/* Generated by Cython 0.23.4 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_4"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__scipy__sparse__csgraph___shortest_path
#define __PYX_HAVE_API__scipy__sparse__csgraph___shortest_path
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "scipy/sparse/csgraph/_shortest_path.pyx",
  "__init__.pxd",
  "type.pxd",
  "scipy/sparse/csgraph/parameters.pxi",
};
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;


/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":725
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":726
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":727
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":728
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":732
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":733
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":734
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":735
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":739
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":740
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":749
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":750
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":751
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":753
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":754
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":755
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":757
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":758
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":760
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":761
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":762
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;

/* "parameters.pxi":3
 * 
 * DTYPE = np.float64
 * ctypedef np.float64_t DTYPE_t             # <<<<<<<<<<<<<<
 * 
 * ITYPE = np.int32
 */
typedef __pyx_t_5numpy_float64_t __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t;

/* "parameters.pxi":6
 * 
 * ITYPE = np.int32
 * ctypedef np.int32_t ITYPE_t             # <<<<<<<<<<<<<<
 * 
 * # Fused type for int32 and int64
 */
typedef __pyx_t_5numpy_int32_t __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t;
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif


/*--- Type declarations ---*/

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":764
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":765
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":766
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":768
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path__floyd_warshall;
struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode;
struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path_initialize_node;
struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap;

/* "scipy/sparse/csgraph/_shortest_path.pyx":1066
 * #  Fibonacci heap.
 * #
 * cdef enum FibonacciState:             # <<<<<<<<<<<<<<
 *     SCANNED
 *     NOT_IN_HEAP
 */
enum __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciState {
  __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_SCANNED,
  __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_NOT_IN_HEAP,
  __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_IN_HEAP
};

/* "scipy/sparse/csgraph/_shortest_path.pyx":250
 * 
 * @cython.boundscheck(False)
 * cdef void _floyd_warshall(             # <<<<<<<<<<<<<<
 *                np.ndarray[DTYPE_t, ndim=2, mode='c'] dist_matrix,
 *                np.ndarray[ITYPE_t, ndim=2, mode='c'] predecessor_matrix,
 */
struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path__floyd_warshall {
  int __pyx_n;
  int directed;
};

/* "scipy/sparse/csgraph/_shortest_path.pyx":1072
 * 
 * 
 * cdef struct FibonacciNode:             # <<<<<<<<<<<<<<
 *     unsigned int index
 *     unsigned int rank
 */
struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode {
  unsigned int index;
  unsigned int rank;
  enum __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciState state;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t val;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *parent;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *left_sibling;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *right_sibling;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *children;
};

/* "scipy/sparse/csgraph/_shortest_path.pyx":1083
 * 
 * 
 * cdef void initialize_node(FibonacciNode* node,             # <<<<<<<<<<<<<<
 *                           unsigned int index,
 *                           DTYPE_t val=0):
 */
struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path_initialize_node {
  int __pyx_n;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t val;
};

/* "scipy/sparse/csgraph/_shortest_path.pyx":1169
 * #  routines to implement a Fibonacci heap
 * 
 * ctypedef FibonacciNode* pFibonacciNode             # <<<<<<<<<<<<<<
 * 
 * 
 */
typedef struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_pFibonacciNode;

/* "scipy/sparse/csgraph/_shortest_path.pyx":1172
 * 
 * 
 * cdef struct FibonacciHeap:             # <<<<<<<<<<<<<<
 *     FibonacciNode* min_node
 *     pFibonacciNode[100] roots_by_rank  # maximum number of nodes is ~2^100.
 */
struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap {
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *min_node;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_pFibonacciNode roots_by_rank[0x64];
};

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

#include <string.h>

static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

static CYTHON_INLINE int  __Pyx_GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);

#define __Pyx_BufPtrCContig2d(type, buf, i0, s0, i1, s1) ((type)((char*)buf + i0 * s0) + i1)
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

static double __Pyx__PyObject_AsDouble(PyObject* obj);
#if CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyObject_AsDouble(obj)\
(likely(PyFloat_CheckExact(obj)) ? PyFloat_AS_DOUBLE(obj) :\
 likely(PyInt_CheckExact(obj)) ?\
 PyFloat_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#else
#define __Pyx_PyObject_AsDouble(obj)\
((likely(PyFloat_CheckExact(obj))) ?\
 PyFloat_AS_DOUBLE(obj) : __Pyx__PyObject_AsDouble(obj))
#endif

static void __Pyx_RaiseBufferIndexError(int axis);

#define __Pyx_BufPtrCContig1d(type, buf, i0, s0) ((type)buf + i0)
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


static Py_ssize_t __Pyx_zeros[] = {0, 0, 0, 0, 0, 0, 0, 0};
static Py_ssize_t __Pyx_minusones[] = {-1, -1, -1, -1, -1, -1, -1, -1};

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int32(npy_int32 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if (defined(_WIN32) || defined(__clang__)) && defined(__cplusplus) && CYTHON_CCOMPLEX
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eqf(a, b)   ((a)==(b))
    #define __Pyx_c_sumf(a, b)  ((a)+(b))
    #define __Pyx_c_difff(a, b) ((a)-(b))
    #define __Pyx_c_prodf(a, b) ((a)*(b))
    #define __Pyx_c_quotf(a, b) ((a)/(b))
    #define __Pyx_c_negf(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zerof(z) ((z)==(float)0)
    #define __Pyx_c_conjf(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_absf(z)     (::std::abs(z))
        #define __Pyx_c_powf(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zerof(z) ((z)==0)
    #define __Pyx_c_conjf(z)    (conjf(z))
    #if 1
        #define __Pyx_c_absf(z)     (cabsf(z))
        #define __Pyx_c_powf(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq(a, b)   ((a)==(b))
    #define __Pyx_c_sum(a, b)  ((a)+(b))
    #define __Pyx_c_diff(a, b) ((a)-(b))
    #define __Pyx_c_prod(a, b) ((a)*(b))
    #define __Pyx_c_quot(a, b) ((a)/(b))
    #define __Pyx_c_neg(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero(z) ((z)==(double)0)
    #define __Pyx_c_conj(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (::std::abs(z))
        #define __Pyx_c_pow(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero(z) ((z)==0)
    #define __Pyx_c_conj(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (cabs(z))
        #define __Pyx_c_pow(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'scipy.sparse.csgraph._shortest_path' */
static __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_EPS;
static __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_NULL_IDX;
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__floyd_warshall(PyArrayObject *, PyArrayObject *, struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path__floyd_warshall *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__dijkstra_directed(PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t); /*proto*/
static PyObject *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__dijkstra_undirected(PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t); /*proto*/
static int __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__bellman_ford_directed(PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *); /*proto*/
static int __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__bellman_ford_undirected(PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *); /*proto*/
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_add_weights(PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *); /*proto*/
static int __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_directed(PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *); /*proto*/
static int __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_undirected(PyArrayObject *, PyArrayObject *, PyArrayObject *, PyArrayObject *); /*proto*/
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_initialize_node(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *, unsigned int, struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path_initialize_node *__pyx_optional_args); /*proto*/
static struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_rightmost_sibling(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *); /*proto*/
static struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_leftmost_sibling(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *); /*proto*/
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_child(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *); /*proto*/
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_sibling(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *); /*proto*/
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *); /*proto*/
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_insert_node(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap *, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *); /*proto*/
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_decrease_val(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap *, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *, __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t); /*proto*/
static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_link(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap *, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *); /*proto*/
static struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove_min(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t = { "DTYPE_t", NULL, sizeof(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t = { "ITYPE_t", NULL, sizeof(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t), 0 };
#define __Pyx_MODULE_NAME "scipy.sparse.csgraph._shortest_path"
int __pyx_module_is_main_scipy__sparse__csgraph___shortest_path = 0;

/* Implementation of 'scipy.sparse.csgraph._shortest_path' */
static PyObject *__pyx_builtin_Exception;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_RuntimeError;
static char __pyx_k_[] = "";
static char __pyx_k_B[] = "B";
static char __pyx_k_C[] = "C";
static char __pyx_k_D[] = "D";
static char __pyx_k_H[] = "H";
static char __pyx_k_I[] = "I";
static char __pyx_k_J[] = "J";
static char __pyx_k_L[] = "L";
static char __pyx_k_N[] = "N";
static char __pyx_k_O[] = "O";
static char __pyx_k_Q[] = "Q";
static char __pyx_k_T[] = "T";
static char __pyx_k_b[] = "b";
static char __pyx_k_d[] = "d";
static char __pyx_k_f[] = "f";
static char __pyx_k_g[] = "g";
static char __pyx_k_h[] = "h";
static char __pyx_k_i[] = "i";
static char __pyx_k_l[] = "l";
static char __pyx_k_q[] = "q";
static char __pyx_k_BF[] = "BF";
static char __pyx_k_FW[] = "FW";
static char __pyx_k_Nk[] = "Nk";
static char __pyx_k_Zd[] = "Zd";
static char __pyx_k_Zf[] = "Zf";
static char __pyx_k_Zg[] = "Zg";
static char __pyx_k_np[] = "np";
static char __pyx_k_any[] = "any";
static char __pyx_k_doc[] = "__doc__";
static char __pyx_k_inf[] = "inf";
static char __pyx_k_nnz[] = "nnz";
static char __pyx_k_ret[] = "ret";
static char __pyx_k_sum[] = "sum";
static char __pyx_k_auto[] = "auto";
static char __pyx_k_copy[] = "copy";
static char __pyx_k_data[] = "data";
static char __pyx_k_fill[] = "fill";
static char __pyx_k_flat[] = "flat";
static char __pyx_k_init[] = "__init__";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_ones[] = "ones";
static char __pyx_k_self[] = "self";
static char __pyx_k_size[] = "size";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_warn[] = "warn";
static char __pyx_k_DTYPE[] = "DTYPE";
static char __pyx_k_ITYPE[] = "ITYPE";
static char __pyx_k_array[] = "array";
static char __pyx_k_dtype[] = "dtype";
static char __pyx_k_empty[] = "empty";
static char __pyx_k_int32[] = "int32";
static char __pyx_k_isinf[] = "isinf";
static char __pyx_k_limit[] = "limit";
static char __pyx_k_numpy[] = "numpy";
static char __pyx_k_order[] = "order";
static char __pyx_k_range[] = "range";
static char __pyx_k_shape[] = "shape";
static char __pyx_k_tocsr[] = "tocsr";
static char __pyx_k_where[] = "where";
static char __pyx_k_zeros[] = "zeros";
static char __pyx_k_arange[] = "arange";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_indptr[] = "indptr";
static char __pyx_k_method[] = "method";
static char __pyx_k_module[] = "__module__";
static char __pyx_k_csgraph[] = "csgraph";
static char __pyx_k_float64[] = "float64";
static char __pyx_k_indices[] = "indices";
static char __pyx_k_johnson[] = "johnson";
static char __pyx_k_message[] = "message";
static char __pyx_k_prepare[] = "__prepare__";
static char __pyx_k_reshape[] = "reshape";
static char __pyx_k_csgraphT[] = "csgraphT";
static char __pyx_k_csr_data[] = "csr_data";
static char __pyx_k_diagonal[] = "diagonal";
static char __pyx_k_dijkstra[] = "dijkstra";
static char __pyx_k_directed[] = "directed";
static char __pyx_k_isscalar[] = "isscalar";
static char __pyx_k_qualname[] = "__qualname__";
static char __pyx_k_warnings[] = "warnings";
static char __pyx_k_Exception[] = "Exception";
static char __pyx_k_TypeError[] = "TypeError";
static char __pyx_k_csrT_data[] = "csrT_data";
static char __pyx_k_metaclass[] = "__metaclass__";
static char __pyx_k_overwrite[] = "overwrite";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_atleast_1d[] = "atleast_1d";
static char __pyx_k_csr_matrix[] = "csr_matrix";
static char __pyx_k_csr_output[] = "csr_output";
static char __pyx_k_dist_array[] = "dist_array";
static char __pyx_k_isspmatrix[] = "isspmatrix";
static char __pyx_k_unweighted[] = "unweighted";
static char __pyx_k_dist_matrix[] = "dist_matrix";
static char __pyx_k_RuntimeError[] = "RuntimeError";
static char __pyx_k_bellman_ford[] = "bellman_ford";
static char __pyx_k_dense_output[] = "dense_output";
static char __pyx_k_return_shape[] = "return_shape";
static char __pyx_k_scipy_sparse[] = "scipy.sparse";
static char __pyx_k_copy_if_dense[] = "copy_if_dense";
static char __pyx_k_shortest_path[] = "shortest_path";
static char __pyx_k_copy_if_sparse[] = "copy_if_sparse";
static char __pyx_k_floyd_warshall[] = "floyd_warshall";
static char __pyx_k_isspmatrix_csc[] = "isspmatrix_csc";
static char __pyx_k_isspmatrix_csr[] = "isspmatrix_csr";
static char __pyx_k_validate_graph[] = "validate_graph";
static char __pyx_k_limit_must_be_0[] = "limit must be >= 0";
static char __pyx_k_NegativeCycleError[] = "NegativeCycleError";
static char __pyx_k_predecessor_matrix[] = "predecessor_matrix";
static char __pyx_k_return_predecessors[] = "return_predecessors";
static char __pyx_k_unrecognized_method_s[] = "unrecognized method '%s'";
static char __pyx_k_indices_out_of_range_0_N[] = "indices out of range 0...N";
static char __pyx_k_NegativeCycleError___init[] = "NegativeCycleError.__init__";
static char __pyx_k_Negative_cycle_in_nodes_s[] = "Negative cycle in nodes %s";
static char __pyx_k_limit_must_be_numeric_float[] = "limit must be numeric (float)";
static char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static char __pyx_k_Negative_cycle_detected_on_node[] = "Negative cycle detected on node %i";
static char __pyx_k_Routines_for_performing_shortes[] = "\nRoutines for performing shortest-path graph searches\n\nThe main interface is in the function :func:`shortest_path`.  This\ncalls cython routines that compute the shortest path using\nthe Floyd-Warshall algorithm, Dijkstra's algorithm with Fibonacci Heaps,\nthe Bellman-Ford algorithm, or Johnson's Algorithm.\n";
static char __pyx_k_home_br_repos_scipy_scipy_spars[] = "/home/br/repos/scipy/scipy/sparse/csgraph/_shortest_path.pyx";
static char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static char __pyx_k_Graph_has_negative_weights_dijks[] = "Graph has negative weights: dijkstra will give inaccurate results if the graph contains negative cycles. Consider johnson or bellman_ford.";
static char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static char __pyx_k_scipy_sparse_csgraph__shortest_p[] = "scipy.sparse.csgraph._shortest_path";
static char __pyx_k_scipy_sparse_csgraph__validation[] = "scipy.sparse.csgraph._validation";
static char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_n_s_BF;
static PyObject *__pyx_n_s_C;
static PyObject *__pyx_n_s_D;
static PyObject *__pyx_n_s_DTYPE;
static PyObject *__pyx_n_s_Exception;
static PyObject *__pyx_n_s_FW;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_kp_s_Graph_has_negative_weights_dijks;
static PyObject *__pyx_n_s_ITYPE;
static PyObject *__pyx_n_s_J;
static PyObject *__pyx_n_s_N;
static PyObject *__pyx_n_s_NegativeCycleError;
static PyObject *__pyx_n_s_NegativeCycleError___init;
static PyObject *__pyx_kp_s_Negative_cycle_detected_on_node;
static PyObject *__pyx_kp_s_Negative_cycle_in_nodes_s;
static PyObject *__pyx_n_s_Nk;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_T;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_any;
static PyObject *__pyx_n_s_arange;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_n_s_atleast_1d;
static PyObject *__pyx_n_s_auto;
static PyObject *__pyx_n_s_bellman_ford;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_copy_if_dense;
static PyObject *__pyx_n_s_copy_if_sparse;
static PyObject *__pyx_n_s_csgraph;
static PyObject *__pyx_n_s_csgraphT;
static PyObject *__pyx_n_s_csrT_data;
static PyObject *__pyx_n_s_csr_data;
static PyObject *__pyx_n_s_csr_matrix;
static PyObject *__pyx_n_s_csr_output;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_dense_output;
static PyObject *__pyx_n_s_diagonal;
static PyObject *__pyx_n_s_dijkstra;
static PyObject *__pyx_n_s_directed;
static PyObject *__pyx_n_s_dist_array;
static PyObject *__pyx_n_s_dist_matrix;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_fill;
static PyObject *__pyx_n_s_flat;
static PyObject *__pyx_n_s_float64;
static PyObject *__pyx_n_s_floyd_warshall;
static PyObject *__pyx_kp_s_home_br_repos_scipy_scipy_spars;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_indices;
static PyObject *__pyx_kp_s_indices_out_of_range_0_N;
static PyObject *__pyx_n_s_indptr;
static PyObject *__pyx_n_s_inf;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_int32;
static PyObject *__pyx_n_s_isinf;
static PyObject *__pyx_n_s_isscalar;
static PyObject *__pyx_n_s_isspmatrix;
static PyObject *__pyx_n_s_isspmatrix_csc;
static PyObject *__pyx_n_s_isspmatrix_csr;
static PyObject *__pyx_n_s_johnson;
static PyObject *__pyx_n_s_limit;
static PyObject *__pyx_kp_s_limit_must_be_0;
static PyObject *__pyx_kp_s_limit_must_be_numeric_float;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_message;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_method;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_nnz;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_ones;
static PyObject *__pyx_n_s_order;
static PyObject *__pyx_n_s_overwrite;
static PyObject *__pyx_n_s_predecessor_matrix;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reshape;
static PyObject *__pyx_n_s_ret;
static PyObject *__pyx_n_s_return_predecessors;
static PyObject *__pyx_n_s_return_shape;
static PyObject *__pyx_n_s_scipy_sparse;
static PyObject *__pyx_n_s_scipy_sparse_csgraph__shortest_p;
static PyObject *__pyx_n_s_scipy_sparse_csgraph__validation;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_shortest_path;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_tocsr;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_kp_s_unrecognized_method_s;
static PyObject *__pyx_n_s_unweighted;
static PyObject *__pyx_n_s_validate_graph;
static PyObject *__pyx_n_s_warn;
static PyObject *__pyx_n_s_warnings;
static PyObject *__pyx_n_s_where;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_18NegativeCycleError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_message); /* proto */
static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_shortest_path(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_method, PyObject *__pyx_v_directed, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted, PyObject *__pyx_v_overwrite); /* proto */
static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_2floyd_warshall(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_directed, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted, PyObject *__pyx_v_overwrite); /* proto */
static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_4dijkstra(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_directed, PyObject *__pyx_v_indices, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted, PyObject *__pyx_v_limit); /* proto */
static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_6bellman_ford(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_directed, PyObject *__pyx_v_indices, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted); /* proto */
static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_8johnson(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_directed, PyObject *__pyx_v_indices, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_k__4;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__14;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_codeobj__23;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__28;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;

/* "scipy/sparse/csgraph/_shortest_path.pyx":28
 * 
 * class NegativeCycleError(Exception):
 *     def __init__(self, message=''):             # <<<<<<<<<<<<<<
 *         Exception.__init__(self, message)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_18NegativeCycleError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_18NegativeCycleError_1__init__ = {"__init__", (PyCFunction)__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_18NegativeCycleError_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_18NegativeCycleError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_message = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_message,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)((PyObject*)__pyx_kp_s_));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_message);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_message = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.NegativeCycleError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_18NegativeCycleError___init__(__pyx_self, __pyx_v_self, __pyx_v_message);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_18NegativeCycleError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_message) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":29
 * class NegativeCycleError(Exception):
 *     def __init__(self, message=''):
 *         Exception.__init__(self, message)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_builtin_Exception, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_INCREF(__pyx_v_self);
  __Pyx_GIVEREF(__pyx_v_self);
  PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_self);
  __Pyx_INCREF(__pyx_v_message);
  __Pyx_GIVEREF(__pyx_v_message);
  PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_message);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":28
 * 
 * class NegativeCycleError(Exception):
 *     def __init__(self, message=''):             # <<<<<<<<<<<<<<
 *         Exception.__init__(self, message)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.NegativeCycleError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":32
 * 
 * 
 * def shortest_path(csgraph, method='auto',             # <<<<<<<<<<<<<<
 *                   directed=True,
 *                   return_predecessors=False,
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_1shortest_path(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_shortest_path[] = "\n    shortest_path(csgraph, method='auto', directed=True, return_predecessors=False,\n                  unweighted=False, overwrite=False)\n\n    Perform a shortest-path graph search on a positive directed or\n    undirected graph.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of distances representing the input graph.\n    method : string ['auto'|'FW'|'D'], optional\n        Algorithm to use for shortest paths.  Options are:\n\n           'auto' -- (default) select the best among 'FW', 'D', 'BF', or 'J'\n                     based on the input data.\n\n           'FW'   -- Floyd-Warshall algorithm.  Computational cost is\n                     approximately ``O[N^3]``.  The input csgraph will be\n                     converted to a dense representation.\n\n           'D'    -- Dijkstra's algorithm with Fibonacci heaps.  Computational\n                     cost is approximately ``O[N(N*k + N*log(N))]``, where\n\t\t     ``k`` is the average number of connected edges per node.\n\t\t     The input csgraph will be converted to a csr\n\t\t     representation.\n\n           'BF'   -- Bellman-Ford algorithm.  This algorithm can be used when\n                     weights are negative.  If a negative cycle is encountered,\n                     an error will be raised.  Computational cost is\n                     approximately ``O[N(N^2 k)]``, where ``k`` is the average\n                     number of connected edges per node. The input csgraph will\n                     be converted to a csr representation.\n\n           'J'    -- Johnson's algorithm.  Like the Bellman-Ford algorithm,\n                     Johnson's algorithm is designed for use when the weights\n                     are negative.  It combines the Bellman-Ford algorithm\n                     with Dijkstra's algorithm for faster computation.\n\n    directed : bool, optional\n        If True (default), the""n find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n    overwrite : bool, optional\n        If True, overwrite csgraph with the result.  This applies only if\n        method == 'FW' and csgraph is a dense, c-ordered array with\n        dtype=float64.\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The N x N matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        information on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Raises\n    ------\n    NegativeCycleError:\n        if there are negative cycles in the graph\n\n    Notes\n    -----\n    As currently implemented, Dijkstra's algorithm and Johnson's algorithm\n    do not work for graphs with direction-dependent distances when\n    directed == False.  i.e., if csgraph[i,j] and csgraph[j,i] are non-equal\n    edges, method='D' may yield an incorrect result.\n    ";
static PyMethodDef __pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_1shortest_path = {"shortest_path", (PyCFunction)__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_1shortest_path, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_shortest_path};
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_1shortest_path(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_csgraph = 0;
  PyObject *__pyx_v_method = 0;
  PyObject *__pyx_v_directed = 0;
  PyObject *__pyx_v_return_predecessors = 0;
  PyObject *__pyx_v_unweighted = 0;
  PyObject *__pyx_v_overwrite = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("shortest_path (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_csgraph,&__pyx_n_s_method,&__pyx_n_s_directed,&__pyx_n_s_return_predecessors,&__pyx_n_s_unweighted,&__pyx_n_s_overwrite,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    values[1] = ((PyObject *)__pyx_n_s_auto);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":33
 * 
 * def shortest_path(csgraph, method='auto',
 *                   directed=True,             # <<<<<<<<<<<<<<
 *                   return_predecessors=False,
 *                   unweighted=False,
 */
    values[2] = ((PyObject *)Py_True);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":34
 * def shortest_path(csgraph, method='auto',
 *                   directed=True,
 *                   return_predecessors=False,             # <<<<<<<<<<<<<<
 *                   unweighted=False,
 *                   overwrite=False):
 */
    values[3] = ((PyObject *)Py_False);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":35
 *                   directed=True,
 *                   return_predecessors=False,
 *                   unweighted=False,             # <<<<<<<<<<<<<<
 *                   overwrite=False):
 *     """
 */
    values[4] = ((PyObject *)Py_False);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":36
 *                   return_predecessors=False,
 *                   unweighted=False,
 *                   overwrite=False):             # <<<<<<<<<<<<<<
 *     """
 *     shortest_path(csgraph, method='auto', directed=True, return_predecessors=False,
 */
    values[5] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_csgraph)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_method);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_directed);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_predecessors);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_unweighted);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_overwrite);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "shortest_path") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_csgraph = values[0];
    __pyx_v_method = values[1];
    __pyx_v_directed = values[2];
    __pyx_v_return_predecessors = values[3];
    __pyx_v_unweighted = values[4];
    __pyx_v_overwrite = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("shortest_path", 0, 1, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.shortest_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_shortest_path(__pyx_self, __pyx_v_csgraph, __pyx_v_method, __pyx_v_directed, __pyx_v_return_predecessors, __pyx_v_unweighted, __pyx_v_overwrite);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":32
 * 
 * 
 * def shortest_path(csgraph, method='auto',             # <<<<<<<<<<<<<<
 *                   directed=True,
 *                   return_predecessors=False,
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_shortest_path(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_method, PyObject *__pyx_v_directed, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted, PyObject *__pyx_v_overwrite) {
  PyObject *__pyx_v_N = NULL;
  PyObject *__pyx_v_Nk = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("shortest_path", 0);
  __Pyx_INCREF(__pyx_v_method);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":124
 *     # validate here to catch errors early but don't store the result;
 *     # we'll validate again later
 *     validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                    copy_if_dense=(not overwrite),
 *                    copy_if_sparse=(not overwrite))
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_validate_graph); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_csgraph);
  __Pyx_GIVEREF(__pyx_v_csgraph);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_csgraph);
  __Pyx_INCREF(__pyx_v_directed);
  __Pyx_GIVEREF(__pyx_v_directed);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_directed);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":125
 *     # we'll validate again later
 *     validate_graph(csgraph, directed, DTYPE,
 *                    copy_if_dense=(not overwrite),             # <<<<<<<<<<<<<<
 *                    copy_if_sparse=(not overwrite))
 * 
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyBool_FromLong((!__pyx_t_4)); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_copy_if_dense, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":126
 *     validate_graph(csgraph, directed, DTYPE,
 *                    copy_if_dense=(not overwrite),
 *                    copy_if_sparse=(not overwrite))             # <<<<<<<<<<<<<<
 * 
 *     if method == 'auto':
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 126; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyBool_FromLong((!__pyx_t_4)); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 126; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_copy_if_sparse, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":124
 *     # validate here to catch errors early but don't store the result;
 *     # we'll validate again later
 *     validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                    copy_if_dense=(not overwrite),
 *                    copy_if_sparse=(not overwrite))
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":128
 *                    copy_if_sparse=(not overwrite))
 * 
 *     if method == 'auto':             # <<<<<<<<<<<<<<
 *         # guess fastest method based on number of nodes and edges
 *         N = csgraph.shape[0]
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_method, __pyx_n_s_auto, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":130
 *     if method == 'auto':
 *         # guess fastest method based on number of nodes and edges
 *         N = csgraph.shape[0]             # <<<<<<<<<<<<<<
 *         if isspmatrix(csgraph):
 *             Nk = csgraph.nnz
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_shape); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_5, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_N = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":131
 *         # guess fastest method based on number of nodes and edges
 *         N = csgraph.shape[0]
 *         if isspmatrix(csgraph):             # <<<<<<<<<<<<<<
 *             Nk = csgraph.nnz
 *         else:
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_isspmatrix); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_csgraph); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_csgraph);
      __Pyx_GIVEREF(__pyx_v_csgraph);
      PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_v_csgraph);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_4) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":132
 *         N = csgraph.shape[0]
 *         if isspmatrix(csgraph):
 *             Nk = csgraph.nnz             # <<<<<<<<<<<<<<
 *         else:
 *             Nk = np.sum(csgraph > 0)
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_nnz); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_v_Nk = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":131
 *         # guess fastest method based on number of nodes and edges
 *         N = csgraph.shape[0]
 *         if isspmatrix(csgraph):             # <<<<<<<<<<<<<<
 *             Nk = csgraph.nnz
 *         else:
 */
      goto __pyx_L4;
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":134
 *             Nk = csgraph.nnz
 *         else:
 *             Nk = np.sum(csgraph > 0)             # <<<<<<<<<<<<<<
 * 
 *         if Nk < N * N / 4:
 */
    /*else*/ {
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_sum); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_csgraph, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_3 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      if (!__pyx_t_3) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_v_Nk = __pyx_t_2;
      __pyx_t_2 = 0;
    }
    __pyx_L4:;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":136
 *             Nk = np.sum(csgraph > 0)
 * 
 *         if Nk < N * N / 4:             # <<<<<<<<<<<<<<
 *             if ((isspmatrix(csgraph) and np.any(csgraph.data < 0))
 *                       or (not isspmatrix(csgraph) and np.any(csgraph < 0))):
 */
    __pyx_t_2 = PyNumber_Multiply(__pyx_v_N, __pyx_v_N); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_int_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_Nk, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_4) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":137
 * 
 *         if Nk < N * N / 4:
 *             if ((isspmatrix(csgraph) and np.any(csgraph.data < 0))             # <<<<<<<<<<<<<<
 *                       or (not isspmatrix(csgraph) and np.any(csgraph < 0))):
 *                 method = 'J'
 */
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_isspmatrix); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_csgraph); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_INCREF(__pyx_v_csgraph);
        __Pyx_GIVEREF(__pyx_v_csgraph);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_csgraph);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!__pyx_t_7) {
        goto __pyx_L8_next_or;
      } else {
      }
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_any); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_data); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_1) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1); __pyx_t_1 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!__pyx_t_7) {
      } else {
        __pyx_t_4 = __pyx_t_7;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_L8_next_or:;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":138
 *         if Nk < N * N / 4:
 *             if ((isspmatrix(csgraph) and np.any(csgraph.data < 0))
 *                       or (not isspmatrix(csgraph) and np.any(csgraph < 0))):             # <<<<<<<<<<<<<<
 *                 method = 'J'
 *             else:
 */
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_isspmatrix); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_3) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_csgraph); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_v_csgraph);
        __Pyx_GIVEREF(__pyx_v_csgraph);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_csgraph);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = ((!__pyx_t_7) != 0);
      if (__pyx_t_8) {
      } else {
        __pyx_t_4 = __pyx_t_8;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_any); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_csgraph, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_3 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      if (!__pyx_t_3) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_4 = __pyx_t_8;
      __pyx_L7_bool_binop_done:;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":137
 * 
 *         if Nk < N * N / 4:
 *             if ((isspmatrix(csgraph) and np.any(csgraph.data < 0))             # <<<<<<<<<<<<<<
 *                       or (not isspmatrix(csgraph) and np.any(csgraph < 0))):
 *                 method = 'J'
 */
      if (__pyx_t_4) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":139
 *             if ((isspmatrix(csgraph) and np.any(csgraph.data < 0))
 *                       or (not isspmatrix(csgraph) and np.any(csgraph < 0))):
 *                 method = 'J'             # <<<<<<<<<<<<<<
 *             else:
 *                 method = 'D'
 */
        __Pyx_INCREF(__pyx_n_s_J);
        __Pyx_DECREF_SET(__pyx_v_method, __pyx_n_s_J);

        /* "scipy/sparse/csgraph/_shortest_path.pyx":137
 * 
 *         if Nk < N * N / 4:
 *             if ((isspmatrix(csgraph) and np.any(csgraph.data < 0))             # <<<<<<<<<<<<<<
 *                       or (not isspmatrix(csgraph) and np.any(csgraph < 0))):
 *                 method = 'J'
 */
        goto __pyx_L6;
      }

      /* "scipy/sparse/csgraph/_shortest_path.pyx":141
 *                 method = 'J'
 *             else:
 *                 method = 'D'             # <<<<<<<<<<<<<<
 *         else:
 *             method = 'FW'
 */
      /*else*/ {
        __Pyx_INCREF(__pyx_n_s_D);
        __Pyx_DECREF_SET(__pyx_v_method, __pyx_n_s_D);
      }
      __pyx_L6:;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":136
 *             Nk = np.sum(csgraph > 0)
 * 
 *         if Nk < N * N / 4:             # <<<<<<<<<<<<<<
 *             if ((isspmatrix(csgraph) and np.any(csgraph.data < 0))
 *                       or (not isspmatrix(csgraph) and np.any(csgraph < 0))):
 */
      goto __pyx_L5;
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":143
 *                 method = 'D'
 *         else:
 *             method = 'FW'             # <<<<<<<<<<<<<<
 * 
 *     if method == 'FW':
 */
    /*else*/ {
      __Pyx_INCREF(__pyx_n_s_FW);
      __Pyx_DECREF_SET(__pyx_v_method, __pyx_n_s_FW);
    }
    __pyx_L5:;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":128
 *                    copy_if_sparse=(not overwrite))
 * 
 *     if method == 'auto':             # <<<<<<<<<<<<<<
 *         # guess fastest method based on number of nodes and edges
 *         N = csgraph.shape[0]
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":145
 *             method = 'FW'
 * 
 *     if method == 'FW':             # <<<<<<<<<<<<<<
 *         return floyd_warshall(csgraph, directed,
 *                               return_predecessors=return_predecessors,
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_method, __pyx_n_s_FW, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":146
 * 
 *     if method == 'FW':
 *         return floyd_warshall(csgraph, directed,             # <<<<<<<<<<<<<<
 *                               return_predecessors=return_predecessors,
 *                               unweighted=unweighted,
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_floyd_warshall); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 146; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 146; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v_csgraph);
    __Pyx_GIVEREF(__pyx_v_csgraph);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_csgraph);
    __Pyx_INCREF(__pyx_v_directed);
    __Pyx_GIVEREF(__pyx_v_directed);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_directed);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":147
 *     if method == 'FW':
 *         return floyd_warshall(csgraph, directed,
 *                               return_predecessors=return_predecessors,             # <<<<<<<<<<<<<<
 *                               unweighted=unweighted,
 *                               overwrite=overwrite)
 */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return_predecessors, __pyx_v_return_predecessors) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":148
 *         return floyd_warshall(csgraph, directed,
 *                               return_predecessors=return_predecessors,
 *                               unweighted=unweighted,             # <<<<<<<<<<<<<<
 *                               overwrite=overwrite)
 * 
 */
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_unweighted, __pyx_v_unweighted) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":149
 *                               return_predecessors=return_predecessors,
 *                               unweighted=unweighted,
 *                               overwrite=overwrite)             # <<<<<<<<<<<<<<
 * 
 *     elif method == 'D':
 */
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_overwrite, __pyx_v_overwrite) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":146
 * 
 *     if method == 'FW':
 *         return floyd_warshall(csgraph, directed,             # <<<<<<<<<<<<<<
 *                               return_predecessors=return_predecessors,
 *                               unweighted=unweighted,
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, __pyx_t_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 146; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":145
 *             method = 'FW'
 * 
 *     if method == 'FW':             # <<<<<<<<<<<<<<
 *         return floyd_warshall(csgraph, directed,
 *                               return_predecessors=return_predecessors,
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":151
 *                               overwrite=overwrite)
 * 
 *     elif method == 'D':             # <<<<<<<<<<<<<<
 *         return dijkstra(csgraph, directed,
 *                         return_predecessors=return_predecessors,
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_method, __pyx_n_s_D, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":152
 * 
 *     elif method == 'D':
 *         return dijkstra(csgraph, directed,             # <<<<<<<<<<<<<<
 *                         return_predecessors=return_predecessors,
 *                         unweighted=unweighted)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_dijkstra); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_csgraph);
    __Pyx_GIVEREF(__pyx_v_csgraph);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_csgraph);
    __Pyx_INCREF(__pyx_v_directed);
    __Pyx_GIVEREF(__pyx_v_directed);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_directed);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":153
 *     elif method == 'D':
 *         return dijkstra(csgraph, directed,
 *                         return_predecessors=return_predecessors,             # <<<<<<<<<<<<<<
 *                         unweighted=unweighted)
 * 
 */
    __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_return_predecessors, __pyx_v_return_predecessors) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":154
 *         return dijkstra(csgraph, directed,
 *                         return_predecessors=return_predecessors,
 *                         unweighted=unweighted)             # <<<<<<<<<<<<<<
 * 
 *     elif method == 'BF':
 */
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_unweighted, __pyx_v_unweighted) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":152
 * 
 *     elif method == 'D':
 *         return dijkstra(csgraph, directed,             # <<<<<<<<<<<<<<
 *                         return_predecessors=return_predecessors,
 *                         unweighted=unweighted)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":151
 *                               overwrite=overwrite)
 * 
 *     elif method == 'D':             # <<<<<<<<<<<<<<
 *         return dijkstra(csgraph, directed,
 *                         return_predecessors=return_predecessors,
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":156
 *                         unweighted=unweighted)
 * 
 *     elif method == 'BF':             # <<<<<<<<<<<<<<
 *         return bellman_ford(csgraph, directed,
 *                             return_predecessors=return_predecessors,
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_method, __pyx_n_s_BF, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":157
 * 
 *     elif method == 'BF':
 *         return bellman_ford(csgraph, directed,             # <<<<<<<<<<<<<<
 *                             return_predecessors=return_predecessors,
 *                             unweighted=unweighted)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_bellman_ford); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v_csgraph);
    __Pyx_GIVEREF(__pyx_v_csgraph);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_csgraph);
    __Pyx_INCREF(__pyx_v_directed);
    __Pyx_GIVEREF(__pyx_v_directed);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_directed);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":158
 *     elif method == 'BF':
 *         return bellman_ford(csgraph, directed,
 *                             return_predecessors=return_predecessors,             # <<<<<<<<<<<<<<
 *                             unweighted=unweighted)
 * 
 */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return_predecessors, __pyx_v_return_predecessors) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":159
 *         return bellman_ford(csgraph, directed,
 *                             return_predecessors=return_predecessors,
 *                             unweighted=unweighted)             # <<<<<<<<<<<<<<
 * 
 *     elif method == 'J':
 */
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_unweighted, __pyx_v_unweighted) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":157
 * 
 *     elif method == 'BF':
 *         return bellman_ford(csgraph, directed,             # <<<<<<<<<<<<<<
 *                             return_predecessors=return_predecessors,
 *                             unweighted=unweighted)
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, __pyx_t_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":156
 *                         unweighted=unweighted)
 * 
 *     elif method == 'BF':             # <<<<<<<<<<<<<<
 *         return bellman_ford(csgraph, directed,
 *                             return_predecessors=return_predecessors,
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":161
 *                             unweighted=unweighted)
 * 
 *     elif method == 'J':             # <<<<<<<<<<<<<<
 *         return johnson(csgraph, directed,
 *                        return_predecessors=return_predecessors,
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_method, __pyx_n_s_J, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":162
 * 
 *     elif method == 'J':
 *         return johnson(csgraph, directed,             # <<<<<<<<<<<<<<
 *                        return_predecessors=return_predecessors,
 *                        unweighted=unweighted)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_johnson); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_csgraph);
    __Pyx_GIVEREF(__pyx_v_csgraph);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_csgraph);
    __Pyx_INCREF(__pyx_v_directed);
    __Pyx_GIVEREF(__pyx_v_directed);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_directed);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":163
 *     elif method == 'J':
 *         return johnson(csgraph, directed,
 *                        return_predecessors=return_predecessors,             # <<<<<<<<<<<<<<
 *                        unweighted=unweighted)
 * 
 */
    __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_return_predecessors, __pyx_v_return_predecessors) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":164
 *         return johnson(csgraph, directed,
 *                        return_predecessors=return_predecessors,
 *                        unweighted=unweighted)             # <<<<<<<<<<<<<<
 * 
 *     else:
 */
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_unweighted, __pyx_v_unweighted) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":162
 * 
 *     elif method == 'J':
 *         return johnson(csgraph, directed,             # <<<<<<<<<<<<<<
 *                        return_predecessors=return_predecessors,
 *                        unweighted=unweighted)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":161
 *                             unweighted=unweighted)
 * 
 *     elif method == 'J':             # <<<<<<<<<<<<<<
 *         return johnson(csgraph, directed,
 *                        return_predecessors=return_predecessors,
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":167
 * 
 *     else:
 *         raise ValueError("unrecognized method '%s'" % method)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_unrecognized_method_s, __pyx_v_method); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":32
 * 
 * 
 * def shortest_path(csgraph, method='auto',             # <<<<<<<<<<<<<<
 *                   directed=True,
 *                   return_predecessors=False,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.shortest_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_N);
  __Pyx_XDECREF(__pyx_v_Nk);
  __Pyx_XDECREF(__pyx_v_method);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":170
 * 
 * 
 * def floyd_warshall(csgraph, directed=True,             # <<<<<<<<<<<<<<
 *                    return_predecessors=False,
 *                    unweighted=False,
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_3floyd_warshall(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_2floyd_warshall[] = "\n    floyd_warshall(csgraph, directed=True, return_predecessors=False,\n                   unweighted=False, overwrite=False)\n\n    Compute the shortest path lengths using the Floyd-Warshall algorithm\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of distances representing the input graph.\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n    overwrite : bool, optional\n        If True, overwrite csgraph with the result.  This applies only if\n        csgraph is a dense, c-ordered array with dtype=float64.\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The N x N matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        information on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Raises""\n    ------\n    NegativeCycleError:\n        if there are negative cycles in the graph\n    ";
static PyMethodDef __pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_3floyd_warshall = {"floyd_warshall", (PyCFunction)__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_3floyd_warshall, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_2floyd_warshall};
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_3floyd_warshall(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_csgraph = 0;
  PyObject *__pyx_v_directed = 0;
  PyObject *__pyx_v_return_predecessors = 0;
  PyObject *__pyx_v_unweighted = 0;
  PyObject *__pyx_v_overwrite = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("floyd_warshall (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_csgraph,&__pyx_n_s_directed,&__pyx_n_s_return_predecessors,&__pyx_n_s_unweighted,&__pyx_n_s_overwrite,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[1] = ((PyObject *)Py_True);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":171
 * 
 * def floyd_warshall(csgraph, directed=True,
 *                    return_predecessors=False,             # <<<<<<<<<<<<<<
 *                    unweighted=False,
 *                    overwrite=False):
 */
    values[2] = ((PyObject *)Py_False);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":172
 * def floyd_warshall(csgraph, directed=True,
 *                    return_predecessors=False,
 *                    unweighted=False,             # <<<<<<<<<<<<<<
 *                    overwrite=False):
 *     """
 */
    values[3] = ((PyObject *)Py_False);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":173
 *                    return_predecessors=False,
 *                    unweighted=False,
 *                    overwrite=False):             # <<<<<<<<<<<<<<
 *     """
 *     floyd_warshall(csgraph, directed=True, return_predecessors=False,
 */
    values[4] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_csgraph)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_directed);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_predecessors);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_unweighted);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_overwrite);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "floyd_warshall") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_csgraph = values[0];
    __pyx_v_directed = values[1];
    __pyx_v_return_predecessors = values[2];
    __pyx_v_unweighted = values[3];
    __pyx_v_overwrite = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("floyd_warshall", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.floyd_warshall", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_2floyd_warshall(__pyx_self, __pyx_v_csgraph, __pyx_v_directed, __pyx_v_return_predecessors, __pyx_v_unweighted, __pyx_v_overwrite);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":170
 * 
 * 
 * def floyd_warshall(csgraph, directed=True,             # <<<<<<<<<<<<<<
 *                    return_predecessors=False,
 *                    unweighted=False,
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_2floyd_warshall(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_directed, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted, PyObject *__pyx_v_overwrite) {
  PyObject *__pyx_v_dist_matrix = NULL;
  PyObject *__pyx_v_predecessor_matrix = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path__floyd_warshall __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("floyd_warshall", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":222
 *         if there are negative cycles in the graph
 *     """
 *     dist_matrix = validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                                  csr_output=False,
 *                                  copy_if_dense=not overwrite)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_validate_graph); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_csgraph);
  __Pyx_GIVEREF(__pyx_v_csgraph);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_csgraph);
  __Pyx_INCREF(__pyx_v_directed);
  __Pyx_GIVEREF(__pyx_v_directed);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_directed);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":223
 *     """
 *     dist_matrix = validate_graph(csgraph, directed, DTYPE,
 *                                  csr_output=False,             # <<<<<<<<<<<<<<
 *                                  copy_if_dense=not overwrite)
 * 
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_csr_output, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":224
 *     dist_matrix = validate_graph(csgraph, directed, DTYPE,
 *                                  csr_output=False,
 *                                  copy_if_dense=not overwrite)             # <<<<<<<<<<<<<<
 * 
 *     if unweighted:
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyBool_FromLong((!__pyx_t_4)); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_copy_if_dense, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":222
 *         if there are negative cycles in the graph
 *     """
 *     dist_matrix = validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                                  csr_output=False,
 *                                  copy_if_dense=not overwrite)
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_dist_matrix = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":226
 *                                  copy_if_dense=not overwrite)
 * 
 *     if unweighted:             # <<<<<<<<<<<<<<
 *         dist_matrix[~np.isinf(dist_matrix)] = 1
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_unweighted); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":227
 * 
 *     if unweighted:
 *         dist_matrix[~np.isinf(dist_matrix)] = 1             # <<<<<<<<<<<<<<
 * 
 *     if return_predecessors:
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_isinf); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_dist_matrix); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(__pyx_v_dist_matrix);
      __Pyx_GIVEREF(__pyx_v_dist_matrix);
      PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_v_dist_matrix);
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Invert(__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_dist_matrix, __pyx_t_3, __pyx_int_1) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":226
 *                                  copy_if_dense=not overwrite)
 * 
 *     if unweighted:             # <<<<<<<<<<<<<<
 *         dist_matrix[~np.isinf(dist_matrix)] = 1
 * 
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":229
 *         dist_matrix[~np.isinf(dist_matrix)] = 1
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         predecessor_matrix = np.empty(dist_matrix.shape,
 *                                       dtype=ITYPE, order='C')
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_return_predecessors); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 229; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":230
 * 
 *     if return_predecessors:
 *         predecessor_matrix = np.empty(dist_matrix.shape,             # <<<<<<<<<<<<<<
 *                                       dtype=ITYPE, order='C')
 *     else:
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":231
 *     if return_predecessors:
 *         predecessor_matrix = np.empty(dist_matrix.shape,
 *                                       dtype=ITYPE, order='C')             # <<<<<<<<<<<<<<
 *     else:
 *         predecessor_matrix = np.empty((0, 0), dtype=ITYPE)
 */
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":230
 * 
 *     if return_predecessors:
 *         predecessor_matrix = np.empty(dist_matrix.shape,             # <<<<<<<<<<<<<<
 *                                       dtype=ITYPE, order='C')
 *     else:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_predecessor_matrix = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":229
 *         dist_matrix[~np.isinf(dist_matrix)] = 1
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         predecessor_matrix = np.empty(dist_matrix.shape,
 *                                       dtype=ITYPE, order='C')
 */
    goto __pyx_L4;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":233
 *                                       dtype=ITYPE, order='C')
 *     else:
 *         predecessor_matrix = np.empty((0, 0), dtype=ITYPE)             # <<<<<<<<<<<<<<
 * 
 *     _floyd_warshall(dist_matrix,
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_predecessor_matrix = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L4:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":235
 *         predecessor_matrix = np.empty((0, 0), dtype=ITYPE)
 * 
 *     _floyd_warshall(dist_matrix,             # <<<<<<<<<<<<<<
 *                     predecessor_matrix,
 *                     int(directed))
 */
  if (!(likely(((__pyx_v_dist_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":236
 * 
 *     _floyd_warshall(dist_matrix,
 *                     predecessor_matrix,             # <<<<<<<<<<<<<<
 *                     int(directed))
 * 
 */
  if (!(likely(((__pyx_v_predecessor_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_predecessor_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":237
 *     _floyd_warshall(dist_matrix,
 *                     predecessor_matrix,
 *                     int(directed))             # <<<<<<<<<<<<<<
 * 
 *     if np.any(dist_matrix.diagonal() < 0):
 */
  __pyx_t_1 = PyNumber_Int(__pyx_v_directed); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":235
 *         predecessor_matrix = np.empty((0, 0), dtype=ITYPE)
 * 
 *     _floyd_warshall(dist_matrix,             # <<<<<<<<<<<<<<
 *                     predecessor_matrix,
 *                     int(directed))
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.directed = __pyx_t_6;
  __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__floyd_warshall(((PyArrayObject *)__pyx_v_dist_matrix), ((PyArrayObject *)__pyx_v_predecessor_matrix), &__pyx_t_7); 

  /* "scipy/sparse/csgraph/_shortest_path.pyx":239
 *                     int(directed))
 * 
 *     if np.any(dist_matrix.diagonal() < 0):             # <<<<<<<<<<<<<<
 *         raise NegativeCycleError("Negative cycle in nodes %s"
 *                                  % np.where(dist_matrix.diagonal() < 0)[0])
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_any); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_diagonal); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (__pyx_t_8) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_2, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_2); __pyx_t_2 = NULL;
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":240
 * 
 *     if np.any(dist_matrix.diagonal() < 0):
 *         raise NegativeCycleError("Negative cycle in nodes %s"             # <<<<<<<<<<<<<<
 *                                  % np.where(dist_matrix.diagonal() < 0)[0])
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_NegativeCycleError); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":241
 *     if np.any(dist_matrix.diagonal() < 0):
 *         raise NegativeCycleError("Negative cycle in nodes %s"
 *                                  % np.where(dist_matrix.diagonal() < 0)[0])             # <<<<<<<<<<<<<<
 * 
 *     if return_predecessors:
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_where); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_diagonal); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    if (__pyx_t_10) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    } else {
      __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_9); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyObject_RichCompare(__pyx_t_5, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_9); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_8);
    } else {
      __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_9);
      __pyx_t_9 = 0;
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_10, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_8, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Negative_cycle_in_nodes_s, __pyx_t_2); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":239
 *                     int(directed))
 * 
 *     if np.any(dist_matrix.diagonal() < 0):             # <<<<<<<<<<<<<<
 *         raise NegativeCycleError("Negative cycle in nodes %s"
 *                                  % np.where(dist_matrix.diagonal() < 0)[0])
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":243
 *                                  % np.where(dist_matrix.diagonal() < 0)[0])
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         return dist_matrix, predecessor_matrix
 *     else:
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_return_predecessors); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":244
 * 
 *     if return_predecessors:
 *         return dist_matrix, predecessor_matrix             # <<<<<<<<<<<<<<
 *     else:
 *         return dist_matrix
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 244; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_dist_matrix);
    __Pyx_GIVEREF(__pyx_v_dist_matrix);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_dist_matrix);
    __Pyx_INCREF(__pyx_v_predecessor_matrix);
    __Pyx_GIVEREF(__pyx_v_predecessor_matrix);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_predecessor_matrix);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":243
 *                                  % np.where(dist_matrix.diagonal() < 0)[0])
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         return dist_matrix, predecessor_matrix
 *     else:
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":246
 *         return dist_matrix, predecessor_matrix
 *     else:
 *         return dist_matrix             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_dist_matrix);
    __pyx_r = __pyx_v_dist_matrix;
    goto __pyx_L0;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":170
 * 
 * 
 * def floyd_warshall(csgraph, directed=True,             # <<<<<<<<<<<<<<
 *                    return_predecessors=False,
 *                    unweighted=False,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.floyd_warshall", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dist_matrix);
  __Pyx_XDECREF(__pyx_v_predecessor_matrix);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":250
 * 
 * @cython.boundscheck(False)
 * cdef void _floyd_warshall(             # <<<<<<<<<<<<<<
 *                np.ndarray[DTYPE_t, ndim=2, mode='c'] dist_matrix,
 *                np.ndarray[ITYPE_t, ndim=2, mode='c'] predecessor_matrix,
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__floyd_warshall(PyArrayObject *__pyx_v_dist_matrix, PyArrayObject *__pyx_v_predecessor_matrix, struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path__floyd_warshall *__pyx_optional_args) {
  int __pyx_v_directed = ((int)0);
  int __pyx_v_N;
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_j;
  unsigned int __pyx_v_k;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_infinity;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d_ijk;
  int __pyx_v_store_predecessors;
  PyObject *__pyx_v_i_edge = NULL;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dist_matrix;
  __Pyx_Buffer __pyx_pybuffer_dist_matrix;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_predecessor_matrix;
  __Pyx_Buffer __pyx_pybuffer_predecessor_matrix;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  size_t __pyx_t_10;
  size_t __pyx_t_11;
  size_t __pyx_t_12;
  size_t __pyx_t_13;
  size_t __pyx_t_14;
  size_t __pyx_t_15;
  size_t __pyx_t_16;
  size_t __pyx_t_17;
  size_t __pyx_t_18;
  size_t __pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  size_t __pyx_t_23;
  size_t __pyx_t_24;
  int __pyx_t_25;
  size_t __pyx_t_26;
  size_t __pyx_t_27;
  size_t __pyx_t_28;
  size_t __pyx_t_29;
  size_t __pyx_t_30;
  size_t __pyx_t_31;
  size_t __pyx_t_32;
  size_t __pyx_t_33;
  size_t __pyx_t_34;
  size_t __pyx_t_35;
  size_t __pyx_t_36;
  size_t __pyx_t_37;
  size_t __pyx_t_38;
  size_t __pyx_t_39;
  size_t __pyx_t_40;
  size_t __pyx_t_41;
  size_t __pyx_t_42;
  size_t __pyx_t_43;
  size_t __pyx_t_44;
  size_t __pyx_t_45;
  size_t __pyx_t_46;
  size_t __pyx_t_47;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_floyd_warshall", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_directed = __pyx_optional_args->directed;
    }
  }
  __pyx_pybuffer_dist_matrix.pybuffer.buf = NULL;
  __pyx_pybuffer_dist_matrix.refcount = 0;
  __pyx_pybuffernd_dist_matrix.data = NULL;
  __pyx_pybuffernd_dist_matrix.rcbuffer = &__pyx_pybuffer_dist_matrix;
  __pyx_pybuffer_predecessor_matrix.pybuffer.buf = NULL;
  __pyx_pybuffer_predecessor_matrix.refcount = 0;
  __pyx_pybuffernd_predecessor_matrix.data = NULL;
  __pyx_pybuffernd_predecessor_matrix.rcbuffer = &__pyx_pybuffer_predecessor_matrix;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer, (PyObject*)__pyx_v_dist_matrix, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 250; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_dist_matrix.diminfo[0].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dist_matrix.diminfo[0].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_dist_matrix.diminfo[1].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_dist_matrix.diminfo[1].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer, (PyObject*)__pyx_v_predecessor_matrix, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 250; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_predecessor_matrix.diminfo[0].strides = __pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_predecessor_matrix.diminfo[0].shape = __pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_predecessor_matrix.diminfo[1].strides = __pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_predecessor_matrix.diminfo[1].shape = __pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer.shape[1];

  /* "scipy/sparse/csgraph/_shortest_path.pyx":261
 *     # the points are not connected.
 *     global NULL_IDX
 *     cdef int N = dist_matrix.shape[0]             # <<<<<<<<<<<<<<
 *     assert dist_matrix.shape[1] == N
 * 
 */
  __pyx_v_N = (__pyx_v_dist_matrix->dimensions[0]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":262
 *     global NULL_IDX
 *     cdef int N = dist_matrix.shape[0]
 *     assert dist_matrix.shape[1] == N             # <<<<<<<<<<<<<<
 * 
 *     cdef unsigned int i, j, k
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!(((__pyx_v_dist_matrix->dimensions[1]) == __pyx_v_N) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 262; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "scipy/sparse/csgraph/_shortest_path.pyx":266
 *     cdef unsigned int i, j, k
 * 
 *     cdef DTYPE_t infinity = np.inf             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t d_ijk
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_3 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_infinity = __pyx_t_3;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":274
 *     #   - set diagonal to zero
 *     #   - symmetrize matrix if non-directed graph is desired
 *     dist_matrix[dist_matrix == 0] = infinity             # <<<<<<<<<<<<<<
 *     dist_matrix.flat[::N + 1] = 0
 *     if not directed:
 */
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_infinity); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)__pyx_v_dist_matrix), __pyx_int_0, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_dist_matrix), __pyx_t_1, __pyx_t_2) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":275
 *     #   - symmetrize matrix if non-directed graph is desired
 *     dist_matrix[dist_matrix == 0] = infinity
 *     dist_matrix.flat[::N + 1] = 0             # <<<<<<<<<<<<<<
 *     if not directed:
 *         for i from 0 <= i < N:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dist_matrix), __pyx_n_s_flat); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_long((__pyx_v_N + 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PySlice_New(Py_None, Py_None, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_t_4, __pyx_int_0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":276
 *     dist_matrix[dist_matrix == 0] = infinity
 *     dist_matrix.flat[::N + 1] = 0
 *     if not directed:             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < N:
 *             for j from i + 1 <= j < N:
 */
  __pyx_t_5 = ((!(__pyx_v_directed != 0)) != 0);
  if (__pyx_t_5) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":277
 *     dist_matrix.flat[::N + 1] = 0
 *     if not directed:
 *         for i from 0 <= i < N:             # <<<<<<<<<<<<<<
 *             for j from i + 1 <= j < N:
 *                 if dist_matrix[j, i] <= dist_matrix[i, j]:
 */
    __pyx_t_6 = __pyx_v_N;
    for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_6; __pyx_v_i++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":278
 *     if not directed:
 *         for i from 0 <= i < N:
 *             for j from i + 1 <= j < N:             # <<<<<<<<<<<<<<
 *                 if dist_matrix[j, i] <= dist_matrix[i, j]:
 *                     dist_matrix[i, j] = dist_matrix[j, i]
 */
      __pyx_t_7 = __pyx_v_N;
      for (__pyx_v_j = (__pyx_v_i + 1); __pyx_v_j < __pyx_t_7; __pyx_v_j++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":279
 *         for i from 0 <= i < N:
 *             for j from i + 1 <= j < N:
 *                 if dist_matrix[j, i] <= dist_matrix[i, j]:             # <<<<<<<<<<<<<<
 *                     dist_matrix[i, j] = dist_matrix[j, i]
 *                 else:
 */
        __pyx_t_8 = __pyx_v_j;
        __pyx_t_9 = __pyx_v_i;
        __pyx_t_10 = __pyx_v_i;
        __pyx_t_11 = __pyx_v_j;
        __pyx_t_5 = (((*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_8, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_9, __pyx_pybuffernd_dist_matrix.diminfo[1].strides)) <= (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_10, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_11, __pyx_pybuffernd_dist_matrix.diminfo[1].strides))) != 0);
        if (__pyx_t_5) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":280
 *             for j from i + 1 <= j < N:
 *                 if dist_matrix[j, i] <= dist_matrix[i, j]:
 *                     dist_matrix[i, j] = dist_matrix[j, i]             # <<<<<<<<<<<<<<
 *                 else:
 *                     dist_matrix[j, i] = dist_matrix[i, j]
 */
          __pyx_t_12 = __pyx_v_j;
          __pyx_t_13 = __pyx_v_i;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = __pyx_v_j;
          *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_14, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_15, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_12, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_13, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":279
 *         for i from 0 <= i < N:
 *             for j from i + 1 <= j < N:
 *                 if dist_matrix[j, i] <= dist_matrix[i, j]:             # <<<<<<<<<<<<<<
 *                     dist_matrix[i, j] = dist_matrix[j, i]
 *                 else:
 */
          goto __pyx_L8;
        }

        /* "scipy/sparse/csgraph/_shortest_path.pyx":282
 *                     dist_matrix[i, j] = dist_matrix[j, i]
 *                 else:
 *                     dist_matrix[j, i] = dist_matrix[i, j]             # <<<<<<<<<<<<<<
 * 
 *     #----------------------------------------------------------------------
 */
        /*else*/ {
          __pyx_t_16 = __pyx_v_i;
          __pyx_t_17 = __pyx_v_j;
          __pyx_t_18 = __pyx_v_j;
          __pyx_t_19 = __pyx_v_i;
          *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_18, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_19, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_16, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_17, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));
        }
        __pyx_L8:;
      }
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":276
 *     dist_matrix[dist_matrix == 0] = infinity
 *     dist_matrix.flat[::N + 1] = 0
 *     if not directed:             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < N:
 *             for j from i + 1 <= j < N:
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":289
 *     #   - initialize diagonal and all non-edges to NULL
 *     #   - initialize all edges to the row index
 *     cdef int store_predecessors = False             # <<<<<<<<<<<<<<
 * 
 *     if predecessor_matrix.size > 0:
 */
  __pyx_v_store_predecessors = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":291
 *     cdef int store_predecessors = False
 * 
 *     if predecessor_matrix.size > 0:             # <<<<<<<<<<<<<<
 *         store_predecessors = True
 *         assert predecessor_matrix.shape[0] == N
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_predecessor_matrix), __pyx_n_s_size); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_4, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":292
 * 
 *     if predecessor_matrix.size > 0:
 *         store_predecessors = True             # <<<<<<<<<<<<<<
 *         assert predecessor_matrix.shape[0] == N
 *         assert predecessor_matrix.shape[1] == N
 */
    __pyx_v_store_predecessors = 1;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":293
 *     if predecessor_matrix.size > 0:
 *         store_predecessors = True
 *         assert predecessor_matrix.shape[0] == N             # <<<<<<<<<<<<<<
 *         assert predecessor_matrix.shape[1] == N
 *         predecessor_matrix.fill(NULL_IDX)
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!(((__pyx_v_predecessor_matrix->dimensions[0]) == __pyx_v_N) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    #endif

    /* "scipy/sparse/csgraph/_shortest_path.pyx":294
 *         store_predecessors = True
 *         assert predecessor_matrix.shape[0] == N
 *         assert predecessor_matrix.shape[1] == N             # <<<<<<<<<<<<<<
 *         predecessor_matrix.fill(NULL_IDX)
 *         i_edge = np.where(~np.isinf(dist_matrix))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!(((__pyx_v_predecessor_matrix->dimensions[1]) == __pyx_v_N) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    #endif

    /* "scipy/sparse/csgraph/_shortest_path.pyx":295
 *         assert predecessor_matrix.shape[0] == N
 *         assert predecessor_matrix.shape[1] == N
 *         predecessor_matrix.fill(NULL_IDX)             # <<<<<<<<<<<<<<
 *         i_edge = np.where(~np.isinf(dist_matrix))
 *         predecessor_matrix[i_edge] = i_edge[0]
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_predecessor_matrix), __pyx_n_s_fill); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyInt_From_npy_int32(__pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_NULL_IDX); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_20 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_20 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_20)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_20);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_20) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_21 = PyTuple_New(1+1); if (unlikely(!__pyx_t_21)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_21);
      __Pyx_GIVEREF(__pyx_t_20); PyTuple_SET_ITEM(__pyx_t_21, 0, __pyx_t_20); __pyx_t_20 = NULL;
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_21, 0+1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_21, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":296
 *         assert predecessor_matrix.shape[1] == N
 *         predecessor_matrix.fill(NULL_IDX)
 *         i_edge = np.where(~np.isinf(dist_matrix))             # <<<<<<<<<<<<<<
 *         predecessor_matrix[i_edge] = i_edge[0]
 *         predecessor_matrix.flat[::N + 1] = NULL_IDX
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_21 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_where); if (unlikely(!__pyx_t_21)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_21);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_isinf); if (unlikely(!__pyx_t_20)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_20);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_20))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_20);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_20);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_20, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_20, ((PyObject *)__pyx_v_dist_matrix)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_22 = PyTuple_New(1+1); if (unlikely(!__pyx_t_22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_22);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_dist_matrix));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_dist_matrix));
      PyTuple_SET_ITEM(__pyx_t_22, 0+1, ((PyObject *)__pyx_v_dist_matrix));
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_20, __pyx_t_22, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
    }
    __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    __pyx_t_20 = PyNumber_Invert(__pyx_t_4); if (unlikely(!__pyx_t_20)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_20);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_21))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_21);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_21);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_21, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_21, __pyx_t_20); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_22 = PyTuple_New(1+1); if (unlikely(!__pyx_t_22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_22);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_20);
      PyTuple_SET_ITEM(__pyx_t_22, 0+1, __pyx_t_20);
      __pyx_t_20 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_21, __pyx_t_22, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
    }
    __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
    __pyx_v_i_edge = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":297
 *         predecessor_matrix.fill(NULL_IDX)
 *         i_edge = np.where(~np.isinf(dist_matrix))
 *         predecessor_matrix[i_edge] = i_edge[0]             # <<<<<<<<<<<<<<
 *         predecessor_matrix.flat[::N + 1] = NULL_IDX
 * 
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_i_edge, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 297; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_predecessor_matrix), __pyx_v_i_edge, __pyx_t_2) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 297; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":298
 *         i_edge = np.where(~np.isinf(dist_matrix))
 *         predecessor_matrix[i_edge] = i_edge[0]
 *         predecessor_matrix.flat[::N + 1] = NULL_IDX             # <<<<<<<<<<<<<<
 * 
 *     # Now perform the Floyd-Warshall algorithm.
 */
    __pyx_t_2 = __Pyx_PyInt_From_npy_int32(__pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_NULL_IDX); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_21 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_predecessor_matrix), __pyx_n_s_flat); if (unlikely(!__pyx_t_21)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_21);
    __pyx_t_22 = __Pyx_PyInt_From_long((__pyx_v_N + 1)); if (unlikely(!__pyx_t_22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_22);
    __pyx_t_20 = PySlice_New(Py_None, Py_None, __pyx_t_22); if (unlikely(!__pyx_t_20)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_20);
    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
    if (unlikely(PyObject_SetItem(__pyx_t_21, __pyx_t_20, __pyx_t_2) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
    __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":291
 *     cdef int store_predecessors = False
 * 
 *     if predecessor_matrix.size > 0:             # <<<<<<<<<<<<<<
 *         store_predecessors = True
 *         assert predecessor_matrix.shape[0] == N
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":303
 *     # In each loop, this finds the shortest path from point i
 *     #  to point j using intermediate nodes 0 ... k
 *     if store_predecessors:             # <<<<<<<<<<<<<<
 *         for k from 0 <= k < N:
 *             for i from 0 <= i < N:
 */
  __pyx_t_5 = (__pyx_v_store_predecessors != 0);
  if (__pyx_t_5) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":304
 *     #  to point j using intermediate nodes 0 ... k
 *     if store_predecessors:
 *         for k from 0 <= k < N:             # <<<<<<<<<<<<<<
 *             for i from 0 <= i < N:
 *                 if dist_matrix[i, k] == infinity:
 */
    __pyx_t_6 = __pyx_v_N;
    for (__pyx_v_k = 0; __pyx_v_k < __pyx_t_6; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":305
 *     if store_predecessors:
 *         for k from 0 <= k < N:
 *             for i from 0 <= i < N:             # <<<<<<<<<<<<<<
 *                 if dist_matrix[i, k] == infinity:
 *                     continue
 */
      __pyx_t_7 = __pyx_v_N;
      for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_7; __pyx_v_i++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":306
 *         for k from 0 <= k < N:
 *             for i from 0 <= i < N:
 *                 if dist_matrix[i, k] == infinity:             # <<<<<<<<<<<<<<
 *                     continue
 *                 for j from 0 <= j < N:
 */
        __pyx_t_23 = __pyx_v_i;
        __pyx_t_24 = __pyx_v_k;
        __pyx_t_5 = (((*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_23, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_24, __pyx_pybuffernd_dist_matrix.diminfo[1].strides)) == __pyx_v_infinity) != 0);
        if (__pyx_t_5) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":307
 *             for i from 0 <= i < N:
 *                 if dist_matrix[i, k] == infinity:
 *                     continue             # <<<<<<<<<<<<<<
 *                 for j from 0 <= j < N:
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 */
          goto __pyx_L13_continue;

          /* "scipy/sparse/csgraph/_shortest_path.pyx":306
 *         for k from 0 <= k < N:
 *             for i from 0 <= i < N:
 *                 if dist_matrix[i, k] == infinity:             # <<<<<<<<<<<<<<
 *                     continue
 *                 for j from 0 <= j < N:
 */
        }

        /* "scipy/sparse/csgraph/_shortest_path.pyx":308
 *                 if dist_matrix[i, k] == infinity:
 *                     continue
 *                 for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 *                     if d_ijk < dist_matrix[i, j]:
 */
        __pyx_t_25 = __pyx_v_N;
        for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_25; __pyx_v_j++) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":309
 *                     continue
 *                 for j from 0 <= j < N:
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]             # <<<<<<<<<<<<<<
 *                     if d_ijk < dist_matrix[i, j]:
 *                         dist_matrix[i, j] = d_ijk
 */
          __pyx_t_26 = __pyx_v_i;
          __pyx_t_27 = __pyx_v_k;
          __pyx_t_28 = __pyx_v_k;
          __pyx_t_29 = __pyx_v_j;
          __pyx_v_d_ijk = ((*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_26, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_27, __pyx_pybuffernd_dist_matrix.diminfo[1].strides)) + (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_28, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_29, __pyx_pybuffernd_dist_matrix.diminfo[1].strides)));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":310
 *                 for j from 0 <= j < N:
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 *                     if d_ijk < dist_matrix[i, j]:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, j] = d_ijk
 *                         predecessor_matrix[i, j] = predecessor_matrix[k, j]
 */
          __pyx_t_30 = __pyx_v_i;
          __pyx_t_31 = __pyx_v_j;
          __pyx_t_5 = ((__pyx_v_d_ijk < (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_30, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_31, __pyx_pybuffernd_dist_matrix.diminfo[1].strides))) != 0);
          if (__pyx_t_5) {

            /* "scipy/sparse/csgraph/_shortest_path.pyx":311
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 *                     if d_ijk < dist_matrix[i, j]:
 *                         dist_matrix[i, j] = d_ijk             # <<<<<<<<<<<<<<
 *                         predecessor_matrix[i, j] = predecessor_matrix[k, j]
 *     else:
 */
            __pyx_t_32 = __pyx_v_i;
            __pyx_t_33 = __pyx_v_j;
            *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_32, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_33, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = __pyx_v_d_ijk;

            /* "scipy/sparse/csgraph/_shortest_path.pyx":312
 *                     if d_ijk < dist_matrix[i, j]:
 *                         dist_matrix[i, j] = d_ijk
 *                         predecessor_matrix[i, j] = predecessor_matrix[k, j]             # <<<<<<<<<<<<<<
 *     else:
 *         for k from 0 <= k < N:
 */
            __pyx_t_34 = __pyx_v_k;
            __pyx_t_35 = __pyx_v_j;
            __pyx_t_36 = __pyx_v_i;
            __pyx_t_37 = __pyx_v_j;
            *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer.buf, __pyx_t_36, __pyx_pybuffernd_predecessor_matrix.diminfo[0].strides, __pyx_t_37, __pyx_pybuffernd_predecessor_matrix.diminfo[1].strides) = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer.buf, __pyx_t_34, __pyx_pybuffernd_predecessor_matrix.diminfo[0].strides, __pyx_t_35, __pyx_pybuffernd_predecessor_matrix.diminfo[1].strides));

            /* "scipy/sparse/csgraph/_shortest_path.pyx":310
 *                 for j from 0 <= j < N:
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 *                     if d_ijk < dist_matrix[i, j]:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, j] = d_ijk
 *                         predecessor_matrix[i, j] = predecessor_matrix[k, j]
 */
          }
        }
        __pyx_L13_continue:;
      }
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":303
 *     # In each loop, this finds the shortest path from point i
 *     #  to point j using intermediate nodes 0 ... k
 *     if store_predecessors:             # <<<<<<<<<<<<<<
 *         for k from 0 <= k < N:
 *             for i from 0 <= i < N:
 */
    goto __pyx_L10;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":314
 *                         predecessor_matrix[i, j] = predecessor_matrix[k, j]
 *     else:
 *         for k from 0 <= k < N:             # <<<<<<<<<<<<<<
 *             for i from 0 <= i < N:
 *                 if dist_matrix[i, k] == infinity:
 */
  /*else*/ {
    __pyx_t_6 = __pyx_v_N;
    for (__pyx_v_k = 0; __pyx_v_k < __pyx_t_6; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":315
 *     else:
 *         for k from 0 <= k < N:
 *             for i from 0 <= i < N:             # <<<<<<<<<<<<<<
 *                 if dist_matrix[i, k] == infinity:
 *                     continue
 */
      __pyx_t_7 = __pyx_v_N;
      for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_7; __pyx_v_i++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":316
 *         for k from 0 <= k < N:
 *             for i from 0 <= i < N:
 *                 if dist_matrix[i, k] == infinity:             # <<<<<<<<<<<<<<
 *                     continue
 *                 for j from 0 <= j < N:
 */
        __pyx_t_38 = __pyx_v_i;
        __pyx_t_39 = __pyx_v_k;
        __pyx_t_5 = (((*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_38, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_39, __pyx_pybuffernd_dist_matrix.diminfo[1].strides)) == __pyx_v_infinity) != 0);
        if (__pyx_t_5) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":317
 *             for i from 0 <= i < N:
 *                 if dist_matrix[i, k] == infinity:
 *                     continue             # <<<<<<<<<<<<<<
 *                 for j from 0 <= j < N:
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 */
          goto __pyx_L21_continue;

          /* "scipy/sparse/csgraph/_shortest_path.pyx":316
 *         for k from 0 <= k < N:
 *             for i from 0 <= i < N:
 *                 if dist_matrix[i, k] == infinity:             # <<<<<<<<<<<<<<
 *                     continue
 *                 for j from 0 <= j < N:
 */
        }

        /* "scipy/sparse/csgraph/_shortest_path.pyx":318
 *                 if dist_matrix[i, k] == infinity:
 *                     continue
 *                 for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 *                     if d_ijk < dist_matrix[i, j]:
 */
        __pyx_t_25 = __pyx_v_N;
        for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_25; __pyx_v_j++) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":319
 *                     continue
 *                 for j from 0 <= j < N:
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]             # <<<<<<<<<<<<<<
 *                     if d_ijk < dist_matrix[i, j]:
 *                         dist_matrix[i, j] = d_ijk
 */
          __pyx_t_40 = __pyx_v_i;
          __pyx_t_41 = __pyx_v_k;
          __pyx_t_42 = __pyx_v_k;
          __pyx_t_43 = __pyx_v_j;
          __pyx_v_d_ijk = ((*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_40, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_41, __pyx_pybuffernd_dist_matrix.diminfo[1].strides)) + (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_42, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_43, __pyx_pybuffernd_dist_matrix.diminfo[1].strides)));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":320
 *                 for j from 0 <= j < N:
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 *                     if d_ijk < dist_matrix[i, j]:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, j] = d_ijk
 * 
 */
          __pyx_t_44 = __pyx_v_i;
          __pyx_t_45 = __pyx_v_j;
          __pyx_t_5 = ((__pyx_v_d_ijk < (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_44, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_45, __pyx_pybuffernd_dist_matrix.diminfo[1].strides))) != 0);
          if (__pyx_t_5) {

            /* "scipy/sparse/csgraph/_shortest_path.pyx":321
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 *                     if d_ijk < dist_matrix[i, j]:
 *                         dist_matrix[i, j] = d_ijk             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_46 = __pyx_v_i;
            __pyx_t_47 = __pyx_v_j;
            *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_46, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_47, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = __pyx_v_d_ijk;

            /* "scipy/sparse/csgraph/_shortest_path.pyx":320
 *                 for j from 0 <= j < N:
 *                     d_ijk = dist_matrix[i, k] + dist_matrix[k, j]
 *                     if d_ijk < dist_matrix[i, j]:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, j] = d_ijk
 * 
 */
          }
        }
        __pyx_L21_continue:;
      }
    }
  }
  __pyx_L10:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":250
 * 
 * @cython.boundscheck(False)
 * cdef void _floyd_warshall(             # <<<<<<<<<<<<<<
 *                np.ndarray[DTYPE_t, ndim=2, mode='c'] dist_matrix,
 *                np.ndarray[ITYPE_t, ndim=2, mode='c'] predecessor_matrix,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_WriteUnraisable("scipy.sparse.csgraph._shortest_path._floyd_warshall", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_predecessor_matrix.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_i_edge);
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":325
 * 
 * 
 * def dijkstra(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *              return_predecessors=False,
 *              unweighted=False, limit=np.inf):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_5dijkstra(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_4dijkstra[] = "\n    dijkstra(csgraph, directed=True, indices=None, return_predecessors=False,\n             unweighted=False)\n\n    Dijkstra algorithm using Fibonacci Heaps\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of non-negative distances representing the input graph.\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    indices : array_like or int, optional\n        if specified, only compute the paths for the points at the given\n        indices.\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n    limit : float, optional\n        The maximum distance to calculate, must be >= 0. Using a smaller limit\n        will decrease computation time by aborting calculations between pairs\n        that are separated by a distance > limit. For such pairs, the distance\n        will be equal to np.inf (i.e., not connected).\n        .. versionadded:: 0.14.0\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains""\n        information on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Notes\n    -----\n    As currently implemented, Dijkstra's algorithm does not work for\n    graphs with direction-dependent distances when directed == False.\n    i.e., if csgraph[i,j] and csgraph[j,i] are not equal and\n    both are nonzero, setting directed=False will not yield the correct\n    result.\n\n    Also, this routine does not work for graphs with negative\n    distances.  Negative distances can lead to infinite cycles that must\n    be handled by specialized algorithms such as Bellman-Ford's algorithm\n    or Johnson's algorithm.\n    ";
static PyMethodDef __pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_5dijkstra = {"dijkstra", (PyCFunction)__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_5dijkstra, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_4dijkstra};
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_5dijkstra(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_csgraph = 0;
  PyObject *__pyx_v_directed = 0;
  PyObject *__pyx_v_indices = 0;
  PyObject *__pyx_v_return_predecessors = 0;
  PyObject *__pyx_v_unweighted = 0;
  PyObject *__pyx_v_limit = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dijkstra (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_csgraph,&__pyx_n_s_directed,&__pyx_n_s_indices,&__pyx_n_s_return_predecessors,&__pyx_n_s_unweighted,&__pyx_n_s_limit,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    values[1] = ((PyObject *)Py_True);
    values[2] = ((PyObject *)Py_None);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":326
 * 
 * def dijkstra(csgraph, directed=True, indices=None,
 *              return_predecessors=False,             # <<<<<<<<<<<<<<
 *              unweighted=False, limit=np.inf):
 *     """
 */
    values[3] = ((PyObject *)Py_False);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":327
 * def dijkstra(csgraph, directed=True, indices=None,
 *              return_predecessors=False,
 *              unweighted=False, limit=np.inf):             # <<<<<<<<<<<<<<
 *     """
 *     dijkstra(csgraph, directed=True, indices=None, return_predecessors=False,
 */
    values[4] = ((PyObject *)Py_False);
    values[5] = __pyx_k__4;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_csgraph)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_directed);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_indices);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_predecessors);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_unweighted);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_limit);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "dijkstra") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_csgraph = values[0];
    __pyx_v_directed = values[1];
    __pyx_v_indices = values[2];
    __pyx_v_return_predecessors = values[3];
    __pyx_v_unweighted = values[4];
    __pyx_v_limit = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("dijkstra", 0, 1, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.dijkstra", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_4dijkstra(__pyx_self, __pyx_v_csgraph, __pyx_v_directed, __pyx_v_indices, __pyx_v_return_predecessors, __pyx_v_unweighted, __pyx_v_limit);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":325
 * 
 * 
 * def dijkstra(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *              return_predecessors=False,
 *              unweighted=False, limit=np.inf):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_4dijkstra(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_directed, PyObject *__pyx_v_indices, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted, PyObject *__pyx_v_limit) {
  PyObject *__pyx_v_N = NULL;
  PyObject *__pyx_v_return_shape = NULL;
  PyObject *__pyx_v_dist_matrix = NULL;
  PyObject *__pyx_v_predecessor_matrix = NULL;
  PyObject *__pyx_v_csr_data = NULL;
  PyObject *__pyx_v_csgraphT = NULL;
  PyObject *__pyx_v_csrT_data = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  double __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("dijkstra", 0);
  __Pyx_INCREF(__pyx_v_csgraph);
  __Pyx_INCREF(__pyx_v_indices);
  __Pyx_INCREF(__pyx_v_limit);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":394
 *     #------------------------------
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                              dense_output=False)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_validate_graph); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_csgraph);
  __Pyx_GIVEREF(__pyx_v_csgraph);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_csgraph);
  __Pyx_INCREF(__pyx_v_directed);
  __Pyx_GIVEREF(__pyx_v_directed);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_directed);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":395
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,
 *                              dense_output=False)             # <<<<<<<<<<<<<<
 * 
 *     if np.any(csgraph.data < 0):
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dense_output, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":394
 *     #------------------------------
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                              dense_output=False)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_csgraph, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":397
 *                              dense_output=False)
 * 
 *     if np.any(csgraph.data < 0):             # <<<<<<<<<<<<<<
 *         warnings.warn("Graph has negative weights: dijkstra will give "
 *                       "inaccurate results if the graph contains negative "
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_any); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_data); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":398
 * 
 *     if np.any(csgraph.data < 0):
 *         warnings.warn("Graph has negative weights: dijkstra will give "             # <<<<<<<<<<<<<<
 *                       "inaccurate results if the graph contains negative "
 *                       "cycles. Consider johnson or bellman_ford.")
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_warnings); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_warn); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":397
 *                              dense_output=False)
 * 
 *     if np.any(csgraph.data < 0):             # <<<<<<<<<<<<<<
 *         warnings.warn("Graph has negative weights: dijkstra will give "
 *                       "inaccurate results if the graph contains negative "
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":402
 *                       "cycles. Consider johnson or bellman_ford.")
 * 
 *     N = csgraph.shape[0]             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_N = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":406
 *     #------------------------------
 *     # intitialize/validate indices
 *     if indices is None:             # <<<<<<<<<<<<<<
 *         indices = np.arange(N, dtype=ITYPE)
 *         return_shape = indices.shape + (N,)
 */
  __pyx_t_6 = (__pyx_v_indices == Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":407
 *     # intitialize/validate indices
 *     if indices is None:
 *         indices = np.arange(N, dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         return_shape = indices.shape + (N,)
 *     else:
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_arange); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_N);
    __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":408
 *     if indices is None:
 *         indices = np.arange(N, dtype=ITYPE)
 *         return_shape = indices.shape + (N,)             # <<<<<<<<<<<<<<
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE, copy=True)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 408; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 408; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_N);
    __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 408; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_return_shape = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":406
 *     #------------------------------
 *     # intitialize/validate indices
 *     if indices is None:             # <<<<<<<<<<<<<<
 *         indices = np.arange(N, dtype=ITYPE)
 *         return_shape = indices.shape + (N,)
 */
    goto __pyx_L4;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":410
 *         return_shape = indices.shape + (N,)
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE, copy=True)             # <<<<<<<<<<<<<<
 *         return_shape = indices.shape + (N,)
 *         indices = np.atleast_1d(indices).reshape(-1)
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_indices);
    __Pyx_GIVEREF(__pyx_v_indices);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_indices);
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_copy, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":411
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE, copy=True)
 *         return_shape = indices.shape + (N,)             # <<<<<<<<<<<<<<
 *         indices = np.atleast_1d(indices).reshape(-1)
 *         indices[indices < 0] += N
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_N);
    __pyx_t_3 = PyNumber_Add(__pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_return_shape = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":412
 *         indices = np.array(indices, order='C', dtype=ITYPE, copy=True)
 *         return_shape = indices.shape + (N,)
 *         indices = np.atleast_1d(indices).reshape(-1)             # <<<<<<<<<<<<<<
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_atleast_1d); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_indices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_INCREF(__pyx_v_indices);
      __Pyx_GIVEREF(__pyx_v_indices);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_indices);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":413
 *         return_shape = indices.shape + (N,)
 *         indices = np.atleast_1d(indices).reshape(-1)
 *         indices[indices < 0] += N             # <<<<<<<<<<<<<<
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_indices, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = PyObject_GetItem(__pyx_v_indices, __pyx_t_3); if (unlikely(__pyx_t_4 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 413; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_t_4, __pyx_v_N); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_indices, __pyx_t_3, __pyx_t_5) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":414
 *         indices = np.atleast_1d(indices).reshape(-1)
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):             # <<<<<<<<<<<<<<
 *             raise ValueError("indices out of range 0...N")
 * 
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_any); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyObject_RichCompare(__pyx_v_indices, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_any); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_indices, __pyx_v_N, Py_GE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = __pyx_t_6;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_7) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":415
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")             # <<<<<<<<<<<<<<
 * 
 *     if not np.isscalar(limit):
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "scipy/sparse/csgraph/_shortest_path.pyx":414
 *         indices = np.atleast_1d(indices).reshape(-1)
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):             # <<<<<<<<<<<<<<
 *             raise ValueError("indices out of range 0...N")
 * 
 */
    }
  }
  __pyx_L4:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":417
 *             raise ValueError("indices out of range 0...N")
 * 
 *     if not np.isscalar(limit):             # <<<<<<<<<<<<<<
 *         raise TypeError('limit must be numeric (float)')
 *     limit = float(limit)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_isscalar); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_limit); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
    __Pyx_INCREF(__pyx_v_limit);
    __Pyx_GIVEREF(__pyx_v_limit);
    PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_limit);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = ((!__pyx_t_7) != 0);
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":418
 * 
 *     if not np.isscalar(limit):
 *         raise TypeError('limit must be numeric (float)')             # <<<<<<<<<<<<<<
 *     limit = float(limit)
 *     if limit < 0:
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":417
 *             raise ValueError("indices out of range 0...N")
 * 
 *     if not np.isscalar(limit):             # <<<<<<<<<<<<<<
 *         raise TypeError('limit must be numeric (float)')
 *     limit = float(limit)
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":419
 *     if not np.isscalar(limit):
 *         raise TypeError('limit must be numeric (float)')
 *     limit = float(limit)             # <<<<<<<<<<<<<<
 *     if limit < 0:
 *         raise ValueError('limit must be >= 0')
 */
  __pyx_t_8 = __Pyx_PyObject_AsDouble(__pyx_v_limit); if (unlikely(__pyx_t_8 == ((double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = PyFloat_FromDouble(__pyx_t_8); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF_SET(__pyx_v_limit, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":420
 *         raise TypeError('limit must be numeric (float)')
 *     limit = float(limit)
 *     if limit < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError('limit must be >= 0')
 * 
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_limit, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":421
 *     limit = float(limit)
 *     if limit < 0:
 *         raise ValueError('limit must be >= 0')             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":420
 *         raise TypeError('limit must be numeric (float)')
 *     limit = float(limit)
 *     if limit < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError('limit must be >= 0')
 * 
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":425
 *     #------------------------------
 *     # initialize dist_matrix for output
 *     dist_matrix = np.zeros((len(indices), N), dtype=DTYPE)             # <<<<<<<<<<<<<<
 *     dist_matrix.fill(np.inf)
 *     dist_matrix[np.arange(len(indices)), indices] = 0
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_9 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_N);
  __Pyx_GIVEREF(__pyx_v_N);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_N);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_dist_matrix = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":426
 *     # initialize dist_matrix for output
 *     dist_matrix = np.zeros((len(indices), N), dtype=DTYPE)
 *     dist_matrix.fill(np.inf)             # <<<<<<<<<<<<<<
 *     dist_matrix[np.arange(len(indices)), indices] = 0
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_fill); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_inf); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":427
 *     dist_matrix = np.zeros((len(indices), N), dtype=DTYPE)
 *     dist_matrix.fill(np.inf)
 *     dist_matrix[np.arange(len(indices)), indices] = 0             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_arange); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_9 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1); __pyx_t_1 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_indices);
  __Pyx_GIVEREF(__pyx_v_indices);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_indices);
  __pyx_t_2 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_dist_matrix, __pyx_t_5, __pyx_int_0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":431
 *     #------------------------------
 *     # initialize predecessors for output
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_return_predecessors); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 431; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":432
 *     # initialize predecessors for output
 *     if return_predecessors:
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         predecessor_matrix.fill(NULL_IDX)
 *     else:
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_N);
    __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_predecessor_matrix = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":433
 *     if return_predecessors:
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)             # <<<<<<<<<<<<<<
 *     else:
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_predecessor_matrix, __pyx_n_s_fill); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyInt_From_npy_int32(__pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_NULL_IDX); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":431
 *     #------------------------------
 *     # initialize predecessors for output
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)
 */
    goto __pyx_L10;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":435
 *         predecessor_matrix.fill(NULL_IDX)
 *     else:
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)             # <<<<<<<<<<<<<<
 * 
 *     if unweighted:
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_GIVEREF(__pyx_int_0);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_int_0);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_N);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_predecessor_matrix = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __pyx_L10:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":437
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)
 * 
 *     if unweighted:             # <<<<<<<<<<<<<<
 *         csr_data = np.ones(csgraph.data.shape)
 *     else:
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_unweighted); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":438
 * 
 *     if unweighted:
 *         csr_data = np.ones(csgraph.data.shape)             # <<<<<<<<<<<<<<
 *     else:
 *         csr_data = csgraph.data
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_ones); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_data); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_csr_data = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":437
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)
 * 
 *     if unweighted:             # <<<<<<<<<<<<<<
 *         csr_data = np.ones(csgraph.data.shape)
 *     else:
 */
    goto __pyx_L11;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":440
 *         csr_data = np.ones(csgraph.data.shape)
 *     else:
 *         csr_data = csgraph.data             # <<<<<<<<<<<<<<
 * 
 *     if directed:
 */
  /*else*/ {
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_data); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_csr_data = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __pyx_L11:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":442
 *         csr_data = csgraph.data
 * 
 *     if directed:             # <<<<<<<<<<<<<<
 *         _dijkstra_directed(indices,
 *                            csr_data, csgraph.indices, csgraph.indptr,
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_directed); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":443
 * 
 *     if directed:
 *         _dijkstra_directed(indices,             # <<<<<<<<<<<<<<
 *                            csr_data, csgraph.indices, csgraph.indptr,
 *                            dist_matrix, predecessor_matrix, limit)
 */
    if (!(likely(((__pyx_v_indices) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_indices, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":444
 *     if directed:
 *         _dijkstra_directed(indices,
 *                            csr_data, csgraph.indices, csgraph.indptr,             # <<<<<<<<<<<<<<
 *                            dist_matrix, predecessor_matrix, limit)
 *     else:
 */
    if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 444; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 444; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 444; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 444; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 444; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":445
 *         _dijkstra_directed(indices,
 *                            csr_data, csgraph.indices, csgraph.indptr,
 *                            dist_matrix, predecessor_matrix, limit)             # <<<<<<<<<<<<<<
 *     else:
 *         csgraphT = csgraph.T.tocsr()
 */
    if (!(likely(((__pyx_v_dist_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_predecessor_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_predecessor_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_v_limit); if (unlikely((__pyx_t_10 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":443
 * 
 *     if directed:
 *         _dijkstra_directed(indices,             # <<<<<<<<<<<<<<
 *                            csr_data, csgraph.indices, csgraph.indptr,
 *                            dist_matrix, predecessor_matrix, limit)
 */
    __pyx_t_2 = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__dijkstra_directed(((PyArrayObject *)__pyx_v_indices), ((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_5), ((PyArrayObject *)__pyx_t_1), ((PyArrayObject *)__pyx_v_dist_matrix), ((PyArrayObject *)__pyx_v_predecessor_matrix), __pyx_t_10); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":442
 *         csr_data = csgraph.data
 * 
 *     if directed:             # <<<<<<<<<<<<<<
 *         _dijkstra_directed(indices,
 *                            csr_data, csgraph.indices, csgraph.indptr,
 */
    goto __pyx_L12;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":447
 *                            dist_matrix, predecessor_matrix, limit)
 *     else:
 *         csgraphT = csgraph.T.tocsr()             # <<<<<<<<<<<<<<
 *         if unweighted:
 *             csrT_data = csr_data
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_T); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_tocsr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (__pyx_t_1) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_csgraphT = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":448
 *     else:
 *         csgraphT = csgraph.T.tocsr()
 *         if unweighted:             # <<<<<<<<<<<<<<
 *             csrT_data = csr_data
 *         else:
 */
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_unweighted); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 448; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_6) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":449
 *         csgraphT = csgraph.T.tocsr()
 *         if unweighted:
 *             csrT_data = csr_data             # <<<<<<<<<<<<<<
 *         else:
 *             csrT_data = csgraphT.data
 */
      __Pyx_INCREF(__pyx_v_csr_data);
      __pyx_v_csrT_data = __pyx_v_csr_data;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":448
 *     else:
 *         csgraphT = csgraph.T.tocsr()
 *         if unweighted:             # <<<<<<<<<<<<<<
 *             csrT_data = csr_data
 *         else:
 */
      goto __pyx_L13;
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":451
 *             csrT_data = csr_data
 *         else:
 *             csrT_data = csgraphT.data             # <<<<<<<<<<<<<<
 *         _dijkstra_undirected(indices,
 *                              csr_data, csgraph.indices, csgraph.indptr,
 */
    /*else*/ {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_data); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 451; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_v_csrT_data = __pyx_t_2;
      __pyx_t_2 = 0;
    }
    __pyx_L13:;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":452
 *         else:
 *             csrT_data = csgraphT.data
 *         _dijkstra_undirected(indices,             # <<<<<<<<<<<<<<
 *                              csr_data, csgraph.indices, csgraph.indptr,
 *                              csrT_data, csgraphT.indices, csgraphT.indptr,
 */
    if (!(likely(((__pyx_v_indices) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_indices, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":453
 *             csrT_data = csgraphT.data
 *         _dijkstra_undirected(indices,
 *                              csr_data, csgraph.indices, csgraph.indptr,             # <<<<<<<<<<<<<<
 *                              csrT_data, csgraphT.indices, csgraphT.indptr,
 *                              dist_matrix, predecessor_matrix, limit)
 */
    if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":454
 *         _dijkstra_undirected(indices,
 *                              csr_data, csgraph.indices, csgraph.indptr,
 *                              csrT_data, csgraphT.indices, csgraphT.indptr,             # <<<<<<<<<<<<<<
 *                              dist_matrix, predecessor_matrix, limit)
 * 
 */
    if (!(likely(((__pyx_v_csrT_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csrT_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 454; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_indices); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 454; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 454; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_indptr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 454; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 454; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":455
 *                              csr_data, csgraph.indices, csgraph.indptr,
 *                              csrT_data, csgraphT.indices, csgraphT.indptr,
 *                              dist_matrix, predecessor_matrix, limit)             # <<<<<<<<<<<<<<
 * 
 *     if return_predecessors:
 */
    if (!(likely(((__pyx_v_dist_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_predecessor_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_predecessor_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_v_limit); if (unlikely((__pyx_t_10 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":452
 *         else:
 *             csrT_data = csgraphT.data
 *         _dijkstra_undirected(indices,             # <<<<<<<<<<<<<<
 *                              csr_data, csgraph.indices, csgraph.indptr,
 *                              csrT_data, csgraphT.indices, csgraphT.indptr,
 */
    __pyx_t_4 = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__dijkstra_undirected(((PyArrayObject *)__pyx_v_indices), ((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_2), ((PyArrayObject *)__pyx_t_5), ((PyArrayObject *)__pyx_v_csrT_data), ((PyArrayObject *)__pyx_t_1), ((PyArrayObject *)__pyx_t_3), ((PyArrayObject *)__pyx_v_dist_matrix), ((PyArrayObject *)__pyx_v_predecessor_matrix), __pyx_t_10); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_L12:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":457
 *                              dist_matrix, predecessor_matrix, limit)
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_return_predecessors); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":458
 * 
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),             # <<<<<<<<<<<<<<
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_return_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_return_shape);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":459
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))             # <<<<<<<<<<<<<<
 *     else:
 *         return dist_matrix.reshape(return_shape)
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_predecessor_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_return_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_v_return_shape);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":458
 * 
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),             # <<<<<<<<<<<<<<
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 */
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":457
 *                              dist_matrix, predecessor_matrix, limit)
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":461
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 *         return dist_matrix.reshape(return_shape)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_return_shape); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_v_return_shape);
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":325
 * 
 * 
 * def dijkstra(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *              return_predecessors=False,
 *              unweighted=False, limit=np.inf):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.dijkstra", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_N);
  __Pyx_XDECREF(__pyx_v_return_shape);
  __Pyx_XDECREF(__pyx_v_dist_matrix);
  __Pyx_XDECREF(__pyx_v_predecessor_matrix);
  __Pyx_XDECREF(__pyx_v_csr_data);
  __Pyx_XDECREF(__pyx_v_csgraphT);
  __Pyx_XDECREF(__pyx_v_csrT_data);
  __Pyx_XDECREF(__pyx_v_csgraph);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XDECREF(__pyx_v_limit);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":464
 * 
 * 
 * cdef _dijkstra_directed(             # <<<<<<<<<<<<<<
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] source_indices,
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 */

static PyObject *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__dijkstra_directed(PyArrayObject *__pyx_v_source_indices, PyArrayObject *__pyx_v_csr_weights, PyArrayObject *__pyx_v_csr_indices, PyArrayObject *__pyx_v_csr_indptr, PyArrayObject *__pyx_v_dist_matrix, PyArrayObject *__pyx_v_pred, __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_limit) {
  unsigned int __pyx_v_Nind;
  unsigned int __pyx_v_N;
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_k;
  unsigned int __pyx_v_j_source;
  unsigned int __pyx_v_j_current;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_v_j;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_next_val;
  int __pyx_v_return_pred;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap __pyx_v_heap;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_v;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_current_node;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_nodes;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indices;
  __Pyx_Buffer __pyx_pybuffer_csr_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indptr;
  __Pyx_Buffer __pyx_pybuffer_csr_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_weights;
  __Pyx_Buffer __pyx_pybuffer_csr_weights;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dist_matrix;
  __Pyx_Buffer __pyx_pybuffer_dist_matrix;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_pred;
  __Pyx_Buffer __pyx_pybuffer_pred;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_source_indices;
  __Pyx_Buffer __pyx_pybuffer_source_indices;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  unsigned int __pyx_t_4;
  size_t __pyx_t_5;
  unsigned int __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_t_9;
  size_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  size_t __pyx_t_15;
  size_t __pyx_t_16;
  size_t __pyx_t_17;
  size_t __pyx_t_18;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_t_19;
  size_t __pyx_t_20;
  size_t __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_dijkstra_directed", 0);
  __pyx_pybuffer_source_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_source_indices.refcount = 0;
  __pyx_pybuffernd_source_indices.data = NULL;
  __pyx_pybuffernd_source_indices.rcbuffer = &__pyx_pybuffer_source_indices;
  __pyx_pybuffer_csr_weights.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_weights.refcount = 0;
  __pyx_pybuffernd_csr_weights.data = NULL;
  __pyx_pybuffernd_csr_weights.rcbuffer = &__pyx_pybuffer_csr_weights;
  __pyx_pybuffer_csr_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indices.refcount = 0;
  __pyx_pybuffernd_csr_indices.data = NULL;
  __pyx_pybuffernd_csr_indices.rcbuffer = &__pyx_pybuffer_csr_indices;
  __pyx_pybuffer_csr_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indptr.refcount = 0;
  __pyx_pybuffernd_csr_indptr.data = NULL;
  __pyx_pybuffernd_csr_indptr.rcbuffer = &__pyx_pybuffer_csr_indptr;
  __pyx_pybuffer_dist_matrix.pybuffer.buf = NULL;
  __pyx_pybuffer_dist_matrix.refcount = 0;
  __pyx_pybuffernd_dist_matrix.data = NULL;
  __pyx_pybuffernd_dist_matrix.rcbuffer = &__pyx_pybuffer_dist_matrix;
  __pyx_pybuffer_pred.pybuffer.buf = NULL;
  __pyx_pybuffer_pred.refcount = 0;
  __pyx_pybuffernd_pred.data = NULL;
  __pyx_pybuffernd_pred.rcbuffer = &__pyx_pybuffer_pred;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_source_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_source_indices.diminfo[0].strides = __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_source_indices.diminfo[0].shape = __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_weights, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_weights.diminfo[0].strides = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_weights.diminfo[0].shape = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indices.diminfo[0].strides = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indices.diminfo[0].shape = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indptr, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indptr.diminfo[0].strides = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indptr.diminfo[0].shape = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer, (PyObject*)__pyx_v_dist_matrix, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_dist_matrix.diminfo[0].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dist_matrix.diminfo[0].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_dist_matrix.diminfo[1].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_dist_matrix.diminfo[1].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_pred.rcbuffer->pybuffer, (PyObject*)__pyx_v_pred, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_pred.diminfo[0].strides = __pyx_pybuffernd_pred.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_pred.diminfo[0].shape = __pyx_pybuffernd_pred.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_pred.diminfo[1].strides = __pyx_pybuffernd_pred.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_pred.diminfo[1].shape = __pyx_pybuffernd_pred.rcbuffer->pybuffer.shape[1];

  /* "scipy/sparse/csgraph/_shortest_path.pyx":472
 *             np.ndarray[ITYPE_t, ndim=2, mode='c'] pred,
 *             DTYPE_t limit):
 *     cdef unsigned int Nind = dist_matrix.shape[0]             # <<<<<<<<<<<<<<
 *     cdef unsigned int N = dist_matrix.shape[1]
 *     cdef unsigned int i, k, j_source, j_current
 */
  __pyx_v_Nind = (__pyx_v_dist_matrix->dimensions[0]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":473
 *             DTYPE_t limit):
 *     cdef unsigned int Nind = dist_matrix.shape[0]
 *     cdef unsigned int N = dist_matrix.shape[1]             # <<<<<<<<<<<<<<
 *     cdef unsigned int i, k, j_source, j_current
 *     cdef ITYPE_t j
 */
  __pyx_v_N = (__pyx_v_dist_matrix->dimensions[1]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":479
 *     cdef DTYPE_t next_val
 * 
 *     cdef int return_pred = (pred.size > 0)             # <<<<<<<<<<<<<<
 * 
 *     cdef FibonacciHeap heap
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_pred), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_return_pred = __pyx_t_3;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":484
 *     cdef FibonacciNode *v
 *     cdef FibonacciNode *current_node
 *     cdef FibonacciNode* nodes = <FibonacciNode*> malloc(N *             # <<<<<<<<<<<<<<
 *                                                         sizeof(FibonacciNode))
 * 
 */
  __pyx_v_nodes = ((struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *)malloc((__pyx_v_N * (sizeof(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode)))));

  /* "scipy/sparse/csgraph/_shortest_path.pyx":487
 *                                                         sizeof(FibonacciNode))
 * 
 *     for i from 0 <= i < Nind:             # <<<<<<<<<<<<<<
 *         j_source = source_indices[i]
 * 
 */
  __pyx_t_4 = __pyx_v_Nind;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_4; __pyx_v_i++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":488
 * 
 *     for i from 0 <= i < Nind:
 *         j_source = source_indices[i]             # <<<<<<<<<<<<<<
 * 
 *         for k from 0 <= k < N:
 */
    __pyx_t_5 = __pyx_v_i;
    __pyx_t_3 = -1;
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_source_indices.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_j_source = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_source_indices.diminfo[0].strides));

    /* "scipy/sparse/csgraph/_shortest_path.pyx":490
 *         j_source = source_indices[i]
 * 
 *         for k from 0 <= k < N:             # <<<<<<<<<<<<<<
 *             initialize_node(&nodes[k], k)
 * 
 */
    __pyx_t_6 = __pyx_v_N;
    for (__pyx_v_k = 0; __pyx_v_k < __pyx_t_6; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":491
 * 
 *         for k from 0 <= k < N:
 *             initialize_node(&nodes[k], k)             # <<<<<<<<<<<<<<
 * 
 *         dist_matrix[i, j_source] = 0
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_initialize_node((&(__pyx_v_nodes[__pyx_v_k])), __pyx_v_k, NULL);
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":493
 *             initialize_node(&nodes[k], k)
 * 
 *         dist_matrix[i, j_source] = 0             # <<<<<<<<<<<<<<
 *         heap.min_node = NULL
 *         insert_node(&heap, &nodes[j_source])
 */
    __pyx_t_7 = __pyx_v_i;
    __pyx_t_8 = __pyx_v_j_source;
    __pyx_t_3 = -1;
    if (unlikely(__pyx_t_7 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_8 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_7, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_8, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = 0.0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":494
 * 
 *         dist_matrix[i, j_source] = 0
 *         heap.min_node = NULL             # <<<<<<<<<<<<<<
 *         insert_node(&heap, &nodes[j_source])
 * 
 */
    __pyx_v_heap.min_node = NULL;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":495
 *         dist_matrix[i, j_source] = 0
 *         heap.min_node = NULL
 *         insert_node(&heap, &nodes[j_source])             # <<<<<<<<<<<<<<
 * 
 *         while heap.min_node:
 */
    __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_insert_node((&__pyx_v_heap), (&(__pyx_v_nodes[__pyx_v_j_source])));

    /* "scipy/sparse/csgraph/_shortest_path.pyx":497
 *         insert_node(&heap, &nodes[j_source])
 * 
 *         while heap.min_node:             # <<<<<<<<<<<<<<
 *             v = remove_min(&heap)
 *             v.state = SCANNED
 */
    while (1) {
      __pyx_t_9 = (__pyx_v_heap.min_node != 0);
      if (!__pyx_t_9) break;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":498
 * 
 *         while heap.min_node:
 *             v = remove_min(&heap)             # <<<<<<<<<<<<<<
 *             v.state = SCANNED
 * 
 */
      __pyx_v_v = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove_min((&__pyx_v_heap));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":499
 *         while heap.min_node:
 *             v = remove_min(&heap)
 *             v.state = SCANNED             # <<<<<<<<<<<<<<
 * 
 *             for j from csr_indptr[v.index] <= j < csr_indptr[v.index + 1]:
 */
      __pyx_v_v->state = __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_SCANNED;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":501
 *             v.state = SCANNED
 * 
 *             for j from csr_indptr[v.index] <= j < csr_indptr[v.index + 1]:             # <<<<<<<<<<<<<<
 *                 j_current = csr_indices[j]
 *                 current_node = &nodes[j_current]
 */
      __pyx_t_10 = __pyx_v_v->index;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_10 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_11 = (__pyx_v_v->index + 1);
      __pyx_t_3 = -1;
      if (__pyx_t_11 < 0) {
        __pyx_t_11 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
        if (unlikely(__pyx_t_11 < 0)) __pyx_t_3 = 0;
      } else if (unlikely(__pyx_t_11 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
      for (__pyx_v_j = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_10, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_j < __pyx_t_12; __pyx_v_j++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":502
 * 
 *             for j from csr_indptr[v.index] <= j < csr_indptr[v.index + 1]:
 *                 j_current = csr_indices[j]             # <<<<<<<<<<<<<<
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:
 */
        __pyx_t_13 = __pyx_v_j;
        __pyx_t_3 = -1;
        if (__pyx_t_13 < 0) {
          __pyx_t_13 += __pyx_pybuffernd_csr_indices.diminfo[0].shape;
          if (unlikely(__pyx_t_13 < 0)) __pyx_t_3 = 0;
        } else if (unlikely(__pyx_t_13 >= __pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_j_current = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_13, __pyx_pybuffernd_csr_indices.diminfo[0].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":503
 *             for j from csr_indptr[v.index] <= j < csr_indptr[v.index + 1]:
 *                 j_current = csr_indices[j]
 *                 current_node = &nodes[j_current]             # <<<<<<<<<<<<<<
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csr_weights[j]
 */
        __pyx_v_current_node = (&(__pyx_v_nodes[__pyx_v_j_current]));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":504
 *                 j_current = csr_indices[j]
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:             # <<<<<<<<<<<<<<
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:
 */
        __pyx_t_9 = ((__pyx_v_current_node->state != __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_SCANNED) != 0);
        if (__pyx_t_9) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":505
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csr_weights[j]             # <<<<<<<<<<<<<<
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:
 */
          __pyx_t_14 = __pyx_v_j;
          __pyx_t_3 = -1;
          if (__pyx_t_14 < 0) {
            __pyx_t_14 += __pyx_pybuffernd_csr_weights.diminfo[0].shape;
            if (unlikely(__pyx_t_14 < 0)) __pyx_t_3 = 0;
          } else if (unlikely(__pyx_t_14 >= __pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_3 = 0;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_v_next_val = (__pyx_v_v->val + (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_14, __pyx_pybuffernd_csr_weights.diminfo[0].strides)));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":506
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:             # <<<<<<<<<<<<<<
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 */
          __pyx_t_9 = ((__pyx_v_next_val <= __pyx_v_limit) != 0);
          if (__pyx_t_9) {

            /* "scipy/sparse/csgraph/_shortest_path.pyx":507
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:             # <<<<<<<<<<<<<<
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 */
            __pyx_t_9 = ((__pyx_v_current_node->state == __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_NOT_IN_HEAP) != 0);
            if (__pyx_t_9) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":508
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP             # <<<<<<<<<<<<<<
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 */
              __pyx_v_current_node->state = __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_IN_HEAP;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":509
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val             # <<<<<<<<<<<<<<
 *                             insert_node(&heap, current_node)
 *                             if return_pred:
 */
              __pyx_v_current_node->val = __pyx_v_next_val;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":510
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)             # <<<<<<<<<<<<<<
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 */
              __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_insert_node((&__pyx_v_heap), __pyx_v_current_node);

              /* "scipy/sparse/csgraph/_shortest_path.pyx":511
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 */
              __pyx_t_9 = (__pyx_v_return_pred != 0);
              if (__pyx_t_9) {

                /* "scipy/sparse/csgraph/_shortest_path.pyx":512
 *                             insert_node(&heap, current_node)
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index             # <<<<<<<<<<<<<<
 *                         elif current_node.val > next_val:
 *                             decrease_val(&heap, current_node,
 */
                __pyx_t_6 = __pyx_v_v->index;
                __pyx_t_15 = __pyx_v_i;
                __pyx_t_16 = __pyx_v_j_current;
                __pyx_t_3 = -1;
                if (unlikely(__pyx_t_15 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
                if (unlikely(__pyx_t_16 >= (size_t)__pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
                if (unlikely(__pyx_t_3 != -1)) {
                  __Pyx_RaiseBufferIndexError(__pyx_t_3);
                  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 512; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
                }
                *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_15, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_16, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_t_6;

                /* "scipy/sparse/csgraph/_shortest_path.pyx":511
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 */
              }

              /* "scipy/sparse/csgraph/_shortest_path.pyx":507
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:             # <<<<<<<<<<<<<<
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 */
              goto __pyx_L13;
            }

            /* "scipy/sparse/csgraph/_shortest_path.pyx":513
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:             # <<<<<<<<<<<<<<
 *                             decrease_val(&heap, current_node,
 *                                          next_val)
 */
            __pyx_t_9 = ((__pyx_v_current_node->val > __pyx_v_next_val) != 0);
            if (__pyx_t_9) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":514
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 *                             decrease_val(&heap, current_node,             # <<<<<<<<<<<<<<
 *                                          next_val)
 *                             if return_pred:
 */
              __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_decrease_val((&__pyx_v_heap), __pyx_v_current_node, __pyx_v_next_val);

              /* "scipy/sparse/csgraph/_shortest_path.pyx":516
 *                             decrease_val(&heap, current_node,
 *                                          next_val)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 * 
 */
              __pyx_t_9 = (__pyx_v_return_pred != 0);
              if (__pyx_t_9) {

                /* "scipy/sparse/csgraph/_shortest_path.pyx":517
 *                                          next_val)
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index             # <<<<<<<<<<<<<<
 * 
 *             #v has now been scanned: add the distance to the results
 */
                __pyx_t_6 = __pyx_v_v->index;
                __pyx_t_17 = __pyx_v_i;
                __pyx_t_18 = __pyx_v_j_current;
                __pyx_t_3 = -1;
                if (unlikely(__pyx_t_17 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
                if (unlikely(__pyx_t_18 >= (size_t)__pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
                if (unlikely(__pyx_t_3 != -1)) {
                  __Pyx_RaiseBufferIndexError(__pyx_t_3);
                  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 517; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
                }
                *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_17, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_18, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_t_6;

                /* "scipy/sparse/csgraph/_shortest_path.pyx":516
 *                             decrease_val(&heap, current_node,
 *                                          next_val)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 * 
 */
              }

              /* "scipy/sparse/csgraph/_shortest_path.pyx":513
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:             # <<<<<<<<<<<<<<
 *                             decrease_val(&heap, current_node,
 *                                          next_val)
 */
            }
            __pyx_L13:;

            /* "scipy/sparse/csgraph/_shortest_path.pyx":506
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:             # <<<<<<<<<<<<<<
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 */
          }

          /* "scipy/sparse/csgraph/_shortest_path.pyx":504
 *                 j_current = csr_indices[j]
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:             # <<<<<<<<<<<<<<
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:
 */
        }
      }

      /* "scipy/sparse/csgraph/_shortest_path.pyx":520
 * 
 *             #v has now been scanned: add the distance to the results
 *             dist_matrix[i, v.index] = v.val             # <<<<<<<<<<<<<<
 * 
 *     free(nodes)
 */
      __pyx_t_19 = __pyx_v_v->val;
      __pyx_t_20 = __pyx_v_i;
      __pyx_t_21 = __pyx_v_v->index;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_20 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_21 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_20, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_21, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = __pyx_t_19;
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":522
 *             dist_matrix[i, v.index] = v.val
 * 
 *     free(nodes)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_nodes);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":464
 * 
 * 
 * cdef _dijkstra_directed(             # <<<<<<<<<<<<<<
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] source_indices,
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_pred.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path._dijkstra_directed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_pred.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":525
 * 
 * 
 * cdef _dijkstra_undirected(             # <<<<<<<<<<<<<<
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] source_indices,
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 */

static PyObject *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__dijkstra_undirected(PyArrayObject *__pyx_v_source_indices, PyArrayObject *__pyx_v_csr_weights, PyArrayObject *__pyx_v_csr_indices, PyArrayObject *__pyx_v_csr_indptr, PyArrayObject *__pyx_v_csrT_weights, PyArrayObject *__pyx_v_csrT_indices, PyArrayObject *__pyx_v_csrT_indptr, PyArrayObject *__pyx_v_dist_matrix, PyArrayObject *__pyx_v_pred, __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_limit) {
  unsigned int __pyx_v_Nind;
  unsigned int __pyx_v_N;
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_k;
  unsigned int __pyx_v_j_source;
  unsigned int __pyx_v_j_current;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_v_j;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_next_val;
  int __pyx_v_return_pred;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap __pyx_v_heap;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_v;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_current_node;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_nodes;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csrT_indices;
  __Pyx_Buffer __pyx_pybuffer_csrT_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csrT_indptr;
  __Pyx_Buffer __pyx_pybuffer_csrT_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csrT_weights;
  __Pyx_Buffer __pyx_pybuffer_csrT_weights;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indices;
  __Pyx_Buffer __pyx_pybuffer_csr_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indptr;
  __Pyx_Buffer __pyx_pybuffer_csr_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_weights;
  __Pyx_Buffer __pyx_pybuffer_csr_weights;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dist_matrix;
  __Pyx_Buffer __pyx_pybuffer_dist_matrix;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_pred;
  __Pyx_Buffer __pyx_pybuffer_pred;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_source_indices;
  __Pyx_Buffer __pyx_pybuffer_source_indices;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  unsigned int __pyx_t_4;
  size_t __pyx_t_5;
  unsigned int __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_t_9;
  size_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  size_t __pyx_t_15;
  size_t __pyx_t_16;
  size_t __pyx_t_17;
  size_t __pyx_t_18;
  size_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  size_t __pyx_t_23;
  size_t __pyx_t_24;
  size_t __pyx_t_25;
  size_t __pyx_t_26;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_t_27;
  size_t __pyx_t_28;
  size_t __pyx_t_29;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_dijkstra_undirected", 0);
  __pyx_pybuffer_source_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_source_indices.refcount = 0;
  __pyx_pybuffernd_source_indices.data = NULL;
  __pyx_pybuffernd_source_indices.rcbuffer = &__pyx_pybuffer_source_indices;
  __pyx_pybuffer_csr_weights.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_weights.refcount = 0;
  __pyx_pybuffernd_csr_weights.data = NULL;
  __pyx_pybuffernd_csr_weights.rcbuffer = &__pyx_pybuffer_csr_weights;
  __pyx_pybuffer_csr_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indices.refcount = 0;
  __pyx_pybuffernd_csr_indices.data = NULL;
  __pyx_pybuffernd_csr_indices.rcbuffer = &__pyx_pybuffer_csr_indices;
  __pyx_pybuffer_csr_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indptr.refcount = 0;
  __pyx_pybuffernd_csr_indptr.data = NULL;
  __pyx_pybuffernd_csr_indptr.rcbuffer = &__pyx_pybuffer_csr_indptr;
  __pyx_pybuffer_csrT_weights.pybuffer.buf = NULL;
  __pyx_pybuffer_csrT_weights.refcount = 0;
  __pyx_pybuffernd_csrT_weights.data = NULL;
  __pyx_pybuffernd_csrT_weights.rcbuffer = &__pyx_pybuffer_csrT_weights;
  __pyx_pybuffer_csrT_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_csrT_indices.refcount = 0;
  __pyx_pybuffernd_csrT_indices.data = NULL;
  __pyx_pybuffernd_csrT_indices.rcbuffer = &__pyx_pybuffer_csrT_indices;
  __pyx_pybuffer_csrT_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_csrT_indptr.refcount = 0;
  __pyx_pybuffernd_csrT_indptr.data = NULL;
  __pyx_pybuffernd_csrT_indptr.rcbuffer = &__pyx_pybuffer_csrT_indptr;
  __pyx_pybuffer_dist_matrix.pybuffer.buf = NULL;
  __pyx_pybuffer_dist_matrix.refcount = 0;
  __pyx_pybuffernd_dist_matrix.data = NULL;
  __pyx_pybuffernd_dist_matrix.rcbuffer = &__pyx_pybuffer_dist_matrix;
  __pyx_pybuffer_pred.pybuffer.buf = NULL;
  __pyx_pybuffer_pred.refcount = 0;
  __pyx_pybuffernd_pred.data = NULL;
  __pyx_pybuffernd_pred.rcbuffer = &__pyx_pybuffer_pred;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_source_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_source_indices.diminfo[0].strides = __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_source_indices.diminfo[0].shape = __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_weights, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_weights.diminfo[0].strides = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_weights.diminfo[0].shape = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indices.diminfo[0].strides = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indices.diminfo[0].shape = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indptr, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indptr.diminfo[0].strides = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indptr.diminfo[0].shape = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csrT_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_csrT_weights, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csrT_weights.diminfo[0].strides = __pyx_pybuffernd_csrT_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csrT_weights.diminfo[0].shape = __pyx_pybuffernd_csrT_weights.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csrT_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_csrT_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csrT_indices.diminfo[0].strides = __pyx_pybuffernd_csrT_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csrT_indices.diminfo[0].shape = __pyx_pybuffernd_csrT_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csrT_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_v_csrT_indptr, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csrT_indptr.diminfo[0].strides = __pyx_pybuffernd_csrT_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csrT_indptr.diminfo[0].shape = __pyx_pybuffernd_csrT_indptr.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer, (PyObject*)__pyx_v_dist_matrix, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_dist_matrix.diminfo[0].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dist_matrix.diminfo[0].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_dist_matrix.diminfo[1].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_dist_matrix.diminfo[1].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_pred.rcbuffer->pybuffer, (PyObject*)__pyx_v_pred, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_pred.diminfo[0].strides = __pyx_pybuffernd_pred.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_pred.diminfo[0].shape = __pyx_pybuffernd_pred.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_pred.diminfo[1].strides = __pyx_pybuffernd_pred.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_pred.diminfo[1].shape = __pyx_pybuffernd_pred.rcbuffer->pybuffer.shape[1];

  /* "scipy/sparse/csgraph/_shortest_path.pyx":536
 *             np.ndarray[ITYPE_t, ndim=2, mode='c'] pred,
 *             DTYPE_t limit):
 *     cdef unsigned int Nind = dist_matrix.shape[0]             # <<<<<<<<<<<<<<
 *     cdef unsigned int N = dist_matrix.shape[1]
 *     cdef unsigned int i, k, j_source, j_current
 */
  __pyx_v_Nind = (__pyx_v_dist_matrix->dimensions[0]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":537
 *             DTYPE_t limit):
 *     cdef unsigned int Nind = dist_matrix.shape[0]
 *     cdef unsigned int N = dist_matrix.shape[1]             # <<<<<<<<<<<<<<
 *     cdef unsigned int i, k, j_source, j_current
 *     cdef ITYPE_t j
 */
  __pyx_v_N = (__pyx_v_dist_matrix->dimensions[1]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":543
 *     cdef DTYPE_t next_val
 * 
 *     cdef int return_pred = (pred.size > 0)             # <<<<<<<<<<<<<<
 * 
 *     cdef FibonacciHeap heap
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_pred), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 543; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 543; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 543; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_return_pred = __pyx_t_3;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":548
 *     cdef FibonacciNode *v
 *     cdef FibonacciNode *current_node
 *     cdef FibonacciNode* nodes = <FibonacciNode*> malloc(N *             # <<<<<<<<<<<<<<
 *                                                         sizeof(FibonacciNode))
 * 
 */
  __pyx_v_nodes = ((struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *)malloc((__pyx_v_N * (sizeof(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode)))));

  /* "scipy/sparse/csgraph/_shortest_path.pyx":551
 *                                                         sizeof(FibonacciNode))
 * 
 *     for i from 0 <= i < Nind:             # <<<<<<<<<<<<<<
 *         j_source = source_indices[i]
 * 
 */
  __pyx_t_4 = __pyx_v_Nind;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_4; __pyx_v_i++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":552
 * 
 *     for i from 0 <= i < Nind:
 *         j_source = source_indices[i]             # <<<<<<<<<<<<<<
 * 
 *         for k from 0 <= k < N:
 */
    __pyx_t_5 = __pyx_v_i;
    __pyx_t_3 = -1;
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_source_indices.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 552; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_j_source = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_source_indices.diminfo[0].strides));

    /* "scipy/sparse/csgraph/_shortest_path.pyx":554
 *         j_source = source_indices[i]
 * 
 *         for k from 0 <= k < N:             # <<<<<<<<<<<<<<
 *             initialize_node(&nodes[k], k)
 * 
 */
    __pyx_t_6 = __pyx_v_N;
    for (__pyx_v_k = 0; __pyx_v_k < __pyx_t_6; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":555
 * 
 *         for k from 0 <= k < N:
 *             initialize_node(&nodes[k], k)             # <<<<<<<<<<<<<<
 * 
 *         dist_matrix[i, j_source] = 0
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_initialize_node((&(__pyx_v_nodes[__pyx_v_k])), __pyx_v_k, NULL);
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":557
 *             initialize_node(&nodes[k], k)
 * 
 *         dist_matrix[i, j_source] = 0             # <<<<<<<<<<<<<<
 *         heap.min_node = NULL
 *         insert_node(&heap, &nodes[j_source])
 */
    __pyx_t_7 = __pyx_v_i;
    __pyx_t_8 = __pyx_v_j_source;
    __pyx_t_3 = -1;
    if (unlikely(__pyx_t_7 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_8 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 557; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_7, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_8, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = 0.0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":558
 * 
 *         dist_matrix[i, j_source] = 0
 *         heap.min_node = NULL             # <<<<<<<<<<<<<<
 *         insert_node(&heap, &nodes[j_source])
 * 
 */
    __pyx_v_heap.min_node = NULL;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":559
 *         dist_matrix[i, j_source] = 0
 *         heap.min_node = NULL
 *         insert_node(&heap, &nodes[j_source])             # <<<<<<<<<<<<<<
 * 
 *         while heap.min_node:
 */
    __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_insert_node((&__pyx_v_heap), (&(__pyx_v_nodes[__pyx_v_j_source])));

    /* "scipy/sparse/csgraph/_shortest_path.pyx":561
 *         insert_node(&heap, &nodes[j_source])
 * 
 *         while heap.min_node:             # <<<<<<<<<<<<<<
 *             v = remove_min(&heap)
 *             v.state = SCANNED
 */
    while (1) {
      __pyx_t_9 = (__pyx_v_heap.min_node != 0);
      if (!__pyx_t_9) break;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":562
 * 
 *         while heap.min_node:
 *             v = remove_min(&heap)             # <<<<<<<<<<<<<<
 *             v.state = SCANNED
 * 
 */
      __pyx_v_v = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove_min((&__pyx_v_heap));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":563
 *         while heap.min_node:
 *             v = remove_min(&heap)
 *             v.state = SCANNED             # <<<<<<<<<<<<<<
 * 
 *             for j from csr_indptr[v.index] <= j < csr_indptr[v.index + 1]:
 */
      __pyx_v_v->state = __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_SCANNED;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":565
 *             v.state = SCANNED
 * 
 *             for j from csr_indptr[v.index] <= j < csr_indptr[v.index + 1]:             # <<<<<<<<<<<<<<
 *                 j_current = csr_indices[j]
 *                 current_node = &nodes[j_current]
 */
      __pyx_t_10 = __pyx_v_v->index;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_10 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 565; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_11 = (__pyx_v_v->index + 1);
      __pyx_t_3 = -1;
      if (__pyx_t_11 < 0) {
        __pyx_t_11 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
        if (unlikely(__pyx_t_11 < 0)) __pyx_t_3 = 0;
      } else if (unlikely(__pyx_t_11 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 565; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
      for (__pyx_v_j = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_10, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_j < __pyx_t_12; __pyx_v_j++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":566
 * 
 *             for j from csr_indptr[v.index] <= j < csr_indptr[v.index + 1]:
 *                 j_current = csr_indices[j]             # <<<<<<<<<<<<<<
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:
 */
        __pyx_t_13 = __pyx_v_j;
        __pyx_t_3 = -1;
        if (__pyx_t_13 < 0) {
          __pyx_t_13 += __pyx_pybuffernd_csr_indices.diminfo[0].shape;
          if (unlikely(__pyx_t_13 < 0)) __pyx_t_3 = 0;
        } else if (unlikely(__pyx_t_13 >= __pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 566; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_j_current = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_13, __pyx_pybuffernd_csr_indices.diminfo[0].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":567
 *             for j from csr_indptr[v.index] <= j < csr_indptr[v.index + 1]:
 *                 j_current = csr_indices[j]
 *                 current_node = &nodes[j_current]             # <<<<<<<<<<<<<<
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csr_weights[j]
 */
        __pyx_v_current_node = (&(__pyx_v_nodes[__pyx_v_j_current]));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":568
 *                 j_current = csr_indices[j]
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:             # <<<<<<<<<<<<<<
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:
 */
        __pyx_t_9 = ((__pyx_v_current_node->state != __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_SCANNED) != 0);
        if (__pyx_t_9) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":569
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csr_weights[j]             # <<<<<<<<<<<<<<
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:
 */
          __pyx_t_14 = __pyx_v_j;
          __pyx_t_3 = -1;
          if (__pyx_t_14 < 0) {
            __pyx_t_14 += __pyx_pybuffernd_csr_weights.diminfo[0].shape;
            if (unlikely(__pyx_t_14 < 0)) __pyx_t_3 = 0;
          } else if (unlikely(__pyx_t_14 >= __pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_3 = 0;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 569; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_v_next_val = (__pyx_v_v->val + (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_14, __pyx_pybuffernd_csr_weights.diminfo[0].strides)));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":570
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:             # <<<<<<<<<<<<<<
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 */
          __pyx_t_9 = ((__pyx_v_next_val <= __pyx_v_limit) != 0);
          if (__pyx_t_9) {

            /* "scipy/sparse/csgraph/_shortest_path.pyx":571
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:             # <<<<<<<<<<<<<<
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 */
            __pyx_t_9 = ((__pyx_v_current_node->state == __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_NOT_IN_HEAP) != 0);
            if (__pyx_t_9) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":572
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP             # <<<<<<<<<<<<<<
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 */
              __pyx_v_current_node->state = __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_IN_HEAP;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":573
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val             # <<<<<<<<<<<<<<
 *                             insert_node(&heap, current_node)
 *                             if return_pred:
 */
              __pyx_v_current_node->val = __pyx_v_next_val;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":574
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)             # <<<<<<<<<<<<<<
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 */
              __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_insert_node((&__pyx_v_heap), __pyx_v_current_node);

              /* "scipy/sparse/csgraph/_shortest_path.pyx":575
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 */
              __pyx_t_9 = (__pyx_v_return_pred != 0);
              if (__pyx_t_9) {

                /* "scipy/sparse/csgraph/_shortest_path.pyx":576
 *                             insert_node(&heap, current_node)
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index             # <<<<<<<<<<<<<<
 *                         elif current_node.val > next_val:
 *                             decrease_val(&heap, current_node,
 */
                __pyx_t_6 = __pyx_v_v->index;
                __pyx_t_15 = __pyx_v_i;
                __pyx_t_16 = __pyx_v_j_current;
                __pyx_t_3 = -1;
                if (unlikely(__pyx_t_15 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
                if (unlikely(__pyx_t_16 >= (size_t)__pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
                if (unlikely(__pyx_t_3 != -1)) {
                  __Pyx_RaiseBufferIndexError(__pyx_t_3);
                  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 576; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
                }
                *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_15, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_16, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_t_6;

                /* "scipy/sparse/csgraph/_shortest_path.pyx":575
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 */
              }

              /* "scipy/sparse/csgraph/_shortest_path.pyx":571
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:             # <<<<<<<<<<<<<<
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 */
              goto __pyx_L13;
            }

            /* "scipy/sparse/csgraph/_shortest_path.pyx":577
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:             # <<<<<<<<<<<<<<
 *                             decrease_val(&heap, current_node,
 *                                          next_val)
 */
            __pyx_t_9 = ((__pyx_v_current_node->val > __pyx_v_next_val) != 0);
            if (__pyx_t_9) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":578
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 *                             decrease_val(&heap, current_node,             # <<<<<<<<<<<<<<
 *                                          next_val)
 *                             if return_pred:
 */
              __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_decrease_val((&__pyx_v_heap), __pyx_v_current_node, __pyx_v_next_val);

              /* "scipy/sparse/csgraph/_shortest_path.pyx":580
 *                             decrease_val(&heap, current_node,
 *                                          next_val)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 * 
 */
              __pyx_t_9 = (__pyx_v_return_pred != 0);
              if (__pyx_t_9) {

                /* "scipy/sparse/csgraph/_shortest_path.pyx":581
 *                                          next_val)
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index             # <<<<<<<<<<<<<<
 * 
 *             for j from csrT_indptr[v.index] <= j < csrT_indptr[v.index + 1]:
 */
                __pyx_t_6 = __pyx_v_v->index;
                __pyx_t_17 = __pyx_v_i;
                __pyx_t_18 = __pyx_v_j_current;
                __pyx_t_3 = -1;
                if (unlikely(__pyx_t_17 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
                if (unlikely(__pyx_t_18 >= (size_t)__pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
                if (unlikely(__pyx_t_3 != -1)) {
                  __Pyx_RaiseBufferIndexError(__pyx_t_3);
                  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 581; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
                }
                *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_17, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_18, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_t_6;

                /* "scipy/sparse/csgraph/_shortest_path.pyx":580
 *                             decrease_val(&heap, current_node,
 *                                          next_val)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 * 
 */
              }

              /* "scipy/sparse/csgraph/_shortest_path.pyx":577
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:             # <<<<<<<<<<<<<<
 *                             decrease_val(&heap, current_node,
 *                                          next_val)
 */
            }
            __pyx_L13:;

            /* "scipy/sparse/csgraph/_shortest_path.pyx":570
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:             # <<<<<<<<<<<<<<
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 */
          }

          /* "scipy/sparse/csgraph/_shortest_path.pyx":568
 *                 j_current = csr_indices[j]
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:             # <<<<<<<<<<<<<<
 *                     next_val = v.val + csr_weights[j]
 *                     if next_val <= limit:
 */
        }
      }

      /* "scipy/sparse/csgraph/_shortest_path.pyx":583
 *                                 pred[i, j_current] = v.index
 * 
 *             for j from csrT_indptr[v.index] <= j < csrT_indptr[v.index + 1]:             # <<<<<<<<<<<<<<
 *                 j_current = csrT_indices[j]
 *                 current_node = &nodes[j_current]
 */
      __pyx_t_19 = __pyx_v_v->index;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_19 >= (size_t)__pyx_pybuffernd_csrT_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_20 = (__pyx_v_v->index + 1);
      __pyx_t_3 = -1;
      if (__pyx_t_20 < 0) {
        __pyx_t_20 += __pyx_pybuffernd_csrT_indptr.diminfo[0].shape;
        if (unlikely(__pyx_t_20 < 0)) __pyx_t_3 = 0;
      } else if (unlikely(__pyx_t_20 >= __pyx_pybuffernd_csrT_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csrT_indptr.rcbuffer->pybuffer.buf, __pyx_t_20, __pyx_pybuffernd_csrT_indptr.diminfo[0].strides));
      for (__pyx_v_j = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csrT_indptr.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_csrT_indptr.diminfo[0].strides)); __pyx_v_j < __pyx_t_12; __pyx_v_j++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":584
 * 
 *             for j from csrT_indptr[v.index] <= j < csrT_indptr[v.index + 1]:
 *                 j_current = csrT_indices[j]             # <<<<<<<<<<<<<<
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:
 */
        __pyx_t_21 = __pyx_v_j;
        __pyx_t_3 = -1;
        if (__pyx_t_21 < 0) {
          __pyx_t_21 += __pyx_pybuffernd_csrT_indices.diminfo[0].shape;
          if (unlikely(__pyx_t_21 < 0)) __pyx_t_3 = 0;
        } else if (unlikely(__pyx_t_21 >= __pyx_pybuffernd_csrT_indices.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 584; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_j_current = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csrT_indices.rcbuffer->pybuffer.buf, __pyx_t_21, __pyx_pybuffernd_csrT_indices.diminfo[0].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":585
 *             for j from csrT_indptr[v.index] <= j < csrT_indptr[v.index + 1]:
 *                 j_current = csrT_indices[j]
 *                 current_node = &nodes[j_current]             # <<<<<<<<<<<<<<
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csrT_weights[j]
 */
        __pyx_v_current_node = (&(__pyx_v_nodes[__pyx_v_j_current]));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":586
 *                 j_current = csrT_indices[j]
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:             # <<<<<<<<<<<<<<
 *                     next_val = v.val + csrT_weights[j]
 *                     if next_val <= limit:
 */
        __pyx_t_9 = ((__pyx_v_current_node->state != __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_SCANNED) != 0);
        if (__pyx_t_9) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":587
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csrT_weights[j]             # <<<<<<<<<<<<<<
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:
 */
          __pyx_t_22 = __pyx_v_j;
          __pyx_t_3 = -1;
          if (__pyx_t_22 < 0) {
            __pyx_t_22 += __pyx_pybuffernd_csrT_weights.diminfo[0].shape;
            if (unlikely(__pyx_t_22 < 0)) __pyx_t_3 = 0;
          } else if (unlikely(__pyx_t_22 >= __pyx_pybuffernd_csrT_weights.diminfo[0].shape)) __pyx_t_3 = 0;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 587; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_v_next_val = (__pyx_v_v->val + (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csrT_weights.rcbuffer->pybuffer.buf, __pyx_t_22, __pyx_pybuffernd_csrT_weights.diminfo[0].strides)));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":588
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csrT_weights[j]
 *                     if next_val <= limit:             # <<<<<<<<<<<<<<
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 */
          __pyx_t_9 = ((__pyx_v_next_val <= __pyx_v_limit) != 0);
          if (__pyx_t_9) {

            /* "scipy/sparse/csgraph/_shortest_path.pyx":589
 *                     next_val = v.val + csrT_weights[j]
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:             # <<<<<<<<<<<<<<
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 */
            __pyx_t_9 = ((__pyx_v_current_node->state == __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_NOT_IN_HEAP) != 0);
            if (__pyx_t_9) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":590
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP             # <<<<<<<<<<<<<<
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 */
              __pyx_v_current_node->state = __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_IN_HEAP;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":591
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val             # <<<<<<<<<<<<<<
 *                             insert_node(&heap, current_node)
 *                             if return_pred:
 */
              __pyx_v_current_node->val = __pyx_v_next_val;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":592
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)             # <<<<<<<<<<<<<<
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 */
              __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_insert_node((&__pyx_v_heap), __pyx_v_current_node);

              /* "scipy/sparse/csgraph/_shortest_path.pyx":593
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 */
              __pyx_t_9 = (__pyx_v_return_pred != 0);
              if (__pyx_t_9) {

                /* "scipy/sparse/csgraph/_shortest_path.pyx":594
 *                             insert_node(&heap, current_node)
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index             # <<<<<<<<<<<<<<
 *                         elif current_node.val > next_val:
 *                             decrease_val(&heap, current_node, next_val)
 */
                __pyx_t_6 = __pyx_v_v->index;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j_current;
                __pyx_t_3 = -1;
                if (unlikely(__pyx_t_23 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
                if (unlikely(__pyx_t_24 >= (size_t)__pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
                if (unlikely(__pyx_t_3 != -1)) {
                  __Pyx_RaiseBufferIndexError(__pyx_t_3);
                  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 594; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
                }
                *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_23, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_24, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_t_6;

                /* "scipy/sparse/csgraph/_shortest_path.pyx":593
 *                             current_node.val = next_val
 *                             insert_node(&heap, current_node)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 */
              }

              /* "scipy/sparse/csgraph/_shortest_path.pyx":589
 *                     next_val = v.val + csrT_weights[j]
 *                     if next_val <= limit:
 *                         if current_node.state == NOT_IN_HEAP:             # <<<<<<<<<<<<<<
 *                             current_node.state = IN_HEAP
 *                             current_node.val = next_val
 */
              goto __pyx_L20;
            }

            /* "scipy/sparse/csgraph/_shortest_path.pyx":595
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:             # <<<<<<<<<<<<<<
 *                             decrease_val(&heap, current_node, next_val)
 *                             if return_pred:
 */
            __pyx_t_9 = ((__pyx_v_current_node->val > __pyx_v_next_val) != 0);
            if (__pyx_t_9) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":596
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:
 *                             decrease_val(&heap, current_node, next_val)             # <<<<<<<<<<<<<<
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 */
              __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_decrease_val((&__pyx_v_heap), __pyx_v_current_node, __pyx_v_next_val);

              /* "scipy/sparse/csgraph/_shortest_path.pyx":597
 *                         elif current_node.val > next_val:
 *                             decrease_val(&heap, current_node, next_val)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 * 
 */
              __pyx_t_9 = (__pyx_v_return_pred != 0);
              if (__pyx_t_9) {

                /* "scipy/sparse/csgraph/_shortest_path.pyx":598
 *                             decrease_val(&heap, current_node, next_val)
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index             # <<<<<<<<<<<<<<
 * 
 *             #v has now been scanned: add the distance to the results
 */
                __pyx_t_6 = __pyx_v_v->index;
                __pyx_t_25 = __pyx_v_i;
                __pyx_t_26 = __pyx_v_j_current;
                __pyx_t_3 = -1;
                if (unlikely(__pyx_t_25 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
                if (unlikely(__pyx_t_26 >= (size_t)__pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
                if (unlikely(__pyx_t_3 != -1)) {
                  __Pyx_RaiseBufferIndexError(__pyx_t_3);
                  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
                }
                *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_25, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_26, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_t_6;

                /* "scipy/sparse/csgraph/_shortest_path.pyx":597
 *                         elif current_node.val > next_val:
 *                             decrease_val(&heap, current_node, next_val)
 *                             if return_pred:             # <<<<<<<<<<<<<<
 *                                 pred[i, j_current] = v.index
 * 
 */
              }

              /* "scipy/sparse/csgraph/_shortest_path.pyx":595
 *                             if return_pred:
 *                                 pred[i, j_current] = v.index
 *                         elif current_node.val > next_val:             # <<<<<<<<<<<<<<
 *                             decrease_val(&heap, current_node, next_val)
 *                             if return_pred:
 */
            }
            __pyx_L20:;

            /* "scipy/sparse/csgraph/_shortest_path.pyx":588
 *                 if current_node.state != SCANNED:
 *                     next_val = v.val + csrT_weights[j]
 *                     if next_val <= limit:             # <<<<<<<<<<<<<<
 *                         if current_node.state == NOT_IN_HEAP:
 *                             current_node.state = IN_HEAP
 */
          }

          /* "scipy/sparse/csgraph/_shortest_path.pyx":586
 *                 j_current = csrT_indices[j]
 *                 current_node = &nodes[j_current]
 *                 if current_node.state != SCANNED:             # <<<<<<<<<<<<<<
 *                     next_val = v.val + csrT_weights[j]
 *                     if next_val <= limit:
 */
        }
      }

      /* "scipy/sparse/csgraph/_shortest_path.pyx":601
 * 
 *             #v has now been scanned: add the distance to the results
 *             dist_matrix[i, v.index] = v.val             # <<<<<<<<<<<<<<
 * 
 *     free(nodes)
 */
      __pyx_t_27 = __pyx_v_v->val;
      __pyx_t_28 = __pyx_v_i;
      __pyx_t_29 = __pyx_v_v->index;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_28 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_29 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_28, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_29, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = __pyx_t_27;
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":603
 *             dist_matrix[i, v.index] = v.val
 * 
 *     free(nodes)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_nodes);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":525
 * 
 * 
 * cdef _dijkstra_undirected(             # <<<<<<<<<<<<<<
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] source_indices,
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csrT_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csrT_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csrT_weights.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_pred.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path._dijkstra_undirected", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csrT_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csrT_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csrT_weights.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_pred.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":606
 * 
 * 
 * def bellman_ford(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *                  return_predecessors=False,
 *                  unweighted=False):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_7bellman_ford(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_6bellman_ford[] = "\n    bellman_ford(csgraph, directed=True, indices=None, return_predecessors=False,\n                 unweighted=False)\n\n    Compute the shortest path lengths using the Bellman-Ford algorithm.\n\n    The Bellman-ford algorithm can robustly deal with graphs with negative\n    weights.  If a negative cycle is detected, an error is raised.  For\n    graphs without negative edge weights, dijkstra's algorithm may be faster.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of distances representing the input graph.\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    indices : array_like or int, optional\n        if specified, only compute the paths for the points at the given\n        indices.\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The N x N matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        information on the shortest paths from point i: each entry\n        predecessors[i, ""j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Raises\n    ------\n    NegativeCycleError:\n        if there are negative cycles in the graph\n\n    Notes\n    -----\n    This routine is specially designed for graphs with negative edge weights.\n    If all edge weights are positive, then Dijkstra's algorithm is a better\n    choice.\n    ";
static PyMethodDef __pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_7bellman_ford = {"bellman_ford", (PyCFunction)__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_7bellman_ford, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_6bellman_ford};
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_7bellman_ford(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_csgraph = 0;
  PyObject *__pyx_v_directed = 0;
  PyObject *__pyx_v_indices = 0;
  PyObject *__pyx_v_return_predecessors = 0;
  PyObject *__pyx_v_unweighted = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("bellman_ford (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_csgraph,&__pyx_n_s_directed,&__pyx_n_s_indices,&__pyx_n_s_return_predecessors,&__pyx_n_s_unweighted,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[1] = ((PyObject *)Py_True);
    values[2] = ((PyObject *)Py_None);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":607
 * 
 * def bellman_ford(csgraph, directed=True, indices=None,
 *                  return_predecessors=False,             # <<<<<<<<<<<<<<
 *                  unweighted=False):
 *     """
 */
    values[3] = ((PyObject *)Py_False);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":608
 * def bellman_ford(csgraph, directed=True, indices=None,
 *                  return_predecessors=False,
 *                  unweighted=False):             # <<<<<<<<<<<<<<
 *     """
 *     bellman_ford(csgraph, directed=True, indices=None, return_predecessors=False,
 */
    values[4] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_csgraph)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_directed);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_indices);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_predecessors);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_unweighted);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "bellman_ford") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_csgraph = values[0];
    __pyx_v_directed = values[1];
    __pyx_v_indices = values[2];
    __pyx_v_return_predecessors = values[3];
    __pyx_v_unweighted = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("bellman_ford", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.bellman_ford", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_6bellman_ford(__pyx_self, __pyx_v_csgraph, __pyx_v_directed, __pyx_v_indices, __pyx_v_return_predecessors, __pyx_v_unweighted);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":606
 * 
 * 
 * def bellman_ford(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *                  return_predecessors=False,
 *                  unweighted=False):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_6bellman_ford(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_directed, PyObject *__pyx_v_indices, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted) {
  PyObject *__pyx_v_N = NULL;
  PyObject *__pyx_v_return_shape = NULL;
  PyObject *__pyx_v_dist_matrix = NULL;
  PyObject *__pyx_v_predecessor_matrix = NULL;
  PyObject *__pyx_v_csr_data = NULL;
  PyObject *__pyx_v_ret = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bellman_ford", 0);
  __Pyx_INCREF(__pyx_v_csgraph);
  __Pyx_INCREF(__pyx_v_indices);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":671
 *     #------------------------------
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                              dense_output=False)
 *     N = csgraph.shape[0]
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_validate_graph); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 671; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 671; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 671; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_csgraph);
  __Pyx_GIVEREF(__pyx_v_csgraph);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_csgraph);
  __Pyx_INCREF(__pyx_v_directed);
  __Pyx_GIVEREF(__pyx_v_directed);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_directed);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":672
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,
 *                              dense_output=False)             # <<<<<<<<<<<<<<
 *     N = csgraph.shape[0]
 * 
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 672; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dense_output, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 672; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":671
 *     #------------------------------
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                              dense_output=False)
 *     N = csgraph.shape[0]
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 671; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_csgraph, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":673
 *     csgraph = validate_graph(csgraph, directed, DTYPE,
 *                              dense_output=False)
 *     N = csgraph.shape[0]             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 673; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 673; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_N = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":677
 *     #------------------------------
 *     # intitialize/validate indices
 *     if indices is None:             # <<<<<<<<<<<<<<
 *         indices = np.arange(N, dtype=ITYPE)
 *     else:
 */
  __pyx_t_5 = (__pyx_v_indices == Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":678
 *     # intitialize/validate indices
 *     if indices is None:
 *         indices = np.arange(N, dtype=ITYPE)             # <<<<<<<<<<<<<<
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE)
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 678; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_arange); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 678; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 678; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_N);
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 678; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 678; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 678; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 678; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":677
 *     #------------------------------
 *     # intitialize/validate indices
 *     if indices is None:             # <<<<<<<<<<<<<<
 *         indices = np.arange(N, dtype=ITYPE)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":680
 *         indices = np.arange(N, dtype=ITYPE)
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_indices);
    __Pyx_GIVEREF(__pyx_v_indices);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_indices);
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":681
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE)
 *         indices[indices < 0] += N             # <<<<<<<<<<<<<<
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")
 */
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_indices, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 681; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_2 = PyObject_GetItem(__pyx_v_indices, __pyx_t_4); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 681; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_t_2, __pyx_v_N); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 681; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_indices, __pyx_t_4, __pyx_t_1) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 681; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":682
 *         indices = np.array(indices, order='C', dtype=ITYPE)
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):             # <<<<<<<<<<<<<<
 *             raise ValueError("indices out of range 0...N")
 *     return_shape = indices.shape + (N,)
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_any); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_indices, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_any); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_indices, __pyx_v_N, Py_GE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = __pyx_t_5;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_6) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":683
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")             # <<<<<<<<<<<<<<
 *     return_shape = indices.shape + (N,)
 *     indices = np.atleast_1d(indices).reshape(-1)
 */
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "scipy/sparse/csgraph/_shortest_path.pyx":682
 *         indices = np.array(indices, order='C', dtype=ITYPE)
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):             # <<<<<<<<<<<<<<
 *             raise ValueError("indices out of range 0...N")
 *     return_shape = indices.shape + (N,)
 */
    }
  }
  __pyx_L3:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":684
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")
 *     return_shape = indices.shape + (N,)             # <<<<<<<<<<<<<<
 *     indices = np.atleast_1d(indices).reshape(-1)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 684; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 684; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_N);
  __Pyx_GIVEREF(__pyx_v_N);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_N);
  __pyx_t_3 = PyNumber_Add(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 684; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_return_shape = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":685
 *             raise ValueError("indices out of range 0...N")
 *     return_shape = indices.shape + (N,)
 *     indices = np.atleast_1d(indices).reshape(-1)             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 685; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_atleast_1d); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 685; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_7) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_indices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 685; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 685; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_7); __pyx_t_7 = NULL;
    __Pyx_INCREF(__pyx_v_indices);
    __Pyx_GIVEREF(__pyx_v_indices);
    PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_v_indices);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 685; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 685; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 685; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":689
 *     #------------------------------
 *     # initialize dist_matrix for output
 *     dist_matrix = np.empty((len(indices), N), dtype=DTYPE)             # <<<<<<<<<<<<<<
 *     dist_matrix.fill(np.inf)
 *     dist_matrix[np.arange(len(indices)), indices] = 0
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_8 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_N);
  __Pyx_GIVEREF(__pyx_v_N);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_N);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_dist_matrix = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":690
 *     # initialize dist_matrix for output
 *     dist_matrix = np.empty((len(indices), N), dtype=DTYPE)
 *     dist_matrix.fill(np.inf)             # <<<<<<<<<<<<<<
 *     dist_matrix[np.arange(len(indices)), indices] = 0
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_fill); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 690; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 690; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_inf); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 690; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 690; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 690; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 690; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":691
 *     dist_matrix = np.empty((len(indices), N), dtype=DTYPE)
 *     dist_matrix.fill(np.inf)
 *     dist_matrix[np.arange(len(indices)), indices] = 0             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_arange); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_7);
  __Pyx_INCREF(__pyx_v_indices);
  __Pyx_GIVEREF(__pyx_v_indices);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_indices);
  __pyx_t_7 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_dist_matrix, __pyx_t_1, __pyx_int_0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 691; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":695
 *     #------------------------------
 *     # initialize predecessors for output
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_return_predecessors); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":696
 *     # initialize predecessors for output
 *     if return_predecessors:
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         predecessor_matrix.fill(NULL_IDX)
 *     else:
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_N);
    __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_predecessor_matrix = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":697
 *     if return_predecessors:
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)             # <<<<<<<<<<<<<<
 *     else:
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_predecessor_matrix, __pyx_n_s_fill); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 697; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyInt_From_npy_int32(__pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_NULL_IDX); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 697; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_7) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 697; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 697; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 697; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":695
 *     #------------------------------
 *     # initialize predecessors for output
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)
 */
    goto __pyx_L7;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":699
 *         predecessor_matrix.fill(NULL_IDX)
 *     else:
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)             # <<<<<<<<<<<<<<
 * 
 *     if unweighted:
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_GIVEREF(__pyx_int_0);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_int_0);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_N);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_predecessor_matrix = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L7:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":701
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)
 * 
 *     if unweighted:             # <<<<<<<<<<<<<<
 *         csr_data = np.ones(csgraph.data.shape)
 *     else:
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_unweighted); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 701; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":702
 * 
 *     if unweighted:
 *         csr_data = np.ones(csgraph.data.shape)             # <<<<<<<<<<<<<<
 *     else:
 *         csr_data = csgraph.data
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 702; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ones); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 702; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_data); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 702; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 702; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 702; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 702; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 702; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_csr_data = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":701
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)
 * 
 *     if unweighted:             # <<<<<<<<<<<<<<
 *         csr_data = np.ones(csgraph.data.shape)
 *     else:
 */
    goto __pyx_L8;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":704
 *         csr_data = np.ones(csgraph.data.shape)
 *     else:
 *         csr_data = csgraph.data             # <<<<<<<<<<<<<<
 * 
 *     if directed:
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_data); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_csr_data = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L8:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":706
 *         csr_data = csgraph.data
 * 
 *     if directed:             # <<<<<<<<<<<<<<
 *         ret = _bellman_ford_directed(indices,
 *                                      csr_data, csgraph.indices,
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_directed); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":707
 * 
 *     if directed:
 *         ret = _bellman_ford_directed(indices,             # <<<<<<<<<<<<<<
 *                                      csr_data, csgraph.indices,
 *                                      csgraph.indptr,
 */
    if (!(likely(((__pyx_v_indices) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_indices, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 707; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":708
 *     if directed:
 *         ret = _bellman_ford_directed(indices,
 *                                      csr_data, csgraph.indices,             # <<<<<<<<<<<<<<
 *                                      csgraph.indptr,
 *                                      dist_matrix, predecessor_matrix)
 */
    if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 708; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 708; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 708; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":709
 *         ret = _bellman_ford_directed(indices,
 *                                      csr_data, csgraph.indices,
 *                                      csgraph.indptr,             # <<<<<<<<<<<<<<
 *                                      dist_matrix, predecessor_matrix)
 *     else:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 709; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 709; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":710
 *                                      csr_data, csgraph.indices,
 *                                      csgraph.indptr,
 *                                      dist_matrix, predecessor_matrix)             # <<<<<<<<<<<<<<
 *     else:
 *         ret = _bellman_ford_undirected(indices,
 */
    if (!(likely(((__pyx_v_dist_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 710; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_predecessor_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_predecessor_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 710; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":707
 * 
 *     if directed:
 *         ret = _bellman_ford_directed(indices,             # <<<<<<<<<<<<<<
 *                                      csr_data, csgraph.indices,
 *                                      csgraph.indptr,
 */
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__bellman_ford_directed(((PyArrayObject *)__pyx_v_indices), ((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_1), ((PyArrayObject *)__pyx_t_4), ((PyArrayObject *)__pyx_v_dist_matrix), ((PyArrayObject *)__pyx_v_predecessor_matrix))); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 707; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_ret = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":706
 *         csr_data = csgraph.data
 * 
 *     if directed:             # <<<<<<<<<<<<<<
 *         ret = _bellman_ford_directed(indices,
 *                                      csr_data, csgraph.indices,
 */
    goto __pyx_L9;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":712
 *                                      dist_matrix, predecessor_matrix)
 *     else:
 *         ret = _bellman_ford_undirected(indices,             # <<<<<<<<<<<<<<
 *                                        csr_data, csgraph.indices,
 *                                        csgraph.indptr,
 */
  /*else*/ {
    if (!(likely(((__pyx_v_indices) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_indices, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":713
 *     else:
 *         ret = _bellman_ford_undirected(indices,
 *                                        csr_data, csgraph.indices,             # <<<<<<<<<<<<<<
 *                                        csgraph.indptr,
 *                                        dist_matrix, predecessor_matrix)
 */
    if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":714
 *         ret = _bellman_ford_undirected(indices,
 *                                        csr_data, csgraph.indices,
 *                                        csgraph.indptr,             # <<<<<<<<<<<<<<
 *                                        dist_matrix, predecessor_matrix)
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 714; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 714; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":715
 *                                        csr_data, csgraph.indices,
 *                                        csgraph.indptr,
 *                                        dist_matrix, predecessor_matrix)             # <<<<<<<<<<<<<<
 * 
 *     if ret >= 0:
 */
    if (!(likely(((__pyx_v_dist_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_predecessor_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_predecessor_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":712
 *                                      dist_matrix, predecessor_matrix)
 *     else:
 *         ret = _bellman_ford_undirected(indices,             # <<<<<<<<<<<<<<
 *                                        csr_data, csgraph.indices,
 *                                        csgraph.indptr,
 */
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__bellman_ford_undirected(((PyArrayObject *)__pyx_v_indices), ((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_7), ((PyArrayObject *)__pyx_t_4), ((PyArrayObject *)__pyx_v_dist_matrix), ((PyArrayObject *)__pyx_v_predecessor_matrix))); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_ret = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L9:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":717
 *                                        dist_matrix, predecessor_matrix)
 * 
 *     if ret >= 0:             # <<<<<<<<<<<<<<
 *         raise NegativeCycleError("Negative cycle detected on node %i" % ret)
 * 
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_ret, __pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 717; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 717; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":718
 * 
 *     if ret >= 0:
 *         raise NegativeCycleError("Negative cycle detected on node %i" % ret)             # <<<<<<<<<<<<<<
 * 
 *     if return_predecessors:
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_NegativeCycleError); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyString_Format(__pyx_kp_s_Negative_cycle_detected_on_node, __pyx_v_ret); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":717
 *                                        dist_matrix, predecessor_matrix)
 * 
 *     if ret >= 0:             # <<<<<<<<<<<<<<
 *         raise NegativeCycleError("Negative cycle detected on node %i" % ret)
 * 
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":720
 *         raise NegativeCycleError("Negative cycle detected on node %i" % ret)
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_return_predecessors); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":721
 * 
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),             # <<<<<<<<<<<<<<
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_return_shape); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_return_shape);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":722
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))             # <<<<<<<<<<<<<<
 *     else:
 *         return dist_matrix.reshape(return_shape)
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_predecessor_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_return_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_return_shape);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":721
 * 
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),             # <<<<<<<<<<<<<<
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 */
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4);
    __pyx_t_1 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":720
 *         raise NegativeCycleError("Negative cycle detected on node %i" % ret)
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":724
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 *         return dist_matrix.reshape(return_shape)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 724; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_return_shape); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 724; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
    } else {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 724; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_return_shape);
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 724; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":606
 * 
 * 
 * def bellman_ford(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *                  return_predecessors=False,
 *                  unweighted=False):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.bellman_ford", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_N);
  __Pyx_XDECREF(__pyx_v_return_shape);
  __Pyx_XDECREF(__pyx_v_dist_matrix);
  __Pyx_XDECREF(__pyx_v_predecessor_matrix);
  __Pyx_XDECREF(__pyx_v_csr_data);
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_csgraph);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":727
 * 
 * 
 * cdef int _bellman_ford_directed(             # <<<<<<<<<<<<<<
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] source_indices,
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 */

static int __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__bellman_ford_directed(PyArrayObject *__pyx_v_source_indices, PyArrayObject *__pyx_v_csr_weights, PyArrayObject *__pyx_v_csr_indices, PyArrayObject *__pyx_v_csr_indptr, PyArrayObject *__pyx_v_dist_matrix, PyArrayObject *__pyx_v_pred) {
  unsigned int __pyx_v_Nind;
  unsigned int __pyx_v_N;
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_j;
  unsigned int __pyx_v_k;
  unsigned int __pyx_v_j_source;
  CYTHON_UNUSED unsigned int __pyx_v_count;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d1;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d2;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_w12;
  int __pyx_v_return_pred;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indices;
  __Pyx_Buffer __pyx_pybuffer_csr_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indptr;
  __Pyx_Buffer __pyx_pybuffer_csr_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_weights;
  __Pyx_Buffer __pyx_pybuffer_csr_weights;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dist_matrix;
  __Pyx_Buffer __pyx_pybuffer_dist_matrix;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_pred;
  __Pyx_Buffer __pyx_pybuffer_pred;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_source_indices;
  __Pyx_Buffer __pyx_pybuffer_source_indices;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  unsigned int __pyx_t_4;
  size_t __pyx_t_5;
  long __pyx_t_6;
  unsigned int __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  size_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_t_12;
  size_t __pyx_t_13;
  size_t __pyx_t_14;
  size_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_t_17;
  size_t __pyx_t_18;
  size_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  size_t __pyx_t_21;
  size_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  size_t __pyx_t_24;
  size_t __pyx_t_25;
  size_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  size_t __pyx_t_28;
  size_t __pyx_t_29;
  size_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_bellman_ford_directed", 0);
  __pyx_pybuffer_source_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_source_indices.refcount = 0;
  __pyx_pybuffernd_source_indices.data = NULL;
  __pyx_pybuffernd_source_indices.rcbuffer = &__pyx_pybuffer_source_indices;
  __pyx_pybuffer_csr_weights.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_weights.refcount = 0;
  __pyx_pybuffernd_csr_weights.data = NULL;
  __pyx_pybuffernd_csr_weights.rcbuffer = &__pyx_pybuffer_csr_weights;
  __pyx_pybuffer_csr_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indices.refcount = 0;
  __pyx_pybuffernd_csr_indices.data = NULL;
  __pyx_pybuffernd_csr_indices.rcbuffer = &__pyx_pybuffer_csr_indices;
  __pyx_pybuffer_csr_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indptr.refcount = 0;
  __pyx_pybuffernd_csr_indptr.data = NULL;
  __pyx_pybuffernd_csr_indptr.rcbuffer = &__pyx_pybuffer_csr_indptr;
  __pyx_pybuffer_dist_matrix.pybuffer.buf = NULL;
  __pyx_pybuffer_dist_matrix.refcount = 0;
  __pyx_pybuffernd_dist_matrix.data = NULL;
  __pyx_pybuffernd_dist_matrix.rcbuffer = &__pyx_pybuffer_dist_matrix;
  __pyx_pybuffer_pred.pybuffer.buf = NULL;
  __pyx_pybuffer_pred.refcount = 0;
  __pyx_pybuffernd_pred.data = NULL;
  __pyx_pybuffernd_pred.rcbuffer = &__pyx_pybuffer_pred;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_source_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 727; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_source_indices.diminfo[0].strides = __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_source_indices.diminfo[0].shape = __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_weights, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 727; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_weights.diminfo[0].strides = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_weights.diminfo[0].shape = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 727; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indices.diminfo[0].strides = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indices.diminfo[0].shape = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indptr, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 727; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indptr.diminfo[0].strides = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indptr.diminfo[0].shape = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer, (PyObject*)__pyx_v_dist_matrix, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 727; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_dist_matrix.diminfo[0].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dist_matrix.diminfo[0].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_dist_matrix.diminfo[1].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_dist_matrix.diminfo[1].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_pred.rcbuffer->pybuffer, (PyObject*)__pyx_v_pred, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 727; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_pred.diminfo[0].strides = __pyx_pybuffernd_pred.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_pred.diminfo[0].shape = __pyx_pybuffernd_pred.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_pred.diminfo[1].strides = __pyx_pybuffernd_pred.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_pred.diminfo[1].shape = __pyx_pybuffernd_pred.rcbuffer->pybuffer.shape[1];

  /* "scipy/sparse/csgraph/_shortest_path.pyx":735
 *             np.ndarray[ITYPE_t, ndim=2, mode='c'] pred):
 *     global DTYPE_EPS
 *     cdef unsigned int Nind = dist_matrix.shape[0]             # <<<<<<<<<<<<<<
 *     cdef unsigned int N = dist_matrix.shape[1]
 *     cdef unsigned int i, j, k, j_source, count
 */
  __pyx_v_Nind = (__pyx_v_dist_matrix->dimensions[0]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":736
 *     global DTYPE_EPS
 *     cdef unsigned int Nind = dist_matrix.shape[0]
 *     cdef unsigned int N = dist_matrix.shape[1]             # <<<<<<<<<<<<<<
 *     cdef unsigned int i, j, k, j_source, count
 * 
 */
  __pyx_v_N = (__pyx_v_dist_matrix->dimensions[1]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":741
 *     cdef DTYPE_t d1, d2, w12
 * 
 *     cdef int return_pred = (pred.size > 0)             # <<<<<<<<<<<<<<
 * 
 *     for i from 0 <= i < Nind:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_pred), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 741; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 741; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 741; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_return_pred = __pyx_t_3;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":743
 *     cdef int return_pred = (pred.size > 0)
 * 
 *     for i from 0 <= i < Nind:             # <<<<<<<<<<<<<<
 *         j_source = source_indices[i]
 * 
 */
  __pyx_t_4 = __pyx_v_Nind;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_4; __pyx_v_i++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":744
 * 
 *     for i from 0 <= i < Nind:
 *         j_source = source_indices[i]             # <<<<<<<<<<<<<<
 * 
 *         # relax all edges N-1 times
 */
    __pyx_t_5 = __pyx_v_i;
    __pyx_t_3 = -1;
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_source_indices.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 744; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_j_source = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_source_indices.diminfo[0].strides));

    /* "scipy/sparse/csgraph/_shortest_path.pyx":747
 * 
 *         # relax all edges N-1 times
 *         for count from 0 <= count < N - 1:             # <<<<<<<<<<<<<<
 *             for j from 0 <= j < N:
 *                 d1 = dist_matrix[i, j]
 */
    __pyx_t_6 = (__pyx_v_N - 1);
    for (__pyx_v_count = 0; __pyx_v_count < __pyx_t_6; __pyx_v_count++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":748
 *         # relax all edges N-1 times
 *         for count from 0 <= count < N - 1:
 *             for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *                 d1 = dist_matrix[i, j]
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 */
      __pyx_t_7 = __pyx_v_N;
      for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_7; __pyx_v_j++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":749
 *         for count from 0 <= count < N - 1:
 *             for j from 0 <= j < N:
 *                 d1 = dist_matrix[i, j]             # <<<<<<<<<<<<<<
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                     w12 = csr_weights[k]
 */
        __pyx_t_8 = __pyx_v_i;
        __pyx_t_9 = __pyx_v_j;
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_8 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_9 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 749; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_d1 = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_8, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_9, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":750
 *             for j from 0 <= j < N:
 *                 d1 = dist_matrix[i, j]
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *                     w12 = csr_weights[k]
 *                     d2 = dist_matrix[i, csr_indices[k]]
 */
        __pyx_t_10 = __pyx_v_j;
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_10 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 750; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_t_11 = (__pyx_v_j + 1);
        __pyx_t_3 = -1;
        if (__pyx_t_11 < 0) {
          __pyx_t_11 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
          if (unlikely(__pyx_t_11 < 0)) __pyx_t_3 = 0;
        } else if (unlikely(__pyx_t_11 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 750; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_t_12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
        for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_10, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_12; __pyx_v_k++) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":751
 *                 d1 = dist_matrix[i, j]
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                     w12 = csr_weights[k]             # <<<<<<<<<<<<<<
 *                     d2 = dist_matrix[i, csr_indices[k]]
 *                     if d1 + w12 < d2:
 */
          __pyx_t_13 = __pyx_v_k;
          __pyx_t_3 = -1;
          if (unlikely(__pyx_t_13 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_3 = 0;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 751; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_v_w12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_13, __pyx_pybuffernd_csr_weights.diminfo[0].strides));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":752
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                     w12 = csr_weights[k]
 *                     d2 = dist_matrix[i, csr_indices[k]]             # <<<<<<<<<<<<<<
 *                     if d1 + w12 < d2:
 *                         dist_matrix[i, csr_indices[k]] = d1 + w12
 */
          __pyx_t_14 = __pyx_v_k;
          __pyx_t_3 = -1;
          if (unlikely(__pyx_t_14 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 752; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_t_15 = __pyx_v_i;
          __pyx_t_16 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_14, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
          __pyx_t_3 = -1;
          if (unlikely(__pyx_t_15 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
          if (__pyx_t_16 < 0) {
            __pyx_t_16 += __pyx_pybuffernd_dist_matrix.diminfo[1].shape;
            if (unlikely(__pyx_t_16 < 0)) __pyx_t_3 = 1;
          } else if (unlikely(__pyx_t_16 >= __pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 752; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_v_d2 = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_15, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_16, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":753
 *                     w12 = csr_weights[k]
 *                     d2 = dist_matrix[i, csr_indices[k]]
 *                     if d1 + w12 < d2:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, csr_indices[k]] = d1 + w12
 *                         if return_pred:
 */
          __pyx_t_17 = (((__pyx_v_d1 + __pyx_v_w12) < __pyx_v_d2) != 0);
          if (__pyx_t_17) {

            /* "scipy/sparse/csgraph/_shortest_path.pyx":754
 *                     d2 = dist_matrix[i, csr_indices[k]]
 *                     if d1 + w12 < d2:
 *                         dist_matrix[i, csr_indices[k]] = d1 + w12             # <<<<<<<<<<<<<<
 *                         if return_pred:
 *                             pred[i, csr_indices[k]] = j
 */
            __pyx_t_18 = __pyx_v_k;
            __pyx_t_3 = -1;
            if (unlikely(__pyx_t_18 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
            if (unlikely(__pyx_t_3 != -1)) {
              __Pyx_RaiseBufferIndexError(__pyx_t_3);
              {__pyx_filename = __pyx_f[0]; __pyx_lineno = 754; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            }
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_18, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
            __pyx_t_3 = -1;
            if (unlikely(__pyx_t_19 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
            if (__pyx_t_20 < 0) {
              __pyx_t_20 += __pyx_pybuffernd_dist_matrix.diminfo[1].shape;
              if (unlikely(__pyx_t_20 < 0)) __pyx_t_3 = 1;
            } else if (unlikely(__pyx_t_20 >= __pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
            if (unlikely(__pyx_t_3 != -1)) {
              __Pyx_RaiseBufferIndexError(__pyx_t_3);
              {__pyx_filename = __pyx_f[0]; __pyx_lineno = 754; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            }
            *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_20, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = (__pyx_v_d1 + __pyx_v_w12);

            /* "scipy/sparse/csgraph/_shortest_path.pyx":755
 *                     if d1 + w12 < d2:
 *                         dist_matrix[i, csr_indices[k]] = d1 + w12
 *                         if return_pred:             # <<<<<<<<<<<<<<
 *                             pred[i, csr_indices[k]] = j
 * 
 */
            __pyx_t_17 = (__pyx_v_return_pred != 0);
            if (__pyx_t_17) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":756
 *                         dist_matrix[i, csr_indices[k]] = d1 + w12
 *                         if return_pred:
 *                             pred[i, csr_indices[k]] = j             # <<<<<<<<<<<<<<
 * 
 *         # check for negative-weight cycles
 */
              __pyx_t_21 = __pyx_v_k;
              __pyx_t_3 = -1;
              if (unlikely(__pyx_t_21 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
              if (unlikely(__pyx_t_3 != -1)) {
                __Pyx_RaiseBufferIndexError(__pyx_t_3);
                {__pyx_filename = __pyx_f[0]; __pyx_lineno = 756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
              }
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_21, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
              __pyx_t_3 = -1;
              if (unlikely(__pyx_t_22 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
              if (__pyx_t_23 < 0) {
                __pyx_t_23 += __pyx_pybuffernd_pred.diminfo[1].shape;
                if (unlikely(__pyx_t_23 < 0)) __pyx_t_3 = 1;
              } else if (unlikely(__pyx_t_23 >= __pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
              if (unlikely(__pyx_t_3 != -1)) {
                __Pyx_RaiseBufferIndexError(__pyx_t_3);
                {__pyx_filename = __pyx_f[0]; __pyx_lineno = 756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
              }
              *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_22, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_23, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_v_j;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":755
 *                     if d1 + w12 < d2:
 *                         dist_matrix[i, csr_indices[k]] = d1 + w12
 *                         if return_pred:             # <<<<<<<<<<<<<<
 *                             pred[i, csr_indices[k]] = j
 * 
 */
            }

            /* "scipy/sparse/csgraph/_shortest_path.pyx":753
 *                     w12 = csr_weights[k]
 *                     d2 = dist_matrix[i, csr_indices[k]]
 *                     if d1 + w12 < d2:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, csr_indices[k]] = d1 + w12
 *                         if return_pred:
 */
          }
        }
      }
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":759
 * 
 *         # check for negative-weight cycles
 *         for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *             d1 = dist_matrix[i, j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 */
    __pyx_t_7 = __pyx_v_N;
    for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_7; __pyx_v_j++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":760
 *         # check for negative-weight cycles
 *         for j from 0 <= j < N:
 *             d1 = dist_matrix[i, j]             # <<<<<<<<<<<<<<
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]
 */
      __pyx_t_24 = __pyx_v_i;
      __pyx_t_25 = __pyx_v_j;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_24 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_25 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 760; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_v_d1 = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_24, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_25, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":761
 *         for j from 0 <= j < N:
 *             d1 = dist_matrix[i, j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *                 w12 = csr_weights[k]
 *                 d2 = dist_matrix[i, csr_indices[k]]
 */
      __pyx_t_26 = __pyx_v_j;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_26 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 761; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_27 = (__pyx_v_j + 1);
      __pyx_t_3 = -1;
      if (__pyx_t_27 < 0) {
        __pyx_t_27 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
        if (unlikely(__pyx_t_27 < 0)) __pyx_t_3 = 0;
      } else if (unlikely(__pyx_t_27 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 761; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_27, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
      for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_26, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_12; __pyx_v_k++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":762
 *             d1 = dist_matrix[i, j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]             # <<<<<<<<<<<<<<
 *                 d2 = dist_matrix[i, csr_indices[k]]
 *                 if d1 + w12 + DTYPE_EPS < d2:
 */
        __pyx_t_28 = __pyx_v_k;
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_28 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 762; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_w12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_28, __pyx_pybuffernd_csr_weights.diminfo[0].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":763
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]
 *                 d2 = dist_matrix[i, csr_indices[k]]             # <<<<<<<<<<<<<<
 *                 if d1 + w12 + DTYPE_EPS < d2:
 *                     return j_source
 */
        __pyx_t_29 = __pyx_v_k;
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_29 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 763; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_t_30 = __pyx_v_i;
        __pyx_t_31 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_29, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_30 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
        if (__pyx_t_31 < 0) {
          __pyx_t_31 += __pyx_pybuffernd_dist_matrix.diminfo[1].shape;
          if (unlikely(__pyx_t_31 < 0)) __pyx_t_3 = 1;
        } else if (unlikely(__pyx_t_31 >= __pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 763; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_d2 = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_30, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_31, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":764
 *                 w12 = csr_weights[k]
 *                 d2 = dist_matrix[i, csr_indices[k]]
 *                 if d1 + w12 + DTYPE_EPS < d2:             # <<<<<<<<<<<<<<
 *                     return j_source
 * 
 */
        __pyx_t_17 = ((((__pyx_v_d1 + __pyx_v_w12) + __pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_EPS) < __pyx_v_d2) != 0);
        if (__pyx_t_17) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":765
 *                 d2 = dist_matrix[i, csr_indices[k]]
 *                 if d1 + w12 + DTYPE_EPS < d2:
 *                     return j_source             # <<<<<<<<<<<<<<
 * 
 *     return -1
 */
          __pyx_r = __pyx_v_j_source;
          goto __pyx_L0;

          /* "scipy/sparse/csgraph/_shortest_path.pyx":764
 *                 w12 = csr_weights[k]
 *                 d2 = dist_matrix[i, csr_indices[k]]
 *                 if d1 + w12 + DTYPE_EPS < d2:             # <<<<<<<<<<<<<<
 *                     return j_source
 * 
 */
        }
      }
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":767
 *                     return j_source
 * 
 *     return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = -1;
  goto __pyx_L0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":727
 * 
 * 
 * cdef int _bellman_ford_directed(             # <<<<<<<<<<<<<<
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] source_indices,
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_pred.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_WriteUnraisable("scipy.sparse.csgraph._shortest_path._bellman_ford_directed", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_pred.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":770
 * 
 * 
 * cdef int _bellman_ford_undirected(             # <<<<<<<<<<<<<<
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] source_indices,
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 */

static int __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__bellman_ford_undirected(PyArrayObject *__pyx_v_source_indices, PyArrayObject *__pyx_v_csr_weights, PyArrayObject *__pyx_v_csr_indices, PyArrayObject *__pyx_v_csr_indptr, PyArrayObject *__pyx_v_dist_matrix, PyArrayObject *__pyx_v_pred) {
  unsigned int __pyx_v_Nind;
  unsigned int __pyx_v_N;
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_j;
  unsigned int __pyx_v_k;
  unsigned int __pyx_v_j_source;
  unsigned int __pyx_v_ind_k;
  CYTHON_UNUSED unsigned int __pyx_v_count;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d1;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d2;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_w12;
  int __pyx_v_return_pred;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indices;
  __Pyx_Buffer __pyx_pybuffer_csr_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indptr;
  __Pyx_Buffer __pyx_pybuffer_csr_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_weights;
  __Pyx_Buffer __pyx_pybuffer_csr_weights;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dist_matrix;
  __Pyx_Buffer __pyx_pybuffer_dist_matrix;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_pred;
  __Pyx_Buffer __pyx_pybuffer_pred;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_source_indices;
  __Pyx_Buffer __pyx_pybuffer_source_indices;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  unsigned int __pyx_t_4;
  size_t __pyx_t_5;
  long __pyx_t_6;
  unsigned int __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  size_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_t_12;
  size_t __pyx_t_13;
  size_t __pyx_t_14;
  size_t __pyx_t_15;
  size_t __pyx_t_16;
  int __pyx_t_17;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_t_18;
  size_t __pyx_t_19;
  size_t __pyx_t_20;
  size_t __pyx_t_21;
  size_t __pyx_t_22;
  size_t __pyx_t_23;
  size_t __pyx_t_24;
  size_t __pyx_t_25;
  size_t __pyx_t_26;
  size_t __pyx_t_27;
  size_t __pyx_t_28;
  size_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  size_t __pyx_t_31;
  size_t __pyx_t_32;
  size_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_bellman_ford_undirected", 0);
  __pyx_pybuffer_source_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_source_indices.refcount = 0;
  __pyx_pybuffernd_source_indices.data = NULL;
  __pyx_pybuffernd_source_indices.rcbuffer = &__pyx_pybuffer_source_indices;
  __pyx_pybuffer_csr_weights.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_weights.refcount = 0;
  __pyx_pybuffernd_csr_weights.data = NULL;
  __pyx_pybuffernd_csr_weights.rcbuffer = &__pyx_pybuffer_csr_weights;
  __pyx_pybuffer_csr_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indices.refcount = 0;
  __pyx_pybuffernd_csr_indices.data = NULL;
  __pyx_pybuffernd_csr_indices.rcbuffer = &__pyx_pybuffer_csr_indices;
  __pyx_pybuffer_csr_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indptr.refcount = 0;
  __pyx_pybuffernd_csr_indptr.data = NULL;
  __pyx_pybuffernd_csr_indptr.rcbuffer = &__pyx_pybuffer_csr_indptr;
  __pyx_pybuffer_dist_matrix.pybuffer.buf = NULL;
  __pyx_pybuffer_dist_matrix.refcount = 0;
  __pyx_pybuffernd_dist_matrix.data = NULL;
  __pyx_pybuffernd_dist_matrix.rcbuffer = &__pyx_pybuffer_dist_matrix;
  __pyx_pybuffer_pred.pybuffer.buf = NULL;
  __pyx_pybuffer_pred.refcount = 0;
  __pyx_pybuffernd_pred.data = NULL;
  __pyx_pybuffernd_pred.rcbuffer = &__pyx_pybuffer_pred;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_source_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 770; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_source_indices.diminfo[0].strides = __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_source_indices.diminfo[0].shape = __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_weights, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 770; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_weights.diminfo[0].strides = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_weights.diminfo[0].shape = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 770; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indices.diminfo[0].strides = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indices.diminfo[0].shape = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indptr, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 770; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indptr.diminfo[0].strides = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indptr.diminfo[0].shape = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer, (PyObject*)__pyx_v_dist_matrix, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 770; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_dist_matrix.diminfo[0].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dist_matrix.diminfo[0].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_dist_matrix.diminfo[1].strides = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_dist_matrix.diminfo[1].shape = __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_pred.rcbuffer->pybuffer, (PyObject*)__pyx_v_pred, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 770; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_pred.diminfo[0].strides = __pyx_pybuffernd_pred.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_pred.diminfo[0].shape = __pyx_pybuffernd_pred.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_pred.diminfo[1].strides = __pyx_pybuffernd_pred.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_pred.diminfo[1].shape = __pyx_pybuffernd_pred.rcbuffer->pybuffer.shape[1];

  /* "scipy/sparse/csgraph/_shortest_path.pyx":778
 *             np.ndarray[ITYPE_t, ndim=2, mode='c'] pred):
 *     global DTYPE_EPS
 *     cdef unsigned int Nind = dist_matrix.shape[0]             # <<<<<<<<<<<<<<
 *     cdef unsigned int N = dist_matrix.shape[1]
 *     cdef unsigned int i, j, k, j_source, ind_k, count
 */
  __pyx_v_Nind = (__pyx_v_dist_matrix->dimensions[0]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":779
 *     global DTYPE_EPS
 *     cdef unsigned int Nind = dist_matrix.shape[0]
 *     cdef unsigned int N = dist_matrix.shape[1]             # <<<<<<<<<<<<<<
 *     cdef unsigned int i, j, k, j_source, ind_k, count
 * 
 */
  __pyx_v_N = (__pyx_v_dist_matrix->dimensions[1]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":784
 *     cdef DTYPE_t d1, d2, w12
 * 
 *     cdef int return_pred = (pred.size > 0)             # <<<<<<<<<<<<<<
 * 
 *     for i from 0 <= i < Nind:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_pred), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_return_pred = __pyx_t_3;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":786
 *     cdef int return_pred = (pred.size > 0)
 * 
 *     for i from 0 <= i < Nind:             # <<<<<<<<<<<<<<
 *         j_source = source_indices[i]
 * 
 */
  __pyx_t_4 = __pyx_v_Nind;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_4; __pyx_v_i++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":787
 * 
 *     for i from 0 <= i < Nind:
 *         j_source = source_indices[i]             # <<<<<<<<<<<<<<
 * 
 *         # relax all edges N-1 times
 */
    __pyx_t_5 = __pyx_v_i;
    __pyx_t_3 = -1;
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_source_indices.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 787; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_j_source = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_source_indices.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_source_indices.diminfo[0].strides));

    /* "scipy/sparse/csgraph/_shortest_path.pyx":790
 * 
 *         # relax all edges N-1 times
 *         for count from 0 <= count < N - 1:             # <<<<<<<<<<<<<<
 *             for j from 0 <= j < N:
 *                 d1 = dist_matrix[i, j]
 */
    __pyx_t_6 = (__pyx_v_N - 1);
    for (__pyx_v_count = 0; __pyx_v_count < __pyx_t_6; __pyx_v_count++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":791
 *         # relax all edges N-1 times
 *         for count from 0 <= count < N - 1:
 *             for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *                 d1 = dist_matrix[i, j]
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 */
      __pyx_t_7 = __pyx_v_N;
      for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_7; __pyx_v_j++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":792
 *         for count from 0 <= count < N - 1:
 *             for j from 0 <= j < N:
 *                 d1 = dist_matrix[i, j]             # <<<<<<<<<<<<<<
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                     w12 = csr_weights[k]
 */
        __pyx_t_8 = __pyx_v_i;
        __pyx_t_9 = __pyx_v_j;
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_8 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_9 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_d1 = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_8, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_9, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":793
 *             for j from 0 <= j < N:
 *                 d1 = dist_matrix[i, j]
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *                     w12 = csr_weights[k]
 *                     ind_k = csr_indices[k]
 */
        __pyx_t_10 = __pyx_v_j;
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_10 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_t_11 = (__pyx_v_j + 1);
        __pyx_t_3 = -1;
        if (__pyx_t_11 < 0) {
          __pyx_t_11 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
          if (unlikely(__pyx_t_11 < 0)) __pyx_t_3 = 0;
        } else if (unlikely(__pyx_t_11 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_t_12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
        for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_10, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_12; __pyx_v_k++) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":794
 *                 d1 = dist_matrix[i, j]
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                     w12 = csr_weights[k]             # <<<<<<<<<<<<<<
 *                     ind_k = csr_indices[k]
 *                     d2 = dist_matrix[i, ind_k]
 */
          __pyx_t_13 = __pyx_v_k;
          __pyx_t_3 = -1;
          if (unlikely(__pyx_t_13 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_3 = 0;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_v_w12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_13, __pyx_pybuffernd_csr_weights.diminfo[0].strides));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":795
 *                 for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                     w12 = csr_weights[k]
 *                     ind_k = csr_indices[k]             # <<<<<<<<<<<<<<
 *                     d2 = dist_matrix[i, ind_k]
 *                     if d1 + w12 < d2:
 */
          __pyx_t_14 = __pyx_v_k;
          __pyx_t_3 = -1;
          if (unlikely(__pyx_t_14 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_v_ind_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_14, __pyx_pybuffernd_csr_indices.diminfo[0].strides));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":796
 *                     w12 = csr_weights[k]
 *                     ind_k = csr_indices[k]
 *                     d2 = dist_matrix[i, ind_k]             # <<<<<<<<<<<<<<
 *                     if d1 + w12 < d2:
 *                         dist_matrix[i, ind_k] = d2 = d1 + w12
 */
          __pyx_t_15 = __pyx_v_i;
          __pyx_t_16 = __pyx_v_ind_k;
          __pyx_t_3 = -1;
          if (unlikely(__pyx_t_15 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
          if (unlikely(__pyx_t_16 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
          if (unlikely(__pyx_t_3 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_3);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_v_d2 = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_15, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_16, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

          /* "scipy/sparse/csgraph/_shortest_path.pyx":797
 *                     ind_k = csr_indices[k]
 *                     d2 = dist_matrix[i, ind_k]
 *                     if d1 + w12 < d2:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, ind_k] = d2 = d1 + w12
 *                         if return_pred:
 */
          __pyx_t_17 = (((__pyx_v_d1 + __pyx_v_w12) < __pyx_v_d2) != 0);
          if (__pyx_t_17) {

            /* "scipy/sparse/csgraph/_shortest_path.pyx":798
 *                     d2 = dist_matrix[i, ind_k]
 *                     if d1 + w12 < d2:
 *                         dist_matrix[i, ind_k] = d2 = d1 + w12             # <<<<<<<<<<<<<<
 *                         if return_pred:
 *                             pred[i, ind_k] = j
 */
            __pyx_t_18 = (__pyx_v_d1 + __pyx_v_w12);
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_ind_k;
            __pyx_t_3 = -1;
            if (unlikely(__pyx_t_19 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
            if (unlikely(__pyx_t_20 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
            if (unlikely(__pyx_t_3 != -1)) {
              __Pyx_RaiseBufferIndexError(__pyx_t_3);
              {__pyx_filename = __pyx_f[0]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            }
            *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_20, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = __pyx_t_18;
            __pyx_v_d2 = __pyx_t_18;

            /* "scipy/sparse/csgraph/_shortest_path.pyx":799
 *                     if d1 + w12 < d2:
 *                         dist_matrix[i, ind_k] = d2 = d1 + w12
 *                         if return_pred:             # <<<<<<<<<<<<<<
 *                             pred[i, ind_k] = j
 *                     if d2 + w12 < d1:
 */
            __pyx_t_17 = (__pyx_v_return_pred != 0);
            if (__pyx_t_17) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":800
 *                         dist_matrix[i, ind_k] = d2 = d1 + w12
 *                         if return_pred:
 *                             pred[i, ind_k] = j             # <<<<<<<<<<<<<<
 *                     if d2 + w12 < d1:
 *                         dist_matrix[i, j] = d1 = d2 + w12
 */
              __pyx_t_21 = __pyx_v_i;
              __pyx_t_22 = __pyx_v_ind_k;
              __pyx_t_3 = -1;
              if (unlikely(__pyx_t_21 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
              if (unlikely(__pyx_t_22 >= (size_t)__pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
              if (unlikely(__pyx_t_3 != -1)) {
                __Pyx_RaiseBufferIndexError(__pyx_t_3);
                {__pyx_filename = __pyx_f[0]; __pyx_lineno = 800; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
              }
              *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_21, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_22, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_v_j;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":799
 *                     if d1 + w12 < d2:
 *                         dist_matrix[i, ind_k] = d2 = d1 + w12
 *                         if return_pred:             # <<<<<<<<<<<<<<
 *                             pred[i, ind_k] = j
 *                     if d2 + w12 < d1:
 */
            }

            /* "scipy/sparse/csgraph/_shortest_path.pyx":797
 *                     ind_k = csr_indices[k]
 *                     d2 = dist_matrix[i, ind_k]
 *                     if d1 + w12 < d2:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, ind_k] = d2 = d1 + w12
 *                         if return_pred:
 */
          }

          /* "scipy/sparse/csgraph/_shortest_path.pyx":801
 *                         if return_pred:
 *                             pred[i, ind_k] = j
 *                     if d2 + w12 < d1:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, j] = d1 = d2 + w12
 *                         if return_pred:
 */
          __pyx_t_17 = (((__pyx_v_d2 + __pyx_v_w12) < __pyx_v_d1) != 0);
          if (__pyx_t_17) {

            /* "scipy/sparse/csgraph/_shortest_path.pyx":802
 *                             pred[i, ind_k] = j
 *                     if d2 + w12 < d1:
 *                         dist_matrix[i, j] = d1 = d2 + w12             # <<<<<<<<<<<<<<
 *                         if return_pred:
 *                             pred[i, j] = ind_k
 */
            __pyx_t_18 = (__pyx_v_d2 + __pyx_v_w12);
            __pyx_t_23 = __pyx_v_i;
            __pyx_t_24 = __pyx_v_j;
            __pyx_t_3 = -1;
            if (unlikely(__pyx_t_23 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
            if (unlikely(__pyx_t_24 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
            if (unlikely(__pyx_t_3 != -1)) {
              __Pyx_RaiseBufferIndexError(__pyx_t_3);
              {__pyx_filename = __pyx_f[0]; __pyx_lineno = 802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            }
            *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_23, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_24, __pyx_pybuffernd_dist_matrix.diminfo[1].strides) = __pyx_t_18;
            __pyx_v_d1 = __pyx_t_18;

            /* "scipy/sparse/csgraph/_shortest_path.pyx":803
 *                     if d2 + w12 < d1:
 *                         dist_matrix[i, j] = d1 = d2 + w12
 *                         if return_pred:             # <<<<<<<<<<<<<<
 *                             pred[i, j] = ind_k
 * 
 */
            __pyx_t_17 = (__pyx_v_return_pred != 0);
            if (__pyx_t_17) {

              /* "scipy/sparse/csgraph/_shortest_path.pyx":804
 *                         dist_matrix[i, j] = d1 = d2 + w12
 *                         if return_pred:
 *                             pred[i, j] = ind_k             # <<<<<<<<<<<<<<
 * 
 *         # check for negative-weight cycles
 */
              __pyx_t_25 = __pyx_v_i;
              __pyx_t_26 = __pyx_v_j;
              __pyx_t_3 = -1;
              if (unlikely(__pyx_t_25 >= (size_t)__pyx_pybuffernd_pred.diminfo[0].shape)) __pyx_t_3 = 0;
              if (unlikely(__pyx_t_26 >= (size_t)__pyx_pybuffernd_pred.diminfo[1].shape)) __pyx_t_3 = 1;
              if (unlikely(__pyx_t_3 != -1)) {
                __Pyx_RaiseBufferIndexError(__pyx_t_3);
                {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
              }
              *__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_pred.rcbuffer->pybuffer.buf, __pyx_t_25, __pyx_pybuffernd_pred.diminfo[0].strides, __pyx_t_26, __pyx_pybuffernd_pred.diminfo[1].strides) = __pyx_v_ind_k;

              /* "scipy/sparse/csgraph/_shortest_path.pyx":803
 *                     if d2 + w12 < d1:
 *                         dist_matrix[i, j] = d1 = d2 + w12
 *                         if return_pred:             # <<<<<<<<<<<<<<
 *                             pred[i, j] = ind_k
 * 
 */
            }

            /* "scipy/sparse/csgraph/_shortest_path.pyx":801
 *                         if return_pred:
 *                             pred[i, ind_k] = j
 *                     if d2 + w12 < d1:             # <<<<<<<<<<<<<<
 *                         dist_matrix[i, j] = d1 = d2 + w12
 *                         if return_pred:
 */
          }
        }
      }
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":807
 * 
 *         # check for negative-weight cycles
 *         for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *             d1 = dist_matrix[i, j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 */
    __pyx_t_7 = __pyx_v_N;
    for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_7; __pyx_v_j++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":808
 *         # check for negative-weight cycles
 *         for j from 0 <= j < N:
 *             d1 = dist_matrix[i, j]             # <<<<<<<<<<<<<<
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]
 */
      __pyx_t_27 = __pyx_v_i;
      __pyx_t_28 = __pyx_v_j;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_27 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_28 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_v_d1 = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_27, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_28, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":809
 *         for j from 0 <= j < N:
 *             d1 = dist_matrix[i, j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *                 w12 = csr_weights[k]
 *                 d2 = dist_matrix[i, csr_indices[k]]
 */
      __pyx_t_29 = __pyx_v_j;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_29 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_30 = (__pyx_v_j + 1);
      __pyx_t_3 = -1;
      if (__pyx_t_30 < 0) {
        __pyx_t_30 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
        if (unlikely(__pyx_t_30 < 0)) __pyx_t_3 = 0;
      } else if (unlikely(__pyx_t_30 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_30, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
      for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_29, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_12; __pyx_v_k++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":810
 *             d1 = dist_matrix[i, j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]             # <<<<<<<<<<<<<<
 *                 d2 = dist_matrix[i, csr_indices[k]]
 *                 if abs(d2 - d1) > w12 + DTYPE_EPS:
 */
        __pyx_t_31 = __pyx_v_k;
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_31 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_w12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_31, __pyx_pybuffernd_csr_weights.diminfo[0].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":811
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]
 *                 d2 = dist_matrix[i, csr_indices[k]]             # <<<<<<<<<<<<<<
 *                 if abs(d2 - d1) > w12 + DTYPE_EPS:
 *                     return j_source
 */
        __pyx_t_32 = __pyx_v_k;
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_32 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_t_33 = __pyx_v_i;
        __pyx_t_34 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_32, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
        __pyx_t_3 = -1;
        if (unlikely(__pyx_t_33 >= (size_t)__pyx_pybuffernd_dist_matrix.diminfo[0].shape)) __pyx_t_3 = 0;
        if (__pyx_t_34 < 0) {
          __pyx_t_34 += __pyx_pybuffernd_dist_matrix.diminfo[1].shape;
          if (unlikely(__pyx_t_34 < 0)) __pyx_t_3 = 1;
        } else if (unlikely(__pyx_t_34 >= __pyx_pybuffernd_dist_matrix.diminfo[1].shape)) __pyx_t_3 = 1;
        if (unlikely(__pyx_t_3 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_3);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_d2 = (*__Pyx_BufPtrCContig2d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer.buf, __pyx_t_33, __pyx_pybuffernd_dist_matrix.diminfo[0].strides, __pyx_t_34, __pyx_pybuffernd_dist_matrix.diminfo[1].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":812
 *                 w12 = csr_weights[k]
 *                 d2 = dist_matrix[i, csr_indices[k]]
 *                 if abs(d2 - d1) > w12 + DTYPE_EPS:             # <<<<<<<<<<<<<<
 *                     return j_source
 * 
 */
        __pyx_t_17 = ((fabs((__pyx_v_d2 - __pyx_v_d1)) > (__pyx_v_w12 + __pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_EPS)) != 0);
        if (__pyx_t_17) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":813
 *                 d2 = dist_matrix[i, csr_indices[k]]
 *                 if abs(d2 - d1) > w12 + DTYPE_EPS:
 *                     return j_source             # <<<<<<<<<<<<<<
 * 
 *     return -1
 */
          __pyx_r = __pyx_v_j_source;
          goto __pyx_L0;

          /* "scipy/sparse/csgraph/_shortest_path.pyx":812
 *                 w12 = csr_weights[k]
 *                 d2 = dist_matrix[i, csr_indices[k]]
 *                 if abs(d2 - d1) > w12 + DTYPE_EPS:             # <<<<<<<<<<<<<<
 *                     return j_source
 * 
 */
        }
      }
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":815
 *                     return j_source
 * 
 *     return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = -1;
  goto __pyx_L0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":770
 * 
 * 
 * cdef int _bellman_ford_undirected(             # <<<<<<<<<<<<<<
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] source_indices,
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_pred.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_WriteUnraisable("scipy.sparse.csgraph._shortest_path._bellman_ford_undirected", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_matrix.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_pred.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_source_indices.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":818
 * 
 * 
 * def johnson(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *             return_predecessors=False,
 *             unweighted=False):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_9johnson(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_8johnson[] = "\n    johnson(csgraph, directed=True, indices=None, return_predecessors=False,\n            unweighted=False)\n\n    Compute the shortest path lengths using Johnson's algorithm.\n\n    Johnson's algorithm combines the Bellman-Ford algorithm and Dijkstra's\n    algorithm to quickly find shortest paths in a way that is robust to\n    the presence of negative cycles.  If a negative cycle is detected,\n    an error is raised.  For graphs without negative edge weights,\n    dijkstra() may be faster.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array, matrix, or sparse matrix, 2 dimensions\n        The N x N array of distances representing the input graph.\n    directed : bool, optional\n        If True (default), then find the shortest path on a directed graph:\n        only move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i]\n    indices : array_like or int, optional\n        if specified, only compute the paths for the points at the given\n        indices.\n    return_predecessors : bool, optional\n        If True, return the size (N, N) predecesor matrix\n    unweighted : bool, optional\n        If True, then find unweighted distances.  That is, rather than finding\n        the path between each point such that the sum of weights is minimized,\n        find the path such that the number of edges is minimized.\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between graph nodes. dist_matrix[i,j]\n        gives the shortest distance from point i to point j along the graph.\n\n    predecessors : ndarray\n        Returned only if return_predecessors == True.\n        The N x N matrix of predecessors, which can be used to reconstruct\n        the shortest paths.  Row i of the predecessor matrix contains\n        informati""on on the shortest paths from point i: each entry\n        predecessors[i, j] gives the index of the previous node in the\n        path from point i to point j.  If no path exists between point\n        i and j, then predecessors[i, j] = -9999\n\n    Raises\n    ------\n    NegativeCycleError:\n        if there are negative cycles in the graph\n\n    Notes\n    -----\n    This routine is specially designed for graphs with negative edge weights.\n    If all edge weights are positive, then Dijkstra's algorithm is a better\n    choice.\n    ";
static PyMethodDef __pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_9johnson = {"johnson", (PyCFunction)__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_9johnson, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6sparse_7csgraph_14_shortest_path_8johnson};
static PyObject *__pyx_pw_5scipy_6sparse_7csgraph_14_shortest_path_9johnson(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_csgraph = 0;
  PyObject *__pyx_v_directed = 0;
  PyObject *__pyx_v_indices = 0;
  PyObject *__pyx_v_return_predecessors = 0;
  PyObject *__pyx_v_unweighted = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("johnson (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_csgraph,&__pyx_n_s_directed,&__pyx_n_s_indices,&__pyx_n_s_return_predecessors,&__pyx_n_s_unweighted,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[1] = ((PyObject *)Py_True);
    values[2] = ((PyObject *)Py_None);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":819
 * 
 * def johnson(csgraph, directed=True, indices=None,
 *             return_predecessors=False,             # <<<<<<<<<<<<<<
 *             unweighted=False):
 *     """
 */
    values[3] = ((PyObject *)Py_False);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":820
 * def johnson(csgraph, directed=True, indices=None,
 *             return_predecessors=False,
 *             unweighted=False):             # <<<<<<<<<<<<<<
 *     """
 *     johnson(csgraph, directed=True, indices=None, return_predecessors=False,
 */
    values[4] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_csgraph)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_directed);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_indices);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_predecessors);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_unweighted);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "johnson") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_csgraph = values[0];
    __pyx_v_directed = values[1];
    __pyx_v_indices = values[2];
    __pyx_v_return_predecessors = values[3];
    __pyx_v_unweighted = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("johnson", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.johnson", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_8johnson(__pyx_self, __pyx_v_csgraph, __pyx_v_directed, __pyx_v_indices, __pyx_v_return_predecessors, __pyx_v_unweighted);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":818
 * 
 * 
 * def johnson(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *             return_predecessors=False,
 *             unweighted=False):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6sparse_7csgraph_14_shortest_path_8johnson(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_csgraph, PyObject *__pyx_v_directed, PyObject *__pyx_v_indices, PyObject *__pyx_v_return_predecessors, PyObject *__pyx_v_unweighted) {
  PyObject *__pyx_v_N = NULL;
  PyObject *__pyx_v_return_shape = NULL;
  PyObject *__pyx_v_dist_matrix = NULL;
  PyObject *__pyx_v_predecessor_matrix = NULL;
  PyObject *__pyx_v_dist_array = NULL;
  PyObject *__pyx_v_csr_data = NULL;
  PyObject *__pyx_v_ret = NULL;
  PyObject *__pyx_v_csgraphT = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("johnson", 0);
  __Pyx_INCREF(__pyx_v_csgraph);
  __Pyx_INCREF(__pyx_v_indices);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":883
 *     #------------------------------
 *     # if unweighted, there are no negative weights: we just use dijkstra
 *     if unweighted:             # <<<<<<<<<<<<<<
 *         return dijkstra(csgraph, directed, indices,
 *                         return_predecessors, unweighted)
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_unweighted); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 883; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":884
 *     # if unweighted, there are no negative weights: we just use dijkstra
 *     if unweighted:
 *         return dijkstra(csgraph, directed, indices,             # <<<<<<<<<<<<<<
 *                         return_predecessors, unweighted)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_dijkstra); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 884; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":885
 *     if unweighted:
 *         return dijkstra(csgraph, directed, indices,
 *                         return_predecessors, unweighted)             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    __pyx_t_6 = PyTuple_New(5+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 884; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_csgraph);
    __Pyx_GIVEREF(__pyx_v_csgraph);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_csgraph);
    __Pyx_INCREF(__pyx_v_directed);
    __Pyx_GIVEREF(__pyx_v_directed);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_directed);
    __Pyx_INCREF(__pyx_v_indices);
    __Pyx_GIVEREF(__pyx_v_indices);
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_indices);
    __Pyx_INCREF(__pyx_v_return_predecessors);
    __Pyx_GIVEREF(__pyx_v_return_predecessors);
    PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_return_predecessors);
    __Pyx_INCREF(__pyx_v_unweighted);
    __Pyx_GIVEREF(__pyx_v_unweighted);
    PyTuple_SET_ITEM(__pyx_t_6, 4+__pyx_t_5, __pyx_v_unweighted);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 884; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":883
 *     #------------------------------
 *     # if unweighted, there are no negative weights: we just use dijkstra
 *     if unweighted:             # <<<<<<<<<<<<<<
 *         return dijkstra(csgraph, directed, indices,
 *                         return_predecessors, unweighted)
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":889
 *     #------------------------------
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                              dense_output=False)
 *     N = csgraph.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_validate_graph); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_csgraph);
  __Pyx_GIVEREF(__pyx_v_csgraph);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_csgraph);
  __Pyx_INCREF(__pyx_v_directed);
  __Pyx_GIVEREF(__pyx_v_directed);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_directed);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":890
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,
 *                              dense_output=False)             # <<<<<<<<<<<<<<
 *     N = csgraph.shape[0]
 * 
 */
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dense_output, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":889
 *     #------------------------------
 *     # validate csgraph and convert to csr matrix
 *     csgraph = validate_graph(csgraph, directed, DTYPE,             # <<<<<<<<<<<<<<
 *                              dense_output=False)
 *     N = csgraph.shape[0]
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_csgraph, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":891
 *     csgraph = validate_graph(csgraph, directed, DTYPE,
 *                              dense_output=False)
 *     N = csgraph.shape[0]             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 891; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_N = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":895
 *     #------------------------------
 *     # initialize/validate indices
 *     if indices is None:             # <<<<<<<<<<<<<<
 *         indices = np.arange(N, dtype=ITYPE)
 *         return_shape = indices.shape + (N,)
 */
  __pyx_t_1 = (__pyx_v_indices == Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":896
 *     # initialize/validate indices
 *     if indices is None:
 *         indices = np.arange(N, dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         return_shape = indices.shape + (N,)
 *     else:
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_arange); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_N);
    __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":897
 *     if indices is None:
 *         indices = np.arange(N, dtype=ITYPE)
 *         return_shape = indices.shape + (N,)             # <<<<<<<<<<<<<<
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_shape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 897; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 897; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_N);
    __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_t_6); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 897; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_return_shape = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":895
 *     #------------------------------
 *     # initialize/validate indices
 *     if indices is None:             # <<<<<<<<<<<<<<
 *         indices = np.arange(N, dtype=ITYPE)
 *         return_shape = indices.shape + (N,)
 */
    goto __pyx_L4;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":899
 *         return_shape = indices.shape + (N,)
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         return_shape = indices.shape + (N,)
 *         indices = np.atleast_1d(indices).reshape(-1)
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_indices);
    __Pyx_GIVEREF(__pyx_v_indices);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_indices);
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":900
 *     else:
 *         indices = np.array(indices, order='C', dtype=ITYPE)
 *         return_shape = indices.shape + (N,)             # <<<<<<<<<<<<<<
 *         indices = np.atleast_1d(indices).reshape(-1)
 *         indices[indices < 0] += N
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_N);
    __pyx_t_3 = PyNumber_Add(__pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_return_shape = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":901
 *         indices = np.array(indices, order='C', dtype=ITYPE)
 *         return_shape = indices.shape + (N,)
 *         indices = np.atleast_1d(indices).reshape(-1)             # <<<<<<<<<<<<<<
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_atleast_1d); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_indices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(__pyx_v_indices);
      __Pyx_GIVEREF(__pyx_v_indices);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_indices);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_indices, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":902
 *         return_shape = indices.shape + (N,)
 *         indices = np.atleast_1d(indices).reshape(-1)
 *         indices[indices < 0] += N             # <<<<<<<<<<<<<<
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_indices, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 902; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = PyObject_GetItem(__pyx_v_indices, __pyx_t_3); if (unlikely(__pyx_t_4 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 902; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyNumber_InPlaceAdd(__pyx_t_4, __pyx_v_N); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 902; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_indices, __pyx_t_3, __pyx_t_6) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 902; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":903
 *         indices = np.atleast_1d(indices).reshape(-1)
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):             # <<<<<<<<<<<<<<
 *             raise ValueError("indices out of range 0...N")
 * 
 */
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_any); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyObject_RichCompare(__pyx_v_indices, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_1) {
    } else {
      __pyx_t_7 = __pyx_t_1;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_any); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_indices, __pyx_v_N, Py_GE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_6 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = __pyx_t_1;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_7) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":904
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 904; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 904; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "scipy/sparse/csgraph/_shortest_path.pyx":903
 *         indices = np.atleast_1d(indices).reshape(-1)
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):             # <<<<<<<<<<<<<<
 *             raise ValueError("indices out of range 0...N")
 * 
 */
    }
  }
  __pyx_L4:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":908
 *     #------------------------------
 *     # initialize dist_matrix for output
 *     dist_matrix = np.empty((len(indices), N), dtype=DTYPE)             # <<<<<<<<<<<<<<
 *     dist_matrix.fill(np.inf)
 *     dist_matrix[np.arange(len(indices)), indices] = 0
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_N);
  __Pyx_GIVEREF(__pyx_v_N);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_N);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_dist_matrix = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":909
 *     # initialize dist_matrix for output
 *     dist_matrix = np.empty((len(indices), N), dtype=DTYPE)
 *     dist_matrix.fill(np.inf)             # <<<<<<<<<<<<<<
 *     dist_matrix[np.arange(len(indices)), indices] = 0
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_fill); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 909; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 909; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_inf); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 909; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 909; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 909; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 909; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":910
 *     dist_matrix = np.empty((len(indices), N), dtype=DTYPE)
 *     dist_matrix.fill(np.inf)
 *     dist_matrix[np.arange(len(indices)), indices] = 0             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_arange); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_8) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_8); __pyx_t_8 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_indices);
  __Pyx_GIVEREF(__pyx_v_indices);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_indices);
  __pyx_t_4 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_dist_matrix, __pyx_t_6, __pyx_int_0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":914
 *     #------------------------------
 *     # initialize predecessors for output
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_return_predecessors); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_7) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":915
 *     # initialize predecessors for output
 *     if return_predecessors:
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         predecessor_matrix.fill(NULL_IDX)
 *     else:
 */
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_5 = PyObject_Length(__pyx_v_indices); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_N);
    __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_predecessor_matrix = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":916
 *     if return_predecessors:
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)             # <<<<<<<<<<<<<<
 *     else:
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_predecessor_matrix, __pyx_n_s_fill); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 916; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyInt_From_npy_int32(__pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_NULL_IDX); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 916; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 916; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 916; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 916; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":914
 *     #------------------------------
 *     # initialize predecessors for output
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         predecessor_matrix = np.empty((len(indices), N), dtype=ITYPE)
 *         predecessor_matrix.fill(NULL_IDX)
 */
    goto __pyx_L8;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":918
 *         predecessor_matrix.fill(NULL_IDX)
 *     else:
 *         predecessor_matrix = np.empty((0, N), dtype=ITYPE)             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_GIVEREF(__pyx_int_0);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_int_0);
    __Pyx_INCREF(__pyx_v_N);
    __Pyx_GIVEREF(__pyx_v_N);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_N);
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, __pyx_t_2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_predecessor_matrix = __pyx_t_6;
    __pyx_t_6 = 0;
  }
  __pyx_L8:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":922
 *     #------------------------------
 *     # initialize distance array
 *     dist_array = np.zeros(N, dtype=DTYPE)             # <<<<<<<<<<<<<<
 * 
 *     csr_data = csgraph.data.copy()
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_N);
  __Pyx_GIVEREF(__pyx_v_N);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_N);
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, __pyx_t_8); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_dist_array = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":924
 *     dist_array = np.zeros(N, dtype=DTYPE)
 * 
 *     csr_data = csgraph.data.copy()             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_data); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 924; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_copy); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 924; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (__pyx_t_8) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_8); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 924; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 924; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_csr_data = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":929
 *     # here we first add a single node to the graph, connected by a
 *     # directed edge of weight zero to each node, and perform bellman-ford
 *     if directed:             # <<<<<<<<<<<<<<
 *         ret = _johnson_directed(csr_data, csgraph.indices,
 *                                 csgraph.indptr, dist_array)
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_directed); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 929; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_7) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":930
 *     # directed edge of weight zero to each node, and perform bellman-ford
 *     if directed:
 *         ret = _johnson_directed(csr_data, csgraph.indices,             # <<<<<<<<<<<<<<
 *                                 csgraph.indptr, dist_array)
 *     else:
 */
    if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 930; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 930; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 930; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":931
 *     if directed:
 *         ret = _johnson_directed(csr_data, csgraph.indices,
 *                                 csgraph.indptr, dist_array)             # <<<<<<<<<<<<<<
 *     else:
 *         ret = _johnson_undirected(csr_data, csgraph.indices,
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 931; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 931; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_dist_array) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_array, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 931; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":930
 *     # directed edge of weight zero to each node, and perform bellman-ford
 *     if directed:
 *         ret = _johnson_directed(csr_data, csgraph.indices,             # <<<<<<<<<<<<<<
 *                                 csgraph.indptr, dist_array)
 *     else:
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_directed(((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_3), ((PyArrayObject *)__pyx_t_6), ((PyArrayObject *)__pyx_v_dist_array))); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 930; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_ret = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":929
 *     # here we first add a single node to the graph, connected by a
 *     # directed edge of weight zero to each node, and perform bellman-ford
 *     if directed:             # <<<<<<<<<<<<<<
 *         ret = _johnson_directed(csr_data, csgraph.indices,
 *                                 csgraph.indptr, dist_array)
 */
    goto __pyx_L9;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":933
 *                                 csgraph.indptr, dist_array)
 *     else:
 *         ret = _johnson_undirected(csr_data, csgraph.indices,             # <<<<<<<<<<<<<<
 *                                   csgraph.indptr, dist_array)
 * 
 */
  /*else*/ {
    if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 933; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 933; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    if (!(likely(((__pyx_t_8) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_8, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 933; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":934
 *     else:
 *         ret = _johnson_undirected(csr_data, csgraph.indices,
 *                                   csgraph.indptr, dist_array)             # <<<<<<<<<<<<<<
 * 
 *     if ret >= 0:
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_dist_array) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_array, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":933
 *                                 csgraph.indptr, dist_array)
 *     else:
 *         ret = _johnson_undirected(csr_data, csgraph.indices,             # <<<<<<<<<<<<<<
 *                                   csgraph.indptr, dist_array)
 * 
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_undirected(((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_8), ((PyArrayObject *)__pyx_t_6), ((PyArrayObject *)__pyx_v_dist_array))); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 933; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_ret = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L9:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":936
 *                                   csgraph.indptr, dist_array)
 * 
 *     if ret >= 0:             # <<<<<<<<<<<<<<
 *         raise NegativeCycleError("Negative cycle detected on node %i" % ret)
 * 
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_ret, __pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_7) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":937
 * 
 *     if ret >= 0:
 *         raise NegativeCycleError("Negative cycle detected on node %i" % ret)             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_NegativeCycleError); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Negative_cycle_detected_on_node, __pyx_v_ret); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_2 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_8); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":936
 *                                   csgraph.indptr, dist_array)
 * 
 *     if ret >= 0:             # <<<<<<<<<<<<<<
 *         raise NegativeCycleError("Negative cycle detected on node %i" % ret)
 * 
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":941
 *     #------------------------------
 *     # add the bellman-ford weights to the data
 *     _johnson_add_weights(csr_data, csgraph.indices,             # <<<<<<<<<<<<<<
 *                          csgraph.indptr, dist_array)
 * 
 */
  if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 941; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 941; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 941; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":942
 *     # add the bellman-ford weights to the data
 *     _johnson_add_weights(csr_data, csgraph.indices,
 *                          csgraph.indptr, dist_array)             # <<<<<<<<<<<<<<
 * 
 *     if directed:
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 942; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 942; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (!(likely(((__pyx_v_dist_array) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_array, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 942; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":941
 *     #------------------------------
 *     # add the bellman-ford weights to the data
 *     _johnson_add_weights(csr_data, csgraph.indices,             # <<<<<<<<<<<<<<
 *                          csgraph.indptr, dist_array)
 * 
 */
  __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_add_weights(((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_3), ((PyArrayObject *)__pyx_t_6), ((PyArrayObject *)__pyx_v_dist_array));
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":944
 *                          csgraph.indptr, dist_array)
 * 
 *     if directed:             # <<<<<<<<<<<<<<
 *         _dijkstra_directed(indices,
 *                            csr_data, csgraph.indices, csgraph.indptr,
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_directed); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_7) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":945
 * 
 *     if directed:
 *         _dijkstra_directed(indices,             # <<<<<<<<<<<<<<
 *                            csr_data, csgraph.indices, csgraph.indptr,
 *                            dist_matrix, predecessor_matrix, np.inf)
 */
    if (!(likely(((__pyx_v_indices) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_indices, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":946
 *     if directed:
 *         _dijkstra_directed(indices,
 *                            csr_data, csgraph.indices, csgraph.indptr,             # <<<<<<<<<<<<<<
 *                            dist_matrix, predecessor_matrix, np.inf)
 *     else:
 */
    if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":947
 *         _dijkstra_directed(indices,
 *                            csr_data, csgraph.indices, csgraph.indptr,
 *                            dist_matrix, predecessor_matrix, np.inf)             # <<<<<<<<<<<<<<
 *     else:
 *         csgraphT = csr_matrix((csr_data, csgraph.indices, csgraph.indptr),
 */
    if (!(likely(((__pyx_v_dist_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_predecessor_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_predecessor_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_inf); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __pyx_PyFloat_AsDouble(__pyx_t_8); if (unlikely((__pyx_t_9 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":945
 * 
 *     if directed:
 *         _dijkstra_directed(indices,             # <<<<<<<<<<<<<<
 *                            csr_data, csgraph.indices, csgraph.indptr,
 *                            dist_matrix, predecessor_matrix, np.inf)
 */
    __pyx_t_8 = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__dijkstra_directed(((PyArrayObject *)__pyx_v_indices), ((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_6), ((PyArrayObject *)__pyx_t_3), ((PyArrayObject *)__pyx_v_dist_matrix), ((PyArrayObject *)__pyx_v_predecessor_matrix), __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":944
 *                          csgraph.indptr, dist_array)
 * 
 *     if directed:             # <<<<<<<<<<<<<<
 *         _dijkstra_directed(indices,
 *                            csr_data, csgraph.indices, csgraph.indptr,
 */
    goto __pyx_L11;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":949
 *                            dist_matrix, predecessor_matrix, np.inf)
 *     else:
 *         csgraphT = csr_matrix((csr_data, csgraph.indices, csgraph.indptr),             # <<<<<<<<<<<<<<
 *                           csgraph.shape).T.tocsr()
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,
 */
  /*else*/ {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":950
 *     else:
 *         csgraphT = csr_matrix((csr_data, csgraph.indices, csgraph.indptr),
 *                           csgraph.shape).T.tocsr()             # <<<<<<<<<<<<<<
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,
 *                              csgraphT.indptr, dist_array)
 */
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_csr_matrix); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":949
 *                            dist_matrix, predecessor_matrix, np.inf)
 *     else:
 *         csgraphT = csr_matrix((csr_data, csgraph.indices, csgraph.indptr),             # <<<<<<<<<<<<<<
 *                           csgraph.shape).T.tocsr()
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(__pyx_v_csr_data);
    __Pyx_GIVEREF(__pyx_v_csr_data);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_csr_data);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_2);
    __pyx_t_4 = 0;
    __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":950
 *     else:
 *         csgraphT = csr_matrix((csr_data, csgraph.indices, csgraph.indptr),
 *                           csgraph.shape).T.tocsr()             # <<<<<<<<<<<<<<
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,
 *                              csgraphT.indptr, dist_array)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_shape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_5 = 1;
      }
    }
    __pyx_t_11 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_5, __pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_5, __pyx_t_2);
    __pyx_t_10 = 0;
    __pyx_t_2 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_11, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_T); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_tocsr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_6) {
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __pyx_t_8 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_csgraphT = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":951
 *         csgraphT = csr_matrix((csr_data, csgraph.indices, csgraph.indptr),
 *                           csgraph.shape).T.tocsr()
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,             # <<<<<<<<<<<<<<
 *                              csgraphT.indptr, dist_array)
 *         _dijkstra_undirected(indices,
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_data); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    if (!(likely(((__pyx_t_8) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_8, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_indices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":952
 *                           csgraph.shape).T.tocsr()
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,
 *                              csgraphT.indptr, dist_array)             # <<<<<<<<<<<<<<
 *         _dijkstra_undirected(indices,
 *                              csr_data, csgraph.indices, csgraph.indptr,
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_indptr); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 952; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 952; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_dist_array) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_array, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 952; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":951
 *         csgraphT = csr_matrix((csr_data, csgraph.indices, csgraph.indptr),
 *                           csgraph.shape).T.tocsr()
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,             # <<<<<<<<<<<<<<
 *                              csgraphT.indptr, dist_array)
 *         _dijkstra_undirected(indices,
 */
    __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_add_weights(((PyArrayObject *)__pyx_t_8), ((PyArrayObject *)__pyx_t_3), ((PyArrayObject *)__pyx_t_6), ((PyArrayObject *)__pyx_v_dist_array));
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":953
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,
 *                              csgraphT.indptr, dist_array)
 *         _dijkstra_undirected(indices,             # <<<<<<<<<<<<<<
 *                              csr_data, csgraph.indices, csgraph.indptr,
 *                              csgraphT.data, csgraphT.indices, csgraphT.indptr,
 */
    if (!(likely(((__pyx_v_indices) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_indices, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 953; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":954
 *                              csgraphT.indptr, dist_array)
 *         _dijkstra_undirected(indices,
 *                              csr_data, csgraph.indices, csgraph.indptr,             # <<<<<<<<<<<<<<
 *                              csgraphT.data, csgraphT.indices, csgraphT.indptr,
 *                              dist_matrix, predecessor_matrix, np.inf)
 */
    if (!(likely(((__pyx_v_csr_data) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_csr_data, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indices); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraph, __pyx_n_s_indptr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":955
 *         _dijkstra_undirected(indices,
 *                              csr_data, csgraph.indices, csgraph.indptr,
 *                              csgraphT.data, csgraphT.indices, csgraphT.indptr,             # <<<<<<<<<<<<<<
 *                              dist_matrix, predecessor_matrix, np.inf)
 * 
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_data); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    if (!(likely(((__pyx_t_8) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_8, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_indices); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    if (!(likely(((__pyx_t_11) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_11, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_csgraphT, __pyx_n_s_indptr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "scipy/sparse/csgraph/_shortest_path.pyx":956
 *                              csr_data, csgraph.indices, csgraph.indptr,
 *                              csgraphT.data, csgraphT.indices, csgraphT.indptr,
 *                              dist_matrix, predecessor_matrix, np.inf)             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
    if (!(likely(((__pyx_v_dist_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dist_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!(likely(((__pyx_v_predecessor_matrix) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_predecessor_matrix, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_inf); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_9 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_9 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":953
 *         _johnson_add_weights(csgraphT.data, csgraphT.indices,
 *                              csgraphT.indptr, dist_array)
 *         _dijkstra_undirected(indices,             # <<<<<<<<<<<<<<
 *                              csr_data, csgraph.indices, csgraph.indptr,
 *                              csgraphT.data, csgraphT.indices, csgraphT.indptr,
 */
    __pyx_t_4 = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__dijkstra_undirected(((PyArrayObject *)__pyx_v_indices), ((PyArrayObject *)__pyx_v_csr_data), ((PyArrayObject *)__pyx_t_6), ((PyArrayObject *)__pyx_t_3), ((PyArrayObject *)__pyx_t_8), ((PyArrayObject *)__pyx_t_11), ((PyArrayObject *)__pyx_t_2), ((PyArrayObject *)__pyx_v_dist_matrix), ((PyArrayObject *)__pyx_v_predecessor_matrix), __pyx_t_9); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 953; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_L11:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":960
 *     #------------------------------
 *     # correct the distance matrix for the bellman-ford weights
 *     dist_matrix += dist_array             # <<<<<<<<<<<<<<
 *     dist_matrix -= dist_array[:, None][indices]
 * 
 */
  __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_dist_matrix, __pyx_v_dist_array); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 960; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF_SET(__pyx_v_dist_matrix, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":961
 *     # correct the distance matrix for the bellman-ford weights
 *     dist_matrix += dist_array
 *     dist_matrix -= dist_array[:, None][indices]             # <<<<<<<<<<<<<<
 * 
 *     if return_predecessors:
 */
  __pyx_t_4 = PyObject_GetItem(__pyx_v_dist_array, __pyx_tuple__15); if (unlikely(__pyx_t_4 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 961; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyObject_GetItem(__pyx_t_4, __pyx_v_indices); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 961; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyNumber_InPlaceSubtract(__pyx_v_dist_matrix, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 961; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_dist_matrix, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":963
 *     dist_matrix -= dist_array[:, None][indices]
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_return_predecessors); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 963; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_7) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":964
 * 
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),             # <<<<<<<<<<<<<<
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 964; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_11) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_return_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 964; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 964; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_11); __pyx_t_11 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_v_return_shape);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 964; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":965
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))             # <<<<<<<<<<<<<<
 *     else:
 *         return dist_matrix.reshape(return_shape)
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_predecessor_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 965; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_11 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    if (!__pyx_t_11) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_return_shape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 965; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 965; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_11); __pyx_t_11 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_return_shape);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 965; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":964
 * 
 *     if return_predecessors:
 *         return (dist_matrix.reshape(return_shape),             # <<<<<<<<<<<<<<
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 */
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 964; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_2);
    __pyx_t_4 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":963
 *     dist_matrix -= dist_array[:, None][indices]
 * 
 *     if return_predecessors:             # <<<<<<<<<<<<<<
 *         return (dist_matrix.reshape(return_shape),
 *                 predecessor_matrix.reshape(return_shape))
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":967
 *                 predecessor_matrix.reshape(return_shape))
 *     else:
 *         return dist_matrix.reshape(return_shape)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_matrix, __pyx_n_s_reshape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 967; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_return_shape); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 967; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
    } else {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 967; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_return_shape);
      __Pyx_GIVEREF(__pyx_v_return_shape);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_return_shape);
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 967; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":818
 * 
 * 
 * def johnson(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *             return_predecessors=False,
 *             unweighted=False):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("scipy.sparse.csgraph._shortest_path.johnson", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_N);
  __Pyx_XDECREF(__pyx_v_return_shape);
  __Pyx_XDECREF(__pyx_v_dist_matrix);
  __Pyx_XDECREF(__pyx_v_predecessor_matrix);
  __Pyx_XDECREF(__pyx_v_dist_array);
  __Pyx_XDECREF(__pyx_v_csr_data);
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_csgraphT);
  __Pyx_XDECREF(__pyx_v_csgraph);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":970
 * 
 * 
 * cdef void _johnson_add_weights(             # <<<<<<<<<<<<<<
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] csr_indices,
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_add_weights(PyArrayObject *__pyx_v_csr_weights, PyArrayObject *__pyx_v_csr_indices, PyArrayObject *__pyx_v_csr_indptr, PyArrayObject *__pyx_v_dist_array) {
  unsigned int __pyx_v_j;
  unsigned int __pyx_v_k;
  unsigned int __pyx_v_N;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indices;
  __Pyx_Buffer __pyx_pybuffer_csr_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indptr;
  __Pyx_Buffer __pyx_pybuffer_csr_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_weights;
  __Pyx_Buffer __pyx_pybuffer_csr_weights;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dist_array;
  __Pyx_Buffer __pyx_pybuffer_dist_array;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  size_t __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_johnson_add_weights", 0);
  __pyx_pybuffer_csr_weights.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_weights.refcount = 0;
  __pyx_pybuffernd_csr_weights.data = NULL;
  __pyx_pybuffernd_csr_weights.rcbuffer = &__pyx_pybuffer_csr_weights;
  __pyx_pybuffer_csr_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indices.refcount = 0;
  __pyx_pybuffernd_csr_indices.data = NULL;
  __pyx_pybuffernd_csr_indices.rcbuffer = &__pyx_pybuffer_csr_indices;
  __pyx_pybuffer_csr_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indptr.refcount = 0;
  __pyx_pybuffernd_csr_indptr.data = NULL;
  __pyx_pybuffernd_csr_indptr.rcbuffer = &__pyx_pybuffer_csr_indptr;
  __pyx_pybuffer_dist_array.pybuffer.buf = NULL;
  __pyx_pybuffer_dist_array.refcount = 0;
  __pyx_pybuffernd_dist_array.data = NULL;
  __pyx_pybuffernd_dist_array.rcbuffer = &__pyx_pybuffer_dist_array;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_weights, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 970; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_weights.diminfo[0].strides = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_weights.diminfo[0].shape = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 970; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indices.diminfo[0].strides = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indices.diminfo[0].shape = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indptr, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 970; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indptr.diminfo[0].strides = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indptr.diminfo[0].shape = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_dist_array, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 970; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_dist_array.diminfo[0].strides = __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dist_array.diminfo[0].shape = __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.shape[0];

  /* "scipy/sparse/csgraph/_shortest_path.pyx":976
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] dist_array):
 *     # let w(u, v) = w(u, v) + h(u) - h(v)
 *     cdef unsigned int j, k, N = dist_array.shape[0]             # <<<<<<<<<<<<<<
 * 
 *     for j from 0 <= j < N:
 */
  __pyx_v_N = (__pyx_v_dist_array->dimensions[0]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":978
 *     cdef unsigned int j, k, N = dist_array.shape[0]
 * 
 *     for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             csr_weights[k] += dist_array[j]
 */
  __pyx_t_1 = __pyx_v_N;
  for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_1; __pyx_v_j++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":979
 * 
 *     for j from 0 <= j < N:
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *             csr_weights[k] += dist_array[j]
 *             csr_weights[k] -= dist_array[csr_indices[k]]
 */
    __pyx_t_2 = __pyx_v_j;
    __pyx_t_3 = -1;
    if (unlikely(__pyx_t_2 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_4 = (__pyx_v_j + 1);
    __pyx_t_3 = -1;
    if (__pyx_t_4 < 0) {
      __pyx_t_4 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
      if (unlikely(__pyx_t_4 < 0)) __pyx_t_3 = 0;
    } else if (unlikely(__pyx_t_4 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_3 = 0;
    if (unlikely(__pyx_t_3 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_3);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_5 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_4, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
    for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_2, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_5; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":980
 *     for j from 0 <= j < N:
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             csr_weights[k] += dist_array[j]             # <<<<<<<<<<<<<<
 *             csr_weights[k] -= dist_array[csr_indices[k]]
 * 
 */
      __pyx_t_6 = __pyx_v_j;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_6 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 980; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_7 = __pyx_v_k;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_7 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 980; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      *__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_7, __pyx_pybuffernd_csr_weights.diminfo[0].strides) += (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_6, __pyx_pybuffernd_dist_array.diminfo[0].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":981
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             csr_weights[k] += dist_array[j]
 *             csr_weights[k] -= dist_array[csr_indices[k]]             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_8 = __pyx_v_k;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_8 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 981; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_9 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_8, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
      __pyx_t_3 = -1;
      if (__pyx_t_9 < 0) {
        __pyx_t_9 += __pyx_pybuffernd_dist_array.diminfo[0].shape;
        if (unlikely(__pyx_t_9 < 0)) __pyx_t_3 = 0;
      } else if (unlikely(__pyx_t_9 >= __pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 981; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_10 = __pyx_v_k;
      __pyx_t_3 = -1;
      if (unlikely(__pyx_t_10 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_3 = 0;
      if (unlikely(__pyx_t_3 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_3);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 981; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      *__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_10, __pyx_pybuffernd_csr_weights.diminfo[0].strides) -= (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_9, __pyx_pybuffernd_dist_array.diminfo[0].strides));
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":970
 * 
 * 
 * cdef void _johnson_add_weights(             # <<<<<<<<<<<<<<
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] csr_indices,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_WriteUnraisable("scipy.sparse.csgraph._shortest_path._johnson_add_weights", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":984
 * 
 * 
 * cdef int _johnson_directed(             # <<<<<<<<<<<<<<
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] csr_indices,
 */

static int __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_directed(PyArrayObject *__pyx_v_csr_weights, PyArrayObject *__pyx_v_csr_indices, PyArrayObject *__pyx_v_csr_indptr, PyArrayObject *__pyx_v_dist_array) {
  unsigned int __pyx_v_N;
  unsigned int __pyx_v_j;
  unsigned int __pyx_v_k;
  CYTHON_UNUSED unsigned int __pyx_v_count;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d1;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d2;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_w12;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indices;
  __Pyx_Buffer __pyx_pybuffer_csr_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indptr;
  __Pyx_Buffer __pyx_pybuffer_csr_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_weights;
  __Pyx_Buffer __pyx_pybuffer_csr_weights;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dist_array;
  __Pyx_Buffer __pyx_pybuffer_dist_array;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  unsigned int __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_t_10;
  size_t __pyx_t_11;
  size_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  size_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  size_t __pyx_t_16;
  size_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  size_t __pyx_t_19;
  size_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_johnson_directed", 0);
  __pyx_pybuffer_csr_weights.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_weights.refcount = 0;
  __pyx_pybuffernd_csr_weights.data = NULL;
  __pyx_pybuffernd_csr_weights.rcbuffer = &__pyx_pybuffer_csr_weights;
  __pyx_pybuffer_csr_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indices.refcount = 0;
  __pyx_pybuffernd_csr_indices.data = NULL;
  __pyx_pybuffernd_csr_indices.rcbuffer = &__pyx_pybuffer_csr_indices;
  __pyx_pybuffer_csr_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indptr.refcount = 0;
  __pyx_pybuffernd_csr_indptr.data = NULL;
  __pyx_pybuffernd_csr_indptr.rcbuffer = &__pyx_pybuffer_csr_indptr;
  __pyx_pybuffer_dist_array.pybuffer.buf = NULL;
  __pyx_pybuffer_dist_array.refcount = 0;
  __pyx_pybuffernd_dist_array.data = NULL;
  __pyx_pybuffernd_dist_array.rcbuffer = &__pyx_pybuffer_dist_array;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_weights, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_weights.diminfo[0].strides = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_weights.diminfo[0].shape = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indices.diminfo[0].strides = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indices.diminfo[0].shape = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indptr, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indptr.diminfo[0].strides = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indptr.diminfo[0].shape = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_dist_array, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_dist_array.diminfo[0].strides = __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dist_array.diminfo[0].shape = __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.shape[0];

  /* "scipy/sparse/csgraph/_shortest_path.pyx":990
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] dist_array):
 *     global DTYPE_EPS
 *     cdef unsigned int N = dist_array.shape[0]             # <<<<<<<<<<<<<<
 *     cdef unsigned int j, k, j_source, count
 * 
 */
  __pyx_v_N = (__pyx_v_dist_array->dimensions[0]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":996
 * 
 *     # relax all edges (N+1) - 1 times
 *     for count from 0 <= count < N:             # <<<<<<<<<<<<<<
 *         for k from 0 <= k < N:
 *             if dist_array[k] < 0:
 */
  __pyx_t_1 = __pyx_v_N;
  for (__pyx_v_count = 0; __pyx_v_count < __pyx_t_1; __pyx_v_count++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":997
 *     # relax all edges (N+1) - 1 times
 *     for count from 0 <= count < N:
 *         for k from 0 <= k < N:             # <<<<<<<<<<<<<<
 *             if dist_array[k] < 0:
 *                 dist_array[k] = 0
 */
    __pyx_t_2 = __pyx_v_N;
    for (__pyx_v_k = 0; __pyx_v_k < __pyx_t_2; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":998
 *     for count from 0 <= count < N:
 *         for k from 0 <= k < N:
 *             if dist_array[k] < 0:             # <<<<<<<<<<<<<<
 *                 dist_array[k] = 0
 * 
 */
      __pyx_t_3 = __pyx_v_k;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_3 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 998; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_5 = (((*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_dist_array.diminfo[0].strides)) < 0.0) != 0);
      if (__pyx_t_5) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":999
 *         for k from 0 <= k < N:
 *             if dist_array[k] < 0:
 *                 dist_array[k] = 0             # <<<<<<<<<<<<<<
 * 
 *         for j from 0 <= j < N:
 */
        __pyx_t_6 = __pyx_v_k;
        __pyx_t_4 = -1;
        if (unlikely(__pyx_t_6 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
        if (unlikely(__pyx_t_4 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_4);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 999; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        *__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_6, __pyx_pybuffernd_dist_array.diminfo[0].strides) = 0.0;

        /* "scipy/sparse/csgraph/_shortest_path.pyx":998
 *     for count from 0 <= count < N:
 *         for k from 0 <= k < N:
 *             if dist_array[k] < 0:             # <<<<<<<<<<<<<<
 *                 dist_array[k] = 0
 * 
 */
      }
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1001
 *                 dist_array[k] = 0
 * 
 *         for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *             d1 = dist_array[j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 */
    __pyx_t_2 = __pyx_v_N;
    for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_2; __pyx_v_j++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1002
 * 
 *         for j from 0 <= j < N:
 *             d1 = dist_array[j]             # <<<<<<<<<<<<<<
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]
 */
      __pyx_t_7 = __pyx_v_j;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_7 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1002; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_v_d1 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_7, __pyx_pybuffernd_dist_array.diminfo[0].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1003
 *         for j from 0 <= j < N:
 *             d1 = dist_array[j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *                 w12 = csr_weights[k]
 *                 d2 = dist_array[csr_indices[k]]
 */
      __pyx_t_8 = __pyx_v_j;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_8 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1003; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_9 = (__pyx_v_j + 1);
      __pyx_t_4 = -1;
      if (__pyx_t_9 < 0) {
        __pyx_t_9 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
        if (unlikely(__pyx_t_9 < 0)) __pyx_t_4 = 0;
      } else if (unlikely(__pyx_t_9 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1003; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_10 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_9, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
      for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_8, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_10; __pyx_v_k++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1004
 *             d1 = dist_array[j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]             # <<<<<<<<<<<<<<
 *                 d2 = dist_array[csr_indices[k]]
 *                 if d1 + w12 < d2:
 */
        __pyx_t_11 = __pyx_v_k;
        __pyx_t_4 = -1;
        if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_4 = 0;
        if (unlikely(__pyx_t_4 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_4);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1004; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_w12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_csr_weights.diminfo[0].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1005
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]
 *                 d2 = dist_array[csr_indices[k]]             # <<<<<<<<<<<<<<
 *                 if d1 + w12 < d2:
 *                     dist_array[csr_indices[k]] = d1 + w12
 */
        __pyx_t_12 = __pyx_v_k;
        __pyx_t_4 = -1;
        if (unlikely(__pyx_t_12 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_4 = 0;
        if (unlikely(__pyx_t_4 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_4);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1005; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_t_13 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_12, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
        __pyx_t_4 = -1;
        if (__pyx_t_13 < 0) {
          __pyx_t_13 += __pyx_pybuffernd_dist_array.diminfo[0].shape;
          if (unlikely(__pyx_t_13 < 0)) __pyx_t_4 = 0;
        } else if (unlikely(__pyx_t_13 >= __pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
        if (unlikely(__pyx_t_4 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_4);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1005; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_d2 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_13, __pyx_pybuffernd_dist_array.diminfo[0].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1006
 *                 w12 = csr_weights[k]
 *                 d2 = dist_array[csr_indices[k]]
 *                 if d1 + w12 < d2:             # <<<<<<<<<<<<<<
 *                     dist_array[csr_indices[k]] = d1 + w12
 * 
 */
        __pyx_t_5 = (((__pyx_v_d1 + __pyx_v_w12) < __pyx_v_d2) != 0);
        if (__pyx_t_5) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":1007
 *                 d2 = dist_array[csr_indices[k]]
 *                 if d1 + w12 < d2:
 *                     dist_array[csr_indices[k]] = d1 + w12             # <<<<<<<<<<<<<<
 * 
 *     # check for negative-weight cycles
 */
          __pyx_t_14 = __pyx_v_k;
          __pyx_t_4 = -1;
          if (unlikely(__pyx_t_14 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_4 = 0;
          if (unlikely(__pyx_t_4 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_4);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1007; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          __pyx_t_15 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_14, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
          __pyx_t_4 = -1;
          if (__pyx_t_15 < 0) {
            __pyx_t_15 += __pyx_pybuffernd_dist_array.diminfo[0].shape;
            if (unlikely(__pyx_t_15 < 0)) __pyx_t_4 = 0;
          } else if (unlikely(__pyx_t_15 >= __pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
          if (unlikely(__pyx_t_4 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_4);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1007; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          *__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_15, __pyx_pybuffernd_dist_array.diminfo[0].strides) = (__pyx_v_d1 + __pyx_v_w12);

          /* "scipy/sparse/csgraph/_shortest_path.pyx":1006
 *                 w12 = csr_weights[k]
 *                 d2 = dist_array[csr_indices[k]]
 *                 if d1 + w12 < d2:             # <<<<<<<<<<<<<<
 *                     dist_array[csr_indices[k]] = d1 + w12
 * 
 */
        }
      }
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1010
 * 
 *     # check for negative-weight cycles
 *     for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *         d1 = dist_array[j]
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 */
  __pyx_t_1 = __pyx_v_N;
  for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_1; __pyx_v_j++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1011
 *     # check for negative-weight cycles
 *     for j from 0 <= j < N:
 *         d1 = dist_array[j]             # <<<<<<<<<<<<<<
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             w12 = csr_weights[k]
 */
    __pyx_t_16 = __pyx_v_j;
    __pyx_t_4 = -1;
    if (unlikely(__pyx_t_16 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
    if (unlikely(__pyx_t_4 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_4);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1011; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_d1 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_16, __pyx_pybuffernd_dist_array.diminfo[0].strides));

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1012
 *     for j from 0 <= j < N:
 *         d1 = dist_array[j]
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *             w12 = csr_weights[k]
 *             d2 = dist_array[csr_indices[k]]
 */
    __pyx_t_17 = __pyx_v_j;
    __pyx_t_4 = -1;
    if (unlikely(__pyx_t_17 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_4 = 0;
    if (unlikely(__pyx_t_4 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_4);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1012; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_18 = (__pyx_v_j + 1);
    __pyx_t_4 = -1;
    if (__pyx_t_18 < 0) {
      __pyx_t_18 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
      if (unlikely(__pyx_t_18 < 0)) __pyx_t_4 = 0;
    } else if (unlikely(__pyx_t_18 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_4 = 0;
    if (unlikely(__pyx_t_4 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_4);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1012; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_18, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
    for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_17, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_10; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1013
 *         d1 = dist_array[j]
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             w12 = csr_weights[k]             # <<<<<<<<<<<<<<
 *             d2 = dist_array[csr_indices[k]]
 *             if d1 + w12 + DTYPE_EPS < d2:
 */
      __pyx_t_19 = __pyx_v_k;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_19 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_v_w12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_csr_weights.diminfo[0].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1014
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             w12 = csr_weights[k]
 *             d2 = dist_array[csr_indices[k]]             # <<<<<<<<<<<<<<
 *             if d1 + w12 + DTYPE_EPS < d2:
 *                 return j
 */
      __pyx_t_20 = __pyx_v_k;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_20 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1014; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_21 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_20, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
      __pyx_t_4 = -1;
      if (__pyx_t_21 < 0) {
        __pyx_t_21 += __pyx_pybuffernd_dist_array.diminfo[0].shape;
        if (unlikely(__pyx_t_21 < 0)) __pyx_t_4 = 0;
      } else if (unlikely(__pyx_t_21 >= __pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1014; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_v_d2 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_21, __pyx_pybuffernd_dist_array.diminfo[0].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1015
 *             w12 = csr_weights[k]
 *             d2 = dist_array[csr_indices[k]]
 *             if d1 + w12 + DTYPE_EPS < d2:             # <<<<<<<<<<<<<<
 *                 return j
 * 
 */
      __pyx_t_5 = ((((__pyx_v_d1 + __pyx_v_w12) + __pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_EPS) < __pyx_v_d2) != 0);
      if (__pyx_t_5) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1016
 *             d2 = dist_array[csr_indices[k]]
 *             if d1 + w12 + DTYPE_EPS < d2:
 *                 return j             # <<<<<<<<<<<<<<
 * 
 *     return -1
 */
        __pyx_r = __pyx_v_j;
        goto __pyx_L0;

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1015
 *             w12 = csr_weights[k]
 *             d2 = dist_array[csr_indices[k]]
 *             if d1 + w12 + DTYPE_EPS < d2:             # <<<<<<<<<<<<<<
 *                 return j
 * 
 */
      }
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1018
 *                 return j
 * 
 *     return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = -1;
  goto __pyx_L0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":984
 * 
 * 
 * cdef int _johnson_directed(             # <<<<<<<<<<<<<<
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] csr_indices,
 */

  /* function exit code */
  __pyx_L1_error:;
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_WriteUnraisable("scipy.sparse.csgraph._shortest_path._johnson_directed", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1021
 * 
 * 
 * cdef int _johnson_undirected(             # <<<<<<<<<<<<<<
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] csr_indices,
 */

static int __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path__johnson_undirected(PyArrayObject *__pyx_v_csr_weights, PyArrayObject *__pyx_v_csr_indices, PyArrayObject *__pyx_v_csr_indptr, PyArrayObject *__pyx_v_dist_array) {
  unsigned int __pyx_v_N;
  unsigned int __pyx_v_j;
  unsigned int __pyx_v_k;
  CYTHON_UNUSED unsigned int __pyx_v_count;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d1;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_d2;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_w12;
  PyObject *__pyx_v_ind_k = NULL;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indices;
  __Pyx_Buffer __pyx_pybuffer_csr_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_indptr;
  __Pyx_Buffer __pyx_pybuffer_csr_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_csr_weights;
  __Pyx_Buffer __pyx_pybuffer_csr_weights;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_dist_array;
  __Pyx_Buffer __pyx_pybuffer_dist_array;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  unsigned int __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t __pyx_t_10;
  size_t __pyx_t_11;
  size_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_t_14;
  size_t __pyx_t_15;
  size_t __pyx_t_16;
  size_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  size_t __pyx_t_19;
  size_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_johnson_undirected", 0);
  __pyx_pybuffer_csr_weights.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_weights.refcount = 0;
  __pyx_pybuffernd_csr_weights.data = NULL;
  __pyx_pybuffernd_csr_weights.rcbuffer = &__pyx_pybuffer_csr_weights;
  __pyx_pybuffer_csr_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indices.refcount = 0;
  __pyx_pybuffernd_csr_indices.data = NULL;
  __pyx_pybuffernd_csr_indices.rcbuffer = &__pyx_pybuffer_csr_indices;
  __pyx_pybuffer_csr_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_csr_indptr.refcount = 0;
  __pyx_pybuffernd_csr_indptr.data = NULL;
  __pyx_pybuffernd_csr_indptr.rcbuffer = &__pyx_pybuffer_csr_indptr;
  __pyx_pybuffer_dist_array.pybuffer.buf = NULL;
  __pyx_pybuffer_dist_array.refcount = 0;
  __pyx_pybuffernd_dist_array.data = NULL;
  __pyx_pybuffernd_dist_array.rcbuffer = &__pyx_pybuffer_dist_array;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_weights, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1021; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_weights.diminfo[0].strides = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_weights.diminfo[0].shape = __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indices, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1021; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indices.diminfo[0].strides = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indices.diminfo[0].shape = __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_v_csr_indptr, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1021; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_csr_indptr.diminfo[0].strides = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_csr_indptr.diminfo[0].shape = __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.shape[0];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_dist_array, &__Pyx_TypeInfo_nn___pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1021; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_dist_array.diminfo[0].strides = __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_dist_array.diminfo[0].shape = __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.shape[0];

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1027
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] dist_array):
 *     global DTYPE_EPS
 *     cdef unsigned int N = dist_array.shape[0]             # <<<<<<<<<<<<<<
 *     cdef unsigned int j, k, j_source, count
 * 
 */
  __pyx_v_N = (__pyx_v_dist_array->dimensions[0]);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1033
 * 
 *     # relax all edges (N+1) - 1 times
 *     for count from 0 <= count < N:             # <<<<<<<<<<<<<<
 *         for k from 0 <= k < N:
 *             if dist_array[k] < 0:
 */
  __pyx_t_1 = __pyx_v_N;
  for (__pyx_v_count = 0; __pyx_v_count < __pyx_t_1; __pyx_v_count++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1034
 *     # relax all edges (N+1) - 1 times
 *     for count from 0 <= count < N:
 *         for k from 0 <= k < N:             # <<<<<<<<<<<<<<
 *             if dist_array[k] < 0:
 *                 dist_array[k] = 0
 */
    __pyx_t_2 = __pyx_v_N;
    for (__pyx_v_k = 0; __pyx_v_k < __pyx_t_2; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1035
 *     for count from 0 <= count < N:
 *         for k from 0 <= k < N:
 *             if dist_array[k] < 0:             # <<<<<<<<<<<<<<
 *                 dist_array[k] = 0
 * 
 */
      __pyx_t_3 = __pyx_v_k;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_3 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_5 = (((*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_dist_array.diminfo[0].strides)) < 0.0) != 0);
      if (__pyx_t_5) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1036
 *         for k from 0 <= k < N:
 *             if dist_array[k] < 0:
 *                 dist_array[k] = 0             # <<<<<<<<<<<<<<
 * 
 *         for j from 0 <= j < N:
 */
        __pyx_t_6 = __pyx_v_k;
        __pyx_t_4 = -1;
        if (unlikely(__pyx_t_6 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
        if (unlikely(__pyx_t_4 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_4);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        *__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_6, __pyx_pybuffernd_dist_array.diminfo[0].strides) = 0.0;

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1035
 *     for count from 0 <= count < N:
 *         for k from 0 <= k < N:
 *             if dist_array[k] < 0:             # <<<<<<<<<<<<<<
 *                 dist_array[k] = 0
 * 
 */
      }
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1038
 *                 dist_array[k] = 0
 * 
 *         for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *             d1 = dist_array[j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 */
    __pyx_t_2 = __pyx_v_N;
    for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_2; __pyx_v_j++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1039
 * 
 *         for j from 0 <= j < N:
 *             d1 = dist_array[j]             # <<<<<<<<<<<<<<
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]
 */
      __pyx_t_7 = __pyx_v_j;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_7 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1039; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_v_d1 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_7, __pyx_pybuffernd_dist_array.diminfo[0].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1040
 *         for j from 0 <= j < N:
 *             d1 = dist_array[j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *                 w12 = csr_weights[k]
 *                 ind_k = csr_indices[k]
 */
      __pyx_t_8 = __pyx_v_j;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_8 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1040; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_9 = (__pyx_v_j + 1);
      __pyx_t_4 = -1;
      if (__pyx_t_9 < 0) {
        __pyx_t_9 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
        if (unlikely(__pyx_t_9 < 0)) __pyx_t_4 = 0;
      } else if (unlikely(__pyx_t_9 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1040; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_10 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_9, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
      for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_8, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_10; __pyx_v_k++) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1041
 *             d1 = dist_array[j]
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]             # <<<<<<<<<<<<<<
 *                 ind_k = csr_indices[k]
 *                 d2 = dist_array[ind_k]
 */
        __pyx_t_11 = __pyx_v_k;
        __pyx_t_4 = -1;
        if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_4 = 0;
        if (unlikely(__pyx_t_4 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_4);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1041; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_v_w12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_csr_weights.diminfo[0].strides));

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1042
 *             for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *                 w12 = csr_weights[k]
 *                 ind_k = csr_indices[k]             # <<<<<<<<<<<<<<
 *                 d2 = dist_array[ind_k]
 *                 if d1 + w12 < d2:
 */
        __pyx_t_12 = __pyx_v_k;
        __pyx_t_4 = -1;
        if (unlikely(__pyx_t_12 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_4 = 0;
        if (unlikely(__pyx_t_4 != -1)) {
          __Pyx_RaiseBufferIndexError(__pyx_t_4);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1042; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __pyx_t_13 = __Pyx_PyInt_From_npy_int32((*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_12, __pyx_pybuffernd_csr_indices.diminfo[0].strides))); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1042; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_XDECREF_SET(__pyx_v_ind_k, __pyx_t_13);
        __pyx_t_13 = 0;

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1043
 *                 w12 = csr_weights[k]
 *                 ind_k = csr_indices[k]
 *                 d2 = dist_array[ind_k]             # <<<<<<<<<<<<<<
 *                 if d1 + w12 < d2:
 *                     dist_array[ind_k] = d1 + w12
 */
        __pyx_t_13 = PyObject_GetItem(((PyObject *)__pyx_v_dist_array), __pyx_v_ind_k); if (unlikely(__pyx_t_13 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1043; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_14 = __pyx_PyFloat_AsDouble(__pyx_t_13); if (unlikely((__pyx_t_14 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1043; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_v_d2 = __pyx_t_14;

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1044
 *                 ind_k = csr_indices[k]
 *                 d2 = dist_array[ind_k]
 *                 if d1 + w12 < d2:             # <<<<<<<<<<<<<<
 *                     dist_array[ind_k] = d1 + w12
 *                 if d2 + w12 < d1:
 */
        __pyx_t_5 = (((__pyx_v_d1 + __pyx_v_w12) < __pyx_v_d2) != 0);
        if (__pyx_t_5) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":1045
 *                 d2 = dist_array[ind_k]
 *                 if d1 + w12 < d2:
 *                     dist_array[ind_k] = d1 + w12             # <<<<<<<<<<<<<<
 *                 if d2 + w12 < d1:
 *                     dist_array[j] = d1 = d2 + w12
 */
          __pyx_t_13 = PyFloat_FromDouble((__pyx_v_d1 + __pyx_v_w12)); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1045; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_13);
          if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_dist_array), __pyx_v_ind_k, __pyx_t_13) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1045; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

          /* "scipy/sparse/csgraph/_shortest_path.pyx":1044
 *                 ind_k = csr_indices[k]
 *                 d2 = dist_array[ind_k]
 *                 if d1 + w12 < d2:             # <<<<<<<<<<<<<<
 *                     dist_array[ind_k] = d1 + w12
 *                 if d2 + w12 < d1:
 */
        }

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1046
 *                 if d1 + w12 < d2:
 *                     dist_array[ind_k] = d1 + w12
 *                 if d2 + w12 < d1:             # <<<<<<<<<<<<<<
 *                     dist_array[j] = d1 = d2 + w12
 * 
 */
        __pyx_t_5 = (((__pyx_v_d2 + __pyx_v_w12) < __pyx_v_d1) != 0);
        if (__pyx_t_5) {

          /* "scipy/sparse/csgraph/_shortest_path.pyx":1047
 *                     dist_array[ind_k] = d1 + w12
 *                 if d2 + w12 < d1:
 *                     dist_array[j] = d1 = d2 + w12             # <<<<<<<<<<<<<<
 * 
 *     # check for negative-weight cycles
 */
          __pyx_t_14 = (__pyx_v_d2 + __pyx_v_w12);
          __pyx_t_15 = __pyx_v_j;
          __pyx_t_4 = -1;
          if (unlikely(__pyx_t_15 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
          if (unlikely(__pyx_t_4 != -1)) {
            __Pyx_RaiseBufferIndexError(__pyx_t_4);
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1047; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          *__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_15, __pyx_pybuffernd_dist_array.diminfo[0].strides) = __pyx_t_14;
          __pyx_v_d1 = __pyx_t_14;

          /* "scipy/sparse/csgraph/_shortest_path.pyx":1046
 *                 if d1 + w12 < d2:
 *                     dist_array[ind_k] = d1 + w12
 *                 if d2 + w12 < d1:             # <<<<<<<<<<<<<<
 *                     dist_array[j] = d1 = d2 + w12
 * 
 */
        }
      }
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1050
 * 
 *     # check for negative-weight cycles
 *     for j from 0 <= j < N:             # <<<<<<<<<<<<<<
 *         d1 = dist_array[j]
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 */
  __pyx_t_1 = __pyx_v_N;
  for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_1; __pyx_v_j++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1051
 *     # check for negative-weight cycles
 *     for j from 0 <= j < N:
 *         d1 = dist_array[j]             # <<<<<<<<<<<<<<
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             w12 = csr_weights[k]
 */
    __pyx_t_16 = __pyx_v_j;
    __pyx_t_4 = -1;
    if (unlikely(__pyx_t_16 >= (size_t)__pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
    if (unlikely(__pyx_t_4 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_4);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_d1 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_16, __pyx_pybuffernd_dist_array.diminfo[0].strides));

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1052
 *     for j from 0 <= j < N:
 *         d1 = dist_array[j]
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:             # <<<<<<<<<<<<<<
 *             w12 = csr_weights[k]
 *             d2 = dist_array[csr_indices[k]]
 */
    __pyx_t_17 = __pyx_v_j;
    __pyx_t_4 = -1;
    if (unlikely(__pyx_t_17 >= (size_t)__pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_4 = 0;
    if (unlikely(__pyx_t_4 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_4);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_18 = (__pyx_v_j + 1);
    __pyx_t_4 = -1;
    if (__pyx_t_18 < 0) {
      __pyx_t_18 += __pyx_pybuffernd_csr_indptr.diminfo[0].shape;
      if (unlikely(__pyx_t_18 < 0)) __pyx_t_4 = 0;
    } else if (unlikely(__pyx_t_18 >= __pyx_pybuffernd_csr_indptr.diminfo[0].shape)) __pyx_t_4 = 0;
    if (unlikely(__pyx_t_4 != -1)) {
      __Pyx_RaiseBufferIndexError(__pyx_t_4);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_18, __pyx_pybuffernd_csr_indptr.diminfo[0].strides));
    for (__pyx_v_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer.buf, __pyx_t_17, __pyx_pybuffernd_csr_indptr.diminfo[0].strides)); __pyx_v_k < __pyx_t_10; __pyx_v_k++) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1053
 *         d1 = dist_array[j]
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             w12 = csr_weights[k]             # <<<<<<<<<<<<<<
 *             d2 = dist_array[csr_indices[k]]
 *             if abs(d2 - d1) > w12 + DTYPE_EPS:
 */
      __pyx_t_19 = __pyx_v_k;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_19 >= (size_t)__pyx_pybuffernd_csr_weights.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1053; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_v_w12 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_csr_weights.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_csr_weights.diminfo[0].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1054
 *         for k from csr_indptr[j] <= k < csr_indptr[j + 1]:
 *             w12 = csr_weights[k]
 *             d2 = dist_array[csr_indices[k]]             # <<<<<<<<<<<<<<
 *             if abs(d2 - d1) > w12 + DTYPE_EPS:
 *                 return j
 */
      __pyx_t_20 = __pyx_v_k;
      __pyx_t_4 = -1;
      if (unlikely(__pyx_t_20 >= (size_t)__pyx_pybuffernd_csr_indices.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_21 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_ITYPE_t *, __pyx_pybuffernd_csr_indices.rcbuffer->pybuffer.buf, __pyx_t_20, __pyx_pybuffernd_csr_indices.diminfo[0].strides));
      __pyx_t_4 = -1;
      if (__pyx_t_21 < 0) {
        __pyx_t_21 += __pyx_pybuffernd_dist_array.diminfo[0].shape;
        if (unlikely(__pyx_t_21 < 0)) __pyx_t_4 = 0;
      } else if (unlikely(__pyx_t_21 >= __pyx_pybuffernd_dist_array.diminfo[0].shape)) __pyx_t_4 = 0;
      if (unlikely(__pyx_t_4 != -1)) {
        __Pyx_RaiseBufferIndexError(__pyx_t_4);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_v_d2 = (*__Pyx_BufPtrCContig1d(__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t *, __pyx_pybuffernd_dist_array.rcbuffer->pybuffer.buf, __pyx_t_21, __pyx_pybuffernd_dist_array.diminfo[0].strides));

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1055
 *             w12 = csr_weights[k]
 *             d2 = dist_array[csr_indices[k]]
 *             if abs(d2 - d1) > w12 + DTYPE_EPS:             # <<<<<<<<<<<<<<
 *                 return j
 * 
 */
      __pyx_t_5 = ((fabs((__pyx_v_d2 - __pyx_v_d1)) > (__pyx_v_w12 + __pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_EPS)) != 0);
      if (__pyx_t_5) {

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1056
 *             d2 = dist_array[csr_indices[k]]
 *             if abs(d2 - d1) > w12 + DTYPE_EPS:
 *                 return j             # <<<<<<<<<<<<<<
 * 
 *     return -1
 */
        __pyx_r = __pyx_v_j;
        goto __pyx_L0;

        /* "scipy/sparse/csgraph/_shortest_path.pyx":1055
 *             w12 = csr_weights[k]
 *             d2 = dist_array[csr_indices[k]]
 *             if abs(d2 - d1) > w12 + DTYPE_EPS:             # <<<<<<<<<<<<<<
 *                 return j
 * 
 */
      }
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1058
 *                 return j
 * 
 *     return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = -1;
  goto __pyx_L0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1021
 * 
 * 
 * cdef int _johnson_undirected(             # <<<<<<<<<<<<<<
 *             np.ndarray[DTYPE_t, ndim=1, mode='c'] csr_weights,
 *             np.ndarray[ITYPE_t, ndim=1, mode='c'] csr_indices,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_13);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_WriteUnraisable("scipy.sparse.csgraph._shortest_path._johnson_undirected", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_csr_weights.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_dist_array.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_ind_k);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1083
 * 
 * 
 * cdef void initialize_node(FibonacciNode* node,             # <<<<<<<<<<<<<<
 *                           unsigned int index,
 *                           DTYPE_t val=0):
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_initialize_node(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node, unsigned int __pyx_v_index, struct __pyx_opt_args_5scipy_6sparse_7csgraph_14_shortest_path_initialize_node *__pyx_optional_args) {
  __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_val = ((__pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t)0.0);
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("initialize_node", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_val = __pyx_optional_args->val;
    }
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1088
 *     # Assumptions: - node is a valid pointer
 *     #              - node is not currently part of a heap
 *     node.index = index             # <<<<<<<<<<<<<<
 *     node.val = val
 *     node.rank = 0
 */
  __pyx_v_node->index = __pyx_v_index;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1089
 *     #              - node is not currently part of a heap
 *     node.index = index
 *     node.val = val             # <<<<<<<<<<<<<<
 *     node.rank = 0
 *     node.state = NOT_IN_HEAP
 */
  __pyx_v_node->val = __pyx_v_val;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1090
 *     node.index = index
 *     node.val = val
 *     node.rank = 0             # <<<<<<<<<<<<<<
 *     node.state = NOT_IN_HEAP
 * 
 */
  __pyx_v_node->rank = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1091
 *     node.val = val
 *     node.rank = 0
 *     node.state = NOT_IN_HEAP             # <<<<<<<<<<<<<<
 * 
 *     node.parent = NULL
 */
  __pyx_v_node->state = __pyx_e_5scipy_6sparse_7csgraph_14_shortest_path_NOT_IN_HEAP;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1093
 *     node.state = NOT_IN_HEAP
 * 
 *     node.parent = NULL             # <<<<<<<<<<<<<<
 *     node.left_sibling = NULL
 *     node.right_sibling = NULL
 */
  __pyx_v_node->parent = NULL;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1094
 * 
 *     node.parent = NULL
 *     node.left_sibling = NULL             # <<<<<<<<<<<<<<
 *     node.right_sibling = NULL
 *     node.children = NULL
 */
  __pyx_v_node->left_sibling = NULL;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1095
 *     node.parent = NULL
 *     node.left_sibling = NULL
 *     node.right_sibling = NULL             # <<<<<<<<<<<<<<
 *     node.children = NULL
 * 
 */
  __pyx_v_node->right_sibling = NULL;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1096
 *     node.left_sibling = NULL
 *     node.right_sibling = NULL
 *     node.children = NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_node->children = NULL;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1083
 * 
 * 
 * cdef void initialize_node(FibonacciNode* node,             # <<<<<<<<<<<<<<
 *                           unsigned int index,
 *                           DTYPE_t val=0):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1099
 * 
 * 
 * cdef FibonacciNode* rightmost_sibling(FibonacciNode* node):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     cdef FibonacciNode* temp = node
 */

static struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_rightmost_sibling(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node) {
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_temp;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_t_2;
  __Pyx_RefNannySetupContext("rightmost_sibling", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1101
 * cdef FibonacciNode* rightmost_sibling(FibonacciNode* node):
 *     # Assumptions: - node is a valid pointer
 *     cdef FibonacciNode* temp = node             # <<<<<<<<<<<<<<
 *     while(temp.right_sibling):
 *         temp = temp.right_sibling
 */
  __pyx_v_temp = __pyx_v_node;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1102
 *     # Assumptions: - node is a valid pointer
 *     cdef FibonacciNode* temp = node
 *     while(temp.right_sibling):             # <<<<<<<<<<<<<<
 *         temp = temp.right_sibling
 *     return temp
 */
  while (1) {
    __pyx_t_1 = (__pyx_v_temp->right_sibling != 0);
    if (!__pyx_t_1) break;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1103
 *     cdef FibonacciNode* temp = node
 *     while(temp.right_sibling):
 *         temp = temp.right_sibling             # <<<<<<<<<<<<<<
 *     return temp
 * 
 */
    __pyx_t_2 = __pyx_v_temp->right_sibling;
    __pyx_v_temp = __pyx_t_2;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1104
 *     while(temp.right_sibling):
 *         temp = temp.right_sibling
 *     return temp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_temp;
  goto __pyx_L0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1099
 * 
 * 
 * cdef FibonacciNode* rightmost_sibling(FibonacciNode* node):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     cdef FibonacciNode* temp = node
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1107
 * 
 * 
 * cdef FibonacciNode* leftmost_sibling(FibonacciNode* node):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     cdef FibonacciNode* temp = node
 */

static struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_leftmost_sibling(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node) {
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_temp;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_t_2;
  __Pyx_RefNannySetupContext("leftmost_sibling", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1109
 * cdef FibonacciNode* leftmost_sibling(FibonacciNode* node):
 *     # Assumptions: - node is a valid pointer
 *     cdef FibonacciNode* temp = node             # <<<<<<<<<<<<<<
 *     while(temp.left_sibling):
 *         temp = temp.left_sibling
 */
  __pyx_v_temp = __pyx_v_node;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1110
 *     # Assumptions: - node is a valid pointer
 *     cdef FibonacciNode* temp = node
 *     while(temp.left_sibling):             # <<<<<<<<<<<<<<
 *         temp = temp.left_sibling
 *     return temp
 */
  while (1) {
    __pyx_t_1 = (__pyx_v_temp->left_sibling != 0);
    if (!__pyx_t_1) break;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1111
 *     cdef FibonacciNode* temp = node
 *     while(temp.left_sibling):
 *         temp = temp.left_sibling             # <<<<<<<<<<<<<<
 *     return temp
 * 
 */
    __pyx_t_2 = __pyx_v_temp->left_sibling;
    __pyx_v_temp = __pyx_t_2;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1112
 *     while(temp.left_sibling):
 *         temp = temp.left_sibling
 *     return temp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_temp;
  goto __pyx_L0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1107
 * 
 * 
 * cdef FibonacciNode* leftmost_sibling(FibonacciNode* node):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     cdef FibonacciNode* temp = node
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1115
 * 
 * 
 * cdef void add_child(FibonacciNode* node, FibonacciNode* new_child):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     #              - new_child is a valid pointer
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_child(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_new_child) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("add_child", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1119
 *     #              - new_child is a valid pointer
 *     #              - new_child is not the sibling or child of another node
 *     new_child.parent = node             # <<<<<<<<<<<<<<
 * 
 *     if node.children:
 */
  __pyx_v_new_child->parent = __pyx_v_node;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1121
 *     new_child.parent = node
 * 
 *     if node.children:             # <<<<<<<<<<<<<<
 *         add_sibling(node.children, new_child)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_node->children != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1122
 * 
 *     if node.children:
 *         add_sibling(node.children, new_child)             # <<<<<<<<<<<<<<
 *     else:
 *         node.children = new_child
 */
    __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_sibling(__pyx_v_node->children, __pyx_v_new_child);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1121
 *     new_child.parent = node
 * 
 *     if node.children:             # <<<<<<<<<<<<<<
 *         add_sibling(node.children, new_child)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1124
 *         add_sibling(node.children, new_child)
 *     else:
 *         node.children = new_child             # <<<<<<<<<<<<<<
 *         new_child.right_sibling = NULL
 *         new_child.left_sibling = NULL
 */
  /*else*/ {
    __pyx_v_node->children = __pyx_v_new_child;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1125
 *     else:
 *         node.children = new_child
 *         new_child.right_sibling = NULL             # <<<<<<<<<<<<<<
 *         new_child.left_sibling = NULL
 *         node.rank = 1
 */
    __pyx_v_new_child->right_sibling = NULL;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1126
 *         node.children = new_child
 *         new_child.right_sibling = NULL
 *         new_child.left_sibling = NULL             # <<<<<<<<<<<<<<
 *         node.rank = 1
 * 
 */
    __pyx_v_new_child->left_sibling = NULL;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1127
 *         new_child.right_sibling = NULL
 *         new_child.left_sibling = NULL
 *         node.rank = 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_node->rank = 1;
  }
  __pyx_L3:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1115
 * 
 * 
 * cdef void add_child(FibonacciNode* node, FibonacciNode* new_child):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     #              - new_child is a valid pointer
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1130
 * 
 * 
 * cdef void add_sibling(FibonacciNode* node, FibonacciNode* new_sibling):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     #              - new_sibling is a valid pointer
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_sibling(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_new_sibling) {
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_temp;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("add_sibling", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1134
 *     #              - new_sibling is a valid pointer
 *     #              - new_sibling is not the child or sibling of another node
 *     cdef FibonacciNode* temp = rightmost_sibling(node)             # <<<<<<<<<<<<<<
 *     temp.right_sibling = new_sibling
 *     new_sibling.left_sibling = temp
 */
  __pyx_v_temp = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_rightmost_sibling(__pyx_v_node);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1135
 *     #              - new_sibling is not the child or sibling of another node
 *     cdef FibonacciNode* temp = rightmost_sibling(node)
 *     temp.right_sibling = new_sibling             # <<<<<<<<<<<<<<
 *     new_sibling.left_sibling = temp
 *     new_sibling.right_sibling = NULL
 */
  __pyx_v_temp->right_sibling = __pyx_v_new_sibling;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1136
 *     cdef FibonacciNode* temp = rightmost_sibling(node)
 *     temp.right_sibling = new_sibling
 *     new_sibling.left_sibling = temp             # <<<<<<<<<<<<<<
 *     new_sibling.right_sibling = NULL
 *     new_sibling.parent = node.parent
 */
  __pyx_v_new_sibling->left_sibling = __pyx_v_temp;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1137
 *     temp.right_sibling = new_sibling
 *     new_sibling.left_sibling = temp
 *     new_sibling.right_sibling = NULL             # <<<<<<<<<<<<<<
 *     new_sibling.parent = node.parent
 *     if new_sibling.parent:
 */
  __pyx_v_new_sibling->right_sibling = NULL;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1138
 *     new_sibling.left_sibling = temp
 *     new_sibling.right_sibling = NULL
 *     new_sibling.parent = node.parent             # <<<<<<<<<<<<<<
 *     if new_sibling.parent:
 *         new_sibling.parent.rank += 1
 */
  __pyx_t_1 = __pyx_v_node->parent;
  __pyx_v_new_sibling->parent = __pyx_t_1;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1139
 *     new_sibling.right_sibling = NULL
 *     new_sibling.parent = node.parent
 *     if new_sibling.parent:             # <<<<<<<<<<<<<<
 *         new_sibling.parent.rank += 1
 * 
 */
  __pyx_t_2 = (__pyx_v_new_sibling->parent != 0);
  if (__pyx_t_2) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1140
 *     new_sibling.parent = node.parent
 *     if new_sibling.parent:
 *         new_sibling.parent.rank += 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_new_sibling->parent->rank = (__pyx_v_new_sibling->parent->rank + 1);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1139
 *     new_sibling.right_sibling = NULL
 *     new_sibling.parent = node.parent
 *     if new_sibling.parent:             # <<<<<<<<<<<<<<
 *         new_sibling.parent.rank += 1
 * 
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1130
 * 
 * 
 * cdef void add_sibling(FibonacciNode* node, FibonacciNode* new_sibling):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     #              - new_sibling is a valid pointer
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1143
 * 
 * 
 * cdef void remove(FibonacciNode* node):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     if node.parent:
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_t_2;
  __Pyx_RefNannySetupContext("remove", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1145
 * cdef void remove(FibonacciNode* node):
 *     # Assumptions: - node is a valid pointer
 *     if node.parent:             # <<<<<<<<<<<<<<
 *         node.parent.rank -= 1
 *         if node.left_sibling:
 */
  __pyx_t_1 = (__pyx_v_node->parent != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1146
 *     # Assumptions: - node is a valid pointer
 *     if node.parent:
 *         node.parent.rank -= 1             # <<<<<<<<<<<<<<
 *         if node.left_sibling:
 *             node.parent.children = node.left_sibling
 */
    __pyx_v_node->parent->rank = (__pyx_v_node->parent->rank - 1);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1147
 *     if node.parent:
 *         node.parent.rank -= 1
 *         if node.left_sibling:             # <<<<<<<<<<<<<<
 *             node.parent.children = node.left_sibling
 *         elif node.right_sibling:
 */
    __pyx_t_1 = (__pyx_v_node->left_sibling != 0);
    if (__pyx_t_1) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1148
 *         node.parent.rank -= 1
 *         if node.left_sibling:
 *             node.parent.children = node.left_sibling             # <<<<<<<<<<<<<<
 *         elif node.right_sibling:
 *             node.parent.children = node.right_sibling
 */
      __pyx_t_2 = __pyx_v_node->left_sibling;
      __pyx_v_node->parent->children = __pyx_t_2;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1147
 *     if node.parent:
 *         node.parent.rank -= 1
 *         if node.left_sibling:             # <<<<<<<<<<<<<<
 *             node.parent.children = node.left_sibling
 *         elif node.right_sibling:
 */
      goto __pyx_L4;
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1149
 *         if node.left_sibling:
 *             node.parent.children = node.left_sibling
 *         elif node.right_sibling:             # <<<<<<<<<<<<<<
 *             node.parent.children = node.right_sibling
 *         else:
 */
    __pyx_t_1 = (__pyx_v_node->right_sibling != 0);
    if (__pyx_t_1) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1150
 *             node.parent.children = node.left_sibling
 *         elif node.right_sibling:
 *             node.parent.children = node.right_sibling             # <<<<<<<<<<<<<<
 *         else:
 *             node.parent.children = NULL
 */
      __pyx_t_2 = __pyx_v_node->right_sibling;
      __pyx_v_node->parent->children = __pyx_t_2;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1149
 *         if node.left_sibling:
 *             node.parent.children = node.left_sibling
 *         elif node.right_sibling:             # <<<<<<<<<<<<<<
 *             node.parent.children = node.right_sibling
 *         else:
 */
      goto __pyx_L4;
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1152
 *             node.parent.children = node.right_sibling
 *         else:
 *             node.parent.children = NULL             # <<<<<<<<<<<<<<
 * 
 *     if node.left_sibling:
 */
    /*else*/ {
      __pyx_v_node->parent->children = NULL;
    }
    __pyx_L4:;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1145
 * cdef void remove(FibonacciNode* node):
 *     # Assumptions: - node is a valid pointer
 *     if node.parent:             # <<<<<<<<<<<<<<
 *         node.parent.rank -= 1
 *         if node.left_sibling:
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1154
 *             node.parent.children = NULL
 * 
 *     if node.left_sibling:             # <<<<<<<<<<<<<<
 *         node.left_sibling.right_sibling = node.right_sibling
 *     if node.right_sibling:
 */
  __pyx_t_1 = (__pyx_v_node->left_sibling != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1155
 * 
 *     if node.left_sibling:
 *         node.left_sibling.right_sibling = node.right_sibling             # <<<<<<<<<<<<<<
 *     if node.right_sibling:
 *         node.right_sibling.left_sibling = node.left_sibling
 */
    __pyx_t_2 = __pyx_v_node->right_sibling;
    __pyx_v_node->left_sibling->right_sibling = __pyx_t_2;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1154
 *             node.parent.children = NULL
 * 
 *     if node.left_sibling:             # <<<<<<<<<<<<<<
 *         node.left_sibling.right_sibling = node.right_sibling
 *     if node.right_sibling:
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1156
 *     if node.left_sibling:
 *         node.left_sibling.right_sibling = node.right_sibling
 *     if node.right_sibling:             # <<<<<<<<<<<<<<
 *         node.right_sibling.left_sibling = node.left_sibling
 * 
 */
  __pyx_t_1 = (__pyx_v_node->right_sibling != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1157
 *         node.left_sibling.right_sibling = node.right_sibling
 *     if node.right_sibling:
 *         node.right_sibling.left_sibling = node.left_sibling             # <<<<<<<<<<<<<<
 * 
 *     node.left_sibling = NULL
 */
    __pyx_t_2 = __pyx_v_node->left_sibling;
    __pyx_v_node->right_sibling->left_sibling = __pyx_t_2;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1156
 *     if node.left_sibling:
 *         node.left_sibling.right_sibling = node.right_sibling
 *     if node.right_sibling:             # <<<<<<<<<<<<<<
 *         node.right_sibling.left_sibling = node.left_sibling
 * 
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1159
 *         node.right_sibling.left_sibling = node.left_sibling
 * 
 *     node.left_sibling = NULL             # <<<<<<<<<<<<<<
 *     node.right_sibling = NULL
 *     node.parent = NULL
 */
  __pyx_v_node->left_sibling = NULL;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1160
 * 
 *     node.left_sibling = NULL
 *     node.right_sibling = NULL             # <<<<<<<<<<<<<<
 *     node.parent = NULL
 * 
 */
  __pyx_v_node->right_sibling = NULL;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1161
 *     node.left_sibling = NULL
 *     node.right_sibling = NULL
 *     node.parent = NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_node->parent = NULL;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1143
 * 
 * 
 * cdef void remove(FibonacciNode* node):             # <<<<<<<<<<<<<<
 *     # Assumptions: - node is a valid pointer
 *     if node.parent:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1177
 * 
 * 
 * cdef void insert_node(FibonacciHeap* heap,             # <<<<<<<<<<<<<<
 *                       FibonacciNode* node):
 *     # Assumptions: - heap is a valid pointer
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_insert_node(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap *__pyx_v_heap, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("insert_node", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1182
 *     #              - node is a valid pointer
 *     #              - node is not the child or sibling of another node
 *     if heap.min_node:             # <<<<<<<<<<<<<<
 *         add_sibling(heap.min_node, node)
 *         if node.val < heap.min_node.val:
 */
  __pyx_t_1 = (__pyx_v_heap->min_node != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1183
 *     #              - node is not the child or sibling of another node
 *     if heap.min_node:
 *         add_sibling(heap.min_node, node)             # <<<<<<<<<<<<<<
 *         if node.val < heap.min_node.val:
 *             heap.min_node = node
 */
    __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_sibling(__pyx_v_heap->min_node, __pyx_v_node);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1184
 *     if heap.min_node:
 *         add_sibling(heap.min_node, node)
 *         if node.val < heap.min_node.val:             # <<<<<<<<<<<<<<
 *             heap.min_node = node
 *     else:
 */
    __pyx_t_1 = ((__pyx_v_node->val < __pyx_v_heap->min_node->val) != 0);
    if (__pyx_t_1) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1185
 *         add_sibling(heap.min_node, node)
 *         if node.val < heap.min_node.val:
 *             heap.min_node = node             # <<<<<<<<<<<<<<
 *     else:
 *         heap.min_node = node
 */
      __pyx_v_heap->min_node = __pyx_v_node;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1184
 *     if heap.min_node:
 *         add_sibling(heap.min_node, node)
 *         if node.val < heap.min_node.val:             # <<<<<<<<<<<<<<
 *             heap.min_node = node
 *     else:
 */
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1182
 *     #              - node is a valid pointer
 *     #              - node is not the child or sibling of another node
 *     if heap.min_node:             # <<<<<<<<<<<<<<
 *         add_sibling(heap.min_node, node)
 *         if node.val < heap.min_node.val:
 */
    goto __pyx_L3;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1187
 *             heap.min_node = node
 *     else:
 *         heap.min_node = node             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_v_heap->min_node = __pyx_v_node;
  }
  __pyx_L3:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1177
 * 
 * 
 * cdef void insert_node(FibonacciHeap* heap,             # <<<<<<<<<<<<<<
 *                       FibonacciNode* node):
 *     # Assumptions: - heap is a valid pointer
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1190
 * 
 * 
 * cdef void decrease_val(FibonacciHeap* heap,             # <<<<<<<<<<<<<<
 *                        FibonacciNode* node,
 *                        DTYPE_t newval):
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_decrease_val(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap *__pyx_v_heap, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node, __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_t __pyx_v_newval) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("decrease_val", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1198
 *     #              - node is not the child or sibling of another node
 *     #              - node is in the heap
 *     node.val = newval             # <<<<<<<<<<<<<<
 *     if node.parent and (node.parent.val >= newval):
 *         remove(node)
 */
  __pyx_v_node->val = __pyx_v_newval;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1199
 *     #              - node is in the heap
 *     node.val = newval
 *     if node.parent and (node.parent.val >= newval):             # <<<<<<<<<<<<<<
 *         remove(node)
 *         insert_node(heap, node)
 */
  __pyx_t_2 = (__pyx_v_node->parent != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_node->parent->val >= __pyx_v_newval) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1200
 *     node.val = newval
 *     if node.parent and (node.parent.val >= newval):
 *         remove(node)             # <<<<<<<<<<<<<<
 *         insert_node(heap, node)
 *     elif heap.min_node.val > node.val:
 */
    __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove(__pyx_v_node);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1201
 *     if node.parent and (node.parent.val >= newval):
 *         remove(node)
 *         insert_node(heap, node)             # <<<<<<<<<<<<<<
 *     elif heap.min_node.val > node.val:
 *         heap.min_node = node
 */
    __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_insert_node(__pyx_v_heap, __pyx_v_node);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1199
 *     #              - node is in the heap
 *     node.val = newval
 *     if node.parent and (node.parent.val >= newval):             # <<<<<<<<<<<<<<
 *         remove(node)
 *         insert_node(heap, node)
 */
    goto __pyx_L3;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1202
 *         remove(node)
 *         insert_node(heap, node)
 *     elif heap.min_node.val > node.val:             # <<<<<<<<<<<<<<
 *         heap.min_node = node
 * 
 */
  __pyx_t_1 = ((__pyx_v_heap->min_node->val > __pyx_v_node->val) != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1203
 *         insert_node(heap, node)
 *     elif heap.min_node.val > node.val:
 *         heap.min_node = node             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_heap->min_node = __pyx_v_node;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1202
 *         remove(node)
 *         insert_node(heap, node)
 *     elif heap.min_node.val > node.val:             # <<<<<<<<<<<<<<
 *         heap.min_node = node
 * 
 */
  }
  __pyx_L3:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1190
 * 
 * 
 * cdef void decrease_val(FibonacciHeap* heap,             # <<<<<<<<<<<<<<
 *                        FibonacciNode* node,
 *                        DTYPE_t newval):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1206
 * 
 * 
 * cdef void link(FibonacciHeap* heap, FibonacciNode* node):             # <<<<<<<<<<<<<<
 *     # Assumptions: - heap is a valid pointer
 *     #              - node is a valid pointer
 */

static void __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_link(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap *__pyx_v_heap, struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_node) {
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_linknode;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("link", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1215
 *     cdef FibonacciNode *child
 * 
 *     if heap.roots_by_rank[node.rank] == NULL:             # <<<<<<<<<<<<<<
 *         heap.roots_by_rank[node.rank] = node
 *     else:
 */
  __pyx_t_1 = (((__pyx_v_heap->roots_by_rank[__pyx_v_node->rank]) == NULL) != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1216
 * 
 *     if heap.roots_by_rank[node.rank] == NULL:
 *         heap.roots_by_rank[node.rank] = node             # <<<<<<<<<<<<<<
 *     else:
 *         linknode = heap.roots_by_rank[node.rank]
 */
    (__pyx_v_heap->roots_by_rank[__pyx_v_node->rank]) = __pyx_v_node;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1215
 *     cdef FibonacciNode *child
 * 
 *     if heap.roots_by_rank[node.rank] == NULL:             # <<<<<<<<<<<<<<
 *         heap.roots_by_rank[node.rank] = node
 *     else:
 */
    goto __pyx_L3;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1218
 *         heap.roots_by_rank[node.rank] = node
 *     else:
 *         linknode = heap.roots_by_rank[node.rank]             # <<<<<<<<<<<<<<
 *         heap.roots_by_rank[node.rank] = NULL
 * 
 */
  /*else*/ {
    __pyx_v_linknode = (__pyx_v_heap->roots_by_rank[__pyx_v_node->rank]);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1219
 *     else:
 *         linknode = heap.roots_by_rank[node.rank]
 *         heap.roots_by_rank[node.rank] = NULL             # <<<<<<<<<<<<<<
 * 
 *         if node.val < linknode.val or node == heap.min_node:
 */
    (__pyx_v_heap->roots_by_rank[__pyx_v_node->rank]) = NULL;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1221
 *         heap.roots_by_rank[node.rank] = NULL
 * 
 *         if node.val < linknode.val or node == heap.min_node:             # <<<<<<<<<<<<<<
 *             remove(linknode)
 *             add_child(node, linknode)
 */
    __pyx_t_2 = ((__pyx_v_node->val < __pyx_v_linknode->val) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_node == __pyx_v_heap->min_node) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_1) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1222
 * 
 *         if node.val < linknode.val or node == heap.min_node:
 *             remove(linknode)             # <<<<<<<<<<<<<<
 *             add_child(node, linknode)
 *             link(heap, node)
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove(__pyx_v_linknode);

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1223
 *         if node.val < linknode.val or node == heap.min_node:
 *             remove(linknode)
 *             add_child(node, linknode)             # <<<<<<<<<<<<<<
 *             link(heap, node)
 *         else:
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_child(__pyx_v_node, __pyx_v_linknode);

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1224
 *             remove(linknode)
 *             add_child(node, linknode)
 *             link(heap, node)             # <<<<<<<<<<<<<<
 *         else:
 *             remove(node)
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_link(__pyx_v_heap, __pyx_v_node);

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1221
 *         heap.roots_by_rank[node.rank] = NULL
 * 
 *         if node.val < linknode.val or node == heap.min_node:             # <<<<<<<<<<<<<<
 *             remove(linknode)
 *             add_child(node, linknode)
 */
      goto __pyx_L4;
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1226
 *             link(heap, node)
 *         else:
 *             remove(node)             # <<<<<<<<<<<<<<
 *             add_child(linknode, node)
 *             link(heap, linknode)
 */
    /*else*/ {
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove(__pyx_v_node);

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1227
 *         else:
 *             remove(node)
 *             add_child(linknode, node)             # <<<<<<<<<<<<<<
 *             link(heap, linknode)
 * 
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_child(__pyx_v_linknode, __pyx_v_node);

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1228
 *             remove(node)
 *             add_child(linknode, node)
 *             link(heap, linknode)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_link(__pyx_v_heap, __pyx_v_linknode);
    }
    __pyx_L4:;
  }
  __pyx_L3:;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1206
 * 
 * 
 * cdef void link(FibonacciHeap* heap, FibonacciNode* node):             # <<<<<<<<<<<<<<
 *     # Assumptions: - heap is a valid pointer
 *     #              - node is a valid pointer
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "scipy/sparse/csgraph/_shortest_path.pyx":1231
 * 
 * 
 * cdef FibonacciNode* remove_min(FibonacciHeap* heap):             # <<<<<<<<<<<<<<
 *     # Assumptions: - heap is a valid pointer
 *     #              - heap.min_node is a valid pointer
 */

static struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove_min(struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciHeap *__pyx_v_heap) {
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_temp;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_temp_right;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_v_out;
  unsigned int __pyx_v_i;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_t_5scipy_6sparse_7csgraph_14_shortest_path_FibonacciNode *__pyx_t_2;
  __Pyx_RefNannySetupContext("remove_min", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1240
 * 
 *     # make all min_node children into root nodes
 *     if heap.min_node.children:             # <<<<<<<<<<<<<<
 *         temp = leftmost_sibling(heap.min_node.children)
 *         temp_right = NULL
 */
  __pyx_t_1 = (__pyx_v_heap->min_node->children != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1241
 *     # make all min_node children into root nodes
 *     if heap.min_node.children:
 *         temp = leftmost_sibling(heap.min_node.children)             # <<<<<<<<<<<<<<
 *         temp_right = NULL
 * 
 */
    __pyx_v_temp = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_leftmost_sibling(__pyx_v_heap->min_node->children);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1242
 *     if heap.min_node.children:
 *         temp = leftmost_sibling(heap.min_node.children)
 *         temp_right = NULL             # <<<<<<<<<<<<<<
 * 
 *         while temp:
 */
    __pyx_v_temp_right = NULL;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1244
 *         temp_right = NULL
 * 
 *         while temp:             # <<<<<<<<<<<<<<
 *             temp_right = temp.right_sibling
 *             remove(temp)
 */
    while (1) {
      __pyx_t_1 = (__pyx_v_temp != 0);
      if (!__pyx_t_1) break;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1245
 * 
 *         while temp:
 *             temp_right = temp.right_sibling             # <<<<<<<<<<<<<<
 *             remove(temp)
 *             add_sibling(heap.min_node, temp)
 */
      __pyx_t_2 = __pyx_v_temp->right_sibling;
      __pyx_v_temp_right = __pyx_t_2;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1246
 *         while temp:
 *             temp_right = temp.right_sibling
 *             remove(temp)             # <<<<<<<<<<<<<<
 *             add_sibling(heap.min_node, temp)
 *             temp = temp_right
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove(__pyx_v_temp);

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1247
 *             temp_right = temp.right_sibling
 *             remove(temp)
 *             add_sibling(heap.min_node, temp)             # <<<<<<<<<<<<<<
 *             temp = temp_right
 * 
 */
      __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_add_sibling(__pyx_v_heap->min_node, __pyx_v_temp);

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1248
 *             remove(temp)
 *             add_sibling(heap.min_node, temp)
 *             temp = temp_right             # <<<<<<<<<<<<<<
 * 
 *         heap.min_node.children = NULL
 */
      __pyx_v_temp = __pyx_v_temp_right;
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1250
 *             temp = temp_right
 * 
 *         heap.min_node.children = NULL             # <<<<<<<<<<<<<<
 * 
 *     # choose a root node other than min_node
 */
    __pyx_v_heap->min_node->children = NULL;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1240
 * 
 *     # make all min_node children into root nodes
 *     if heap.min_node.children:             # <<<<<<<<<<<<<<
 *         temp = leftmost_sibling(heap.min_node.children)
 *         temp_right = NULL
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1253
 * 
 *     # choose a root node other than min_node
 *     temp = leftmost_sibling(heap.min_node)             # <<<<<<<<<<<<<<
 *     if temp == heap.min_node:
 *         if heap.min_node.right_sibling:
 */
  __pyx_v_temp = __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_leftmost_sibling(__pyx_v_heap->min_node);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1254
 *     # choose a root node other than min_node
 *     temp = leftmost_sibling(heap.min_node)
 *     if temp == heap.min_node:             # <<<<<<<<<<<<<<
 *         if heap.min_node.right_sibling:
 *             temp = heap.min_node.right_sibling
 */
  __pyx_t_1 = ((__pyx_v_temp == __pyx_v_heap->min_node) != 0);
  if (__pyx_t_1) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1255
 *     temp = leftmost_sibling(heap.min_node)
 *     if temp == heap.min_node:
 *         if heap.min_node.right_sibling:             # <<<<<<<<<<<<<<
 *             temp = heap.min_node.right_sibling
 *         else:
 */
    __pyx_t_1 = (__pyx_v_heap->min_node->right_sibling != 0);
    if (__pyx_t_1) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1256
 *     if temp == heap.min_node:
 *         if heap.min_node.right_sibling:
 *             temp = heap.min_node.right_sibling             # <<<<<<<<<<<<<<
 *         else:
 *             out = heap.min_node
 */
      __pyx_t_2 = __pyx_v_heap->min_node->right_sibling;
      __pyx_v_temp = __pyx_t_2;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1255
 *     temp = leftmost_sibling(heap.min_node)
 *     if temp == heap.min_node:
 *         if heap.min_node.right_sibling:             # <<<<<<<<<<<<<<
 *             temp = heap.min_node.right_sibling
 *         else:
 */
      goto __pyx_L7;
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1258
 *             temp = heap.min_node.right_sibling
 *         else:
 *             out = heap.min_node             # <<<<<<<<<<<<<<
 *             heap.min_node = NULL
 *             return out
 */
    /*else*/ {
      __pyx_t_2 = __pyx_v_heap->min_node;
      __pyx_v_out = __pyx_t_2;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1259
 *         else:
 *             out = heap.min_node
 *             heap.min_node = NULL             # <<<<<<<<<<<<<<
 *             return out
 * 
 */
      __pyx_v_heap->min_node = NULL;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1260
 *             out = heap.min_node
 *             heap.min_node = NULL
 *             return out             # <<<<<<<<<<<<<<
 * 
 *     # remove min_node, and point heap to the new min
 */
      __pyx_r = __pyx_v_out;
      goto __pyx_L0;
    }
    __pyx_L7:;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1254
 *     # choose a root node other than min_node
 *     temp = leftmost_sibling(heap.min_node)
 *     if temp == heap.min_node:             # <<<<<<<<<<<<<<
 *         if heap.min_node.right_sibling:
 *             temp = heap.min_node.right_sibling
 */
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1263
 * 
 *     # remove min_node, and point heap to the new min
 *     out = heap.min_node             # <<<<<<<<<<<<<<
 *     remove(heap.min_node)
 *     heap.min_node = temp
 */
  __pyx_t_2 = __pyx_v_heap->min_node;
  __pyx_v_out = __pyx_t_2;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1264
 *     # remove min_node, and point heap to the new min
 *     out = heap.min_node
 *     remove(heap.min_node)             # <<<<<<<<<<<<<<
 *     heap.min_node = temp
 * 
 */
  __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_remove(__pyx_v_heap->min_node);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1265
 *     out = heap.min_node
 *     remove(heap.min_node)
 *     heap.min_node = temp             # <<<<<<<<<<<<<<
 * 
 *     # re-link the heap
 */
  __pyx_v_heap->min_node = __pyx_v_temp;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1268
 * 
 *     # re-link the heap
 *     for i from 0 <= i < 100:             # <<<<<<<<<<<<<<
 *         heap.roots_by_rank[i] = NULL
 * 
 */
  for (__pyx_v_i = 0; __pyx_v_i < 0x64; __pyx_v_i++) {

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1269
 *     # re-link the heap
 *     for i from 0 <= i < 100:
 *         heap.roots_by_rank[i] = NULL             # <<<<<<<<<<<<<<
 * 
 *     while temp:
 */
    (__pyx_v_heap->roots_by_rank[__pyx_v_i]) = NULL;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1271
 *         heap.roots_by_rank[i] = NULL
 * 
 *     while temp:             # <<<<<<<<<<<<<<
 *         if temp.val < heap.min_node.val:
 *             heap.min_node = temp
 */
  while (1) {
    __pyx_t_1 = (__pyx_v_temp != 0);
    if (!__pyx_t_1) break;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1272
 * 
 *     while temp:
 *         if temp.val < heap.min_node.val:             # <<<<<<<<<<<<<<
 *             heap.min_node = temp
 *         temp_right = temp.right_sibling
 */
    __pyx_t_1 = ((__pyx_v_temp->val < __pyx_v_heap->min_node->val) != 0);
    if (__pyx_t_1) {

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1273
 *     while temp:
 *         if temp.val < heap.min_node.val:
 *             heap.min_node = temp             # <<<<<<<<<<<<<<
 *         temp_right = temp.right_sibling
 *         link(heap, temp)
 */
      __pyx_v_heap->min_node = __pyx_v_temp;

      /* "scipy/sparse/csgraph/_shortest_path.pyx":1272
 * 
 *     while temp:
 *         if temp.val < heap.min_node.val:             # <<<<<<<<<<<<<<
 *             heap.min_node = temp
 *         temp_right = temp.right_sibling
 */
    }

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1274
 *         if temp.val < heap.min_node.val:
 *             heap.min_node = temp
 *         temp_right = temp.right_sibling             # <<<<<<<<<<<<<<
 *         link(heap, temp)
 *         temp = temp_right
 */
    __pyx_t_2 = __pyx_v_temp->right_sibling;
    __pyx_v_temp_right = __pyx_t_2;

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1275
 *             heap.min_node = temp
 *         temp_right = temp.right_sibling
 *         link(heap, temp)             # <<<<<<<<<<<<<<
 *         temp = temp_right
 * 
 */
    __pyx_f_5scipy_6sparse_7csgraph_14_shortest_path_link(__pyx_v_heap, __pyx_v_temp);

    /* "scipy/sparse/csgraph/_shortest_path.pyx":1276
 *         temp_right = temp.right_sibling
 *         link(heap, temp)
 *         temp = temp_right             # <<<<<<<<<<<<<<
 * 
 *     return out
 */
    __pyx_v_temp = __pyx_v_temp_right;
  }

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1278
 *         temp = temp_right
 * 
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1231
 * 
 * 
 * cdef FibonacciNode* remove_min(FibonacciHeap* heap):             # <<<<<<<<<<<<<<
 *     # Assumptions: - heap is a valid pointer
 *     #              - heap.min_node is a valid pointer
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_copy_shape;
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_v_hasfields;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":203
 *             # of flags
 * 
 *             if info == NULL: return             # <<<<<<<<<<<<<<
 * 
 *             cdef int copy_shape, i, ndim
 */
  __pyx_t_1 = ((__pyx_v_info == NULL) != 0);
  if (__pyx_t_1) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":206
 * 
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":207
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":209
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":212
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 copy_shape = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 copy_shape = 0
 */
    __pyx_v_copy_shape = 1;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
    goto __pyx_L4;
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":214
 *                 copy_shape = 1
 *             else:
 *                 copy_shape = 0             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  /*else*/ {
    __pyx_v_copy_shape = 0;
  }
  __pyx_L4:;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":217
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":221
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":224
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if copy_shape:
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":225
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if copy_shape:
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (__pyx_v_copy_shape != 0);
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":229
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)malloc((((sizeof(Py_ssize_t)) * ((size_t)__pyx_v_ndim)) * 2)));

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":230
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":231
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":232
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":233
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L11;
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":235
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":236
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L11:;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":237
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":238
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":239
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":242
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = self.descr
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":243
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = self.descr             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->descr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":246
 *             cdef int offset
 * 
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields and not copy_shape:
 */
  __pyx_v_hasfields = PyDataType_HASFIELDS(__pyx_v_descr);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
  __pyx_t_2 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_copy_shape != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":250
 *             if not hasfields and not copy_shape:
 *                 # do not call releasebuffer
 *                 info.obj = None             # <<<<<<<<<<<<<<
 *             else:
 *                 # need to call releasebuffer
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = Py_None;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
    goto __pyx_L14;
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":253
 *             else:
 *                 # need to call releasebuffer
 *                 info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields:
 */
  /*else*/ {
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = ((PyObject *)__pyx_v_self);
  }
  __pyx_L14:;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":256
 * 
 *             if not hasfields:
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L20_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_L20_next_or:;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L19_bool_binop_done:;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_1) {

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":260
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = __pyx_k_b;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":261
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      case NPY_UBYTE:
      __pyx_v_f = __pyx_k_B;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":262
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      case NPY_SHORT:
      __pyx_v_f = __pyx_k_h;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":263
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      case NPY_USHORT:
      __pyx_v_f = __pyx_k_H;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":264
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      case NPY_INT:
      __pyx_v_f = __pyx_k_i;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":265
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      case NPY_UINT:
      __pyx_v_f = __pyx_k_I;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      case NPY_LONG:
      __pyx_v_f = __pyx_k_l;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":267
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      case NPY_ULONG:
      __pyx_v_f = __pyx_k_L;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      case NPY_LONGLONG:
      __pyx_v_f = __pyx_k_q;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":269
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      case NPY_ULONGLONG:
      __pyx_v_f = __pyx_k_Q;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      case NPY_FLOAT:
      __pyx_v_f = __pyx_k_f;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":271
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      case NPY_DOUBLE:
      __pyx_v_f = __pyx_k_d;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      case NPY_LONGDOUBLE:
      __pyx_v_f = __pyx_k_g;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":273
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      case NPY_CFLOAT:
      __pyx_v_f = __pyx_k_Zf;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      case NPY_CDOUBLE:
      __pyx_v_f = __pyx_k_Zd;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":275
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      case NPY_CLONGDOUBLE:
      __pyx_v_f = __pyx_k_Zg;
      break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      case NPY_OBJECT:
      __pyx_v_f = __pyx_k_O;
      break;
      default:

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      break;
    }

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":279
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":282
 *                 return
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)malloc(0xFF));

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_7 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_f = __pyx_t_7;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":288
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":292
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)
 */
    free(__pyx_v_info->format);

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":294
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    free(__pyx_v_info->strides);

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":771
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":774
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 774; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":780
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 780; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":783
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 783; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":790
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":791
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":795
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":796
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (__pyx_t_6) {

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_6) {

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":813
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":814
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":815
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":816
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":818
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (__pyx_t_6) {

        /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":826
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":828
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":829
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":831
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":832
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":834
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":835
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":837
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":838
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":839
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":841
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":844
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L15:;

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":845
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":849
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":850
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  PyObject *__pyx_v_baseptr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
  __pyx_t_1 = (__pyx_v_base == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":969
 *      cdef PyObject* baseptr
 *      if base is None:
 *          baseptr = NULL             # <<<<<<<<<<<<<<
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 */
    __pyx_v_baseptr = NULL;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
    goto __pyx_L3;
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":971
 *          baseptr = NULL
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!             # <<<<<<<<<<<<<<
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 */
  /*else*/ {
    Py_INCREF(__pyx_v_base);

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":972
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base             # <<<<<<<<<<<<<<
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr
 */
    __pyx_v_baseptr = ((PyObject *)__pyx_v_base);
  }
  __pyx_L3:;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":973
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)             # <<<<<<<<<<<<<<
 *      arr.base = baseptr
 * 
 */
  Py_XDECREF(__pyx_v_arr->base);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":974
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_v_arr->base = __pyx_v_baseptr;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arr->base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":978
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return <object>arr.base
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":980
 *         return None
 *     else:
 *         return <object>arr.base             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_arr->base));
    __pyx_r = ((PyObject *)__pyx_v_arr->base);
    goto __pyx_L0;
  }

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_shortest_path",
    __pyx_k_Routines_for_performing_shortes, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_n_s_BF, __pyx_k_BF, sizeof(__pyx_k_BF), 0, 0, 1, 1},
  {&__pyx_n_s_C, __pyx_k_C, sizeof(__pyx_k_C), 0, 0, 1, 1},
  {&__pyx_n_s_D, __pyx_k_D, sizeof(__pyx_k_D), 0, 0, 1, 1},
  {&__pyx_n_s_DTYPE, __pyx_k_DTYPE, sizeof(__pyx_k_DTYPE), 0, 0, 1, 1},
  {&__pyx_n_s_Exception, __pyx_k_Exception, sizeof(__pyx_k_Exception), 0, 0, 1, 1},
  {&__pyx_n_s_FW, __pyx_k_FW, sizeof(__pyx_k_FW), 0, 0, 1, 1},
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_kp_s_Graph_has_negative_weights_dijks, __pyx_k_Graph_has_negative_weights_dijks, sizeof(__pyx_k_Graph_has_negative_weights_dijks), 0, 0, 1, 0},
  {&__pyx_n_s_ITYPE, __pyx_k_ITYPE, sizeof(__pyx_k_ITYPE), 0, 0, 1, 1},
  {&__pyx_n_s_J, __pyx_k_J, sizeof(__pyx_k_J), 0, 0, 1, 1},
  {&__pyx_n_s_N, __pyx_k_N, sizeof(__pyx_k_N), 0, 0, 1, 1},
  {&__pyx_n_s_NegativeCycleError, __pyx_k_NegativeCycleError, sizeof(__pyx_k_NegativeCycleError), 0, 0, 1, 1},
  {&__pyx_n_s_NegativeCycleError___init, __pyx_k_NegativeCycleError___init, sizeof(__pyx_k_NegativeCycleError___init), 0, 0, 1, 1},
  {&__pyx_kp_s_Negative_cycle_detected_on_node, __pyx_k_Negative_cycle_detected_on_node, sizeof(__pyx_k_Negative_cycle_detected_on_node), 0, 0, 1, 0},
  {&__pyx_kp_s_Negative_cycle_in_nodes_s, __pyx_k_Negative_cycle_in_nodes_s, sizeof(__pyx_k_Negative_cycle_in_nodes_s), 0, 0, 1, 0},
  {&__pyx_n_s_Nk, __pyx_k_Nk, sizeof(__pyx_k_Nk), 0, 0, 1, 1},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_T, __pyx_k_T, sizeof(__pyx_k_T), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_any, __pyx_k_any, sizeof(__pyx_k_any), 0, 0, 1, 1},
  {&__pyx_n_s_arange, __pyx_k_arange, sizeof(__pyx_k_arange), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_n_s_atleast_1d, __pyx_k_atleast_1d, sizeof(__pyx_k_atleast_1d), 0, 0, 1, 1},
  {&__pyx_n_s_auto, __pyx_k_auto, sizeof(__pyx_k_auto), 0, 0, 1, 1},
  {&__pyx_n_s_bellman_ford, __pyx_k_bellman_ford, sizeof(__pyx_k_bellman_ford), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_copy_if_dense, __pyx_k_copy_if_dense, sizeof(__pyx_k_copy_if_dense), 0, 0, 1, 1},
  {&__pyx_n_s_copy_if_sparse, __pyx_k_copy_if_sparse, sizeof(__pyx_k_copy_if_sparse), 0, 0, 1, 1},
  {&__pyx_n_s_csgraph, __pyx_k_csgraph, sizeof(__pyx_k_csgraph), 0, 0, 1, 1},
  {&__pyx_n_s_csgraphT, __pyx_k_csgraphT, sizeof(__pyx_k_csgraphT), 0, 0, 1, 1},
  {&__pyx_n_s_csrT_data, __pyx_k_csrT_data, sizeof(__pyx_k_csrT_data), 0, 0, 1, 1},
  {&__pyx_n_s_csr_data, __pyx_k_csr_data, sizeof(__pyx_k_csr_data), 0, 0, 1, 1},
  {&__pyx_n_s_csr_matrix, __pyx_k_csr_matrix, sizeof(__pyx_k_csr_matrix), 0, 0, 1, 1},
  {&__pyx_n_s_csr_output, __pyx_k_csr_output, sizeof(__pyx_k_csr_output), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_dense_output, __pyx_k_dense_output, sizeof(__pyx_k_dense_output), 0, 0, 1, 1},
  {&__pyx_n_s_diagonal, __pyx_k_diagonal, sizeof(__pyx_k_diagonal), 0, 0, 1, 1},
  {&__pyx_n_s_dijkstra, __pyx_k_dijkstra, sizeof(__pyx_k_dijkstra), 0, 0, 1, 1},
  {&__pyx_n_s_directed, __pyx_k_directed, sizeof(__pyx_k_directed), 0, 0, 1, 1},
  {&__pyx_n_s_dist_array, __pyx_k_dist_array, sizeof(__pyx_k_dist_array), 0, 0, 1, 1},
  {&__pyx_n_s_dist_matrix, __pyx_k_dist_matrix, sizeof(__pyx_k_dist_matrix), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_fill, __pyx_k_fill, sizeof(__pyx_k_fill), 0, 0, 1, 1},
  {&__pyx_n_s_flat, __pyx_k_flat, sizeof(__pyx_k_flat), 0, 0, 1, 1},
  {&__pyx_n_s_float64, __pyx_k_float64, sizeof(__pyx_k_float64), 0, 0, 1, 1},
  {&__pyx_n_s_floyd_warshall, __pyx_k_floyd_warshall, sizeof(__pyx_k_floyd_warshall), 0, 0, 1, 1},
  {&__pyx_kp_s_home_br_repos_scipy_scipy_spars, __pyx_k_home_br_repos_scipy_scipy_spars, sizeof(__pyx_k_home_br_repos_scipy_scipy_spars), 0, 0, 1, 0},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_indices, __pyx_k_indices, sizeof(__pyx_k_indices), 0, 0, 1, 1},
  {&__pyx_kp_s_indices_out_of_range_0_N, __pyx_k_indices_out_of_range_0_N, sizeof(__pyx_k_indices_out_of_range_0_N), 0, 0, 1, 0},
  {&__pyx_n_s_indptr, __pyx_k_indptr, sizeof(__pyx_k_indptr), 0, 0, 1, 1},
  {&__pyx_n_s_inf, __pyx_k_inf, sizeof(__pyx_k_inf), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_int32, __pyx_k_int32, sizeof(__pyx_k_int32), 0, 0, 1, 1},
  {&__pyx_n_s_isinf, __pyx_k_isinf, sizeof(__pyx_k_isinf), 0, 0, 1, 1},
  {&__pyx_n_s_isscalar, __pyx_k_isscalar, sizeof(__pyx_k_isscalar), 0, 0, 1, 1},
  {&__pyx_n_s_isspmatrix, __pyx_k_isspmatrix, sizeof(__pyx_k_isspmatrix), 0, 0, 1, 1},
  {&__pyx_n_s_isspmatrix_csc, __pyx_k_isspmatrix_csc, sizeof(__pyx_k_isspmatrix_csc), 0, 0, 1, 1},
  {&__pyx_n_s_isspmatrix_csr, __pyx_k_isspmatrix_csr, sizeof(__pyx_k_isspmatrix_csr), 0, 0, 1, 1},
  {&__pyx_n_s_johnson, __pyx_k_johnson, sizeof(__pyx_k_johnson), 0, 0, 1, 1},
  {&__pyx_n_s_limit, __pyx_k_limit, sizeof(__pyx_k_limit), 0, 0, 1, 1},
  {&__pyx_kp_s_limit_must_be_0, __pyx_k_limit_must_be_0, sizeof(__pyx_k_limit_must_be_0), 0, 0, 1, 0},
  {&__pyx_kp_s_limit_must_be_numeric_float, __pyx_k_limit_must_be_numeric_float, sizeof(__pyx_k_limit_must_be_numeric_float), 0, 0, 1, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_message, __pyx_k_message, sizeof(__pyx_k_message), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_method, __pyx_k_method, sizeof(__pyx_k_method), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_nnz, __pyx_k_nnz, sizeof(__pyx_k_nnz), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_ones, __pyx_k_ones, sizeof(__pyx_k_ones), 0, 0, 1, 1},
  {&__pyx_n_s_order, __pyx_k_order, sizeof(__pyx_k_order), 0, 0, 1, 1},
  {&__pyx_n_s_overwrite, __pyx_k_overwrite, sizeof(__pyx_k_overwrite), 0, 0, 1, 1},
  {&__pyx_n_s_predecessor_matrix, __pyx_k_predecessor_matrix, sizeof(__pyx_k_predecessor_matrix), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reshape, __pyx_k_reshape, sizeof(__pyx_k_reshape), 0, 0, 1, 1},
  {&__pyx_n_s_ret, __pyx_k_ret, sizeof(__pyx_k_ret), 0, 0, 1, 1},
  {&__pyx_n_s_return_predecessors, __pyx_k_return_predecessors, sizeof(__pyx_k_return_predecessors), 0, 0, 1, 1},
  {&__pyx_n_s_return_shape, __pyx_k_return_shape, sizeof(__pyx_k_return_shape), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_sparse, __pyx_k_scipy_sparse, sizeof(__pyx_k_scipy_sparse), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_sparse_csgraph__shortest_p, __pyx_k_scipy_sparse_csgraph__shortest_p, sizeof(__pyx_k_scipy_sparse_csgraph__shortest_p), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_sparse_csgraph__validation, __pyx_k_scipy_sparse_csgraph__validation, sizeof(__pyx_k_scipy_sparse_csgraph__validation), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_shortest_path, __pyx_k_shortest_path, sizeof(__pyx_k_shortest_path), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_tocsr, __pyx_k_tocsr, sizeof(__pyx_k_tocsr), 0, 0, 1, 1},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_kp_s_unrecognized_method_s, __pyx_k_unrecognized_method_s, sizeof(__pyx_k_unrecognized_method_s), 0, 0, 1, 0},
  {&__pyx_n_s_unweighted, __pyx_k_unweighted, sizeof(__pyx_k_unweighted), 0, 0, 1, 1},
  {&__pyx_n_s_validate_graph, __pyx_k_validate_graph, sizeof(__pyx_k_validate_graph), 0, 0, 1, 1},
  {&__pyx_n_s_warn, __pyx_k_warn, sizeof(__pyx_k_warn), 0, 0, 1, 1},
  {&__pyx_n_s_warnings, __pyx_k_warnings, sizeof(__pyx_k_warnings), 0, 0, 1, 1},
  {&__pyx_n_s_where, __pyx_k_where, sizeof(__pyx_k_where), 0, 0, 1, 1},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_Exception = __Pyx_GetBuiltinName(__pyx_n_s_Exception); if (!__pyx_builtin_Exception) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":233
 *                                       dtype=ITYPE, order='C')
 *     else:
 *         predecessor_matrix = np.empty((0, 0), dtype=ITYPE)             # <<<<<<<<<<<<<<
 * 
 *     _floyd_warshall(dist_matrix,
 */
  __pyx_tuple__2 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_0); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_tuple__2); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":398
 * 
 *     if np.any(csgraph.data < 0):
 *         warnings.warn("Graph has negative weights: dijkstra will give "             # <<<<<<<<<<<<<<
 *                       "inaccurate results if the graph contains negative "
 *                       "cycles. Consider johnson or bellman_ford.")
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_Graph_has_negative_weights_dijks); if (unlikely(!__pyx_tuple__5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":412
 *         indices = np.array(indices, order='C', dtype=ITYPE, copy=True)
 *         return_shape = indices.shape + (N,)
 *         indices = np.atleast_1d(indices).reshape(-1)             # <<<<<<<<<<<<<<
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_int_neg_1); if (unlikely(!__pyx_tuple__6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":415
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")             # <<<<<<<<<<<<<<
 * 
 *     if not np.isscalar(limit):
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_indices_out_of_range_0_N); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":418
 * 
 *     if not np.isscalar(limit):
 *         raise TypeError('limit must be numeric (float)')             # <<<<<<<<<<<<<<
 *     limit = float(limit)
 *     if limit < 0:
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_limit_must_be_numeric_float); if (unlikely(!__pyx_tuple__8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":421
 *     limit = float(limit)
 *     if limit < 0:
 *         raise ValueError('limit must be >= 0')             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_limit_must_be_0); if (unlikely(!__pyx_tuple__9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":683
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")             # <<<<<<<<<<<<<<
 *     return_shape = indices.shape + (N,)
 *     indices = np.atleast_1d(indices).reshape(-1)
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_indices_out_of_range_0_N); if (unlikely(!__pyx_tuple__10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":685
 *             raise ValueError("indices out of range 0...N")
 *     return_shape = indices.shape + (N,)
 *     indices = np.atleast_1d(indices).reshape(-1)             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_int_neg_1); if (unlikely(!__pyx_tuple__11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 685; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":901
 *         indices = np.array(indices, order='C', dtype=ITYPE)
 *         return_shape = indices.shape + (N,)
 *         indices = np.atleast_1d(indices).reshape(-1)             # <<<<<<<<<<<<<<
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_int_neg_1); if (unlikely(!__pyx_tuple__12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":904
 *         indices[indices < 0] += N
 *         if np.any(indices < 0) or np.any(indices >= N):
 *             raise ValueError("indices out of range 0...N")             # <<<<<<<<<<<<<<
 * 
 *     #------------------------------
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_indices_out_of_range_0_N); if (unlikely(!__pyx_tuple__13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 904; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":961
 *     # correct the distance matrix for the bellman-ford weights
 *     dist_matrix += dist_array
 *     dist_matrix -= dist_array[:, None][indices]             # <<<<<<<<<<<<<<
 * 
 *     if return_predecessors:
 */
  __pyx_slice__14 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 961; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_slice__14);
  __Pyx_GIVEREF(__pyx_slice__14);
  __pyx_tuple__15 = PyTuple_Pack(2, __pyx_slice__14, Py_None); if (unlikely(!__pyx_tuple__15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 961; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple__16)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__17)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__18)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__19)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__20)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__21)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":28
 * 
 * class NegativeCycleError(Exception):
 *     def __init__(self, message=''):             # <<<<<<<<<<<<<<
 *         Exception.__init__(self, message)
 * 
 */
  __pyx_tuple__22 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_message); if (unlikely(!__pyx_tuple__22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);
  __pyx_codeobj__23 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__22, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_spars, __pyx_n_s_init, 28, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__23)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_tuple__24 = PyTuple_Pack(1, ((PyObject*)__pyx_kp_s_)); if (unlikely(!__pyx_tuple__24)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":32
 * 
 * 
 * def shortest_path(csgraph, method='auto',             # <<<<<<<<<<<<<<
 *                   directed=True,
 *                   return_predecessors=False,
 */
  __pyx_tuple__25 = PyTuple_Pack(8, __pyx_n_s_csgraph, __pyx_n_s_method, __pyx_n_s_directed, __pyx_n_s_return_predecessors, __pyx_n_s_unweighted, __pyx_n_s_overwrite, __pyx_n_s_N, __pyx_n_s_Nk); if (unlikely(!__pyx_tuple__25)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(6, 0, 8, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_spars, __pyx_n_s_shortest_path, 32, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":170
 * 
 * 
 * def floyd_warshall(csgraph, directed=True,             # <<<<<<<<<<<<<<
 *                    return_predecessors=False,
 *                    unweighted=False,
 */
  __pyx_tuple__27 = PyTuple_Pack(7, __pyx_n_s_csgraph, __pyx_n_s_directed, __pyx_n_s_return_predecessors, __pyx_n_s_unweighted, __pyx_n_s_overwrite, __pyx_n_s_dist_matrix, __pyx_n_s_predecessor_matrix); if (unlikely(!__pyx_tuple__27)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__28 = (PyObject*)__Pyx_PyCode_New(5, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_spars, __pyx_n_s_floyd_warshall, 170, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__28)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":325
 * 
 * 
 * def dijkstra(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *              return_predecessors=False,
 *              unweighted=False, limit=np.inf):
 */
  __pyx_tuple__29 = PyTuple_Pack(13, __pyx_n_s_csgraph, __pyx_n_s_directed, __pyx_n_s_indices, __pyx_n_s_return_predecessors, __pyx_n_s_unweighted, __pyx_n_s_limit, __pyx_n_s_N, __pyx_n_s_return_shape, __pyx_n_s_dist_matrix, __pyx_n_s_predecessor_matrix, __pyx_n_s_csr_data, __pyx_n_s_csgraphT, __pyx_n_s_csrT_data); if (unlikely(!__pyx_tuple__29)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(6, 0, 13, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_spars, __pyx_n_s_dijkstra, 325, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":606
 * 
 * 
 * def bellman_ford(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *                  return_predecessors=False,
 *                  unweighted=False):
 */
  __pyx_tuple__31 = PyTuple_Pack(11, __pyx_n_s_csgraph, __pyx_n_s_directed, __pyx_n_s_indices, __pyx_n_s_return_predecessors, __pyx_n_s_unweighted, __pyx_n_s_N, __pyx_n_s_return_shape, __pyx_n_s_dist_matrix, __pyx_n_s_predecessor_matrix, __pyx_n_s_csr_data, __pyx_n_s_ret); if (unlikely(!__pyx_tuple__31)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(5, 0, 11, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_spars, __pyx_n_s_bellman_ford, 606, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/sparse/csgraph/_shortest_path.pyx":818
 * 
 * 
 * def johnson(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *             return_predecessors=False,
 *             unweighted=False):
 */
  __pyx_tuple__33 = PyTuple_Pack(13, __pyx_n_s_csgraph, __pyx_n_s_directed, __pyx_n_s_indices, __pyx_n_s_return_predecessors, __pyx_n_s_unweighted, __pyx_n_s_N, __pyx_n_s_return_shape, __pyx_n_s_dist_matrix, __pyx_n_s_predecessor_matrix, __pyx_n_s_dist_array, __pyx_n_s_csr_data, __pyx_n_s_ret, __pyx_n_s_csgraphT); if (unlikely(!__pyx_tuple__33)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(5, 0, 13, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_spars, __pyx_n_s_johnson, 818, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_shortest_path(void); /*proto*/
PyMODINIT_FUNC init_shortest_path(void)
#else
PyMODINIT_FUNC PyInit__shortest_path(void); /*proto*/
PyMODINIT_FUNC PyInit__shortest_path(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__shortest_path(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_shortest_path", __pyx_methods, __pyx_k_Routines_for_performing_shortes, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_scipy__sparse__csgraph___shortest_path) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "scipy.sparse.csgraph._shortest_path")) {
      if (unlikely(PyDict_SetItemString(modules, "scipy.sparse.csgraph._shortest_path", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_5numpy_dtype)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_flatiter)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_broadcast)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_5numpy_ndarray)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType("numpy", "ufunc", sizeof(PyUFuncObject), 0); if (unlikely(!__pyx_ptype_5numpy_ufunc)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "scipy/sparse/csgraph/_shortest_path.pyx":12
 * # Author: Jake Vanderplas  -- <vanderplas@astro.washington.edu>
 * # License: BSD, (C) 2011
 * import warnings             # <<<<<<<<<<<<<<
 * 
 * import numpy as np
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_warnings, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warnings, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":14
 * import warnings
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":17
 * cimport numpy as np
 * 
 * from scipy.sparse import csr_matrix, isspmatrix, isspmatrix_csr, isspmatrix_csc             # <<<<<<<<<<<<<<
 * from scipy.sparse.csgraph._validation import validate_graph
 * 
 */
  __pyx_t_1 = PyList_New(4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_csr_matrix);
  __Pyx_GIVEREF(__pyx_n_s_csr_matrix);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_csr_matrix);
  __Pyx_INCREF(__pyx_n_s_isspmatrix);
  __Pyx_GIVEREF(__pyx_n_s_isspmatrix);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_isspmatrix);
  __Pyx_INCREF(__pyx_n_s_isspmatrix_csr);
  __Pyx_GIVEREF(__pyx_n_s_isspmatrix_csr);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_isspmatrix_csr);
  __Pyx_INCREF(__pyx_n_s_isspmatrix_csc);
  __Pyx_GIVEREF(__pyx_n_s_isspmatrix_csc);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_s_isspmatrix_csc);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_scipy_sparse, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_csr_matrix); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_csr_matrix, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_isspmatrix); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_isspmatrix, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_isspmatrix_csr); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_isspmatrix_csr, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_isspmatrix_csc); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_isspmatrix_csc, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":18
 * 
 * from scipy.sparse import csr_matrix, isspmatrix, isspmatrix_csr, isspmatrix_csc
 * from scipy.sparse.csgraph._validation import validate_graph             # <<<<<<<<<<<<<<
 * 
 * cimport cython
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 18; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_validate_graph);
  __Pyx_GIVEREF(__pyx_n_s_validate_graph);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_validate_graph);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_scipy_sparse_csgraph__validation, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 18; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_validate_graph); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 18; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_validate_graph, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 18; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "parameters.pxi":2
 * 
 * DTYPE = np.float64             # <<<<<<<<<<<<<<
 * ctypedef np.float64_t DTYPE_t
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 2; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 2; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 2; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "parameters.pxi":5
 * ctypedef np.float64_t DTYPE_t
 * 
 * ITYPE = np.int32             # <<<<<<<<<<<<<<
 * ctypedef np.int32_t ITYPE_t
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 5; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 5; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ITYPE, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 5; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "parameters.pxi":14
 * 
 * # EPS is the precision of DTYPE
 * cdef DTYPE_t DTYPE_EPS = 1E-15             # <<<<<<<<<<<<<<
 * 
 * # NULL_IDX is the index used in predecessor matrices to store a non-path
 */
  __pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_DTYPE_EPS = 1E-15;

  /* "parameters.pxi":17
 * 
 * # NULL_IDX is the index used in predecessor matrices to store a non-path
 * cdef ITYPE_t NULL_IDX = -9999             # <<<<<<<<<<<<<<
 */
  __pyx_v_5scipy_6sparse_7csgraph_14_shortest_path_NULL_IDX = -9999;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":27
 * 
 * 
 * class NegativeCycleError(Exception):             # <<<<<<<<<<<<<<
 *     def __init__(self, message=''):
 *         Exception.__init__(self, message)
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_builtin_Exception);
  __Pyx_GIVEREF(__pyx_builtin_Exception);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_builtin_Exception);
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_NegativeCycleError, __pyx_n_s_NegativeCycleError, (PyObject *) NULL, __pyx_n_s_scipy_sparse_csgraph__shortest_p, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "scipy/sparse/csgraph/_shortest_path.pyx":28
 * 
 * class NegativeCycleError(Exception):
 *     def __init__(self, message=''):             # <<<<<<<<<<<<<<
 *         Exception.__init__(self, message)
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_18NegativeCycleError_1__init__, 0, __pyx_n_s_NegativeCycleError___init, NULL, __pyx_n_s_scipy_sparse_csgraph__shortest_p, __pyx_d, ((PyObject *)__pyx_codeobj__23)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__24);
  if (PyObject_SetItem(__pyx_t_3, __pyx_n_s_init, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":27
 * 
 * 
 * class NegativeCycleError(Exception):             # <<<<<<<<<<<<<<
 *     def __init__(self, message=''):
 *         Exception.__init__(self, message)
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_NegativeCycleError, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_NegativeCycleError, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":32
 * 
 * 
 * def shortest_path(csgraph, method='auto',             # <<<<<<<<<<<<<<
 *                   directed=True,
 *                   return_predecessors=False,
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_1shortest_path, NULL, __pyx_n_s_scipy_sparse_csgraph__shortest_p); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_shortest_path, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":170
 * 
 * 
 * def floyd_warshall(csgraph, directed=True,             # <<<<<<<<<<<<<<
 *                    return_predecessors=False,
 *                    unweighted=False,
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_3floyd_warshall, NULL, __pyx_n_s_scipy_sparse_csgraph__shortest_p); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_floyd_warshall, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":327
 * def dijkstra(csgraph, directed=True, indices=None,
 *              return_predecessors=False,
 *              unweighted=False, limit=np.inf):             # <<<<<<<<<<<<<<
 *     """
 *     dijkstra(csgraph, directed=True, indices=None, return_predecessors=False,
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_k__4 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":325
 * 
 * 
 * def dijkstra(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *              return_predecessors=False,
 *              unweighted=False, limit=np.inf):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_5dijkstra, NULL, __pyx_n_s_scipy_sparse_csgraph__shortest_p); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_dijkstra, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":606
 * 
 * 
 * def bellman_ford(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *                  return_predecessors=False,
 *                  unweighted=False):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_7bellman_ford, NULL, __pyx_n_s_scipy_sparse_csgraph__shortest_p); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bellman_ford, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":818
 * 
 * 
 * def johnson(csgraph, directed=True, indices=None,             # <<<<<<<<<<<<<<
 *             return_predecessors=False,
 *             unweighted=False):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6sparse_7csgraph_14_shortest_path_9johnson, NULL, __pyx_n_s_scipy_sparse_csgraph__shortest_p); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_johnson, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/sparse/csgraph/_shortest_path.pyx":1
 * """             # <<<<<<<<<<<<<<
 * Routines for performing shortest-path graph searches
 * 
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "../../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init scipy.sparse.csgraph._shortest_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init scipy.sparse.csgraph._shortest_path");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyList_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyTuple_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (PyErr_ExceptionMatches(PyExc_OverflowError))
                        PyErr_Clear();
                    else
                        return NULL;
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

static CYTHON_INLINE int __Pyx_IsLittleEndian(void) {
  unsigned int n = 1;
  return *(unsigned char*)(&n) != 0;
}
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t < '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static CYTHON_INLINE PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number;
    int ndim = ctx->head->field->type->ndim;
;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
      case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if (ctx->enc_type == *ts && got_Z == ctx->is_complex &&
            ctx->enc_packmode == ctx->new_packmode) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}
static CYTHON_INLINE void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static CYTHON_INLINE int __Pyx_GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  if (obj == Py_None || obj == NULL) {
    __Pyx_ZeroBuffer(buf);
    return 0;
  }
  buf->buf = NULL;
  if (__Pyx_GetBuffer(obj, buf, flags) == -1) goto fail;
  if (buf->ndim != nd) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if ((unsigned)buf->itemsize != dtype->size) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_ZeroBuffer(buf);
  return -1;
}
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (info->buf == NULL) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#endif
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

static double __Pyx__PyObject_AsDouble(PyObject* obj) {
    PyObject* float_value;
#if CYTHON_COMPILING_IN_PYPY
    float_value = PyNumber_Float(obj);  if (0) goto bad;
#else
    PyNumberMethods *nb = Py_TYPE(obj)->tp_as_number;
    if (likely(nb) && likely(nb->nb_float)) {
        float_value = nb->nb_float(obj);
        if (likely(float_value) && unlikely(!PyFloat_Check(float_value))) {
            PyErr_Format(PyExc_TypeError,
                "__float__ returned non-float (type %.200s)",
                Py_TYPE(float_value)->tp_name);
            Py_DECREF(float_value);
            goto bad;
        }
    } else if (PyUnicode_CheckExact(obj) || PyBytes_CheckExact(obj)) {
#if PY_MAJOR_VERSION >= 3
        float_value = PyFloat_FromString(obj);
#else
        float_value = PyFloat_FromString(obj, 0);
#endif
    } else {
        PyObject* args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        PyTuple_SET_ITEM(args, 0, obj);
        float_value = PyObject_Call((PyObject*)&PyFloat_Type, args, 0);
        PyTuple_SET_ITEM(args, 0, 0);
        Py_DECREF(args);
    }
#endif
    if (likely(float_value)) {
        double value = PyFloat_AS_DOUBLE(float_value);
        Py_DECREF(float_value);
        return value;
    }
bad:
    return (double)-1;
}

static void __Pyx_RaiseBufferIndexError(int axis) {
  PyErr_Format(PyExc_IndexError,
     "Out of bounds on buffer access (axis %d)", axis);
}

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_COMPILING_IN_CPYTHON
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CyFunctionObject, func.m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyMem_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
#if CYTHON_COMPILING_IN_PYPY
static PyObject * __Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    PyObject *self = f->m_self;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0 = PySequence_ITEM(arg, 0);
                if (unlikely(!arg0)) return NULL;
                result = (*meth)(self, arg0);
                Py_DECREF(arg0);
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
#else
static PyObject * __Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
	return PyCFunction_Call(func, arg, kw);
}
#endif
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_Call,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
#if !CYTHON_COMPILING_IN_PYPY
    __pyx_CyFunctionType_type.tp_call = PyCFunction_Call;
#endif
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (__pyx_CyFunctionType == NULL) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyMem_Malloc(size);
    if (!m->defaults)
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) return __pyx_pw_5numpy_7ndarray_1__getbuffer__(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) { __pyx_pw_5numpy_7ndarray_3__releasebuffer__(obj, view); return; }
    Py_DECREF(obj);
    view->obj = NULL;
}
#endif


              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int32(npy_int32 value) {
    const npy_int32 neg_one = (npy_int32) -1, const_zero = (npy_int32) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_int32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_int32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(npy_int32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(npy_int32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_int32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_int32),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        float denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(a, a);
                    case 3:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, a);
                    case 4:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_absf(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        double denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(a, a);
                    case 3:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, a);
                    case 4:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_abs(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) -1, const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
