/* Generated by Cython 0.23.4 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_4"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__scipy__special___ufuncs
#define __PYX_HAVE_API__scipy__special___ufuncs
#include "_complexstuff.h"
#include "numpy/npy_math.h"
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#include "sf_error.h"
#include "math.h"
#include "lapack_defs.h"
#include "specfun_wrappers.h"
#include "c_misc/misc.h"
#include "cephes.h"
#include "amos_wrappers.h"
#include "_ufuncs_defs.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "scipy/special/_ufuncs.pyx",
  "scipy/special/sph_harm.pxd",
  "scipy/special/_exprel.pxd",
  "scipy/special/_convex_analysis.pxd",
  "__init__.pxd",
  "scipy/special/_legacy.pxd",
  "scipy/special/_boxcox.pxd",
  "type.pxd",
};

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":725
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":726
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":727
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":728
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":732
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":733
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":734
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":735
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":739
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":740
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":749
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":750
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":751
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":753
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":754
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":755
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":757
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":758
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":760
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":761
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":762
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif


/*--- Type declarations ---*/

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":764
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":765
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":766
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":768
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "scipy/special/_ufuncs.pyx":1222
 * 
 * from _ellip_harm cimport ellip_harmonic as _func_ellip_harmonic
 * ctypedef double _proto_ellip_harmonic_t(double, double, int, int, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_ellip_harmonic_t *_proto_ellip_harmonic_t_var = &_func_ellip_harmonic
 * from _legacy cimport ellip_harmonic_unsafe as _func_ellip_harmonic_unsafe
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_ellip_harmonic_t(double, double, int, int, double, double, double);

/* "scipy/special/_ufuncs.pyx":1225
 * cdef _proto_ellip_harmonic_t *_proto_ellip_harmonic_t_var = &_func_ellip_harmonic
 * from _legacy cimport ellip_harmonic_unsafe as _func_ellip_harmonic_unsafe
 * ctypedef double _proto_ellip_harmonic_unsafe_t(double, double, double, double, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_ellip_harmonic_unsafe_t *_proto_ellip_harmonic_unsafe_t_var = &_func_ellip_harmonic_unsafe
 * from lambertw cimport lambertw_scalar as _func_lambertw_scalar
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_ellip_harmonic_unsafe_t(double, double, double, double, double, double, double);

/* "scipy/special/_ufuncs.pyx":1228
 * cdef _proto_ellip_harmonic_unsafe_t *_proto_ellip_harmonic_unsafe_t_var = &_func_ellip_harmonic_unsafe
 * from lambertw cimport lambertw_scalar as _func_lambertw_scalar
 * ctypedef double complex _proto_lambertw_scalar_t(double complex, long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_lambertw_scalar_t *_proto_lambertw_scalar_t_var = &_func_lambertw_scalar
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_lambertw_scalar_t(__pyx_t_double_complex, long, double);

/* "scipy/special/_ufuncs.pyx":1247
 *     cdef double _func_bdtr "bdtr"(int, int, double) nogil
 * from _legacy cimport bdtr_unsafe as _func_bdtr_unsafe
 * ctypedef double _proto_bdtr_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_bdtr_unsafe_t *_proto_bdtr_unsafe_t_var = &_func_bdtr_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_bdtr_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1252
 *     cdef double _func_bdtrc "bdtrc"(int, int, double) nogil
 * from _legacy cimport bdtrc_unsafe as _func_bdtrc_unsafe
 * ctypedef double _proto_bdtrc_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_bdtrc_unsafe_t *_proto_bdtrc_unsafe_t_var = &_func_bdtrc_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_bdtrc_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1257
 *     cdef double _func_bdtri "bdtri"(int, int, double) nogil
 * from _legacy cimport bdtri_unsafe as _func_bdtri_unsafe
 * ctypedef double _proto_bdtri_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_bdtri_unsafe_t *_proto_bdtri_unsafe_t_var = &_func_bdtri_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_bdtri_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1282
 *     cdef double _func_lbeta "lbeta"(double, double) nogil
 * from orthogonal_eval cimport binom as _func_binom
 * ctypedef double _proto_binom_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_binom_t *_proto_binom_t_var = &_func_binom
 * from _boxcox cimport boxcox as _func_boxcox
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_binom_t(double, double);

/* "scipy/special/_ufuncs.pyx":1285
 * cdef _proto_binom_t *_proto_binom_t_var = &_func_binom
 * from _boxcox cimport boxcox as _func_boxcox
 * ctypedef double _proto_boxcox_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_boxcox_t *_proto_boxcox_t_var = &_func_boxcox
 * from _boxcox cimport boxcox1p as _func_boxcox1p
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_boxcox_t(double, double);

/* "scipy/special/_ufuncs.pyx":1288
 * cdef _proto_boxcox_t *_proto_boxcox_t_var = &_func_boxcox
 * from _boxcox cimport boxcox1p as _func_boxcox1p
 * ctypedef double _proto_boxcox1p_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_boxcox1p_t *_proto_boxcox1p_t_var = &_func_boxcox1p
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_boxcox1p_t(double, double);

/* "scipy/special/_ufuncs.pyx":1333
 *     cdef double _func_ellpk "ellpk"(double) nogil
 * from _convex_analysis cimport entr as _func_entr
 * ctypedef double _proto_entr_t(double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_entr_t *_proto_entr_t_var = &_func_entr
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_entr_t(double);

/* "scipy/special/_ufuncs.pyx":1340
 *     cdef double _func_erfc "erfc"(double) nogil
 * from orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double _proto_eval_chebyc_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyc_double__t *_proto_eval_chebyc_double__t_var = &_func_eval_chebyc[double]
 * from orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1343
 * cdef _proto_eval_chebyc_double__t *_proto_eval_chebyc_double__t_var = &_func_eval_chebyc[double]
 * from orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double complex _proto_eval_chebyc_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyc_double_complex__t *_proto_eval_chebyc_double_complex__t_var = &_func_eval_chebyc[double_complex]
 * from orthogonal_eval cimport eval_chebyc_l as _func_eval_chebyc_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1346
 * cdef _proto_eval_chebyc_double_complex__t *_proto_eval_chebyc_double_complex__t_var = &_func_eval_chebyc[double_complex]
 * from orthogonal_eval cimport eval_chebyc_l as _func_eval_chebyc_l
 * ctypedef double _proto_eval_chebyc_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyc_l_t *_proto_eval_chebyc_l_t_var = &_func_eval_chebyc_l
 * from orthogonal_eval cimport eval_chebys as _func_eval_chebys
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1349
 * cdef _proto_eval_chebyc_l_t *_proto_eval_chebyc_l_t_var = &_func_eval_chebyc_l
 * from orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double _proto_eval_chebys_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebys_double__t *_proto_eval_chebys_double__t_var = &_func_eval_chebys[double]
 * from orthogonal_eval cimport eval_chebys as _func_eval_chebys
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1352
 * cdef _proto_eval_chebys_double__t *_proto_eval_chebys_double__t_var = &_func_eval_chebys[double]
 * from orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double complex _proto_eval_chebys_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebys_double_complex__t *_proto_eval_chebys_double_complex__t_var = &_func_eval_chebys[double_complex]
 * from orthogonal_eval cimport eval_chebys_l as _func_eval_chebys_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1355
 * cdef _proto_eval_chebys_double_complex__t *_proto_eval_chebys_double_complex__t_var = &_func_eval_chebys[double_complex]
 * from orthogonal_eval cimport eval_chebys_l as _func_eval_chebys_l
 * ctypedef double _proto_eval_chebys_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebys_l_t *_proto_eval_chebys_l_t_var = &_func_eval_chebys_l
 * from orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1358
 * cdef _proto_eval_chebys_l_t *_proto_eval_chebys_l_t_var = &_func_eval_chebys_l
 * from orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double _proto_eval_chebyt_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyt_double__t *_proto_eval_chebyt_double__t_var = &_func_eval_chebyt[double]
 * from orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1361
 * cdef _proto_eval_chebyt_double__t *_proto_eval_chebyt_double__t_var = &_func_eval_chebyt[double]
 * from orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double complex _proto_eval_chebyt_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyt_double_complex__t *_proto_eval_chebyt_double_complex__t_var = &_func_eval_chebyt[double_complex]
 * from orthogonal_eval cimport eval_chebyt_l as _func_eval_chebyt_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1364
 * cdef _proto_eval_chebyt_double_complex__t *_proto_eval_chebyt_double_complex__t_var = &_func_eval_chebyt[double_complex]
 * from orthogonal_eval cimport eval_chebyt_l as _func_eval_chebyt_l
 * ctypedef double _proto_eval_chebyt_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyt_l_t *_proto_eval_chebyt_l_t_var = &_func_eval_chebyt_l
 * from orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1367
 * cdef _proto_eval_chebyt_l_t *_proto_eval_chebyt_l_t_var = &_func_eval_chebyt_l
 * from orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double _proto_eval_chebyu_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyu_double__t *_proto_eval_chebyu_double__t_var = &_func_eval_chebyu[double]
 * from orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1370
 * cdef _proto_eval_chebyu_double__t *_proto_eval_chebyu_double__t_var = &_func_eval_chebyu[double]
 * from orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double complex _proto_eval_chebyu_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyu_double_complex__t *_proto_eval_chebyu_double_complex__t_var = &_func_eval_chebyu[double_complex]
 * from orthogonal_eval cimport eval_chebyu_l as _func_eval_chebyu_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1373
 * cdef _proto_eval_chebyu_double_complex__t *_proto_eval_chebyu_double_complex__t_var = &_func_eval_chebyu[double_complex]
 * from orthogonal_eval cimport eval_chebyu_l as _func_eval_chebyu_l
 * ctypedef double _proto_eval_chebyu_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyu_l_t *_proto_eval_chebyu_l_t_var = &_func_eval_chebyu_l
 * from orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1376
 * cdef _proto_eval_chebyu_l_t *_proto_eval_chebyu_l_t_var = &_func_eval_chebyu_l
 * from orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double _proto_eval_gegenbauer_double__t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_gegenbauer_double__t *_proto_eval_gegenbauer_double__t_var = &_func_eval_gegenbauer[double]
 * from orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double__t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1379
 * cdef _proto_eval_gegenbauer_double__t *_proto_eval_gegenbauer_double__t_var = &_func_eval_gegenbauer[double]
 * from orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double complex _proto_eval_gegenbauer_double_complex__t(double, double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_gegenbauer_double_complex__t *_proto_eval_gegenbauer_double_complex__t_var = &_func_eval_gegenbauer[double_complex]
 * from orthogonal_eval cimport eval_gegenbauer_l as _func_eval_gegenbauer_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double_complex__t(double, double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1382
 * cdef _proto_eval_gegenbauer_double_complex__t *_proto_eval_gegenbauer_double_complex__t_var = &_func_eval_gegenbauer[double_complex]
 * from orthogonal_eval cimport eval_gegenbauer_l as _func_eval_gegenbauer_l
 * ctypedef double _proto_eval_gegenbauer_l_t(long, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_gegenbauer_l_t *_proto_eval_gegenbauer_l_t_var = &_func_eval_gegenbauer_l
 * from orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_l_t(long, double, double);

/* "scipy/special/_ufuncs.pyx":1385
 * cdef _proto_eval_gegenbauer_l_t *_proto_eval_gegenbauer_l_t_var = &_func_eval_gegenbauer_l
 * from orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double _proto_eval_genlaguerre_double__t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_genlaguerre_double__t *_proto_eval_genlaguerre_double__t_var = &_func_eval_genlaguerre[double]
 * from orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double__t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1388
 * cdef _proto_eval_genlaguerre_double__t *_proto_eval_genlaguerre_double__t_var = &_func_eval_genlaguerre[double]
 * from orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double complex _proto_eval_genlaguerre_double_complex__t(double, double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_genlaguerre_double_complex__t *_proto_eval_genlaguerre_double_complex__t_var = &_func_eval_genlaguerre[double_complex]
 * from orthogonal_eval cimport eval_genlaguerre_l as _func_eval_genlaguerre_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double_complex__t(double, double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1391
 * cdef _proto_eval_genlaguerre_double_complex__t *_proto_eval_genlaguerre_double_complex__t_var = &_func_eval_genlaguerre[double_complex]
 * from orthogonal_eval cimport eval_genlaguerre_l as _func_eval_genlaguerre_l
 * ctypedef double _proto_eval_genlaguerre_l_t(long, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_genlaguerre_l_t *_proto_eval_genlaguerre_l_t_var = &_func_eval_genlaguerre_l
 * from orthogonal_eval cimport eval_hermite as _func_eval_hermite
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_l_t(long, double, double);

/* "scipy/special/_ufuncs.pyx":1394
 * cdef _proto_eval_genlaguerre_l_t *_proto_eval_genlaguerre_l_t_var = &_func_eval_genlaguerre_l
 * from orthogonal_eval cimport eval_hermite as _func_eval_hermite
 * ctypedef double _proto_eval_hermite_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_hermite_t *_proto_eval_hermite_t_var = &_func_eval_hermite
 * from orthogonal_eval cimport eval_hermitenorm as _func_eval_hermitenorm
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_hermite_t(long, double);

/* "scipy/special/_ufuncs.pyx":1397
 * cdef _proto_eval_hermite_t *_proto_eval_hermite_t_var = &_func_eval_hermite
 * from orthogonal_eval cimport eval_hermitenorm as _func_eval_hermitenorm
 * ctypedef double _proto_eval_hermitenorm_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_hermitenorm_t *_proto_eval_hermitenorm_t_var = &_func_eval_hermitenorm
 * from orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_hermitenorm_t(long, double);

/* "scipy/special/_ufuncs.pyx":1400
 * cdef _proto_eval_hermitenorm_t *_proto_eval_hermitenorm_t_var = &_func_eval_hermitenorm
 * from orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double _proto_eval_jacobi_double__t(double, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_jacobi_double__t *_proto_eval_jacobi_double__t_var = &_func_eval_jacobi[double]
 * from orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_double__t(double, double, double, double);

/* "scipy/special/_ufuncs.pyx":1403
 * cdef _proto_eval_jacobi_double__t *_proto_eval_jacobi_double__t_var = &_func_eval_jacobi[double]
 * from orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double complex _proto_eval_jacobi_double_complex__t(double, double, double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_jacobi_double_complex__t *_proto_eval_jacobi_double_complex__t_var = &_func_eval_jacobi[double_complex]
 * from orthogonal_eval cimport eval_jacobi_l as _func_eval_jacobi_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_double_complex__t(double, double, double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1406
 * cdef _proto_eval_jacobi_double_complex__t *_proto_eval_jacobi_double_complex__t_var = &_func_eval_jacobi[double_complex]
 * from orthogonal_eval cimport eval_jacobi_l as _func_eval_jacobi_l
 * ctypedef double _proto_eval_jacobi_l_t(long, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_jacobi_l_t *_proto_eval_jacobi_l_t_var = &_func_eval_jacobi_l
 * from orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_l_t(long, double, double, double);

/* "scipy/special/_ufuncs.pyx":1409
 * cdef _proto_eval_jacobi_l_t *_proto_eval_jacobi_l_t_var = &_func_eval_jacobi_l
 * from orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double _proto_eval_laguerre_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_laguerre_double__t *_proto_eval_laguerre_double__t_var = &_func_eval_laguerre[double]
 * from orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1412
 * cdef _proto_eval_laguerre_double__t *_proto_eval_laguerre_double__t_var = &_func_eval_laguerre[double]
 * from orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double complex _proto_eval_laguerre_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_laguerre_double_complex__t *_proto_eval_laguerre_double_complex__t_var = &_func_eval_laguerre[double_complex]
 * from orthogonal_eval cimport eval_laguerre_l as _func_eval_laguerre_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1415
 * cdef _proto_eval_laguerre_double_complex__t *_proto_eval_laguerre_double_complex__t_var = &_func_eval_laguerre[double_complex]
 * from orthogonal_eval cimport eval_laguerre_l as _func_eval_laguerre_l
 * ctypedef double _proto_eval_laguerre_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_laguerre_l_t *_proto_eval_laguerre_l_t_var = &_func_eval_laguerre_l
 * from orthogonal_eval cimport eval_legendre as _func_eval_legendre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1418
 * cdef _proto_eval_laguerre_l_t *_proto_eval_laguerre_l_t_var = &_func_eval_laguerre_l
 * from orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double _proto_eval_legendre_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_legendre_double__t *_proto_eval_legendre_double__t_var = &_func_eval_legendre[double]
 * from orthogonal_eval cimport eval_legendre as _func_eval_legendre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1421
 * cdef _proto_eval_legendre_double__t *_proto_eval_legendre_double__t_var = &_func_eval_legendre[double]
 * from orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double complex _proto_eval_legendre_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_legendre_double_complex__t *_proto_eval_legendre_double_complex__t_var = &_func_eval_legendre[double_complex]
 * from orthogonal_eval cimport eval_legendre_l as _func_eval_legendre_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1424
 * cdef _proto_eval_legendre_double_complex__t *_proto_eval_legendre_double_complex__t_var = &_func_eval_legendre[double_complex]
 * from orthogonal_eval cimport eval_legendre_l as _func_eval_legendre_l
 * ctypedef double _proto_eval_legendre_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_legendre_l_t *_proto_eval_legendre_l_t_var = &_func_eval_legendre_l
 * from orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1427
 * cdef _proto_eval_legendre_l_t *_proto_eval_legendre_l_t_var = &_func_eval_legendre_l
 * from orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double _proto_eval_sh_chebyt_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyt_double__t *_proto_eval_sh_chebyt_double__t_var = &_func_eval_sh_chebyt[double]
 * from orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1430
 * cdef _proto_eval_sh_chebyt_double__t *_proto_eval_sh_chebyt_double__t_var = &_func_eval_sh_chebyt[double]
 * from orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double complex _proto_eval_sh_chebyt_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyt_double_complex__t *_proto_eval_sh_chebyt_double_complex__t_var = &_func_eval_sh_chebyt[double_complex]
 * from orthogonal_eval cimport eval_sh_chebyt_l as _func_eval_sh_chebyt_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1433
 * cdef _proto_eval_sh_chebyt_double_complex__t *_proto_eval_sh_chebyt_double_complex__t_var = &_func_eval_sh_chebyt[double_complex]
 * from orthogonal_eval cimport eval_sh_chebyt_l as _func_eval_sh_chebyt_l
 * ctypedef double _proto_eval_sh_chebyt_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyt_l_t *_proto_eval_sh_chebyt_l_t_var = &_func_eval_sh_chebyt_l
 * from orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1436
 * cdef _proto_eval_sh_chebyt_l_t *_proto_eval_sh_chebyt_l_t_var = &_func_eval_sh_chebyt_l
 * from orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double _proto_eval_sh_chebyu_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyu_double__t *_proto_eval_sh_chebyu_double__t_var = &_func_eval_sh_chebyu[double]
 * from orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1439
 * cdef _proto_eval_sh_chebyu_double__t *_proto_eval_sh_chebyu_double__t_var = &_func_eval_sh_chebyu[double]
 * from orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double complex _proto_eval_sh_chebyu_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyu_double_complex__t *_proto_eval_sh_chebyu_double_complex__t_var = &_func_eval_sh_chebyu[double_complex]
 * from orthogonal_eval cimport eval_sh_chebyu_l as _func_eval_sh_chebyu_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1442
 * cdef _proto_eval_sh_chebyu_double_complex__t *_proto_eval_sh_chebyu_double_complex__t_var = &_func_eval_sh_chebyu[double_complex]
 * from orthogonal_eval cimport eval_sh_chebyu_l as _func_eval_sh_chebyu_l
 * ctypedef double _proto_eval_sh_chebyu_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyu_l_t *_proto_eval_sh_chebyu_l_t_var = &_func_eval_sh_chebyu_l
 * from orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1445
 * cdef _proto_eval_sh_chebyu_l_t *_proto_eval_sh_chebyu_l_t_var = &_func_eval_sh_chebyu_l
 * from orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double _proto_eval_sh_jacobi_double__t(double, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_jacobi_double__t *_proto_eval_sh_jacobi_double__t_var = &_func_eval_sh_jacobi[double]
 * from orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double__t(double, double, double, double);

/* "scipy/special/_ufuncs.pyx":1448
 * cdef _proto_eval_sh_jacobi_double__t *_proto_eval_sh_jacobi_double__t_var = &_func_eval_sh_jacobi[double]
 * from orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double complex _proto_eval_sh_jacobi_double_complex__t(double, double, double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_jacobi_double_complex__t *_proto_eval_sh_jacobi_double_complex__t_var = &_func_eval_sh_jacobi[double_complex]
 * from orthogonal_eval cimport eval_sh_jacobi_l as _func_eval_sh_jacobi_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double_complex__t(double, double, double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1451
 * cdef _proto_eval_sh_jacobi_double_complex__t *_proto_eval_sh_jacobi_double_complex__t_var = &_func_eval_sh_jacobi[double_complex]
 * from orthogonal_eval cimport eval_sh_jacobi_l as _func_eval_sh_jacobi_l
 * ctypedef double _proto_eval_sh_jacobi_l_t(long, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_jacobi_l_t *_proto_eval_sh_jacobi_l_t_var = &_func_eval_sh_jacobi_l
 * from orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_l_t(long, double, double, double);

/* "scipy/special/_ufuncs.pyx":1454
 * cdef _proto_eval_sh_jacobi_l_t *_proto_eval_sh_jacobi_l_t_var = &_func_eval_sh_jacobi_l
 * from orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double _proto_eval_sh_legendre_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_legendre_double__t *_proto_eval_sh_legendre_double__t_var = &_func_eval_sh_legendre[double]
 * from orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1457
 * cdef _proto_eval_sh_legendre_double__t *_proto_eval_sh_legendre_double__t_var = &_func_eval_sh_legendre[double]
 * from orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double complex _proto_eval_sh_legendre_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_legendre_double_complex__t *_proto_eval_sh_legendre_double_complex__t_var = &_func_eval_sh_legendre[double_complex]
 * from orthogonal_eval cimport eval_sh_legendre_l as _func_eval_sh_legendre_l
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1460
 * cdef _proto_eval_sh_legendre_double_complex__t *_proto_eval_sh_legendre_double_complex__t_var = &_func_eval_sh_legendre[double_complex]
 * from orthogonal_eval cimport eval_sh_legendre_l as _func_eval_sh_legendre_l
 * ctypedef double _proto_eval_sh_legendre_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_legendre_l_t *_proto_eval_sh_legendre_l_t_var = &_func_eval_sh_legendre_l
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1485
 *     cdef double _func_expn "expn"(int, double) nogil
 * from _legacy cimport expn_unsafe as _func_expn_unsafe
 * ctypedef double _proto_expn_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_expn_unsafe_t *_proto_expn_unsafe_t_var = &_func_expn_unsafe
 * from _exprel cimport exprel as _func_exprel
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_expn_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1488
 * cdef _proto_expn_unsafe_t *_proto_expn_unsafe_t_var = &_func_expn_unsafe
 * from _exprel cimport exprel as _func_exprel
 * ctypedef double _proto_exprel_t(double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_exprel_t *_proto_exprel_t_var = &_func_exprel
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_exprel_t(double);

/* "scipy/special/_ufuncs.pyx":1539
 *     cdef double complex _func_cbesh_wrap2_e "cbesh_wrap2_e"(double, double complex) nogil
 * from _convex_analysis cimport huber as _func_huber
 * ctypedef double _proto_huber_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_huber_t *_proto_huber_t_var = &_func_huber
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_huber_t(double, double);

/* "scipy/special/_ufuncs.pyx":1550
 *     cdef double _func_hyp2f0 "hyp2f0"(double, double, double, int, double *) nogil
 * from _legacy cimport hyp2f0_unsafe as _func_hyp2f0_unsafe
 * ctypedef double _proto_hyp2f0_unsafe_t(double, double, double, double, double *) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_hyp2f0_unsafe_t *_proto_hyp2f0_unsafe_t_var = &_func_hyp2f0_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_hyp2f0_unsafe_t(double, double, double, double, double *);

/* "scipy/special/_ufuncs.pyx":1569
 *     cdef double _func_i1e "i1e"(double) nogil
 * from _boxcox cimport inv_boxcox as _func_inv_boxcox
 * ctypedef double _proto_inv_boxcox_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_inv_boxcox_t *_proto_inv_boxcox_t_var = &_func_inv_boxcox
 * from _boxcox cimport inv_boxcox1p as _func_inv_boxcox1p
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_inv_boxcox_t(double, double);

/* "scipy/special/_ufuncs.pyx":1572
 * cdef _proto_inv_boxcox_t *_proto_inv_boxcox_t_var = &_func_inv_boxcox
 * from _boxcox cimport inv_boxcox1p as _func_inv_boxcox1p
 * ctypedef double _proto_inv_boxcox1p_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_inv_boxcox1p_t *_proto_inv_boxcox1p_t_var = &_func_inv_boxcox1p
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_inv_boxcox1p_t(double, double);

/* "scipy/special/_ufuncs.pyx":1629
 *     cdef double _func_kerp_wrap "kerp_wrap"(double) nogil
 * from _convex_analysis cimport kl_div as _func_kl_div
 * ctypedef double _proto_kl_div_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_kl_div_t *_proto_kl_div_t_var = &_func_kl_div
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_kl_div_t(double, double);

/* "scipy/special/_ufuncs.pyx":1634
 *     cdef double _func_cbesk_wrap_real_int "cbesk_wrap_real_int"(int, double) nogil
 * from _legacy cimport kn_unsafe as _func_kn_unsafe
 * ctypedef double _proto_kn_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_kn_unsafe_t *_proto_kn_unsafe_t_var = &_func_kn_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_kn_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1685
 *     cdef double _func_nbdtr "nbdtr"(int, int, double) nogil
 * from _legacy cimport nbdtr_unsafe as _func_nbdtr_unsafe
 * ctypedef double _proto_nbdtr_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_nbdtr_unsafe_t *_proto_nbdtr_unsafe_t_var = &_func_nbdtr_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtr_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1690
 *     cdef double _func_nbdtrc "nbdtrc"(int, int, double) nogil
 * from _legacy cimport nbdtrc_unsafe as _func_nbdtrc_unsafe
 * ctypedef double _proto_nbdtrc_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_nbdtrc_unsafe_t *_proto_nbdtrc_unsafe_t_var = &_func_nbdtrc_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtrc_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1695
 *     cdef double _func_nbdtri "nbdtri"(int, int, double) nogil
 * from _legacy cimport nbdtri_unsafe as _func_nbdtri_unsafe
 * ctypedef double _proto_nbdtri_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_nbdtri_unsafe_t *_proto_nbdtri_unsafe_t_var = &_func_nbdtri_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtri_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1750
 *     cdef double _func_pdtr "pdtr"(int, double) nogil
 * from _legacy cimport pdtr_unsafe as _func_pdtr_unsafe
 * ctypedef double _proto_pdtr_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_pdtr_unsafe_t *_proto_pdtr_unsafe_t_var = &_func_pdtr_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_pdtr_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1755
 *     cdef double _func_pdtrc "pdtrc"(int, double) nogil
 * from _legacy cimport pdtrc_unsafe as _func_pdtrc_unsafe
 * ctypedef double _proto_pdtrc_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_pdtrc_unsafe_t *_proto_pdtrc_unsafe_t_var = &_func_pdtrc_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_pdtrc_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1760
 *     cdef double _func_pdtri "pdtri"(int, double) nogil
 * from _legacy cimport pdtri_unsafe as _func_pdtri_unsafe
 * ctypedef double _proto_pdtri_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_pdtri_unsafe_t *_proto_pdtri_unsafe_t_var = &_func_pdtri_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_pdtri_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1781
 *     cdef int _func_prolate_radial2_wrap "prolate_radial2_wrap"(double, double, double, double, double, double *, double *) nogil
 * from _convex_analysis cimport pseudo_huber as _func_pseudo_huber
 * ctypedef double _proto_pseudo_huber_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_pseudo_huber_t *_proto_pseudo_huber_t_var = &_func_pseudo_huber
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_pseudo_huber_t(double, double);

/* "scipy/special/_ufuncs.pyx":1790
 *     cdef double _func_radian "radian"(double, double, double) nogil
 * from _convex_analysis cimport rel_entr as _func_rel_entr
 * ctypedef double _proto_rel_entr_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_rel_entr_t *_proto_rel_entr_t_var = &_func_rel_entr
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_rel_entr_t(double, double);

/* "scipy/special/_ufuncs.pyx":1807
 *     cdef double _func_smirnov "smirnov"(int, double) nogil
 * from _legacy cimport smirnov_unsafe as _func_smirnov_unsafe
 * ctypedef double _proto_smirnov_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_smirnov_unsafe_t *_proto_smirnov_unsafe_t_var = &_func_smirnov_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_smirnov_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1812
 *     cdef double _func_smirnovi "smirnovi"(int, double) nogil
 * from _legacy cimport smirnovi_unsafe as _func_smirnovi_unsafe
 * ctypedef double _proto_smirnovi_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_smirnovi_unsafe_t *_proto_smirnovi_unsafe_t_var = &_func_smirnovi_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovi_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1817
 *     cdef double _func_spence "spence"(double) nogil
 * from sph_harm cimport sph_harmonic as _func_sph_harmonic
 * ctypedef double complex _proto_sph_harmonic_t(int, int, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_sph_harmonic_t *_proto_sph_harmonic_t_var = &_func_sph_harmonic
 * from _legacy cimport sph_harmonic_unsafe as _func_sph_harmonic_unsafe
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_sph_harmonic_t(int, int, double, double);

/* "scipy/special/_ufuncs.pyx":1820
 * cdef _proto_sph_harmonic_t *_proto_sph_harmonic_t_var = &_func_sph_harmonic
 * from _legacy cimport sph_harmonic_unsafe as _func_sph_harmonic_unsafe
 * ctypedef double complex _proto_sph_harmonic_unsafe_t(double, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_sph_harmonic_unsafe_t *_proto_sph_harmonic_unsafe_t_var = &_func_sph_harmonic_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_sph_harmonic_unsafe_t(double, double, double, double);

/* "scipy/special/_ufuncs.pyx":1835
 *     cdef double _func_tukeylambdacdf "tukeylambdacdf"(double, double) nogil
 * from _xlogy cimport xlog1py as _func_xlog1py
 * ctypedef double _proto_xlog1py_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_xlog1py_t *_proto_xlog1py_t_var = &_func_xlog1py
 * from _xlogy cimport xlogy as _func_xlogy
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_xlog1py_t(double, double);

/* "scipy/special/_ufuncs.pyx":1838
 * cdef _proto_xlog1py_t *_proto_xlog1py_t_var = &_func_xlog1py
 * from _xlogy cimport xlogy as _func_xlogy
 * ctypedef double _proto_xlogy_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_xlogy_double__t *_proto_xlogy_double__t_var = &_func_xlogy[double]
 * from _xlogy cimport xlogy as _func_xlogy
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_xlogy_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1841
 * cdef _proto_xlogy_double__t *_proto_xlogy_double__t_var = &_func_xlogy[double]
 * from _xlogy cimport xlogy as _func_xlogy
 * ctypedef double complex _proto_xlogy_double_complex__t(double complex, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_xlogy_double_complex__t *_proto_xlogy_double_complex__t_var = &_func_xlogy[double_complex]
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_xlogy_double_complex__t(__pyx_t_double_complex, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1850
 *     cdef double _func_yn "yn"(int, double) nogil
 * from _legacy cimport yn_unsafe as _func_yn_unsafe
 * ctypedef double _proto_yn_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_yn_unsafe_t *_proto_yn_unsafe_t_var = &_func_yn_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_yn_unsafe_t(double, double);

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

static CYTHON_INLINE long __Pyx_div_long(long, long);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if (defined(_WIN32) || defined(__clang__)) && defined(__cplusplus) && CYTHON_CCOMPLEX
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq(a, b)   ((a)==(b))
    #define __Pyx_c_sum(a, b)  ((a)+(b))
    #define __Pyx_c_diff(a, b) ((a)-(b))
    #define __Pyx_c_prod(a, b) ((a)*(b))
    #define __Pyx_c_quot(a, b) ((a)/(b))
    #define __Pyx_c_neg(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero(z) ((z)==(double)0)
    #define __Pyx_c_conj(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (::std::abs(z))
        #define __Pyx_c_pow(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero(z) ((z)==0)
    #define __Pyx_c_conj(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (cabs(z))
        #define __Pyx_c_pow(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *);

static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eqf(a, b)   ((a)==(b))
    #define __Pyx_c_sumf(a, b)  ((a)+(b))
    #define __Pyx_c_difff(a, b) ((a)-(b))
    #define __Pyx_c_prodf(a, b) ((a)*(b))
    #define __Pyx_c_quotf(a, b) ((a)/(b))
    #define __Pyx_c_negf(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zerof(z) ((z)==(float)0)
    #define __Pyx_c_conjf(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_absf(z)     (::std::abs(z))
        #define __Pyx_c_powf(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zerof(z) ((z)==0)
    #define __Pyx_c_conjf(z)    (conjf(z))
    #if 1
        #define __Pyx_c_absf(z)     (cabsf(z))
        #define __Pyx_c_powf(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

static CYTHON_INLINE long __Pyx_pow_long(long, long);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/

/* Module declarations from 'libc.math' */

/* Module declarations from 'libc' */

/* Module declarations from 'scipy.special.sf_error' */

/* Module declarations from 'scipy.special._ufuncs_cxx' */
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w)
static int (*__pyx_f_5scipy_7special_11_ufuncs_cxx__set_errprint)(int); /*proto*/

/* Module declarations from 'scipy.special' */

/* Module declarations from 'scipy' */

/* Module declarations from 'cython' */

/* Module declarations from 'numpy.math' */

/* Module declarations from 'scipy.special._ellip_harm' */
static CYTHON_INLINE double *__pyx_f_5scipy_7special_11_ellip_harm_lame_coefficients(double, double, int, int, void **, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_11_ellip_harm_ellip_harm_eval(double, double, int, int, double, double *, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic(double, double, int, int, double, double, double); /*proto*/

/* Module declarations from 'scipy.special._complexstuff' */
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisnan(double); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisfinite(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zlog(double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special.sph_harm' */
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8sph_harm_sph_harmonic(int, int, double, double); /*proto*/

/* Module declarations from 'scipy.special._legacy' */
static CYTHON_INLINE void __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(char *, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe(double, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe(double, double, double, double, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtr_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtri_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_expn_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe(double, double, double, double, double *); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtr_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtri_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_kn_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_yn_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnov_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe(double, double); /*proto*/

/* Module declarations from 'scipy.special.lambertw' */
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_scalar(__pyx_t_double_complex, long, double); /*proto*/

/* Module declarations from 'scipy.special.orthogonal_eval' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_binom(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l(long, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi_l(long, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer_l(long, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l(long, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_hermite(long, double); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(double, double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(double, double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(double, double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(double, double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi(double, double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi(double, double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre(double, __pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._boxcox' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_boxcox(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_boxcox1p(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_inv_boxcox(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p(double, double); /*proto*/

/* Module declarations from 'scipy.special._convex_analysis' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_entr(double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_kl_div(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_rel_entr(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_huber(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber(double, double); /*proto*/

/* Module declarations from 'scipy.special._exprel' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_exprel_exprel(double); /*proto*/

/* Module declarations from 'scipy.special._xlogy' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_6_xlogy_xlog1py(double, double); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy(__pyx_t_double_complex, __pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._ufuncs' */
static __pyx_t_5scipy_7special_7_ufuncs__proto_ellip_harmonic_t *__pyx_v_5scipy_7special_7_ufuncs__proto_ellip_harmonic_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_ellip_harmonic_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_ellip_harmonic_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_lambertw_scalar_t *__pyx_v_5scipy_7special_7_ufuncs__proto_lambertw_scalar_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_bdtr_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_bdtr_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_bdtrc_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_bdtrc_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_bdtri_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_bdtri_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_binom_t *__pyx_v_5scipy_7special_7_ufuncs__proto_binom_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_boxcox_t *__pyx_v_5scipy_7special_7_ufuncs__proto_boxcox_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_boxcox1p_t *__pyx_v_5scipy_7special_7_ufuncs__proto_boxcox1p_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_entr_t *__pyx_v_5scipy_7special_7_ufuncs__proto_entr_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_hermite_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_hermite_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_hermitenorm_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_hermitenorm_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_expn_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_expn_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_exprel_t *__pyx_v_5scipy_7special_7_ufuncs__proto_exprel_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_huber_t *__pyx_v_5scipy_7special_7_ufuncs__proto_huber_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_hyp2f0_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_hyp2f0_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_inv_boxcox_t *__pyx_v_5scipy_7special_7_ufuncs__proto_inv_boxcox_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_inv_boxcox1p_t *__pyx_v_5scipy_7special_7_ufuncs__proto_inv_boxcox1p_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_kl_div_t *__pyx_v_5scipy_7special_7_ufuncs__proto_kl_div_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_kn_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_kn_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtr_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_nbdtr_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtrc_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_nbdtrc_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtri_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_nbdtri_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_pdtr_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_pdtr_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_pdtrc_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_pdtrc_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_pdtri_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_pdtri_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_pseudo_huber_t *__pyx_v_5scipy_7special_7_ufuncs__proto_pseudo_huber_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_rel_entr_t *__pyx_v_5scipy_7special_7_ufuncs__proto_rel_entr_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_smirnov_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_smirnov_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovi_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_smirnovi_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_sph_harmonic_t *__pyx_v_5scipy_7special_7_ufuncs__proto_sph_harmonic_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_sph_harmonic_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_sph_harmonic_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_xlog1py_t *__pyx_v_5scipy_7special_7_ufuncs__proto_xlog1py_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_xlogy_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_xlogy_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_xlogy_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_xlogy_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_yn_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_yn_unsafe_t_var;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[24];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[5];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[5];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[5];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types[3];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types[3];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[25];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[25];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[16];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[18];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_doc;
__PYX_EXTERN_C DL_EXPORT(int) wrap_PyUFunc_getfperr(void); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_f_f__As_f_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_iidd__As_lldd_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_F_FF(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_f_FFFF(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_F_FFFF(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddi_d_As_dddl_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ldd__As_ldd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_ff_ffff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_d_DDDD(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_ffff_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_Dld__As_Dld_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_DD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_FF_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_d_DD(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_dddd_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_ddddddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_lddd__As_lddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_fffffff_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_D_DD(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddiiddd__As_ddllddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_dd_dddd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_f_FF(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_D_DDDD(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_g_g__As_g_g(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
#define __Pyx_MODULE_NAME "scipy.special._ufuncs"
int __pyx_module_is_main_scipy__special___ufuncs = 0;

/* Implementation of 'scipy.special._ufuncs' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_RuntimeWarning;
static char __pyx_k_B[] = "B";
static char __pyx_k_H[] = "H";
static char __pyx_k_I[] = "I";
static char __pyx_k_L[] = "L";
static char __pyx_k_O[] = "O";
static char __pyx_k_Q[] = "Q";
static char __pyx_k_V[] = "V";
static char __pyx_k_b[] = "b";
static char __pyx_k_d[] = "d";
static char __pyx_k_f[] = "f";
static char __pyx_k_g[] = "g";
static char __pyx_k_h[] = "h";
static char __pyx_k_i[] = "i";
static char __pyx_k_l[] = "l";
static char __pyx_k_q[] = "q";
static char __pyx_k_Zd[] = "Zd";
static char __pyx_k_Zf[] = "Zf";
static char __pyx_k_Zg[] = "Zg";
static char __pyx_k_i0[] = "i0";
static char __pyx_k_i1[] = "i1";
static char __pyx_k_iv[] = "iv";
static char __pyx_k_j0[] = "j0";
static char __pyx_k_j1[] = "j1";
static char __pyx_k_jn[] = "jn";
static char __pyx_k_jv[] = "jv";
static char __pyx_k_k0[] = "k0";
static char __pyx_k_k1[] = "k1";
static char __pyx_k_kn[] = "kn";
static char __pyx_k_kv[] = "kv";
static char __pyx_k_y0[] = "y0";
static char __pyx_k_y1[] = "y1";
static char __pyx_k_yn[] = "yn";
static char __pyx_k_yv[] = "yv";
static char __pyx_k_bei[] = "bei";
static char __pyx_k_ber[] = "ber";
static char __pyx_k_erf[] = "erf";
static char __pyx_k_i0e[] = "i0e";
static char __pyx_k_i1e[] = "i1e";
static char __pyx_k_ive[] = "ive";
static char __pyx_k_jve[] = "jve";
static char __pyx_k_k0e[] = "k0e";
static char __pyx_k_k1e[] = "k1e";
static char __pyx_k_kei[] = "kei";
static char __pyx_k_ker[] = "ker";
static char __pyx_k_kve[] = "kve";
static char __pyx_k_psi[] = "psi";
static char __pyx_k_yve[] = "yve";
static char __pyx_k_airy[] = "airy";
static char __pyx_k_bdtr[] = "bdtr";
static char __pyx_k_beip[] = "beip";
static char __pyx_k_berp[] = "berp";
static char __pyx_k_beta[] = "beta";
static char __pyx_k_cbrt[] = "cbrt";
static char __pyx_k_entr[] = "entr";
static char __pyx_k_erfc[] = "erfc";
static char __pyx_k_erfi[] = "erfi";
static char __pyx_k_exp1[] = "exp1";
static char __pyx_k_exp2[] = "exp2";
static char __pyx_k_expi[] = "expi";
static char __pyx_k_expn[] = "expn";
static char __pyx_k_fdtr[] = "fdtr";
static char __pyx_k_gdtr[] = "gdtr";
static char __pyx_k_keip[] = "keip";
static char __pyx_k_kerp[] = "kerp";
static char __pyx_k_lpmv[] = "lpmv";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_ndtr[] = "ndtr";
static char __pyx_k_pbdv[] = "pbdv";
static char __pyx_k_pbvv[] = "pbvv";
static char __pyx_k_pbwa[] = "pbwa";
static char __pyx_k_pdtr[] = "pdtr";
static char __pyx_k_poch[] = "poch";
static char __pyx_k_sici[] = "sici";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_wofz[] = "wofz";
static char __pyx_k_zeta[] = "zeta";
static char __pyx_k_airye[] = "airye";
static char __pyx_k_bdtrc[] = "bdtrc";
static char __pyx_k_bdtri[] = "bdtri";
static char __pyx_k_binom[] = "binom";
static char __pyx_k_btdtr[] = "btdtr";
static char __pyx_k_chdtr[] = "chdtr";
static char __pyx_k_cosdg[] = "cosdg";
static char __pyx_k_cosm1[] = "cosm1";
static char __pyx_k_cotdg[] = "cotdg";
static char __pyx_k_dawsn[] = "dawsn";
static char __pyx_k_erfcx[] = "erfcx";
static char __pyx_k_exp10[] = "exp10";
static char __pyx_k_expit[] = "expit";
static char __pyx_k_expm1[] = "expm1";
static char __pyx_k_fdtrc[] = "fdtrc";
static char __pyx_k_fdtri[] = "fdtri";
static char __pyx_k_gamma[] = "gamma";
static char __pyx_k_gdtrc[] = "gdtrc";
static char __pyx_k_huber[] = "huber";
static char __pyx_k_log1p[] = "log1p";
static char __pyx_k_logit[] = "logit";
static char __pyx_k_nbdtr[] = "nbdtr";
static char __pyx_k_ndtri[] = "ndtri";
static char __pyx_k_pdtrc[] = "pdtrc";
static char __pyx_k_pdtri[] = "pdtri";
static char __pyx_k_range[] = "range";
static char __pyx_k_round[] = "round";
static char __pyx_k_sindg[] = "sindg";
static char __pyx_k_stdtr[] = "stdtr";
static char __pyx_k_tandg[] = "tandg";
static char __pyx_k_xlogy[] = "xlogy";
static char __pyx_k_zetac[] = "zetac";
static char __pyx_k_bdtrik[] = "bdtrik";
static char __pyx_k_bdtrin[] = "bdtrin";
static char __pyx_k_betaln[] = "betaln";
static char __pyx_k_boxcox[] = "boxcox";
static char __pyx_k_btdtri[] = "btdtri";
static char __pyx_k_chdtrc[] = "chdtrc";
static char __pyx_k_chdtri[] = "chdtri";
static char __pyx_k_chndtr[] = "chndtr";
static char __pyx_k_ellipe[] = "ellipe";
static char __pyx_k_ellipj[] = "ellipj";
static char __pyx_k_exprel[] = "exprel";
static char __pyx_k_gdtria[] = "gdtria";
static char __pyx_k_gdtrib[] = "gdtrib";
static char __pyx_k_gdtrix[] = "gdtrix";
static char __pyx_k_hyp1f1[] = "hyp1f1";
static char __pyx_k_hyp1f2[] = "hyp1f2";
static char __pyx_k_hyp2f0[] = "hyp2f0";
static char __pyx_k_hyp2f1[] = "hyp2f1";
static char __pyx_k_hyp3f0[] = "hyp3f0";
static char __pyx_k_hyperu[] = "hyperu";
static char __pyx_k_inflag[] = "inflag";
static char __pyx_k_itairy[] = "itairy";
static char __pyx_k_iti0k0[] = "iti0k0";
static char __pyx_k_itj0y0[] = "itj0y0";
static char __pyx_k_kelvin[] = "kelvin";
static char __pyx_k_kl_div[] = "kl_div";
static char __pyx_k_nbdtrc[] = "nbdtrc";
static char __pyx_k_nbdtri[] = "nbdtri";
static char __pyx_k_ncfdtr[] = "ncfdtr";
static char __pyx_k_nctdtr[] = "nctdtr";
static char __pyx_k_obl_cv[] = "obl_cv";
static char __pyx_k_pdtrik[] = "pdtrik";
static char __pyx_k_pro_cv[] = "pro_cv";
static char __pyx_k_radian[] = "radian";
static char __pyx_k_rgamma[] = "rgamma";
static char __pyx_k_shichi[] = "shichi";
static char __pyx_k_spence[] = "spence";
static char __pyx_k_struve[] = "struve";
static char __pyx_k_betainc[] = "betainc";
static char __pyx_k_btdtria[] = "btdtria";
static char __pyx_k_btdtrib[] = "btdtrib";
static char __pyx_k_chdtriv[] = "chdtriv";
static char __pyx_k_fresnel[] = "fresnel";
static char __pyx_k_gammaln[] = "gammaln";
static char __pyx_k_hankel1[] = "hankel1";
static char __pyx_k_hankel2[] = "hankel2";
static char __pyx_k_it2i0k0[] = "it2i0k0";
static char __pyx_k_it2j0y0[] = "it2j0y0";
static char __pyx_k_kolmogi[] = "kolmogi";
static char __pyx_k_nbdtrik[] = "nbdtrik";
static char __pyx_k_nbdtrin[] = "nbdtrin";
static char __pyx_k_ncfdtri[] = "ncfdtri";
static char __pyx_k_smirnov[] = "smirnov";
static char __pyx_k_stdtrit[] = "stdtrit";
static char __pyx_k_tklmbda[] = "tklmbda";
static char __pyx_k_xlog1py[] = "xlog1py";
static char __pyx_k_boxcox1p[] = "boxcox1p";
static char __pyx_k_chndtrix[] = "chndtrix";
static char __pyx_k_ellipkm1[] = "ellipkm1";
static char __pyx_k_errprint[] = "errprint";
static char __pyx_k_fdtridfd[] = "fdtridfd";
static char __pyx_k_gammainc[] = "gammainc";
static char __pyx_k_gammasgn[] = "gammasgn";
static char __pyx_k_hankel1e[] = "hankel1e";
static char __pyx_k_hankel2e[] = "hankel2e";
static char __pyx_k_lambertw[] = "lambertw";
static char __pyx_k_log_ndtr[] = "log_ndtr";
static char __pyx_k_nctdtrit[] = "nctdtrit";
static char __pyx_k_nrdtrimn[] = "nrdtrimn";
static char __pyx_k_nrdtrisd[] = "nrdtrisd";
static char __pyx_k_obl_ang1[] = "obl_ang1";
static char __pyx_k_obl_rad1[] = "obl_rad1";
static char __pyx_k_obl_rad2[] = "obl_rad2";
static char __pyx_k_pro_ang1[] = "pro_ang1";
static char __pyx_k_pro_rad1[] = "pro_rad1";
static char __pyx_k_pro_rad2[] = "pro_rad2";
static char __pyx_k_rel_entr[] = "rel_entr";
static char __pyx_k_smirnovi[] = "smirnovi";
static char __pyx_k_sph_harm[] = "sph_harm";
static char __pyx_k_stdtridf[] = "stdtridf";
static char __pyx_k_chndtridf[] = "chndtridf";
static char __pyx_k_chndtrinc[] = "chndtrinc";
static char __pyx_k_ellipeinc[] = "ellipeinc";
static char __pyx_k_ellipkinc[] = "ellipkinc";
static char __pyx_k_gammaincc[] = "gammaincc";
static char __pyx_k_itstruve0[] = "itstruve0";
static char __pyx_k_mathieu_a[] = "mathieu_a";
static char __pyx_k_mathieu_b[] = "mathieu_b";
static char __pyx_k_modstruve[] = "modstruve";
static char __pyx_k_ncfdtrinc[] = "ncfdtrinc";
static char __pyx_k_nctdtridf[] = "nctdtridf";
static char __pyx_k_nctdtrinc[] = "nctdtrinc";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_besselpoly[] = "besselpoly";
static char __pyx_k_betaincinv[] = "betaincinv";
static char __pyx_k_ellip_harm[] = "ellip_harm";
static char __pyx_k_exp2_x_2_x[] = "exp2(x)\n\n2**x";
static char __pyx_k_inv_boxcox[] = "inv_boxcox";
static char __pyx_k_it2struve0[] = "it2struve0";
static char __pyx_k_kolmogorov[] = "kolmogorov";
static char __pyx_k_lambertw_2[] = "_lambertw";
static char __pyx_k_ncfdtridfd[] = "ncfdtridfd";
static char __pyx_k_ncfdtridfn[] = "ncfdtridfn";
static char __pyx_k_eval_chebyc[] = "eval_chebyc";
static char __pyx_k_eval_chebys[] = "eval_chebys";
static char __pyx_k_eval_chebyt[] = "eval_chebyt";
static char __pyx_k_eval_chebyu[] = "eval_chebyu";
static char __pyx_k_eval_jacobi[] = "eval_jacobi";
static char __pyx_k_gammaincinv[] = "gammaincinv";
static char __pyx_k_mathieu_cem[] = "mathieu_cem";
static char __pyx_k_mathieu_sem[] = "mathieu_sem";
static char __pyx_k_modfresnelm[] = "modfresnelm";
static char __pyx_k_modfresnelp[] = "modfresnelp";
static char __pyx_k_obl_ang1_cv[] = "obl_ang1_cv";
static char __pyx_k_obl_rad1_cv[] = "obl_rad1_cv";
static char __pyx_k_obl_rad2_cv[] = "obl_rad2_cv";
static char __pyx_k_pro_ang1_cv[] = "pro_ang1_cv";
static char __pyx_k_pro_rad1_cv[] = "pro_rad1_cv";
static char __pyx_k_pro_rad2_cv[] = "pro_rad2_cv";
static char __pyx_k_RuntimeError[] = "RuntimeError";
static char __pyx_k_ellip_harm_2[] = "_ellip_harm";
static char __pyx_k_eval_hermite[] = "eval_hermite";
static char __pyx_k_exp10_x_10_x[] = "exp10(x)\n\n10**x";
static char __pyx_k_gammainccinv[] = "gammainccinv";
static char __pyx_k_inv_boxcox1p[] = "inv_boxcox1p";
static char __pyx_k_itmodstruve0[] = "itmodstruve0";
static char __pyx_k_pseudo_huber[] = "pseudo_huber";
static char __pyx_k_eval_laguerre[] = "eval_laguerre";
static char __pyx_k_eval_legendre[] = "eval_legendre";
static char __pyx_k_RuntimeWarning[] = "RuntimeWarning";
static char __pyx_k_eval_sh_chebyt[] = "eval_sh_chebyt";
static char __pyx_k_eval_sh_chebyu[] = "eval_sh_chebyu";
static char __pyx_k_eval_sh_jacobi[] = "eval_sh_jacobi";
static char __pyx_k_eval_gegenbauer[] = "eval_gegenbauer";
static char __pyx_k_mathieu_modcem1[] = "mathieu_modcem1";
static char __pyx_k_mathieu_modcem2[] = "mathieu_modcem2";
static char __pyx_k_mathieu_modsem1[] = "mathieu_modsem1";
static char __pyx_k_mathieu_modsem2[] = "mathieu_modsem2";
static char __pyx_k_eval_genlaguerre[] = "eval_genlaguerre";
static char __pyx_k_eval_hermitenorm[] = "eval_hermitenorm";
static char __pyx_k_eval_sh_legendre[] = "eval_sh_legendre";
static char __pyx_k_invalid_value_for_n[] = "invalid value for n";
static char __pyx_k_invalid_value_for_p[] = "invalid value for p";
static char __pyx_k_struve_power_series[] = "_struve_power_series";
static char __pyx_k_struve_asymp_large_z[] = "_struve_asymp_large_z";
static char __pyx_k_struve_bessel_series[] = "_struve_bessel_series";
static char __pyx_k_cbrt_x_Cube_root_of_x[] = "cbrt(x)\n\nCube root of `x`";
static char __pyx_k_scipy_special__ufuncs[] = "scipy.special._ufuncs";
static char __pyx_k_invalid_input_argument[] = "invalid input argument";
static char __pyx_k_invalid_signm_or_signn[] = "invalid signm or signn";
static char __pyx_k_n_should_not_be_negative[] = "n should not be negative";
static char __pyx_k_bei_x_Kelvin_function_bei[] = "bei(x)\n\nKelvin function bei";
static char __pyx_k_ber_x_Kelvin_function_ber[] = "ber(x)\n\nKelvin function ber.";
static char __pyx_k_failed_to_allocate_memory[] = "failed to allocate memory";
static char __pyx_k_kei_x_Kelvin_function_ker[] = "kei(x)\n\nKelvin function ker";
static char __pyx_k_ker_x_Kelvin_function_ker[] = "ker(x)\n\nKelvin function ker";
static char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static char __pyx_k_binom_n_k_Binomial_coefficient[] = "binom(n, k)\n\nBinomial coefficient";
static char __pyx_k_m_should_not_be_greater_than_n[] = "m should not be greater than n";
static char __pyx_k_airy_z_Airy_functions_and_their[] = "airy(z)\n\nAiry functions and their derivatives.\n\nParameters\n----------\nz : float or complex\n    Argument.\n\nReturns\n-------\nAi, Aip, Bi, Bip\n    Airy functions Ai and Bi, and their derivatives Aip and Bip\n\nNotes\n-----\nThe Airy functions Ai and Bi are two independent solutions of y''(x) = x y.";
static char __pyx_k_bdtri_k_n_y_Inverse_function_to[] = "bdtri(k, n, y)\n\nInverse function to `bdtr` vs. `p`\n\nFinds probability `p` such that for the cumulative binomial\nprobability ``bdtr(k, n, p) == y``.";
static char __pyx_k_beip_x_Derivative_of_the_Kelvin[] = "beip(x)\n\nDerivative of the Kelvin function `bei`";
static char __pyx_k_berp_x_Derivative_of_the_Kelvin[] = "berp(x)\n\nDerivative of the Kelvin function `ber`";
static char __pyx_k_beta_a_b_Beta_function_beta_a_b[] = "beta(a, b)\n\nBeta function.\n\n::\n\n    beta(a, b) =  gamma(a) * gamma(b) / gamma(a+b)";
static char __pyx_k_betaln_a_b_Natural_logarithm_of[] = "betaln(a, b)\n\nNatural logarithm of absolute value of beta function.\n\nComputes ``ln(abs(beta(x)))``.";
static char __pyx_k_chdtr_v_x_Chi_square_cumulative[] = "chdtr(v, x)\n\nChi square cumulative distribution function\n\nReturns the area under the left hand tail (from 0 to `x`) of the Chi\nsquare probability density function with `v` degrees of freedom::\n\n    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=0..x)";
static char __pyx_k_entr_x_Elementwise_function_for[] = "entr(x)\n\nElementwise function for computing entropy.\n\n.. math:: \\text{entr}(x) = \\begin{cases} - x \\log(x) & x > 0  \\\\ 0 & x = 0 \\\\ -\\infty & \\text{otherwise} \\end{cases}\n\nParameters\n----------\nx : ndarray\n    Input array.\n\nReturns\n-------\nres : ndarray\n    The value of the elementwise entropy function at the given points `x`.\n\nSee Also\n--------\nkl_div, rel_entr\n\nNotes\n-----\nThis function is concave.\n\n.. versionadded:: 0.14.0";
static char __pyx_k_erfi_z_Imaginary_error_function[] = "erfi(z)\n\nImaginary error function, -i erf(i z).\n\nNotes\n-----\n\n.. versionadded:: 0.12.0\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva";
static char __pyx_k_eval_sh_jacobi_n_p_q_x_out_None[] = "eval_sh_jacobi(n, p, q, x, out=None)\n\nEvaluate shifted Jacobi polynomial at a point.";
static char __pyx_k_exp1_z_Exponential_integral_E_1[] = "exp1(z)\n\nExponential integral E_1 of complex argument z\n\n::\n\n    integral(exp(-z*t)/t, t=1..inf).";
static char __pyx_k_expn_n_x_Exponential_integral_E[] = "expn(n, x)\n\nExponential integral E_n\n\nReturns the exponential integral for integer `n` and non-negative `x` and\n`n`::\n\n    integral(exp(-x*t) / t**n, t=1..inf).";
static char __pyx_k_fdtri_dfn_dfd_p_Inverse_to_fdtr[] = "fdtri(dfn, dfd, p)\n\nInverse to `fdtr` vs x\n\nFinds the F density argument `x` such that ``fdtr(dfn, dfd, x) == p``.";
static char __pyx_k_floating_point_number_truncated[] = "floating point number truncated to an integer";
static char __pyx_k_gammaln_z_Logarithm_of_absolute[] = "gammaln(z)\n\nLogarithm of absolute value of gamma function\n\nDefined as::\n\n    ln(abs(gamma(z)))\n\nSee Also\n--------\ngammasgn";
static char __pyx_k_home_br_repos_scipy_scipy_speci[] = "/home/br/repos/scipy/scipy/special/_ufuncs.pyx";
static char __pyx_k_iv_v_z_Modified_Bessel_function[] = "iv(v, z)\n\nModified Bessel function of the first kind  of real order\n\nParameters\n----------\nv\n    Order. If `z` is of real type and negative, `v` must be integer valued.\nz\n    Argument.";
static char __pyx_k_k0_x_Modified_Bessel_function_K[] = "k0(x)\n\nModified Bessel function K of order 0\n\nModified Bessel function of the second kind (sometimes called the\nthird kind) of order 0.";
static char __pyx_k_keip_x_Derivative_of_the_Kelvin[] = "keip(x)\n\nDerivative of the Kelvin function kei";
static char __pyx_k_kerp_x_Derivative_of_the_Kelvin[] = "kerp(x)\n\nDerivative of the Kelvin function ker";
static char __pyx_k_kl_div_x_y_Elementwise_function[] = "kl_div(x, y)\n\nElementwise function for computing Kullback-Leibler divergence.\n\n.. math:: \\mathrm{kl\\_div}(x, y) = \\begin{cases} x \\log(x / y) - x + y & x > 0, y > 0 \\\\ y & x = 0, y \\ge 0 \\\\ \\infty & \\text{otherwise} \\end{cases}\n\nParameters\n----------\nx : ndarray\n    First input array.\ny : ndarray\n    Second input array.\n\nReturns\n-------\nres : ndarray\n    Output array.\n\nSee Also\n--------\nentr, rel_entr\n\nNotes\n-----\nThis function is non-negative and is jointly convex in `x` and `y`.\n\n.. versionadded:: 0.14.0";
static char __pyx_k_kn_n_x_Modified_Bessel_function[] = "kn(n, x)\n\nModified Bessel function of the second kind of integer order `n`\n\nReturns the modified Bessel function of the second kind for integer order\n`n` at real `z`.\n\nThese are also sometimes called functions of the third kind, Basset\nfunctions, or Macdonald functions.\n\nParameters\n----------\nn : array_like of int\n    Order of Bessel functions (floats will truncate with a warning)\nz : array_like of float\n    Argument at which to evaluate the Bessel functions\n\nReturns\n-------\nout : ndarray\n    The results\n\nSee Also\n--------\nkv : Same function, but accepts real order and complex argument\nkvp : Derivative of this function\n\nExamples\n--------\nPlot the function of several orders for real input:\n\n>>> from scipy.special import kn\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(0, 5, 1000)\n>>> for N in range(6):\n...     plt.plot(x, kn(N, x), label='$K_{}(x)$'.format(N))\n>>> plt.ylim(0, 10)\n>>> plt.legend()\n>>> plt.title(r'Modified Bessel function of the second kind $K_n(x)$')\n>>> plt.show()\n\nCalculate for a single value at multiple orders:\n\n>>> kn([4, 5, 6], 1)\narray([   44.23241585,   360.9605896 ,  3653.83831186])";
static char __pyx_k_kv_v_z_Modified_Bessel_function[] = "kv(v, z)\n\nModified Bessel function of the second kind of real order `v`\n\nReturns the modified Bessel function of the second kind for real order\n`v` at complex `z`.\n\nThese are also sometimes called functions of the third kind, Basset\nfunctions, or Macdonald functions.\n\nParameters\n----------\nv : array_like of float\n    Order of Bessel functions\nz : array_like of complex\n    Argument at which to evaluate the Bessel functions\n\nReturns\n-------\nout : ndarray\n    The results. Note that input must be of complex type to get complex\n    output, e.g. ``kv(3, -2+0j)`` instead of ``kv(3, -2)``.\n\nSee Also\n--------\nkvp : Derivative of this function\n\nExamples\n--------\nPlot the function of several orders for real input:\n\n>>> from scipy.special import kv\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(0, 5, 1000)\n>>> for N in np.linspace(0, 6, 5):\n...     plt.plot(x, kv(N, x), label='$K_{{{}}}(x)$'.format(N))\n>>> plt.ylim(0, 10)\n>>> plt.legend()\n>>> plt.title(r'Modified Bessel function of the second kind $K_\\nu(x)$')\n>>> plt.show()\n\nCalculate for a single value at multiple orders:\n\n>>> kv([4, 4.5, 5], 1+2j)\narray([ 0.1992+2.3892j,  2.3493+3.6j   ,  7.2827+3.8104j])";
static char __pyx_k_ncfdtridfd_p_f_dfn_nc_Calculate[] = "ncfdtridfd(p, f, dfn, nc)\n\nCalculate degrees of freedom (denominator) for the noncentral F-distribution.\n\nSee `ncfdtr` for more details.";
static char __pyx_k_ncfdtridfn_p_f_dfd_nc_Calculate[] = "ncfdtridfn(p, f, dfd, nc)\n\nCalculate degrees of freedom (numerator) for the noncentral F-distribution.\n\nSee `ncfdtr` for more details.";
static char __pyx_k_ncfdtrinc_p_f_dfn_dfd_Calculate[] = "ncfdtrinc(p, f, dfn, dfd)\n\nCalculate non-centrality parameter for non-central F distribution.\n\nSee `ncfdtr` for more details.";
static char __pyx_k_nrdtrimn_p_x_std_Calculate_mean[] = "nrdtrimn(p, x, std)\n\nCalculate mean of normal distribution given other params.\n\nParameters\n----------\np : array_like\n    CDF values, in range (0, 1].\nx : array_like\n    Quantiles, i.e. the upper limit of integration.\nstd : array_like\n    Standard deviation.\n\nReturns\n-------\nmn : float or ndarray\n    The mean of the normal distribution.\n\nSee Also\n--------\nnrdtrimn, ndtr";
static char __pyx_k_pdtrik_p_m_Inverse_to_pdtr_vs_k[] = "pdtrik(p, m)\n\nInverse to `pdtr` vs k\n\nReturns the quantile k such that ``pdtr(k, m) = p``";
static char __pyx_k_smirnovi_n_y_Inverse_to_smirnov[] = "smirnovi(n, y)\n\nInverse to `smirnov`\n\nReturns ``e`` such that ``smirnov(n, e) = y``.";
static char __pyx_k_struve_asymp_large_z_v_z_is_h_I[] = "_struve_asymp_large_z(v, z, is_h)\n\nInternal function for testing `struve` & `modstruve`\n\nEvaluates using asymptotic expansion\n\nReturns\n-------\nv, err";
static char __pyx_k_struve_bessel_series_v_z_is_h_I[] = "_struve_bessel_series(v, z, is_h)\n\nInternal function for testing `struve` & `modstruve`\n\nEvaluates using Bessel function series\n\nReturns\n-------\nv, err";
static char __pyx_k_struve_power_series_v_z_is_h_In[] = "_struve_power_series(v, z, is_h)\n\nInternal function for testing `struve` & `modstruve`\n\nEvaluates using power series\n\nReturns\n-------\nv, err";
static char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static char __pyx_k_Internal_function_use_ellip_harm[] = "Internal function, use `ellip_harm` instead.";
static char __pyx_k_Internal_function_use_lambertw_i[] = "Internal function, use `lambertw` instead.";
static char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static char __pyx_k_airye_z_Exponentially_scaled_Air[] = "airye(z)\n\nExponentially scaled Airy functions and their derivatives.\n\nScaling::\n\n    eAi  = Ai  * exp(2.0/3.0*z*sqrt(z))\n    eAip = Aip * exp(2.0/3.0*z*sqrt(z))\n    eBi  = Bi  * exp(-abs((2.0/3.0*z*sqrt(z)).real))\n    eBip = Bip * exp(-abs((2.0/3.0*z*sqrt(z)).real))\n\nParameters\n----------\nz : float or complex\n    Argument.\n\nReturns\n-------\neAi, eAip, eBi, eBip\n    Airy functions Ai and Bi, and their derivatives Aip and Bip";
static char __pyx_k_bdtr_k_n_p_Binomial_distribution[] = "bdtr(k, n, p)\n\nBinomial distribution cumulative distribution function.\n\nSum of the terms 0 through `k` of the Binomial probability density.\n\n::\n\n    y = sum(nCj p**j (1-p)**(n-j), j=0..k)\n\nParameters\n----------\nk, n : int\n    Terms to include\np : float\n    Probability\n\nReturns\n-------\ny : float\n    Sum of terms";
static char __pyx_k_bdtrc_k_n_p_Binomial_distributio[] = "bdtrc(k, n, p)\n\nBinomial distribution survival function.\n\nSum of the terms k+1 through `n` of the Binomial probability density\n\n::\n\n    y = sum(nCj p**j (1-p)**(n-j), j=k+1..n)\n\nParameters\n----------\nk, n : int\n    Terms to include\np : float\n    Probability\n\nReturns\n-------\ny : float\n    Sum of terms";
static char __pyx_k_bdtrik_y_n_p_Inverse_function_to[] = "bdtrik(y, n, p)\n\nInverse function to `bdtr` vs `k`";
static char __pyx_k_bdtrin_k_y_p_Inverse_function_to[] = "bdtrin(k, y, p)\n\nInverse function to `bdtr` vs `n`";
static char __pyx_k_besselpoly_a_lmb_nu_Weighted_int[] = "besselpoly(a, lmb, nu)\n\nWeighted integral of a Bessel function.\n\n.. math::\n\n   \\int_0^1 x^\\lambda J_\\nu(2 a x) \\, dx\n\nwhere :math:`J_\\nu` is a Bessel function and :math:`\\lambda=lmb`,\n:math:`\\nu=nu`.";
static char __pyx_k_betainc_a_b_x_Incomplete_beta_in[] = "betainc(a, b, x)\n\nIncomplete beta integral.\n\nCompute the incomplete beta integral of the arguments, evaluated\nfrom zero to `x`::\n\n    gamma(a+b) / (gamma(a)*gamma(b)) * integral(t**(a-1) (1-t)**(b-1), t=0..x).\n\nNotes\n-----\nThe incomplete beta is also sometimes defined without the terms\nin gamma, in which case the above definition is the so-called regularized\nincomplete beta. Under this definition, you can get the incomplete beta by\nmultiplying the result of the scipy function by beta(a, b).";
static char __pyx_k_betaincinv_a_b_y_Inverse_functio[] = "betaincinv(a, b, y)\n\nInverse function to beta integral.\n\nCompute `x` such that betainc(a, b, x) = y.";
static char __pyx_k_boxcox1p_x_lmbda_Compute_the_Box[] = "boxcox1p(x, lmbda)\n\nCompute the Box-Cox transformation of 1 + `x`.\n\nThe Box-Cox transformation computed by `boxcox1p` is::\n\n    y = ((1+x)**lmbda - 1) / lmbda  if lmbda != 0\n        log(1+x)                    if lmbda == 0\n\nReturns `nan` if ``x < -1``.\nReturns `-inf` if ``x == -1`` and ``lmbda < 0``.\n\nParameters\n----------\nx : array_like\n    Data to be transformed.\nlmbda : array_like\n    Power parameter of the Box-Cox transform.\n\nReturns\n-------\ny : array\n    Transformed data.\n\nNotes\n-----\n\n.. versionadded:: 0.14.0\n\nExamples\n--------\n>>> from scipy.special import boxcox1p\n>>> boxcox1p(1e-4, [0, 0.5, 1])\narray([  9.99950003e-05,   9.99975001e-05,   1.00000000e-04])\n>>> boxcox1p([0.01, 0.1], 0.25)\narray([ 0.00996272,  0.09645476])";
static char __pyx_k_boxcox_x_lmbda_Compute_the_Box_C[] = "boxcox(x, lmbda)\n\nCompute the Box-Cox transformation.\n\nThe Box-Cox transformation is::\n\n    y = (x**lmbda - 1) / lmbda  if lmbda != 0\n        log(x)                  if lmbda == 0\n\nReturns `nan` if ``x < 0``.\nReturns `-inf` if ``x == 0`` and ``lmbda < 0``.\n\nParameters\n----------\nx : array_like\n    Data to be transformed.\nlmbda : array_like\n    Power parameter of the Box-Cox transform.\n\nReturns\n-------\ny : array\n    Transformed data.\n\nNotes\n-----\n\n.. versionadded:: 0.14.0\n\nExamples\n--------\n>>> from scipy.special import boxcox\n>>> boxcox([1, 4, 10], 2.5)\narray([   0.        ,   12.4       ,  126.09110641])\n>>> boxcox(2, [0, 1, 2])\narray([ 0.69314718,  1.        ,  1.5       ])";
static char __pyx_k_btdtr_a_b_x_Cumulative_beta_dist[] = "btdtr(a, b, x)\n\nCumulative beta distribution.\n\nReturns the area from zero to `x` under the beta density function::\n\n    gamma(a+b)/(gamma(a)*gamma(b)))*integral(t**(a-1) (1-t)**(b-1), t=0..x)\n\nSee Also\n--------\nbetainc";
static char __pyx_k_btdtri_a_b_p_p_th_quantile_of_th[] = "btdtri(a, b, p)\n\np-th quantile of the beta distribution.\n\nThis is effectively the inverse of `btdtr` returning the value of `x` for which\n``btdtr(a, b, x) = p``\n\nSee Also\n--------\nbetaincinv";
static char __pyx_k_btdtria_a_p_x_Inverse_of_btdtr_v[] = "btdtria(a, p, x)\n\nInverse of `btdtr` vs `b`";
static char __pyx_k_btdtria_p_b_x_Inverse_of_btdtr_v[] = "btdtria(p, b, x)\n\nInverse of `btdtr` vs `a`";
static char __pyx_k_chdtrc_v_x_Chi_square_survival_f[] = "chdtrc(v, x)\n\nChi square survival function\n\nReturns the area under the right hand tail (from `x` to\ninfinity) of the Chi square probability density function with `v`\ndegrees of freedom::\n\n    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=x..inf)";
static char __pyx_k_chdtri_p_x_Inverse_to_chdtr_vs_v[] = "chdtri(p, x)\n\nInverse to `chdtr` vs `v`\n\nReturns the argument v such that ``chdtr(v, x) == p``.";
static char __pyx_k_chdtri_v_p_Inverse_to_chdtrc_Ret[] = "chdtri(v, p)\n\nInverse to `chdtrc`\n\nReturns the argument x such that ``chdtrc(v, x) == p``.";
static char __pyx_k_chndtr_x_df_nc_Non_central_chi_s[] = "chndtr(x, df, nc)\n\nNon-central chi square cumulative distribution function";
static char __pyx_k_chndtridf_x_p_nc_Inverse_to_chnd[] = "chndtridf(x, p, nc)\n\nInverse to `chndtr` vs `df`";
static char __pyx_k_chndtrinc_x_df_p_Inverse_to_chnd[] = "chndtrinc(x, df, p)\n\nInverse to `chndtr` vs `nc`";
static char __pyx_k_chndtrix_p_df_nc_Inverse_to_chnd[] = "chndtrix(p, df, nc)\n\nInverse to `chndtr` vs `x`";
static char __pyx_k_cosdg_x_Cosine_of_the_angle_x_gi[] = "cosdg(x)\n\nCosine of the angle `x` given in degrees.";
static char __pyx_k_cosm1_x_cos_x_1_for_use_when_x_i[] = "cosm1(x)\n\ncos(x) - 1 for use when `x` is near zero.";
static char __pyx_k_cotdg_x_Cotangent_of_the_angle_x[] = "cotdg(x)\n\nCotangent of the angle `x` given in degrees.";
static char __pyx_k_dawsn_x_Dawson_s_integral_Comput[] = "dawsn(x)\n\nDawson's integral.\n\nComputes::\n\n    exp(-x**2) * integral(exp(t**2), t=0..x).\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva";
static char __pyx_k_ellipe_m_Complete_elliptic_integ[] = "ellipe(m)\n\nComplete elliptic integral of the second kind\n\nThis function is defined as\n\n.. math:: E(m) = \\int_0^{\\pi/2} [1 - m \\sin(t)^2]^{1/2} dt\n\nParameters\n----------\nm : array_like\n    Defines the parameter of the elliptic integral.\n\nReturns\n-------\nE : ndarray\n    Value of the elliptic integral.\n\nSee Also\n--------\nellipkm1 : Complete elliptic integral of the first kind, near `m` = 1\nellipk : Complete elliptic integral of the first kind\nellipkinc : Incomplete elliptic integral of the first kind\nellipeinc : Incomplete elliptic integral of the second kind";
static char __pyx_k_ellipeinc_phi_m_Incomplete_ellip[] = "ellipeinc(phi, m)\n\nIncomplete elliptic integral of the second kind\n\nThis function is defined as\n\n.. math:: E(\\phi, m) = \\int_0^{\\phi} [1 - m \\sin(t)^2]^{1/2} dt\n\nParameters\n----------\nphi : array_like\n    amplitude of the elliptic integral.\n\nm : array_like\n    parameter of the elliptic integral.\n\nReturns\n-------\nE : ndarray\n    Value of the elliptic integral.\n\nSee Also\n--------\nellipkm1 : Complete elliptic integral of the first kind, near `m` = 1\nellipk : Complete elliptic integral of the first kind\nellipkinc : Incomplete elliptic integral of the first kind\nellipe : Complete elliptic integral of the second kind";
static char __pyx_k_ellipj_u_m_Jacobian_elliptic_fun[] = "ellipj(u, m)\n\nJacobian elliptic functions\n\nCalculates the Jacobian elliptic functions of parameter `m` between\n0 and 1, and real `u`.\n\nParameters\n----------\nm, u\n    Parameters\n\nReturns\n-------\nsn, cn, dn, ph\n    The returned functions::\n\n        sn(u|m), cn(u|m), dn(u|m)\n\n    The value ``ph`` is such that if ``u = ellik(ph, m)``,\n    then ``sn(u|m) = sin(ph)`` and ``cn(u|m) = cos(ph)``.";
static char __pyx_k_ellipkinc_phi_m_Incomplete_ellip[] = "ellipkinc(phi, m)\n\nIncomplete elliptic integral of the first kind\n\nThis function is defined as\n\n.. math:: K(\\phi, m) = \\int_0^{\\phi} [1 - m \\sin(t)^2]^{-1/2} dt\n\nParameters\n----------\nphi : array_like\n    amplitude of the elliptic integral\n\nm : array_like\n    parameter of the elliptic integral\n\nReturns\n-------\nK : ndarray\n    Value of the elliptic integral\n\nNotes\n-----\nThis function is also called ``F(phi, m)``.\n\nSee Also\n--------\nellipkm1 : Complete elliptic integral of the first kind, near `m` = 1\nellipk : Complete elliptic integral of the first kind\nellipe : Complete elliptic integral of the second kind\nellipeinc : Incomplete elliptic integral of the second kind";
static char __pyx_k_ellipkm1_p_Complete_elliptic_int[] = "ellipkm1(p)\n\nComplete elliptic integral of the first kind around `m` = 1\n\nThis function is defined as\n\n.. math:: K(p) = \\int_0^{\\pi/2} [1 - m \\sin(t)^2]^{-1/2} dt\n\nwhere `m = 1 - p`.\n\nParameters\n----------\np : array_like\n    Defines the parameter of the elliptic integral as `m` = 1 - p.\n\nReturns\n-------\nK : ndarray\n    Value of the elliptic integral.\n\nSee Also\n--------\nellipk : Complete elliptic integral of the first kind\nellipkinc : Incomplete elliptic integral of the first kind\nellipe : Complete elliptic integral of the second kind\nellipeinc : Incomplete elliptic integral of the second kind";
static char __pyx_k_erf_z_Returns_the_error_function[] = "erf(z)\n\nReturns the error function of complex argument.\n\nIt is defined as ``2/sqrt(pi)*integral(exp(-t**2), t=0..z)``.\n\nParameters\n----------\nx : ndarray\n    Input array.\n\nReturns\n-------\nres : ndarray\n    The values of the error function at the given points `x`.\n\nSee Also\n--------\nerfc, erfinv, erfcinv\n\nNotes\n-----\nThe cumulative of the unit normal distribution is given by\n``Phi(z) = 1/2[1 + erf(z/sqrt(2))]``.\n\nReferences\n----------\n.. [1] http://en.wikipedia.org/wiki/Error_function\n.. [2] Milton Abramowitz and Irene A. Stegun, eds.\n    Handbook of Mathematical Functions with Formulas,\n    Graphs, and Mathematical Tables. New York: Dover,\n    1972. http://www.math.sfu.ca/~cbm/aands/page_297.htm\n.. [3] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva";
static char __pyx_k_erfc_x_Complementary_error_funct[] = "erfc(x)\n\nComplementary error function, 1 - erf(x).\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva";
static char __pyx_k_erfcx_x_Scaled_complementary_err[] = "erfcx(x)\n\nScaled complementary error function, exp(x^2) erfc(x).\n\nNotes\n-----\n\n.. versionadded:: 0.12.0\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva";
static char __pyx_k_eval_chebyc_n_x_out_None_Evaluat[] = "eval_chebyc(n, x, out=None)\n\nEvaluate Chebyshev C polynomial at a point.";
static char __pyx_k_eval_chebys_n_x_out_None_Evaluat[] = "eval_chebys(n, x, out=None)\n\nEvaluate Chebyshev S polynomial at a point.";
static char __pyx_k_eval_chebyt_n_x_out_None_Evaluat[] = "eval_chebyt(n, x, out=None)\n\nEvaluate Chebyshev T polynomial at a point.\n\nThis routine is numerically stable for `x` in ``[-1, 1]`` at least\nup to order ``10000``.";
static char __pyx_k_eval_chebyu_n_x_out_None_Evaluat[] = "eval_chebyu(n, x, out=None)\n\nEvaluate Chebyshev U polynomial at a point.";
static char __pyx_k_eval_gegenbauer_n_alpha_x_out_No[] = "eval_gegenbauer(n, alpha, x, out=None)\n\nEvaluate Gegenbauer polynomial at a point.";
static char __pyx_k_eval_genlaguerre_n_alpha_x_out_N[] = "eval_genlaguerre(n, alpha, x, out=None)\n\nEvaluate generalized Laguerre polynomial at a point.";
static char __pyx_k_eval_hermite_n_x_out_None_Evalua[] = "eval_hermite(n, x, out=None)\n\nEvaluate Hermite polynomial at a point.";
static char __pyx_k_eval_hermitenorm_n_x_out_None_Ev[] = "eval_hermitenorm(n, x, out=None)\n\nEvaluate normalized Hermite polynomial at a point.";
static char __pyx_k_eval_jacobi_n_alpha_beta_x_out_N[] = "eval_jacobi(n, alpha, beta, x, out=None)\n\nEvaluate Jacobi polynomial at a point.";
static char __pyx_k_eval_laguerre_n_x_out_None_Evalu[] = "eval_laguerre(n, x, out=None)\n\nEvaluate Laguerre polynomial at a point.";
static char __pyx_k_eval_legendre_n_x_out_None_Evalu[] = "eval_legendre(n, x, out=None)\n\nEvaluate Legendre polynomial at a point.";
static char __pyx_k_eval_sh_chebyt_n_x_out_None_Eval[] = "eval_sh_chebyt(n, x, out=None)\n\nEvaluate shifted Chebyshev T polynomial at a point.";
static char __pyx_k_eval_sh_chebyu_n_x_out_None_Eval[] = "eval_sh_chebyu(n, x, out=None)\n\nEvaluate shifted Chebyshev U polynomial at a point.";
static char __pyx_k_eval_sh_legendre_n_x_out_None_Ev[] = "eval_sh_legendre(n, x, out=None)\n\nEvaluate shifted Legendre polynomial at a point.";
static char __pyx_k_expi_x_Exponential_integral_Ei_D[] = "expi(x)\n\nExponential integral Ei\n\nDefined as::\n\n    integral(exp(t)/t, t=-inf..x)\n\nSee `expn` for a different exponential integral.";
static char __pyx_k_expit_x_Expit_ufunc_for_ndarrays[] = "expit(x)\n\nExpit ufunc for ndarrays.\n\nThe expit function, also known as the logistic function, is defined as\nexpit(x) = 1/(1+exp(-x)). It is the inverse of the logit function.\n\nParameters\n----------\nx : ndarray\n    The ndarray to apply expit to element-wise.\n\nReturns\n-------\nout : ndarray\n    An ndarray of the same shape as x. Its entries\n    are expit of the corresponding entry of x.\n\nNotes\n-----\nAs a ufunc expit takes a number of optional\nkeyword arguments. For more information\nsee `ufuncs <http://docs.scipy.org/doc/numpy/reference/ufuncs.html>`_\n\n.. versionadded:: 0.10.0";
static char __pyx_k_expm1_x_exp_x_1_for_use_when_x_i[] = "expm1(x)\n\nexp(x) - 1 for use when `x` is near zero.";
static char __pyx_k_exprel_x_Relative_error_exponent[] = "exprel(x)\n\nRelative error exponential, (exp(x)-1)/x, for use when `x` is near zero.\n\nParameters\n----------\nx : ndarray\n    Input array.\n\nReturns\n-------\nres : ndarray\n    Output array.\n\nSee Also\n--------\nexpm1\n\n.. versionadded:: 0.17.0";
static char __pyx_k_fdtr_dfn_dfd_x_F_cumulative_dist[] = "fdtr(dfn, dfd, x)\n\nF cumulative distribution function\n\nReturns the area from zero to `x` under the F density function (also\nknown as Snedcor's density or the variance ratio density).  This\nis the density of X = (unum/dfn)/(uden/dfd), where unum and uden\nare random variables having Chi square distributions with dfn and\ndfd degrees of freedom, respectively.";
static char __pyx_k_fdtrc_dfn_dfd_x_F_survival_funct[] = "fdtrc(dfn, dfd, x)\n\nF survival function\n\nReturns the complemented F distribution function.";
static char __pyx_k_fdtridfd_dfn_p_x_Inverse_to_fdtr[] = "fdtridfd(dfn, p, x)\n\nInverse to `fdtr` vs dfd\n\nFinds the F density argument dfd such that ``fdtr(dfn, dfd, x) == p``.";
static char __pyx_k_fresnel_z_Fresnel_sin_and_cos_in[] = "fresnel(z)\n\nFresnel sin and cos integrals\n\nDefined as::\n\n    ssa = integral(sin(pi/2 * t**2), t=0..z)\n    csa = integral(cos(pi/2 * t**2), t=0..z)\n\nParameters\n----------\nz : float or complex array_like\n    Argument\n\nReturns\n-------\nssa, csa\n    Fresnel sin and cos integral values";
static char __pyx_k_gamma_z_Gamma_function_The_gamma[] = "gamma(z)\n\nGamma function\n\nThe gamma function is often referred to as the generalized\nfactorial since ``z*gamma(z) = gamma(z+1)`` and ``gamma(n+1) =\nn!`` for natural number *n*.";
static char __pyx_k_gammainc_a_x_Incomplete_gamma_fu[] = "gammainc(a, x)\n\nIncomplete gamma function\n\nDefined as::\n\n    1 / gamma(a) * integral(exp(-t) * t**(a-1), t=0..x)\n\n`a` must be positive and `x` must be >= 0.";
static char __pyx_k_gammaincc_a_x_Complemented_incom[] = "gammaincc(a, x)\n\nComplemented incomplete gamma integral\n\nDefined as::\n\n    1 / gamma(a) * integral(exp(-t) * t**(a-1), t=x..inf) = 1 - gammainc(a, x)\n\n`a` must be positive and `x` must be >= 0.";
static char __pyx_k_gammainccinv_a_y_Inverse_to_gamm[] = "gammainccinv(a, y)\n\nInverse to `gammaincc`\n\nReturns `x` such that ``gammaincc(a, x) == y``.";
static char __pyx_k_gammaincinv_a_y_Inverse_to_gamma[] = "gammaincinv(a, y)\n\nInverse to `gammainc`\n\nReturns `x` such that ``gammainc(a, x) = y``.";
static char __pyx_k_gammasgn_x_Sign_of_the_gamma_fun[] = "gammasgn(x)\n\nSign of the gamma function.\n\nSee Also\n--------\ngammaln";
static char __pyx_k_gdtr_a_b_x_Gamma_distribution_cu[] = "gdtr(a, b, x)\n\nGamma distribution cumulative density function.\n\nReturns the integral from zero to `x` of the gamma probability\ndensity function::\n\n    a**b / gamma(b) * integral(t**(b-1) exp(-at), t=0..x).\n\nThe arguments `a` and `b` are used differently here than in other\ndefinitions.";
static char __pyx_k_gdtrc_a_b_x_Gamma_distribution_s[] = "gdtrc(a, b, x)\n\nGamma distribution survival function.\n\nIntegral from `x` to infinity of the gamma probability density\nfunction.\n\nSee Also\n--------\ngdtr, gdtri";
static char __pyx_k_gdtria_p_b_x_out_None_Inverse_of[] = "gdtria(p, b, x, out=None)\n\nInverse of `gdtr` vs a.\n\nReturns the inverse with respect to the parameter `a` of ``p =\ngdtr(a, b, x)``, the cumulative distribution function of the gamma\ndistribution.\n\nParameters\n----------\np : array_like\n    Probability values.\nb : array_like\n    `b` parameter values of `gdtr(a, b, x)`.  `b` is the \"shape\" parameter\n    of the gamma distribution.\nx : array_like\n    Nonnegative real values, from the domain of the gamma distribution.\nout : ndarray, optional\n    If a fourth argument is given, it must be a numpy.ndarray whose size\n    matches the broadcast result of `a`, `b` and `x`.  `out` is then the\n    array returned by the function.\n\nReturns\n-------\na : ndarray\n    Values of the `a` parameter such that `p = gdtr(a, b, x)`.  `1/a`\n    is the \"scale\" parameter of the gamma distribution.\n\nSee Also\n--------\ngdtr : CDF of the gamma distribution.\ngdtrib : Inverse with respect to `b` of `gdtr(a, b, x)`.\ngdtrix : Inverse with respect to `x` of `gdtr(a, b, x)`.\n\nExamples\n--------\nFirst evaluate `gdtr`.\n\n>>> from scipy.special import gdtr, gdtria\n>>> p = gdtr(1.2, 3.4, 5.6)\n>>> print(p)\n0.94378087442\n\nVerify the inverse.\n\n>>> gdtria(p, 3.4, 5.6)\n1.2";
static char __pyx_k_gdtrib_a_p_x_out_None_Inverse_of[] = "gdtrib(a, p, x, out=None)\n\nInverse of `gdtr` vs b.\n\nReturns the inverse with respect to the parameter `b` of ``p =\ngdtr(a, b, x)``, the cumulative distribution function of the gamma\ndistribution.\n\nParameters\n----------\na : array_like\n    `a` parameter values of `gdtr(a, b, x)`. `1/a` is the \"scale\"\n    parameter of the gamma distribution.\np : array_like\n    Probability values.\nx : array_like\n    Nonnegative real values, from the domain of the gamma distribution.\nout : ndarray, optional\n    If a fourth argument is given, it must be a numpy.ndarray whose size\n    matches the broadcast result of `a`, `b` and `x`.  `out` is then the\n    array returned by the function.\n\nReturns\n-------\nb : ndarray\n    Values of the `b` parameter such that `p = gdtr(a, b, x)`.  `b` is\n    the \"shape\" parameter of the gamma distribution.\n\nSee Also\n--------\ngdtr : CDF of the gamma distribution.\ngdtria : Inverse with respect to `a` of `gdtr(a, b, x)`.\ngdtrix : Inverse with respect to `x` of `gdtr(a, b, x)`.\n\nExamples\n--------\nFirst evaluate `gdtr`.\n\n>>> from scipy.special import gdtr, gdtrib\n>>> p = gdtr(1.2, 3.4, 5.6)\n>>> print(p)\n0.94378087442\n\nVerify the inverse.\n\n>>> gdtrib(1.2, p, 5.6)\n3.3999999999723882";
static char __pyx_k_gdtrix_a_b_p_out_None_Inverse_of[] = "gdtrix(a, b, p, out=None)\n\nInverse of `gdtr` vs x.\n\nReturns the inverse with respect to the parameter `x` of ``p =\ngdtr(a, b, x)``, the cumulative distribution function of the gamma\ndistribution. This is also known as the p'th quantile of the\ndistribution.\n\nParameters\n----------\na : array_like\n    `a` parameter values of `gdtr(a, b, x)`.  `1/a` is the \"scale\"\n    parameter of the gamma distribution.\nb : array_like\n    `b` parameter values of `gdtr(a, b, x)`.  `b` is the \"shape\" parameter\n    of the gamma distribution.\np : array_like\n    Probability values.\nout : ndarray, optional\n    If a fourth argument is given, it must be a numpy.ndarray whose size\n    matches the broadcast result of `a`, `b` and `x`.  `out` is then the\n    array returned by the function.\n\nReturns\n-------\nx : ndarray\n    Values of the `x` parameter such that `p = gdtr(a, b, x)`.\n\nSee Also\n--------\ngdtr : CDF of the gamma distribution.\ngdtria : Inverse with respect to `a` of `gdtr(a, b, x)`.\ngdtrib : Inverse with respect to `b` of `gdtr(a, b, x)`.\n\nExamples\n--------\nFirst evaluate `gdtr`.\n\n>>> from scipy.special import gdtr, gdtrix\n>>> p = gdtr(1.2, 3.4, 5.6)\n>>> print(p)\n0.94378087442\n\nVerify the inverse.\n\n>>> gdtrix(1.2, 3.4, p)\n5.5999999999999996";
static char __pyx_k_hankel1_v_z_Hankel_function_of_t[] = "hankel1(v, z)\n\nHankel function of the first kind\n\nParameters\n----------\nv : float\n    Order\nz : float or complex\n    Argument";
static char __pyx_k_hankel1e_v_z_Exponentially_scale[] = "hankel1e(v, z)\n\nExponentially scaled Hankel function of the first kind\n\nDefined as::\n\n    hankel1e(v, z) = hankel1(v, z) * exp(-1j * z)\n\nParameters\n----------\nv : float\n    Order\nz : complex\n    Argument";
static char __pyx_k_hankel2_v_z_Hankel_function_of_t[] = "hankel2(v, z)\n\nHankel function of the second kind\n\nParameters\n----------\nv : float\n    Order\nz : complex\n    Argument";
static char __pyx_k_hankel2e_v_z_Exponentially_scale[] = "hankel2e(v, z)\n\nExponentially scaled Hankel function of the second kind\n\nDefined as::\n\n    hankel1e(v, z) = hankel1(v, z) * exp(1j * z)\n\nParameters\n----------\nv : float\n    Order\nz : complex\n    Argument";
static char __pyx_k_huber_delta_r_Huber_loss_functio[] = "huber(delta, r)\n\nHuber loss function.\n\n.. math:: \\text{huber}(\\delta, r) = \\begin{cases} \\infty & \\delta < 0  \\\\ \\frac{1}{2}r^2 & 0 \\le \\delta, | r | \\le \\delta \\\\ \\delta ( |r| - \\frac{1}{2}\\delta ) & \\text{otherwise} \\end{cases}\n\nParameters\n----------\ndelta : ndarray\n    Input array, indicating the quadratic vs. linear loss changepoint.\nr : ndarray\n    Input array, possibly representing residuals.\n\nReturns\n-------\nres : ndarray\n    The computed Huber loss function values.\n\nNotes\n-----\nThis function is convex in r.\n\n.. versionadded:: 0.15.0";
static char __pyx_k_hyp1f1_a_b_x_Confluent_hypergeom[] = "hyp1f1(a, b, x)\n\nConfluent hypergeometric function 1F1(a, b; x)";
static char __pyx_k_hyp1f2_a_b_c_x_Hypergeometric_fu[] = "hyp1f2(a, b, c, x)\n\nHypergeometric function 1F2 and error estimate\n\nReturns\n-------\ny\n    Value of the function\nerr\n    Error estimate";
static char __pyx_k_hyp2f0_a_b_x_type_Hypergeometric[] = "hyp2f0(a, b, x, type)\n\nHypergeometric function 2F0 in y and an error estimate\n\nThe parameter `type` determines a convergence factor and can be\neither 1 or 2.\n\nReturns\n-------\ny\n    Value of the function\nerr\n    Error estimate";
static char __pyx_k_hyp2f1_a_b_c_z_Gauss_hypergeomet[] = "hyp2f1(a, b, c, z)\n\nGauss hypergeometric function 2F1(a, b; c; z).";
static char __pyx_k_hyp3f0_a_b_c_x_Hypergeometric_fu[] = "hyp3f0(a, b, c, x)\n\nHypergeometric function 3F0 in y and an error estimate\n\nReturns\n-------\ny\n    Value of the function\nerr\n    Error estimate";
static char __pyx_k_hyperu_a_b_x_Confluent_hypergeom[] = "hyperu(a, b, x)\n\nConfluent hypergeometric function U(a, b, x) of the second kind";
static char __pyx_k_i0_x_Modified_Bessel_function_of[] = "i0(x)\n\nModified Bessel function of order 0";
static char __pyx_k_i0e_x_Exponentially_scaled_modif[] = "i0e(x)\n\nExponentially scaled modified Bessel function of order 0.\n\nDefined as::\n\n    i0e(x) = exp(-abs(x)) * i0(x).";
static char __pyx_k_i1_x_Modified_Bessel_function_of[] = "i1(x)\n\nModified Bessel function of order 1";
static char __pyx_k_i1e_x_Exponentially_scaled_modif[] = "i1e(x)\n\nExponentially scaled modified Bessel function of order 1.\n\nDefined as::\n\n    i1e(x) = exp(-abs(x)) * i1(x)";
static char __pyx_k_inv_boxcox1p_y_lmbda_Compute_the[] = "inv_boxcox1p(y, lmbda)\n\nCompute the inverse of the Box-Cox transformation.\n\nFind ``x`` such that::\n\n    y = ((1+x)**lmbda - 1) / lmbda  if lmbda != 0\n        log(1+x)                    if lmbda == 0\n\nParameters\n----------\ny : array_like\n    Data to be transformed.\nlmbda : array_like\n    Power parameter of the Box-Cox transform.\n\nReturns\n-------\nx : array\n    Transformed data.\n\nNotes\n-----\n\n.. versionadded:: 0.16.0\n\nExamples\n--------\n>>> from scipy.special import boxcox1p, inv_boxcox1p\n>>> y = boxcox1p([1, 4, 10], 2.5)\n>>> inv_boxcox1p(y, 2.5)\narray([1., 4., 10.])";
static char __pyx_k_inv_boxcox_y_lmbda_Compute_the_i[] = "inv_boxcox(y, lmbda)\n\nCompute the inverse of the Box-Cox transformation.\n\nFind ``x`` such that::\n\n    y = (x**lmbda - 1) / lmbda  if lmbda != 0\n        log(x)                  if lmbda == 0\n\nParameters\n----------\ny : array_like\n    Data to be transformed.\nlmbda : array_like\n    Power parameter of the Box-Cox transform.\n\nReturns\n-------\nx : array\n    Transformed data.\n\nNotes\n-----\n\n.. versionadded:: 0.16.0\n\nExamples\n--------\n>>> from scipy.special import boxcox, inv_boxcox\n>>> y = boxcox([1, 4, 10], 2.5)\n>>> inv_boxcox(y, 2.5)\narray([1., 4., 10.])";
static char __pyx_k_it2i0k0_x_Integrals_related_to_m[] = "it2i0k0(x)\n\nIntegrals related to modified Bessel functions of order 0\n\nReturns\n-------\nii0\n    ``integral((i0(t)-1)/t, t=0..x)``\nik0\n    ``int(k0(t)/t, t=x..inf)``";
static char __pyx_k_it2j0y0_x_Integrals_related_to_B[] = "it2j0y0(x)\n\nIntegrals related to Bessel functions of order 0\n\nReturns\n-------\nij0\n    ``integral((1-j0(t))/t, t=0..x)``\niy0\n    ``integral(y0(t)/t, t=x..inf)``";
static char __pyx_k_it2struve0_x_Integral_related_to[] = "it2struve0(x)\n\nIntegral related to Struve function of order 0\n\nReturns\n-------\ni\n    ``integral(H0(t)/t, t=x..inf)``";
static char __pyx_k_itairy_x_Integrals_of_Airy_funct[] = "itairy(x)\n\nIntegrals of Airy functions\n\nCalculates the integral of Airy functions from 0 to `x`\n\nReturns\n-------\nApt, Bpt\n    Integrals for positive arguments\nAnt, Bnt\n    Integrals for negative arguments";
static char __pyx_k_iteration_failed_to_converge_g_g[] = "iteration failed to converge: %g + %gj";
static char __pyx_k_iti0k0_x_Integrals_of_modified_B[] = "iti0k0(x)\n\nIntegrals of modified Bessel functions of order 0\n\nReturns simple integrals from 0 to `x` of the zeroth order modified\nBessel functions `i0` and `k0`.\n\nReturns\n-------\nii0, ik0";
static char __pyx_k_itj0y0_x_Integrals_of_Bessel_fun[] = "itj0y0(x)\n\nIntegrals of Bessel functions of order 0\n\nReturns simple integrals from 0 to `x` of the zeroth order Bessel\nfunctions `j0` and `y0`.\n\nReturns\n-------\nij0, iy0";
static char __pyx_k_itmodstruve0_x_Integral_of_the_m[] = "itmodstruve0(x)\n\nIntegral of the modified Struve function of order 0\n\nReturns\n-------\ni\n    ``integral(L0(t), t=0..x)``";
static char __pyx_k_itstruve0_x_Integral_of_the_Stru[] = "itstruve0(x)\n\nIntegral of the Struve function of order 0\n\nReturns\n-------\ni\n    ``integral(H0(t), t=0..x)``";
static char __pyx_k_ive_v_z_Exponentially_scaled_mod[] = "ive(v, z)\n\nExponentially scaled modified Bessel function of the first kind\n\nDefined as::\n\n    ive(v, z) = iv(v, z) * exp(-abs(z.real))";
static char __pyx_k_j0_x_Bessel_function_the_first_k[] = "j0(x)\n\nBessel function the first kind of order 0";
static char __pyx_k_j1_x_Bessel_function_of_the_firs[] = "j1(x)\n\nBessel function of the first kind of order 1";
static char __pyx_k_jv_v_z_Bessel_function_of_the_fi[] = "jv(v, z)\n\nBessel function of the first kind of real order `v`";
static char __pyx_k_jve_v_z_Exponentially_scaled_Bes[] = "jve(v, z)\n\nExponentially scaled Bessel function of order `v`\n\nDefined as::\n\n    jve(v, z) = jv(v, z) * exp(-abs(z.imag))";
static char __pyx_k_k0e_x_Exponentially_scaled_modif[] = "k0e(x)\n\nExponentially scaled modified Bessel function K of order 0\n\nDefined as::\n\n    k0e(x) = exp(x) * k0(x).";
static char __pyx_k_k1e_x_Exponentially_scaled_modif[] = "k1e(x)\n\nExponentially scaled modified Bessel function K of order 1\n\nDefined as::\n\n    k1e(x) = exp(x) * k1(x)";
static char __pyx_k_kelvin_x_Kelvin_functions_as_com[] = "kelvin(x)\n\nKelvin functions as complex numbers\n\nReturns\n-------\nBe, Ke, Bep, Kep\n    The tuple (Be, Ke, Bep, Kep) contains complex numbers\n    representing the real and imaginary Kelvin functions and their\n    derivatives evaluated at `x`.  For example, kelvin(x)[0].real =\n    ber x and kelvin(x)[0].imag = bei x with similar relationships\n    for ker and kei.";
static char __pyx_k_kolmogi_p_Inverse_function_to_ko[] = "kolmogi(p)\n\nInverse function to kolmogorov\n\nReturns y such that ``kolmogorov(y) == p``.";
static char __pyx_k_kolmogorov_y_Complementary_cumul[] = "kolmogorov(y)\n\nComplementary cumulative distribution function of Kolmogorov distribution\n\nReturns the complementary cumulative distribution function of\nKolmogorov's limiting distribution (Kn* for large n) of a\ntwo-sided test for equality between an empirical and a theoretical\ndistribution. It is equal to the (limit as n->infinity of the)\nprobability that sqrt(n) * max absolute deviation > y.";
static char __pyx_k_kve_v_z_Exponentially_scaled_mod[] = "kve(v, z)\n\nExponentially scaled modified Bessel function of the second kind.\n\nReturns the exponentially scaled, modified Bessel function of the\nsecond kind (sometimes called the third kind) for real order `v` at\ncomplex `z`::\n\n    kve(v, z) = kv(v, z) * exp(z)";
static char __pyx_k_log1p_x_Calculates_log_1_x_for_u[] = "log1p(x)\n\nCalculates log(1+x) for use when `x` is near zero";
static char __pyx_k_log_ndtr_x_Logarithm_of_Gaussian[] = "log_ndtr(x)\n\nLogarithm of Gaussian cumulative distribution function\n\nReturns the log of the area under the standard Gaussian probability\ndensity function, integrated from minus infinity to `x`::\n\n    log(1/sqrt(2*pi) * integral(exp(-t**2 / 2), t=-inf..x))";
static char __pyx_k_logit_x_Logit_ufunc_for_ndarrays[] = "logit(x)\n\nLogit ufunc for ndarrays.\n\nThe logit function is defined as logit(p) = log(p/(1-p)).\nNote that logit(0) = -inf, logit(1) = inf, and logit(p)\nfor p<0 or p>1 yields nan.\n\nParameters\n----------\nx : ndarray\n    The ndarray to apply logit to element-wise.\n\nReturns\n-------\nout : ndarray\n    An ndarray of the same shape as x. Its entries\n    are logit of the corresponding entry of x.\n\nNotes\n-----\nAs a ufunc logit takes a number of optional\nkeyword arguments. For more information\nsee `ufuncs <http://docs.scipy.org/doc/numpy/reference/ufuncs.html>`_\n\n.. versionadded:: 0.10.0";
static char __pyx_k_lpmv_m_v_x_Associated_legendre_f[] = "lpmv(m, v, x)\n\nAssociated legendre function of integer order.\n\nParameters\n----------\nm : int\n    Order\nv : float\n    Degree.\nx : float\n    Argument. Must be ``|x| <= 1``.\n\nReturns\n-------\nres : float\n    The value of the function.\n\nSee Also\n--------\nlpmn : Similar, but computes values for all orders 0..m and degrees 0..n.\nclpmn : Similar to `lpmn` but allows a complex argument.\n\nNotes\n-----\nIt is possible to extend the domain of this function to all\ncomplex m, v, x, but this is not yet implemented.";
static char __pyx_k_mathieu_a_m_q_Characteristic_val[] = "mathieu_a(m, q)\n\nCharacteristic value of even Mathieu functions\n\nReturns the characteristic value for the even solution,\n``ce_m(z, q)``, of Mathieu's equation.";
static char __pyx_k_mathieu_b_m_q_Characteristic_val[] = "mathieu_b(m, q)\n\nCharacteristic value of odd Mathieu functions\n\nReturns the characteristic value for the odd solution,\n``se_m(z, q)``, of Mathieu's equation.";
static char __pyx_k_mathieu_cem_m_q_x_Even_Mathieu_f[] = "mathieu_cem(m, q, x)\n\nEven Mathieu function and its derivative\n\nReturns the even Mathieu function, ``ce_m(x, q)``, of order `m` and\nparameter `q` evaluated at `x` (given in degrees).  Also returns the\nderivative with respect to `x` of ce_m(x, q)\n\nParameters\n----------\nm\n    Order of the function\nq\n    Parameter of the function\nx\n    Argument of the function, *given in degrees, not radians*\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x";
static char __pyx_k_mathieu_modcem1_m_q_x_Even_modif[] = "mathieu_modcem1(m, q, x)\n\nEven modified Mathieu function of the first kind and its derivative\n\nEvaluates the even modified Mathieu function of the first kind,\n``Mc1m(x, q)``, and its derivative at `x` for order `m` and parameter\n`q`.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x";
static char __pyx_k_mathieu_modcem2_m_q_x_Even_modif[] = "mathieu_modcem2(m, q, x)\n\nEven modified Mathieu function of the second kind and its derivative\n\nEvaluates the even modified Mathieu function of the second kind,\nMc2m(x, q), and its derivative at `x` (given in degrees) for order `m`\nand parameter `q`.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x";
static char __pyx_k_mathieu_modsem1_m_q_x_Odd_modifi[] = "mathieu_modsem1(m, q, x)\n\nOdd modified Mathieu function of the first kind and its derivative\n\nEvaluates the odd modified Mathieu function of the first kind,\nMs1m(x, q), and its derivative at `x` (given in degrees) for order `m`\nand parameter `q`.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x";
static char __pyx_k_mathieu_modsem2_m_q_x_Odd_modifi[] = "mathieu_modsem2(m, q, x)\n\nOdd modified Mathieu function of the second kind and its derivative\n\nEvaluates the odd modified Mathieu function of the second kind,\nMs2m(x, q), and its derivative at `x` (given in degrees) for order `m`\nand parameter q.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x";
static char __pyx_k_mathieu_sem_m_q_x_Odd_Mathieu_fu[] = "mathieu_sem(m, q, x)\n\nOdd Mathieu function and its derivative\n\nReturns the odd Mathieu function, se_m(x, q), of order `m` and\nparameter `q` evaluated at `x` (given in degrees).  Also returns the\nderivative with respect to `x` of se_m(x, q).\n\nParameters\n----------\nm\n    Order of the function\nq\n    Parameter of the function\nx\n    Argument of the function, *given in degrees, not radians*.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x";
static char __pyx_k_modfresnelm_x_Modified_Fresnel_n[] = "modfresnelm(x)\n\nModified Fresnel negative integrals\n\nReturns\n-------\nfm\n    Integral ``F_-(x)``: ``integral(exp(-1j*t*t), t=x..inf)``\nkm\n    Integral ``K_-(x)``: ``1/sqrt(pi)*exp(1j*(x*x+pi/4))*fp``";
static char __pyx_k_modfresnelp_x_Modified_Fresnel_p[] = "modfresnelp(x)\n\nModified Fresnel positive integrals\n\nReturns\n-------\nfp\n    Integral ``F_+(x)``: ``integral(exp(1j*t*t), t=x..inf)``\nkp\n    Integral ``K_+(x)``: ``1/sqrt(pi)*exp(-1j*(x*x+pi/4))*fp``";
static char __pyx_k_modstruve_v_x_Modified_Struve_fu[] = "modstruve(v, x)\n\nModified Struve function\n\nReturns the modified Struve function Lv(x) of order `v` at `x`, `x` must\nbe positive unless `v` is an integer.";
static char __pyx_k_nbdtr_k_n_p_Negative_binomial_cu[] = "nbdtr(k, n, p)\n\nNegative binomial cumulative distribution function\n\nReturns the sum of the terms 0 through `k` of the negative binomial\ndistribution::\n\n    sum((n+j-1)Cj p**n (1-p)**j, j=0..k).\n\nIn a sequence of Bernoulli trials this is the probability that k\nor fewer failures precede the nth success.";
static char __pyx_k_nbdtrc_k_n_p_Negative_binomial_s[] = "nbdtrc(k, n, p)\n\nNegative binomial survival function\n\nReturns the sum of the terms k+1 to infinity of the negative\nbinomial distribution.";
static char __pyx_k_nbdtri_k_n_y_Inverse_of_nbdtr_vs[] = "nbdtri(k, n, y)\n\nInverse of `nbdtr` vs `p`\n\nFinds the argument p such that ``nbdtr(k, n, p) = y``.";
static char __pyx_k_nbdtrik_y_n_p_Inverse_of_nbdtr_v[] = "nbdtrik(y, n, p)\n\nInverse of `nbdtr` vs `k`\n\nFinds the argument k such that ``nbdtr(k, n, p) = y``.";
static char __pyx_k_nbdtrin_k_y_p_Inverse_of_nbdtr_v[] = "nbdtrin(k, y, p)\n\nInverse of `nbdtr` vs `n`\n\nFinds the argument `n` such that ``nbdtr(k, n, p) = y``.";
static char __pyx_k_ncfdtr_dfn_dfd_nc_f_Cumulative_d[] = "ncfdtr(dfn, dfd, nc, f)\n\nCumulative distribution function of the non-central F distribution.\n\nParameters\n----------\ndfn : array_like\n    Degrees of freedom of the numerator sum of squares.  Range (0, inf).\ndfd : array_like\n    Degrees of freedom of the denominator sum of squares.  Range (0, inf).\nnc : array_like\n    Noncentrality parameter.  Should be in range (0, 1e4).\nf : array_like\n    Quantiles, i.e. the upper limit of integration.\n\nReturns\n-------\ncdf : float or ndarray\n    The calculated CDF.  If all inputs are scalar, the return will be a\n    float.  Otherwise it will be an array.\n\nSee Also\n--------\nncdfdtri : Inverse CDF (iCDF) of the non-central F distribution.\nncdfdtridfd : Calculate dfd, given CDF and iCDF values.\nncdfdtridfn : Calculate dfn, given CDF and iCDF values.\nncdfdtrinc : Calculate noncentrality parameter, given CDF, iCDF, dfn, dfd.\n\nExamples\n--------\n>>> from scipy import special\n>>> from scipy import stats\n>>> import matplotlib.pyplot as plt\n\nPlot the CDF of the non-central F distribution, for nc=0.  Compare with the\nF-distribution from scipy.stats:\n\n>>> x = np.linspace(-1, 8, num=500)\n>>> dfn = 3\n>>> dfd = 2\n>>> ncf_stats = stats.f.cdf(x, dfn, dfd)\n>>> ncf_special = special.ncfdtr(dfn, dfd, 0, x)\n\n>>> fig = plt.figure()\n>>> ax = fig.add_subplot(111)\n>>> ax.plot(x, ncf_stats, 'b-', lw=3)\n>>> ax.plot(x, ncf_special, 'r-')\n>>> plt.show()";
static char __pyx_k_ncfdtri_p_dfn_dfd_nc_Inverse_cum[] = "ncfdtri(p, dfn, dfd, nc)\n\nInverse cumulative distribution function of the non-central F distribution.\n\nSee `ncfdtr` for more details.";
static char __pyx_k_nctdtr_df_nc_t_Cumulative_distri[] = "nctdtr(df, nc, t)\n\nCumulative distribution function of the non-central `t` distribution.\n\nParameters\n----------\ndf : array_like\n    Degrees of freedom of the distribution.  Should be in range (0, inf).\nnc : array_like\n    Noncentrality parameter.  Should be in range (-1e6, 1e6).\nt : array_like\n    Quantiles, i.e. the upper limit of integration.\n\nReturns\n-------\ncdf : float or ndarray\n    The calculated CDF.  If all inputs are scalar, the return will be a\n    float.  Otherwise it will be an array.\n\nSee Also\n--------\nnctdtrit : Inverse CDF (iCDF) of the non-central t distribution.\nnctdtridf : Calculate degrees of freedom, given CDF and iCDF values.\nnctdtrinc : Calculate non-centrality parameter, given CDF iCDF values.\n\nExamples\n--------\n>>> from scipy import special\n>>> from scipy import stats\n>>> import matplotlib.pyplot as plt\n\nPlot the CDF of the non-central t distribution, for nc=0.  Compare with the\nt-distribution from scipy.stats:\n\n>>> x = np.linspace(-5, 5, num=500)\n>>> df = 3\n>>> nct_stats = stats.t.cdf(x, df)\n>>> nct_special = special.nctdtr(df, 0, x)\n\n>>> fig = plt.figure()\n>>> ax = fig.add_subplot(111)\n>>> ax.plot(x, nct_stats, 'b-', lw=3)\n>>> ax.plot(x, nct_special, 'r-')\n>>> plt.show()";
static char __pyx_k_nctdtridf_p_nc_t_Calculate_degre[] = "nctdtridf(p, nc, t)\n\nCalculate degrees of freedom for non-central t distribution.\n\nSee `nctdtr` for more details.\n\nParameters\n----------\np : array_like\n    CDF values, in range (0, 1].\nnc : array_like\n    Noncentrality parameter.  Should be in range (-1e6, 1e6).\nt : array_like\n    Quantiles, i.e. the upper limit of integration.";
static char __pyx_k_nctdtrinc_df_p_t_Calculate_non_c[] = "nctdtrinc(df, p, t)\n\nCalculate non-centrality parameter for non-central t distribution.\n\nSee `nctdtr` for more details.\n\nParameters\n----------\ndf : array_like\n    Degrees of freedom of the distribution.  Should be in range (0, inf).\np : array_like\n    CDF values, in range (0, 1].\nt : array_like\n    Quantiles, i.e. the upper limit of integration.";
static char __pyx_k_nctdtrit_df_nc_p_Inverse_cumulat[] = "nctdtrit(df, nc, p)\n\nInverse cumulative distribution function of the non-central t distribution.\n\nSee `nctdtr` for more details.\n\nParameters\n----------\ndf : array_like\n    Degrees of freedom of the distribution.  Should be in range (0, inf).\nnc : array_like\n    Noncentrality parameter.  Should be in range (-1e6, 1e6).\np : array_like\n    CDF values, in range (0, 1].";
static char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static char __pyx_k_ndtr_x_Gaussian_cumulative_distr[] = "ndtr(x)\n\nGaussian cumulative distribution function\n\nReturns the area under the standard Gaussian probability\ndensity function, integrated from minus infinity to `x`::\n\n    1/sqrt(2*pi) * integral(exp(-t**2 / 2), t=-inf..x)";
static char __pyx_k_ndtri_y_Inverse_of_ndtr_vs_x_Ret[] = "ndtri(y)\n\nInverse of `ndtr` vs x\n\nReturns the argument x for which the area under the Gaussian\nprobability density function (integrated from minus infinity to `x`)\nis equal to y.";
static char __pyx_k_nrdtrisd_p_x_mn_Calculate_standa[] = "nrdtrisd(p, x, mn)\n\nCalculate standard deviation of normal distribution given other params.\n\nParameters\n----------\np : array_like\n    CDF values, in range (0, 1].\nx : array_like\n    Quantiles, i.e. the upper limit of integration.\nmn : float or ndarray\n    The mean of the normal distribution.\n\nReturns\n-------\nstd : array_like\n    Standard deviation.\n\nSee Also\n--------\nnrdtristd, ndtr";
static char __pyx_k_obl_ang1_cv_m_n_c_cv_x_Oblate_sp[] = "obl_ang1_cv(m, n, c, cv, x)\n\nOblate spheroidal angular function obl_ang1 for precomputed characteristic value\n\nComputes the oblate spheroidal angular function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_obl_ang1_m_n_c_x_Oblate_spheroid[] = "obl_ang1(m, n, c, x)\n\nOblate spheroidal angular function of the first kind and its derivative\n\nComputes the oblate spheroidal angular function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_obl_cv_m_n_c_Characteristic_valu[] = "obl_cv(m, n, c)\n\nCharacteristic value of oblate spheroidal function\n\nComputes the characteristic value of oblate spheroidal wave\nfunctions of order `m`, `n` (n>=m) and spheroidal parameter `c`.";
static char __pyx_k_obl_rad1_cv_m_n_c_cv_x_Oblate_sp[] = "obl_rad1_cv(m, n, c, cv, x)\n\nOblate spheroidal radial function obl_rad1 for precomputed characteristic value\n\nComputes the oblate spheroidal radial function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_obl_rad1_m_n_c_x_Oblate_spheroid[] = "obl_rad1(m, n, c, x)\n\nOblate spheroidal radial function of the first kind and its derivative\n\nComputes the oblate spheroidal radial function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_obl_rad2_cv_m_n_c_cv_x_Oblate_sp[] = "obl_rad2_cv(m, n, c, cv, x)\n\nOblate spheroidal radial function obl_rad2 for precomputed characteristic value\n\nComputes the oblate spheroidal radial function of the second kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_obl_rad2_m_n_c_x_Oblate_spheroid[] = "obl_rad2(m, n, c, x)\n\nOblate spheroidal radial function of the second kind and its derivative.\n\nComputes the oblate spheroidal radial function of the second kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_pbdv_v_x_Parabolic_cylinder_func[] = "pbdv(v, x)\n\nParabolic cylinder function D\n\nReturns (d, dp) the parabolic cylinder function Dv(x) in d and the\nderivative, Dv'(x) in dp.\n\nReturns\n-------\nd\n    Value of the function\ndp\n    Value of the derivative vs x";
static char __pyx_k_pbvv_v_x_Parabolic_cylinder_func[] = "pbvv(v, x)\n\nParabolic cylinder function V\n\nReturns the parabolic cylinder function Vv(x) in v and the\nderivative, Vv'(x) in vp.\n\nReturns\n-------\nv\n    Value of the function\nvp\n    Value of the derivative vs x";
static char __pyx_k_pbwa_a_x_Parabolic_cylinder_func[] = "pbwa(a, x)\n\nParabolic cylinder function W\n\nReturns the parabolic cylinder function W(a, x) in w and the\nderivative, W'(a, x) in wp.\n\n.. warning::\n\n   May not be accurate for large (>5) arguments in a and/or x.\n\nReturns\n-------\nw\n    Value of the function\nwp\n    Value of the derivative vs x";
static char __pyx_k_pdtr_k_m_Poisson_cumulative_dist[] = "pdtr(k, m)\n\nPoisson cumulative distribution function\n\nReturns the sum of the first `k` terms of the Poisson distribution:\nsum(exp(-m) * m**j / j!, j=0..k) = gammaincc( k+1, m).  Arguments\nmust both be positive and `k` an integer.";
static char __pyx_k_pdtrc_k_m_Poisson_survival_funct[] = "pdtrc(k, m)\n\nPoisson survival function\n\nReturns the sum of the terms from k+1 to infinity of the Poisson\ndistribution: sum(exp(-m) * m**j / j!, j=k+1..inf) = gammainc(\nk+1, m).  Arguments must both be positive and `k` an integer.";
static char __pyx_k_pdtri_k_y_Inverse_to_pdtr_vs_m_R[] = "pdtri(k, y)\n\nInverse to `pdtr` vs m\n\nReturns the Poisson variable `m` such that the sum from 0 to `k` of\nthe Poisson density is equal to the given probability `y`:\ncalculated by gammaincinv(k+1, y). `k` must be a nonnegative\ninteger and `y` between 0 and 1.";
static char __pyx_k_poch_z_m_Rising_factorial_z__m_T[] = "poch(z, m)\n\nRising factorial (z)_m\n\nThe Pochhammer symbol (rising factorial), is defined as::\n\n    (z)_m = gamma(z + m) / gamma(z)\n\nFor positive integer `m` it reads::\n\n    (z)_m = z * (z + 1) * ... * (z + m - 1)";
static char __pyx_k_polynomial_defined_only_for_alph[] = "polynomial defined only for alpha > -1";
static char __pyx_k_pro_ang1_cv_m_n_c_cv_x_Prolate_s[] = "pro_ang1_cv(m, n, c, cv, x)\n\nProlate spheroidal angular function pro_ang1 for precomputed characteristic value\n\nComputes the prolate spheroidal angular function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_pro_ang1_m_n_c_x_Prolate_spheroi[] = "pro_ang1(m, n, c, x)\n\nProlate spheroidal angular function of the first kind and its derivative\n\nComputes the prolate spheroidal angular function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_pro_cv_m_n_c_Characteristic_valu[] = "pro_cv(m, n, c)\n\nCharacteristic value of prolate spheroidal function\n\nComputes the characteristic value of prolate spheroidal wave\nfunctions of order `m`, `n` (n>=m) and spheroidal parameter `c`.";
static char __pyx_k_pro_rad1_cv_m_n_c_cv_x_Prolate_s[] = "pro_rad1_cv(m, n, c, cv, x)\n\nProlate spheroidal radial function pro_rad1 for precomputed characteristic value\n\nComputes the prolate spheroidal radial function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_pro_rad1_m_n_c_x_Prolate_spheroi[] = "pro_rad1(m, n, c, x)\n\nProlate spheroidal radial function of the first kind and its derivative\n\nComputes the prolate spheroidal radial function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_pro_rad2_cv_m_n_c_cv_x_Prolate_s[] = "pro_rad2_cv(m, n, c, cv, x)\n\nProlate spheroidal radial function pro_rad2 for precomputed characteristic value\n\nComputes the prolate spheroidal radial function of the second kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_pro_rad2_m_n_c_x_Prolate_spheroi[] = "pro_rad2(m, n, c, x)\n\nProlate spheroidal radial function of the secon kind and its derivative\n\nComputes the prolate spheroidal radial function of the second kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x";
static char __pyx_k_pseudo_huber_delta_r_Pseudo_Hube[] = "pseudo_huber(delta, r)\n\nPseudo-Huber loss function.\n\n.. math:: \\mathrm{pseudo\\_huber}(\\delta, r) = \\delta^2 \\left( \\sqrt{ 1 + \\left( \\frac{r}{\\delta} \\right)^2 } - 1 \\right)\n\nParameters\n----------\ndelta : ndarray\n    Input array, indicating the soft quadratic vs. linear loss changepoint.\nr : ndarray\n    Input array, possibly representing residuals.\n\nReturns\n-------\nres : ndarray\n    The computed Pseudo-Huber loss function values.\n\nNotes\n-----\nThis function is convex in :math:`r`.\n\n.. versionadded:: 0.15.0";
static char __pyx_k_psi_z_Digamma_function_The_deriv[] = "psi(z)\n\nDigamma function\n\nThe derivative of the logarithm of the gamma function evaluated at\n`z` (also called the digamma function).";
static char __pyx_k_radian_d_m_s_Convert_from_degree[] = "radian(d, m, s)\n\nConvert from degrees to radians\n\nReturns the angle given in (d)egrees, (m)inutes, and (s)econds in\nradians.";
static char __pyx_k_rel_entr_x_y_Elementwise_functio[] = "rel_entr(x, y)\n\nElementwise function for computing relative entropy.\n\n.. math:: \\mathrm{rel\\_entr}(x, y) = \\begin{cases} x \\log(x / y) & x > 0, y > 0 \\\\ 0 & x = 0, y \\ge 0 \\\\ \\infty & \\text{otherwise} \\end{cases}\n\nParameters\n----------\nx : ndarray\n    First input array.\ny : ndarray\n    Second input array.\n\nReturns\n-------\nres : ndarray\n    Output array.\n\nSee Also\n--------\nentr, kl_div\n\nNotes\n-----\nThis function is jointly convex in x and y.\n\n.. versionadded:: 0.14.0";
static char __pyx_k_rgamma_z_Gamma_function_inverted[] = "rgamma(z)\n\nGamma function inverted\n\nReturns ``1/gamma(x)``";
static char __pyx_k_round_x_Round_to_nearest_integer[] = "round(x)\n\nRound to nearest integer\n\nReturns the nearest integer to `x` as a double precision floating\npoint result.  If `x` ends in 0.5 exactly, the nearest even integer\nis chosen.";
static char __pyx_k_shichi_x_Hyperbolic_sine_and_cos[] = "shichi(x)\n\nHyperbolic sine and cosine integrals\n\nReturns\n-------\nshi\n    ``integral(sinh(t)/t, t=0..x)``\nchi\n    ``eul + ln x + integral((cosh(t)-1)/t, t=0..x)``\n    where ``eul`` is Euler's constant.";
static char __pyx_k_sici_x_Sine_and_cosine_integrals[] = "sici(x)\n\nSine and cosine integrals\n\nReturns\n-------\nsi\n    ``integral(sin(t)/t, t=0..x)``\nci\n    ``eul + ln x + integral((cos(t) - 1)/t, t=0..x)``\n    where ``eul`` is Euler's constant.";
static char __pyx_k_sindg_x_Sine_of_angle_given_in_d[] = "sindg(x)\n\nSine of angle given in degrees";
static char __pyx_k_smirnov_n_e_Kolmogorov_Smirnov_c[] = "smirnov(n, e)\n\nKolmogorov-Smirnov complementary cumulative distribution function\n\nReturns the exact Kolmogorov-Smirnov complementary cumulative\ndistribution function (Dn+ or Dn-) for a one-sided test of\nequality between an empirical and a theoretical distribution. It\nis equal to the probability that the maximum difference between a\ntheoretical distribution and an empirical one based on `n` samples\nis greater than e.";
static char __pyx_k_spence_x_Dilogarithm_integral_Re[] = "spence(x)\n\nDilogarithm integral\n\nReturns the dilogarithm integral::\n\n    -integral(log t / (t-1), t=1..x)";
static char __pyx_k_sph_harm_m_n_theta_phi_Compute_s[] = "sph_harm(m, n, theta, phi)\n\nCompute spherical harmonics.\n\n.. math:: Y^m_n(\\theta,\\phi) = \\sqrt{\\frac{2n+1}{4\\pi}\\frac{(n-m)!}{(n+m)!}} e^{i m \\theta} P^m_n(\\cos(\\phi))\n\nParameters\n----------\nm : int\n   ``|m| <= n``; the order of the harmonic.\nn : int\n   where `n` >= 0; the degree of the harmonic.  This is often called\n   ``l`` (lower case L) in descriptions of spherical harmonics.\ntheta : float\n   [0, 2*pi]; the azimuthal (longitudinal) coordinate.\nphi : float\n   [0, pi]; the polar (colatitudinal) coordinate.\n\nReturns\n-------\ny_mn : complex float\n   The harmonic :math:`Y^m_n` sampled at `theta` and `phi`\n\nNotes\n-----\nThere are different conventions for the meaning of input arguments\n`theta` and `phi`.  We take `theta` to be the azimuthal angle and\n`phi` to be the polar angle.  It is common to see the opposite\nconvention - that is `theta` as the polar angle and `phi` as the\nazimuthal angle.\n\nReferences\n----------\n.. [1] Digital Library of Mathematical Functions, 14.30. http://dlmf.nist.gov/14.30";
static char __pyx_k_stdtr_df_t_Student_t_distributio[] = "stdtr(df, t)\n\nStudent t distribution cumulative density function\n\nReturns the integral from minus infinity to t of the Student t\ndistribution with df > 0 degrees of freedom::\n\n   gamma((df+1)/2)/(sqrt(df*pi)*gamma(df/2)) *\n   integral((1+x**2/df)**(-df/2-1/2), x=-inf..t)";
static char __pyx_k_stdtridf_p_t_Inverse_of_stdtr_vs[] = "stdtridf(p, t)\n\nInverse of `stdtr` vs df\n\nReturns the argument df such that stdtr(df, t) is equal to `p`.";
static char __pyx_k_stdtrit_df_p_Inverse_of_stdtr_vs[] = "stdtrit(df, p)\n\nInverse of `stdtr` vs `t`\n\nReturns the argument `t` such that stdtr(df, t) is equal to `p`.";
static char __pyx_k_struve_v_x_Struve_function_Compu[] = "struve(v, x)\n\nStruve function\n\nComputes the struve function Hv(x) of order `v` at `x`, `x` must be\npositive unless `v` is an integer.";
static char __pyx_k_tandg_x_Tangent_of_angle_x_given[] = "tandg(x)\n\nTangent of angle x given in degrees.";
static char __pyx_k_tklmbda_x_lmbda_Tukey_Lambda_cum[] = "tklmbda(x, lmbda)\n\nTukey-Lambda cumulative distribution function";
static char __pyx_k_wofz_z_Faddeeva_function_Returns[] = "wofz(z)\n\nFaddeeva function\n\nReturns the value of the Faddeeva function for complex argument::\n\n    exp(-z**2)*erfc(-i*z)\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva";
static char __pyx_k_xlog1py_x_y_Compute_x_log1p_y_so[] = "xlog1py(x, y)\n\nCompute ``x*log1p(y)`` so that the result is 0 if ``x = 0``.\n\nParameters\n----------\nx : array_like\n    Multiplier\ny : array_like\n    Argument\n\nReturns\n-------\nz : array_like\n    Computed x*log1p(y)\n\nNotes\n-----\n\n.. versionadded:: 0.13.0";
static char __pyx_k_xlogy_x_y_Compute_x_log_y_so_tha[] = "xlogy(x, y)\n\nCompute ``x*log(y)`` so that the result is 0 if ``x = 0``.\n\nParameters\n----------\nx : array_like\n    Multiplier\ny : array_like\n    Argument\n\nReturns\n-------\nz : array_like\n    Computed x*log(y)\n\nNotes\n-----\n\n.. versionadded:: 0.13.0";
static char __pyx_k_y0_x_Bessel_function_of_the_seco[] = "y0(x)\n\nBessel function of the second kind of order 0\n\nReturns the Bessel function of the second kind of order 0 at `x`.";
static char __pyx_k_y1_x_Bessel_function_of_the_seco[] = "y1(x)\n\nBessel function of the second kind of order 1\n\nReturns the Bessel function of the second kind of order 1 at `x`.";
static char __pyx_k_yn_n_x_Bessel_function_of_the_se[] = "yn(n, x)\n\nBessel function of the second kind of integer order\n\nReturns the Bessel function of the second kind of integer order `n`\nat `x`.";
static char __pyx_k_yv_v_z_Bessel_function_of_the_se[] = "yv(v, z)\n\nBessel function of the second kind of real order\n\nReturns the Bessel function of the second kind of real order `v` at\ncomplex `z`.";
static char __pyx_k_yve_v_z_Exponentially_scaled_Bes[] = "yve(v, z)\n\nExponentially scaled Bessel function of the second kind of real order\n\nReturns the exponentially scaled Bessel function of the second\nkind of real order `v` at complex `z`::\n\n    yve(v, z) = yv(v, z) * exp(-abs(z.imag))";
static char __pyx_k_zeta_x_q_Hurwitz_zeta_function_T[] = "zeta(x, q)\n\nHurwitz zeta function\n\nThe Riemann zeta function of two arguments (also known as the\nHurwitz zeta function).\n\nThis function is defined as\n\n.. math:: \\zeta(x, q) = \\sum_{k=0}^{\\infty} 1 / (k+q)^x,\n\nwhere ``x > 1`` and ``q > 0``.\n\nSee also\n--------\nzetac";
static char __pyx_k_zetac_x_Riemann_zeta_function_mi[] = "zetac(x)\n\nRiemann zeta function minus 1.\n\nThis function is defined as\n\n.. math:: \\zeta(x) = \\sum_{k=2}^{\\infty} 1 / k^x,\n\nwhere ``x > 1``.\n\nSee Also\n--------\nzeta";
static char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static char __pyx_k_i1_x_Modified_Bessel_function_of_2[] = "i1(x)\n\nModified Bessel function of the first kind of order 1";
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_RuntimeWarning;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_airy;
static PyObject *__pyx_n_s_airye;
static PyObject *__pyx_n_s_bdtr;
static PyObject *__pyx_n_s_bdtrc;
static PyObject *__pyx_n_s_bdtri;
static PyObject *__pyx_n_s_bdtrik;
static PyObject *__pyx_n_s_bdtrin;
static PyObject *__pyx_n_s_bei;
static PyObject *__pyx_n_s_beip;
static PyObject *__pyx_n_s_ber;
static PyObject *__pyx_n_s_berp;
static PyObject *__pyx_n_s_besselpoly;
static PyObject *__pyx_n_s_beta;
static PyObject *__pyx_n_s_betainc;
static PyObject *__pyx_n_s_betaincinv;
static PyObject *__pyx_n_s_betaln;
static PyObject *__pyx_n_s_binom;
static PyObject *__pyx_n_s_boxcox;
static PyObject *__pyx_n_s_boxcox1p;
static PyObject *__pyx_n_s_btdtr;
static PyObject *__pyx_n_s_btdtri;
static PyObject *__pyx_n_s_btdtria;
static PyObject *__pyx_n_s_btdtrib;
static PyObject *__pyx_n_s_cbrt;
static PyObject *__pyx_n_s_chdtr;
static PyObject *__pyx_n_s_chdtrc;
static PyObject *__pyx_n_s_chdtri;
static PyObject *__pyx_n_s_chdtriv;
static PyObject *__pyx_n_s_chndtr;
static PyObject *__pyx_n_s_chndtridf;
static PyObject *__pyx_n_s_chndtrinc;
static PyObject *__pyx_n_s_chndtrix;
static PyObject *__pyx_n_s_cosdg;
static PyObject *__pyx_n_s_cosm1;
static PyObject *__pyx_n_s_cotdg;
static PyObject *__pyx_n_s_dawsn;
static PyObject *__pyx_n_s_ellip_harm_2;
static PyObject *__pyx_n_s_ellipe;
static PyObject *__pyx_n_s_ellipeinc;
static PyObject *__pyx_n_s_ellipj;
static PyObject *__pyx_n_s_ellipkinc;
static PyObject *__pyx_n_s_ellipkm1;
static PyObject *__pyx_n_s_entr;
static PyObject *__pyx_n_s_erf;
static PyObject *__pyx_n_s_erfc;
static PyObject *__pyx_n_s_erfcx;
static PyObject *__pyx_n_s_erfi;
static PyObject *__pyx_n_s_errprint;
static PyObject *__pyx_n_s_eval_chebyc;
static PyObject *__pyx_n_s_eval_chebys;
static PyObject *__pyx_n_s_eval_chebyt;
static PyObject *__pyx_n_s_eval_chebyu;
static PyObject *__pyx_n_s_eval_gegenbauer;
static PyObject *__pyx_n_s_eval_genlaguerre;
static PyObject *__pyx_n_s_eval_hermite;
static PyObject *__pyx_n_s_eval_hermitenorm;
static PyObject *__pyx_n_s_eval_jacobi;
static PyObject *__pyx_n_s_eval_laguerre;
static PyObject *__pyx_n_s_eval_legendre;
static PyObject *__pyx_n_s_eval_sh_chebyt;
static PyObject *__pyx_n_s_eval_sh_chebyu;
static PyObject *__pyx_n_s_eval_sh_jacobi;
static PyObject *__pyx_n_s_eval_sh_legendre;
static PyObject *__pyx_n_s_exp1;
static PyObject *__pyx_n_s_exp10;
static PyObject *__pyx_n_s_exp2;
static PyObject *__pyx_n_s_expi;
static PyObject *__pyx_n_s_expit;
static PyObject *__pyx_n_s_expm1;
static PyObject *__pyx_n_s_expn;
static PyObject *__pyx_n_s_exprel;
static PyObject *__pyx_n_s_fdtr;
static PyObject *__pyx_n_s_fdtrc;
static PyObject *__pyx_n_s_fdtri;
static PyObject *__pyx_n_s_fdtridfd;
static PyObject *__pyx_n_s_fresnel;
static PyObject *__pyx_n_s_gamma;
static PyObject *__pyx_n_s_gammainc;
static PyObject *__pyx_n_s_gammaincc;
static PyObject *__pyx_n_s_gammainccinv;
static PyObject *__pyx_n_s_gammaincinv;
static PyObject *__pyx_n_s_gammaln;
static PyObject *__pyx_n_s_gammasgn;
static PyObject *__pyx_n_s_gdtr;
static PyObject *__pyx_n_s_gdtrc;
static PyObject *__pyx_n_s_gdtria;
static PyObject *__pyx_n_s_gdtrib;
static PyObject *__pyx_n_s_gdtrix;
static PyObject *__pyx_n_s_hankel1;
static PyObject *__pyx_n_s_hankel1e;
static PyObject *__pyx_n_s_hankel2;
static PyObject *__pyx_n_s_hankel2e;
static PyObject *__pyx_kp_s_home_br_repos_scipy_scipy_speci;
static PyObject *__pyx_n_s_huber;
static PyObject *__pyx_n_s_hyp1f1;
static PyObject *__pyx_n_s_hyp1f2;
static PyObject *__pyx_n_s_hyp2f0;
static PyObject *__pyx_n_s_hyp2f1;
static PyObject *__pyx_n_s_hyp3f0;
static PyObject *__pyx_n_s_hyperu;
static PyObject *__pyx_n_s_i0;
static PyObject *__pyx_n_s_i0e;
static PyObject *__pyx_n_s_i1;
static PyObject *__pyx_n_s_i1e;
static PyObject *__pyx_n_s_inflag;
static PyObject *__pyx_n_s_inv_boxcox;
static PyObject *__pyx_n_s_inv_boxcox1p;
static PyObject *__pyx_n_s_it2i0k0;
static PyObject *__pyx_n_s_it2j0y0;
static PyObject *__pyx_n_s_it2struve0;
static PyObject *__pyx_n_s_itairy;
static PyObject *__pyx_n_s_iti0k0;
static PyObject *__pyx_n_s_itj0y0;
static PyObject *__pyx_n_s_itmodstruve0;
static PyObject *__pyx_n_s_itstruve0;
static PyObject *__pyx_n_s_iv;
static PyObject *__pyx_n_s_ive;
static PyObject *__pyx_n_s_j0;
static PyObject *__pyx_n_s_j1;
static PyObject *__pyx_n_s_jn;
static PyObject *__pyx_n_s_jv;
static PyObject *__pyx_n_s_jve;
static PyObject *__pyx_n_s_k0;
static PyObject *__pyx_n_s_k0e;
static PyObject *__pyx_n_s_k1;
static PyObject *__pyx_n_s_k1e;
static PyObject *__pyx_n_s_kei;
static PyObject *__pyx_n_s_keip;
static PyObject *__pyx_n_s_kelvin;
static PyObject *__pyx_n_s_ker;
static PyObject *__pyx_n_s_kerp;
static PyObject *__pyx_n_s_kl_div;
static PyObject *__pyx_n_s_kn;
static PyObject *__pyx_n_s_kolmogi;
static PyObject *__pyx_n_s_kolmogorov;
static PyObject *__pyx_n_s_kv;
static PyObject *__pyx_n_s_kve;
static PyObject *__pyx_n_s_lambertw_2;
static PyObject *__pyx_n_s_log1p;
static PyObject *__pyx_n_s_log_ndtr;
static PyObject *__pyx_n_s_logit;
static PyObject *__pyx_n_s_lpmv;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mathieu_a;
static PyObject *__pyx_n_s_mathieu_b;
static PyObject *__pyx_n_s_mathieu_cem;
static PyObject *__pyx_n_s_mathieu_modcem1;
static PyObject *__pyx_n_s_mathieu_modcem2;
static PyObject *__pyx_n_s_mathieu_modsem1;
static PyObject *__pyx_n_s_mathieu_modsem2;
static PyObject *__pyx_n_s_mathieu_sem;
static PyObject *__pyx_n_s_modfresnelm;
static PyObject *__pyx_n_s_modfresnelp;
static PyObject *__pyx_n_s_modstruve;
static PyObject *__pyx_n_s_nbdtr;
static PyObject *__pyx_n_s_nbdtrc;
static PyObject *__pyx_n_s_nbdtri;
static PyObject *__pyx_n_s_nbdtrik;
static PyObject *__pyx_n_s_nbdtrin;
static PyObject *__pyx_n_s_ncfdtr;
static PyObject *__pyx_n_s_ncfdtri;
static PyObject *__pyx_n_s_ncfdtridfd;
static PyObject *__pyx_n_s_ncfdtridfn;
static PyObject *__pyx_n_s_ncfdtrinc;
static PyObject *__pyx_n_s_nctdtr;
static PyObject *__pyx_n_s_nctdtridf;
static PyObject *__pyx_n_s_nctdtrinc;
static PyObject *__pyx_n_s_nctdtrit;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_ndtr;
static PyObject *__pyx_n_s_ndtri;
static PyObject *__pyx_n_s_nrdtrimn;
static PyObject *__pyx_n_s_nrdtrisd;
static PyObject *__pyx_n_s_obl_ang1;
static PyObject *__pyx_n_s_obl_ang1_cv;
static PyObject *__pyx_n_s_obl_cv;
static PyObject *__pyx_n_s_obl_rad1;
static PyObject *__pyx_n_s_obl_rad1_cv;
static PyObject *__pyx_n_s_obl_rad2;
static PyObject *__pyx_n_s_obl_rad2_cv;
static PyObject *__pyx_n_s_pbdv;
static PyObject *__pyx_n_s_pbvv;
static PyObject *__pyx_n_s_pbwa;
static PyObject *__pyx_n_s_pdtr;
static PyObject *__pyx_n_s_pdtrc;
static PyObject *__pyx_n_s_pdtri;
static PyObject *__pyx_n_s_pdtrik;
static PyObject *__pyx_n_s_poch;
static PyObject *__pyx_n_s_pro_ang1;
static PyObject *__pyx_n_s_pro_ang1_cv;
static PyObject *__pyx_n_s_pro_cv;
static PyObject *__pyx_n_s_pro_rad1;
static PyObject *__pyx_n_s_pro_rad1_cv;
static PyObject *__pyx_n_s_pro_rad2;
static PyObject *__pyx_n_s_pro_rad2_cv;
static PyObject *__pyx_n_s_pseudo_huber;
static PyObject *__pyx_n_s_psi;
static PyObject *__pyx_n_s_radian;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_rel_entr;
static PyObject *__pyx_n_s_rgamma;
static PyObject *__pyx_n_s_round;
static PyObject *__pyx_n_s_scipy_special__ufuncs;
static PyObject *__pyx_n_s_shichi;
static PyObject *__pyx_n_s_sici;
static PyObject *__pyx_n_s_sindg;
static PyObject *__pyx_n_s_smirnov;
static PyObject *__pyx_n_s_smirnovi;
static PyObject *__pyx_n_s_spence;
static PyObject *__pyx_n_s_sph_harm;
static PyObject *__pyx_n_s_stdtr;
static PyObject *__pyx_n_s_stdtridf;
static PyObject *__pyx_n_s_stdtrit;
static PyObject *__pyx_n_s_struve;
static PyObject *__pyx_n_s_struve_asymp_large_z;
static PyObject *__pyx_n_s_struve_bessel_series;
static PyObject *__pyx_n_s_struve_power_series;
static PyObject *__pyx_n_s_tandg;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_tklmbda;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_n_s_wofz;
static PyObject *__pyx_n_s_xlog1py;
static PyObject *__pyx_n_s_xlogy;
static PyObject *__pyx_n_s_y0;
static PyObject *__pyx_n_s_y1;
static PyObject *__pyx_n_s_yn;
static PyObject *__pyx_n_s_yv;
static PyObject *__pyx_n_s_yve;
static PyObject *__pyx_n_s_zeta;
static PyObject *__pyx_n_s_zetac;
static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_errprint(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_inflag); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_codeobj__8;

/* "scipy/special/_ufuncs.pyx":26
 *     int PyUFunc_getfperr() nogil
 * 
 * cdef public int wrap_PyUFunc_getfperr() nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Call PyUFunc_getfperr in a context where PyUFunc_API array is initialized;
 */

int wrap_PyUFunc_getfperr(void) {
  int __pyx_r;

  /* "scipy/special/_ufuncs.pyx":31
 *     this avoids messing with the UNIQUE_SYMBOL #defines
 *     """
 *     return PyUFunc_getfperr()             # <<<<<<<<<<<<<<
 * 
 * cimport libc
 */
  __pyx_r = PyUFunc_getfperr();
  goto __pyx_L0;

  /* "scipy/special/_ufuncs.pyx":26
 *     int PyUFunc_getfperr() nogil
 * 
 * cdef public int wrap_PyUFunc_getfperr() nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Call PyUFunc_getfperr in a context where PyUFunc_API array is initialized;
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/special/_ufuncs.pyx":40
 * np.import_ufunc()
 * 
 * cdef int _set_errprint(int flag) nogil:             # <<<<<<<<<<<<<<
 *     return sf_error.set_print(flag)
 * 
 */

static int __pyx_f_5scipy_7special_7_ufuncs__set_errprint(int __pyx_v_flag) {
  int __pyx_r;

  /* "scipy/special/_ufuncs.pyx":41
 * 
 * cdef int _set_errprint(int flag) nogil:
 *     return sf_error.set_print(flag)             # <<<<<<<<<<<<<<
 * 
 * cimport scipy.special._ufuncs_cxx
 */
  __pyx_r = sf_error_set_print(__pyx_v_flag);
  goto __pyx_L0;

  /* "scipy/special/_ufuncs.pyx":40
 * np.import_ufunc()
 * 
 * cdef int _set_errprint(int flag) nogil:             # <<<<<<<<<<<<<<
 *     return sf_error.set_print(flag)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/special/_ufuncs.pyx":45
 * cimport scipy.special._ufuncs_cxx
 * 
 * def errprint(inflag=None):             # <<<<<<<<<<<<<<
 *     """
 *     errprint(inflag=None)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_1errprint(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_7special_7_ufuncs_errprint[] = "\n    errprint(inflag=None)\n\n    Sets or returns the error printing flag for special functions.\n\n    Parameters\n    ----------\n    inflag : bool, optional\n        Whether warnings concerning evaluation of special functions in\n        scipy.special are shown. If omitted, no change is made to the\n        current setting.\n\n    Returns\n    -------\n    old_flag\n        Previous value of the error flag\n\n    ";
static PyMethodDef __pyx_mdef_5scipy_7special_7_ufuncs_1errprint = {"errprint", (PyCFunction)__pyx_pw_5scipy_7special_7_ufuncs_1errprint, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_7special_7_ufuncs_errprint};
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_1errprint(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_inflag = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("errprint (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_inflag,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inflag);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "errprint") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_inflag = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("errprint", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.special._ufuncs.errprint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_7special_7_ufuncs_errprint(__pyx_self, __pyx_v_inflag);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_errprint(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_inflag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("errprint", 0);

  /* "scipy/special/_ufuncs.pyx":64
 * 
 *     """
 *     if inflag is not None:             # <<<<<<<<<<<<<<
 *         scipy.special._ufuncs_cxx._set_errprint(int(bool(inflag)))
 *         return sf_error.set_print(int(bool(inflag)))
 */
  __pyx_t_1 = (__pyx_v_inflag != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "scipy/special/_ufuncs.pyx":65
 *     """
 *     if inflag is not None:
 *         scipy.special._ufuncs_cxx._set_errprint(int(bool(inflag)))             # <<<<<<<<<<<<<<
 *         return sf_error.set_print(int(bool(inflag)))
 *     else:
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_inflag); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 65; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_f_5scipy_7special_11_ufuncs_cxx__set_errprint(((int)(!(!__pyx_t_2))));

    /* "scipy/special/_ufuncs.pyx":66
 *     if inflag is not None:
 *         scipy.special._ufuncs_cxx._set_errprint(int(bool(inflag)))
 *         return sf_error.set_print(int(bool(inflag)))             # <<<<<<<<<<<<<<
 *     else:
 *         return sf_error.get_print()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_inflag); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 66; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyInt_From_int(sf_error_set_print(((int)(!(!__pyx_t_2))))); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 66; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "scipy/special/_ufuncs.pyx":64
 * 
 *     """
 *     if inflag is not None:             # <<<<<<<<<<<<<<
 *         scipy.special._ufuncs_cxx._set_errprint(int(bool(inflag)))
 *         return sf_error.set_print(int(bool(inflag)))
 */
  }

  /* "scipy/special/_ufuncs.pyx":68
 *         return sf_error.set_print(int(bool(inflag)))
 *     else:
 *         return sf_error.get_print()             # <<<<<<<<<<<<<<
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_int(sf_error_get_print()); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "scipy/special/_ufuncs.pyx":45
 * cimport scipy.special._ufuncs_cxx
 * 
 * def errprint(inflag=None):             # <<<<<<<<<<<<<<
 *     """
 *     errprint(inflag=None)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("scipy.special._ufuncs.errprint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/special/_ufuncs.pyx":69
 *     else:
 *         return sf_error.get_print()
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":70
 *         return sf_error.get_print()
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":71
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":72
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":73
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":74
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":75
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":76
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":77
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":79
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":80
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":81
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":82
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":83
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":84
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":85
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":86
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":87
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":69
 *     else:
 *         return sf_error.get_print()
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":89
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_f_f__As_f_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  float __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":90
 * 
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":91
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":92
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":93
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef float ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":94
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef float ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":96
 *     cdef char *op0 = args[1]
 *     cdef float ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<float(*)(float) nogil>func)(<float>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":97
 *     cdef float ov0
 *     for i in range(n):
 *         ov0 = (<float(*)(float) nogil>func)(<float>(<float*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((float (*)(float))__pyx_v_func)(((float)(((float *)__pyx_v_ip0)[0])));

    /* "scipy/special/_ufuncs.pyx":98
 *     for i in range(n):
 *         ov0 = (<float(*)(float) nogil>func)(<float>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":99
 *         ov0 = (<float(*)(float) nogil>func)(<float>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":100
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":101
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":89
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":103
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":104
 * 
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":105
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":106
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":107
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":108
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":109
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":110
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":111
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":114
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0]:
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":115
 *     cdef double ov1
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)
 *         else:
 */
    __pyx_t_3 = ((((int)(((long *)__pyx_v_ip2)[0])) == (((long *)__pyx_v_ip2)[0])) != 0);
    if (__pyx_t_3) {

      /* "scipy/special/_ufuncs.pyx":116
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0]:
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(double, double, int, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((int)(((long *)__pyx_v_ip2)[0])), (&__pyx_v_ov1));

      /* "scipy/special/_ufuncs.pyx":115
 *     cdef double ov1
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":118
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, __pyx_k_invalid_input_argument);

      /* "scipy/special/_ufuncs.pyx":119
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
      __pyx_v_ov0 = ((double)NPY_NAN);

      /* "scipy/special/_ufuncs.pyx":120
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
      __pyx_v_ov1 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":121
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":122
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":123
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":124
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":125
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *         op1 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":126
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":127
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *         op1 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":128
 *         op0 += steps[3]
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":103
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":130
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":131
 * 
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":132
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":133
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":134
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":135
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":136
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":137
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":138
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[5]
 *     cdef char *op1 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":139
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[6]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":140
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]
 *     cdef char *op1 = args[6]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":143
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":144
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    ((int (*)(double, double, double, double, double, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])), ((double)(((double *)__pyx_v_ip4)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":145
 *     for i in range(n):
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":146
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":147
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":148
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":149
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":150
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         op0 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":151
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         op0 += steps[5]
 *         op1 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":152
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         op0 += steps[5]             # <<<<<<<<<<<<<<
 *         op1 += steps[6]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":153
 *         ip4 += steps[4]
 *         op0 += steps[5]
 *         op1 += steps[6]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[6]));
  }

  /* "scipy/special/_ufuncs.pyx":154
 *         op0 += steps[5]
 *         op1 += steps[6]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":130
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":156
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":157
 * 
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":158
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":159
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":160
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":161
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":162
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":164
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":165
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double complex>(<double complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, __pyx_t_double_complex))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":166
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":167
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":168
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":169
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":170
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":156
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":172
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":173
 * 
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":174
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":175
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":176
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":177
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":178
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":181
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":182
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    ((int (*)(double, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":183
 *     for i in range(n):
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":184
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":185
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":186
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":187
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":188
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":172
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":190
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":191
 * 
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":192
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":193
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":194
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef double complex ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":195
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":197
 *     cdef char *op0 = args[1]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<double complex*>ip0)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":198
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<double complex*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))));

    /* "scipy/special/_ufuncs.pyx":199
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<double complex*>ip0)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":200
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<double complex*>ip0)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":201
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":202
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":190
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":204
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_iidd__As_lldd_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/special/_ufuncs.pyx":205
 * 
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":206
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":207
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":208
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":209
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":210
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":211
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":212
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":214
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":215
 *     cdef double complex ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         else:
 */
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip0)[0])) == (((long *)__pyx_v_ip0)[0])) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_3 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip1)[0])) == (((long *)__pyx_v_ip1)[0])) != 0);
    __pyx_t_3 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_3) {

      /* "scipy/special/_ufuncs.pyx":216
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((__pyx_t_double_complex (*)(int, int, double, double))__pyx_v_func)(((int)(((long *)__pyx_v_ip0)[0])), ((int)(((long *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])));

      /* "scipy/special/_ufuncs.pyx":215
 *     cdef double complex ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":218
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double complex>NPY_NAN
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, __pyx_k_invalid_input_argument);

      /* "scipy/special/_ufuncs.pyx":219
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double complex>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = __pyx_t_double_complex_from_parts(((double)NPY_NAN), 0);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":220
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double complex>NPY_NAN
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":221
 *             ov0 = <double complex>NPY_NAN
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":222
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":223
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":224
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":225
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":226
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":204
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":228
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_F_FF(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":229
 * 
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":230
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":231
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":232
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":233
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":234
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":237
 *     cdef double complex ov0
 *     cdef double complex ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":238
 *     cdef double complex ov1
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 */
    ((int (*)(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip0)[0])))), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":239
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":240
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op1)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov1)), ((float)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":241
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":242
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":243
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":244
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":228
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":246
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_f_FFFF(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  __pyx_t_double_complex __pyx_v_ov2;
  __pyx_t_double_complex __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":247
 * 
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":248
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":249
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":250
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":251
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":252
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":253
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":254
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":259
 *     cdef double complex ov2
 *     cdef double complex ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":260
 *     cdef double complex ov3
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 */
    ((int (*)(double, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3));

    /* "scipy/special/_ufuncs.pyx":261
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":262
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1             # <<<<<<<<<<<<<<
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3
 */
    (((__pyx_t_float_complex *)__pyx_v_op1)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov1)), ((float)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":263
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2             # <<<<<<<<<<<<<<
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]
 */
    (((__pyx_t_float_complex *)__pyx_v_op2)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov2)), ((float)__Pyx_CIMAG(__pyx_v_ov2)));

    /* "scipy/special/_ufuncs.pyx":264
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op3)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov3)), ((float)__Pyx_CIMAG(__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":265
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":266
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":267
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":268
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":269
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":270
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":246
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":272
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_F_FFFF(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  __pyx_t_double_complex __pyx_v_ov2;
  __pyx_t_double_complex __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":273
 * 
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":274
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":275
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":276
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":277
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":278
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":279
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":280
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":285
 *     cdef double complex ov2
 *     cdef double complex ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":286
 *     cdef double complex ov3
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 */
    ((int (*)(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip0)[0])))), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3));

    /* "scipy/special/_ufuncs.pyx":287
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":288
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1             # <<<<<<<<<<<<<<
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3
 */
    (((__pyx_t_float_complex *)__pyx_v_op1)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov1)), ((float)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":289
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2             # <<<<<<<<<<<<<<
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]
 */
    (((__pyx_t_float_complex *)__pyx_v_op2)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov2)), ((float)__Pyx_CIMAG(__pyx_v_ov2)));

    /* "scipy/special/_ufuncs.pyx":290
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op3)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov3)), ((float)__Pyx_CIMAG(__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":291
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":292
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":293
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":294
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":295
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":296
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":272
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":298
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddi_d_As_dddl_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":299
 * 
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":300
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":301
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":302
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":303
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":304
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":305
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":306
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":307
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":310
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip3)[0] == (<long*>ip3)[0]:
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":311
 *     cdef double ov1
 *     for i in range(n):
 *         if <int>(<long*>ip3)[0] == (<long*>ip3)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)
 *         else:
 */
    __pyx_t_3 = ((((int)(((long *)__pyx_v_ip3)[0])) == (((long *)__pyx_v_ip3)[0])) != 0);
    if (__pyx_t_3) {

      /* "scipy/special/_ufuncs.pyx":312
 *     for i in range(n):
 *         if <int>(<long*>ip3)[0] == (<long*>ip3)[0]:
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(double, double, double, int, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((int)(((long *)__pyx_v_ip3)[0])), (&__pyx_v_ov1));

      /* "scipy/special/_ufuncs.pyx":311
 *     cdef double ov1
 *     for i in range(n):
 *         if <int>(<long*>ip3)[0] == (<long*>ip3)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":314
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, __pyx_k_invalid_input_argument);

      /* "scipy/special/_ufuncs.pyx":315
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
      __pyx_v_ov0 = ((double)NPY_NAN);

      /* "scipy/special/_ufuncs.pyx":316
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
      __pyx_v_ov1 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":317
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":318
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":319
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":320
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":321
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":322
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *         op1 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":323
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":324
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *         op1 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":325
 *         op0 += steps[4]
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":298
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":327
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ldd__As_ldd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":328
 * 
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":329
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":330
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":331
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":332
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":333
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":334
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":336
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(long, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":337
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(long, double, double))__pyx_v_func)(((long)(((long *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])));

    /* "scipy/special/_ufuncs.pyx":338
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":339
 *         ov0 = (<double(*)(long, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":340
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":341
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":342
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":343
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":327
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":345
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":346
 * 
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":347
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":348
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":349
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":350
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":351
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":352
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":354
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":355
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])));

    /* "scipy/special/_ufuncs.pyx":356
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":357
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":358
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":359
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":360
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":361
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":345
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":363
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":364
 * 
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":365
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":366
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":367
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":368
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":369
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":371
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":372
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])));

    /* "scipy/special/_ufuncs.pyx":373
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":374
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":375
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":376
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":377
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":363
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":379
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_ff_ffff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  double __pyx_v_ov2;
  double __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":380
 * 
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":381
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":382
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":383
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":384
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":385
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":386
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[4]
 *     cdef char *op3 = args[5]
 */
  __pyx_v_op1 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":387
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op2 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":388
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]
 *     cdef char *op3 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op3 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":393
 *     cdef double ov2
 *     cdef double ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":394
 *     cdef double ov3
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    ((int (*)(double, double, double *, double *, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3));

    /* "scipy/special/_ufuncs.pyx":395
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":396
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":397
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2             # <<<<<<<<<<<<<<
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op2)[0]) = ((float)__pyx_v_ov2);

    /* "scipy/special/_ufuncs.pyx":398
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op3)[0]) = ((float)__pyx_v_ov3);

    /* "scipy/special/_ufuncs.pyx":399
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":400
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *         op1 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":401
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *         op1 += steps[3]
 *         op2 += steps[4]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":402
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *         op1 += steps[3]             # <<<<<<<<<<<<<<
 *         op2 += steps[4]
 *         op3 += steps[5]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":403
 *         op0 += steps[2]
 *         op1 += steps[3]
 *         op2 += steps[4]             # <<<<<<<<<<<<<<
 *         op3 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":404
 *         op1 += steps[3]
 *         op2 += steps[4]
 *         op3 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":405
 *         op2 += steps[4]
 *         op3 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":379
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":407
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":408
 * 
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":409
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":410
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":411
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":412
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":413
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[3]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":414
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":417
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":418
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    ((int (*)(double, double, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":419
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":420
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":421
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":422
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *         op1 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":423
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *         op1 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":424
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *         op1 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":425
 *         op0 += steps[2]
 *         op1 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":407
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":427
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":428
 * 
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":429
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":430
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":431
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":432
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":433
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":434
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":435
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":438
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":439
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    ((int (*)(double, double, double, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":440
 *     for i in range(n):
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":441
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":442
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":443
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":444
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *         op1 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":445
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":446
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *         op1 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":447
 *         op0 += steps[3]
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":427
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":449
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":450
 * 
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":451
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":452
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":453
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":454
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":455
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":456
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":458
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double complex>(<double complex*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":459
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double complex>(<double complex*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, __pyx_t_double_complex))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip2)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip2)[0])))));

    /* "scipy/special/_ufuncs.pyx":460
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double complex>(<double complex*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":461
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double complex>(<double complex*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":462
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":463
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":464
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":465
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":449
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":467
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":468
 * 
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":469
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":470
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":471
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":472
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":473
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":474
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":475
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":477
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double complex>(<double complex*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":478
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double complex>(<double complex*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, double, __pyx_t_double_complex))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip3)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip3)[0])))));

    /* "scipy/special/_ufuncs.pyx":479
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double complex>(<double complex*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":480
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double complex>(<double complex*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":481
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":482
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":483
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":484
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":485
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":467
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":487
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  double __pyx_v_ov2;
  double __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":488
 * 
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":489
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":490
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":491
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":492
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":493
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":494
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":495
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":500
 *     cdef double ov2
 *     cdef double ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":501
 *     cdef double ov3
 *     for i in range(n):
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    ((int (*)(double, double *, double *, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3));

    /* "scipy/special/_ufuncs.pyx":502
 *     for i in range(n):
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":503
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":504
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2             # <<<<<<<<<<<<<<
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op2)[0]) = ((double)__pyx_v_ov2);

    /* "scipy/special/_ufuncs.pyx":505
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((double *)__pyx_v_op3)[0]) = ((double)__pyx_v_ov3);

    /* "scipy/special/_ufuncs.pyx":506
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":507
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":508
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":509
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":510
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":511
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":487
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":513
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":514
 * 
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":515
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":516
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":517
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":518
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":519
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":520
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":521
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[5]
 *     cdef char *op1 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":522
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[6]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":523
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]
 *     cdef char *op1 = args[6]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":526
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":527
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    ((int (*)(double, double, double, double, double, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])), ((double)(((float *)__pyx_v_ip4)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":528
 *     for i in range(n):
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":529
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":530
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":531
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":532
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":533
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         op0 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":534
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         op0 += steps[5]
 *         op1 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":535
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         op0 += steps[5]             # <<<<<<<<<<<<<<
 *         op1 += steps[6]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":536
 *         ip4 += steps[4]
 *         op0 += steps[5]
 *         op1 += steps[6]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[6]));
  }

  /* "scipy/special/_ufuncs.pyx":537
 *         op0 += steps[5]
 *         op1 += steps[6]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":513
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":539
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":540
 * 
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":541
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":542
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":543
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef double complex ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":544
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":546
 *     cdef char *op0 = args[1]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<float complex*>ip0)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":547
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<float complex*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip0)[0])))));

    /* "scipy/special/_ufuncs.pyx":548
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<float complex*>ip0)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":549
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<float complex*>ip0)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":550
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":551
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":539
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":553
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_d_DDDD(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  __pyx_t_double_complex __pyx_v_ov2;
  __pyx_t_double_complex __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":554
 * 
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":555
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":556
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":557
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":558
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":559
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":560
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":561
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":566
 *     cdef double complex ov2
 *     cdef double complex ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":567
 *     cdef double complex ov3
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 */
    ((int (*)(double, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3));

    /* "scipy/special/_ufuncs.pyx":568
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":569
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1             # <<<<<<<<<<<<<<
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3
 */
    (((__pyx_t_double_complex *)__pyx_v_op1)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov1)), ((double)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":570
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2             # <<<<<<<<<<<<<<
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]
 */
    (((__pyx_t_double_complex *)__pyx_v_op2)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov2)), ((double)__Pyx_CIMAG(__pyx_v_ov2)));

    /* "scipy/special/_ufuncs.pyx":571
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op3)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov3)), ((double)__Pyx_CIMAG(__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":572
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":573
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":574
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":575
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":576
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":577
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":553
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":579
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":580
 * 
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":581
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":582
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":583
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":584
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":585
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":586
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":587
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":590
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":591
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    ((int (*)(double, double, double, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":592
 *     for i in range(n):
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":593
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":594
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":595
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":596
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *         op1 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":597
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":598
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *         op1 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":599
 *         op0 += steps[3]
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":579
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":601
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_ffff_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":602
 * 
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":603
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":604
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":605
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":606
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":607
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":608
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":609
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":611
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":612
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":613
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":614
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":615
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":616
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":617
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":618
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":619
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":601
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":621
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_Dld__As_Dld_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":622
 * 
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":623
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":624
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":625
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":626
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":627
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":628
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":630
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex, long, double) nogil>func)(<double complex>(<double complex*>ip0)[0], <long>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":631
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, long, double) nogil>func)(<double complex>(<double complex*>ip0)[0], <long>(<long*>ip1)[0], <double>(<double*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex, long, double))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))), ((long)(((long *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])));

    /* "scipy/special/_ufuncs.pyx":632
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, long, double) nogil>func)(<double complex>(<double complex*>ip0)[0], <long>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":633
 *         ov0 = (<double complex(*)(double complex, long, double) nogil>func)(<double complex>(<double complex*>ip0)[0], <long>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":634
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":635
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":636
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":637
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":621
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":639
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_DD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":640
 * 
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":641
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":642
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":643
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":644
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":645
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":647
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<double complex*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":648
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<double complex*>ip0)[0], <double complex>(<double complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex, __pyx_t_double_complex))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":649
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<double complex*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":650
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<double complex*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":651
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":652
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":653
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":639
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":655
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":656
 * 
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":657
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":658
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":659
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":660
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":661
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[3]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":662
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":665
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":666
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    ((int (*)(double, double, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":667
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":668
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":669
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":670
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *         op1 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":671
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *         op1 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":672
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *         op1 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":673
 *         op0 += steps[2]
 *         op1 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":655
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":675
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":676
 * 
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":677
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":678
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":679
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":680
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":681
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":682
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":683
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":685
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double complex>(<float complex*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":686
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double complex>(<float complex*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, double, __pyx_t_double_complex))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip3)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip3)[0])))));

    /* "scipy/special/_ufuncs.pyx":687
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double complex>(<float complex*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":688
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double complex>(<float complex*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":689
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":690
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":691
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":692
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":693
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":675
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":695
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":696
 * 
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":697
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":698
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":699
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":700
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":701
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":702
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":703
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":705
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":706
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":707
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":708
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":709
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":710
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":711
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":712
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":713
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":695
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":715
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_FF_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":716
 * 
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":717
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":718
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":719
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":720
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":721
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":723
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<float complex*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":724
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<float complex*>ip0)[0], <double complex>(<float complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex, __pyx_t_double_complex))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip0)[0])))), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":725
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<float complex*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":726
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<float complex*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":727
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":728
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":729
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":715
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":731
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/special/_ufuncs.pyx":732
 * 
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":733
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":734
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":735
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":736
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":737
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":738
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":740
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":741
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         else:
 */
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip0)[0])) == (((long *)__pyx_v_ip0)[0])) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_3 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip1)[0])) == (((long *)__pyx_v_ip1)[0])) != 0);
    __pyx_t_3 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_3) {

      /* "scipy/special/_ufuncs.pyx":742
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(int, int, double))__pyx_v_func)(((int)(((long *)__pyx_v_ip0)[0])), ((int)(((long *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])));

      /* "scipy/special/_ufuncs.pyx":741
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":744
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, __pyx_k_invalid_input_argument);

      /* "scipy/special/_ufuncs.pyx":745
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":746
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":747
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":748
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":749
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":750
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":751
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":731
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":753
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_d_DD(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":754
 * 
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":755
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":756
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":757
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":758
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":759
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":762
 *     cdef double complex ov0
 *     cdef double complex ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":763
 *     cdef double complex ov1
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 */
    ((int (*)(double, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":764
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":765
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op1)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov1)), ((double)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":766
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":767
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":768
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":769
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":753
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":771
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":772
 * 
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":773
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":774
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":775
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef double ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":776
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":778
 *     cdef char *op0 = args[1]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":779
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<float*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])));

    /* "scipy/special/_ufuncs.pyx":780
 *     for i in range(n):
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":781
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":782
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":783
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":771
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":785
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":786
 * 
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":787
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":788
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":789
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef double ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":790
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":792
 *     cdef char *op0 = args[1]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<double*>ip0)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":793
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<double*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])));

    /* "scipy/special/_ufuncs.pyx":794
 *     for i in range(n):
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<double*>ip0)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":795
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<double*>ip0)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":796
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":797
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":785
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":799
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":800
 * 
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":801
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":802
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":803
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":804
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":805
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":806
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":807
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":808
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":811
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":812
 *     cdef double ov1
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":813
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":814
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":815
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":816
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":817
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":818
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *         op1 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":819
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":820
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *         op1 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":821
 *         op0 += steps[4]
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":799
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":823
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_dddd_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":824
 * 
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":825
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":826
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":827
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":828
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":829
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":830
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":831
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":833
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":834
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":835
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":836
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":837
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":838
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":839
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":840
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":841
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":823
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":843
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_ddddddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_ip5;
  char *__pyx_v_ip6;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":844
 * 
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":845
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":846
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":847
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":848
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":849
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":850
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":851
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":852
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]
 */
  __pyx_v_ip5 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":853
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 */
  __pyx_v_ip6 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":854
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[7]);

  /* "scipy/special/_ufuncs.pyx":856
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":857
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double, double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])), ((double)(((double *)__pyx_v_ip4)[0])), ((double)(((double *)__pyx_v_ip5)[0])), ((double)(((double *)__pyx_v_ip6)[0])));

    /* "scipy/special/_ufuncs.pyx":858
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":859
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":860
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":861
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":862
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":863
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":864
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         ip5 += steps[5]             # <<<<<<<<<<<<<<
 *         ip6 += steps[6]
 *         op0 += steps[7]
 */
    __pyx_v_ip5 = (__pyx_v_ip5 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":865
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 *         ip6 += steps[6]             # <<<<<<<<<<<<<<
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip6 = (__pyx_v_ip6 + (__pyx_v_steps[6]));

    /* "scipy/special/_ufuncs.pyx":866
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 *         op0 += steps[7]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[7]));
  }

  /* "scipy/special/_ufuncs.pyx":867
 *         ip6 += steps[6]
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":843
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":869
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_lddd__As_lddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":870
 * 
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":871
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":872
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":873
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":874
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":875
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":876
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":877
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":879
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(long, double, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":880
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(long, double, double, double))__pyx_v_func)(((long)(((long *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":881
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":882
 *         ov0 = (<double(*)(long, double, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":883
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":884
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":885
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":886
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":887
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":869
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":889
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":890
 * 
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":891
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":892
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":893
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":894
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":895
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":896
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":898
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double complex>(<float complex*>ip2)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":899
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double complex>(<float complex*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, __pyx_t_double_complex))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip2)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip2)[0])))));

    /* "scipy/special/_ufuncs.pyx":900
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double complex>(<float complex*>ip2)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":901
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double complex>(<float complex*>ip2)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":902
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":903
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":904
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":905
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":889
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":907
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  double __pyx_v_ov2;
  double __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":908
 * 
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":909
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":910
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":911
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":912
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":913
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":914
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":915
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":920
 *     cdef double ov2
 *     cdef double ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":921
 *     cdef double ov3
 *     for i in range(n):
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    ((int (*)(double, double *, double *, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3));

    /* "scipy/special/_ufuncs.pyx":922
 *     for i in range(n):
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":923
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":924
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2             # <<<<<<<<<<<<<<
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op2)[0]) = ((float)__pyx_v_ov2);

    /* "scipy/special/_ufuncs.pyx":925
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((float *)__pyx_v_op3)[0]) = ((float)__pyx_v_ov3);

    /* "scipy/special/_ufuncs.pyx":926
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":927
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":928
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":929
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":930
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":931
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":907
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":933
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_fffffff_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_ip5;
  char *__pyx_v_ip6;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":934
 * 
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":935
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":936
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":937
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":938
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":939
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":940
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":941
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":942
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]
 */
  __pyx_v_ip5 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":943
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 */
  __pyx_v_ip6 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":944
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[7]);

  /* "scipy/special/_ufuncs.pyx":946
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], <double>(<float*>ip5)[0], <double>(<float*>ip6)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":947
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], <double>(<float*>ip5)[0], <double>(<float*>ip6)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double, double, double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])), ((double)(((float *)__pyx_v_ip4)[0])), ((double)(((float *)__pyx_v_ip5)[0])), ((double)(((float *)__pyx_v_ip6)[0])));

    /* "scipy/special/_ufuncs.pyx":948
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], <double>(<float*>ip5)[0], <double>(<float*>ip6)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":949
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], <double>(<float*>ip5)[0], <double>(<float*>ip6)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":950
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":951
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":952
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":953
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":954
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         ip5 += steps[5]             # <<<<<<<<<<<<<<
 *         ip6 += steps[6]
 *         op0 += steps[7]
 */
    __pyx_v_ip5 = (__pyx_v_ip5 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":955
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 *         ip6 += steps[6]             # <<<<<<<<<<<<<<
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip6 = (__pyx_v_ip6 + (__pyx_v_steps[6]));

    /* "scipy/special/_ufuncs.pyx":956
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 *         op0 += steps[7]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[7]));
  }

  /* "scipy/special/_ufuncs.pyx":957
 *         ip6 += steps[6]
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":933
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":959
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":960
 * 
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":961
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":962
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":963
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":964
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":965
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":966
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":967
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":968
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":971
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":972
 *     cdef double ov1
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":973
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":974
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":975
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":976
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":977
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":978
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *         op1 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":979
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":980
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *         op1 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":981
 *         op0 += steps[4]
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":959
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":983
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_D_DD(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":984
 * 
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":985
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":986
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":987
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":988
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":989
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":992
 *     cdef double complex ov0
 *     cdef double complex ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":993
 *     cdef double complex ov1
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 */
    ((int (*)(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":994
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":995
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op1)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov1)), ((double)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":996
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":997
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":998
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":999
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":983
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1001
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddiiddd__As_ddllddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_ip5;
  char *__pyx_v_ip6;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/special/_ufuncs.pyx":1002
 * 
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1003
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1004
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1005
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1006
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1007
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1008
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1009
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1010
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]
 */
  __pyx_v_ip5 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":1011
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 */
  __pyx_v_ip6 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":1012
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[7]);

  /* "scipy/special/_ufuncs.pyx":1014
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0] and <int>(<long*>ip3)[0] == (<long*>ip3)[0]:
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1015
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0] and <int>(<long*>ip3)[0] == (<long*>ip3)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         else:
 */
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip2)[0])) == (((long *)__pyx_v_ip2)[0])) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_3 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip3)[0])) == (((long *)__pyx_v_ip3)[0])) != 0);
    __pyx_t_3 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_3) {

      /* "scipy/special/_ufuncs.pyx":1016
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0] and <int>(<long*>ip3)[0] == (<long*>ip3)[0]:
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(double, double, int, int, double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((int)(((long *)__pyx_v_ip2)[0])), ((int)(((long *)__pyx_v_ip3)[0])), ((double)(((double *)__pyx_v_ip4)[0])), ((double)(((double *)__pyx_v_ip5)[0])), ((double)(((double *)__pyx_v_ip6)[0])));

      /* "scipy/special/_ufuncs.pyx":1015
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0] and <int>(<long*>ip3)[0] == (<long*>ip3)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":1018
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, __pyx_k_invalid_input_argument);

      /* "scipy/special/_ufuncs.pyx":1019
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":1020
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1021
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1022
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1023
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1024
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1025
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":1026
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         ip5 += steps[5]             # <<<<<<<<<<<<<<
 *         ip6 += steps[6]
 *         op0 += steps[7]
 */
    __pyx_v_ip5 = (__pyx_v_ip5 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":1027
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 *         ip6 += steps[6]             # <<<<<<<<<<<<<<
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip6 = (__pyx_v_ip6 + (__pyx_v_steps[6]));

    /* "scipy/special/_ufuncs.pyx":1028
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 *         op0 += steps[7]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[7]));
  }

  /* "scipy/special/_ufuncs.pyx":1029
 *         ip6 += steps[6]
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1001
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1031
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_dd_dddd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  double __pyx_v_ov2;
  double __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1032
 * 
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1033
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1034
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1035
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1036
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1037
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1038
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[4]
 *     cdef char *op3 = args[5]
 */
  __pyx_v_op1 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1039
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op2 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1040
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]
 *     cdef char *op3 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op3 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":1045
 *     cdef double ov2
 *     cdef double ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1046
 *     cdef double ov3
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    ((int (*)(double, double, double *, double *, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3));

    /* "scipy/special/_ufuncs.pyx":1047
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1048
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1049
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2             # <<<<<<<<<<<<<<
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op2)[0]) = ((double)__pyx_v_ov2);

    /* "scipy/special/_ufuncs.pyx":1050
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op3)[0]) = ((double)__pyx_v_ov3);

    /* "scipy/special/_ufuncs.pyx":1051
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1052
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *         op1 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1053
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *         op1 += steps[3]
 *         op2 += steps[4]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1054
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *         op1 += steps[3]             # <<<<<<<<<<<<<<
 *         op2 += steps[4]
 *         op3 += steps[5]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1055
 *         op0 += steps[2]
 *         op1 += steps[3]
 *         op2 += steps[4]             # <<<<<<<<<<<<<<
 *         op3 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":1056
 *         op1 += steps[3]
 *         op2 += steps[4]
 *         op3 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":1057
 *         op2 += steps[4]
 *         op3 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1031
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1059
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1060
 * 
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1061
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1062
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1063
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1064
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1065
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1067
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1068
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double complex>(<float complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, __pyx_t_double_complex))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":1069
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":1070
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1071
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1072
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":1073
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1059
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1075
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1076
 * 
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1077
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1078
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1079
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1080
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1081
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1084
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1085
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    ((int (*)(double, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":1086
 *     for i in range(n):
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1087
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1088
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1089
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1090
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":1091
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1075
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1093
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":1094
 * 
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1095
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1096
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1097
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1098
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1099
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1101
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1102
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         else:
 */
    __pyx_t_3 = ((((int)(((long *)__pyx_v_ip0)[0])) == (((long *)__pyx_v_ip0)[0])) != 0);
    if (__pyx_t_3) {

      /* "scipy/special/_ufuncs.pyx":1103
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(int, double))__pyx_v_func)(((int)(((long *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])));

      /* "scipy/special/_ufuncs.pyx":1102
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":1105
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, __pyx_k_invalid_input_argument);

      /* "scipy/special/_ufuncs.pyx":1106
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":1107
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1108
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1109
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1110
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":1111
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1093
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1113
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1114
 * 
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1115
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1116
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1117
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1118
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1119
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1121
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1122
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])));

    /* "scipy/special/_ufuncs.pyx":1123
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1124
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1125
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1126
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":1127
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1113
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1129
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_f_FF(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1130
 * 
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1131
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1132
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1133
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1134
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1135
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1138
 *     cdef double complex ov0
 *     cdef double complex ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1139
 *     cdef double complex ov1
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 */
    ((int (*)(double, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":1140
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":1141
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op1)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov1)), ((float)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":1142
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1143
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1144
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":1145
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1129
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1147
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1148
 * 
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1149
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1150
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1151
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1152
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1153
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1154
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1156
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1157
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])));

    /* "scipy/special/_ufuncs.pyx":1158
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1159
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1160
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1161
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1162
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":1163
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1147
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1165
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_D_DDDD(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  __pyx_t_double_complex __pyx_v_ov2;
  __pyx_t_double_complex __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1166
 * 
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1167
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1168
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1169
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1170
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1171
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1172
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1173
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1178
 *     cdef double complex ov2
 *     cdef double complex ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1179
 *     cdef double complex ov3
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 */
    ((int (*)(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3));

    /* "scipy/special/_ufuncs.pyx":1180
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":1181
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1             # <<<<<<<<<<<<<<
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3
 */
    (((__pyx_t_double_complex *)__pyx_v_op1)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov1)), ((double)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":1182
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2             # <<<<<<<<<<<<<<
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]
 */
    (((__pyx_t_double_complex *)__pyx_v_op2)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov2)), ((double)__Pyx_CIMAG(__pyx_v_ov2)));

    /* "scipy/special/_ufuncs.pyx":1183
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op3)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov3)), ((double)__Pyx_CIMAG(__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":1184
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1185
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1186
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1187
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1188
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":1189
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1165
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1191
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_g_g__As_g_g(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  long double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1192
 * 
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1193
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1194
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1195
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef long double ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1196
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef long double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1198
 *     cdef char *op0 = args[1]
 *     cdef long double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<long double(*)(long double) nogil>func)(<long double>(<long double*>ip0)[0])
 *         (<long double *>op0)[0] = <long double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1199
 *     cdef long double ov0
 *     for i in range(n):
 *         ov0 = (<long double(*)(long double) nogil>func)(<long double>(<long double*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<long double *>op0)[0] = <long double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((long double (*)(long double))__pyx_v_func)(((long double)(((long double *)__pyx_v_ip0)[0])));

    /* "scipy/special/_ufuncs.pyx":1200
 *     for i in range(n):
 *         ov0 = (<long double(*)(long double) nogil>func)(<long double>(<long double*>ip0)[0])
 *         (<long double *>op0)[0] = <long double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((long double *)__pyx_v_op0)[0]) = ((long double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1201
 *         ov0 = (<long double(*)(long double) nogil>func)(<long double>(<long double*>ip0)[0])
 *         (<long double *>op0)[0] = <long double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1202
 *         (<long double *>op0)[0] = <long double>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":1203
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1191
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1205
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;

  /* "scipy/special/_ufuncs.pyx":1206
 * 
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1207
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1208
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1209
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1210
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1211
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1213
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(long, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "scipy/special/_ufuncs.pyx":1214
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(long, double))__pyx_v_func)(((long)(((long *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])));

    /* "scipy/special/_ufuncs.pyx":1215
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1216
 *         ov0 = (<double(*)(long, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1217
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1218
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":1219
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * from _ellip_harm cimport ellip_harmonic as _func_ellip_harmonic
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1205
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "sph_harm.pxd":13
 * from libc.stdlib cimport abs
 * 
 * cdef inline double complex sph_harmonic(int m, int n, double theta, double phi) nogil:             # <<<<<<<<<<<<<<
 *     cdef double x, prefactor
 *     cdef double complex val
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8sph_harm_sph_harmonic(int __pyx_v_m, int __pyx_v_n, double __pyx_v_theta, double __pyx_v_phi) {
  double __pyx_v_x;
  double __pyx_v_prefactor;
  __pyx_t_double_complex __pyx_v_val;
  int __pyx_v_mp;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "sph_harm.pxd":17
 *     cdef double complex val
 *     cdef int mp
 *     x = cos(phi)             # <<<<<<<<<<<<<<
 *     if abs(m) > n :
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 */
  __pyx_v_x = cos(__pyx_v_phi);

  /* "sph_harm.pxd":18
 *     cdef int mp
 *     x = cos(phi)
 *     if abs(m) > n :             # <<<<<<<<<<<<<<
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan
 */
  __pyx_t_1 = ((abs(__pyx_v_m) > __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "sph_harm.pxd":19
 *     x = cos(phi)
 *     if abs(m) > n :
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")             # <<<<<<<<<<<<<<
 *         return nan
 *     if n < 0:
 */
    sf_error(__pyx_k_sph_harm, SF_ERROR_ARG, __pyx_k_m_should_not_be_greater_than_n);

    /* "sph_harm.pxd":20
 *     if abs(m) > n :
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "sph_harm.pxd":18
 *     cdef int mp
 *     x = cos(phi)
 *     if abs(m) > n :             # <<<<<<<<<<<<<<
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan
 */
  }

  /* "sph_harm.pxd":21
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "sph_harm.pxd":22
 *         return nan
 *     if n < 0:
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")             # <<<<<<<<<<<<<<
 *         return nan
 *     if m < 0:
 */
    sf_error(__pyx_k_sph_harm, SF_ERROR_ARG, __pyx_k_n_should_not_be_negative);

    /* "sph_harm.pxd":23
 *     if n < 0:
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan             # <<<<<<<<<<<<<<
 *     if m < 0:
 *         mp = -m
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "sph_harm.pxd":21
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan
 */
  }

  /* "sph_harm.pxd":24
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan
 *     if m < 0:             # <<<<<<<<<<<<<<
 *         mp = -m
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)
 */
  __pyx_t_1 = ((__pyx_v_m < 0) != 0);
  if (__pyx_t_1) {

    /* "sph_harm.pxd":25
 *         return nan
 *     if m < 0:
 *         mp = -m             # <<<<<<<<<<<<<<
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)
 *     else:
 */
    __pyx_v_mp = (-__pyx_v_m);

    /* "sph_harm.pxd":26
 *     if m < 0:
 *         mp = -m
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)             # <<<<<<<<<<<<<<
 *     else:
 *         mp = m
 */
    __pyx_v_prefactor = (__Pyx_pow_long(-1L, ((long)__pyx_v_mp)) * poch(((__pyx_v_n + __pyx_v_mp) + 1), (-2L * __pyx_v_mp)));

    /* "sph_harm.pxd":24
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan
 *     if m < 0:             # <<<<<<<<<<<<<<
 *         mp = -m
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)
 */
    goto __pyx_L5;
  }

  /* "sph_harm.pxd":28
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)
 *     else:
 *         mp = m             # <<<<<<<<<<<<<<
 *     val = pmv_wrap(mp, n, x)
 *     if  m < 0:
 */
  /*else*/ {
    __pyx_v_mp = __pyx_v_m;
  }
  __pyx_L5:;

  /* "sph_harm.pxd":29
 *     else:
 *         mp = m
 *     val = pmv_wrap(mp, n, x)             # <<<<<<<<<<<<<<
 *     if  m < 0:
 *         val *= prefactor
 */
  __pyx_v_val = __pyx_t_double_complex_from_parts(pmv_wrap(__pyx_v_mp, __pyx_v_n, __pyx_v_x), 0);

  /* "sph_harm.pxd":30
 *         mp = m
 *     val = pmv_wrap(mp, n, x)
 *     if  m < 0:             # <<<<<<<<<<<<<<
 *         val *= prefactor
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 */
  __pyx_t_1 = ((__pyx_v_m < 0) != 0);
  if (__pyx_t_1) {

    /* "sph_harm.pxd":31
 *     val = pmv_wrap(mp, n, x)
 *     if  m < 0:
 *         val *= prefactor             # <<<<<<<<<<<<<<
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 *     val *= sqrt(poch(n + m + 1, -2 * m))
 */
    __pyx_v_val = __Pyx_c_prod(__pyx_v_val, __pyx_t_double_complex_from_parts(__pyx_v_prefactor, 0));

    /* "sph_harm.pxd":30
 *         mp = m
 *     val = pmv_wrap(mp, n, x)
 *     if  m < 0:             # <<<<<<<<<<<<<<
 *         val *= prefactor
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 */
  }

  /* "sph_harm.pxd":32
 *     if  m < 0:
 *         val *= prefactor
 *     val *= sqrt((2*n + 1) / 4.0 / pi)             # <<<<<<<<<<<<<<
 *     val *= sqrt(poch(n + m + 1, -2 * m))
 *     val *= zexp(1j * m * theta)
 */
  __pyx_t_2 = (((2 * __pyx_v_n) + 1) / 4.0);
  if (unlikely(NPY_PI == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_v_val = __Pyx_c_prod(__pyx_v_val, __pyx_t_double_complex_from_parts(sqrt((__pyx_t_2 / NPY_PI)), 0));

  /* "sph_harm.pxd":33
 *         val *= prefactor
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 *     val *= sqrt(poch(n + m + 1, -2 * m))             # <<<<<<<<<<<<<<
 *     val *= zexp(1j * m * theta)
 *     return val
 */
  __pyx_v_val = __Pyx_c_prod(__pyx_v_val, __pyx_t_double_complex_from_parts(sqrt(poch(((__pyx_v_n + __pyx_v_m) + 1), (-2L * __pyx_v_m))), 0));

  /* "sph_harm.pxd":34
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 *     val *= sqrt(poch(n + m + 1, -2 * m))
 *     val *= zexp(1j * m * theta)             # <<<<<<<<<<<<<<
 *     return val
 */
  __pyx_v_val = __Pyx_c_prod(__pyx_v_val, __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__Pyx_c_prod(__Pyx_c_prod(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(__pyx_v_m, 0)), __pyx_t_double_complex_from_parts(__pyx_v_theta, 0))));

  /* "sph_harm.pxd":35
 *     val *= sqrt(poch(n + m + 1, -2 * m))
 *     val *= zexp(1j * m * theta)
 *     return val             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_val;
  goto __pyx_L0;

  /* "sph_harm.pxd":13
 * from libc.stdlib cimport abs
 * 
 * cdef inline double complex sph_harmonic(int m, int n, double theta, double phi) nogil:             # <<<<<<<<<<<<<<
 *     cdef double x, prefactor
 *     cdef double complex val
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special.sph_harm.sph_harmonic", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
  __pyx_L0:;
  return __pyx_r;
}

/* "_ellip_harm.pxd":49
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * cdef inline double* lame_coefficients(double h2, double k2, int n, int p,             # <<<<<<<<<<<<<<
 *                                       void **bufferp, double signm,
 *                                       double signn) nogil:
 */

static CYTHON_INLINE double *__pyx_f_5scipy_7special_11_ellip_harm_lame_coefficients(double __pyx_v_h2, double __pyx_v_k2, int __pyx_v_n, int __pyx_v_p, void **__pyx_v_bufferp, double __pyx_v_signm, double __pyx_v_signn) {
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_gamma;
  double __pyx_v_tol;
  double __pyx_v_vl;
  double __pyx_v_vu;
  int __pyx_v_r;
  int __pyx_v_tp;
  int __pyx_v_j;
  int __pyx_v_size;
  int __pyx_v_i;
  int __pyx_v_info;
  int __pyx_v_lwork;
  int __pyx_v_liwork;
  int __pyx_v_c;
  char __pyx_v_t;
  void *__pyx_v_buffer;
  double *__pyx_v_g;
  double *__pyx_v_d;
  double *__pyx_v_f;
  double *__pyx_v_ss;
  double *__pyx_v_w;
  double *__pyx_v_dd;
  double *__pyx_v_eigv;
  double *__pyx_v_work;
  int *__pyx_v_iwork;
  int *__pyx_v_isuppz;
  double *__pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  char __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;

  /* "_ellip_harm.pxd":52
 *                                       void **bufferp, double signm,
 *                                       double signn) nogil:
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for n")
 *         return NULL
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":53
 *                                       double signn) nogil:
 *     if n < 0:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for n")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(__pyx_k_ellip_harm, SF_ERROR_ARG, __pyx_k_invalid_value_for_n);

    /* "_ellip_harm.pxd":54
 *     if n < 0:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for n")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     if p < 1 or p > 2*n + 1:
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":52
 *                                       void **bufferp, double signm,
 *                                       double signn) nogil:
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for n")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":56
 *         return NULL
 * 
 *     if p < 1 or p > 2*n + 1:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for p")
 *         return NULL
 */
  __pyx_t_2 = ((__pyx_v_p < 1) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_p > ((2 * __pyx_v_n) + 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":57
 * 
 *     if p < 1 or p > 2*n + 1:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for p")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(__pyx_k_ellip_harm, SF_ERROR_ARG, __pyx_k_invalid_value_for_p);

    /* "_ellip_harm.pxd":58
 *     if p < 1 or p > 2*n + 1:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for p")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     if fabs(signm) != 1 or fabs(signn) != 1:
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":56
 *         return NULL
 * 
 *     if p < 1 or p > 2*n + 1:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for p")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":60
 *         return NULL
 * 
 *     if fabs(signm) != 1 or fabs(signn) != 1:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid signm or signn")
 *         return NULL
 */
  __pyx_t_2 = ((fabs(__pyx_v_signm) != 1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__pyx_v_signn) != 1.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":61
 * 
 *     if fabs(signm) != 1 or fabs(signn) != 1:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid signm or signn")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(__pyx_k_ellip_harm, SF_ERROR_ARG, __pyx_k_invalid_signm_or_signn);

    /* "_ellip_harm.pxd":62
 *     if fabs(signm) != 1 or fabs(signn) != 1:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid signm or signn")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef double s2, alpha, beta, gamma, lamba_romain, pp, psi, t1, tol, vl, vu
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":60
 *         return NULL
 * 
 *     if fabs(signm) != 1 or fabs(signn) != 1:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid signm or signn")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":68
 *     cdef char t
 * 
 *     r = n/2             # <<<<<<<<<<<<<<
 *     alpha = h2
 *     beta = k2 - h2
 */
  __pyx_v_r = __Pyx_div_long(__pyx_v_n, 2);

  /* "_ellip_harm.pxd":69
 * 
 *     r = n/2
 *     alpha = h2             # <<<<<<<<<<<<<<
 *     beta = k2 - h2
 *     gamma = alpha - beta
 */
  __pyx_v_alpha = __pyx_v_h2;

  /* "_ellip_harm.pxd":70
 *     r = n/2
 *     alpha = h2
 *     beta = k2 - h2             # <<<<<<<<<<<<<<
 *     gamma = alpha - beta
 * 
 */
  __pyx_v_beta = (__pyx_v_k2 - __pyx_v_h2);

  /* "_ellip_harm.pxd":71
 *     alpha = h2
 *     beta = k2 - h2
 *     gamma = alpha - beta             # <<<<<<<<<<<<<<
 * 
 *     if p - 1 < r + 1:
 */
  __pyx_v_gamma = (__pyx_v_alpha - __pyx_v_beta);

  /* "_ellip_harm.pxd":73
 *     gamma = alpha - beta
 * 
 *     if p - 1 < r + 1:             # <<<<<<<<<<<<<<
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < (__pyx_v_r + 1)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":74
 * 
 *     if p - 1 < r + 1:
 *         t, tp, size = 'K', p, r + 1             # <<<<<<<<<<<<<<
 *     elif p - 1 < (n - r) + (r + 1):
 *         t, tp, size = 'L', p - (r + 1), n - r
 */
    __pyx_t_3 = 'K';
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_5 = (__pyx_v_r + 1);
    __pyx_v_t = __pyx_t_3;
    __pyx_v_tp = __pyx_t_4;
    __pyx_v_size = __pyx_t_5;

    /* "_ellip_harm.pxd":73
 *     gamma = alpha - beta
 * 
 *     if p - 1 < r + 1:             # <<<<<<<<<<<<<<
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):
 */
    goto __pyx_L10;
  }

  /* "_ellip_harm.pxd":75
 *     if p - 1 < r + 1:
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < ((__pyx_v_n - __pyx_v_r) + (__pyx_v_r + 1))) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":76
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):
 *         t, tp, size = 'L', p - (r + 1), n - r             # <<<<<<<<<<<<<<
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 */
    __pyx_t_3 = 'L';
    __pyx_t_5 = (__pyx_v_p - (__pyx_v_r + 1));
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_v_t = __pyx_t_3;
    __pyx_v_tp = __pyx_t_5;
    __pyx_v_size = __pyx_t_4;

    /* "_ellip_harm.pxd":75
 *     if p - 1 < r + 1:
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 */
    goto __pyx_L10;
  }

  /* "_ellip_harm.pxd":77
 *     elif p - 1 < (n - r) + (r + 1):
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < (((__pyx_v_n - __pyx_v_r) + (__pyx_v_n - __pyx_v_r)) + (__pyx_v_r + 1))) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":78
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r             # <<<<<<<<<<<<<<
 *     elif p - 1 < 2*n + 1:
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r
 */
    __pyx_t_3 = 'M';
    __pyx_t_5 = ((__pyx_v_p - (__pyx_v_n - __pyx_v_r)) - (__pyx_v_r + 1));
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_v_t = __pyx_t_3;
    __pyx_v_tp = __pyx_t_5;
    __pyx_v_size = __pyx_t_4;

    /* "_ellip_harm.pxd":77
 *     elif p - 1 < (n - r) + (r + 1):
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:
 */
    goto __pyx_L10;
  }

  /* "_ellip_harm.pxd":79
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:             # <<<<<<<<<<<<<<
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r
 * 
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < ((2 * __pyx_v_n) + 1)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":80
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r             # <<<<<<<<<<<<<<
 * 
 *     lwork = 60*size
 */
    __pyx_t_3 = 'N';
    __pyx_t_5 = (((__pyx_v_p - (__pyx_v_n - __pyx_v_r)) - (__pyx_v_n - __pyx_v_r)) - (__pyx_v_r + 1));
    __pyx_t_4 = __pyx_v_r;
    __pyx_v_t = __pyx_t_3;
    __pyx_v_tp = __pyx_t_5;
    __pyx_v_size = __pyx_t_4;

    /* "_ellip_harm.pxd":79
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:             # <<<<<<<<<<<<<<
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r
 * 
 */
  }
  __pyx_L10:;

  /* "_ellip_harm.pxd":82
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r
 * 
 *     lwork = 60*size             # <<<<<<<<<<<<<<
 *     liwork = 30*size
 *     tol = 0.0
 */
  __pyx_v_lwork = (60 * __pyx_v_size);

  /* "_ellip_harm.pxd":83
 * 
 *     lwork = 60*size
 *     liwork = 30*size             # <<<<<<<<<<<<<<
 *     tol = 0.0
 *     vl = 0
 */
  __pyx_v_liwork = (30 * __pyx_v_size);

  /* "_ellip_harm.pxd":84
 *     lwork = 60*size
 *     liwork = 30*size
 *     tol = 0.0             # <<<<<<<<<<<<<<
 *     vl = 0
 *     vu = 0
 */
  __pyx_v_tol = 0.0;

  /* "_ellip_harm.pxd":85
 *     liwork = 30*size
 *     tol = 0.0
 *     vl = 0             # <<<<<<<<<<<<<<
 *     vu = 0
 * 
 */
  __pyx_v_vl = 0.0;

  /* "_ellip_harm.pxd":86
 *     tol = 0.0
 *     vl = 0
 *     vu = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef void *buffer = malloc((sizeof(double)*(7*size + lwork))
 */
  __pyx_v_vu = 0.0;

  /* "_ellip_harm.pxd":88
 *     vu = 0
 * 
 *     cdef void *buffer = malloc((sizeof(double)*(7*size + lwork))             # <<<<<<<<<<<<<<
 *                                + (sizeof(int)*(2*size + liwork)))
 *     bufferp[0] = buffer
 */
  __pyx_v_buffer = malloc((((sizeof(double)) * ((7 * __pyx_v_size) + __pyx_v_lwork)) + ((sizeof(int)) * ((2 * __pyx_v_size) + __pyx_v_liwork))));

  /* "_ellip_harm.pxd":90
 *     cdef void *buffer = malloc((sizeof(double)*(7*size + lwork))
 *                                + (sizeof(int)*(2*size + liwork)))
 *     bufferp[0] = buffer             # <<<<<<<<<<<<<<
 *     if not buffer:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 */
  (__pyx_v_bufferp[0]) = __pyx_v_buffer;

  /* "_ellip_harm.pxd":91
 *                                + (sizeof(int)*(2*size + liwork)))
 *     bufferp[0] = buffer
 *     if not buffer:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL
 */
  __pyx_t_1 = ((!(__pyx_v_buffer != 0)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":92
 *     bufferp[0] = buffer
 *     if not buffer:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(__pyx_k_ellip_harm, SF_ERROR_NO_RESULT, __pyx_k_failed_to_allocate_memory);

    /* "_ellip_harm.pxd":93
 *     if not buffer:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef double *g = <double *>buffer
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":91
 *                                + (sizeof(int)*(2*size + liwork)))
 *     bufferp[0] = buffer
 *     if not buffer:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":95
 *         return NULL
 * 
 *     cdef double *g = <double *>buffer             # <<<<<<<<<<<<<<
 *     cdef double *d = g + size
 *     cdef double *f = d + size
 */
  __pyx_v_g = ((double *)__pyx_v_buffer);

  /* "_ellip_harm.pxd":96
 * 
 *     cdef double *g = <double *>buffer
 *     cdef double *d = g + size             # <<<<<<<<<<<<<<
 *     cdef double *f = d + size
 *     cdef double *ss =  f + size
 */
  __pyx_v_d = (__pyx_v_g + __pyx_v_size);

  /* "_ellip_harm.pxd":97
 *     cdef double *g = <double *>buffer
 *     cdef double *d = g + size
 *     cdef double *f = d + size             # <<<<<<<<<<<<<<
 *     cdef double *ss =  f + size
 *     cdef double *w =  ss + size
 */
  __pyx_v_f = (__pyx_v_d + __pyx_v_size);

  /* "_ellip_harm.pxd":98
 *     cdef double *d = g + size
 *     cdef double *f = d + size
 *     cdef double *ss =  f + size             # <<<<<<<<<<<<<<
 *     cdef double *w =  ss + size
 *     cdef double *dd = w + size
 */
  __pyx_v_ss = (__pyx_v_f + __pyx_v_size);

  /* "_ellip_harm.pxd":99
 *     cdef double *f = d + size
 *     cdef double *ss =  f + size
 *     cdef double *w =  ss + size             # <<<<<<<<<<<<<<
 *     cdef double *dd = w + size
 *     cdef double *eigv = dd + size
 */
  __pyx_v_w = (__pyx_v_ss + __pyx_v_size);

  /* "_ellip_harm.pxd":100
 *     cdef double *ss =  f + size
 *     cdef double *w =  ss + size
 *     cdef double *dd = w + size             # <<<<<<<<<<<<<<
 *     cdef double *eigv = dd + size
 *     cdef double *work = eigv + size
 */
  __pyx_v_dd = (__pyx_v_w + __pyx_v_size);

  /* "_ellip_harm.pxd":101
 *     cdef double *w =  ss + size
 *     cdef double *dd = w + size
 *     cdef double *eigv = dd + size             # <<<<<<<<<<<<<<
 *     cdef double *work = eigv + size
 * 
 */
  __pyx_v_eigv = (__pyx_v_dd + __pyx_v_size);

  /* "_ellip_harm.pxd":102
 *     cdef double *dd = w + size
 *     cdef double *eigv = dd + size
 *     cdef double *work = eigv + size             # <<<<<<<<<<<<<<
 * 
 *     cdef int *iwork = <int *>(work + lwork)
 */
  __pyx_v_work = (__pyx_v_eigv + __pyx_v_size);

  /* "_ellip_harm.pxd":104
 *     cdef double *work = eigv + size
 * 
 *     cdef int *iwork = <int *>(work + lwork)             # <<<<<<<<<<<<<<
 *     cdef int *isuppz = iwork + liwork
 * 
 */
  __pyx_v_iwork = ((int *)(__pyx_v_work + __pyx_v_lwork));

  /* "_ellip_harm.pxd":105
 * 
 *     cdef int *iwork = <int *>(work + lwork)
 *     cdef int *isuppz = iwork + liwork             # <<<<<<<<<<<<<<
 * 
 *     if t == 'K':
 */
  __pyx_v_isuppz = (__pyx_v_iwork + __pyx_v_liwork);

  /* "_ellip_harm.pxd":107
 *     cdef int *isuppz = iwork + liwork
 * 
 *     if t == 'K':             # <<<<<<<<<<<<<<
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 */
  switch (__pyx_v_t) {
    case 'K':

    /* "_ellip_harm.pxd":108
 * 
 *     if t == 'K':
 *         for j in range(0, r + 1):             # <<<<<<<<<<<<<<
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:
 */
    __pyx_t_5 = (__pyx_v_r + 1);
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_5; __pyx_t_4+=1) {
      __pyx_v_j = __pyx_t_4;

      /* "_ellip_harm.pxd":109
 *     if t == 'K':
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)             # <<<<<<<<<<<<<<
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 */
      (__pyx_v_g[__pyx_v_j]) = (((-((2 * __pyx_v_j) + 2)) * ((2 * __pyx_v_j) + 1)) * __pyx_v_beta);

      /* "_ellip_harm.pxd":110
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta
 */
      __pyx_t_1 = ((__pyx_v_n % 2) != 0);
      if (__pyx_t_1) {

        /* "_ellip_harm.pxd":111
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta
 *            else:
 */
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * ((2 * (__pyx_v_r - (__pyx_v_j + 1))) + 2)) * ((2 * ((__pyx_v_j + 1) + __pyx_v_r)) + 1));

        /* "_ellip_harm.pxd":112
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta             # <<<<<<<<<<<<<<
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*(r + (j + 1)) - 1))
 */
        (__pyx_v_d[__pyx_v_j]) = ((((((2 * __pyx_v_r) + 1) * ((2 * __pyx_v_r) + 2)) - ((4 * __pyx_v_j) * __pyx_v_j)) * __pyx_v_alpha) + ((((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1)) * __pyx_v_beta));

        /* "_ellip_harm.pxd":110
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta
 */
        goto __pyx_L14;
      }

      /* "_ellip_harm.pxd":114
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*(r + (j + 1)) - 1))             # <<<<<<<<<<<<<<
 *                d[j] = 2*r*(2*r + 1)*alpha - 4*j*j*gamma
 * 
 */
      /*else*/ {
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * ((2 * (__pyx_v_r - (__pyx_v_j + 1))) + 2)) * ((2 * (__pyx_v_r + (__pyx_v_j + 1))) - 1));

        /* "_ellip_harm.pxd":115
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*(r + (j + 1)) - 1))
 *                d[j] = 2*r*(2*r + 1)*alpha - 4*j*j*gamma             # <<<<<<<<<<<<<<
 * 
 *     elif t == 'L':
 */
        (__pyx_v_d[__pyx_v_j]) = ((((2 * __pyx_v_r) * ((2 * __pyx_v_r) + 1)) * __pyx_v_alpha) - (((4 * __pyx_v_j) * __pyx_v_j) * __pyx_v_gamma));
      }
      __pyx_L14:;
    }

    /* "_ellip_harm.pxd":107
 *     cdef int *isuppz = iwork + liwork
 * 
 *     if t == 'K':             # <<<<<<<<<<<<<<
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 */
    break;

    /* "_ellip_harm.pxd":117
 *                d[j] = 2*r*(2*r + 1)*alpha - 4*j*j*gamma
 * 
 *     elif t == 'L':             # <<<<<<<<<<<<<<
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 */
    case 'L':

    /* "_ellip_harm.pxd":118
 * 
 *     elif t == 'L':
 *         for j in range(0, n - r):             # <<<<<<<<<<<<<<
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:
 */
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_4; __pyx_t_6+=1) {
      __pyx_v_j = __pyx_t_6;

      /* "_ellip_harm.pxd":119
 *     elif t == 'L':
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)             # <<<<<<<<<<<<<<
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 */
      (__pyx_v_g[__pyx_v_j]) = (((-((2 * __pyx_v_j) + 2)) * ((2 * __pyx_v_j) + 3)) * __pyx_v_beta);

      /* "_ellip_harm.pxd":120
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma
 */
      __pyx_t_1 = ((__pyx_v_n % 2) != 0);
      if (__pyx_t_1) {

        /* "_ellip_harm.pxd":121
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma
 *            else:
 */
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * ((2 * (__pyx_v_r - (__pyx_v_j + 1))) + 2)) * ((2 * ((__pyx_v_j + 1) + __pyx_v_r)) + 1));

        /* "_ellip_harm.pxd":122
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma             # <<<<<<<<<<<<<<
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 */
        (__pyx_v_d[__pyx_v_j]) = (((((2 * __pyx_v_r) + 1) * ((2 * __pyx_v_r) + 2)) * __pyx_v_alpha) - ((((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1)) * __pyx_v_gamma));

        /* "_ellip_harm.pxd":120
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma
 */
        goto __pyx_L17;
      }

      /* "_ellip_harm.pxd":124
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = (2*r*(2*r + 1) - (2*j + 1)*(2*j + 1))*alpha + (2*j + 2)*(2*j + 2)*beta
 * 
 */
      /*else*/ {
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * (2 * (__pyx_v_r - (__pyx_v_j + 1)))) * ((2 * (__pyx_v_r + (__pyx_v_j + 1))) + 1));

        /* "_ellip_harm.pxd":125
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 *                d[j] = (2*r*(2*r + 1) - (2*j + 1)*(2*j + 1))*alpha + (2*j + 2)*(2*j + 2)*beta             # <<<<<<<<<<<<<<
 * 
 *     elif t == 'M':
 */
        (__pyx_v_d[__pyx_v_j]) = (((((2 * __pyx_v_r) * ((2 * __pyx_v_r) + 1)) - (((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1))) * __pyx_v_alpha) + ((((2 * __pyx_v_j) + 2) * ((2 * __pyx_v_j) + 2)) * __pyx_v_beta));
      }
      __pyx_L17:;
    }

    /* "_ellip_harm.pxd":117
 *                d[j] = 2*r*(2*r + 1)*alpha - 4*j*j*gamma
 * 
 *     elif t == 'L':             # <<<<<<<<<<<<<<
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 */
    break;

    /* "_ellip_harm.pxd":127
 *                d[j] = (2*r*(2*r + 1) - (2*j + 1)*(2*j + 1))*alpha + (2*j + 2)*(2*j + 2)*beta
 * 
 *     elif t == 'M':             # <<<<<<<<<<<<<<
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 */
    case 'M':

    /* "_ellip_harm.pxd":128
 * 
 *     elif t == 'M':
 *         for j in range(0, n - r):             # <<<<<<<<<<<<<<
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:
 */
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_4; __pyx_t_6+=1) {
      __pyx_v_j = __pyx_t_6;

      /* "_ellip_harm.pxd":129
 *     elif t == 'M':
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)             # <<<<<<<<<<<<<<
 *            if n%2:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 */
      (__pyx_v_g[__pyx_v_j]) = (((-((2 * __pyx_v_j) + 2)) * ((2 * __pyx_v_j) + 1)) * __pyx_v_beta);

      /* "_ellip_harm.pxd":130
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta
 */
      __pyx_t_1 = ((__pyx_v_n % 2) != 0);
      if (__pyx_t_1) {

        /* "_ellip_harm.pxd":131
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta
 *            else:
 */
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * ((2 * (__pyx_v_r - (__pyx_v_j + 1))) + 2)) * ((2 * ((__pyx_v_j + 1) + __pyx_v_r)) + 1));

        /* "_ellip_harm.pxd":132
 *            if n%2:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta             # <<<<<<<<<<<<<<
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 */
        (__pyx_v_d[__pyx_v_j]) = ((((((2 * __pyx_v_r) + 1) * ((2 * __pyx_v_r) + 2)) - (((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1))) * __pyx_v_alpha) + (((4 * __pyx_v_j) * __pyx_v_j) * __pyx_v_beta));

        /* "_ellip_harm.pxd":130
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta
 */
        goto __pyx_L20;
      }

      /* "_ellip_harm.pxd":134
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 1)*(2*j + 1)*gamma
 * 
 */
      /*else*/ {
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * (2 * (__pyx_v_r - (__pyx_v_j + 1)))) * ((2 * (__pyx_v_r + (__pyx_v_j + 1))) + 1));

        /* "_ellip_harm.pxd":135
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 1)*(2*j + 1)*gamma             # <<<<<<<<<<<<<<
 * 
 *     elif t == 'N':
 */
        (__pyx_v_d[__pyx_v_j]) = ((((2 * __pyx_v_r) * ((2 * __pyx_v_r) + 1)) * __pyx_v_alpha) - ((((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1)) * __pyx_v_gamma));
      }
      __pyx_L20:;
    }

    /* "_ellip_harm.pxd":127
 *                d[j] = (2*r*(2*r + 1) - (2*j + 1)*(2*j + 1))*alpha + (2*j + 2)*(2*j + 2)*beta
 * 
 *     elif t == 'M':             # <<<<<<<<<<<<<<
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 */
    break;

    /* "_ellip_harm.pxd":137
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 1)*(2*j + 1)*gamma
 * 
 *     elif t == 'N':             # <<<<<<<<<<<<<<
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 */
    case 'N':

    /* "_ellip_harm.pxd":138
 * 
 *     elif t == 'N':
 *         for j in range(0, r):             # <<<<<<<<<<<<<<
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:
 */
    __pyx_t_4 = __pyx_v_r;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_4; __pyx_t_6+=1) {
      __pyx_v_j = __pyx_t_6;

      /* "_ellip_harm.pxd":139
 *     elif t == 'N':
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)             # <<<<<<<<<<<<<<
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))
 */
      (__pyx_v_g[__pyx_v_j]) = (((-((2 * __pyx_v_j) + 2)) * ((2 * __pyx_v_j) + 3)) * __pyx_v_beta);

      /* "_ellip_harm.pxd":140
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma
 */
      __pyx_t_1 = ((__pyx_v_n % 2) != 0);
      if (__pyx_t_1) {

        /* "_ellip_harm.pxd":141
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))             # <<<<<<<<<<<<<<
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma
 *            else:
 */
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * (2 * (__pyx_v_r - (__pyx_v_j + 1)))) * ((2 * ((__pyx_v_j + 1) + __pyx_v_r)) + 3));

        /* "_ellip_harm.pxd":142
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma             # <<<<<<<<<<<<<<
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 */
        (__pyx_v_d[__pyx_v_j]) = (((((2 * __pyx_v_r) + 1) * ((2 * __pyx_v_r) + 2)) * __pyx_v_alpha) - ((((2 * __pyx_v_j) + 2) * ((2 * __pyx_v_j) + 2)) * __pyx_v_gamma));

        /* "_ellip_harm.pxd":140
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma
 */
        goto __pyx_L23;
      }

      /* "_ellip_harm.pxd":144
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 2)*(2*j +2)*alpha + (2*j + 1)*(2*j + 1)*beta
 * 
 */
      /*else*/ {
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * (2 * (__pyx_v_r - (__pyx_v_j + 1)))) * ((2 * (__pyx_v_r + (__pyx_v_j + 1))) + 1));

        /* "_ellip_harm.pxd":145
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 2)*(2*j +2)*alpha + (2*j + 1)*(2*j + 1)*beta             # <<<<<<<<<<<<<<
 * 
 *     for i in range(0, size):
 */
        (__pyx_v_d[__pyx_v_j]) = (((((2 * __pyx_v_r) * ((2 * __pyx_v_r) + 1)) * __pyx_v_alpha) - ((((2 * __pyx_v_j) + 2) * ((2 * __pyx_v_j) + 2)) * __pyx_v_alpha)) + ((((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1)) * __pyx_v_beta));
      }
      __pyx_L23:;
    }

    /* "_ellip_harm.pxd":137
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 1)*(2*j + 1)*gamma
 * 
 *     elif t == 'N':             # <<<<<<<<<<<<<<
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 */
    break;
    default: break;
  }

  /* "_ellip_harm.pxd":147
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 2)*(2*j +2)*alpha + (2*j + 1)*(2*j + 1)*beta
 * 
 *     for i in range(0, size):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *            ss[i] = 1
 */
  __pyx_t_4 = __pyx_v_size;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_4; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "_ellip_harm.pxd":148
 * 
 *     for i in range(0, size):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *            ss[i] = 1
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_i == 0) != 0);
    if (__pyx_t_1) {

      /* "_ellip_harm.pxd":149
 *     for i in range(0, size):
 *         if i == 0:
 *            ss[i] = 1             # <<<<<<<<<<<<<<
 *         else:
 *            ss[i] = sqrt(g[i - 1]/f[i - 1])*ss[i - 1]
 */
      (__pyx_v_ss[__pyx_v_i]) = 1.0;

      /* "_ellip_harm.pxd":148
 * 
 *     for i in range(0, size):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *            ss[i] = 1
 *         else:
 */
      goto __pyx_L26;
    }

    /* "_ellip_harm.pxd":151
 *            ss[i] = 1
 *         else:
 *            ss[i] = sqrt(g[i - 1]/f[i - 1])*ss[i - 1]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(0, size-1):
 */
    /*else*/ {
      (__pyx_v_ss[__pyx_v_i]) = (sqrt(((__pyx_v_g[(__pyx_v_i - 1)]) / (__pyx_v_f[(__pyx_v_i - 1)]))) * (__pyx_v_ss[(__pyx_v_i - 1)]));
    }
    __pyx_L26:;
  }

  /* "_ellip_harm.pxd":153
 *            ss[i] = sqrt(g[i - 1]/f[i - 1])*ss[i - 1]
 * 
 *     for i in range(0, size-1):             # <<<<<<<<<<<<<<
 *         dd[i] = g[i]*ss[i]/ss[i+1]
 * 
 */
  __pyx_t_5 = (__pyx_v_size - 1);
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_5; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "_ellip_harm.pxd":154
 * 
 *     for i in range(0, size-1):
 *         dd[i] = g[i]*ss[i]/ss[i+1]             # <<<<<<<<<<<<<<
 * 
 *     c_dstevr("V", "I", &size, d, dd, &vl, &vu, &tp, &tp, &tol, &c, w, eigv,
 */
    (__pyx_v_dd[__pyx_v_i]) = (((__pyx_v_g[__pyx_v_i]) * (__pyx_v_ss[__pyx_v_i])) / (__pyx_v_ss[(__pyx_v_i + 1)]));
  }

  /* "_ellip_harm.pxd":156
 *         dd[i] = g[i]*ss[i]/ss[i+1]
 * 
 *     c_dstevr("V", "I", &size, d, dd, &vl, &vu, &tp, &tp, &tol, &c, w, eigv,             # <<<<<<<<<<<<<<
 *              &size, isuppz, work, &lwork, iwork, &liwork, &info)
 * 
 */
  c_dstevr(__pyx_k_V, __pyx_k_I, (&__pyx_v_size), __pyx_v_d, __pyx_v_dd, (&__pyx_v_vl), (&__pyx_v_vu), (&__pyx_v_tp), (&__pyx_v_tp), (&__pyx_v_tol), (&__pyx_v_c), __pyx_v_w, __pyx_v_eigv, (&__pyx_v_size), __pyx_v_isuppz, __pyx_v_work, (&__pyx_v_lwork), __pyx_v_iwork, (&__pyx_v_liwork), (&__pyx_v_info));

  /* "_ellip_harm.pxd":159
 *              &size, isuppz, work, &lwork, iwork, &liwork, &info)
 * 
 *     if info != 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL
 */
  __pyx_t_1 = ((__pyx_v_info != 0) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":160
 * 
 *     if info != 0:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(__pyx_k_ellip_harm, SF_ERROR_NO_RESULT, __pyx_k_failed_to_allocate_memory);

    /* "_ellip_harm.pxd":161
 *     if info != 0:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     for i in range(0, size):
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":159
 *              &size, isuppz, work, &lwork, iwork, &liwork, &info)
 * 
 *     if info != 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":163
 *         return NULL
 * 
 *     for i in range(0, size):             # <<<<<<<<<<<<<<
 *         eigv[i] /= ss[i]
 * 
 */
  __pyx_t_4 = __pyx_v_size;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_4; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "_ellip_harm.pxd":164
 * 
 *     for i in range(0, size):
 *         eigv[i] /= ss[i]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(0, size):
 */
    __pyx_t_7 = __pyx_v_i;
    (__pyx_v_eigv[__pyx_t_7]) = ((__pyx_v_eigv[__pyx_t_7]) / (__pyx_v_ss[__pyx_v_i]));
  }

  /* "_ellip_harm.pxd":166
 *         eigv[i] /= ss[i]
 * 
 *     for i in range(0, size):             # <<<<<<<<<<<<<<
 *         eigv[i] = eigv[i]/(eigv[size - 1]/pow(-h2, size - 1))
 *     return eigv
 */
  __pyx_t_4 = __pyx_v_size;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_4; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "_ellip_harm.pxd":167
 * 
 *     for i in range(0, size):
 *         eigv[i] = eigv[i]/(eigv[size - 1]/pow(-h2, size - 1))             # <<<<<<<<<<<<<<
 *     return eigv
 * 
 */
    (__pyx_v_eigv[__pyx_v_i]) = ((__pyx_v_eigv[__pyx_v_i]) / ((__pyx_v_eigv[(__pyx_v_size - 1)]) / pow((-__pyx_v_h2), (__pyx_v_size - 1))));
  }

  /* "_ellip_harm.pxd":168
 *     for i in range(0, size):
 *         eigv[i] = eigv[i]/(eigv[size - 1]/pow(-h2, size - 1))
 *     return eigv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_eigv;
  goto __pyx_L0;

  /* "_ellip_harm.pxd":49
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * cdef inline double* lame_coefficients(double h2, double k2, int n, int p,             # <<<<<<<<<<<<<<
 *                                       void **bufferp, double signm,
 *                                       double signn) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_ellip_harm.pxd":174
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * cdef inline double ellip_harm_eval(double h2, double k2, int n, int p,             # <<<<<<<<<<<<<<
 *                                    double s, double *eigv, double signm,
 *                                    double signn) nogil:
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_11_ellip_harm_ellip_harm_eval(double __pyx_v_h2, double __pyx_v_k2, int __pyx_v_n, int __pyx_v_p, double __pyx_v_s, double *__pyx_v_eigv, double __pyx_v_signm, double __pyx_v_signn) {
  int __pyx_v_size;
  int __pyx_v_r;
  int __pyx_v_j;
  double __pyx_v_s2;
  double __pyx_v_pp;
  double __pyx_v_lambda_romain;
  double __pyx_v_psi;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  double __pyx_t_3;
  int __pyx_t_4;

  /* "_ellip_harm.pxd":179
 *     cdef int size, tp, r, j
 *     cdef double s2, pp, lambda_romain, psi
 *     s2 = s*s             # <<<<<<<<<<<<<<
 *     r = n/2
 *     if p - 1 < r + 1:
 */
  __pyx_v_s2 = (__pyx_v_s * __pyx_v_s);

  /* "_ellip_harm.pxd":180
 *     cdef double s2, pp, lambda_romain, psi
 *     s2 = s*s
 *     r = n/2             # <<<<<<<<<<<<<<
 *     if p - 1 < r + 1:
 *         size, psi = r + 1, pow(s, n - 2*r)
 */
  __pyx_v_r = __Pyx_div_long(__pyx_v_n, 2);

  /* "_ellip_harm.pxd":181
 *     s2 = s*s
 *     r = n/2
 *     if p - 1 < r + 1:             # <<<<<<<<<<<<<<
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < (__pyx_v_r + 1)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":182
 *     r = n/2
 *     if p - 1 < r + 1:
 *         size, psi = r + 1, pow(s, n - 2*r)             # <<<<<<<<<<<<<<
 *     elif p - 1 < (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 */
    __pyx_t_2 = (__pyx_v_r + 1);
    __pyx_t_3 = pow(__pyx_v_s, (__pyx_v_n - (2 * __pyx_v_r)));
    __pyx_v_size = __pyx_t_2;
    __pyx_v_psi = __pyx_t_3;

    /* "_ellip_harm.pxd":181
 *     s2 = s*s
 *     r = n/2
 *     if p - 1 < r + 1:             # <<<<<<<<<<<<<<
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):
 */
    goto __pyx_L3;
  }

  /* "_ellip_harm.pxd":183
 *     if p - 1 < r + 1:
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < ((__pyx_v_n - __pyx_v_r) + (__pyx_v_r + 1))) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":184
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))             # <<<<<<<<<<<<<<
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 */
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_t_3 = ((pow(__pyx_v_s, ((1 - __pyx_v_n) + (2 * __pyx_v_r))) * __pyx_v_signm) * sqrt(fabs((__pyx_v_s2 - __pyx_v_h2))));
    __pyx_v_size = __pyx_t_4;
    __pyx_v_psi = __pyx_t_3;

    /* "_ellip_harm.pxd":183
 *     if p - 1 < r + 1:
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 */
    goto __pyx_L3;
  }

  /* "_ellip_harm.pxd":185
 *     elif p - 1 < (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < (((__pyx_v_n - __pyx_v_r) + (__pyx_v_n - __pyx_v_r)) + (__pyx_v_r + 1))) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":186
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))             # <<<<<<<<<<<<<<
 *     elif p - 1 < 2*n + 1:
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 */
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_t_3 = ((pow(__pyx_v_s, ((1 - __pyx_v_n) + (2 * __pyx_v_r))) * __pyx_v_signn) * sqrt(fabs((__pyx_v_s2 - __pyx_v_k2))));
    __pyx_v_size = __pyx_t_4;
    __pyx_v_psi = __pyx_t_3;

    /* "_ellip_harm.pxd":185
 *     elif p - 1 < (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:
 */
    goto __pyx_L3;
  }

  /* "_ellip_harm.pxd":187
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:             # <<<<<<<<<<<<<<
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 *     lambda_romain = 1.0 - <double>s2/<double>h2
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < ((2 * __pyx_v_n) + 1)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":188
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))             # <<<<<<<<<<<<<<
 *     lambda_romain = 1.0 - <double>s2/<double>h2
 *     pp = eigv[size - 1]
 */
    __pyx_t_4 = __pyx_v_r;
    __pyx_t_3 = (((pow(__pyx_v_s, (__pyx_v_n - (2 * __pyx_v_r))) * __pyx_v_signm) * __pyx_v_signn) * sqrt(fabs(((__pyx_v_s2 - __pyx_v_h2) * (__pyx_v_s2 - __pyx_v_k2)))));
    __pyx_v_size = __pyx_t_4;
    __pyx_v_psi = __pyx_t_3;

    /* "_ellip_harm.pxd":187
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:             # <<<<<<<<<<<<<<
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 *     lambda_romain = 1.0 - <double>s2/<double>h2
 */
  }
  __pyx_L3:;

  /* "_ellip_harm.pxd":189
 *     elif p - 1 < 2*n + 1:
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 *     lambda_romain = 1.0 - <double>s2/<double>h2             # <<<<<<<<<<<<<<
 *     pp = eigv[size - 1]
 * 
 */
  __pyx_v_lambda_romain = (1.0 - (((double)__pyx_v_s2) / ((double)__pyx_v_h2)));

  /* "_ellip_harm.pxd":190
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 *     lambda_romain = 1.0 - <double>s2/<double>h2
 *     pp = eigv[size - 1]             # <<<<<<<<<<<<<<
 * 
 *     for j in range(size - 2, -1, -1):
 */
  __pyx_v_pp = (__pyx_v_eigv[(__pyx_v_size - 1)]);

  /* "_ellip_harm.pxd":192
 *     pp = eigv[size - 1]
 * 
 *     for j in range(size - 2, -1, -1):             # <<<<<<<<<<<<<<
 *         pp = pp*lambda_romain + eigv[j]
 *     pp = pp*psi
 */
  for (__pyx_t_4 = (__pyx_v_size - 2); __pyx_t_4 > -1L; __pyx_t_4-=1) {
    __pyx_v_j = __pyx_t_4;

    /* "_ellip_harm.pxd":193
 * 
 *     for j in range(size - 2, -1, -1):
 *         pp = pp*lambda_romain + eigv[j]             # <<<<<<<<<<<<<<
 *     pp = pp*psi
 *     return pp
 */
    __pyx_v_pp = ((__pyx_v_pp * __pyx_v_lambda_romain) + (__pyx_v_eigv[__pyx_v_j]));
  }

  /* "_ellip_harm.pxd":194
 *     for j in range(size - 2, -1, -1):
 *         pp = pp*lambda_romain + eigv[j]
 *     pp = pp*psi             # <<<<<<<<<<<<<<
 *     return pp
 * 
 */
  __pyx_v_pp = (__pyx_v_pp * __pyx_v_psi);

  /* "_ellip_harm.pxd":195
 *         pp = pp*lambda_romain + eigv[j]
 *     pp = pp*psi
 *     return pp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_pp;
  goto __pyx_L0;

  /* "_ellip_harm.pxd":174
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * cdef inline double ellip_harm_eval(double h2, double k2, int n, int p,             # <<<<<<<<<<<<<<
 *                                    double s, double *eigv, double signm,
 *                                    double signn) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_ellip_harm.pxd":198
 * 
 * 
 * cdef inline double ellip_harmonic(double h2, double k2, int n, int p, double s,             # <<<<<<<<<<<<<<
 *                                   double signm, double signn) nogil:
 *     cdef double result
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic(double __pyx_v_h2, double __pyx_v_k2, int __pyx_v_n, int __pyx_v_p, double __pyx_v_s, double __pyx_v_signm, double __pyx_v_signn) {
  double __pyx_v_result;
  double *__pyx_v_eigv;
  void *__pyx_v_bufferp;
  double __pyx_r;
  int __pyx_t_1;

  /* "_ellip_harm.pxd":203
 *     cdef double *eigv
 *     cdef void *bufferp
 *     eigv = lame_coefficients(h2, k2, n, p, &bufferp, signm, signn)             # <<<<<<<<<<<<<<
 *     if not eigv:
 *         free(bufferp)
 */
  __pyx_v_eigv = __pyx_f_5scipy_7special_11_ellip_harm_lame_coefficients(__pyx_v_h2, __pyx_v_k2, __pyx_v_n, __pyx_v_p, (&__pyx_v_bufferp), __pyx_v_signm, __pyx_v_signn);

  /* "_ellip_harm.pxd":204
 *     cdef void *bufferp
 *     eigv = lame_coefficients(h2, k2, n, p, &bufferp, signm, signn)
 *     if not eigv:             # <<<<<<<<<<<<<<
 *         free(bufferp)
 *         return NAN
 */
  __pyx_t_1 = ((!(__pyx_v_eigv != 0)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":205
 *     eigv = lame_coefficients(h2, k2, n, p, &bufferp, signm, signn)
 *     if not eigv:
 *         free(bufferp)             # <<<<<<<<<<<<<<
 *         return NAN
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)
 */
    free(__pyx_v_bufferp);

    /* "_ellip_harm.pxd":206
 *     if not eigv:
 *         free(bufferp)
 *         return NAN             # <<<<<<<<<<<<<<
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)
 *     free(bufferp)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":204
 *     cdef void *bufferp
 *     eigv = lame_coefficients(h2, k2, n, p, &bufferp, signm, signn)
 *     if not eigv:             # <<<<<<<<<<<<<<
 *         free(bufferp)
 *         return NAN
 */
  }

  /* "_ellip_harm.pxd":207
 *         free(bufferp)
 *         return NAN
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)             # <<<<<<<<<<<<<<
 *     free(bufferp)
 *     return result
 */
  __pyx_v_result = __pyx_f_5scipy_7special_11_ellip_harm_ellip_harm_eval(__pyx_v_h2, __pyx_v_k2, __pyx_v_n, __pyx_v_p, __pyx_v_s, __pyx_v_eigv, __pyx_v_signm, __pyx_v_signn);

  /* "_ellip_harm.pxd":208
 *         return NAN
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)
 *     free(bufferp)             # <<<<<<<<<<<<<<
 *     return result
 */
  free(__pyx_v_bufferp);

  /* "_ellip_harm.pxd":209
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)
 *     free(bufferp)
 *     return result             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "_ellip_harm.pxd":198
 * 
 * 
 * cdef inline double ellip_harmonic(double h2, double k2, int n, int p, double s,             # <<<<<<<<<<<<<<
 *                                   double signm, double signn) nogil:
 *     cdef double result
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_exprel.pxd":7
 * from libc.math cimport expm1, fabs, log
 * 
 * cdef inline double exprel(double x) nogil:             # <<<<<<<<<<<<<<
 *     if fabs(x) < 1e-16:
 *         return 1.0
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_exprel_exprel(double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "_exprel.pxd":8
 * 
 * cdef inline double exprel(double x) nogil:
 *     if fabs(x) < 1e-16:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)
 */
  __pyx_t_1 = ((fabs(__pyx_v_x) < 1e-16) != 0);
  if (__pyx_t_1) {

    /* "_exprel.pxd":9
 * cdef inline double exprel(double x) nogil:
 *     if fabs(x) < 1e-16:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif x > 717:  # near log(DBL_MAX)
 *         return NPY_INFINITY
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "_exprel.pxd":8
 * 
 * cdef inline double exprel(double x) nogil:
 *     if fabs(x) < 1e-16:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)
 */
  }

  /* "_exprel.pxd":10
 *     if fabs(x) < 1e-16:
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)             # <<<<<<<<<<<<<<
 *         return NPY_INFINITY
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_x > 717.0) != 0);
  if (__pyx_t_1) {

    /* "_exprel.pxd":11
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)
 *         return NPY_INFINITY             # <<<<<<<<<<<<<<
 *     else:
 *         return expm1(x) / x
 */
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;

    /* "_exprel.pxd":10
 *     if fabs(x) < 1e-16:
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)             # <<<<<<<<<<<<<<
 *         return NPY_INFINITY
 *     else:
 */
  }

  /* "_exprel.pxd":13
 *         return NPY_INFINITY
 *     else:
 *         return expm1(x) / x             # <<<<<<<<<<<<<<
 * 
 */
  /*else*/ {
    __pyx_t_2 = expm1(__pyx_v_x);
    if (unlikely(__pyx_v_x == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 13; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_r = (__pyx_t_2 / __pyx_v_x);
    goto __pyx_L0;
  }

  /* "_exprel.pxd":7
 * from libc.math cimport expm1, fabs, log
 * 
 * cdef inline double exprel(double x) nogil:             # <<<<<<<<<<<<<<
 *     if fabs(x) < 1e-16:
 *         return 1.0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._exprel.exprel", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":6
 * from libc.math cimport log, sqrt, fabs
 * 
 * cdef inline double entr(double x) nogil:             # <<<<<<<<<<<<<<
 *     if x > 0:
 *         return -x * log(x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_entr(double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_convex_analysis.pxd":7
 * 
 * cdef inline double entr(double x) nogil:
 *     if x > 0:             # <<<<<<<<<<<<<<
 *         return -x * log(x)
 *     elif x == 0:
 */
  __pyx_t_1 = ((__pyx_v_x > 0.0) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":8
 * cdef inline double entr(double x) nogil:
 *     if x > 0:
 *         return -x * log(x)             # <<<<<<<<<<<<<<
 *     elif x == 0:
 *         return 0
 */
    __pyx_r = ((-__pyx_v_x) * log(__pyx_v_x));
    goto __pyx_L0;

    /* "_convex_analysis.pxd":7
 * 
 * cdef inline double entr(double x) nogil:
 *     if x > 0:             # <<<<<<<<<<<<<<
 *         return -x * log(x)
 *     elif x == 0:
 */
  }

  /* "_convex_analysis.pxd":9
 *     if x > 0:
 *         return -x * log(x)
 *     elif x == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":10
 *         return -x * log(x)
 *     elif x == 0:
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return -inf
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":9
 *     if x > 0:
 *         return -x * log(x)
 *     elif x == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_convex_analysis.pxd":12
 *         return 0
 *     else:
 *         return -inf             # <<<<<<<<<<<<<<
 * 
 * cdef inline double kl_div(double x, double y) nogil:
 */
  /*else*/ {
    __pyx_r = (-NPY_INFINITY);
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":6
 * from libc.math cimport log, sqrt, fabs
 * 
 * cdef inline double entr(double x) nogil:             # <<<<<<<<<<<<<<
 *     if x > 0:
 *         return -x * log(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":14
 *         return -inf
 * 
 * cdef inline double kl_div(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if x > 0 and y > 0:
 *         return x * log(x / y) - x + y
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_kl_div(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "_convex_analysis.pxd":15
 * 
 * cdef inline double kl_div(double x, double y) nogil:
 *     if x > 0 and y > 0:             # <<<<<<<<<<<<<<
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:
 */
  __pyx_t_2 = ((__pyx_v_x > 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_y > 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":16
 * cdef inline double kl_div(double x, double y) nogil:
 *     if x > 0 and y > 0:
 *         return x * log(x / y) - x + y             # <<<<<<<<<<<<<<
 *     elif x == 0 and y >= 0:
 *         return y
 */
    if (unlikely(__pyx_v_y == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_r = (((__pyx_v_x * log((__pyx_v_x / __pyx_v_y))) - __pyx_v_x) + __pyx_v_y);
    goto __pyx_L0;

    /* "_convex_analysis.pxd":15
 * 
 * cdef inline double kl_div(double x, double y) nogil:
 *     if x > 0 and y > 0:             # <<<<<<<<<<<<<<
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:
 */
  }

  /* "_convex_analysis.pxd":17
 *     if x > 0 and y > 0:
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:             # <<<<<<<<<<<<<<
 *         return y
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_y >= 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":18
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:
 *         return y             # <<<<<<<<<<<<<<
 *     else:
 *         return inf
 */
    __pyx_r = __pyx_v_y;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":17
 *     if x > 0 and y > 0:
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:             # <<<<<<<<<<<<<<
 *         return y
 *     else:
 */
  }

  /* "_convex_analysis.pxd":20
 *         return y
 *     else:
 *         return inf             # <<<<<<<<<<<<<<
 * 
 * cdef inline double rel_entr(double x, double y) nogil:
 */
  /*else*/ {
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":14
 *         return -inf
 * 
 * cdef inline double kl_div(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if x > 0 and y > 0:
 *         return x * log(x / y) - x + y
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._convex_analysis.kl_div", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":22
 *         return inf
 * 
 * cdef inline double rel_entr(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if x > 0 and y > 0:
 *         return x * log(x / y)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_rel_entr(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "_convex_analysis.pxd":23
 * 
 * cdef inline double rel_entr(double x, double y) nogil:
 *     if x > 0 and y > 0:             # <<<<<<<<<<<<<<
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:
 */
  __pyx_t_2 = ((__pyx_v_x > 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_y > 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":24
 * cdef inline double rel_entr(double x, double y) nogil:
 *     if x > 0 and y > 0:
 *         return x * log(x / y)             # <<<<<<<<<<<<<<
 *     elif x == 0 and y >= 0:
 *         return 0
 */
    if (unlikely(__pyx_v_y == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_r = (__pyx_v_x * log((__pyx_v_x / __pyx_v_y)));
    goto __pyx_L0;

    /* "_convex_analysis.pxd":23
 * 
 * cdef inline double rel_entr(double x, double y) nogil:
 *     if x > 0 and y > 0:             # <<<<<<<<<<<<<<
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:
 */
  }

  /* "_convex_analysis.pxd":25
 *     if x > 0 and y > 0:
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_y >= 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":26
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return inf
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":25
 *     if x > 0 and y > 0:
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_convex_analysis.pxd":28
 *         return 0
 *     else:
 *         return inf             # <<<<<<<<<<<<<<
 * 
 * cdef inline double huber(double delta, double r) nogil:
 */
  /*else*/ {
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":22
 *         return inf
 * 
 * cdef inline double rel_entr(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if x > 0 and y > 0:
 *         return x * log(x / y)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._convex_analysis.rel_entr", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":30
 *         return inf
 * 
 * cdef inline double huber(double delta, double r) nogil:             # <<<<<<<<<<<<<<
 *     if delta < 0:
 *         return inf
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_huber(double __pyx_v_delta, double __pyx_v_r) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_convex_analysis.pxd":31
 * 
 * cdef inline double huber(double delta, double r) nogil:
 *     if delta < 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     elif fabs(r) <= delta:
 */
  __pyx_t_1 = ((__pyx_v_delta < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":32
 * cdef inline double huber(double delta, double r) nogil:
 *     if delta < 0:
 *         return inf             # <<<<<<<<<<<<<<
 *     elif fabs(r) <= delta:
 *         return 0.5 * r * r;
 */
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":31
 * 
 * cdef inline double huber(double delta, double r) nogil:
 *     if delta < 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     elif fabs(r) <= delta:
 */
  }

  /* "_convex_analysis.pxd":33
 *     if delta < 0:
 *         return inf
 *     elif fabs(r) <= delta:             # <<<<<<<<<<<<<<
 *         return 0.5 * r * r;
 *     else:
 */
  __pyx_t_1 = ((fabs(__pyx_v_r) <= __pyx_v_delta) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":34
 *         return inf
 *     elif fabs(r) <= delta:
 *         return 0.5 * r * r;             # <<<<<<<<<<<<<<
 *     else:
 *         return delta * (fabs(r) - 0.5 * delta);
 */
    __pyx_r = ((0.5 * __pyx_v_r) * __pyx_v_r);
    goto __pyx_L0;

    /* "_convex_analysis.pxd":33
 *     if delta < 0:
 *         return inf
 *     elif fabs(r) <= delta:             # <<<<<<<<<<<<<<
 *         return 0.5 * r * r;
 *     else:
 */
  }

  /* "_convex_analysis.pxd":36
 *         return 0.5 * r * r;
 *     else:
 *         return delta * (fabs(r) - 0.5 * delta);             # <<<<<<<<<<<<<<
 * 
 * cdef inline double pseudo_huber(double delta, double r) nogil:
 */
  /*else*/ {
    __pyx_r = (__pyx_v_delta * (fabs(__pyx_v_r) - (0.5 * __pyx_v_delta)));
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":30
 *         return inf
 * 
 * cdef inline double huber(double delta, double r) nogil:             # <<<<<<<<<<<<<<
 *     if delta < 0:
 *         return inf
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":38
 *         return delta * (fabs(r) - 0.5 * delta);
 * 
 * cdef inline double pseudo_huber(double delta, double r) nogil:             # <<<<<<<<<<<<<<
 *     cdef double u, v
 *     if delta < 0:
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber(double __pyx_v_delta, double __pyx_v_r) {
  double __pyx_v_u;
  double __pyx_v_v;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "_convex_analysis.pxd":40
 * cdef inline double pseudo_huber(double delta, double r) nogil:
 *     cdef double u, v
 *     if delta < 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     elif delta == 0 or r == 0:
 */
  __pyx_t_1 = ((__pyx_v_delta < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":41
 *     cdef double u, v
 *     if delta < 0:
 *         return inf             # <<<<<<<<<<<<<<
 *     elif delta == 0 or r == 0:
 *         return 0
 */
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":40
 * cdef inline double pseudo_huber(double delta, double r) nogil:
 *     cdef double u, v
 *     if delta < 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     elif delta == 0 or r == 0:
 */
  }

  /* "_convex_analysis.pxd":42
 *     if delta < 0:
 *         return inf
 *     elif delta == 0 or r == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_delta == 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_r == 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":43
 *         return inf
 *     elif delta == 0 or r == 0:
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         u = delta
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":42
 *     if delta < 0:
 *         return inf
 *     elif delta == 0 or r == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_convex_analysis.pxd":45
 *         return 0
 *     else:
 *         u = delta             # <<<<<<<<<<<<<<
 *         v = r / delta
 *         return u*u*(sqrt(1 + v*v) - 1)
 */
  /*else*/ {
    __pyx_v_u = __pyx_v_delta;

    /* "_convex_analysis.pxd":46
 *     else:
 *         u = delta
 *         v = r / delta             # <<<<<<<<<<<<<<
 *         return u*u*(sqrt(1 + v*v) - 1)
 */
    if (unlikely(__pyx_v_delta == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 46; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_v = (__pyx_v_r / __pyx_v_delta);

    /* "_convex_analysis.pxd":47
 *         u = delta
 *         v = r / delta
 *         return u*u*(sqrt(1 + v*v) - 1)             # <<<<<<<<<<<<<<
 */
    __pyx_r = ((__pyx_v_u * __pyx_v_u) * (sqrt((1.0 + (__pyx_v_v * __pyx_v_v))) - 1.0));
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":38
 *         return delta * (fabs(r) - 0.5 * delta);
 * 
 * cdef inline double pseudo_huber(double delta, double r) nogil:             # <<<<<<<<<<<<<<
 *     cdef double u, v
 *     if delta < 0:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._convex_analysis.pseudo_huber", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_copy_shape;
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_v_hasfields;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":203
 *             # of flags
 * 
 *             if info == NULL: return             # <<<<<<<<<<<<<<
 * 
 *             cdef int copy_shape, i, ndim
 */
  __pyx_t_1 = ((__pyx_v_info == NULL) != 0);
  if (__pyx_t_1) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":206
 * 
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":207
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":209
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":212
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 copy_shape = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 copy_shape = 0
 */
    __pyx_v_copy_shape = 1;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
    goto __pyx_L4;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":214
 *                 copy_shape = 1
 *             else:
 *                 copy_shape = 0             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  /*else*/ {
    __pyx_v_copy_shape = 0;
  }
  __pyx_L4:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":217
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[4]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":221
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[4]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":224
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if copy_shape:
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":225
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if copy_shape:
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (__pyx_v_copy_shape != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":229
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)malloc((((sizeof(Py_ssize_t)) * ((size_t)__pyx_v_ndim)) * 2)));

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":230
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":231
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":232
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":233
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L11;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":235
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":236
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L11:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":237
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":238
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":239
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":242
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = self.descr
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":243
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = self.descr             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->descr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":246
 *             cdef int offset
 * 
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields and not copy_shape:
 */
  __pyx_v_hasfields = PyDataType_HASFIELDS(__pyx_v_descr);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
  __pyx_t_2 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_copy_shape != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":250
 *             if not hasfields and not copy_shape:
 *                 # do not call releasebuffer
 *                 info.obj = None             # <<<<<<<<<<<<<<
 *             else:
 *                 # need to call releasebuffer
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = Py_None;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
    goto __pyx_L14;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":253
 *             else:
 *                 # need to call releasebuffer
 *                 info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields:
 */
  /*else*/ {
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = ((PyObject *)__pyx_v_self);
  }
  __pyx_L14:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":256
 * 
 *             if not hasfields:
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L20_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_L20_next_or:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L19_bool_binop_done:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_1) {

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[4]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":260
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = __pyx_k_b;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":261
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      case NPY_UBYTE:
      __pyx_v_f = __pyx_k_B;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":262
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      case NPY_SHORT:
      __pyx_v_f = __pyx_k_h;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":263
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      case NPY_USHORT:
      __pyx_v_f = __pyx_k_H;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":264
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      case NPY_INT:
      __pyx_v_f = __pyx_k_i;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":265
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      case NPY_UINT:
      __pyx_v_f = __pyx_k_I;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      case NPY_LONG:
      __pyx_v_f = __pyx_k_l;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":267
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      case NPY_ULONG:
      __pyx_v_f = __pyx_k_L;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      case NPY_LONGLONG:
      __pyx_v_f = __pyx_k_q;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":269
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      case NPY_ULONGLONG:
      __pyx_v_f = __pyx_k_Q;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      case NPY_FLOAT:
      __pyx_v_f = __pyx_k_f;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":271
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      case NPY_DOUBLE:
      __pyx_v_f = __pyx_k_d;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      case NPY_LONGDOUBLE:
      __pyx_v_f = __pyx_k_g;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":273
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      case NPY_CFLOAT:
      __pyx_v_f = __pyx_k_Zf;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      case NPY_CDOUBLE:
      __pyx_v_f = __pyx_k_Zd;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":275
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      case NPY_CLONGDOUBLE:
      __pyx_v_f = __pyx_k_Zg;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      case NPY_OBJECT:
      __pyx_v_f = __pyx_k_O;
      break;
      default:

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[4]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      break;
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":279
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":282
 *                 return
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)malloc(0xFF));

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_7 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_f = __pyx_t_7;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":288
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":292
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)
 */
    free(__pyx_v_info->format);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":294
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    free(__pyx_v_info->strides);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":771
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":774
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 774; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":780
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 780; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":783
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 783; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":790
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":791
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[4]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":795
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[4]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":796
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[4]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[4]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (__pyx_t_6) {

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[4]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_6) {

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[4]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":813
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":814
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":815
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":816
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":818
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (__pyx_t_6) {

        /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        {__pyx_filename = __pyx_f[4]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":826
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":828
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":829
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":831
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":832
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":834
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":835
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":837
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":838
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":839
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":841
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":844
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        {__pyx_filename = __pyx_f[4]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L15:;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":845
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":849
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":850
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  PyObject *__pyx_v_baseptr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
  __pyx_t_1 = (__pyx_v_base == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":969
 *      cdef PyObject* baseptr
 *      if base is None:
 *          baseptr = NULL             # <<<<<<<<<<<<<<
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 */
    __pyx_v_baseptr = NULL;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
    goto __pyx_L3;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":971
 *          baseptr = NULL
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!             # <<<<<<<<<<<<<<
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 */
  /*else*/ {
    Py_INCREF(__pyx_v_base);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":972
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base             # <<<<<<<<<<<<<<
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr
 */
    __pyx_v_baseptr = ((PyObject *)__pyx_v_base);
  }
  __pyx_L3:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":973
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)             # <<<<<<<<<<<<<<
 *      arr.base = baseptr
 * 
 */
  Py_XDECREF(__pyx_v_arr->base);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":974
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_v_arr->base = __pyx_v_baseptr;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arr->base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":978
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return <object>arr.base
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":980
 *         return None
 *     else:
 *         return <object>arr.base             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_arr->base));
    __pyx_r = ((PyObject *)__pyx_v_arr->base);
    goto __pyx_L0;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_xlogy.pxd":5
 * from _complexstuff cimport zlog, npy_log1p, zisnan, number_t
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_xlogy.pxd":6
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_y) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_xlogy.pxd":7
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return x * zlog(y)
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_xlogy.pxd":6
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_xlogy.pxd":9
 *         return 0
 *     else:
 *         return x * zlog(y)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double xlog1py(double x, double y) nogil:
 */
  /*else*/ {
    __pyx_r = (__pyx_v_x * __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_y));
    goto __pyx_L0;
  }

  /* "_xlogy.pxd":5
 * from _complexstuff cimport zlog, npy_log1p, zisnan, number_t
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy(__pyx_t_double_complex __pyx_v_x, __pyx_t_double_complex __pyx_v_y) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_xlogy.pxd":6
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__Pyx_c_eq(__pyx_v_x, __pyx_t_double_complex_from_parts(0, 0))) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_y) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_xlogy.pxd":7
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return x * zlog(y)
 */
    __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
    goto __pyx_L0;

    /* "_xlogy.pxd":6
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_xlogy.pxd":9
 *         return 0
 *     else:
 *         return x * zlog(y)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double xlog1py(double x, double y) nogil:
 */
  /*else*/ {
    __pyx_r = __Pyx_c_prod(__pyx_v_x, __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_y));
    goto __pyx_L0;
  }

  /* "_xlogy.pxd":5
 * from _complexstuff cimport zlog, npy_log1p, zisnan, number_t
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_xlogy.pxd":11
 *         return x * zlog(y)
 * 
 * cdef inline double xlog1py(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_6_xlogy_xlog1py(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_xlogy.pxd":12
 * 
 * cdef inline double xlog1py(double x, double y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_y) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_xlogy.pxd":13
 * cdef inline double xlog1py(double x, double y) nogil:
 *     if x == 0 and not zisnan(y):
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return x * npy_log1p(y)
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_xlogy.pxd":12
 * 
 * cdef inline double xlog1py(double x, double y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_xlogy.pxd":15
 *         return 0
 *     else:
 *         return x * npy_log1p(y)             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __pyx_r = (__pyx_v_x * npy_log1p(__pyx_v_y));
    goto __pyx_L0;
  }

  /* "_xlogy.pxd":11
 *         return x * zlog(y)
 * 
 * cdef inline double xlog1py(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":40
 *     int PyErr_WarnEx_noerr "PyErr_WarnEx" (object, char *, int)
 * 
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if <int>x != x or <int>y != y:
 *         with gil:
 */

static CYTHON_INLINE void __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(CYTHON_UNUSED char *__pyx_v_func_name, double __pyx_v_x, double __pyx_v_y) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save;
  #endif
  __Pyx_RefNannySetupContext("_legacy_cast_check", 1);

  /* "_legacy.pxd":41
 * 
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:
 *     if <int>x != x or <int>y != y:             # <<<<<<<<<<<<<<
 *         with gil:
 *             PyErr_WarnEx_noerr(RuntimeWarning,
 */
  /*try:*/ {
    __pyx_t_2 = ((((int)__pyx_v_x) != __pyx_v_x) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_2 = ((((int)__pyx_v_y) != __pyx_v_y) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_1) {

      /* "_legacy.pxd":42
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:
 *     if <int>x != x or <int>y != y:
 *         with gil:             # <<<<<<<<<<<<<<
 *             PyErr_WarnEx_noerr(RuntimeWarning,
 *                                "floating point number truncated to an integer",
 */
      {
          #ifdef WITH_THREAD
          PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
          #endif
          /*try:*/ {

            /* "_legacy.pxd":43
 *     if <int>x != x or <int>y != y:
 *         with gil:
 *             PyErr_WarnEx_noerr(RuntimeWarning,             # <<<<<<<<<<<<<<
 *                                "floating point number truncated to an integer",
 *                                1)
 */
            PyErr_WarnEx(__pyx_builtin_RuntimeWarning, __pyx_k_floating_point_number_truncated, 1);
          }

          /* "_legacy.pxd":42
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:
 *     if <int>x != x or <int>y != y:
 *         with gil:             # <<<<<<<<<<<<<<
 *             PyErr_WarnEx_noerr(RuntimeWarning,
 *                                "floating point number truncated to an integer",
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              PyGILState_Release(__pyx_gilstate_save);
              #endif
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "_legacy.pxd":41
 * 
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:
 *     if <int>x != x or <int>y != y:             # <<<<<<<<<<<<<<
 *         with gil:
 *             PyErr_WarnEx_noerr(RuntimeWarning,
 */
    }
  }
  /*finally:*/ {
    /*normal exit:*/{
      #ifdef WITH_THREAD
      __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      goto __pyx_L5;
    }
    __pyx_L5:;
  }

  /* "_legacy.pxd":40
 *     int PyErr_WarnEx_noerr "PyErr_WarnEx" (object, char *, int)
 * 
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if <int>x != x or <int>y != y:
 *         with gil:
 */

  /* function exit code */
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "_legacy.pxd":47
 *                                1)
 * 
 * cdef inline double complex sph_harmonic_unsafe(double m, double n, double theta, double phi) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("sph_harm", m, n)
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe(double __pyx_v_m, double __pyx_v_n, double __pyx_v_theta, double __pyx_v_phi) {
  __pyx_t_double_complex __pyx_r;

  /* "_legacy.pxd":48
 * 
 * cdef inline double complex sph_harmonic_unsafe(double m, double n, double theta, double phi) nogil:
 *     _legacy_cast_check("sph_harm", m, n)             # <<<<<<<<<<<<<<
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n, double p, double l, double signm, double signn) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_sph_harm, __pyx_v_m, __pyx_v_n);

  /* "_legacy.pxd":49
 * cdef inline double complex sph_harmonic_unsafe(double m, double n, double theta, double phi) nogil:
 *     _legacy_cast_check("sph_harm", m, n)
 *     return sph_harmonic(<int>m, <int> n, theta, phi)             # <<<<<<<<<<<<<<
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n, double p, double l, double signm, double signn) nogil:
 *     _legacy_cast_check("_ellip_harm", n, p)
 */
  __pyx_r = __pyx_f_5scipy_7special_8sph_harm_sph_harmonic(((int)__pyx_v_m), ((int)__pyx_v_n), __pyx_v_theta, __pyx_v_phi);
  goto __pyx_L0;

  /* "_legacy.pxd":47
 *                                1)
 * 
 * cdef inline double complex sph_harmonic_unsafe(double m, double n, double theta, double phi) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("sph_harm", m, n)
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":50
 *     _legacy_cast_check("sph_harm", m, n)
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n, double p, double l, double signm, double signn) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("_ellip_harm", n, p)
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe(double __pyx_v_h2, double __pyx_v_k2, double __pyx_v_n, double __pyx_v_p, double __pyx_v_l, double __pyx_v_signm, double __pyx_v_signn) {
  double __pyx_r;

  /* "_legacy.pxd":51
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n, double p, double l, double signm, double signn) nogil:
 *     _legacy_cast_check("_ellip_harm", n, p)             # <<<<<<<<<<<<<<
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_ellip_harm_2, __pyx_v_n, __pyx_v_p);

  /* "_legacy.pxd":52
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n, double p, double l, double signm, double signn) nogil:
 *     _legacy_cast_check("_ellip_harm", n, p)
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)             # <<<<<<<<<<<<<<
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("bdtrc", k, n)
 */
  __pyx_r = __pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic(__pyx_v_h2, __pyx_v_k2, ((int)__pyx_v_n), ((int)__pyx_v_p), __pyx_v_l, __pyx_v_signm, __pyx_v_signn);
  goto __pyx_L0;

  /* "_legacy.pxd":50
 *     _legacy_cast_check("sph_harm", m, n)
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n, double p, double l, double signm, double signn) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("_ellip_harm", n, p)
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":53
 *     _legacy_cast_check("_ellip_harm", n, p)
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtrc", k, n)
 *     return bdtrc(<int>k, <int>n, p)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;

  /* "_legacy.pxd":54
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("bdtrc", k, n)             # <<<<<<<<<<<<<<
 *     return bdtrc(<int>k, <int>n, p)
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_bdtrc, __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":55
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("bdtrc", k, n)
 *     return bdtrc(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("bdtr", k, n)
 */
  __pyx_r = bdtrc(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":53
 *     _legacy_cast_check("_ellip_harm", n, p)
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtrc", k, n)
 *     return bdtrc(<int>k, <int>n, p)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":56
 *     _legacy_cast_check("bdtrc", k, n)
 *     return bdtrc(<int>k, <int>n, p)
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtr", k, n)
 *     return bdtr(<int>k, <int>n, p)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtr_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;

  /* "_legacy.pxd":57
 *     return bdtrc(<int>k, <int>n, p)
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("bdtr", k, n)             # <<<<<<<<<<<<<<
 *     return bdtr(<int>k, <int>n, p)
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_bdtr, __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":58
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("bdtr", k, n)
 *     return bdtr(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:
 *     _legacy_cast_check("bdtri", k, n)
 */
  __pyx_r = bdtr(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":56
 *     _legacy_cast_check("bdtrc", k, n)
 *     return bdtrc(<int>k, <int>n, p)
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtr", k, n)
 *     return bdtr(<int>k, <int>n, p)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":59
 *     _legacy_cast_check("bdtr", k, n)
 *     return bdtr(<int>k, <int>n, p)
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtri", k, n)
 *     return bdtri(<int>k, <int>n, y)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtri_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_y) {
  double __pyx_r;

  /* "_legacy.pxd":60
 *     return bdtr(<int>k, <int>n, p)
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:
 *     _legacy_cast_check("bdtri", k, n)             # <<<<<<<<<<<<<<
 *     return bdtri(<int>k, <int>n, y)
 * cdef inline double expn_unsafe(double n, double x) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_bdtri, __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":61
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:
 *     _legacy_cast_check("bdtri", k, n)
 *     return bdtri(<int>k, <int>n, y)             # <<<<<<<<<<<<<<
 * cdef inline double expn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("expn", n, 0)
 */
  __pyx_r = bdtri(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_y);
  goto __pyx_L0;

  /* "_legacy.pxd":59
 *     _legacy_cast_check("bdtr", k, n)
 *     return bdtr(<int>k, <int>n, p)
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtri", k, n)
 *     return bdtri(<int>k, <int>n, y)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":62
 *     _legacy_cast_check("bdtri", k, n)
 *     return bdtri(<int>k, <int>n, y)
 * cdef inline double expn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("expn", n, 0)
 *     return expn(<int>n, x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_expn_unsafe(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "_legacy.pxd":63
 *     return bdtri(<int>k, <int>n, y)
 * cdef inline double expn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("expn", n, 0)             # <<<<<<<<<<<<<<
 *     return expn(<int>n, x)
 * cdef inline double hyp2f0_unsafe(double a, double b, double x, double type, double *err) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_expn, __pyx_v_n, 0.0);

  /* "_legacy.pxd":64
 * cdef inline double expn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("expn", n, 0)
 *     return expn(<int>n, x)             # <<<<<<<<<<<<<<
 * cdef inline double hyp2f0_unsafe(double a, double b, double x, double type, double *err) nogil:
 *     _legacy_cast_check("hyp2f0", type, 0)
 */
  __pyx_r = expn(((int)__pyx_v_n), __pyx_v_x);
  goto __pyx_L0;

  /* "_legacy.pxd":62
 *     _legacy_cast_check("bdtri", k, n)
 *     return bdtri(<int>k, <int>n, y)
 * cdef inline double expn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("expn", n, 0)
 *     return expn(<int>n, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":65
 *     _legacy_cast_check("expn", n, 0)
 *     return expn(<int>n, x)
 * cdef inline double hyp2f0_unsafe(double a, double b, double x, double type, double *err) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("hyp2f0", type, 0)
 *     return hyp2f0(a, b, x, <int>type, err)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe(double __pyx_v_a, double __pyx_v_b, double __pyx_v_x, double __pyx_v_type, double *__pyx_v_err) {
  double __pyx_r;

  /* "_legacy.pxd":66
 *     return expn(<int>n, x)
 * cdef inline double hyp2f0_unsafe(double a, double b, double x, double type, double *err) nogil:
 *     _legacy_cast_check("hyp2f0", type, 0)             # <<<<<<<<<<<<<<
 *     return hyp2f0(a, b, x, <int>type, err)
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_hyp2f0, __pyx_v_type, 0.0);

  /* "_legacy.pxd":67
 * cdef inline double hyp2f0_unsafe(double a, double b, double x, double type, double *err) nogil:
 *     _legacy_cast_check("hyp2f0", type, 0)
 *     return hyp2f0(a, b, x, <int>type, err)             # <<<<<<<<<<<<<<
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("nbdtrc", k, n)
 */
  __pyx_r = hyp2f0(__pyx_v_a, __pyx_v_b, __pyx_v_x, ((int)__pyx_v_type), __pyx_v_err);
  goto __pyx_L0;

  /* "_legacy.pxd":65
 *     _legacy_cast_check("expn", n, 0)
 *     return expn(<int>n, x)
 * cdef inline double hyp2f0_unsafe(double a, double b, double x, double type, double *err) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("hyp2f0", type, 0)
 *     return hyp2f0(a, b, x, <int>type, err)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":68
 *     _legacy_cast_check("hyp2f0", type, 0)
 *     return hyp2f0(a, b, x, <int>type, err)
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtrc", k, n)
 *     return nbdtrc(<int>k, <int>n, p)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;

  /* "_legacy.pxd":69
 *     return hyp2f0(a, b, x, <int>type, err)
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("nbdtrc", k, n)             # <<<<<<<<<<<<<<
 *     return nbdtrc(<int>k, <int>n, p)
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_nbdtrc, __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":70
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("nbdtrc", k, n)
 *     return nbdtrc(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:
 *     _legacy_cast_check("nbdtr", k, n)
 */
  __pyx_r = nbdtrc(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":68
 *     _legacy_cast_check("hyp2f0", type, 0)
 *     return hyp2f0(a, b, x, <int>type, err)
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtrc", k, n)
 *     return nbdtrc(<int>k, <int>n, p)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":71
 *     _legacy_cast_check("nbdtrc", k, n)
 *     return nbdtrc(<int>k, <int>n, p)
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtr", k, n)
 *     return nbdtr(<int>k, <int>n, p)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;

  /* "_legacy.pxd":72
 *     return nbdtrc(<int>k, <int>n, p)
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:
 *     _legacy_cast_check("nbdtr", k, n)             # <<<<<<<<<<<<<<
 *     return nbdtr(<int>k, <int>n, p)
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_nbdtr, __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":73
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:
 *     _legacy_cast_check("nbdtr", k, n)
 *     return nbdtr(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("nbdtri", k, n)
 */
  __pyx_r = nbdtr(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":71
 *     _legacy_cast_check("nbdtrc", k, n)
 *     return nbdtrc(<int>k, <int>n, p)
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtr", k, n)
 *     return nbdtr(<int>k, <int>n, p)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":74
 *     _legacy_cast_check("nbdtr", k, n)
 *     return nbdtr(<int>k, <int>n, p)
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtri", k, n)
 *     return nbdtri(<int>k, <int>n, p)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;

  /* "_legacy.pxd":75
 *     return nbdtr(<int>k, <int>n, p)
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("nbdtri", k, n)             # <<<<<<<<<<<<<<
 *     return nbdtri(<int>k, <int>n, p)
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_nbdtri, __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":76
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:
 *     _legacy_cast_check("nbdtri", k, n)
 *     return nbdtri(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:
 *     _legacy_cast_check("pdtrc", k, 0)
 */
  __pyx_r = nbdtri(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":74
 *     _legacy_cast_check("nbdtr", k, n)
 *     return nbdtr(<int>k, <int>n, p)
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtri", k, n)
 *     return nbdtri(<int>k, <int>n, p)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":77
 *     _legacy_cast_check("nbdtri", k, n)
 *     return nbdtri(<int>k, <int>n, p)
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtrc", k, 0)
 *     return pdtrc(<int>k, m)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe(double __pyx_v_k, double __pyx_v_m) {
  double __pyx_r;

  /* "_legacy.pxd":78
 *     return nbdtri(<int>k, <int>n, p)
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:
 *     _legacy_cast_check("pdtrc", k, 0)             # <<<<<<<<<<<<<<
 *     return pdtrc(<int>k, m)
 * cdef inline double pdtr_unsafe(double k, double m) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_pdtrc, __pyx_v_k, 0.0);

  /* "_legacy.pxd":79
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:
 *     _legacy_cast_check("pdtrc", k, 0)
 *     return pdtrc(<int>k, m)             # <<<<<<<<<<<<<<
 * cdef inline double pdtr_unsafe(double k, double m) nogil:
 *     _legacy_cast_check("pdtr", k, 0)
 */
  __pyx_r = pdtrc(((int)__pyx_v_k), __pyx_v_m);
  goto __pyx_L0;

  /* "_legacy.pxd":77
 *     _legacy_cast_check("nbdtri", k, n)
 *     return nbdtri(<int>k, <int>n, p)
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtrc", k, 0)
 *     return pdtrc(<int>k, m)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":80
 *     _legacy_cast_check("pdtrc", k, 0)
 *     return pdtrc(<int>k, m)
 * cdef inline double pdtr_unsafe(double k, double m) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtr", k, 0)
 *     return pdtr(<int>k, m)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtr_unsafe(double __pyx_v_k, double __pyx_v_m) {
  double __pyx_r;

  /* "_legacy.pxd":81
 *     return pdtrc(<int>k, m)
 * cdef inline double pdtr_unsafe(double k, double m) nogil:
 *     _legacy_cast_check("pdtr", k, 0)             # <<<<<<<<<<<<<<
 *     return pdtr(<int>k, m)
 * cdef inline double pdtri_unsafe(double k, double y) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_pdtr, __pyx_v_k, 0.0);

  /* "_legacy.pxd":82
 * cdef inline double pdtr_unsafe(double k, double m) nogil:
 *     _legacy_cast_check("pdtr", k, 0)
 *     return pdtr(<int>k, m)             # <<<<<<<<<<<<<<
 * cdef inline double pdtri_unsafe(double k, double y) nogil:
 *     _legacy_cast_check("pdtri", k, 0)
 */
  __pyx_r = pdtr(((int)__pyx_v_k), __pyx_v_m);
  goto __pyx_L0;

  /* "_legacy.pxd":80
 *     _legacy_cast_check("pdtrc", k, 0)
 *     return pdtrc(<int>k, m)
 * cdef inline double pdtr_unsafe(double k, double m) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtr", k, 0)
 *     return pdtr(<int>k, m)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":83
 *     _legacy_cast_check("pdtr", k, 0)
 *     return pdtr(<int>k, m)
 * cdef inline double pdtri_unsafe(double k, double y) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtri", k, 0)
 *     return pdtri(<int>k, y)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtri_unsafe(double __pyx_v_k, double __pyx_v_y) {
  double __pyx_r;

  /* "_legacy.pxd":84
 *     return pdtr(<int>k, m)
 * cdef inline double pdtri_unsafe(double k, double y) nogil:
 *     _legacy_cast_check("pdtri", k, 0)             # <<<<<<<<<<<<<<
 *     return pdtri(<int>k, y)
 * cdef inline double kn_unsafe(double n, double x) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_pdtri, __pyx_v_k, 0.0);

  /* "_legacy.pxd":85
 * cdef inline double pdtri_unsafe(double k, double y) nogil:
 *     _legacy_cast_check("pdtri", k, 0)
 *     return pdtri(<int>k, y)             # <<<<<<<<<<<<<<
 * cdef inline double kn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("kn", n, 0)
 */
  __pyx_r = pdtri(((int)__pyx_v_k), __pyx_v_y);
  goto __pyx_L0;

  /* "_legacy.pxd":83
 *     _legacy_cast_check("pdtr", k, 0)
 *     return pdtr(<int>k, m)
 * cdef inline double pdtri_unsafe(double k, double y) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtri", k, 0)
 *     return pdtri(<int>k, y)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":86
 *     _legacy_cast_check("pdtri", k, 0)
 *     return pdtri(<int>k, y)
 * cdef inline double kn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("kn", n, 0)
 *     return cbesk_wrap_real_int(<int>n, x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_kn_unsafe(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "_legacy.pxd":87
 *     return pdtri(<int>k, y)
 * cdef inline double kn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("kn", n, 0)             # <<<<<<<<<<<<<<
 *     return cbesk_wrap_real_int(<int>n, x)
 * cdef inline double yn_unsafe(double n, double x) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_kn, __pyx_v_n, 0.0);

  /* "_legacy.pxd":88
 * cdef inline double kn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("kn", n, 0)
 *     return cbesk_wrap_real_int(<int>n, x)             # <<<<<<<<<<<<<<
 * cdef inline double yn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("yn", n, 0)
 */
  __pyx_r = cbesk_wrap_real_int(((int)__pyx_v_n), __pyx_v_x);
  goto __pyx_L0;

  /* "_legacy.pxd":86
 *     _legacy_cast_check("pdtri", k, 0)
 *     return pdtri(<int>k, y)
 * cdef inline double kn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("kn", n, 0)
 *     return cbesk_wrap_real_int(<int>n, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":89
 *     _legacy_cast_check("kn", n, 0)
 *     return cbesk_wrap_real_int(<int>n, x)
 * cdef inline double yn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("yn", n, 0)
 *     return yn(<int>n, x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_yn_unsafe(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "_legacy.pxd":90
 *     return cbesk_wrap_real_int(<int>n, x)
 * cdef inline double yn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("yn", n, 0)             # <<<<<<<<<<<<<<
 *     return yn(<int>n, x)
 * cdef inline double smirnov_unsafe(double n, double e) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_yn, __pyx_v_n, 0.0);

  /* "_legacy.pxd":91
 * cdef inline double yn_unsafe(double n, double x) nogil:
 *     _legacy_cast_check("yn", n, 0)
 *     return yn(<int>n, x)             # <<<<<<<<<<<<<<
 * cdef inline double smirnov_unsafe(double n, double e) nogil:
 *     _legacy_cast_check("smirnov", n, 0)
 */
  __pyx_r = yn(((int)__pyx_v_n), __pyx_v_x);
  goto __pyx_L0;

  /* "_legacy.pxd":89
 *     _legacy_cast_check("kn", n, 0)
 *     return cbesk_wrap_real_int(<int>n, x)
 * cdef inline double yn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("yn", n, 0)
 *     return yn(<int>n, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":92
 *     _legacy_cast_check("yn", n, 0)
 *     return yn(<int>n, x)
 * cdef inline double smirnov_unsafe(double n, double e) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnov", n, 0)
 *     return smirnov(<int>n, e)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnov_unsafe(double __pyx_v_n, double __pyx_v_e) {
  double __pyx_r;

  /* "_legacy.pxd":93
 *     return yn(<int>n, x)
 * cdef inline double smirnov_unsafe(double n, double e) nogil:
 *     _legacy_cast_check("smirnov", n, 0)             # <<<<<<<<<<<<<<
 *     return smirnov(<int>n, e)
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_smirnov, __pyx_v_n, 0.0);

  /* "_legacy.pxd":94
 * cdef inline double smirnov_unsafe(double n, double e) nogil:
 *     _legacy_cast_check("smirnov", n, 0)
 *     return smirnov(<int>n, e)             # <<<<<<<<<<<<<<
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:
 *     _legacy_cast_check("smirnovi", n, 0)
 */
  __pyx_r = smirnov(((int)__pyx_v_n), __pyx_v_e);
  goto __pyx_L0;

  /* "_legacy.pxd":92
 *     _legacy_cast_check("yn", n, 0)
 *     return yn(<int>n, x)
 * cdef inline double smirnov_unsafe(double n, double e) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnov", n, 0)
 *     return smirnov(<int>n, e)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":95
 *     _legacy_cast_check("smirnov", n, 0)
 *     return smirnov(<int>n, e)
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnovi", n, 0)
 *     return smirnovi(<int>n, p)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe(double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;

  /* "_legacy.pxd":96
 *     return smirnov(<int>n, e)
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:
 *     _legacy_cast_check("smirnovi", n, 0)             # <<<<<<<<<<<<<<
 *     return smirnovi(<int>n, p)
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(__pyx_k_smirnovi, __pyx_v_n, 0.0);

  /* "_legacy.pxd":97
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:
 *     _legacy_cast_check("smirnovi", n, 0)
 *     return smirnovi(<int>n, p)             # <<<<<<<<<<<<<<
 */
  __pyx_r = smirnovi(((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":95
 *     _legacy_cast_check("smirnov", n, 0)
 *     return smirnov(<int>n, e)
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnovi", n, 0)
 *     return smirnovi(<int>n, p)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "lambertw.pxd":38
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_scalar(double complex z, long k, double tol) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     This is just the implementation of W for a single input z.
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_scalar(__pyx_t_double_complex __pyx_v_z, long __pyx_v_k, double __pyx_v_tol) {
  __pyx_t_double_complex __pyx_v_w;
  double __pyx_v_u;
  double __pyx_v_absz;
  __pyx_t_double_complex __pyx_v_ew;
  __pyx_t_double_complex __pyx_v_wew;
  __pyx_t_double_complex __pyx_v_wewz;
  __pyx_t_double_complex __pyx_v_wn;
  CYTHON_UNUSED int __pyx_v_i;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "lambertw.pxd":44
 *     """
 *     # Comments copied verbatim from [2] are marked with '>'
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 * 
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "lambertw.pxd":45
 *     # Comments copied verbatim from [2] are marked with '>'
 *     if zisnan(z):
 *         return z             # <<<<<<<<<<<<<<
 * 
 *     # Return value:
 */
    __pyx_r = __pyx_v_z;
    goto __pyx_L0;

    /* "lambertw.pxd":44
 *     """
 *     # Comments copied verbatim from [2] are marked with '>'
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 * 
 */
  }

  /* "lambertw.pxd":52
 *     #> We must be extremely careful near the singularities at -1/e and 0
 *     cdef double u
 *     u = exp(-1)             # <<<<<<<<<<<<<<
 * 
 *     cdef double absz
 */
  __pyx_v_u = exp(-1.0);

  /* "lambertw.pxd":55
 * 
 *     cdef double absz
 *     absz = zabs(z)             # <<<<<<<<<<<<<<
 *     if absz <= u:
 *         if z == 0:
 */
  __pyx_v_absz = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z);

  /* "lambertw.pxd":56
 *     cdef double absz
 *     absz = zabs(z)
 *     if absz <= u:             # <<<<<<<<<<<<<<
 *         if z == 0:
 *             #> w(0,0) = 0; for all other branches we hit the pole
 */
  __pyx_t_1 = ((__pyx_v_absz <= __pyx_v_u) != 0);
  if (__pyx_t_1) {

    /* "lambertw.pxd":57
 *     absz = zabs(z)
 *     if absz <= u:
 *         if z == 0:             # <<<<<<<<<<<<<<
 *             #> w(0,0) = 0; for all other branches we hit the pole
 *             if k == 0:
 */
    __pyx_t_1 = ((__Pyx_c_eq(__pyx_v_z, __pyx_t_double_complex_from_parts(0, 0))) != 0);
    if (__pyx_t_1) {

      /* "lambertw.pxd":59
 *         if z == 0:
 *             #> w(0,0) = 0; for all other branches we hit the pole
 *             if k == 0:             # <<<<<<<<<<<<<<
 *                 return z
 *             sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 */
      __pyx_t_1 = ((__pyx_v_k == 0) != 0);
      if (__pyx_t_1) {

        /* "lambertw.pxd":60
 *             #> w(0,0) = 0; for all other branches we hit the pole
 *             if k == 0:
 *                 return z             # <<<<<<<<<<<<<<
 *             sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 *             return -inf
 */
        __pyx_r = __pyx_v_z;
        goto __pyx_L0;

        /* "lambertw.pxd":59
 *         if z == 0:
 *             #> w(0,0) = 0; for all other branches we hit the pole
 *             if k == 0:             # <<<<<<<<<<<<<<
 *                 return z
 *             sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 */
      }

      /* "lambertw.pxd":61
 *             if k == 0:
 *                 return z
 *             sf_error.error("lambertw", sf_error.SINGULAR, NULL)             # <<<<<<<<<<<<<<
 *             return -inf
 * 
 */
      sf_error(__pyx_k_lambertw, SF_ERROR_SINGULAR, NULL);

      /* "lambertw.pxd":62
 *                 return z
 *             sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 *             return -inf             # <<<<<<<<<<<<<<
 * 
 *         if k == 0:
 */
      __pyx_r = __pyx_t_double_complex_from_parts((-NPY_INFINITY), 0);
      goto __pyx_L0;

      /* "lambertw.pxd":57
 *     absz = zabs(z)
 *     if absz <= u:
 *         if z == 0:             # <<<<<<<<<<<<<<
 *             #> w(0,0) = 0; for all other branches we hit the pole
 *             if k == 0:
 */
    }

    /* "lambertw.pxd":64
 *             return -inf
 * 
 *         if k == 0:             # <<<<<<<<<<<<<<
 *             w = z # Initial guess for iteration
 *         #> For small real z < 0, the -1 branch beaves roughly like log(-z)
 */
    __pyx_t_1 = ((__pyx_v_k == 0) != 0);
    if (__pyx_t_1) {

      /* "lambertw.pxd":65
 * 
 *         if k == 0:
 *             w = z # Initial guess for iteration             # <<<<<<<<<<<<<<
 *         #> For small real z < 0, the -1 branch beaves roughly like log(-z)
 *         elif k == -1 and z.imag ==0 and z.real < 0:
 */
      __pyx_v_w = __pyx_v_z;

      /* "lambertw.pxd":64
 *             return -inf
 * 
 *         if k == 0:             # <<<<<<<<<<<<<<
 *             w = z # Initial guess for iteration
 *         #> For small real z < 0, the -1 branch beaves roughly like log(-z)
 */
      goto __pyx_L7;
    }

    /* "lambertw.pxd":67
 *             w = z # Initial guess for iteration
 *         #> For small real z < 0, the -1 branch beaves roughly like log(-z)
 *         elif k == -1 and z.imag ==0 and z.real < 0:             # <<<<<<<<<<<<<<
 *             w = log(-z.real)
 *         #> Use a simple asymptotic approximation.
 */
    __pyx_t_2 = ((__pyx_v_k == -1L) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_2 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) < 0.0) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_1) {

      /* "lambertw.pxd":68
 *         #> For small real z < 0, the -1 branch beaves roughly like log(-z)
 *         elif k == -1 and z.imag ==0 and z.real < 0:
 *             w = log(-z.real)             # <<<<<<<<<<<<<<
 *         #> Use a simple asymptotic approximation.
 *         else:
 */
      __pyx_v_w = __pyx_t_double_complex_from_parts(log((-__Pyx_CREAL(__pyx_v_z))), 0);

      /* "lambertw.pxd":67
 *             w = z # Initial guess for iteration
 *         #> For small real z < 0, the -1 branch beaves roughly like log(-z)
 *         elif k == -1 and z.imag ==0 and z.real < 0:             # <<<<<<<<<<<<<<
 *             w = log(-z.real)
 *         #> Use a simple asymptotic approximation.
 */
      goto __pyx_L7;
    }

    /* "lambertw.pxd":71
 *         #> Use a simple asymptotic approximation.
 *         else:
 *             w = zlog(z)             # <<<<<<<<<<<<<<
 *             #> The branches are roughly logarithmic. This approximation
 *             #> gets better for large |k|; need to check that this always
 */
    /*else*/ {
      __pyx_v_w = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_z);

      /* "lambertw.pxd":75
 *             #> gets better for large |k|; need to check that this always
 *             #> works for k ~= -1, 0, 1.
 *             if k: w = w + k*2*pi*1j             # <<<<<<<<<<<<<<
 * 
 *     elif k == 0 and z.imag and zabs(z) <= 0.7:
 */
      __pyx_t_1 = (__pyx_v_k != 0);
      if (__pyx_t_1) {
        __pyx_v_w = __Pyx_c_sum(__pyx_v_w, __Pyx_c_prod(__pyx_t_double_complex_from_parts(((__pyx_v_k * 2) * NPY_PI), 0), __pyx_t_double_complex_from_parts(0, 1.0)));
      }
    }
    __pyx_L7:;

    /* "lambertw.pxd":56
 *     cdef double absz
 *     absz = zabs(z)
 *     if absz <= u:             # <<<<<<<<<<<<<<
 *         if z == 0:
 *             #> w(0,0) = 0; for all other branches we hit the pole
 */
    goto __pyx_L4;
  }

  /* "lambertw.pxd":77
 *             if k: w = w + k*2*pi*1j
 * 
 *     elif k == 0 and z.imag and zabs(z) <= 0.7:             # <<<<<<<<<<<<<<
 *         #> Both the W(z) ~= z and W(z) ~= ln(z) approximations break
 *         #> down around z ~= -0.5 (converging to the wrong branch), so patch
 */
  __pyx_t_2 = ((__pyx_v_k == 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_2 = (__Pyx_CIMAG(__pyx_v_z) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z) <= 0.7) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_1) {

    /* "lambertw.pxd":81
 *         #> down around z ~= -0.5 (converging to the wrong branch), so patch
 *         #> with a constant approximation (adjusted for sign)
 *         if zabs(z+0.5) < 0.1:             # <<<<<<<<<<<<<<
 *             if z.imag > 0:
 *                 w = 0.7 + 0.7j
 */
    __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_sum(__pyx_v_z, __pyx_t_double_complex_from_parts(0.5, 0))) < 0.1) != 0);
    if (__pyx_t_1) {

      /* "lambertw.pxd":82
 *         #> with a constant approximation (adjusted for sign)
 *         if zabs(z+0.5) < 0.1:
 *             if z.imag > 0:             # <<<<<<<<<<<<<<
 *                 w = 0.7 + 0.7j
 *             else:
 */
      __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) > 0.0) != 0);
      if (__pyx_t_1) {

        /* "lambertw.pxd":83
 *         if zabs(z+0.5) < 0.1:
 *             if z.imag > 0:
 *                 w = 0.7 + 0.7j             # <<<<<<<<<<<<<<
 *             else:
 *                 w = 0.7 - 0.7j
 */
        __pyx_v_w = __Pyx_c_sum(__pyx_t_double_complex_from_parts(0.7, 0), __pyx_t_double_complex_from_parts(0, 0.7));

        /* "lambertw.pxd":82
 *         #> with a constant approximation (adjusted for sign)
 *         if zabs(z+0.5) < 0.1:
 *             if z.imag > 0:             # <<<<<<<<<<<<<<
 *                 w = 0.7 + 0.7j
 *             else:
 */
        goto __pyx_L16;
      }

      /* "lambertw.pxd":85
 *                 w = 0.7 + 0.7j
 *             else:
 *                 w = 0.7 - 0.7j             # <<<<<<<<<<<<<<
 *         else:
 *             w = z
 */
      /*else*/ {
        __pyx_v_w = __Pyx_c_diff(__pyx_t_double_complex_from_parts(0.7, 0), __pyx_t_double_complex_from_parts(0, 0.7));
      }
      __pyx_L16:;

      /* "lambertw.pxd":81
 *         #> down around z ~= -0.5 (converging to the wrong branch), so patch
 *         #> with a constant approximation (adjusted for sign)
 *         if zabs(z+0.5) < 0.1:             # <<<<<<<<<<<<<<
 *             if z.imag > 0:
 *                 w = 0.7 + 0.7j
 */
      goto __pyx_L15;
    }

    /* "lambertw.pxd":87
 *                 w = 0.7 - 0.7j
 *         else:
 *             w = z             # <<<<<<<<<<<<<<
 * 
 *     else:
 */
    /*else*/ {
      __pyx_v_w = __pyx_v_z;
    }
    __pyx_L15:;

    /* "lambertw.pxd":77
 *             if k: w = w + k*2*pi*1j
 * 
 *     elif k == 0 and z.imag and zabs(z) <= 0.7:             # <<<<<<<<<<<<<<
 *         #> Both the W(z) ~= z and W(z) ~= ln(z) approximations break
 *         #> down around z ~= -0.5 (converging to the wrong branch), so patch
 */
    goto __pyx_L4;
  }

  /* "lambertw.pxd":90
 * 
 *     else:
 *         if z.real == inf:             # <<<<<<<<<<<<<<
 *             if k == 0:
 *                 return z
 */
  /*else*/ {
    __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) == NPY_INFINITY) != 0);
    if (__pyx_t_1) {

      /* "lambertw.pxd":91
 *     else:
 *         if z.real == inf:
 *             if k == 0:             # <<<<<<<<<<<<<<
 *                 return z
 *             else:
 */
      __pyx_t_1 = ((__pyx_v_k == 0) != 0);
      if (__pyx_t_1) {

        /* "lambertw.pxd":92
 *         if z.real == inf:
 *             if k == 0:
 *                 return z             # <<<<<<<<<<<<<<
 *             else:
 *                 return z + 2*k*pi*1j
 */
        __pyx_r = __pyx_v_z;
        goto __pyx_L0;

        /* "lambertw.pxd":91
 *     else:
 *         if z.real == inf:
 *             if k == 0:             # <<<<<<<<<<<<<<
 *                 return z
 *             else:
 */
      }

      /* "lambertw.pxd":94
 *                 return z
 *             else:
 *                 return z + 2*k*pi*1j             # <<<<<<<<<<<<<<
 * 
 *         if z.real == -inf:
 */
      /*else*/ {
        __pyx_r = __Pyx_c_sum(__pyx_v_z, __Pyx_c_prod(__pyx_t_double_complex_from_parts(((2 * __pyx_v_k) * NPY_PI), 0), __pyx_t_double_complex_from_parts(0, 1.0)));
        goto __pyx_L0;
      }

      /* "lambertw.pxd":90
 * 
 *     else:
 *         if z.real == inf:             # <<<<<<<<<<<<<<
 *             if k == 0:
 *                 return z
 */
    }

    /* "lambertw.pxd":96
 *                 return z + 2*k*pi*1j
 * 
 *         if z.real == -inf:             # <<<<<<<<<<<<<<
 *             return (-z) + (2*k+1)*pi*1j
 * 
 */
    __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) == (-NPY_INFINITY)) != 0);
    if (__pyx_t_1) {

      /* "lambertw.pxd":97
 * 
 *         if z.real == -inf:
 *             return (-z) + (2*k+1)*pi*1j             # <<<<<<<<<<<<<<
 * 
 *         #> Simple asymptotic approximation as above
 */
      __pyx_r = __Pyx_c_sum(__Pyx_c_neg(__pyx_v_z), __Pyx_c_prod(__pyx_t_double_complex_from_parts((((2 * __pyx_v_k) + 1) * NPY_PI), 0), __pyx_t_double_complex_from_parts(0, 1.0)));
      goto __pyx_L0;

      /* "lambertw.pxd":96
 *                 return z + 2*k*pi*1j
 * 
 *         if z.real == -inf:             # <<<<<<<<<<<<<<
 *             return (-z) + (2*k+1)*pi*1j
 * 
 */
    }

    /* "lambertw.pxd":100
 * 
 *         #> Simple asymptotic approximation as above
 *         w = zlog(z)             # <<<<<<<<<<<<<<
 *         if k: w = w + k*2*pi*1j
 * 
 */
    __pyx_v_w = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_z);

    /* "lambertw.pxd":101
 *         #> Simple asymptotic approximation as above
 *         w = zlog(z)
 *         if k: w = w + k*2*pi*1j             # <<<<<<<<<<<<<<
 * 
 *     #> Use Halley iteration to solve w*exp(w) = z
 */
    __pyx_t_1 = (__pyx_v_k != 0);
    if (__pyx_t_1) {
      __pyx_v_w = __Pyx_c_sum(__pyx_v_w, __Pyx_c_prod(__pyx_t_double_complex_from_parts(((__pyx_v_k * 2) * NPY_PI), 0), __pyx_t_double_complex_from_parts(0, 1.0)));
    }
  }
  __pyx_L4:;

  /* "lambertw.pxd":106
 *     cdef double complex ew, wew, wewz, wn
 *     cdef int i
 *     for i in range(100):             # <<<<<<<<<<<<<<
 *         ew = zexp(w)
 *         wew = w*ew
 */
  for (__pyx_t_3 = 0; __pyx_t_3 < 0x64; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "lambertw.pxd":107
 *     cdef int i
 *     for i in range(100):
 *         ew = zexp(w)             # <<<<<<<<<<<<<<
 *         wew = w*ew
 *         wewz = wew-z
 */
    __pyx_v_ew = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__pyx_v_w);

    /* "lambertw.pxd":108
 *     for i in range(100):
 *         ew = zexp(w)
 *         wew = w*ew             # <<<<<<<<<<<<<<
 *         wewz = wew-z
 *         wn = w - wewz / (wew + ew - (w + 2)*wewz/(2*w + 2))
 */
    __pyx_v_wew = __Pyx_c_prod(__pyx_v_w, __pyx_v_ew);

    /* "lambertw.pxd":109
 *         ew = zexp(w)
 *         wew = w*ew
 *         wewz = wew-z             # <<<<<<<<<<<<<<
 *         wn = w - wewz / (wew + ew - (w + 2)*wewz/(2*w + 2))
 *         if zabs(wn-w) < tol*zabs(wn):
 */
    __pyx_v_wewz = __Pyx_c_diff(__pyx_v_wew, __pyx_v_z);

    /* "lambertw.pxd":110
 *         wew = w*ew
 *         wewz = wew-z
 *         wn = w - wewz / (wew + ew - (w + 2)*wewz/(2*w + 2))             # <<<<<<<<<<<<<<
 *         if zabs(wn-w) < tol*zabs(wn):
 *             return wn
 */
    __pyx_v_wn = __Pyx_c_diff(__pyx_v_w, __Pyx_c_quot(__pyx_v_wewz, __Pyx_c_diff(__Pyx_c_sum(__pyx_v_wew, __pyx_v_ew), __Pyx_c_quot(__Pyx_c_prod(__Pyx_c_sum(__pyx_v_w, __pyx_t_double_complex_from_parts(2, 0)), __pyx_v_wewz), __Pyx_c_sum(__Pyx_c_prod(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_w), __pyx_t_double_complex_from_parts(2, 0))))));

    /* "lambertw.pxd":111
 *         wewz = wew-z
 *         wn = w - wewz / (wew + ew - (w + 2)*wewz/(2*w + 2))
 *         if zabs(wn-w) < tol*zabs(wn):             # <<<<<<<<<<<<<<
 *             return wn
 *         else:
 */
    __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff(__pyx_v_wn, __pyx_v_w)) < (__pyx_v_tol * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_wn))) != 0);
    if (__pyx_t_1) {

      /* "lambertw.pxd":112
 *         wn = w - wewz / (wew + ew - (w + 2)*wewz/(2*w + 2))
 *         if zabs(wn-w) < tol*zabs(wn):
 *             return wn             # <<<<<<<<<<<<<<
 *         else:
 *             w = wn
 */
      __pyx_r = __pyx_v_wn;
      goto __pyx_L0;

      /* "lambertw.pxd":111
 *         wewz = wew-z
 *         wn = w - wewz / (wew + ew - (w + 2)*wewz/(2*w + 2))
 *         if zabs(wn-w) < tol*zabs(wn):             # <<<<<<<<<<<<<<
 *             return wn
 *         else:
 */
    }

    /* "lambertw.pxd":114
 *             return wn
 *         else:
 *             w = wn             # <<<<<<<<<<<<<<
 * 
 *     sf_error.error("lambertw", sf_error.SLOW,
 */
    /*else*/ {
      __pyx_v_w = __pyx_v_wn;
    }
  }

  /* "lambertw.pxd":116
 *             w = wn
 * 
 *     sf_error.error("lambertw", sf_error.SLOW,             # <<<<<<<<<<<<<<
 *                    "iteration failed to converge: %g + %gj",
 *                    <double>z.real, <double>z.imag)
 */
  sf_error(__pyx_k_lambertw, SF_ERROR_SLOW, __pyx_k_iteration_failed_to_converge_g_g, ((double)__Pyx_CREAL(__pyx_v_z)), ((double)__Pyx_CIMAG(__pyx_v_z)));

  /* "lambertw.pxd":119
 *                    "iteration failed to converge: %g + %gj",
 *                    <double>z.real, <double>z.imag)
 *     return nan             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
  goto __pyx_L0;

  /* "lambertw.pxd":38
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_scalar(double complex z, long k, double tol) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     This is just the implementation of W for a single input z.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_boxcox.pxd":5
 * 
 * 
 * cdef inline double boxcox(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     # if lmbda << 1 and log(x) < 1.0, the lmbda*log(x) product can lose
 *     # precision, furthermore, expm1(x) == x for x < eps.
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_boxcox(double __pyx_v_x, double __pyx_v_lmbda) {
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "_boxcox.pxd":12
 *     # abs(lmbda)*log(x) < eps whenever abs(lmbda) <= eps/-log(min double)
 *     # which is ~2.98e-19.
 *     if fabs(lmbda) < 1e-19:             # <<<<<<<<<<<<<<
 *         return log(x)
 *     else:
 */
  __pyx_t_1 = ((fabs(__pyx_v_lmbda) < 1e-19) != 0);
  if (__pyx_t_1) {

    /* "_boxcox.pxd":13
 *     # which is ~2.98e-19.
 *     if fabs(lmbda) < 1e-19:
 *         return log(x)             # <<<<<<<<<<<<<<
 *     else:
 *         return expm1(lmbda * log(x)) / lmbda
 */
    __pyx_r = log(__pyx_v_x);
    goto __pyx_L0;

    /* "_boxcox.pxd":12
 *     # abs(lmbda)*log(x) < eps whenever abs(lmbda) <= eps/-log(min double)
 *     # which is ~2.98e-19.
 *     if fabs(lmbda) < 1e-19:             # <<<<<<<<<<<<<<
 *         return log(x)
 *     else:
 */
  }

  /* "_boxcox.pxd":15
 *         return log(x)
 *     else:
 *         return expm1(lmbda * log(x)) / lmbda             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_2 = expm1((__pyx_v_lmbda * log(__pyx_v_x)));
    if (unlikely(__pyx_v_lmbda == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[6]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_r = (__pyx_t_2 / __pyx_v_lmbda);
    goto __pyx_L0;
  }

  /* "_boxcox.pxd":5
 * 
 * 
 * cdef inline double boxcox(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     # if lmbda << 1 and log(x) < 1.0, the lmbda*log(x) product can lose
 *     # precision, furthermore, expm1(x) == x for x < eps.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._boxcox.boxcox", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_boxcox.pxd":18
 * 
 * 
 * cdef inline double boxcox1p(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     # The argument given above in boxcox applies here with the modification
 *     # that the smallest value produced by log1p is the minimum representable
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_boxcox1p(double __pyx_v_x, double __pyx_v_lmbda) {
  double __pyx_v_lgx;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "_boxcox.pxd":23
 *     # value, rather than eps.  The second condition here prevents unflow
 *     # when log1p(x) is < eps.
 *     cdef double lgx = log1p(x)             # <<<<<<<<<<<<<<
 *     if fabs(lmbda) < 1e-19 or (fabs(lgx) < 1e-289 and fabs(lmbda) < 1e273):
 *         return lgx
 */
  __pyx_v_lgx = log1p(__pyx_v_x);

  /* "_boxcox.pxd":24
 *     # when log1p(x) is < eps.
 *     cdef double lgx = log1p(x)
 *     if fabs(lmbda) < 1e-19 or (fabs(lgx) < 1e-289 and fabs(lmbda) < 1e273):             # <<<<<<<<<<<<<<
 *         return lgx
 *     else:
 */
  __pyx_t_2 = ((fabs(__pyx_v_lmbda) < 1e-19) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__pyx_v_lgx) < 1e-289) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__pyx_v_lmbda) < 1e273) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_boxcox.pxd":25
 *     cdef double lgx = log1p(x)
 *     if fabs(lmbda) < 1e-19 or (fabs(lgx) < 1e-289 and fabs(lmbda) < 1e273):
 *         return lgx             # <<<<<<<<<<<<<<
 *     else:
 *         return expm1(lmbda * lgx) / lmbda
 */
    __pyx_r = __pyx_v_lgx;
    goto __pyx_L0;

    /* "_boxcox.pxd":24
 *     # when log1p(x) is < eps.
 *     cdef double lgx = log1p(x)
 *     if fabs(lmbda) < 1e-19 or (fabs(lgx) < 1e-289 and fabs(lmbda) < 1e273):             # <<<<<<<<<<<<<<
 *         return lgx
 *     else:
 */
  }

  /* "_boxcox.pxd":27
 *         return lgx
 *     else:
 *         return expm1(lmbda * lgx) / lmbda             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_3 = expm1((__pyx_v_lmbda * __pyx_v_lgx));
    if (unlikely(__pyx_v_lmbda == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[6]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_r = (__pyx_t_3 / __pyx_v_lmbda);
    goto __pyx_L0;
  }

  /* "_boxcox.pxd":18
 * 
 * 
 * cdef inline double boxcox1p(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     # The argument given above in boxcox applies here with the modification
 *     # that the smallest value produced by log1p is the minimum representable
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._boxcox.boxcox1p", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_boxcox.pxd":30
 * 
 * 
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     if lmbda == 0:
 *         return exp(x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_inv_boxcox(double __pyx_v_x, double __pyx_v_lmbda) {
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "_boxcox.pxd":31
 * 
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:
 *     if lmbda == 0:             # <<<<<<<<<<<<<<
 *         return exp(x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_lmbda == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_boxcox.pxd":32
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:
 *     if lmbda == 0:
 *         return exp(x)             # <<<<<<<<<<<<<<
 *     else:
 *         return exp(log1p(lmbda * x) / lmbda)
 */
    __pyx_r = exp(__pyx_v_x);
    goto __pyx_L0;

    /* "_boxcox.pxd":31
 * 
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:
 *     if lmbda == 0:             # <<<<<<<<<<<<<<
 *         return exp(x)
 *     else:
 */
  }

  /* "_boxcox.pxd":34
 *         return exp(x)
 *     else:
 *         return exp(log1p(lmbda * x) / lmbda)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_2 = log1p((__pyx_v_lmbda * __pyx_v_x));
    if (unlikely(__pyx_v_lmbda == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[6]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_r = exp((__pyx_t_2 / __pyx_v_lmbda));
    goto __pyx_L0;
  }

  /* "_boxcox.pxd":30
 * 
 * 
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     if lmbda == 0:
 *         return exp(x)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._boxcox.inv_boxcox", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_boxcox.pxd":37
 * 
 * 
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     if lmbda == 0:
 *         return expm1(x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p(double __pyx_v_x, double __pyx_v_lmbda) {
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "_boxcox.pxd":38
 * 
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:
 *     if lmbda == 0:             # <<<<<<<<<<<<<<
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:
 */
  __pyx_t_1 = ((__pyx_v_lmbda == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_boxcox.pxd":39
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:
 *     if lmbda == 0:
 *         return expm1(x)             # <<<<<<<<<<<<<<
 *     elif fabs(lmbda * x) < 1e-154:
 *         return x
 */
    __pyx_r = expm1(__pyx_v_x);
    goto __pyx_L0;

    /* "_boxcox.pxd":38
 * 
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:
 *     if lmbda == 0:             # <<<<<<<<<<<<<<
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:
 */
  }

  /* "_boxcox.pxd":40
 *     if lmbda == 0:
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  __pyx_t_1 = ((fabs((__pyx_v_lmbda * __pyx_v_x)) < 1e-154) != 0);
  if (__pyx_t_1) {

    /* "_boxcox.pxd":41
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:
 *         return x             # <<<<<<<<<<<<<<
 *     else:
 *         return expm1(log1p(lmbda * x) / lmbda)
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "_boxcox.pxd":40
 *     if lmbda == 0:
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  }

  /* "_boxcox.pxd":43
 *         return x
 *     else:
 *         return expm1(log1p(lmbda * x) / lmbda)             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __pyx_t_2 = log1p((__pyx_v_lmbda * __pyx_v_x));
    if (unlikely(__pyx_v_lmbda == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[6]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_r = expm1((__pyx_t_2 / __pyx_v_lmbda));
    goto __pyx_L0;
  }

  /* "_boxcox.pxd":37
 * 
 * 
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     if lmbda == 0:
 *         return expm1(x)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._boxcox.inv_boxcox1p", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":50
 * # Fused type wrappers
 * 
 * cdef inline number_t hyp2f1(double a, double b, double c, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(double __pyx_v_a, double __pyx_v_b, double __pyx_v_c, double __pyx_v_z) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":53
 *     cdef npy_cdouble r
 *     if number_t is double:
 *         return hyp2f1_wrap(a, b, c, z)             # <<<<<<<<<<<<<<
 *     else:
 *         r = chyp2f1_wrap(a, b, c, (<npy_cdouble*>&z)[0])
 */
  __pyx_r = hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_z);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":50
 * # Fused type wrappers
 * 
 * cdef inline number_t hyp2f1(double a, double b, double c, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(double __pyx_v_a, double __pyx_v_b, double __pyx_v_c, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":55
 *         return hyp2f1_wrap(a, b, c, z)
 *     else:
 *         r = chyp2f1_wrap(a, b, c, (<npy_cdouble*>&z)[0])             # <<<<<<<<<<<<<<
 *         return (<number_t*>&r)[0]
 * 
 */
  __pyx_v_r = chyp2f1_wrap(__pyx_v_a, __pyx_v_b, __pyx_v_c, (((npy_cdouble *)(&__pyx_v_z))[0]));

  /* "orthogonal_eval.pxd":56
 *     else:
 *         r = chyp2f1_wrap(a, b, c, (<npy_cdouble*>&z)[0])
 *         return (<number_t*>&r)[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t hyp1f1(double a, double b, number_t z) nogil:
 */
  __pyx_r = (((__pyx_t_double_complex *)(&__pyx_v_r))[0]);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":50
 * # Fused type wrappers
 * 
 * cdef inline number_t hyp2f1(double a, double b, double c, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":58
 *         return (<number_t*>&r)[0]
 * 
 * cdef inline number_t hyp1f1(double a, double b, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(double __pyx_v_a, double __pyx_v_b, double __pyx_v_z) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":61
 *     cdef npy_cdouble r
 *     if number_t is double:
 *         return hyp1f1_wrap(a, b, z)             # <<<<<<<<<<<<<<
 *     else:
 *         r = chyp1f1_wrap(a, b, (<npy_cdouble*>&z)[0])
 */
  __pyx_r = hyp1f1_wrap(__pyx_v_a, __pyx_v_b, __pyx_v_z);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":58
 *         return (<number_t*>&r)[0]
 * 
 * cdef inline number_t hyp1f1(double a, double b, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(double __pyx_v_a, double __pyx_v_b, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":63
 *         return hyp1f1_wrap(a, b, z)
 *     else:
 *         r = chyp1f1_wrap(a, b, (<npy_cdouble*>&z)[0])             # <<<<<<<<<<<<<<
 *         return (<number_t*>&r)[0]
 * 
 */
  __pyx_v_r = chyp1f1_wrap(__pyx_v_a, __pyx_v_b, (((npy_cdouble *)(&__pyx_v_z))[0]));

  /* "orthogonal_eval.pxd":64
 *     else:
 *         r = chyp1f1_wrap(a, b, (<npy_cdouble*>&z)[0])
 *         return (<number_t*>&r)[0]             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = (((__pyx_t_double_complex *)(&__pyx_v_r))[0]);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":58
 *         return (<number_t*>&r)[0]
 * 
 * cdef inline number_t hyp1f1(double a, double b, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":71
 * 
 * @cython.cdivision(True)
 * cdef inline double binom(double n, double k) nogil:             # <<<<<<<<<<<<<<
 *     cdef double kx, nx, num, den, dk, sgn
 *     cdef int i
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_binom(double __pyx_v_n, double __pyx_v_k) {
  double __pyx_v_kx;
  double __pyx_v_nx;
  double __pyx_v_num;
  double __pyx_v_den;
  double __pyx_v_dk;
  double __pyx_v_sgn;
  int __pyx_v_i;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  double __pyx_t_5;

  /* "orthogonal_eval.pxd":75
 *     cdef int i
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         nx = floor(n)
 *         if n == nx:
 */
  __pyx_t_1 = ((__pyx_v_n < 0.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":76
 * 
 *     if n < 0:
 *         nx = floor(n)             # <<<<<<<<<<<<<<
 *         if n == nx:
 *             # undefined
 */
    __pyx_v_nx = floor(__pyx_v_n);

    /* "orthogonal_eval.pxd":77
 *     if n < 0:
 *         nx = floor(n)
 *         if n == nx:             # <<<<<<<<<<<<<<
 *             # undefined
 *             return nan
 */
    __pyx_t_1 = ((__pyx_v_n == __pyx_v_nx) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":79
 *         if n == nx:
 *             # undefined
 *             return nan             # <<<<<<<<<<<<<<
 * 
 *     kx = floor(k)
 */
      __pyx_r = NPY_NAN;
      goto __pyx_L0;

      /* "orthogonal_eval.pxd":77
 *     if n < 0:
 *         nx = floor(n)
 *         if n == nx:             # <<<<<<<<<<<<<<
 *             # undefined
 *             return nan
 */
    }

    /* "orthogonal_eval.pxd":75
 *     cdef int i
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         nx = floor(n)
 *         if n == nx:
 */
  }

  /* "orthogonal_eval.pxd":81
 *             return nan
 * 
 *     kx = floor(k)             # <<<<<<<<<<<<<<
 *     if k == kx and (fabs(n) > 1e-8 or n == 0):
 *         # Integer case: use multiplication formula for less rounding error
 */
  __pyx_v_kx = floor(__pyx_v_k);

  /* "orthogonal_eval.pxd":82
 * 
 *     kx = floor(k)
 *     if k == kx and (fabs(n) > 1e-8 or n == 0):             # <<<<<<<<<<<<<<
 *         # Integer case: use multiplication formula for less rounding error
 *         # for cases where the result is an integer.
 */
  __pyx_t_2 = ((__pyx_v_k == __pyx_v_kx) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__pyx_v_n) > 1e-8) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_n == 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":89
 *         # precision.
 * 
 *         nx = floor(n)             # <<<<<<<<<<<<<<
 *         if nx == n and kx > nx/2 and nx > 0:
 *             # Reduce kx by symmetry
 */
    __pyx_v_nx = floor(__pyx_v_n);

    /* "orthogonal_eval.pxd":90
 * 
 *         nx = floor(n)
 *         if nx == n and kx > nx/2 and nx > 0:             # <<<<<<<<<<<<<<
 *             # Reduce kx by symmetry
 *             kx = nx - kx
 */
    __pyx_t_2 = ((__pyx_v_nx == __pyx_v_n) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L10_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_kx > (__pyx_v_nx / 2.0)) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L10_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_nx > 0.0) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L10_bool_binop_done:;
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":92
 *         if nx == n and kx > nx/2 and nx > 0:
 *             # Reduce kx by symmetry
 *             kx = nx - kx             # <<<<<<<<<<<<<<
 * 
 *         if kx >= 0 and kx < 20:
 */
      __pyx_v_kx = (__pyx_v_nx - __pyx_v_kx);

      /* "orthogonal_eval.pxd":90
 * 
 *         nx = floor(n)
 *         if nx == n and kx > nx/2 and nx > 0:             # <<<<<<<<<<<<<<
 *             # Reduce kx by symmetry
 *             kx = nx - kx
 */
    }

    /* "orthogonal_eval.pxd":94
 *             kx = nx - kx
 * 
 *         if kx >= 0 and kx < 20:             # <<<<<<<<<<<<<<
 *             num = 1.0
 *             den = 1.0
 */
    __pyx_t_2 = ((__pyx_v_kx >= 0.0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_kx < 20.0) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L14_bool_binop_done:;
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":95
 * 
 *         if kx >= 0 and kx < 20:
 *             num = 1.0             # <<<<<<<<<<<<<<
 *             den = 1.0
 *             for i in range(1, 1 + <int>kx):
 */
      __pyx_v_num = 1.0;

      /* "orthogonal_eval.pxd":96
 *         if kx >= 0 and kx < 20:
 *             num = 1.0
 *             den = 1.0             # <<<<<<<<<<<<<<
 *             for i in range(1, 1 + <int>kx):
 *                 num *= i + n - kx
 */
      __pyx_v_den = 1.0;

      /* "orthogonal_eval.pxd":97
 *             num = 1.0
 *             den = 1.0
 *             for i in range(1, 1 + <int>kx):             # <<<<<<<<<<<<<<
 *                 num *= i + n - kx
 *                 den *= i
 */
      __pyx_t_3 = (1 + ((int)__pyx_v_kx));
      for (__pyx_t_4 = 1; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
        __pyx_v_i = __pyx_t_4;

        /* "orthogonal_eval.pxd":98
 *             den = 1.0
 *             for i in range(1, 1 + <int>kx):
 *                 num *= i + n - kx             # <<<<<<<<<<<<<<
 *                 den *= i
 *                 if fabs(num) > 1e50:
 */
        __pyx_v_num = (__pyx_v_num * ((__pyx_v_i + __pyx_v_n) - __pyx_v_kx));

        /* "orthogonal_eval.pxd":99
 *             for i in range(1, 1 + <int>kx):
 *                 num *= i + n - kx
 *                 den *= i             # <<<<<<<<<<<<<<
 *                 if fabs(num) > 1e50:
 *                     num /= den
 */
        __pyx_v_den = (__pyx_v_den * __pyx_v_i);

        /* "orthogonal_eval.pxd":100
 *                 num *= i + n - kx
 *                 den *= i
 *                 if fabs(num) > 1e50:             # <<<<<<<<<<<<<<
 *                     num /= den
 *                     den = 1.0
 */
        __pyx_t_1 = ((fabs(__pyx_v_num) > 1e50) != 0);
        if (__pyx_t_1) {

          /* "orthogonal_eval.pxd":101
 *                 den *= i
 *                 if fabs(num) > 1e50:
 *                     num /= den             # <<<<<<<<<<<<<<
 *                     den = 1.0
 *             return num/den
 */
          __pyx_v_num = (__pyx_v_num / __pyx_v_den);

          /* "orthogonal_eval.pxd":102
 *                 if fabs(num) > 1e50:
 *                     num /= den
 *                     den = 1.0             # <<<<<<<<<<<<<<
 *             return num/den
 * 
 */
          __pyx_v_den = 1.0;

          /* "orthogonal_eval.pxd":100
 *                 num *= i + n - kx
 *                 den *= i
 *                 if fabs(num) > 1e50:             # <<<<<<<<<<<<<<
 *                     num /= den
 *                     den = 1.0
 */
        }
      }

      /* "orthogonal_eval.pxd":103
 *                     num /= den
 *                     den = 1.0
 *             return num/den             # <<<<<<<<<<<<<<
 * 
 *     # general case:
 */
      __pyx_r = (__pyx_v_num / __pyx_v_den);
      goto __pyx_L0;

      /* "orthogonal_eval.pxd":94
 *             kx = nx - kx
 * 
 *         if kx >= 0 and kx < 20:             # <<<<<<<<<<<<<<
 *             num = 1.0
 *             den = 1.0
 */
    }

    /* "orthogonal_eval.pxd":82
 * 
 *     kx = floor(k)
 *     if k == kx and (fabs(n) > 1e-8 or n == 0):             # <<<<<<<<<<<<<<
 *         # Integer case: use multiplication formula for less rounding error
 *         # for cases where the result is an integer.
 */
  }

  /* "orthogonal_eval.pxd":106
 * 
 *     # general case:
 *     if n >= 1e10*k and k > 0:             # <<<<<<<<<<<<<<
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))
 */
  __pyx_t_2 = ((__pyx_v_n >= (1e10 * __pyx_v_k)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_k > 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L20_bool_binop_done:;
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":108
 *     if n >= 1e10*k and k > 0:
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))             # <<<<<<<<<<<<<<
 *     elif k > 1e8*fabs(n):
 *         # avoid loss of precision
 */
    __pyx_r = exp(((-lbeta(((1.0 + __pyx_v_n) - __pyx_v_k), (1.0 + __pyx_v_k))) - log((__pyx_v_n + 1.0))));
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":106
 * 
 *     # general case:
 *     if n >= 1e10*k and k > 0:             # <<<<<<<<<<<<<<
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))
 */
  }

  /* "orthogonal_eval.pxd":109
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))
 *     elif k > 1e8*fabs(n):             # <<<<<<<<<<<<<<
 *         # avoid loss of precision
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 */
  __pyx_t_1 = ((__pyx_v_k > (1e8 * fabs(__pyx_v_n))) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":111
 *     elif k > 1e8*fabs(n):
 *         # avoid loss of precision
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...             # <<<<<<<<<<<<<<
 *         num /= pi * fabs(k)**n
 *         if k > 0:
 */
    __pyx_v_num = ((Gamma((1.0 + __pyx_v_n)) / fabs(__pyx_v_k)) + ((Gamma((1.0 + __pyx_v_n)) * __pyx_v_n) / (2.0 * pow(__pyx_v_k, 2.0))));

    /* "orthogonal_eval.pxd":112
 *         # avoid loss of precision
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 *         num /= pi * fabs(k)**n             # <<<<<<<<<<<<<<
 *         if k > 0:
 *             kx = floor(k)
 */
    __pyx_v_num = (__pyx_v_num / (M_PI * pow(fabs(__pyx_v_k), __pyx_v_n)));

    /* "orthogonal_eval.pxd":113
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 *         num /= pi * fabs(k)**n
 *         if k > 0:             # <<<<<<<<<<<<<<
 *             kx = floor(k)
 *             if <int>kx == kx:
 */
    __pyx_t_1 = ((__pyx_v_k > 0.0) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":114
 *         num /= pi * fabs(k)**n
 *         if k > 0:
 *             kx = floor(k)             # <<<<<<<<<<<<<<
 *             if <int>kx == kx:
 *                 dk = k - kx
 */
      __pyx_v_kx = floor(__pyx_v_k);

      /* "orthogonal_eval.pxd":115
 *         if k > 0:
 *             kx = floor(k)
 *             if <int>kx == kx:             # <<<<<<<<<<<<<<
 *                 dk = k - kx
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1
 */
      __pyx_t_1 = ((((int)__pyx_v_kx) == __pyx_v_kx) != 0);
      if (__pyx_t_1) {

        /* "orthogonal_eval.pxd":116
 *             kx = floor(k)
 *             if <int>kx == kx:
 *                 dk = k - kx             # <<<<<<<<<<<<<<
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1
 *             else:
 */
        __pyx_v_dk = (__pyx_v_k - __pyx_v_kx);

        /* "orthogonal_eval.pxd":117
 *             if <int>kx == kx:
 *                 dk = k - kx
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1             # <<<<<<<<<<<<<<
 *             else:
 *                 dk = k
 */
        if ((((((int)__pyx_v_kx) % 2) == 0) != 0)) {
          __pyx_t_5 = 1.0;
        } else {
          __pyx_t_5 = -1.0;
        }
        __pyx_v_sgn = __pyx_t_5;

        /* "orthogonal_eval.pxd":115
 *         if k > 0:
 *             kx = floor(k)
 *             if <int>kx == kx:             # <<<<<<<<<<<<<<
 *                 dk = k - kx
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1
 */
        goto __pyx_L23;
      }

      /* "orthogonal_eval.pxd":119
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1
 *             else:
 *                 dk = k             # <<<<<<<<<<<<<<
 *                 sgn = 1
 *             return num * sin((dk-n)*pi) * sgn
 */
      /*else*/ {
        __pyx_v_dk = __pyx_v_k;

        /* "orthogonal_eval.pxd":120
 *             else:
 *                 dk = k
 *                 sgn = 1             # <<<<<<<<<<<<<<
 *             return num * sin((dk-n)*pi) * sgn
 *         else:
 */
        __pyx_v_sgn = 1.0;
      }
      __pyx_L23:;

      /* "orthogonal_eval.pxd":121
 *                 dk = k
 *                 sgn = 1
 *             return num * sin((dk-n)*pi) * sgn             # <<<<<<<<<<<<<<
 *         else:
 *             kx = floor(k)
 */
      __pyx_r = ((__pyx_v_num * sin(((__pyx_v_dk - __pyx_v_n) * M_PI))) * __pyx_v_sgn);
      goto __pyx_L0;

      /* "orthogonal_eval.pxd":113
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 *         num /= pi * fabs(k)**n
 *         if k > 0:             # <<<<<<<<<<<<<<
 *             kx = floor(k)
 *             if <int>kx == kx:
 */
    }

    /* "orthogonal_eval.pxd":123
 *             return num * sin((dk-n)*pi) * sgn
 *         else:
 *             kx = floor(k)             # <<<<<<<<<<<<<<
 *             if <int>kx == kx:
 *                 return 0
 */
    /*else*/ {
      __pyx_v_kx = floor(__pyx_v_k);

      /* "orthogonal_eval.pxd":124
 *         else:
 *             kx = floor(k)
 *             if <int>kx == kx:             # <<<<<<<<<<<<<<
 *                 return 0
 *             else:
 */
      __pyx_t_1 = ((((int)__pyx_v_kx) == __pyx_v_kx) != 0);
      if (__pyx_t_1) {

        /* "orthogonal_eval.pxd":125
 *             kx = floor(k)
 *             if <int>kx == kx:
 *                 return 0             # <<<<<<<<<<<<<<
 *             else:
 *                 return num * sin(k*pi)
 */
        __pyx_r = 0.0;
        goto __pyx_L0;

        /* "orthogonal_eval.pxd":124
 *         else:
 *             kx = floor(k)
 *             if <int>kx == kx:             # <<<<<<<<<<<<<<
 *                 return 0
 *             else:
 */
      }

      /* "orthogonal_eval.pxd":127
 *                 return 0
 *             else:
 *                 return num * sin(k*pi)             # <<<<<<<<<<<<<<
 *     else:
 *         return 1/beta(1 + n - k, 1 + k)/(n + 1)
 */
      /*else*/ {
        __pyx_r = (__pyx_v_num * sin((__pyx_v_k * M_PI)));
        goto __pyx_L0;
      }
    }

    /* "orthogonal_eval.pxd":109
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))
 *     elif k > 1e8*fabs(n):             # <<<<<<<<<<<<<<
 *         # avoid loss of precision
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 */
  }

  /* "orthogonal_eval.pxd":129
 *                 return num * sin(k*pi)
 *     else:
 *         return 1/beta(1 + n - k, 1 + k)/(n + 1)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  /*else*/ {
    __pyx_r = ((1.0 / beta(((1.0 + __pyx_v_n) - __pyx_v_k), (1.0 + __pyx_v_k))) / (__pyx_v_n + 1.0));
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":71
 * 
 * @cython.cdivision(True)
 * cdef inline double binom(double n, double k) nogil:             # <<<<<<<<<<<<<<
 *     cdef double kx, nx, num, den, dk, sgn
 *     cdef int i
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":135
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_jacobi(double n, double alpha, double beta, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(double __pyx_v_n, double __pyx_v_alpha, double __pyx_v_beta, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":139
 *     cdef number_t g
 * 
 *     d = binom(n+alpha, n)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n + alpha + beta + 1
 */
  __pyx_v_d = __pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n);

  /* "orthogonal_eval.pxd":140
 * 
 *     d = binom(n+alpha, n)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n + alpha + beta + 1
 *     c = alpha + 1
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":141
 *     d = binom(n+alpha, n)
 *     a = -n
 *     b = n + alpha + beta + 1             # <<<<<<<<<<<<<<
 *     c = alpha + 1
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (((__pyx_v_n + __pyx_v_alpha) + __pyx_v_beta) + 1.0);

  /* "orthogonal_eval.pxd":142
 *     a = -n
 *     b = n + alpha + beta + 1
 *     c = alpha + 1             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d * hyp2f1(a, b, c, g)
 */
  __pyx_v_c = (__pyx_v_alpha + 1.0);

  /* "orthogonal_eval.pxd":143
 *     b = n + alpha + beta + 1
 *     c = alpha + 1
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d * hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = (0.5 * (1.0 - __pyx_v_x));

  /* "orthogonal_eval.pxd":144
 *     c = alpha + 1
 *     g = 0.5*(1-x)
 *     return d * hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":135
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_jacobi(double n, double alpha, double beta, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(double __pyx_v_n, double __pyx_v_alpha, double __pyx_v_beta, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":139
 *     cdef number_t g
 * 
 *     d = binom(n+alpha, n)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n + alpha + beta + 1
 */
  __pyx_v_d = __pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n);

  /* "orthogonal_eval.pxd":140
 * 
 *     d = binom(n+alpha, n)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n + alpha + beta + 1
 *     c = alpha + 1
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":141
 *     d = binom(n+alpha, n)
 *     a = -n
 *     b = n + alpha + beta + 1             # <<<<<<<<<<<<<<
 *     c = alpha + 1
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (((__pyx_v_n + __pyx_v_alpha) + __pyx_v_beta) + 1.0);

  /* "orthogonal_eval.pxd":142
 *     a = -n
 *     b = n + alpha + beta + 1
 *     c = alpha + 1             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d * hyp2f1(a, b, c, g)
 */
  __pyx_v_c = (__pyx_v_alpha + 1.0);

  /* "orthogonal_eval.pxd":143
 *     b = n + alpha + beta + 1
 *     c = alpha + 1
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d * hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_prod(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x));

  /* "orthogonal_eval.pxd":144
 *     c = alpha + 1
 *     g = 0.5*(1-x)
 *     return d * hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_prod(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":135
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_jacobi(double n, double alpha, double beta, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":147
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_jacobi_l(long n, double alpha, double beta, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef double p, d
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l(long __pyx_v_n, double __pyx_v_alpha, double __pyx_v_beta, double __pyx_v_x) {
  long __pyx_v_kk;
  double __pyx_v_p;
  double __pyx_v_d;
  double __pyx_v_k;
  double __pyx_v_t;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;

  /* "orthogonal_eval.pxd":152
 *     cdef double k, t
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":153
 * 
 *     if n < 0:
 *         return eval_jacobi(n, alpha, beta, x)             # <<<<<<<<<<<<<<
 *     elif n == 0:
 *         return 1.0
 */
    __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(__pyx_v_n, __pyx_v_alpha, __pyx_v_beta, __pyx_v_x);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":152
 *     cdef double k, t
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:
 */
  }

  /* "orthogonal_eval.pxd":154
 *     if n < 0:
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":155
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":154
 *     if n < 0:
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":156
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":157
 *         return 1.0
 *     elif n == 1:
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))             # <<<<<<<<<<<<<<
 *     else:
 *         d = (alpha+beta+2)*(x - 1) / (2*(alpha+1))
 */
    __pyx_r = (0.5 * ((2.0 * (__pyx_v_alpha + 1.0)) + (((__pyx_v_alpha + __pyx_v_beta) + 2.0) * (__pyx_v_x - 1.0))));
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":156
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))
 *     else:
 */
  }

  /* "orthogonal_eval.pxd":159
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))
 *     else:
 *         d = (alpha+beta+2)*(x - 1) / (2*(alpha+1))             # <<<<<<<<<<<<<<
 *         p = d + 1
 *         for kk in range(n-1):
 */
  /*else*/ {
    __pyx_v_d = ((((__pyx_v_alpha + __pyx_v_beta) + 2.0) * (__pyx_v_x - 1.0)) / (2.0 * (__pyx_v_alpha + 1.0)));

    /* "orthogonal_eval.pxd":160
 *     else:
 *         d = (alpha+beta+2)*(x - 1) / (2*(alpha+1))
 *         p = d + 1             # <<<<<<<<<<<<<<
 *         for kk in range(n-1):
 *             k = kk+1.0
 */
    __pyx_v_p = (__pyx_v_d + 1.0);

    /* "orthogonal_eval.pxd":161
 *         d = (alpha+beta+2)*(x - 1) / (2*(alpha+1))
 *         p = d + 1
 *         for kk in range(n-1):             # <<<<<<<<<<<<<<
 *             k = kk+1.0
 *             t = 2*k+alpha+beta
 */
    __pyx_t_2 = (__pyx_v_n - 1);
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_kk = __pyx_t_3;

      /* "orthogonal_eval.pxd":162
 *         p = d + 1
 *         for kk in range(n-1):
 *             k = kk+1.0             # <<<<<<<<<<<<<<
 *             t = 2*k+alpha+beta
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)
 */
      __pyx_v_k = (__pyx_v_kk + 1.0);

      /* "orthogonal_eval.pxd":163
 *         for kk in range(n-1):
 *             k = kk+1.0
 *             t = 2*k+alpha+beta             # <<<<<<<<<<<<<<
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)
 *             p = d + p
 */
      __pyx_v_t = (((2.0 * __pyx_v_k) + __pyx_v_alpha) + __pyx_v_beta);

      /* "orthogonal_eval.pxd":164
 *             k = kk+1.0
 *             t = 2*k+alpha+beta
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)             # <<<<<<<<<<<<<<
 *             p = d + p
 *         return binom(n+alpha, n)*p
 */
      __pyx_v_d = ((((((__pyx_v_t * (__pyx_v_t + 1.0)) * (__pyx_v_t + 2.0)) * (__pyx_v_x - 1.0)) * __pyx_v_p) + ((((2.0 * __pyx_v_k) * (__pyx_v_k + __pyx_v_beta)) * (__pyx_v_t + 2.0)) * __pyx_v_d)) / (((2.0 * ((__pyx_v_k + __pyx_v_alpha) + 1.0)) * (((__pyx_v_k + __pyx_v_alpha) + __pyx_v_beta) + 1.0)) * __pyx_v_t));

      /* "orthogonal_eval.pxd":165
 *             t = 2*k+alpha+beta
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)
 *             p = d + p             # <<<<<<<<<<<<<<
 *         return binom(n+alpha, n)*p
 * 
 */
      __pyx_v_p = (__pyx_v_d + __pyx_v_p);
    }

    /* "orthogonal_eval.pxd":166
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)
 *             p = d + p
 *         return binom(n+alpha, n)*p             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n) * __pyx_v_p);
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":147
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_jacobi_l(long n, double alpha, double beta, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef double p, d
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":173
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi(double __pyx_v_n, double __pyx_v_p, double __pyx_v_q, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":174
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(__pyx_v_n, (__pyx_v_p - __pyx_v_q), (__pyx_v_q - 1.0), ((2.0 * __pyx_v_x) - 1.0)) / __pyx_f_5scipy_7special_15orthogonal_eval_binom((((2.0 * __pyx_v_n) + __pyx_v_p) - 1.0), __pyx_v_n));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":173
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi(double __pyx_v_n, double __pyx_v_p, double __pyx_v_q, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":174
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_quot(__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(__pyx_v_n, (__pyx_v_p - __pyx_v_q), (__pyx_v_q - 1.0), __Pyx_c_diff(__Pyx_c_prod(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(1, 0))), __pyx_t_double_complex_from_parts(__pyx_f_5scipy_7special_15orthogonal_eval_binom((((2.0 * __pyx_v_n) + __pyx_v_p) - 1.0), __pyx_v_n), 0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":173
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":177
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_sh_jacobi_l(long n, double p, double q, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi_l(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi_l(long __pyx_v_n, double __pyx_v_p, double __pyx_v_q, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":178
 * @cython.cdivision(True)
 * cdef inline double eval_sh_jacobi_l(long n, double p, double q, double x) nogil:
 *     return eval_jacobi_l(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l(__pyx_v_n, (__pyx_v_p - __pyx_v_q), (__pyx_v_q - 1.0), ((2.0 * __pyx_v_x) - 1.0)) / __pyx_f_5scipy_7special_15orthogonal_eval_binom((((2 * __pyx_v_n) + __pyx_v_p) - 1.0), __pyx_v_n));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":177
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_sh_jacobi_l(long n, double p, double q, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi_l(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":185
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_gegenbauer(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(double __pyx_v_n, double __pyx_v_alpha, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":189
 *     cdef number_t g
 * 
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n + 2*alpha
 */
  __pyx_v_d = ((Gamma((__pyx_v_n + (2.0 * __pyx_v_alpha))) / Gamma((1.0 + __pyx_v_n))) / Gamma((2.0 * __pyx_v_alpha)));

  /* "orthogonal_eval.pxd":190
 * 
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n + 2*alpha
 *     c = alpha + 0.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":191
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)
 *     a = -n
 *     b = n + 2*alpha             # <<<<<<<<<<<<<<
 *     c = alpha + 0.5
 *     g = (1-x)/2.0
 */
  __pyx_v_b = (__pyx_v_n + (2.0 * __pyx_v_alpha));

  /* "orthogonal_eval.pxd":192
 *     a = -n
 *     b = n + 2*alpha
 *     c = alpha + 0.5             # <<<<<<<<<<<<<<
 *     g = (1-x)/2.0
 *     return d * hyp2f1(a, b, c, g)
 */
  __pyx_v_c = (__pyx_v_alpha + 0.5);

  /* "orthogonal_eval.pxd":193
 *     b = n + 2*alpha
 *     c = alpha + 0.5
 *     g = (1-x)/2.0             # <<<<<<<<<<<<<<
 *     return d * hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = ((1.0 - __pyx_v_x) / 2.0);

  /* "orthogonal_eval.pxd":194
 *     c = alpha + 0.5
 *     g = (1-x)/2.0
 *     return d * hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":185
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_gegenbauer(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(double __pyx_v_n, double __pyx_v_alpha, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":189
 *     cdef number_t g
 * 
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n + 2*alpha
 */
  __pyx_v_d = ((Gamma((__pyx_v_n + (2.0 * __pyx_v_alpha))) / Gamma((1.0 + __pyx_v_n))) / Gamma((2.0 * __pyx_v_alpha)));

  /* "orthogonal_eval.pxd":190
 * 
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n + 2*alpha
 *     c = alpha + 0.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":191
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)
 *     a = -n
 *     b = n + 2*alpha             # <<<<<<<<<<<<<<
 *     c = alpha + 0.5
 *     g = (1-x)/2.0
 */
  __pyx_v_b = (__pyx_v_n + (2.0 * __pyx_v_alpha));

  /* "orthogonal_eval.pxd":192
 *     a = -n
 *     b = n + 2*alpha
 *     c = alpha + 0.5             # <<<<<<<<<<<<<<
 *     g = (1-x)/2.0
 *     return d * hyp2f1(a, b, c, g)
 */
  __pyx_v_c = (__pyx_v_alpha + 0.5);

  /* "orthogonal_eval.pxd":193
 *     b = n + 2*alpha
 *     c = alpha + 0.5
 *     g = (1-x)/2.0             # <<<<<<<<<<<<<<
 *     return d * hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_quot(__Pyx_c_diff(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(2.0, 0));

  /* "orthogonal_eval.pxd":194
 *     c = alpha + 0.5
 *     g = (1-x)/2.0
 *     return d * hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_prod(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":185
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_gegenbauer(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":197
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_gegenbauer_l(long n, double alpha, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef long a, b
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer_l(long __pyx_v_n, double __pyx_v_alpha, double __pyx_v_x) {
  long __pyx_v_kk;
  long __pyx_v_a;
  double __pyx_v_p;
  double __pyx_v_d;
  double __pyx_v_k;
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;

  /* "orthogonal_eval.pxd":203
 *     cdef double k
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":204
 * 
 *     if n < 0:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     elif n == 0:
 *         return 1.0
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":203
 *     cdef double k
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  }

  /* "orthogonal_eval.pxd":205
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":206
 *         return 0.0
 *     elif n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return 2*alpha*x
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":205
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":207
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return 2*alpha*x
 *     elif alpha == 0.0:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":208
 *         return 1.0
 *     elif n == 1:
 *         return 2*alpha*x             # <<<<<<<<<<<<<<
 *     elif alpha == 0.0:
 *         return eval_gegenbauer(n, alpha, x)
 */
    __pyx_r = ((2.0 * __pyx_v_alpha) * __pyx_v_x);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":207
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return 2*alpha*x
 *     elif alpha == 0.0:
 */
  }

  /* "orthogonal_eval.pxd":209
 *     elif n == 1:
 *         return 2*alpha*x
 *     elif alpha == 0.0:             # <<<<<<<<<<<<<<
 *         return eval_gegenbauer(n, alpha, x)
 *     elif fabs(x) < 1e-5:
 */
  __pyx_t_1 = ((__pyx_v_alpha == 0.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":210
 *         return 2*alpha*x
 *     elif alpha == 0.0:
 *         return eval_gegenbauer(n, alpha, x)             # <<<<<<<<<<<<<<
 *     elif fabs(x) < 1e-5:
 *         # Power series rather than recurrence due to loss of precision
 */
    __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(__pyx_v_n, __pyx_v_alpha, __pyx_v_x);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":209
 *     elif n == 1:
 *         return 2*alpha*x
 *     elif alpha == 0.0:             # <<<<<<<<<<<<<<
 *         return eval_gegenbauer(n, alpha, x)
 *     elif fabs(x) < 1e-5:
 */
  }

  /* "orthogonal_eval.pxd":211
 *     elif alpha == 0.0:
 *         return eval_gegenbauer(n, alpha, x)
 *     elif fabs(x) < 1e-5:             # <<<<<<<<<<<<<<
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/GegenbauerC3/02/
 */
  __pyx_t_1 = ((fabs(__pyx_v_x) < 1e-5) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":214
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/GegenbauerC3/02/
 *         a = n//2             # <<<<<<<<<<<<<<
 * 
 *         d = 1 if a % 2 == 0 else -1
 */
    __pyx_v_a = __Pyx_div_long(__pyx_v_n, 2);

    /* "orthogonal_eval.pxd":216
 *         a = n//2
 * 
 *         d = 1 if a % 2 == 0 else -1             # <<<<<<<<<<<<<<
 *         d /= beta(alpha, 1 + a)
 *         if n == 2*a:
 */
    if ((((__pyx_v_a % 2) == 0) != 0)) {
      __pyx_t_2 = 1.0;
    } else {
      __pyx_t_2 = -1.0;
    }
    __pyx_v_d = __pyx_t_2;

    /* "orthogonal_eval.pxd":217
 * 
 *         d = 1 if a % 2 == 0 else -1
 *         d /= beta(alpha, 1 + a)             # <<<<<<<<<<<<<<
 *         if n == 2*a:
 *             d /= (a + alpha)
 */
    __pyx_v_d = (__pyx_v_d / beta(__pyx_v_alpha, (1 + __pyx_v_a)));

    /* "orthogonal_eval.pxd":218
 *         d = 1 if a % 2 == 0 else -1
 *         d /= beta(alpha, 1 + a)
 *         if n == 2*a:             # <<<<<<<<<<<<<<
 *             d /= (a + alpha)
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_n == (2 * __pyx_v_a)) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":219
 *         d /= beta(alpha, 1 + a)
 *         if n == 2*a:
 *             d /= (a + alpha)             # <<<<<<<<<<<<<<
 *         else:
 *             d *= 2*x
 */
      __pyx_v_d = (__pyx_v_d / (__pyx_v_a + __pyx_v_alpha));

      /* "orthogonal_eval.pxd":218
 *         d = 1 if a % 2 == 0 else -1
 *         d /= beta(alpha, 1 + a)
 *         if n == 2*a:             # <<<<<<<<<<<<<<
 *             d /= (a + alpha)
 *         else:
 */
      goto __pyx_L4;
    }

    /* "orthogonal_eval.pxd":221
 *             d /= (a + alpha)
 *         else:
 *             d *= 2*x             # <<<<<<<<<<<<<<
 * 
 *         p = 0
 */
    /*else*/ {
      __pyx_v_d = (__pyx_v_d * (2.0 * __pyx_v_x));
    }
    __pyx_L4:;

    /* "orthogonal_eval.pxd":223
 *             d *= 2*x
 * 
 *         p = 0             # <<<<<<<<<<<<<<
 *         for kk in range(a+1):
 *             p += d
 */
    __pyx_v_p = 0.0;

    /* "orthogonal_eval.pxd":224
 * 
 *         p = 0
 *         for kk in range(a+1):             # <<<<<<<<<<<<<<
 *             p += d
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (
 */
    __pyx_t_3 = (__pyx_v_a + 1);
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_kk = __pyx_t_4;

      /* "orthogonal_eval.pxd":225
 *         p = 0
 *         for kk in range(a+1):
 *             p += d             # <<<<<<<<<<<<<<
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 */
      __pyx_v_p = (__pyx_v_p + __pyx_v_d);

      /* "orthogonal_eval.pxd":226
 *         for kk in range(a+1):
 *             p += d
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (             # <<<<<<<<<<<<<<
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):
 */
      __pyx_v_d = (__pyx_v_d * ((((-4.0 * pow(__pyx_v_x, 2.0)) * (__pyx_v_a - __pyx_v_kk)) * ((((-__pyx_v_a) + __pyx_v_alpha) + __pyx_v_kk) + __pyx_v_n)) / ((((__pyx_v_n + 1) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk)) * (((__pyx_v_n + 2) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk)))));

      /* "orthogonal_eval.pxd":228
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):             # <<<<<<<<<<<<<<
 *                 # converged
 *                 break
 */
      __pyx_t_1 = ((fabs(__pyx_v_d) == (1e-20 * fabs(__pyx_v_p))) != 0);
      if (__pyx_t_1) {

        /* "orthogonal_eval.pxd":230
 *             if fabs(d) == 1e-20*fabs(p):
 *                 # converged
 *                 break             # <<<<<<<<<<<<<<
 *         return p
 *     else:
 */
        goto __pyx_L6_break;

        /* "orthogonal_eval.pxd":228
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):             # <<<<<<<<<<<<<<
 *                 # converged
 *                 break
 */
      }
    }
    __pyx_L6_break:;

    /* "orthogonal_eval.pxd":231
 *                 # converged
 *                 break
 *         return p             # <<<<<<<<<<<<<<
 *     else:
 *         d = x - 1
 */
    __pyx_r = __pyx_v_p;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":211
 *     elif alpha == 0.0:
 *         return eval_gegenbauer(n, alpha, x)
 *     elif fabs(x) < 1e-5:             # <<<<<<<<<<<<<<
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/GegenbauerC3/02/
 */
  }

  /* "orthogonal_eval.pxd":233
 *         return p
 *     else:
 *         d = x - 1             # <<<<<<<<<<<<<<
 *         p = x
 *         for kk in range(n-1):
 */
  /*else*/ {
    __pyx_v_d = (__pyx_v_x - 1.0);

    /* "orthogonal_eval.pxd":234
 *     else:
 *         d = x - 1
 *         p = x             # <<<<<<<<<<<<<<
 *         for kk in range(n-1):
 *             k = kk+1.0
 */
    __pyx_v_p = __pyx_v_x;

    /* "orthogonal_eval.pxd":235
 *         d = x - 1
 *         p = x
 *         for kk in range(n-1):             # <<<<<<<<<<<<<<
 *             k = kk+1.0
 *             d = (2*(k+alpha)/(k+2*alpha))*(x-1)*p + (k/(k+2*alpha)) * d
 */
    __pyx_t_3 = (__pyx_v_n - 1);
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_kk = __pyx_t_4;

      /* "orthogonal_eval.pxd":236
 *         p = x
 *         for kk in range(n-1):
 *             k = kk+1.0             # <<<<<<<<<<<<<<
 *             d = (2*(k+alpha)/(k+2*alpha))*(x-1)*p + (k/(k+2*alpha)) * d
 *             p = d + p
 */
      __pyx_v_k = (__pyx_v_kk + 1.0);

      /* "orthogonal_eval.pxd":237
 *         for kk in range(n-1):
 *             k = kk+1.0
 *             d = (2*(k+alpha)/(k+2*alpha))*(x-1)*p + (k/(k+2*alpha)) * d             # <<<<<<<<<<<<<<
 *             p = d + p
 * 
 */
      __pyx_v_d = (((((2.0 * (__pyx_v_k + __pyx_v_alpha)) / (__pyx_v_k + (2.0 * __pyx_v_alpha))) * (__pyx_v_x - 1.0)) * __pyx_v_p) + ((__pyx_v_k / (__pyx_v_k + (2.0 * __pyx_v_alpha))) * __pyx_v_d));

      /* "orthogonal_eval.pxd":238
 *             k = kk+1.0
 *             d = (2*(k+alpha)/(k+2*alpha))*(x-1)*p + (k/(k+2*alpha)) * d
 *             p = d + p             # <<<<<<<<<<<<<<
 * 
 *         if fabs(alpha/n) < 1e-8:
 */
      __pyx_v_p = (__pyx_v_d + __pyx_v_p);
    }

    /* "orthogonal_eval.pxd":240
 *             p = d + p
 * 
 *         if fabs(alpha/n) < 1e-8:             # <<<<<<<<<<<<<<
 *             # avoid loss of precision
 *             return 2*alpha/n * p
 */
    __pyx_t_1 = ((fabs((__pyx_v_alpha / __pyx_v_n)) < 1e-8) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":242
 *         if fabs(alpha/n) < 1e-8:
 *             # avoid loss of precision
 *             return 2*alpha/n * p             # <<<<<<<<<<<<<<
 *         else:
 *             return binom(n+2*alpha-1, n)*p
 */
      __pyx_r = (((2.0 * __pyx_v_alpha) / __pyx_v_n) * __pyx_v_p);
      goto __pyx_L0;

      /* "orthogonal_eval.pxd":240
 *             p = d + p
 * 
 *         if fabs(alpha/n) < 1e-8:             # <<<<<<<<<<<<<<
 *             # avoid loss of precision
 *             return 2*alpha/n * p
 */
    }

    /* "orthogonal_eval.pxd":244
 *             return 2*alpha/n * p
 *         else:
 *             return binom(n+2*alpha-1, n)*p             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    /*else*/ {
      __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_binom(((__pyx_v_n + (2.0 * __pyx_v_alpha)) - 1.0), __pyx_v_n) * __pyx_v_p);
      goto __pyx_L0;
    }
  }

  /* "orthogonal_eval.pxd":197
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_gegenbauer_l(long n, double alpha, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef long a, b
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":250
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  CYTHON_UNUSED double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":254
 *     cdef number_t g
 * 
 *     d = 1.0             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n
 */
  __pyx_v_d = 1.0;

  /* "orthogonal_eval.pxd":255
 * 
 *     d = 1.0
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n
 *     c = 0.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":256
 *     d = 1.0
 *     a = -n
 *     b = n             # <<<<<<<<<<<<<<
 *     c = 0.5
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = __pyx_v_n;

  /* "orthogonal_eval.pxd":257
 *     a = -n
 *     b = n
 *     c = 0.5             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 0.5;

  /* "orthogonal_eval.pxd":258
 *     b = n
 *     c = 0.5
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = (0.5 * (1.0 - __pyx_v_x));

  /* "orthogonal_eval.pxd":259
 *     c = 0.5
 *     g = 0.5*(1-x)
 *     return hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyt_l(long k, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":250
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  CYTHON_UNUSED double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":254
 *     cdef number_t g
 * 
 *     d = 1.0             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n
 */
  __pyx_v_d = 1.0;

  /* "orthogonal_eval.pxd":255
 * 
 *     d = 1.0
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n
 *     c = 0.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":256
 *     d = 1.0
 *     a = -n
 *     b = n             # <<<<<<<<<<<<<<
 *     c = 0.5
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = __pyx_v_n;

  /* "orthogonal_eval.pxd":257
 *     a = -n
 *     b = n
 *     c = 0.5             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 0.5;

  /* "orthogonal_eval.pxd":258
 *     b = n
 *     c = 0.5
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_prod(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x));

  /* "orthogonal_eval.pxd":259
 *     c = 0.5
 *     g = 0.5*(1-x)
 *     return hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyt_l(long k, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":250
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":261
 *     return hyp2f1(a, b, c, g)
 * 
 * cdef inline double eval_chebyt_l(long k, double x) nogil:             # <<<<<<<<<<<<<<
 *     # Use Chebyshev T recurrence directly, see [MH]
 *     cdef long m
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l(long __pyx_v_k, double __pyx_v_x) {
  CYTHON_UNUSED long __pyx_v_m;
  double __pyx_v_b2;
  double __pyx_v_b1;
  double __pyx_v_b0;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;

  /* "orthogonal_eval.pxd":266
 *     cdef double b2, b1, b0
 * 
 *     if k < 0:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         k = -k
 */
  __pyx_t_1 = ((__pyx_v_k < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":268
 *     if k < 0:
 *         # symmetry
 *         k = -k             # <<<<<<<<<<<<<<
 * 
 *     b2 = 0
 */
    __pyx_v_k = (-__pyx_v_k);

    /* "orthogonal_eval.pxd":266
 *     cdef double b2, b1, b0
 * 
 *     if k < 0:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         k = -k
 */
  }

  /* "orthogonal_eval.pxd":270
 *         k = -k
 * 
 *     b2 = 0             # <<<<<<<<<<<<<<
 *     b1 = -1
 *     b0 = 0
 */
  __pyx_v_b2 = 0.0;

  /* "orthogonal_eval.pxd":271
 * 
 *     b2 = 0
 *     b1 = -1             # <<<<<<<<<<<<<<
 *     b0 = 0
 *     x = 2*x
 */
  __pyx_v_b1 = -1.0;

  /* "orthogonal_eval.pxd":272
 *     b2 = 0
 *     b1 = -1
 *     b0 = 0             # <<<<<<<<<<<<<<
 *     x = 2*x
 *     for m in range(k+1):
 */
  __pyx_v_b0 = 0.0;

  /* "orthogonal_eval.pxd":273
 *     b1 = -1
 *     b0 = 0
 *     x = 2*x             # <<<<<<<<<<<<<<
 *     for m in range(k+1):
 *         b2 = b1
 */
  __pyx_v_x = (2.0 * __pyx_v_x);

  /* "orthogonal_eval.pxd":274
 *     b0 = 0
 *     x = 2*x
 *     for m in range(k+1):             # <<<<<<<<<<<<<<
 *         b2 = b1
 *         b1 = b0
 */
  __pyx_t_2 = (__pyx_v_k + 1);
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_m = __pyx_t_3;

    /* "orthogonal_eval.pxd":275
 *     x = 2*x
 *     for m in range(k+1):
 *         b2 = b1             # <<<<<<<<<<<<<<
 *         b1 = b0
 *         b0 = x*b1 - b2
 */
    __pyx_v_b2 = __pyx_v_b1;

    /* "orthogonal_eval.pxd":276
 *     for m in range(k+1):
 *         b2 = b1
 *         b1 = b0             # <<<<<<<<<<<<<<
 *         b0 = x*b1 - b2
 *     return (b0 - b2)/2.0
 */
    __pyx_v_b1 = __pyx_v_b0;

    /* "orthogonal_eval.pxd":277
 *         b2 = b1
 *         b1 = b0
 *         b0 = x*b1 - b2             # <<<<<<<<<<<<<<
 *     return (b0 - b2)/2.0
 * 
 */
    __pyx_v_b0 = ((__pyx_v_x * __pyx_v_b1) - __pyx_v_b2);
  }

  /* "orthogonal_eval.pxd":278
 *         b1 = b0
 *         b0 = x*b1 - b2
 *     return (b0 - b2)/2.0             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = ((__pyx_v_b0 - __pyx_v_b2) / 2.0);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":261
 *     return hyp2f1(a, b, c, g)
 * 
 * cdef inline double eval_chebyt_l(long k, double x) nogil:             # <<<<<<<<<<<<<<
 *     # Use Chebyshev T recurrence directly, see [MH]
 *     cdef long m
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":284
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":288
 *     cdef number_t g
 * 
 *     d = n+1             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n+2
 */
  __pyx_v_d = (__pyx_v_n + 1.0);

  /* "orthogonal_eval.pxd":289
 * 
 *     d = n+1
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n+2
 *     c = 1.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":290
 *     d = n+1
 *     a = -n
 *     b = n+2             # <<<<<<<<<<<<<<
 *     c = 1.5
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (__pyx_v_n + 2.0);

  /* "orthogonal_eval.pxd":291
 *     a = -n
 *     b = n+2
 *     c = 1.5             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 1.5;

  /* "orthogonal_eval.pxd":292
 *     b = n+2
 *     c = 1.5
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d*hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = (0.5 * (1.0 - __pyx_v_x));

  /* "orthogonal_eval.pxd":293
 *     c = 1.5
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyu_l(long k, double x) nogil:
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":284
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":288
 *     cdef number_t g
 * 
 *     d = n+1             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n+2
 */
  __pyx_v_d = (__pyx_v_n + 1.0);

  /* "orthogonal_eval.pxd":289
 * 
 *     d = n+1
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n+2
 *     c = 1.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":290
 *     d = n+1
 *     a = -n
 *     b = n+2             # <<<<<<<<<<<<<<
 *     c = 1.5
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (__pyx_v_n + 2.0);

  /* "orthogonal_eval.pxd":291
 *     a = -n
 *     b = n+2
 *     c = 1.5             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 1.5;

  /* "orthogonal_eval.pxd":292
 *     b = n+2
 *     c = 1.5
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d*hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_prod(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x));

  /* "orthogonal_eval.pxd":293
 *     c = 1.5
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyu_l(long k, double x) nogil:
 */
  __pyx_r = __Pyx_c_prod(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":284
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":295
 *     return d*hyp2f1(a, b, c, g)
 * 
 * cdef inline double eval_chebyu_l(long k, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long m
 *     cdef int sign
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l(long __pyx_v_k, double __pyx_v_x) {
  CYTHON_UNUSED long __pyx_v_m;
  int __pyx_v_sign;
  double __pyx_v_b2;
  double __pyx_v_b1;
  double __pyx_v_b0;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;

  /* "orthogonal_eval.pxd":300
 *     cdef double b2, b1, b0
 * 
 *     if k == -1:             # <<<<<<<<<<<<<<
 *         return 0
 *     elif k < -1:
 */
  __pyx_t_1 = ((__pyx_v_k == -1L) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":301
 * 
 *     if k == -1:
 *         return 0             # <<<<<<<<<<<<<<
 *     elif k < -1:
 *         # symmetry
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":300
 *     cdef double b2, b1, b0
 * 
 *     if k == -1:             # <<<<<<<<<<<<<<
 *         return 0
 *     elif k < -1:
 */
  }

  /* "orthogonal_eval.pxd":302
 *     if k == -1:
 *         return 0
 *     elif k < -1:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         k = -k - 2
 */
  __pyx_t_1 = ((__pyx_v_k < -1L) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":304
 *     elif k < -1:
 *         # symmetry
 *         k = -k - 2             # <<<<<<<<<<<<<<
 *         sign = -1
 *     else:
 */
    __pyx_v_k = ((-__pyx_v_k) - 2);

    /* "orthogonal_eval.pxd":305
 *         # symmetry
 *         k = -k - 2
 *         sign = -1             # <<<<<<<<<<<<<<
 *     else:
 *         sign = 1
 */
    __pyx_v_sign = -1;

    /* "orthogonal_eval.pxd":302
 *     if k == -1:
 *         return 0
 *     elif k < -1:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         k = -k - 2
 */
    goto __pyx_L3;
  }

  /* "orthogonal_eval.pxd":307
 *         sign = -1
 *     else:
 *         sign = 1             # <<<<<<<<<<<<<<
 * 
 *     b2 = 0
 */
  /*else*/ {
    __pyx_v_sign = 1;
  }
  __pyx_L3:;

  /* "orthogonal_eval.pxd":309
 *         sign = 1
 * 
 *     b2 = 0             # <<<<<<<<<<<<<<
 *     b1 = -1
 *     b0 = 0
 */
  __pyx_v_b2 = 0.0;

  /* "orthogonal_eval.pxd":310
 * 
 *     b2 = 0
 *     b1 = -1             # <<<<<<<<<<<<<<
 *     b0 = 0
 *     x = 2*x
 */
  __pyx_v_b1 = -1.0;

  /* "orthogonal_eval.pxd":311
 *     b2 = 0
 *     b1 = -1
 *     b0 = 0             # <<<<<<<<<<<<<<
 *     x = 2*x
 *     for m in range(k+1):
 */
  __pyx_v_b0 = 0.0;

  /* "orthogonal_eval.pxd":312
 *     b1 = -1
 *     b0 = 0
 *     x = 2*x             # <<<<<<<<<<<<<<
 *     for m in range(k+1):
 *         b2 = b1
 */
  __pyx_v_x = (2.0 * __pyx_v_x);

  /* "orthogonal_eval.pxd":313
 *     b0 = 0
 *     x = 2*x
 *     for m in range(k+1):             # <<<<<<<<<<<<<<
 *         b2 = b1
 *         b1 = b0
 */
  __pyx_t_2 = (__pyx_v_k + 1);
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_m = __pyx_t_3;

    /* "orthogonal_eval.pxd":314
 *     x = 2*x
 *     for m in range(k+1):
 *         b2 = b1             # <<<<<<<<<<<<<<
 *         b1 = b0
 *         b0 = x*b1 - b2
 */
    __pyx_v_b2 = __pyx_v_b1;

    /* "orthogonal_eval.pxd":315
 *     for m in range(k+1):
 *         b2 = b1
 *         b1 = b0             # <<<<<<<<<<<<<<
 *         b0 = x*b1 - b2
 *     return b0 * sign
 */
    __pyx_v_b1 = __pyx_v_b0;

    /* "orthogonal_eval.pxd":316
 *         b2 = b1
 *         b1 = b0
 *         b0 = x*b1 - b2             # <<<<<<<<<<<<<<
 *     return b0 * sign
 * 
 */
    __pyx_v_b0 = ((__pyx_v_x * __pyx_v_b1) - __pyx_v_b2);
  }

  /* "orthogonal_eval.pxd":317
 *         b1 = b0
 *         b0 = x*b1 - b2
 *     return b0 * sign             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = (__pyx_v_b0 * __pyx_v_sign);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":295
 *     return d*hyp2f1(a, b, c, g)
 * 
 * cdef inline double eval_chebyu_l(long k, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long m
 *     cdef int sign
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":323
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 0.5*x)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":324
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:
 *     return eval_chebyu(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(__pyx_v_n, (0.5 * __pyx_v_x));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":323
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":324
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:
 *     return eval_chebyu(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(__pyx_v_n, __Pyx_c_prod(__pyx_t_double_complex_from_parts(0.5, 0), __pyx_v_x));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":323
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":326
 *     return eval_chebyu(n, 0.5*x)
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu_l(n, 0.5*x)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":327
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:
 *     return eval_chebyu_l(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l(__pyx_v_n, (0.5 * __pyx_v_x));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":326
 *     return eval_chebyu(n, 0.5*x)
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu_l(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":333
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":334
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:
 *     return 2*eval_chebyt(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:
 */
  __pyx_r = (2.0 * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(__pyx_v_n, (0.5 * __pyx_v_x)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":333
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":334
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:
 *     return 2*eval_chebyt(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:
 */
  __pyx_r = __Pyx_c_prod(__pyx_t_double_complex_from_parts(2, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(__pyx_v_n, __Pyx_c_prod(__pyx_t_double_complex_from_parts(0.5, 0), __pyx_v_x)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":333
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":336
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt_l(n, 0.5*x)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":337
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:
 *     return 2*eval_chebyt_l(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = (2.0 * __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l(__pyx_v_n, (0.5 * __pyx_v_x)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":336
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt_l(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":343
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":344
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:
 *     return eval_chebyt(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":343
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":344
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:
 *     return eval_chebyt(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(__pyx_v_n, __Pyx_c_diff(__Pyx_c_prod(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(1, 0)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":343
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":346
 *     return eval_chebyt(n, 2*x-1)
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt_l(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":347
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:
 *     return eval_chebyt_l(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":346
 *     return eval_chebyt(n, 2*x-1)
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt_l(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":353
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":354
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:
 *     return eval_chebyu(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":353
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":354
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:
 *     return eval_chebyu(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(__pyx_v_n, __Pyx_c_diff(__Pyx_c_prod(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(1, 0)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":353
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":356
 *     return eval_chebyu(n, 2*x-1)
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu_l(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":357
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:
 *     return eval_chebyu_l(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":356
 *     return eval_chebyu(n, 2*x-1)
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu_l(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":363
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":367
 *     cdef number_t g
 * 
 *     d = 1             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n+1
 */
  __pyx_v_d = 1.0;

  /* "orthogonal_eval.pxd":368
 * 
 *     d = 1
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n+1
 *     c = 1
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":369
 *     d = 1
 *     a = -n
 *     b = n+1             # <<<<<<<<<<<<<<
 *     c = 1
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (__pyx_v_n + 1.0);

  /* "orthogonal_eval.pxd":370
 *     a = -n
 *     b = n+1
 *     c = 1             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 1.0;

  /* "orthogonal_eval.pxd":371
 *     b = n+1
 *     c = 1
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d*hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = (0.5 * (1.0 - __pyx_v_x));

  /* "orthogonal_eval.pxd":372
 *     c = 1
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":363
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":367
 *     cdef number_t g
 * 
 *     d = 1             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n+1
 */
  __pyx_v_d = 1.0;

  /* "orthogonal_eval.pxd":368
 * 
 *     d = 1
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n+1
 *     c = 1
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":369
 *     d = 1
 *     a = -n
 *     b = n+1             # <<<<<<<<<<<<<<
 *     c = 1
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (__pyx_v_n + 1.0);

  /* "orthogonal_eval.pxd":370
 *     a = -n
 *     b = n+1
 *     c = 1             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 1.0;

  /* "orthogonal_eval.pxd":371
 *     b = n+1
 *     c = 1
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d*hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_prod(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x));

  /* "orthogonal_eval.pxd":372
 *     c = 1
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_prod(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":363
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":375
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_legendre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk, a
 *     cdef double p, d
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l(long __pyx_v_n, double __pyx_v_x) {
  long __pyx_v_kk;
  long __pyx_v_a;
  double __pyx_v_p;
  double __pyx_v_d;
  double __pyx_v_k;
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;

  /* "orthogonal_eval.pxd":380
 *     cdef double k
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         n = -n - 1
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":382
 *     if n < 0:
 *         # symmetry
 *         n = -n - 1             # <<<<<<<<<<<<<<
 * 
 *     if n == 0:
 */
    __pyx_v_n = ((-__pyx_v_n) - 1);

    /* "orthogonal_eval.pxd":380
 *     cdef double k
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         n = -n - 1
 */
  }

  /* "orthogonal_eval.pxd":384
 *         n = -n - 1
 * 
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":385
 * 
 *     if n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return x
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":384
 *         n = -n - 1
 * 
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":386
 *     if n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return x
 *     elif fabs(x) < 1e-5:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":387
 *         return 1.0
 *     elif n == 1:
 *         return x             # <<<<<<<<<<<<<<
 *     elif fabs(x) < 1e-5:
 *         # Power series rather than recurrence due to loss of precision
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":386
 *     if n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return x
 *     elif fabs(x) < 1e-5:
 */
  }

  /* "orthogonal_eval.pxd":388
 *     elif n == 1:
 *         return x
 *     elif fabs(x) < 1e-5:             # <<<<<<<<<<<<<<
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/LegendreP/02/
 */
  __pyx_t_1 = ((fabs(__pyx_v_x) < 1e-5) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":391
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/LegendreP/02/
 *         a = n//2             # <<<<<<<<<<<<<<
 * 
 *         d = 1 if a % 2 == 0 else -1
 */
    __pyx_v_a = __Pyx_div_long(__pyx_v_n, 2);

    /* "orthogonal_eval.pxd":393
 *         a = n//2
 * 
 *         d = 1 if a % 2 == 0 else -1             # <<<<<<<<<<<<<<
 *         if n == 2*a:
 *             d *= -2 / beta(a + 1, -0.5)
 */
    if ((((__pyx_v_a % 2) == 0) != 0)) {
      __pyx_t_2 = 1.0;
    } else {
      __pyx_t_2 = -1.0;
    }
    __pyx_v_d = __pyx_t_2;

    /* "orthogonal_eval.pxd":394
 * 
 *         d = 1 if a % 2 == 0 else -1
 *         if n == 2*a:             # <<<<<<<<<<<<<<
 *             d *= -2 / beta(a + 1, -0.5)
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_n == (2 * __pyx_v_a)) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":395
 *         d = 1 if a % 2 == 0 else -1
 *         if n == 2*a:
 *             d *= -2 / beta(a + 1, -0.5)             # <<<<<<<<<<<<<<
 *         else:
 *             d *= 2 * x / beta(a + 1, 0.5)
 */
      __pyx_v_d = (__pyx_v_d * (-2.0 / beta((__pyx_v_a + 1), -0.5)));

      /* "orthogonal_eval.pxd":394
 * 
 *         d = 1 if a % 2 == 0 else -1
 *         if n == 2*a:             # <<<<<<<<<<<<<<
 *             d *= -2 / beta(a + 1, -0.5)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "orthogonal_eval.pxd":397
 *             d *= -2 / beta(a + 1, -0.5)
 *         else:
 *             d *= 2 * x / beta(a + 1, 0.5)             # <<<<<<<<<<<<<<
 * 
 *         p = 0
 */
    /*else*/ {
      __pyx_v_d = (__pyx_v_d * ((2.0 * __pyx_v_x) / beta((__pyx_v_a + 1), 0.5)));
    }
    __pyx_L5:;

    /* "orthogonal_eval.pxd":399
 *             d *= 2 * x / beta(a + 1, 0.5)
 * 
 *         p = 0             # <<<<<<<<<<<<<<
 *         for kk in range(a+1):
 *             p += d
 */
    __pyx_v_p = 0.0;

    /* "orthogonal_eval.pxd":400
 * 
 *         p = 0
 *         for kk in range(a+1):             # <<<<<<<<<<<<<<
 *             p += d
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (
 */
    __pyx_t_3 = (__pyx_v_a + 1);
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_kk = __pyx_t_4;

      /* "orthogonal_eval.pxd":401
 *         p = 0
 *         for kk in range(a+1):
 *             p += d             # <<<<<<<<<<<<<<
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 */
      __pyx_v_p = (__pyx_v_p + __pyx_v_d);

      /* "orthogonal_eval.pxd":402
 *         for kk in range(a+1):
 *             p += d
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (             # <<<<<<<<<<<<<<
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):
 */
      __pyx_v_d = (__pyx_v_d * ((((-2.0 * pow(__pyx_v_x, 2.0)) * (__pyx_v_a - __pyx_v_kk)) * ((((2 * __pyx_v_n) + 1) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk))) / ((((__pyx_v_n + 1) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk)) * (((__pyx_v_n + 2) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk)))));

      /* "orthogonal_eval.pxd":404
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):             # <<<<<<<<<<<<<<
 *                 # converged
 *                 break
 */
      __pyx_t_1 = ((fabs(__pyx_v_d) == (1e-20 * fabs(__pyx_v_p))) != 0);
      if (__pyx_t_1) {

        /* "orthogonal_eval.pxd":406
 *             if fabs(d) == 1e-20*fabs(p):
 *                 # converged
 *                 break             # <<<<<<<<<<<<<<
 *         return p
 *     else:
 */
        goto __pyx_L7_break;

        /* "orthogonal_eval.pxd":404
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):             # <<<<<<<<<<<<<<
 *                 # converged
 *                 break
 */
      }
    }
    __pyx_L7_break:;

    /* "orthogonal_eval.pxd":407
 *                 # converged
 *                 break
 *         return p             # <<<<<<<<<<<<<<
 *     else:
 *         d = x - 1
 */
    __pyx_r = __pyx_v_p;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":388
 *     elif n == 1:
 *         return x
 *     elif fabs(x) < 1e-5:             # <<<<<<<<<<<<<<
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/LegendreP/02/
 */
  }

  /* "orthogonal_eval.pxd":409
 *         return p
 *     else:
 *         d = x - 1             # <<<<<<<<<<<<<<
 *         p = x
 *         for kk in range(n-1):
 */
  /*else*/ {
    __pyx_v_d = (__pyx_v_x - 1.0);

    /* "orthogonal_eval.pxd":410
 *     else:
 *         d = x - 1
 *         p = x             # <<<<<<<<<<<<<<
 *         for kk in range(n-1):
 *             k = kk+1.0
 */
    __pyx_v_p = __pyx_v_x;

    /* "orthogonal_eval.pxd":411
 *         d = x - 1
 *         p = x
 *         for kk in range(n-1):             # <<<<<<<<<<<<<<
 *             k = kk+1.0
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d
 */
    __pyx_t_3 = (__pyx_v_n - 1);
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_kk = __pyx_t_4;

      /* "orthogonal_eval.pxd":412
 *         p = x
 *         for kk in range(n-1):
 *             k = kk+1.0             # <<<<<<<<<<<<<<
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d
 *             p = d + p
 */
      __pyx_v_k = (__pyx_v_kk + 1.0);

      /* "orthogonal_eval.pxd":413
 *         for kk in range(n-1):
 *             k = kk+1.0
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d             # <<<<<<<<<<<<<<
 *             p = d + p
 *         return p
 */
      __pyx_v_d = ((((((2.0 * __pyx_v_k) + 1.0) / (__pyx_v_k + 1.0)) * (__pyx_v_x - 1.0)) * __pyx_v_p) + ((__pyx_v_k / (__pyx_v_k + 1.0)) * __pyx_v_d));

      /* "orthogonal_eval.pxd":414
 *             k = kk+1.0
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d
 *             p = d + p             # <<<<<<<<<<<<<<
 *         return p
 * 
 */
      __pyx_v_p = (__pyx_v_d + __pyx_v_p);
    }

    /* "orthogonal_eval.pxd":415
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d
 *             p = d + p
 *         return p             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    __pyx_r = __pyx_v_p;
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":375
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_legendre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk, a
 *     cdef double p, d
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":421
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":422
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:
 *     return eval_legendre(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":421
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":422
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:
 *     return eval_legendre(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(__pyx_v_n, __Pyx_c_diff(__Pyx_c_prod(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(1, 0)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":421
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":424
 *     return eval_legendre(n, 2*x-1)
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre_l(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":425
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:
 *     return eval_legendre_l(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":424
 *     return eval_legendre(n, 2*x-1)
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre_l(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":431
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_genlaguerre(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(double __pyx_v_n, double __pyx_v_alpha, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;
  int __pyx_t_1;

  /* "orthogonal_eval.pxd":435
 *     cdef number_t g
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  __pyx_t_1 = ((__pyx_v_alpha <= -1.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":436
 * 
 *     if alpha <= -1:
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,             # <<<<<<<<<<<<<<
 *                        "polynomial defined only for alpha > -1")
 *         return nan
 */
    sf_error(__pyx_k_eval_genlaguerre, SF_ERROR_DOMAIN, __pyx_k_polynomial_defined_only_for_alph);

    /* "orthogonal_eval.pxd":438
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     d = binom(n+alpha, n)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":435
 *     cdef number_t g
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  }

  /* "orthogonal_eval.pxd":440
 *         return nan
 * 
 *     d = binom(n+alpha, n)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = alpha + 1
 */
  __pyx_v_d = __pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n);

  /* "orthogonal_eval.pxd":441
 * 
 *     d = binom(n+alpha, n)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = alpha + 1
 *     g = x
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":442
 *     d = binom(n+alpha, n)
 *     a = -n
 *     b = alpha + 1             # <<<<<<<<<<<<<<
 *     g = x
 *     return d * hyp1f1(a, b, g)
 */
  __pyx_v_b = (__pyx_v_alpha + 1.0);

  /* "orthogonal_eval.pxd":443
 *     a = -n
 *     b = alpha + 1
 *     g = x             # <<<<<<<<<<<<<<
 *     return d * hyp1f1(a, b, g)
 * 
 */
  __pyx_v_g = __pyx_v_x;

  /* "orthogonal_eval.pxd":444
 *     b = alpha + 1
 *     g = x
 *     return d * hyp1f1(a, b, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(__pyx_v_a, __pyx_v_b, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":431
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_genlaguerre(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(double __pyx_v_n, double __pyx_v_alpha, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "orthogonal_eval.pxd":435
 *     cdef number_t g
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  __pyx_t_1 = ((__pyx_v_alpha <= -1.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":436
 * 
 *     if alpha <= -1:
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,             # <<<<<<<<<<<<<<
 *                        "polynomial defined only for alpha > -1")
 *         return nan
 */
    sf_error(__pyx_k_eval_genlaguerre, SF_ERROR_DOMAIN, __pyx_k_polynomial_defined_only_for_alph);

    /* "orthogonal_eval.pxd":438
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     d = binom(n+alpha, n)
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":435
 *     cdef number_t g
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  }

  /* "orthogonal_eval.pxd":440
 *         return nan
 * 
 *     d = binom(n+alpha, n)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = alpha + 1
 */
  __pyx_v_d = __pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n);

  /* "orthogonal_eval.pxd":441
 * 
 *     d = binom(n+alpha, n)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = alpha + 1
 *     g = x
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":442
 *     d = binom(n+alpha, n)
 *     a = -n
 *     b = alpha + 1             # <<<<<<<<<<<<<<
 *     g = x
 *     return d * hyp1f1(a, b, g)
 */
  __pyx_v_b = (__pyx_v_alpha + 1.0);

  /* "orthogonal_eval.pxd":443
 *     a = -n
 *     b = alpha + 1
 *     g = x             # <<<<<<<<<<<<<<
 *     return d * hyp1f1(a, b, g)
 * 
 */
  __pyx_v_g = __pyx_v_x;

  /* "orthogonal_eval.pxd":444
 *     b = alpha + 1
 *     g = x
 *     return d * hyp1f1(a, b, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_prod(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(__pyx_v_a, __pyx_v_b, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":431
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_genlaguerre(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":447
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_genlaguerre_l(long n, double alpha, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef double p, d
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l(long __pyx_v_n, double __pyx_v_alpha, double __pyx_v_x) {
  long __pyx_v_kk;
  double __pyx_v_p;
  double __pyx_v_d;
  double __pyx_v_k;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;

  /* "orthogonal_eval.pxd":452
 *     cdef double k
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  __pyx_t_1 = ((__pyx_v_alpha <= -1.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":453
 * 
 *     if alpha <= -1:
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,             # <<<<<<<<<<<<<<
 *                        "polynomial defined only for alpha > -1")
 *         return nan
 */
    sf_error(__pyx_k_eval_genlaguerre, SF_ERROR_DOMAIN, __pyx_k_polynomial_defined_only_for_alph);

    /* "orthogonal_eval.pxd":455
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     if n < 0:
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":452
 *     cdef double k
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  }

  /* "orthogonal_eval.pxd":457
 *         return nan
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":458
 * 
 *     if n < 0:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     elif n == 0:
 *         return 1.0
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":457
 *         return nan
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  }

  /* "orthogonal_eval.pxd":459
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":460
 *         return 0.0
 *     elif n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return -x+alpha+1
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":459
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":461
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return -x+alpha+1
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":462
 *         return 1.0
 *     elif n == 1:
 *         return -x+alpha+1             # <<<<<<<<<<<<<<
 *     else:
 *         d = -x/(alpha+1)
 */
    __pyx_r = (((-__pyx_v_x) + __pyx_v_alpha) + 1.0);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":461
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return -x+alpha+1
 *     else:
 */
  }

  /* "orthogonal_eval.pxd":464
 *         return -x+alpha+1
 *     else:
 *         d = -x/(alpha+1)             # <<<<<<<<<<<<<<
 *         p = d + 1
 *         for kk in range(n-1):
 */
  /*else*/ {
    __pyx_v_d = ((-__pyx_v_x) / (__pyx_v_alpha + 1.0));

    /* "orthogonal_eval.pxd":465
 *     else:
 *         d = -x/(alpha+1)
 *         p = d + 1             # <<<<<<<<<<<<<<
 *         for kk in range(n-1):
 *             k = kk+1.0
 */
    __pyx_v_p = (__pyx_v_d + 1.0);

    /* "orthogonal_eval.pxd":466
 *         d = -x/(alpha+1)
 *         p = d + 1
 *         for kk in range(n-1):             # <<<<<<<<<<<<<<
 *             k = kk+1.0
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d
 */
    __pyx_t_2 = (__pyx_v_n - 1);
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_kk = __pyx_t_3;

      /* "orthogonal_eval.pxd":467
 *         p = d + 1
 *         for kk in range(n-1):
 *             k = kk+1.0             # <<<<<<<<<<<<<<
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d
 *             p = d + p
 */
      __pyx_v_k = (__pyx_v_kk + 1.0);

      /* "orthogonal_eval.pxd":468
 *         for kk in range(n-1):
 *             k = kk+1.0
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d             # <<<<<<<<<<<<<<
 *             p = d + p
 *         return binom(n+alpha, n)*p
 */
      __pyx_v_d = ((((-__pyx_v_x) / ((__pyx_v_k + __pyx_v_alpha) + 1.0)) * __pyx_v_p) + ((__pyx_v_k / ((__pyx_v_k + __pyx_v_alpha) + 1.0)) * __pyx_v_d));

      /* "orthogonal_eval.pxd":469
 *             k = kk+1.0
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d
 *             p = d + p             # <<<<<<<<<<<<<<
 *         return binom(n+alpha, n)*p
 * 
 */
      __pyx_v_p = (__pyx_v_d + __pyx_v_p);
    }

    /* "orthogonal_eval.pxd":470
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d
 *             p = d + p
 *         return binom(n+alpha, n)*p             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n) * __pyx_v_p);
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":447
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_genlaguerre_l(long n, double alpha, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef double p, d
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":476
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre(n, 0., x)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":477
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:
 *     return eval_genlaguerre(n, 0., x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(__pyx_v_n, 0., __pyx_v_x);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":476
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre(n, 0., x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":477
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:
 *     return eval_genlaguerre(n, 0., x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(__pyx_v_n, 0., __pyx_v_x);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":476
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre(n, 0., x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":479
 *     return eval_genlaguerre(n, 0., x)
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre_l(n, 0., x)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":480
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:
 *     return eval_genlaguerre_l(n, 0., x)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l(__pyx_v_n, 0., __pyx_v_x);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":479
 *     return eval_genlaguerre(n, 0., x)
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre_l(n, 0., x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":486
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline double eval_hermitenorm(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long k
 *     cdef double y1, y2, y3
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm(long __pyx_v_n, double __pyx_v_x) {
  long __pyx_v_k;
  double __pyx_v_y1;
  double __pyx_v_y2;
  double __pyx_v_y3;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;

  /* "orthogonal_eval.pxd":490
 *     cdef double y1, y2, y3
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":491
 * 
 *     if n < 0:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     elif n == 0:
 *         return 1.0
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":490
 *     cdef double y1, y2, y3
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  }

  /* "orthogonal_eval.pxd":492
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":493
 *         return 0.0
 *     elif n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return x
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":492
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":494
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":495
 *         return 1.0
 *     elif n == 1:
 *         return x             # <<<<<<<<<<<<<<
 *     else:
 *         y3 = 0.0
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":494
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  }

  /* "orthogonal_eval.pxd":497
 *         return x
 *     else:
 *         y3 = 0.0             # <<<<<<<<<<<<<<
 *         y2 = 1.0
 *         for k in range(n, 1, -1):
 */
  /*else*/ {
    __pyx_v_y3 = 0.0;

    /* "orthogonal_eval.pxd":498
 *     else:
 *         y3 = 0.0
 *         y2 = 1.0             # <<<<<<<<<<<<<<
 *         for k in range(n, 1, -1):
 *             y1 = x*y2 - k*y3
 */
    __pyx_v_y2 = 1.0;

    /* "orthogonal_eval.pxd":499
 *         y3 = 0.0
 *         y2 = 1.0
 *         for k in range(n, 1, -1):             # <<<<<<<<<<<<<<
 *             y1 = x*y2 - k*y3
 *             y3 = y2
 */
    for (__pyx_t_2 = __pyx_v_n; __pyx_t_2 > 1; __pyx_t_2-=1) {
      __pyx_v_k = __pyx_t_2;

      /* "orthogonal_eval.pxd":500
 *         y2 = 1.0
 *         for k in range(n, 1, -1):
 *             y1 = x*y2 - k*y3             # <<<<<<<<<<<<<<
 *             y3 = y2
 *             y2 = y1
 */
      __pyx_v_y1 = ((__pyx_v_x * __pyx_v_y2) - (__pyx_v_k * __pyx_v_y3));

      /* "orthogonal_eval.pxd":501
 *         for k in range(n, 1, -1):
 *             y1 = x*y2 - k*y3
 *             y3 = y2             # <<<<<<<<<<<<<<
 *             y2 = y1
 *         return x*y2 - y3
 */
      __pyx_v_y3 = __pyx_v_y2;

      /* "orthogonal_eval.pxd":502
 *             y1 = x*y2 - k*y3
 *             y3 = y2
 *             y2 = y1             # <<<<<<<<<<<<<<
 *         return x*y2 - y3
 * 
 */
      __pyx_v_y2 = __pyx_v_y1;
    }

    /* "orthogonal_eval.pxd":503
 *             y3 = y2
 *             y2 = y1
 *         return x*y2 - y3             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    __pyx_r = ((__pyx_v_x * __pyx_v_y2) - __pyx_v_y3);
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":486
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline double eval_hermitenorm(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long k
 *     cdef double y1, y2, y3
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":510
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_hermite(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_hermitenorm(n, sqrt(2)*x) * 2**(n/2.0)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_hermite(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":511
 * @cython.cdivision(True)
 * cdef inline double eval_hermite(long n, double x) nogil:
 *     return eval_hermitenorm(n, sqrt(2)*x) * 2**(n/2.0)             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm(__pyx_v_n, (sqrt(2.0) * __pyx_v_x)) * pow(2.0, (__pyx_v_n / 2.0)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":510
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_hermite(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_hermitenorm(n, sqrt(2)*x) * 2**(n/2.0)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":27
 *     double_complex
 * 
 * cdef inline bint zisnan(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isnan(x.real) or npy_isnan(x.imag)
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisnan(double __pyx_v_x) {
  int __pyx_r;

  /* "_complexstuff.pxd":31
 *         return npy_isnan(x.real) or npy_isnan(x.imag)
 *     else:
 *         return npy_isnan(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline bint zisfinite(number_t x) nogil:
 */
  __pyx_r = npy_isnan(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":27
 *     double_complex
 * 
 * cdef inline bint zisnan(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isnan(x.real) or npy_isnan(x.imag)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_t_double_complex __pyx_v_x) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_complexstuff.pxd":29
 * cdef inline bint zisnan(number_t x) nogil:
 *     if number_t is double_complex:
 *         return npy_isnan(x.real) or npy_isnan(x.imag)             # <<<<<<<<<<<<<<
 *     else:
 *         return npy_isnan(x)
 */
  __pyx_t_2 = (npy_isnan(__Pyx_CREAL(__pyx_v_x)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__Pyx_CIMAG(__pyx_v_x)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "_complexstuff.pxd":27
 *     double_complex
 * 
 * cdef inline bint zisnan(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isnan(x.real) or npy_isnan(x.imag)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":33
 *         return npy_isnan(x)
 * 
 * cdef inline bint zisfinite(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisfinite(double __pyx_v_x) {
  int __pyx_r;

  /* "_complexstuff.pxd":37
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)
 *     else:
 *         return npy_isfinite(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline bint zisinf(number_t x) nogil:
 */
  __pyx_r = npy_isfinite(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":33
 *         return npy_isnan(x)
 * 
 * cdef inline bint zisfinite(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisfinite(__pyx_t_double_complex __pyx_v_x) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_complexstuff.pxd":35
 * cdef inline bint zisfinite(number_t x) nogil:
 *     if number_t is double_complex:
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)             # <<<<<<<<<<<<<<
 *     else:
 *         return npy_isfinite(x)
 */
  __pyx_t_2 = (npy_isfinite(__Pyx_CREAL(__pyx_v_x)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (npy_isfinite(__Pyx_CIMAG(__pyx_v_x)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "_complexstuff.pxd":33
 *         return npy_isnan(x)
 * 
 * cdef inline bint zisfinite(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":39
 *         return npy_isfinite(x)
 * 
 * cdef inline bint zisinf(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return not zisnan(x) and not zisfinite(x)
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisinf(double __pyx_v_x) {
  int __pyx_r;

  /* "_complexstuff.pxd":43
 *         return not zisnan(x) and not zisfinite(x)
 *     else:
 *         return npy_isinf(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double zabs(number_t x) nogil:
 */
  __pyx_r = npy_isinf(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":39
 *         return npy_isfinite(x)
 * 
 * cdef inline bint zisinf(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return not zisnan(x) and not zisfinite(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisinf(__pyx_t_double_complex __pyx_v_x) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_complexstuff.pxd":41
 * cdef inline bint zisinf(number_t x) nogil:
 *     if number_t is double_complex:
 *         return not zisnan(x) and not zisfinite(x)             # <<<<<<<<<<<<<<
 *     else:
 *         return npy_isinf(x)
 */
  __pyx_t_2 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_x) != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisfinite(__pyx_v_x) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "_complexstuff.pxd":39
 *         return npy_isfinite(x)
 * 
 * cdef inline bint zisinf(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return not zisnan(x) and not zisfinite(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":45
 *         return npy_isinf(x)
 * 
 * cdef inline double zabs(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_cabs((<np.npy_cdouble*>&x)[0])
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":49
 *         return npy_cabs((<np.npy_cdouble*>&x)[0])
 *     else:
 *         return libc.math.fabs(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t zlog(number_t x) nogil:
 */
  __pyx_r = fabs(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":45
 *         return npy_isinf(x)
 * 
 * cdef inline double zabs(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_cabs((<np.npy_cdouble*>&x)[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE double __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_t_double_complex __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":47
 * cdef inline double zabs(number_t x) nogil:
 *     if number_t is double_complex:
 *         return npy_cabs((<np.npy_cdouble*>&x)[0])             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.fabs(x)
 */
  __pyx_r = npy_cabs((((npy_cdouble *)(&__pyx_v_x))[0]));
  goto __pyx_L0;

  /* "_complexstuff.pxd":45
 *         return npy_isinf(x)
 * 
 * cdef inline double zabs(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_cabs((<np.npy_cdouble*>&x)[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":51
 *         return libc.math.fabs(x)
 * 
 * cdef inline number_t zlog(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zlog(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":57
 *         return (<double_complex*>&r)[0]
 *     else:
 *         return libc.math.log(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t zexp(number_t x) nogil:
 */
  __pyx_r = log(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":51
 *         return libc.math.fabs(x)
 * 
 * cdef inline number_t zlog(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_t_double_complex __pyx_v_x) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":54
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 *         r = npy_clog((<np.npy_cdouble*>&x)[0])             # <<<<<<<<<<<<<<
 *         return (<double_complex*>&r)[0]
 *     else:
 */
  __pyx_v_r = npy_clog((((npy_cdouble *)(&__pyx_v_x))[0]));

  /* "_complexstuff.pxd":55
 *     if number_t is double_complex:
 *         r = npy_clog((<np.npy_cdouble*>&x)[0])
 *         return (<double_complex*>&r)[0]             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.log(x)
 */
  __pyx_r = (((__pyx_t_double_complex *)(&__pyx_v_r))[0]);
  goto __pyx_L0;

  /* "_complexstuff.pxd":51
 *         return libc.math.fabs(x)
 * 
 * cdef inline number_t zlog(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":59
 *         return libc.math.log(x)
 * 
 * cdef inline number_t zexp(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zexp(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":65
 *         return (<double_complex*>&r)[0]
 *     else:
 *         return libc.math.exp(x)             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_r = exp(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":59
 *         return libc.math.log(x)
 * 
 * cdef inline number_t zexp(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__pyx_t_double_complex __pyx_v_x) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":62
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 *         r = npy_cexp((<np.npy_cdouble*>&x)[0])             # <<<<<<<<<<<<<<
 *         return (<double_complex*>&r)[0]
 *     else:
 */
  __pyx_v_r = npy_cexp((((npy_cdouble *)(&__pyx_v_x))[0]));

  /* "_complexstuff.pxd":63
 *     if number_t is double_complex:
 *         r = npy_cexp((<np.npy_cdouble*>&x)[0])
 *         return (<double_complex*>&r)[0]             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.exp(x)
 */
  __pyx_r = (((__pyx_t_double_complex *)(&__pyx_v_r))[0]);
  goto __pyx_L0;

  /* "_complexstuff.pxd":59
 *         return libc.math.log(x)
 * 
 * cdef inline number_t zexp(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_ufuncs",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeWarning, __pyx_k_RuntimeWarning, sizeof(__pyx_k_RuntimeWarning), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_airy, __pyx_k_airy, sizeof(__pyx_k_airy), 0, 0, 1, 1},
  {&__pyx_n_s_airye, __pyx_k_airye, sizeof(__pyx_k_airye), 0, 0, 1, 1},
  {&__pyx_n_s_bdtr, __pyx_k_bdtr, sizeof(__pyx_k_bdtr), 0, 0, 1, 1},
  {&__pyx_n_s_bdtrc, __pyx_k_bdtrc, sizeof(__pyx_k_bdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_bdtri, __pyx_k_bdtri, sizeof(__pyx_k_bdtri), 0, 0, 1, 1},
  {&__pyx_n_s_bdtrik, __pyx_k_bdtrik, sizeof(__pyx_k_bdtrik), 0, 0, 1, 1},
  {&__pyx_n_s_bdtrin, __pyx_k_bdtrin, sizeof(__pyx_k_bdtrin), 0, 0, 1, 1},
  {&__pyx_n_s_bei, __pyx_k_bei, sizeof(__pyx_k_bei), 0, 0, 1, 1},
  {&__pyx_n_s_beip, __pyx_k_beip, sizeof(__pyx_k_beip), 0, 0, 1, 1},
  {&__pyx_n_s_ber, __pyx_k_ber, sizeof(__pyx_k_ber), 0, 0, 1, 1},
  {&__pyx_n_s_berp, __pyx_k_berp, sizeof(__pyx_k_berp), 0, 0, 1, 1},
  {&__pyx_n_s_besselpoly, __pyx_k_besselpoly, sizeof(__pyx_k_besselpoly), 0, 0, 1, 1},
  {&__pyx_n_s_beta, __pyx_k_beta, sizeof(__pyx_k_beta), 0, 0, 1, 1},
  {&__pyx_n_s_betainc, __pyx_k_betainc, sizeof(__pyx_k_betainc), 0, 0, 1, 1},
  {&__pyx_n_s_betaincinv, __pyx_k_betaincinv, sizeof(__pyx_k_betaincinv), 0, 0, 1, 1},
  {&__pyx_n_s_betaln, __pyx_k_betaln, sizeof(__pyx_k_betaln), 0, 0, 1, 1},
  {&__pyx_n_s_binom, __pyx_k_binom, sizeof(__pyx_k_binom), 0, 0, 1, 1},
  {&__pyx_n_s_boxcox, __pyx_k_boxcox, sizeof(__pyx_k_boxcox), 0, 0, 1, 1},
  {&__pyx_n_s_boxcox1p, __pyx_k_boxcox1p, sizeof(__pyx_k_boxcox1p), 0, 0, 1, 1},
  {&__pyx_n_s_btdtr, __pyx_k_btdtr, sizeof(__pyx_k_btdtr), 0, 0, 1, 1},
  {&__pyx_n_s_btdtri, __pyx_k_btdtri, sizeof(__pyx_k_btdtri), 0, 0, 1, 1},
  {&__pyx_n_s_btdtria, __pyx_k_btdtria, sizeof(__pyx_k_btdtria), 0, 0, 1, 1},
  {&__pyx_n_s_btdtrib, __pyx_k_btdtrib, sizeof(__pyx_k_btdtrib), 0, 0, 1, 1},
  {&__pyx_n_s_cbrt, __pyx_k_cbrt, sizeof(__pyx_k_cbrt), 0, 0, 1, 1},
  {&__pyx_n_s_chdtr, __pyx_k_chdtr, sizeof(__pyx_k_chdtr), 0, 0, 1, 1},
  {&__pyx_n_s_chdtrc, __pyx_k_chdtrc, sizeof(__pyx_k_chdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_chdtri, __pyx_k_chdtri, sizeof(__pyx_k_chdtri), 0, 0, 1, 1},
  {&__pyx_n_s_chdtriv, __pyx_k_chdtriv, sizeof(__pyx_k_chdtriv), 0, 0, 1, 1},
  {&__pyx_n_s_chndtr, __pyx_k_chndtr, sizeof(__pyx_k_chndtr), 0, 0, 1, 1},
  {&__pyx_n_s_chndtridf, __pyx_k_chndtridf, sizeof(__pyx_k_chndtridf), 0, 0, 1, 1},
  {&__pyx_n_s_chndtrinc, __pyx_k_chndtrinc, sizeof(__pyx_k_chndtrinc), 0, 0, 1, 1},
  {&__pyx_n_s_chndtrix, __pyx_k_chndtrix, sizeof(__pyx_k_chndtrix), 0, 0, 1, 1},
  {&__pyx_n_s_cosdg, __pyx_k_cosdg, sizeof(__pyx_k_cosdg), 0, 0, 1, 1},
  {&__pyx_n_s_cosm1, __pyx_k_cosm1, sizeof(__pyx_k_cosm1), 0, 0, 1, 1},
  {&__pyx_n_s_cotdg, __pyx_k_cotdg, sizeof(__pyx_k_cotdg), 0, 0, 1, 1},
  {&__pyx_n_s_dawsn, __pyx_k_dawsn, sizeof(__pyx_k_dawsn), 0, 0, 1, 1},
  {&__pyx_n_s_ellip_harm_2, __pyx_k_ellip_harm_2, sizeof(__pyx_k_ellip_harm_2), 0, 0, 1, 1},
  {&__pyx_n_s_ellipe, __pyx_k_ellipe, sizeof(__pyx_k_ellipe), 0, 0, 1, 1},
  {&__pyx_n_s_ellipeinc, __pyx_k_ellipeinc, sizeof(__pyx_k_ellipeinc), 0, 0, 1, 1},
  {&__pyx_n_s_ellipj, __pyx_k_ellipj, sizeof(__pyx_k_ellipj), 0, 0, 1, 1},
  {&__pyx_n_s_ellipkinc, __pyx_k_ellipkinc, sizeof(__pyx_k_ellipkinc), 0, 0, 1, 1},
  {&__pyx_n_s_ellipkm1, __pyx_k_ellipkm1, sizeof(__pyx_k_ellipkm1), 0, 0, 1, 1},
  {&__pyx_n_s_entr, __pyx_k_entr, sizeof(__pyx_k_entr), 0, 0, 1, 1},
  {&__pyx_n_s_erf, __pyx_k_erf, sizeof(__pyx_k_erf), 0, 0, 1, 1},
  {&__pyx_n_s_erfc, __pyx_k_erfc, sizeof(__pyx_k_erfc), 0, 0, 1, 1},
  {&__pyx_n_s_erfcx, __pyx_k_erfcx, sizeof(__pyx_k_erfcx), 0, 0, 1, 1},
  {&__pyx_n_s_erfi, __pyx_k_erfi, sizeof(__pyx_k_erfi), 0, 0, 1, 1},
  {&__pyx_n_s_errprint, __pyx_k_errprint, sizeof(__pyx_k_errprint), 0, 0, 1, 1},
  {&__pyx_n_s_eval_chebyc, __pyx_k_eval_chebyc, sizeof(__pyx_k_eval_chebyc), 0, 0, 1, 1},
  {&__pyx_n_s_eval_chebys, __pyx_k_eval_chebys, sizeof(__pyx_k_eval_chebys), 0, 0, 1, 1},
  {&__pyx_n_s_eval_chebyt, __pyx_k_eval_chebyt, sizeof(__pyx_k_eval_chebyt), 0, 0, 1, 1},
  {&__pyx_n_s_eval_chebyu, __pyx_k_eval_chebyu, sizeof(__pyx_k_eval_chebyu), 0, 0, 1, 1},
  {&__pyx_n_s_eval_gegenbauer, __pyx_k_eval_gegenbauer, sizeof(__pyx_k_eval_gegenbauer), 0, 0, 1, 1},
  {&__pyx_n_s_eval_genlaguerre, __pyx_k_eval_genlaguerre, sizeof(__pyx_k_eval_genlaguerre), 0, 0, 1, 1},
  {&__pyx_n_s_eval_hermite, __pyx_k_eval_hermite, sizeof(__pyx_k_eval_hermite), 0, 0, 1, 1},
  {&__pyx_n_s_eval_hermitenorm, __pyx_k_eval_hermitenorm, sizeof(__pyx_k_eval_hermitenorm), 0, 0, 1, 1},
  {&__pyx_n_s_eval_jacobi, __pyx_k_eval_jacobi, sizeof(__pyx_k_eval_jacobi), 0, 0, 1, 1},
  {&__pyx_n_s_eval_laguerre, __pyx_k_eval_laguerre, sizeof(__pyx_k_eval_laguerre), 0, 0, 1, 1},
  {&__pyx_n_s_eval_legendre, __pyx_k_eval_legendre, sizeof(__pyx_k_eval_legendre), 0, 0, 1, 1},
  {&__pyx_n_s_eval_sh_chebyt, __pyx_k_eval_sh_chebyt, sizeof(__pyx_k_eval_sh_chebyt), 0, 0, 1, 1},
  {&__pyx_n_s_eval_sh_chebyu, __pyx_k_eval_sh_chebyu, sizeof(__pyx_k_eval_sh_chebyu), 0, 0, 1, 1},
  {&__pyx_n_s_eval_sh_jacobi, __pyx_k_eval_sh_jacobi, sizeof(__pyx_k_eval_sh_jacobi), 0, 0, 1, 1},
  {&__pyx_n_s_eval_sh_legendre, __pyx_k_eval_sh_legendre, sizeof(__pyx_k_eval_sh_legendre), 0, 0, 1, 1},
  {&__pyx_n_s_exp1, __pyx_k_exp1, sizeof(__pyx_k_exp1), 0, 0, 1, 1},
  {&__pyx_n_s_exp10, __pyx_k_exp10, sizeof(__pyx_k_exp10), 0, 0, 1, 1},
  {&__pyx_n_s_exp2, __pyx_k_exp2, sizeof(__pyx_k_exp2), 0, 0, 1, 1},
  {&__pyx_n_s_expi, __pyx_k_expi, sizeof(__pyx_k_expi), 0, 0, 1, 1},
  {&__pyx_n_s_expit, __pyx_k_expit, sizeof(__pyx_k_expit), 0, 0, 1, 1},
  {&__pyx_n_s_expm1, __pyx_k_expm1, sizeof(__pyx_k_expm1), 0, 0, 1, 1},
  {&__pyx_n_s_expn, __pyx_k_expn, sizeof(__pyx_k_expn), 0, 0, 1, 1},
  {&__pyx_n_s_exprel, __pyx_k_exprel, sizeof(__pyx_k_exprel), 0, 0, 1, 1},
  {&__pyx_n_s_fdtr, __pyx_k_fdtr, sizeof(__pyx_k_fdtr), 0, 0, 1, 1},
  {&__pyx_n_s_fdtrc, __pyx_k_fdtrc, sizeof(__pyx_k_fdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_fdtri, __pyx_k_fdtri, sizeof(__pyx_k_fdtri), 0, 0, 1, 1},
  {&__pyx_n_s_fdtridfd, __pyx_k_fdtridfd, sizeof(__pyx_k_fdtridfd), 0, 0, 1, 1},
  {&__pyx_n_s_fresnel, __pyx_k_fresnel, sizeof(__pyx_k_fresnel), 0, 0, 1, 1},
  {&__pyx_n_s_gamma, __pyx_k_gamma, sizeof(__pyx_k_gamma), 0, 0, 1, 1},
  {&__pyx_n_s_gammainc, __pyx_k_gammainc, sizeof(__pyx_k_gammainc), 0, 0, 1, 1},
  {&__pyx_n_s_gammaincc, __pyx_k_gammaincc, sizeof(__pyx_k_gammaincc), 0, 0, 1, 1},
  {&__pyx_n_s_gammainccinv, __pyx_k_gammainccinv, sizeof(__pyx_k_gammainccinv), 0, 0, 1, 1},
  {&__pyx_n_s_gammaincinv, __pyx_k_gammaincinv, sizeof(__pyx_k_gammaincinv), 0, 0, 1, 1},
  {&__pyx_n_s_gammaln, __pyx_k_gammaln, sizeof(__pyx_k_gammaln), 0, 0, 1, 1},
  {&__pyx_n_s_gammasgn, __pyx_k_gammasgn, sizeof(__pyx_k_gammasgn), 0, 0, 1, 1},
  {&__pyx_n_s_gdtr, __pyx_k_gdtr, sizeof(__pyx_k_gdtr), 0, 0, 1, 1},
  {&__pyx_n_s_gdtrc, __pyx_k_gdtrc, sizeof(__pyx_k_gdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_gdtria, __pyx_k_gdtria, sizeof(__pyx_k_gdtria), 0, 0, 1, 1},
  {&__pyx_n_s_gdtrib, __pyx_k_gdtrib, sizeof(__pyx_k_gdtrib), 0, 0, 1, 1},
  {&__pyx_n_s_gdtrix, __pyx_k_gdtrix, sizeof(__pyx_k_gdtrix), 0, 0, 1, 1},
  {&__pyx_n_s_hankel1, __pyx_k_hankel1, sizeof(__pyx_k_hankel1), 0, 0, 1, 1},
  {&__pyx_n_s_hankel1e, __pyx_k_hankel1e, sizeof(__pyx_k_hankel1e), 0, 0, 1, 1},
  {&__pyx_n_s_hankel2, __pyx_k_hankel2, sizeof(__pyx_k_hankel2), 0, 0, 1, 1},
  {&__pyx_n_s_hankel2e, __pyx_k_hankel2e, sizeof(__pyx_k_hankel2e), 0, 0, 1, 1},
  {&__pyx_kp_s_home_br_repos_scipy_scipy_speci, __pyx_k_home_br_repos_scipy_scipy_speci, sizeof(__pyx_k_home_br_repos_scipy_scipy_speci), 0, 0, 1, 0},
  {&__pyx_n_s_huber, __pyx_k_huber, sizeof(__pyx_k_huber), 0, 0, 1, 1},
  {&__pyx_n_s_hyp1f1, __pyx_k_hyp1f1, sizeof(__pyx_k_hyp1f1), 0, 0, 1, 1},
  {&__pyx_n_s_hyp1f2, __pyx_k_hyp1f2, sizeof(__pyx_k_hyp1f2), 0, 0, 1, 1},
  {&__pyx_n_s_hyp2f0, __pyx_k_hyp2f0, sizeof(__pyx_k_hyp2f0), 0, 0, 1, 1},
  {&__pyx_n_s_hyp2f1, __pyx_k_hyp2f1, sizeof(__pyx_k_hyp2f1), 0, 0, 1, 1},
  {&__pyx_n_s_hyp3f0, __pyx_k_hyp3f0, sizeof(__pyx_k_hyp3f0), 0, 0, 1, 1},
  {&__pyx_n_s_hyperu, __pyx_k_hyperu, sizeof(__pyx_k_hyperu), 0, 0, 1, 1},
  {&__pyx_n_s_i0, __pyx_k_i0, sizeof(__pyx_k_i0), 0, 0, 1, 1},
  {&__pyx_n_s_i0e, __pyx_k_i0e, sizeof(__pyx_k_i0e), 0, 0, 1, 1},
  {&__pyx_n_s_i1, __pyx_k_i1, sizeof(__pyx_k_i1), 0, 0, 1, 1},
  {&__pyx_n_s_i1e, __pyx_k_i1e, sizeof(__pyx_k_i1e), 0, 0, 1, 1},
  {&__pyx_n_s_inflag, __pyx_k_inflag, sizeof(__pyx_k_inflag), 0, 0, 1, 1},
  {&__pyx_n_s_inv_boxcox, __pyx_k_inv_boxcox, sizeof(__pyx_k_inv_boxcox), 0, 0, 1, 1},
  {&__pyx_n_s_inv_boxcox1p, __pyx_k_inv_boxcox1p, sizeof(__pyx_k_inv_boxcox1p), 0, 0, 1, 1},
  {&__pyx_n_s_it2i0k0, __pyx_k_it2i0k0, sizeof(__pyx_k_it2i0k0), 0, 0, 1, 1},
  {&__pyx_n_s_it2j0y0, __pyx_k_it2j0y0, sizeof(__pyx_k_it2j0y0), 0, 0, 1, 1},
  {&__pyx_n_s_it2struve0, __pyx_k_it2struve0, sizeof(__pyx_k_it2struve0), 0, 0, 1, 1},
  {&__pyx_n_s_itairy, __pyx_k_itairy, sizeof(__pyx_k_itairy), 0, 0, 1, 1},
  {&__pyx_n_s_iti0k0, __pyx_k_iti0k0, sizeof(__pyx_k_iti0k0), 0, 0, 1, 1},
  {&__pyx_n_s_itj0y0, __pyx_k_itj0y0, sizeof(__pyx_k_itj0y0), 0, 0, 1, 1},
  {&__pyx_n_s_itmodstruve0, __pyx_k_itmodstruve0, sizeof(__pyx_k_itmodstruve0), 0, 0, 1, 1},
  {&__pyx_n_s_itstruve0, __pyx_k_itstruve0, sizeof(__pyx_k_itstruve0), 0, 0, 1, 1},
  {&__pyx_n_s_iv, __pyx_k_iv, sizeof(__pyx_k_iv), 0, 0, 1, 1},
  {&__pyx_n_s_ive, __pyx_k_ive, sizeof(__pyx_k_ive), 0, 0, 1, 1},
  {&__pyx_n_s_j0, __pyx_k_j0, sizeof(__pyx_k_j0), 0, 0, 1, 1},
  {&__pyx_n_s_j1, __pyx_k_j1, sizeof(__pyx_k_j1), 0, 0, 1, 1},
  {&__pyx_n_s_jn, __pyx_k_jn, sizeof(__pyx_k_jn), 0, 0, 1, 1},
  {&__pyx_n_s_jv, __pyx_k_jv, sizeof(__pyx_k_jv), 0, 0, 1, 1},
  {&__pyx_n_s_jve, __pyx_k_jve, sizeof(__pyx_k_jve), 0, 0, 1, 1},
  {&__pyx_n_s_k0, __pyx_k_k0, sizeof(__pyx_k_k0), 0, 0, 1, 1},
  {&__pyx_n_s_k0e, __pyx_k_k0e, sizeof(__pyx_k_k0e), 0, 0, 1, 1},
  {&__pyx_n_s_k1, __pyx_k_k1, sizeof(__pyx_k_k1), 0, 0, 1, 1},
  {&__pyx_n_s_k1e, __pyx_k_k1e, sizeof(__pyx_k_k1e), 0, 0, 1, 1},
  {&__pyx_n_s_kei, __pyx_k_kei, sizeof(__pyx_k_kei), 0, 0, 1, 1},
  {&__pyx_n_s_keip, __pyx_k_keip, sizeof(__pyx_k_keip), 0, 0, 1, 1},
  {&__pyx_n_s_kelvin, __pyx_k_kelvin, sizeof(__pyx_k_kelvin), 0, 0, 1, 1},
  {&__pyx_n_s_ker, __pyx_k_ker, sizeof(__pyx_k_ker), 0, 0, 1, 1},
  {&__pyx_n_s_kerp, __pyx_k_kerp, sizeof(__pyx_k_kerp), 0, 0, 1, 1},
  {&__pyx_n_s_kl_div, __pyx_k_kl_div, sizeof(__pyx_k_kl_div), 0, 0, 1, 1},
  {&__pyx_n_s_kn, __pyx_k_kn, sizeof(__pyx_k_kn), 0, 0, 1, 1},
  {&__pyx_n_s_kolmogi, __pyx_k_kolmogi, sizeof(__pyx_k_kolmogi), 0, 0, 1, 1},
  {&__pyx_n_s_kolmogorov, __pyx_k_kolmogorov, sizeof(__pyx_k_kolmogorov), 0, 0, 1, 1},
  {&__pyx_n_s_kv, __pyx_k_kv, sizeof(__pyx_k_kv), 0, 0, 1, 1},
  {&__pyx_n_s_kve, __pyx_k_kve, sizeof(__pyx_k_kve), 0, 0, 1, 1},
  {&__pyx_n_s_lambertw_2, __pyx_k_lambertw_2, sizeof(__pyx_k_lambertw_2), 0, 0, 1, 1},
  {&__pyx_n_s_log1p, __pyx_k_log1p, sizeof(__pyx_k_log1p), 0, 0, 1, 1},
  {&__pyx_n_s_log_ndtr, __pyx_k_log_ndtr, sizeof(__pyx_k_log_ndtr), 0, 0, 1, 1},
  {&__pyx_n_s_logit, __pyx_k_logit, sizeof(__pyx_k_logit), 0, 0, 1, 1},
  {&__pyx_n_s_lpmv, __pyx_k_lpmv, sizeof(__pyx_k_lpmv), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_a, __pyx_k_mathieu_a, sizeof(__pyx_k_mathieu_a), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_b, __pyx_k_mathieu_b, sizeof(__pyx_k_mathieu_b), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_cem, __pyx_k_mathieu_cem, sizeof(__pyx_k_mathieu_cem), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_modcem1, __pyx_k_mathieu_modcem1, sizeof(__pyx_k_mathieu_modcem1), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_modcem2, __pyx_k_mathieu_modcem2, sizeof(__pyx_k_mathieu_modcem2), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_modsem1, __pyx_k_mathieu_modsem1, sizeof(__pyx_k_mathieu_modsem1), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_modsem2, __pyx_k_mathieu_modsem2, sizeof(__pyx_k_mathieu_modsem2), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_sem, __pyx_k_mathieu_sem, sizeof(__pyx_k_mathieu_sem), 0, 0, 1, 1},
  {&__pyx_n_s_modfresnelm, __pyx_k_modfresnelm, sizeof(__pyx_k_modfresnelm), 0, 0, 1, 1},
  {&__pyx_n_s_modfresnelp, __pyx_k_modfresnelp, sizeof(__pyx_k_modfresnelp), 0, 0, 1, 1},
  {&__pyx_n_s_modstruve, __pyx_k_modstruve, sizeof(__pyx_k_modstruve), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtr, __pyx_k_nbdtr, sizeof(__pyx_k_nbdtr), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtrc, __pyx_k_nbdtrc, sizeof(__pyx_k_nbdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtri, __pyx_k_nbdtri, sizeof(__pyx_k_nbdtri), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtrik, __pyx_k_nbdtrik, sizeof(__pyx_k_nbdtrik), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtrin, __pyx_k_nbdtrin, sizeof(__pyx_k_nbdtrin), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtr, __pyx_k_ncfdtr, sizeof(__pyx_k_ncfdtr), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtri, __pyx_k_ncfdtri, sizeof(__pyx_k_ncfdtri), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtridfd, __pyx_k_ncfdtridfd, sizeof(__pyx_k_ncfdtridfd), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtridfn, __pyx_k_ncfdtridfn, sizeof(__pyx_k_ncfdtridfn), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtrinc, __pyx_k_ncfdtrinc, sizeof(__pyx_k_ncfdtrinc), 0, 0, 1, 1},
  {&__pyx_n_s_nctdtr, __pyx_k_nctdtr, sizeof(__pyx_k_nctdtr), 0, 0, 1, 1},
  {&__pyx_n_s_nctdtridf, __pyx_k_nctdtridf, sizeof(__pyx_k_nctdtridf), 0, 0, 1, 1},
  {&__pyx_n_s_nctdtrinc, __pyx_k_nctdtrinc, sizeof(__pyx_k_nctdtrinc), 0, 0, 1, 1},
  {&__pyx_n_s_nctdtrit, __pyx_k_nctdtrit, sizeof(__pyx_k_nctdtrit), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_ndtr, __pyx_k_ndtr, sizeof(__pyx_k_ndtr), 0, 0, 1, 1},
  {&__pyx_n_s_ndtri, __pyx_k_ndtri, sizeof(__pyx_k_ndtri), 0, 0, 1, 1},
  {&__pyx_n_s_nrdtrimn, __pyx_k_nrdtrimn, sizeof(__pyx_k_nrdtrimn), 0, 0, 1, 1},
  {&__pyx_n_s_nrdtrisd, __pyx_k_nrdtrisd, sizeof(__pyx_k_nrdtrisd), 0, 0, 1, 1},
  {&__pyx_n_s_obl_ang1, __pyx_k_obl_ang1, sizeof(__pyx_k_obl_ang1), 0, 0, 1, 1},
  {&__pyx_n_s_obl_ang1_cv, __pyx_k_obl_ang1_cv, sizeof(__pyx_k_obl_ang1_cv), 0, 0, 1, 1},
  {&__pyx_n_s_obl_cv, __pyx_k_obl_cv, sizeof(__pyx_k_obl_cv), 0, 0, 1, 1},
  {&__pyx_n_s_obl_rad1, __pyx_k_obl_rad1, sizeof(__pyx_k_obl_rad1), 0, 0, 1, 1},
  {&__pyx_n_s_obl_rad1_cv, __pyx_k_obl_rad1_cv, sizeof(__pyx_k_obl_rad1_cv), 0, 0, 1, 1},
  {&__pyx_n_s_obl_rad2, __pyx_k_obl_rad2, sizeof(__pyx_k_obl_rad2), 0, 0, 1, 1},
  {&__pyx_n_s_obl_rad2_cv, __pyx_k_obl_rad2_cv, sizeof(__pyx_k_obl_rad2_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pbdv, __pyx_k_pbdv, sizeof(__pyx_k_pbdv), 0, 0, 1, 1},
  {&__pyx_n_s_pbvv, __pyx_k_pbvv, sizeof(__pyx_k_pbvv), 0, 0, 1, 1},
  {&__pyx_n_s_pbwa, __pyx_k_pbwa, sizeof(__pyx_k_pbwa), 0, 0, 1, 1},
  {&__pyx_n_s_pdtr, __pyx_k_pdtr, sizeof(__pyx_k_pdtr), 0, 0, 1, 1},
  {&__pyx_n_s_pdtrc, __pyx_k_pdtrc, sizeof(__pyx_k_pdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_pdtri, __pyx_k_pdtri, sizeof(__pyx_k_pdtri), 0, 0, 1, 1},
  {&__pyx_n_s_pdtrik, __pyx_k_pdtrik, sizeof(__pyx_k_pdtrik), 0, 0, 1, 1},
  {&__pyx_n_s_poch, __pyx_k_poch, sizeof(__pyx_k_poch), 0, 0, 1, 1},
  {&__pyx_n_s_pro_ang1, __pyx_k_pro_ang1, sizeof(__pyx_k_pro_ang1), 0, 0, 1, 1},
  {&__pyx_n_s_pro_ang1_cv, __pyx_k_pro_ang1_cv, sizeof(__pyx_k_pro_ang1_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pro_cv, __pyx_k_pro_cv, sizeof(__pyx_k_pro_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pro_rad1, __pyx_k_pro_rad1, sizeof(__pyx_k_pro_rad1), 0, 0, 1, 1},
  {&__pyx_n_s_pro_rad1_cv, __pyx_k_pro_rad1_cv, sizeof(__pyx_k_pro_rad1_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pro_rad2, __pyx_k_pro_rad2, sizeof(__pyx_k_pro_rad2), 0, 0, 1, 1},
  {&__pyx_n_s_pro_rad2_cv, __pyx_k_pro_rad2_cv, sizeof(__pyx_k_pro_rad2_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pseudo_huber, __pyx_k_pseudo_huber, sizeof(__pyx_k_pseudo_huber), 0, 0, 1, 1},
  {&__pyx_n_s_psi, __pyx_k_psi, sizeof(__pyx_k_psi), 0, 0, 1, 1},
  {&__pyx_n_s_radian, __pyx_k_radian, sizeof(__pyx_k_radian), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_rel_entr, __pyx_k_rel_entr, sizeof(__pyx_k_rel_entr), 0, 0, 1, 1},
  {&__pyx_n_s_rgamma, __pyx_k_rgamma, sizeof(__pyx_k_rgamma), 0, 0, 1, 1},
  {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_special__ufuncs, __pyx_k_scipy_special__ufuncs, sizeof(__pyx_k_scipy_special__ufuncs), 0, 0, 1, 1},
  {&__pyx_n_s_shichi, __pyx_k_shichi, sizeof(__pyx_k_shichi), 0, 0, 1, 1},
  {&__pyx_n_s_sici, __pyx_k_sici, sizeof(__pyx_k_sici), 0, 0, 1, 1},
  {&__pyx_n_s_sindg, __pyx_k_sindg, sizeof(__pyx_k_sindg), 0, 0, 1, 1},
  {&__pyx_n_s_smirnov, __pyx_k_smirnov, sizeof(__pyx_k_smirnov), 0, 0, 1, 1},
  {&__pyx_n_s_smirnovi, __pyx_k_smirnovi, sizeof(__pyx_k_smirnovi), 0, 0, 1, 1},
  {&__pyx_n_s_spence, __pyx_k_spence, sizeof(__pyx_k_spence), 0, 0, 1, 1},
  {&__pyx_n_s_sph_harm, __pyx_k_sph_harm, sizeof(__pyx_k_sph_harm), 0, 0, 1, 1},
  {&__pyx_n_s_stdtr, __pyx_k_stdtr, sizeof(__pyx_k_stdtr), 0, 0, 1, 1},
  {&__pyx_n_s_stdtridf, __pyx_k_stdtridf, sizeof(__pyx_k_stdtridf), 0, 0, 1, 1},
  {&__pyx_n_s_stdtrit, __pyx_k_stdtrit, sizeof(__pyx_k_stdtrit), 0, 0, 1, 1},
  {&__pyx_n_s_struve, __pyx_k_struve, sizeof(__pyx_k_struve), 0, 0, 1, 1},
  {&__pyx_n_s_struve_asymp_large_z, __pyx_k_struve_asymp_large_z, sizeof(__pyx_k_struve_asymp_large_z), 0, 0, 1, 1},
  {&__pyx_n_s_struve_bessel_series, __pyx_k_struve_bessel_series, sizeof(__pyx_k_struve_bessel_series), 0, 0, 1, 1},
  {&__pyx_n_s_struve_power_series, __pyx_k_struve_power_series, sizeof(__pyx_k_struve_power_series), 0, 0, 1, 1},
  {&__pyx_n_s_tandg, __pyx_k_tandg, sizeof(__pyx_k_tandg), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_tklmbda, __pyx_k_tklmbda, sizeof(__pyx_k_tklmbda), 0, 0, 1, 1},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_n_s_wofz, __pyx_k_wofz, sizeof(__pyx_k_wofz), 0, 0, 1, 1},
  {&__pyx_n_s_xlog1py, __pyx_k_xlog1py, sizeof(__pyx_k_xlog1py), 0, 0, 1, 1},
  {&__pyx_n_s_xlogy, __pyx_k_xlogy, sizeof(__pyx_k_xlogy), 0, 0, 1, 1},
  {&__pyx_n_s_y0, __pyx_k_y0, sizeof(__pyx_k_y0), 0, 0, 1, 1},
  {&__pyx_n_s_y1, __pyx_k_y1, sizeof(__pyx_k_y1), 0, 0, 1, 1},
  {&__pyx_n_s_yn, __pyx_k_yn, sizeof(__pyx_k_yn), 0, 0, 1, 1},
  {&__pyx_n_s_yv, __pyx_k_yv, sizeof(__pyx_k_yv), 0, 0, 1, 1},
  {&__pyx_n_s_yve, __pyx_k_yve, sizeof(__pyx_k_yve), 0, 0, 1, 1},
  {&__pyx_n_s_zeta, __pyx_k_zeta, sizeof(__pyx_k_zeta), 0, 0, 1, 1},
  {&__pyx_n_s_zetac, __pyx_k_zetac, sizeof(__pyx_k_zetac), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 79; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeWarning = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeWarning); if (!__pyx_builtin_RuntimeWarning) {__pyx_filename = __pyx_f[5]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__5)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__6)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "scipy/special/_ufuncs.pyx":45
 * cimport scipy.special._ufuncs_cxx
 * 
 * def errprint(inflag=None):             # <<<<<<<<<<<<<<
 *     """
 *     errprint(inflag=None)
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_n_s_inflag); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __pyx_codeobj__8 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__7, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_speci, __pyx_n_s_errprint, 45, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_ufuncs(void); /*proto*/
PyMODINIT_FUNC init_ufuncs(void)
#else
PyMODINIT_FUNC PyInit__ufuncs(void); /*proto*/
PyMODINIT_FUNC PyInit__ufuncs(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__ufuncs(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_ufuncs", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_scipy__special___ufuncs) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "scipy.special._ufuncs")) {
      if (unlikely(PyDict_SetItemString(modules, "scipy.special._ufuncs", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[7]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_5numpy_dtype)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_flatiter)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_broadcast)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_5numpy_ndarray)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType("numpy", "ufunc", sizeof(PyUFuncObject), 0); if (unlikely(!__pyx_ptype_5numpy_ufunc)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("scipy.special._ufuncs_cxx"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_dawsn", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_dawsn_complex", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erf", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfc", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfcx", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfcx_complex", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfi", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfi_complex", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_w", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w, "void *") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Function import code ---*/
  __pyx_t_2 = __Pyx_ImportModule("scipy.special._ufuncs_cxx"); if (!__pyx_t_2) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "_set_errprint", (void (**)(void))&__pyx_f_5scipy_7special_11_ufuncs_cxx__set_errprint, "int (int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "scipy/special/_ufuncs.pyx":37
 * cimport sf_error
 * 
 * np.import_array()             # <<<<<<<<<<<<<<
 * np.import_ufunc()
 * 
 */
  import_array();

  /* "scipy/special/_ufuncs.pyx":38
 * 
 * np.import_array()
 * np.import_ufunc()             # <<<<<<<<<<<<<<
 * 
 * cdef int _set_errprint(int flag) nogil:
 */
  import_ufunc();

  /* "scipy/special/_ufuncs.pyx":45
 * cimport scipy.special._ufuncs_cxx
 * 
 * def errprint(inflag=None):             # <<<<<<<<<<<<<<
 *     """
 *     errprint(inflag=None)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5scipy_7special_7_ufuncs_1errprint, NULL, __pyx_n_s_scipy_special__ufuncs); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_errprint, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":1223
 * from _ellip_harm cimport ellip_harmonic as _func_ellip_harmonic
 * ctypedef double _proto_ellip_harmonic_t(double, double, int, int, double, double, double) nogil
 * cdef _proto_ellip_harmonic_t *_proto_ellip_harmonic_t_var = &_func_ellip_harmonic             # <<<<<<<<<<<<<<
 * from _legacy cimport ellip_harmonic_unsafe as _func_ellip_harmonic_unsafe
 * ctypedef double _proto_ellip_harmonic_unsafe_t(double, double, double, double, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_ellip_harmonic_t_var = (&__pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic);

  /* "scipy/special/_ufuncs.pyx":1226
 * from _legacy cimport ellip_harmonic_unsafe as _func_ellip_harmonic_unsafe
 * ctypedef double _proto_ellip_harmonic_unsafe_t(double, double, double, double, double, double, double) nogil
 * cdef _proto_ellip_harmonic_unsafe_t *_proto_ellip_harmonic_unsafe_t_var = &_func_ellip_harmonic_unsafe             # <<<<<<<<<<<<<<
 * from lambertw cimport lambertw_scalar as _func_lambertw_scalar
 * ctypedef double complex _proto_lambertw_scalar_t(double complex, long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_ellip_harmonic_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":1229
 * from lambertw cimport lambertw_scalar as _func_lambertw_scalar
 * ctypedef double complex _proto_lambertw_scalar_t(double complex, long, double) nogil
 * cdef _proto_lambertw_scalar_t *_proto_lambertw_scalar_t_var = &_func_lambertw_scalar             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_struve_asymp_large_z "struve_asymp_large_z"(double, double, int, double *) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_lambertw_scalar_t_var = (&__pyx_f_5scipy_7special_8lambertw_lambertw_scalar);

  /* "scipy/special/_ufuncs.pyx":1248
 * from _legacy cimport bdtr_unsafe as _func_bdtr_unsafe
 * ctypedef double _proto_bdtr_unsafe_t(double, double, double) nogil
 * cdef _proto_bdtr_unsafe_t *_proto_bdtr_unsafe_t_var = &_func_bdtr_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_bdtrc "bdtrc"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_bdtr_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_bdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":1253
 * from _legacy cimport bdtrc_unsafe as _func_bdtrc_unsafe
 * ctypedef double _proto_bdtrc_unsafe_t(double, double, double) nogil
 * cdef _proto_bdtrc_unsafe_t *_proto_bdtrc_unsafe_t_var = &_func_bdtrc_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_bdtri "bdtri"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_bdtrc_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":1258
 * from _legacy cimport bdtri_unsafe as _func_bdtri_unsafe
 * ctypedef double _proto_bdtri_unsafe_t(double, double, double) nogil
 * cdef _proto_bdtri_unsafe_t *_proto_bdtri_unsafe_t_var = &_func_bdtri_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_cdfbin2_wrap "cdfbin2_wrap"(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_bdtri_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_bdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":1283
 * from orthogonal_eval cimport binom as _func_binom
 * ctypedef double _proto_binom_t(double, double) nogil
 * cdef _proto_binom_t *_proto_binom_t_var = &_func_binom             # <<<<<<<<<<<<<<
 * from _boxcox cimport boxcox as _func_boxcox
 * ctypedef double _proto_boxcox_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_binom_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_binom);

  /* "scipy/special/_ufuncs.pyx":1286
 * from _boxcox cimport boxcox as _func_boxcox
 * ctypedef double _proto_boxcox_t(double, double) nogil
 * cdef _proto_boxcox_t *_proto_boxcox_t_var = &_func_boxcox             # <<<<<<<<<<<<<<
 * from _boxcox cimport boxcox1p as _func_boxcox1p
 * ctypedef double _proto_boxcox1p_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_boxcox_t_var = (&__pyx_f_5scipy_7special_7_boxcox_boxcox);

  /* "scipy/special/_ufuncs.pyx":1289
 * from _boxcox cimport boxcox1p as _func_boxcox1p
 * ctypedef double _proto_boxcox1p_t(double, double) nogil
 * cdef _proto_boxcox1p_t *_proto_boxcox1p_t_var = &_func_boxcox1p             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_btdtr "btdtr"(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_boxcox1p_t_var = (&__pyx_f_5scipy_7special_7_boxcox_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":1334
 * from _convex_analysis cimport entr as _func_entr
 * ctypedef double _proto_entr_t(double) nogil
 * cdef _proto_entr_t *_proto_entr_t_var = &_func_entr             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_erf "erf"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_entr_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_entr);

  /* "scipy/special/_ufuncs.pyx":1341
 * from orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double _proto_eval_chebyc_double__t(double, double) nogil
 * cdef _proto_eval_chebyc_double__t *_proto_eval_chebyc_double__t_var = &_func_eval_chebyc[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double complex _proto_eval_chebyc_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":1344
 * from orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double complex _proto_eval_chebyc_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_chebyc_double_complex__t *_proto_eval_chebyc_double_complex__t_var = &_func_eval_chebyc[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebyc_l as _func_eval_chebyc_l
 * ctypedef double _proto_eval_chebyc_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":1347
 * from orthogonal_eval cimport eval_chebyc_l as _func_eval_chebyc_l
 * ctypedef double _proto_eval_chebyc_l_t(long, double) nogil
 * cdef _proto_eval_chebyc_l_t *_proto_eval_chebyc_l_t_var = &_func_eval_chebyc_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double _proto_eval_chebys_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc_l);

  /* "scipy/special/_ufuncs.pyx":1350
 * from orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double _proto_eval_chebys_double__t(double, double) nogil
 * cdef _proto_eval_chebys_double__t *_proto_eval_chebys_double__t_var = &_func_eval_chebys[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double complex _proto_eval_chebys_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":1353
 * from orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double complex _proto_eval_chebys_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_chebys_double_complex__t *_proto_eval_chebys_double_complex__t_var = &_func_eval_chebys[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebys_l as _func_eval_chebys_l
 * ctypedef double _proto_eval_chebys_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":1356
 * from orthogonal_eval cimport eval_chebys_l as _func_eval_chebys_l
 * ctypedef double _proto_eval_chebys_l_t(long, double) nogil
 * cdef _proto_eval_chebys_l_t *_proto_eval_chebys_l_t_var = &_func_eval_chebys_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double _proto_eval_chebyt_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys_l);

  /* "scipy/special/_ufuncs.pyx":1359
 * from orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double _proto_eval_chebyt_double__t(double, double) nogil
 * cdef _proto_eval_chebyt_double__t *_proto_eval_chebyt_double__t_var = &_func_eval_chebyt[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double complex _proto_eval_chebyt_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":1362
 * from orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double complex _proto_eval_chebyt_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_chebyt_double_complex__t *_proto_eval_chebyt_double_complex__t_var = &_func_eval_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebyt_l as _func_eval_chebyt_l
 * ctypedef double _proto_eval_chebyt_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":1365
 * from orthogonal_eval cimport eval_chebyt_l as _func_eval_chebyt_l
 * ctypedef double _proto_eval_chebyt_l_t(long, double) nogil
 * cdef _proto_eval_chebyt_l_t *_proto_eval_chebyt_l_t_var = &_func_eval_chebyt_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double _proto_eval_chebyu_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l);

  /* "scipy/special/_ufuncs.pyx":1368
 * from orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double _proto_eval_chebyu_double__t(double, double) nogil
 * cdef _proto_eval_chebyu_double__t *_proto_eval_chebyu_double__t_var = &_func_eval_chebyu[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double complex _proto_eval_chebyu_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":1371
 * from orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double complex _proto_eval_chebyu_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_chebyu_double_complex__t *_proto_eval_chebyu_double_complex__t_var = &_func_eval_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_chebyu_l as _func_eval_chebyu_l
 * ctypedef double _proto_eval_chebyu_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":1374
 * from orthogonal_eval cimport eval_chebyu_l as _func_eval_chebyu_l
 * ctypedef double _proto_eval_chebyu_l_t(long, double) nogil
 * cdef _proto_eval_chebyu_l_t *_proto_eval_chebyu_l_t_var = &_func_eval_chebyu_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double _proto_eval_gegenbauer_double__t(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l);

  /* "scipy/special/_ufuncs.pyx":1377
 * from orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double _proto_eval_gegenbauer_double__t(double, double, double) nogil
 * cdef _proto_eval_gegenbauer_double__t *_proto_eval_gegenbauer_double__t_var = &_func_eval_gegenbauer[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double complex _proto_eval_gegenbauer_double_complex__t(double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":1380
 * from orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double complex _proto_eval_gegenbauer_double_complex__t(double, double, double complex) nogil
 * cdef _proto_eval_gegenbauer_double_complex__t *_proto_eval_gegenbauer_double_complex__t_var = &_func_eval_gegenbauer[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_gegenbauer_l as _func_eval_gegenbauer_l
 * ctypedef double _proto_eval_gegenbauer_l_t(long, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":1383
 * from orthogonal_eval cimport eval_gegenbauer_l as _func_eval_gegenbauer_l
 * ctypedef double _proto_eval_gegenbauer_l_t(long, double, double) nogil
 * cdef _proto_eval_gegenbauer_l_t *_proto_eval_gegenbauer_l_t_var = &_func_eval_gegenbauer_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double _proto_eval_genlaguerre_double__t(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer_l);

  /* "scipy/special/_ufuncs.pyx":1386
 * from orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double _proto_eval_genlaguerre_double__t(double, double, double) nogil
 * cdef _proto_eval_genlaguerre_double__t *_proto_eval_genlaguerre_double__t_var = &_func_eval_genlaguerre[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double complex _proto_eval_genlaguerre_double_complex__t(double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":1389
 * from orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double complex _proto_eval_genlaguerre_double_complex__t(double, double, double complex) nogil
 * cdef _proto_eval_genlaguerre_double_complex__t *_proto_eval_genlaguerre_double_complex__t_var = &_func_eval_genlaguerre[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_genlaguerre_l as _func_eval_genlaguerre_l
 * ctypedef double _proto_eval_genlaguerre_l_t(long, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":1392
 * from orthogonal_eval cimport eval_genlaguerre_l as _func_eval_genlaguerre_l
 * ctypedef double _proto_eval_genlaguerre_l_t(long, double, double) nogil
 * cdef _proto_eval_genlaguerre_l_t *_proto_eval_genlaguerre_l_t_var = &_func_eval_genlaguerre_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_hermite as _func_eval_hermite
 * ctypedef double _proto_eval_hermite_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l);

  /* "scipy/special/_ufuncs.pyx":1395
 * from orthogonal_eval cimport eval_hermite as _func_eval_hermite
 * ctypedef double _proto_eval_hermite_t(long, double) nogil
 * cdef _proto_eval_hermite_t *_proto_eval_hermite_t_var = &_func_eval_hermite             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_hermitenorm as _func_eval_hermitenorm
 * ctypedef double _proto_eval_hermitenorm_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_hermite_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermite);

  /* "scipy/special/_ufuncs.pyx":1398
 * from orthogonal_eval cimport eval_hermitenorm as _func_eval_hermitenorm
 * ctypedef double _proto_eval_hermitenorm_t(long, double) nogil
 * cdef _proto_eval_hermitenorm_t *_proto_eval_hermitenorm_t_var = &_func_eval_hermitenorm             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double _proto_eval_jacobi_double__t(double, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_hermitenorm_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm);

  /* "scipy/special/_ufuncs.pyx":1401
 * from orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double _proto_eval_jacobi_double__t(double, double, double, double) nogil
 * cdef _proto_eval_jacobi_double__t *_proto_eval_jacobi_double__t_var = &_func_eval_jacobi[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double complex _proto_eval_jacobi_double_complex__t(double, double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":1404
 * from orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double complex _proto_eval_jacobi_double_complex__t(double, double, double, double complex) nogil
 * cdef _proto_eval_jacobi_double_complex__t *_proto_eval_jacobi_double_complex__t_var = &_func_eval_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_jacobi_l as _func_eval_jacobi_l
 * ctypedef double _proto_eval_jacobi_l_t(long, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":1407
 * from orthogonal_eval cimport eval_jacobi_l as _func_eval_jacobi_l
 * ctypedef double _proto_eval_jacobi_l_t(long, double, double, double) nogil
 * cdef _proto_eval_jacobi_l_t *_proto_eval_jacobi_l_t_var = &_func_eval_jacobi_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double _proto_eval_laguerre_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l);

  /* "scipy/special/_ufuncs.pyx":1410
 * from orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double _proto_eval_laguerre_double__t(double, double) nogil
 * cdef _proto_eval_laguerre_double__t *_proto_eval_laguerre_double__t_var = &_func_eval_laguerre[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double complex _proto_eval_laguerre_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":1413
 * from orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double complex _proto_eval_laguerre_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_laguerre_double_complex__t *_proto_eval_laguerre_double_complex__t_var = &_func_eval_laguerre[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_laguerre_l as _func_eval_laguerre_l
 * ctypedef double _proto_eval_laguerre_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":1416
 * from orthogonal_eval cimport eval_laguerre_l as _func_eval_laguerre_l
 * ctypedef double _proto_eval_laguerre_l_t(long, double) nogil
 * cdef _proto_eval_laguerre_l_t *_proto_eval_laguerre_l_t_var = &_func_eval_laguerre_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double _proto_eval_legendre_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre_l);

  /* "scipy/special/_ufuncs.pyx":1419
 * from orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double _proto_eval_legendre_double__t(double, double) nogil
 * cdef _proto_eval_legendre_double__t *_proto_eval_legendre_double__t_var = &_func_eval_legendre[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double complex _proto_eval_legendre_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":1422
 * from orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double complex _proto_eval_legendre_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_legendre_double_complex__t *_proto_eval_legendre_double_complex__t_var = &_func_eval_legendre[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_legendre_l as _func_eval_legendre_l
 * ctypedef double _proto_eval_legendre_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":1425
 * from orthogonal_eval cimport eval_legendre_l as _func_eval_legendre_l
 * ctypedef double _proto_eval_legendre_l_t(long, double) nogil
 * cdef _proto_eval_legendre_l_t *_proto_eval_legendre_l_t_var = &_func_eval_legendre_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double _proto_eval_sh_chebyt_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l);

  /* "scipy/special/_ufuncs.pyx":1428
 * from orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double _proto_eval_sh_chebyt_double__t(double, double) nogil
 * cdef _proto_eval_sh_chebyt_double__t *_proto_eval_sh_chebyt_double__t_var = &_func_eval_sh_chebyt[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double complex _proto_eval_sh_chebyt_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":1431
 * from orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double complex _proto_eval_sh_chebyt_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_sh_chebyt_double_complex__t *_proto_eval_sh_chebyt_double_complex__t_var = &_func_eval_sh_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_chebyt_l as _func_eval_sh_chebyt_l
 * ctypedef double _proto_eval_sh_chebyt_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":1434
 * from orthogonal_eval cimport eval_sh_chebyt_l as _func_eval_sh_chebyt_l
 * ctypedef double _proto_eval_sh_chebyt_l_t(long, double) nogil
 * cdef _proto_eval_sh_chebyt_l_t *_proto_eval_sh_chebyt_l_t_var = &_func_eval_sh_chebyt_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double _proto_eval_sh_chebyu_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt_l);

  /* "scipy/special/_ufuncs.pyx":1437
 * from orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double _proto_eval_sh_chebyu_double__t(double, double) nogil
 * cdef _proto_eval_sh_chebyu_double__t *_proto_eval_sh_chebyu_double__t_var = &_func_eval_sh_chebyu[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double complex _proto_eval_sh_chebyu_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":1440
 * from orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double complex _proto_eval_sh_chebyu_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_sh_chebyu_double_complex__t *_proto_eval_sh_chebyu_double_complex__t_var = &_func_eval_sh_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_chebyu_l as _func_eval_sh_chebyu_l
 * ctypedef double _proto_eval_sh_chebyu_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":1443
 * from orthogonal_eval cimport eval_sh_chebyu_l as _func_eval_sh_chebyu_l
 * ctypedef double _proto_eval_sh_chebyu_l_t(long, double) nogil
 * cdef _proto_eval_sh_chebyu_l_t *_proto_eval_sh_chebyu_l_t_var = &_func_eval_sh_chebyu_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double _proto_eval_sh_jacobi_double__t(double, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu_l);

  /* "scipy/special/_ufuncs.pyx":1446
 * from orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double _proto_eval_sh_jacobi_double__t(double, double, double, double) nogil
 * cdef _proto_eval_sh_jacobi_double__t *_proto_eval_sh_jacobi_double__t_var = &_func_eval_sh_jacobi[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double complex _proto_eval_sh_jacobi_double_complex__t(double, double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":1449
 * from orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double complex _proto_eval_sh_jacobi_double_complex__t(double, double, double, double complex) nogil
 * cdef _proto_eval_sh_jacobi_double_complex__t *_proto_eval_sh_jacobi_double_complex__t_var = &_func_eval_sh_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_jacobi_l as _func_eval_sh_jacobi_l
 * ctypedef double _proto_eval_sh_jacobi_l_t(long, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":1452
 * from orthogonal_eval cimport eval_sh_jacobi_l as _func_eval_sh_jacobi_l
 * ctypedef double _proto_eval_sh_jacobi_l_t(long, double, double, double) nogil
 * cdef _proto_eval_sh_jacobi_l_t *_proto_eval_sh_jacobi_l_t_var = &_func_eval_sh_jacobi_l             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double _proto_eval_sh_legendre_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi_l);

  /* "scipy/special/_ufuncs.pyx":1455
 * from orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double _proto_eval_sh_legendre_double__t(double, double) nogil
 * cdef _proto_eval_sh_legendre_double__t *_proto_eval_sh_legendre_double__t_var = &_func_eval_sh_legendre[double]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double complex _proto_eval_sh_legendre_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":1458
 * from orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double complex _proto_eval_sh_legendre_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_sh_legendre_double_complex__t *_proto_eval_sh_legendre_double_complex__t_var = &_func_eval_sh_legendre[double_complex]             # <<<<<<<<<<<<<<
 * from orthogonal_eval cimport eval_sh_legendre_l as _func_eval_sh_legendre_l
 * ctypedef double _proto_eval_sh_legendre_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":1461
 * from orthogonal_eval cimport eval_sh_legendre_l as _func_eval_sh_legendre_l
 * ctypedef double _proto_eval_sh_legendre_l_t(long, double) nogil
 * cdef _proto_eval_sh_legendre_l_t *_proto_eval_sh_legendre_l_t_var = &_func_eval_sh_legendre_l             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_exp1_wrap "exp1_wrap"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre_l);

  /* "scipy/special/_ufuncs.pyx":1486
 * from _legacy cimport expn_unsafe as _func_expn_unsafe
 * ctypedef double _proto_expn_unsafe_t(double, double) nogil
 * cdef _proto_expn_unsafe_t *_proto_expn_unsafe_t_var = &_func_expn_unsafe             # <<<<<<<<<<<<<<
 * from _exprel cimport exprel as _func_exprel
 * ctypedef double _proto_exprel_t(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_expn_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_expn_unsafe);

  /* "scipy/special/_ufuncs.pyx":1489
 * from _exprel cimport exprel as _func_exprel
 * ctypedef double _proto_exprel_t(double) nogil
 * cdef _proto_exprel_t *_proto_exprel_t_var = &_func_exprel             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_fdtr "fdtr"(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_exprel_t_var = (&__pyx_f_5scipy_7special_7_exprel_exprel);

  /* "scipy/special/_ufuncs.pyx":1540
 * from _convex_analysis cimport huber as _func_huber
 * ctypedef double _proto_huber_t(double, double) nogil
 * cdef _proto_huber_t *_proto_huber_t_var = &_func_huber             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_hyp1f1_wrap "hyp1f1_wrap"(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_huber_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_huber);

  /* "scipy/special/_ufuncs.pyx":1551
 * from _legacy cimport hyp2f0_unsafe as _func_hyp2f0_unsafe
 * ctypedef double _proto_hyp2f0_unsafe_t(double, double, double, double, double *) nogil
 * cdef _proto_hyp2f0_unsafe_t *_proto_hyp2f0_unsafe_t_var = &_func_hyp2f0_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_hyp2f1 "hyp2f1"(double, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_hyp2f0_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe);

  /* "scipy/special/_ufuncs.pyx":1570
 * from _boxcox cimport inv_boxcox as _func_inv_boxcox
 * ctypedef double _proto_inv_boxcox_t(double, double) nogil
 * cdef _proto_inv_boxcox_t *_proto_inv_boxcox_t_var = &_func_inv_boxcox             # <<<<<<<<<<<<<<
 * from _boxcox cimport inv_boxcox1p as _func_inv_boxcox1p
 * ctypedef double _proto_inv_boxcox1p_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_inv_boxcox_t_var = (&__pyx_f_5scipy_7special_7_boxcox_inv_boxcox);

  /* "scipy/special/_ufuncs.pyx":1573
 * from _boxcox cimport inv_boxcox1p as _func_inv_boxcox1p
 * ctypedef double _proto_inv_boxcox1p_t(double, double) nogil
 * cdef _proto_inv_boxcox1p_t *_proto_inv_boxcox1p_t_var = &_func_inv_boxcox1p             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef int _func_it2i0k0_wrap "it2i0k0_wrap"(double, double *, double *) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_inv_boxcox1p_t_var = (&__pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":1630
 * from _convex_analysis cimport kl_div as _func_kl_div
 * ctypedef double _proto_kl_div_t(double, double) nogil
 * cdef _proto_kl_div_t *_proto_kl_div_t_var = &_func_kl_div             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_cbesk_wrap_real_int "cbesk_wrap_real_int"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_kl_div_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_kl_div);

  /* "scipy/special/_ufuncs.pyx":1635
 * from _legacy cimport kn_unsafe as _func_kn_unsafe
 * ctypedef double _proto_kn_unsafe_t(double, double) nogil
 * cdef _proto_kn_unsafe_t *_proto_kn_unsafe_t_var = &_func_kn_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_kolmogi "kolmogi"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_kn_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_kn_unsafe);

  /* "scipy/special/_ufuncs.pyx":1686
 * from _legacy cimport nbdtr_unsafe as _func_nbdtr_unsafe
 * ctypedef double _proto_nbdtr_unsafe_t(double, double, double) nogil
 * cdef _proto_nbdtr_unsafe_t *_proto_nbdtr_unsafe_t_var = &_func_nbdtr_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_nbdtrc "nbdtrc"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_nbdtr_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":1691
 * from _legacy cimport nbdtrc_unsafe as _func_nbdtrc_unsafe
 * ctypedef double _proto_nbdtrc_unsafe_t(double, double, double) nogil
 * cdef _proto_nbdtrc_unsafe_t *_proto_nbdtrc_unsafe_t_var = &_func_nbdtrc_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_nbdtri "nbdtri"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_nbdtrc_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":1696
 * from _legacy cimport nbdtri_unsafe as _func_nbdtri_unsafe
 * ctypedef double _proto_nbdtri_unsafe_t(double, double, double) nogil
 * cdef _proto_nbdtri_unsafe_t *_proto_nbdtri_unsafe_t_var = &_func_nbdtri_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_cdfnbn2_wrap "cdfnbn2_wrap"(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_nbdtri_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":1751
 * from _legacy cimport pdtr_unsafe as _func_pdtr_unsafe
 * ctypedef double _proto_pdtr_unsafe_t(double, double) nogil
 * cdef _proto_pdtr_unsafe_t *_proto_pdtr_unsafe_t_var = &_func_pdtr_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_pdtrc "pdtrc"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_pdtr_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_pdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":1756
 * from _legacy cimport pdtrc_unsafe as _func_pdtrc_unsafe
 * ctypedef double _proto_pdtrc_unsafe_t(double, double) nogil
 * cdef _proto_pdtrc_unsafe_t *_proto_pdtrc_unsafe_t_var = &_func_pdtrc_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_pdtri "pdtri"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_pdtrc_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":1761
 * from _legacy cimport pdtri_unsafe as _func_pdtri_unsafe
 * ctypedef double _proto_pdtri_unsafe_t(double, double) nogil
 * cdef _proto_pdtri_unsafe_t *_proto_pdtri_unsafe_t_var = &_func_pdtri_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_cdfpoi2_wrap "cdfpoi2_wrap"(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_pdtri_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_pdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":1782
 * from _convex_analysis cimport pseudo_huber as _func_pseudo_huber
 * ctypedef double _proto_pseudo_huber_t(double, double) nogil
 * cdef _proto_pseudo_huber_t *_proto_pseudo_huber_t_var = &_func_pseudo_huber             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_psi "psi"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_pseudo_huber_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber);

  /* "scipy/special/_ufuncs.pyx":1791
 * from _convex_analysis cimport rel_entr as _func_rel_entr
 * ctypedef double _proto_rel_entr_t(double, double) nogil
 * cdef _proto_rel_entr_t *_proto_rel_entr_t_var = &_func_rel_entr             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_rgamma "rgamma"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_rel_entr_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_rel_entr);

  /* "scipy/special/_ufuncs.pyx":1808
 * from _legacy cimport smirnov_unsafe as _func_smirnov_unsafe
 * ctypedef double _proto_smirnov_unsafe_t(double, double) nogil
 * cdef _proto_smirnov_unsafe_t *_proto_smirnov_unsafe_t_var = &_func_smirnov_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_smirnovi "smirnovi"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_smirnov_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_smirnov_unsafe);

  /* "scipy/special/_ufuncs.pyx":1813
 * from _legacy cimport smirnovi_unsafe as _func_smirnovi_unsafe
 * ctypedef double _proto_smirnovi_unsafe_t(double, double) nogil
 * cdef _proto_smirnovi_unsafe_t *_proto_smirnovi_unsafe_t_var = &_func_smirnovi_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_spence "spence"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_smirnovi_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe);

  /* "scipy/special/_ufuncs.pyx":1818
 * from sph_harm cimport sph_harmonic as _func_sph_harmonic
 * ctypedef double complex _proto_sph_harmonic_t(int, int, double, double) nogil
 * cdef _proto_sph_harmonic_t *_proto_sph_harmonic_t_var = &_func_sph_harmonic             # <<<<<<<<<<<<<<
 * from _legacy cimport sph_harmonic_unsafe as _func_sph_harmonic_unsafe
 * ctypedef double complex _proto_sph_harmonic_unsafe_t(double, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_sph_harmonic_t_var = (&__pyx_f_5scipy_7special_8sph_harm_sph_harmonic);

  /* "scipy/special/_ufuncs.pyx":1821
 * from _legacy cimport sph_harmonic_unsafe as _func_sph_harmonic_unsafe
 * ctypedef double complex _proto_sph_harmonic_unsafe_t(double, double, double, double) nogil
 * cdef _proto_sph_harmonic_unsafe_t *_proto_sph_harmonic_unsafe_t_var = &_func_sph_harmonic_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_cdft1_wrap "cdft1_wrap"(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_sph_harmonic_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":1836
 * from _xlogy cimport xlog1py as _func_xlog1py
 * ctypedef double _proto_xlog1py_t(double, double) nogil
 * cdef _proto_xlog1py_t *_proto_xlog1py_t_var = &_func_xlog1py             # <<<<<<<<<<<<<<
 * from _xlogy cimport xlogy as _func_xlogy
 * ctypedef double _proto_xlogy_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_xlog1py_t_var = (&__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":1839
 * from _xlogy cimport xlogy as _func_xlogy
 * ctypedef double _proto_xlogy_double__t(double, double) nogil
 * cdef _proto_xlogy_double__t *_proto_xlogy_double__t_var = &_func_xlogy[double]             # <<<<<<<<<<<<<<
 * from _xlogy cimport xlogy as _func_xlogy
 * ctypedef double complex _proto_xlogy_double_complex__t(double complex, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_xlogy_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":1842
 * from _xlogy cimport xlogy as _func_xlogy
 * ctypedef double complex _proto_xlogy_double_complex__t(double complex, double complex) nogil
 * cdef _proto_xlogy_double_complex__t *_proto_xlogy_double_complex__t_var = &_func_xlogy[double_complex]             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_y0 "y0"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_xlogy_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":1851
 * from _legacy cimport yn_unsafe as _func_yn_unsafe
 * ctypedef double _proto_yn_unsafe_t(double, double) nogil
 * cdef _proto_yn_unsafe_t *_proto_yn_unsafe_t_var = &_func_yn_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_cbesy_wrap_real "cbesy_wrap_real"(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_yn_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_yn_unsafe);

  /* "scipy/special/_ufuncs.pyx":1868
 * cdef void *ufunc__ellip_harm_data[3]
 * cdef char ufunc__ellip_harm_types[24]
 * cdef char *ufunc__ellip_harm_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `ellip_harm` instead.")
 * ufunc__ellip_harm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_fffffff_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_doc = __pyx_k_Internal_function_use_ellip_harm;

  /* "scipy/special/_ufuncs.pyx":1870
 * cdef char *ufunc__ellip_harm_doc = (
 *     "Internal function, use `ellip_harm` instead.")
 * ufunc__ellip_harm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_fffffff_f             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddiiddd__As_ddllddd_d
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_fffffff_f);

  /* "scipy/special/_ufuncs.pyx":1871
 *     "Internal function, use `ellip_harm` instead.")
 * ufunc__ellip_harm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_fffffff_f
 * ufunc__ellip_harm_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddiiddd__As_ddllddd_d             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddiiddd__As_ddllddd_d);

  /* "scipy/special/_ufuncs.pyx":1872
 * ufunc__ellip_harm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_fffffff_f
 * ufunc__ellip_harm_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddiiddd__As_ddllddd_d
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_ddddddd_d);

  /* "scipy/special/_ufuncs.pyx":1873
 * ufunc__ellip_harm_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddiiddd__As_ddllddd_d
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1874
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1875
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1876
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1877
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1878
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1879
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1880
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1881
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1882
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1883
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[10]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":1884
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[11]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":1885
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1886
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1887
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1888
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1889
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1890
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1891
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[18]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1892
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[19]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1893
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[20]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1894
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[21]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1895
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[22]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1896
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[23]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1897
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":1898
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[1]) = ((void *)((char *)__pyx_k_ellip_harm_2));

  /* "scipy/special/_ufuncs.pyx":1899
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[2]) = ((void *)__pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic);

  /* "scipy/special/_ufuncs.pyx":1900
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[3]) = ((void *)((char *)__pyx_k_ellip_harm_2));

  /* "scipy/special/_ufuncs.pyx":1901
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":1902
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[5]) = ((void *)((char *)__pyx_k_ellip_harm_2));

  /* "scipy/special/_ufuncs.pyx":1903
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]
 * ufunc__ellip_harm_data[2] = &ufunc__ellip_harm_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":1904
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_data[2] = &ufunc__ellip_harm_ptr[2*2]
 * _ellip_harm = np.PyUFunc_FromFuncAndData(ufunc__ellip_harm_loops, ufunc__ellip_harm_data, ufunc__ellip_harm_types, 3, 7, 1, 0, "_ellip_harm", ufunc__ellip_harm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":1905
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]
 * ufunc__ellip_harm_data[2] = &ufunc__ellip_harm_ptr[2*2]             # <<<<<<<<<<<<<<
 * _ellip_harm = np.PyUFunc_FromFuncAndData(ufunc__ellip_harm_loops, ufunc__ellip_harm_data, ufunc__ellip_harm_types, 3, 7, 1, 0, "_ellip_harm", ufunc__ellip_harm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":1906
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]
 * ufunc__ellip_harm_data[2] = &ufunc__ellip_harm_ptr[2*2]
 * _ellip_harm = np.PyUFunc_FromFuncAndData(ufunc__ellip_harm_loops, ufunc__ellip_harm_data, ufunc__ellip_harm_types, 3, 7, 1, 0, "_ellip_harm", ufunc__ellip_harm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__lambertw_loops[1]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types, 3, 7, 1, 0, __pyx_k_ellip_harm_2, __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellip_harm_2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":1912
 * cdef void *ufunc__lambertw_data[1]
 * cdef char ufunc__lambertw_types[4]
 * cdef char *ufunc__lambertw_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `lambertw` instead.")
 * ufunc__lambertw_loops[0] = <np.PyUFuncGenericFunction>loop_D_Dld__As_Dld_D
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_doc = __pyx_k_Internal_function_use_lambertw_i;

  /* "scipy/special/_ufuncs.pyx":1914
 * cdef char *ufunc__lambertw_doc = (
 *     "Internal function, use `lambertw` instead.")
 * ufunc__lambertw_loops[0] = <np.PyUFuncGenericFunction>loop_D_Dld__As_Dld_D             # <<<<<<<<<<<<<<
 * ufunc__lambertw_types[0] = <char>NPY_CDOUBLE
 * ufunc__lambertw_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_Dld__As_Dld_D);

  /* "scipy/special/_ufuncs.pyx":1915
 *     "Internal function, use `lambertw` instead.")
 * ufunc__lambertw_loops[0] = <np.PyUFuncGenericFunction>loop_D_Dld__As_Dld_D
 * ufunc__lambertw_types[0] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lambertw_types[1] = <char>NPY_LONG
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[0]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":1916
 * ufunc__lambertw_loops[0] = <np.PyUFuncGenericFunction>loop_D_Dld__As_Dld_D
 * ufunc__lambertw_types[0] = <char>NPY_CDOUBLE
 * ufunc__lambertw_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":1917
 * ufunc__lambertw_types[0] = <char>NPY_CDOUBLE
 * ufunc__lambertw_types[1] = <char>NPY_LONG
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1918
 * ufunc__lambertw_types[1] = <char>NPY_LONG
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[3]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":1919
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar             # <<<<<<<<<<<<<<
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")
 * ufunc__lambertw_data[0] = &ufunc__lambertw_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_ptr[0]) = ((void *)__pyx_f_5scipy_7special_8lambertw_lambertw_scalar);

  /* "scipy/special/_ufuncs.pyx":1920
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")             # <<<<<<<<<<<<<<
 * ufunc__lambertw_data[0] = &ufunc__lambertw_ptr[2*0]
 * _lambertw = np.PyUFunc_FromFuncAndData(ufunc__lambertw_loops, ufunc__lambertw_data, ufunc__lambertw_types, 1, 3, 1, 0, "_lambertw", ufunc__lambertw_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_ptr[1]) = ((void *)((char *)__pyx_k_lambertw_2));

  /* "scipy/special/_ufuncs.pyx":1921
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")
 * ufunc__lambertw_data[0] = &ufunc__lambertw_ptr[2*0]             # <<<<<<<<<<<<<<
 * _lambertw = np.PyUFunc_FromFuncAndData(ufunc__lambertw_loops, ufunc__lambertw_data, ufunc__lambertw_types, 1, 3, 1, 0, "_lambertw", ufunc__lambertw_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":1922
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")
 * ufunc__lambertw_data[0] = &ufunc__lambertw_ptr[2*0]
 * _lambertw = np.PyUFunc_FromFuncAndData(ufunc__lambertw_loops, ufunc__lambertw_data, ufunc__lambertw_types, 1, 3, 1, 0, "_lambertw", ufunc__lambertw_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__struve_asymp_large_z_loops[1]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types, 1, 3, 1, 0, __pyx_k_lambertw_2, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_lambertw_2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":1928
 * cdef void *ufunc__struve_asymp_large_z_data[1]
 * cdef char ufunc__struve_asymp_large_z_types[5]
 * cdef char *ufunc__struve_asymp_large_z_doc = (             # <<<<<<<<<<<<<<
 *     "_struve_asymp_large_z(v, z, is_h)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_doc = __pyx_k_struve_asymp_large_z_v_z_is_h_I;

  /* "scipy/special/_ufuncs.pyx":1938
 *     "-------\n"
 *     "v, err")
 * ufunc__struve_asymp_large_z_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd);

  /* "scipy/special/_ufuncs.pyx":1939
 *     "v, err")
 * ufunc__struve_asymp_large_z_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_asymp_large_z_types[0] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[0]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1940
 * ufunc__struve_asymp_large_z_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_asymp_large_z_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1941
 * ufunc__struve_asymp_large_z_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[2]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":1942
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1943
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1944
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")
 * ufunc__struve_asymp_large_z_data[0] = &ufunc__struve_asymp_large_z_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_ptr[0]) = ((void *)struve_asymp_large_z);

  /* "scipy/special/_ufuncs.pyx":1945
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_data[0] = &ufunc__struve_asymp_large_z_ptr[2*0]
 * _struve_asymp_large_z = np.PyUFunc_FromFuncAndData(ufunc__struve_asymp_large_z_loops, ufunc__struve_asymp_large_z_data, ufunc__struve_asymp_large_z_types, 1, 3, 2, 0, "_struve_asymp_large_z", ufunc__struve_asymp_large_z_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_ptr[1]) = ((void *)((char *)__pyx_k_struve_asymp_large_z));

  /* "scipy/special/_ufuncs.pyx":1946
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")
 * ufunc__struve_asymp_large_z_data[0] = &ufunc__struve_asymp_large_z_ptr[2*0]             # <<<<<<<<<<<<<<
 * _struve_asymp_large_z = np.PyUFunc_FromFuncAndData(ufunc__struve_asymp_large_z_loops, ufunc__struve_asymp_large_z_data, ufunc__struve_asymp_large_z_types, 1, 3, 2, 0, "_struve_asymp_large_z", ufunc__struve_asymp_large_z_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":1947
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")
 * ufunc__struve_asymp_large_z_data[0] = &ufunc__struve_asymp_large_z_ptr[2*0]
 * _struve_asymp_large_z = np.PyUFunc_FromFuncAndData(ufunc__struve_asymp_large_z_loops, ufunc__struve_asymp_large_z_data, ufunc__struve_asymp_large_z_types, 1, 3, 2, 0, "_struve_asymp_large_z", ufunc__struve_asymp_large_z_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__struve_bessel_series_loops[1]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types, 1, 3, 2, 0, __pyx_k_struve_asymp_large_z, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struve_asymp_large_z, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":1953
 * cdef void *ufunc__struve_bessel_series_data[1]
 * cdef char ufunc__struve_bessel_series_types[5]
 * cdef char *ufunc__struve_bessel_series_doc = (             # <<<<<<<<<<<<<<
 *     "_struve_bessel_series(v, z, is_h)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_doc = __pyx_k_struve_bessel_series_v_z_is_h_I;

  /* "scipy/special/_ufuncs.pyx":1963
 *     "-------\n"
 *     "v, err")
 * ufunc__struve_bessel_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd);

  /* "scipy/special/_ufuncs.pyx":1964
 *     "v, err")
 * ufunc__struve_bessel_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_bessel_series_types[0] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[0]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1965
 * ufunc__struve_bessel_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_bessel_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1966
 * ufunc__struve_bessel_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[2]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":1967
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1968
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1969
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")
 * ufunc__struve_bessel_series_data[0] = &ufunc__struve_bessel_series_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_ptr[0]) = ((void *)struve_bessel_series);

  /* "scipy/special/_ufuncs.pyx":1970
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_data[0] = &ufunc__struve_bessel_series_ptr[2*0]
 * _struve_bessel_series = np.PyUFunc_FromFuncAndData(ufunc__struve_bessel_series_loops, ufunc__struve_bessel_series_data, ufunc__struve_bessel_series_types, 1, 3, 2, 0, "_struve_bessel_series", ufunc__struve_bessel_series_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_ptr[1]) = ((void *)((char *)__pyx_k_struve_bessel_series));

  /* "scipy/special/_ufuncs.pyx":1971
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")
 * ufunc__struve_bessel_series_data[0] = &ufunc__struve_bessel_series_ptr[2*0]             # <<<<<<<<<<<<<<
 * _struve_bessel_series = np.PyUFunc_FromFuncAndData(ufunc__struve_bessel_series_loops, ufunc__struve_bessel_series_data, ufunc__struve_bessel_series_types, 1, 3, 2, 0, "_struve_bessel_series", ufunc__struve_bessel_series_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":1972
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")
 * ufunc__struve_bessel_series_data[0] = &ufunc__struve_bessel_series_ptr[2*0]
 * _struve_bessel_series = np.PyUFunc_FromFuncAndData(ufunc__struve_bessel_series_loops, ufunc__struve_bessel_series_data, ufunc__struve_bessel_series_types, 1, 3, 2, 0, "_struve_bessel_series", ufunc__struve_bessel_series_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__struve_power_series_loops[1]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types, 1, 3, 2, 0, __pyx_k_struve_bessel_series, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struve_bessel_series, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":1978
 * cdef void *ufunc__struve_power_series_data[1]
 * cdef char ufunc__struve_power_series_types[5]
 * cdef char *ufunc__struve_power_series_doc = (             # <<<<<<<<<<<<<<
 *     "_struve_power_series(v, z, is_h)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_doc = __pyx_k_struve_power_series_v_z_is_h_In;

  /* "scipy/special/_ufuncs.pyx":1988
 *     "-------\n"
 *     "v, err")
 * ufunc__struve_power_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd);

  /* "scipy/special/_ufuncs.pyx":1989
 *     "v, err")
 * ufunc__struve_power_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_power_series_types[0] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[0]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1990
 * ufunc__struve_power_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_power_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1991
 * ufunc__struve_power_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[2]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":1992
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1993
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1994
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")
 * ufunc__struve_power_series_data[0] = &ufunc__struve_power_series_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_ptr[0]) = ((void *)struve_power_series);

  /* "scipy/special/_ufuncs.pyx":1995
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_data[0] = &ufunc__struve_power_series_ptr[2*0]
 * _struve_power_series = np.PyUFunc_FromFuncAndData(ufunc__struve_power_series_loops, ufunc__struve_power_series_data, ufunc__struve_power_series_types, 1, 3, 2, 0, "_struve_power_series", ufunc__struve_power_series_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_ptr[1]) = ((void *)((char *)__pyx_k_struve_power_series));

  /* "scipy/special/_ufuncs.pyx":1996
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")
 * ufunc__struve_power_series_data[0] = &ufunc__struve_power_series_ptr[2*0]             # <<<<<<<<<<<<<<
 * _struve_power_series = np.PyUFunc_FromFuncAndData(ufunc__struve_power_series_loops, ufunc__struve_power_series_data, ufunc__struve_power_series_types, 1, 3, 2, 0, "_struve_power_series", ufunc__struve_power_series_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":1997
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")
 * ufunc__struve_power_series_data[0] = &ufunc__struve_power_series_ptr[2*0]
 * _struve_power_series = np.PyUFunc_FromFuncAndData(ufunc__struve_power_series_loops, ufunc__struve_power_series_data, ufunc__struve_power_series_types, 1, 3, 2, 0, "_struve_power_series", ufunc__struve_power_series_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_airy_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types, 1, 3, 2, 0, __pyx_k_struve_power_series, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1997; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struve_power_series, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1997; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2003
 * cdef void *ufunc_airy_data[4]
 * cdef char ufunc_airy_types[20]
 * cdef char *ufunc_airy_doc = (             # <<<<<<<<<<<<<<
 *     "airy(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_doc = __pyx_k_airy_z_Airy_functions_and_their;

  /* "scipy/special/_ufuncs.pyx":2021
 *     "-----\n"
 *     "The Airy functions Ai and Bi are two independent solutions of y''(x) = x y.")
 * ufunc_airy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff             # <<<<<<<<<<<<<<
 * ufunc_airy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff);

  /* "scipy/special/_ufuncs.pyx":2022
 *     "The Airy functions Ai and Bi are two independent solutions of y''(x) = x y.")
 * ufunc_airy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_airy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd             # <<<<<<<<<<<<<<
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd);

  /* "scipy/special/_ufuncs.pyx":2023
 * ufunc_airy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_airy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF             # <<<<<<<<<<<<<<
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airy_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_F_FFFF);

  /* "scipy/special/_ufuncs.pyx":2024
 * ufunc_airy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD             # <<<<<<<<<<<<<<
 * ufunc_airy_types[0] = <char>NPY_FLOAT
 * ufunc_airy_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_D_DDDD);

  /* "scipy/special/_ufuncs.pyx":2025
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airy_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[1] = <char>NPY_FLOAT
 * ufunc_airy_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2026
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airy_types[0] = <char>NPY_FLOAT
 * ufunc_airy_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[2] = <char>NPY_FLOAT
 * ufunc_airy_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2027
 * ufunc_airy_types[0] = <char>NPY_FLOAT
 * ufunc_airy_types[1] = <char>NPY_FLOAT
 * ufunc_airy_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[3] = <char>NPY_FLOAT
 * ufunc_airy_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2028
 * ufunc_airy_types[1] = <char>NPY_FLOAT
 * ufunc_airy_types[2] = <char>NPY_FLOAT
 * ufunc_airy_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[4] = <char>NPY_FLOAT
 * ufunc_airy_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2029
 * ufunc_airy_types[2] = <char>NPY_FLOAT
 * ufunc_airy_types[3] = <char>NPY_FLOAT
 * ufunc_airy_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[5] = <char>NPY_DOUBLE
 * ufunc_airy_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2030
 * ufunc_airy_types[3] = <char>NPY_FLOAT
 * ufunc_airy_types[4] = <char>NPY_FLOAT
 * ufunc_airy_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[6] = <char>NPY_DOUBLE
 * ufunc_airy_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2031
 * ufunc_airy_types[4] = <char>NPY_FLOAT
 * ufunc_airy_types[5] = <char>NPY_DOUBLE
 * ufunc_airy_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[7] = <char>NPY_DOUBLE
 * ufunc_airy_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2032
 * ufunc_airy_types[5] = <char>NPY_DOUBLE
 * ufunc_airy_types[6] = <char>NPY_DOUBLE
 * ufunc_airy_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[8] = <char>NPY_DOUBLE
 * ufunc_airy_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2033
 * ufunc_airy_types[6] = <char>NPY_DOUBLE
 * ufunc_airy_types[7] = <char>NPY_DOUBLE
 * ufunc_airy_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[9] = <char>NPY_DOUBLE
 * ufunc_airy_types[10] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2034
 * ufunc_airy_types[7] = <char>NPY_DOUBLE
 * ufunc_airy_types[8] = <char>NPY_DOUBLE
 * ufunc_airy_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[10] = <char>NPY_CFLOAT
 * ufunc_airy_types[11] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2035
 * ufunc_airy_types[8] = <char>NPY_DOUBLE
 * ufunc_airy_types[9] = <char>NPY_DOUBLE
 * ufunc_airy_types[10] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[11] = <char>NPY_CFLOAT
 * ufunc_airy_types[12] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[10]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2036
 * ufunc_airy_types[9] = <char>NPY_DOUBLE
 * ufunc_airy_types[10] = <char>NPY_CFLOAT
 * ufunc_airy_types[11] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[12] = <char>NPY_CFLOAT
 * ufunc_airy_types[13] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[11]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2037
 * ufunc_airy_types[10] = <char>NPY_CFLOAT
 * ufunc_airy_types[11] = <char>NPY_CFLOAT
 * ufunc_airy_types[12] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[13] = <char>NPY_CFLOAT
 * ufunc_airy_types[14] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[12]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2038
 * ufunc_airy_types[11] = <char>NPY_CFLOAT
 * ufunc_airy_types[12] = <char>NPY_CFLOAT
 * ufunc_airy_types[13] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[14] = <char>NPY_CFLOAT
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[13]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2039
 * ufunc_airy_types[12] = <char>NPY_CFLOAT
 * ufunc_airy_types[13] = <char>NPY_CFLOAT
 * ufunc_airy_types[14] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[14]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2040
 * ufunc_airy_types[13] = <char>NPY_CFLOAT
 * ufunc_airy_types[14] = <char>NPY_CFLOAT
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[15]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2041
 * ufunc_airy_types[14] = <char>NPY_CFLOAT
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[16]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2042
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[17]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2043
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2044
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2045
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[0]) = ((void *)airy_wrap);

  /* "scipy/special/_ufuncs.pyx":2046
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[1]) = ((void *)((char *)__pyx_k_airy));

  /* "scipy/special/_ufuncs.pyx":2047
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[2]) = ((void *)airy_wrap);

  /* "scipy/special/_ufuncs.pyx":2048
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[3]) = ((void *)((char *)__pyx_k_airy));

  /* "scipy/special/_ufuncs.pyx":2049
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[4]) = ((void *)cairy_wrap);

  /* "scipy/special/_ufuncs.pyx":2050
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[5]) = ((void *)((char *)__pyx_k_airy));

  /* "scipy/special/_ufuncs.pyx":2051
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[6]) = ((void *)cairy_wrap);

  /* "scipy/special/_ufuncs.pyx":2052
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")             # <<<<<<<<<<<<<<
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[7]) = ((void *)((char *)__pyx_k_airy));

  /* "scipy/special/_ufuncs.pyx":2053
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2054
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]
 * ufunc_airy_data[3] = &ufunc_airy_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2055
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_airy_data[3] = &ufunc_airy_ptr[2*3]
 * airy = np.PyUFunc_FromFuncAndData(ufunc_airy_loops, ufunc_airy_data, ufunc_airy_types, 4, 1, 4, 0, "airy", ufunc_airy_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2056
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]
 * ufunc_airy_data[3] = &ufunc_airy_ptr[2*3]             # <<<<<<<<<<<<<<
 * airy = np.PyUFunc_FromFuncAndData(ufunc_airy_loops, ufunc_airy_data, ufunc_airy_types, 4, 1, 4, 0, "airy", ufunc_airy_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":2057
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]
 * ufunc_airy_data[3] = &ufunc_airy_ptr[2*3]
 * airy = np.PyUFunc_FromFuncAndData(ufunc_airy_loops, ufunc_airy_data, ufunc_airy_types, 4, 1, 4, 0, "airy", ufunc_airy_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_airye_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types, 4, 1, 4, 0, __pyx_k_airy, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2057; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_airy, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2057; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2063
 * cdef void *ufunc_airye_data[4]
 * cdef char ufunc_airye_types[20]
 * cdef char *ufunc_airye_doc = (             # <<<<<<<<<<<<<<
 *     "airye(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_doc = __pyx_k_airye_z_Exponentially_scaled_Air;

  /* "scipy/special/_ufuncs.pyx":2084
 *     "eAi, eAip, eBi, eBip\n"
 *     "    Airy functions Ai and Bi, and their derivatives Aip and Bip")
 * ufunc_airye_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff             # <<<<<<<<<<<<<<
 * ufunc_airye_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff);

  /* "scipy/special/_ufuncs.pyx":2085
 *     "    Airy functions Ai and Bi, and their derivatives Aip and Bip")
 * ufunc_airye_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_airye_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd             # <<<<<<<<<<<<<<
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd);

  /* "scipy/special/_ufuncs.pyx":2086
 * ufunc_airye_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_airye_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF             # <<<<<<<<<<<<<<
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airye_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_F_FFFF);

  /* "scipy/special/_ufuncs.pyx":2087
 * ufunc_airye_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD             # <<<<<<<<<<<<<<
 * ufunc_airye_types[0] = <char>NPY_FLOAT
 * ufunc_airye_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_D_DDDD);

  /* "scipy/special/_ufuncs.pyx":2088
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airye_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[1] = <char>NPY_FLOAT
 * ufunc_airye_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2089
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airye_types[0] = <char>NPY_FLOAT
 * ufunc_airye_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[2] = <char>NPY_FLOAT
 * ufunc_airye_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2090
 * ufunc_airye_types[0] = <char>NPY_FLOAT
 * ufunc_airye_types[1] = <char>NPY_FLOAT
 * ufunc_airye_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[3] = <char>NPY_FLOAT
 * ufunc_airye_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2091
 * ufunc_airye_types[1] = <char>NPY_FLOAT
 * ufunc_airye_types[2] = <char>NPY_FLOAT
 * ufunc_airye_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[4] = <char>NPY_FLOAT
 * ufunc_airye_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2092
 * ufunc_airye_types[2] = <char>NPY_FLOAT
 * ufunc_airye_types[3] = <char>NPY_FLOAT
 * ufunc_airye_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[5] = <char>NPY_DOUBLE
 * ufunc_airye_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2093
 * ufunc_airye_types[3] = <char>NPY_FLOAT
 * ufunc_airye_types[4] = <char>NPY_FLOAT
 * ufunc_airye_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[6] = <char>NPY_DOUBLE
 * ufunc_airye_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2094
 * ufunc_airye_types[4] = <char>NPY_FLOAT
 * ufunc_airye_types[5] = <char>NPY_DOUBLE
 * ufunc_airye_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[7] = <char>NPY_DOUBLE
 * ufunc_airye_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2095
 * ufunc_airye_types[5] = <char>NPY_DOUBLE
 * ufunc_airye_types[6] = <char>NPY_DOUBLE
 * ufunc_airye_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[8] = <char>NPY_DOUBLE
 * ufunc_airye_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2096
 * ufunc_airye_types[6] = <char>NPY_DOUBLE
 * ufunc_airye_types[7] = <char>NPY_DOUBLE
 * ufunc_airye_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[9] = <char>NPY_DOUBLE
 * ufunc_airye_types[10] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2097
 * ufunc_airye_types[7] = <char>NPY_DOUBLE
 * ufunc_airye_types[8] = <char>NPY_DOUBLE
 * ufunc_airye_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[10] = <char>NPY_CFLOAT
 * ufunc_airye_types[11] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2098
 * ufunc_airye_types[8] = <char>NPY_DOUBLE
 * ufunc_airye_types[9] = <char>NPY_DOUBLE
 * ufunc_airye_types[10] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[11] = <char>NPY_CFLOAT
 * ufunc_airye_types[12] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[10]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2099
 * ufunc_airye_types[9] = <char>NPY_DOUBLE
 * ufunc_airye_types[10] = <char>NPY_CFLOAT
 * ufunc_airye_types[11] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[12] = <char>NPY_CFLOAT
 * ufunc_airye_types[13] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[11]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2100
 * ufunc_airye_types[10] = <char>NPY_CFLOAT
 * ufunc_airye_types[11] = <char>NPY_CFLOAT
 * ufunc_airye_types[12] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[13] = <char>NPY_CFLOAT
 * ufunc_airye_types[14] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[12]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2101
 * ufunc_airye_types[11] = <char>NPY_CFLOAT
 * ufunc_airye_types[12] = <char>NPY_CFLOAT
 * ufunc_airye_types[13] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[14] = <char>NPY_CFLOAT
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[13]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2102
 * ufunc_airye_types[12] = <char>NPY_CFLOAT
 * ufunc_airye_types[13] = <char>NPY_CFLOAT
 * ufunc_airye_types[14] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[14]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2103
 * ufunc_airye_types[13] = <char>NPY_CFLOAT
 * ufunc_airye_types[14] = <char>NPY_CFLOAT
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[15]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2104
 * ufunc_airye_types[14] = <char>NPY_CFLOAT
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[16]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2105
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[17]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2106
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2107
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2108
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[0]) = ((void *)cairy_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":2109
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[1]) = ((void *)((char *)__pyx_k_airye));

  /* "scipy/special/_ufuncs.pyx":2110
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[2]) = ((void *)cairy_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":2111
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[3]) = ((void *)((char *)__pyx_k_airye));

  /* "scipy/special/_ufuncs.pyx":2112
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[4]) = ((void *)cairy_wrap_e);

  /* "scipy/special/_ufuncs.pyx":2113
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[5]) = ((void *)((char *)__pyx_k_airye));

  /* "scipy/special/_ufuncs.pyx":2114
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[6]) = ((void *)cairy_wrap_e);

  /* "scipy/special/_ufuncs.pyx":2115
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")             # <<<<<<<<<<<<<<
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[7]) = ((void *)((char *)__pyx_k_airye));

  /* "scipy/special/_ufuncs.pyx":2116
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2117
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]
 * ufunc_airye_data[3] = &ufunc_airye_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2118
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_airye_data[3] = &ufunc_airye_ptr[2*3]
 * airye = np.PyUFunc_FromFuncAndData(ufunc_airye_loops, ufunc_airye_data, ufunc_airye_types, 4, 1, 4, 0, "airye", ufunc_airye_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2119
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]
 * ufunc_airye_data[3] = &ufunc_airye_ptr[2*3]             # <<<<<<<<<<<<<<
 * airye = np.PyUFunc_FromFuncAndData(ufunc_airye_loops, ufunc_airye_data, ufunc_airye_types, 4, 1, 4, 0, "airye", ufunc_airye_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":2120
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]
 * ufunc_airye_data[3] = &ufunc_airye_ptr[2*3]
 * airye = np.PyUFunc_FromFuncAndData(ufunc_airye_loops, ufunc_airye_data, ufunc_airye_types, 4, 1, 4, 0, "airye", ufunc_airye_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtr_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types, 4, 1, 4, 0, __pyx_k_airye, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_airye, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2126
 * cdef void *ufunc_bdtr_data[3]
 * cdef char ufunc_bdtr_types[12]
 * cdef char *ufunc_bdtr_doc = (             # <<<<<<<<<<<<<<
 *     "bdtr(k, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_doc = __pyx_k_bdtr_k_n_p_Binomial_distribution;

  /* "scipy/special/_ufuncs.pyx":2148
 *     "y : float\n"
 *     "    Sum of terms")
 * ufunc_bdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_bdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":2149
 *     "    Sum of terms")
 * ufunc_bdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtr_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2150
 * ufunc_bdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[0] = <char>NPY_LONG
 * ufunc_bdtr_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2151
 * ufunc_bdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtr_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[1] = <char>NPY_LONG
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2152
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtr_types[0] = <char>NPY_LONG
 * ufunc_bdtr_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2153
 * ufunc_bdtr_types[0] = <char>NPY_LONG
 * ufunc_bdtr_types[1] = <char>NPY_LONG
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2154
 * ufunc_bdtr_types[1] = <char>NPY_LONG
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2155
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2156
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2157
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2158
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2159
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2160
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2161
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2162
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2163
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[0]) = ((void *)bdtr);

  /* "scipy/special/_ufuncs.pyx":2164
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[1]) = ((void *)((char *)__pyx_k_bdtr));

  /* "scipy/special/_ufuncs.pyx":2165
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":2166
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[3]) = ((void *)((char *)__pyx_k_bdtr));

  /* "scipy/special/_ufuncs.pyx":2167
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":2168
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")             # <<<<<<<<<<<<<<
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[5]) = ((void *)((char *)__pyx_k_bdtr));

  /* "scipy/special/_ufuncs.pyx":2169
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]
 * ufunc_bdtr_data[2] = &ufunc_bdtr_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2170
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_bdtr_data[2] = &ufunc_bdtr_ptr[2*2]
 * bdtr = np.PyUFunc_FromFuncAndData(ufunc_bdtr_loops, ufunc_bdtr_data, ufunc_bdtr_types, 3, 3, 1, 0, "bdtr", ufunc_bdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2171
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]
 * ufunc_bdtr_data[2] = &ufunc_bdtr_ptr[2*2]             # <<<<<<<<<<<<<<
 * bdtr = np.PyUFunc_FromFuncAndData(ufunc_bdtr_loops, ufunc_bdtr_data, ufunc_bdtr_types, 3, 3, 1, 0, "bdtr", ufunc_bdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2172
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]
 * ufunc_bdtr_data[2] = &ufunc_bdtr_ptr[2*2]
 * bdtr = np.PyUFunc_FromFuncAndData(ufunc_bdtr_loops, ufunc_bdtr_data, ufunc_bdtr_types, 3, 3, 1, 0, "bdtr", ufunc_bdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtrc_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types, 3, 3, 1, 0, __pyx_k_bdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2178
 * cdef void *ufunc_bdtrc_data[3]
 * cdef char ufunc_bdtrc_types[12]
 * cdef char *ufunc_bdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "bdtrc(k, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_doc = __pyx_k_bdtrc_k_n_p_Binomial_distributio;

  /* "scipy/special/_ufuncs.pyx":2200
 *     "y : float\n"
 *     "    Sum of terms")
 * ufunc_bdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":2201
 *     "    Sum of terms")
 * ufunc_bdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2202
 * ufunc_bdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[0] = <char>NPY_LONG
 * ufunc_bdtrc_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2203
 * ufunc_bdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[1] = <char>NPY_LONG
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2204
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrc_types[0] = <char>NPY_LONG
 * ufunc_bdtrc_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2205
 * ufunc_bdtrc_types[0] = <char>NPY_LONG
 * ufunc_bdtrc_types[1] = <char>NPY_LONG
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2206
 * ufunc_bdtrc_types[1] = <char>NPY_LONG
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2207
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2208
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2209
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2210
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2211
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2212
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2213
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2214
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2215
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[0]) = ((void *)bdtrc);

  /* "scipy/special/_ufuncs.pyx":2216
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[1]) = ((void *)((char *)__pyx_k_bdtrc));

  /* "scipy/special/_ufuncs.pyx":2217
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":2218
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[3]) = ((void *)((char *)__pyx_k_bdtrc));

  /* "scipy/special/_ufuncs.pyx":2219
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":2220
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[5]) = ((void *)((char *)__pyx_k_bdtrc));

  /* "scipy/special/_ufuncs.pyx":2221
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]
 * ufunc_bdtrc_data[2] = &ufunc_bdtrc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2222
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_data[2] = &ufunc_bdtrc_ptr[2*2]
 * bdtrc = np.PyUFunc_FromFuncAndData(ufunc_bdtrc_loops, ufunc_bdtrc_data, ufunc_bdtrc_types, 3, 3, 1, 0, "bdtrc", ufunc_bdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2223
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]
 * ufunc_bdtrc_data[2] = &ufunc_bdtrc_ptr[2*2]             # <<<<<<<<<<<<<<
 * bdtrc = np.PyUFunc_FromFuncAndData(ufunc_bdtrc_loops, ufunc_bdtrc_data, ufunc_bdtrc_types, 3, 3, 1, 0, "bdtrc", ufunc_bdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2224
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]
 * ufunc_bdtrc_data[2] = &ufunc_bdtrc_ptr[2*2]
 * bdtrc = np.PyUFunc_FromFuncAndData(ufunc_bdtrc_loops, ufunc_bdtrc_data, ufunc_bdtrc_types, 3, 3, 1, 0, "bdtrc", ufunc_bdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtri_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types, 3, 3, 1, 0, __pyx_k_bdtrc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtrc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2230
 * cdef void *ufunc_bdtri_data[3]
 * cdef char ufunc_bdtri_types[12]
 * cdef char *ufunc_bdtri_doc = (             # <<<<<<<<<<<<<<
 *     "bdtri(k, n, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_doc = __pyx_k_bdtri_k_n_y_Inverse_function_to;

  /* "scipy/special/_ufuncs.pyx":2237
 *     "Finds probability `p` such that for the cumulative binomial\n"
 *     "probability ``bdtr(k, n, p) == y``.")
 * ufunc_bdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_bdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":2238
 *     "probability ``bdtr(k, n, p) == y``.")
 * ufunc_bdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtri_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2239
 * ufunc_bdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[0] = <char>NPY_LONG
 * ufunc_bdtri_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2240
 * ufunc_bdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtri_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[1] = <char>NPY_LONG
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2241
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtri_types[0] = <char>NPY_LONG
 * ufunc_bdtri_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2242
 * ufunc_bdtri_types[0] = <char>NPY_LONG
 * ufunc_bdtri_types[1] = <char>NPY_LONG
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2243
 * ufunc_bdtri_types[1] = <char>NPY_LONG
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2244
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2245
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2246
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2247
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2248
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2249
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2250
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2251
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2252
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[0]) = ((void *)bdtri);

  /* "scipy/special/_ufuncs.pyx":2253
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[1]) = ((void *)((char *)__pyx_k_bdtri));

  /* "scipy/special/_ufuncs.pyx":2254
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":2255
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[3]) = ((void *)((char *)__pyx_k_bdtri));

  /* "scipy/special/_ufuncs.pyx":2256
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":2257
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")             # <<<<<<<<<<<<<<
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[5]) = ((void *)((char *)__pyx_k_bdtri));

  /* "scipy/special/_ufuncs.pyx":2258
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]
 * ufunc_bdtri_data[2] = &ufunc_bdtri_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2259
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_bdtri_data[2] = &ufunc_bdtri_ptr[2*2]
 * bdtri = np.PyUFunc_FromFuncAndData(ufunc_bdtri_loops, ufunc_bdtri_data, ufunc_bdtri_types, 3, 3, 1, 0, "bdtri", ufunc_bdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2260
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]
 * ufunc_bdtri_data[2] = &ufunc_bdtri_ptr[2*2]             # <<<<<<<<<<<<<<
 * bdtri = np.PyUFunc_FromFuncAndData(ufunc_bdtri_loops, ufunc_bdtri_data, ufunc_bdtri_types, 3, 3, 1, 0, "bdtri", ufunc_bdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2261
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]
 * ufunc_bdtri_data[2] = &ufunc_bdtri_ptr[2*2]
 * bdtri = np.PyUFunc_FromFuncAndData(ufunc_bdtri_loops, ufunc_bdtri_data, ufunc_bdtri_types, 3, 3, 1, 0, "bdtri", ufunc_bdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtrik_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types, 3, 3, 1, 0, __pyx_k_bdtri, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2261; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtri, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2261; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2267
 * cdef void *ufunc_bdtrik_data[2]
 * cdef char ufunc_bdtrik_types[8]
 * cdef char *ufunc_bdtrik_doc = (             # <<<<<<<<<<<<<<
 *     "bdtrik(y, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_doc = __pyx_k_bdtrik_y_n_p_Inverse_function_to;

  /* "scipy/special/_ufuncs.pyx":2271
 *     "\n"
 *     "Inverse function to `bdtr` vs `k`")
 * ufunc_bdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2272
 *     "Inverse function to `bdtr` vs `k`")
 * ufunc_bdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2273
 * ufunc_bdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2274
 * ufunc_bdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2275
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2276
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2277
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2278
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2279
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2280
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2281
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[0]) = ((void *)cdfbin2_wrap);

  /* "scipy/special/_ufuncs.pyx":2282
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[1]) = ((void *)((char *)__pyx_k_bdtrik));

  /* "scipy/special/_ufuncs.pyx":2283
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[2]) = ((void *)cdfbin2_wrap);

  /* "scipy/special/_ufuncs.pyx":2284
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]
 * ufunc_bdtrik_data[1] = &ufunc_bdtrik_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[3]) = ((void *)((char *)__pyx_k_bdtrik));

  /* "scipy/special/_ufuncs.pyx":2285
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_data[1] = &ufunc_bdtrik_ptr[2*1]
 * bdtrik = np.PyUFunc_FromFuncAndData(ufunc_bdtrik_loops, ufunc_bdtrik_data, ufunc_bdtrik_types, 2, 3, 1, 0, "bdtrik", ufunc_bdtrik_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2286
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]
 * ufunc_bdtrik_data[1] = &ufunc_bdtrik_ptr[2*1]             # <<<<<<<<<<<<<<
 * bdtrik = np.PyUFunc_FromFuncAndData(ufunc_bdtrik_loops, ufunc_bdtrik_data, ufunc_bdtrik_types, 2, 3, 1, 0, "bdtrik", ufunc_bdtrik_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2287
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]
 * ufunc_bdtrik_data[1] = &ufunc_bdtrik_ptr[2*1]
 * bdtrik = np.PyUFunc_FromFuncAndData(ufunc_bdtrik_loops, ufunc_bdtrik_data, ufunc_bdtrik_types, 2, 3, 1, 0, "bdtrik", ufunc_bdtrik_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtrin_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types, 2, 3, 1, 0, __pyx_k_bdtrik, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtrik, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2293
 * cdef void *ufunc_bdtrin_data[2]
 * cdef char ufunc_bdtrin_types[8]
 * cdef char *ufunc_bdtrin_doc = (             # <<<<<<<<<<<<<<
 *     "bdtrin(k, y, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_doc = __pyx_k_bdtrin_k_y_p_Inverse_function_to;

  /* "scipy/special/_ufuncs.pyx":2297
 *     "\n"
 *     "Inverse function to `bdtr` vs `n`")
 * ufunc_bdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2298
 *     "Inverse function to `bdtr` vs `n`")
 * ufunc_bdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2299
 * ufunc_bdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2300
 * ufunc_bdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2301
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2302
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2303
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2304
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2305
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2306
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2307
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[0]) = ((void *)cdfbin3_wrap);

  /* "scipy/special/_ufuncs.pyx":2308
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[1]) = ((void *)((char *)__pyx_k_bdtrin));

  /* "scipy/special/_ufuncs.pyx":2309
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[2]) = ((void *)cdfbin3_wrap);

  /* "scipy/special/_ufuncs.pyx":2310
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]
 * ufunc_bdtrin_data[1] = &ufunc_bdtrin_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[3]) = ((void *)((char *)__pyx_k_bdtrin));

  /* "scipy/special/_ufuncs.pyx":2311
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_data[1] = &ufunc_bdtrin_ptr[2*1]
 * bdtrin = np.PyUFunc_FromFuncAndData(ufunc_bdtrin_loops, ufunc_bdtrin_data, ufunc_bdtrin_types, 2, 3, 1, 0, "bdtrin", ufunc_bdtrin_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2312
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]
 * ufunc_bdtrin_data[1] = &ufunc_bdtrin_ptr[2*1]             # <<<<<<<<<<<<<<
 * bdtrin = np.PyUFunc_FromFuncAndData(ufunc_bdtrin_loops, ufunc_bdtrin_data, ufunc_bdtrin_types, 2, 3, 1, 0, "bdtrin", ufunc_bdtrin_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2313
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]
 * ufunc_bdtrin_data[1] = &ufunc_bdtrin_ptr[2*1]
 * bdtrin = np.PyUFunc_FromFuncAndData(ufunc_bdtrin_loops, ufunc_bdtrin_data, ufunc_bdtrin_types, 2, 3, 1, 0, "bdtrin", ufunc_bdtrin_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bei_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types, 2, 3, 1, 0, __pyx_k_bdtrin, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtrin, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2319
 * cdef void *ufunc_bei_data[2]
 * cdef char ufunc_bei_types[4]
 * cdef char *ufunc_bei_doc = (             # <<<<<<<<<<<<<<
 *     "bei(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_doc = __pyx_k_bei_x_Kelvin_function_bei;

  /* "scipy/special/_ufuncs.pyx":2323
 *     "\n"
 *     "Kelvin function bei")
 * ufunc_bei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_bei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_bei_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2324
 *     "Kelvin function bei")
 * ufunc_bei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_bei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_bei_types[0] = <char>NPY_FLOAT
 * ufunc_bei_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2325
 * ufunc_bei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_bei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_bei_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bei_types[1] = <char>NPY_FLOAT
 * ufunc_bei_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2326
 * ufunc_bei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_bei_types[0] = <char>NPY_FLOAT
 * ufunc_bei_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bei_types[2] = <char>NPY_DOUBLE
 * ufunc_bei_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2327
 * ufunc_bei_types[0] = <char>NPY_FLOAT
 * ufunc_bei_types[1] = <char>NPY_FLOAT
 * ufunc_bei_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bei_types[3] = <char>NPY_DOUBLE
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2328
 * ufunc_bei_types[1] = <char>NPY_FLOAT
 * ufunc_bei_types[2] = <char>NPY_DOUBLE
 * ufunc_bei_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2329
 * ufunc_bei_types[2] = <char>NPY_DOUBLE
 * ufunc_bei_types[3] = <char>NPY_DOUBLE
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap             # <<<<<<<<<<<<<<
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[0]) = ((void *)bei_wrap);

  /* "scipy/special/_ufuncs.pyx":2330
 * ufunc_bei_types[3] = <char>NPY_DOUBLE
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")             # <<<<<<<<<<<<<<
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[1]) = ((void *)((char *)__pyx_k_bei));

  /* "scipy/special/_ufuncs.pyx":2331
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap             # <<<<<<<<<<<<<<
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[2]) = ((void *)bei_wrap);

  /* "scipy/special/_ufuncs.pyx":2332
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")             # <<<<<<<<<<<<<<
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]
 * ufunc_bei_data[1] = &ufunc_bei_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[3]) = ((void *)((char *)__pyx_k_bei));

  /* "scipy/special/_ufuncs.pyx":2333
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bei_data[1] = &ufunc_bei_ptr[2*1]
 * bei = np.PyUFunc_FromFuncAndData(ufunc_bei_loops, ufunc_bei_data, ufunc_bei_types, 2, 1, 1, 0, "bei", ufunc_bei_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2334
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]
 * ufunc_bei_data[1] = &ufunc_bei_ptr[2*1]             # <<<<<<<<<<<<<<
 * bei = np.PyUFunc_FromFuncAndData(ufunc_bei_loops, ufunc_bei_data, ufunc_bei_types, 2, 1, 1, 0, "bei", ufunc_bei_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2335
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]
 * ufunc_bei_data[1] = &ufunc_bei_ptr[2*1]
 * bei = np.PyUFunc_FromFuncAndData(ufunc_bei_loops, ufunc_bei_data, ufunc_bei_types, 2, 1, 1, 0, "bei", ufunc_bei_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_beip_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types, 2, 1, 1, 0, __pyx_k_bei, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bei, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2341
 * cdef void *ufunc_beip_data[2]
 * cdef char ufunc_beip_types[4]
 * cdef char *ufunc_beip_doc = (             # <<<<<<<<<<<<<<
 *     "beip(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_doc = __pyx_k_beip_x_Derivative_of_the_Kelvin;

  /* "scipy/special/_ufuncs.pyx":2345
 *     "\n"
 *     "Derivative of the Kelvin function `bei`")
 * ufunc_beip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_beip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_beip_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2346
 *     "Derivative of the Kelvin function `bei`")
 * ufunc_beip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_beip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_beip_types[0] = <char>NPY_FLOAT
 * ufunc_beip_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2347
 * ufunc_beip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_beip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_beip_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beip_types[1] = <char>NPY_FLOAT
 * ufunc_beip_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2348
 * ufunc_beip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_beip_types[0] = <char>NPY_FLOAT
 * ufunc_beip_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beip_types[2] = <char>NPY_DOUBLE
 * ufunc_beip_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2349
 * ufunc_beip_types[0] = <char>NPY_FLOAT
 * ufunc_beip_types[1] = <char>NPY_FLOAT
 * ufunc_beip_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beip_types[3] = <char>NPY_DOUBLE
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2350
 * ufunc_beip_types[1] = <char>NPY_FLOAT
 * ufunc_beip_types[2] = <char>NPY_DOUBLE
 * ufunc_beip_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2351
 * ufunc_beip_types[2] = <char>NPY_DOUBLE
 * ufunc_beip_types[3] = <char>NPY_DOUBLE
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap             # <<<<<<<<<<<<<<
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[0]) = ((void *)beip_wrap);

  /* "scipy/special/_ufuncs.pyx":2352
 * ufunc_beip_types[3] = <char>NPY_DOUBLE
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")             # <<<<<<<<<<<<<<
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[1]) = ((void *)((char *)__pyx_k_beip));

  /* "scipy/special/_ufuncs.pyx":2353
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap             # <<<<<<<<<<<<<<
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[2]) = ((void *)beip_wrap);

  /* "scipy/special/_ufuncs.pyx":2354
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")             # <<<<<<<<<<<<<<
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]
 * ufunc_beip_data[1] = &ufunc_beip_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[3]) = ((void *)((char *)__pyx_k_beip));

  /* "scipy/special/_ufuncs.pyx":2355
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_beip_data[1] = &ufunc_beip_ptr[2*1]
 * beip = np.PyUFunc_FromFuncAndData(ufunc_beip_loops, ufunc_beip_data, ufunc_beip_types, 2, 1, 1, 0, "beip", ufunc_beip_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2356
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]
 * ufunc_beip_data[1] = &ufunc_beip_ptr[2*1]             # <<<<<<<<<<<<<<
 * beip = np.PyUFunc_FromFuncAndData(ufunc_beip_loops, ufunc_beip_data, ufunc_beip_types, 2, 1, 1, 0, "beip", ufunc_beip_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2357
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]
 * ufunc_beip_data[1] = &ufunc_beip_ptr[2*1]
 * beip = np.PyUFunc_FromFuncAndData(ufunc_beip_loops, ufunc_beip_data, ufunc_beip_types, 2, 1, 1, 0, "beip", ufunc_beip_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ber_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types, 2, 1, 1, 0, __pyx_k_beip, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_beip, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2363
 * cdef void *ufunc_ber_data[2]
 * cdef char ufunc_ber_types[4]
 * cdef char *ufunc_ber_doc = (             # <<<<<<<<<<<<<<
 *     "ber(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_doc = __pyx_k_ber_x_Kelvin_function_ber;

  /* "scipy/special/_ufuncs.pyx":2367
 *     "\n"
 *     "Kelvin function ber.")
 * ufunc_ber_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ber_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ber_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2368
 *     "Kelvin function ber.")
 * ufunc_ber_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ber_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ber_types[0] = <char>NPY_FLOAT
 * ufunc_ber_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2369
 * ufunc_ber_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ber_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ber_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ber_types[1] = <char>NPY_FLOAT
 * ufunc_ber_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2370
 * ufunc_ber_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ber_types[0] = <char>NPY_FLOAT
 * ufunc_ber_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ber_types[2] = <char>NPY_DOUBLE
 * ufunc_ber_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2371
 * ufunc_ber_types[0] = <char>NPY_FLOAT
 * ufunc_ber_types[1] = <char>NPY_FLOAT
 * ufunc_ber_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ber_types[3] = <char>NPY_DOUBLE
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2372
 * ufunc_ber_types[1] = <char>NPY_FLOAT
 * ufunc_ber_types[2] = <char>NPY_DOUBLE
 * ufunc_ber_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2373
 * ufunc_ber_types[2] = <char>NPY_DOUBLE
 * ufunc_ber_types[3] = <char>NPY_DOUBLE
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap             # <<<<<<<<<<<<<<
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[0]) = ((void *)ber_wrap);

  /* "scipy/special/_ufuncs.pyx":2374
 * ufunc_ber_types[3] = <char>NPY_DOUBLE
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")             # <<<<<<<<<<<<<<
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[1]) = ((void *)((char *)__pyx_k_ber));

  /* "scipy/special/_ufuncs.pyx":2375
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap             # <<<<<<<<<<<<<<
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[2]) = ((void *)ber_wrap);

  /* "scipy/special/_ufuncs.pyx":2376
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")             # <<<<<<<<<<<<<<
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]
 * ufunc_ber_data[1] = &ufunc_ber_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[3]) = ((void *)((char *)__pyx_k_ber));

  /* "scipy/special/_ufuncs.pyx":2377
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ber_data[1] = &ufunc_ber_ptr[2*1]
 * ber = np.PyUFunc_FromFuncAndData(ufunc_ber_loops, ufunc_ber_data, ufunc_ber_types, 2, 1, 1, 0, "ber", ufunc_ber_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2378
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]
 * ufunc_ber_data[1] = &ufunc_ber_ptr[2*1]             # <<<<<<<<<<<<<<
 * ber = np.PyUFunc_FromFuncAndData(ufunc_ber_loops, ufunc_ber_data, ufunc_ber_types, 2, 1, 1, 0, "ber", ufunc_ber_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2379
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]
 * ufunc_ber_data[1] = &ufunc_ber_ptr[2*1]
 * ber = np.PyUFunc_FromFuncAndData(ufunc_ber_loops, ufunc_ber_data, ufunc_ber_types, 2, 1, 1, 0, "ber", ufunc_ber_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_berp_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types, 2, 1, 1, 0, __pyx_k_ber, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2379; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ber, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2379; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2385
 * cdef void *ufunc_berp_data[2]
 * cdef char ufunc_berp_types[4]
 * cdef char *ufunc_berp_doc = (             # <<<<<<<<<<<<<<
 *     "berp(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_doc = __pyx_k_berp_x_Derivative_of_the_Kelvin;

  /* "scipy/special/_ufuncs.pyx":2389
 *     "\n"
 *     "Derivative of the Kelvin function `ber`")
 * ufunc_berp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_berp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_berp_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2390
 *     "Derivative of the Kelvin function `ber`")
 * ufunc_berp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_berp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_berp_types[0] = <char>NPY_FLOAT
 * ufunc_berp_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2391
 * ufunc_berp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_berp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_berp_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_berp_types[1] = <char>NPY_FLOAT
 * ufunc_berp_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2392
 * ufunc_berp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_berp_types[0] = <char>NPY_FLOAT
 * ufunc_berp_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_berp_types[2] = <char>NPY_DOUBLE
 * ufunc_berp_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2393
 * ufunc_berp_types[0] = <char>NPY_FLOAT
 * ufunc_berp_types[1] = <char>NPY_FLOAT
 * ufunc_berp_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_berp_types[3] = <char>NPY_DOUBLE
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2394
 * ufunc_berp_types[1] = <char>NPY_FLOAT
 * ufunc_berp_types[2] = <char>NPY_DOUBLE
 * ufunc_berp_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2395
 * ufunc_berp_types[2] = <char>NPY_DOUBLE
 * ufunc_berp_types[3] = <char>NPY_DOUBLE
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap             # <<<<<<<<<<<<<<
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[0]) = ((void *)berp_wrap);

  /* "scipy/special/_ufuncs.pyx":2396
 * ufunc_berp_types[3] = <char>NPY_DOUBLE
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")             # <<<<<<<<<<<<<<
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[1]) = ((void *)((char *)__pyx_k_berp));

  /* "scipy/special/_ufuncs.pyx":2397
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap             # <<<<<<<<<<<<<<
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[2]) = ((void *)berp_wrap);

  /* "scipy/special/_ufuncs.pyx":2398
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")             # <<<<<<<<<<<<<<
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]
 * ufunc_berp_data[1] = &ufunc_berp_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[3]) = ((void *)((char *)__pyx_k_berp));

  /* "scipy/special/_ufuncs.pyx":2399
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_berp_data[1] = &ufunc_berp_ptr[2*1]
 * berp = np.PyUFunc_FromFuncAndData(ufunc_berp_loops, ufunc_berp_data, ufunc_berp_types, 2, 1, 1, 0, "berp", ufunc_berp_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2400
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]
 * ufunc_berp_data[1] = &ufunc_berp_ptr[2*1]             # <<<<<<<<<<<<<<
 * berp = np.PyUFunc_FromFuncAndData(ufunc_berp_loops, ufunc_berp_data, ufunc_berp_types, 2, 1, 1, 0, "berp", ufunc_berp_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2401
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]
 * ufunc_berp_data[1] = &ufunc_berp_ptr[2*1]
 * berp = np.PyUFunc_FromFuncAndData(ufunc_berp_loops, ufunc_berp_data, ufunc_berp_types, 2, 1, 1, 0, "berp", ufunc_berp_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_besselpoly_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types, 2, 1, 1, 0, __pyx_k_berp, __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2401; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_berp, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2401; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2407
 * cdef void *ufunc_besselpoly_data[2]
 * cdef char ufunc_besselpoly_types[8]
 * cdef char *ufunc_besselpoly_doc = (             # <<<<<<<<<<<<<<
 *     "besselpoly(a, lmb, nu)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_doc = __pyx_k_besselpoly_a_lmb_nu_Weighted_int;

  /* "scipy/special/_ufuncs.pyx":2418
 *     "where :math:`J_\\nu` is a Bessel function and :math:`\\lambda=lmb`,\n"
 *     ":math:`\\nu=nu`.")
 * ufunc_besselpoly_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2419
 *     ":math:`\\nu=nu`.")
 * ufunc_besselpoly_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_besselpoly_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2420
 * ufunc_besselpoly_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_besselpoly_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2421
 * ufunc_besselpoly_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2422
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2423
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2424
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2425
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2426
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2427
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2428
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[0]) = ((void *)besselpoly);

  /* "scipy/special/_ufuncs.pyx":2429
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[1]) = ((void *)((char *)__pyx_k_besselpoly));

  /* "scipy/special/_ufuncs.pyx":2430
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[2]) = ((void *)besselpoly);

  /* "scipy/special/_ufuncs.pyx":2431
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]
 * ufunc_besselpoly_data[1] = &ufunc_besselpoly_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[3]) = ((void *)((char *)__pyx_k_besselpoly));

  /* "scipy/special/_ufuncs.pyx":2432
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_data[1] = &ufunc_besselpoly_ptr[2*1]
 * besselpoly = np.PyUFunc_FromFuncAndData(ufunc_besselpoly_loops, ufunc_besselpoly_data, ufunc_besselpoly_types, 2, 3, 1, 0, "besselpoly", ufunc_besselpoly_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2433
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]
 * ufunc_besselpoly_data[1] = &ufunc_besselpoly_ptr[2*1]             # <<<<<<<<<<<<<<
 * besselpoly = np.PyUFunc_FromFuncAndData(ufunc_besselpoly_loops, ufunc_besselpoly_data, ufunc_besselpoly_types, 2, 3, 1, 0, "besselpoly", ufunc_besselpoly_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2434
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]
 * ufunc_besselpoly_data[1] = &ufunc_besselpoly_ptr[2*1]
 * besselpoly = np.PyUFunc_FromFuncAndData(ufunc_besselpoly_loops, ufunc_besselpoly_data, ufunc_besselpoly_types, 2, 3, 1, 0, "besselpoly", ufunc_besselpoly_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_beta_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types, 2, 3, 1, 0, __pyx_k_besselpoly, __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_besselpoly, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2440
 * cdef void *ufunc_beta_data[2]
 * cdef char ufunc_beta_types[6]
 * cdef char *ufunc_beta_doc = (             # <<<<<<<<<<<<<<
 *     "beta(a, b)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_doc = __pyx_k_beta_a_b_Beta_function_beta_a_b;

  /* "scipy/special/_ufuncs.pyx":2448
 *     "\n"
 *     "    beta(a, b) =  gamma(a) * gamma(b) / gamma(a+b)")
 * ufunc_beta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_beta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_beta_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2449
 *     "    beta(a, b) =  gamma(a) * gamma(b) / gamma(a+b)")
 * ufunc_beta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_beta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_beta_types[0] = <char>NPY_FLOAT
 * ufunc_beta_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2450
 * ufunc_beta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_beta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_beta_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beta_types[1] = <char>NPY_FLOAT
 * ufunc_beta_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2451
 * ufunc_beta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_beta_types[0] = <char>NPY_FLOAT
 * ufunc_beta_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beta_types[2] = <char>NPY_FLOAT
 * ufunc_beta_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2452
 * ufunc_beta_types[0] = <char>NPY_FLOAT
 * ufunc_beta_types[1] = <char>NPY_FLOAT
 * ufunc_beta_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beta_types[3] = <char>NPY_DOUBLE
 * ufunc_beta_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2453
 * ufunc_beta_types[1] = <char>NPY_FLOAT
 * ufunc_beta_types[2] = <char>NPY_FLOAT
 * ufunc_beta_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beta_types[4] = <char>NPY_DOUBLE
 * ufunc_beta_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2454
 * ufunc_beta_types[2] = <char>NPY_FLOAT
 * ufunc_beta_types[3] = <char>NPY_DOUBLE
 * ufunc_beta_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beta_types[5] = <char>NPY_DOUBLE
 * ufunc_beta_ptr[2*0] = <void*>_func_beta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2455
 * ufunc_beta_types[3] = <char>NPY_DOUBLE
 * ufunc_beta_types[4] = <char>NPY_DOUBLE
 * ufunc_beta_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beta_ptr[2*0] = <void*>_func_beta
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2456
 * ufunc_beta_types[4] = <char>NPY_DOUBLE
 * ufunc_beta_types[5] = <char>NPY_DOUBLE
 * ufunc_beta_ptr[2*0] = <void*>_func_beta             # <<<<<<<<<<<<<<
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")
 * ufunc_beta_ptr[2*1] = <void*>_func_beta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[0]) = ((void *)beta);

  /* "scipy/special/_ufuncs.pyx":2457
 * ufunc_beta_types[5] = <char>NPY_DOUBLE
 * ufunc_beta_ptr[2*0] = <void*>_func_beta
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")             # <<<<<<<<<<<<<<
 * ufunc_beta_ptr[2*1] = <void*>_func_beta
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[1]) = ((void *)((char *)__pyx_k_beta));

  /* "scipy/special/_ufuncs.pyx":2458
 * ufunc_beta_ptr[2*0] = <void*>_func_beta
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")
 * ufunc_beta_ptr[2*1] = <void*>_func_beta             # <<<<<<<<<<<<<<
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[2]) = ((void *)beta);

  /* "scipy/special/_ufuncs.pyx":2459
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")
 * ufunc_beta_ptr[2*1] = <void*>_func_beta
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")             # <<<<<<<<<<<<<<
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]
 * ufunc_beta_data[1] = &ufunc_beta_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[3]) = ((void *)((char *)__pyx_k_beta));

  /* "scipy/special/_ufuncs.pyx":2460
 * ufunc_beta_ptr[2*1] = <void*>_func_beta
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_beta_data[1] = &ufunc_beta_ptr[2*1]
 * beta = np.PyUFunc_FromFuncAndData(ufunc_beta_loops, ufunc_beta_data, ufunc_beta_types, 2, 2, 1, 0, "beta", ufunc_beta_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2461
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]
 * ufunc_beta_data[1] = &ufunc_beta_ptr[2*1]             # <<<<<<<<<<<<<<
 * beta = np.PyUFunc_FromFuncAndData(ufunc_beta_loops, ufunc_beta_data, ufunc_beta_types, 2, 2, 1, 0, "beta", ufunc_beta_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2462
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]
 * ufunc_beta_data[1] = &ufunc_beta_ptr[2*1]
 * beta = np.PyUFunc_FromFuncAndData(ufunc_beta_loops, ufunc_beta_data, ufunc_beta_types, 2, 2, 1, 0, "beta", ufunc_beta_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_betainc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types, 2, 2, 1, 0, __pyx_k_beta, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_beta, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2468
 * cdef void *ufunc_betainc_data[2]
 * cdef char ufunc_betainc_types[8]
 * cdef char *ufunc_betainc_doc = (             # <<<<<<<<<<<<<<
 *     "betainc(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_doc = __pyx_k_betainc_a_b_x_Incomplete_beta_in;

  /* "scipy/special/_ufuncs.pyx":2484
 *     "incomplete beta. Under this definition, you can get the incomplete beta by\n"
 *     "multiplying the result of the scipy function by beta(a, b).")
 * ufunc_betainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_betainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betainc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2485
 *     "multiplying the result of the scipy function by beta(a, b).")
 * ufunc_betainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_betainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[0] = <char>NPY_FLOAT
 * ufunc_betainc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2486
 * ufunc_betainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_betainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betainc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[1] = <char>NPY_FLOAT
 * ufunc_betainc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2487
 * ufunc_betainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betainc_types[0] = <char>NPY_FLOAT
 * ufunc_betainc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[2] = <char>NPY_FLOAT
 * ufunc_betainc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2488
 * ufunc_betainc_types[0] = <char>NPY_FLOAT
 * ufunc_betainc_types[1] = <char>NPY_FLOAT
 * ufunc_betainc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[3] = <char>NPY_FLOAT
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2489
 * ufunc_betainc_types[1] = <char>NPY_FLOAT
 * ufunc_betainc_types[2] = <char>NPY_FLOAT
 * ufunc_betainc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2490
 * ufunc_betainc_types[2] = <char>NPY_FLOAT
 * ufunc_betainc_types[3] = <char>NPY_FLOAT
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2491
 * ufunc_betainc_types[3] = <char>NPY_FLOAT
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2492
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2493
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2494
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet             # <<<<<<<<<<<<<<
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[0]) = ((void *)incbet);

  /* "scipy/special/_ufuncs.pyx":2495
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")             # <<<<<<<<<<<<<<
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[1]) = ((void *)((char *)__pyx_k_betainc));

  /* "scipy/special/_ufuncs.pyx":2496
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet             # <<<<<<<<<<<<<<
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[2]) = ((void *)incbet);

  /* "scipy/special/_ufuncs.pyx":2497
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")             # <<<<<<<<<<<<<<
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]
 * ufunc_betainc_data[1] = &ufunc_betainc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[3]) = ((void *)((char *)__pyx_k_betainc));

  /* "scipy/special/_ufuncs.pyx":2498
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_betainc_data[1] = &ufunc_betainc_ptr[2*1]
 * betainc = np.PyUFunc_FromFuncAndData(ufunc_betainc_loops, ufunc_betainc_data, ufunc_betainc_types, 2, 3, 1, 0, "betainc", ufunc_betainc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2499
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]
 * ufunc_betainc_data[1] = &ufunc_betainc_ptr[2*1]             # <<<<<<<<<<<<<<
 * betainc = np.PyUFunc_FromFuncAndData(ufunc_betainc_loops, ufunc_betainc_data, ufunc_betainc_types, 2, 3, 1, 0, "betainc", ufunc_betainc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2500
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]
 * ufunc_betainc_data[1] = &ufunc_betainc_ptr[2*1]
 * betainc = np.PyUFunc_FromFuncAndData(ufunc_betainc_loops, ufunc_betainc_data, ufunc_betainc_types, 2, 3, 1, 0, "betainc", ufunc_betainc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_betaincinv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types, 2, 3, 1, 0, __pyx_k_betainc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_betainc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2506
 * cdef void *ufunc_betaincinv_data[2]
 * cdef char ufunc_betaincinv_types[8]
 * cdef char *ufunc_betaincinv_doc = (             # <<<<<<<<<<<<<<
 *     "betaincinv(a, b, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_doc = __pyx_k_betaincinv_a_b_y_Inverse_functio;

  /* "scipy/special/_ufuncs.pyx":2512
 *     "\n"
 *     "Compute `x` such that betainc(a, b, x) = y.")
 * ufunc_betaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2513
 *     "Compute `x` such that betainc(a, b, x) = y.")
 * ufunc_betaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_betaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2514
 * ufunc_betaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_betaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2515
 * ufunc_betaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2516
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2517
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2518
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2519
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2520
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2521
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2522
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[0]) = ((void *)incbi);

  /* "scipy/special/_ufuncs.pyx":2523
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[1]) = ((void *)((char *)__pyx_k_betaincinv));

  /* "scipy/special/_ufuncs.pyx":2524
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[2]) = ((void *)incbi);

  /* "scipy/special/_ufuncs.pyx":2525
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]
 * ufunc_betaincinv_data[1] = &ufunc_betaincinv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[3]) = ((void *)((char *)__pyx_k_betaincinv));

  /* "scipy/special/_ufuncs.pyx":2526
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_data[1] = &ufunc_betaincinv_ptr[2*1]
 * betaincinv = np.PyUFunc_FromFuncAndData(ufunc_betaincinv_loops, ufunc_betaincinv_data, ufunc_betaincinv_types, 2, 3, 1, 0, "betaincinv", ufunc_betaincinv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2527
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]
 * ufunc_betaincinv_data[1] = &ufunc_betaincinv_ptr[2*1]             # <<<<<<<<<<<<<<
 * betaincinv = np.PyUFunc_FromFuncAndData(ufunc_betaincinv_loops, ufunc_betaincinv_data, ufunc_betaincinv_types, 2, 3, 1, 0, "betaincinv", ufunc_betaincinv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2528
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]
 * ufunc_betaincinv_data[1] = &ufunc_betaincinv_ptr[2*1]
 * betaincinv = np.PyUFunc_FromFuncAndData(ufunc_betaincinv_loops, ufunc_betaincinv_data, ufunc_betaincinv_types, 2, 3, 1, 0, "betaincinv", ufunc_betaincinv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_betaln_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types, 2, 3, 1, 0, __pyx_k_betaincinv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_betaincinv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2534
 * cdef void *ufunc_betaln_data[2]
 * cdef char ufunc_betaln_types[6]
 * cdef char *ufunc_betaln_doc = (             # <<<<<<<<<<<<<<
 *     "betaln(a, b)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_doc = __pyx_k_betaln_a_b_Natural_logarithm_of;

  /* "scipy/special/_ufuncs.pyx":2540
 *     "\n"
 *     "Computes ``ln(abs(beta(x)))``.")
 * ufunc_betaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_betaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_betaln_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2541
 *     "Computes ``ln(abs(beta(x)))``.")
 * ufunc_betaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_betaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[0] = <char>NPY_FLOAT
 * ufunc_betaln_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2542
 * ufunc_betaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_betaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_betaln_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[1] = <char>NPY_FLOAT
 * ufunc_betaln_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2543
 * ufunc_betaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_betaln_types[0] = <char>NPY_FLOAT
 * ufunc_betaln_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[2] = <char>NPY_FLOAT
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2544
 * ufunc_betaln_types[0] = <char>NPY_FLOAT
 * ufunc_betaln_types[1] = <char>NPY_FLOAT
 * ufunc_betaln_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2545
 * ufunc_betaln_types[1] = <char>NPY_FLOAT
 * ufunc_betaln_types[2] = <char>NPY_FLOAT
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2546
 * ufunc_betaln_types[2] = <char>NPY_FLOAT
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2547
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2548
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta             # <<<<<<<<<<<<<<
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[0]) = ((void *)lbeta);

  /* "scipy/special/_ufuncs.pyx":2549
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")             # <<<<<<<<<<<<<<
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[1]) = ((void *)((char *)__pyx_k_betaln));

  /* "scipy/special/_ufuncs.pyx":2550
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta             # <<<<<<<<<<<<<<
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[2]) = ((void *)lbeta);

  /* "scipy/special/_ufuncs.pyx":2551
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")             # <<<<<<<<<<<<<<
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]
 * ufunc_betaln_data[1] = &ufunc_betaln_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[3]) = ((void *)((char *)__pyx_k_betaln));

  /* "scipy/special/_ufuncs.pyx":2552
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_betaln_data[1] = &ufunc_betaln_ptr[2*1]
 * betaln = np.PyUFunc_FromFuncAndData(ufunc_betaln_loops, ufunc_betaln_data, ufunc_betaln_types, 2, 2, 1, 0, "betaln", ufunc_betaln_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2553
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]
 * ufunc_betaln_data[1] = &ufunc_betaln_ptr[2*1]             # <<<<<<<<<<<<<<
 * betaln = np.PyUFunc_FromFuncAndData(ufunc_betaln_loops, ufunc_betaln_data, ufunc_betaln_types, 2, 2, 1, 0, "betaln", ufunc_betaln_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2554
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]
 * ufunc_betaln_data[1] = &ufunc_betaln_ptr[2*1]
 * betaln = np.PyUFunc_FromFuncAndData(ufunc_betaln_loops, ufunc_betaln_data, ufunc_betaln_types, 2, 2, 1, 0, "betaln", ufunc_betaln_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_binom_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types, 2, 2, 1, 0, __pyx_k_betaln, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_betaln, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2560
 * cdef void *ufunc_binom_data[2]
 * cdef char ufunc_binom_types[6]
 * cdef char *ufunc_binom_doc = (             # <<<<<<<<<<<<<<
 *     "binom(n, k)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_doc = __pyx_k_binom_n_k_Binomial_coefficient;

  /* "scipy/special/_ufuncs.pyx":2564
 *     "\n"
 *     "Binomial coefficient")
 * ufunc_binom_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_binom_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_binom_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2565
 *     "Binomial coefficient")
 * ufunc_binom_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_binom_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_binom_types[0] = <char>NPY_FLOAT
 * ufunc_binom_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2566
 * ufunc_binom_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_binom_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_binom_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_binom_types[1] = <char>NPY_FLOAT
 * ufunc_binom_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2567
 * ufunc_binom_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_binom_types[0] = <char>NPY_FLOAT
 * ufunc_binom_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_binom_types[2] = <char>NPY_FLOAT
 * ufunc_binom_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2568
 * ufunc_binom_types[0] = <char>NPY_FLOAT
 * ufunc_binom_types[1] = <char>NPY_FLOAT
 * ufunc_binom_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_binom_types[3] = <char>NPY_DOUBLE
 * ufunc_binom_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2569
 * ufunc_binom_types[1] = <char>NPY_FLOAT
 * ufunc_binom_types[2] = <char>NPY_FLOAT
 * ufunc_binom_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_binom_types[4] = <char>NPY_DOUBLE
 * ufunc_binom_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2570
 * ufunc_binom_types[2] = <char>NPY_FLOAT
 * ufunc_binom_types[3] = <char>NPY_DOUBLE
 * ufunc_binom_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_binom_types[5] = <char>NPY_DOUBLE
 * ufunc_binom_ptr[2*0] = <void*>_func_binom
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2571
 * ufunc_binom_types[3] = <char>NPY_DOUBLE
 * ufunc_binom_types[4] = <char>NPY_DOUBLE
 * ufunc_binom_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_binom_ptr[2*0] = <void*>_func_binom
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2572
 * ufunc_binom_types[4] = <char>NPY_DOUBLE
 * ufunc_binom_types[5] = <char>NPY_DOUBLE
 * ufunc_binom_ptr[2*0] = <void*>_func_binom             # <<<<<<<<<<<<<<
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")
 * ufunc_binom_ptr[2*1] = <void*>_func_binom
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_binom);

  /* "scipy/special/_ufuncs.pyx":2573
 * ufunc_binom_types[5] = <char>NPY_DOUBLE
 * ufunc_binom_ptr[2*0] = <void*>_func_binom
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")             # <<<<<<<<<<<<<<
 * ufunc_binom_ptr[2*1] = <void*>_func_binom
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[1]) = ((void *)((char *)__pyx_k_binom));

  /* "scipy/special/_ufuncs.pyx":2574
 * ufunc_binom_ptr[2*0] = <void*>_func_binom
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")
 * ufunc_binom_ptr[2*1] = <void*>_func_binom             # <<<<<<<<<<<<<<
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[2]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_binom);

  /* "scipy/special/_ufuncs.pyx":2575
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")
 * ufunc_binom_ptr[2*1] = <void*>_func_binom
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")             # <<<<<<<<<<<<<<
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]
 * ufunc_binom_data[1] = &ufunc_binom_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[3]) = ((void *)((char *)__pyx_k_binom));

  /* "scipy/special/_ufuncs.pyx":2576
 * ufunc_binom_ptr[2*1] = <void*>_func_binom
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_binom_data[1] = &ufunc_binom_ptr[2*1]
 * binom = np.PyUFunc_FromFuncAndData(ufunc_binom_loops, ufunc_binom_data, ufunc_binom_types, 2, 2, 1, 0, "binom", ufunc_binom_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2577
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]
 * ufunc_binom_data[1] = &ufunc_binom_ptr[2*1]             # <<<<<<<<<<<<<<
 * binom = np.PyUFunc_FromFuncAndData(ufunc_binom_loops, ufunc_binom_data, ufunc_binom_types, 2, 2, 1, 0, "binom", ufunc_binom_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2578
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]
 * ufunc_binom_data[1] = &ufunc_binom_ptr[2*1]
 * binom = np.PyUFunc_FromFuncAndData(ufunc_binom_loops, ufunc_binom_data, ufunc_binom_types, 2, 2, 1, 0, "binom", ufunc_binom_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_boxcox_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types, 2, 2, 1, 0, __pyx_k_binom, __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2578; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_binom, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2578; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2584
 * cdef void *ufunc_boxcox_data[2]
 * cdef char ufunc_boxcox_types[6]
 * cdef char *ufunc_boxcox_doc = (             # <<<<<<<<<<<<<<
 *     "boxcox(x, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_doc = __pyx_k_boxcox_x_lmbda_Compute_the_Box_C;

  /* "scipy/special/_ufuncs.pyx":2621
 *     ">>> boxcox(2, [0, 1, 2])\n"
 *     "array([ 0.69314718,  1.        ,  1.5       ])")
 * ufunc_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2622
 *     "array([ 0.69314718,  1.        ,  1.5       ])")
 * ufunc_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2623
 * ufunc_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2624
 * ufunc_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2625
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2626
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2627
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2628
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2629
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox             # <<<<<<<<<<<<<<
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_boxcox);

  /* "scipy/special/_ufuncs.pyx":2630
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")             # <<<<<<<<<<<<<<
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[1]) = ((void *)((char *)__pyx_k_boxcox));

  /* "scipy/special/_ufuncs.pyx":2631
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox             # <<<<<<<<<<<<<<
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_boxcox);

  /* "scipy/special/_ufuncs.pyx":2632
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")             # <<<<<<<<<<<<<<
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]
 * ufunc_boxcox_data[1] = &ufunc_boxcox_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[3]) = ((void *)((char *)__pyx_k_boxcox));

  /* "scipy/special/_ufuncs.pyx":2633
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_boxcox_data[1] = &ufunc_boxcox_ptr[2*1]
 * boxcox = np.PyUFunc_FromFuncAndData(ufunc_boxcox_loops, ufunc_boxcox_data, ufunc_boxcox_types, 2, 2, 1, 0, "boxcox", ufunc_boxcox_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2634
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]
 * ufunc_boxcox_data[1] = &ufunc_boxcox_ptr[2*1]             # <<<<<<<<<<<<<<
 * boxcox = np.PyUFunc_FromFuncAndData(ufunc_boxcox_loops, ufunc_boxcox_data, ufunc_boxcox_types, 2, 2, 1, 0, "boxcox", ufunc_boxcox_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2635
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]
 * ufunc_boxcox_data[1] = &ufunc_boxcox_ptr[2*1]
 * boxcox = np.PyUFunc_FromFuncAndData(ufunc_boxcox_loops, ufunc_boxcox_data, ufunc_boxcox_types, 2, 2, 1, 0, "boxcox", ufunc_boxcox_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_boxcox1p_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types, 2, 2, 1, 0, __pyx_k_boxcox, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_boxcox, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2641
 * cdef void *ufunc_boxcox1p_data[2]
 * cdef char ufunc_boxcox1p_types[6]
 * cdef char *ufunc_boxcox1p_doc = (             # <<<<<<<<<<<<<<
 *     "boxcox1p(x, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_doc = __pyx_k_boxcox1p_x_lmbda_Compute_the_Box;

  /* "scipy/special/_ufuncs.pyx":2678
 *     ">>> boxcox1p([0.01, 0.1], 0.25)\n"
 *     "array([ 0.00996272,  0.09645476])")
 * ufunc_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2679
 *     "array([ 0.00996272,  0.09645476])")
 * ufunc_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2680
 * ufunc_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2681
 * ufunc_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2682
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2683
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2684
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2685
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2686
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":2687
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[1]) = ((void *)((char *)__pyx_k_boxcox1p));

  /* "scipy/special/_ufuncs.pyx":2688
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":2689
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]
 * ufunc_boxcox1p_data[1] = &ufunc_boxcox1p_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[3]) = ((void *)((char *)__pyx_k_boxcox1p));

  /* "scipy/special/_ufuncs.pyx":2690
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_data[1] = &ufunc_boxcox1p_ptr[2*1]
 * boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_boxcox1p_loops, ufunc_boxcox1p_data, ufunc_boxcox1p_types, 2, 2, 1, 0, "boxcox1p", ufunc_boxcox1p_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2691
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]
 * ufunc_boxcox1p_data[1] = &ufunc_boxcox1p_ptr[2*1]             # <<<<<<<<<<<<<<
 * boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_boxcox1p_loops, ufunc_boxcox1p_data, ufunc_boxcox1p_types, 2, 2, 1, 0, "boxcox1p", ufunc_boxcox1p_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2692
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]
 * ufunc_boxcox1p_data[1] = &ufunc_boxcox1p_ptr[2*1]
 * boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_boxcox1p_loops, ufunc_boxcox1p_data, ufunc_boxcox1p_types, 2, 2, 1, 0, "boxcox1p", ufunc_boxcox1p_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_btdtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types, 2, 2, 1, 0, __pyx_k_boxcox1p, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2692; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_boxcox1p, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2692; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2698
 * cdef void *ufunc_btdtr_data[2]
 * cdef char ufunc_btdtr_types[8]
 * cdef char *ufunc_btdtr_doc = (             # <<<<<<<<<<<<<<
 *     "btdtr(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_doc = __pyx_k_btdtr_a_b_x_Cumulative_beta_dist;

  /* "scipy/special/_ufuncs.pyx":2710
 *     "--------\n"
 *     "betainc")
 * ufunc_btdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_btdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2711
 *     "betainc")
 * ufunc_btdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2712
 * ufunc_btdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2713
 * ufunc_btdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2714
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2715
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2716
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2717
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2718
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2719
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2720
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr             # <<<<<<<<<<<<<<
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[0]) = ((void *)btdtr);

  /* "scipy/special/_ufuncs.pyx":2721
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")             # <<<<<<<<<<<<<<
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[1]) = ((void *)((char *)__pyx_k_btdtr));

  /* "scipy/special/_ufuncs.pyx":2722
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr             # <<<<<<<<<<<<<<
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[2]) = ((void *)btdtr);

  /* "scipy/special/_ufuncs.pyx":2723
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")             # <<<<<<<<<<<<<<
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]
 * ufunc_btdtr_data[1] = &ufunc_btdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[3]) = ((void *)((char *)__pyx_k_btdtr));

  /* "scipy/special/_ufuncs.pyx":2724
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_btdtr_data[1] = &ufunc_btdtr_ptr[2*1]
 * btdtr = np.PyUFunc_FromFuncAndData(ufunc_btdtr_loops, ufunc_btdtr_data, ufunc_btdtr_types, 2, 3, 1, 0, "btdtr", ufunc_btdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2725
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]
 * ufunc_btdtr_data[1] = &ufunc_btdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * btdtr = np.PyUFunc_FromFuncAndData(ufunc_btdtr_loops, ufunc_btdtr_data, ufunc_btdtr_types, 2, 3, 1, 0, "btdtr", ufunc_btdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2726
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]
 * ufunc_btdtr_data[1] = &ufunc_btdtr_ptr[2*1]
 * btdtr = np.PyUFunc_FromFuncAndData(ufunc_btdtr_loops, ufunc_btdtr_data, ufunc_btdtr_types, 2, 3, 1, 0, "btdtr", ufunc_btdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_btdtri_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types, 2, 3, 1, 0, __pyx_k_btdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2726; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_btdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2726; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2732
 * cdef void *ufunc_btdtri_data[2]
 * cdef char ufunc_btdtri_types[8]
 * cdef char *ufunc_btdtri_doc = (             # <<<<<<<<<<<<<<
 *     "btdtri(a, b, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_doc = __pyx_k_btdtri_a_b_p_p_th_quantile_of_th;

  /* "scipy/special/_ufuncs.pyx":2743
 *     "--------\n"
 *     "betaincinv")
 * ufunc_btdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_btdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2744
 *     "betaincinv")
 * ufunc_btdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2745
 * ufunc_btdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2746
 * ufunc_btdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2747
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2748
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2749
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2750
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2751
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2752
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2753
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi             # <<<<<<<<<<<<<<
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[0]) = ((void *)incbi);

  /* "scipy/special/_ufuncs.pyx":2754
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")             # <<<<<<<<<<<<<<
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[1]) = ((void *)((char *)__pyx_k_btdtri));

  /* "scipy/special/_ufuncs.pyx":2755
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi             # <<<<<<<<<<<<<<
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[2]) = ((void *)incbi);

  /* "scipy/special/_ufuncs.pyx":2756
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")             # <<<<<<<<<<<<<<
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]
 * ufunc_btdtri_data[1] = &ufunc_btdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[3]) = ((void *)((char *)__pyx_k_btdtri));

  /* "scipy/special/_ufuncs.pyx":2757
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_btdtri_data[1] = &ufunc_btdtri_ptr[2*1]
 * btdtri = np.PyUFunc_FromFuncAndData(ufunc_btdtri_loops, ufunc_btdtri_data, ufunc_btdtri_types, 2, 3, 1, 0, "btdtri", ufunc_btdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2758
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]
 * ufunc_btdtri_data[1] = &ufunc_btdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * btdtri = np.PyUFunc_FromFuncAndData(ufunc_btdtri_loops, ufunc_btdtri_data, ufunc_btdtri_types, 2, 3, 1, 0, "btdtri", ufunc_btdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2759
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]
 * ufunc_btdtri_data[1] = &ufunc_btdtri_ptr[2*1]
 * btdtri = np.PyUFunc_FromFuncAndData(ufunc_btdtri_loops, ufunc_btdtri_data, ufunc_btdtri_types, 2, 3, 1, 0, "btdtri", ufunc_btdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_btdtria_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types, 2, 3, 1, 0, __pyx_k_btdtri, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_btdtri, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2765
 * cdef void *ufunc_btdtria_data[2]
 * cdef char ufunc_btdtria_types[8]
 * cdef char *ufunc_btdtria_doc = (             # <<<<<<<<<<<<<<
 *     "btdtria(p, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_doc = __pyx_k_btdtria_p_b_x_Inverse_of_btdtr_v;

  /* "scipy/special/_ufuncs.pyx":2769
 *     "\n"
 *     "Inverse of `btdtr` vs `a`")
 * ufunc_btdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_btdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2770
 *     "Inverse of `btdtr` vs `a`")
 * ufunc_btdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2771
 * ufunc_btdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2772
 * ufunc_btdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2773
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2774
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2775
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2776
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2777
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2778
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2779
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap             # <<<<<<<<<<<<<<
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[0]) = ((void *)cdfbet3_wrap);

  /* "scipy/special/_ufuncs.pyx":2780
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")             # <<<<<<<<<<<<<<
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[1]) = ((void *)((char *)__pyx_k_btdtria));

  /* "scipy/special/_ufuncs.pyx":2781
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap             # <<<<<<<<<<<<<<
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[2]) = ((void *)cdfbet3_wrap);

  /* "scipy/special/_ufuncs.pyx":2782
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")             # <<<<<<<<<<<<<<
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]
 * ufunc_btdtria_data[1] = &ufunc_btdtria_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[3]) = ((void *)((char *)__pyx_k_btdtria));

  /* "scipy/special/_ufuncs.pyx":2783
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_btdtria_data[1] = &ufunc_btdtria_ptr[2*1]
 * btdtria = np.PyUFunc_FromFuncAndData(ufunc_btdtria_loops, ufunc_btdtria_data, ufunc_btdtria_types, 2, 3, 1, 0, "btdtria", ufunc_btdtria_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2784
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]
 * ufunc_btdtria_data[1] = &ufunc_btdtria_ptr[2*1]             # <<<<<<<<<<<<<<
 * btdtria = np.PyUFunc_FromFuncAndData(ufunc_btdtria_loops, ufunc_btdtria_data, ufunc_btdtria_types, 2, 3, 1, 0, "btdtria", ufunc_btdtria_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2785
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]
 * ufunc_btdtria_data[1] = &ufunc_btdtria_ptr[2*1]
 * btdtria = np.PyUFunc_FromFuncAndData(ufunc_btdtria_loops, ufunc_btdtria_data, ufunc_btdtria_types, 2, 3, 1, 0, "btdtria", ufunc_btdtria_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_btdtrib_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types, 2, 3, 1, 0, __pyx_k_btdtria, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2785; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_btdtria, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2785; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2791
 * cdef void *ufunc_btdtrib_data[2]
 * cdef char ufunc_btdtrib_types[8]
 * cdef char *ufunc_btdtrib_doc = (             # <<<<<<<<<<<<<<
 *     "btdtria(a, p, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_doc = __pyx_k_btdtria_a_p_x_Inverse_of_btdtr_v;

  /* "scipy/special/_ufuncs.pyx":2795
 *     "\n"
 *     "Inverse of `btdtr` vs `b`")
 * ufunc_btdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2796
 *     "Inverse of `btdtr` vs `b`")
 * ufunc_btdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2797
 * ufunc_btdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2798
 * ufunc_btdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2799
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2800
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2801
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2802
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2803
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2804
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2805
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[0]) = ((void *)cdfbet4_wrap);

  /* "scipy/special/_ufuncs.pyx":2806
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[1]) = ((void *)((char *)__pyx_k_btdtrib));

  /* "scipy/special/_ufuncs.pyx":2807
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[2]) = ((void *)cdfbet4_wrap);

  /* "scipy/special/_ufuncs.pyx":2808
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]
 * ufunc_btdtrib_data[1] = &ufunc_btdtrib_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[3]) = ((void *)((char *)__pyx_k_btdtrib));

  /* "scipy/special/_ufuncs.pyx":2809
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_data[1] = &ufunc_btdtrib_ptr[2*1]
 * btdtrib = np.PyUFunc_FromFuncAndData(ufunc_btdtrib_loops, ufunc_btdtrib_data, ufunc_btdtrib_types, 2, 3, 1, 0, "btdtrib", ufunc_btdtrib_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2810
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]
 * ufunc_btdtrib_data[1] = &ufunc_btdtrib_ptr[2*1]             # <<<<<<<<<<<<<<
 * btdtrib = np.PyUFunc_FromFuncAndData(ufunc_btdtrib_loops, ufunc_btdtrib_data, ufunc_btdtrib_types, 2, 3, 1, 0, "btdtrib", ufunc_btdtrib_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2811
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]
 * ufunc_btdtrib_data[1] = &ufunc_btdtrib_ptr[2*1]
 * btdtrib = np.PyUFunc_FromFuncAndData(ufunc_btdtrib_loops, ufunc_btdtrib_data, ufunc_btdtrib_types, 2, 3, 1, 0, "btdtrib", ufunc_btdtrib_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_cbrt_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types, 2, 3, 1, 0, __pyx_k_btdtrib, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_btdtrib, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2817
 * cdef void *ufunc_cbrt_data[2]
 * cdef char ufunc_cbrt_types[4]
 * cdef char *ufunc_cbrt_doc = (             # <<<<<<<<<<<<<<
 *     "cbrt(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_doc = __pyx_k_cbrt_x_Cube_root_of_x;

  /* "scipy/special/_ufuncs.pyx":2821
 *     "\n"
 *     "Cube root of `x`")
 * ufunc_cbrt_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_cbrt_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2822
 *     "Cube root of `x`")
 * ufunc_cbrt_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cbrt_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2823
 * ufunc_cbrt_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cbrt_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2824
 * ufunc_cbrt_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2825
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2826
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2827
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt             # <<<<<<<<<<<<<<
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[0]) = ((void *)cbrt);

  /* "scipy/special/_ufuncs.pyx":2828
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")             # <<<<<<<<<<<<<<
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[1]) = ((void *)((char *)__pyx_k_cbrt));

  /* "scipy/special/_ufuncs.pyx":2829
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt             # <<<<<<<<<<<<<<
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[2]) = ((void *)cbrt);

  /* "scipy/special/_ufuncs.pyx":2830
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")             # <<<<<<<<<<<<<<
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]
 * ufunc_cbrt_data[1] = &ufunc_cbrt_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[3]) = ((void *)((char *)__pyx_k_cbrt));

  /* "scipy/special/_ufuncs.pyx":2831
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_cbrt_data[1] = &ufunc_cbrt_ptr[2*1]
 * cbrt = np.PyUFunc_FromFuncAndData(ufunc_cbrt_loops, ufunc_cbrt_data, ufunc_cbrt_types, 2, 1, 1, 0, "cbrt", ufunc_cbrt_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2832
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]
 * ufunc_cbrt_data[1] = &ufunc_cbrt_ptr[2*1]             # <<<<<<<<<<<<<<
 * cbrt = np.PyUFunc_FromFuncAndData(ufunc_cbrt_loops, ufunc_cbrt_data, ufunc_cbrt_types, 2, 1, 1, 0, "cbrt", ufunc_cbrt_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2833
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]
 * ufunc_cbrt_data[1] = &ufunc_cbrt_ptr[2*1]
 * cbrt = np.PyUFunc_FromFuncAndData(ufunc_cbrt_loops, ufunc_cbrt_data, ufunc_cbrt_types, 2, 1, 1, 0, "cbrt", ufunc_cbrt_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chdtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types, 2, 1, 1, 0, __pyx_k_cbrt, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cbrt, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2839
 * cdef void *ufunc_chdtr_data[2]
 * cdef char ufunc_chdtr_types[6]
 * cdef char *ufunc_chdtr_doc = (             # <<<<<<<<<<<<<<
 *     "chdtr(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_doc = __pyx_k_chdtr_v_x_Chi_square_cumulative;

  /* "scipy/special/_ufuncs.pyx":2848
 *     "\n"
 *     "    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=0..x)")
 * ufunc_chdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_chdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2849
 *     "    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=0..x)")
 * ufunc_chdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2850
 * ufunc_chdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2851
 * ufunc_chdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2852
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2853
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2854
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2855
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2856
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr             # <<<<<<<<<<<<<<
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[0]) = ((void *)chdtr);

  /* "scipy/special/_ufuncs.pyx":2857
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")             # <<<<<<<<<<<<<<
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[1]) = ((void *)((char *)__pyx_k_chdtr));

  /* "scipy/special/_ufuncs.pyx":2858
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr             # <<<<<<<<<<<<<<
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[2]) = ((void *)chdtr);

  /* "scipy/special/_ufuncs.pyx":2859
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")             # <<<<<<<<<<<<<<
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]
 * ufunc_chdtr_data[1] = &ufunc_chdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[3]) = ((void *)((char *)__pyx_k_chdtr));

  /* "scipy/special/_ufuncs.pyx":2860
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chdtr_data[1] = &ufunc_chdtr_ptr[2*1]
 * chdtr = np.PyUFunc_FromFuncAndData(ufunc_chdtr_loops, ufunc_chdtr_data, ufunc_chdtr_types, 2, 2, 1, 0, "chdtr", ufunc_chdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2861
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]
 * ufunc_chdtr_data[1] = &ufunc_chdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * chdtr = np.PyUFunc_FromFuncAndData(ufunc_chdtr_loops, ufunc_chdtr_data, ufunc_chdtr_types, 2, 2, 1, 0, "chdtr", ufunc_chdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2862
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]
 * ufunc_chdtr_data[1] = &ufunc_chdtr_ptr[2*1]
 * chdtr = np.PyUFunc_FromFuncAndData(ufunc_chdtr_loops, ufunc_chdtr_data, ufunc_chdtr_types, 2, 2, 1, 0, "chdtr", ufunc_chdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chdtrc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types, 2, 2, 1, 0, __pyx_k_chdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2862; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2862; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2868
 * cdef void *ufunc_chdtrc_data[2]
 * cdef char ufunc_chdtrc_types[6]
 * cdef char *ufunc_chdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "chdtrc(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_doc = __pyx_k_chdtrc_v_x_Chi_square_survival_f;

  /* "scipy/special/_ufuncs.pyx":2878
 *     "\n"
 *     "    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=x..inf)")
 * ufunc_chdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2879
 *     "    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=x..inf)")
 * ufunc_chdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2880
 * ufunc_chdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2881
 * ufunc_chdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2882
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2883
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2884
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2885
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2886
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[0]) = ((void *)chdtrc);

  /* "scipy/special/_ufuncs.pyx":2887
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[1]) = ((void *)((char *)__pyx_k_chdtrc));

  /* "scipy/special/_ufuncs.pyx":2888
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[2]) = ((void *)chdtrc);

  /* "scipy/special/_ufuncs.pyx":2889
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]
 * ufunc_chdtrc_data[1] = &ufunc_chdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[3]) = ((void *)((char *)__pyx_k_chdtrc));

  /* "scipy/special/_ufuncs.pyx":2890
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_data[1] = &ufunc_chdtrc_ptr[2*1]
 * chdtrc = np.PyUFunc_FromFuncAndData(ufunc_chdtrc_loops, ufunc_chdtrc_data, ufunc_chdtrc_types, 2, 2, 1, 0, "chdtrc", ufunc_chdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2891
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]
 * ufunc_chdtrc_data[1] = &ufunc_chdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * chdtrc = np.PyUFunc_FromFuncAndData(ufunc_chdtrc_loops, ufunc_chdtrc_data, ufunc_chdtrc_types, 2, 2, 1, 0, "chdtrc", ufunc_chdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2892
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]
 * ufunc_chdtrc_data[1] = &ufunc_chdtrc_ptr[2*1]
 * chdtrc = np.PyUFunc_FromFuncAndData(ufunc_chdtrc_loops, ufunc_chdtrc_data, ufunc_chdtrc_types, 2, 2, 1, 0, "chdtrc", ufunc_chdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chdtri_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types, 2, 2, 1, 0, __pyx_k_chdtrc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2892; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chdtrc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2892; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2898
 * cdef void *ufunc_chdtri_data[2]
 * cdef char ufunc_chdtri_types[6]
 * cdef char *ufunc_chdtri_doc = (             # <<<<<<<<<<<<<<
 *     "chdtri(v, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_doc = __pyx_k_chdtri_v_p_Inverse_to_chdtrc_Ret;

  /* "scipy/special/_ufuncs.pyx":2904
 *     "\n"
 *     "Returns the argument x such that ``chdtrc(v, x) == p``.")
 * ufunc_chdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_chdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2905
 *     "Returns the argument x such that ``chdtrc(v, x) == p``.")
 * ufunc_chdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2906
 * ufunc_chdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2907
 * ufunc_chdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2908
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2909
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2910
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2911
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2912
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri             # <<<<<<<<<<<<<<
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[0]) = ((void *)chdtri);

  /* "scipy/special/_ufuncs.pyx":2913
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")             # <<<<<<<<<<<<<<
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[1]) = ((void *)((char *)__pyx_k_chdtri));

  /* "scipy/special/_ufuncs.pyx":2914
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri             # <<<<<<<<<<<<<<
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[2]) = ((void *)chdtri);

  /* "scipy/special/_ufuncs.pyx":2915
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")             # <<<<<<<<<<<<<<
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]
 * ufunc_chdtri_data[1] = &ufunc_chdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[3]) = ((void *)((char *)__pyx_k_chdtri));

  /* "scipy/special/_ufuncs.pyx":2916
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chdtri_data[1] = &ufunc_chdtri_ptr[2*1]
 * chdtri = np.PyUFunc_FromFuncAndData(ufunc_chdtri_loops, ufunc_chdtri_data, ufunc_chdtri_types, 2, 2, 1, 0, "chdtri", ufunc_chdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2917
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]
 * ufunc_chdtri_data[1] = &ufunc_chdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * chdtri = np.PyUFunc_FromFuncAndData(ufunc_chdtri_loops, ufunc_chdtri_data, ufunc_chdtri_types, 2, 2, 1, 0, "chdtri", ufunc_chdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2918
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]
 * ufunc_chdtri_data[1] = &ufunc_chdtri_ptr[2*1]
 * chdtri = np.PyUFunc_FromFuncAndData(ufunc_chdtri_loops, ufunc_chdtri_data, ufunc_chdtri_types, 2, 2, 1, 0, "chdtri", ufunc_chdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chdtriv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types, 2, 2, 1, 0, __pyx_k_chdtri, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chdtri, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2924
 * cdef void *ufunc_chdtriv_data[2]
 * cdef char ufunc_chdtriv_types[6]
 * cdef char *ufunc_chdtriv_doc = (             # <<<<<<<<<<<<<<
 *     "chdtri(p, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_doc = __pyx_k_chdtri_p_x_Inverse_to_chdtr_vs_v;

  /* "scipy/special/_ufuncs.pyx":2930
 *     "\n"
 *     "Returns the argument v such that ``chdtr(v, x) == p``.")
 * ufunc_chdtriv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2931
 *     "Returns the argument v such that ``chdtr(v, x) == p``.")
 * ufunc_chdtriv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtriv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2932
 * ufunc_chdtriv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtriv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2933
 * ufunc_chdtriv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2934
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2935
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2936
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2937
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2938
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[0]) = ((void *)cdfchi3_wrap);

  /* "scipy/special/_ufuncs.pyx":2939
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[1]) = ((void *)((char *)__pyx_k_chdtriv));

  /* "scipy/special/_ufuncs.pyx":2940
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[2]) = ((void *)cdfchi3_wrap);

  /* "scipy/special/_ufuncs.pyx":2941
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]
 * ufunc_chdtriv_data[1] = &ufunc_chdtriv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[3]) = ((void *)((char *)__pyx_k_chdtriv));

  /* "scipy/special/_ufuncs.pyx":2942
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_data[1] = &ufunc_chdtriv_ptr[2*1]
 * chdtriv = np.PyUFunc_FromFuncAndData(ufunc_chdtriv_loops, ufunc_chdtriv_data, ufunc_chdtriv_types, 2, 2, 1, 0, "chdtriv", ufunc_chdtriv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2943
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]
 * ufunc_chdtriv_data[1] = &ufunc_chdtriv_ptr[2*1]             # <<<<<<<<<<<<<<
 * chdtriv = np.PyUFunc_FromFuncAndData(ufunc_chdtriv_loops, ufunc_chdtriv_data, ufunc_chdtriv_types, 2, 2, 1, 0, "chdtriv", ufunc_chdtriv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2944
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]
 * ufunc_chdtriv_data[1] = &ufunc_chdtriv_ptr[2*1]
 * chdtriv = np.PyUFunc_FromFuncAndData(ufunc_chdtriv_loops, ufunc_chdtriv_data, ufunc_chdtriv_types, 2, 2, 1, 0, "chdtriv", ufunc_chdtriv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chndtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types, 2, 2, 1, 0, __pyx_k_chdtriv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chdtriv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2950
 * cdef void *ufunc_chndtr_data[2]
 * cdef char ufunc_chndtr_types[8]
 * cdef char *ufunc_chndtr_doc = (             # <<<<<<<<<<<<<<
 *     "chndtr(x, df, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_doc = __pyx_k_chndtr_x_df_nc_Non_central_chi_s;

  /* "scipy/special/_ufuncs.pyx":2954
 *     "\n"
 *     "Non-central chi square cumulative distribution function")
 * ufunc_chndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_chndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2955
 *     "Non-central chi square cumulative distribution function")
 * ufunc_chndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2956
 * ufunc_chndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2957
 * ufunc_chndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2958
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2959
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2960
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2961
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2962
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2963
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2964
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[0]) = ((void *)cdfchn1_wrap);

  /* "scipy/special/_ufuncs.pyx":2965
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")             # <<<<<<<<<<<<<<
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[1]) = ((void *)((char *)__pyx_k_chndtr));

  /* "scipy/special/_ufuncs.pyx":2966
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[2]) = ((void *)cdfchn1_wrap);

  /* "scipy/special/_ufuncs.pyx":2967
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")             # <<<<<<<<<<<<<<
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]
 * ufunc_chndtr_data[1] = &ufunc_chndtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[3]) = ((void *)((char *)__pyx_k_chndtr));

  /* "scipy/special/_ufuncs.pyx":2968
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chndtr_data[1] = &ufunc_chndtr_ptr[2*1]
 * chndtr = np.PyUFunc_FromFuncAndData(ufunc_chndtr_loops, ufunc_chndtr_data, ufunc_chndtr_types, 2, 3, 1, 0, "chndtr", ufunc_chndtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2969
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]
 * ufunc_chndtr_data[1] = &ufunc_chndtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * chndtr = np.PyUFunc_FromFuncAndData(ufunc_chndtr_loops, ufunc_chndtr_data, ufunc_chndtr_types, 2, 3, 1, 0, "chndtr", ufunc_chndtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2970
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]
 * ufunc_chndtr_data[1] = &ufunc_chndtr_ptr[2*1]
 * chndtr = np.PyUFunc_FromFuncAndData(ufunc_chndtr_loops, ufunc_chndtr_data, ufunc_chndtr_types, 2, 3, 1, 0, "chndtr", ufunc_chndtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chndtridf_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types, 2, 3, 1, 0, __pyx_k_chndtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2970; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chndtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2970; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":2976
 * cdef void *ufunc_chndtridf_data[2]
 * cdef char ufunc_chndtridf_types[8]
 * cdef char *ufunc_chndtridf_doc = (             # <<<<<<<<<<<<<<
 *     "chndtridf(x, p, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_doc = __pyx_k_chndtridf_x_p_nc_Inverse_to_chnd;

  /* "scipy/special/_ufuncs.pyx":2980
 *     "\n"
 *     "Inverse to `chndtr` vs `df`")
 * ufunc_chndtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2981
 *     "Inverse to `chndtr` vs `df`")
 * ufunc_chndtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2982
 * ufunc_chndtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2983
 * ufunc_chndtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2984
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2985
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2986
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2987
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2988
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2989
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2990
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[0]) = ((void *)cdfchn3_wrap);

  /* "scipy/special/_ufuncs.pyx":2991
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[1]) = ((void *)((char *)__pyx_k_chndtridf));

  /* "scipy/special/_ufuncs.pyx":2992
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[2]) = ((void *)cdfchn3_wrap);

  /* "scipy/special/_ufuncs.pyx":2993
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]
 * ufunc_chndtridf_data[1] = &ufunc_chndtridf_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[3]) = ((void *)((char *)__pyx_k_chndtridf));

  /* "scipy/special/_ufuncs.pyx":2994
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_data[1] = &ufunc_chndtridf_ptr[2*1]
 * chndtridf = np.PyUFunc_FromFuncAndData(ufunc_chndtridf_loops, ufunc_chndtridf_data, ufunc_chndtridf_types, 2, 3, 1, 0, "chndtridf", ufunc_chndtridf_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2995
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]
 * ufunc_chndtridf_data[1] = &ufunc_chndtridf_ptr[2*1]             # <<<<<<<<<<<<<<
 * chndtridf = np.PyUFunc_FromFuncAndData(ufunc_chndtridf_loops, ufunc_chndtridf_data, ufunc_chndtridf_types, 2, 3, 1, 0, "chndtridf", ufunc_chndtridf_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2996
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]
 * ufunc_chndtridf_data[1] = &ufunc_chndtridf_ptr[2*1]
 * chndtridf = np.PyUFunc_FromFuncAndData(ufunc_chndtridf_loops, ufunc_chndtridf_data, ufunc_chndtridf_types, 2, 3, 1, 0, "chndtridf", ufunc_chndtridf_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chndtrinc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types, 2, 3, 1, 0, __pyx_k_chndtridf, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2996; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chndtridf, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2996; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3002
 * cdef void *ufunc_chndtrinc_data[2]
 * cdef char ufunc_chndtrinc_types[8]
 * cdef char *ufunc_chndtrinc_doc = (             # <<<<<<<<<<<<<<
 *     "chndtrinc(x, df, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_doc = __pyx_k_chndtrinc_x_df_p_Inverse_to_chnd;

  /* "scipy/special/_ufuncs.pyx":3006
 *     "\n"
 *     "Inverse to `chndtr` vs `nc`")
 * ufunc_chndtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3007
 *     "Inverse to `chndtr` vs `nc`")
 * ufunc_chndtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3008
 * ufunc_chndtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3009
 * ufunc_chndtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3010
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3011
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3012
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3013
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3014
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3015
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3016
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[0]) = ((void *)cdfchn4_wrap);

  /* "scipy/special/_ufuncs.pyx":3017
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[1]) = ((void *)((char *)__pyx_k_chndtrinc));

  /* "scipy/special/_ufuncs.pyx":3018
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[2]) = ((void *)cdfchn4_wrap);

  /* "scipy/special/_ufuncs.pyx":3019
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]
 * ufunc_chndtrinc_data[1] = &ufunc_chndtrinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[3]) = ((void *)((char *)__pyx_k_chndtrinc));

  /* "scipy/special/_ufuncs.pyx":3020
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_data[1] = &ufunc_chndtrinc_ptr[2*1]
 * chndtrinc = np.PyUFunc_FromFuncAndData(ufunc_chndtrinc_loops, ufunc_chndtrinc_data, ufunc_chndtrinc_types, 2, 3, 1, 0, "chndtrinc", ufunc_chndtrinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3021
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]
 * ufunc_chndtrinc_data[1] = &ufunc_chndtrinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * chndtrinc = np.PyUFunc_FromFuncAndData(ufunc_chndtrinc_loops, ufunc_chndtrinc_data, ufunc_chndtrinc_types, 2, 3, 1, 0, "chndtrinc", ufunc_chndtrinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3022
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]
 * ufunc_chndtrinc_data[1] = &ufunc_chndtrinc_ptr[2*1]
 * chndtrinc = np.PyUFunc_FromFuncAndData(ufunc_chndtrinc_loops, ufunc_chndtrinc_data, ufunc_chndtrinc_types, 2, 3, 1, 0, "chndtrinc", ufunc_chndtrinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chndtrix_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types, 2, 3, 1, 0, __pyx_k_chndtrinc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chndtrinc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3028
 * cdef void *ufunc_chndtrix_data[2]
 * cdef char ufunc_chndtrix_types[8]
 * cdef char *ufunc_chndtrix_doc = (             # <<<<<<<<<<<<<<
 *     "chndtrix(p, df, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_doc = __pyx_k_chndtrix_p_df_nc_Inverse_to_chnd;

  /* "scipy/special/_ufuncs.pyx":3032
 *     "\n"
 *     "Inverse to `chndtr` vs `x`")
 * ufunc_chndtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3033
 *     "Inverse to `chndtr` vs `x`")
 * ufunc_chndtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3034
 * ufunc_chndtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3035
 * ufunc_chndtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3036
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3037
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3038
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3039
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3040
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3041
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3042
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[0]) = ((void *)cdfchn2_wrap);

  /* "scipy/special/_ufuncs.pyx":3043
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[1]) = ((void *)((char *)__pyx_k_chndtrix));

  /* "scipy/special/_ufuncs.pyx":3044
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[2]) = ((void *)cdfchn2_wrap);

  /* "scipy/special/_ufuncs.pyx":3045
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]
 * ufunc_chndtrix_data[1] = &ufunc_chndtrix_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[3]) = ((void *)((char *)__pyx_k_chndtrix));

  /* "scipy/special/_ufuncs.pyx":3046
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_data[1] = &ufunc_chndtrix_ptr[2*1]
 * chndtrix = np.PyUFunc_FromFuncAndData(ufunc_chndtrix_loops, ufunc_chndtrix_data, ufunc_chndtrix_types, 2, 3, 1, 0, "chndtrix", ufunc_chndtrix_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3047
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]
 * ufunc_chndtrix_data[1] = &ufunc_chndtrix_ptr[2*1]             # <<<<<<<<<<<<<<
 * chndtrix = np.PyUFunc_FromFuncAndData(ufunc_chndtrix_loops, ufunc_chndtrix_data, ufunc_chndtrix_types, 2, 3, 1, 0, "chndtrix", ufunc_chndtrix_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3048
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]
 * ufunc_chndtrix_data[1] = &ufunc_chndtrix_ptr[2*1]
 * chndtrix = np.PyUFunc_FromFuncAndData(ufunc_chndtrix_loops, ufunc_chndtrix_data, ufunc_chndtrix_types, 2, 3, 1, 0, "chndtrix", ufunc_chndtrix_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_cosdg_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types, 2, 3, 1, 0, __pyx_k_chndtrix, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chndtrix, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3054
 * cdef void *ufunc_cosdg_data[2]
 * cdef char ufunc_cosdg_types[4]
 * cdef char *ufunc_cosdg_doc = (             # <<<<<<<<<<<<<<
 *     "cosdg(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_doc = __pyx_k_cosdg_x_Cosine_of_the_angle_x_gi;

  /* "scipy/special/_ufuncs.pyx":3058
 *     "\n"
 *     "Cosine of the angle `x` given in degrees.")
 * ufunc_cosdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_cosdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3059
 *     "Cosine of the angle `x` given in degrees.")
 * ufunc_cosdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cosdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3060
 * ufunc_cosdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cosdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3061
 * ufunc_cosdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3062
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3063
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3064
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg             # <<<<<<<<<<<<<<
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[0]) = ((void *)cosdg);

  /* "scipy/special/_ufuncs.pyx":3065
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")             # <<<<<<<<<<<<<<
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[1]) = ((void *)((char *)__pyx_k_cosdg));

  /* "scipy/special/_ufuncs.pyx":3066
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg             # <<<<<<<<<<<<<<
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[2]) = ((void *)cosdg);

  /* "scipy/special/_ufuncs.pyx":3067
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")             # <<<<<<<<<<<<<<
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]
 * ufunc_cosdg_data[1] = &ufunc_cosdg_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[3]) = ((void *)((char *)__pyx_k_cosdg));

  /* "scipy/special/_ufuncs.pyx":3068
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_cosdg_data[1] = &ufunc_cosdg_ptr[2*1]
 * cosdg = np.PyUFunc_FromFuncAndData(ufunc_cosdg_loops, ufunc_cosdg_data, ufunc_cosdg_types, 2, 1, 1, 0, "cosdg", ufunc_cosdg_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3069
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]
 * ufunc_cosdg_data[1] = &ufunc_cosdg_ptr[2*1]             # <<<<<<<<<<<<<<
 * cosdg = np.PyUFunc_FromFuncAndData(ufunc_cosdg_loops, ufunc_cosdg_data, ufunc_cosdg_types, 2, 1, 1, 0, "cosdg", ufunc_cosdg_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3070
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]
 * ufunc_cosdg_data[1] = &ufunc_cosdg_ptr[2*1]
 * cosdg = np.PyUFunc_FromFuncAndData(ufunc_cosdg_loops, ufunc_cosdg_data, ufunc_cosdg_types, 2, 1, 1, 0, "cosdg", ufunc_cosdg_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_cosm1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types, 2, 1, 1, 0, __pyx_k_cosdg, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cosdg, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3076
 * cdef void *ufunc_cosm1_data[2]
 * cdef char ufunc_cosm1_types[4]
 * cdef char *ufunc_cosm1_doc = (             # <<<<<<<<<<<<<<
 *     "cosm1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_doc = __pyx_k_cosm1_x_cos_x_1_for_use_when_x_i;

  /* "scipy/special/_ufuncs.pyx":3080
 *     "\n"
 *     "cos(x) - 1 for use when `x` is near zero.")
 * ufunc_cosm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_cosm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3081
 *     "cos(x) - 1 for use when `x` is near zero.")
 * ufunc_cosm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cosm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3082
 * ufunc_cosm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cosm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3083
 * ufunc_cosm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3084
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3085
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3086
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1             # <<<<<<<<<<<<<<
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[0]) = ((void *)cosm1);

  /* "scipy/special/_ufuncs.pyx":3087
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")             # <<<<<<<<<<<<<<
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[1]) = ((void *)((char *)__pyx_k_cosm1));

  /* "scipy/special/_ufuncs.pyx":3088
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1             # <<<<<<<<<<<<<<
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[2]) = ((void *)cosm1);

  /* "scipy/special/_ufuncs.pyx":3089
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")             # <<<<<<<<<<<<<<
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]
 * ufunc_cosm1_data[1] = &ufunc_cosm1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[3]) = ((void *)((char *)__pyx_k_cosm1));

  /* "scipy/special/_ufuncs.pyx":3090
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_cosm1_data[1] = &ufunc_cosm1_ptr[2*1]
 * cosm1 = np.PyUFunc_FromFuncAndData(ufunc_cosm1_loops, ufunc_cosm1_data, ufunc_cosm1_types, 2, 1, 1, 0, "cosm1", ufunc_cosm1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3091
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]
 * ufunc_cosm1_data[1] = &ufunc_cosm1_ptr[2*1]             # <<<<<<<<<<<<<<
 * cosm1 = np.PyUFunc_FromFuncAndData(ufunc_cosm1_loops, ufunc_cosm1_data, ufunc_cosm1_types, 2, 1, 1, 0, "cosm1", ufunc_cosm1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3092
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]
 * ufunc_cosm1_data[1] = &ufunc_cosm1_ptr[2*1]
 * cosm1 = np.PyUFunc_FromFuncAndData(ufunc_cosm1_loops, ufunc_cosm1_data, ufunc_cosm1_types, 2, 1, 1, 0, "cosm1", ufunc_cosm1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_cotdg_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types, 2, 1, 1, 0, __pyx_k_cosm1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3092; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cosm1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3092; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3098
 * cdef void *ufunc_cotdg_data[2]
 * cdef char ufunc_cotdg_types[4]
 * cdef char *ufunc_cotdg_doc = (             # <<<<<<<<<<<<<<
 *     "cotdg(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_doc = __pyx_k_cotdg_x_Cotangent_of_the_angle_x;

  /* "scipy/special/_ufuncs.pyx":3102
 *     "\n"
 *     "Cotangent of the angle `x` given in degrees.")
 * ufunc_cotdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_cotdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3103
 *     "Cotangent of the angle `x` given in degrees.")
 * ufunc_cotdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cotdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3104
 * ufunc_cotdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cotdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3105
 * ufunc_cotdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3106
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3107
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3108
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg             # <<<<<<<<<<<<<<
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[0]) = ((void *)cotdg);

  /* "scipy/special/_ufuncs.pyx":3109
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")             # <<<<<<<<<<<<<<
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[1]) = ((void *)((char *)__pyx_k_cotdg));

  /* "scipy/special/_ufuncs.pyx":3110
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg             # <<<<<<<<<<<<<<
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[2]) = ((void *)cotdg);

  /* "scipy/special/_ufuncs.pyx":3111
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")             # <<<<<<<<<<<<<<
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]
 * ufunc_cotdg_data[1] = &ufunc_cotdg_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[3]) = ((void *)((char *)__pyx_k_cotdg));

  /* "scipy/special/_ufuncs.pyx":3112
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_cotdg_data[1] = &ufunc_cotdg_ptr[2*1]
 * cotdg = np.PyUFunc_FromFuncAndData(ufunc_cotdg_loops, ufunc_cotdg_data, ufunc_cotdg_types, 2, 1, 1, 0, "cotdg", ufunc_cotdg_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3113
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]
 * ufunc_cotdg_data[1] = &ufunc_cotdg_ptr[2*1]             # <<<<<<<<<<<<<<
 * cotdg = np.PyUFunc_FromFuncAndData(ufunc_cotdg_loops, ufunc_cotdg_data, ufunc_cotdg_types, 2, 1, 1, 0, "cotdg", ufunc_cotdg_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3114
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]
 * ufunc_cotdg_data[1] = &ufunc_cotdg_ptr[2*1]
 * cotdg = np.PyUFunc_FromFuncAndData(ufunc_cotdg_loops, ufunc_cotdg_data, ufunc_cotdg_types, 2, 1, 1, 0, "cotdg", ufunc_cotdg_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_dawsn_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types, 2, 1, 1, 0, __pyx_k_cotdg, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cotdg, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3120
 * cdef void *ufunc_dawsn_data[4]
 * cdef char ufunc_dawsn_types[8]
 * cdef char *ufunc_dawsn_doc = (             # <<<<<<<<<<<<<<
 *     "dawsn(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_doc = __pyx_k_dawsn_x_Dawson_s_integral_Comput;

  /* "scipy/special/_ufuncs.pyx":3133
 *     ".. [1] Steven G. Johnson, Faddeeva W function implementation.\n"
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_dawsn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_dawsn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3134
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_dawsn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_dawsn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3135
 * ufunc_dawsn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_dawsn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":3136
 * ufunc_dawsn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":3137
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3138
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3139
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3140
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3141
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3142
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3143
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3144
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3145
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn);

  /* "scipy/special/_ufuncs.pyx":3146
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[1]) = ((void *)((char *)__pyx_k_dawsn));

  /* "scipy/special/_ufuncs.pyx":3147
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn);

  /* "scipy/special/_ufuncs.pyx":3148
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[3]) = ((void *)((char *)__pyx_k_dawsn));

  /* "scipy/special/_ufuncs.pyx":3149
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex);

  /* "scipy/special/_ufuncs.pyx":3150
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[5]) = ((void *)((char *)__pyx_k_dawsn));

  /* "scipy/special/_ufuncs.pyx":3151
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex);

  /* "scipy/special/_ufuncs.pyx":3152
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")             # <<<<<<<<<<<<<<
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[7]) = ((void *)((char *)__pyx_k_dawsn));

  /* "scipy/special/_ufuncs.pyx":3153
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3154
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]
 * ufunc_dawsn_data[3] = &ufunc_dawsn_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3155
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_dawsn_data[3] = &ufunc_dawsn_ptr[2*3]
 * dawsn = np.PyUFunc_FromFuncAndData(ufunc_dawsn_loops, ufunc_dawsn_data, ufunc_dawsn_types, 4, 1, 1, 0, "dawsn", ufunc_dawsn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3156
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]
 * ufunc_dawsn_data[3] = &ufunc_dawsn_ptr[2*3]             # <<<<<<<<<<<<<<
 * dawsn = np.PyUFunc_FromFuncAndData(ufunc_dawsn_loops, ufunc_dawsn_data, ufunc_dawsn_types, 4, 1, 1, 0, "dawsn", ufunc_dawsn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3157
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]
 * ufunc_dawsn_data[3] = &ufunc_dawsn_ptr[2*3]
 * dawsn = np.PyUFunc_FromFuncAndData(ufunc_dawsn_loops, ufunc_dawsn_data, ufunc_dawsn_types, 4, 1, 1, 0, "dawsn", ufunc_dawsn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipe_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types, 4, 1, 1, 0, __pyx_k_dawsn, __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_dawsn, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3163
 * cdef void *ufunc_ellipe_data[2]
 * cdef char ufunc_ellipe_types[4]
 * cdef char *ufunc_ellipe_doc = (             # <<<<<<<<<<<<<<
 *     "ellipe(m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_doc = __pyx_k_ellipe_m_Complete_elliptic_integ;

  /* "scipy/special/_ufuncs.pyx":3188
 *     "ellipkinc : Incomplete elliptic integral of the first kind\n"
 *     "ellipeinc : Incomplete elliptic integral of the second kind")
 * ufunc_ellipe_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ellipe_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3189
 *     "ellipeinc : Incomplete elliptic integral of the second kind")
 * ufunc_ellipe_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ellipe_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3190
 * ufunc_ellipe_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ellipe_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3191
 * ufunc_ellipe_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3192
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3193
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3194
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe             # <<<<<<<<<<<<<<
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[0]) = ((void *)ellpe);

  /* "scipy/special/_ufuncs.pyx":3195
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")             # <<<<<<<<<<<<<<
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[1]) = ((void *)((char *)__pyx_k_ellipe));

  /* "scipy/special/_ufuncs.pyx":3196
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe             # <<<<<<<<<<<<<<
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[2]) = ((void *)ellpe);

  /* "scipy/special/_ufuncs.pyx":3197
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")             # <<<<<<<<<<<<<<
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]
 * ufunc_ellipe_data[1] = &ufunc_ellipe_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[3]) = ((void *)((char *)__pyx_k_ellipe));

  /* "scipy/special/_ufuncs.pyx":3198
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipe_data[1] = &ufunc_ellipe_ptr[2*1]
 * ellipe = np.PyUFunc_FromFuncAndData(ufunc_ellipe_loops, ufunc_ellipe_data, ufunc_ellipe_types, 2, 1, 1, 0, "ellipe", ufunc_ellipe_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3199
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]
 * ufunc_ellipe_data[1] = &ufunc_ellipe_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipe = np.PyUFunc_FromFuncAndData(ufunc_ellipe_loops, ufunc_ellipe_data, ufunc_ellipe_types, 2, 1, 1, 0, "ellipe", ufunc_ellipe_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3200
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]
 * ufunc_ellipe_data[1] = &ufunc_ellipe_ptr[2*1]
 * ellipe = np.PyUFunc_FromFuncAndData(ufunc_ellipe_loops, ufunc_ellipe_data, ufunc_ellipe_types, 2, 1, 1, 0, "ellipe", ufunc_ellipe_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipeinc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types, 2, 1, 1, 0, __pyx_k_ellipe, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipe, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3206
 * cdef void *ufunc_ellipeinc_data[2]
 * cdef char ufunc_ellipeinc_types[6]
 * cdef char *ufunc_ellipeinc_doc = (             # <<<<<<<<<<<<<<
 *     "ellipeinc(phi, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_doc = __pyx_k_ellipeinc_phi_m_Incomplete_ellip;

  /* "scipy/special/_ufuncs.pyx":3234
 *     "ellipkinc : Incomplete elliptic integral of the first kind\n"
 *     "ellipe : Complete elliptic integral of the second kind")
 * ufunc_ellipeinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3235
 *     "ellipe : Complete elliptic integral of the second kind")
 * ufunc_ellipeinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ellipeinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3236
 * ufunc_ellipeinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ellipeinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3237
 * ufunc_ellipeinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3238
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3239
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3240
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3241
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3242
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[0]) = ((void *)ellie);

  /* "scipy/special/_ufuncs.pyx":3243
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[1]) = ((void *)((char *)__pyx_k_ellipeinc));

  /* "scipy/special/_ufuncs.pyx":3244
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[2]) = ((void *)ellie);

  /* "scipy/special/_ufuncs.pyx":3245
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]
 * ufunc_ellipeinc_data[1] = &ufunc_ellipeinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[3]) = ((void *)((char *)__pyx_k_ellipeinc));

  /* "scipy/special/_ufuncs.pyx":3246
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_data[1] = &ufunc_ellipeinc_ptr[2*1]
 * ellipeinc = np.PyUFunc_FromFuncAndData(ufunc_ellipeinc_loops, ufunc_ellipeinc_data, ufunc_ellipeinc_types, 2, 2, 1, 0, "ellipeinc", ufunc_ellipeinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3247
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]
 * ufunc_ellipeinc_data[1] = &ufunc_ellipeinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipeinc = np.PyUFunc_FromFuncAndData(ufunc_ellipeinc_loops, ufunc_ellipeinc_data, ufunc_ellipeinc_types, 2, 2, 1, 0, "ellipeinc", ufunc_ellipeinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3248
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]
 * ufunc_ellipeinc_data[1] = &ufunc_ellipeinc_ptr[2*1]
 * ellipeinc = np.PyUFunc_FromFuncAndData(ufunc_ellipeinc_loops, ufunc_ellipeinc_data, ufunc_ellipeinc_types, 2, 2, 1, 0, "ellipeinc", ufunc_ellipeinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipj_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types, 2, 2, 1, 0, __pyx_k_ellipeinc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3248; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipeinc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3248; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3254
 * cdef void *ufunc_ellipj_data[2]
 * cdef char ufunc_ellipj_types[12]
 * cdef char *ufunc_ellipj_doc = (             # <<<<<<<<<<<<<<
 *     "ellipj(u, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_doc = __pyx_k_ellipj_u_m_Jacobian_elliptic_fun;

  /* "scipy/special/_ufuncs.pyx":3276
 *     "    The value ``ph`` is such that if ``u = ellik(ph, m)``,\n"
 *     "    then ``sn(u|m) = sin(ph)`` and ``cn(u|m) = cos(ph)``.")
 * ufunc_ellipj_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_ff_ffff             # <<<<<<<<<<<<<<
 * ufunc_ellipj_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_dd_dddd
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_ff_ffff);

  /* "scipy/special/_ufuncs.pyx":3277
 *     "    then ``sn(u|m) = sin(ph)`` and ``cn(u|m) = cos(ph)``.")
 * ufunc_ellipj_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_ff_ffff
 * ufunc_ellipj_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_dd_dddd             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_dd_dddd);

  /* "scipy/special/_ufuncs.pyx":3278
 * ufunc_ellipj_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_ff_ffff
 * ufunc_ellipj_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_dd_dddd
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3279
 * ufunc_ellipj_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_dd_dddd
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3280
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3281
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3282
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3283
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3284
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3285
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3286
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3287
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3288
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3289
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3290
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj             # <<<<<<<<<<<<<<
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[0]) = ((void *)ellpj);

  /* "scipy/special/_ufuncs.pyx":3291
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")             # <<<<<<<<<<<<<<
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[1]) = ((void *)((char *)__pyx_k_ellipj));

  /* "scipy/special/_ufuncs.pyx":3292
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj             # <<<<<<<<<<<<<<
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[2]) = ((void *)ellpj);

  /* "scipy/special/_ufuncs.pyx":3293
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")             # <<<<<<<<<<<<<<
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]
 * ufunc_ellipj_data[1] = &ufunc_ellipj_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[3]) = ((void *)((char *)__pyx_k_ellipj));

  /* "scipy/special/_ufuncs.pyx":3294
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipj_data[1] = &ufunc_ellipj_ptr[2*1]
 * ellipj = np.PyUFunc_FromFuncAndData(ufunc_ellipj_loops, ufunc_ellipj_data, ufunc_ellipj_types, 2, 2, 4, 0, "ellipj", ufunc_ellipj_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3295
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]
 * ufunc_ellipj_data[1] = &ufunc_ellipj_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipj = np.PyUFunc_FromFuncAndData(ufunc_ellipj_loops, ufunc_ellipj_data, ufunc_ellipj_types, 2, 2, 4, 0, "ellipj", ufunc_ellipj_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3296
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]
 * ufunc_ellipj_data[1] = &ufunc_ellipj_ptr[2*1]
 * ellipj = np.PyUFunc_FromFuncAndData(ufunc_ellipj_loops, ufunc_ellipj_data, ufunc_ellipj_types, 2, 2, 4, 0, "ellipj", ufunc_ellipj_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipkinc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types, 2, 2, 4, 0, __pyx_k_ellipj, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipj, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3302
 * cdef void *ufunc_ellipkinc_data[2]
 * cdef char ufunc_ellipkinc_types[6]
 * cdef char *ufunc_ellipkinc_doc = (             # <<<<<<<<<<<<<<
 *     "ellipkinc(phi, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_doc = __pyx_k_ellipkinc_phi_m_Incomplete_ellip;

  /* "scipy/special/_ufuncs.pyx":3334
 *     "ellipe : Complete elliptic integral of the second kind\n"
 *     "ellipeinc : Incomplete elliptic integral of the second kind")
 * ufunc_ellipkinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3335
 *     "ellipeinc : Incomplete elliptic integral of the second kind")
 * ufunc_ellipkinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ellipkinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3336
 * ufunc_ellipkinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ellipkinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3337
 * ufunc_ellipkinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3338
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3339
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3340
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3341
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3342
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[0]) = ((void *)ellik);

  /* "scipy/special/_ufuncs.pyx":3343
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[1]) = ((void *)((char *)__pyx_k_ellipkinc));

  /* "scipy/special/_ufuncs.pyx":3344
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[2]) = ((void *)ellik);

  /* "scipy/special/_ufuncs.pyx":3345
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]
 * ufunc_ellipkinc_data[1] = &ufunc_ellipkinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[3]) = ((void *)((char *)__pyx_k_ellipkinc));

  /* "scipy/special/_ufuncs.pyx":3346
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_data[1] = &ufunc_ellipkinc_ptr[2*1]
 * ellipkinc = np.PyUFunc_FromFuncAndData(ufunc_ellipkinc_loops, ufunc_ellipkinc_data, ufunc_ellipkinc_types, 2, 2, 1, 0, "ellipkinc", ufunc_ellipkinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3347
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]
 * ufunc_ellipkinc_data[1] = &ufunc_ellipkinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipkinc = np.PyUFunc_FromFuncAndData(ufunc_ellipkinc_loops, ufunc_ellipkinc_data, ufunc_ellipkinc_types, 2, 2, 1, 0, "ellipkinc", ufunc_ellipkinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3348
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]
 * ufunc_ellipkinc_data[1] = &ufunc_ellipkinc_ptr[2*1]
 * ellipkinc = np.PyUFunc_FromFuncAndData(ufunc_ellipkinc_loops, ufunc_ellipkinc_data, ufunc_ellipkinc_types, 2, 2, 1, 0, "ellipkinc", ufunc_ellipkinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipkm1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types, 2, 2, 1, 0, __pyx_k_ellipkinc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3348; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipkinc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3348; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3354
 * cdef void *ufunc_ellipkm1_data[2]
 * cdef char ufunc_ellipkm1_types[4]
 * cdef char *ufunc_ellipkm1_doc = (             # <<<<<<<<<<<<<<
 *     "ellipkm1(p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_doc = __pyx_k_ellipkm1_p_Complete_elliptic_int;

  /* "scipy/special/_ufuncs.pyx":3381
 *     "ellipe : Complete elliptic integral of the second kind\n"
 *     "ellipeinc : Incomplete elliptic integral of the second kind")
 * ufunc_ellipkm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3382
 *     "ellipeinc : Incomplete elliptic integral of the second kind")
 * ufunc_ellipkm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ellipkm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3383
 * ufunc_ellipkm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ellipkm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3384
 * ufunc_ellipkm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3385
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3386
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3387
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[0]) = ((void *)ellpk);

  /* "scipy/special/_ufuncs.pyx":3388
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[1]) = ((void *)((char *)__pyx_k_ellipkm1));

  /* "scipy/special/_ufuncs.pyx":3389
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[2]) = ((void *)ellpk);

  /* "scipy/special/_ufuncs.pyx":3390
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]
 * ufunc_ellipkm1_data[1] = &ufunc_ellipkm1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[3]) = ((void *)((char *)__pyx_k_ellipkm1));

  /* "scipy/special/_ufuncs.pyx":3391
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_data[1] = &ufunc_ellipkm1_ptr[2*1]
 * ellipkm1 = np.PyUFunc_FromFuncAndData(ufunc_ellipkm1_loops, ufunc_ellipkm1_data, ufunc_ellipkm1_types, 2, 1, 1, 0, "ellipkm1", ufunc_ellipkm1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3392
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]
 * ufunc_ellipkm1_data[1] = &ufunc_ellipkm1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipkm1 = np.PyUFunc_FromFuncAndData(ufunc_ellipkm1_loops, ufunc_ellipkm1_data, ufunc_ellipkm1_types, 2, 1, 1, 0, "ellipkm1", ufunc_ellipkm1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3393
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]
 * ufunc_ellipkm1_data[1] = &ufunc_ellipkm1_ptr[2*1]
 * ellipkm1 = np.PyUFunc_FromFuncAndData(ufunc_ellipkm1_loops, ufunc_ellipkm1_data, ufunc_ellipkm1_types, 2, 1, 1, 0, "ellipkm1", ufunc_ellipkm1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_entr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types, 2, 1, 1, 0, __pyx_k_ellipkm1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3393; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipkm1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3393; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3399
 * cdef void *ufunc_entr_data[2]
 * cdef char ufunc_entr_types[4]
 * cdef char *ufunc_entr_doc = (             # <<<<<<<<<<<<<<
 *     "entr(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_doc = __pyx_k_entr_x_Elementwise_function_for;

  /* "scipy/special/_ufuncs.pyx":3425
 *     "\n"
 *     ".. versionadded:: 0.14.0")
 * ufunc_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_entr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3426
 *     ".. versionadded:: 0.14.0")
 * ufunc_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_entr_types[0] = <char>NPY_FLOAT
 * ufunc_entr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3427
 * ufunc_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_entr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_entr_types[1] = <char>NPY_FLOAT
 * ufunc_entr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3428
 * ufunc_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_entr_types[0] = <char>NPY_FLOAT
 * ufunc_entr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_entr_types[2] = <char>NPY_DOUBLE
 * ufunc_entr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3429
 * ufunc_entr_types[0] = <char>NPY_FLOAT
 * ufunc_entr_types[1] = <char>NPY_FLOAT
 * ufunc_entr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_entr_ptr[2*0] = <void*>_func_entr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3430
 * ufunc_entr_types[1] = <char>NPY_FLOAT
 * ufunc_entr_types[2] = <char>NPY_DOUBLE
 * ufunc_entr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_entr_ptr[2*0] = <void*>_func_entr
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3431
 * ufunc_entr_types[2] = <char>NPY_DOUBLE
 * ufunc_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_entr_ptr[2*0] = <void*>_func_entr             # <<<<<<<<<<<<<<
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")
 * ufunc_entr_ptr[2*1] = <void*>_func_entr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_entr);

  /* "scipy/special/_ufuncs.pyx":3432
 * ufunc_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_entr_ptr[2*0] = <void*>_func_entr
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")             # <<<<<<<<<<<<<<
 * ufunc_entr_ptr[2*1] = <void*>_func_entr
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[1]) = ((void *)((char *)__pyx_k_entr));

  /* "scipy/special/_ufuncs.pyx":3433
 * ufunc_entr_ptr[2*0] = <void*>_func_entr
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")
 * ufunc_entr_ptr[2*1] = <void*>_func_entr             # <<<<<<<<<<<<<<
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_entr);

  /* "scipy/special/_ufuncs.pyx":3434
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")
 * ufunc_entr_ptr[2*1] = <void*>_func_entr
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")             # <<<<<<<<<<<<<<
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]
 * ufunc_entr_data[1] = &ufunc_entr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[3]) = ((void *)((char *)__pyx_k_entr));

  /* "scipy/special/_ufuncs.pyx":3435
 * ufunc_entr_ptr[2*1] = <void*>_func_entr
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_entr_data[1] = &ufunc_entr_ptr[2*1]
 * entr = np.PyUFunc_FromFuncAndData(ufunc_entr_loops, ufunc_entr_data, ufunc_entr_types, 2, 1, 1, 0, "entr", ufunc_entr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3436
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]
 * ufunc_entr_data[1] = &ufunc_entr_ptr[2*1]             # <<<<<<<<<<<<<<
 * entr = np.PyUFunc_FromFuncAndData(ufunc_entr_loops, ufunc_entr_data, ufunc_entr_types, 2, 1, 1, 0, "entr", ufunc_entr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3437
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]
 * ufunc_entr_data[1] = &ufunc_entr_ptr[2*1]
 * entr = np.PyUFunc_FromFuncAndData(ufunc_entr_loops, ufunc_entr_data, ufunc_entr_types, 2, 1, 1, 0, "entr", ufunc_entr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_erf_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types, 2, 1, 1, 0, __pyx_k_entr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_entr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3443
 * cdef void *ufunc_erf_data[4]
 * cdef char ufunc_erf_types[8]
 * cdef char *ufunc_erf_doc = (             # <<<<<<<<<<<<<<
 *     "erf(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_doc = __pyx_k_erf_z_Returns_the_error_function;

  /* "scipy/special/_ufuncs.pyx":3478
 *     ".. [3] Steven G. Johnson, Faddeeva W function implementation.\n"
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_erf_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_erf_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3479
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_erf_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erf_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3480
 * ufunc_erf_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erf_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erf_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":3481
 * ufunc_erf_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_erf_types[0] = <char>NPY_FLOAT
 * ufunc_erf_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":3482
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erf_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erf_types[1] = <char>NPY_FLOAT
 * ufunc_erf_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3483
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erf_types[0] = <char>NPY_FLOAT
 * ufunc_erf_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erf_types[2] = <char>NPY_DOUBLE
 * ufunc_erf_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3484
 * ufunc_erf_types[0] = <char>NPY_FLOAT
 * ufunc_erf_types[1] = <char>NPY_FLOAT
 * ufunc_erf_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erf_types[3] = <char>NPY_DOUBLE
 * ufunc_erf_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3485
 * ufunc_erf_types[1] = <char>NPY_FLOAT
 * ufunc_erf_types[2] = <char>NPY_DOUBLE
 * ufunc_erf_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erf_types[4] = <char>NPY_CFLOAT
 * ufunc_erf_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3486
 * ufunc_erf_types[2] = <char>NPY_DOUBLE
 * ufunc_erf_types[3] = <char>NPY_DOUBLE
 * ufunc_erf_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erf_types[5] = <char>NPY_CFLOAT
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3487
 * ufunc_erf_types[3] = <char>NPY_DOUBLE
 * ufunc_erf_types[4] = <char>NPY_CFLOAT
 * ufunc_erf_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3488
 * ufunc_erf_types[4] = <char>NPY_CFLOAT
 * ufunc_erf_types[5] = <char>NPY_CFLOAT
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE
 * ufunc_erf_ptr[2*0] = <void*>_func_erf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3489
 * ufunc_erf_types[5] = <char>NPY_CFLOAT
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*0] = <void*>_func_erf
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3490
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE
 * ufunc_erf_ptr[2*0] = <void*>_func_erf             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*1] = <void*>_func_erf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[0]) = ((void *)erf);

  /* "scipy/special/_ufuncs.pyx":3491
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE
 * ufunc_erf_ptr[2*0] = <void*>_func_erf
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*1] = <void*>_func_erf
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[1]) = ((void *)((char *)__pyx_k_erf));

  /* "scipy/special/_ufuncs.pyx":3492
 * ufunc_erf_ptr[2*0] = <void*>_func_erf
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*1] = <void*>_func_erf             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[2]) = ((void *)erf);

  /* "scipy/special/_ufuncs.pyx":3493
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*1] = <void*>_func_erf
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[3]) = ((void *)((char *)__pyx_k_erf));

  /* "scipy/special/_ufuncs.pyx":3494
 * ufunc_erf_ptr[2*1] = <void*>_func_erf
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf);

  /* "scipy/special/_ufuncs.pyx":3495
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[5]) = ((void *)((char *)__pyx_k_erf));

  /* "scipy/special/_ufuncs.pyx":3496
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf);

  /* "scipy/special/_ufuncs.pyx":3497
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")             # <<<<<<<<<<<<<<
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[7]) = ((void *)((char *)__pyx_k_erf));

  /* "scipy/special/_ufuncs.pyx":3498
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3499
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]
 * ufunc_erf_data[3] = &ufunc_erf_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3500
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_erf_data[3] = &ufunc_erf_ptr[2*3]
 * erf = np.PyUFunc_FromFuncAndData(ufunc_erf_loops, ufunc_erf_data, ufunc_erf_types, 4, 1, 1, 0, "erf", ufunc_erf_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3501
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]
 * ufunc_erf_data[3] = &ufunc_erf_ptr[2*3]             # <<<<<<<<<<<<<<
 * erf = np.PyUFunc_FromFuncAndData(ufunc_erf_loops, ufunc_erf_data, ufunc_erf_types, 4, 1, 1, 0, "erf", ufunc_erf_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3502
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]
 * ufunc_erf_data[3] = &ufunc_erf_ptr[2*3]
 * erf = np.PyUFunc_FromFuncAndData(ufunc_erf_loops, ufunc_erf_data, ufunc_erf_types, 4, 1, 1, 0, "erf", ufunc_erf_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_erfc_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types, 4, 1, 1, 0, __pyx_k_erf, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erf, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3508
 * cdef void *ufunc_erfc_data[4]
 * cdef char ufunc_erfc_types[8]
 * cdef char *ufunc_erfc_doc = (             # <<<<<<<<<<<<<<
 *     "erfc(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_doc = __pyx_k_erfc_x_Complementary_error_funct;

  /* "scipy/special/_ufuncs.pyx":3517
 *     ".. [1] Steven G. Johnson, Faddeeva W function implementation.\n"
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_erfc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_erfc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3518
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_erfc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3519
 * ufunc_erfc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":3520
 * ufunc_erfc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[0] = <char>NPY_FLOAT
 * ufunc_erfc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":3521
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[1] = <char>NPY_FLOAT
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3522
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfc_types[0] = <char>NPY_FLOAT
 * ufunc_erfc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3523
 * ufunc_erfc_types[0] = <char>NPY_FLOAT
 * ufunc_erfc_types[1] = <char>NPY_FLOAT
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3524
 * ufunc_erfc_types[1] = <char>NPY_FLOAT
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3525
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3526
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3527
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3528
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3529
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[0]) = ((void *)erfc);

  /* "scipy/special/_ufuncs.pyx":3530
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[1]) = ((void *)((char *)__pyx_k_erfc));

  /* "scipy/special/_ufuncs.pyx":3531
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[2]) = ((void *)erfc);

  /* "scipy/special/_ufuncs.pyx":3532
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[3]) = ((void *)((char *)__pyx_k_erfc));

  /* "scipy/special/_ufuncs.pyx":3533
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc);

  /* "scipy/special/_ufuncs.pyx":3534
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[5]) = ((void *)((char *)__pyx_k_erfc));

  /* "scipy/special/_ufuncs.pyx":3535
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc);

  /* "scipy/special/_ufuncs.pyx":3536
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")             # <<<<<<<<<<<<<<
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[7]) = ((void *)((char *)__pyx_k_erfc));

  /* "scipy/special/_ufuncs.pyx":3537
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3538
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]
 * ufunc_erfc_data[3] = &ufunc_erfc_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3539
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_erfc_data[3] = &ufunc_erfc_ptr[2*3]
 * erfc = np.PyUFunc_FromFuncAndData(ufunc_erfc_loops, ufunc_erfc_data, ufunc_erfc_types, 4, 1, 1, 0, "erfc", ufunc_erfc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3540
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]
 * ufunc_erfc_data[3] = &ufunc_erfc_ptr[2*3]             # <<<<<<<<<<<<<<
 * erfc = np.PyUFunc_FromFuncAndData(ufunc_erfc_loops, ufunc_erfc_data, ufunc_erfc_types, 4, 1, 1, 0, "erfc", ufunc_erfc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3541
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]
 * ufunc_erfc_data[3] = &ufunc_erfc_ptr[2*3]
 * erfc = np.PyUFunc_FromFuncAndData(ufunc_erfc_loops, ufunc_erfc_data, ufunc_erfc_types, 4, 1, 1, 0, "erfc", ufunc_erfc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_erfcx_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types, 4, 1, 1, 0, __pyx_k_erfc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3541; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erfc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3541; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3547
 * cdef void *ufunc_erfcx_data[4]
 * cdef char ufunc_erfcx_types[8]
 * cdef char *ufunc_erfcx_doc = (             # <<<<<<<<<<<<<<
 *     "erfcx(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_doc = __pyx_k_erfcx_x_Scaled_complementary_err;

  /* "scipy/special/_ufuncs.pyx":3561
 *     ".. [1] Steven G. Johnson, Faddeeva W function implementation.\n"
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_erfcx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_erfcx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3562
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_erfcx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfcx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3563
 * ufunc_erfcx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfcx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":3564
 * ufunc_erfcx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":3565
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3566
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3567
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3568
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3569
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3570
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3571
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3572
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3573
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx);

  /* "scipy/special/_ufuncs.pyx":3574
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[1]) = ((void *)((char *)__pyx_k_erfcx));

  /* "scipy/special/_ufuncs.pyx":3575
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx);

  /* "scipy/special/_ufuncs.pyx":3576
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[3]) = ((void *)((char *)__pyx_k_erfcx));

  /* "scipy/special/_ufuncs.pyx":3577
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex);

  /* "scipy/special/_ufuncs.pyx":3578
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[5]) = ((void *)((char *)__pyx_k_erfcx));

  /* "scipy/special/_ufuncs.pyx":3579
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex);

  /* "scipy/special/_ufuncs.pyx":3580
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")             # <<<<<<<<<<<<<<
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[7]) = ((void *)((char *)__pyx_k_erfcx));

  /* "scipy/special/_ufuncs.pyx":3581
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3582
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]
 * ufunc_erfcx_data[3] = &ufunc_erfcx_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3583
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_erfcx_data[3] = &ufunc_erfcx_ptr[2*3]
 * erfcx = np.PyUFunc_FromFuncAndData(ufunc_erfcx_loops, ufunc_erfcx_data, ufunc_erfcx_types, 4, 1, 1, 0, "erfcx", ufunc_erfcx_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3584
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]
 * ufunc_erfcx_data[3] = &ufunc_erfcx_ptr[2*3]             # <<<<<<<<<<<<<<
 * erfcx = np.PyUFunc_FromFuncAndData(ufunc_erfcx_loops, ufunc_erfcx_data, ufunc_erfcx_types, 4, 1, 1, 0, "erfcx", ufunc_erfcx_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3585
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]
 * ufunc_erfcx_data[3] = &ufunc_erfcx_ptr[2*3]
 * erfcx = np.PyUFunc_FromFuncAndData(ufunc_erfcx_loops, ufunc_erfcx_data, ufunc_erfcx_types, 4, 1, 1, 0, "erfcx", ufunc_erfcx_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_erfi_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types, 4, 1, 1, 0, __pyx_k_erfcx, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erfcx, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3591
 * cdef void *ufunc_erfi_data[4]
 * cdef char ufunc_erfi_types[8]
 * cdef char *ufunc_erfi_doc = (             # <<<<<<<<<<<<<<
 *     "erfi(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_doc = __pyx_k_erfi_z_Imaginary_error_function;

  /* "scipy/special/_ufuncs.pyx":3605
 *     ".. [1] Steven G. Johnson, Faddeeva W function implementation.\n"
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_erfi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_erfi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3606
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_erfi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3607
 * ufunc_erfi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":3608
 * ufunc_erfi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[0] = <char>NPY_FLOAT
 * ufunc_erfi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":3609
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[1] = <char>NPY_FLOAT
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3610
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfi_types[0] = <char>NPY_FLOAT
 * ufunc_erfi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3611
 * ufunc_erfi_types[0] = <char>NPY_FLOAT
 * ufunc_erfi_types[1] = <char>NPY_FLOAT
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3612
 * ufunc_erfi_types[1] = <char>NPY_FLOAT
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3613
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3614
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3615
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3616
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3617
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi);

  /* "scipy/special/_ufuncs.pyx":3618
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[1]) = ((void *)((char *)__pyx_k_erfi));

  /* "scipy/special/_ufuncs.pyx":3619
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi);

  /* "scipy/special/_ufuncs.pyx":3620
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[3]) = ((void *)((char *)__pyx_k_erfi));

  /* "scipy/special/_ufuncs.pyx":3621
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex);

  /* "scipy/special/_ufuncs.pyx":3622
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[5]) = ((void *)((char *)__pyx_k_erfi));

  /* "scipy/special/_ufuncs.pyx":3623
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex);

  /* "scipy/special/_ufuncs.pyx":3624
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")             # <<<<<<<<<<<<<<
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[7]) = ((void *)((char *)__pyx_k_erfi));

  /* "scipy/special/_ufuncs.pyx":3625
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3626
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]
 * ufunc_erfi_data[3] = &ufunc_erfi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3627
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_erfi_data[3] = &ufunc_erfi_ptr[2*3]
 * erfi = np.PyUFunc_FromFuncAndData(ufunc_erfi_loops, ufunc_erfi_data, ufunc_erfi_types, 4, 1, 1, 0, "erfi", ufunc_erfi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3628
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]
 * ufunc_erfi_data[3] = &ufunc_erfi_ptr[2*3]             # <<<<<<<<<<<<<<
 * erfi = np.PyUFunc_FromFuncAndData(ufunc_erfi_loops, ufunc_erfi_data, ufunc_erfi_types, 4, 1, 1, 0, "erfi", ufunc_erfi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3629
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]
 * ufunc_erfi_data[3] = &ufunc_erfi_ptr[2*3]
 * erfi = np.PyUFunc_FromFuncAndData(ufunc_erfi_loops, ufunc_erfi_data, ufunc_erfi_types, 4, 1, 1, 0, "erfi", ufunc_erfi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_chebyc_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types, 4, 1, 1, 0, __pyx_k_erfi, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erfi, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3635
 * cdef void *ufunc_eval_chebyc_data[5]
 * cdef char ufunc_eval_chebyc_types[15]
 * cdef char *ufunc_eval_chebyc_doc = (             # <<<<<<<<<<<<<<
 *     "eval_chebyc(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_doc = __pyx_k_eval_chebyc_n_x_out_None_Evaluat;

  /* "scipy/special/_ufuncs.pyx":3639
 *     "\n"
 *     "Evaluate Chebyshev C polynomial at a point.")
 * ufunc_eval_chebyc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":3640
 *     "Evaluate Chebyshev C polynomial at a point.")
 * ufunc_eval_chebyc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3641
 * ufunc_eval_chebyc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":3642
 * ufunc_eval_chebyc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3643
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":3644
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3645
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3646
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3647
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3648
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3649
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3650
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3651
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3652
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3653
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3654
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3655
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3656
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3657
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3658
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3659
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc_l);

  /* "scipy/special/_ufuncs.pyx":3660
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[1]) = ((void *)((char *)__pyx_k_eval_chebyc));

  /* "scipy/special/_ufuncs.pyx":3661
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":3662
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[3]) = ((void *)((char *)__pyx_k_eval_chebyc));

  /* "scipy/special/_ufuncs.pyx":3663
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":3664
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[5]) = ((void *)((char *)__pyx_k_eval_chebyc));

  /* "scipy/special/_ufuncs.pyx":3665
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":3666
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[7]) = ((void *)((char *)__pyx_k_eval_chebyc));

  /* "scipy/special/_ufuncs.pyx":3667
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":3668
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[9]) = ((void *)((char *)__pyx_k_eval_chebyc));

  /* "scipy/special/_ufuncs.pyx":3669
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3670
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3671
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]
 * ufunc_eval_chebyc_data[4] = &ufunc_eval_chebyc_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3672
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[4] = &ufunc_eval_chebyc_ptr[2*4]
 * eval_chebyc = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyc_loops, ufunc_eval_chebyc_data, ufunc_eval_chebyc_types, 5, 2, 1, 0, "eval_chebyc", ufunc_eval_chebyc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3673
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]
 * ufunc_eval_chebyc_data[4] = &ufunc_eval_chebyc_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_chebyc = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyc_loops, ufunc_eval_chebyc_data, ufunc_eval_chebyc_types, 5, 2, 1, 0, "eval_chebyc", ufunc_eval_chebyc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":3674
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]
 * ufunc_eval_chebyc_data[4] = &ufunc_eval_chebyc_ptr[2*4]
 * eval_chebyc = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyc_loops, ufunc_eval_chebyc_data, ufunc_eval_chebyc_types, 5, 2, 1, 0, "eval_chebyc", ufunc_eval_chebyc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_chebys_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types, 5, 2, 1, 0, __pyx_k_eval_chebyc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3674; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_chebyc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3674; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3680
 * cdef void *ufunc_eval_chebys_data[5]
 * cdef char ufunc_eval_chebys_types[15]
 * cdef char *ufunc_eval_chebys_doc = (             # <<<<<<<<<<<<<<
 *     "eval_chebys(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_doc = __pyx_k_eval_chebys_n_x_out_None_Evaluat;

  /* "scipy/special/_ufuncs.pyx":3684
 *     "\n"
 *     "Evaluate Chebyshev S polynomial at a point.")
 * ufunc_eval_chebys_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":3685
 *     "Evaluate Chebyshev S polynomial at a point.")
 * ufunc_eval_chebys_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebys_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3686
 * ufunc_eval_chebys_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebys_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":3687
 * ufunc_eval_chebys_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3688
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":3689
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3690
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3691
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3692
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3693
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3694
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3695
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3696
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3697
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3698
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3699
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3700
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3701
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3702
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3703
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3704
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys_l);

  /* "scipy/special/_ufuncs.pyx":3705
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[1]) = ((void *)((char *)__pyx_k_eval_chebys));

  /* "scipy/special/_ufuncs.pyx":3706
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":3707
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[3]) = ((void *)((char *)__pyx_k_eval_chebys));

  /* "scipy/special/_ufuncs.pyx":3708
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":3709
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[5]) = ((void *)((char *)__pyx_k_eval_chebys));

  /* "scipy/special/_ufuncs.pyx":3710
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":3711
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[7]) = ((void *)((char *)__pyx_k_eval_chebys));

  /* "scipy/special/_ufuncs.pyx":3712
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":3713
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[9]) = ((void *)((char *)__pyx_k_eval_chebys));

  /* "scipy/special/_ufuncs.pyx":3714
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3715
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3716
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]
 * ufunc_eval_chebys_data[4] = &ufunc_eval_chebys_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3717
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[4] = &ufunc_eval_chebys_ptr[2*4]
 * eval_chebys = np.PyUFunc_FromFuncAndData(ufunc_eval_chebys_loops, ufunc_eval_chebys_data, ufunc_eval_chebys_types, 5, 2, 1, 0, "eval_chebys", ufunc_eval_chebys_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3718
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]
 * ufunc_eval_chebys_data[4] = &ufunc_eval_chebys_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_chebys = np.PyUFunc_FromFuncAndData(ufunc_eval_chebys_loops, ufunc_eval_chebys_data, ufunc_eval_chebys_types, 5, 2, 1, 0, "eval_chebys", ufunc_eval_chebys_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":3719
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]
 * ufunc_eval_chebys_data[4] = &ufunc_eval_chebys_ptr[2*4]
 * eval_chebys = np.PyUFunc_FromFuncAndData(ufunc_eval_chebys_loops, ufunc_eval_chebys_data, ufunc_eval_chebys_types, 5, 2, 1, 0, "eval_chebys", ufunc_eval_chebys_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_chebyt_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types, 5, 2, 1, 0, __pyx_k_eval_chebys, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_chebys, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3725
 * cdef void *ufunc_eval_chebyt_data[5]
 * cdef char ufunc_eval_chebyt_types[15]
 * cdef char *ufunc_eval_chebyt_doc = (             # <<<<<<<<<<<<<<
 *     "eval_chebyt(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_doc = __pyx_k_eval_chebyt_n_x_out_None_Evaluat;

  /* "scipy/special/_ufuncs.pyx":3732
 *     "This routine is numerically stable for `x` in ``[-1, 1]`` at least\n"
 *     "up to order ``10000``.")
 * ufunc_eval_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":3733
 *     "up to order ``10000``.")
 * ufunc_eval_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3734
 * ufunc_eval_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":3735
 * ufunc_eval_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3736
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":3737
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3738
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3739
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3740
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3741
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3742
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3743
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3744
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3745
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3746
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3747
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3748
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3749
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3750
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3751
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3752
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l);

  /* "scipy/special/_ufuncs.pyx":3753
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[1]) = ((void *)((char *)__pyx_k_eval_chebyt));

  /* "scipy/special/_ufuncs.pyx":3754
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":3755
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[3]) = ((void *)((char *)__pyx_k_eval_chebyt));

  /* "scipy/special/_ufuncs.pyx":3756
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":3757
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[5]) = ((void *)((char *)__pyx_k_eval_chebyt));

  /* "scipy/special/_ufuncs.pyx":3758
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":3759
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[7]) = ((void *)((char *)__pyx_k_eval_chebyt));

  /* "scipy/special/_ufuncs.pyx":3760
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":3761
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[9]) = ((void *)((char *)__pyx_k_eval_chebyt));

  /* "scipy/special/_ufuncs.pyx":3762
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3763
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3764
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]
 * ufunc_eval_chebyt_data[4] = &ufunc_eval_chebyt_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3765
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[4] = &ufunc_eval_chebyt_ptr[2*4]
 * eval_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyt_loops, ufunc_eval_chebyt_data, ufunc_eval_chebyt_types, 5, 2, 1, 0, "eval_chebyt", ufunc_eval_chebyt_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3766
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]
 * ufunc_eval_chebyt_data[4] = &ufunc_eval_chebyt_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyt_loops, ufunc_eval_chebyt_data, ufunc_eval_chebyt_types, 5, 2, 1, 0, "eval_chebyt", ufunc_eval_chebyt_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":3767
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]
 * ufunc_eval_chebyt_data[4] = &ufunc_eval_chebyt_ptr[2*4]
 * eval_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyt_loops, ufunc_eval_chebyt_data, ufunc_eval_chebyt_types, 5, 2, 1, 0, "eval_chebyt", ufunc_eval_chebyt_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_chebyu_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types, 5, 2, 1, 0, __pyx_k_eval_chebyt, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_chebyt, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3773
 * cdef void *ufunc_eval_chebyu_data[5]
 * cdef char ufunc_eval_chebyu_types[15]
 * cdef char *ufunc_eval_chebyu_doc = (             # <<<<<<<<<<<<<<
 *     "eval_chebyu(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_doc = __pyx_k_eval_chebyu_n_x_out_None_Evaluat;

  /* "scipy/special/_ufuncs.pyx":3777
 *     "\n"
 *     "Evaluate Chebyshev U polynomial at a point.")
 * ufunc_eval_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":3778
 *     "Evaluate Chebyshev U polynomial at a point.")
 * ufunc_eval_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3779
 * ufunc_eval_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":3780
 * ufunc_eval_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3781
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":3782
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3783
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3784
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3785
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3786
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3787
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3788
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3789
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3790
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3791
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3792
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3793
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3794
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3795
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3796
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3797
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l);

  /* "scipy/special/_ufuncs.pyx":3798
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[1]) = ((void *)((char *)__pyx_k_eval_chebyu));

  /* "scipy/special/_ufuncs.pyx":3799
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":3800
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[3]) = ((void *)((char *)__pyx_k_eval_chebyu));

  /* "scipy/special/_ufuncs.pyx":3801
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":3802
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[5]) = ((void *)((char *)__pyx_k_eval_chebyu));

  /* "scipy/special/_ufuncs.pyx":3803
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":3804
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[7]) = ((void *)((char *)__pyx_k_eval_chebyu));

  /* "scipy/special/_ufuncs.pyx":3805
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":3806
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[9]) = ((void *)((char *)__pyx_k_eval_chebyu));

  /* "scipy/special/_ufuncs.pyx":3807
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3808
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3809
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]
 * ufunc_eval_chebyu_data[4] = &ufunc_eval_chebyu_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3810
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[4] = &ufunc_eval_chebyu_ptr[2*4]
 * eval_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyu_loops, ufunc_eval_chebyu_data, ufunc_eval_chebyu_types, 5, 2, 1, 0, "eval_chebyu", ufunc_eval_chebyu_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3811
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]
 * ufunc_eval_chebyu_data[4] = &ufunc_eval_chebyu_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyu_loops, ufunc_eval_chebyu_data, ufunc_eval_chebyu_types, 5, 2, 1, 0, "eval_chebyu", ufunc_eval_chebyu_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":3812
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]
 * ufunc_eval_chebyu_data[4] = &ufunc_eval_chebyu_ptr[2*4]
 * eval_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyu_loops, ufunc_eval_chebyu_data, ufunc_eval_chebyu_types, 5, 2, 1, 0, "eval_chebyu", ufunc_eval_chebyu_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_gegenbauer_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types, 5, 2, 1, 0, __pyx_k_eval_chebyu, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3812; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_chebyu, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3812; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3818
 * cdef void *ufunc_eval_gegenbauer_data[5]
 * cdef char ufunc_eval_gegenbauer_types[20]
 * cdef char *ufunc_eval_gegenbauer_doc = (             # <<<<<<<<<<<<<<
 *     "eval_gegenbauer(n, alpha, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_doc = __pyx_k_eval_gegenbauer_n_alpha_x_out_No;

  /* "scipy/special/_ufuncs.pyx":3822
 *     "\n"
 *     "Evaluate Gegenbauer polynomial at a point.")
 * ufunc_eval_gegenbauer_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ldd__As_ldd_d);

  /* "scipy/special/_ufuncs.pyx":3823
 *     "Evaluate Gegenbauer polynomial at a point.")
 * ufunc_eval_gegenbauer_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d
 * ufunc_eval_gegenbauer_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3824
 * ufunc_eval_gegenbauer_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d
 * ufunc_eval_gegenbauer_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F);

  /* "scipy/special/_ufuncs.pyx":3825
 * ufunc_eval_gegenbauer_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3826
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D);

  /* "scipy/special/_ufuncs.pyx":3827
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3828
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3829
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3830
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3831
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3832
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3833
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3834
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3835
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3836
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[9]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3837
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[10]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3838
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[11]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3839
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3840
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3841
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3842
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3843
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3844
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3845
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3846
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3847
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer_l);

  /* "scipy/special/_ufuncs.pyx":3848
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[1]) = ((void *)((char *)__pyx_k_eval_gegenbauer));

  /* "scipy/special/_ufuncs.pyx":3849
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":3850
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[3]) = ((void *)((char *)__pyx_k_eval_gegenbauer));

  /* "scipy/special/_ufuncs.pyx":3851
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":3852
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[5]) = ((void *)((char *)__pyx_k_eval_gegenbauer));

  /* "scipy/special/_ufuncs.pyx":3853
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":3854
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[7]) = ((void *)((char *)__pyx_k_eval_gegenbauer));

  /* "scipy/special/_ufuncs.pyx":3855
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":3856
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[9]) = ((void *)((char *)__pyx_k_eval_gegenbauer));

  /* "scipy/special/_ufuncs.pyx":3857
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3858
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3859
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]
 * ufunc_eval_gegenbauer_data[4] = &ufunc_eval_gegenbauer_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3860
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[4] = &ufunc_eval_gegenbauer_ptr[2*4]
 * eval_gegenbauer = np.PyUFunc_FromFuncAndData(ufunc_eval_gegenbauer_loops, ufunc_eval_gegenbauer_data, ufunc_eval_gegenbauer_types, 5, 3, 1, 0, "eval_gegenbauer", ufunc_eval_gegenbauer_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3861
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]
 * ufunc_eval_gegenbauer_data[4] = &ufunc_eval_gegenbauer_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_gegenbauer = np.PyUFunc_FromFuncAndData(ufunc_eval_gegenbauer_loops, ufunc_eval_gegenbauer_data, ufunc_eval_gegenbauer_types, 5, 3, 1, 0, "eval_gegenbauer", ufunc_eval_gegenbauer_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":3862
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]
 * ufunc_eval_gegenbauer_data[4] = &ufunc_eval_gegenbauer_ptr[2*4]
 * eval_gegenbauer = np.PyUFunc_FromFuncAndData(ufunc_eval_gegenbauer_loops, ufunc_eval_gegenbauer_data, ufunc_eval_gegenbauer_types, 5, 3, 1, 0, "eval_gegenbauer", ufunc_eval_gegenbauer_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_genlaguerre_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types, 5, 3, 1, 0, __pyx_k_eval_gegenbauer, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3862; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_gegenbauer, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3862; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3868
 * cdef void *ufunc_eval_genlaguerre_data[5]
 * cdef char ufunc_eval_genlaguerre_types[20]
 * cdef char *ufunc_eval_genlaguerre_doc = (             # <<<<<<<<<<<<<<
 *     "eval_genlaguerre(n, alpha, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_doc = __pyx_k_eval_genlaguerre_n_alpha_x_out_N;

  /* "scipy/special/_ufuncs.pyx":3872
 *     "\n"
 *     "Evaluate generalized Laguerre polynomial at a point.")
 * ufunc_eval_genlaguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ldd__As_ldd_d);

  /* "scipy/special/_ufuncs.pyx":3873
 *     "Evaluate generalized Laguerre polynomial at a point.")
 * ufunc_eval_genlaguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d
 * ufunc_eval_genlaguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3874
 * ufunc_eval_genlaguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d
 * ufunc_eval_genlaguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F);

  /* "scipy/special/_ufuncs.pyx":3875
 * ufunc_eval_genlaguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3876
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D);

  /* "scipy/special/_ufuncs.pyx":3877
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3878
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3879
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3880
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3881
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3882
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3883
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3884
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3885
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3886
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[9]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3887
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[10]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3888
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[11]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3889
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3890
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3891
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3892
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3893
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3894
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3895
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3896
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3897
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l);

  /* "scipy/special/_ufuncs.pyx":3898
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[1]) = ((void *)((char *)__pyx_k_eval_genlaguerre));

  /* "scipy/special/_ufuncs.pyx":3899
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":3900
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[3]) = ((void *)((char *)__pyx_k_eval_genlaguerre));

  /* "scipy/special/_ufuncs.pyx":3901
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":3902
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[5]) = ((void *)((char *)__pyx_k_eval_genlaguerre));

  /* "scipy/special/_ufuncs.pyx":3903
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":3904
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[7]) = ((void *)((char *)__pyx_k_eval_genlaguerre));

  /* "scipy/special/_ufuncs.pyx":3905
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":3906
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[9]) = ((void *)((char *)__pyx_k_eval_genlaguerre));

  /* "scipy/special/_ufuncs.pyx":3907
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3908
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3909
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]
 * ufunc_eval_genlaguerre_data[4] = &ufunc_eval_genlaguerre_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3910
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[4] = &ufunc_eval_genlaguerre_ptr[2*4]
 * eval_genlaguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_genlaguerre_loops, ufunc_eval_genlaguerre_data, ufunc_eval_genlaguerre_types, 5, 3, 1, 0, "eval_genlaguerre", ufunc_eval_genlaguerre_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":3911
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]
 * ufunc_eval_genlaguerre_data[4] = &ufunc_eval_genlaguerre_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_genlaguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_genlaguerre_loops, ufunc_eval_genlaguerre_data, ufunc_eval_genlaguerre_types, 5, 3, 1, 0, "eval_genlaguerre", ufunc_eval_genlaguerre_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":3912
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]
 * ufunc_eval_genlaguerre_data[4] = &ufunc_eval_genlaguerre_ptr[2*4]
 * eval_genlaguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_genlaguerre_loops, ufunc_eval_genlaguerre_data, ufunc_eval_genlaguerre_types, 5, 3, 1, 0, "eval_genlaguerre", ufunc_eval_genlaguerre_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_hermite_loops[1]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types, 5, 3, 1, 0, __pyx_k_eval_genlaguerre, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_genlaguerre, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3918
 * cdef void *ufunc_eval_hermite_data[1]
 * cdef char ufunc_eval_hermite_types[3]
 * cdef char *ufunc_eval_hermite_doc = (             # <<<<<<<<<<<<<<
 *     "eval_hermite(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_doc = __pyx_k_eval_hermite_n_x_out_None_Evalua;

  /* "scipy/special/_ufuncs.pyx":3922
 *     "\n"
 *     "Evaluate Hermite polynomial at a point.")
 * ufunc_eval_hermite_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_types[0] = <char>NPY_LONG
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":3923
 *     "Evaluate Hermite polynomial at a point.")
 * ufunc_eval_hermite_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_hermite_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3924
 * ufunc_eval_hermite_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_hermite_types[0] = <char>NPY_LONG
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3925
 * ufunc_eval_hermite_types[0] = <char>NPY_LONG
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3926
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")
 * ufunc_eval_hermite_data[0] = &ufunc_eval_hermite_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermite);

  /* "scipy/special/_ufuncs.pyx":3927
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_data[0] = &ufunc_eval_hermite_ptr[2*0]
 * eval_hermite = np.PyUFunc_FromFuncAndData(ufunc_eval_hermite_loops, ufunc_eval_hermite_data, ufunc_eval_hermite_types, 1, 2, 1, 0, "eval_hermite", ufunc_eval_hermite_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_ptr[1]) = ((void *)((char *)__pyx_k_eval_hermite));

  /* "scipy/special/_ufuncs.pyx":3928
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")
 * ufunc_eval_hermite_data[0] = &ufunc_eval_hermite_ptr[2*0]             # <<<<<<<<<<<<<<
 * eval_hermite = np.PyUFunc_FromFuncAndData(ufunc_eval_hermite_loops, ufunc_eval_hermite_data, ufunc_eval_hermite_types, 1, 2, 1, 0, "eval_hermite", ufunc_eval_hermite_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3929
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")
 * ufunc_eval_hermite_data[0] = &ufunc_eval_hermite_ptr[2*0]
 * eval_hermite = np.PyUFunc_FromFuncAndData(ufunc_eval_hermite_loops, ufunc_eval_hermite_data, ufunc_eval_hermite_types, 1, 2, 1, 0, "eval_hermite", ufunc_eval_hermite_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_hermitenorm_loops[1]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types, 1, 2, 1, 0, __pyx_k_eval_hermite, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3929; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_hermite, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3929; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3935
 * cdef void *ufunc_eval_hermitenorm_data[1]
 * cdef char ufunc_eval_hermitenorm_types[3]
 * cdef char *ufunc_eval_hermitenorm_doc = (             # <<<<<<<<<<<<<<
 *     "eval_hermitenorm(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_doc = __pyx_k_eval_hermitenorm_n_x_out_None_Ev;

  /* "scipy/special/_ufuncs.pyx":3939
 *     "\n"
 *     "Evaluate normalized Hermite polynomial at a point.")
 * ufunc_eval_hermitenorm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_types[0] = <char>NPY_LONG
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":3940
 *     "Evaluate normalized Hermite polynomial at a point.")
 * ufunc_eval_hermitenorm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_hermitenorm_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3941
 * ufunc_eval_hermitenorm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_hermitenorm_types[0] = <char>NPY_LONG
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3942
 * ufunc_eval_hermitenorm_types[0] = <char>NPY_LONG
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3943
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")
 * ufunc_eval_hermitenorm_data[0] = &ufunc_eval_hermitenorm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm);

  /* "scipy/special/_ufuncs.pyx":3944
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_data[0] = &ufunc_eval_hermitenorm_ptr[2*0]
 * eval_hermitenorm = np.PyUFunc_FromFuncAndData(ufunc_eval_hermitenorm_loops, ufunc_eval_hermitenorm_data, ufunc_eval_hermitenorm_types, 1, 2, 1, 0, "eval_hermitenorm", ufunc_eval_hermitenorm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_ptr[1]) = ((void *)((char *)__pyx_k_eval_hermitenorm));

  /* "scipy/special/_ufuncs.pyx":3945
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")
 * ufunc_eval_hermitenorm_data[0] = &ufunc_eval_hermitenorm_ptr[2*0]             # <<<<<<<<<<<<<<
 * eval_hermitenorm = np.PyUFunc_FromFuncAndData(ufunc_eval_hermitenorm_loops, ufunc_eval_hermitenorm_data, ufunc_eval_hermitenorm_types, 1, 2, 1, 0, "eval_hermitenorm", ufunc_eval_hermitenorm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3946
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")
 * ufunc_eval_hermitenorm_data[0] = &ufunc_eval_hermitenorm_ptr[2*0]
 * eval_hermitenorm = np.PyUFunc_FromFuncAndData(ufunc_eval_hermitenorm_loops, ufunc_eval_hermitenorm_data, ufunc_eval_hermitenorm_types, 1, 2, 1, 0, "eval_hermitenorm", ufunc_eval_hermitenorm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_jacobi_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types, 1, 2, 1, 0, __pyx_k_eval_hermitenorm, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_hermitenorm, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":3952
 * cdef void *ufunc_eval_jacobi_data[5]
 * cdef char ufunc_eval_jacobi_types[25]
 * cdef char *ufunc_eval_jacobi_doc = (             # <<<<<<<<<<<<<<
 *     "eval_jacobi(n, alpha, beta, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_doc = __pyx_k_eval_jacobi_n_alpha_beta_x_out_N;

  /* "scipy/special/_ufuncs.pyx":3956
 *     "\n"
 *     "Evaluate Jacobi polynomial at a point.")
 * ufunc_eval_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_lddd__As_lddd_d);

  /* "scipy/special/_ufuncs.pyx":3957
 *     "Evaluate Jacobi polynomial at a point.")
 * ufunc_eval_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d
 * ufunc_eval_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":3958
 * ufunc_eval_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d
 * ufunc_eval_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F);

  /* "scipy/special/_ufuncs.pyx":3959
 * ufunc_eval_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":3960
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D);

  /* "scipy/special/_ufuncs.pyx":3961
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3962
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3963
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3964
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3965
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3966
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3967
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3968
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3969
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3970
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[9]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3971
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[10]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3972
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[11]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3973
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[12]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3974
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[13]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3975
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[14]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":3976
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3977
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3978
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3979
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[18]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3980
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[19]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3981
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[20]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3982
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[21]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3983
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[22]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3984
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[23]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3985
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[24]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":3986
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l);

  /* "scipy/special/_ufuncs.pyx":3987
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[1]) = ((void *)((char *)__pyx_k_eval_jacobi));

  /* "scipy/special/_ufuncs.pyx":3988
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":3989
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[3]) = ((void *)((char *)__pyx_k_eval_jacobi));

  /* "scipy/special/_ufuncs.pyx":3990
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":3991
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[5]) = ((void *)((char *)__pyx_k_eval_jacobi));

  /* "scipy/special/_ufuncs.pyx":3992
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":3993
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[7]) = ((void *)((char *)__pyx_k_eval_jacobi));

  /* "scipy/special/_ufuncs.pyx":3994
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":3995
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[9]) = ((void *)((char *)__pyx_k_eval_jacobi));

  /* "scipy/special/_ufuncs.pyx":3996
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3997
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3998
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]
 * ufunc_eval_jacobi_data[4] = &ufunc_eval_jacobi_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3999
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[4] = &ufunc_eval_jacobi_ptr[2*4]
 * eval_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_jacobi_loops, ufunc_eval_jacobi_data, ufunc_eval_jacobi_types, 5, 4, 1, 0, "eval_jacobi", ufunc_eval_jacobi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4000
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]
 * ufunc_eval_jacobi_data[4] = &ufunc_eval_jacobi_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_jacobi_loops, ufunc_eval_jacobi_data, ufunc_eval_jacobi_types, 5, 4, 1, 0, "eval_jacobi", ufunc_eval_jacobi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":4001
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]
 * ufunc_eval_jacobi_data[4] = &ufunc_eval_jacobi_ptr[2*4]
 * eval_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_jacobi_loops, ufunc_eval_jacobi_data, ufunc_eval_jacobi_types, 5, 4, 1, 0, "eval_jacobi", ufunc_eval_jacobi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_laguerre_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types, 5, 4, 1, 0, __pyx_k_eval_jacobi, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4001; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_jacobi, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4001; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4007
 * cdef void *ufunc_eval_laguerre_data[5]
 * cdef char ufunc_eval_laguerre_types[15]
 * cdef char *ufunc_eval_laguerre_doc = (             # <<<<<<<<<<<<<<
 *     "eval_laguerre(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_doc = __pyx_k_eval_laguerre_n_x_out_None_Evalu;

  /* "scipy/special/_ufuncs.pyx":4011
 *     "\n"
 *     "Evaluate Laguerre polynomial at a point.")
 * ufunc_eval_laguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":4012
 *     "Evaluate Laguerre polynomial at a point.")
 * ufunc_eval_laguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_laguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4013
 * ufunc_eval_laguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_laguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":4014
 * ufunc_eval_laguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4015
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":4016
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":4017
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4018
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4019
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4020
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4021
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4022
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4023
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4024
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4025
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4026
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4027
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4028
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4029
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4030
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4031
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre_l);

  /* "scipy/special/_ufuncs.pyx":4032
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[1]) = ((void *)((char *)__pyx_k_eval_laguerre));

  /* "scipy/special/_ufuncs.pyx":4033
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":4034
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[3]) = ((void *)((char *)__pyx_k_eval_laguerre));

  /* "scipy/special/_ufuncs.pyx":4035
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":4036
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[5]) = ((void *)((char *)__pyx_k_eval_laguerre));

  /* "scipy/special/_ufuncs.pyx":4037
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":4038
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[7]) = ((void *)((char *)__pyx_k_eval_laguerre));

  /* "scipy/special/_ufuncs.pyx":4039
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":4040
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[9]) = ((void *)((char *)__pyx_k_eval_laguerre));

  /* "scipy/special/_ufuncs.pyx":4041
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4042
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4043
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]
 * ufunc_eval_laguerre_data[4] = &ufunc_eval_laguerre_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4044
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[4] = &ufunc_eval_laguerre_ptr[2*4]
 * eval_laguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_laguerre_loops, ufunc_eval_laguerre_data, ufunc_eval_laguerre_types, 5, 2, 1, 0, "eval_laguerre", ufunc_eval_laguerre_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4045
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]
 * ufunc_eval_laguerre_data[4] = &ufunc_eval_laguerre_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_laguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_laguerre_loops, ufunc_eval_laguerre_data, ufunc_eval_laguerre_types, 5, 2, 1, 0, "eval_laguerre", ufunc_eval_laguerre_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":4046
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]
 * ufunc_eval_laguerre_data[4] = &ufunc_eval_laguerre_ptr[2*4]
 * eval_laguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_laguerre_loops, ufunc_eval_laguerre_data, ufunc_eval_laguerre_types, 5, 2, 1, 0, "eval_laguerre", ufunc_eval_laguerre_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_legendre_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types, 5, 2, 1, 0, __pyx_k_eval_laguerre, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4046; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_laguerre, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4046; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4052
 * cdef void *ufunc_eval_legendre_data[5]
 * cdef char ufunc_eval_legendre_types[15]
 * cdef char *ufunc_eval_legendre_doc = (             # <<<<<<<<<<<<<<
 *     "eval_legendre(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_doc = __pyx_k_eval_legendre_n_x_out_None_Evalu;

  /* "scipy/special/_ufuncs.pyx":4056
 *     "\n"
 *     "Evaluate Legendre polynomial at a point.")
 * ufunc_eval_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":4057
 *     "Evaluate Legendre polynomial at a point.")
 * ufunc_eval_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4058
 * ufunc_eval_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":4059
 * ufunc_eval_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4060
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":4061
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":4062
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4063
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4064
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4065
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4066
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4067
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4068
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4069
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4070
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4071
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4072
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4073
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4074
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4075
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4076
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l);

  /* "scipy/special/_ufuncs.pyx":4077
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[1]) = ((void *)((char *)__pyx_k_eval_legendre));

  /* "scipy/special/_ufuncs.pyx":4078
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":4079
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[3]) = ((void *)((char *)__pyx_k_eval_legendre));

  /* "scipy/special/_ufuncs.pyx":4080
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":4081
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[5]) = ((void *)((char *)__pyx_k_eval_legendre));

  /* "scipy/special/_ufuncs.pyx":4082
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":4083
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[7]) = ((void *)((char *)__pyx_k_eval_legendre));

  /* "scipy/special/_ufuncs.pyx":4084
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":4085
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[9]) = ((void *)((char *)__pyx_k_eval_legendre));

  /* "scipy/special/_ufuncs.pyx":4086
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4087
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4088
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]
 * ufunc_eval_legendre_data[4] = &ufunc_eval_legendre_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4089
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[4] = &ufunc_eval_legendre_ptr[2*4]
 * eval_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_legendre_loops, ufunc_eval_legendre_data, ufunc_eval_legendre_types, 5, 2, 1, 0, "eval_legendre", ufunc_eval_legendre_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4090
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]
 * ufunc_eval_legendre_data[4] = &ufunc_eval_legendre_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_legendre_loops, ufunc_eval_legendre_data, ufunc_eval_legendre_types, 5, 2, 1, 0, "eval_legendre", ufunc_eval_legendre_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":4091
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]
 * ufunc_eval_legendre_data[4] = &ufunc_eval_legendre_ptr[2*4]
 * eval_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_legendre_loops, ufunc_eval_legendre_data, ufunc_eval_legendre_types, 5, 2, 1, 0, "eval_legendre", ufunc_eval_legendre_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_sh_chebyt_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types, 5, 2, 1, 0, __pyx_k_eval_legendre, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4091; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_legendre, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4091; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4097
 * cdef void *ufunc_eval_sh_chebyt_data[5]
 * cdef char ufunc_eval_sh_chebyt_types[15]
 * cdef char *ufunc_eval_sh_chebyt_doc = (             # <<<<<<<<<<<<<<
 *     "eval_sh_chebyt(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_doc = __pyx_k_eval_sh_chebyt_n_x_out_None_Eval;

  /* "scipy/special/_ufuncs.pyx":4101
 *     "\n"
 *     "Evaluate shifted Chebyshev T polynomial at a point.")
 * ufunc_eval_sh_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":4102
 *     "Evaluate shifted Chebyshev T polynomial at a point.")
 * ufunc_eval_sh_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4103
 * ufunc_eval_sh_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":4104
 * ufunc_eval_sh_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4105
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":4106
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":4107
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4108
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4109
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4110
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4111
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4112
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4113
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4114
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4115
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4116
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4117
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4118
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4119
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4120
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4121
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt_l);

  /* "scipy/special/_ufuncs.pyx":4122
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[1]) = ((void *)((char *)__pyx_k_eval_sh_chebyt));

  /* "scipy/special/_ufuncs.pyx":4123
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":4124
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[3]) = ((void *)((char *)__pyx_k_eval_sh_chebyt));

  /* "scipy/special/_ufuncs.pyx":4125
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":4126
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[5]) = ((void *)((char *)__pyx_k_eval_sh_chebyt));

  /* "scipy/special/_ufuncs.pyx":4127
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":4128
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[7]) = ((void *)((char *)__pyx_k_eval_sh_chebyt));

  /* "scipy/special/_ufuncs.pyx":4129
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":4130
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[9]) = ((void *)((char *)__pyx_k_eval_sh_chebyt));

  /* "scipy/special/_ufuncs.pyx":4131
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4132
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4133
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]
 * ufunc_eval_sh_chebyt_data[4] = &ufunc_eval_sh_chebyt_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4134
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[4] = &ufunc_eval_sh_chebyt_ptr[2*4]
 * eval_sh_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyt_loops, ufunc_eval_sh_chebyt_data, ufunc_eval_sh_chebyt_types, 5, 2, 1, 0, "eval_sh_chebyt", ufunc_eval_sh_chebyt_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4135
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]
 * ufunc_eval_sh_chebyt_data[4] = &ufunc_eval_sh_chebyt_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_sh_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyt_loops, ufunc_eval_sh_chebyt_data, ufunc_eval_sh_chebyt_types, 5, 2, 1, 0, "eval_sh_chebyt", ufunc_eval_sh_chebyt_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":4136
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]
 * ufunc_eval_sh_chebyt_data[4] = &ufunc_eval_sh_chebyt_ptr[2*4]
 * eval_sh_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyt_loops, ufunc_eval_sh_chebyt_data, ufunc_eval_sh_chebyt_types, 5, 2, 1, 0, "eval_sh_chebyt", ufunc_eval_sh_chebyt_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_sh_chebyu_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types, 5, 2, 1, 0, __pyx_k_eval_sh_chebyt, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_sh_chebyt, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4142
 * cdef void *ufunc_eval_sh_chebyu_data[5]
 * cdef char ufunc_eval_sh_chebyu_types[15]
 * cdef char *ufunc_eval_sh_chebyu_doc = (             # <<<<<<<<<<<<<<
 *     "eval_sh_chebyu(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_doc = __pyx_k_eval_sh_chebyu_n_x_out_None_Eval;

  /* "scipy/special/_ufuncs.pyx":4146
 *     "\n"
 *     "Evaluate shifted Chebyshev U polynomial at a point.")
 * ufunc_eval_sh_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":4147
 *     "Evaluate shifted Chebyshev U polynomial at a point.")
 * ufunc_eval_sh_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4148
 * ufunc_eval_sh_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":4149
 * ufunc_eval_sh_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4150
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":4151
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":4152
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4153
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4154
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4155
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4156
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4157
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4158
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4159
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4160
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4161
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4162
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4163
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4164
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4165
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4166
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu_l);

  /* "scipy/special/_ufuncs.pyx":4167
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[1]) = ((void *)((char *)__pyx_k_eval_sh_chebyu));

  /* "scipy/special/_ufuncs.pyx":4168
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":4169
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[3]) = ((void *)((char *)__pyx_k_eval_sh_chebyu));

  /* "scipy/special/_ufuncs.pyx":4170
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":4171
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[5]) = ((void *)((char *)__pyx_k_eval_sh_chebyu));

  /* "scipy/special/_ufuncs.pyx":4172
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":4173
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[7]) = ((void *)((char *)__pyx_k_eval_sh_chebyu));

  /* "scipy/special/_ufuncs.pyx":4174
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":4175
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[9]) = ((void *)((char *)__pyx_k_eval_sh_chebyu));

  /* "scipy/special/_ufuncs.pyx":4176
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4177
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4178
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]
 * ufunc_eval_sh_chebyu_data[4] = &ufunc_eval_sh_chebyu_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4179
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[4] = &ufunc_eval_sh_chebyu_ptr[2*4]
 * eval_sh_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyu_loops, ufunc_eval_sh_chebyu_data, ufunc_eval_sh_chebyu_types, 5, 2, 1, 0, "eval_sh_chebyu", ufunc_eval_sh_chebyu_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4180
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]
 * ufunc_eval_sh_chebyu_data[4] = &ufunc_eval_sh_chebyu_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_sh_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyu_loops, ufunc_eval_sh_chebyu_data, ufunc_eval_sh_chebyu_types, 5, 2, 1, 0, "eval_sh_chebyu", ufunc_eval_sh_chebyu_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":4181
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]
 * ufunc_eval_sh_chebyu_data[4] = &ufunc_eval_sh_chebyu_ptr[2*4]
 * eval_sh_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyu_loops, ufunc_eval_sh_chebyu_data, ufunc_eval_sh_chebyu_types, 5, 2, 1, 0, "eval_sh_chebyu", ufunc_eval_sh_chebyu_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_sh_jacobi_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types, 5, 2, 1, 0, __pyx_k_eval_sh_chebyu, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_sh_chebyu, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4187
 * cdef void *ufunc_eval_sh_jacobi_data[5]
 * cdef char ufunc_eval_sh_jacobi_types[25]
 * cdef char *ufunc_eval_sh_jacobi_doc = (             # <<<<<<<<<<<<<<
 *     "eval_sh_jacobi(n, p, q, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_doc = __pyx_k_eval_sh_jacobi_n_p_q_x_out_None;

  /* "scipy/special/_ufuncs.pyx":4191
 *     "\n"
 *     "Evaluate shifted Jacobi polynomial at a point.")
 * ufunc_eval_sh_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_lddd__As_lddd_d);

  /* "scipy/special/_ufuncs.pyx":4192
 *     "Evaluate shifted Jacobi polynomial at a point.")
 * ufunc_eval_sh_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d
 * ufunc_eval_sh_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":4193
 * ufunc_eval_sh_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d
 * ufunc_eval_sh_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F);

  /* "scipy/special/_ufuncs.pyx":4194
 * ufunc_eval_sh_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":4195
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D);

  /* "scipy/special/_ufuncs.pyx":4196
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":4197
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4198
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4199
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4200
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4201
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4202
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4203
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4204
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4205
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[9]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4206
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[10]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4207
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[11]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4208
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[12]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4209
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[13]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4210
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[14]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4211
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4212
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4213
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4214
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[18]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4215
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[19]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4216
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[20]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4217
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[21]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4218
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[22]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4219
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[23]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4220
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[24]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4221
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi_l);

  /* "scipy/special/_ufuncs.pyx":4222
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[1]) = ((void *)((char *)__pyx_k_eval_sh_jacobi));

  /* "scipy/special/_ufuncs.pyx":4223
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":4224
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[3]) = ((void *)((char *)__pyx_k_eval_sh_jacobi));

  /* "scipy/special/_ufuncs.pyx":4225
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":4226
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[5]) = ((void *)((char *)__pyx_k_eval_sh_jacobi));

  /* "scipy/special/_ufuncs.pyx":4227
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":4228
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[7]) = ((void *)((char *)__pyx_k_eval_sh_jacobi));

  /* "scipy/special/_ufuncs.pyx":4229
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":4230
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[9]) = ((void *)((char *)__pyx_k_eval_sh_jacobi));

  /* "scipy/special/_ufuncs.pyx":4231
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4232
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4233
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]
 * ufunc_eval_sh_jacobi_data[4] = &ufunc_eval_sh_jacobi_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4234
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[4] = &ufunc_eval_sh_jacobi_ptr[2*4]
 * eval_sh_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_jacobi_loops, ufunc_eval_sh_jacobi_data, ufunc_eval_sh_jacobi_types, 5, 4, 1, 0, "eval_sh_jacobi", ufunc_eval_sh_jacobi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4235
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]
 * ufunc_eval_sh_jacobi_data[4] = &ufunc_eval_sh_jacobi_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_sh_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_jacobi_loops, ufunc_eval_sh_jacobi_data, ufunc_eval_sh_jacobi_types, 5, 4, 1, 0, "eval_sh_jacobi", ufunc_eval_sh_jacobi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":4236
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]
 * ufunc_eval_sh_jacobi_data[4] = &ufunc_eval_sh_jacobi_ptr[2*4]
 * eval_sh_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_jacobi_loops, ufunc_eval_sh_jacobi_data, ufunc_eval_sh_jacobi_types, 5, 4, 1, 0, "eval_sh_jacobi", ufunc_eval_sh_jacobi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_sh_legendre_loops[5]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types, 5, 4, 1, 0, __pyx_k_eval_sh_jacobi, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_sh_jacobi, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4242
 * cdef void *ufunc_eval_sh_legendre_data[5]
 * cdef char ufunc_eval_sh_legendre_types[15]
 * cdef char *ufunc_eval_sh_legendre_doc = (             # <<<<<<<<<<<<<<
 *     "eval_sh_legendre(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_doc = __pyx_k_eval_sh_legendre_n_x_out_None_Ev;

  /* "scipy/special/_ufuncs.pyx":4246
 *     "\n"
 *     "Evaluate shifted Legendre polynomial at a point.")
 * ufunc_eval_sh_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":4247
 *     "Evaluate shifted Legendre polynomial at a point.")
 * ufunc_eval_sh_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4248
 * ufunc_eval_sh_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":4249
 * ufunc_eval_sh_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4250
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":4251
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":4252
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4253
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4254
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4255
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4256
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4257
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4258
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4259
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4260
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4261
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4262
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4263
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4264
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4265
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4266
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre_l);

  /* "scipy/special/_ufuncs.pyx":4267
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[1]) = ((void *)((char *)__pyx_k_eval_sh_legendre));

  /* "scipy/special/_ufuncs.pyx":4268
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":4269
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[3]) = ((void *)((char *)__pyx_k_eval_sh_legendre));

  /* "scipy/special/_ufuncs.pyx":4270
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":4271
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[5]) = ((void *)((char *)__pyx_k_eval_sh_legendre));

  /* "scipy/special/_ufuncs.pyx":4272
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":4273
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[7]) = ((void *)((char *)__pyx_k_eval_sh_legendre));

  /* "scipy/special/_ufuncs.pyx":4274
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":4275
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[9]) = ((void *)((char *)__pyx_k_eval_sh_legendre));

  /* "scipy/special/_ufuncs.pyx":4276
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4277
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4278
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]
 * ufunc_eval_sh_legendre_data[4] = &ufunc_eval_sh_legendre_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4279
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[4] = &ufunc_eval_sh_legendre_ptr[2*4]
 * eval_sh_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_legendre_loops, ufunc_eval_sh_legendre_data, ufunc_eval_sh_legendre_types, 5, 2, 1, 0, "eval_sh_legendre", ufunc_eval_sh_legendre_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4280
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]
 * ufunc_eval_sh_legendre_data[4] = &ufunc_eval_sh_legendre_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_sh_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_legendre_loops, ufunc_eval_sh_legendre_data, ufunc_eval_sh_legendre_types, 5, 2, 1, 0, "eval_sh_legendre", ufunc_eval_sh_legendre_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":4281
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]
 * ufunc_eval_sh_legendre_data[4] = &ufunc_eval_sh_legendre_ptr[2*4]
 * eval_sh_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_legendre_loops, ufunc_eval_sh_legendre_data, ufunc_eval_sh_legendre_types, 5, 2, 1, 0, "eval_sh_legendre", ufunc_eval_sh_legendre_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_exp1_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types, 5, 2, 1, 0, __pyx_k_eval_sh_legendre, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_sh_legendre, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4287
 * cdef void *ufunc_exp1_data[4]
 * cdef char ufunc_exp1_types[8]
 * cdef char *ufunc_exp1_doc = (             # <<<<<<<<<<<<<<
 *     "exp1(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_doc = __pyx_k_exp1_z_Exponential_integral_E_1;

  /* "scipy/special/_ufuncs.pyx":4295
 *     "\n"
 *     "    integral(exp(-z*t)/t, t=1..inf).")
 * ufunc_exp1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_exp1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4296
 *     "    integral(exp(-z*t)/t, t=1..inf).")
 * ufunc_exp1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4297
 * ufunc_exp1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_exp1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4298
 * ufunc_exp1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[0] = <char>NPY_FLOAT
 * ufunc_exp1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4299
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_exp1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[1] = <char>NPY_FLOAT
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4300
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_exp1_types[0] = <char>NPY_FLOAT
 * ufunc_exp1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4301
 * ufunc_exp1_types[0] = <char>NPY_FLOAT
 * ufunc_exp1_types[1] = <char>NPY_FLOAT
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4302
 * ufunc_exp1_types[1] = <char>NPY_FLOAT
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4303
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4304
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4305
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4306
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4307
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[0]) = ((void *)exp1_wrap);

  /* "scipy/special/_ufuncs.pyx":4308
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[1]) = ((void *)((char *)__pyx_k_exp1));

  /* "scipy/special/_ufuncs.pyx":4309
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[2]) = ((void *)exp1_wrap);

  /* "scipy/special/_ufuncs.pyx":4310
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[3]) = ((void *)((char *)__pyx_k_exp1));

  /* "scipy/special/_ufuncs.pyx":4311
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[4]) = ((void *)cexp1_wrap);

  /* "scipy/special/_ufuncs.pyx":4312
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[5]) = ((void *)((char *)__pyx_k_exp1));

  /* "scipy/special/_ufuncs.pyx":4313
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[6]) = ((void *)cexp1_wrap);

  /* "scipy/special/_ufuncs.pyx":4314
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")             # <<<<<<<<<<<<<<
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[7]) = ((void *)((char *)__pyx_k_exp1));

  /* "scipy/special/_ufuncs.pyx":4315
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4316
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]
 * ufunc_exp1_data[3] = &ufunc_exp1_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4317
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_exp1_data[3] = &ufunc_exp1_ptr[2*3]
 * exp1 = np.PyUFunc_FromFuncAndData(ufunc_exp1_loops, ufunc_exp1_data, ufunc_exp1_types, 4, 1, 1, 0, "exp1", ufunc_exp1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4318
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]
 * ufunc_exp1_data[3] = &ufunc_exp1_ptr[2*3]             # <<<<<<<<<<<<<<
 * exp1 = np.PyUFunc_FromFuncAndData(ufunc_exp1_loops, ufunc_exp1_data, ufunc_exp1_types, 4, 1, 1, 0, "exp1", ufunc_exp1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4319
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]
 * ufunc_exp1_data[3] = &ufunc_exp1_ptr[2*3]
 * exp1 = np.PyUFunc_FromFuncAndData(ufunc_exp1_loops, ufunc_exp1_data, ufunc_exp1_types, 4, 1, 1, 0, "exp1", ufunc_exp1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_exp10_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types, 4, 1, 1, 0, __pyx_k_exp1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_exp1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4325
 * cdef void *ufunc_exp10_data[2]
 * cdef char ufunc_exp10_types[4]
 * cdef char *ufunc_exp10_doc = (             # <<<<<<<<<<<<<<
 *     "exp10(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_doc = __pyx_k_exp10_x_10_x;

  /* "scipy/special/_ufuncs.pyx":4329
 *     "\n"
 *     "10**x")
 * ufunc_exp10_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_exp10_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp10_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4330
 *     "10**x")
 * ufunc_exp10_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp10_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_exp10_types[0] = <char>NPY_FLOAT
 * ufunc_exp10_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4331
 * ufunc_exp10_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp10_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp10_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp10_types[1] = <char>NPY_FLOAT
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4332
 * ufunc_exp10_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp10_types[0] = <char>NPY_FLOAT
 * ufunc_exp10_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4333
 * ufunc_exp10_types[0] = <char>NPY_FLOAT
 * ufunc_exp10_types[1] = <char>NPY_FLOAT
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4334
 * ufunc_exp10_types[1] = <char>NPY_FLOAT
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4335
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10             # <<<<<<<<<<<<<<
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[0]) = ((void *)exp10);

  /* "scipy/special/_ufuncs.pyx":4336
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")             # <<<<<<<<<<<<<<
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[1]) = ((void *)((char *)__pyx_k_exp10));

  /* "scipy/special/_ufuncs.pyx":4337
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10             # <<<<<<<<<<<<<<
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[2]) = ((void *)exp10);

  /* "scipy/special/_ufuncs.pyx":4338
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")             # <<<<<<<<<<<<<<
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]
 * ufunc_exp10_data[1] = &ufunc_exp10_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[3]) = ((void *)((char *)__pyx_k_exp10));

  /* "scipy/special/_ufuncs.pyx":4339
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_exp10_data[1] = &ufunc_exp10_ptr[2*1]
 * exp10 = np.PyUFunc_FromFuncAndData(ufunc_exp10_loops, ufunc_exp10_data, ufunc_exp10_types, 2, 1, 1, 0, "exp10", ufunc_exp10_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4340
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]
 * ufunc_exp10_data[1] = &ufunc_exp10_ptr[2*1]             # <<<<<<<<<<<<<<
 * exp10 = np.PyUFunc_FromFuncAndData(ufunc_exp10_loops, ufunc_exp10_data, ufunc_exp10_types, 2, 1, 1, 0, "exp10", ufunc_exp10_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4341
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]
 * ufunc_exp10_data[1] = &ufunc_exp10_ptr[2*1]
 * exp10 = np.PyUFunc_FromFuncAndData(ufunc_exp10_loops, ufunc_exp10_data, ufunc_exp10_types, 2, 1, 1, 0, "exp10", ufunc_exp10_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_exp2_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types, 2, 1, 1, 0, __pyx_k_exp10, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_exp10, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4347
 * cdef void *ufunc_exp2_data[2]
 * cdef char ufunc_exp2_types[4]
 * cdef char *ufunc_exp2_doc = (             # <<<<<<<<<<<<<<
 *     "exp2(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_doc = __pyx_k_exp2_x_2_x;

  /* "scipy/special/_ufuncs.pyx":4351
 *     "\n"
 *     "2**x")
 * ufunc_exp2_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_exp2_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4352
 *     "2**x")
 * ufunc_exp2_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp2_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_exp2_types[0] = <char>NPY_FLOAT
 * ufunc_exp2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4353
 * ufunc_exp2_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp2_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp2_types[1] = <char>NPY_FLOAT
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4354
 * ufunc_exp2_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp2_types[0] = <char>NPY_FLOAT
 * ufunc_exp2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4355
 * ufunc_exp2_types[0] = <char>NPY_FLOAT
 * ufunc_exp2_types[1] = <char>NPY_FLOAT
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4356
 * ufunc_exp2_types[1] = <char>NPY_FLOAT
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4357
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2             # <<<<<<<<<<<<<<
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[0]) = ((void *)exp2);

  /* "scipy/special/_ufuncs.pyx":4358
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")             # <<<<<<<<<<<<<<
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[1]) = ((void *)((char *)__pyx_k_exp2));

  /* "scipy/special/_ufuncs.pyx":4359
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2             # <<<<<<<<<<<<<<
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[2]) = ((void *)exp2);

  /* "scipy/special/_ufuncs.pyx":4360
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")             # <<<<<<<<<<<<<<
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]
 * ufunc_exp2_data[1] = &ufunc_exp2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[3]) = ((void *)((char *)__pyx_k_exp2));

  /* "scipy/special/_ufuncs.pyx":4361
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_exp2_data[1] = &ufunc_exp2_ptr[2*1]
 * exp2 = np.PyUFunc_FromFuncAndData(ufunc_exp2_loops, ufunc_exp2_data, ufunc_exp2_types, 2, 1, 1, 0, "exp2", ufunc_exp2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4362
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]
 * ufunc_exp2_data[1] = &ufunc_exp2_ptr[2*1]             # <<<<<<<<<<<<<<
 * exp2 = np.PyUFunc_FromFuncAndData(ufunc_exp2_loops, ufunc_exp2_data, ufunc_exp2_types, 2, 1, 1, 0, "exp2", ufunc_exp2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4363
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]
 * ufunc_exp2_data[1] = &ufunc_exp2_ptr[2*1]
 * exp2 = np.PyUFunc_FromFuncAndData(ufunc_exp2_loops, ufunc_exp2_data, ufunc_exp2_types, 2, 1, 1, 0, "exp2", ufunc_exp2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_expi_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types, 2, 1, 1, 0, __pyx_k_exp2, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4363; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_exp2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4363; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4369
 * cdef void *ufunc_expi_data[4]
 * cdef char ufunc_expi_types[8]
 * cdef char *ufunc_expi_doc = (             # <<<<<<<<<<<<<<
 *     "expi(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_doc = __pyx_k_expi_x_Exponential_integral_Ei_D;

  /* "scipy/special/_ufuncs.pyx":4379
 *     "\n"
 *     "See `expn` for a different exponential integral.")
 * ufunc_expi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_expi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4380
 *     "See `expn` for a different exponential integral.")
 * ufunc_expi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_expi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4381
 * ufunc_expi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_expi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4382
 * ufunc_expi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_expi_types[0] = <char>NPY_FLOAT
 * ufunc_expi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4383
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expi_types[1] = <char>NPY_FLOAT
 * ufunc_expi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4384
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expi_types[0] = <char>NPY_FLOAT
 * ufunc_expi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expi_types[2] = <char>NPY_DOUBLE
 * ufunc_expi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4385
 * ufunc_expi_types[0] = <char>NPY_FLOAT
 * ufunc_expi_types[1] = <char>NPY_FLOAT
 * ufunc_expi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expi_types[3] = <char>NPY_DOUBLE
 * ufunc_expi_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4386
 * ufunc_expi_types[1] = <char>NPY_FLOAT
 * ufunc_expi_types[2] = <char>NPY_DOUBLE
 * ufunc_expi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expi_types[4] = <char>NPY_CFLOAT
 * ufunc_expi_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4387
 * ufunc_expi_types[2] = <char>NPY_DOUBLE
 * ufunc_expi_types[3] = <char>NPY_DOUBLE
 * ufunc_expi_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_expi_types[5] = <char>NPY_CFLOAT
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4388
 * ufunc_expi_types[3] = <char>NPY_DOUBLE
 * ufunc_expi_types[4] = <char>NPY_CFLOAT
 * ufunc_expi_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4389
 * ufunc_expi_types[4] = <char>NPY_CFLOAT
 * ufunc_expi_types[5] = <char>NPY_CFLOAT
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4390
 * ufunc_expi_types[5] = <char>NPY_CFLOAT
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4391
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[0]) = ((void *)expi_wrap);

  /* "scipy/special/_ufuncs.pyx":4392
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[1]) = ((void *)((char *)__pyx_k_expi));

  /* "scipy/special/_ufuncs.pyx":4393
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[2]) = ((void *)expi_wrap);

  /* "scipy/special/_ufuncs.pyx":4394
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[3]) = ((void *)((char *)__pyx_k_expi));

  /* "scipy/special/_ufuncs.pyx":4395
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[4]) = ((void *)cexpi_wrap);

  /* "scipy/special/_ufuncs.pyx":4396
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[5]) = ((void *)((char *)__pyx_k_expi));

  /* "scipy/special/_ufuncs.pyx":4397
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[6]) = ((void *)cexpi_wrap);

  /* "scipy/special/_ufuncs.pyx":4398
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")             # <<<<<<<<<<<<<<
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[7]) = ((void *)((char *)__pyx_k_expi));

  /* "scipy/special/_ufuncs.pyx":4399
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4400
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]
 * ufunc_expi_data[3] = &ufunc_expi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4401
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_expi_data[3] = &ufunc_expi_ptr[2*3]
 * expi = np.PyUFunc_FromFuncAndData(ufunc_expi_loops, ufunc_expi_data, ufunc_expi_types, 4, 1, 1, 0, "expi", ufunc_expi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4402
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]
 * ufunc_expi_data[3] = &ufunc_expi_ptr[2*3]             # <<<<<<<<<<<<<<
 * expi = np.PyUFunc_FromFuncAndData(ufunc_expi_loops, ufunc_expi_data, ufunc_expi_types, 4, 1, 1, 0, "expi", ufunc_expi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4403
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]
 * ufunc_expi_data[3] = &ufunc_expi_ptr[2*3]
 * expi = np.PyUFunc_FromFuncAndData(ufunc_expi_loops, ufunc_expi_data, ufunc_expi_types, 4, 1, 1, 0, "expi", ufunc_expi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_expit_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types, 4, 1, 1, 0, __pyx_k_expi, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expi, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4409
 * cdef void *ufunc_expit_data[3]
 * cdef char ufunc_expit_types[6]
 * cdef char *ufunc_expit_doc = (             # <<<<<<<<<<<<<<
 *     "expit(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_doc = __pyx_k_expit_x_Expit_ufunc_for_ndarrays;

  /* "scipy/special/_ufuncs.pyx":4435
 *     "\n"
 *     ".. versionadded:: 0.10.0")
 * ufunc_expit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_expit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_f_f__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4436
 *     ".. versionadded:: 0.10.0")
 * ufunc_expit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f
 * ufunc_expit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_expit_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4437
 * ufunc_expit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f
 * ufunc_expit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g             # <<<<<<<<<<<<<<
 * ufunc_expit_types[0] = <char>NPY_FLOAT
 * ufunc_expit_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_g_g__As_g_g);

  /* "scipy/special/_ufuncs.pyx":4438
 * ufunc_expit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_expit_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expit_types[1] = <char>NPY_FLOAT
 * ufunc_expit_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4439
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_expit_types[0] = <char>NPY_FLOAT
 * ufunc_expit_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expit_types[2] = <char>NPY_DOUBLE
 * ufunc_expit_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4440
 * ufunc_expit_types[0] = <char>NPY_FLOAT
 * ufunc_expit_types[1] = <char>NPY_FLOAT
 * ufunc_expit_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expit_types[3] = <char>NPY_DOUBLE
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4441
 * ufunc_expit_types[1] = <char>NPY_FLOAT
 * ufunc_expit_types[2] = <char>NPY_DOUBLE
 * ufunc_expit_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4442
 * ufunc_expit_types[2] = <char>NPY_DOUBLE
 * ufunc_expit_types[3] = <char>NPY_DOUBLE
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[4]) = ((char)NPY_LONGDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4443
 * ufunc_expit_types[3] = <char>NPY_DOUBLE
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[5]) = ((char)NPY_LONGDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4444
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*1] = <void*>_func_expit
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[0]) = ((void *)expitf);

  /* "scipy/special/_ufuncs.pyx":4445
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*1] = <void*>_func_expit
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[1]) = ((void *)((char *)__pyx_k_expit));

  /* "scipy/special/_ufuncs.pyx":4446
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*1] = <void*>_func_expit             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[2]) = ((void *)expit);

  /* "scipy/special/_ufuncs.pyx":4447
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*1] = <void*>_func_expit
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[3]) = ((void *)((char *)__pyx_k_expit));

  /* "scipy/special/_ufuncs.pyx":4448
 * ufunc_expit_ptr[2*1] = <void*>_func_expit
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[4]) = ((void *)expitl);

  /* "scipy/special/_ufuncs.pyx":4449
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")             # <<<<<<<<<<<<<<
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[5]) = ((void *)((char *)__pyx_k_expit));

  /* "scipy/special/_ufuncs.pyx":4450
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]
 * ufunc_expit_data[2] = &ufunc_expit_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4451
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_expit_data[2] = &ufunc_expit_ptr[2*2]
 * expit = np.PyUFunc_FromFuncAndData(ufunc_expit_loops, ufunc_expit_data, ufunc_expit_types, 3, 1, 1, 0, "expit", ufunc_expit_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4452
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]
 * ufunc_expit_data[2] = &ufunc_expit_ptr[2*2]             # <<<<<<<<<<<<<<
 * expit = np.PyUFunc_FromFuncAndData(ufunc_expit_loops, ufunc_expit_data, ufunc_expit_types, 3, 1, 1, 0, "expit", ufunc_expit_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4453
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]
 * ufunc_expit_data[2] = &ufunc_expit_ptr[2*2]
 * expit = np.PyUFunc_FromFuncAndData(ufunc_expit_loops, ufunc_expit_data, ufunc_expit_types, 3, 1, 1, 0, "expit", ufunc_expit_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_expm1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types, 3, 1, 1, 0, __pyx_k_expit, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expit, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4459
 * cdef void *ufunc_expm1_data[2]
 * cdef char ufunc_expm1_types[4]
 * cdef char *ufunc_expm1_doc = (             # <<<<<<<<<<<<<<
 *     "expm1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_doc = __pyx_k_expm1_x_exp_x_1_for_use_when_x_i;

  /* "scipy/special/_ufuncs.pyx":4463
 *     "\n"
 *     "exp(x) - 1 for use when `x` is near zero.")
 * ufunc_expm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_expm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expm1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4464
 *     "exp(x) - 1 for use when `x` is near zero.")
 * ufunc_expm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_expm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[0] = <char>NPY_FLOAT
 * ufunc_expm1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4465
 * ufunc_expm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_expm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expm1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[1] = <char>NPY_FLOAT
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4466
 * ufunc_expm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expm1_types[0] = <char>NPY_FLOAT
 * ufunc_expm1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4467
 * ufunc_expm1_types[0] = <char>NPY_FLOAT
 * ufunc_expm1_types[1] = <char>NPY_FLOAT
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4468
 * ufunc_expm1_types[1] = <char>NPY_FLOAT
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4469
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[0]) = ((void *)expm1);

  /* "scipy/special/_ufuncs.pyx":4470
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[1]) = ((void *)((char *)__pyx_k_expm1));

  /* "scipy/special/_ufuncs.pyx":4471
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[2]) = ((void *)expm1);

  /* "scipy/special/_ufuncs.pyx":4472
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")             # <<<<<<<<<<<<<<
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[3]) = ((void *)((char *)__pyx_k_expm1));

  /* "scipy/special/_ufuncs.pyx":4473
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]
 * expm1 = np.PyUFunc_FromFuncAndData(ufunc_expm1_loops, ufunc_expm1_data, ufunc_expm1_types, 2, 1, 1, 0, "expm1", ufunc_expm1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4474
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]             # <<<<<<<<<<<<<<
 * expm1 = np.PyUFunc_FromFuncAndData(ufunc_expm1_loops, ufunc_expm1_data, ufunc_expm1_types, 2, 1, 1, 0, "expm1", ufunc_expm1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4475
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]
 * expm1 = np.PyUFunc_FromFuncAndData(ufunc_expm1_loops, ufunc_expm1_data, ufunc_expm1_types, 2, 1, 1, 0, "expm1", ufunc_expm1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_expn_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types, 2, 1, 1, 0, __pyx_k_expm1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expm1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4481
 * cdef void *ufunc_expn_data[3]
 * cdef char ufunc_expn_types[9]
 * cdef char *ufunc_expn_doc = (             # <<<<<<<<<<<<<<
 *     "expn(n, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_doc = __pyx_k_expn_n_x_Exponential_integral_E;

  /* "scipy/special/_ufuncs.pyx":4490
 *     "\n"
 *     "    integral(exp(-x*t) / t**n, t=1..inf).")
 * ufunc_expn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_expn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":4491
 *     "    integral(exp(-x*t) / t**n, t=1..inf).")
 * ufunc_expn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_expn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_expn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4492
 * ufunc_expn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_expn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_expn_types[0] = <char>NPY_LONG
 * ufunc_expn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4493
 * ufunc_expn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_expn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_expn_types[1] = <char>NPY_DOUBLE
 * ufunc_expn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":4494
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_expn_types[0] = <char>NPY_LONG
 * ufunc_expn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_types[2] = <char>NPY_DOUBLE
 * ufunc_expn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4495
 * ufunc_expn_types[0] = <char>NPY_LONG
 * ufunc_expn_types[1] = <char>NPY_DOUBLE
 * ufunc_expn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_types[3] = <char>NPY_FLOAT
 * ufunc_expn_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4496
 * ufunc_expn_types[1] = <char>NPY_DOUBLE
 * ufunc_expn_types[2] = <char>NPY_DOUBLE
 * ufunc_expn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expn_types[4] = <char>NPY_FLOAT
 * ufunc_expn_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4497
 * ufunc_expn_types[2] = <char>NPY_DOUBLE
 * ufunc_expn_types[3] = <char>NPY_FLOAT
 * ufunc_expn_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expn_types[5] = <char>NPY_FLOAT
 * ufunc_expn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4498
 * ufunc_expn_types[3] = <char>NPY_FLOAT
 * ufunc_expn_types[4] = <char>NPY_FLOAT
 * ufunc_expn_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expn_types[6] = <char>NPY_DOUBLE
 * ufunc_expn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4499
 * ufunc_expn_types[4] = <char>NPY_FLOAT
 * ufunc_expn_types[5] = <char>NPY_FLOAT
 * ufunc_expn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_types[7] = <char>NPY_DOUBLE
 * ufunc_expn_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4500
 * ufunc_expn_types[5] = <char>NPY_FLOAT
 * ufunc_expn_types[6] = <char>NPY_DOUBLE
 * ufunc_expn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_types[8] = <char>NPY_DOUBLE
 * ufunc_expn_ptr[2*0] = <void*>_func_expn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4501
 * ufunc_expn_types[6] = <char>NPY_DOUBLE
 * ufunc_expn_types[7] = <char>NPY_DOUBLE
 * ufunc_expn_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*0] = <void*>_func_expn
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4502
 * ufunc_expn_types[7] = <char>NPY_DOUBLE
 * ufunc_expn_types[8] = <char>NPY_DOUBLE
 * ufunc_expn_ptr[2*0] = <void*>_func_expn             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[0]) = ((void *)expn);

  /* "scipy/special/_ufuncs.pyx":4503
 * ufunc_expn_types[8] = <char>NPY_DOUBLE
 * ufunc_expn_ptr[2*0] = <void*>_func_expn
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[1]) = ((void *)((char *)__pyx_k_expn));

  /* "scipy/special/_ufuncs.pyx":4504
 * ufunc_expn_ptr[2*0] = <void*>_func_expn
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_expn_unsafe);

  /* "scipy/special/_ufuncs.pyx":4505
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[3]) = ((void *)((char *)__pyx_k_expn));

  /* "scipy/special/_ufuncs.pyx":4506
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_expn_unsafe);

  /* "scipy/special/_ufuncs.pyx":4507
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")             # <<<<<<<<<<<<<<
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[5]) = ((void *)((char *)__pyx_k_expn));

  /* "scipy/special/_ufuncs.pyx":4508
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]
 * ufunc_expn_data[2] = &ufunc_expn_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4509
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_expn_data[2] = &ufunc_expn_ptr[2*2]
 * expn = np.PyUFunc_FromFuncAndData(ufunc_expn_loops, ufunc_expn_data, ufunc_expn_types, 3, 2, 1, 0, "expn", ufunc_expn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4510
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]
 * ufunc_expn_data[2] = &ufunc_expn_ptr[2*2]             # <<<<<<<<<<<<<<
 * expn = np.PyUFunc_FromFuncAndData(ufunc_expn_loops, ufunc_expn_data, ufunc_expn_types, 3, 2, 1, 0, "expn", ufunc_expn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4511
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]
 * ufunc_expn_data[2] = &ufunc_expn_ptr[2*2]
 * expn = np.PyUFunc_FromFuncAndData(ufunc_expn_loops, ufunc_expn_data, ufunc_expn_types, 3, 2, 1, 0, "expn", ufunc_expn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_exprel_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types, 3, 2, 1, 0, __pyx_k_expn, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4511; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expn, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4511; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4517
 * cdef void *ufunc_exprel_data[2]
 * cdef char ufunc_exprel_types[4]
 * cdef char *ufunc_exprel_doc = (             # <<<<<<<<<<<<<<
 *     "exprel(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_doc = __pyx_k_exprel_x_Relative_error_exponent;

  /* "scipy/special/_ufuncs.pyx":4537
 *     "\n"
 *     ".. versionadded:: 0.17.0")
 * ufunc_exprel_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_exprel_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exprel_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4538
 *     ".. versionadded:: 0.17.0")
 * ufunc_exprel_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exprel_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_exprel_types[0] = <char>NPY_FLOAT
 * ufunc_exprel_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4539
 * ufunc_exprel_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exprel_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exprel_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exprel_types[1] = <char>NPY_FLOAT
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4540
 * ufunc_exprel_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exprel_types[0] = <char>NPY_FLOAT
 * ufunc_exprel_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4541
 * ufunc_exprel_types[0] = <char>NPY_FLOAT
 * ufunc_exprel_types[1] = <char>NPY_FLOAT
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4542
 * ufunc_exprel_types[1] = <char>NPY_FLOAT
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4543
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel             # <<<<<<<<<<<<<<
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_exprel_exprel);

  /* "scipy/special/_ufuncs.pyx":4544
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")             # <<<<<<<<<<<<<<
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[1]) = ((void *)((char *)__pyx_k_exprel));

  /* "scipy/special/_ufuncs.pyx":4545
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel             # <<<<<<<<<<<<<<
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_exprel_exprel);

  /* "scipy/special/_ufuncs.pyx":4546
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")             # <<<<<<<<<<<<<<
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]
 * ufunc_exprel_data[1] = &ufunc_exprel_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[3]) = ((void *)((char *)__pyx_k_exprel));

  /* "scipy/special/_ufuncs.pyx":4547
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_exprel_data[1] = &ufunc_exprel_ptr[2*1]
 * exprel = np.PyUFunc_FromFuncAndData(ufunc_exprel_loops, ufunc_exprel_data, ufunc_exprel_types, 2, 1, 1, 0, "exprel", ufunc_exprel_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4548
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]
 * ufunc_exprel_data[1] = &ufunc_exprel_ptr[2*1]             # <<<<<<<<<<<<<<
 * exprel = np.PyUFunc_FromFuncAndData(ufunc_exprel_loops, ufunc_exprel_data, ufunc_exprel_types, 2, 1, 1, 0, "exprel", ufunc_exprel_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4549
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]
 * ufunc_exprel_data[1] = &ufunc_exprel_ptr[2*1]
 * exprel = np.PyUFunc_FromFuncAndData(ufunc_exprel_loops, ufunc_exprel_data, ufunc_exprel_types, 2, 1, 1, 0, "exprel", ufunc_exprel_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fdtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types, 2, 1, 1, 0, __pyx_k_exprel, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4549; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_exprel, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4549; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4555
 * cdef void *ufunc_fdtr_data[2]
 * cdef char ufunc_fdtr_types[8]
 * cdef char *ufunc_fdtr_doc = (             # <<<<<<<<<<<<<<
 *     "fdtr(dfn, dfd, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_doc = __pyx_k_fdtr_dfn_dfd_x_F_cumulative_dist;

  /* "scipy/special/_ufuncs.pyx":4565
 *     "are random variables having Chi square distributions with dfn and\n"
 *     "dfd degrees of freedom, respectively.")
 * ufunc_fdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_fdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4566
 *     "dfd degrees of freedom, respectively.")
 * ufunc_fdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4567
 * ufunc_fdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4568
 * ufunc_fdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4569
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4570
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4571
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4572
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4573
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4574
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4575
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr             # <<<<<<<<<<<<<<
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[0]) = ((void *)fdtr);

  /* "scipy/special/_ufuncs.pyx":4576
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")             # <<<<<<<<<<<<<<
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[1]) = ((void *)((char *)__pyx_k_fdtr));

  /* "scipy/special/_ufuncs.pyx":4577
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr             # <<<<<<<<<<<<<<
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[2]) = ((void *)fdtr);

  /* "scipy/special/_ufuncs.pyx":4578
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")             # <<<<<<<<<<<<<<
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]
 * ufunc_fdtr_data[1] = &ufunc_fdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[3]) = ((void *)((char *)__pyx_k_fdtr));

  /* "scipy/special/_ufuncs.pyx":4579
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fdtr_data[1] = &ufunc_fdtr_ptr[2*1]
 * fdtr = np.PyUFunc_FromFuncAndData(ufunc_fdtr_loops, ufunc_fdtr_data, ufunc_fdtr_types, 2, 3, 1, 0, "fdtr", ufunc_fdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4580
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]
 * ufunc_fdtr_data[1] = &ufunc_fdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * fdtr = np.PyUFunc_FromFuncAndData(ufunc_fdtr_loops, ufunc_fdtr_data, ufunc_fdtr_types, 2, 3, 1, 0, "fdtr", ufunc_fdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4581
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]
 * ufunc_fdtr_data[1] = &ufunc_fdtr_ptr[2*1]
 * fdtr = np.PyUFunc_FromFuncAndData(ufunc_fdtr_loops, ufunc_fdtr_data, ufunc_fdtr_types, 2, 3, 1, 0, "fdtr", ufunc_fdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fdtrc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types, 2, 3, 1, 0, __pyx_k_fdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4581; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4581; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4587
 * cdef void *ufunc_fdtrc_data[2]
 * cdef char ufunc_fdtrc_types[8]
 * cdef char *ufunc_fdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "fdtrc(dfn, dfd, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_doc = __pyx_k_fdtrc_dfn_dfd_x_F_survival_funct;

  /* "scipy/special/_ufuncs.pyx":4593
 *     "\n"
 *     "Returns the complemented F distribution function.")
 * ufunc_fdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4594
 *     "Returns the complemented F distribution function.")
 * ufunc_fdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4595
 * ufunc_fdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4596
 * ufunc_fdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4597
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4598
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4599
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4600
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4601
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4602
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4603
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[0]) = ((void *)fdtrc);

  /* "scipy/special/_ufuncs.pyx":4604
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[1]) = ((void *)((char *)__pyx_k_fdtrc));

  /* "scipy/special/_ufuncs.pyx":4605
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[2]) = ((void *)fdtrc);

  /* "scipy/special/_ufuncs.pyx":4606
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]
 * ufunc_fdtrc_data[1] = &ufunc_fdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[3]) = ((void *)((char *)__pyx_k_fdtrc));

  /* "scipy/special/_ufuncs.pyx":4607
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_data[1] = &ufunc_fdtrc_ptr[2*1]
 * fdtrc = np.PyUFunc_FromFuncAndData(ufunc_fdtrc_loops, ufunc_fdtrc_data, ufunc_fdtrc_types, 2, 3, 1, 0, "fdtrc", ufunc_fdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4608
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]
 * ufunc_fdtrc_data[1] = &ufunc_fdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * fdtrc = np.PyUFunc_FromFuncAndData(ufunc_fdtrc_loops, ufunc_fdtrc_data, ufunc_fdtrc_types, 2, 3, 1, 0, "fdtrc", ufunc_fdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4609
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]
 * ufunc_fdtrc_data[1] = &ufunc_fdtrc_ptr[2*1]
 * fdtrc = np.PyUFunc_FromFuncAndData(ufunc_fdtrc_loops, ufunc_fdtrc_data, ufunc_fdtrc_types, 2, 3, 1, 0, "fdtrc", ufunc_fdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fdtri_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types, 2, 3, 1, 0, __pyx_k_fdtrc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fdtrc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4615
 * cdef void *ufunc_fdtri_data[2]
 * cdef char ufunc_fdtri_types[8]
 * cdef char *ufunc_fdtri_doc = (             # <<<<<<<<<<<<<<
 *     "fdtri(dfn, dfd, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_doc = __pyx_k_fdtri_dfn_dfd_p_Inverse_to_fdtr;

  /* "scipy/special/_ufuncs.pyx":4621
 *     "\n"
 *     "Finds the F density argument `x` such that ``fdtr(dfn, dfd, x) == p``.")
 * ufunc_fdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_fdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4622
 *     "Finds the F density argument `x` such that ``fdtr(dfn, dfd, x) == p``.")
 * ufunc_fdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4623
 * ufunc_fdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4624
 * ufunc_fdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4625
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4626
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4627
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4628
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4629
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4630
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4631
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri             # <<<<<<<<<<<<<<
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[0]) = ((void *)fdtri);

  /* "scipy/special/_ufuncs.pyx":4632
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")             # <<<<<<<<<<<<<<
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[1]) = ((void *)((char *)__pyx_k_fdtri));

  /* "scipy/special/_ufuncs.pyx":4633
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri             # <<<<<<<<<<<<<<
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[2]) = ((void *)fdtri);

  /* "scipy/special/_ufuncs.pyx":4634
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")             # <<<<<<<<<<<<<<
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]
 * ufunc_fdtri_data[1] = &ufunc_fdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[3]) = ((void *)((char *)__pyx_k_fdtri));

  /* "scipy/special/_ufuncs.pyx":4635
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fdtri_data[1] = &ufunc_fdtri_ptr[2*1]
 * fdtri = np.PyUFunc_FromFuncAndData(ufunc_fdtri_loops, ufunc_fdtri_data, ufunc_fdtri_types, 2, 3, 1, 0, "fdtri", ufunc_fdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4636
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]
 * ufunc_fdtri_data[1] = &ufunc_fdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * fdtri = np.PyUFunc_FromFuncAndData(ufunc_fdtri_loops, ufunc_fdtri_data, ufunc_fdtri_types, 2, 3, 1, 0, "fdtri", ufunc_fdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4637
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]
 * ufunc_fdtri_data[1] = &ufunc_fdtri_ptr[2*1]
 * fdtri = np.PyUFunc_FromFuncAndData(ufunc_fdtri_loops, ufunc_fdtri_data, ufunc_fdtri_types, 2, 3, 1, 0, "fdtri", ufunc_fdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fdtridfd_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types, 2, 3, 1, 0, __pyx_k_fdtri, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fdtri, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4643
 * cdef void *ufunc_fdtridfd_data[2]
 * cdef char ufunc_fdtridfd_types[8]
 * cdef char *ufunc_fdtridfd_doc = (             # <<<<<<<<<<<<<<
 *     "fdtridfd(dfn, p, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_doc = __pyx_k_fdtridfd_dfn_p_x_Inverse_to_fdtr;

  /* "scipy/special/_ufuncs.pyx":4649
 *     "\n"
 *     "Finds the F density argument dfd such that ``fdtr(dfn, dfd, x) == p``.")
 * ufunc_fdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4650
 *     "Finds the F density argument dfd such that ``fdtr(dfn, dfd, x) == p``.")
 * ufunc_fdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4651
 * ufunc_fdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4652
 * ufunc_fdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4653
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4654
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4655
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4656
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4657
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4658
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4659
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[0]) = ((void *)cdff4_wrap);

  /* "scipy/special/_ufuncs.pyx":4660
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[1]) = ((void *)((char *)__pyx_k_fdtridfd));

  /* "scipy/special/_ufuncs.pyx":4661
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[2]) = ((void *)cdff4_wrap);

  /* "scipy/special/_ufuncs.pyx":4662
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]
 * ufunc_fdtridfd_data[1] = &ufunc_fdtridfd_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[3]) = ((void *)((char *)__pyx_k_fdtridfd));

  /* "scipy/special/_ufuncs.pyx":4663
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_data[1] = &ufunc_fdtridfd_ptr[2*1]
 * fdtridfd = np.PyUFunc_FromFuncAndData(ufunc_fdtridfd_loops, ufunc_fdtridfd_data, ufunc_fdtridfd_types, 2, 3, 1, 0, "fdtridfd", ufunc_fdtridfd_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4664
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]
 * ufunc_fdtridfd_data[1] = &ufunc_fdtridfd_ptr[2*1]             # <<<<<<<<<<<<<<
 * fdtridfd = np.PyUFunc_FromFuncAndData(ufunc_fdtridfd_loops, ufunc_fdtridfd_data, ufunc_fdtridfd_types, 2, 3, 1, 0, "fdtridfd", ufunc_fdtridfd_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4665
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]
 * ufunc_fdtridfd_data[1] = &ufunc_fdtridfd_ptr[2*1]
 * fdtridfd = np.PyUFunc_FromFuncAndData(ufunc_fdtridfd_loops, ufunc_fdtridfd_data, ufunc_fdtridfd_types, 2, 3, 1, 0, "fdtridfd", ufunc_fdtridfd_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fresnel_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types, 2, 3, 1, 0, __pyx_k_fdtridfd, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4665; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fdtridfd, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4665; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4671
 * cdef void *ufunc_fresnel_data[4]
 * cdef char ufunc_fresnel_types[12]
 * cdef char *ufunc_fresnel_doc = (             # <<<<<<<<<<<<<<
 *     "fresnel(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_doc = __pyx_k_fresnel_z_Fresnel_sin_and_cos_in;

  /* "scipy/special/_ufuncs.pyx":4690
 *     "ssa, csa\n"
 *     "    Fresnel sin and cos integral values")
 * ufunc_fresnel_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_fresnel_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":4691
 *     "    Fresnel sin and cos integral values")
 * ufunc_fresnel_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_fresnel_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":4692
 * ufunc_fresnel_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_fresnel_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF             # <<<<<<<<<<<<<<
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_F_FF);

  /* "scipy/special/_ufuncs.pyx":4693
 * ufunc_fresnel_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_D_DD);

  /* "scipy/special/_ufuncs.pyx":4694
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4695
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4696
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4697
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4698
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4699
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4700
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4701
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4702
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4703
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4704
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4705
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4706
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[0]) = ((void *)fresnl);

  /* "scipy/special/_ufuncs.pyx":4707
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[1]) = ((void *)((char *)__pyx_k_fresnel));

  /* "scipy/special/_ufuncs.pyx":4708
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[2]) = ((void *)fresnl);

  /* "scipy/special/_ufuncs.pyx":4709
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[3]) = ((void *)((char *)__pyx_k_fresnel));

  /* "scipy/special/_ufuncs.pyx":4710
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[4]) = ((void *)cfresnl_wrap);

  /* "scipy/special/_ufuncs.pyx":4711
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[5]) = ((void *)((char *)__pyx_k_fresnel));

  /* "scipy/special/_ufuncs.pyx":4712
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[6]) = ((void *)cfresnl_wrap);

  /* "scipy/special/_ufuncs.pyx":4713
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")             # <<<<<<<<<<<<<<
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[7]) = ((void *)((char *)__pyx_k_fresnel));

  /* "scipy/special/_ufuncs.pyx":4714
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4715
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]
 * ufunc_fresnel_data[3] = &ufunc_fresnel_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4716
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_fresnel_data[3] = &ufunc_fresnel_ptr[2*3]
 * fresnel = np.PyUFunc_FromFuncAndData(ufunc_fresnel_loops, ufunc_fresnel_data, ufunc_fresnel_types, 4, 1, 2, 0, "fresnel", ufunc_fresnel_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4717
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]
 * ufunc_fresnel_data[3] = &ufunc_fresnel_ptr[2*3]             # <<<<<<<<<<<<<<
 * fresnel = np.PyUFunc_FromFuncAndData(ufunc_fresnel_loops, ufunc_fresnel_data, ufunc_fresnel_types, 4, 1, 2, 0, "fresnel", ufunc_fresnel_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4718
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]
 * ufunc_fresnel_data[3] = &ufunc_fresnel_ptr[2*3]
 * fresnel = np.PyUFunc_FromFuncAndData(ufunc_fresnel_loops, ufunc_fresnel_data, ufunc_fresnel_types, 4, 1, 2, 0, "fresnel", ufunc_fresnel_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gamma_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types, 4, 1, 2, 0, __pyx_k_fresnel, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fresnel, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4724
 * cdef void *ufunc_gamma_data[4]
 * cdef char ufunc_gamma_types[8]
 * cdef char *ufunc_gamma_doc = (             # <<<<<<<<<<<<<<
 *     "gamma(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_doc = __pyx_k_gamma_z_Gamma_function_The_gamma;

  /* "scipy/special/_ufuncs.pyx":4732
 *     "factorial since ``z*gamma(z) = gamma(z+1)`` and ``gamma(n+1) =\n"
 *     "n!`` for natural number *n*.")
 * ufunc_gamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_gamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4733
 *     "n!`` for natural number *n*.")
 * ufunc_gamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4734
 * ufunc_gamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gamma_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4735
 * ufunc_gamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[0] = <char>NPY_FLOAT
 * ufunc_gamma_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4736
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gamma_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[1] = <char>NPY_FLOAT
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4737
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gamma_types[0] = <char>NPY_FLOAT
 * ufunc_gamma_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4738
 * ufunc_gamma_types[0] = <char>NPY_FLOAT
 * ufunc_gamma_types[1] = <char>NPY_FLOAT
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4739
 * ufunc_gamma_types[1] = <char>NPY_FLOAT
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4740
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4741
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4742
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4743
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4744
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[0]) = ((void *)Gamma);

  /* "scipy/special/_ufuncs.pyx":4745
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[1]) = ((void *)((char *)__pyx_k_gamma));

  /* "scipy/special/_ufuncs.pyx":4746
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[2]) = ((void *)Gamma);

  /* "scipy/special/_ufuncs.pyx":4747
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma_wrap
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[3]) = ((void *)((char *)__pyx_k_gamma));

  /* "scipy/special/_ufuncs.pyx":4748
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma_wrap             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[4]) = ((void *)cgamma_wrap);

  /* "scipy/special/_ufuncs.pyx":4749
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma_wrap
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma_wrap
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[5]) = ((void *)((char *)__pyx_k_gamma));

  /* "scipy/special/_ufuncs.pyx":4750
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma_wrap
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma_wrap             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[6]) = ((void *)cgamma_wrap);

  /* "scipy/special/_ufuncs.pyx":4751
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma_wrap
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")             # <<<<<<<<<<<<<<
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[7]) = ((void *)((char *)__pyx_k_gamma));

  /* "scipy/special/_ufuncs.pyx":4752
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma_wrap
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4753
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]
 * ufunc_gamma_data[3] = &ufunc_gamma_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4754
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_gamma_data[3] = &ufunc_gamma_ptr[2*3]
 * gamma = np.PyUFunc_FromFuncAndData(ufunc_gamma_loops, ufunc_gamma_data, ufunc_gamma_types, 4, 1, 1, 0, "gamma", ufunc_gamma_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4755
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]
 * ufunc_gamma_data[3] = &ufunc_gamma_ptr[2*3]             # <<<<<<<<<<<<<<
 * gamma = np.PyUFunc_FromFuncAndData(ufunc_gamma_loops, ufunc_gamma_data, ufunc_gamma_types, 4, 1, 1, 0, "gamma", ufunc_gamma_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4756
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]
 * ufunc_gamma_data[3] = &ufunc_gamma_ptr[2*3]
 * gamma = np.PyUFunc_FromFuncAndData(ufunc_gamma_loops, ufunc_gamma_data, ufunc_gamma_types, 4, 1, 1, 0, "gamma", ufunc_gamma_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammainc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types, 4, 1, 1, 0, __pyx_k_gamma, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gamma, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4762
 * cdef void *ufunc_gammainc_data[2]
 * cdef char ufunc_gammainc_types[6]
 * cdef char *ufunc_gammainc_doc = (             # <<<<<<<<<<<<<<
 *     "gammainc(a, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_doc = __pyx_k_gammainc_a_x_Incomplete_gamma_fu;

  /* "scipy/special/_ufuncs.pyx":4772
 *     "\n"
 *     "`a` must be positive and `x` must be >= 0.")
 * ufunc_gammainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_gammainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4773
 *     "`a` must be positive and `x` must be >= 0.")
 * ufunc_gammainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4774
 * ufunc_gammainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4775
 * ufunc_gammainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4776
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4777
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4778
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4779
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4780
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam             # <<<<<<<<<<<<<<
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[0]) = ((void *)igam);

  /* "scipy/special/_ufuncs.pyx":4781
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")             # <<<<<<<<<<<<<<
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[1]) = ((void *)((char *)__pyx_k_gammainc));

  /* "scipy/special/_ufuncs.pyx":4782
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam             # <<<<<<<<<<<<<<
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[2]) = ((void *)igam);

  /* "scipy/special/_ufuncs.pyx":4783
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")             # <<<<<<<<<<<<<<
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]
 * ufunc_gammainc_data[1] = &ufunc_gammainc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[3]) = ((void *)((char *)__pyx_k_gammainc));

  /* "scipy/special/_ufuncs.pyx":4784
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammainc_data[1] = &ufunc_gammainc_ptr[2*1]
 * gammainc = np.PyUFunc_FromFuncAndData(ufunc_gammainc_loops, ufunc_gammainc_data, ufunc_gammainc_types, 2, 2, 1, 0, "gammainc", ufunc_gammainc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4785
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]
 * ufunc_gammainc_data[1] = &ufunc_gammainc_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammainc = np.PyUFunc_FromFuncAndData(ufunc_gammainc_loops, ufunc_gammainc_data, ufunc_gammainc_types, 2, 2, 1, 0, "gammainc", ufunc_gammainc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4786
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]
 * ufunc_gammainc_data[1] = &ufunc_gammainc_ptr[2*1]
 * gammainc = np.PyUFunc_FromFuncAndData(ufunc_gammainc_loops, ufunc_gammainc_data, ufunc_gammainc_types, 2, 2, 1, 0, "gammainc", ufunc_gammainc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammaincc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types, 2, 2, 1, 0, __pyx_k_gammainc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4786; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammainc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4786; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4792
 * cdef void *ufunc_gammaincc_data[2]
 * cdef char ufunc_gammaincc_types[6]
 * cdef char *ufunc_gammaincc_doc = (             # <<<<<<<<<<<<<<
 *     "gammaincc(a, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_doc = __pyx_k_gammaincc_a_x_Complemented_incom;

  /* "scipy/special/_ufuncs.pyx":4802
 *     "\n"
 *     "`a` must be positive and `x` must be >= 0.")
 * ufunc_gammaincc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4803
 *     "`a` must be positive and `x` must be >= 0.")
 * ufunc_gammaincc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammaincc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4804
 * ufunc_gammaincc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammaincc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4805
 * ufunc_gammaincc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4806
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4807
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4808
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4809
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4810
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[0]) = ((void *)igamc);

  /* "scipy/special/_ufuncs.pyx":4811
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[1]) = ((void *)((char *)__pyx_k_gammaincc));

  /* "scipy/special/_ufuncs.pyx":4812
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[2]) = ((void *)igamc);

  /* "scipy/special/_ufuncs.pyx":4813
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]
 * ufunc_gammaincc_data[1] = &ufunc_gammaincc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[3]) = ((void *)((char *)__pyx_k_gammaincc));

  /* "scipy/special/_ufuncs.pyx":4814
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_data[1] = &ufunc_gammaincc_ptr[2*1]
 * gammaincc = np.PyUFunc_FromFuncAndData(ufunc_gammaincc_loops, ufunc_gammaincc_data, ufunc_gammaincc_types, 2, 2, 1, 0, "gammaincc", ufunc_gammaincc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4815
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]
 * ufunc_gammaincc_data[1] = &ufunc_gammaincc_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammaincc = np.PyUFunc_FromFuncAndData(ufunc_gammaincc_loops, ufunc_gammaincc_data, ufunc_gammaincc_types, 2, 2, 1, 0, "gammaincc", ufunc_gammaincc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4816
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]
 * ufunc_gammaincc_data[1] = &ufunc_gammaincc_ptr[2*1]
 * gammaincc = np.PyUFunc_FromFuncAndData(ufunc_gammaincc_loops, ufunc_gammaincc_data, ufunc_gammaincc_types, 2, 2, 1, 0, "gammaincc", ufunc_gammaincc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammainccinv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types, 2, 2, 1, 0, __pyx_k_gammaincc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4816; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammaincc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4816; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4822
 * cdef void *ufunc_gammainccinv_data[2]
 * cdef char ufunc_gammainccinv_types[6]
 * cdef char *ufunc_gammainccinv_doc = (             # <<<<<<<<<<<<<<
 *     "gammainccinv(a, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_doc = __pyx_k_gammainccinv_a_y_Inverse_to_gamm;

  /* "scipy/special/_ufuncs.pyx":4828
 *     "\n"
 *     "Returns `x` such that ``gammaincc(a, x) == y``.")
 * ufunc_gammainccinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4829
 *     "Returns `x` such that ``gammaincc(a, x) == y``.")
 * ufunc_gammainccinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammainccinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4830
 * ufunc_gammainccinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammainccinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4831
 * ufunc_gammainccinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4832
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4833
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4834
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igami
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4835
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igami
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4836
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igami             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igami
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[0]) = ((void *)igami);

  /* "scipy/special/_ufuncs.pyx":4837
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igami
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igami
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[1]) = ((void *)((char *)__pyx_k_gammainccinv));

  /* "scipy/special/_ufuncs.pyx":4838
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igami
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igami             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[2]) = ((void *)igami);

  /* "scipy/special/_ufuncs.pyx":4839
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igami
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]
 * ufunc_gammainccinv_data[1] = &ufunc_gammainccinv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[3]) = ((void *)((char *)__pyx_k_gammainccinv));

  /* "scipy/special/_ufuncs.pyx":4840
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igami
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_data[1] = &ufunc_gammainccinv_ptr[2*1]
 * gammainccinv = np.PyUFunc_FromFuncAndData(ufunc_gammainccinv_loops, ufunc_gammainccinv_data, ufunc_gammainccinv_types, 2, 2, 1, 0, "gammainccinv", ufunc_gammainccinv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4841
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]
 * ufunc_gammainccinv_data[1] = &ufunc_gammainccinv_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammainccinv = np.PyUFunc_FromFuncAndData(ufunc_gammainccinv_loops, ufunc_gammainccinv_data, ufunc_gammainccinv_types, 2, 2, 1, 0, "gammainccinv", ufunc_gammainccinv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4842
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]
 * ufunc_gammainccinv_data[1] = &ufunc_gammainccinv_ptr[2*1]
 * gammainccinv = np.PyUFunc_FromFuncAndData(ufunc_gammainccinv_loops, ufunc_gammainccinv_data, ufunc_gammainccinv_types, 2, 2, 1, 0, "gammainccinv", ufunc_gammainccinv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammaincinv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types, 2, 2, 1, 0, __pyx_k_gammainccinv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammainccinv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4848
 * cdef void *ufunc_gammaincinv_data[2]
 * cdef char ufunc_gammaincinv_types[6]
 * cdef char *ufunc_gammaincinv_doc = (             # <<<<<<<<<<<<<<
 *     "gammaincinv(a, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_doc = __pyx_k_gammaincinv_a_y_Inverse_to_gamma;

  /* "scipy/special/_ufuncs.pyx":4854
 *     "\n"
 *     "Returns `x` such that ``gammainc(a, x) = y``.")
 * ufunc_gammaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4855
 *     "Returns `x` such that ``gammainc(a, x) = y``.")
 * ufunc_gammaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4856
 * ufunc_gammaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4857
 * ufunc_gammaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4858
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4859
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4860
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_gammaincinv
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4861
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_gammaincinv
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4862
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_gammaincinv             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_gammaincinv
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[0]) = ((void *)gammaincinv);

  /* "scipy/special/_ufuncs.pyx":4863
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_gammaincinv
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_gammaincinv
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[1]) = ((void *)((char *)__pyx_k_gammaincinv));

  /* "scipy/special/_ufuncs.pyx":4864
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_gammaincinv
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_gammaincinv             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[2]) = ((void *)gammaincinv);

  /* "scipy/special/_ufuncs.pyx":4865
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_gammaincinv
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]
 * ufunc_gammaincinv_data[1] = &ufunc_gammaincinv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[3]) = ((void *)((char *)__pyx_k_gammaincinv));

  /* "scipy/special/_ufuncs.pyx":4866
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_gammaincinv
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_data[1] = &ufunc_gammaincinv_ptr[2*1]
 * gammaincinv = np.PyUFunc_FromFuncAndData(ufunc_gammaincinv_loops, ufunc_gammaincinv_data, ufunc_gammaincinv_types, 2, 2, 1, 0, "gammaincinv", ufunc_gammaincinv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4867
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]
 * ufunc_gammaincinv_data[1] = &ufunc_gammaincinv_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammaincinv = np.PyUFunc_FromFuncAndData(ufunc_gammaincinv_loops, ufunc_gammaincinv_data, ufunc_gammaincinv_types, 2, 2, 1, 0, "gammaincinv", ufunc_gammaincinv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4868
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]
 * ufunc_gammaincinv_data[1] = &ufunc_gammaincinv_ptr[2*1]
 * gammaincinv = np.PyUFunc_FromFuncAndData(ufunc_gammaincinv_loops, ufunc_gammaincinv_data, ufunc_gammaincinv_types, 2, 2, 1, 0, "gammaincinv", ufunc_gammaincinv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammaln_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types, 2, 2, 1, 0, __pyx_k_gammaincinv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4868; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammaincinv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4868; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4874
 * cdef void *ufunc_gammaln_data[4]
 * cdef char ufunc_gammaln_types[8]
 * cdef char *ufunc_gammaln_doc = (             # <<<<<<<<<<<<<<
 *     "gammaln(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_doc = __pyx_k_gammaln_z_Logarithm_of_absolute;

  /* "scipy/special/_ufuncs.pyx":4886
 *     "--------\n"
 *     "gammasgn")
 * ufunc_gammaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_gammaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammaln_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4887
 *     "gammasgn")
 * ufunc_gammaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gammaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_gammaln_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gammaln_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4888
 * ufunc_gammaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gammaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammaln_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_gammaln_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4889
 * ufunc_gammaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammaln_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gammaln_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4890
 * ufunc_gammaln_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gammaln_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4891
 * ufunc_gammaln_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4892
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4893
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[4] = <char>NPY_CFLOAT
 * ufunc_gammaln_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4894
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[5] = <char>NPY_CFLOAT
 * ufunc_gammaln_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4895
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[4] = <char>NPY_CFLOAT
 * ufunc_gammaln_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[6] = <char>NPY_CDOUBLE
 * ufunc_gammaln_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4896
 * ufunc_gammaln_types[4] = <char>NPY_CFLOAT
 * ufunc_gammaln_types[5] = <char>NPY_CFLOAT
 * ufunc_gammaln_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[7] = <char>NPY_CDOUBLE
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4897
 * ufunc_gammaln_types[5] = <char>NPY_CFLOAT
 * ufunc_gammaln_types[6] = <char>NPY_CDOUBLE
 * ufunc_gammaln_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4898
 * ufunc_gammaln_types[6] = <char>NPY_CDOUBLE
 * ufunc_gammaln_types[7] = <char>NPY_CDOUBLE
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[0]) = ((void *)lgam);

  /* "scipy/special/_ufuncs.pyx":4899
 * ufunc_gammaln_types[7] = <char>NPY_CDOUBLE
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[1]) = ((void *)((char *)__pyx_k_gammaln));

  /* "scipy/special/_ufuncs.pyx":4900
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*2] = <void*>_func_clngamma_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[2]) = ((void *)lgam);

  /* "scipy/special/_ufuncs.pyx":4901
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*2] = <void*>_func_clngamma_wrap
 * ufunc_gammaln_ptr[2*2+1] = <void*>(<char*>"gammaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[3]) = ((void *)((char *)__pyx_k_gammaln));

  /* "scipy/special/_ufuncs.pyx":4902
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*2] = <void*>_func_clngamma_wrap             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*2+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*3] = <void*>_func_clngamma_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[4]) = ((void *)clngamma_wrap);

  /* "scipy/special/_ufuncs.pyx":4903
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*2] = <void*>_func_clngamma_wrap
 * ufunc_gammaln_ptr[2*2+1] = <void*>(<char*>"gammaln")             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*3] = <void*>_func_clngamma_wrap
 * ufunc_gammaln_ptr[2*3+1] = <void*>(<char*>"gammaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[5]) = ((void *)((char *)__pyx_k_gammaln));

  /* "scipy/special/_ufuncs.pyx":4904
 * ufunc_gammaln_ptr[2*2] = <void*>_func_clngamma_wrap
 * ufunc_gammaln_ptr[2*2+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*3] = <void*>_func_clngamma_wrap             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*3+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[6]) = ((void *)clngamma_wrap);

  /* "scipy/special/_ufuncs.pyx":4905
 * ufunc_gammaln_ptr[2*2+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*3] = <void*>_func_clngamma_wrap
 * ufunc_gammaln_ptr[2*3+1] = <void*>(<char*>"gammaln")             # <<<<<<<<<<<<<<
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[7]) = ((void *)((char *)__pyx_k_gammaln));

  /* "scipy/special/_ufuncs.pyx":4906
 * ufunc_gammaln_ptr[2*3] = <void*>_func_clngamma_wrap
 * ufunc_gammaln_ptr[2*3+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]
 * ufunc_gammaln_data[2] = &ufunc_gammaln_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4907
 * ufunc_gammaln_ptr[2*3+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_gammaln_data[2] = &ufunc_gammaln_ptr[2*2]
 * ufunc_gammaln_data[3] = &ufunc_gammaln_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4908
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]
 * ufunc_gammaln_data[2] = &ufunc_gammaln_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_gammaln_data[3] = &ufunc_gammaln_ptr[2*3]
 * gammaln = np.PyUFunc_FromFuncAndData(ufunc_gammaln_loops, ufunc_gammaln_data, ufunc_gammaln_types, 4, 1, 1, 0, "gammaln", ufunc_gammaln_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4909
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]
 * ufunc_gammaln_data[2] = &ufunc_gammaln_ptr[2*2]
 * ufunc_gammaln_data[3] = &ufunc_gammaln_ptr[2*3]             # <<<<<<<<<<<<<<
 * gammaln = np.PyUFunc_FromFuncAndData(ufunc_gammaln_loops, ufunc_gammaln_data, ufunc_gammaln_types, 4, 1, 1, 0, "gammaln", ufunc_gammaln_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4910
 * ufunc_gammaln_data[2] = &ufunc_gammaln_ptr[2*2]
 * ufunc_gammaln_data[3] = &ufunc_gammaln_ptr[2*3]
 * gammaln = np.PyUFunc_FromFuncAndData(ufunc_gammaln_loops, ufunc_gammaln_data, ufunc_gammaln_types, 4, 1, 1, 0, "gammaln", ufunc_gammaln_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammasgn_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types, 4, 1, 1, 0, __pyx_k_gammaln, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammaln, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4916
 * cdef void *ufunc_gammasgn_data[2]
 * cdef char ufunc_gammasgn_types[4]
 * cdef char *ufunc_gammasgn_doc = (             # <<<<<<<<<<<<<<
 *     "gammasgn(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_doc = __pyx_k_gammasgn_x_Sign_of_the_gamma_fun;

  /* "scipy/special/_ufuncs.pyx":4924
 *     "--------\n"
 *     "gammaln")
 * ufunc_gammasgn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4925
 *     "gammaln")
 * ufunc_gammasgn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gammasgn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4926
 * ufunc_gammasgn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gammasgn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4927
 * ufunc_gammasgn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4928
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4929
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4930
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[0]) = ((void *)gammasgn);

  /* "scipy/special/_ufuncs.pyx":4931
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[1]) = ((void *)((char *)__pyx_k_gammasgn));

  /* "scipy/special/_ufuncs.pyx":4932
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[2]) = ((void *)gammasgn);

  /* "scipy/special/_ufuncs.pyx":4933
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]
 * ufunc_gammasgn_data[1] = &ufunc_gammasgn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[3]) = ((void *)((char *)__pyx_k_gammasgn));

  /* "scipy/special/_ufuncs.pyx":4934
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_data[1] = &ufunc_gammasgn_ptr[2*1]
 * gammasgn = np.PyUFunc_FromFuncAndData(ufunc_gammasgn_loops, ufunc_gammasgn_data, ufunc_gammasgn_types, 2, 1, 1, 0, "gammasgn", ufunc_gammasgn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4935
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]
 * ufunc_gammasgn_data[1] = &ufunc_gammasgn_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammasgn = np.PyUFunc_FromFuncAndData(ufunc_gammasgn_loops, ufunc_gammasgn_data, ufunc_gammasgn_types, 2, 1, 1, 0, "gammasgn", ufunc_gammasgn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4936
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]
 * ufunc_gammasgn_data[1] = &ufunc_gammasgn_ptr[2*1]
 * gammasgn = np.PyUFunc_FromFuncAndData(ufunc_gammasgn_loops, ufunc_gammasgn_data, ufunc_gammasgn_types, 2, 1, 1, 0, "gammasgn", ufunc_gammasgn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types, 2, 1, 1, 0, __pyx_k_gammasgn, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammasgn, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4942
 * cdef void *ufunc_gdtr_data[2]
 * cdef char ufunc_gdtr_types[8]
 * cdef char *ufunc_gdtr_doc = (             # <<<<<<<<<<<<<<
 *     "gdtr(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_doc = __pyx_k_gdtr_a_b_x_Gamma_distribution_cu;

  /* "scipy/special/_ufuncs.pyx":4954
 *     "The arguments `a` and `b` are used differently here than in other\n"
 *     "definitions.")
 * ufunc_gdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4955
 *     "definitions.")
 * ufunc_gdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4956
 * ufunc_gdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4957
 * ufunc_gdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4958
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4959
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4960
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4961
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4962
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4963
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4964
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr             # <<<<<<<<<<<<<<
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[0]) = ((void *)gdtr);

  /* "scipy/special/_ufuncs.pyx":4965
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")             # <<<<<<<<<<<<<<
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[1]) = ((void *)((char *)__pyx_k_gdtr));

  /* "scipy/special/_ufuncs.pyx":4966
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr             # <<<<<<<<<<<<<<
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[2]) = ((void *)gdtr);

  /* "scipy/special/_ufuncs.pyx":4967
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")             # <<<<<<<<<<<<<<
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]
 * ufunc_gdtr_data[1] = &ufunc_gdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[3]) = ((void *)((char *)__pyx_k_gdtr));

  /* "scipy/special/_ufuncs.pyx":4968
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtr_data[1] = &ufunc_gdtr_ptr[2*1]
 * gdtr = np.PyUFunc_FromFuncAndData(ufunc_gdtr_loops, ufunc_gdtr_data, ufunc_gdtr_types, 2, 3, 1, 0, "gdtr", ufunc_gdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4969
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]
 * ufunc_gdtr_data[1] = &ufunc_gdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtr = np.PyUFunc_FromFuncAndData(ufunc_gdtr_loops, ufunc_gdtr_data, ufunc_gdtr_types, 2, 3, 1, 0, "gdtr", ufunc_gdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4970
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]
 * ufunc_gdtr_data[1] = &ufunc_gdtr_ptr[2*1]
 * gdtr = np.PyUFunc_FromFuncAndData(ufunc_gdtr_loops, ufunc_gdtr_data, ufunc_gdtr_types, 2, 3, 1, 0, "gdtr", ufunc_gdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtrc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types, 2, 3, 1, 0, __pyx_k_gdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4970; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 4970; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":4976
 * cdef void *ufunc_gdtrc_data[2]
 * cdef char ufunc_gdtrc_types[8]
 * cdef char *ufunc_gdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "gdtrc(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_doc = __pyx_k_gdtrc_a_b_x_Gamma_distribution_s;

  /* "scipy/special/_ufuncs.pyx":4987
 *     "--------\n"
 *     "gdtr, gdtri")
 * ufunc_gdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4988
 *     "gdtr, gdtri")
 * ufunc_gdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4989
 * ufunc_gdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4990
 * ufunc_gdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4991
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4992
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4993
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4994
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4995
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4996
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4997
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[0]) = ((void *)gdtrc);

  /* "scipy/special/_ufuncs.pyx":4998
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[1]) = ((void *)((char *)__pyx_k_gdtrc));

  /* "scipy/special/_ufuncs.pyx":4999
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[2]) = ((void *)gdtrc);

  /* "scipy/special/_ufuncs.pyx":5000
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]
 * ufunc_gdtrc_data[1] = &ufunc_gdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[3]) = ((void *)((char *)__pyx_k_gdtrc));

  /* "scipy/special/_ufuncs.pyx":5001
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_data[1] = &ufunc_gdtrc_ptr[2*1]
 * gdtrc = np.PyUFunc_FromFuncAndData(ufunc_gdtrc_loops, ufunc_gdtrc_data, ufunc_gdtrc_types, 2, 3, 1, 0, "gdtrc", ufunc_gdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5002
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]
 * ufunc_gdtrc_data[1] = &ufunc_gdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtrc = np.PyUFunc_FromFuncAndData(ufunc_gdtrc_loops, ufunc_gdtrc_data, ufunc_gdtrc_types, 2, 3, 1, 0, "gdtrc", ufunc_gdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5003
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]
 * ufunc_gdtrc_data[1] = &ufunc_gdtrc_ptr[2*1]
 * gdtrc = np.PyUFunc_FromFuncAndData(ufunc_gdtrc_loops, ufunc_gdtrc_data, ufunc_gdtrc_types, 2, 3, 1, 0, "gdtrc", ufunc_gdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtria_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types, 2, 3, 1, 0, __pyx_k_gdtrc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5003; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtrc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5003; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5009
 * cdef void *ufunc_gdtria_data[2]
 * cdef char ufunc_gdtria_types[8]
 * cdef char *ufunc_gdtria_doc = (             # <<<<<<<<<<<<<<
 *     "gdtria(p, b, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_doc = __pyx_k_gdtria_p_b_x_out_None_Inverse_of;

  /* "scipy/special/_ufuncs.pyx":5057
 *     ">>> gdtria(p, 3.4, 5.6)\n"
 *     "1.2")
 * ufunc_gdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":5058
 *     "1.2")
 * ufunc_gdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":5059
 * ufunc_gdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5060
 * ufunc_gdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5061
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5062
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5063
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5064
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5065
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5066
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5067
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[0]) = ((void *)cdfgam4_wrap);

  /* "scipy/special/_ufuncs.pyx":5068
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")             # <<<<<<<<<<<<<<
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[1]) = ((void *)((char *)__pyx_k_gdtria));

  /* "scipy/special/_ufuncs.pyx":5069
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[2]) = ((void *)cdfgam4_wrap);

  /* "scipy/special/_ufuncs.pyx":5070
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")             # <<<<<<<<<<<<<<
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]
 * ufunc_gdtria_data[1] = &ufunc_gdtria_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[3]) = ((void *)((char *)__pyx_k_gdtria));

  /* "scipy/special/_ufuncs.pyx":5071
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtria_data[1] = &ufunc_gdtria_ptr[2*1]
 * gdtria = np.PyUFunc_FromFuncAndData(ufunc_gdtria_loops, ufunc_gdtria_data, ufunc_gdtria_types, 2, 3, 1, 0, "gdtria", ufunc_gdtria_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5072
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]
 * ufunc_gdtria_data[1] = &ufunc_gdtria_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtria = np.PyUFunc_FromFuncAndData(ufunc_gdtria_loops, ufunc_gdtria_data, ufunc_gdtria_types, 2, 3, 1, 0, "gdtria", ufunc_gdtria_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5073
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]
 * ufunc_gdtria_data[1] = &ufunc_gdtria_ptr[2*1]
 * gdtria = np.PyUFunc_FromFuncAndData(ufunc_gdtria_loops, ufunc_gdtria_data, ufunc_gdtria_types, 2, 3, 1, 0, "gdtria", ufunc_gdtria_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtrib_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types, 2, 3, 1, 0, __pyx_k_gdtria, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtria, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5079
 * cdef void *ufunc_gdtrib_data[2]
 * cdef char ufunc_gdtrib_types[8]
 * cdef char *ufunc_gdtrib_doc = (             # <<<<<<<<<<<<<<
 *     "gdtrib(a, p, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_doc = __pyx_k_gdtrib_a_p_x_out_None_Inverse_of;

  /* "scipy/special/_ufuncs.pyx":5127
 *     ">>> gdtrib(1.2, p, 5.6)\n"
 *     "3.3999999999723882")
 * ufunc_gdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":5128
 *     "3.3999999999723882")
 * ufunc_gdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":5129
 * ufunc_gdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5130
 * ufunc_gdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5131
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5132
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5133
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5134
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5135
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5136
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5137
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[0]) = ((void *)cdfgam3_wrap);

  /* "scipy/special/_ufuncs.pyx":5138
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[1]) = ((void *)((char *)__pyx_k_gdtrib));

  /* "scipy/special/_ufuncs.pyx":5139
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[2]) = ((void *)cdfgam3_wrap);

  /* "scipy/special/_ufuncs.pyx":5140
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]
 * ufunc_gdtrib_data[1] = &ufunc_gdtrib_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[3]) = ((void *)((char *)__pyx_k_gdtrib));

  /* "scipy/special/_ufuncs.pyx":5141
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_data[1] = &ufunc_gdtrib_ptr[2*1]
 * gdtrib = np.PyUFunc_FromFuncAndData(ufunc_gdtrib_loops, ufunc_gdtrib_data, ufunc_gdtrib_types, 2, 3, 1, 0, "gdtrib", ufunc_gdtrib_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5142
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]
 * ufunc_gdtrib_data[1] = &ufunc_gdtrib_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtrib = np.PyUFunc_FromFuncAndData(ufunc_gdtrib_loops, ufunc_gdtrib_data, ufunc_gdtrib_types, 2, 3, 1, 0, "gdtrib", ufunc_gdtrib_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5143
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]
 * ufunc_gdtrib_data[1] = &ufunc_gdtrib_ptr[2*1]
 * gdtrib = np.PyUFunc_FromFuncAndData(ufunc_gdtrib_loops, ufunc_gdtrib_data, ufunc_gdtrib_types, 2, 3, 1, 0, "gdtrib", ufunc_gdtrib_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtrix_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types, 2, 3, 1, 0, __pyx_k_gdtrib, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5143; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtrib, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5143; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5149
 * cdef void *ufunc_gdtrix_data[2]
 * cdef char ufunc_gdtrix_types[8]
 * cdef char *ufunc_gdtrix_doc = (             # <<<<<<<<<<<<<<
 *     "gdtrix(a, b, p, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_doc = __pyx_k_gdtrix_a_b_p_out_None_Inverse_of;

  /* "scipy/special/_ufuncs.pyx":5198
 *     ">>> gdtrix(1.2, 3.4, p)\n"
 *     "5.5999999999999996")
 * ufunc_gdtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":5199
 *     "5.5999999999999996")
 * ufunc_gdtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":5200
 * ufunc_gdtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5201
 * ufunc_gdtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5202
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5203
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5204
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5205
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5206
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5207
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5208
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[0]) = ((void *)cdfgam2_wrap);

  /* "scipy/special/_ufuncs.pyx":5209
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[1]) = ((void *)((char *)__pyx_k_gdtrix));

  /* "scipy/special/_ufuncs.pyx":5210
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[2]) = ((void *)cdfgam2_wrap);

  /* "scipy/special/_ufuncs.pyx":5211
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]
 * ufunc_gdtrix_data[1] = &ufunc_gdtrix_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[3]) = ((void *)((char *)__pyx_k_gdtrix));

  /* "scipy/special/_ufuncs.pyx":5212
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_data[1] = &ufunc_gdtrix_ptr[2*1]
 * gdtrix = np.PyUFunc_FromFuncAndData(ufunc_gdtrix_loops, ufunc_gdtrix_data, ufunc_gdtrix_types, 2, 3, 1, 0, "gdtrix", ufunc_gdtrix_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5213
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]
 * ufunc_gdtrix_data[1] = &ufunc_gdtrix_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtrix = np.PyUFunc_FromFuncAndData(ufunc_gdtrix_loops, ufunc_gdtrix_data, ufunc_gdtrix_types, 2, 3, 1, 0, "gdtrix", ufunc_gdtrix_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5214
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]
 * ufunc_gdtrix_data[1] = &ufunc_gdtrix_ptr[2*1]
 * gdtrix = np.PyUFunc_FromFuncAndData(ufunc_gdtrix_loops, ufunc_gdtrix_data, ufunc_gdtrix_types, 2, 3, 1, 0, "gdtrix", ufunc_gdtrix_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hankel1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types, 2, 3, 1, 0, __pyx_k_gdtrix, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtrix, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5220
 * cdef void *ufunc_hankel1_data[2]
 * cdef char ufunc_hankel1_types[6]
 * cdef char *ufunc_hankel1_doc = (             # <<<<<<<<<<<<<<
 *     "hankel1(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_doc = __pyx_k_hankel1_v_z_Hankel_function_of_t;

  /* "scipy/special/_ufuncs.pyx":5231
 *     "z : float or complex\n"
 *     "    Argument")
 * ufunc_hankel1_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hankel1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5232
 *     "    Argument")
 * ufunc_hankel1_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5233
 * ufunc_hankel1_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5234
 * ufunc_hankel1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5235
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5236
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5237
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5238
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5239
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1             # <<<<<<<<<<<<<<
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[0]) = ((void *)cbesh_wrap1);

  /* "scipy/special/_ufuncs.pyx":5240
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")             # <<<<<<<<<<<<<<
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[1]) = ((void *)((char *)__pyx_k_hankel1));

  /* "scipy/special/_ufuncs.pyx":5241
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1             # <<<<<<<<<<<<<<
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[2]) = ((void *)cbesh_wrap1);

  /* "scipy/special/_ufuncs.pyx":5242
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")             # <<<<<<<<<<<<<<
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]
 * ufunc_hankel1_data[1] = &ufunc_hankel1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[3]) = ((void *)((char *)__pyx_k_hankel1));

  /* "scipy/special/_ufuncs.pyx":5243
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hankel1_data[1] = &ufunc_hankel1_ptr[2*1]
 * hankel1 = np.PyUFunc_FromFuncAndData(ufunc_hankel1_loops, ufunc_hankel1_data, ufunc_hankel1_types, 2, 2, 1, 0, "hankel1", ufunc_hankel1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5244
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]
 * ufunc_hankel1_data[1] = &ufunc_hankel1_ptr[2*1]             # <<<<<<<<<<<<<<
 * hankel1 = np.PyUFunc_FromFuncAndData(ufunc_hankel1_loops, ufunc_hankel1_data, ufunc_hankel1_types, 2, 2, 1, 0, "hankel1", ufunc_hankel1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5245
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]
 * ufunc_hankel1_data[1] = &ufunc_hankel1_ptr[2*1]
 * hankel1 = np.PyUFunc_FromFuncAndData(ufunc_hankel1_loops, ufunc_hankel1_data, ufunc_hankel1_types, 2, 2, 1, 0, "hankel1", ufunc_hankel1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hankel1e_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types, 2, 2, 1, 0, __pyx_k_hankel1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hankel1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5251
 * cdef void *ufunc_hankel1e_data[2]
 * cdef char ufunc_hankel1e_types[6]
 * cdef char *ufunc_hankel1e_doc = (             # <<<<<<<<<<<<<<
 *     "hankel1e(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_doc = __pyx_k_hankel1e_v_z_Exponentially_scale;

  /* "scipy/special/_ufuncs.pyx":5266
 *     "z : complex\n"
 *     "    Argument")
 * ufunc_hankel1e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5267
 *     "    Argument")
 * ufunc_hankel1e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel1e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5268
 * ufunc_hankel1e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel1e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5269
 * ufunc_hankel1e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5270
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5271
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5272
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5273
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5274
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[0]) = ((void *)cbesh_wrap1_e);

  /* "scipy/special/_ufuncs.pyx":5275
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[1]) = ((void *)((char *)__pyx_k_hankel1e));

  /* "scipy/special/_ufuncs.pyx":5276
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[2]) = ((void *)cbesh_wrap1_e);

  /* "scipy/special/_ufuncs.pyx":5277
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]
 * ufunc_hankel1e_data[1] = &ufunc_hankel1e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[3]) = ((void *)((char *)__pyx_k_hankel1e));

  /* "scipy/special/_ufuncs.pyx":5278
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_data[1] = &ufunc_hankel1e_ptr[2*1]
 * hankel1e = np.PyUFunc_FromFuncAndData(ufunc_hankel1e_loops, ufunc_hankel1e_data, ufunc_hankel1e_types, 2, 2, 1, 0, "hankel1e", ufunc_hankel1e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5279
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]
 * ufunc_hankel1e_data[1] = &ufunc_hankel1e_ptr[2*1]             # <<<<<<<<<<<<<<
 * hankel1e = np.PyUFunc_FromFuncAndData(ufunc_hankel1e_loops, ufunc_hankel1e_data, ufunc_hankel1e_types, 2, 2, 1, 0, "hankel1e", ufunc_hankel1e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5280
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]
 * ufunc_hankel1e_data[1] = &ufunc_hankel1e_ptr[2*1]
 * hankel1e = np.PyUFunc_FromFuncAndData(ufunc_hankel1e_loops, ufunc_hankel1e_data, ufunc_hankel1e_types, 2, 2, 1, 0, "hankel1e", ufunc_hankel1e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hankel2_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types, 2, 2, 1, 0, __pyx_k_hankel1e, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hankel1e, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5286
 * cdef void *ufunc_hankel2_data[2]
 * cdef char ufunc_hankel2_types[6]
 * cdef char *ufunc_hankel2_doc = (             # <<<<<<<<<<<<<<
 *     "hankel2(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_doc = __pyx_k_hankel2_v_z_Hankel_function_of_t;

  /* "scipy/special/_ufuncs.pyx":5297
 *     "z : complex\n"
 *     "    Argument")
 * ufunc_hankel2_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hankel2_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5298
 *     "    Argument")
 * ufunc_hankel2_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel2_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5299
 * ufunc_hankel2_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel2_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5300
 * ufunc_hankel2_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5301
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5302
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5303
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5304
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5305
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2             # <<<<<<<<<<<<<<
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[0]) = ((void *)cbesh_wrap2);

  /* "scipy/special/_ufuncs.pyx":5306
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")             # <<<<<<<<<<<<<<
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[1]) = ((void *)((char *)__pyx_k_hankel2));

  /* "scipy/special/_ufuncs.pyx":5307
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2             # <<<<<<<<<<<<<<
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[2]) = ((void *)cbesh_wrap2);

  /* "scipy/special/_ufuncs.pyx":5308
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")             # <<<<<<<<<<<<<<
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]
 * ufunc_hankel2_data[1] = &ufunc_hankel2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[3]) = ((void *)((char *)__pyx_k_hankel2));

  /* "scipy/special/_ufuncs.pyx":5309
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hankel2_data[1] = &ufunc_hankel2_ptr[2*1]
 * hankel2 = np.PyUFunc_FromFuncAndData(ufunc_hankel2_loops, ufunc_hankel2_data, ufunc_hankel2_types, 2, 2, 1, 0, "hankel2", ufunc_hankel2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5310
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]
 * ufunc_hankel2_data[1] = &ufunc_hankel2_ptr[2*1]             # <<<<<<<<<<<<<<
 * hankel2 = np.PyUFunc_FromFuncAndData(ufunc_hankel2_loops, ufunc_hankel2_data, ufunc_hankel2_types, 2, 2, 1, 0, "hankel2", ufunc_hankel2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5311
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]
 * ufunc_hankel2_data[1] = &ufunc_hankel2_ptr[2*1]
 * hankel2 = np.PyUFunc_FromFuncAndData(ufunc_hankel2_loops, ufunc_hankel2_data, ufunc_hankel2_types, 2, 2, 1, 0, "hankel2", ufunc_hankel2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hankel2e_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types, 2, 2, 1, 0, __pyx_k_hankel2, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5311; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hankel2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5311; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5317
 * cdef void *ufunc_hankel2e_data[2]
 * cdef char ufunc_hankel2e_types[6]
 * cdef char *ufunc_hankel2e_doc = (             # <<<<<<<<<<<<<<
 *     "hankel2e(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_doc = __pyx_k_hankel2e_v_z_Exponentially_scale;

  /* "scipy/special/_ufuncs.pyx":5332
 *     "z : complex\n"
 *     "    Argument")
 * ufunc_hankel2e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5333
 *     "    Argument")
 * ufunc_hankel2e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel2e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5334
 * ufunc_hankel2e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel2e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5335
 * ufunc_hankel2e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5336
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5337
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5338
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5339
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5340
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[0]) = ((void *)cbesh_wrap2_e);

  /* "scipy/special/_ufuncs.pyx":5341
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[1]) = ((void *)((char *)__pyx_k_hankel2e));

  /* "scipy/special/_ufuncs.pyx":5342
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[2]) = ((void *)cbesh_wrap2_e);

  /* "scipy/special/_ufuncs.pyx":5343
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]
 * ufunc_hankel2e_data[1] = &ufunc_hankel2e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[3]) = ((void *)((char *)__pyx_k_hankel2e));

  /* "scipy/special/_ufuncs.pyx":5344
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_data[1] = &ufunc_hankel2e_ptr[2*1]
 * hankel2e = np.PyUFunc_FromFuncAndData(ufunc_hankel2e_loops, ufunc_hankel2e_data, ufunc_hankel2e_types, 2, 2, 1, 0, "hankel2e", ufunc_hankel2e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5345
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]
 * ufunc_hankel2e_data[1] = &ufunc_hankel2e_ptr[2*1]             # <<<<<<<<<<<<<<
 * hankel2e = np.PyUFunc_FromFuncAndData(ufunc_hankel2e_loops, ufunc_hankel2e_data, ufunc_hankel2e_types, 2, 2, 1, 0, "hankel2e", ufunc_hankel2e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5346
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]
 * ufunc_hankel2e_data[1] = &ufunc_hankel2e_ptr[2*1]
 * hankel2e = np.PyUFunc_FromFuncAndData(ufunc_hankel2e_loops, ufunc_hankel2e_data, ufunc_hankel2e_types, 2, 2, 1, 0, "hankel2e", ufunc_hankel2e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_huber_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types, 2, 2, 1, 0, __pyx_k_hankel2e, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hankel2e, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5352
 * cdef void *ufunc_huber_data[2]
 * cdef char ufunc_huber_types[6]
 * cdef char *ufunc_huber_doc = (             # <<<<<<<<<<<<<<
 *     "huber(delta, r)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_doc = __pyx_k_huber_delta_r_Huber_loss_functio;

  /* "scipy/special/_ufuncs.pyx":5376
 *     "\n"
 *     ".. versionadded:: 0.15.0")
 * ufunc_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_huber_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5377
 *     ".. versionadded:: 0.15.0")
 * ufunc_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_huber_types[0] = <char>NPY_FLOAT
 * ufunc_huber_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5378
 * ufunc_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_huber_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_huber_types[1] = <char>NPY_FLOAT
 * ufunc_huber_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5379
 * ufunc_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_huber_types[0] = <char>NPY_FLOAT
 * ufunc_huber_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_huber_types[2] = <char>NPY_FLOAT
 * ufunc_huber_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5380
 * ufunc_huber_types[0] = <char>NPY_FLOAT
 * ufunc_huber_types[1] = <char>NPY_FLOAT
 * ufunc_huber_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_huber_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5381
 * ufunc_huber_types[1] = <char>NPY_FLOAT
 * ufunc_huber_types[2] = <char>NPY_FLOAT
 * ufunc_huber_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_huber_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5382
 * ufunc_huber_types[2] = <char>NPY_FLOAT
 * ufunc_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_huber_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_huber_ptr[2*0] = <void*>_func_huber
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5383
 * ufunc_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_huber_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_huber_ptr[2*0] = <void*>_func_huber
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5384
 * ufunc_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_huber_ptr[2*0] = <void*>_func_huber             # <<<<<<<<<<<<<<
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")
 * ufunc_huber_ptr[2*1] = <void*>_func_huber
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_huber);

  /* "scipy/special/_ufuncs.pyx":5385
 * ufunc_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_huber_ptr[2*0] = <void*>_func_huber
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")             # <<<<<<<<<<<<<<
 * ufunc_huber_ptr[2*1] = <void*>_func_huber
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[1]) = ((void *)((char *)__pyx_k_huber));

  /* "scipy/special/_ufuncs.pyx":5386
 * ufunc_huber_ptr[2*0] = <void*>_func_huber
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")
 * ufunc_huber_ptr[2*1] = <void*>_func_huber             # <<<<<<<<<<<<<<
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_huber);

  /* "scipy/special/_ufuncs.pyx":5387
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")
 * ufunc_huber_ptr[2*1] = <void*>_func_huber
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")             # <<<<<<<<<<<<<<
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]
 * ufunc_huber_data[1] = &ufunc_huber_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[3]) = ((void *)((char *)__pyx_k_huber));

  /* "scipy/special/_ufuncs.pyx":5388
 * ufunc_huber_ptr[2*1] = <void*>_func_huber
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_huber_data[1] = &ufunc_huber_ptr[2*1]
 * huber = np.PyUFunc_FromFuncAndData(ufunc_huber_loops, ufunc_huber_data, ufunc_huber_types, 2, 2, 1, 0, "huber", ufunc_huber_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5389
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]
 * ufunc_huber_data[1] = &ufunc_huber_ptr[2*1]             # <<<<<<<<<<<<<<
 * huber = np.PyUFunc_FromFuncAndData(ufunc_huber_loops, ufunc_huber_data, ufunc_huber_types, 2, 2, 1, 0, "huber", ufunc_huber_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5390
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]
 * ufunc_huber_data[1] = &ufunc_huber_ptr[2*1]
 * huber = np.PyUFunc_FromFuncAndData(ufunc_huber_loops, ufunc_huber_data, ufunc_huber_types, 2, 2, 1, 0, "huber", ufunc_huber_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp1f1_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types, 2, 2, 1, 0, __pyx_k_huber, __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5390; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_huber, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5390; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5396
 * cdef void *ufunc_hyp1f1_data[4]
 * cdef char ufunc_hyp1f1_types[16]
 * cdef char *ufunc_hyp1f1_doc = (             # <<<<<<<<<<<<<<
 *     "hyp1f1(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_doc = __pyx_k_hyp1f1_a_b_x_Confluent_hypergeom;

  /* "scipy/special/_ufuncs.pyx":5400
 *     "\n"
 *     "Confluent hypergeometric function 1F1(a, b; x)")
 * ufunc_hyp1f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":5401
 *     "Confluent hypergeometric function 1F1(a, b; x)")
 * ufunc_hyp1f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_hyp1f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F);

  /* "scipy/special/_ufuncs.pyx":5402
 * ufunc_hyp1f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_hyp1f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":5403
 * ufunc_hyp1f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D);

  /* "scipy/special/_ufuncs.pyx":5404
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5405
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5406
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5407
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5408
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5409
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5410
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5411
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5412
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5413
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5414
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5415
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5416
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5417
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5418
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5419
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[15]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5420
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[0]) = ((void *)hyp1f1_wrap);

  /* "scipy/special/_ufuncs.pyx":5421
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[1]) = ((void *)((char *)__pyx_k_hyp1f1));

  /* "scipy/special/_ufuncs.pyx":5422
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[2]) = ((void *)chyp1f1_wrap);

  /* "scipy/special/_ufuncs.pyx":5423
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[3]) = ((void *)((char *)__pyx_k_hyp1f1));

  /* "scipy/special/_ufuncs.pyx":5424
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[4]) = ((void *)hyp1f1_wrap);

  /* "scipy/special/_ufuncs.pyx":5425
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[5]) = ((void *)((char *)__pyx_k_hyp1f1));

  /* "scipy/special/_ufuncs.pyx":5426
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[6]) = ((void *)chyp1f1_wrap);

  /* "scipy/special/_ufuncs.pyx":5427
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[7]) = ((void *)((char *)__pyx_k_hyp1f1));

  /* "scipy/special/_ufuncs.pyx":5428
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5429
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]
 * ufunc_hyp1f1_data[3] = &ufunc_hyp1f1_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5430
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_data[3] = &ufunc_hyp1f1_ptr[2*3]
 * hyp1f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f1_loops, ufunc_hyp1f1_data, ufunc_hyp1f1_types, 4, 3, 1, 0, "hyp1f1", ufunc_hyp1f1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5431
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]
 * ufunc_hyp1f1_data[3] = &ufunc_hyp1f1_ptr[2*3]             # <<<<<<<<<<<<<<
 * hyp1f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f1_loops, ufunc_hyp1f1_data, ufunc_hyp1f1_types, 4, 3, 1, 0, "hyp1f1", ufunc_hyp1f1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5432
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]
 * ufunc_hyp1f1_data[3] = &ufunc_hyp1f1_ptr[2*3]
 * hyp1f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f1_loops, ufunc_hyp1f1_data, ufunc_hyp1f1_types, 4, 3, 1, 0, "hyp1f1", ufunc_hyp1f1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp1f2_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types, 4, 3, 1, 0, __pyx_k_hyp1f1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp1f1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5438
 * cdef void *ufunc_hyp1f2_data[2]
 * cdef char ufunc_hyp1f2_types[12]
 * cdef char *ufunc_hyp1f2_doc = (             # <<<<<<<<<<<<<<
 *     "hyp1f2(a, b, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_doc = __pyx_k_hyp1f2_a_b_c_x_Hypergeometric_fu;

  /* "scipy/special/_ufuncs.pyx":5449
 *     "err\n"
 *     "    Error estimate")
 * ufunc_hyp1f2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":5450
 *     "    Error estimate")
 * ufunc_hyp1f2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp1f2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":5451
 * ufunc_hyp1f2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp1f2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5452
 * ufunc_hyp1f2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5453
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5454
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5455
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5456
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5457
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5458
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5459
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5460
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5461
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5462
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5463
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[0]) = ((void *)onef2);

  /* "scipy/special/_ufuncs.pyx":5464
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[1]) = ((void *)((char *)__pyx_k_hyp1f2));

  /* "scipy/special/_ufuncs.pyx":5465
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[2]) = ((void *)onef2);

  /* "scipy/special/_ufuncs.pyx":5466
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]
 * ufunc_hyp1f2_data[1] = &ufunc_hyp1f2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[3]) = ((void *)((char *)__pyx_k_hyp1f2));

  /* "scipy/special/_ufuncs.pyx":5467
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_data[1] = &ufunc_hyp1f2_ptr[2*1]
 * hyp1f2 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f2_loops, ufunc_hyp1f2_data, ufunc_hyp1f2_types, 2, 4, 2, 0, "hyp1f2", ufunc_hyp1f2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5468
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]
 * ufunc_hyp1f2_data[1] = &ufunc_hyp1f2_ptr[2*1]             # <<<<<<<<<<<<<<
 * hyp1f2 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f2_loops, ufunc_hyp1f2_data, ufunc_hyp1f2_types, 2, 4, 2, 0, "hyp1f2", ufunc_hyp1f2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5469
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]
 * ufunc_hyp1f2_data[1] = &ufunc_hyp1f2_ptr[2*1]
 * hyp1f2 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f2_loops, ufunc_hyp1f2_data, ufunc_hyp1f2_types, 2, 4, 2, 0, "hyp1f2", ufunc_hyp1f2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp2f0_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types, 2, 4, 2, 0, __pyx_k_hyp1f2, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5469; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp1f2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5469; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5475
 * cdef void *ufunc_hyp2f0_data[3]
 * cdef char ufunc_hyp2f0_types[18]
 * cdef char *ufunc_hyp2f0_doc = (             # <<<<<<<<<<<<<<
 *     "hyp2f0(a, b, x, type)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_doc = __pyx_k_hyp2f0_a_b_x_type_Hypergeometric;

  /* "scipy/special/_ufuncs.pyx":5489
 *     "err\n"
 *     "    Error estimate")
 * ufunc_hyp2f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddi_d_As_dddl_dd
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":5490
 *     "    Error estimate")
 * ufunc_hyp2f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp2f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddi_d_As_dddl_dd             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddi_d_As_dddl_dd);

  /* "scipy/special/_ufuncs.pyx":5491
 * ufunc_hyp2f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp2f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddi_d_As_dddl_dd
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":5492
 * ufunc_hyp2f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddi_d_As_dddl_dd
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5493
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5494
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5495
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5496
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5497
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5498
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5499
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5500
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5501
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[9]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5502
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5503
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5504
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5505
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5506
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5507
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5508
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5509
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5510
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe);

  /* "scipy/special/_ufuncs.pyx":5511
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[1]) = ((void *)((char *)__pyx_k_hyp2f0));

  /* "scipy/special/_ufuncs.pyx":5512
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[2]) = ((void *)hyp2f0);

  /* "scipy/special/_ufuncs.pyx":5513
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[3]) = ((void *)((char *)__pyx_k_hyp2f0));

  /* "scipy/special/_ufuncs.pyx":5514
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe);

  /* "scipy/special/_ufuncs.pyx":5515
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[5]) = ((void *)((char *)__pyx_k_hyp2f0));

  /* "scipy/special/_ufuncs.pyx":5516
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]
 * ufunc_hyp2f0_data[2] = &ufunc_hyp2f0_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5517
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_data[2] = &ufunc_hyp2f0_ptr[2*2]
 * hyp2f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f0_loops, ufunc_hyp2f0_data, ufunc_hyp2f0_types, 3, 4, 2, 0, "hyp2f0", ufunc_hyp2f0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5518
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]
 * ufunc_hyp2f0_data[2] = &ufunc_hyp2f0_ptr[2*2]             # <<<<<<<<<<<<<<
 * hyp2f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f0_loops, ufunc_hyp2f0_data, ufunc_hyp2f0_types, 3, 4, 2, 0, "hyp2f0", ufunc_hyp2f0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5519
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]
 * ufunc_hyp2f0_data[2] = &ufunc_hyp2f0_ptr[2*2]
 * hyp2f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f0_loops, ufunc_hyp2f0_data, ufunc_hyp2f0_types, 3, 4, 2, 0, "hyp2f0", ufunc_hyp2f0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp2f1_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types, 3, 4, 2, 0, __pyx_k_hyp2f0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5519; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp2f0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5519; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5525
 * cdef void *ufunc_hyp2f1_data[4]
 * cdef char ufunc_hyp2f1_types[20]
 * cdef char *ufunc_hyp2f1_doc = (             # <<<<<<<<<<<<<<
 *     "hyp2f1(a, b, c, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_doc = __pyx_k_hyp2f1_a_b_c_z_Gauss_hypergeomet;

  /* "scipy/special/_ufuncs.pyx":5529
 *     "\n"
 *     "Gauss hypergeometric function 2F1(a, b; c; z).")
 * ufunc_hyp2f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":5530
 *     "Gauss hypergeometric function 2F1(a, b; c; z).")
 * ufunc_hyp2f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_hyp2f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F);

  /* "scipy/special/_ufuncs.pyx":5531
 * ufunc_hyp2f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_hyp2f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":5532
 * ufunc_hyp2f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D);

  /* "scipy/special/_ufuncs.pyx":5533
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5534
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5535
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5536
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5537
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5538
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5539
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5540
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5541
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5542
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[9]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5543
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5544
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5545
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5546
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5547
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5548
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5549
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5550
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5551
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5552
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5553
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[0]) = ((void *)hyp2f1);

  /* "scipy/special/_ufuncs.pyx":5554
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[1]) = ((void *)((char *)__pyx_k_hyp2f1));

  /* "scipy/special/_ufuncs.pyx":5555
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[2]) = ((void *)chyp2f1_wrap);

  /* "scipy/special/_ufuncs.pyx":5556
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[3]) = ((void *)((char *)__pyx_k_hyp2f1));

  /* "scipy/special/_ufuncs.pyx":5557
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[4]) = ((void *)hyp2f1);

  /* "scipy/special/_ufuncs.pyx":5558
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[5]) = ((void *)((char *)__pyx_k_hyp2f1));

  /* "scipy/special/_ufuncs.pyx":5559
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[6]) = ((void *)chyp2f1_wrap);

  /* "scipy/special/_ufuncs.pyx":5560
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[7]) = ((void *)((char *)__pyx_k_hyp2f1));

  /* "scipy/special/_ufuncs.pyx":5561
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5562
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]
 * ufunc_hyp2f1_data[3] = &ufunc_hyp2f1_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5563
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_data[3] = &ufunc_hyp2f1_ptr[2*3]
 * hyp2f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f1_loops, ufunc_hyp2f1_data, ufunc_hyp2f1_types, 4, 4, 1, 0, "hyp2f1", ufunc_hyp2f1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5564
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]
 * ufunc_hyp2f1_data[3] = &ufunc_hyp2f1_ptr[2*3]             # <<<<<<<<<<<<<<
 * hyp2f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f1_loops, ufunc_hyp2f1_data, ufunc_hyp2f1_types, 4, 4, 1, 0, "hyp2f1", ufunc_hyp2f1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5565
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]
 * ufunc_hyp2f1_data[3] = &ufunc_hyp2f1_ptr[2*3]
 * hyp2f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f1_loops, ufunc_hyp2f1_data, ufunc_hyp2f1_types, 4, 4, 1, 0, "hyp2f1", ufunc_hyp2f1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp3f0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types, 4, 4, 1, 0, __pyx_k_hyp2f1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5565; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp2f1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5565; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5571
 * cdef void *ufunc_hyp3f0_data[2]
 * cdef char ufunc_hyp3f0_types[12]
 * cdef char *ufunc_hyp3f0_doc = (             # <<<<<<<<<<<<<<
 *     "hyp3f0(a, b, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_doc = __pyx_k_hyp3f0_a_b_c_x_Hypergeometric_fu;

  /* "scipy/special/_ufuncs.pyx":5582
 *     "err\n"
 *     "    Error estimate")
 * ufunc_hyp3f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":5583
 *     "    Error estimate")
 * ufunc_hyp3f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp3f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":5584
 * ufunc_hyp3f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp3f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5585
 * ufunc_hyp3f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5586
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5587
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5588
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5589
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5590
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5591
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5592
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5593
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5594
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5595
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5596
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[0]) = ((void *)threef0);

  /* "scipy/special/_ufuncs.pyx":5597
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[1]) = ((void *)((char *)__pyx_k_hyp3f0));

  /* "scipy/special/_ufuncs.pyx":5598
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[2]) = ((void *)threef0);

  /* "scipy/special/_ufuncs.pyx":5599
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]
 * ufunc_hyp3f0_data[1] = &ufunc_hyp3f0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[3]) = ((void *)((char *)__pyx_k_hyp3f0));

  /* "scipy/special/_ufuncs.pyx":5600
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_data[1] = &ufunc_hyp3f0_ptr[2*1]
 * hyp3f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp3f0_loops, ufunc_hyp3f0_data, ufunc_hyp3f0_types, 2, 4, 2, 0, "hyp3f0", ufunc_hyp3f0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5601
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]
 * ufunc_hyp3f0_data[1] = &ufunc_hyp3f0_ptr[2*1]             # <<<<<<<<<<<<<<
 * hyp3f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp3f0_loops, ufunc_hyp3f0_data, ufunc_hyp3f0_types, 2, 4, 2, 0, "hyp3f0", ufunc_hyp3f0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5602
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]
 * ufunc_hyp3f0_data[1] = &ufunc_hyp3f0_ptr[2*1]
 * hyp3f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp3f0_loops, ufunc_hyp3f0_data, ufunc_hyp3f0_types, 2, 4, 2, 0, "hyp3f0", ufunc_hyp3f0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyperu_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types, 2, 4, 2, 0, __pyx_k_hyp3f0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5602; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp3f0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5602; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5608
 * cdef void *ufunc_hyperu_data[2]
 * cdef char ufunc_hyperu_types[8]
 * cdef char *ufunc_hyperu_doc = (             # <<<<<<<<<<<<<<
 *     "hyperu(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_doc = __pyx_k_hyperu_a_b_x_Confluent_hypergeom;

  /* "scipy/special/_ufuncs.pyx":5612
 *     "\n"
 *     "Confluent hypergeometric function U(a, b, x) of the second kind")
 * ufunc_hyperu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_hyperu_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":5613
 *     "Confluent hypergeometric function U(a, b, x) of the second kind")
 * ufunc_hyperu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_hyperu_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":5614
 * ufunc_hyperu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_hyperu_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5615
 * ufunc_hyperu_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5616
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5617
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5618
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5619
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5620
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5621
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5622
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[0]) = ((void *)hypU_wrap);

  /* "scipy/special/_ufuncs.pyx":5623
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")             # <<<<<<<<<<<<<<
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[1]) = ((void *)((char *)__pyx_k_hyperu));

  /* "scipy/special/_ufuncs.pyx":5624
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[2]) = ((void *)hypU_wrap);

  /* "scipy/special/_ufuncs.pyx":5625
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")             # <<<<<<<<<<<<<<
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]
 * ufunc_hyperu_data[1] = &ufunc_hyperu_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[3]) = ((void *)((char *)__pyx_k_hyperu));

  /* "scipy/special/_ufuncs.pyx":5626
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyperu_data[1] = &ufunc_hyperu_ptr[2*1]
 * hyperu = np.PyUFunc_FromFuncAndData(ufunc_hyperu_loops, ufunc_hyperu_data, ufunc_hyperu_types, 2, 3, 1, 0, "hyperu", ufunc_hyperu_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5627
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]
 * ufunc_hyperu_data[1] = &ufunc_hyperu_ptr[2*1]             # <<<<<<<<<<<<<<
 * hyperu = np.PyUFunc_FromFuncAndData(ufunc_hyperu_loops, ufunc_hyperu_data, ufunc_hyperu_types, 2, 3, 1, 0, "hyperu", ufunc_hyperu_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5628
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]
 * ufunc_hyperu_data[1] = &ufunc_hyperu_ptr[2*1]
 * hyperu = np.PyUFunc_FromFuncAndData(ufunc_hyperu_loops, ufunc_hyperu_data, ufunc_hyperu_types, 2, 3, 1, 0, "hyperu", ufunc_hyperu_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_i0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types, 2, 3, 1, 0, __pyx_k_hyperu, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyperu, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5634
 * cdef void *ufunc_i0_data[2]
 * cdef char ufunc_i0_types[4]
 * cdef char *ufunc_i0_doc = (             # <<<<<<<<<<<<<<
 *     "i0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_doc = __pyx_k_i0_x_Modified_Bessel_function_of;

  /* "scipy/special/_ufuncs.pyx":5638
 *     "\n"
 *     "Modified Bessel function of order 0")
 * ufunc_i0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_i0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":5639
 *     "Modified Bessel function of order 0")
 * ufunc_i0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_i0_types[0] = <char>NPY_FLOAT
 * ufunc_i0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":5640
 * ufunc_i0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i0_types[1] = <char>NPY_FLOAT
 * ufunc_i0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5641
 * ufunc_i0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0_types[0] = <char>NPY_FLOAT
 * ufunc_i0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i0_types[2] = <char>NPY_DOUBLE
 * ufunc_i0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5642
 * ufunc_i0_types[0] = <char>NPY_FLOAT
 * ufunc_i0_types[1] = <char>NPY_FLOAT
 * ufunc_i0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i0_types[3] = <char>NPY_DOUBLE
 * ufunc_i0_ptr[2*0] = <void*>_func_i0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5643
 * ufunc_i0_types[1] = <char>NPY_FLOAT
 * ufunc_i0_types[2] = <char>NPY_DOUBLE
 * ufunc_i0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i0_ptr[2*0] = <void*>_func_i0
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5644
 * ufunc_i0_types[2] = <char>NPY_DOUBLE
 * ufunc_i0_types[3] = <char>NPY_DOUBLE
 * ufunc_i0_ptr[2*0] = <void*>_func_i0             # <<<<<<<<<<<<<<
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")
 * ufunc_i0_ptr[2*1] = <void*>_func_i0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[0]) = ((void *)i0);

  /* "scipy/special/_ufuncs.pyx":5645
 * ufunc_i0_types[3] = <char>NPY_DOUBLE
 * ufunc_i0_ptr[2*0] = <void*>_func_i0
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")             # <<<<<<<<<<<<<<
 * ufunc_i0_ptr[2*1] = <void*>_func_i0
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[1]) = ((void *)((char *)__pyx_k_i0));

  /* "scipy/special/_ufuncs.pyx":5646
 * ufunc_i0_ptr[2*0] = <void*>_func_i0
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")
 * ufunc_i0_ptr[2*1] = <void*>_func_i0             # <<<<<<<<<<<<<<
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[2]) = ((void *)i0);

  /* "scipy/special/_ufuncs.pyx":5647
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")
 * ufunc_i0_ptr[2*1] = <void*>_func_i0
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")             # <<<<<<<<<<<<<<
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]
 * ufunc_i0_data[1] = &ufunc_i0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[3]) = ((void *)((char *)__pyx_k_i0));

  /* "scipy/special/_ufuncs.pyx":5648
 * ufunc_i0_ptr[2*1] = <void*>_func_i0
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_i0_data[1] = &ufunc_i0_ptr[2*1]
 * i0 = np.PyUFunc_FromFuncAndData(ufunc_i0_loops, ufunc_i0_data, ufunc_i0_types, 2, 1, 1, 0, "i0", ufunc_i0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5649
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]
 * ufunc_i0_data[1] = &ufunc_i0_ptr[2*1]             # <<<<<<<<<<<<<<
 * i0 = np.PyUFunc_FromFuncAndData(ufunc_i0_loops, ufunc_i0_data, ufunc_i0_types, 2, 1, 1, 0, "i0", ufunc_i0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5650
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]
 * ufunc_i0_data[1] = &ufunc_i0_ptr[2*1]
 * i0 = np.PyUFunc_FromFuncAndData(ufunc_i0_loops, ufunc_i0_data, ufunc_i0_types, 2, 1, 1, 0, "i0", ufunc_i0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_i0e_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types, 2, 1, 1, 0, __pyx_k_i0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_i0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5656
 * cdef void *ufunc_i0e_data[2]
 * cdef char ufunc_i0e_types[4]
 * cdef char *ufunc_i0e_doc = (             # <<<<<<<<<<<<<<
 *     "i0e(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_doc = __pyx_k_i0e_x_Exponentially_scaled_modif;

  /* "scipy/special/_ufuncs.pyx":5664
 *     "\n"
 *     "    i0e(x) = exp(-abs(x)) * i0(x).")
 * ufunc_i0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_i0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":5665
 *     "    i0e(x) = exp(-abs(x)) * i0(x).")
 * ufunc_i0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_i0e_types[0] = <char>NPY_FLOAT
 * ufunc_i0e_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":5666
 * ufunc_i0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i0e_types[1] = <char>NPY_FLOAT
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5667
 * ufunc_i0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0e_types[0] = <char>NPY_FLOAT
 * ufunc_i0e_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5668
 * ufunc_i0e_types[0] = <char>NPY_FLOAT
 * ufunc_i0e_types[1] = <char>NPY_FLOAT
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5669
 * ufunc_i0e_types[1] = <char>NPY_FLOAT
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5670
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e             # <<<<<<<<<<<<<<
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[0]) = ((void *)i0e);

  /* "scipy/special/_ufuncs.pyx":5671
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")             # <<<<<<<<<<<<<<
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[1]) = ((void *)((char *)__pyx_k_i0e));

  /* "scipy/special/_ufuncs.pyx":5672
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e             # <<<<<<<<<<<<<<
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[2]) = ((void *)i0e);

  /* "scipy/special/_ufuncs.pyx":5673
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")             # <<<<<<<<<<<<<<
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]
 * ufunc_i0e_data[1] = &ufunc_i0e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[3]) = ((void *)((char *)__pyx_k_i0e));

  /* "scipy/special/_ufuncs.pyx":5674
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_i0e_data[1] = &ufunc_i0e_ptr[2*1]
 * i0e = np.PyUFunc_FromFuncAndData(ufunc_i0e_loops, ufunc_i0e_data, ufunc_i0e_types, 2, 1, 1, 0, "i0e", ufunc_i0e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5675
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]
 * ufunc_i0e_data[1] = &ufunc_i0e_ptr[2*1]             # <<<<<<<<<<<<<<
 * i0e = np.PyUFunc_FromFuncAndData(ufunc_i0e_loops, ufunc_i0e_data, ufunc_i0e_types, 2, 1, 1, 0, "i0e", ufunc_i0e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5676
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]
 * ufunc_i0e_data[1] = &ufunc_i0e_ptr[2*1]
 * i0e = np.PyUFunc_FromFuncAndData(ufunc_i0e_loops, ufunc_i0e_data, ufunc_i0e_types, 2, 1, 1, 0, "i0e", ufunc_i0e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_i1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types, 2, 1, 1, 0, __pyx_k_i0e, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5676; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_i0e, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5676; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5682
 * cdef void *ufunc_i1_data[2]
 * cdef char ufunc_i1_types[4]
 * cdef char *ufunc_i1_doc = (             # <<<<<<<<<<<<<<
 *     "i1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_doc = __pyx_k_i1_x_Modified_Bessel_function_of;

  /* "scipy/special/_ufuncs.pyx":5686
 *     "\n"
 *     "Modified Bessel function of order 1")
 * ufunc_i1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_i1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":5687
 *     "Modified Bessel function of order 1")
 * ufunc_i1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_i1_types[0] = <char>NPY_FLOAT
 * ufunc_i1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":5688
 * ufunc_i1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i1_types[1] = <char>NPY_FLOAT
 * ufunc_i1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5689
 * ufunc_i1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1_types[0] = <char>NPY_FLOAT
 * ufunc_i1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i1_types[2] = <char>NPY_DOUBLE
 * ufunc_i1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5690
 * ufunc_i1_types[0] = <char>NPY_FLOAT
 * ufunc_i1_types[1] = <char>NPY_FLOAT
 * ufunc_i1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i1_types[3] = <char>NPY_DOUBLE
 * ufunc_i1_ptr[2*0] = <void*>_func_i1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5691
 * ufunc_i1_types[1] = <char>NPY_FLOAT
 * ufunc_i1_types[2] = <char>NPY_DOUBLE
 * ufunc_i1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i1_ptr[2*0] = <void*>_func_i1
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5692
 * ufunc_i1_types[2] = <char>NPY_DOUBLE
 * ufunc_i1_types[3] = <char>NPY_DOUBLE
 * ufunc_i1_ptr[2*0] = <void*>_func_i1             # <<<<<<<<<<<<<<
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")
 * ufunc_i1_ptr[2*1] = <void*>_func_i1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[0]) = ((void *)i1);

  /* "scipy/special/_ufuncs.pyx":5693
 * ufunc_i1_types[3] = <char>NPY_DOUBLE
 * ufunc_i1_ptr[2*0] = <void*>_func_i1
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")             # <<<<<<<<<<<<<<
 * ufunc_i1_ptr[2*1] = <void*>_func_i1
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[1]) = ((void *)((char *)__pyx_k_i1));

  /* "scipy/special/_ufuncs.pyx":5694
 * ufunc_i1_ptr[2*0] = <void*>_func_i1
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")
 * ufunc_i1_ptr[2*1] = <void*>_func_i1             # <<<<<<<<<<<<<<
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[2]) = ((void *)i1);

  /* "scipy/special/_ufuncs.pyx":5695
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")
 * ufunc_i1_ptr[2*1] = <void*>_func_i1
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")             # <<<<<<<<<<<<<<
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]
 * ufunc_i1_data[1] = &ufunc_i1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[3]) = ((void *)((char *)__pyx_k_i1));

  /* "scipy/special/_ufuncs.pyx":5696
 * ufunc_i1_ptr[2*1] = <void*>_func_i1
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_i1_data[1] = &ufunc_i1_ptr[2*1]
 * i1 = np.PyUFunc_FromFuncAndData(ufunc_i1_loops, ufunc_i1_data, ufunc_i1_types, 2, 1, 1, 0, "i1", ufunc_i1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5697
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]
 * ufunc_i1_data[1] = &ufunc_i1_ptr[2*1]             # <<<<<<<<<<<<<<
 * i1 = np.PyUFunc_FromFuncAndData(ufunc_i1_loops, ufunc_i1_data, ufunc_i1_types, 2, 1, 1, 0, "i1", ufunc_i1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5698
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]
 * ufunc_i1_data[1] = &ufunc_i1_ptr[2*1]
 * i1 = np.PyUFunc_FromFuncAndData(ufunc_i1_loops, ufunc_i1_data, ufunc_i1_types, 2, 1, 1, 0, "i1", ufunc_i1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_i1e_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types, 2, 1, 1, 0, __pyx_k_i1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5698; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_i1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5698; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5704
 * cdef void *ufunc_i1e_data[2]
 * cdef char ufunc_i1e_types[4]
 * cdef char *ufunc_i1e_doc = (             # <<<<<<<<<<<<<<
 *     "i1e(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_doc = __pyx_k_i1e_x_Exponentially_scaled_modif;

  /* "scipy/special/_ufuncs.pyx":5712
 *     "\n"
 *     "    i1e(x) = exp(-abs(x)) * i1(x)")
 * ufunc_i1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_i1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":5713
 *     "    i1e(x) = exp(-abs(x)) * i1(x)")
 * ufunc_i1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_i1e_types[0] = <char>NPY_FLOAT
 * ufunc_i1e_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":5714
 * ufunc_i1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i1e_types[1] = <char>NPY_FLOAT
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5715
 * ufunc_i1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1e_types[0] = <char>NPY_FLOAT
 * ufunc_i1e_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5716
 * ufunc_i1e_types[0] = <char>NPY_FLOAT
 * ufunc_i1e_types[1] = <char>NPY_FLOAT
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5717
 * ufunc_i1e_types[1] = <char>NPY_FLOAT
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5718
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e             # <<<<<<<<<<<<<<
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[0]) = ((void *)i1e);

  /* "scipy/special/_ufuncs.pyx":5719
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")             # <<<<<<<<<<<<<<
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[1]) = ((void *)((char *)__pyx_k_i1e));

  /* "scipy/special/_ufuncs.pyx":5720
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e             # <<<<<<<<<<<<<<
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[2]) = ((void *)i1e);

  /* "scipy/special/_ufuncs.pyx":5721
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")             # <<<<<<<<<<<<<<
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]
 * ufunc_i1e_data[1] = &ufunc_i1e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[3]) = ((void *)((char *)__pyx_k_i1e));

  /* "scipy/special/_ufuncs.pyx":5722
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_i1e_data[1] = &ufunc_i1e_ptr[2*1]
 * i1e = np.PyUFunc_FromFuncAndData(ufunc_i1e_loops, ufunc_i1e_data, ufunc_i1e_types, 2, 1, 1, 0, "i1e", ufunc_i1e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5723
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]
 * ufunc_i1e_data[1] = &ufunc_i1e_ptr[2*1]             # <<<<<<<<<<<<<<
 * i1e = np.PyUFunc_FromFuncAndData(ufunc_i1e_loops, ufunc_i1e_data, ufunc_i1e_types, 2, 1, 1, 0, "i1e", ufunc_i1e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5724
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]
 * ufunc_i1e_data[1] = &ufunc_i1e_ptr[2*1]
 * i1e = np.PyUFunc_FromFuncAndData(ufunc_i1e_loops, ufunc_i1e_data, ufunc_i1e_types, 2, 1, 1, 0, "i1e", ufunc_i1e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_inv_boxcox_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types, 2, 1, 1, 0, __pyx_k_i1e, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5724; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_i1e, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5724; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5730
 * cdef void *ufunc_inv_boxcox_data[2]
 * cdef char ufunc_inv_boxcox_types[6]
 * cdef char *ufunc_inv_boxcox_doc = (             # <<<<<<<<<<<<<<
 *     "inv_boxcox(y, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_doc = __pyx_k_inv_boxcox_y_lmbda_Compute_the_i;

  /* "scipy/special/_ufuncs.pyx":5763
 *     ">>> inv_boxcox(y, 2.5)\n"
 *     "array([1., 4., 10.])")
 * ufunc_inv_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5764
 *     "array([1., 4., 10.])")
 * ufunc_inv_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_inv_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5765
 * ufunc_inv_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_inv_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5766
 * ufunc_inv_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5767
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5768
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5769
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5770
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5771
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_inv_boxcox);

  /* "scipy/special/_ufuncs.pyx":5772
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[1]) = ((void *)((char *)__pyx_k_inv_boxcox));

  /* "scipy/special/_ufuncs.pyx":5773
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_inv_boxcox);

  /* "scipy/special/_ufuncs.pyx":5774
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]
 * ufunc_inv_boxcox_data[1] = &ufunc_inv_boxcox_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[3]) = ((void *)((char *)__pyx_k_inv_boxcox));

  /* "scipy/special/_ufuncs.pyx":5775
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_data[1] = &ufunc_inv_boxcox_ptr[2*1]
 * inv_boxcox = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox_loops, ufunc_inv_boxcox_data, ufunc_inv_boxcox_types, 2, 2, 1, 0, "inv_boxcox", ufunc_inv_boxcox_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5776
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]
 * ufunc_inv_boxcox_data[1] = &ufunc_inv_boxcox_ptr[2*1]             # <<<<<<<<<<<<<<
 * inv_boxcox = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox_loops, ufunc_inv_boxcox_data, ufunc_inv_boxcox_types, 2, 2, 1, 0, "inv_boxcox", ufunc_inv_boxcox_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5777
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]
 * ufunc_inv_boxcox_data[1] = &ufunc_inv_boxcox_ptr[2*1]
 * inv_boxcox = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox_loops, ufunc_inv_boxcox_data, ufunc_inv_boxcox_types, 2, 2, 1, 0, "inv_boxcox", ufunc_inv_boxcox_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_inv_boxcox1p_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types, 2, 2, 1, 0, __pyx_k_inv_boxcox, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_inv_boxcox, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5783
 * cdef void *ufunc_inv_boxcox1p_data[2]
 * cdef char ufunc_inv_boxcox1p_types[6]
 * cdef char *ufunc_inv_boxcox1p_doc = (             # <<<<<<<<<<<<<<
 *     "inv_boxcox1p(y, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_doc = __pyx_k_inv_boxcox1p_y_lmbda_Compute_the;

  /* "scipy/special/_ufuncs.pyx":5816
 *     ">>> inv_boxcox1p(y, 2.5)\n"
 *     "array([1., 4., 10.])")
 * ufunc_inv_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5817
 *     "array([1., 4., 10.])")
 * ufunc_inv_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_inv_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5818
 * ufunc_inv_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_inv_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5819
 * ufunc_inv_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5820
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5821
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5822
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5823
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5824
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":5825
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[1]) = ((void *)((char *)__pyx_k_inv_boxcox1p));

  /* "scipy/special/_ufuncs.pyx":5826
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":5827
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]
 * ufunc_inv_boxcox1p_data[1] = &ufunc_inv_boxcox1p_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[3]) = ((void *)((char *)__pyx_k_inv_boxcox1p));

  /* "scipy/special/_ufuncs.pyx":5828
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_data[1] = &ufunc_inv_boxcox1p_ptr[2*1]
 * inv_boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox1p_loops, ufunc_inv_boxcox1p_data, ufunc_inv_boxcox1p_types, 2, 2, 1, 0, "inv_boxcox1p", ufunc_inv_boxcox1p_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5829
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]
 * ufunc_inv_boxcox1p_data[1] = &ufunc_inv_boxcox1p_ptr[2*1]             # <<<<<<<<<<<<<<
 * inv_boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox1p_loops, ufunc_inv_boxcox1p_data, ufunc_inv_boxcox1p_types, 2, 2, 1, 0, "inv_boxcox1p", ufunc_inv_boxcox1p_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5830
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]
 * ufunc_inv_boxcox1p_data[1] = &ufunc_inv_boxcox1p_ptr[2*1]
 * inv_boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox1p_loops, ufunc_inv_boxcox1p_data, ufunc_inv_boxcox1p_types, 2, 2, 1, 0, "inv_boxcox1p", ufunc_inv_boxcox1p_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_it2i0k0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types, 2, 2, 1, 0, __pyx_k_inv_boxcox1p, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_inv_boxcox1p, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5836
 * cdef void *ufunc_it2i0k0_data[2]
 * cdef char ufunc_it2i0k0_types[6]
 * cdef char *ufunc_it2i0k0_doc = (             # <<<<<<<<<<<<<<
 *     "it2i0k0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_doc = __pyx_k_it2i0k0_x_Integrals_related_to_m;

  /* "scipy/special/_ufuncs.pyx":5847
 *     "ik0\n"
 *     "    ``int(k0(t)/t, t=x..inf)``")
 * ufunc_it2i0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":5848
 *     "    ``int(k0(t)/t, t=x..inf)``")
 * ufunc_it2i0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_it2i0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":5849
 * ufunc_it2i0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_it2i0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5850
 * ufunc_it2i0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5851
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5852
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5853
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5854
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5855
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[0]) = ((void *)it2i0k0_wrap);

  /* "scipy/special/_ufuncs.pyx":5856
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[1]) = ((void *)((char *)__pyx_k_it2i0k0));

  /* "scipy/special/_ufuncs.pyx":5857
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[2]) = ((void *)it2i0k0_wrap);

  /* "scipy/special/_ufuncs.pyx":5858
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]
 * ufunc_it2i0k0_data[1] = &ufunc_it2i0k0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[3]) = ((void *)((char *)__pyx_k_it2i0k0));

  /* "scipy/special/_ufuncs.pyx":5859
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_data[1] = &ufunc_it2i0k0_ptr[2*1]
 * it2i0k0 = np.PyUFunc_FromFuncAndData(ufunc_it2i0k0_loops, ufunc_it2i0k0_data, ufunc_it2i0k0_types, 2, 1, 2, 0, "it2i0k0", ufunc_it2i0k0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5860
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]
 * ufunc_it2i0k0_data[1] = &ufunc_it2i0k0_ptr[2*1]             # <<<<<<<<<<<<<<
 * it2i0k0 = np.PyUFunc_FromFuncAndData(ufunc_it2i0k0_loops, ufunc_it2i0k0_data, ufunc_it2i0k0_types, 2, 1, 2, 0, "it2i0k0", ufunc_it2i0k0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5861
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]
 * ufunc_it2i0k0_data[1] = &ufunc_it2i0k0_ptr[2*1]
 * it2i0k0 = np.PyUFunc_FromFuncAndData(ufunc_it2i0k0_loops, ufunc_it2i0k0_data, ufunc_it2i0k0_types, 2, 1, 2, 0, "it2i0k0", ufunc_it2i0k0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_it2j0y0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types, 2, 1, 2, 0, __pyx_k_it2i0k0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_it2i0k0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5867
 * cdef void *ufunc_it2j0y0_data[2]
 * cdef char ufunc_it2j0y0_types[6]
 * cdef char *ufunc_it2j0y0_doc = (             # <<<<<<<<<<<<<<
 *     "it2j0y0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_doc = __pyx_k_it2j0y0_x_Integrals_related_to_B;

  /* "scipy/special/_ufuncs.pyx":5878
 *     "iy0\n"
 *     "    ``integral(y0(t)/t, t=x..inf)``")
 * ufunc_it2j0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":5879
 *     "    ``integral(y0(t)/t, t=x..inf)``")
 * ufunc_it2j0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_it2j0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":5880
 * ufunc_it2j0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_it2j0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5881
 * ufunc_it2j0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5882
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5883
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5884
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5885
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5886
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[0]) = ((void *)it2j0y0_wrap);

  /* "scipy/special/_ufuncs.pyx":5887
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[1]) = ((void *)((char *)__pyx_k_it2j0y0));

  /* "scipy/special/_ufuncs.pyx":5888
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[2]) = ((void *)it2j0y0_wrap);

  /* "scipy/special/_ufuncs.pyx":5889
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]
 * ufunc_it2j0y0_data[1] = &ufunc_it2j0y0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[3]) = ((void *)((char *)__pyx_k_it2j0y0));

  /* "scipy/special/_ufuncs.pyx":5890
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_data[1] = &ufunc_it2j0y0_ptr[2*1]
 * it2j0y0 = np.PyUFunc_FromFuncAndData(ufunc_it2j0y0_loops, ufunc_it2j0y0_data, ufunc_it2j0y0_types, 2, 1, 2, 0, "it2j0y0", ufunc_it2j0y0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5891
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]
 * ufunc_it2j0y0_data[1] = &ufunc_it2j0y0_ptr[2*1]             # <<<<<<<<<<<<<<
 * it2j0y0 = np.PyUFunc_FromFuncAndData(ufunc_it2j0y0_loops, ufunc_it2j0y0_data, ufunc_it2j0y0_types, 2, 1, 2, 0, "it2j0y0", ufunc_it2j0y0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5892
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]
 * ufunc_it2j0y0_data[1] = &ufunc_it2j0y0_ptr[2*1]
 * it2j0y0 = np.PyUFunc_FromFuncAndData(ufunc_it2j0y0_loops, ufunc_it2j0y0_data, ufunc_it2j0y0_types, 2, 1, 2, 0, "it2j0y0", ufunc_it2j0y0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_it2struve0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types, 2, 1, 2, 0, __pyx_k_it2j0y0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5892; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_it2j0y0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5892; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5898
 * cdef void *ufunc_it2struve0_data[2]
 * cdef char ufunc_it2struve0_types[4]
 * cdef char *ufunc_it2struve0_doc = (             # <<<<<<<<<<<<<<
 *     "it2struve0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_doc = __pyx_k_it2struve0_x_Integral_related_to;

  /* "scipy/special/_ufuncs.pyx":5907
 *     "i\n"
 *     "    ``integral(H0(t)/t, t=x..inf)``")
 * ufunc_it2struve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":5908
 *     "    ``integral(H0(t)/t, t=x..inf)``")
 * ufunc_it2struve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_it2struve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":5909
 * ufunc_it2struve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_it2struve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5910
 * ufunc_it2struve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5911
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5912
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5913
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[0]) = ((void *)it2struve0_wrap);

  /* "scipy/special/_ufuncs.pyx":5914
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[1]) = ((void *)((char *)__pyx_k_it2struve0));

  /* "scipy/special/_ufuncs.pyx":5915
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[2]) = ((void *)it2struve0_wrap);

  /* "scipy/special/_ufuncs.pyx":5916
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]
 * ufunc_it2struve0_data[1] = &ufunc_it2struve0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[3]) = ((void *)((char *)__pyx_k_it2struve0));

  /* "scipy/special/_ufuncs.pyx":5917
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_data[1] = &ufunc_it2struve0_ptr[2*1]
 * it2struve0 = np.PyUFunc_FromFuncAndData(ufunc_it2struve0_loops, ufunc_it2struve0_data, ufunc_it2struve0_types, 2, 1, 1, 0, "it2struve0", ufunc_it2struve0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5918
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]
 * ufunc_it2struve0_data[1] = &ufunc_it2struve0_ptr[2*1]             # <<<<<<<<<<<<<<
 * it2struve0 = np.PyUFunc_FromFuncAndData(ufunc_it2struve0_loops, ufunc_it2struve0_data, ufunc_it2struve0_types, 2, 1, 1, 0, "it2struve0", ufunc_it2struve0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5919
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]
 * ufunc_it2struve0_data[1] = &ufunc_it2struve0_ptr[2*1]
 * it2struve0 = np.PyUFunc_FromFuncAndData(ufunc_it2struve0_loops, ufunc_it2struve0_data, ufunc_it2struve0_types, 2, 1, 1, 0, "it2struve0", ufunc_it2struve0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_itairy_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types, 2, 1, 1, 0, __pyx_k_it2struve0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5919; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_it2struve0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5919; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5925
 * cdef void *ufunc_itairy_data[2]
 * cdef char ufunc_itairy_types[10]
 * cdef char *ufunc_itairy_doc = (             # <<<<<<<<<<<<<<
 *     "itairy(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_doc = __pyx_k_itairy_x_Integrals_of_Airy_funct;

  /* "scipy/special/_ufuncs.pyx":5938
 *     "Ant, Bnt\n"
 *     "    Integrals for negative arguments")
 * ufunc_itairy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff             # <<<<<<<<<<<<<<
 * ufunc_itairy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_itairy_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff);

  /* "scipy/special/_ufuncs.pyx":5939
 *     "    Integrals for negative arguments")
 * ufunc_itairy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_itairy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[0] = <char>NPY_FLOAT
 * ufunc_itairy_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd);

  /* "scipy/special/_ufuncs.pyx":5940
 * ufunc_itairy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_itairy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_itairy_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[1] = <char>NPY_FLOAT
 * ufunc_itairy_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5941
 * ufunc_itairy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_itairy_types[0] = <char>NPY_FLOAT
 * ufunc_itairy_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[2] = <char>NPY_FLOAT
 * ufunc_itairy_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5942
 * ufunc_itairy_types[0] = <char>NPY_FLOAT
 * ufunc_itairy_types[1] = <char>NPY_FLOAT
 * ufunc_itairy_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[3] = <char>NPY_FLOAT
 * ufunc_itairy_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5943
 * ufunc_itairy_types[1] = <char>NPY_FLOAT
 * ufunc_itairy_types[2] = <char>NPY_FLOAT
 * ufunc_itairy_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[4] = <char>NPY_FLOAT
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5944
 * ufunc_itairy_types[2] = <char>NPY_FLOAT
 * ufunc_itairy_types[3] = <char>NPY_FLOAT
 * ufunc_itairy_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5945
 * ufunc_itairy_types[3] = <char>NPY_FLOAT
 * ufunc_itairy_types[4] = <char>NPY_FLOAT
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5946
 * ufunc_itairy_types[4] = <char>NPY_FLOAT
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5947
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5948
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5949
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5950
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap             # <<<<<<<<<<<<<<
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[0]) = ((void *)itairy_wrap);

  /* "scipy/special/_ufuncs.pyx":5951
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")             # <<<<<<<<<<<<<<
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[1]) = ((void *)((char *)__pyx_k_itairy));

  /* "scipy/special/_ufuncs.pyx":5952
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap             # <<<<<<<<<<<<<<
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[2]) = ((void *)itairy_wrap);

  /* "scipy/special/_ufuncs.pyx":5953
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")             # <<<<<<<<<<<<<<
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]
 * ufunc_itairy_data[1] = &ufunc_itairy_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[3]) = ((void *)((char *)__pyx_k_itairy));

  /* "scipy/special/_ufuncs.pyx":5954
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_itairy_data[1] = &ufunc_itairy_ptr[2*1]
 * itairy = np.PyUFunc_FromFuncAndData(ufunc_itairy_loops, ufunc_itairy_data, ufunc_itairy_types, 2, 1, 4, 0, "itairy", ufunc_itairy_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5955
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]
 * ufunc_itairy_data[1] = &ufunc_itairy_ptr[2*1]             # <<<<<<<<<<<<<<
 * itairy = np.PyUFunc_FromFuncAndData(ufunc_itairy_loops, ufunc_itairy_data, ufunc_itairy_types, 2, 1, 4, 0, "itairy", ufunc_itairy_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5956
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]
 * ufunc_itairy_data[1] = &ufunc_itairy_ptr[2*1]
 * itairy = np.PyUFunc_FromFuncAndData(ufunc_itairy_loops, ufunc_itairy_data, ufunc_itairy_types, 2, 1, 4, 0, "itairy", ufunc_itairy_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_iti0k0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types, 2, 1, 4, 0, __pyx_k_itairy, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_itairy, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5962
 * cdef void *ufunc_iti0k0_data[2]
 * cdef char ufunc_iti0k0_types[6]
 * cdef char *ufunc_iti0k0_doc = (             # <<<<<<<<<<<<<<
 *     "iti0k0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_doc = __pyx_k_iti0k0_x_Integrals_of_modified_B;

  /* "scipy/special/_ufuncs.pyx":5973
 *     "-------\n"
 *     "ii0, ik0")
 * ufunc_iti0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":5974
 *     "ii0, ik0")
 * ufunc_iti0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_iti0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":5975
 * ufunc_iti0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_iti0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5976
 * ufunc_iti0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5977
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5978
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5979
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5980
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5981
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[0]) = ((void *)it1i0k0_wrap);

  /* "scipy/special/_ufuncs.pyx":5982
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[1]) = ((void *)((char *)__pyx_k_iti0k0));

  /* "scipy/special/_ufuncs.pyx":5983
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[2]) = ((void *)it1i0k0_wrap);

  /* "scipy/special/_ufuncs.pyx":5984
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]
 * ufunc_iti0k0_data[1] = &ufunc_iti0k0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[3]) = ((void *)((char *)__pyx_k_iti0k0));

  /* "scipy/special/_ufuncs.pyx":5985
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_data[1] = &ufunc_iti0k0_ptr[2*1]
 * iti0k0 = np.PyUFunc_FromFuncAndData(ufunc_iti0k0_loops, ufunc_iti0k0_data, ufunc_iti0k0_types, 2, 1, 2, 0, "iti0k0", ufunc_iti0k0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5986
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]
 * ufunc_iti0k0_data[1] = &ufunc_iti0k0_ptr[2*1]             # <<<<<<<<<<<<<<
 * iti0k0 = np.PyUFunc_FromFuncAndData(ufunc_iti0k0_loops, ufunc_iti0k0_data, ufunc_iti0k0_types, 2, 1, 2, 0, "iti0k0", ufunc_iti0k0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5987
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]
 * ufunc_iti0k0_data[1] = &ufunc_iti0k0_ptr[2*1]
 * iti0k0 = np.PyUFunc_FromFuncAndData(ufunc_iti0k0_loops, ufunc_iti0k0_data, ufunc_iti0k0_types, 2, 1, 2, 0, "iti0k0", ufunc_iti0k0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_itj0y0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types, 2, 1, 2, 0, __pyx_k_iti0k0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5987; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_iti0k0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 5987; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":5993
 * cdef void *ufunc_itj0y0_data[2]
 * cdef char ufunc_itj0y0_types[6]
 * cdef char *ufunc_itj0y0_doc = (             # <<<<<<<<<<<<<<
 *     "itj0y0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_doc = __pyx_k_itj0y0_x_Integrals_of_Bessel_fun;

  /* "scipy/special/_ufuncs.pyx":6004
 *     "-------\n"
 *     "ij0, iy0")
 * ufunc_itj0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":6005
 *     "ij0, iy0")
 * ufunc_itj0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_itj0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":6006
 * ufunc_itj0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_itj0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6007
 * ufunc_itj0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6008
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6009
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6010
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6011
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6012
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[0]) = ((void *)it1j0y0_wrap);

  /* "scipy/special/_ufuncs.pyx":6013
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[1]) = ((void *)((char *)__pyx_k_itj0y0));

  /* "scipy/special/_ufuncs.pyx":6014
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[2]) = ((void *)it1j0y0_wrap);

  /* "scipy/special/_ufuncs.pyx":6015
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]
 * ufunc_itj0y0_data[1] = &ufunc_itj0y0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[3]) = ((void *)((char *)__pyx_k_itj0y0));

  /* "scipy/special/_ufuncs.pyx":6016
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_data[1] = &ufunc_itj0y0_ptr[2*1]
 * itj0y0 = np.PyUFunc_FromFuncAndData(ufunc_itj0y0_loops, ufunc_itj0y0_data, ufunc_itj0y0_types, 2, 1, 2, 0, "itj0y0", ufunc_itj0y0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6017
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]
 * ufunc_itj0y0_data[1] = &ufunc_itj0y0_ptr[2*1]             # <<<<<<<<<<<<<<
 * itj0y0 = np.PyUFunc_FromFuncAndData(ufunc_itj0y0_loops, ufunc_itj0y0_data, ufunc_itj0y0_types, 2, 1, 2, 0, "itj0y0", ufunc_itj0y0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6018
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]
 * ufunc_itj0y0_data[1] = &ufunc_itj0y0_ptr[2*1]
 * itj0y0 = np.PyUFunc_FromFuncAndData(ufunc_itj0y0_loops, ufunc_itj0y0_data, ufunc_itj0y0_types, 2, 1, 2, 0, "itj0y0", ufunc_itj0y0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_itmodstruve0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types, 2, 1, 2, 0, __pyx_k_itj0y0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6018; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_itj0y0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6018; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6024
 * cdef void *ufunc_itmodstruve0_data[2]
 * cdef char ufunc_itmodstruve0_types[4]
 * cdef char *ufunc_itmodstruve0_doc = (             # <<<<<<<<<<<<<<
 *     "itmodstruve0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_doc = __pyx_k_itmodstruve0_x_Integral_of_the_m;

  /* "scipy/special/_ufuncs.pyx":6033
 *     "i\n"
 *     "    ``integral(L0(t), t=0..x)``")
 * ufunc_itmodstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6034
 *     "    ``integral(L0(t), t=0..x)``")
 * ufunc_itmodstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_itmodstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6035
 * ufunc_itmodstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_itmodstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6036
 * ufunc_itmodstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6037
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6038
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6039
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[0]) = ((void *)itmodstruve0_wrap);

  /* "scipy/special/_ufuncs.pyx":6040
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[1]) = ((void *)((char *)__pyx_k_itmodstruve0));

  /* "scipy/special/_ufuncs.pyx":6041
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[2]) = ((void *)itmodstruve0_wrap);

  /* "scipy/special/_ufuncs.pyx":6042
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]
 * ufunc_itmodstruve0_data[1] = &ufunc_itmodstruve0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[3]) = ((void *)((char *)__pyx_k_itmodstruve0));

  /* "scipy/special/_ufuncs.pyx":6043
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_data[1] = &ufunc_itmodstruve0_ptr[2*1]
 * itmodstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itmodstruve0_loops, ufunc_itmodstruve0_data, ufunc_itmodstruve0_types, 2, 1, 1, 0, "itmodstruve0", ufunc_itmodstruve0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6044
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]
 * ufunc_itmodstruve0_data[1] = &ufunc_itmodstruve0_ptr[2*1]             # <<<<<<<<<<<<<<
 * itmodstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itmodstruve0_loops, ufunc_itmodstruve0_data, ufunc_itmodstruve0_types, 2, 1, 1, 0, "itmodstruve0", ufunc_itmodstruve0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6045
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]
 * ufunc_itmodstruve0_data[1] = &ufunc_itmodstruve0_ptr[2*1]
 * itmodstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itmodstruve0_loops, ufunc_itmodstruve0_data, ufunc_itmodstruve0_types, 2, 1, 1, 0, "itmodstruve0", ufunc_itmodstruve0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_itstruve0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types, 2, 1, 1, 0, __pyx_k_itmodstruve0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6045; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_itmodstruve0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6045; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6051
 * cdef void *ufunc_itstruve0_data[2]
 * cdef char ufunc_itstruve0_types[4]
 * cdef char *ufunc_itstruve0_doc = (             # <<<<<<<<<<<<<<
 *     "itstruve0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_doc = __pyx_k_itstruve0_x_Integral_of_the_Stru;

  /* "scipy/special/_ufuncs.pyx":6060
 *     "i\n"
 *     "    ``integral(H0(t), t=0..x)``")
 * ufunc_itstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6061
 *     "    ``integral(H0(t), t=0..x)``")
 * ufunc_itstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_itstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6062
 * ufunc_itstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_itstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6063
 * ufunc_itstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6064
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6065
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6066
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[0]) = ((void *)itstruve0_wrap);

  /* "scipy/special/_ufuncs.pyx":6067
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[1]) = ((void *)((char *)__pyx_k_itstruve0));

  /* "scipy/special/_ufuncs.pyx":6068
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[2]) = ((void *)itstruve0_wrap);

  /* "scipy/special/_ufuncs.pyx":6069
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]
 * ufunc_itstruve0_data[1] = &ufunc_itstruve0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[3]) = ((void *)((char *)__pyx_k_itstruve0));

  /* "scipy/special/_ufuncs.pyx":6070
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_data[1] = &ufunc_itstruve0_ptr[2*1]
 * itstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itstruve0_loops, ufunc_itstruve0_data, ufunc_itstruve0_types, 2, 1, 1, 0, "itstruve0", ufunc_itstruve0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6071
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]
 * ufunc_itstruve0_data[1] = &ufunc_itstruve0_ptr[2*1]             # <<<<<<<<<<<<<<
 * itstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itstruve0_loops, ufunc_itstruve0_data, ufunc_itstruve0_types, 2, 1, 1, 0, "itstruve0", ufunc_itstruve0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6072
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]
 * ufunc_itstruve0_data[1] = &ufunc_itstruve0_ptr[2*1]
 * itstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itstruve0_loops, ufunc_itstruve0_data, ufunc_itstruve0_types, 2, 1, 1, 0, "itstruve0", ufunc_itstruve0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_iv_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types, 2, 1, 1, 0, __pyx_k_itstruve0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6072; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_itstruve0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6072; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6078
 * cdef void *ufunc_iv_data[4]
 * cdef char ufunc_iv_types[12]
 * cdef char *ufunc_iv_doc = (             # <<<<<<<<<<<<<<
 *     "iv(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_doc = __pyx_k_iv_v_z_Modified_Bessel_function;

  /* "scipy/special/_ufuncs.pyx":6089
 *     "z\n"
 *     "    Argument.")
 * ufunc_iv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_iv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6090
 *     "    Argument.")
 * ufunc_iv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_iv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":6091
 * ufunc_iv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_iv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_iv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6092
 * ufunc_iv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_iv_types[0] = <char>NPY_FLOAT
 * ufunc_iv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":6093
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_iv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[1] = <char>NPY_FLOAT
 * ufunc_iv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6094
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_iv_types[0] = <char>NPY_FLOAT
 * ufunc_iv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[2] = <char>NPY_FLOAT
 * ufunc_iv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6095
 * ufunc_iv_types[0] = <char>NPY_FLOAT
 * ufunc_iv_types[1] = <char>NPY_FLOAT
 * ufunc_iv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[3] = <char>NPY_FLOAT
 * ufunc_iv_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6096
 * ufunc_iv_types[1] = <char>NPY_FLOAT
 * ufunc_iv_types[2] = <char>NPY_FLOAT
 * ufunc_iv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[4] = <char>NPY_CFLOAT
 * ufunc_iv_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6097
 * ufunc_iv_types[2] = <char>NPY_FLOAT
 * ufunc_iv_types[3] = <char>NPY_FLOAT
 * ufunc_iv_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[5] = <char>NPY_CFLOAT
 * ufunc_iv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6098
 * ufunc_iv_types[3] = <char>NPY_FLOAT
 * ufunc_iv_types[4] = <char>NPY_CFLOAT
 * ufunc_iv_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[6] = <char>NPY_DOUBLE
 * ufunc_iv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6099
 * ufunc_iv_types[4] = <char>NPY_CFLOAT
 * ufunc_iv_types[5] = <char>NPY_CFLOAT
 * ufunc_iv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[7] = <char>NPY_DOUBLE
 * ufunc_iv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6100
 * ufunc_iv_types[5] = <char>NPY_CFLOAT
 * ufunc_iv_types[6] = <char>NPY_DOUBLE
 * ufunc_iv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[8] = <char>NPY_DOUBLE
 * ufunc_iv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6101
 * ufunc_iv_types[6] = <char>NPY_DOUBLE
 * ufunc_iv_types[7] = <char>NPY_DOUBLE
 * ufunc_iv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[9] = <char>NPY_DOUBLE
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6102
 * ufunc_iv_types[7] = <char>NPY_DOUBLE
 * ufunc_iv_types[8] = <char>NPY_DOUBLE
 * ufunc_iv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6103
 * ufunc_iv_types[8] = <char>NPY_DOUBLE
 * ufunc_iv_types[9] = <char>NPY_DOUBLE
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE
 * ufunc_iv_ptr[2*0] = <void*>_func_iv
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6104
 * ufunc_iv_types[9] = <char>NPY_DOUBLE
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*0] = <void*>_func_iv
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6105
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE
 * ufunc_iv_ptr[2*0] = <void*>_func_iv             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[0]) = ((void *)iv);

  /* "scipy/special/_ufuncs.pyx":6106
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE
 * ufunc_iv_ptr[2*0] = <void*>_func_iv
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[1]) = ((void *)((char *)__pyx_k_iv));

  /* "scipy/special/_ufuncs.pyx":6107
 * ufunc_iv_ptr[2*0] = <void*>_func_iv
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*2] = <void*>_func_iv
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[2]) = ((void *)cbesi_wrap);

  /* "scipy/special/_ufuncs.pyx":6108
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*2] = <void*>_func_iv
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[3]) = ((void *)((char *)__pyx_k_iv));

  /* "scipy/special/_ufuncs.pyx":6109
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*2] = <void*>_func_iv             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[4]) = ((void *)iv);

  /* "scipy/special/_ufuncs.pyx":6110
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*2] = <void*>_func_iv
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[5]) = ((void *)((char *)__pyx_k_iv));

  /* "scipy/special/_ufuncs.pyx":6111
 * ufunc_iv_ptr[2*2] = <void*>_func_iv
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[6]) = ((void *)cbesi_wrap);

  /* "scipy/special/_ufuncs.pyx":6112
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")             # <<<<<<<<<<<<<<
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[7]) = ((void *)((char *)__pyx_k_iv));

  /* "scipy/special/_ufuncs.pyx":6113
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6114
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]
 * ufunc_iv_data[3] = &ufunc_iv_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6115
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_iv_data[3] = &ufunc_iv_ptr[2*3]
 * iv = np.PyUFunc_FromFuncAndData(ufunc_iv_loops, ufunc_iv_data, ufunc_iv_types, 4, 2, 1, 0, "iv", ufunc_iv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6116
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]
 * ufunc_iv_data[3] = &ufunc_iv_ptr[2*3]             # <<<<<<<<<<<<<<
 * iv = np.PyUFunc_FromFuncAndData(ufunc_iv_loops, ufunc_iv_data, ufunc_iv_types, 4, 2, 1, 0, "iv", ufunc_iv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6117
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]
 * ufunc_iv_data[3] = &ufunc_iv_ptr[2*3]
 * iv = np.PyUFunc_FromFuncAndData(ufunc_iv_loops, ufunc_iv_data, ufunc_iv_types, 4, 2, 1, 0, "iv", ufunc_iv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ive_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types, 4, 2, 1, 0, __pyx_k_iv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_iv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6123
 * cdef void *ufunc_ive_data[4]
 * cdef char ufunc_ive_types[12]
 * cdef char *ufunc_ive_doc = (             # <<<<<<<<<<<<<<
 *     "ive(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_doc = __pyx_k_ive_v_z_Exponentially_scaled_mod;

  /* "scipy/special/_ufuncs.pyx":6131
 *     "\n"
 *     "    ive(v, z) = iv(v, z) * exp(-abs(z.real))")
 * ufunc_ive_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_ive_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6132
 *     "    ive(v, z) = iv(v, z) * exp(-abs(z.real))")
 * ufunc_ive_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ive_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":6133
 * ufunc_ive_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ive_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_ive_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6134
 * ufunc_ive_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_ive_types[0] = <char>NPY_FLOAT
 * ufunc_ive_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":6135
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_ive_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[1] = <char>NPY_FLOAT
 * ufunc_ive_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6136
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_ive_types[0] = <char>NPY_FLOAT
 * ufunc_ive_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[2] = <char>NPY_FLOAT
 * ufunc_ive_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6137
 * ufunc_ive_types[0] = <char>NPY_FLOAT
 * ufunc_ive_types[1] = <char>NPY_FLOAT
 * ufunc_ive_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[3] = <char>NPY_FLOAT
 * ufunc_ive_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6138
 * ufunc_ive_types[1] = <char>NPY_FLOAT
 * ufunc_ive_types[2] = <char>NPY_FLOAT
 * ufunc_ive_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[4] = <char>NPY_CFLOAT
 * ufunc_ive_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6139
 * ufunc_ive_types[2] = <char>NPY_FLOAT
 * ufunc_ive_types[3] = <char>NPY_FLOAT
 * ufunc_ive_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[5] = <char>NPY_CFLOAT
 * ufunc_ive_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6140
 * ufunc_ive_types[3] = <char>NPY_FLOAT
 * ufunc_ive_types[4] = <char>NPY_CFLOAT
 * ufunc_ive_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[6] = <char>NPY_DOUBLE
 * ufunc_ive_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6141
 * ufunc_ive_types[4] = <char>NPY_CFLOAT
 * ufunc_ive_types[5] = <char>NPY_CFLOAT
 * ufunc_ive_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[7] = <char>NPY_DOUBLE
 * ufunc_ive_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6142
 * ufunc_ive_types[5] = <char>NPY_CFLOAT
 * ufunc_ive_types[6] = <char>NPY_DOUBLE
 * ufunc_ive_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[8] = <char>NPY_DOUBLE
 * ufunc_ive_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6143
 * ufunc_ive_types[6] = <char>NPY_DOUBLE
 * ufunc_ive_types[7] = <char>NPY_DOUBLE
 * ufunc_ive_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[9] = <char>NPY_DOUBLE
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6144
 * ufunc_ive_types[7] = <char>NPY_DOUBLE
 * ufunc_ive_types[8] = <char>NPY_DOUBLE
 * ufunc_ive_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6145
 * ufunc_ive_types[8] = <char>NPY_DOUBLE
 * ufunc_ive_types[9] = <char>NPY_DOUBLE
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6146
 * ufunc_ive_types[9] = <char>NPY_DOUBLE
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6147
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[0]) = ((void *)cbesi_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":6148
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[1]) = ((void *)((char *)__pyx_k_ive));

  /* "scipy/special/_ufuncs.pyx":6149
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[2]) = ((void *)cbesi_wrap_e);

  /* "scipy/special/_ufuncs.pyx":6150
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[3]) = ((void *)((char *)__pyx_k_ive));

  /* "scipy/special/_ufuncs.pyx":6151
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[4]) = ((void *)cbesi_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":6152
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[5]) = ((void *)((char *)__pyx_k_ive));

  /* "scipy/special/_ufuncs.pyx":6153
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[6]) = ((void *)cbesi_wrap_e);

  /* "scipy/special/_ufuncs.pyx":6154
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")             # <<<<<<<<<<<<<<
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[7]) = ((void *)((char *)__pyx_k_ive));

  /* "scipy/special/_ufuncs.pyx":6155
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6156
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]
 * ufunc_ive_data[3] = &ufunc_ive_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6157
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_ive_data[3] = &ufunc_ive_ptr[2*3]
 * ive = np.PyUFunc_FromFuncAndData(ufunc_ive_loops, ufunc_ive_data, ufunc_ive_types, 4, 2, 1, 0, "ive", ufunc_ive_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6158
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]
 * ufunc_ive_data[3] = &ufunc_ive_ptr[2*3]             # <<<<<<<<<<<<<<
 * ive = np.PyUFunc_FromFuncAndData(ufunc_ive_loops, ufunc_ive_data, ufunc_ive_types, 4, 2, 1, 0, "ive", ufunc_ive_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6159
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]
 * ufunc_ive_data[3] = &ufunc_ive_ptr[2*3]
 * ive = np.PyUFunc_FromFuncAndData(ufunc_ive_loops, ufunc_ive_data, ufunc_ive_types, 4, 2, 1, 0, "ive", ufunc_ive_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_j0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types, 4, 2, 1, 0, __pyx_k_ive, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6159; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ive, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6159; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6165
 * cdef void *ufunc_j0_data[2]
 * cdef char ufunc_j0_types[4]
 * cdef char *ufunc_j0_doc = (             # <<<<<<<<<<<<<<
 *     "j0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_doc = __pyx_k_j0_x_Bessel_function_the_first_k;

  /* "scipy/special/_ufuncs.pyx":6169
 *     "\n"
 *     "Bessel function the first kind of order 0")
 * ufunc_j0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_j0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6170
 *     "Bessel function the first kind of order 0")
 * ufunc_j0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_j0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_j0_types[0] = <char>NPY_FLOAT
 * ufunc_j0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6171
 * ufunc_j0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_j0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_j0_types[1] = <char>NPY_FLOAT
 * ufunc_j0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6172
 * ufunc_j0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j0_types[0] = <char>NPY_FLOAT
 * ufunc_j0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_j0_types[2] = <char>NPY_DOUBLE
 * ufunc_j0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6173
 * ufunc_j0_types[0] = <char>NPY_FLOAT
 * ufunc_j0_types[1] = <char>NPY_FLOAT
 * ufunc_j0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_j0_types[3] = <char>NPY_DOUBLE
 * ufunc_j0_ptr[2*0] = <void*>_func_j0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6174
 * ufunc_j0_types[1] = <char>NPY_FLOAT
 * ufunc_j0_types[2] = <char>NPY_DOUBLE
 * ufunc_j0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_j0_ptr[2*0] = <void*>_func_j0
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6175
 * ufunc_j0_types[2] = <char>NPY_DOUBLE
 * ufunc_j0_types[3] = <char>NPY_DOUBLE
 * ufunc_j0_ptr[2*0] = <void*>_func_j0             # <<<<<<<<<<<<<<
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")
 * ufunc_j0_ptr[2*1] = <void*>_func_j0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[0]) = ((void *)j0);

  /* "scipy/special/_ufuncs.pyx":6176
 * ufunc_j0_types[3] = <char>NPY_DOUBLE
 * ufunc_j0_ptr[2*0] = <void*>_func_j0
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")             # <<<<<<<<<<<<<<
 * ufunc_j0_ptr[2*1] = <void*>_func_j0
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[1]) = ((void *)((char *)__pyx_k_j0));

  /* "scipy/special/_ufuncs.pyx":6177
 * ufunc_j0_ptr[2*0] = <void*>_func_j0
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")
 * ufunc_j0_ptr[2*1] = <void*>_func_j0             # <<<<<<<<<<<<<<
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[2]) = ((void *)j0);

  /* "scipy/special/_ufuncs.pyx":6178
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")
 * ufunc_j0_ptr[2*1] = <void*>_func_j0
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")             # <<<<<<<<<<<<<<
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]
 * ufunc_j0_data[1] = &ufunc_j0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[3]) = ((void *)((char *)__pyx_k_j0));

  /* "scipy/special/_ufuncs.pyx":6179
 * ufunc_j0_ptr[2*1] = <void*>_func_j0
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_j0_data[1] = &ufunc_j0_ptr[2*1]
 * j0 = np.PyUFunc_FromFuncAndData(ufunc_j0_loops, ufunc_j0_data, ufunc_j0_types, 2, 1, 1, 0, "j0", ufunc_j0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6180
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]
 * ufunc_j0_data[1] = &ufunc_j0_ptr[2*1]             # <<<<<<<<<<<<<<
 * j0 = np.PyUFunc_FromFuncAndData(ufunc_j0_loops, ufunc_j0_data, ufunc_j0_types, 2, 1, 1, 0, "j0", ufunc_j0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6181
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]
 * ufunc_j0_data[1] = &ufunc_j0_ptr[2*1]
 * j0 = np.PyUFunc_FromFuncAndData(ufunc_j0_loops, ufunc_j0_data, ufunc_j0_types, 2, 1, 1, 0, "j0", ufunc_j0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_j1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types, 2, 1, 1, 0, __pyx_k_j0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_j0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6187
 * cdef void *ufunc_j1_data[2]
 * cdef char ufunc_j1_types[4]
 * cdef char *ufunc_j1_doc = (             # <<<<<<<<<<<<<<
 *     "j1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_doc = __pyx_k_j1_x_Bessel_function_of_the_firs;

  /* "scipy/special/_ufuncs.pyx":6191
 *     "\n"
 *     "Bessel function of the first kind of order 1")
 * ufunc_j1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_j1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6192
 *     "Bessel function of the first kind of order 1")
 * ufunc_j1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_j1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_j1_types[0] = <char>NPY_FLOAT
 * ufunc_j1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6193
 * ufunc_j1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_j1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_j1_types[1] = <char>NPY_FLOAT
 * ufunc_j1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6194
 * ufunc_j1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j1_types[0] = <char>NPY_FLOAT
 * ufunc_j1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_j1_types[2] = <char>NPY_DOUBLE
 * ufunc_j1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6195
 * ufunc_j1_types[0] = <char>NPY_FLOAT
 * ufunc_j1_types[1] = <char>NPY_FLOAT
 * ufunc_j1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_j1_types[3] = <char>NPY_DOUBLE
 * ufunc_j1_ptr[2*0] = <void*>_func_j1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6196
 * ufunc_j1_types[1] = <char>NPY_FLOAT
 * ufunc_j1_types[2] = <char>NPY_DOUBLE
 * ufunc_j1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_j1_ptr[2*0] = <void*>_func_j1
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6197
 * ufunc_j1_types[2] = <char>NPY_DOUBLE
 * ufunc_j1_types[3] = <char>NPY_DOUBLE
 * ufunc_j1_ptr[2*0] = <void*>_func_j1             # <<<<<<<<<<<<<<
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")
 * ufunc_j1_ptr[2*1] = <void*>_func_j1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[0]) = ((void *)j1);

  /* "scipy/special/_ufuncs.pyx":6198
 * ufunc_j1_types[3] = <char>NPY_DOUBLE
 * ufunc_j1_ptr[2*0] = <void*>_func_j1
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")             # <<<<<<<<<<<<<<
 * ufunc_j1_ptr[2*1] = <void*>_func_j1
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[1]) = ((void *)((char *)__pyx_k_j1));

  /* "scipy/special/_ufuncs.pyx":6199
 * ufunc_j1_ptr[2*0] = <void*>_func_j1
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")
 * ufunc_j1_ptr[2*1] = <void*>_func_j1             # <<<<<<<<<<<<<<
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[2]) = ((void *)j1);

  /* "scipy/special/_ufuncs.pyx":6200
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")
 * ufunc_j1_ptr[2*1] = <void*>_func_j1
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")             # <<<<<<<<<<<<<<
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]
 * ufunc_j1_data[1] = &ufunc_j1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[3]) = ((void *)((char *)__pyx_k_j1));

  /* "scipy/special/_ufuncs.pyx":6201
 * ufunc_j1_ptr[2*1] = <void*>_func_j1
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_j1_data[1] = &ufunc_j1_ptr[2*1]
 * j1 = np.PyUFunc_FromFuncAndData(ufunc_j1_loops, ufunc_j1_data, ufunc_j1_types, 2, 1, 1, 0, "j1", ufunc_j1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6202
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]
 * ufunc_j1_data[1] = &ufunc_j1_ptr[2*1]             # <<<<<<<<<<<<<<
 * j1 = np.PyUFunc_FromFuncAndData(ufunc_j1_loops, ufunc_j1_data, ufunc_j1_types, 2, 1, 1, 0, "j1", ufunc_j1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6203
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]
 * ufunc_j1_data[1] = &ufunc_j1_ptr[2*1]
 * j1 = np.PyUFunc_FromFuncAndData(ufunc_j1_loops, ufunc_j1_data, ufunc_j1_types, 2, 1, 1, 0, "j1", ufunc_j1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_jv_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types, 2, 1, 1, 0, __pyx_k_j1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6203; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_j1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6203; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6209
 * cdef void *ufunc_jv_data[4]
 * cdef char ufunc_jv_types[12]
 * cdef char *ufunc_jv_doc = (             # <<<<<<<<<<<<<<
 *     "jv(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_doc = __pyx_k_jv_v_z_Bessel_function_of_the_fi;

  /* "scipy/special/_ufuncs.pyx":6213
 *     "\n"
 *     "Bessel function of the first kind of real order `v`")
 * ufunc_jv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_jv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6214
 *     "Bessel function of the first kind of real order `v`")
 * ufunc_jv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_jv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":6215
 * ufunc_jv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_jv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6216
 * ufunc_jv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_jv_types[0] = <char>NPY_FLOAT
 * ufunc_jv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":6217
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[1] = <char>NPY_FLOAT
 * ufunc_jv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6218
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jv_types[0] = <char>NPY_FLOAT
 * ufunc_jv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[2] = <char>NPY_FLOAT
 * ufunc_jv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6219
 * ufunc_jv_types[0] = <char>NPY_FLOAT
 * ufunc_jv_types[1] = <char>NPY_FLOAT
 * ufunc_jv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[3] = <char>NPY_FLOAT
 * ufunc_jv_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6220
 * ufunc_jv_types[1] = <char>NPY_FLOAT
 * ufunc_jv_types[2] = <char>NPY_FLOAT
 * ufunc_jv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[4] = <char>NPY_CFLOAT
 * ufunc_jv_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6221
 * ufunc_jv_types[2] = <char>NPY_FLOAT
 * ufunc_jv_types[3] = <char>NPY_FLOAT
 * ufunc_jv_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[5] = <char>NPY_CFLOAT
 * ufunc_jv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6222
 * ufunc_jv_types[3] = <char>NPY_FLOAT
 * ufunc_jv_types[4] = <char>NPY_CFLOAT
 * ufunc_jv_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[6] = <char>NPY_DOUBLE
 * ufunc_jv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6223
 * ufunc_jv_types[4] = <char>NPY_CFLOAT
 * ufunc_jv_types[5] = <char>NPY_CFLOAT
 * ufunc_jv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[7] = <char>NPY_DOUBLE
 * ufunc_jv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6224
 * ufunc_jv_types[5] = <char>NPY_CFLOAT
 * ufunc_jv_types[6] = <char>NPY_DOUBLE
 * ufunc_jv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[8] = <char>NPY_DOUBLE
 * ufunc_jv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6225
 * ufunc_jv_types[6] = <char>NPY_DOUBLE
 * ufunc_jv_types[7] = <char>NPY_DOUBLE
 * ufunc_jv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[9] = <char>NPY_DOUBLE
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6226
 * ufunc_jv_types[7] = <char>NPY_DOUBLE
 * ufunc_jv_types[8] = <char>NPY_DOUBLE
 * ufunc_jv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6227
 * ufunc_jv_types[8] = <char>NPY_DOUBLE
 * ufunc_jv_types[9] = <char>NPY_DOUBLE
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6228
 * ufunc_jv_types[9] = <char>NPY_DOUBLE
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6229
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[0]) = ((void *)cbesj_wrap_real);

  /* "scipy/special/_ufuncs.pyx":6230
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[1]) = ((void *)((char *)__pyx_k_jv));

  /* "scipy/special/_ufuncs.pyx":6231
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[2]) = ((void *)cbesj_wrap);

  /* "scipy/special/_ufuncs.pyx":6232
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[3]) = ((void *)((char *)__pyx_k_jv));

  /* "scipy/special/_ufuncs.pyx":6233
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[4]) = ((void *)cbesj_wrap_real);

  /* "scipy/special/_ufuncs.pyx":6234
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[5]) = ((void *)((char *)__pyx_k_jv));

  /* "scipy/special/_ufuncs.pyx":6235
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[6]) = ((void *)cbesj_wrap);

  /* "scipy/special/_ufuncs.pyx":6236
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")             # <<<<<<<<<<<<<<
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[7]) = ((void *)((char *)__pyx_k_jv));

  /* "scipy/special/_ufuncs.pyx":6237
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6238
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]
 * ufunc_jv_data[3] = &ufunc_jv_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6239
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_jv_data[3] = &ufunc_jv_ptr[2*3]
 * jv = np.PyUFunc_FromFuncAndData(ufunc_jv_loops, ufunc_jv_data, ufunc_jv_types, 4, 2, 1, 0, "jv", ufunc_jv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6240
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]
 * ufunc_jv_data[3] = &ufunc_jv_ptr[2*3]             # <<<<<<<<<<<<<<
 * jv = np.PyUFunc_FromFuncAndData(ufunc_jv_loops, ufunc_jv_data, ufunc_jv_types, 4, 2, 1, 0, "jv", ufunc_jv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6241
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]
 * ufunc_jv_data[3] = &ufunc_jv_ptr[2*3]
 * jv = np.PyUFunc_FromFuncAndData(ufunc_jv_loops, ufunc_jv_data, ufunc_jv_types, 4, 2, 1, 0, "jv", ufunc_jv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_jve_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types, 4, 2, 1, 0, __pyx_k_jv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_jv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6247
 * cdef void *ufunc_jve_data[4]
 * cdef char ufunc_jve_types[12]
 * cdef char *ufunc_jve_doc = (             # <<<<<<<<<<<<<<
 *     "jve(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_doc = __pyx_k_jve_v_z_Exponentially_scaled_Bes;

  /* "scipy/special/_ufuncs.pyx":6255
 *     "\n"
 *     "    jve(v, z) = jv(v, z) * exp(-abs(z.imag))")
 * ufunc_jve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_jve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6256
 *     "    jve(v, z) = jv(v, z) * exp(-abs(z.imag))")
 * ufunc_jve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_jve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":6257
 * ufunc_jve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_jve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6258
 * ufunc_jve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_jve_types[0] = <char>NPY_FLOAT
 * ufunc_jve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":6259
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[1] = <char>NPY_FLOAT
 * ufunc_jve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6260
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jve_types[0] = <char>NPY_FLOAT
 * ufunc_jve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[2] = <char>NPY_FLOAT
 * ufunc_jve_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6261
 * ufunc_jve_types[0] = <char>NPY_FLOAT
 * ufunc_jve_types[1] = <char>NPY_FLOAT
 * ufunc_jve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[3] = <char>NPY_FLOAT
 * ufunc_jve_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6262
 * ufunc_jve_types[1] = <char>NPY_FLOAT
 * ufunc_jve_types[2] = <char>NPY_FLOAT
 * ufunc_jve_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[4] = <char>NPY_CFLOAT
 * ufunc_jve_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6263
 * ufunc_jve_types[2] = <char>NPY_FLOAT
 * ufunc_jve_types[3] = <char>NPY_FLOAT
 * ufunc_jve_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[5] = <char>NPY_CFLOAT
 * ufunc_jve_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6264
 * ufunc_jve_types[3] = <char>NPY_FLOAT
 * ufunc_jve_types[4] = <char>NPY_CFLOAT
 * ufunc_jve_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[6] = <char>NPY_DOUBLE
 * ufunc_jve_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6265
 * ufunc_jve_types[4] = <char>NPY_CFLOAT
 * ufunc_jve_types[5] = <char>NPY_CFLOAT
 * ufunc_jve_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[7] = <char>NPY_DOUBLE
 * ufunc_jve_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6266
 * ufunc_jve_types[5] = <char>NPY_CFLOAT
 * ufunc_jve_types[6] = <char>NPY_DOUBLE
 * ufunc_jve_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[8] = <char>NPY_DOUBLE
 * ufunc_jve_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6267
 * ufunc_jve_types[6] = <char>NPY_DOUBLE
 * ufunc_jve_types[7] = <char>NPY_DOUBLE
 * ufunc_jve_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[9] = <char>NPY_DOUBLE
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6268
 * ufunc_jve_types[7] = <char>NPY_DOUBLE
 * ufunc_jve_types[8] = <char>NPY_DOUBLE
 * ufunc_jve_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6269
 * ufunc_jve_types[8] = <char>NPY_DOUBLE
 * ufunc_jve_types[9] = <char>NPY_DOUBLE
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6270
 * ufunc_jve_types[9] = <char>NPY_DOUBLE
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6271
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[0]) = ((void *)cbesj_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":6272
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[1]) = ((void *)((char *)__pyx_k_jve));

  /* "scipy/special/_ufuncs.pyx":6273
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[2]) = ((void *)cbesj_wrap_e);

  /* "scipy/special/_ufuncs.pyx":6274
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[3]) = ((void *)((char *)__pyx_k_jve));

  /* "scipy/special/_ufuncs.pyx":6275
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[4]) = ((void *)cbesj_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":6276
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[5]) = ((void *)((char *)__pyx_k_jve));

  /* "scipy/special/_ufuncs.pyx":6277
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[6]) = ((void *)cbesj_wrap_e);

  /* "scipy/special/_ufuncs.pyx":6278
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")             # <<<<<<<<<<<<<<
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[7]) = ((void *)((char *)__pyx_k_jve));

  /* "scipy/special/_ufuncs.pyx":6279
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6280
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]
 * ufunc_jve_data[3] = &ufunc_jve_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6281
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_jve_data[3] = &ufunc_jve_ptr[2*3]
 * jve = np.PyUFunc_FromFuncAndData(ufunc_jve_loops, ufunc_jve_data, ufunc_jve_types, 4, 2, 1, 0, "jve", ufunc_jve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6282
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]
 * ufunc_jve_data[3] = &ufunc_jve_ptr[2*3]             # <<<<<<<<<<<<<<
 * jve = np.PyUFunc_FromFuncAndData(ufunc_jve_loops, ufunc_jve_data, ufunc_jve_types, 4, 2, 1, 0, "jve", ufunc_jve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6283
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]
 * ufunc_jve_data[3] = &ufunc_jve_ptr[2*3]
 * jve = np.PyUFunc_FromFuncAndData(ufunc_jve_loops, ufunc_jve_data, ufunc_jve_types, 4, 2, 1, 0, "jve", ufunc_jve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_k0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types, 4, 2, 1, 0, __pyx_k_jve, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_jve, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6289
 * cdef void *ufunc_k0_data[2]
 * cdef char ufunc_k0_types[4]
 * cdef char *ufunc_k0_doc = (             # <<<<<<<<<<<<<<
 *     "k0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_doc = __pyx_k_k0_x_Modified_Bessel_function_K;

  /* "scipy/special/_ufuncs.pyx":6296
 *     "Modified Bessel function of the second kind (sometimes called the\n"
 *     "third kind) of order 0.")
 * ufunc_k0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_k0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6297
 *     "third kind) of order 0.")
 * ufunc_k0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_k0_types[0] = <char>NPY_FLOAT
 * ufunc_k0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6298
 * ufunc_k0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k0_types[1] = <char>NPY_FLOAT
 * ufunc_k0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6299
 * ufunc_k0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0_types[0] = <char>NPY_FLOAT
 * ufunc_k0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k0_types[2] = <char>NPY_DOUBLE
 * ufunc_k0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6300
 * ufunc_k0_types[0] = <char>NPY_FLOAT
 * ufunc_k0_types[1] = <char>NPY_FLOAT
 * ufunc_k0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k0_types[3] = <char>NPY_DOUBLE
 * ufunc_k0_ptr[2*0] = <void*>_func_k0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6301
 * ufunc_k0_types[1] = <char>NPY_FLOAT
 * ufunc_k0_types[2] = <char>NPY_DOUBLE
 * ufunc_k0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k0_ptr[2*0] = <void*>_func_k0
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6302
 * ufunc_k0_types[2] = <char>NPY_DOUBLE
 * ufunc_k0_types[3] = <char>NPY_DOUBLE
 * ufunc_k0_ptr[2*0] = <void*>_func_k0             # <<<<<<<<<<<<<<
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")
 * ufunc_k0_ptr[2*1] = <void*>_func_k0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[0]) = ((void *)k0);

  /* "scipy/special/_ufuncs.pyx":6303
 * ufunc_k0_types[3] = <char>NPY_DOUBLE
 * ufunc_k0_ptr[2*0] = <void*>_func_k0
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")             # <<<<<<<<<<<<<<
 * ufunc_k0_ptr[2*1] = <void*>_func_k0
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[1]) = ((void *)((char *)__pyx_k_k0));

  /* "scipy/special/_ufuncs.pyx":6304
 * ufunc_k0_ptr[2*0] = <void*>_func_k0
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")
 * ufunc_k0_ptr[2*1] = <void*>_func_k0             # <<<<<<<<<<<<<<
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[2]) = ((void *)k0);

  /* "scipy/special/_ufuncs.pyx":6305
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")
 * ufunc_k0_ptr[2*1] = <void*>_func_k0
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")             # <<<<<<<<<<<<<<
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]
 * ufunc_k0_data[1] = &ufunc_k0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[3]) = ((void *)((char *)__pyx_k_k0));

  /* "scipy/special/_ufuncs.pyx":6306
 * ufunc_k0_ptr[2*1] = <void*>_func_k0
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_k0_data[1] = &ufunc_k0_ptr[2*1]
 * k0 = np.PyUFunc_FromFuncAndData(ufunc_k0_loops, ufunc_k0_data, ufunc_k0_types, 2, 1, 1, 0, "k0", ufunc_k0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6307
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]
 * ufunc_k0_data[1] = &ufunc_k0_ptr[2*1]             # <<<<<<<<<<<<<<
 * k0 = np.PyUFunc_FromFuncAndData(ufunc_k0_loops, ufunc_k0_data, ufunc_k0_types, 2, 1, 1, 0, "k0", ufunc_k0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6308
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]
 * ufunc_k0_data[1] = &ufunc_k0_ptr[2*1]
 * k0 = np.PyUFunc_FromFuncAndData(ufunc_k0_loops, ufunc_k0_data, ufunc_k0_types, 2, 1, 1, 0, "k0", ufunc_k0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_k0e_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types, 2, 1, 1, 0, __pyx_k_k0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6308; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_k0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6308; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6314
 * cdef void *ufunc_k0e_data[2]
 * cdef char ufunc_k0e_types[4]
 * cdef char *ufunc_k0e_doc = (             # <<<<<<<<<<<<<<
 *     "k0e(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_doc = __pyx_k_k0e_x_Exponentially_scaled_modif;

  /* "scipy/special/_ufuncs.pyx":6322
 *     "\n"
 *     "    k0e(x) = exp(x) * k0(x).")
 * ufunc_k0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_k0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6323
 *     "    k0e(x) = exp(x) * k0(x).")
 * ufunc_k0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_k0e_types[0] = <char>NPY_FLOAT
 * ufunc_k0e_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6324
 * ufunc_k0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k0e_types[1] = <char>NPY_FLOAT
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6325
 * ufunc_k0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0e_types[0] = <char>NPY_FLOAT
 * ufunc_k0e_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6326
 * ufunc_k0e_types[0] = <char>NPY_FLOAT
 * ufunc_k0e_types[1] = <char>NPY_FLOAT
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6327
 * ufunc_k0e_types[1] = <char>NPY_FLOAT
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6328
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e             # <<<<<<<<<<<<<<
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[0]) = ((void *)k0e);

  /* "scipy/special/_ufuncs.pyx":6329
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")             # <<<<<<<<<<<<<<
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[1]) = ((void *)((char *)__pyx_k_k0e));

  /* "scipy/special/_ufuncs.pyx":6330
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e             # <<<<<<<<<<<<<<
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[2]) = ((void *)k0e);

  /* "scipy/special/_ufuncs.pyx":6331
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")             # <<<<<<<<<<<<<<
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]
 * ufunc_k0e_data[1] = &ufunc_k0e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[3]) = ((void *)((char *)__pyx_k_k0e));

  /* "scipy/special/_ufuncs.pyx":6332
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_k0e_data[1] = &ufunc_k0e_ptr[2*1]
 * k0e = np.PyUFunc_FromFuncAndData(ufunc_k0e_loops, ufunc_k0e_data, ufunc_k0e_types, 2, 1, 1, 0, "k0e", ufunc_k0e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6333
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]
 * ufunc_k0e_data[1] = &ufunc_k0e_ptr[2*1]             # <<<<<<<<<<<<<<
 * k0e = np.PyUFunc_FromFuncAndData(ufunc_k0e_loops, ufunc_k0e_data, ufunc_k0e_types, 2, 1, 1, 0, "k0e", ufunc_k0e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6334
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]
 * ufunc_k0e_data[1] = &ufunc_k0e_ptr[2*1]
 * k0e = np.PyUFunc_FromFuncAndData(ufunc_k0e_loops, ufunc_k0e_data, ufunc_k0e_types, 2, 1, 1, 0, "k0e", ufunc_k0e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_k1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types, 2, 1, 1, 0, __pyx_k_k0e, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_k0e, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6340
 * cdef void *ufunc_k1_data[2]
 * cdef char ufunc_k1_types[4]
 * cdef char *ufunc_k1_doc = (             # <<<<<<<<<<<<<<
 *     "i1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_doc = __pyx_k_i1_x_Modified_Bessel_function_of_2;

  /* "scipy/special/_ufuncs.pyx":6344
 *     "\n"
 *     "Modified Bessel function of the first kind of order 1")
 * ufunc_k1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_k1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6345
 *     "Modified Bessel function of the first kind of order 1")
 * ufunc_k1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_k1_types[0] = <char>NPY_FLOAT
 * ufunc_k1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6346
 * ufunc_k1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k1_types[1] = <char>NPY_FLOAT
 * ufunc_k1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6347
 * ufunc_k1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1_types[0] = <char>NPY_FLOAT
 * ufunc_k1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k1_types[2] = <char>NPY_DOUBLE
 * ufunc_k1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6348
 * ufunc_k1_types[0] = <char>NPY_FLOAT
 * ufunc_k1_types[1] = <char>NPY_FLOAT
 * ufunc_k1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k1_types[3] = <char>NPY_DOUBLE
 * ufunc_k1_ptr[2*0] = <void*>_func_k1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6349
 * ufunc_k1_types[1] = <char>NPY_FLOAT
 * ufunc_k1_types[2] = <char>NPY_DOUBLE
 * ufunc_k1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k1_ptr[2*0] = <void*>_func_k1
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6350
 * ufunc_k1_types[2] = <char>NPY_DOUBLE
 * ufunc_k1_types[3] = <char>NPY_DOUBLE
 * ufunc_k1_ptr[2*0] = <void*>_func_k1             # <<<<<<<<<<<<<<
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")
 * ufunc_k1_ptr[2*1] = <void*>_func_k1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[0]) = ((void *)k1);

  /* "scipy/special/_ufuncs.pyx":6351
 * ufunc_k1_types[3] = <char>NPY_DOUBLE
 * ufunc_k1_ptr[2*0] = <void*>_func_k1
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")             # <<<<<<<<<<<<<<
 * ufunc_k1_ptr[2*1] = <void*>_func_k1
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[1]) = ((void *)((char *)__pyx_k_k1));

  /* "scipy/special/_ufuncs.pyx":6352
 * ufunc_k1_ptr[2*0] = <void*>_func_k1
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")
 * ufunc_k1_ptr[2*1] = <void*>_func_k1             # <<<<<<<<<<<<<<
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[2]) = ((void *)k1);

  /* "scipy/special/_ufuncs.pyx":6353
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")
 * ufunc_k1_ptr[2*1] = <void*>_func_k1
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")             # <<<<<<<<<<<<<<
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]
 * ufunc_k1_data[1] = &ufunc_k1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[3]) = ((void *)((char *)__pyx_k_k1));

  /* "scipy/special/_ufuncs.pyx":6354
 * ufunc_k1_ptr[2*1] = <void*>_func_k1
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_k1_data[1] = &ufunc_k1_ptr[2*1]
 * k1 = np.PyUFunc_FromFuncAndData(ufunc_k1_loops, ufunc_k1_data, ufunc_k1_types, 2, 1, 1, 0, "k1", ufunc_k1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6355
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]
 * ufunc_k1_data[1] = &ufunc_k1_ptr[2*1]             # <<<<<<<<<<<<<<
 * k1 = np.PyUFunc_FromFuncAndData(ufunc_k1_loops, ufunc_k1_data, ufunc_k1_types, 2, 1, 1, 0, "k1", ufunc_k1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6356
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]
 * ufunc_k1_data[1] = &ufunc_k1_ptr[2*1]
 * k1 = np.PyUFunc_FromFuncAndData(ufunc_k1_loops, ufunc_k1_data, ufunc_k1_types, 2, 1, 1, 0, "k1", ufunc_k1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_k1e_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types, 2, 1, 1, 0, __pyx_k_k1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_k1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6362
 * cdef void *ufunc_k1e_data[2]
 * cdef char ufunc_k1e_types[4]
 * cdef char *ufunc_k1e_doc = (             # <<<<<<<<<<<<<<
 *     "k1e(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_doc = __pyx_k_k1e_x_Exponentially_scaled_modif;

  /* "scipy/special/_ufuncs.pyx":6370
 *     "\n"
 *     "    k1e(x) = exp(x) * k1(x)")
 * ufunc_k1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_k1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6371
 *     "    k1e(x) = exp(x) * k1(x)")
 * ufunc_k1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_k1e_types[0] = <char>NPY_FLOAT
 * ufunc_k1e_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6372
 * ufunc_k1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k1e_types[1] = <char>NPY_FLOAT
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6373
 * ufunc_k1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1e_types[0] = <char>NPY_FLOAT
 * ufunc_k1e_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6374
 * ufunc_k1e_types[0] = <char>NPY_FLOAT
 * ufunc_k1e_types[1] = <char>NPY_FLOAT
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6375
 * ufunc_k1e_types[1] = <char>NPY_FLOAT
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6376
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e             # <<<<<<<<<<<<<<
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[0]) = ((void *)k1e);

  /* "scipy/special/_ufuncs.pyx":6377
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")             # <<<<<<<<<<<<<<
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[1]) = ((void *)((char *)__pyx_k_k1e));

  /* "scipy/special/_ufuncs.pyx":6378
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e             # <<<<<<<<<<<<<<
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[2]) = ((void *)k1e);

  /* "scipy/special/_ufuncs.pyx":6379
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")             # <<<<<<<<<<<<<<
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]
 * ufunc_k1e_data[1] = &ufunc_k1e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[3]) = ((void *)((char *)__pyx_k_k1e));

  /* "scipy/special/_ufuncs.pyx":6380
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_k1e_data[1] = &ufunc_k1e_ptr[2*1]
 * k1e = np.PyUFunc_FromFuncAndData(ufunc_k1e_loops, ufunc_k1e_data, ufunc_k1e_types, 2, 1, 1, 0, "k1e", ufunc_k1e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6381
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]
 * ufunc_k1e_data[1] = &ufunc_k1e_ptr[2*1]             # <<<<<<<<<<<<<<
 * k1e = np.PyUFunc_FromFuncAndData(ufunc_k1e_loops, ufunc_k1e_data, ufunc_k1e_types, 2, 1, 1, 0, "k1e", ufunc_k1e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6382
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]
 * ufunc_k1e_data[1] = &ufunc_k1e_ptr[2*1]
 * k1e = np.PyUFunc_FromFuncAndData(ufunc_k1e_loops, ufunc_k1e_data, ufunc_k1e_types, 2, 1, 1, 0, "k1e", ufunc_k1e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kei_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types, 2, 1, 1, 0, __pyx_k_k1e, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6382; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_k1e, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6382; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6388
 * cdef void *ufunc_kei_data[2]
 * cdef char ufunc_kei_types[4]
 * cdef char *ufunc_kei_doc = (             # <<<<<<<<<<<<<<
 *     "kei(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_doc = __pyx_k_kei_x_Kelvin_function_ker;

  /* "scipy/special/_ufuncs.pyx":6392
 *     "\n"
 *     "Kelvin function ker")
 * ufunc_kei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_kei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kei_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6393
 *     "Kelvin function ker")
 * ufunc_kei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_kei_types[0] = <char>NPY_FLOAT
 * ufunc_kei_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6394
 * ufunc_kei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kei_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kei_types[1] = <char>NPY_FLOAT
 * ufunc_kei_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6395
 * ufunc_kei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kei_types[0] = <char>NPY_FLOAT
 * ufunc_kei_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kei_types[2] = <char>NPY_DOUBLE
 * ufunc_kei_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6396
 * ufunc_kei_types[0] = <char>NPY_FLOAT
 * ufunc_kei_types[1] = <char>NPY_FLOAT
 * ufunc_kei_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kei_types[3] = <char>NPY_DOUBLE
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6397
 * ufunc_kei_types[1] = <char>NPY_FLOAT
 * ufunc_kei_types[2] = <char>NPY_DOUBLE
 * ufunc_kei_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6398
 * ufunc_kei_types[2] = <char>NPY_DOUBLE
 * ufunc_kei_types[3] = <char>NPY_DOUBLE
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap             # <<<<<<<<<<<<<<
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[0]) = ((void *)kei_wrap);

  /* "scipy/special/_ufuncs.pyx":6399
 * ufunc_kei_types[3] = <char>NPY_DOUBLE
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")             # <<<<<<<<<<<<<<
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[1]) = ((void *)((char *)__pyx_k_kei));

  /* "scipy/special/_ufuncs.pyx":6400
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap             # <<<<<<<<<<<<<<
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[2]) = ((void *)kei_wrap);

  /* "scipy/special/_ufuncs.pyx":6401
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")             # <<<<<<<<<<<<<<
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]
 * ufunc_kei_data[1] = &ufunc_kei_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[3]) = ((void *)((char *)__pyx_k_kei));

  /* "scipy/special/_ufuncs.pyx":6402
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kei_data[1] = &ufunc_kei_ptr[2*1]
 * kei = np.PyUFunc_FromFuncAndData(ufunc_kei_loops, ufunc_kei_data, ufunc_kei_types, 2, 1, 1, 0, "kei", ufunc_kei_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6403
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]
 * ufunc_kei_data[1] = &ufunc_kei_ptr[2*1]             # <<<<<<<<<<<<<<
 * kei = np.PyUFunc_FromFuncAndData(ufunc_kei_loops, ufunc_kei_data, ufunc_kei_types, 2, 1, 1, 0, "kei", ufunc_kei_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6404
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]
 * ufunc_kei_data[1] = &ufunc_kei_ptr[2*1]
 * kei = np.PyUFunc_FromFuncAndData(ufunc_kei_loops, ufunc_kei_data, ufunc_kei_types, 2, 1, 1, 0, "kei", ufunc_kei_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_keip_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types, 2, 1, 1, 0, __pyx_k_kei, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6404; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kei, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6404; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6410
 * cdef void *ufunc_keip_data[2]
 * cdef char ufunc_keip_types[4]
 * cdef char *ufunc_keip_doc = (             # <<<<<<<<<<<<<<
 *     "keip(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_doc = __pyx_k_keip_x_Derivative_of_the_Kelvin;

  /* "scipy/special/_ufuncs.pyx":6414
 *     "\n"
 *     "Derivative of the Kelvin function kei")
 * ufunc_keip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_keip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_keip_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6415
 *     "Derivative of the Kelvin function kei")
 * ufunc_keip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_keip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_keip_types[0] = <char>NPY_FLOAT
 * ufunc_keip_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6416
 * ufunc_keip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_keip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_keip_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_keip_types[1] = <char>NPY_FLOAT
 * ufunc_keip_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6417
 * ufunc_keip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_keip_types[0] = <char>NPY_FLOAT
 * ufunc_keip_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_keip_types[2] = <char>NPY_DOUBLE
 * ufunc_keip_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6418
 * ufunc_keip_types[0] = <char>NPY_FLOAT
 * ufunc_keip_types[1] = <char>NPY_FLOAT
 * ufunc_keip_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_keip_types[3] = <char>NPY_DOUBLE
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6419
 * ufunc_keip_types[1] = <char>NPY_FLOAT
 * ufunc_keip_types[2] = <char>NPY_DOUBLE
 * ufunc_keip_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6420
 * ufunc_keip_types[2] = <char>NPY_DOUBLE
 * ufunc_keip_types[3] = <char>NPY_DOUBLE
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap             # <<<<<<<<<<<<<<
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[0]) = ((void *)keip_wrap);

  /* "scipy/special/_ufuncs.pyx":6421
 * ufunc_keip_types[3] = <char>NPY_DOUBLE
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")             # <<<<<<<<<<<<<<
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[1]) = ((void *)((char *)__pyx_k_keip));

  /* "scipy/special/_ufuncs.pyx":6422
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap             # <<<<<<<<<<<<<<
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[2]) = ((void *)keip_wrap);

  /* "scipy/special/_ufuncs.pyx":6423
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")             # <<<<<<<<<<<<<<
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]
 * ufunc_keip_data[1] = &ufunc_keip_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[3]) = ((void *)((char *)__pyx_k_keip));

  /* "scipy/special/_ufuncs.pyx":6424
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_keip_data[1] = &ufunc_keip_ptr[2*1]
 * keip = np.PyUFunc_FromFuncAndData(ufunc_keip_loops, ufunc_keip_data, ufunc_keip_types, 2, 1, 1, 0, "keip", ufunc_keip_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6425
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]
 * ufunc_keip_data[1] = &ufunc_keip_ptr[2*1]             # <<<<<<<<<<<<<<
 * keip = np.PyUFunc_FromFuncAndData(ufunc_keip_loops, ufunc_keip_data, ufunc_keip_types, 2, 1, 1, 0, "keip", ufunc_keip_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6426
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]
 * ufunc_keip_data[1] = &ufunc_keip_ptr[2*1]
 * keip = np.PyUFunc_FromFuncAndData(ufunc_keip_loops, ufunc_keip_data, ufunc_keip_types, 2, 1, 1, 0, "keip", ufunc_keip_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kelvin_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types, 2, 1, 1, 0, __pyx_k_keip, __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_keip, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6432
 * cdef void *ufunc_kelvin_data[2]
 * cdef char ufunc_kelvin_types[10]
 * cdef char *ufunc_kelvin_doc = (             # <<<<<<<<<<<<<<
 *     "kelvin(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_doc = __pyx_k_kelvin_x_Kelvin_functions_as_com;

  /* "scipy/special/_ufuncs.pyx":6445
 *     "    ber x and kelvin(x)[0].imag = bei x with similar relationships\n"
 *     "    for ker and kei.")
 * ufunc_kelvin_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_f_FFFF             # <<<<<<<<<<<<<<
 * ufunc_kelvin_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_d_DDDD
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_f_FFFF);

  /* "scipy/special/_ufuncs.pyx":6446
 *     "    for ker and kei.")
 * ufunc_kelvin_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_f_FFFF
 * ufunc_kelvin_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_d_DDDD             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_d_DDDD);

  /* "scipy/special/_ufuncs.pyx":6447
 * ufunc_kelvin_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_f_FFFF
 * ufunc_kelvin_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_d_DDDD
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6448
 * ufunc_kelvin_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_d_DDDD
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6449
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6450
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[3]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6451
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6452
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6453
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6454
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6455
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[8]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6456
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6457
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap             # <<<<<<<<<<<<<<
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[0]) = ((void *)kelvin_wrap);

  /* "scipy/special/_ufuncs.pyx":6458
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")             # <<<<<<<<<<<<<<
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[1]) = ((void *)((char *)__pyx_k_kelvin));

  /* "scipy/special/_ufuncs.pyx":6459
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap             # <<<<<<<<<<<<<<
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[2]) = ((void *)kelvin_wrap);

  /* "scipy/special/_ufuncs.pyx":6460
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")             # <<<<<<<<<<<<<<
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]
 * ufunc_kelvin_data[1] = &ufunc_kelvin_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[3]) = ((void *)((char *)__pyx_k_kelvin));

  /* "scipy/special/_ufuncs.pyx":6461
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kelvin_data[1] = &ufunc_kelvin_ptr[2*1]
 * kelvin = np.PyUFunc_FromFuncAndData(ufunc_kelvin_loops, ufunc_kelvin_data, ufunc_kelvin_types, 2, 1, 4, 0, "kelvin", ufunc_kelvin_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6462
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]
 * ufunc_kelvin_data[1] = &ufunc_kelvin_ptr[2*1]             # <<<<<<<<<<<<<<
 * kelvin = np.PyUFunc_FromFuncAndData(ufunc_kelvin_loops, ufunc_kelvin_data, ufunc_kelvin_types, 2, 1, 4, 0, "kelvin", ufunc_kelvin_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6463
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]
 * ufunc_kelvin_data[1] = &ufunc_kelvin_ptr[2*1]
 * kelvin = np.PyUFunc_FromFuncAndData(ufunc_kelvin_loops, ufunc_kelvin_data, ufunc_kelvin_types, 2, 1, 4, 0, "kelvin", ufunc_kelvin_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ker_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types, 2, 1, 4, 0, __pyx_k_kelvin, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kelvin, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6469
 * cdef void *ufunc_ker_data[2]
 * cdef char ufunc_ker_types[4]
 * cdef char *ufunc_ker_doc = (             # <<<<<<<<<<<<<<
 *     "ker(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_doc = __pyx_k_ker_x_Kelvin_function_ker;

  /* "scipy/special/_ufuncs.pyx":6473
 *     "\n"
 *     "Kelvin function ker")
 * ufunc_ker_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ker_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ker_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6474
 *     "Kelvin function ker")
 * ufunc_ker_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ker_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ker_types[0] = <char>NPY_FLOAT
 * ufunc_ker_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6475
 * ufunc_ker_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ker_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ker_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ker_types[1] = <char>NPY_FLOAT
 * ufunc_ker_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6476
 * ufunc_ker_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ker_types[0] = <char>NPY_FLOAT
 * ufunc_ker_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ker_types[2] = <char>NPY_DOUBLE
 * ufunc_ker_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6477
 * ufunc_ker_types[0] = <char>NPY_FLOAT
 * ufunc_ker_types[1] = <char>NPY_FLOAT
 * ufunc_ker_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ker_types[3] = <char>NPY_DOUBLE
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6478
 * ufunc_ker_types[1] = <char>NPY_FLOAT
 * ufunc_ker_types[2] = <char>NPY_DOUBLE
 * ufunc_ker_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6479
 * ufunc_ker_types[2] = <char>NPY_DOUBLE
 * ufunc_ker_types[3] = <char>NPY_DOUBLE
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap             # <<<<<<<<<<<<<<
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[0]) = ((void *)ker_wrap);

  /* "scipy/special/_ufuncs.pyx":6480
 * ufunc_ker_types[3] = <char>NPY_DOUBLE
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")             # <<<<<<<<<<<<<<
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[1]) = ((void *)((char *)__pyx_k_ker));

  /* "scipy/special/_ufuncs.pyx":6481
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap             # <<<<<<<<<<<<<<
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[2]) = ((void *)ker_wrap);

  /* "scipy/special/_ufuncs.pyx":6482
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")             # <<<<<<<<<<<<<<
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]
 * ufunc_ker_data[1] = &ufunc_ker_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[3]) = ((void *)((char *)__pyx_k_ker));

  /* "scipy/special/_ufuncs.pyx":6483
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ker_data[1] = &ufunc_ker_ptr[2*1]
 * ker = np.PyUFunc_FromFuncAndData(ufunc_ker_loops, ufunc_ker_data, ufunc_ker_types, 2, 1, 1, 0, "ker", ufunc_ker_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6484
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]
 * ufunc_ker_data[1] = &ufunc_ker_ptr[2*1]             # <<<<<<<<<<<<<<
 * ker = np.PyUFunc_FromFuncAndData(ufunc_ker_loops, ufunc_ker_data, ufunc_ker_types, 2, 1, 1, 0, "ker", ufunc_ker_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6485
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]
 * ufunc_ker_data[1] = &ufunc_ker_ptr[2*1]
 * ker = np.PyUFunc_FromFuncAndData(ufunc_ker_loops, ufunc_ker_data, ufunc_ker_types, 2, 1, 1, 0, "ker", ufunc_ker_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kerp_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types, 2, 1, 1, 0, __pyx_k_ker, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6485; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ker, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6485; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6491
 * cdef void *ufunc_kerp_data[2]
 * cdef char ufunc_kerp_types[4]
 * cdef char *ufunc_kerp_doc = (             # <<<<<<<<<<<<<<
 *     "kerp(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_doc = __pyx_k_kerp_x_Derivative_of_the_Kelvin;

  /* "scipy/special/_ufuncs.pyx":6495
 *     "\n"
 *     "Derivative of the Kelvin function ker")
 * ufunc_kerp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_kerp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kerp_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6496
 *     "Derivative of the Kelvin function ker")
 * ufunc_kerp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kerp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_kerp_types[0] = <char>NPY_FLOAT
 * ufunc_kerp_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6497
 * ufunc_kerp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kerp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kerp_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kerp_types[1] = <char>NPY_FLOAT
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6498
 * ufunc_kerp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kerp_types[0] = <char>NPY_FLOAT
 * ufunc_kerp_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6499
 * ufunc_kerp_types[0] = <char>NPY_FLOAT
 * ufunc_kerp_types[1] = <char>NPY_FLOAT
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6500
 * ufunc_kerp_types[1] = <char>NPY_FLOAT
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6501
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap             # <<<<<<<<<<<<<<
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[0]) = ((void *)kerp_wrap);

  /* "scipy/special/_ufuncs.pyx":6502
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")             # <<<<<<<<<<<<<<
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[1]) = ((void *)((char *)__pyx_k_kerp));

  /* "scipy/special/_ufuncs.pyx":6503
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap             # <<<<<<<<<<<<<<
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[2]) = ((void *)kerp_wrap);

  /* "scipy/special/_ufuncs.pyx":6504
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")             # <<<<<<<<<<<<<<
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]
 * ufunc_kerp_data[1] = &ufunc_kerp_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[3]) = ((void *)((char *)__pyx_k_kerp));

  /* "scipy/special/_ufuncs.pyx":6505
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kerp_data[1] = &ufunc_kerp_ptr[2*1]
 * kerp = np.PyUFunc_FromFuncAndData(ufunc_kerp_loops, ufunc_kerp_data, ufunc_kerp_types, 2, 1, 1, 0, "kerp", ufunc_kerp_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6506
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]
 * ufunc_kerp_data[1] = &ufunc_kerp_ptr[2*1]             # <<<<<<<<<<<<<<
 * kerp = np.PyUFunc_FromFuncAndData(ufunc_kerp_loops, ufunc_kerp_data, ufunc_kerp_types, 2, 1, 1, 0, "kerp", ufunc_kerp_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6507
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]
 * ufunc_kerp_data[1] = &ufunc_kerp_ptr[2*1]
 * kerp = np.PyUFunc_FromFuncAndData(ufunc_kerp_loops, ufunc_kerp_data, ufunc_kerp_types, 2, 1, 1, 0, "kerp", ufunc_kerp_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kl_div_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types, 2, 1, 1, 0, __pyx_k_kerp, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6507; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kerp, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6507; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6513
 * cdef void *ufunc_kl_div_data[2]
 * cdef char ufunc_kl_div_types[6]
 * cdef char *ufunc_kl_div_doc = (             # <<<<<<<<<<<<<<
 *     "kl_div(x, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_doc = __pyx_k_kl_div_x_y_Elementwise_function;

  /* "scipy/special/_ufuncs.pyx":6541
 *     "\n"
 *     ".. versionadded:: 0.14.0")
 * ufunc_kl_div_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_kl_div_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6542
 *     ".. versionadded:: 0.14.0")
 * ufunc_kl_div_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kl_div_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6543
 * ufunc_kl_div_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kl_div_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6544
 * ufunc_kl_div_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6545
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6546
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6547
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6548
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6549
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div             # <<<<<<<<<<<<<<
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_kl_div);

  /* "scipy/special/_ufuncs.pyx":6550
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")             # <<<<<<<<<<<<<<
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[1]) = ((void *)((char *)__pyx_k_kl_div));

  /* "scipy/special/_ufuncs.pyx":6551
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div             # <<<<<<<<<<<<<<
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_kl_div);

  /* "scipy/special/_ufuncs.pyx":6552
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")             # <<<<<<<<<<<<<<
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]
 * ufunc_kl_div_data[1] = &ufunc_kl_div_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[3]) = ((void *)((char *)__pyx_k_kl_div));

  /* "scipy/special/_ufuncs.pyx":6553
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kl_div_data[1] = &ufunc_kl_div_ptr[2*1]
 * kl_div = np.PyUFunc_FromFuncAndData(ufunc_kl_div_loops, ufunc_kl_div_data, ufunc_kl_div_types, 2, 2, 1, 0, "kl_div", ufunc_kl_div_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6554
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]
 * ufunc_kl_div_data[1] = &ufunc_kl_div_ptr[2*1]             # <<<<<<<<<<<<<<
 * kl_div = np.PyUFunc_FromFuncAndData(ufunc_kl_div_loops, ufunc_kl_div_data, ufunc_kl_div_types, 2, 2, 1, 0, "kl_div", ufunc_kl_div_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6555
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]
 * ufunc_kl_div_data[1] = &ufunc_kl_div_ptr[2*1]
 * kl_div = np.PyUFunc_FromFuncAndData(ufunc_kl_div_loops, ufunc_kl_div_data, ufunc_kl_div_types, 2, 2, 1, 0, "kl_div", ufunc_kl_div_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kn_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types, 2, 2, 1, 0, __pyx_k_kl_div, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6555; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kl_div, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6555; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6561
 * cdef void *ufunc_kn_data[3]
 * cdef char ufunc_kn_types[9]
 * cdef char *ufunc_kn_doc = (             # <<<<<<<<<<<<<<
 *     "kn(n, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_doc = __pyx_k_kn_n_x_Modified_Bessel_function;

  /* "scipy/special/_ufuncs.pyx":6607
 *     ">>> kn([4, 5, 6], 1)\n"
 *     "array([   44.23241585,   360.9605896 ,  3653.83831186])")
 * ufunc_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_kn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":6608
 *     "array([   44.23241585,   360.9605896 ,  3653.83831186])")
 * ufunc_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_kn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6609
 * ufunc_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_kn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_kn_types[0] = <char>NPY_LONG
 * ufunc_kn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6610
 * ufunc_kn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_kn_types[1] = <char>NPY_DOUBLE
 * ufunc_kn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":6611
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kn_types[0] = <char>NPY_LONG
 * ufunc_kn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_types[2] = <char>NPY_DOUBLE
 * ufunc_kn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6612
 * ufunc_kn_types[0] = <char>NPY_LONG
 * ufunc_kn_types[1] = <char>NPY_DOUBLE
 * ufunc_kn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_types[3] = <char>NPY_FLOAT
 * ufunc_kn_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6613
 * ufunc_kn_types[1] = <char>NPY_DOUBLE
 * ufunc_kn_types[2] = <char>NPY_DOUBLE
 * ufunc_kn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kn_types[4] = <char>NPY_FLOAT
 * ufunc_kn_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6614
 * ufunc_kn_types[2] = <char>NPY_DOUBLE
 * ufunc_kn_types[3] = <char>NPY_FLOAT
 * ufunc_kn_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kn_types[5] = <char>NPY_FLOAT
 * ufunc_kn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6615
 * ufunc_kn_types[3] = <char>NPY_FLOAT
 * ufunc_kn_types[4] = <char>NPY_FLOAT
 * ufunc_kn_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kn_types[6] = <char>NPY_DOUBLE
 * ufunc_kn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6616
 * ufunc_kn_types[4] = <char>NPY_FLOAT
 * ufunc_kn_types[5] = <char>NPY_FLOAT
 * ufunc_kn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_types[7] = <char>NPY_DOUBLE
 * ufunc_kn_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6617
 * ufunc_kn_types[5] = <char>NPY_FLOAT
 * ufunc_kn_types[6] = <char>NPY_DOUBLE
 * ufunc_kn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_types[8] = <char>NPY_DOUBLE
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6618
 * ufunc_kn_types[6] = <char>NPY_DOUBLE
 * ufunc_kn_types[7] = <char>NPY_DOUBLE
 * ufunc_kn_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6619
 * ufunc_kn_types[7] = <char>NPY_DOUBLE
 * ufunc_kn_types[8] = <char>NPY_DOUBLE
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[0]) = ((void *)cbesk_wrap_real_int);

  /* "scipy/special/_ufuncs.pyx":6620
 * ufunc_kn_types[8] = <char>NPY_DOUBLE
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[1]) = ((void *)((char *)__pyx_k_kn));

  /* "scipy/special/_ufuncs.pyx":6621
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_kn_unsafe);

  /* "scipy/special/_ufuncs.pyx":6622
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[3]) = ((void *)((char *)__pyx_k_kn));

  /* "scipy/special/_ufuncs.pyx":6623
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_kn_unsafe);

  /* "scipy/special/_ufuncs.pyx":6624
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")             # <<<<<<<<<<<<<<
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[5]) = ((void *)((char *)__pyx_k_kn));

  /* "scipy/special/_ufuncs.pyx":6625
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]
 * ufunc_kn_data[2] = &ufunc_kn_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6626
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_kn_data[2] = &ufunc_kn_ptr[2*2]
 * kn = np.PyUFunc_FromFuncAndData(ufunc_kn_loops, ufunc_kn_data, ufunc_kn_types, 3, 2, 1, 0, "kn", ufunc_kn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6627
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]
 * ufunc_kn_data[2] = &ufunc_kn_ptr[2*2]             # <<<<<<<<<<<<<<
 * kn = np.PyUFunc_FromFuncAndData(ufunc_kn_loops, ufunc_kn_data, ufunc_kn_types, 3, 2, 1, 0, "kn", ufunc_kn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6628
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]
 * ufunc_kn_data[2] = &ufunc_kn_ptr[2*2]
 * kn = np.PyUFunc_FromFuncAndData(ufunc_kn_loops, ufunc_kn_data, ufunc_kn_types, 3, 2, 1, 0, "kn", ufunc_kn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kolmogi_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types, 3, 2, 1, 0, __pyx_k_kn, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kn, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6634
 * cdef void *ufunc_kolmogi_data[2]
 * cdef char ufunc_kolmogi_types[4]
 * cdef char *ufunc_kolmogi_doc = (             # <<<<<<<<<<<<<<
 *     "kolmogi(p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_doc = __pyx_k_kolmogi_p_Inverse_function_to_ko;

  /* "scipy/special/_ufuncs.pyx":6640
 *     "\n"
 *     "Returns y such that ``kolmogorov(y) == p``.")
 * ufunc_kolmogi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6641
 *     "Returns y such that ``kolmogorov(y) == p``.")
 * ufunc_kolmogi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kolmogi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6642
 * ufunc_kolmogi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kolmogi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6643
 * ufunc_kolmogi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6644
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6645
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6646
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[0]) = ((void *)kolmogi);

  /* "scipy/special/_ufuncs.pyx":6647
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[1]) = ((void *)((char *)__pyx_k_kolmogi));

  /* "scipy/special/_ufuncs.pyx":6648
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[2]) = ((void *)kolmogi);

  /* "scipy/special/_ufuncs.pyx":6649
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]
 * ufunc_kolmogi_data[1] = &ufunc_kolmogi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[3]) = ((void *)((char *)__pyx_k_kolmogi));

  /* "scipy/special/_ufuncs.pyx":6650
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_data[1] = &ufunc_kolmogi_ptr[2*1]
 * kolmogi = np.PyUFunc_FromFuncAndData(ufunc_kolmogi_loops, ufunc_kolmogi_data, ufunc_kolmogi_types, 2, 1, 1, 0, "kolmogi", ufunc_kolmogi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6651
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]
 * ufunc_kolmogi_data[1] = &ufunc_kolmogi_ptr[2*1]             # <<<<<<<<<<<<<<
 * kolmogi = np.PyUFunc_FromFuncAndData(ufunc_kolmogi_loops, ufunc_kolmogi_data, ufunc_kolmogi_types, 2, 1, 1, 0, "kolmogi", ufunc_kolmogi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6652
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]
 * ufunc_kolmogi_data[1] = &ufunc_kolmogi_ptr[2*1]
 * kolmogi = np.PyUFunc_FromFuncAndData(ufunc_kolmogi_loops, ufunc_kolmogi_data, ufunc_kolmogi_types, 2, 1, 1, 0, "kolmogi", ufunc_kolmogi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kolmogorov_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types, 2, 1, 1, 0, __pyx_k_kolmogi, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6652; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kolmogi, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6652; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6658
 * cdef void *ufunc_kolmogorov_data[2]
 * cdef char ufunc_kolmogorov_types[4]
 * cdef char *ufunc_kolmogorov_doc = (             # <<<<<<<<<<<<<<
 *     "kolmogorov(y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_doc = __pyx_k_kolmogorov_y_Complementary_cumul;

  /* "scipy/special/_ufuncs.pyx":6668
 *     "distribution. It is equal to the (limit as n->infinity of the)\n"
 *     "probability that sqrt(n) * max absolute deviation > y.")
 * ufunc_kolmogorov_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6669
 *     "probability that sqrt(n) * max absolute deviation > y.")
 * ufunc_kolmogorov_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kolmogorov_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6670
 * ufunc_kolmogorov_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kolmogorov_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6671
 * ufunc_kolmogorov_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6672
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6673
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6674
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[0]) = ((void *)kolmogorov);

  /* "scipy/special/_ufuncs.pyx":6675
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[1]) = ((void *)((char *)__pyx_k_kolmogorov));

  /* "scipy/special/_ufuncs.pyx":6676
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[2]) = ((void *)kolmogorov);

  /* "scipy/special/_ufuncs.pyx":6677
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]
 * ufunc_kolmogorov_data[1] = &ufunc_kolmogorov_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[3]) = ((void *)((char *)__pyx_k_kolmogorov));

  /* "scipy/special/_ufuncs.pyx":6678
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_data[1] = &ufunc_kolmogorov_ptr[2*1]
 * kolmogorov = np.PyUFunc_FromFuncAndData(ufunc_kolmogorov_loops, ufunc_kolmogorov_data, ufunc_kolmogorov_types, 2, 1, 1, 0, "kolmogorov", ufunc_kolmogorov_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6679
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]
 * ufunc_kolmogorov_data[1] = &ufunc_kolmogorov_ptr[2*1]             # <<<<<<<<<<<<<<
 * kolmogorov = np.PyUFunc_FromFuncAndData(ufunc_kolmogorov_loops, ufunc_kolmogorov_data, ufunc_kolmogorov_types, 2, 1, 1, 0, "kolmogorov", ufunc_kolmogorov_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6680
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]
 * ufunc_kolmogorov_data[1] = &ufunc_kolmogorov_ptr[2*1]
 * kolmogorov = np.PyUFunc_FromFuncAndData(ufunc_kolmogorov_loops, ufunc_kolmogorov_data, ufunc_kolmogorov_types, 2, 1, 1, 0, "kolmogorov", ufunc_kolmogorov_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kv_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types, 2, 1, 1, 0, __pyx_k_kolmogorov, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kolmogorov, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6686
 * cdef void *ufunc_kv_data[4]
 * cdef char ufunc_kv_types[12]
 * cdef char *ufunc_kv_doc = (             # <<<<<<<<<<<<<<
 *     "kv(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_doc = __pyx_k_kv_v_z_Modified_Bessel_function;

  /* "scipy/special/_ufuncs.pyx":6732
 *     ">>> kv([4, 4.5, 5], 1+2j)\n"
 *     "array([ 0.1992+2.3892j,  2.3493+3.6j   ,  7.2827+3.8104j])")
 * ufunc_kv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_kv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6733
 *     "array([ 0.1992+2.3892j,  2.3493+3.6j   ,  7.2827+3.8104j])")
 * ufunc_kv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":6734
 * ufunc_kv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6735
 * ufunc_kv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_kv_types[0] = <char>NPY_FLOAT
 * ufunc_kv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":6736
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[1] = <char>NPY_FLOAT
 * ufunc_kv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6737
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kv_types[0] = <char>NPY_FLOAT
 * ufunc_kv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[2] = <char>NPY_FLOAT
 * ufunc_kv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6738
 * ufunc_kv_types[0] = <char>NPY_FLOAT
 * ufunc_kv_types[1] = <char>NPY_FLOAT
 * ufunc_kv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[3] = <char>NPY_FLOAT
 * ufunc_kv_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6739
 * ufunc_kv_types[1] = <char>NPY_FLOAT
 * ufunc_kv_types[2] = <char>NPY_FLOAT
 * ufunc_kv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[4] = <char>NPY_CFLOAT
 * ufunc_kv_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6740
 * ufunc_kv_types[2] = <char>NPY_FLOAT
 * ufunc_kv_types[3] = <char>NPY_FLOAT
 * ufunc_kv_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[5] = <char>NPY_CFLOAT
 * ufunc_kv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6741
 * ufunc_kv_types[3] = <char>NPY_FLOAT
 * ufunc_kv_types[4] = <char>NPY_CFLOAT
 * ufunc_kv_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[6] = <char>NPY_DOUBLE
 * ufunc_kv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6742
 * ufunc_kv_types[4] = <char>NPY_CFLOAT
 * ufunc_kv_types[5] = <char>NPY_CFLOAT
 * ufunc_kv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[7] = <char>NPY_DOUBLE
 * ufunc_kv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6743
 * ufunc_kv_types[5] = <char>NPY_CFLOAT
 * ufunc_kv_types[6] = <char>NPY_DOUBLE
 * ufunc_kv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[8] = <char>NPY_DOUBLE
 * ufunc_kv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6744
 * ufunc_kv_types[6] = <char>NPY_DOUBLE
 * ufunc_kv_types[7] = <char>NPY_DOUBLE
 * ufunc_kv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[9] = <char>NPY_DOUBLE
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6745
 * ufunc_kv_types[7] = <char>NPY_DOUBLE
 * ufunc_kv_types[8] = <char>NPY_DOUBLE
 * ufunc_kv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6746
 * ufunc_kv_types[8] = <char>NPY_DOUBLE
 * ufunc_kv_types[9] = <char>NPY_DOUBLE
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6747
 * ufunc_kv_types[9] = <char>NPY_DOUBLE
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6748
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[0]) = ((void *)cbesk_wrap_real);

  /* "scipy/special/_ufuncs.pyx":6749
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[1]) = ((void *)((char *)__pyx_k_kv));

  /* "scipy/special/_ufuncs.pyx":6750
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[2]) = ((void *)cbesk_wrap);

  /* "scipy/special/_ufuncs.pyx":6751
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[3]) = ((void *)((char *)__pyx_k_kv));

  /* "scipy/special/_ufuncs.pyx":6752
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[4]) = ((void *)cbesk_wrap_real);

  /* "scipy/special/_ufuncs.pyx":6753
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[5]) = ((void *)((char *)__pyx_k_kv));

  /* "scipy/special/_ufuncs.pyx":6754
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[6]) = ((void *)cbesk_wrap);

  /* "scipy/special/_ufuncs.pyx":6755
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")             # <<<<<<<<<<<<<<
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[7]) = ((void *)((char *)__pyx_k_kv));

  /* "scipy/special/_ufuncs.pyx":6756
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6757
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]
 * ufunc_kv_data[3] = &ufunc_kv_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6758
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_kv_data[3] = &ufunc_kv_ptr[2*3]
 * kv = np.PyUFunc_FromFuncAndData(ufunc_kv_loops, ufunc_kv_data, ufunc_kv_types, 4, 2, 1, 0, "kv", ufunc_kv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6759
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]
 * ufunc_kv_data[3] = &ufunc_kv_ptr[2*3]             # <<<<<<<<<<<<<<
 * kv = np.PyUFunc_FromFuncAndData(ufunc_kv_loops, ufunc_kv_data, ufunc_kv_types, 4, 2, 1, 0, "kv", ufunc_kv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6760
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]
 * ufunc_kv_data[3] = &ufunc_kv_ptr[2*3]
 * kv = np.PyUFunc_FromFuncAndData(ufunc_kv_loops, ufunc_kv_data, ufunc_kv_types, 4, 2, 1, 0, "kv", ufunc_kv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kve_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types, 4, 2, 1, 0, __pyx_k_kv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6760; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6760; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6766
 * cdef void *ufunc_kve_data[4]
 * cdef char ufunc_kve_types[12]
 * cdef char *ufunc_kve_doc = (             # <<<<<<<<<<<<<<
 *     "kve(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_doc = __pyx_k_kve_v_z_Exponentially_scaled_mod;

  /* "scipy/special/_ufuncs.pyx":6776
 *     "\n"
 *     "    kve(v, z) = kv(v, z) * exp(z)")
 * ufunc_kve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_kve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6777
 *     "    kve(v, z) = kv(v, z) * exp(z)")
 * ufunc_kve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":6778
 * ufunc_kve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6779
 * ufunc_kve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_kve_types[0] = <char>NPY_FLOAT
 * ufunc_kve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":6780
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[1] = <char>NPY_FLOAT
 * ufunc_kve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6781
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kve_types[0] = <char>NPY_FLOAT
 * ufunc_kve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[2] = <char>NPY_FLOAT
 * ufunc_kve_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6782
 * ufunc_kve_types[0] = <char>NPY_FLOAT
 * ufunc_kve_types[1] = <char>NPY_FLOAT
 * ufunc_kve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[3] = <char>NPY_FLOAT
 * ufunc_kve_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6783
 * ufunc_kve_types[1] = <char>NPY_FLOAT
 * ufunc_kve_types[2] = <char>NPY_FLOAT
 * ufunc_kve_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[4] = <char>NPY_CFLOAT
 * ufunc_kve_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6784
 * ufunc_kve_types[2] = <char>NPY_FLOAT
 * ufunc_kve_types[3] = <char>NPY_FLOAT
 * ufunc_kve_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[5] = <char>NPY_CFLOAT
 * ufunc_kve_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6785
 * ufunc_kve_types[3] = <char>NPY_FLOAT
 * ufunc_kve_types[4] = <char>NPY_CFLOAT
 * ufunc_kve_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[6] = <char>NPY_DOUBLE
 * ufunc_kve_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6786
 * ufunc_kve_types[4] = <char>NPY_CFLOAT
 * ufunc_kve_types[5] = <char>NPY_CFLOAT
 * ufunc_kve_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[7] = <char>NPY_DOUBLE
 * ufunc_kve_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6787
 * ufunc_kve_types[5] = <char>NPY_CFLOAT
 * ufunc_kve_types[6] = <char>NPY_DOUBLE
 * ufunc_kve_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[8] = <char>NPY_DOUBLE
 * ufunc_kve_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6788
 * ufunc_kve_types[6] = <char>NPY_DOUBLE
 * ufunc_kve_types[7] = <char>NPY_DOUBLE
 * ufunc_kve_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[9] = <char>NPY_DOUBLE
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6789
 * ufunc_kve_types[7] = <char>NPY_DOUBLE
 * ufunc_kve_types[8] = <char>NPY_DOUBLE
 * ufunc_kve_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6790
 * ufunc_kve_types[8] = <char>NPY_DOUBLE
 * ufunc_kve_types[9] = <char>NPY_DOUBLE
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6791
 * ufunc_kve_types[9] = <char>NPY_DOUBLE
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6792
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[0]) = ((void *)cbesk_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":6793
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[1]) = ((void *)((char *)__pyx_k_kve));

  /* "scipy/special/_ufuncs.pyx":6794
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[2]) = ((void *)cbesk_wrap_e);

  /* "scipy/special/_ufuncs.pyx":6795
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[3]) = ((void *)((char *)__pyx_k_kve));

  /* "scipy/special/_ufuncs.pyx":6796
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[4]) = ((void *)cbesk_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":6797
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[5]) = ((void *)((char *)__pyx_k_kve));

  /* "scipy/special/_ufuncs.pyx":6798
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[6]) = ((void *)cbesk_wrap_e);

  /* "scipy/special/_ufuncs.pyx":6799
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")             # <<<<<<<<<<<<<<
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[7]) = ((void *)((char *)__pyx_k_kve));

  /* "scipy/special/_ufuncs.pyx":6800
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6801
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]
 * ufunc_kve_data[3] = &ufunc_kve_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6802
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_kve_data[3] = &ufunc_kve_ptr[2*3]
 * kve = np.PyUFunc_FromFuncAndData(ufunc_kve_loops, ufunc_kve_data, ufunc_kve_types, 4, 2, 1, 0, "kve", ufunc_kve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6803
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]
 * ufunc_kve_data[3] = &ufunc_kve_ptr[2*3]             # <<<<<<<<<<<<<<
 * kve = np.PyUFunc_FromFuncAndData(ufunc_kve_loops, ufunc_kve_data, ufunc_kve_types, 4, 2, 1, 0, "kve", ufunc_kve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6804
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]
 * ufunc_kve_data[3] = &ufunc_kve_ptr[2*3]
 * kve = np.PyUFunc_FromFuncAndData(ufunc_kve_loops, ufunc_kve_data, ufunc_kve_types, 4, 2, 1, 0, "kve", ufunc_kve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_log1p_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types, 4, 2, 1, 0, __pyx_k_kve, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kve, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6810
 * cdef void *ufunc_log1p_data[2]
 * cdef char ufunc_log1p_types[4]
 * cdef char *ufunc_log1p_doc = (             # <<<<<<<<<<<<<<
 *     "log1p(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_doc = __pyx_k_log1p_x_Calculates_log_1_x_for_u;

  /* "scipy/special/_ufuncs.pyx":6814
 *     "\n"
 *     "Calculates log(1+x) for use when `x` is near zero")
 * ufunc_log1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_log1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log1p_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6815
 *     "Calculates log(1+x) for use when `x` is near zero")
 * ufunc_log1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_log1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[0] = <char>NPY_FLOAT
 * ufunc_log1p_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6816
 * ufunc_log1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_log1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log1p_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[1] = <char>NPY_FLOAT
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6817
 * ufunc_log1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log1p_types[0] = <char>NPY_FLOAT
 * ufunc_log1p_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6818
 * ufunc_log1p_types[0] = <char>NPY_FLOAT
 * ufunc_log1p_types[1] = <char>NPY_FLOAT
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6819
 * ufunc_log1p_types[1] = <char>NPY_FLOAT
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6820
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[0]) = ((void *)log1p);

  /* "scipy/special/_ufuncs.pyx":6821
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[1]) = ((void *)((char *)__pyx_k_log1p));

  /* "scipy/special/_ufuncs.pyx":6822
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[2]) = ((void *)log1p);

  /* "scipy/special/_ufuncs.pyx":6823
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")             # <<<<<<<<<<<<<<
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[3]) = ((void *)((char *)__pyx_k_log1p));

  /* "scipy/special/_ufuncs.pyx":6824
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]
 * log1p = np.PyUFunc_FromFuncAndData(ufunc_log1p_loops, ufunc_log1p_data, ufunc_log1p_types, 2, 1, 1, 0, "log1p", ufunc_log1p_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6825
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]             # <<<<<<<<<<<<<<
 * log1p = np.PyUFunc_FromFuncAndData(ufunc_log1p_loops, ufunc_log1p_data, ufunc_log1p_types, 2, 1, 1, 0, "log1p", ufunc_log1p_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6826
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]
 * log1p = np.PyUFunc_FromFuncAndData(ufunc_log1p_loops, ufunc_log1p_data, ufunc_log1p_types, 2, 1, 1, 0, "log1p", ufunc_log1p_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_log_ndtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types, 2, 1, 1, 0, __pyx_k_log1p, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_log1p, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6832
 * cdef void *ufunc_log_ndtr_data[2]
 * cdef char ufunc_log_ndtr_types[4]
 * cdef char *ufunc_log_ndtr_doc = (             # <<<<<<<<<<<<<<
 *     "log_ndtr(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_doc = __pyx_k_log_ndtr_x_Logarithm_of_Gaussian;

  /* "scipy/special/_ufuncs.pyx":6841
 *     "\n"
 *     "    log(1/sqrt(2*pi) * integral(exp(-t**2 / 2), t=-inf..x))")
 * ufunc_log_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6842
 *     "    log(1/sqrt(2*pi) * integral(exp(-t**2 / 2), t=-inf..x))")
 * ufunc_log_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_log_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6843
 * ufunc_log_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_log_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6844
 * ufunc_log_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6845
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6846
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6847
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[0]) = ((void *)log_ndtr);

  /* "scipy/special/_ufuncs.pyx":6848
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[1]) = ((void *)((char *)__pyx_k_log_ndtr));

  /* "scipy/special/_ufuncs.pyx":6849
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[2]) = ((void *)log_ndtr);

  /* "scipy/special/_ufuncs.pyx":6850
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[3]) = ((void *)((char *)__pyx_k_log_ndtr));

  /* "scipy/special/_ufuncs.pyx":6851
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]
 * log_ndtr = np.PyUFunc_FromFuncAndData(ufunc_log_ndtr_loops, ufunc_log_ndtr_data, ufunc_log_ndtr_types, 2, 1, 1, 0, "log_ndtr", ufunc_log_ndtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6852
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * log_ndtr = np.PyUFunc_FromFuncAndData(ufunc_log_ndtr_loops, ufunc_log_ndtr_data, ufunc_log_ndtr_types, 2, 1, 1, 0, "log_ndtr", ufunc_log_ndtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6853
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]
 * log_ndtr = np.PyUFunc_FromFuncAndData(ufunc_log_ndtr_loops, ufunc_log_ndtr_data, ufunc_log_ndtr_types, 2, 1, 1, 0, "log_ndtr", ufunc_log_ndtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_logit_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types, 2, 1, 1, 0, __pyx_k_log_ndtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6853; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_log_ndtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6853; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6859
 * cdef void *ufunc_logit_data[3]
 * cdef char ufunc_logit_types[6]
 * cdef char *ufunc_logit_doc = (             # <<<<<<<<<<<<<<
 *     "logit(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_doc = __pyx_k_logit_x_Logit_ufunc_for_ndarrays;

  /* "scipy/special/_ufuncs.pyx":6886
 *     "\n"
 *     ".. versionadded:: 0.10.0")
 * ufunc_logit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_logit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_f_f__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6887
 *     ".. versionadded:: 0.10.0")
 * ufunc_logit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f
 * ufunc_logit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_logit_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6888
 * ufunc_logit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f
 * ufunc_logit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g             # <<<<<<<<<<<<<<
 * ufunc_logit_types[0] = <char>NPY_FLOAT
 * ufunc_logit_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_g_g__As_g_g);

  /* "scipy/special/_ufuncs.pyx":6889
 * ufunc_logit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_logit_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_logit_types[1] = <char>NPY_FLOAT
 * ufunc_logit_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6890
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_logit_types[0] = <char>NPY_FLOAT
 * ufunc_logit_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_logit_types[2] = <char>NPY_DOUBLE
 * ufunc_logit_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6891
 * ufunc_logit_types[0] = <char>NPY_FLOAT
 * ufunc_logit_types[1] = <char>NPY_FLOAT
 * ufunc_logit_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_logit_types[3] = <char>NPY_DOUBLE
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6892
 * ufunc_logit_types[1] = <char>NPY_FLOAT
 * ufunc_logit_types[2] = <char>NPY_DOUBLE
 * ufunc_logit_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6893
 * ufunc_logit_types[2] = <char>NPY_DOUBLE
 * ufunc_logit_types[3] = <char>NPY_DOUBLE
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[4]) = ((char)NPY_LONGDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6894
 * ufunc_logit_types[3] = <char>NPY_DOUBLE
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[5]) = ((char)NPY_LONGDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6895
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*1] = <void*>_func_logit
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[0]) = ((void *)logitf);

  /* "scipy/special/_ufuncs.pyx":6896
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*1] = <void*>_func_logit
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[1]) = ((void *)((char *)__pyx_k_logit));

  /* "scipy/special/_ufuncs.pyx":6897
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*1] = <void*>_func_logit             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[2]) = ((void *)logit);

  /* "scipy/special/_ufuncs.pyx":6898
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*1] = <void*>_func_logit
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[3]) = ((void *)((char *)__pyx_k_logit));

  /* "scipy/special/_ufuncs.pyx":6899
 * ufunc_logit_ptr[2*1] = <void*>_func_logit
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[4]) = ((void *)logitl);

  /* "scipy/special/_ufuncs.pyx":6900
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")             # <<<<<<<<<<<<<<
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[5]) = ((void *)((char *)__pyx_k_logit));

  /* "scipy/special/_ufuncs.pyx":6901
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]
 * ufunc_logit_data[2] = &ufunc_logit_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6902
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_logit_data[2] = &ufunc_logit_ptr[2*2]
 * logit = np.PyUFunc_FromFuncAndData(ufunc_logit_loops, ufunc_logit_data, ufunc_logit_types, 3, 1, 1, 0, "logit", ufunc_logit_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6903
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]
 * ufunc_logit_data[2] = &ufunc_logit_ptr[2*2]             # <<<<<<<<<<<<<<
 * logit = np.PyUFunc_FromFuncAndData(ufunc_logit_loops, ufunc_logit_data, ufunc_logit_types, 3, 1, 1, 0, "logit", ufunc_logit_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6904
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]
 * ufunc_logit_data[2] = &ufunc_logit_ptr[2*2]
 * logit = np.PyUFunc_FromFuncAndData(ufunc_logit_loops, ufunc_logit_data, ufunc_logit_types, 3, 1, 1, 0, "logit", ufunc_logit_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_lpmv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types, 3, 1, 1, 0, __pyx_k_logit, __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6904; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logit, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6904; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6910
 * cdef void *ufunc_lpmv_data[2]
 * cdef char ufunc_lpmv_types[8]
 * cdef char *ufunc_lpmv_doc = (             # <<<<<<<<<<<<<<
 *     "lpmv(m, v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_doc = __pyx_k_lpmv_m_v_x_Associated_legendre_f;

  /* "scipy/special/_ufuncs.pyx":6938
 *     "It is possible to extend the domain of this function to all\n"
 *     "complex m, v, x, but this is not yet implemented.")
 * ufunc_lpmv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_lpmv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":6939
 *     "complex m, v, x, but this is not yet implemented.")
 * ufunc_lpmv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_lpmv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":6940
 * ufunc_lpmv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_lpmv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6941
 * ufunc_lpmv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6942
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6943
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6944
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6945
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6946
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6947
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6948
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap             # <<<<<<<<<<<<<<
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[0]) = ((void *)pmv_wrap);

  /* "scipy/special/_ufuncs.pyx":6949
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")             # <<<<<<<<<<<<<<
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[1]) = ((void *)((char *)__pyx_k_lpmv));

  /* "scipy/special/_ufuncs.pyx":6950
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap             # <<<<<<<<<<<<<<
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[2]) = ((void *)pmv_wrap);

  /* "scipy/special/_ufuncs.pyx":6951
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")             # <<<<<<<<<<<<<<
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]
 * ufunc_lpmv_data[1] = &ufunc_lpmv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[3]) = ((void *)((char *)__pyx_k_lpmv));

  /* "scipy/special/_ufuncs.pyx":6952
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_lpmv_data[1] = &ufunc_lpmv_ptr[2*1]
 * lpmv = np.PyUFunc_FromFuncAndData(ufunc_lpmv_loops, ufunc_lpmv_data, ufunc_lpmv_types, 2, 3, 1, 0, "lpmv", ufunc_lpmv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6953
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]
 * ufunc_lpmv_data[1] = &ufunc_lpmv_ptr[2*1]             # <<<<<<<<<<<<<<
 * lpmv = np.PyUFunc_FromFuncAndData(ufunc_lpmv_loops, ufunc_lpmv_data, ufunc_lpmv_types, 2, 3, 1, 0, "lpmv", ufunc_lpmv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6954
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]
 * ufunc_lpmv_data[1] = &ufunc_lpmv_ptr[2*1]
 * lpmv = np.PyUFunc_FromFuncAndData(ufunc_lpmv_loops, ufunc_lpmv_data, ufunc_lpmv_types, 2, 3, 1, 0, "lpmv", ufunc_lpmv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_a_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types, 2, 3, 1, 0, __pyx_k_lpmv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_lpmv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6960
 * cdef void *ufunc_mathieu_a_data[2]
 * cdef char ufunc_mathieu_a_types[6]
 * cdef char *ufunc_mathieu_a_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_a(m, q)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_doc = __pyx_k_mathieu_a_m_q_Characteristic_val;

  /* "scipy/special/_ufuncs.pyx":6967
 *     "Returns the characteristic value for the even solution,\n"
 *     "``ce_m(z, q)``, of Mathieu's equation.")
 * ufunc_mathieu_a_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6968
 *     "``ce_m(z, q)``, of Mathieu's equation.")
 * ufunc_mathieu_a_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_mathieu_a_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6969
 * ufunc_mathieu_a_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_mathieu_a_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6970
 * ufunc_mathieu_a_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6971
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6972
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6973
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6974
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6975
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[0]) = ((void *)cem_cva_wrap);

  /* "scipy/special/_ufuncs.pyx":6976
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[1]) = ((void *)((char *)__pyx_k_mathieu_a));

  /* "scipy/special/_ufuncs.pyx":6977
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[2]) = ((void *)cem_cva_wrap);

  /* "scipy/special/_ufuncs.pyx":6978
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]
 * ufunc_mathieu_a_data[1] = &ufunc_mathieu_a_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[3]) = ((void *)((char *)__pyx_k_mathieu_a));

  /* "scipy/special/_ufuncs.pyx":6979
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_data[1] = &ufunc_mathieu_a_ptr[2*1]
 * mathieu_a = np.PyUFunc_FromFuncAndData(ufunc_mathieu_a_loops, ufunc_mathieu_a_data, ufunc_mathieu_a_types, 2, 2, 1, 0, "mathieu_a", ufunc_mathieu_a_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6980
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]
 * ufunc_mathieu_a_data[1] = &ufunc_mathieu_a_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_a = np.PyUFunc_FromFuncAndData(ufunc_mathieu_a_loops, ufunc_mathieu_a_data, ufunc_mathieu_a_types, 2, 2, 1, 0, "mathieu_a", ufunc_mathieu_a_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6981
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]
 * ufunc_mathieu_a_data[1] = &ufunc_mathieu_a_ptr[2*1]
 * mathieu_a = np.PyUFunc_FromFuncAndData(ufunc_mathieu_a_loops, ufunc_mathieu_a_data, ufunc_mathieu_a_types, 2, 2, 1, 0, "mathieu_a", ufunc_mathieu_a_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_b_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types, 2, 2, 1, 0, __pyx_k_mathieu_a, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6981; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_a, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6981; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":6987
 * cdef void *ufunc_mathieu_b_data[2]
 * cdef char ufunc_mathieu_b_types[6]
 * cdef char *ufunc_mathieu_b_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_b(m, q)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_doc = __pyx_k_mathieu_b_m_q_Characteristic_val;

  /* "scipy/special/_ufuncs.pyx":6994
 *     "Returns the characteristic value for the odd solution,\n"
 *     "``se_m(z, q)``, of Mathieu's equation.")
 * ufunc_mathieu_b_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6995
 *     "``se_m(z, q)``, of Mathieu's equation.")
 * ufunc_mathieu_b_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_mathieu_b_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6996
 * ufunc_mathieu_b_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_mathieu_b_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6997
 * ufunc_mathieu_b_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6998
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6999
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7000
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7001
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7002
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[0]) = ((void *)sem_cva_wrap);

  /* "scipy/special/_ufuncs.pyx":7003
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[1]) = ((void *)((char *)__pyx_k_mathieu_b));

  /* "scipy/special/_ufuncs.pyx":7004
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[2]) = ((void *)sem_cva_wrap);

  /* "scipy/special/_ufuncs.pyx":7005
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]
 * ufunc_mathieu_b_data[1] = &ufunc_mathieu_b_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[3]) = ((void *)((char *)__pyx_k_mathieu_b));

  /* "scipy/special/_ufuncs.pyx":7006
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_data[1] = &ufunc_mathieu_b_ptr[2*1]
 * mathieu_b = np.PyUFunc_FromFuncAndData(ufunc_mathieu_b_loops, ufunc_mathieu_b_data, ufunc_mathieu_b_types, 2, 2, 1, 0, "mathieu_b", ufunc_mathieu_b_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7007
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]
 * ufunc_mathieu_b_data[1] = &ufunc_mathieu_b_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_b = np.PyUFunc_FromFuncAndData(ufunc_mathieu_b_loops, ufunc_mathieu_b_data, ufunc_mathieu_b_types, 2, 2, 1, 0, "mathieu_b", ufunc_mathieu_b_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7008
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]
 * ufunc_mathieu_b_data[1] = &ufunc_mathieu_b_ptr[2*1]
 * mathieu_b = np.PyUFunc_FromFuncAndData(ufunc_mathieu_b_loops, ufunc_mathieu_b_data, ufunc_mathieu_b_types, 2, 2, 1, 0, "mathieu_b", ufunc_mathieu_b_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_cem_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types, 2, 2, 1, 0, __pyx_k_mathieu_b, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7008; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_b, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7008; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7014
 * cdef void *ufunc_mathieu_cem_data[2]
 * cdef char ufunc_mathieu_cem_types[10]
 * cdef char *ufunc_mathieu_cem_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_cem(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_doc = __pyx_k_mathieu_cem_m_q_x_Even_Mathieu_f;

  /* "scipy/special/_ufuncs.pyx":7038
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_cem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":7039
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_cem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_cem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":7040
 * ufunc_mathieu_cem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_cem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7041
 * ufunc_mathieu_cem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7042
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7043
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7044
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7045
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7046
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7047
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7048
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7049
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7050
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[0]) = ((void *)cem_wrap);

  /* "scipy/special/_ufuncs.pyx":7051
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[1]) = ((void *)((char *)__pyx_k_mathieu_cem));

  /* "scipy/special/_ufuncs.pyx":7052
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[2]) = ((void *)cem_wrap);

  /* "scipy/special/_ufuncs.pyx":7053
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]
 * ufunc_mathieu_cem_data[1] = &ufunc_mathieu_cem_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[3]) = ((void *)((char *)__pyx_k_mathieu_cem));

  /* "scipy/special/_ufuncs.pyx":7054
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_data[1] = &ufunc_mathieu_cem_ptr[2*1]
 * mathieu_cem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_cem_loops, ufunc_mathieu_cem_data, ufunc_mathieu_cem_types, 2, 3, 2, 0, "mathieu_cem", ufunc_mathieu_cem_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7055
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]
 * ufunc_mathieu_cem_data[1] = &ufunc_mathieu_cem_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_cem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_cem_loops, ufunc_mathieu_cem_data, ufunc_mathieu_cem_types, 2, 3, 2, 0, "mathieu_cem", ufunc_mathieu_cem_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7056
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]
 * ufunc_mathieu_cem_data[1] = &ufunc_mathieu_cem_ptr[2*1]
 * mathieu_cem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_cem_loops, ufunc_mathieu_cem_data, ufunc_mathieu_cem_types, 2, 3, 2, 0, "mathieu_cem", ufunc_mathieu_cem_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_modcem1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types, 2, 3, 2, 0, __pyx_k_mathieu_cem, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7056; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_cem, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7056; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7062
 * cdef void *ufunc_mathieu_modcem1_data[2]
 * cdef char ufunc_mathieu_modcem1_types[10]
 * cdef char *ufunc_mathieu_modcem1_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_modcem1(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_doc = __pyx_k_mathieu_modcem1_m_q_x_Even_modif;

  /* "scipy/special/_ufuncs.pyx":7077
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modcem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":7078
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modcem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modcem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":7079
 * ufunc_mathieu_modcem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modcem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7080
 * ufunc_mathieu_modcem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7081
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7082
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7083
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7084
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7085
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7086
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7087
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7088
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7089
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[0]) = ((void *)mcm1_wrap);

  /* "scipy/special/_ufuncs.pyx":7090
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[1]) = ((void *)((char *)__pyx_k_mathieu_modcem1));

  /* "scipy/special/_ufuncs.pyx":7091
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[2]) = ((void *)mcm1_wrap);

  /* "scipy/special/_ufuncs.pyx":7092
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]
 * ufunc_mathieu_modcem1_data[1] = &ufunc_mathieu_modcem1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[3]) = ((void *)((char *)__pyx_k_mathieu_modcem1));

  /* "scipy/special/_ufuncs.pyx":7093
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_data[1] = &ufunc_mathieu_modcem1_ptr[2*1]
 * mathieu_modcem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem1_loops, ufunc_mathieu_modcem1_data, ufunc_mathieu_modcem1_types, 2, 3, 2, 0, "mathieu_modcem1", ufunc_mathieu_modcem1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7094
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]
 * ufunc_mathieu_modcem1_data[1] = &ufunc_mathieu_modcem1_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_modcem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem1_loops, ufunc_mathieu_modcem1_data, ufunc_mathieu_modcem1_types, 2, 3, 2, 0, "mathieu_modcem1", ufunc_mathieu_modcem1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7095
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]
 * ufunc_mathieu_modcem1_data[1] = &ufunc_mathieu_modcem1_ptr[2*1]
 * mathieu_modcem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem1_loops, ufunc_mathieu_modcem1_data, ufunc_mathieu_modcem1_types, 2, 3, 2, 0, "mathieu_modcem1", ufunc_mathieu_modcem1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_modcem2_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types, 2, 3, 2, 0, __pyx_k_mathieu_modcem1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7095; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_modcem1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7095; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7101
 * cdef void *ufunc_mathieu_modcem2_data[2]
 * cdef char ufunc_mathieu_modcem2_types[10]
 * cdef char *ufunc_mathieu_modcem2_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_modcem2(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_doc = __pyx_k_mathieu_modcem2_m_q_x_Even_modif;

  /* "scipy/special/_ufuncs.pyx":7116
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modcem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":7117
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modcem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modcem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":7118
 * ufunc_mathieu_modcem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modcem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7119
 * ufunc_mathieu_modcem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7120
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7121
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7122
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7123
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7124
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7125
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7126
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7127
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7128
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[0]) = ((void *)mcm2_wrap);

  /* "scipy/special/_ufuncs.pyx":7129
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[1]) = ((void *)((char *)__pyx_k_mathieu_modcem2));

  /* "scipy/special/_ufuncs.pyx":7130
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[2]) = ((void *)mcm2_wrap);

  /* "scipy/special/_ufuncs.pyx":7131
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]
 * ufunc_mathieu_modcem2_data[1] = &ufunc_mathieu_modcem2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[3]) = ((void *)((char *)__pyx_k_mathieu_modcem2));

  /* "scipy/special/_ufuncs.pyx":7132
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_data[1] = &ufunc_mathieu_modcem2_ptr[2*1]
 * mathieu_modcem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem2_loops, ufunc_mathieu_modcem2_data, ufunc_mathieu_modcem2_types, 2, 3, 2, 0, "mathieu_modcem2", ufunc_mathieu_modcem2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7133
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]
 * ufunc_mathieu_modcem2_data[1] = &ufunc_mathieu_modcem2_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_modcem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem2_loops, ufunc_mathieu_modcem2_data, ufunc_mathieu_modcem2_types, 2, 3, 2, 0, "mathieu_modcem2", ufunc_mathieu_modcem2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7134
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]
 * ufunc_mathieu_modcem2_data[1] = &ufunc_mathieu_modcem2_ptr[2*1]
 * mathieu_modcem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem2_loops, ufunc_mathieu_modcem2_data, ufunc_mathieu_modcem2_types, 2, 3, 2, 0, "mathieu_modcem2", ufunc_mathieu_modcem2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_modsem1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types, 2, 3, 2, 0, __pyx_k_mathieu_modcem2, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_modcem2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7140
 * cdef void *ufunc_mathieu_modsem1_data[2]
 * cdef char ufunc_mathieu_modsem1_types[10]
 * cdef char *ufunc_mathieu_modsem1_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_modsem1(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_doc = __pyx_k_mathieu_modsem1_m_q_x_Odd_modifi;

  /* "scipy/special/_ufuncs.pyx":7155
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modsem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":7156
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modsem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modsem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":7157
 * ufunc_mathieu_modsem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modsem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7158
 * ufunc_mathieu_modsem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7159
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7160
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7161
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7162
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7163
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7164
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7165
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7166
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7167
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[0]) = ((void *)msm1_wrap);

  /* "scipy/special/_ufuncs.pyx":7168
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[1]) = ((void *)((char *)__pyx_k_mathieu_modsem1));

  /* "scipy/special/_ufuncs.pyx":7169
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[2]) = ((void *)msm1_wrap);

  /* "scipy/special/_ufuncs.pyx":7170
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]
 * ufunc_mathieu_modsem1_data[1] = &ufunc_mathieu_modsem1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[3]) = ((void *)((char *)__pyx_k_mathieu_modsem1));

  /* "scipy/special/_ufuncs.pyx":7171
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_data[1] = &ufunc_mathieu_modsem1_ptr[2*1]
 * mathieu_modsem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem1_loops, ufunc_mathieu_modsem1_data, ufunc_mathieu_modsem1_types, 2, 3, 2, 0, "mathieu_modsem1", ufunc_mathieu_modsem1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7172
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]
 * ufunc_mathieu_modsem1_data[1] = &ufunc_mathieu_modsem1_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_modsem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem1_loops, ufunc_mathieu_modsem1_data, ufunc_mathieu_modsem1_types, 2, 3, 2, 0, "mathieu_modsem1", ufunc_mathieu_modsem1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7173
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]
 * ufunc_mathieu_modsem1_data[1] = &ufunc_mathieu_modsem1_ptr[2*1]
 * mathieu_modsem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem1_loops, ufunc_mathieu_modsem1_data, ufunc_mathieu_modsem1_types, 2, 3, 2, 0, "mathieu_modsem1", ufunc_mathieu_modsem1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_modsem2_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types, 2, 3, 2, 0, __pyx_k_mathieu_modsem1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_modsem1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7179
 * cdef void *ufunc_mathieu_modsem2_data[2]
 * cdef char ufunc_mathieu_modsem2_types[10]
 * cdef char *ufunc_mathieu_modsem2_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_modsem2(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_doc = __pyx_k_mathieu_modsem2_m_q_x_Odd_modifi;

  /* "scipy/special/_ufuncs.pyx":7194
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modsem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":7195
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modsem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modsem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":7196
 * ufunc_mathieu_modsem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modsem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7197
 * ufunc_mathieu_modsem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7198
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7199
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7200
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7201
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7202
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7203
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7204
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7205
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7206
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[0]) = ((void *)msm2_wrap);

  /* "scipy/special/_ufuncs.pyx":7207
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[1]) = ((void *)((char *)__pyx_k_mathieu_modsem2));

  /* "scipy/special/_ufuncs.pyx":7208
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[2]) = ((void *)msm2_wrap);

  /* "scipy/special/_ufuncs.pyx":7209
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]
 * ufunc_mathieu_modsem2_data[1] = &ufunc_mathieu_modsem2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[3]) = ((void *)((char *)__pyx_k_mathieu_modsem2));

  /* "scipy/special/_ufuncs.pyx":7210
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_data[1] = &ufunc_mathieu_modsem2_ptr[2*1]
 * mathieu_modsem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem2_loops, ufunc_mathieu_modsem2_data, ufunc_mathieu_modsem2_types, 2, 3, 2, 0, "mathieu_modsem2", ufunc_mathieu_modsem2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7211
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]
 * ufunc_mathieu_modsem2_data[1] = &ufunc_mathieu_modsem2_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_modsem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem2_loops, ufunc_mathieu_modsem2_data, ufunc_mathieu_modsem2_types, 2, 3, 2, 0, "mathieu_modsem2", ufunc_mathieu_modsem2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7212
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]
 * ufunc_mathieu_modsem2_data[1] = &ufunc_mathieu_modsem2_ptr[2*1]
 * mathieu_modsem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem2_loops, ufunc_mathieu_modsem2_data, ufunc_mathieu_modsem2_types, 2, 3, 2, 0, "mathieu_modsem2", ufunc_mathieu_modsem2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_sem_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types, 2, 3, 2, 0, __pyx_k_mathieu_modsem2, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_modsem2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7218
 * cdef void *ufunc_mathieu_sem_data[2]
 * cdef char ufunc_mathieu_sem_types[10]
 * cdef char *ufunc_mathieu_sem_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_sem(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_doc = __pyx_k_mathieu_sem_m_q_x_Odd_Mathieu_fu;

  /* "scipy/special/_ufuncs.pyx":7242
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_sem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":7243
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_sem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_sem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":7244
 * ufunc_mathieu_sem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_sem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7245
 * ufunc_mathieu_sem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7246
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7247
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7248
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7249
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7250
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7251
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7252
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7253
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7254
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[0]) = ((void *)sem_wrap);

  /* "scipy/special/_ufuncs.pyx":7255
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[1]) = ((void *)((char *)__pyx_k_mathieu_sem));

  /* "scipy/special/_ufuncs.pyx":7256
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[2]) = ((void *)sem_wrap);

  /* "scipy/special/_ufuncs.pyx":7257
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]
 * ufunc_mathieu_sem_data[1] = &ufunc_mathieu_sem_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[3]) = ((void *)((char *)__pyx_k_mathieu_sem));

  /* "scipy/special/_ufuncs.pyx":7258
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_data[1] = &ufunc_mathieu_sem_ptr[2*1]
 * mathieu_sem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_sem_loops, ufunc_mathieu_sem_data, ufunc_mathieu_sem_types, 2, 3, 2, 0, "mathieu_sem", ufunc_mathieu_sem_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7259
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]
 * ufunc_mathieu_sem_data[1] = &ufunc_mathieu_sem_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_sem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_sem_loops, ufunc_mathieu_sem_data, ufunc_mathieu_sem_types, 2, 3, 2, 0, "mathieu_sem", ufunc_mathieu_sem_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7260
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]
 * ufunc_mathieu_sem_data[1] = &ufunc_mathieu_sem_ptr[2*1]
 * mathieu_sem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_sem_loops, ufunc_mathieu_sem_data, ufunc_mathieu_sem_types, 2, 3, 2, 0, "mathieu_sem", ufunc_mathieu_sem_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_modfresnelm_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types, 2, 3, 2, 0, __pyx_k_mathieu_sem, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_sem, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7266
 * cdef void *ufunc_modfresnelm_data[2]
 * cdef char ufunc_modfresnelm_types[6]
 * cdef char *ufunc_modfresnelm_doc = (             # <<<<<<<<<<<<<<
 *     "modfresnelm(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_doc = __pyx_k_modfresnelm_x_Modified_Fresnel_n;

  /* "scipy/special/_ufuncs.pyx":7277
 *     "km\n"
 *     "    Integral ``K_-(x)``: ``1/sqrt(pi)*exp(1j*(x*x+pi/4))*fp``")
 * ufunc_modfresnelm_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_f_FF);

  /* "scipy/special/_ufuncs.pyx":7278
 *     "    Integral ``K_-(x)``: ``1/sqrt(pi)*exp(1j*(x*x+pi/4))*fp``")
 * ufunc_modfresnelm_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF
 * ufunc_modfresnelm_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_d_DD);

  /* "scipy/special/_ufuncs.pyx":7279
 * ufunc_modfresnelm_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF
 * ufunc_modfresnelm_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7280
 * ufunc_modfresnelm_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7281
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7282
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7283
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7284
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7285
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[0]) = ((void *)modified_fresnel_minus_wrap);

  /* "scipy/special/_ufuncs.pyx":7286
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[1]) = ((void *)((char *)__pyx_k_modfresnelm));

  /* "scipy/special/_ufuncs.pyx":7287
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[2]) = ((void *)modified_fresnel_minus_wrap);

  /* "scipy/special/_ufuncs.pyx":7288
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]
 * ufunc_modfresnelm_data[1] = &ufunc_modfresnelm_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[3]) = ((void *)((char *)__pyx_k_modfresnelm));

  /* "scipy/special/_ufuncs.pyx":7289
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_data[1] = &ufunc_modfresnelm_ptr[2*1]
 * modfresnelm = np.PyUFunc_FromFuncAndData(ufunc_modfresnelm_loops, ufunc_modfresnelm_data, ufunc_modfresnelm_types, 2, 1, 2, 0, "modfresnelm", ufunc_modfresnelm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7290
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]
 * ufunc_modfresnelm_data[1] = &ufunc_modfresnelm_ptr[2*1]             # <<<<<<<<<<<<<<
 * modfresnelm = np.PyUFunc_FromFuncAndData(ufunc_modfresnelm_loops, ufunc_modfresnelm_data, ufunc_modfresnelm_types, 2, 1, 2, 0, "modfresnelm", ufunc_modfresnelm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7291
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]
 * ufunc_modfresnelm_data[1] = &ufunc_modfresnelm_ptr[2*1]
 * modfresnelm = np.PyUFunc_FromFuncAndData(ufunc_modfresnelm_loops, ufunc_modfresnelm_data, ufunc_modfresnelm_types, 2, 1, 2, 0, "modfresnelm", ufunc_modfresnelm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_modfresnelp_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types, 2, 1, 2, 0, __pyx_k_modfresnelm, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_modfresnelm, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7297
 * cdef void *ufunc_modfresnelp_data[2]
 * cdef char ufunc_modfresnelp_types[6]
 * cdef char *ufunc_modfresnelp_doc = (             # <<<<<<<<<<<<<<
 *     "modfresnelp(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_doc = __pyx_k_modfresnelp_x_Modified_Fresnel_p;

  /* "scipy/special/_ufuncs.pyx":7308
 *     "kp\n"
 *     "    Integral ``K_+(x)``: ``1/sqrt(pi)*exp(-1j*(x*x+pi/4))*fp``")
 * ufunc_modfresnelp_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_f_FF);

  /* "scipy/special/_ufuncs.pyx":7309
 *     "    Integral ``K_+(x)``: ``1/sqrt(pi)*exp(-1j*(x*x+pi/4))*fp``")
 * ufunc_modfresnelp_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF
 * ufunc_modfresnelp_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_d_DD);

  /* "scipy/special/_ufuncs.pyx":7310
 * ufunc_modfresnelp_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF
 * ufunc_modfresnelp_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7311
 * ufunc_modfresnelp_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7312
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7313
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7314
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7315
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7316
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[0]) = ((void *)modified_fresnel_plus_wrap);

  /* "scipy/special/_ufuncs.pyx":7317
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[1]) = ((void *)((char *)__pyx_k_modfresnelp));

  /* "scipy/special/_ufuncs.pyx":7318
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[2]) = ((void *)modified_fresnel_plus_wrap);

  /* "scipy/special/_ufuncs.pyx":7319
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]
 * ufunc_modfresnelp_data[1] = &ufunc_modfresnelp_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[3]) = ((void *)((char *)__pyx_k_modfresnelp));

  /* "scipy/special/_ufuncs.pyx":7320
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_data[1] = &ufunc_modfresnelp_ptr[2*1]
 * modfresnelp = np.PyUFunc_FromFuncAndData(ufunc_modfresnelp_loops, ufunc_modfresnelp_data, ufunc_modfresnelp_types, 2, 1, 2, 0, "modfresnelp", ufunc_modfresnelp_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7321
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]
 * ufunc_modfresnelp_data[1] = &ufunc_modfresnelp_ptr[2*1]             # <<<<<<<<<<<<<<
 * modfresnelp = np.PyUFunc_FromFuncAndData(ufunc_modfresnelp_loops, ufunc_modfresnelp_data, ufunc_modfresnelp_types, 2, 1, 2, 0, "modfresnelp", ufunc_modfresnelp_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7322
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]
 * ufunc_modfresnelp_data[1] = &ufunc_modfresnelp_ptr[2*1]
 * modfresnelp = np.PyUFunc_FromFuncAndData(ufunc_modfresnelp_loops, ufunc_modfresnelp_data, ufunc_modfresnelp_types, 2, 1, 2, 0, "modfresnelp", ufunc_modfresnelp_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_modstruve_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types, 2, 1, 2, 0, __pyx_k_modfresnelp, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_modfresnelp, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7328
 * cdef void *ufunc_modstruve_data[2]
 * cdef char ufunc_modstruve_types[6]
 * cdef char *ufunc_modstruve_doc = (             # <<<<<<<<<<<<<<
 *     "modstruve(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_doc = __pyx_k_modstruve_v_x_Modified_Struve_fu;

  /* "scipy/special/_ufuncs.pyx":7335
 *     "Returns the modified Struve function Lv(x) of order `v` at `x`, `x` must\n"
 *     "be positive unless `v` is an integer.")
 * ufunc_modstruve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_modstruve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":7336
 *     "be positive unless `v` is an integer.")
 * ufunc_modstruve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_modstruve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":7337
 * ufunc_modstruve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_modstruve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7338
 * ufunc_modstruve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7339
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7340
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7341
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7342
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7343
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l             # <<<<<<<<<<<<<<
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[0]) = ((void *)struve_l);

  /* "scipy/special/_ufuncs.pyx":7344
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")             # <<<<<<<<<<<<<<
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[1]) = ((void *)((char *)__pyx_k_modstruve));

  /* "scipy/special/_ufuncs.pyx":7345
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l             # <<<<<<<<<<<<<<
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[2]) = ((void *)struve_l);

  /* "scipy/special/_ufuncs.pyx":7346
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")             # <<<<<<<<<<<<<<
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]
 * ufunc_modstruve_data[1] = &ufunc_modstruve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[3]) = ((void *)((char *)__pyx_k_modstruve));

  /* "scipy/special/_ufuncs.pyx":7347
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_modstruve_data[1] = &ufunc_modstruve_ptr[2*1]
 * modstruve = np.PyUFunc_FromFuncAndData(ufunc_modstruve_loops, ufunc_modstruve_data, ufunc_modstruve_types, 2, 2, 1, 0, "modstruve", ufunc_modstruve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7348
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]
 * ufunc_modstruve_data[1] = &ufunc_modstruve_ptr[2*1]             # <<<<<<<<<<<<<<
 * modstruve = np.PyUFunc_FromFuncAndData(ufunc_modstruve_loops, ufunc_modstruve_data, ufunc_modstruve_types, 2, 2, 1, 0, "modstruve", ufunc_modstruve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7349
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]
 * ufunc_modstruve_data[1] = &ufunc_modstruve_ptr[2*1]
 * modstruve = np.PyUFunc_FromFuncAndData(ufunc_modstruve_loops, ufunc_modstruve_data, ufunc_modstruve_types, 2, 2, 1, 0, "modstruve", ufunc_modstruve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtr_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types, 2, 2, 1, 0, __pyx_k_modstruve, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_modstruve, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7355
 * cdef void *ufunc_nbdtr_data[3]
 * cdef char ufunc_nbdtr_types[12]
 * cdef char *ufunc_nbdtr_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtr(k, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_doc = __pyx_k_nbdtr_k_n_p_Negative_binomial_cu;

  /* "scipy/special/_ufuncs.pyx":7367
 *     "In a sequence of Bernoulli trials this is the probability that k\n"
 *     "or fewer failures precede the nth success.")
 * ufunc_nbdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":7368
 *     "or fewer failures precede the nth success.")
 * ufunc_nbdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtr_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7369
 * ufunc_nbdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[0] = <char>NPY_LONG
 * ufunc_nbdtr_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7370
 * ufunc_nbdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtr_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[1] = <char>NPY_LONG
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":7371
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtr_types[0] = <char>NPY_LONG
 * ufunc_nbdtr_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":7372
 * ufunc_nbdtr_types[0] = <char>NPY_LONG
 * ufunc_nbdtr_types[1] = <char>NPY_LONG
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7373
 * ufunc_nbdtr_types[1] = <char>NPY_LONG
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7374
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7375
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7376
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7377
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7378
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7379
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7380
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7381
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7382
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[0]) = ((void *)nbdtr);

  /* "scipy/special/_ufuncs.pyx":7383
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[1]) = ((void *)((char *)__pyx_k_nbdtr));

  /* "scipy/special/_ufuncs.pyx":7384
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":7385
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[3]) = ((void *)((char *)__pyx_k_nbdtr));

  /* "scipy/special/_ufuncs.pyx":7386
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":7387
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[5]) = ((void *)((char *)__pyx_k_nbdtr));

  /* "scipy/special/_ufuncs.pyx":7388
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]
 * ufunc_nbdtr_data[2] = &ufunc_nbdtr_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7389
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_data[2] = &ufunc_nbdtr_ptr[2*2]
 * nbdtr = np.PyUFunc_FromFuncAndData(ufunc_nbdtr_loops, ufunc_nbdtr_data, ufunc_nbdtr_types, 3, 3, 1, 0, "nbdtr", ufunc_nbdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7390
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]
 * ufunc_nbdtr_data[2] = &ufunc_nbdtr_ptr[2*2]             # <<<<<<<<<<<<<<
 * nbdtr = np.PyUFunc_FromFuncAndData(ufunc_nbdtr_loops, ufunc_nbdtr_data, ufunc_nbdtr_types, 3, 3, 1, 0, "nbdtr", ufunc_nbdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":7391
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]
 * ufunc_nbdtr_data[2] = &ufunc_nbdtr_ptr[2*2]
 * nbdtr = np.PyUFunc_FromFuncAndData(ufunc_nbdtr_loops, ufunc_nbdtr_data, ufunc_nbdtr_types, 3, 3, 1, 0, "nbdtr", ufunc_nbdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtrc_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types, 3, 3, 1, 0, __pyx_k_nbdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7391; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7391; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7397
 * cdef void *ufunc_nbdtrc_data[3]
 * cdef char ufunc_nbdtrc_types[12]
 * cdef char *ufunc_nbdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtrc(k, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_doc = __pyx_k_nbdtrc_k_n_p_Negative_binomial_s;

  /* "scipy/special/_ufuncs.pyx":7404
 *     "Returns the sum of the terms k+1 to infinity of the negative\n"
 *     "binomial distribution.")
 * ufunc_nbdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":7405
 *     "binomial distribution.")
 * ufunc_nbdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7406
 * ufunc_nbdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7407
 * ufunc_nbdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":7408
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":7409
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7410
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7411
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7412
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7413
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7414
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7415
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7416
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7417
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7418
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7419
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[0]) = ((void *)nbdtrc);

  /* "scipy/special/_ufuncs.pyx":7420
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[1]) = ((void *)((char *)__pyx_k_nbdtrc));

  /* "scipy/special/_ufuncs.pyx":7421
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":7422
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[3]) = ((void *)((char *)__pyx_k_nbdtrc));

  /* "scipy/special/_ufuncs.pyx":7423
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":7424
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[5]) = ((void *)((char *)__pyx_k_nbdtrc));

  /* "scipy/special/_ufuncs.pyx":7425
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]
 * ufunc_nbdtrc_data[2] = &ufunc_nbdtrc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7426
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_data[2] = &ufunc_nbdtrc_ptr[2*2]
 * nbdtrc = np.PyUFunc_FromFuncAndData(ufunc_nbdtrc_loops, ufunc_nbdtrc_data, ufunc_nbdtrc_types, 3, 3, 1, 0, "nbdtrc", ufunc_nbdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7427
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]
 * ufunc_nbdtrc_data[2] = &ufunc_nbdtrc_ptr[2*2]             # <<<<<<<<<<<<<<
 * nbdtrc = np.PyUFunc_FromFuncAndData(ufunc_nbdtrc_loops, ufunc_nbdtrc_data, ufunc_nbdtrc_types, 3, 3, 1, 0, "nbdtrc", ufunc_nbdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":7428
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]
 * ufunc_nbdtrc_data[2] = &ufunc_nbdtrc_ptr[2*2]
 * nbdtrc = np.PyUFunc_FromFuncAndData(ufunc_nbdtrc_loops, ufunc_nbdtrc_data, ufunc_nbdtrc_types, 3, 3, 1, 0, "nbdtrc", ufunc_nbdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtri_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types, 3, 3, 1, 0, __pyx_k_nbdtrc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7428; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtrc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7428; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7434
 * cdef void *ufunc_nbdtri_data[3]
 * cdef char ufunc_nbdtri_types[12]
 * cdef char *ufunc_nbdtri_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtri(k, n, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_doc = __pyx_k_nbdtri_k_n_y_Inverse_of_nbdtr_vs;

  /* "scipy/special/_ufuncs.pyx":7440
 *     "\n"
 *     "Finds the argument p such that ``nbdtr(k, n, p) = y``.")
 * ufunc_nbdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":7441
 *     "Finds the argument p such that ``nbdtr(k, n, p) = y``.")
 * ufunc_nbdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtri_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7442
 * ufunc_nbdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[0] = <char>NPY_LONG
 * ufunc_nbdtri_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7443
 * ufunc_nbdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtri_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[1] = <char>NPY_LONG
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":7444
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtri_types[0] = <char>NPY_LONG
 * ufunc_nbdtri_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":7445
 * ufunc_nbdtri_types[0] = <char>NPY_LONG
 * ufunc_nbdtri_types[1] = <char>NPY_LONG
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7446
 * ufunc_nbdtri_types[1] = <char>NPY_LONG
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7447
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7448
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7449
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7450
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7451
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7452
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7453
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7454
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7455
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[0]) = ((void *)nbdtri);

  /* "scipy/special/_ufuncs.pyx":7456
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[1]) = ((void *)((char *)__pyx_k_nbdtri));

  /* "scipy/special/_ufuncs.pyx":7457
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":7458
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[3]) = ((void *)((char *)__pyx_k_nbdtri));

  /* "scipy/special/_ufuncs.pyx":7459
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":7460
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[5]) = ((void *)((char *)__pyx_k_nbdtri));

  /* "scipy/special/_ufuncs.pyx":7461
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]
 * ufunc_nbdtri_data[2] = &ufunc_nbdtri_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7462
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_data[2] = &ufunc_nbdtri_ptr[2*2]
 * nbdtri = np.PyUFunc_FromFuncAndData(ufunc_nbdtri_loops, ufunc_nbdtri_data, ufunc_nbdtri_types, 3, 3, 1, 0, "nbdtri", ufunc_nbdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7463
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]
 * ufunc_nbdtri_data[2] = &ufunc_nbdtri_ptr[2*2]             # <<<<<<<<<<<<<<
 * nbdtri = np.PyUFunc_FromFuncAndData(ufunc_nbdtri_loops, ufunc_nbdtri_data, ufunc_nbdtri_types, 3, 3, 1, 0, "nbdtri", ufunc_nbdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":7464
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]
 * ufunc_nbdtri_data[2] = &ufunc_nbdtri_ptr[2*2]
 * nbdtri = np.PyUFunc_FromFuncAndData(ufunc_nbdtri_loops, ufunc_nbdtri_data, ufunc_nbdtri_types, 3, 3, 1, 0, "nbdtri", ufunc_nbdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtrik_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types, 3, 3, 1, 0, __pyx_k_nbdtri, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtri, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7470
 * cdef void *ufunc_nbdtrik_data[2]
 * cdef char ufunc_nbdtrik_types[8]
 * cdef char *ufunc_nbdtrik_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtrik(y, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_doc = __pyx_k_nbdtrik_y_n_p_Inverse_of_nbdtr_v;

  /* "scipy/special/_ufuncs.pyx":7476
 *     "\n"
 *     "Finds the argument k such that ``nbdtr(k, n, p) = y``.")
 * ufunc_nbdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7477
 *     "Finds the argument k such that ``nbdtr(k, n, p) = y``.")
 * ufunc_nbdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7478
 * ufunc_nbdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7479
 * ufunc_nbdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7480
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7481
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7482
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7483
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7484
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7485
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7486
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[0]) = ((void *)cdfnbn2_wrap);

  /* "scipy/special/_ufuncs.pyx":7487
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[1]) = ((void *)((char *)__pyx_k_nbdtrik));

  /* "scipy/special/_ufuncs.pyx":7488
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[2]) = ((void *)cdfnbn2_wrap);

  /* "scipy/special/_ufuncs.pyx":7489
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]
 * ufunc_nbdtrik_data[1] = &ufunc_nbdtrik_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[3]) = ((void *)((char *)__pyx_k_nbdtrik));

  /* "scipy/special/_ufuncs.pyx":7490
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_data[1] = &ufunc_nbdtrik_ptr[2*1]
 * nbdtrik = np.PyUFunc_FromFuncAndData(ufunc_nbdtrik_loops, ufunc_nbdtrik_data, ufunc_nbdtrik_types, 2, 3, 1, 0, "nbdtrik", ufunc_nbdtrik_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7491
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]
 * ufunc_nbdtrik_data[1] = &ufunc_nbdtrik_ptr[2*1]             # <<<<<<<<<<<<<<
 * nbdtrik = np.PyUFunc_FromFuncAndData(ufunc_nbdtrik_loops, ufunc_nbdtrik_data, ufunc_nbdtrik_types, 2, 3, 1, 0, "nbdtrik", ufunc_nbdtrik_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7492
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]
 * ufunc_nbdtrik_data[1] = &ufunc_nbdtrik_ptr[2*1]
 * nbdtrik = np.PyUFunc_FromFuncAndData(ufunc_nbdtrik_loops, ufunc_nbdtrik_data, ufunc_nbdtrik_types, 2, 3, 1, 0, "nbdtrik", ufunc_nbdtrik_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtrin_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types, 2, 3, 1, 0, __pyx_k_nbdtrik, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtrik, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7498
 * cdef void *ufunc_nbdtrin_data[2]
 * cdef char ufunc_nbdtrin_types[8]
 * cdef char *ufunc_nbdtrin_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtrin(k, y, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_doc = __pyx_k_nbdtrin_k_y_p_Inverse_of_nbdtr_v;

  /* "scipy/special/_ufuncs.pyx":7504
 *     "\n"
 *     "Finds the argument `n` such that ``nbdtr(k, n, p) = y``.")
 * ufunc_nbdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7505
 *     "Finds the argument `n` such that ``nbdtr(k, n, p) = y``.")
 * ufunc_nbdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7506
 * ufunc_nbdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7507
 * ufunc_nbdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7508
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7509
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7510
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7511
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7512
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7513
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7514
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[0]) = ((void *)cdfnbn3_wrap);

  /* "scipy/special/_ufuncs.pyx":7515
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[1]) = ((void *)((char *)__pyx_k_nbdtrin));

  /* "scipy/special/_ufuncs.pyx":7516
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[2]) = ((void *)cdfnbn3_wrap);

  /* "scipy/special/_ufuncs.pyx":7517
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]
 * ufunc_nbdtrin_data[1] = &ufunc_nbdtrin_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[3]) = ((void *)((char *)__pyx_k_nbdtrin));

  /* "scipy/special/_ufuncs.pyx":7518
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_data[1] = &ufunc_nbdtrin_ptr[2*1]
 * nbdtrin = np.PyUFunc_FromFuncAndData(ufunc_nbdtrin_loops, ufunc_nbdtrin_data, ufunc_nbdtrin_types, 2, 3, 1, 0, "nbdtrin", ufunc_nbdtrin_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7519
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]
 * ufunc_nbdtrin_data[1] = &ufunc_nbdtrin_ptr[2*1]             # <<<<<<<<<<<<<<
 * nbdtrin = np.PyUFunc_FromFuncAndData(ufunc_nbdtrin_loops, ufunc_nbdtrin_data, ufunc_nbdtrin_types, 2, 3, 1, 0, "nbdtrin", ufunc_nbdtrin_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7520
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]
 * ufunc_nbdtrin_data[1] = &ufunc_nbdtrin_ptr[2*1]
 * nbdtrin = np.PyUFunc_FromFuncAndData(ufunc_nbdtrin_loops, ufunc_nbdtrin_data, ufunc_nbdtrin_types, 2, 3, 1, 0, "nbdtrin", ufunc_nbdtrin_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types, 2, 3, 1, 0, __pyx_k_nbdtrin, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtrin, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7526
 * cdef void *ufunc_ncfdtr_data[2]
 * cdef char ufunc_ncfdtr_types[10]
 * cdef char *ufunc_ncfdtr_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtr(dfn, dfd, nc, f)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_doc = __pyx_k_ncfdtr_dfn_dfd_nc_f_Cumulative_d;

  /* "scipy/special/_ufuncs.pyx":7575
 *     ">>> ax.plot(x, ncf_special, 'r-')\n"
 *     ">>> plt.show()")
 * ufunc_ncfdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":7576
 *     ">>> plt.show()")
 * ufunc_ncfdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":7577
 * ufunc_ncfdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7578
 * ufunc_ncfdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7579
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7580
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7581
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7582
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7583
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7584
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7585
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7586
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7587
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[0]) = ((void *)cdffnc1_wrap);

  /* "scipy/special/_ufuncs.pyx":7588
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[1]) = ((void *)((char *)__pyx_k_ncfdtr));

  /* "scipy/special/_ufuncs.pyx":7589
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[2]) = ((void *)cdffnc1_wrap);

  /* "scipy/special/_ufuncs.pyx":7590
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]
 * ufunc_ncfdtr_data[1] = &ufunc_ncfdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[3]) = ((void *)((char *)__pyx_k_ncfdtr));

  /* "scipy/special/_ufuncs.pyx":7591
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_data[1] = &ufunc_ncfdtr_ptr[2*1]
 * ncfdtr = np.PyUFunc_FromFuncAndData(ufunc_ncfdtr_loops, ufunc_ncfdtr_data, ufunc_ncfdtr_types, 2, 4, 1, 0, "ncfdtr", ufunc_ncfdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7592
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]
 * ufunc_ncfdtr_data[1] = &ufunc_ncfdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtr = np.PyUFunc_FromFuncAndData(ufunc_ncfdtr_loops, ufunc_ncfdtr_data, ufunc_ncfdtr_types, 2, 4, 1, 0, "ncfdtr", ufunc_ncfdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7593
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]
 * ufunc_ncfdtr_data[1] = &ufunc_ncfdtr_ptr[2*1]
 * ncfdtr = np.PyUFunc_FromFuncAndData(ufunc_ncfdtr_loops, ufunc_ncfdtr_data, ufunc_ncfdtr_types, 2, 4, 1, 0, "ncfdtr", ufunc_ncfdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtri_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types, 2, 4, 1, 0, __pyx_k_ncfdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7599
 * cdef void *ufunc_ncfdtri_data[2]
 * cdef char ufunc_ncfdtri_types[10]
 * cdef char *ufunc_ncfdtri_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtri(p, dfn, dfd, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_doc = __pyx_k_ncfdtri_p_dfn_dfd_nc_Inverse_cum;

  /* "scipy/special/_ufuncs.pyx":7605
 *     "\n"
 *     "See `ncfdtr` for more details.")
 * ufunc_ncfdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":7606
 *     "See `ncfdtr` for more details.")
 * ufunc_ncfdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":7607
 * ufunc_ncfdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7608
 * ufunc_ncfdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7609
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7610
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7611
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7612
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7613
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7614
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7615
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7616
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7617
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[0]) = ((void *)cdffnc2_wrap);

  /* "scipy/special/_ufuncs.pyx":7618
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[1]) = ((void *)((char *)__pyx_k_ncfdtri));

  /* "scipy/special/_ufuncs.pyx":7619
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[2]) = ((void *)cdffnc2_wrap);

  /* "scipy/special/_ufuncs.pyx":7620
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]
 * ufunc_ncfdtri_data[1] = &ufunc_ncfdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[3]) = ((void *)((char *)__pyx_k_ncfdtri));

  /* "scipy/special/_ufuncs.pyx":7621
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_data[1] = &ufunc_ncfdtri_ptr[2*1]
 * ncfdtri = np.PyUFunc_FromFuncAndData(ufunc_ncfdtri_loops, ufunc_ncfdtri_data, ufunc_ncfdtri_types, 2, 4, 1, 0, "ncfdtri", ufunc_ncfdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7622
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]
 * ufunc_ncfdtri_data[1] = &ufunc_ncfdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtri = np.PyUFunc_FromFuncAndData(ufunc_ncfdtri_loops, ufunc_ncfdtri_data, ufunc_ncfdtri_types, 2, 4, 1, 0, "ncfdtri", ufunc_ncfdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7623
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]
 * ufunc_ncfdtri_data[1] = &ufunc_ncfdtri_ptr[2*1]
 * ncfdtri = np.PyUFunc_FromFuncAndData(ufunc_ncfdtri_loops, ufunc_ncfdtri_data, ufunc_ncfdtri_types, 2, 4, 1, 0, "ncfdtri", ufunc_ncfdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtridfd_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types, 2, 4, 1, 0, __pyx_k_ncfdtri, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtri, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7629
 * cdef void *ufunc_ncfdtridfd_data[2]
 * cdef char ufunc_ncfdtridfd_types[10]
 * cdef char *ufunc_ncfdtridfd_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtridfd(p, f, dfn, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_doc = __pyx_k_ncfdtridfd_p_f_dfn_nc_Calculate;

  /* "scipy/special/_ufuncs.pyx":7635
 *     "\n"
 *     "See `ncfdtr` for more details.")
 * ufunc_ncfdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":7636
 *     "See `ncfdtr` for more details.")
 * ufunc_ncfdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":7637
 * ufunc_ncfdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7638
 * ufunc_ncfdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7639
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7640
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7641
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7642
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7643
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7644
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7645
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7646
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7647
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[0]) = ((void *)cdffnc4_wrap);

  /* "scipy/special/_ufuncs.pyx":7648
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[1]) = ((void *)((char *)__pyx_k_ncfdtridfd));

  /* "scipy/special/_ufuncs.pyx":7649
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[2]) = ((void *)cdffnc4_wrap);

  /* "scipy/special/_ufuncs.pyx":7650
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]
 * ufunc_ncfdtridfd_data[1] = &ufunc_ncfdtridfd_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[3]) = ((void *)((char *)__pyx_k_ncfdtridfd));

  /* "scipy/special/_ufuncs.pyx":7651
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_data[1] = &ufunc_ncfdtridfd_ptr[2*1]
 * ncfdtridfd = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfd_loops, ufunc_ncfdtridfd_data, ufunc_ncfdtridfd_types, 2, 4, 1, 0, "ncfdtridfd", ufunc_ncfdtridfd_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7652
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]
 * ufunc_ncfdtridfd_data[1] = &ufunc_ncfdtridfd_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtridfd = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfd_loops, ufunc_ncfdtridfd_data, ufunc_ncfdtridfd_types, 2, 4, 1, 0, "ncfdtridfd", ufunc_ncfdtridfd_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7653
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]
 * ufunc_ncfdtridfd_data[1] = &ufunc_ncfdtridfd_ptr[2*1]
 * ncfdtridfd = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfd_loops, ufunc_ncfdtridfd_data, ufunc_ncfdtridfd_types, 2, 4, 1, 0, "ncfdtridfd", ufunc_ncfdtridfd_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtridfn_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types, 2, 4, 1, 0, __pyx_k_ncfdtridfd, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7653; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtridfd, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7653; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7659
 * cdef void *ufunc_ncfdtridfn_data[2]
 * cdef char ufunc_ncfdtridfn_types[10]
 * cdef char *ufunc_ncfdtridfn_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtridfn(p, f, dfd, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_doc = __pyx_k_ncfdtridfn_p_f_dfd_nc_Calculate;

  /* "scipy/special/_ufuncs.pyx":7665
 *     "\n"
 *     "See `ncfdtr` for more details.")
 * ufunc_ncfdtridfn_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":7666
 *     "See `ncfdtr` for more details.")
 * ufunc_ncfdtridfn_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtridfn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":7667
 * ufunc_ncfdtridfn_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtridfn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7668
 * ufunc_ncfdtridfn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7669
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7670
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7671
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7672
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7673
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7674
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7675
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7676
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7677
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[0]) = ((void *)cdffnc3_wrap);

  /* "scipy/special/_ufuncs.pyx":7678
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[1]) = ((void *)((char *)__pyx_k_ncfdtridfn));

  /* "scipy/special/_ufuncs.pyx":7679
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[2]) = ((void *)cdffnc3_wrap);

  /* "scipy/special/_ufuncs.pyx":7680
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]
 * ufunc_ncfdtridfn_data[1] = &ufunc_ncfdtridfn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[3]) = ((void *)((char *)__pyx_k_ncfdtridfn));

  /* "scipy/special/_ufuncs.pyx":7681
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_data[1] = &ufunc_ncfdtridfn_ptr[2*1]
 * ncfdtridfn = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfn_loops, ufunc_ncfdtridfn_data, ufunc_ncfdtridfn_types, 2, 4, 1, 0, "ncfdtridfn", ufunc_ncfdtridfn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7682
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]
 * ufunc_ncfdtridfn_data[1] = &ufunc_ncfdtridfn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtridfn = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfn_loops, ufunc_ncfdtridfn_data, ufunc_ncfdtridfn_types, 2, 4, 1, 0, "ncfdtridfn", ufunc_ncfdtridfn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7683
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]
 * ufunc_ncfdtridfn_data[1] = &ufunc_ncfdtridfn_ptr[2*1]
 * ncfdtridfn = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfn_loops, ufunc_ncfdtridfn_data, ufunc_ncfdtridfn_types, 2, 4, 1, 0, "ncfdtridfn", ufunc_ncfdtridfn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtrinc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types, 2, 4, 1, 0, __pyx_k_ncfdtridfn, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7683; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtridfn, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7683; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7689
 * cdef void *ufunc_ncfdtrinc_data[2]
 * cdef char ufunc_ncfdtrinc_types[10]
 * cdef char *ufunc_ncfdtrinc_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtrinc(p, f, dfn, dfd)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_doc = __pyx_k_ncfdtrinc_p_f_dfn_dfd_Calculate;

  /* "scipy/special/_ufuncs.pyx":7695
 *     "\n"
 *     "See `ncfdtr` for more details.")
 * ufunc_ncfdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":7696
 *     "See `ncfdtr` for more details.")
 * ufunc_ncfdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":7697
 * ufunc_ncfdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7698
 * ufunc_ncfdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7699
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7700
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7701
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7702
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7703
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7704
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7705
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7706
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7707
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[0]) = ((void *)cdffnc5_wrap);

  /* "scipy/special/_ufuncs.pyx":7708
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[1]) = ((void *)((char *)__pyx_k_ncfdtrinc));

  /* "scipy/special/_ufuncs.pyx":7709
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[2]) = ((void *)cdffnc5_wrap);

  /* "scipy/special/_ufuncs.pyx":7710
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]
 * ufunc_ncfdtrinc_data[1] = &ufunc_ncfdtrinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[3]) = ((void *)((char *)__pyx_k_ncfdtrinc));

  /* "scipy/special/_ufuncs.pyx":7711
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_data[1] = &ufunc_ncfdtrinc_ptr[2*1]
 * ncfdtrinc = np.PyUFunc_FromFuncAndData(ufunc_ncfdtrinc_loops, ufunc_ncfdtrinc_data, ufunc_ncfdtrinc_types, 2, 4, 1, 0, "ncfdtrinc", ufunc_ncfdtrinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7712
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]
 * ufunc_ncfdtrinc_data[1] = &ufunc_ncfdtrinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtrinc = np.PyUFunc_FromFuncAndData(ufunc_ncfdtrinc_loops, ufunc_ncfdtrinc_data, ufunc_ncfdtrinc_types, 2, 4, 1, 0, "ncfdtrinc", ufunc_ncfdtrinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7713
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]
 * ufunc_ncfdtrinc_data[1] = &ufunc_ncfdtrinc_ptr[2*1]
 * ncfdtrinc = np.PyUFunc_FromFuncAndData(ufunc_ncfdtrinc_loops, ufunc_ncfdtrinc_data, ufunc_ncfdtrinc_types, 2, 4, 1, 0, "ncfdtrinc", ufunc_ncfdtrinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nctdtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types, 2, 4, 1, 0, __pyx_k_ncfdtrinc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtrinc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7719
 * cdef void *ufunc_nctdtr_data[2]
 * cdef char ufunc_nctdtr_types[8]
 * cdef char *ufunc_nctdtr_doc = (             # <<<<<<<<<<<<<<
 *     "nctdtr(df, nc, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_doc = __pyx_k_nctdtr_df_nc_t_Cumulative_distri;

  /* "scipy/special/_ufuncs.pyx":7764
 *     ">>> ax.plot(x, nct_special, 'r-')\n"
 *     ">>> plt.show()")
 * ufunc_nctdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7765
 *     ">>> plt.show()")
 * ufunc_nctdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7766
 * ufunc_nctdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7767
 * ufunc_nctdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7768
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7769
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7770
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7771
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7772
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7773
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7774
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[0]) = ((void *)cdftnc1_wrap);

  /* "scipy/special/_ufuncs.pyx":7775
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[1]) = ((void *)((char *)__pyx_k_nctdtr));

  /* "scipy/special/_ufuncs.pyx":7776
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[2]) = ((void *)cdftnc1_wrap);

  /* "scipy/special/_ufuncs.pyx":7777
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]
 * ufunc_nctdtr_data[1] = &ufunc_nctdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[3]) = ((void *)((char *)__pyx_k_nctdtr));

  /* "scipy/special/_ufuncs.pyx":7778
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_data[1] = &ufunc_nctdtr_ptr[2*1]
 * nctdtr = np.PyUFunc_FromFuncAndData(ufunc_nctdtr_loops, ufunc_nctdtr_data, ufunc_nctdtr_types, 2, 3, 1, 0, "nctdtr", ufunc_nctdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7779
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]
 * ufunc_nctdtr_data[1] = &ufunc_nctdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * nctdtr = np.PyUFunc_FromFuncAndData(ufunc_nctdtr_loops, ufunc_nctdtr_data, ufunc_nctdtr_types, 2, 3, 1, 0, "nctdtr", ufunc_nctdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7780
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]
 * ufunc_nctdtr_data[1] = &ufunc_nctdtr_ptr[2*1]
 * nctdtr = np.PyUFunc_FromFuncAndData(ufunc_nctdtr_loops, ufunc_nctdtr_data, ufunc_nctdtr_types, 2, 3, 1, 0, "nctdtr", ufunc_nctdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nctdtridf_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types, 2, 3, 1, 0, __pyx_k_nctdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7780; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nctdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7780; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7786
 * cdef void *ufunc_nctdtridf_data[2]
 * cdef char ufunc_nctdtridf_types[8]
 * cdef char *ufunc_nctdtridf_doc = (             # <<<<<<<<<<<<<<
 *     "nctdtridf(p, nc, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_doc = __pyx_k_nctdtridf_p_nc_t_Calculate_degre;

  /* "scipy/special/_ufuncs.pyx":7801
 *     "t : array_like\n"
 *     "    Quantiles, i.e. the upper limit of integration.")
 * ufunc_nctdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7802
 *     "    Quantiles, i.e. the upper limit of integration.")
 * ufunc_nctdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7803
 * ufunc_nctdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7804
 * ufunc_nctdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7805
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7806
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7807
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7808
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7809
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7810
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7811
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[0]) = ((void *)cdftnc3_wrap);

  /* "scipy/special/_ufuncs.pyx":7812
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[1]) = ((void *)((char *)__pyx_k_nctdtridf));

  /* "scipy/special/_ufuncs.pyx":7813
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[2]) = ((void *)cdftnc3_wrap);

  /* "scipy/special/_ufuncs.pyx":7814
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]
 * ufunc_nctdtridf_data[1] = &ufunc_nctdtridf_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[3]) = ((void *)((char *)__pyx_k_nctdtridf));

  /* "scipy/special/_ufuncs.pyx":7815
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_data[1] = &ufunc_nctdtridf_ptr[2*1]
 * nctdtridf = np.PyUFunc_FromFuncAndData(ufunc_nctdtridf_loops, ufunc_nctdtridf_data, ufunc_nctdtridf_types, 2, 3, 1, 0, "nctdtridf", ufunc_nctdtridf_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7816
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]
 * ufunc_nctdtridf_data[1] = &ufunc_nctdtridf_ptr[2*1]             # <<<<<<<<<<<<<<
 * nctdtridf = np.PyUFunc_FromFuncAndData(ufunc_nctdtridf_loops, ufunc_nctdtridf_data, ufunc_nctdtridf_types, 2, 3, 1, 0, "nctdtridf", ufunc_nctdtridf_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7817
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]
 * ufunc_nctdtridf_data[1] = &ufunc_nctdtridf_ptr[2*1]
 * nctdtridf = np.PyUFunc_FromFuncAndData(ufunc_nctdtridf_loops, ufunc_nctdtridf_data, ufunc_nctdtridf_types, 2, 3, 1, 0, "nctdtridf", ufunc_nctdtridf_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nctdtrinc_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types, 2, 3, 1, 0, __pyx_k_nctdtridf, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7817; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nctdtridf, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7817; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7823
 * cdef void *ufunc_nctdtrinc_data[2]
 * cdef char ufunc_nctdtrinc_types[8]
 * cdef char *ufunc_nctdtrinc_doc = (             # <<<<<<<<<<<<<<
 *     "nctdtrinc(df, p, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_doc = __pyx_k_nctdtrinc_df_p_t_Calculate_non_c;

  /* "scipy/special/_ufuncs.pyx":7838
 *     "t : array_like\n"
 *     "    Quantiles, i.e. the upper limit of integration.")
 * ufunc_nctdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7839
 *     "    Quantiles, i.e. the upper limit of integration.")
 * ufunc_nctdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7840
 * ufunc_nctdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7841
 * ufunc_nctdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7842
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7843
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7844
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7845
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7846
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7847
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7848
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[0]) = ((void *)cdftnc4_wrap);

  /* "scipy/special/_ufuncs.pyx":7849
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[1]) = ((void *)((char *)__pyx_k_nctdtrinc));

  /* "scipy/special/_ufuncs.pyx":7850
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[2]) = ((void *)cdftnc4_wrap);

  /* "scipy/special/_ufuncs.pyx":7851
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]
 * ufunc_nctdtrinc_data[1] = &ufunc_nctdtrinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[3]) = ((void *)((char *)__pyx_k_nctdtrinc));

  /* "scipy/special/_ufuncs.pyx":7852
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_data[1] = &ufunc_nctdtrinc_ptr[2*1]
 * nctdtrinc = np.PyUFunc_FromFuncAndData(ufunc_nctdtrinc_loops, ufunc_nctdtrinc_data, ufunc_nctdtrinc_types, 2, 3, 1, 0, "nctdtrinc", ufunc_nctdtrinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7853
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]
 * ufunc_nctdtrinc_data[1] = &ufunc_nctdtrinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * nctdtrinc = np.PyUFunc_FromFuncAndData(ufunc_nctdtrinc_loops, ufunc_nctdtrinc_data, ufunc_nctdtrinc_types, 2, 3, 1, 0, "nctdtrinc", ufunc_nctdtrinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7854
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]
 * ufunc_nctdtrinc_data[1] = &ufunc_nctdtrinc_ptr[2*1]
 * nctdtrinc = np.PyUFunc_FromFuncAndData(ufunc_nctdtrinc_loops, ufunc_nctdtrinc_data, ufunc_nctdtrinc_types, 2, 3, 1, 0, "nctdtrinc", ufunc_nctdtrinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nctdtrit_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types, 2, 3, 1, 0, __pyx_k_nctdtrinc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7854; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nctdtrinc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7854; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7860
 * cdef void *ufunc_nctdtrit_data[2]
 * cdef char ufunc_nctdtrit_types[8]
 * cdef char *ufunc_nctdtrit_doc = (             # <<<<<<<<<<<<<<
 *     "nctdtrit(df, nc, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_doc = __pyx_k_nctdtrit_df_nc_p_Inverse_cumulat;

  /* "scipy/special/_ufuncs.pyx":7875
 *     "p : array_like\n"
 *     "    CDF values, in range (0, 1].")
 * ufunc_nctdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7876
 *     "    CDF values, in range (0, 1].")
 * ufunc_nctdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7877
 * ufunc_nctdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7878
 * ufunc_nctdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7879
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7880
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7881
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7882
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7883
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7884
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7885
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[0]) = ((void *)cdftnc2_wrap);

  /* "scipy/special/_ufuncs.pyx":7886
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[1]) = ((void *)((char *)__pyx_k_nctdtrit));

  /* "scipy/special/_ufuncs.pyx":7887
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[2]) = ((void *)cdftnc2_wrap);

  /* "scipy/special/_ufuncs.pyx":7888
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]
 * ufunc_nctdtrit_data[1] = &ufunc_nctdtrit_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[3]) = ((void *)((char *)__pyx_k_nctdtrit));

  /* "scipy/special/_ufuncs.pyx":7889
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_data[1] = &ufunc_nctdtrit_ptr[2*1]
 * nctdtrit = np.PyUFunc_FromFuncAndData(ufunc_nctdtrit_loops, ufunc_nctdtrit_data, ufunc_nctdtrit_types, 2, 3, 1, 0, "nctdtrit", ufunc_nctdtrit_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7890
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]
 * ufunc_nctdtrit_data[1] = &ufunc_nctdtrit_ptr[2*1]             # <<<<<<<<<<<<<<
 * nctdtrit = np.PyUFunc_FromFuncAndData(ufunc_nctdtrit_loops, ufunc_nctdtrit_data, ufunc_nctdtrit_types, 2, 3, 1, 0, "nctdtrit", ufunc_nctdtrit_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7891
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]
 * ufunc_nctdtrit_data[1] = &ufunc_nctdtrit_ptr[2*1]
 * nctdtrit = np.PyUFunc_FromFuncAndData(ufunc_nctdtrit_loops, ufunc_nctdtrit_data, ufunc_nctdtrit_types, 2, 3, 1, 0, "nctdtrit", ufunc_nctdtrit_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ndtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types, 2, 3, 1, 0, __pyx_k_nctdtrit, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nctdtrit, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7897
 * cdef void *ufunc_ndtr_data[2]
 * cdef char ufunc_ndtr_types[4]
 * cdef char *ufunc_ndtr_doc = (             # <<<<<<<<<<<<<<
 *     "ndtr(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_doc = __pyx_k_ndtr_x_Gaussian_cumulative_distr;

  /* "scipy/special/_ufuncs.pyx":7906
 *     "\n"
 *     "    1/sqrt(2*pi) * integral(exp(-t**2 / 2), t=-inf..x)")
 * ufunc_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":7907
 *     "    1/sqrt(2*pi) * integral(exp(-t**2 / 2), t=-inf..x)")
 * ufunc_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":7908
 * ufunc_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7909
 * ufunc_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7910
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7911
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7912
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[0]) = ((void *)ndtr);

  /* "scipy/special/_ufuncs.pyx":7913
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[1]) = ((void *)((char *)__pyx_k_ndtr));

  /* "scipy/special/_ufuncs.pyx":7914
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[2]) = ((void *)ndtr);

  /* "scipy/special/_ufuncs.pyx":7915
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")             # <<<<<<<<<<<<<<
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[3]) = ((void *)((char *)__pyx_k_ndtr));

  /* "scipy/special/_ufuncs.pyx":7916
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]
 * ndtr = np.PyUFunc_FromFuncAndData(ufunc_ndtr_loops, ufunc_ndtr_data, ufunc_ndtr_types, 2, 1, 1, 0, "ndtr", ufunc_ndtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7917
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ndtr = np.PyUFunc_FromFuncAndData(ufunc_ndtr_loops, ufunc_ndtr_data, ufunc_ndtr_types, 2, 1, 1, 0, "ndtr", ufunc_ndtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7918
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]
 * ndtr = np.PyUFunc_FromFuncAndData(ufunc_ndtr_loops, ufunc_ndtr_data, ufunc_ndtr_types, 2, 1, 1, 0, "ndtr", ufunc_ndtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ndtri_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types, 2, 1, 1, 0, __pyx_k_ndtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ndtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7918; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7924
 * cdef void *ufunc_ndtri_data[2]
 * cdef char ufunc_ndtri_types[4]
 * cdef char *ufunc_ndtri_doc = (             # <<<<<<<<<<<<<<
 *     "ndtri(y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_doc = __pyx_k_ndtri_y_Inverse_of_ndtr_vs_x_Ret;

  /* "scipy/special/_ufuncs.pyx":7932
 *     "probability density function (integrated from minus infinity to `x`)\n"
 *     "is equal to y.")
 * ufunc_ndtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ndtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":7933
 *     "is equal to y.")
 * ufunc_ndtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ndtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":7934
 * ufunc_ndtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ndtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7935
 * ufunc_ndtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7936
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7937
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7938
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri             # <<<<<<<<<<<<<<
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[0]) = ((void *)ndtri);

  /* "scipy/special/_ufuncs.pyx":7939
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")             # <<<<<<<<<<<<<<
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[1]) = ((void *)((char *)__pyx_k_ndtri));

  /* "scipy/special/_ufuncs.pyx":7940
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri             # <<<<<<<<<<<<<<
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[2]) = ((void *)ndtri);

  /* "scipy/special/_ufuncs.pyx":7941
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")             # <<<<<<<<<<<<<<
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]
 * ufunc_ndtri_data[1] = &ufunc_ndtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[3]) = ((void *)((char *)__pyx_k_ndtri));

  /* "scipy/special/_ufuncs.pyx":7942
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ndtri_data[1] = &ufunc_ndtri_ptr[2*1]
 * ndtri = np.PyUFunc_FromFuncAndData(ufunc_ndtri_loops, ufunc_ndtri_data, ufunc_ndtri_types, 2, 1, 1, 0, "ndtri", ufunc_ndtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7943
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]
 * ufunc_ndtri_data[1] = &ufunc_ndtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ndtri = np.PyUFunc_FromFuncAndData(ufunc_ndtri_loops, ufunc_ndtri_data, ufunc_ndtri_types, 2, 1, 1, 0, "ndtri", ufunc_ndtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7944
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]
 * ufunc_ndtri_data[1] = &ufunc_ndtri_ptr[2*1]
 * ndtri = np.PyUFunc_FromFuncAndData(ufunc_ndtri_loops, ufunc_ndtri_data, ufunc_ndtri_types, 2, 1, 1, 0, "ndtri", ufunc_ndtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nrdtrimn_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types, 2, 1, 1, 0, __pyx_k_ndtri, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ndtri, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7950
 * cdef void *ufunc_nrdtrimn_data[2]
 * cdef char ufunc_nrdtrimn_types[8]
 * cdef char *ufunc_nrdtrimn_doc = (             # <<<<<<<<<<<<<<
 *     "nrdtrimn(p, x, std)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_doc = __pyx_k_nrdtrimn_p_x_std_Calculate_mean;

  /* "scipy/special/_ufuncs.pyx":7972
 *     "--------\n"
 *     "nrdtrimn, ndtr")
 * ufunc_nrdtrimn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7973
 *     "nrdtrimn, ndtr")
 * ufunc_nrdtrimn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nrdtrimn_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7974
 * ufunc_nrdtrimn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nrdtrimn_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7975
 * ufunc_nrdtrimn_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7976
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7977
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7978
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7979
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7980
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7981
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7982
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[0]) = ((void *)cdfnor3_wrap);

  /* "scipy/special/_ufuncs.pyx":7983
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[1]) = ((void *)((char *)__pyx_k_nrdtrimn));

  /* "scipy/special/_ufuncs.pyx":7984
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[2]) = ((void *)cdfnor3_wrap);

  /* "scipy/special/_ufuncs.pyx":7985
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]
 * ufunc_nrdtrimn_data[1] = &ufunc_nrdtrimn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[3]) = ((void *)((char *)__pyx_k_nrdtrimn));

  /* "scipy/special/_ufuncs.pyx":7986
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_data[1] = &ufunc_nrdtrimn_ptr[2*1]
 * nrdtrimn = np.PyUFunc_FromFuncAndData(ufunc_nrdtrimn_loops, ufunc_nrdtrimn_data, ufunc_nrdtrimn_types, 2, 3, 1, 0, "nrdtrimn", ufunc_nrdtrimn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7987
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]
 * ufunc_nrdtrimn_data[1] = &ufunc_nrdtrimn_ptr[2*1]             # <<<<<<<<<<<<<<
 * nrdtrimn = np.PyUFunc_FromFuncAndData(ufunc_nrdtrimn_loops, ufunc_nrdtrimn_data, ufunc_nrdtrimn_types, 2, 3, 1, 0, "nrdtrimn", ufunc_nrdtrimn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7988
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]
 * ufunc_nrdtrimn_data[1] = &ufunc_nrdtrimn_ptr[2*1]
 * nrdtrimn = np.PyUFunc_FromFuncAndData(ufunc_nrdtrimn_loops, ufunc_nrdtrimn_data, ufunc_nrdtrimn_types, 2, 3, 1, 0, "nrdtrimn", ufunc_nrdtrimn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nrdtrisd_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types, 2, 3, 1, 0, __pyx_k_nrdtrimn, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7988; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nrdtrimn, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7988; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":7994
 * cdef void *ufunc_nrdtrisd_data[2]
 * cdef char ufunc_nrdtrisd_types[8]
 * cdef char *ufunc_nrdtrisd_doc = (             # <<<<<<<<<<<<<<
 *     "nrdtrisd(p, x, mn)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_doc = __pyx_k_nrdtrisd_p_x_mn_Calculate_standa;

  /* "scipy/special/_ufuncs.pyx":8016
 *     "--------\n"
 *     "nrdtristd, ndtr")
 * ufunc_nrdtrisd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":8017
 *     "nrdtristd, ndtr")
 * ufunc_nrdtrisd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nrdtrisd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":8018
 * ufunc_nrdtrisd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nrdtrisd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8019
 * ufunc_nrdtrisd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8020
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8021
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8022
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8023
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8024
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8025
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8026
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[0]) = ((void *)cdfnor4_wrap);

  /* "scipy/special/_ufuncs.pyx":8027
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[1]) = ((void *)((char *)__pyx_k_nrdtrisd));

  /* "scipy/special/_ufuncs.pyx":8028
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[2]) = ((void *)cdfnor4_wrap);

  /* "scipy/special/_ufuncs.pyx":8029
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]
 * ufunc_nrdtrisd_data[1] = &ufunc_nrdtrisd_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[3]) = ((void *)((char *)__pyx_k_nrdtrisd));

  /* "scipy/special/_ufuncs.pyx":8030
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_data[1] = &ufunc_nrdtrisd_ptr[2*1]
 * nrdtrisd = np.PyUFunc_FromFuncAndData(ufunc_nrdtrisd_loops, ufunc_nrdtrisd_data, ufunc_nrdtrisd_types, 2, 3, 1, 0, "nrdtrisd", ufunc_nrdtrisd_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8031
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]
 * ufunc_nrdtrisd_data[1] = &ufunc_nrdtrisd_ptr[2*1]             # <<<<<<<<<<<<<<
 * nrdtrisd = np.PyUFunc_FromFuncAndData(ufunc_nrdtrisd_loops, ufunc_nrdtrisd_data, ufunc_nrdtrisd_types, 2, 3, 1, 0, "nrdtrisd", ufunc_nrdtrisd_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8032
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]
 * ufunc_nrdtrisd_data[1] = &ufunc_nrdtrisd_ptr[2*1]
 * nrdtrisd = np.PyUFunc_FromFuncAndData(ufunc_nrdtrisd_loops, ufunc_nrdtrisd_data, ufunc_nrdtrisd_types, 2, 3, 1, 0, "nrdtrisd", ufunc_nrdtrisd_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_ang1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types, 2, 3, 1, 0, __pyx_k_nrdtrisd, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nrdtrisd, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8038
 * cdef void *ufunc_obl_ang1_data[2]
 * cdef char ufunc_obl_ang1_types[12]
 * cdef char *ufunc_obl_ang1_doc = (             # <<<<<<<<<<<<<<
 *     "obl_ang1(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_doc = __pyx_k_obl_ang1_m_n_c_x_Oblate_spheroid;

  /* "scipy/special/_ufuncs.pyx":8053
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":8054
 *     "    Value of the derivative vs x")
 * ufunc_obl_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":8055
 * ufunc_obl_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8056
 * ufunc_obl_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8057
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8058
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8059
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8060
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8061
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8062
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8063
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8064
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8065
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8066
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8067
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[0]) = ((void *)oblate_aswfa_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8068
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[1]) = ((void *)((char *)__pyx_k_obl_ang1));

  /* "scipy/special/_ufuncs.pyx":8069
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[2]) = ((void *)oblate_aswfa_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8070
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]
 * ufunc_obl_ang1_data[1] = &ufunc_obl_ang1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[3]) = ((void *)((char *)__pyx_k_obl_ang1));

  /* "scipy/special/_ufuncs.pyx":8071
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_data[1] = &ufunc_obl_ang1_ptr[2*1]
 * obl_ang1 = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_loops, ufunc_obl_ang1_data, ufunc_obl_ang1_types, 2, 4, 2, 0, "obl_ang1", ufunc_obl_ang1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8072
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]
 * ufunc_obl_ang1_data[1] = &ufunc_obl_ang1_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_ang1 = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_loops, ufunc_obl_ang1_data, ufunc_obl_ang1_types, 2, 4, 2, 0, "obl_ang1", ufunc_obl_ang1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8073
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]
 * ufunc_obl_ang1_data[1] = &ufunc_obl_ang1_ptr[2*1]
 * obl_ang1 = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_loops, ufunc_obl_ang1_data, ufunc_obl_ang1_types, 2, 4, 2, 0, "obl_ang1", ufunc_obl_ang1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_ang1_cv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types, 2, 4, 2, 0, __pyx_k_obl_ang1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_ang1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8079
 * cdef void *ufunc_obl_ang1_cv_data[2]
 * cdef char ufunc_obl_ang1_cv_types[14]
 * cdef char *ufunc_obl_ang1_cv_doc = (             # <<<<<<<<<<<<<<
 *     "obl_ang1_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_doc = __pyx_k_obl_ang1_cv_m_n_c_cv_x_Oblate_sp;

  /* "scipy/special/_ufuncs.pyx":8095
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":8096
 *     "    Value of the derivative vs x")
 * ufunc_obl_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":8097
 * ufunc_obl_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8098
 * ufunc_obl_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8099
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8100
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8101
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8102
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8103
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8104
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8105
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8106
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8107
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8108
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8109
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8110
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8111
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[0]) = ((void *)oblate_aswfa_wrap);

  /* "scipy/special/_ufuncs.pyx":8112
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[1]) = ((void *)((char *)__pyx_k_obl_ang1_cv));

  /* "scipy/special/_ufuncs.pyx":8113
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[2]) = ((void *)oblate_aswfa_wrap);

  /* "scipy/special/_ufuncs.pyx":8114
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]
 * ufunc_obl_ang1_cv_data[1] = &ufunc_obl_ang1_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[3]) = ((void *)((char *)__pyx_k_obl_ang1_cv));

  /* "scipy/special/_ufuncs.pyx":8115
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_data[1] = &ufunc_obl_ang1_cv_ptr[2*1]
 * obl_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_cv_loops, ufunc_obl_ang1_cv_data, ufunc_obl_ang1_cv_types, 2, 5, 2, 0, "obl_ang1_cv", ufunc_obl_ang1_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8116
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]
 * ufunc_obl_ang1_cv_data[1] = &ufunc_obl_ang1_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_cv_loops, ufunc_obl_ang1_cv_data, ufunc_obl_ang1_cv_types, 2, 5, 2, 0, "obl_ang1_cv", ufunc_obl_ang1_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8117
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]
 * ufunc_obl_ang1_cv_data[1] = &ufunc_obl_ang1_cv_ptr[2*1]
 * obl_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_cv_loops, ufunc_obl_ang1_cv_data, ufunc_obl_ang1_cv_types, 2, 5, 2, 0, "obl_ang1_cv", ufunc_obl_ang1_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_cv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types, 2, 5, 2, 0, __pyx_k_obl_ang1_cv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_ang1_cv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8123
 * cdef void *ufunc_obl_cv_data[2]
 * cdef char ufunc_obl_cv_types[8]
 * cdef char *ufunc_obl_cv_doc = (             # <<<<<<<<<<<<<<
 *     "obl_cv(m, n, c)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_doc = __pyx_k_obl_cv_m_n_c_Characteristic_valu;

  /* "scipy/special/_ufuncs.pyx":8130
 *     "Computes the characteristic value of oblate spheroidal wave\n"
 *     "functions of order `m`, `n` (n>=m) and spheroidal parameter `c`.")
 * ufunc_obl_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":8131
 *     "functions of order `m`, `n` (n>=m) and spheroidal parameter `c`.")
 * ufunc_obl_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_obl_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":8132
 * ufunc_obl_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_obl_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8133
 * ufunc_obl_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8134
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8135
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8136
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8137
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8138
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8139
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8140
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[0]) = ((void *)oblate_segv_wrap);

  /* "scipy/special/_ufuncs.pyx":8141
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[1]) = ((void *)((char *)__pyx_k_obl_cv));

  /* "scipy/special/_ufuncs.pyx":8142
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[2]) = ((void *)oblate_segv_wrap);

  /* "scipy/special/_ufuncs.pyx":8143
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]
 * ufunc_obl_cv_data[1] = &ufunc_obl_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[3]) = ((void *)((char *)__pyx_k_obl_cv));

  /* "scipy/special/_ufuncs.pyx":8144
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_data[1] = &ufunc_obl_cv_ptr[2*1]
 * obl_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_cv_loops, ufunc_obl_cv_data, ufunc_obl_cv_types, 2, 3, 1, 0, "obl_cv", ufunc_obl_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8145
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]
 * ufunc_obl_cv_data[1] = &ufunc_obl_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_cv_loops, ufunc_obl_cv_data, ufunc_obl_cv_types, 2, 3, 1, 0, "obl_cv", ufunc_obl_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8146
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]
 * ufunc_obl_cv_data[1] = &ufunc_obl_cv_ptr[2*1]
 * obl_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_cv_loops, ufunc_obl_cv_data, ufunc_obl_cv_types, 2, 3, 1, 0, "obl_cv", ufunc_obl_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_rad1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types, 2, 3, 1, 0, __pyx_k_obl_cv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8146; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_cv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8146; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8152
 * cdef void *ufunc_obl_rad1_data[2]
 * cdef char ufunc_obl_rad1_types[12]
 * cdef char *ufunc_obl_rad1_doc = (             # <<<<<<<<<<<<<<
 *     "obl_rad1(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_doc = __pyx_k_obl_rad1_m_n_c_x_Oblate_spheroid;

  /* "scipy/special/_ufuncs.pyx":8167
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":8168
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":8169
 * ufunc_obl_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8170
 * ufunc_obl_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8171
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8172
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8173
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8174
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8175
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8176
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8177
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8178
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8179
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8180
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8181
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[0]) = ((void *)oblate_radial1_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8182
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[1]) = ((void *)((char *)__pyx_k_obl_rad1));

  /* "scipy/special/_ufuncs.pyx":8183
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[2]) = ((void *)oblate_radial1_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8184
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]
 * ufunc_obl_rad1_data[1] = &ufunc_obl_rad1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[3]) = ((void *)((char *)__pyx_k_obl_rad1));

  /* "scipy/special/_ufuncs.pyx":8185
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_data[1] = &ufunc_obl_rad1_ptr[2*1]
 * obl_rad1 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_loops, ufunc_obl_rad1_data, ufunc_obl_rad1_types, 2, 4, 2, 0, "obl_rad1", ufunc_obl_rad1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8186
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]
 * ufunc_obl_rad1_data[1] = &ufunc_obl_rad1_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_rad1 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_loops, ufunc_obl_rad1_data, ufunc_obl_rad1_types, 2, 4, 2, 0, "obl_rad1", ufunc_obl_rad1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8187
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]
 * ufunc_obl_rad1_data[1] = &ufunc_obl_rad1_ptr[2*1]
 * obl_rad1 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_loops, ufunc_obl_rad1_data, ufunc_obl_rad1_types, 2, 4, 2, 0, "obl_rad1", ufunc_obl_rad1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_rad1_cv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types, 2, 4, 2, 0, __pyx_k_obl_rad1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_rad1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8193
 * cdef void *ufunc_obl_rad1_cv_data[2]
 * cdef char ufunc_obl_rad1_cv_types[14]
 * cdef char *ufunc_obl_rad1_cv_doc = (             # <<<<<<<<<<<<<<
 *     "obl_rad1_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_doc = __pyx_k_obl_rad1_cv_m_n_c_cv_x_Oblate_sp;

  /* "scipy/special/_ufuncs.pyx":8209
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":8210
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":8211
 * ufunc_obl_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8212
 * ufunc_obl_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8213
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8214
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8215
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8216
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8217
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8218
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8219
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8220
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8221
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8222
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8223
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8224
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8225
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[0]) = ((void *)oblate_radial1_wrap);

  /* "scipy/special/_ufuncs.pyx":8226
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[1]) = ((void *)((char *)__pyx_k_obl_rad1_cv));

  /* "scipy/special/_ufuncs.pyx":8227
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[2]) = ((void *)oblate_radial1_wrap);

  /* "scipy/special/_ufuncs.pyx":8228
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]
 * ufunc_obl_rad1_cv_data[1] = &ufunc_obl_rad1_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[3]) = ((void *)((char *)__pyx_k_obl_rad1_cv));

  /* "scipy/special/_ufuncs.pyx":8229
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_data[1] = &ufunc_obl_rad1_cv_ptr[2*1]
 * obl_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_cv_loops, ufunc_obl_rad1_cv_data, ufunc_obl_rad1_cv_types, 2, 5, 2, 0, "obl_rad1_cv", ufunc_obl_rad1_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8230
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]
 * ufunc_obl_rad1_cv_data[1] = &ufunc_obl_rad1_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_cv_loops, ufunc_obl_rad1_cv_data, ufunc_obl_rad1_cv_types, 2, 5, 2, 0, "obl_rad1_cv", ufunc_obl_rad1_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8231
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]
 * ufunc_obl_rad1_cv_data[1] = &ufunc_obl_rad1_cv_ptr[2*1]
 * obl_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_cv_loops, ufunc_obl_rad1_cv_data, ufunc_obl_rad1_cv_types, 2, 5, 2, 0, "obl_rad1_cv", ufunc_obl_rad1_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_rad2_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types, 2, 5, 2, 0, __pyx_k_obl_rad1_cv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_rad1_cv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8237
 * cdef void *ufunc_obl_rad2_data[2]
 * cdef char ufunc_obl_rad2_types[12]
 * cdef char *ufunc_obl_rad2_doc = (             # <<<<<<<<<<<<<<
 *     "obl_rad2(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_doc = __pyx_k_obl_rad2_m_n_c_x_Oblate_spheroid;

  /* "scipy/special/_ufuncs.pyx":8252
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":8253
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":8254
 * ufunc_obl_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8255
 * ufunc_obl_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8256
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8257
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8258
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8259
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8260
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8261
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8262
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8263
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8264
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8265
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8266
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[0]) = ((void *)oblate_radial2_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8267
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[1]) = ((void *)((char *)__pyx_k_obl_rad2));

  /* "scipy/special/_ufuncs.pyx":8268
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[2]) = ((void *)oblate_radial2_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8269
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]
 * ufunc_obl_rad2_data[1] = &ufunc_obl_rad2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[3]) = ((void *)((char *)__pyx_k_obl_rad2));

  /* "scipy/special/_ufuncs.pyx":8270
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_data[1] = &ufunc_obl_rad2_ptr[2*1]
 * obl_rad2 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_loops, ufunc_obl_rad2_data, ufunc_obl_rad2_types, 2, 4, 2, 0, "obl_rad2", ufunc_obl_rad2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8271
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]
 * ufunc_obl_rad2_data[1] = &ufunc_obl_rad2_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_rad2 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_loops, ufunc_obl_rad2_data, ufunc_obl_rad2_types, 2, 4, 2, 0, "obl_rad2", ufunc_obl_rad2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8272
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]
 * ufunc_obl_rad2_data[1] = &ufunc_obl_rad2_ptr[2*1]
 * obl_rad2 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_loops, ufunc_obl_rad2_data, ufunc_obl_rad2_types, 2, 4, 2, 0, "obl_rad2", ufunc_obl_rad2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_rad2_cv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types, 2, 4, 2, 0, __pyx_k_obl_rad2, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_rad2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8278
 * cdef void *ufunc_obl_rad2_cv_data[2]
 * cdef char ufunc_obl_rad2_cv_types[14]
 * cdef char *ufunc_obl_rad2_cv_doc = (             # <<<<<<<<<<<<<<
 *     "obl_rad2_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_doc = __pyx_k_obl_rad2_cv_m_n_c_cv_x_Oblate_sp;

  /* "scipy/special/_ufuncs.pyx":8294
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":8295
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":8296
 * ufunc_obl_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8297
 * ufunc_obl_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8298
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8299
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8300
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8301
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8302
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8303
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8304
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8305
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8306
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8307
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8308
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8309
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8310
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[0]) = ((void *)oblate_radial2_wrap);

  /* "scipy/special/_ufuncs.pyx":8311
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[1]) = ((void *)((char *)__pyx_k_obl_rad2_cv));

  /* "scipy/special/_ufuncs.pyx":8312
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[2]) = ((void *)oblate_radial2_wrap);

  /* "scipy/special/_ufuncs.pyx":8313
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]
 * ufunc_obl_rad2_cv_data[1] = &ufunc_obl_rad2_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[3]) = ((void *)((char *)__pyx_k_obl_rad2_cv));

  /* "scipy/special/_ufuncs.pyx":8314
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_data[1] = &ufunc_obl_rad2_cv_ptr[2*1]
 * obl_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_cv_loops, ufunc_obl_rad2_cv_data, ufunc_obl_rad2_cv_types, 2, 5, 2, 0, "obl_rad2_cv", ufunc_obl_rad2_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8315
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]
 * ufunc_obl_rad2_cv_data[1] = &ufunc_obl_rad2_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_cv_loops, ufunc_obl_rad2_cv_data, ufunc_obl_rad2_cv_types, 2, 5, 2, 0, "obl_rad2_cv", ufunc_obl_rad2_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8316
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]
 * ufunc_obl_rad2_cv_data[1] = &ufunc_obl_rad2_cv_ptr[2*1]
 * obl_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_cv_loops, ufunc_obl_rad2_cv_data, ufunc_obl_rad2_cv_types, 2, 5, 2, 0, "obl_rad2_cv", ufunc_obl_rad2_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pbdv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types, 2, 5, 2, 0, __pyx_k_obl_rad2_cv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8316; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_rad2_cv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8316; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8322
 * cdef void *ufunc_pbdv_data[2]
 * cdef char ufunc_pbdv_types[8]
 * cdef char *ufunc_pbdv_doc = (             # <<<<<<<<<<<<<<
 *     "pbdv(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_doc = __pyx_k_pbdv_v_x_Parabolic_cylinder_func;

  /* "scipy/special/_ufuncs.pyx":8336
 *     "dp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pbdv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff             # <<<<<<<<<<<<<<
 * ufunc_pbdv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff);

  /* "scipy/special/_ufuncs.pyx":8337
 *     "    Value of the derivative vs x")
 * ufunc_pbdv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbdv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd);

  /* "scipy/special/_ufuncs.pyx":8338
 * ufunc_pbdv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbdv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8339
 * ufunc_pbdv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8340
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8341
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8342
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8343
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8344
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8345
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8346
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[0]) = ((void *)pbdv_wrap);

  /* "scipy/special/_ufuncs.pyx":8347
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")             # <<<<<<<<<<<<<<
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[1]) = ((void *)((char *)__pyx_k_pbdv));

  /* "scipy/special/_ufuncs.pyx":8348
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[2]) = ((void *)pbdv_wrap);

  /* "scipy/special/_ufuncs.pyx":8349
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")             # <<<<<<<<<<<<<<
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]
 * ufunc_pbdv_data[1] = &ufunc_pbdv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[3]) = ((void *)((char *)__pyx_k_pbdv));

  /* "scipy/special/_ufuncs.pyx":8350
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pbdv_data[1] = &ufunc_pbdv_ptr[2*1]
 * pbdv = np.PyUFunc_FromFuncAndData(ufunc_pbdv_loops, ufunc_pbdv_data, ufunc_pbdv_types, 2, 2, 2, 0, "pbdv", ufunc_pbdv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8351
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]
 * ufunc_pbdv_data[1] = &ufunc_pbdv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pbdv = np.PyUFunc_FromFuncAndData(ufunc_pbdv_loops, ufunc_pbdv_data, ufunc_pbdv_types, 2, 2, 2, 0, "pbdv", ufunc_pbdv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8352
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]
 * ufunc_pbdv_data[1] = &ufunc_pbdv_ptr[2*1]
 * pbdv = np.PyUFunc_FromFuncAndData(ufunc_pbdv_loops, ufunc_pbdv_data, ufunc_pbdv_types, 2, 2, 2, 0, "pbdv", ufunc_pbdv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pbvv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types, 2, 2, 2, 0, __pyx_k_pbdv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pbdv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8358
 * cdef void *ufunc_pbvv_data[2]
 * cdef char ufunc_pbvv_types[8]
 * cdef char *ufunc_pbvv_doc = (             # <<<<<<<<<<<<<<
 *     "pbvv(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_doc = __pyx_k_pbvv_v_x_Parabolic_cylinder_func;

  /* "scipy/special/_ufuncs.pyx":8372
 *     "vp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pbvv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff             # <<<<<<<<<<<<<<
 * ufunc_pbvv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff);

  /* "scipy/special/_ufuncs.pyx":8373
 *     "    Value of the derivative vs x")
 * ufunc_pbvv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbvv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd);

  /* "scipy/special/_ufuncs.pyx":8374
 * ufunc_pbvv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbvv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8375
 * ufunc_pbvv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8376
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8377
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8378
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8379
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8380
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8381
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8382
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[0]) = ((void *)pbvv_wrap);

  /* "scipy/special/_ufuncs.pyx":8383
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")             # <<<<<<<<<<<<<<
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[1]) = ((void *)((char *)__pyx_k_pbvv));

  /* "scipy/special/_ufuncs.pyx":8384
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[2]) = ((void *)pbvv_wrap);

  /* "scipy/special/_ufuncs.pyx":8385
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")             # <<<<<<<<<<<<<<
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]
 * ufunc_pbvv_data[1] = &ufunc_pbvv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[3]) = ((void *)((char *)__pyx_k_pbvv));

  /* "scipy/special/_ufuncs.pyx":8386
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pbvv_data[1] = &ufunc_pbvv_ptr[2*1]
 * pbvv = np.PyUFunc_FromFuncAndData(ufunc_pbvv_loops, ufunc_pbvv_data, ufunc_pbvv_types, 2, 2, 2, 0, "pbvv", ufunc_pbvv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8387
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]
 * ufunc_pbvv_data[1] = &ufunc_pbvv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pbvv = np.PyUFunc_FromFuncAndData(ufunc_pbvv_loops, ufunc_pbvv_data, ufunc_pbvv_types, 2, 2, 2, 0, "pbvv", ufunc_pbvv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8388
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]
 * ufunc_pbvv_data[1] = &ufunc_pbvv_ptr[2*1]
 * pbvv = np.PyUFunc_FromFuncAndData(ufunc_pbvv_loops, ufunc_pbvv_data, ufunc_pbvv_types, 2, 2, 2, 0, "pbvv", ufunc_pbvv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pbwa_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types, 2, 2, 2, 0, __pyx_k_pbvv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pbvv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8394
 * cdef void *ufunc_pbwa_data[2]
 * cdef char ufunc_pbwa_types[8]
 * cdef char *ufunc_pbwa_doc = (             # <<<<<<<<<<<<<<
 *     "pbwa(a, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_doc = __pyx_k_pbwa_a_x_Parabolic_cylinder_func;

  /* "scipy/special/_ufuncs.pyx":8412
 *     "wp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pbwa_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff             # <<<<<<<<<<<<<<
 * ufunc_pbwa_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff);

  /* "scipy/special/_ufuncs.pyx":8413
 *     "    Value of the derivative vs x")
 * ufunc_pbwa_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbwa_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd);

  /* "scipy/special/_ufuncs.pyx":8414
 * ufunc_pbwa_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbwa_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8415
 * ufunc_pbwa_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8416
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8417
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8418
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8419
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8420
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8421
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8422
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[0]) = ((void *)pbwa_wrap);

  /* "scipy/special/_ufuncs.pyx":8423
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")             # <<<<<<<<<<<<<<
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[1]) = ((void *)((char *)__pyx_k_pbwa));

  /* "scipy/special/_ufuncs.pyx":8424
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[2]) = ((void *)pbwa_wrap);

  /* "scipy/special/_ufuncs.pyx":8425
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")             # <<<<<<<<<<<<<<
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]
 * ufunc_pbwa_data[1] = &ufunc_pbwa_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[3]) = ((void *)((char *)__pyx_k_pbwa));

  /* "scipy/special/_ufuncs.pyx":8426
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pbwa_data[1] = &ufunc_pbwa_ptr[2*1]
 * pbwa = np.PyUFunc_FromFuncAndData(ufunc_pbwa_loops, ufunc_pbwa_data, ufunc_pbwa_types, 2, 2, 2, 0, "pbwa", ufunc_pbwa_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8427
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]
 * ufunc_pbwa_data[1] = &ufunc_pbwa_ptr[2*1]             # <<<<<<<<<<<<<<
 * pbwa = np.PyUFunc_FromFuncAndData(ufunc_pbwa_loops, ufunc_pbwa_data, ufunc_pbwa_types, 2, 2, 2, 0, "pbwa", ufunc_pbwa_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8428
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]
 * ufunc_pbwa_data[1] = &ufunc_pbwa_ptr[2*1]
 * pbwa = np.PyUFunc_FromFuncAndData(ufunc_pbwa_loops, ufunc_pbwa_data, ufunc_pbwa_types, 2, 2, 2, 0, "pbwa", ufunc_pbwa_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pdtr_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types, 2, 2, 2, 0, __pyx_k_pbwa, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8428; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pbwa, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8428; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8434
 * cdef void *ufunc_pdtr_data[3]
 * cdef char ufunc_pdtr_types[9]
 * cdef char *ufunc_pdtr_doc = (             # <<<<<<<<<<<<<<
 *     "pdtr(k, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_doc = __pyx_k_pdtr_k_m_Poisson_cumulative_dist;

  /* "scipy/special/_ufuncs.pyx":8442
 *     "sum(exp(-m) * m**j / j!, j=0..k) = gammaincc( k+1, m).  Arguments\n"
 *     "must both be positive and `k` an integer.")
 * ufunc_pdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_pdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":8443
 *     "must both be positive and `k` an integer.")
 * ufunc_pdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtr_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8444
 * ufunc_pdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[0] = <char>NPY_LONG
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8445
 * ufunc_pdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtr_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":8446
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtr_types[0] = <char>NPY_LONG
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8447
 * ufunc_pdtr_types[0] = <char>NPY_LONG
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8448
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8449
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8450
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8451
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8452
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8453
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8454
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[0]) = ((void *)pdtr);

  /* "scipy/special/_ufuncs.pyx":8455
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[1]) = ((void *)((char *)__pyx_k_pdtr));

  /* "scipy/special/_ufuncs.pyx":8456
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":8457
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[3]) = ((void *)((char *)__pyx_k_pdtr));

  /* "scipy/special/_ufuncs.pyx":8458
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":8459
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")             # <<<<<<<<<<<<<<
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[5]) = ((void *)((char *)__pyx_k_pdtr));

  /* "scipy/special/_ufuncs.pyx":8460
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]
 * ufunc_pdtr_data[2] = &ufunc_pdtr_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8461
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_pdtr_data[2] = &ufunc_pdtr_ptr[2*2]
 * pdtr = np.PyUFunc_FromFuncAndData(ufunc_pdtr_loops, ufunc_pdtr_data, ufunc_pdtr_types, 3, 2, 1, 0, "pdtr", ufunc_pdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8462
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]
 * ufunc_pdtr_data[2] = &ufunc_pdtr_ptr[2*2]             # <<<<<<<<<<<<<<
 * pdtr = np.PyUFunc_FromFuncAndData(ufunc_pdtr_loops, ufunc_pdtr_data, ufunc_pdtr_types, 3, 2, 1, 0, "pdtr", ufunc_pdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":8463
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]
 * ufunc_pdtr_data[2] = &ufunc_pdtr_ptr[2*2]
 * pdtr = np.PyUFunc_FromFuncAndData(ufunc_pdtr_loops, ufunc_pdtr_data, ufunc_pdtr_types, 3, 2, 1, 0, "pdtr", ufunc_pdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pdtrc_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types, 3, 2, 1, 0, __pyx_k_pdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8469
 * cdef void *ufunc_pdtrc_data[3]
 * cdef char ufunc_pdtrc_types[9]
 * cdef char *ufunc_pdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "pdtrc(k, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_doc = __pyx_k_pdtrc_k_m_Poisson_survival_funct;

  /* "scipy/special/_ufuncs.pyx":8477
 *     "distribution: sum(exp(-m) * m**j / j!, j=k+1..inf) = gammainc(\n"
 *     "k+1, m).  Arguments must both be positive and `k` an integer.")
 * ufunc_pdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":8478
 *     "k+1, m).  Arguments must both be positive and `k` an integer.")
 * ufunc_pdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8479
 * ufunc_pdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[0] = <char>NPY_LONG
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8480
 * ufunc_pdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":8481
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrc_types[0] = <char>NPY_LONG
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8482
 * ufunc_pdtrc_types[0] = <char>NPY_LONG
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8483
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8484
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8485
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8486
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8487
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8488
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8489
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[0]) = ((void *)pdtrc);

  /* "scipy/special/_ufuncs.pyx":8490
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[1]) = ((void *)((char *)__pyx_k_pdtrc));

  /* "scipy/special/_ufuncs.pyx":8491
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":8492
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[3]) = ((void *)((char *)__pyx_k_pdtrc));

  /* "scipy/special/_ufuncs.pyx":8493
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":8494
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[5]) = ((void *)((char *)__pyx_k_pdtrc));

  /* "scipy/special/_ufuncs.pyx":8495
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]
 * ufunc_pdtrc_data[2] = &ufunc_pdtrc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8496
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_data[2] = &ufunc_pdtrc_ptr[2*2]
 * pdtrc = np.PyUFunc_FromFuncAndData(ufunc_pdtrc_loops, ufunc_pdtrc_data, ufunc_pdtrc_types, 3, 2, 1, 0, "pdtrc", ufunc_pdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8497
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]
 * ufunc_pdtrc_data[2] = &ufunc_pdtrc_ptr[2*2]             # <<<<<<<<<<<<<<
 * pdtrc = np.PyUFunc_FromFuncAndData(ufunc_pdtrc_loops, ufunc_pdtrc_data, ufunc_pdtrc_types, 3, 2, 1, 0, "pdtrc", ufunc_pdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":8498
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]
 * ufunc_pdtrc_data[2] = &ufunc_pdtrc_ptr[2*2]
 * pdtrc = np.PyUFunc_FromFuncAndData(ufunc_pdtrc_loops, ufunc_pdtrc_data, ufunc_pdtrc_types, 3, 2, 1, 0, "pdtrc", ufunc_pdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pdtri_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types, 3, 2, 1, 0, __pyx_k_pdtrc, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pdtrc, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8504
 * cdef void *ufunc_pdtri_data[3]
 * cdef char ufunc_pdtri_types[9]
 * cdef char *ufunc_pdtri_doc = (             # <<<<<<<<<<<<<<
 *     "pdtri(k, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_doc = __pyx_k_pdtri_k_y_Inverse_to_pdtr_vs_m_R;

  /* "scipy/special/_ufuncs.pyx":8513
 *     "calculated by gammaincinv(k+1, y). `k` must be a nonnegative\n"
 *     "integer and `y` between 0 and 1.")
 * ufunc_pdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_pdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":8514
 *     "integer and `y` between 0 and 1.")
 * ufunc_pdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtri_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8515
 * ufunc_pdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[0] = <char>NPY_LONG
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8516
 * ufunc_pdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtri_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":8517
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtri_types[0] = <char>NPY_LONG
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8518
 * ufunc_pdtri_types[0] = <char>NPY_LONG
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8519
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8520
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8521
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8522
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8523
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8524
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8525
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[0]) = ((void *)pdtri);

  /* "scipy/special/_ufuncs.pyx":8526
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[1]) = ((void *)((char *)__pyx_k_pdtri));

  /* "scipy/special/_ufuncs.pyx":8527
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":8528
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[3]) = ((void *)((char *)__pyx_k_pdtri));

  /* "scipy/special/_ufuncs.pyx":8529
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":8530
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")             # <<<<<<<<<<<<<<
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[5]) = ((void *)((char *)__pyx_k_pdtri));

  /* "scipy/special/_ufuncs.pyx":8531
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]
 * ufunc_pdtri_data[2] = &ufunc_pdtri_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8532
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_pdtri_data[2] = &ufunc_pdtri_ptr[2*2]
 * pdtri = np.PyUFunc_FromFuncAndData(ufunc_pdtri_loops, ufunc_pdtri_data, ufunc_pdtri_types, 3, 2, 1, 0, "pdtri", ufunc_pdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8533
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]
 * ufunc_pdtri_data[2] = &ufunc_pdtri_ptr[2*2]             # <<<<<<<<<<<<<<
 * pdtri = np.PyUFunc_FromFuncAndData(ufunc_pdtri_loops, ufunc_pdtri_data, ufunc_pdtri_types, 3, 2, 1, 0, "pdtri", ufunc_pdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":8534
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]
 * ufunc_pdtri_data[2] = &ufunc_pdtri_ptr[2*2]
 * pdtri = np.PyUFunc_FromFuncAndData(ufunc_pdtri_loops, ufunc_pdtri_data, ufunc_pdtri_types, 3, 2, 1, 0, "pdtri", ufunc_pdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pdtrik_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types, 3, 2, 1, 0, __pyx_k_pdtri, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8534; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pdtri, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8534; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8540
 * cdef void *ufunc_pdtrik_data[2]
 * cdef char ufunc_pdtrik_types[6]
 * cdef char *ufunc_pdtrik_doc = (             # <<<<<<<<<<<<<<
 *     "pdtrik(p, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_doc = __pyx_k_pdtrik_p_m_Inverse_to_pdtr_vs_k;

  /* "scipy/special/_ufuncs.pyx":8546
 *     "\n"
 *     "Returns the quantile k such that ``pdtr(k, m) = p``")
 * ufunc_pdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8547
 *     "Returns the quantile k such that ``pdtr(k, m) = p``")
 * ufunc_pdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8548
 * ufunc_pdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8549
 * ufunc_pdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8550
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8551
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8552
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8553
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8554
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[0]) = ((void *)cdfpoi2_wrap);

  /* "scipy/special/_ufuncs.pyx":8555
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[1]) = ((void *)((char *)__pyx_k_pdtrik));

  /* "scipy/special/_ufuncs.pyx":8556
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[2]) = ((void *)cdfpoi2_wrap);

  /* "scipy/special/_ufuncs.pyx":8557
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]
 * ufunc_pdtrik_data[1] = &ufunc_pdtrik_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[3]) = ((void *)((char *)__pyx_k_pdtrik));

  /* "scipy/special/_ufuncs.pyx":8558
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_data[1] = &ufunc_pdtrik_ptr[2*1]
 * pdtrik = np.PyUFunc_FromFuncAndData(ufunc_pdtrik_loops, ufunc_pdtrik_data, ufunc_pdtrik_types, 2, 2, 1, 0, "pdtrik", ufunc_pdtrik_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8559
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]
 * ufunc_pdtrik_data[1] = &ufunc_pdtrik_ptr[2*1]             # <<<<<<<<<<<<<<
 * pdtrik = np.PyUFunc_FromFuncAndData(ufunc_pdtrik_loops, ufunc_pdtrik_data, ufunc_pdtrik_types, 2, 2, 1, 0, "pdtrik", ufunc_pdtrik_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8560
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]
 * ufunc_pdtrik_data[1] = &ufunc_pdtrik_ptr[2*1]
 * pdtrik = np.PyUFunc_FromFuncAndData(ufunc_pdtrik_loops, ufunc_pdtrik_data, ufunc_pdtrik_types, 2, 2, 1, 0, "pdtrik", ufunc_pdtrik_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_poch_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types, 2, 2, 1, 0, __pyx_k_pdtrik, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8560; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pdtrik, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8560; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8566
 * cdef void *ufunc_poch_data[2]
 * cdef char ufunc_poch_types[6]
 * cdef char *ufunc_poch_doc = (             # <<<<<<<<<<<<<<
 *     "poch(z, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_doc = __pyx_k_poch_z_m_Rising_factorial_z__m_T;

  /* "scipy/special/_ufuncs.pyx":8578
 *     "\n"
 *     "    (z)_m = z * (z + 1) * ... * (z + m - 1)")
 * ufunc_poch_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_poch_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_poch_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8579
 *     "    (z)_m = z * (z + 1) * ... * (z + m - 1)")
 * ufunc_poch_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_poch_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_poch_types[0] = <char>NPY_FLOAT
 * ufunc_poch_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8580
 * ufunc_poch_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_poch_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_poch_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_poch_types[1] = <char>NPY_FLOAT
 * ufunc_poch_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8581
 * ufunc_poch_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_poch_types[0] = <char>NPY_FLOAT
 * ufunc_poch_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_poch_types[2] = <char>NPY_FLOAT
 * ufunc_poch_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8582
 * ufunc_poch_types[0] = <char>NPY_FLOAT
 * ufunc_poch_types[1] = <char>NPY_FLOAT
 * ufunc_poch_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_poch_types[3] = <char>NPY_DOUBLE
 * ufunc_poch_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8583
 * ufunc_poch_types[1] = <char>NPY_FLOAT
 * ufunc_poch_types[2] = <char>NPY_FLOAT
 * ufunc_poch_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_poch_types[4] = <char>NPY_DOUBLE
 * ufunc_poch_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8584
 * ufunc_poch_types[2] = <char>NPY_FLOAT
 * ufunc_poch_types[3] = <char>NPY_DOUBLE
 * ufunc_poch_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_poch_types[5] = <char>NPY_DOUBLE
 * ufunc_poch_ptr[2*0] = <void*>_func_poch
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8585
 * ufunc_poch_types[3] = <char>NPY_DOUBLE
 * ufunc_poch_types[4] = <char>NPY_DOUBLE
 * ufunc_poch_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_poch_ptr[2*0] = <void*>_func_poch
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8586
 * ufunc_poch_types[4] = <char>NPY_DOUBLE
 * ufunc_poch_types[5] = <char>NPY_DOUBLE
 * ufunc_poch_ptr[2*0] = <void*>_func_poch             # <<<<<<<<<<<<<<
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")
 * ufunc_poch_ptr[2*1] = <void*>_func_poch
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[0]) = ((void *)poch);

  /* "scipy/special/_ufuncs.pyx":8587
 * ufunc_poch_types[5] = <char>NPY_DOUBLE
 * ufunc_poch_ptr[2*0] = <void*>_func_poch
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")             # <<<<<<<<<<<<<<
 * ufunc_poch_ptr[2*1] = <void*>_func_poch
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[1]) = ((void *)((char *)__pyx_k_poch));

  /* "scipy/special/_ufuncs.pyx":8588
 * ufunc_poch_ptr[2*0] = <void*>_func_poch
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")
 * ufunc_poch_ptr[2*1] = <void*>_func_poch             # <<<<<<<<<<<<<<
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[2]) = ((void *)poch);

  /* "scipy/special/_ufuncs.pyx":8589
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")
 * ufunc_poch_ptr[2*1] = <void*>_func_poch
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")             # <<<<<<<<<<<<<<
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]
 * ufunc_poch_data[1] = &ufunc_poch_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[3]) = ((void *)((char *)__pyx_k_poch));

  /* "scipy/special/_ufuncs.pyx":8590
 * ufunc_poch_ptr[2*1] = <void*>_func_poch
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_poch_data[1] = &ufunc_poch_ptr[2*1]
 * poch = np.PyUFunc_FromFuncAndData(ufunc_poch_loops, ufunc_poch_data, ufunc_poch_types, 2, 2, 1, 0, "poch", ufunc_poch_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8591
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]
 * ufunc_poch_data[1] = &ufunc_poch_ptr[2*1]             # <<<<<<<<<<<<<<
 * poch = np.PyUFunc_FromFuncAndData(ufunc_poch_loops, ufunc_poch_data, ufunc_poch_types, 2, 2, 1, 0, "poch", ufunc_poch_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8592
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]
 * ufunc_poch_data[1] = &ufunc_poch_ptr[2*1]
 * poch = np.PyUFunc_FromFuncAndData(ufunc_poch_loops, ufunc_poch_data, ufunc_poch_types, 2, 2, 1, 0, "poch", ufunc_poch_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_ang1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types, 2, 2, 1, 0, __pyx_k_poch, __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_poch, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8598
 * cdef void *ufunc_pro_ang1_data[2]
 * cdef char ufunc_pro_ang1_types[12]
 * cdef char *ufunc_pro_ang1_doc = (             # <<<<<<<<<<<<<<
 *     "pro_ang1(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_doc = __pyx_k_pro_ang1_m_n_c_x_Prolate_spheroi;

  /* "scipy/special/_ufuncs.pyx":8613
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":8614
 *     "    Value of the derivative vs x")
 * ufunc_pro_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":8615
 * ufunc_pro_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8616
 * ufunc_pro_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8617
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8618
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8619
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8620
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8621
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8622
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8623
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8624
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8625
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8626
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8627
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[0]) = ((void *)prolate_aswfa_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8628
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[1]) = ((void *)((char *)__pyx_k_pro_ang1));

  /* "scipy/special/_ufuncs.pyx":8629
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[2]) = ((void *)prolate_aswfa_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8630
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]
 * ufunc_pro_ang1_data[1] = &ufunc_pro_ang1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[3]) = ((void *)((char *)__pyx_k_pro_ang1));

  /* "scipy/special/_ufuncs.pyx":8631
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_data[1] = &ufunc_pro_ang1_ptr[2*1]
 * pro_ang1 = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_loops, ufunc_pro_ang1_data, ufunc_pro_ang1_types, 2, 4, 2, 0, "pro_ang1", ufunc_pro_ang1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8632
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]
 * ufunc_pro_ang1_data[1] = &ufunc_pro_ang1_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_ang1 = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_loops, ufunc_pro_ang1_data, ufunc_pro_ang1_types, 2, 4, 2, 0, "pro_ang1", ufunc_pro_ang1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8633
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]
 * ufunc_pro_ang1_data[1] = &ufunc_pro_ang1_ptr[2*1]
 * pro_ang1 = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_loops, ufunc_pro_ang1_data, ufunc_pro_ang1_types, 2, 4, 2, 0, "pro_ang1", ufunc_pro_ang1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_ang1_cv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types, 2, 4, 2, 0, __pyx_k_pro_ang1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8633; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_ang1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8633; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8639
 * cdef void *ufunc_pro_ang1_cv_data[2]
 * cdef char ufunc_pro_ang1_cv_types[14]
 * cdef char *ufunc_pro_ang1_cv_doc = (             # <<<<<<<<<<<<<<
 *     "pro_ang1_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_doc = __pyx_k_pro_ang1_cv_m_n_c_cv_x_Prolate_s;

  /* "scipy/special/_ufuncs.pyx":8655
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":8656
 *     "    Value of the derivative vs x")
 * ufunc_pro_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":8657
 * ufunc_pro_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8658
 * ufunc_pro_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8659
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8660
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8661
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8662
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8663
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8664
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8665
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8666
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8667
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8668
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8669
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8670
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8671
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[0]) = ((void *)prolate_aswfa_wrap);

  /* "scipy/special/_ufuncs.pyx":8672
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[1]) = ((void *)((char *)__pyx_k_pro_ang1_cv));

  /* "scipy/special/_ufuncs.pyx":8673
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[2]) = ((void *)prolate_aswfa_wrap);

  /* "scipy/special/_ufuncs.pyx":8674
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]
 * ufunc_pro_ang1_cv_data[1] = &ufunc_pro_ang1_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[3]) = ((void *)((char *)__pyx_k_pro_ang1_cv));

  /* "scipy/special/_ufuncs.pyx":8675
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_data[1] = &ufunc_pro_ang1_cv_ptr[2*1]
 * pro_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_cv_loops, ufunc_pro_ang1_cv_data, ufunc_pro_ang1_cv_types, 2, 5, 2, 0, "pro_ang1_cv", ufunc_pro_ang1_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8676
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]
 * ufunc_pro_ang1_cv_data[1] = &ufunc_pro_ang1_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_cv_loops, ufunc_pro_ang1_cv_data, ufunc_pro_ang1_cv_types, 2, 5, 2, 0, "pro_ang1_cv", ufunc_pro_ang1_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8677
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]
 * ufunc_pro_ang1_cv_data[1] = &ufunc_pro_ang1_cv_ptr[2*1]
 * pro_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_cv_loops, ufunc_pro_ang1_cv_data, ufunc_pro_ang1_cv_types, 2, 5, 2, 0, "pro_ang1_cv", ufunc_pro_ang1_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_cv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types, 2, 5, 2, 0, __pyx_k_pro_ang1_cv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8677; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_ang1_cv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8677; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8683
 * cdef void *ufunc_pro_cv_data[2]
 * cdef char ufunc_pro_cv_types[8]
 * cdef char *ufunc_pro_cv_doc = (             # <<<<<<<<<<<<<<
 *     "pro_cv(m, n, c)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_doc = __pyx_k_pro_cv_m_n_c_Characteristic_valu;

  /* "scipy/special/_ufuncs.pyx":8690
 *     "Computes the characteristic value of prolate spheroidal wave\n"
 *     "functions of order `m`, `n` (n>=m) and spheroidal parameter `c`.")
 * ufunc_pro_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":8691
 *     "functions of order `m`, `n` (n>=m) and spheroidal parameter `c`.")
 * ufunc_pro_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_pro_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":8692
 * ufunc_pro_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_pro_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8693
 * ufunc_pro_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8694
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8695
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8696
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8697
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8698
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8699
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8700
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[0]) = ((void *)prolate_segv_wrap);

  /* "scipy/special/_ufuncs.pyx":8701
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[1]) = ((void *)((char *)__pyx_k_pro_cv));

  /* "scipy/special/_ufuncs.pyx":8702
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[2]) = ((void *)prolate_segv_wrap);

  /* "scipy/special/_ufuncs.pyx":8703
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]
 * ufunc_pro_cv_data[1] = &ufunc_pro_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[3]) = ((void *)((char *)__pyx_k_pro_cv));

  /* "scipy/special/_ufuncs.pyx":8704
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_data[1] = &ufunc_pro_cv_ptr[2*1]
 * pro_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_cv_loops, ufunc_pro_cv_data, ufunc_pro_cv_types, 2, 3, 1, 0, "pro_cv", ufunc_pro_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8705
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]
 * ufunc_pro_cv_data[1] = &ufunc_pro_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_cv_loops, ufunc_pro_cv_data, ufunc_pro_cv_types, 2, 3, 1, 0, "pro_cv", ufunc_pro_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8706
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]
 * ufunc_pro_cv_data[1] = &ufunc_pro_cv_ptr[2*1]
 * pro_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_cv_loops, ufunc_pro_cv_data, ufunc_pro_cv_types, 2, 3, 1, 0, "pro_cv", ufunc_pro_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_rad1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types, 2, 3, 1, 0, __pyx_k_pro_cv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_cv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8712
 * cdef void *ufunc_pro_rad1_data[2]
 * cdef char ufunc_pro_rad1_types[12]
 * cdef char *ufunc_pro_rad1_doc = (             # <<<<<<<<<<<<<<
 *     "pro_rad1(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_doc = __pyx_k_pro_rad1_m_n_c_x_Prolate_spheroi;

  /* "scipy/special/_ufuncs.pyx":8727
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":8728
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":8729
 * ufunc_pro_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8730
 * ufunc_pro_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8731
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8732
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8733
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8734
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8735
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8736
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8737
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8738
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8739
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8740
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8741
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[0]) = ((void *)prolate_radial1_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8742
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[1]) = ((void *)((char *)__pyx_k_pro_rad1));

  /* "scipy/special/_ufuncs.pyx":8743
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[2]) = ((void *)prolate_radial1_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8744
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]
 * ufunc_pro_rad1_data[1] = &ufunc_pro_rad1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[3]) = ((void *)((char *)__pyx_k_pro_rad1));

  /* "scipy/special/_ufuncs.pyx":8745
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_data[1] = &ufunc_pro_rad1_ptr[2*1]
 * pro_rad1 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_loops, ufunc_pro_rad1_data, ufunc_pro_rad1_types, 2, 4, 2, 0, "pro_rad1", ufunc_pro_rad1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8746
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]
 * ufunc_pro_rad1_data[1] = &ufunc_pro_rad1_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_rad1 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_loops, ufunc_pro_rad1_data, ufunc_pro_rad1_types, 2, 4, 2, 0, "pro_rad1", ufunc_pro_rad1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8747
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]
 * ufunc_pro_rad1_data[1] = &ufunc_pro_rad1_ptr[2*1]
 * pro_rad1 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_loops, ufunc_pro_rad1_data, ufunc_pro_rad1_types, 2, 4, 2, 0, "pro_rad1", ufunc_pro_rad1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_rad1_cv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types, 2, 4, 2, 0, __pyx_k_pro_rad1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8747; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_rad1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8747; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8753
 * cdef void *ufunc_pro_rad1_cv_data[2]
 * cdef char ufunc_pro_rad1_cv_types[14]
 * cdef char *ufunc_pro_rad1_cv_doc = (             # <<<<<<<<<<<<<<
 *     "pro_rad1_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_doc = __pyx_k_pro_rad1_cv_m_n_c_cv_x_Prolate_s;

  /* "scipy/special/_ufuncs.pyx":8769
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":8770
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":8771
 * ufunc_pro_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8772
 * ufunc_pro_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8773
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8774
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8775
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8776
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8777
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8778
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8779
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8780
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8781
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8782
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8783
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8784
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8785
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[0]) = ((void *)prolate_radial1_wrap);

  /* "scipy/special/_ufuncs.pyx":8786
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[1]) = ((void *)((char *)__pyx_k_pro_rad1_cv));

  /* "scipy/special/_ufuncs.pyx":8787
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[2]) = ((void *)prolate_radial1_wrap);

  /* "scipy/special/_ufuncs.pyx":8788
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]
 * ufunc_pro_rad1_cv_data[1] = &ufunc_pro_rad1_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[3]) = ((void *)((char *)__pyx_k_pro_rad1_cv));

  /* "scipy/special/_ufuncs.pyx":8789
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_data[1] = &ufunc_pro_rad1_cv_ptr[2*1]
 * pro_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_cv_loops, ufunc_pro_rad1_cv_data, ufunc_pro_rad1_cv_types, 2, 5, 2, 0, "pro_rad1_cv", ufunc_pro_rad1_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8790
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]
 * ufunc_pro_rad1_cv_data[1] = &ufunc_pro_rad1_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_cv_loops, ufunc_pro_rad1_cv_data, ufunc_pro_rad1_cv_types, 2, 5, 2, 0, "pro_rad1_cv", ufunc_pro_rad1_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8791
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]
 * ufunc_pro_rad1_cv_data[1] = &ufunc_pro_rad1_cv_ptr[2*1]
 * pro_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_cv_loops, ufunc_pro_rad1_cv_data, ufunc_pro_rad1_cv_types, 2, 5, 2, 0, "pro_rad1_cv", ufunc_pro_rad1_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_rad2_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types, 2, 5, 2, 0, __pyx_k_pro_rad1_cv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_rad1_cv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8797
 * cdef void *ufunc_pro_rad2_data[2]
 * cdef char ufunc_pro_rad2_types[12]
 * cdef char *ufunc_pro_rad2_doc = (             # <<<<<<<<<<<<<<
 *     "pro_rad2(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_doc = __pyx_k_pro_rad2_m_n_c_x_Prolate_spheroi;

  /* "scipy/special/_ufuncs.pyx":8812
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":8813
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":8814
 * ufunc_pro_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8815
 * ufunc_pro_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8816
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8817
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8818
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8819
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8820
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8821
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8822
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8823
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8824
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8825
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8826
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[0]) = ((void *)prolate_radial2_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8827
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[1]) = ((void *)((char *)__pyx_k_pro_rad2));

  /* "scipy/special/_ufuncs.pyx":8828
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[2]) = ((void *)prolate_radial2_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":8829
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]
 * ufunc_pro_rad2_data[1] = &ufunc_pro_rad2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[3]) = ((void *)((char *)__pyx_k_pro_rad2));

  /* "scipy/special/_ufuncs.pyx":8830
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_data[1] = &ufunc_pro_rad2_ptr[2*1]
 * pro_rad2 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_loops, ufunc_pro_rad2_data, ufunc_pro_rad2_types, 2, 4, 2, 0, "pro_rad2", ufunc_pro_rad2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8831
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]
 * ufunc_pro_rad2_data[1] = &ufunc_pro_rad2_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_rad2 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_loops, ufunc_pro_rad2_data, ufunc_pro_rad2_types, 2, 4, 2, 0, "pro_rad2", ufunc_pro_rad2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8832
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]
 * ufunc_pro_rad2_data[1] = &ufunc_pro_rad2_ptr[2*1]
 * pro_rad2 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_loops, ufunc_pro_rad2_data, ufunc_pro_rad2_types, 2, 4, 2, 0, "pro_rad2", ufunc_pro_rad2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_rad2_cv_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types, 2, 4, 2, 0, __pyx_k_pro_rad2, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_rad2, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8838
 * cdef void *ufunc_pro_rad2_cv_data[2]
 * cdef char ufunc_pro_rad2_cv_types[14]
 * cdef char *ufunc_pro_rad2_cv_doc = (             # <<<<<<<<<<<<<<
 *     "pro_rad2_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_doc = __pyx_k_pro_rad2_cv_m_n_c_cv_x_Prolate_s;

  /* "scipy/special/_ufuncs.pyx":8854
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":8855
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":8856
 * ufunc_pro_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8857
 * ufunc_pro_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8858
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8859
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8860
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8861
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8862
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8863
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8864
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8865
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8866
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8867
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8868
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8869
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8870
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[0]) = ((void *)prolate_radial2_wrap);

  /* "scipy/special/_ufuncs.pyx":8871
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[1]) = ((void *)((char *)__pyx_k_pro_rad2_cv));

  /* "scipy/special/_ufuncs.pyx":8872
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[2]) = ((void *)prolate_radial2_wrap);

  /* "scipy/special/_ufuncs.pyx":8873
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]
 * ufunc_pro_rad2_cv_data[1] = &ufunc_pro_rad2_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[3]) = ((void *)((char *)__pyx_k_pro_rad2_cv));

  /* "scipy/special/_ufuncs.pyx":8874
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_data[1] = &ufunc_pro_rad2_cv_ptr[2*1]
 * pro_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_cv_loops, ufunc_pro_rad2_cv_data, ufunc_pro_rad2_cv_types, 2, 5, 2, 0, "pro_rad2_cv", ufunc_pro_rad2_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8875
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]
 * ufunc_pro_rad2_cv_data[1] = &ufunc_pro_rad2_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_cv_loops, ufunc_pro_rad2_cv_data, ufunc_pro_rad2_cv_types, 2, 5, 2, 0, "pro_rad2_cv", ufunc_pro_rad2_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8876
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]
 * ufunc_pro_rad2_cv_data[1] = &ufunc_pro_rad2_cv_ptr[2*1]
 * pro_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_cv_loops, ufunc_pro_rad2_cv_data, ufunc_pro_rad2_cv_types, 2, 5, 2, 0, "pro_rad2_cv", ufunc_pro_rad2_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pseudo_huber_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types, 2, 5, 2, 0, __pyx_k_pro_rad2_cv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8876; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_rad2_cv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8876; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8882
 * cdef void *ufunc_pseudo_huber_data[2]
 * cdef char ufunc_pseudo_huber_types[6]
 * cdef char *ufunc_pseudo_huber_doc = (             # <<<<<<<<<<<<<<
 *     "pseudo_huber(delta, r)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_doc = __pyx_k_pseudo_huber_delta_r_Pseudo_Hube;

  /* "scipy/special/_ufuncs.pyx":8906
 *     "\n"
 *     ".. versionadded:: 0.15.0")
 * ufunc_pseudo_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8907
 *     ".. versionadded:: 0.15.0")
 * ufunc_pseudo_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pseudo_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8908
 * ufunc_pseudo_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pseudo_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8909
 * ufunc_pseudo_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8910
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8911
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8912
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8913
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8914
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber);

  /* "scipy/special/_ufuncs.pyx":8915
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[1]) = ((void *)((char *)__pyx_k_pseudo_huber));

  /* "scipy/special/_ufuncs.pyx":8916
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber);

  /* "scipy/special/_ufuncs.pyx":8917
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]
 * ufunc_pseudo_huber_data[1] = &ufunc_pseudo_huber_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[3]) = ((void *)((char *)__pyx_k_pseudo_huber));

  /* "scipy/special/_ufuncs.pyx":8918
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_data[1] = &ufunc_pseudo_huber_ptr[2*1]
 * pseudo_huber = np.PyUFunc_FromFuncAndData(ufunc_pseudo_huber_loops, ufunc_pseudo_huber_data, ufunc_pseudo_huber_types, 2, 2, 1, 0, "pseudo_huber", ufunc_pseudo_huber_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8919
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]
 * ufunc_pseudo_huber_data[1] = &ufunc_pseudo_huber_ptr[2*1]             # <<<<<<<<<<<<<<
 * pseudo_huber = np.PyUFunc_FromFuncAndData(ufunc_pseudo_huber_loops, ufunc_pseudo_huber_data, ufunc_pseudo_huber_types, 2, 2, 1, 0, "pseudo_huber", ufunc_pseudo_huber_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8920
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]
 * ufunc_pseudo_huber_data[1] = &ufunc_pseudo_huber_ptr[2*1]
 * pseudo_huber = np.PyUFunc_FromFuncAndData(ufunc_pseudo_huber_loops, ufunc_pseudo_huber_data, ufunc_pseudo_huber_types, 2, 2, 1, 0, "pseudo_huber", ufunc_pseudo_huber_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_psi_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types, 2, 2, 1, 0, __pyx_k_pseudo_huber, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8920; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pseudo_huber, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8920; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8926
 * cdef void *ufunc_psi_data[4]
 * cdef char ufunc_psi_types[8]
 * cdef char *ufunc_psi_doc = (             # <<<<<<<<<<<<<<
 *     "psi(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_doc = __pyx_k_psi_z_Digamma_function_The_deriv;

  /* "scipy/special/_ufuncs.pyx":8933
 *     "The derivative of the logarithm of the gamma function evaluated at\n"
 *     "`z` (also called the digamma function).")
 * ufunc_psi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_psi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8934
 *     "`z` (also called the digamma function).")
 * ufunc_psi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_psi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8935
 * ufunc_psi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_psi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_psi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":8936
 * ufunc_psi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_psi_types[0] = <char>NPY_FLOAT
 * ufunc_psi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":8937
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_psi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_psi_types[1] = <char>NPY_FLOAT
 * ufunc_psi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8938
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_psi_types[0] = <char>NPY_FLOAT
 * ufunc_psi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_psi_types[2] = <char>NPY_DOUBLE
 * ufunc_psi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8939
 * ufunc_psi_types[0] = <char>NPY_FLOAT
 * ufunc_psi_types[1] = <char>NPY_FLOAT
 * ufunc_psi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_psi_types[3] = <char>NPY_DOUBLE
 * ufunc_psi_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8940
 * ufunc_psi_types[1] = <char>NPY_FLOAT
 * ufunc_psi_types[2] = <char>NPY_DOUBLE
 * ufunc_psi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_psi_types[4] = <char>NPY_CFLOAT
 * ufunc_psi_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8941
 * ufunc_psi_types[2] = <char>NPY_DOUBLE
 * ufunc_psi_types[3] = <char>NPY_DOUBLE
 * ufunc_psi_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_psi_types[5] = <char>NPY_CFLOAT
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":8942
 * ufunc_psi_types[3] = <char>NPY_DOUBLE
 * ufunc_psi_types[4] = <char>NPY_CFLOAT
 * ufunc_psi_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":8943
 * ufunc_psi_types[4] = <char>NPY_CFLOAT
 * ufunc_psi_types[5] = <char>NPY_CFLOAT
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE
 * ufunc_psi_ptr[2*0] = <void*>_func_psi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":8944
 * ufunc_psi_types[5] = <char>NPY_CFLOAT
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*0] = <void*>_func_psi
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":8945
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE
 * ufunc_psi_ptr[2*0] = <void*>_func_psi             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*1] = <void*>_func_psi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[0]) = ((void *)psi);

  /* "scipy/special/_ufuncs.pyx":8946
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE
 * ufunc_psi_ptr[2*0] = <void*>_func_psi
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*1] = <void*>_func_psi
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[1]) = ((void *)((char *)__pyx_k_psi));

  /* "scipy/special/_ufuncs.pyx":8947
 * ufunc_psi_ptr[2*0] = <void*>_func_psi
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*1] = <void*>_func_psi             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*2] = <void*>_func_cpsi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[2]) = ((void *)psi);

  /* "scipy/special/_ufuncs.pyx":8948
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*1] = <void*>_func_psi
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*2] = <void*>_func_cpsi_wrap
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[3]) = ((void *)((char *)__pyx_k_psi));

  /* "scipy/special/_ufuncs.pyx":8949
 * ufunc_psi_ptr[2*1] = <void*>_func_psi
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*2] = <void*>_func_cpsi_wrap             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*3] = <void*>_func_cpsi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[4]) = ((void *)cpsi_wrap);

  /* "scipy/special/_ufuncs.pyx":8950
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*2] = <void*>_func_cpsi_wrap
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*3] = <void*>_func_cpsi_wrap
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[5]) = ((void *)((char *)__pyx_k_psi));

  /* "scipy/special/_ufuncs.pyx":8951
 * ufunc_psi_ptr[2*2] = <void*>_func_cpsi_wrap
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*3] = <void*>_func_cpsi_wrap             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[6]) = ((void *)cpsi_wrap);

  /* "scipy/special/_ufuncs.pyx":8952
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*3] = <void*>_func_cpsi_wrap
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")             # <<<<<<<<<<<<<<
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[7]) = ((void *)((char *)__pyx_k_psi));

  /* "scipy/special/_ufuncs.pyx":8953
 * ufunc_psi_ptr[2*3] = <void*>_func_cpsi_wrap
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8954
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]
 * ufunc_psi_data[3] = &ufunc_psi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8955
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_psi_data[3] = &ufunc_psi_ptr[2*3]
 * psi = np.PyUFunc_FromFuncAndData(ufunc_psi_loops, ufunc_psi_data, ufunc_psi_types, 4, 1, 1, 0, "psi", ufunc_psi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":8956
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]
 * ufunc_psi_data[3] = &ufunc_psi_ptr[2*3]             # <<<<<<<<<<<<<<
 * psi = np.PyUFunc_FromFuncAndData(ufunc_psi_loops, ufunc_psi_data, ufunc_psi_types, 4, 1, 1, 0, "psi", ufunc_psi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":8957
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]
 * ufunc_psi_data[3] = &ufunc_psi_ptr[2*3]
 * psi = np.PyUFunc_FromFuncAndData(ufunc_psi_loops, ufunc_psi_data, ufunc_psi_types, 4, 1, 1, 0, "psi", ufunc_psi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_radian_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types, 4, 1, 1, 0, __pyx_k_psi, __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8957; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_psi, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8957; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8963
 * cdef void *ufunc_radian_data[2]
 * cdef char ufunc_radian_types[8]
 * cdef char *ufunc_radian_doc = (             # <<<<<<<<<<<<<<
 *     "radian(d, m, s)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_doc = __pyx_k_radian_d_m_s_Convert_from_degree;

  /* "scipy/special/_ufuncs.pyx":8970
 *     "Returns the angle given in (d)egrees, (m)inutes, and (s)econds in\n"
 *     "radians.")
 * ufunc_radian_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_radian_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_radian_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":8971
 *     "radians.")
 * ufunc_radian_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_radian_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_radian_types[0] = <char>NPY_FLOAT
 * ufunc_radian_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":8972
 * ufunc_radian_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_radian_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_radian_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_radian_types[1] = <char>NPY_FLOAT
 * ufunc_radian_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8973
 * ufunc_radian_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_radian_types[0] = <char>NPY_FLOAT
 * ufunc_radian_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_radian_types[2] = <char>NPY_FLOAT
 * ufunc_radian_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8974
 * ufunc_radian_types[0] = <char>NPY_FLOAT
 * ufunc_radian_types[1] = <char>NPY_FLOAT
 * ufunc_radian_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_radian_types[3] = <char>NPY_FLOAT
 * ufunc_radian_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8975
 * ufunc_radian_types[1] = <char>NPY_FLOAT
 * ufunc_radian_types[2] = <char>NPY_FLOAT
 * ufunc_radian_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_radian_types[4] = <char>NPY_DOUBLE
 * ufunc_radian_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8976
 * ufunc_radian_types[2] = <char>NPY_FLOAT
 * ufunc_radian_types[3] = <char>NPY_FLOAT
 * ufunc_radian_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_radian_types[5] = <char>NPY_DOUBLE
 * ufunc_radian_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8977
 * ufunc_radian_types[3] = <char>NPY_FLOAT
 * ufunc_radian_types[4] = <char>NPY_DOUBLE
 * ufunc_radian_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_radian_types[6] = <char>NPY_DOUBLE
 * ufunc_radian_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8978
 * ufunc_radian_types[4] = <char>NPY_DOUBLE
 * ufunc_radian_types[5] = <char>NPY_DOUBLE
 * ufunc_radian_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_radian_types[7] = <char>NPY_DOUBLE
 * ufunc_radian_ptr[2*0] = <void*>_func_radian
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8979
 * ufunc_radian_types[5] = <char>NPY_DOUBLE
 * ufunc_radian_types[6] = <char>NPY_DOUBLE
 * ufunc_radian_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_radian_ptr[2*0] = <void*>_func_radian
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8980
 * ufunc_radian_types[6] = <char>NPY_DOUBLE
 * ufunc_radian_types[7] = <char>NPY_DOUBLE
 * ufunc_radian_ptr[2*0] = <void*>_func_radian             # <<<<<<<<<<<<<<
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")
 * ufunc_radian_ptr[2*1] = <void*>_func_radian
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[0]) = ((void *)radian);

  /* "scipy/special/_ufuncs.pyx":8981
 * ufunc_radian_types[7] = <char>NPY_DOUBLE
 * ufunc_radian_ptr[2*0] = <void*>_func_radian
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")             # <<<<<<<<<<<<<<
 * ufunc_radian_ptr[2*1] = <void*>_func_radian
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[1]) = ((void *)((char *)__pyx_k_radian));

  /* "scipy/special/_ufuncs.pyx":8982
 * ufunc_radian_ptr[2*0] = <void*>_func_radian
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")
 * ufunc_radian_ptr[2*1] = <void*>_func_radian             # <<<<<<<<<<<<<<
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[2]) = ((void *)radian);

  /* "scipy/special/_ufuncs.pyx":8983
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")
 * ufunc_radian_ptr[2*1] = <void*>_func_radian
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")             # <<<<<<<<<<<<<<
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]
 * ufunc_radian_data[1] = &ufunc_radian_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[3]) = ((void *)((char *)__pyx_k_radian));

  /* "scipy/special/_ufuncs.pyx":8984
 * ufunc_radian_ptr[2*1] = <void*>_func_radian
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_radian_data[1] = &ufunc_radian_ptr[2*1]
 * radian = np.PyUFunc_FromFuncAndData(ufunc_radian_loops, ufunc_radian_data, ufunc_radian_types, 2, 3, 1, 0, "radian", ufunc_radian_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8985
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]
 * ufunc_radian_data[1] = &ufunc_radian_ptr[2*1]             # <<<<<<<<<<<<<<
 * radian = np.PyUFunc_FromFuncAndData(ufunc_radian_loops, ufunc_radian_data, ufunc_radian_types, 2, 3, 1, 0, "radian", ufunc_radian_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8986
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]
 * ufunc_radian_data[1] = &ufunc_radian_ptr[2*1]
 * radian = np.PyUFunc_FromFuncAndData(ufunc_radian_loops, ufunc_radian_data, ufunc_radian_types, 2, 3, 1, 0, "radian", ufunc_radian_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_rel_entr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types, 2, 3, 1, 0, __pyx_k_radian, __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8986; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_radian, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8986; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":8992
 * cdef void *ufunc_rel_entr_data[2]
 * cdef char ufunc_rel_entr_types[6]
 * cdef char *ufunc_rel_entr_doc = (             # <<<<<<<<<<<<<<
 *     "rel_entr(x, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_doc = __pyx_k_rel_entr_x_y_Elementwise_functio;

  /* "scipy/special/_ufuncs.pyx":9020
 *     "\n"
 *     ".. versionadded:: 0.14.0")
 * ufunc_rel_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9021
 *     ".. versionadded:: 0.14.0")
 * ufunc_rel_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_rel_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9022
 * ufunc_rel_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_rel_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9023
 * ufunc_rel_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9024
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9025
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9026
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9027
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9028
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_rel_entr);

  /* "scipy/special/_ufuncs.pyx":9029
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[1]) = ((void *)((char *)__pyx_k_rel_entr));

  /* "scipy/special/_ufuncs.pyx":9030
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_rel_entr);

  /* "scipy/special/_ufuncs.pyx":9031
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]
 * ufunc_rel_entr_data[1] = &ufunc_rel_entr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[3]) = ((void *)((char *)__pyx_k_rel_entr));

  /* "scipy/special/_ufuncs.pyx":9032
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_data[1] = &ufunc_rel_entr_ptr[2*1]
 * rel_entr = np.PyUFunc_FromFuncAndData(ufunc_rel_entr_loops, ufunc_rel_entr_data, ufunc_rel_entr_types, 2, 2, 1, 0, "rel_entr", ufunc_rel_entr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9033
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]
 * ufunc_rel_entr_data[1] = &ufunc_rel_entr_ptr[2*1]             # <<<<<<<<<<<<<<
 * rel_entr = np.PyUFunc_FromFuncAndData(ufunc_rel_entr_loops, ufunc_rel_entr_data, ufunc_rel_entr_types, 2, 2, 1, 0, "rel_entr", ufunc_rel_entr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9034
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]
 * ufunc_rel_entr_data[1] = &ufunc_rel_entr_ptr[2*1]
 * rel_entr = np.PyUFunc_FromFuncAndData(ufunc_rel_entr_loops, ufunc_rel_entr_data, ufunc_rel_entr_types, 2, 2, 1, 0, "rel_entr", ufunc_rel_entr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_rgamma_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types, 2, 2, 1, 0, __pyx_k_rel_entr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_rel_entr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9040
 * cdef void *ufunc_rgamma_data[4]
 * cdef char ufunc_rgamma_types[8]
 * cdef char *ufunc_rgamma_doc = (             # <<<<<<<<<<<<<<
 *     "rgamma(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_doc = __pyx_k_rgamma_z_Gamma_function_inverted;

  /* "scipy/special/_ufuncs.pyx":9046
 *     "\n"
 *     "Returns ``1/gamma(x)``")
 * ufunc_rgamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_rgamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9047
 *     "Returns ``1/gamma(x)``")
 * ufunc_rgamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_rgamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9048
 * ufunc_rgamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_rgamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":9049
 * ufunc_rgamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":9050
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9051
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9052
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9053
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9054
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9055
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9056
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9057
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9058
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[0]) = ((void *)rgamma);

  /* "scipy/special/_ufuncs.pyx":9059
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[1]) = ((void *)((char *)__pyx_k_rgamma));

  /* "scipy/special/_ufuncs.pyx":9060
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[2]) = ((void *)rgamma);

  /* "scipy/special/_ufuncs.pyx":9061
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma_wrap
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[3]) = ((void *)((char *)__pyx_k_rgamma));

  /* "scipy/special/_ufuncs.pyx":9062
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma_wrap             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[4]) = ((void *)crgamma_wrap);

  /* "scipy/special/_ufuncs.pyx":9063
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma_wrap
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma_wrap
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[5]) = ((void *)((char *)__pyx_k_rgamma));

  /* "scipy/special/_ufuncs.pyx":9064
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma_wrap
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma_wrap             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[6]) = ((void *)crgamma_wrap);

  /* "scipy/special/_ufuncs.pyx":9065
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma_wrap
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")             # <<<<<<<<<<<<<<
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[7]) = ((void *)((char *)__pyx_k_rgamma));

  /* "scipy/special/_ufuncs.pyx":9066
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma_wrap
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9067
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]
 * ufunc_rgamma_data[3] = &ufunc_rgamma_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9068
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_rgamma_data[3] = &ufunc_rgamma_ptr[2*3]
 * rgamma = np.PyUFunc_FromFuncAndData(ufunc_rgamma_loops, ufunc_rgamma_data, ufunc_rgamma_types, 4, 1, 1, 0, "rgamma", ufunc_rgamma_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9069
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]
 * ufunc_rgamma_data[3] = &ufunc_rgamma_ptr[2*3]             # <<<<<<<<<<<<<<
 * rgamma = np.PyUFunc_FromFuncAndData(ufunc_rgamma_loops, ufunc_rgamma_data, ufunc_rgamma_types, 4, 1, 1, 0, "rgamma", ufunc_rgamma_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":9070
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]
 * ufunc_rgamma_data[3] = &ufunc_rgamma_ptr[2*3]
 * rgamma = np.PyUFunc_FromFuncAndData(ufunc_rgamma_loops, ufunc_rgamma_data, ufunc_rgamma_types, 4, 1, 1, 0, "rgamma", ufunc_rgamma_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_round_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types, 4, 1, 1, 0, __pyx_k_rgamma, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_rgamma, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9076
 * cdef void *ufunc_round_data[2]
 * cdef char ufunc_round_types[4]
 * cdef char *ufunc_round_doc = (             # <<<<<<<<<<<<<<
 *     "round(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_doc = __pyx_k_round_x_Round_to_nearest_integer;

  /* "scipy/special/_ufuncs.pyx":9084
 *     "point result.  If `x` ends in 0.5 exactly, the nearest even integer\n"
 *     "is chosen.")
 * ufunc_round_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_round_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_round_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9085
 *     "is chosen.")
 * ufunc_round_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_round_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_round_types[0] = <char>NPY_FLOAT
 * ufunc_round_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9086
 * ufunc_round_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_round_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_round_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_round_types[1] = <char>NPY_FLOAT
 * ufunc_round_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9087
 * ufunc_round_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_round_types[0] = <char>NPY_FLOAT
 * ufunc_round_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_round_types[2] = <char>NPY_DOUBLE
 * ufunc_round_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9088
 * ufunc_round_types[0] = <char>NPY_FLOAT
 * ufunc_round_types[1] = <char>NPY_FLOAT
 * ufunc_round_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_round_types[3] = <char>NPY_DOUBLE
 * ufunc_round_ptr[2*0] = <void*>_func_round
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9089
 * ufunc_round_types[1] = <char>NPY_FLOAT
 * ufunc_round_types[2] = <char>NPY_DOUBLE
 * ufunc_round_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_round_ptr[2*0] = <void*>_func_round
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9090
 * ufunc_round_types[2] = <char>NPY_DOUBLE
 * ufunc_round_types[3] = <char>NPY_DOUBLE
 * ufunc_round_ptr[2*0] = <void*>_func_round             # <<<<<<<<<<<<<<
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")
 * ufunc_round_ptr[2*1] = <void*>_func_round
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[0]) = ((void *)round);

  /* "scipy/special/_ufuncs.pyx":9091
 * ufunc_round_types[3] = <char>NPY_DOUBLE
 * ufunc_round_ptr[2*0] = <void*>_func_round
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")             # <<<<<<<<<<<<<<
 * ufunc_round_ptr[2*1] = <void*>_func_round
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[1]) = ((void *)((char *)__pyx_k_round));

  /* "scipy/special/_ufuncs.pyx":9092
 * ufunc_round_ptr[2*0] = <void*>_func_round
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")
 * ufunc_round_ptr[2*1] = <void*>_func_round             # <<<<<<<<<<<<<<
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[2]) = ((void *)round);

  /* "scipy/special/_ufuncs.pyx":9093
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")
 * ufunc_round_ptr[2*1] = <void*>_func_round
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")             # <<<<<<<<<<<<<<
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]
 * ufunc_round_data[1] = &ufunc_round_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[3]) = ((void *)((char *)__pyx_k_round));

  /* "scipy/special/_ufuncs.pyx":9094
 * ufunc_round_ptr[2*1] = <void*>_func_round
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_round_data[1] = &ufunc_round_ptr[2*1]
 * round = np.PyUFunc_FromFuncAndData(ufunc_round_loops, ufunc_round_data, ufunc_round_types, 2, 1, 1, 0, "round", ufunc_round_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9095
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]
 * ufunc_round_data[1] = &ufunc_round_ptr[2*1]             # <<<<<<<<<<<<<<
 * round = np.PyUFunc_FromFuncAndData(ufunc_round_loops, ufunc_round_data, ufunc_round_types, 2, 1, 1, 0, "round", ufunc_round_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9096
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]
 * ufunc_round_data[1] = &ufunc_round_ptr[2*1]
 * round = np.PyUFunc_FromFuncAndData(ufunc_round_loops, ufunc_round_data, ufunc_round_types, 2, 1, 1, 0, "round", ufunc_round_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_shichi_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types, 2, 1, 1, 0, __pyx_k_round, __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9096; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_round, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9096; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9102
 * cdef void *ufunc_shichi_data[2]
 * cdef char ufunc_shichi_types[6]
 * cdef char *ufunc_shichi_doc = (             # <<<<<<<<<<<<<<
 *     "shichi(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_doc = __pyx_k_shichi_x_Hyperbolic_sine_and_cos;

  /* "scipy/special/_ufuncs.pyx":9114
 *     "    ``eul + ln x + integral((cosh(t)-1)/t, t=0..x)``\n"
 *     "    where ``eul`` is Euler's constant.")
 * ufunc_shichi_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_shichi_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_shichi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":9115
 *     "    where ``eul`` is Euler's constant.")
 * ufunc_shichi_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_shichi_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[0] = <char>NPY_FLOAT
 * ufunc_shichi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":9116
 * ufunc_shichi_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_shichi_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_shichi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[1] = <char>NPY_FLOAT
 * ufunc_shichi_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9117
 * ufunc_shichi_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_shichi_types[0] = <char>NPY_FLOAT
 * ufunc_shichi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[2] = <char>NPY_FLOAT
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9118
 * ufunc_shichi_types[0] = <char>NPY_FLOAT
 * ufunc_shichi_types[1] = <char>NPY_FLOAT
 * ufunc_shichi_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9119
 * ufunc_shichi_types[1] = <char>NPY_FLOAT
 * ufunc_shichi_types[2] = <char>NPY_FLOAT
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9120
 * ufunc_shichi_types[2] = <char>NPY_FLOAT
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9121
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9122
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[0]) = ((void *)shichi);

  /* "scipy/special/_ufuncs.pyx":9123
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[1]) = ((void *)((char *)__pyx_k_shichi));

  /* "scipy/special/_ufuncs.pyx":9124
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[2]) = ((void *)shichi);

  /* "scipy/special/_ufuncs.pyx":9125
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")             # <<<<<<<<<<<<<<
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[3]) = ((void *)((char *)__pyx_k_shichi));

  /* "scipy/special/_ufuncs.pyx":9126
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]
 * shichi = np.PyUFunc_FromFuncAndData(ufunc_shichi_loops, ufunc_shichi_data, ufunc_shichi_types, 2, 1, 2, 0, "shichi", ufunc_shichi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9127
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]             # <<<<<<<<<<<<<<
 * shichi = np.PyUFunc_FromFuncAndData(ufunc_shichi_loops, ufunc_shichi_data, ufunc_shichi_types, 2, 1, 2, 0, "shichi", ufunc_shichi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9128
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]
 * shichi = np.PyUFunc_FromFuncAndData(ufunc_shichi_loops, ufunc_shichi_data, ufunc_shichi_types, 2, 1, 2, 0, "shichi", ufunc_shichi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_sici_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types, 2, 1, 2, 0, __pyx_k_shichi, __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_shichi, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9134
 * cdef void *ufunc_sici_data[2]
 * cdef char ufunc_sici_types[6]
 * cdef char *ufunc_sici_doc = (             # <<<<<<<<<<<<<<
 *     "sici(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_doc = __pyx_k_sici_x_Sine_and_cosine_integrals;

  /* "scipy/special/_ufuncs.pyx":9146
 *     "    ``eul + ln x + integral((cos(t) - 1)/t, t=0..x)``\n"
 *     "    where ``eul`` is Euler's constant.")
 * ufunc_sici_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_sici_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_sici_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":9147
 *     "    where ``eul`` is Euler's constant.")
 * ufunc_sici_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_sici_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_sici_types[0] = <char>NPY_FLOAT
 * ufunc_sici_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":9148
 * ufunc_sici_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_sici_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_sici_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[1] = <char>NPY_FLOAT
 * ufunc_sici_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9149
 * ufunc_sici_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_sici_types[0] = <char>NPY_FLOAT
 * ufunc_sici_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[2] = <char>NPY_FLOAT
 * ufunc_sici_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9150
 * ufunc_sici_types[0] = <char>NPY_FLOAT
 * ufunc_sici_types[1] = <char>NPY_FLOAT
 * ufunc_sici_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[3] = <char>NPY_DOUBLE
 * ufunc_sici_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9151
 * ufunc_sici_types[1] = <char>NPY_FLOAT
 * ufunc_sici_types[2] = <char>NPY_FLOAT
 * ufunc_sici_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_types[4] = <char>NPY_DOUBLE
 * ufunc_sici_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9152
 * ufunc_sici_types[2] = <char>NPY_FLOAT
 * ufunc_sici_types[3] = <char>NPY_DOUBLE
 * ufunc_sici_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_types[5] = <char>NPY_DOUBLE
 * ufunc_sici_ptr[2*0] = <void*>_func_sici
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9153
 * ufunc_sici_types[3] = <char>NPY_DOUBLE
 * ufunc_sici_types[4] = <char>NPY_DOUBLE
 * ufunc_sici_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*0] = <void*>_func_sici
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9154
 * ufunc_sici_types[4] = <char>NPY_DOUBLE
 * ufunc_sici_types[5] = <char>NPY_DOUBLE
 * ufunc_sici_ptr[2*0] = <void*>_func_sici             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*1] = <void*>_func_sici
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[0]) = ((void *)sici);

  /* "scipy/special/_ufuncs.pyx":9155
 * ufunc_sici_types[5] = <char>NPY_DOUBLE
 * ufunc_sici_ptr[2*0] = <void*>_func_sici
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*1] = <void*>_func_sici
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[1]) = ((void *)((char *)__pyx_k_sici));

  /* "scipy/special/_ufuncs.pyx":9156
 * ufunc_sici_ptr[2*0] = <void*>_func_sici
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*1] = <void*>_func_sici             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[2]) = ((void *)sici);

  /* "scipy/special/_ufuncs.pyx":9157
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*1] = <void*>_func_sici
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")             # <<<<<<<<<<<<<<
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[3]) = ((void *)((char *)__pyx_k_sici));

  /* "scipy/special/_ufuncs.pyx":9158
 * ufunc_sici_ptr[2*1] = <void*>_func_sici
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]
 * sici = np.PyUFunc_FromFuncAndData(ufunc_sici_loops, ufunc_sici_data, ufunc_sici_types, 2, 1, 2, 0, "sici", ufunc_sici_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9159
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]             # <<<<<<<<<<<<<<
 * sici = np.PyUFunc_FromFuncAndData(ufunc_sici_loops, ufunc_sici_data, ufunc_sici_types, 2, 1, 2, 0, "sici", ufunc_sici_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9160
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]
 * sici = np.PyUFunc_FromFuncAndData(ufunc_sici_loops, ufunc_sici_data, ufunc_sici_types, 2, 1, 2, 0, "sici", ufunc_sici_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_sindg_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types, 2, 1, 2, 0, __pyx_k_sici, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sici, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9166
 * cdef void *ufunc_sindg_data[2]
 * cdef char ufunc_sindg_types[4]
 * cdef char *ufunc_sindg_doc = (             # <<<<<<<<<<<<<<
 *     "sindg(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_doc = __pyx_k_sindg_x_Sine_of_angle_given_in_d;

  /* "scipy/special/_ufuncs.pyx":9170
 *     "\n"
 *     "Sine of angle given in degrees")
 * ufunc_sindg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_sindg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_sindg_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9171
 *     "Sine of angle given in degrees")
 * ufunc_sindg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_sindg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_sindg_types[0] = <char>NPY_FLOAT
 * ufunc_sindg_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9172
 * ufunc_sindg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_sindg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_sindg_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sindg_types[1] = <char>NPY_FLOAT
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9173
 * ufunc_sindg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_sindg_types[0] = <char>NPY_FLOAT
 * ufunc_sindg_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9174
 * ufunc_sindg_types[0] = <char>NPY_FLOAT
 * ufunc_sindg_types[1] = <char>NPY_FLOAT
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9175
 * ufunc_sindg_types[1] = <char>NPY_FLOAT
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9176
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg             # <<<<<<<<<<<<<<
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[0]) = ((void *)sindg);

  /* "scipy/special/_ufuncs.pyx":9177
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")             # <<<<<<<<<<<<<<
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[1]) = ((void *)((char *)__pyx_k_sindg));

  /* "scipy/special/_ufuncs.pyx":9178
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg             # <<<<<<<<<<<<<<
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[2]) = ((void *)sindg);

  /* "scipy/special/_ufuncs.pyx":9179
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")             # <<<<<<<<<<<<<<
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]
 * ufunc_sindg_data[1] = &ufunc_sindg_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[3]) = ((void *)((char *)__pyx_k_sindg));

  /* "scipy/special/_ufuncs.pyx":9180
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_sindg_data[1] = &ufunc_sindg_ptr[2*1]
 * sindg = np.PyUFunc_FromFuncAndData(ufunc_sindg_loops, ufunc_sindg_data, ufunc_sindg_types, 2, 1, 1, 0, "sindg", ufunc_sindg_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9181
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]
 * ufunc_sindg_data[1] = &ufunc_sindg_ptr[2*1]             # <<<<<<<<<<<<<<
 * sindg = np.PyUFunc_FromFuncAndData(ufunc_sindg_loops, ufunc_sindg_data, ufunc_sindg_types, 2, 1, 1, 0, "sindg", ufunc_sindg_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9182
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]
 * ufunc_sindg_data[1] = &ufunc_sindg_ptr[2*1]
 * sindg = np.PyUFunc_FromFuncAndData(ufunc_sindg_loops, ufunc_sindg_data, ufunc_sindg_types, 2, 1, 1, 0, "sindg", ufunc_sindg_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_smirnov_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types, 2, 1, 1, 0, __pyx_k_sindg, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9182; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sindg, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9182; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9188
 * cdef void *ufunc_smirnov_data[3]
 * cdef char ufunc_smirnov_types[9]
 * cdef char *ufunc_smirnov_doc = (             # <<<<<<<<<<<<<<
 *     "smirnov(n, e)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_doc = __pyx_k_smirnov_n_e_Kolmogorov_Smirnov_c;

  /* "scipy/special/_ufuncs.pyx":9199
 *     "theoretical distribution and an empirical one based on `n` samples\n"
 *     "is greater than e.")
 * ufunc_smirnov_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_smirnov_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":9200
 *     "is greater than e.")
 * ufunc_smirnov_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_smirnov_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnov_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9201
 * ufunc_smirnov_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_smirnov_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[0] = <char>NPY_LONG
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9202
 * ufunc_smirnov_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnov_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":9203
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnov_types[0] = <char>NPY_LONG
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9204
 * ufunc_smirnov_types[0] = <char>NPY_LONG
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9205
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9206
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9207
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9208
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9209
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9210
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9211
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[0]) = ((void *)smirnov);

  /* "scipy/special/_ufuncs.pyx":9212
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[1]) = ((void *)((char *)__pyx_k_smirnov));

  /* "scipy/special/_ufuncs.pyx":9213
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnov_unsafe);

  /* "scipy/special/_ufuncs.pyx":9214
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[3]) = ((void *)((char *)__pyx_k_smirnov));

  /* "scipy/special/_ufuncs.pyx":9215
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnov_unsafe);

  /* "scipy/special/_ufuncs.pyx":9216
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")             # <<<<<<<<<<<<<<
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[5]) = ((void *)((char *)__pyx_k_smirnov));

  /* "scipy/special/_ufuncs.pyx":9217
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]
 * ufunc_smirnov_data[2] = &ufunc_smirnov_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9218
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_smirnov_data[2] = &ufunc_smirnov_ptr[2*2]
 * smirnov = np.PyUFunc_FromFuncAndData(ufunc_smirnov_loops, ufunc_smirnov_data, ufunc_smirnov_types, 3, 2, 1, 0, "smirnov", ufunc_smirnov_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9219
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]
 * ufunc_smirnov_data[2] = &ufunc_smirnov_ptr[2*2]             # <<<<<<<<<<<<<<
 * smirnov = np.PyUFunc_FromFuncAndData(ufunc_smirnov_loops, ufunc_smirnov_data, ufunc_smirnov_types, 3, 2, 1, 0, "smirnov", ufunc_smirnov_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9220
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]
 * ufunc_smirnov_data[2] = &ufunc_smirnov_ptr[2*2]
 * smirnov = np.PyUFunc_FromFuncAndData(ufunc_smirnov_loops, ufunc_smirnov_data, ufunc_smirnov_types, 3, 2, 1, 0, "smirnov", ufunc_smirnov_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_smirnovi_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types, 3, 2, 1, 0, __pyx_k_smirnov, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_smirnov, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9226
 * cdef void *ufunc_smirnovi_data[3]
 * cdef char ufunc_smirnovi_types[9]
 * cdef char *ufunc_smirnovi_doc = (             # <<<<<<<<<<<<<<
 *     "smirnovi(n, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_doc = __pyx_k_smirnovi_n_y_Inverse_to_smirnov;

  /* "scipy/special/_ufuncs.pyx":9232
 *     "\n"
 *     "Returns ``e`` such that ``smirnov(n, e) = y``.")
 * ufunc_smirnovi_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":9233
 *     "Returns ``e`` such that ``smirnov(n, e) = y``.")
 * ufunc_smirnovi_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_smirnovi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnovi_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9234
 * ufunc_smirnovi_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_smirnovi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[0] = <char>NPY_LONG
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9235
 * ufunc_smirnovi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnovi_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":9236
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnovi_types[0] = <char>NPY_LONG
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9237
 * ufunc_smirnovi_types[0] = <char>NPY_LONG
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9238
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9239
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9240
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9241
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9242
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9243
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9244
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[0]) = ((void *)smirnovi);

  /* "scipy/special/_ufuncs.pyx":9245
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[1]) = ((void *)((char *)__pyx_k_smirnovi));

  /* "scipy/special/_ufuncs.pyx":9246
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe);

  /* "scipy/special/_ufuncs.pyx":9247
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[3]) = ((void *)((char *)__pyx_k_smirnovi));

  /* "scipy/special/_ufuncs.pyx":9248
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe);

  /* "scipy/special/_ufuncs.pyx":9249
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[5]) = ((void *)((char *)__pyx_k_smirnovi));

  /* "scipy/special/_ufuncs.pyx":9250
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]
 * ufunc_smirnovi_data[2] = &ufunc_smirnovi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9251
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_data[2] = &ufunc_smirnovi_ptr[2*2]
 * smirnovi = np.PyUFunc_FromFuncAndData(ufunc_smirnovi_loops, ufunc_smirnovi_data, ufunc_smirnovi_types, 3, 2, 1, 0, "smirnovi", ufunc_smirnovi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9252
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]
 * ufunc_smirnovi_data[2] = &ufunc_smirnovi_ptr[2*2]             # <<<<<<<<<<<<<<
 * smirnovi = np.PyUFunc_FromFuncAndData(ufunc_smirnovi_loops, ufunc_smirnovi_data, ufunc_smirnovi_types, 3, 2, 1, 0, "smirnovi", ufunc_smirnovi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9253
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]
 * ufunc_smirnovi_data[2] = &ufunc_smirnovi_ptr[2*2]
 * smirnovi = np.PyUFunc_FromFuncAndData(ufunc_smirnovi_loops, ufunc_smirnovi_data, ufunc_smirnovi_types, 3, 2, 1, 0, "smirnovi", ufunc_smirnovi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_spence_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types, 3, 2, 1, 0, __pyx_k_smirnovi, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_smirnovi, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9259
 * cdef void *ufunc_spence_data[2]
 * cdef char ufunc_spence_types[4]
 * cdef char *ufunc_spence_doc = (             # <<<<<<<<<<<<<<
 *     "spence(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_doc = __pyx_k_spence_x_Dilogarithm_integral_Re;

  /* "scipy/special/_ufuncs.pyx":9267
 *     "\n"
 *     "    -integral(log t / (t-1), t=1..x)")
 * ufunc_spence_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_spence_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_spence_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9268
 *     "    -integral(log t / (t-1), t=1..x)")
 * ufunc_spence_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_spence_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_spence_types[0] = <char>NPY_FLOAT
 * ufunc_spence_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9269
 * ufunc_spence_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_spence_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_spence_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_spence_types[1] = <char>NPY_FLOAT
 * ufunc_spence_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9270
 * ufunc_spence_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_spence_types[0] = <char>NPY_FLOAT
 * ufunc_spence_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_spence_types[2] = <char>NPY_DOUBLE
 * ufunc_spence_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9271
 * ufunc_spence_types[0] = <char>NPY_FLOAT
 * ufunc_spence_types[1] = <char>NPY_FLOAT
 * ufunc_spence_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_spence_types[3] = <char>NPY_DOUBLE
 * ufunc_spence_ptr[2*0] = <void*>_func_spence
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9272
 * ufunc_spence_types[1] = <char>NPY_FLOAT
 * ufunc_spence_types[2] = <char>NPY_DOUBLE
 * ufunc_spence_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*0] = <void*>_func_spence
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9273
 * ufunc_spence_types[2] = <char>NPY_DOUBLE
 * ufunc_spence_types[3] = <char>NPY_DOUBLE
 * ufunc_spence_ptr[2*0] = <void*>_func_spence             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*1] = <void*>_func_spence
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[0]) = ((void *)spence);

  /* "scipy/special/_ufuncs.pyx":9274
 * ufunc_spence_types[3] = <char>NPY_DOUBLE
 * ufunc_spence_ptr[2*0] = <void*>_func_spence
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*1] = <void*>_func_spence
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[1]) = ((void *)((char *)__pyx_k_spence));

  /* "scipy/special/_ufuncs.pyx":9275
 * ufunc_spence_ptr[2*0] = <void*>_func_spence
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*1] = <void*>_func_spence             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[2]) = ((void *)spence);

  /* "scipy/special/_ufuncs.pyx":9276
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*1] = <void*>_func_spence
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")             # <<<<<<<<<<<<<<
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[3]) = ((void *)((char *)__pyx_k_spence));

  /* "scipy/special/_ufuncs.pyx":9277
 * ufunc_spence_ptr[2*1] = <void*>_func_spence
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]
 * spence = np.PyUFunc_FromFuncAndData(ufunc_spence_loops, ufunc_spence_data, ufunc_spence_types, 2, 1, 1, 0, "spence", ufunc_spence_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9278
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]             # <<<<<<<<<<<<<<
 * spence = np.PyUFunc_FromFuncAndData(ufunc_spence_loops, ufunc_spence_data, ufunc_spence_types, 2, 1, 1, 0, "spence", ufunc_spence_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9279
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]
 * spence = np.PyUFunc_FromFuncAndData(ufunc_spence_loops, ufunc_spence_data, ufunc_spence_types, 2, 1, 1, 0, "spence", ufunc_spence_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_sph_harm_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types, 2, 1, 1, 0, __pyx_k_spence, __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spence, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9285
 * cdef void *ufunc_sph_harm_data[3]
 * cdef char ufunc_sph_harm_types[15]
 * cdef char *ufunc_sph_harm_doc = (             # <<<<<<<<<<<<<<
 *     "sph_harm(m, n, theta, phi)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_doc = __pyx_k_sph_harm_m_n_theta_phi_Compute_s;

  /* "scipy/special/_ufuncs.pyx":9320
 *     "----------\n"
 *     ".. [1] Digital Library of Mathematical Functions, 14.30. http://dlmf.nist.gov/14.30")
 * ufunc_sph_harm_loops[0] = <np.PyUFuncGenericFunction>loop_D_iidd__As_lldd_D             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddd__As_ffff_F
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_iidd__As_lldd_D);

  /* "scipy/special/_ufuncs.pyx":9321
 *     ".. [1] Digital Library of Mathematical Functions, 14.30. http://dlmf.nist.gov/14.30")
 * ufunc_sph_harm_loops[0] = <np.PyUFuncGenericFunction>loop_D_iidd__As_lldd_D
 * ufunc_sph_harm_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddd__As_ffff_F             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D
 * ufunc_sph_harm_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_ffff_F);

  /* "scipy/special/_ufuncs.pyx":9322
 * ufunc_sph_harm_loops[0] = <np.PyUFuncGenericFunction>loop_D_iidd__As_lldd_D
 * ufunc_sph_harm_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddd__As_ffff_F
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[0] = <char>NPY_LONG
 * ufunc_sph_harm_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_dddd_D);

  /* "scipy/special/_ufuncs.pyx":9323
 * ufunc_sph_harm_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddd__As_ffff_F
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D
 * ufunc_sph_harm_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[1] = <char>NPY_LONG
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":9324
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D
 * ufunc_sph_harm_types[0] = <char>NPY_LONG
 * ufunc_sph_harm_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":9325
 * ufunc_sph_harm_types[0] = <char>NPY_LONG
 * ufunc_sph_harm_types[1] = <char>NPY_LONG
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9326
 * ufunc_sph_harm_types[1] = <char>NPY_LONG
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9327
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9328
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9329
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9330
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9331
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9332
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[9]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9333
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9334
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9335
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9336
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9337
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9338
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[0]) = ((void *)__pyx_f_5scipy_7special_8sph_harm_sph_harmonic);

  /* "scipy/special/_ufuncs.pyx":9339
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[1]) = ((void *)((char *)__pyx_k_sph_harm));

  /* "scipy/special/_ufuncs.pyx":9340
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":9341
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[3]) = ((void *)((char *)__pyx_k_sph_harm));

  /* "scipy/special/_ufuncs.pyx":9342
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":9343
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[5]) = ((void *)((char *)__pyx_k_sph_harm));

  /* "scipy/special/_ufuncs.pyx":9344
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]
 * ufunc_sph_harm_data[2] = &ufunc_sph_harm_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9345
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_data[2] = &ufunc_sph_harm_ptr[2*2]
 * sph_harm = np.PyUFunc_FromFuncAndData(ufunc_sph_harm_loops, ufunc_sph_harm_data, ufunc_sph_harm_types, 3, 4, 1, 0, "sph_harm", ufunc_sph_harm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9346
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]
 * ufunc_sph_harm_data[2] = &ufunc_sph_harm_ptr[2*2]             # <<<<<<<<<<<<<<
 * sph_harm = np.PyUFunc_FromFuncAndData(ufunc_sph_harm_loops, ufunc_sph_harm_data, ufunc_sph_harm_types, 3, 4, 1, 0, "sph_harm", ufunc_sph_harm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9347
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]
 * ufunc_sph_harm_data[2] = &ufunc_sph_harm_ptr[2*2]
 * sph_harm = np.PyUFunc_FromFuncAndData(ufunc_sph_harm_loops, ufunc_sph_harm_data, ufunc_sph_harm_types, 3, 4, 1, 0, "sph_harm", ufunc_sph_harm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_stdtr_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types, 3, 4, 1, 0, __pyx_k_sph_harm, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9347; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sph_harm, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9347; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9353
 * cdef void *ufunc_stdtr_data[2]
 * cdef char ufunc_stdtr_types[6]
 * cdef char *ufunc_stdtr_doc = (             # <<<<<<<<<<<<<<
 *     "stdtr(df, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_doc = __pyx_k_stdtr_df_t_Student_t_distributio;

  /* "scipy/special/_ufuncs.pyx":9363
 *     "   gamma((df+1)/2)/(sqrt(df*pi)*gamma(df/2)) *\n"
 *     "   integral((1+x**2/df)**(-df/2-1/2), x=-inf..t)")
 * ufunc_stdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_stdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9364
 *     "   integral((1+x**2/df)**(-df/2-1/2), x=-inf..t)")
 * ufunc_stdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9365
 * ufunc_stdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9366
 * ufunc_stdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9367
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9368
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9369
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9370
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9371
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[0]) = ((void *)cdft1_wrap);

  /* "scipy/special/_ufuncs.pyx":9372
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")             # <<<<<<<<<<<<<<
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[1]) = ((void *)((char *)__pyx_k_stdtr));

  /* "scipy/special/_ufuncs.pyx":9373
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[2]) = ((void *)cdft1_wrap);

  /* "scipy/special/_ufuncs.pyx":9374
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")             # <<<<<<<<<<<<<<
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]
 * ufunc_stdtr_data[1] = &ufunc_stdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[3]) = ((void *)((char *)__pyx_k_stdtr));

  /* "scipy/special/_ufuncs.pyx":9375
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_stdtr_data[1] = &ufunc_stdtr_ptr[2*1]
 * stdtr = np.PyUFunc_FromFuncAndData(ufunc_stdtr_loops, ufunc_stdtr_data, ufunc_stdtr_types, 2, 2, 1, 0, "stdtr", ufunc_stdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9376
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]
 * ufunc_stdtr_data[1] = &ufunc_stdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * stdtr = np.PyUFunc_FromFuncAndData(ufunc_stdtr_loops, ufunc_stdtr_data, ufunc_stdtr_types, 2, 2, 1, 0, "stdtr", ufunc_stdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9377
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]
 * ufunc_stdtr_data[1] = &ufunc_stdtr_ptr[2*1]
 * stdtr = np.PyUFunc_FromFuncAndData(ufunc_stdtr_loops, ufunc_stdtr_data, ufunc_stdtr_types, 2, 2, 1, 0, "stdtr", ufunc_stdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_stdtridf_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types, 2, 2, 1, 0, __pyx_k_stdtr, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_stdtr, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9383
 * cdef void *ufunc_stdtridf_data[2]
 * cdef char ufunc_stdtridf_types[6]
 * cdef char *ufunc_stdtridf_doc = (             # <<<<<<<<<<<<<<
 *     "stdtridf(p, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_doc = __pyx_k_stdtridf_p_t_Inverse_of_stdtr_vs;

  /* "scipy/special/_ufuncs.pyx":9389
 *     "\n"
 *     "Returns the argument df such that stdtr(df, t) is equal to `p`.")
 * ufunc_stdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9390
 *     "Returns the argument df such that stdtr(df, t) is equal to `p`.")
 * ufunc_stdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9391
 * ufunc_stdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9392
 * ufunc_stdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9393
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9394
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9395
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9396
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9397
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[0]) = ((void *)cdft3_wrap);

  /* "scipy/special/_ufuncs.pyx":9398
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[1]) = ((void *)((char *)__pyx_k_stdtridf));

  /* "scipy/special/_ufuncs.pyx":9399
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[2]) = ((void *)cdft3_wrap);

  /* "scipy/special/_ufuncs.pyx":9400
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]
 * ufunc_stdtridf_data[1] = &ufunc_stdtridf_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[3]) = ((void *)((char *)__pyx_k_stdtridf));

  /* "scipy/special/_ufuncs.pyx":9401
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_data[1] = &ufunc_stdtridf_ptr[2*1]
 * stdtridf = np.PyUFunc_FromFuncAndData(ufunc_stdtridf_loops, ufunc_stdtridf_data, ufunc_stdtridf_types, 2, 2, 1, 0, "stdtridf", ufunc_stdtridf_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9402
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]
 * ufunc_stdtridf_data[1] = &ufunc_stdtridf_ptr[2*1]             # <<<<<<<<<<<<<<
 * stdtridf = np.PyUFunc_FromFuncAndData(ufunc_stdtridf_loops, ufunc_stdtridf_data, ufunc_stdtridf_types, 2, 2, 1, 0, "stdtridf", ufunc_stdtridf_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9403
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]
 * ufunc_stdtridf_data[1] = &ufunc_stdtridf_ptr[2*1]
 * stdtridf = np.PyUFunc_FromFuncAndData(ufunc_stdtridf_loops, ufunc_stdtridf_data, ufunc_stdtridf_types, 2, 2, 1, 0, "stdtridf", ufunc_stdtridf_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_stdtrit_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types, 2, 2, 1, 0, __pyx_k_stdtridf, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_stdtridf, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9409
 * cdef void *ufunc_stdtrit_data[2]
 * cdef char ufunc_stdtrit_types[6]
 * cdef char *ufunc_stdtrit_doc = (             # <<<<<<<<<<<<<<
 *     "stdtrit(df, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_doc = __pyx_k_stdtrit_df_p_Inverse_of_stdtr_vs;

  /* "scipy/special/_ufuncs.pyx":9415
 *     "\n"
 *     "Returns the argument `t` such that stdtr(df, t) is equal to `p`.")
 * ufunc_stdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9416
 *     "Returns the argument `t` such that stdtr(df, t) is equal to `p`.")
 * ufunc_stdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9417
 * ufunc_stdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9418
 * ufunc_stdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9419
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9420
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9421
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9422
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9423
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[0]) = ((void *)cdft2_wrap);

  /* "scipy/special/_ufuncs.pyx":9424
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[1]) = ((void *)((char *)__pyx_k_stdtrit));

  /* "scipy/special/_ufuncs.pyx":9425
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[2]) = ((void *)cdft2_wrap);

  /* "scipy/special/_ufuncs.pyx":9426
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]
 * ufunc_stdtrit_data[1] = &ufunc_stdtrit_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[3]) = ((void *)((char *)__pyx_k_stdtrit));

  /* "scipy/special/_ufuncs.pyx":9427
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_data[1] = &ufunc_stdtrit_ptr[2*1]
 * stdtrit = np.PyUFunc_FromFuncAndData(ufunc_stdtrit_loops, ufunc_stdtrit_data, ufunc_stdtrit_types, 2, 2, 1, 0, "stdtrit", ufunc_stdtrit_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9428
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]
 * ufunc_stdtrit_data[1] = &ufunc_stdtrit_ptr[2*1]             # <<<<<<<<<<<<<<
 * stdtrit = np.PyUFunc_FromFuncAndData(ufunc_stdtrit_loops, ufunc_stdtrit_data, ufunc_stdtrit_types, 2, 2, 1, 0, "stdtrit", ufunc_stdtrit_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9429
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]
 * ufunc_stdtrit_data[1] = &ufunc_stdtrit_ptr[2*1]
 * stdtrit = np.PyUFunc_FromFuncAndData(ufunc_stdtrit_loops, ufunc_stdtrit_data, ufunc_stdtrit_types, 2, 2, 1, 0, "stdtrit", ufunc_stdtrit_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_struve_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types, 2, 2, 1, 0, __pyx_k_stdtrit, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9429; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_stdtrit, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9429; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9435
 * cdef void *ufunc_struve_data[2]
 * cdef char ufunc_struve_types[6]
 * cdef char *ufunc_struve_doc = (             # <<<<<<<<<<<<<<
 *     "struve(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_doc = __pyx_k_struve_v_x_Struve_function_Compu;

  /* "scipy/special/_ufuncs.pyx":9442
 *     "Computes the struve function Hv(x) of order `v` at `x`, `x` must be\n"
 *     "positive unless `v` is an integer.")
 * ufunc_struve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_struve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_struve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9443
 *     "positive unless `v` is an integer.")
 * ufunc_struve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_struve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_struve_types[0] = <char>NPY_FLOAT
 * ufunc_struve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9444
 * ufunc_struve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_struve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_struve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_struve_types[1] = <char>NPY_FLOAT
 * ufunc_struve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9445
 * ufunc_struve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_struve_types[0] = <char>NPY_FLOAT
 * ufunc_struve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_struve_types[2] = <char>NPY_FLOAT
 * ufunc_struve_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9446
 * ufunc_struve_types[0] = <char>NPY_FLOAT
 * ufunc_struve_types[1] = <char>NPY_FLOAT
 * ufunc_struve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_struve_types[3] = <char>NPY_DOUBLE
 * ufunc_struve_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9447
 * ufunc_struve_types[1] = <char>NPY_FLOAT
 * ufunc_struve_types[2] = <char>NPY_FLOAT
 * ufunc_struve_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_struve_types[4] = <char>NPY_DOUBLE
 * ufunc_struve_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9448
 * ufunc_struve_types[2] = <char>NPY_FLOAT
 * ufunc_struve_types[3] = <char>NPY_DOUBLE
 * ufunc_struve_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_struve_types[5] = <char>NPY_DOUBLE
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9449
 * ufunc_struve_types[3] = <char>NPY_DOUBLE
 * ufunc_struve_types[4] = <char>NPY_DOUBLE
 * ufunc_struve_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9450
 * ufunc_struve_types[4] = <char>NPY_DOUBLE
 * ufunc_struve_types[5] = <char>NPY_DOUBLE
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h             # <<<<<<<<<<<<<<
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[0]) = ((void *)struve_h);

  /* "scipy/special/_ufuncs.pyx":9451
 * ufunc_struve_types[5] = <char>NPY_DOUBLE
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")             # <<<<<<<<<<<<<<
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[1]) = ((void *)((char *)__pyx_k_struve));

  /* "scipy/special/_ufuncs.pyx":9452
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h             # <<<<<<<<<<<<<<
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[2]) = ((void *)struve_h);

  /* "scipy/special/_ufuncs.pyx":9453
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")             # <<<<<<<<<<<<<<
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]
 * ufunc_struve_data[1] = &ufunc_struve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[3]) = ((void *)((char *)__pyx_k_struve));

  /* "scipy/special/_ufuncs.pyx":9454
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_struve_data[1] = &ufunc_struve_ptr[2*1]
 * struve = np.PyUFunc_FromFuncAndData(ufunc_struve_loops, ufunc_struve_data, ufunc_struve_types, 2, 2, 1, 0, "struve", ufunc_struve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9455
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]
 * ufunc_struve_data[1] = &ufunc_struve_ptr[2*1]             # <<<<<<<<<<<<<<
 * struve = np.PyUFunc_FromFuncAndData(ufunc_struve_loops, ufunc_struve_data, ufunc_struve_types, 2, 2, 1, 0, "struve", ufunc_struve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9456
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]
 * ufunc_struve_data[1] = &ufunc_struve_ptr[2*1]
 * struve = np.PyUFunc_FromFuncAndData(ufunc_struve_loops, ufunc_struve_data, ufunc_struve_types, 2, 2, 1, 0, "struve", ufunc_struve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_tandg_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types, 2, 2, 1, 0, __pyx_k_struve, __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struve, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9462
 * cdef void *ufunc_tandg_data[2]
 * cdef char ufunc_tandg_types[4]
 * cdef char *ufunc_tandg_doc = (             # <<<<<<<<<<<<<<
 *     "tandg(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_doc = __pyx_k_tandg_x_Tangent_of_angle_x_given;

  /* "scipy/special/_ufuncs.pyx":9466
 *     "\n"
 *     "Tangent of angle x given in degrees.")
 * ufunc_tandg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_tandg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_tandg_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9467
 *     "Tangent of angle x given in degrees.")
 * ufunc_tandg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_tandg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_tandg_types[0] = <char>NPY_FLOAT
 * ufunc_tandg_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9468
 * ufunc_tandg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_tandg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_tandg_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tandg_types[1] = <char>NPY_FLOAT
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9469
 * ufunc_tandg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_tandg_types[0] = <char>NPY_FLOAT
 * ufunc_tandg_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9470
 * ufunc_tandg_types[0] = <char>NPY_FLOAT
 * ufunc_tandg_types[1] = <char>NPY_FLOAT
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9471
 * ufunc_tandg_types[1] = <char>NPY_FLOAT
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9472
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg             # <<<<<<<<<<<<<<
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[0]) = ((void *)tandg);

  /* "scipy/special/_ufuncs.pyx":9473
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")             # <<<<<<<<<<<<<<
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[1]) = ((void *)((char *)__pyx_k_tandg));

  /* "scipy/special/_ufuncs.pyx":9474
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg             # <<<<<<<<<<<<<<
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[2]) = ((void *)tandg);

  /* "scipy/special/_ufuncs.pyx":9475
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")             # <<<<<<<<<<<<<<
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]
 * ufunc_tandg_data[1] = &ufunc_tandg_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[3]) = ((void *)((char *)__pyx_k_tandg));

  /* "scipy/special/_ufuncs.pyx":9476
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_tandg_data[1] = &ufunc_tandg_ptr[2*1]
 * tandg = np.PyUFunc_FromFuncAndData(ufunc_tandg_loops, ufunc_tandg_data, ufunc_tandg_types, 2, 1, 1, 0, "tandg", ufunc_tandg_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9477
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]
 * ufunc_tandg_data[1] = &ufunc_tandg_ptr[2*1]             # <<<<<<<<<<<<<<
 * tandg = np.PyUFunc_FromFuncAndData(ufunc_tandg_loops, ufunc_tandg_data, ufunc_tandg_types, 2, 1, 1, 0, "tandg", ufunc_tandg_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9478
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]
 * ufunc_tandg_data[1] = &ufunc_tandg_ptr[2*1]
 * tandg = np.PyUFunc_FromFuncAndData(ufunc_tandg_loops, ufunc_tandg_data, ufunc_tandg_types, 2, 1, 1, 0, "tandg", ufunc_tandg_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_tklmbda_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types, 2, 1, 1, 0, __pyx_k_tandg, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_tandg, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9484
 * cdef void *ufunc_tklmbda_data[2]
 * cdef char ufunc_tklmbda_types[6]
 * cdef char *ufunc_tklmbda_doc = (             # <<<<<<<<<<<<<<
 *     "tklmbda(x, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_doc = __pyx_k_tklmbda_x_lmbda_Tukey_Lambda_cum;

  /* "scipy/special/_ufuncs.pyx":9488
 *     "\n"
 *     "Tukey-Lambda cumulative distribution function")
 * ufunc_tklmbda_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9489
 *     "Tukey-Lambda cumulative distribution function")
 * ufunc_tklmbda_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_tklmbda_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9490
 * ufunc_tklmbda_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_tklmbda_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9491
 * ufunc_tklmbda_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9492
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9493
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9494
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9495
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9496
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[0]) = ((void *)tukeylambdacdf);

  /* "scipy/special/_ufuncs.pyx":9497
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[1]) = ((void *)((char *)__pyx_k_tklmbda));

  /* "scipy/special/_ufuncs.pyx":9498
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[2]) = ((void *)tukeylambdacdf);

  /* "scipy/special/_ufuncs.pyx":9499
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]
 * ufunc_tklmbda_data[1] = &ufunc_tklmbda_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[3]) = ((void *)((char *)__pyx_k_tklmbda));

  /* "scipy/special/_ufuncs.pyx":9500
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_data[1] = &ufunc_tklmbda_ptr[2*1]
 * tklmbda = np.PyUFunc_FromFuncAndData(ufunc_tklmbda_loops, ufunc_tklmbda_data, ufunc_tklmbda_types, 2, 2, 1, 0, "tklmbda", ufunc_tklmbda_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9501
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]
 * ufunc_tklmbda_data[1] = &ufunc_tklmbda_ptr[2*1]             # <<<<<<<<<<<<<<
 * tklmbda = np.PyUFunc_FromFuncAndData(ufunc_tklmbda_loops, ufunc_tklmbda_data, ufunc_tklmbda_types, 2, 2, 1, 0, "tklmbda", ufunc_tklmbda_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9502
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]
 * ufunc_tklmbda_data[1] = &ufunc_tklmbda_ptr[2*1]
 * tklmbda = np.PyUFunc_FromFuncAndData(ufunc_tklmbda_loops, ufunc_tklmbda_data, ufunc_tklmbda_types, 2, 2, 1, 0, "tklmbda", ufunc_tklmbda_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_wofz_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types, 2, 2, 1, 0, __pyx_k_tklmbda, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_tklmbda, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9508
 * cdef void *ufunc_wofz_data[2]
 * cdef char ufunc_wofz_types[4]
 * cdef char *ufunc_wofz_doc = (             # <<<<<<<<<<<<<<
 *     "wofz(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_doc = __pyx_k_wofz_z_Faddeeva_function_Returns;

  /* "scipy/special/_ufuncs.pyx":9521
 *     ".. [1] Steven G. Johnson, Faddeeva W function implementation.\n"
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_wofz_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_wofz_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":9522
 *     "   http://ab-initio.mit.edu/Faddeeva")
 * ufunc_wofz_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_wofz_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":9523
 * ufunc_wofz_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_wofz_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[0]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9524
 * ufunc_wofz_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9525
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[2]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9526
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[3]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9527
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w             # <<<<<<<<<<<<<<
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w);

  /* "scipy/special/_ufuncs.pyx":9528
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")             # <<<<<<<<<<<<<<
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[1]) = ((void *)((char *)__pyx_k_wofz));

  /* "scipy/special/_ufuncs.pyx":9529
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w             # <<<<<<<<<<<<<<
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w);

  /* "scipy/special/_ufuncs.pyx":9530
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")             # <<<<<<<<<<<<<<
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]
 * ufunc_wofz_data[1] = &ufunc_wofz_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[3]) = ((void *)((char *)__pyx_k_wofz));

  /* "scipy/special/_ufuncs.pyx":9531
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_wofz_data[1] = &ufunc_wofz_ptr[2*1]
 * wofz = np.PyUFunc_FromFuncAndData(ufunc_wofz_loops, ufunc_wofz_data, ufunc_wofz_types, 2, 1, 1, 0, "wofz", ufunc_wofz_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9532
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]
 * ufunc_wofz_data[1] = &ufunc_wofz_ptr[2*1]             # <<<<<<<<<<<<<<
 * wofz = np.PyUFunc_FromFuncAndData(ufunc_wofz_loops, ufunc_wofz_data, ufunc_wofz_types, 2, 1, 1, 0, "wofz", ufunc_wofz_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9533
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]
 * ufunc_wofz_data[1] = &ufunc_wofz_ptr[2*1]
 * wofz = np.PyUFunc_FromFuncAndData(ufunc_wofz_loops, ufunc_wofz_data, ufunc_wofz_types, 2, 1, 1, 0, "wofz", ufunc_wofz_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_xlog1py_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types, 2, 1, 1, 0, __pyx_k_wofz, __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_wofz, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9539
 * cdef void *ufunc_xlog1py_data[2]
 * cdef char ufunc_xlog1py_types[6]
 * cdef char *ufunc_xlog1py_doc = (             # <<<<<<<<<<<<<<
 *     "xlog1py(x, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_doc = __pyx_k_xlog1py_x_y_Compute_x_log1p_y_so;

  /* "scipy/special/_ufuncs.pyx":9560
 *     "\n"
 *     ".. versionadded:: 0.13.0")
 * ufunc_xlog1py_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9561
 *     ".. versionadded:: 0.13.0")
 * ufunc_xlog1py_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_xlog1py_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9562
 * ufunc_xlog1py_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_xlog1py_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9563
 * ufunc_xlog1py_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9564
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9565
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9566
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9567
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9568
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[0]) = ((void *)__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":9569
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[1]) = ((void *)((char *)__pyx_k_xlog1py));

  /* "scipy/special/_ufuncs.pyx":9570
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[2]) = ((void *)__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":9571
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[3]) = ((void *)((char *)__pyx_k_xlog1py));

  /* "scipy/special/_ufuncs.pyx":9572
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]
 * xlog1py = np.PyUFunc_FromFuncAndData(ufunc_xlog1py_loops, ufunc_xlog1py_data, ufunc_xlog1py_types, 2, 2, 1, 0, "xlog1py", ufunc_xlog1py_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9573
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]             # <<<<<<<<<<<<<<
 * xlog1py = np.PyUFunc_FromFuncAndData(ufunc_xlog1py_loops, ufunc_xlog1py_data, ufunc_xlog1py_types, 2, 2, 1, 0, "xlog1py", ufunc_xlog1py_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9574
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]
 * xlog1py = np.PyUFunc_FromFuncAndData(ufunc_xlog1py_loops, ufunc_xlog1py_data, ufunc_xlog1py_types, 2, 2, 1, 0, "xlog1py", ufunc_xlog1py_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_xlogy_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types, 2, 2, 1, 0, __pyx_k_xlog1py, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_xlog1py, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9580
 * cdef void *ufunc_xlogy_data[4]
 * cdef char ufunc_xlogy_types[12]
 * cdef char *ufunc_xlogy_doc = (             # <<<<<<<<<<<<<<
 *     "xlogy(x, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_doc = __pyx_k_xlogy_x_y_Compute_x_log_y_so_tha;

  /* "scipy/special/_ufuncs.pyx":9601
 *     "\n"
 *     ".. versionadded:: 0.13.0")
 * ufunc_xlogy_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_xlogy_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9602
 *     ".. versionadded:: 0.13.0")
 * ufunc_xlogy_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_xlogy_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9603
 * ufunc_xlogy_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_xlogy_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F             # <<<<<<<<<<<<<<
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_FF_F);

  /* "scipy/special/_ufuncs.pyx":9604
 * ufunc_xlogy_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_DD_D);

  /* "scipy/special/_ufuncs.pyx":9605
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9606
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9607
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9608
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9609
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9610
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9611
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9612
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9613
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9614
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9615
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9616
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9617
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[0]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":9618
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[1]) = ((void *)((char *)__pyx_k_xlogy));

  /* "scipy/special/_ufuncs.pyx":9619
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":9620
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[3]) = ((void *)((char *)__pyx_k_xlogy));

  /* "scipy/special/_ufuncs.pyx":9621
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":9622
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[5]) = ((void *)((char *)__pyx_k_xlogy));

  /* "scipy/special/_ufuncs.pyx":9623
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[6]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":9624
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")             # <<<<<<<<<<<<<<
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[7]) = ((void *)((char *)__pyx_k_xlogy));

  /* "scipy/special/_ufuncs.pyx":9625
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9626
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]
 * ufunc_xlogy_data[3] = &ufunc_xlogy_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9627
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_data[3] = &ufunc_xlogy_ptr[2*3]
 * xlogy = np.PyUFunc_FromFuncAndData(ufunc_xlogy_loops, ufunc_xlogy_data, ufunc_xlogy_types, 4, 2, 1, 0, "xlogy", ufunc_xlogy_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9628
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]
 * ufunc_xlogy_data[3] = &ufunc_xlogy_ptr[2*3]             # <<<<<<<<<<<<<<
 * xlogy = np.PyUFunc_FromFuncAndData(ufunc_xlogy_loops, ufunc_xlogy_data, ufunc_xlogy_types, 4, 2, 1, 0, "xlogy", ufunc_xlogy_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":9629
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]
 * ufunc_xlogy_data[3] = &ufunc_xlogy_ptr[2*3]
 * xlogy = np.PyUFunc_FromFuncAndData(ufunc_xlogy_loops, ufunc_xlogy_data, ufunc_xlogy_types, 4, 2, 1, 0, "xlogy", ufunc_xlogy_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_y0_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types, 4, 2, 1, 0, __pyx_k_xlogy, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_xlogy, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9635
 * cdef void *ufunc_y0_data[2]
 * cdef char ufunc_y0_types[4]
 * cdef char *ufunc_y0_doc = (             # <<<<<<<<<<<<<<
 *     "y0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_doc = __pyx_k_y0_x_Bessel_function_of_the_seco;

  /* "scipy/special/_ufuncs.pyx":9641
 *     "\n"
 *     "Returns the Bessel function of the second kind of order 0 at `x`.")
 * ufunc_y0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_y0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9642
 *     "Returns the Bessel function of the second kind of order 0 at `x`.")
 * ufunc_y0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_y0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_y0_types[0] = <char>NPY_FLOAT
 * ufunc_y0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9643
 * ufunc_y0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_y0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_y0_types[1] = <char>NPY_FLOAT
 * ufunc_y0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9644
 * ufunc_y0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y0_types[0] = <char>NPY_FLOAT
 * ufunc_y0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_y0_types[2] = <char>NPY_DOUBLE
 * ufunc_y0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9645
 * ufunc_y0_types[0] = <char>NPY_FLOAT
 * ufunc_y0_types[1] = <char>NPY_FLOAT
 * ufunc_y0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_y0_types[3] = <char>NPY_DOUBLE
 * ufunc_y0_ptr[2*0] = <void*>_func_y0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9646
 * ufunc_y0_types[1] = <char>NPY_FLOAT
 * ufunc_y0_types[2] = <char>NPY_DOUBLE
 * ufunc_y0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_y0_ptr[2*0] = <void*>_func_y0
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9647
 * ufunc_y0_types[2] = <char>NPY_DOUBLE
 * ufunc_y0_types[3] = <char>NPY_DOUBLE
 * ufunc_y0_ptr[2*0] = <void*>_func_y0             # <<<<<<<<<<<<<<
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")
 * ufunc_y0_ptr[2*1] = <void*>_func_y0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[0]) = ((void *)y0);

  /* "scipy/special/_ufuncs.pyx":9648
 * ufunc_y0_types[3] = <char>NPY_DOUBLE
 * ufunc_y0_ptr[2*0] = <void*>_func_y0
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")             # <<<<<<<<<<<<<<
 * ufunc_y0_ptr[2*1] = <void*>_func_y0
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[1]) = ((void *)((char *)__pyx_k_y0));

  /* "scipy/special/_ufuncs.pyx":9649
 * ufunc_y0_ptr[2*0] = <void*>_func_y0
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")
 * ufunc_y0_ptr[2*1] = <void*>_func_y0             # <<<<<<<<<<<<<<
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[2]) = ((void *)y0);

  /* "scipy/special/_ufuncs.pyx":9650
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")
 * ufunc_y0_ptr[2*1] = <void*>_func_y0
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")             # <<<<<<<<<<<<<<
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]
 * ufunc_y0_data[1] = &ufunc_y0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[3]) = ((void *)((char *)__pyx_k_y0));

  /* "scipy/special/_ufuncs.pyx":9651
 * ufunc_y0_ptr[2*1] = <void*>_func_y0
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_y0_data[1] = &ufunc_y0_ptr[2*1]
 * y0 = np.PyUFunc_FromFuncAndData(ufunc_y0_loops, ufunc_y0_data, ufunc_y0_types, 2, 1, 1, 0, "y0", ufunc_y0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9652
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]
 * ufunc_y0_data[1] = &ufunc_y0_ptr[2*1]             # <<<<<<<<<<<<<<
 * y0 = np.PyUFunc_FromFuncAndData(ufunc_y0_loops, ufunc_y0_data, ufunc_y0_types, 2, 1, 1, 0, "y0", ufunc_y0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9653
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]
 * ufunc_y0_data[1] = &ufunc_y0_ptr[2*1]
 * y0 = np.PyUFunc_FromFuncAndData(ufunc_y0_loops, ufunc_y0_data, ufunc_y0_types, 2, 1, 1, 0, "y0", ufunc_y0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_y1_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types, 2, 1, 1, 0, __pyx_k_y0, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9653; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_y0, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9653; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9659
 * cdef void *ufunc_y1_data[2]
 * cdef char ufunc_y1_types[4]
 * cdef char *ufunc_y1_doc = (             # <<<<<<<<<<<<<<
 *     "y1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_doc = __pyx_k_y1_x_Bessel_function_of_the_seco;

  /* "scipy/special/_ufuncs.pyx":9665
 *     "\n"
 *     "Returns the Bessel function of the second kind of order 1 at `x`.")
 * ufunc_y1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_y1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9666
 *     "Returns the Bessel function of the second kind of order 1 at `x`.")
 * ufunc_y1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_y1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_y1_types[0] = <char>NPY_FLOAT
 * ufunc_y1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9667
 * ufunc_y1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_y1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_y1_types[1] = <char>NPY_FLOAT
 * ufunc_y1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9668
 * ufunc_y1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y1_types[0] = <char>NPY_FLOAT
 * ufunc_y1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_y1_types[2] = <char>NPY_DOUBLE
 * ufunc_y1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9669
 * ufunc_y1_types[0] = <char>NPY_FLOAT
 * ufunc_y1_types[1] = <char>NPY_FLOAT
 * ufunc_y1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_y1_types[3] = <char>NPY_DOUBLE
 * ufunc_y1_ptr[2*0] = <void*>_func_y1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9670
 * ufunc_y1_types[1] = <char>NPY_FLOAT
 * ufunc_y1_types[2] = <char>NPY_DOUBLE
 * ufunc_y1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_y1_ptr[2*0] = <void*>_func_y1
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9671
 * ufunc_y1_types[2] = <char>NPY_DOUBLE
 * ufunc_y1_types[3] = <char>NPY_DOUBLE
 * ufunc_y1_ptr[2*0] = <void*>_func_y1             # <<<<<<<<<<<<<<
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")
 * ufunc_y1_ptr[2*1] = <void*>_func_y1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[0]) = ((void *)y1);

  /* "scipy/special/_ufuncs.pyx":9672
 * ufunc_y1_types[3] = <char>NPY_DOUBLE
 * ufunc_y1_ptr[2*0] = <void*>_func_y1
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")             # <<<<<<<<<<<<<<
 * ufunc_y1_ptr[2*1] = <void*>_func_y1
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[1]) = ((void *)((char *)__pyx_k_y1));

  /* "scipy/special/_ufuncs.pyx":9673
 * ufunc_y1_ptr[2*0] = <void*>_func_y1
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")
 * ufunc_y1_ptr[2*1] = <void*>_func_y1             # <<<<<<<<<<<<<<
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[2]) = ((void *)y1);

  /* "scipy/special/_ufuncs.pyx":9674
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")
 * ufunc_y1_ptr[2*1] = <void*>_func_y1
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")             # <<<<<<<<<<<<<<
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]
 * ufunc_y1_data[1] = &ufunc_y1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[3]) = ((void *)((char *)__pyx_k_y1));

  /* "scipy/special/_ufuncs.pyx":9675
 * ufunc_y1_ptr[2*1] = <void*>_func_y1
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_y1_data[1] = &ufunc_y1_ptr[2*1]
 * y1 = np.PyUFunc_FromFuncAndData(ufunc_y1_loops, ufunc_y1_data, ufunc_y1_types, 2, 1, 1, 0, "y1", ufunc_y1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9676
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]
 * ufunc_y1_data[1] = &ufunc_y1_ptr[2*1]             # <<<<<<<<<<<<<<
 * y1 = np.PyUFunc_FromFuncAndData(ufunc_y1_loops, ufunc_y1_data, ufunc_y1_types, 2, 1, 1, 0, "y1", ufunc_y1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9677
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]
 * ufunc_y1_data[1] = &ufunc_y1_ptr[2*1]
 * y1 = np.PyUFunc_FromFuncAndData(ufunc_y1_loops, ufunc_y1_data, ufunc_y1_types, 2, 1, 1, 0, "y1", ufunc_y1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_yn_loops[3]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types, 2, 1, 1, 0, __pyx_k_y1, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9677; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_y1, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9677; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9683
 * cdef void *ufunc_yn_data[3]
 * cdef char ufunc_yn_types[9]
 * cdef char *ufunc_yn_doc = (             # <<<<<<<<<<<<<<
 *     "yn(n, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_doc = __pyx_k_yn_n_x_Bessel_function_of_the_se;

  /* "scipy/special/_ufuncs.pyx":9690
 *     "Returns the Bessel function of the second kind of integer order `n`\n"
 *     "at `x`.")
 * ufunc_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_yn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":9691
 *     "at `x`.")
 * ufunc_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_yn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9692
 * ufunc_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_yn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_yn_types[0] = <char>NPY_LONG
 * ufunc_yn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9693
 * ufunc_yn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_yn_types[1] = <char>NPY_DOUBLE
 * ufunc_yn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":9694
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yn_types[0] = <char>NPY_LONG
 * ufunc_yn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_types[2] = <char>NPY_DOUBLE
 * ufunc_yn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9695
 * ufunc_yn_types[0] = <char>NPY_LONG
 * ufunc_yn_types[1] = <char>NPY_DOUBLE
 * ufunc_yn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_types[3] = <char>NPY_FLOAT
 * ufunc_yn_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9696
 * ufunc_yn_types[1] = <char>NPY_DOUBLE
 * ufunc_yn_types[2] = <char>NPY_DOUBLE
 * ufunc_yn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yn_types[4] = <char>NPY_FLOAT
 * ufunc_yn_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9697
 * ufunc_yn_types[2] = <char>NPY_DOUBLE
 * ufunc_yn_types[3] = <char>NPY_FLOAT
 * ufunc_yn_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yn_types[5] = <char>NPY_FLOAT
 * ufunc_yn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9698
 * ufunc_yn_types[3] = <char>NPY_FLOAT
 * ufunc_yn_types[4] = <char>NPY_FLOAT
 * ufunc_yn_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yn_types[6] = <char>NPY_DOUBLE
 * ufunc_yn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9699
 * ufunc_yn_types[4] = <char>NPY_FLOAT
 * ufunc_yn_types[5] = <char>NPY_FLOAT
 * ufunc_yn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_types[7] = <char>NPY_DOUBLE
 * ufunc_yn_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9700
 * ufunc_yn_types[5] = <char>NPY_FLOAT
 * ufunc_yn_types[6] = <char>NPY_DOUBLE
 * ufunc_yn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_types[8] = <char>NPY_DOUBLE
 * ufunc_yn_ptr[2*0] = <void*>_func_yn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9701
 * ufunc_yn_types[6] = <char>NPY_DOUBLE
 * ufunc_yn_types[7] = <char>NPY_DOUBLE
 * ufunc_yn_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*0] = <void*>_func_yn
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9702
 * ufunc_yn_types[7] = <char>NPY_DOUBLE
 * ufunc_yn_types[8] = <char>NPY_DOUBLE
 * ufunc_yn_ptr[2*0] = <void*>_func_yn             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[0]) = ((void *)yn);

  /* "scipy/special/_ufuncs.pyx":9703
 * ufunc_yn_types[8] = <char>NPY_DOUBLE
 * ufunc_yn_ptr[2*0] = <void*>_func_yn
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[1]) = ((void *)((char *)__pyx_k_yn));

  /* "scipy/special/_ufuncs.pyx":9704
 * ufunc_yn_ptr[2*0] = <void*>_func_yn
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_yn_unsafe);

  /* "scipy/special/_ufuncs.pyx":9705
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[3]) = ((void *)((char *)__pyx_k_yn));

  /* "scipy/special/_ufuncs.pyx":9706
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_yn_unsafe);

  /* "scipy/special/_ufuncs.pyx":9707
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")             # <<<<<<<<<<<<<<
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[5]) = ((void *)((char *)__pyx_k_yn));

  /* "scipy/special/_ufuncs.pyx":9708
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]
 * ufunc_yn_data[2] = &ufunc_yn_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9709
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_yn_data[2] = &ufunc_yn_ptr[2*2]
 * yn = np.PyUFunc_FromFuncAndData(ufunc_yn_loops, ufunc_yn_data, ufunc_yn_types, 3, 2, 1, 0, "yn", ufunc_yn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9710
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]
 * ufunc_yn_data[2] = &ufunc_yn_ptr[2*2]             # <<<<<<<<<<<<<<
 * yn = np.PyUFunc_FromFuncAndData(ufunc_yn_loops, ufunc_yn_data, ufunc_yn_types, 3, 2, 1, 0, "yn", ufunc_yn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9711
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]
 * ufunc_yn_data[2] = &ufunc_yn_ptr[2*2]
 * yn = np.PyUFunc_FromFuncAndData(ufunc_yn_loops, ufunc_yn_data, ufunc_yn_types, 3, 2, 1, 0, "yn", ufunc_yn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_yv_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types, 3, 2, 1, 0, __pyx_k_yn, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9711; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_yn, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9711; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9717
 * cdef void *ufunc_yv_data[4]
 * cdef char ufunc_yv_types[12]
 * cdef char *ufunc_yv_doc = (             # <<<<<<<<<<<<<<
 *     "yv(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_doc = __pyx_k_yv_v_z_Bessel_function_of_the_se;

  /* "scipy/special/_ufuncs.pyx":9724
 *     "Returns the Bessel function of the second kind of real order `v` at\n"
 *     "complex `z`.")
 * ufunc_yv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_yv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9725
 *     "complex `z`.")
 * ufunc_yv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":9726
 * ufunc_yv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9727
 * ufunc_yv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_yv_types[0] = <char>NPY_FLOAT
 * ufunc_yv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":9728
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[1] = <char>NPY_FLOAT
 * ufunc_yv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9729
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yv_types[0] = <char>NPY_FLOAT
 * ufunc_yv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[2] = <char>NPY_FLOAT
 * ufunc_yv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9730
 * ufunc_yv_types[0] = <char>NPY_FLOAT
 * ufunc_yv_types[1] = <char>NPY_FLOAT
 * ufunc_yv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[3] = <char>NPY_FLOAT
 * ufunc_yv_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9731
 * ufunc_yv_types[1] = <char>NPY_FLOAT
 * ufunc_yv_types[2] = <char>NPY_FLOAT
 * ufunc_yv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[4] = <char>NPY_CFLOAT
 * ufunc_yv_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9732
 * ufunc_yv_types[2] = <char>NPY_FLOAT
 * ufunc_yv_types[3] = <char>NPY_FLOAT
 * ufunc_yv_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[5] = <char>NPY_CFLOAT
 * ufunc_yv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9733
 * ufunc_yv_types[3] = <char>NPY_FLOAT
 * ufunc_yv_types[4] = <char>NPY_CFLOAT
 * ufunc_yv_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[6] = <char>NPY_DOUBLE
 * ufunc_yv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9734
 * ufunc_yv_types[4] = <char>NPY_CFLOAT
 * ufunc_yv_types[5] = <char>NPY_CFLOAT
 * ufunc_yv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[7] = <char>NPY_DOUBLE
 * ufunc_yv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9735
 * ufunc_yv_types[5] = <char>NPY_CFLOAT
 * ufunc_yv_types[6] = <char>NPY_DOUBLE
 * ufunc_yv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[8] = <char>NPY_DOUBLE
 * ufunc_yv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9736
 * ufunc_yv_types[6] = <char>NPY_DOUBLE
 * ufunc_yv_types[7] = <char>NPY_DOUBLE
 * ufunc_yv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[9] = <char>NPY_DOUBLE
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9737
 * ufunc_yv_types[7] = <char>NPY_DOUBLE
 * ufunc_yv_types[8] = <char>NPY_DOUBLE
 * ufunc_yv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9738
 * ufunc_yv_types[8] = <char>NPY_DOUBLE
 * ufunc_yv_types[9] = <char>NPY_DOUBLE
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9739
 * ufunc_yv_types[9] = <char>NPY_DOUBLE
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9740
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[0]) = ((void *)cbesy_wrap_real);

  /* "scipy/special/_ufuncs.pyx":9741
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[1]) = ((void *)((char *)__pyx_k_yv));

  /* "scipy/special/_ufuncs.pyx":9742
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[2]) = ((void *)cbesy_wrap);

  /* "scipy/special/_ufuncs.pyx":9743
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[3]) = ((void *)((char *)__pyx_k_yv));

  /* "scipy/special/_ufuncs.pyx":9744
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[4]) = ((void *)cbesy_wrap_real);

  /* "scipy/special/_ufuncs.pyx":9745
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[5]) = ((void *)((char *)__pyx_k_yv));

  /* "scipy/special/_ufuncs.pyx":9746
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[6]) = ((void *)cbesy_wrap);

  /* "scipy/special/_ufuncs.pyx":9747
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")             # <<<<<<<<<<<<<<
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[7]) = ((void *)((char *)__pyx_k_yv));

  /* "scipy/special/_ufuncs.pyx":9748
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9749
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]
 * ufunc_yv_data[3] = &ufunc_yv_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9750
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_yv_data[3] = &ufunc_yv_ptr[2*3]
 * yv = np.PyUFunc_FromFuncAndData(ufunc_yv_loops, ufunc_yv_data, ufunc_yv_types, 4, 2, 1, 0, "yv", ufunc_yv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9751
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]
 * ufunc_yv_data[3] = &ufunc_yv_ptr[2*3]             # <<<<<<<<<<<<<<
 * yv = np.PyUFunc_FromFuncAndData(ufunc_yv_loops, ufunc_yv_data, ufunc_yv_types, 4, 2, 1, 0, "yv", ufunc_yv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":9752
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]
 * ufunc_yv_data[3] = &ufunc_yv_ptr[2*3]
 * yv = np.PyUFunc_FromFuncAndData(ufunc_yv_loops, ufunc_yv_data, ufunc_yv_types, 4, 2, 1, 0, "yv", ufunc_yv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_yve_loops[4]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types, 4, 2, 1, 0, __pyx_k_yv, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9752; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_yv, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9752; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9758
 * cdef void *ufunc_yve_data[4]
 * cdef char ufunc_yve_types[12]
 * cdef char *ufunc_yve_doc = (             # <<<<<<<<<<<<<<
 *     "yve(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_doc = __pyx_k_yve_v_z_Exponentially_scaled_Bes;

  /* "scipy/special/_ufuncs.pyx":9767
 *     "\n"
 *     "    yve(v, z) = yv(v, z) * exp(-abs(z.imag))")
 * ufunc_yve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_yve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9768
 *     "    yve(v, z) = yv(v, z) * exp(-abs(z.imag))")
 * ufunc_yve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":9769
 * ufunc_yve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9770
 * ufunc_yve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_yve_types[0] = <char>NPY_FLOAT
 * ufunc_yve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":9771
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[1] = <char>NPY_FLOAT
 * ufunc_yve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9772
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yve_types[0] = <char>NPY_FLOAT
 * ufunc_yve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[2] = <char>NPY_FLOAT
 * ufunc_yve_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9773
 * ufunc_yve_types[0] = <char>NPY_FLOAT
 * ufunc_yve_types[1] = <char>NPY_FLOAT
 * ufunc_yve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[3] = <char>NPY_FLOAT
 * ufunc_yve_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9774
 * ufunc_yve_types[1] = <char>NPY_FLOAT
 * ufunc_yve_types[2] = <char>NPY_FLOAT
 * ufunc_yve_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[4] = <char>NPY_CFLOAT
 * ufunc_yve_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9775
 * ufunc_yve_types[2] = <char>NPY_FLOAT
 * ufunc_yve_types[3] = <char>NPY_FLOAT
 * ufunc_yve_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[5] = <char>NPY_CFLOAT
 * ufunc_yve_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9776
 * ufunc_yve_types[3] = <char>NPY_FLOAT
 * ufunc_yve_types[4] = <char>NPY_CFLOAT
 * ufunc_yve_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[6] = <char>NPY_DOUBLE
 * ufunc_yve_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9777
 * ufunc_yve_types[4] = <char>NPY_CFLOAT
 * ufunc_yve_types[5] = <char>NPY_CFLOAT
 * ufunc_yve_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[7] = <char>NPY_DOUBLE
 * ufunc_yve_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9778
 * ufunc_yve_types[5] = <char>NPY_CFLOAT
 * ufunc_yve_types[6] = <char>NPY_DOUBLE
 * ufunc_yve_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[8] = <char>NPY_DOUBLE
 * ufunc_yve_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9779
 * ufunc_yve_types[6] = <char>NPY_DOUBLE
 * ufunc_yve_types[7] = <char>NPY_DOUBLE
 * ufunc_yve_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[9] = <char>NPY_DOUBLE
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9780
 * ufunc_yve_types[7] = <char>NPY_DOUBLE
 * ufunc_yve_types[8] = <char>NPY_DOUBLE
 * ufunc_yve_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9781
 * ufunc_yve_types[8] = <char>NPY_DOUBLE
 * ufunc_yve_types[9] = <char>NPY_DOUBLE
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9782
 * ufunc_yve_types[9] = <char>NPY_DOUBLE
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9783
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[0]) = ((void *)cbesy_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":9784
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[1]) = ((void *)((char *)__pyx_k_yve));

  /* "scipy/special/_ufuncs.pyx":9785
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[2]) = ((void *)cbesy_wrap_e);

  /* "scipy/special/_ufuncs.pyx":9786
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[3]) = ((void *)((char *)__pyx_k_yve));

  /* "scipy/special/_ufuncs.pyx":9787
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[4]) = ((void *)cbesy_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":9788
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[5]) = ((void *)((char *)__pyx_k_yve));

  /* "scipy/special/_ufuncs.pyx":9789
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[6]) = ((void *)cbesy_wrap_e);

  /* "scipy/special/_ufuncs.pyx":9790
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")             # <<<<<<<<<<<<<<
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[7]) = ((void *)((char *)__pyx_k_yve));

  /* "scipy/special/_ufuncs.pyx":9791
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9792
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]
 * ufunc_yve_data[3] = &ufunc_yve_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9793
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_yve_data[3] = &ufunc_yve_ptr[2*3]
 * yve = np.PyUFunc_FromFuncAndData(ufunc_yve_loops, ufunc_yve_data, ufunc_yve_types, 4, 2, 1, 0, "yve", ufunc_yve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9794
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]
 * ufunc_yve_data[3] = &ufunc_yve_ptr[2*3]             # <<<<<<<<<<<<<<
 * yve = np.PyUFunc_FromFuncAndData(ufunc_yve_loops, ufunc_yve_data, ufunc_yve_types, 4, 2, 1, 0, "yve", ufunc_yve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":9795
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]
 * ufunc_yve_data[3] = &ufunc_yve_ptr[2*3]
 * yve = np.PyUFunc_FromFuncAndData(ufunc_yve_loops, ufunc_yve_data, ufunc_yve_types, 4, 2, 1, 0, "yve", ufunc_yve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_zeta_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types, 4, 2, 1, 0, __pyx_k_yve, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_yve, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9801
 * cdef void *ufunc_zeta_data[2]
 * cdef char ufunc_zeta_types[6]
 * cdef char *ufunc_zeta_doc = (             # <<<<<<<<<<<<<<
 *     "zeta(x, q)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_doc = __pyx_k_zeta_x_q_Hurwitz_zeta_function_T;

  /* "scipy/special/_ufuncs.pyx":9818
 *     "--------\n"
 *     "zetac")
 * ufunc_zeta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_zeta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_zeta_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9819
 *     "zetac")
 * ufunc_zeta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_zeta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_zeta_types[0] = <char>NPY_FLOAT
 * ufunc_zeta_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9820
 * ufunc_zeta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_zeta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_zeta_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_zeta_types[1] = <char>NPY_FLOAT
 * ufunc_zeta_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9821
 * ufunc_zeta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_zeta_types[0] = <char>NPY_FLOAT
 * ufunc_zeta_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_zeta_types[2] = <char>NPY_FLOAT
 * ufunc_zeta_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9822
 * ufunc_zeta_types[0] = <char>NPY_FLOAT
 * ufunc_zeta_types[1] = <char>NPY_FLOAT
 * ufunc_zeta_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_zeta_types[3] = <char>NPY_DOUBLE
 * ufunc_zeta_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9823
 * ufunc_zeta_types[1] = <char>NPY_FLOAT
 * ufunc_zeta_types[2] = <char>NPY_FLOAT
 * ufunc_zeta_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_zeta_types[4] = <char>NPY_DOUBLE
 * ufunc_zeta_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9824
 * ufunc_zeta_types[2] = <char>NPY_FLOAT
 * ufunc_zeta_types[3] = <char>NPY_DOUBLE
 * ufunc_zeta_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_zeta_types[5] = <char>NPY_DOUBLE
 * ufunc_zeta_ptr[2*0] = <void*>_func_zeta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9825
 * ufunc_zeta_types[3] = <char>NPY_DOUBLE
 * ufunc_zeta_types[4] = <char>NPY_DOUBLE
 * ufunc_zeta_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_zeta_ptr[2*0] = <void*>_func_zeta
 * ufunc_zeta_ptr[2*0+1] = <void*>(<char*>"zeta")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9826
 * ufunc_zeta_types[4] = <char>NPY_DOUBLE
 * ufunc_zeta_types[5] = <char>NPY_DOUBLE
 * ufunc_zeta_ptr[2*0] = <void*>_func_zeta             # <<<<<<<<<<<<<<
 * ufunc_zeta_ptr[2*0+1] = <void*>(<char*>"zeta")
 * ufunc_zeta_ptr[2*1] = <void*>_func_zeta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_ptr[0]) = ((void *)zeta);

  /* "scipy/special/_ufuncs.pyx":9827
 * ufunc_zeta_types[5] = <char>NPY_DOUBLE
 * ufunc_zeta_ptr[2*0] = <void*>_func_zeta
 * ufunc_zeta_ptr[2*0+1] = <void*>(<char*>"zeta")             # <<<<<<<<<<<<<<
 * ufunc_zeta_ptr[2*1] = <void*>_func_zeta
 * ufunc_zeta_ptr[2*1+1] = <void*>(<char*>"zeta")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_ptr[1]) = ((void *)((char *)__pyx_k_zeta));

  /* "scipy/special/_ufuncs.pyx":9828
 * ufunc_zeta_ptr[2*0] = <void*>_func_zeta
 * ufunc_zeta_ptr[2*0+1] = <void*>(<char*>"zeta")
 * ufunc_zeta_ptr[2*1] = <void*>_func_zeta             # <<<<<<<<<<<<<<
 * ufunc_zeta_ptr[2*1+1] = <void*>(<char*>"zeta")
 * ufunc_zeta_data[0] = &ufunc_zeta_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_ptr[2]) = ((void *)zeta);

  /* "scipy/special/_ufuncs.pyx":9829
 * ufunc_zeta_ptr[2*0+1] = <void*>(<char*>"zeta")
 * ufunc_zeta_ptr[2*1] = <void*>_func_zeta
 * ufunc_zeta_ptr[2*1+1] = <void*>(<char*>"zeta")             # <<<<<<<<<<<<<<
 * ufunc_zeta_data[0] = &ufunc_zeta_ptr[2*0]
 * ufunc_zeta_data[1] = &ufunc_zeta_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_ptr[3]) = ((void *)((char *)__pyx_k_zeta));

  /* "scipy/special/_ufuncs.pyx":9830
 * ufunc_zeta_ptr[2*1] = <void*>_func_zeta
 * ufunc_zeta_ptr[2*1+1] = <void*>(<char*>"zeta")
 * ufunc_zeta_data[0] = &ufunc_zeta_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_zeta_data[1] = &ufunc_zeta_ptr[2*1]
 * zeta = np.PyUFunc_FromFuncAndData(ufunc_zeta_loops, ufunc_zeta_data, ufunc_zeta_types, 2, 2, 1, 0, "zeta", ufunc_zeta_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9831
 * ufunc_zeta_ptr[2*1+1] = <void*>(<char*>"zeta")
 * ufunc_zeta_data[0] = &ufunc_zeta_ptr[2*0]
 * ufunc_zeta_data[1] = &ufunc_zeta_ptr[2*1]             # <<<<<<<<<<<<<<
 * zeta = np.PyUFunc_FromFuncAndData(ufunc_zeta_loops, ufunc_zeta_data, ufunc_zeta_types, 2, 2, 1, 0, "zeta", ufunc_zeta_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9832
 * ufunc_zeta_data[0] = &ufunc_zeta_ptr[2*0]
 * ufunc_zeta_data[1] = &ufunc_zeta_ptr[2*1]
 * zeta = np.PyUFunc_FromFuncAndData(ufunc_zeta_loops, ufunc_zeta_data, ufunc_zeta_types, 2, 2, 1, 0, "zeta", ufunc_zeta_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_zetac_loops[2]
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_types, 2, 2, 1, 0, __pyx_k_zeta, __pyx_v_5scipy_7special_7_ufuncs_ufunc_zeta_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_zeta, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9838
 * cdef void *ufunc_zetac_data[2]
 * cdef char ufunc_zetac_types[4]
 * cdef char *ufunc_zetac_doc = (             # <<<<<<<<<<<<<<
 *     "zetac(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_doc = __pyx_k_zetac_x_Riemann_zeta_function_mi;

  /* "scipy/special/_ufuncs.pyx":9852
 *     "--------\n"
 *     "zeta")
 * ufunc_zetac_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_zetac_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_zetac_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9853
 *     "zeta")
 * ufunc_zetac_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_zetac_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_zetac_types[0] = <char>NPY_FLOAT
 * ufunc_zetac_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9854
 * ufunc_zetac_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_zetac_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_zetac_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_zetac_types[1] = <char>NPY_FLOAT
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9855
 * ufunc_zetac_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_zetac_types[0] = <char>NPY_FLOAT
 * ufunc_zetac_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9856
 * ufunc_zetac_types[0] = <char>NPY_FLOAT
 * ufunc_zetac_types[1] = <char>NPY_FLOAT
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9857
 * ufunc_zetac_types[1] = <char>NPY_FLOAT
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9858
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac             # <<<<<<<<<<<<<<
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[0]) = ((void *)zetac);

  /* "scipy/special/_ufuncs.pyx":9859
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")             # <<<<<<<<<<<<<<
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[1]) = ((void *)((char *)__pyx_k_zetac));

  /* "scipy/special/_ufuncs.pyx":9860
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac             # <<<<<<<<<<<<<<
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[2]) = ((void *)zetac);

  /* "scipy/special/_ufuncs.pyx":9861
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")             # <<<<<<<<<<<<<<
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]
 * ufunc_zetac_data[1] = &ufunc_zetac_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[3]) = ((void *)((char *)__pyx_k_zetac));

  /* "scipy/special/_ufuncs.pyx":9862
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_zetac_data[1] = &ufunc_zetac_ptr[2*1]
 * zetac = np.PyUFunc_FromFuncAndData(ufunc_zetac_loops, ufunc_zetac_data, ufunc_zetac_types, 2, 1, 1, 0, "zetac", ufunc_zetac_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9863
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]
 * ufunc_zetac_data[1] = &ufunc_zetac_ptr[2*1]             # <<<<<<<<<<<<<<
 * zetac = np.PyUFunc_FromFuncAndData(ufunc_zetac_loops, ufunc_zetac_data, ufunc_zetac_types, 2, 1, 1, 0, "zetac", ufunc_zetac_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9864
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]
 * ufunc_zetac_data[1] = &ufunc_zetac_ptr[2*1]
 * zetac = np.PyUFunc_FromFuncAndData(ufunc_zetac_loops, ufunc_zetac_data, ufunc_zetac_types, 2, 1, 1, 0, "zetac", ufunc_zetac_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * #
 */
  __pyx_t_3 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types, 2, 1, 1, 0, __pyx_k_zetac, __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_doc, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9864; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_zetac, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9864; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":9869
 * # Aliases
 * #
 * jn = jv             # <<<<<<<<<<<<<<
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_jv); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9869; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_jn, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9869; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/special/_ufuncs.pyx":1
 * # This file is automatically generated by generate_ufuncs.py.             # <<<<<<<<<<<<<<
 * # Do not edit manually!
 * 
 */
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "_complexstuff.pxd":59
 *         return libc.math.log(x)
 * 
 * cdef inline number_t zexp(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init scipy.special._ufuncs", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init scipy.special._ufuncs");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#endif
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        double denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(a, a);
                    case 3:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, a);
                    case 4:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_abs(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *x) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Py_intptr_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Py_intptr_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(Py_intptr_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, long, PyLong_AsLong(x))
            } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Py_intptr_t val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Py_intptr_t) -1;
        }
    } else {
        Py_intptr_t val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (Py_intptr_t) -1;
        val = __Pyx_PyInt_As_Py_intptr_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Py_intptr_t");
    return (Py_intptr_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Py_intptr_t");
    return (Py_intptr_t) -1;
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        float denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(a, a);
                    case 3:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, a);
                    case 4:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_absf(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

static CYTHON_INLINE long __Pyx_pow_long(long b, long e) {
    long t = b;
    switch (e) {
        case 3:
            t *= b;
        case 2:
            t *= b;
        case 1:
            return t;
        case 0:
            return 1;
    }
    #if 1
    if (unlikely(e<0)) return 0;
    #endif
    t = 1;
    while (likely(e)) {
        t *= (b * (e&1)) | ((~e)&1);
        b *= b;
        e >>= 1;
    }
    return t;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) -1, const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
