/* Generated by Cython 0.23.4 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_4"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__scipy__linalg__cython_lapack
#define __PYX_HAVE_API__scipy__linalg__cython_lapack
#include "fortran_defs.h"
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#include "_lapack_subroutines.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "scipy/linalg/cython_lapack.pyx",
  "__init__.pxd",
  "type.pxd",
};

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":725
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":726
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":727
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":728
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":732
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":733
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":734
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":735
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":739
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":740
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":749
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":750
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":751
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":753
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":754
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":755
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":757
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":758
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":760
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":761
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":762
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;

/* "scipy/linalg/cython_lapack.pxd":15
 * # The original libraries should be linked directly.
 * 
 * ctypedef float s             # <<<<<<<<<<<<<<
 * ctypedef double d
 * ctypedef float complex c
 */
typedef float __pyx_t_5scipy_6linalg_13cython_lapack_s;

/* "scipy/linalg/cython_lapack.pxd":16
 * 
 * ctypedef float s
 * ctypedef double d             # <<<<<<<<<<<<<<
 * ctypedef float complex c
 * ctypedef double complex z
 */
typedef double __pyx_t_5scipy_6linalg_13cython_lapack_d;
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif


/*--- Type declarations ---*/

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":764
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":765
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":766
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":768
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "scipy/linalg/cython_lapack.pxd":22
 * # Function pointer type declarations for
 * # gees and gges families of functions.
 * ctypedef bint cselect1(c*)             # <<<<<<<<<<<<<<
 * ctypedef bint cselect2(c*, c*)
 * ctypedef bint dselect2(d*, d*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_cselect1(__pyx_t_float_complex *);

/* "scipy/linalg/cython_lapack.pxd":23
 * # gees and gges families of functions.
 * ctypedef bint cselect1(c*)
 * ctypedef bint cselect2(c*, c*)             # <<<<<<<<<<<<<<
 * ctypedef bint dselect2(d*, d*)
 * ctypedef bint dselect3(d*, d*, d*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_cselect2(__pyx_t_float_complex *, __pyx_t_float_complex *);

/* "scipy/linalg/cython_lapack.pxd":24
 * ctypedef bint cselect1(c*)
 * ctypedef bint cselect2(c*, c*)
 * ctypedef bint dselect2(d*, d*)             # <<<<<<<<<<<<<<
 * ctypedef bint dselect3(d*, d*, d*)
 * ctypedef bint sselect2(s*, s*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_dselect2(__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *);

/* "scipy/linalg/cython_lapack.pxd":25
 * ctypedef bint cselect2(c*, c*)
 * ctypedef bint dselect2(d*, d*)
 * ctypedef bint dselect3(d*, d*, d*)             # <<<<<<<<<<<<<<
 * ctypedef bint sselect2(s*, s*)
 * ctypedef bint sselect3(s*, s*, s*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_dselect3(__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *);

/* "scipy/linalg/cython_lapack.pxd":26
 * ctypedef bint dselect2(d*, d*)
 * ctypedef bint dselect3(d*, d*, d*)
 * ctypedef bint sselect2(s*, s*)             # <<<<<<<<<<<<<<
 * ctypedef bint sselect3(s*, s*, s*)
 * ctypedef bint zselect1(z*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_sselect2(__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *);

/* "scipy/linalg/cython_lapack.pxd":27
 * ctypedef bint dselect3(d*, d*, d*)
 * ctypedef bint sselect2(s*, s*)
 * ctypedef bint sselect3(s*, s*, s*)             # <<<<<<<<<<<<<<
 * ctypedef bint zselect1(z*)
 * ctypedef bint zselect2(z*, z*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_sselect3(__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *);

/* "scipy/linalg/cython_lapack.pxd":28
 * ctypedef bint sselect2(s*, s*)
 * ctypedef bint sselect3(s*, s*, s*)
 * ctypedef bint zselect1(z*)             # <<<<<<<<<<<<<<
 * ctypedef bint zselect2(z*, z*)
 * 
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_zselect1(__pyx_t_double_complex *);

/* "scipy/linalg/cython_lapack.pxd":29
 * ctypedef bint sselect3(s*, s*, s*)
 * ctypedef bint zselect1(z*)
 * ctypedef bint zselect2(z*, z*)             # <<<<<<<<<<<<<<
 * 
 * cdef void cbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, c *vt, int *ldvt, c *u, int *ldu, c *c, int *ldc, s *rwork, int *info) nogil
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_zselect2(__pyx_t_double_complex *, __pyx_t_double_complex *);

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if (defined(_WIN32) || defined(__clang__)) && defined(__cplusplus) && CYTHON_CCOMPLEX
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eqf(a, b)   ((a)==(b))
    #define __Pyx_c_sumf(a, b)  ((a)+(b))
    #define __Pyx_c_difff(a, b) ((a)-(b))
    #define __Pyx_c_prodf(a, b) ((a)*(b))
    #define __Pyx_c_quotf(a, b) ((a)/(b))
    #define __Pyx_c_negf(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zerof(z) ((z)==(float)0)
    #define __Pyx_c_conjf(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_absf(z)     (::std::abs(z))
        #define __Pyx_c_powf(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zerof(z) ((z)==0)
    #define __Pyx_c_conjf(z)    (conjf(z))
    #if 1
        #define __Pyx_c_absf(z)     (cabsf(z))
        #define __Pyx_c_powf(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq(a, b)   ((a)==(b))
    #define __Pyx_c_sum(a, b)  ((a)+(b))
    #define __Pyx_c_diff(a, b) ((a)-(b))
    #define __Pyx_c_prod(a, b) ((a)*(b))
    #define __Pyx_c_quot(a, b) ((a)/(b))
    #define __Pyx_c_neg(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero(z) ((z)==(double)0)
    #define __Pyx_c_conj(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (::std::abs(z))
        #define __Pyx_c_pow(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero(z) ((z)==0)
    #define __Pyx_c_conj(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (cabs(z))
        #define __Pyx_c_pow(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static int __Pyx_check_binary_version(void);

static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/

/* Module declarations from 'scipy.linalg.cython_lapack' */
static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlamch(char *); /*proto*/
static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slamch(char *); /*proto*/
#define __Pyx_MODULE_NAME "scipy.linalg.cython_lapack"
int __pyx_module_is_main_scipy__linalg__cython_lapack = 0;

/* Implementation of 'scipy.linalg.cython_lapack' */
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_RuntimeError;
static char __pyx_k_B[] = "B";
static char __pyx_k_H[] = "H";
static char __pyx_k_I[] = "I";
static char __pyx_k_L[] = "L";
static char __pyx_k_O[] = "O";
static char __pyx_k_Q[] = "Q";
static char __pyx_k_b[] = "b";
static char __pyx_k_d[] = "d";
static char __pyx_k_f[] = "f";
static char __pyx_k_g[] = "g";
static char __pyx_k_h[] = "h";
static char __pyx_k_i[] = "i";
static char __pyx_k_l[] = "l";
static char __pyx_k_q[] = "q";
static char __pyx_k_Zd[] = "Zd";
static char __pyx_k_Zf[] = "Zf";
static char __pyx_k_Zg[] = "Zg";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_cmach[] = "cmach";
static char __pyx_k_range[] = "range";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_cmach_char[] = "cmach_char";
static char __pyx_k_cmach_bytes[] = "cmach_bytes";
static char __pyx_k_test_dlamch[] = "_test_dlamch";
static char __pyx_k_test_slamch[] = "_test_slamch";
static char __pyx_k_RuntimeError[] = "RuntimeError";
static char __pyx_k_scipy_linalg_cython_lapack[] = "scipy.linalg.cython_lapack";
static char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static char __pyx_k_LAPACK_functions_for_Cython_Usa[] = "\nLAPACK functions for Cython\n===========================\n\nUsable from Cython via::\n\n    cimport scipy.linalg.cython_lapack\n\nThis module provides Cython-level wrappers for all primary routines included\nin LAPACK 3.1.0 except for ``zcgesv`` since its interface is not consistent\nfrom LAPACK 3.1.0 to 3.6.0. It also provides some of the\nfixed-api auxiliary routines.\n\nThese wrappers do not check for alignment of arrays.\nAlignment should be checked before these wrappers are used.\n\nRaw function pointers (Fortran-style pointer arguments):\n\n- cbdsqr\n- cgbbrd\n- cgbcon\n- cgbequ\n- cgbrfs\n- cgbsv\n- cgbsvx\n- cgbtf2\n- cgbtrf\n- cgbtrs\n- cgebak\n- cgebal\n- cgebd2\n- cgebrd\n- cgecon\n- cgeequ\n- cgees\n- cgeesx\n- cgeev\n- cgeevx\n- cgehd2\n- cgehrd\n- cgelq2\n- cgelqf\n- cgels\n- cgelsd\n- cgelss\n- cgelsy\n- cgeql2\n- cgeqlf\n- cgeqp3\n- cgeqr2\n- cgeqrf\n- cgerfs\n- cgerq2\n- cgerqf\n- cgesc2\n- cgesdd\n- cgesv\n- cgesvd\n- cgesvx\n- cgetc2\n- cgetf2\n- cgetrf\n- cgetri\n- cgetrs\n- cggbak\n- cggbal\n- cgges\n- cggesx\n- cggev\n- cggevx\n- cggglm\n- cgghrd\n- cgglse\n- cggqrf\n- cggrqf\n- cgtcon\n- cgtrfs\n- cgtsv\n- cgtsvx\n- cgttrf\n- cgttrs\n- cgtts2\n- chbev\n- chbevd\n- chbevx\n- chbgst\n- chbgv\n- chbgvd\n- chbgvx\n- chbtrd\n- checon\n- cheev\n- cheevd\n- cheevr\n- cheevx\n- chegs2\n- chegst\n- chegv\n- chegvd\n- chegvx\n- cherfs\n- chesv\n- chesvx\n- chetd2\n- chetf2\n- chetrd\n- chetrf\n- chetri\n- chetrs\n- chgeqz\n- chpcon\n- chpev\n- chpevd\n- chpevx\n- chpgst\n- chpgv\n- chpgvd\n- chpgvx\n- chprfs\n- chpsv\n- chpsvx\n- chptrd\n- chptrf\n- chptri\n- chptrs\n- chsein\n- chseqr\n- clabrd\n- clacgv\n- clacn2\n- clacon\n- clacp2\n- clacpy\n- clacrm\n- clacrt\n- cladiv\n- claed0\n- claed7\n- claed8\n- claein\n- claesy\n- claev2\n- clag2z\n- clags2\n- clagtm\n- clahef\n- clahqr\n- clahr2\n- claic1\n- clals0\n- clalsa\n- clalsd\n- clangb\n- clange\n- clangt\n- clanhb\n- clanhe\n- clanhp\n- clanhs\n- clanht\n- clansb\n- clansp\n- clansy\n- clantb""\n- clantp\n- clantr\n- clapll\n- clapmt\n- claqgb\n- claqge\n- claqhb\n- claqhe\n- claqhp\n- claqp2\n- claqps\n- claqr0\n- claqr1\n- claqr2\n- claqr3\n- claqr4\n- claqr5\n- claqsb\n- claqsp\n- claqsy\n- clar1v\n- clar2v\n- clarcm\n- clarf\n- clarfb\n- clarfg\n- clarft\n- clarfx\n- clargv\n- clarnv\n- clarrv\n- clartg\n- clartv\n- clarz\n- clarzb\n- clarzt\n- clascl\n- claset\n- clasr\n- classq\n- claswp\n- clasyf\n- clatbs\n- clatdf\n- clatps\n- clatrd\n- clatrs\n- clatrz\n- clauu2\n- clauum\n- cpbcon\n- cpbequ\n- cpbrfs\n- cpbstf\n- cpbsv\n- cpbsvx\n- cpbtf2\n- cpbtrf\n- cpbtrs\n- cpocon\n- cpoequ\n- cporfs\n- cposv\n- cposvx\n- cpotf2\n- cpotrf\n- cpotri\n- cpotrs\n- cppcon\n- cppequ\n- cpprfs\n- cppsv\n- cppsvx\n- cpptrf\n- cpptri\n- cpptrs\n- cptcon\n- cpteqr\n- cptrfs\n- cptsv\n- cptsvx\n- cpttrf\n- cpttrs\n- cptts2\n- crot\n- cspcon\n- cspmv\n- cspr\n- csprfs\n- cspsv\n- cspsvx\n- csptrf\n- csptri\n- csptrs\n- csrscl\n- cstedc\n- cstegr\n- cstein\n- cstemr\n- csteqr\n- csycon\n- csymv\n- csyr\n- csyrfs\n- csysv\n- csysvx\n- csytf2\n- csytrf\n- csytri\n- csytrs\n- ctbcon\n- ctbrfs\n- ctbtrs\n- ctgevc\n- ctgex2\n- ctgexc\n- ctgsen\n- ctgsja\n- ctgsna\n- ctgsy2\n- ctgsyl\n- ctpcon\n- ctprfs\n- ctptri\n- ctptrs\n- ctrcon\n- ctrevc\n- ctrexc\n- ctrrfs\n- ctrsen\n- ctrsna\n- ctrsyl\n- ctrti2\n- ctrtri\n- ctrtrs\n- ctzrzf\n- cung2l\n- cung2r\n- cungbr\n- cunghr\n- cungl2\n- cunglq\n- cungql\n- cungqr\n- cungr2\n- cungrq\n- cungtr\n- cunm2l\n- cunm2r\n- cunmbr\n- cunmhr\n- cunml2\n- cunmlq\n- cunmql\n- cunmqr\n- cunmr2\n- cunmr3\n- cunmrq\n- cunmrz\n- cunmtr\n- cupgtr\n- cupmtr\n- dbdsdc\n- dbdsqr\n- ddisna\n- dgbbrd\n- dgbcon\n- dgbequ\n- dgbrfs\n- dgbsv\n- dgbsvx\n- dgbtf2\n- dgbtrf\n- dgbtrs\n- dgebak\n- dgebal\n- dgebd2\n- dgebrd\n- dgecon\n- dgeequ\n- dgees\n- dgeesx\n- dgeev\n- dgeevx\n- dgehd2\n- dgehrd\n- dgelq2\n- dgelqf\n- dgels\n- dgelsd\n- dgelss\n- dgelsy\n- dgeql2\n- dgeqlf\n- dgeqp3\n- dgeqr2\n- dgeqrf\n- dgerfs\n- dgerq2\n- dgerqf\n- dgesc2\n- dgesdd""\n- dgesv\n- dgesvd\n- dgesvx\n- dgetc2\n- dgetf2\n- dgetrf\n- dgetri\n- dgetrs\n- dggbak\n- dggbal\n- dgges\n- dggesx\n- dggev\n- dggevx\n- dggglm\n- dgghrd\n- dgglse\n- dggqrf\n- dggrqf\n- dgtcon\n- dgtrfs\n- dgtsv\n- dgtsvx\n- dgttrf\n- dgttrs\n- dgtts2\n- dhgeqz\n- dhsein\n- dhseqr\n- disnan\n- dlabad\n- dlabrd\n- dlacn2\n- dlacon\n- dlacpy\n- dladiv\n- dlae2\n- dlaebz\n- dlaed0\n- dlaed1\n- dlaed2\n- dlaed3\n- dlaed4\n- dlaed5\n- dlaed6\n- dlaed7\n- dlaed8\n- dlaed9\n- dlaeda\n- dlaein\n- dlaev2\n- dlaexc\n- dlag2\n- dlag2s\n- dlags2\n- dlagtf\n- dlagtm\n- dlagts\n- dlagv2\n- dlahqr\n- dlahr2\n- dlaic1\n- dlaln2\n- dlals0\n- dlalsa\n- dlalsd\n- dlamch\n- dlamrg\n- dlaneg\n- dlangb\n- dlange\n- dlangt\n- dlanhs\n- dlansb\n- dlansp\n- dlanst\n- dlansy\n- dlantb\n- dlantp\n- dlantr\n- dlanv2\n- dlapll\n- dlapmt\n- dlapy2\n- dlapy3\n- dlaqgb\n- dlaqge\n- dlaqp2\n- dlaqps\n- dlaqr0\n- dlaqr1\n- dlaqr2\n- dlaqr3\n- dlaqr4\n- dlaqr5\n- dlaqsb\n- dlaqsp\n- dlaqsy\n- dlaqtr\n- dlar1v\n- dlar2v\n- dlarf\n- dlarfb\n- dlarfg\n- dlarft\n- dlarfx\n- dlargv\n- dlarnv\n- dlarra\n- dlarrb\n- dlarrc\n- dlarrd\n- dlarre\n- dlarrf\n- dlarrj\n- dlarrk\n- dlarrr\n- dlarrv\n- dlartg\n- dlartv\n- dlaruv\n- dlarz\n- dlarzb\n- dlarzt\n- dlas2\n- dlascl\n- dlasd0\n- dlasd1\n- dlasd2\n- dlasd3\n- dlasd4\n- dlasd5\n- dlasd6\n- dlasd7\n- dlasd8\n- dlasda\n- dlasdq\n- dlasdt\n- dlaset\n- dlasq1\n- dlasq2\n- dlasq6\n- dlasr\n- dlasrt\n- dlassq\n- dlasv2\n- dlaswp\n- dlasy2\n- dlasyf\n- dlatbs\n- dlatdf\n- dlatps\n- dlatrd\n- dlatrs\n- dlatrz\n- dlauu2\n- dlauum\n- dopgtr\n- dopmtr\n- dorg2l\n- dorg2r\n- dorgbr\n- dorghr\n- dorgl2\n- dorglq\n- dorgql\n- dorgqr\n- dorgr2\n- dorgrq\n- dorgtr\n- dorm2l\n- dorm2r\n- dormbr\n- dormhr\n- dorml2\n- dormlq\n- dormql\n- dormqr\n- dormr2\n- dormr3\n- dormrq\n- dormrz\n- dormtr\n- dpbcon\n- dpbequ\n- dpbrfs\n- dpbstf\n- dpbsv\n- dpbsvx\n- dpbtf2\n- dpbtrf\n- dpbtrs\n- dpocon\n- dpoequ\n- dporfs\n- dposv\n- dposvx\n- dpotf2\n- dpotrf\n- dpotri\n- dpotrs""\n- dppcon\n- dppequ\n- dpprfs\n- dppsv\n- dppsvx\n- dpptrf\n- dpptri\n- dpptrs\n- dptcon\n- dpteqr\n- dptrfs\n- dptsv\n- dptsvx\n- dpttrf\n- dpttrs\n- dptts2\n- drscl\n- dsbev\n- dsbevd\n- dsbevx\n- dsbgst\n- dsbgv\n- dsbgvd\n- dsbgvx\n- dsbtrd\n- dsgesv\n- dspcon\n- dspev\n- dspevd\n- dspevx\n- dspgst\n- dspgv\n- dspgvd\n- dspgvx\n- dsprfs\n- dspsv\n- dspsvx\n- dsptrd\n- dsptrf\n- dsptri\n- dsptrs\n- dstebz\n- dstedc\n- dstegr\n- dstein\n- dstemr\n- dsteqr\n- dsterf\n- dstev\n- dstevd\n- dstevr\n- dstevx\n- dsycon\n- dsyev\n- dsyevd\n- dsyevr\n- dsyevx\n- dsygs2\n- dsygst\n- dsygv\n- dsygvd\n- dsygvx\n- dsyrfs\n- dsysv\n- dsysvx\n- dsytd2\n- dsytf2\n- dsytrd\n- dsytrf\n- dsytri\n- dsytrs\n- dtbcon\n- dtbrfs\n- dtbtrs\n- dtgevc\n- dtgex2\n- dtgexc\n- dtgsen\n- dtgsja\n- dtgsna\n- dtgsy2\n- dtgsyl\n- dtpcon\n- dtprfs\n- dtptri\n- dtptrs\n- dtrcon\n- dtrevc\n- dtrexc\n- dtrrfs\n- dtrsen\n- dtrsna\n- dtrsyl\n- dtrti2\n- dtrtri\n- dtrtrs\n- dtzrzf\n- dzsum1\n- icmax1\n- ieeeck\n- ilaver\n- izmax1\n- sbdsdc\n- sbdsqr\n- scsum1\n- sdisna\n- sgbbrd\n- sgbcon\n- sgbequ\n- sgbrfs\n- sgbsv\n- sgbsvx\n- sgbtf2\n- sgbtrf\n- sgbtrs\n- sgebak\n- sgebal\n- sgebd2\n- sgebrd\n- sgecon\n- sgeequ\n- sgees\n- sgeesx\n- sgeev\n- sgeevx\n- sgehd2\n- sgehrd\n- sgelq2\n- sgelqf\n- sgels\n- sgelsd\n- sgelss\n- sgelsy\n- sgeql2\n- sgeqlf\n- sgeqp3\n- sgeqr2\n- sgeqrf\n- sgerfs\n- sgerq2\n- sgerqf\n- sgesc2\n- sgesdd\n- sgesv\n- sgesvd\n- sgesvx\n- sgetc2\n- sgetf2\n- sgetrf\n- sgetri\n- sgetrs\n- sggbak\n- sggbal\n- sgges\n- sggesx\n- sggev\n- sggevx\n- sggglm\n- sgghrd\n- sgglse\n- sggqrf\n- sggrqf\n- sgtcon\n- sgtrfs\n- sgtsv\n- sgtsvx\n- sgttrf\n- sgttrs\n- sgtts2\n- shgeqz\n- shsein\n- shseqr\n- slabad\n- slabrd\n- slacn2\n- slacon\n- slacpy\n- sladiv\n- slae2\n- slaebz\n- slaed0\n- slaed1\n- slaed2\n- slaed3\n- slaed4\n- slaed5\n- slaed6\n- slaed7\n- slaed8\n- slaed9\n- slaeda\n- slaein\n- slaev2\n- slaexc\n- slag2\n- slag2d\n- slags2\n- slagtf\n- slagtm\n- slagts\n- slagv2\n- slahqr""\n- slahr2\n- slaic1\n- slaln2\n- slals0\n- slalsa\n- slalsd\n- slamch\n- slamrg\n- slangb\n- slange\n- slangt\n- slanhs\n- slansb\n- slansp\n- slanst\n- slansy\n- slantb\n- slantp\n- slantr\n- slanv2\n- slapll\n- slapmt\n- slapy2\n- slapy3\n- slaqgb\n- slaqge\n- slaqp2\n- slaqps\n- slaqr0\n- slaqr1\n- slaqr2\n- slaqr3\n- slaqr4\n- slaqr5\n- slaqsb\n- slaqsp\n- slaqsy\n- slaqtr\n- slar1v\n- slar2v\n- slarf\n- slarfb\n- slarfg\n- slarft\n- slarfx\n- slargv\n- slarnv\n- slarra\n- slarrb\n- slarrc\n- slarrd\n- slarre\n- slarrf\n- slarrj\n- slarrk\n- slarrr\n- slarrv\n- slartg\n- slartv\n- slaruv\n- slarz\n- slarzb\n- slarzt\n- slas2\n- slascl\n- slasd0\n- slasd1\n- slasd2\n- slasd3\n- slasd4\n- slasd5\n- slasd6\n- slasd7\n- slasd8\n- slasda\n- slasdq\n- slasdt\n- slaset\n- slasq1\n- slasq2\n- slasq6\n- slasr\n- slasrt\n- slassq\n- slasv2\n- slaswp\n- slasy2\n- slasyf\n- slatbs\n- slatdf\n- slatps\n- slatrd\n- slatrs\n- slatrz\n- slauu2\n- slauum\n- sopgtr\n- sopmtr\n- sorg2l\n- sorg2r\n- sorgbr\n- sorghr\n- sorgl2\n- sorglq\n- sorgql\n- sorgqr\n- sorgr2\n- sorgrq\n- sorgtr\n- sorm2l\n- sorm2r\n- sormbr\n- sormhr\n- sorml2\n- sormlq\n- sormql\n- sormqr\n- sormr2\n- sormr3\n- sormrq\n- sormrz\n- sormtr\n- spbcon\n- spbequ\n- spbrfs\n- spbstf\n- spbsv\n- spbsvx\n- spbtf2\n- spbtrf\n- spbtrs\n- spocon\n- spoequ\n- sporfs\n- sposv\n- sposvx\n- spotf2\n- spotrf\n- spotri\n- spotrs\n- sppcon\n- sppequ\n- spprfs\n- sppsv\n- sppsvx\n- spptrf\n- spptri\n- spptrs\n- sptcon\n- spteqr\n- sptrfs\n- sptsv\n- sptsvx\n- spttrf\n- spttrs\n- sptts2\n- srscl\n- ssbev\n- ssbevd\n- ssbevx\n- ssbgst\n- ssbgv\n- ssbgvd\n- ssbgvx\n- ssbtrd\n- sspcon\n- sspev\n- sspevd\n- sspevx\n- sspgst\n- sspgv\n- sspgvd\n- sspgvx\n- ssprfs\n- sspsv\n- sspsvx\n- ssptrd\n- ssptrf\n- ssptri\n- ssptrs\n- sstebz\n- sstedc\n- sstegr\n- sstein\n- sstemr\n- ssteqr\n- ssterf\n- sstev\n- sstevd\n- sstevr\n- sstevx\n- ssycon\n- ssyev\n- ssyevd\n- ssyevr\n- ssyevx\n- ssygs2\n- ssygst\n- ssygv\n- ssygvd\n- ssygvx\n- ssy""rfs\n- ssysv\n- ssysvx\n- ssytd2\n- ssytf2\n- ssytrd\n- ssytrf\n- ssytri\n- ssytrs\n- stbcon\n- stbrfs\n- stbtrs\n- stgevc\n- stgex2\n- stgexc\n- stgsen\n- stgsja\n- stgsna\n- stgsy2\n- stgsyl\n- stpcon\n- stprfs\n- stptri\n- stptrs\n- strcon\n- strevc\n- strexc\n- strrfs\n- strsen\n- strsna\n- strsyl\n- strti2\n- strtri\n- strtrs\n- stzrzf\n- zbdsqr\n- zdrscl\n- zgbbrd\n- zgbcon\n- zgbequ\n- zgbrfs\n- zgbsv\n- zgbsvx\n- zgbtf2\n- zgbtrf\n- zgbtrs\n- zgebak\n- zgebal\n- zgebd2\n- zgebrd\n- zgecon\n- zgeequ\n- zgees\n- zgeesx\n- zgeev\n- zgeevx\n- zgehd2\n- zgehrd\n- zgelq2\n- zgelqf\n- zgels\n- zgelsd\n- zgelss\n- zgelsy\n- zgeql2\n- zgeqlf\n- zgeqp3\n- zgeqr2\n- zgeqrf\n- zgerfs\n- zgerq2\n- zgerqf\n- zgesc2\n- zgesdd\n- zgesv\n- zgesvd\n- zgesvx\n- zgetc2\n- zgetf2\n- zgetrf\n- zgetri\n- zgetrs\n- zggbak\n- zggbal\n- zgges\n- zggesx\n- zggev\n- zggevx\n- zggglm\n- zgghrd\n- zgglse\n- zggqrf\n- zggrqf\n- zgtcon\n- zgtrfs\n- zgtsv\n- zgtsvx\n- zgttrf\n- zgttrs\n- zgtts2\n- zhbev\n- zhbevd\n- zhbevx\n- zhbgst\n- zhbgv\n- zhbgvd\n- zhbgvx\n- zhbtrd\n- zhecon\n- zheev\n- zheevd\n- zheevr\n- zheevx\n- zhegs2\n- zhegst\n- zhegv\n- zhegvd\n- zhegvx\n- zherfs\n- zhesv\n- zhesvx\n- zhetd2\n- zhetf2\n- zhetrd\n- zhetrf\n- zhetri\n- zhetrs\n- zhgeqz\n- zhpcon\n- zhpev\n- zhpevd\n- zhpevx\n- zhpgst\n- zhpgv\n- zhpgvd\n- zhpgvx\n- zhprfs\n- zhpsv\n- zhpsvx\n- zhptrd\n- zhptrf\n- zhptri\n- zhptrs\n- zhsein\n- zhseqr\n- zlabrd\n- zlacgv\n- zlacn2\n- zlacon\n- zlacp2\n- zlacpy\n- zlacrm\n- zlacrt\n- zladiv\n- zlaed0\n- zlaed7\n- zlaed8\n- zlaein\n- zlaesy\n- zlaev2\n- zlag2c\n- zlags2\n- zlagtm\n- zlahef\n- zlahqr\n- zlahr2\n- zlaic1\n- zlals0\n- zlalsa\n- zlalsd\n- zlangb\n- zlange\n- zlangt\n- zlanhb\n- zlanhe\n- zlanhp\n- zlanhs\n- zlanht\n- zlansb\n- zlansp\n- zlansy\n- zlantb\n- zlantp\n- zlantr\n- zlapll\n- zlapmt\n- zlaqgb\n- zlaqge\n- zlaqhb\n- zlaqhe\n- zlaqhp\n- zlaqp2\n- zlaqps\n- zlaqr0\n- zlaqr1\n- zlaqr2\n- zlaqr3\n- zlaqr4\n- zlaqr5\n- zlaqsb\n- zlaqsp\n- zlaqsy""\n- zlar1v\n- zlar2v\n- zlarcm\n- zlarf\n- zlarfb\n- zlarfg\n- zlarft\n- zlarfx\n- zlargv\n- zlarnv\n- zlarrv\n- zlartg\n- zlartv\n- zlarz\n- zlarzb\n- zlarzt\n- zlascl\n- zlaset\n- zlasr\n- zlassq\n- zlaswp\n- zlasyf\n- zlatbs\n- zlatdf\n- zlatps\n- zlatrd\n- zlatrs\n- zlatrz\n- zlauu2\n- zlauum\n- zpbcon\n- zpbequ\n- zpbrfs\n- zpbstf\n- zpbsv\n- zpbsvx\n- zpbtf2\n- zpbtrf\n- zpbtrs\n- zpocon\n- zpoequ\n- zporfs\n- zposv\n- zposvx\n- zpotf2\n- zpotrf\n- zpotri\n- zpotrs\n- zppcon\n- zppequ\n- zpprfs\n- zppsv\n- zppsvx\n- zpptrf\n- zpptri\n- zpptrs\n- zptcon\n- zpteqr\n- zptrfs\n- zptsv\n- zptsvx\n- zpttrf\n- zpttrs\n- zptts2\n- zrot\n- zspcon\n- zspmv\n- zspr\n- zsprfs\n- zspsv\n- zspsvx\n- zsptrf\n- zsptri\n- zsptrs\n- zstedc\n- zstegr\n- zstein\n- zstemr\n- zsteqr\n- zsycon\n- zsymv\n- zsyr\n- zsyrfs\n- zsysv\n- zsysvx\n- zsytf2\n- zsytrf\n- zsytri\n- zsytrs\n- ztbcon\n- ztbrfs\n- ztbtrs\n- ztgevc\n- ztgex2\n- ztgexc\n- ztgsen\n- ztgsja\n- ztgsna\n- ztgsy2\n- ztgsyl\n- ztpcon\n- ztprfs\n- ztptri\n- ztptrs\n- ztrcon\n- ztrevc\n- ztrexc\n- ztrrfs\n- ztrsen\n- ztrsna\n- ztrsyl\n- ztrti2\n- ztrtri\n- ztrtrs\n- ztzrzf\n- zung2l\n- zung2r\n- zungbr\n- zunghr\n- zungl2\n- zunglq\n- zungql\n- zungqr\n- zungr2\n- zungrq\n- zungtr\n- zunm2l\n- zunm2r\n- zunmbr\n- zunmhr\n- zunml2\n- zunmlq\n- zunmql\n- zunmqr\n- zunmr2\n- zunmr3\n- zunmrq\n- zunmrz\n- zunmtr\n- zupgtr\n- zupmtr\n\n\n";
static char __pyx_k_home_br_repos_scipy_scipy_linal[] = "/home/br/repos/scipy/scipy/linalg/cython_lapack.pyx";
static char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_cmach;
static PyObject *__pyx_n_s_cmach_bytes;
static PyObject *__pyx_n_s_cmach_char;
static PyObject *__pyx_kp_s_home_br_repos_scipy_scipy_linal;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_scipy_linalg_cython_lapack;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_test_dlamch;
static PyObject *__pyx_n_s_test_slamch;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_pf_5scipy_6linalg_13cython_lapack__test_dlamch(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cmach); /* proto */
static PyObject *__pyx_pf_5scipy_6linalg_13cython_lapack_2_test_slamch(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cmach); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_codeobj__8;
static PyObject *__pyx_codeobj__10;

/* "scipy/linalg/cython_lapack.pyx":1352
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cladiv "F_FUNC(cladivwrp, CLADIVWRP)"(c *out, npy_complex64 *x, npy_complex64 *y) nogil
 * cdef c cladiv(c *x, c *y) nogil:             # <<<<<<<<<<<<<<
 *     cdef c out
 *     _fortran_cladiv(&out, <npy_complex64*>x, <npy_complex64*>y)
 */

static __pyx_t_float_complex __pyx_f_5scipy_6linalg_13cython_lapack_cladiv(__pyx_t_float_complex *__pyx_v_x, __pyx_t_float_complex *__pyx_v_y) {
  __pyx_t_float_complex __pyx_v_out;
  __pyx_t_float_complex __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1354
 * cdef c cladiv(c *x, c *y) nogil:
 *     cdef c out
 *     _fortran_cladiv(&out, <npy_complex64*>x, <npy_complex64*>y)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(cladivwrp, CLADIVWRP)((&__pyx_v_out), ((npy_complex64 *)__pyx_v_x), ((npy_complex64 *)__pyx_v_y));

  /* "scipy/linalg/cython_lapack.pyx":1355
 *     cdef c out
 *     _fortran_cladiv(&out, <npy_complex64*>x, <npy_complex64*>y)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1352
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cladiv "F_FUNC(cladivwrp, CLADIVWRP)"(c *out, npy_complex64 *x, npy_complex64 *y) nogil
 * cdef c cladiv(c *x, c *y) nogil:             # <<<<<<<<<<<<<<
 *     cdef c out
 *     _fortran_cladiv(&out, <npy_complex64*>x, <npy_complex64*>y)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1360
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clangb "F_FUNC(clangbwrp, CLANGBWRP)"(s *out, char *norm, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *work) nogil
 * cdef s clangb(char *norm, int *n, int *kl, int *ku, c *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clangb(&out, norm, n, kl, ku, <npy_complex64*>ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clangb(char *__pyx_v_norm, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1362
 * cdef s clangb(char *norm, int *n, int *kl, int *ku, c *ab, int *ldab, s *work) nogil:
 *     cdef s out
 *     _fortran_clangb(&out, norm, n, kl, ku, <npy_complex64*>ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clangbwrp, CLANGBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1363
 *     cdef s out
 *     _fortran_clangb(&out, norm, n, kl, ku, <npy_complex64*>ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1360
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clangb "F_FUNC(clangbwrp, CLANGBWRP)"(s *out, char *norm, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *work) nogil
 * cdef s clangb(char *norm, int *n, int *kl, int *ku, c *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clangb(&out, norm, n, kl, ku, <npy_complex64*>ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1368
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clange "F_FUNC(clangewrp, CLANGEWRP)"(s *out, char *norm, int *m, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clange(char *norm, int *m, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clange(&out, norm, m, n, <npy_complex64*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clange(char *__pyx_v_norm, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1370
 * cdef s clange(char *norm, int *m, int *n, c *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_clange(&out, norm, m, n, <npy_complex64*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clangewrp, CLANGEWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1371
 *     cdef s out
 *     _fortran_clange(&out, norm, m, n, <npy_complex64*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1368
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clange "F_FUNC(clangewrp, CLANGEWRP)"(s *out, char *norm, int *m, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clange(char *norm, int *m, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clange(&out, norm, m, n, <npy_complex64*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clangt "F_FUNC(clangtwrp, CLANGTWRP)"(s *out, char *norm, int *n, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du) nogil
 * cdef s clangt(char *norm, int *n, c *dl, c *d, c *du) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clangt(&out, norm, n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clangt(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1378
 * cdef s clangt(char *norm, int *n, c *dl, c *d, c *du) nogil:
 *     cdef s out
 *     _fortran_clangt(&out, norm, n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clangtwrp, CLANGTWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du));

  /* "scipy/linalg/cython_lapack.pyx":1379
 *     cdef s out
 *     _fortran_clangt(&out, norm, n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clangt "F_FUNC(clangtwrp, CLANGTWRP)"(s *out, char *norm, int *n, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du) nogil
 * cdef s clangt(char *norm, int *n, c *dl, c *d, c *du) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clangt(&out, norm, n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1384
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanhb "F_FUNC(clanhbwrp, CLANHBWRP)"(s *out, char *norm, char *uplo, int *n, int *k, npy_complex64 *ab, int *ldab, s *work) nogil
 * cdef s clanhb(char *norm, char *uplo, int *n, int *k, c *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanhb(&out, norm, uplo, n, k, <npy_complex64*>ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clanhb(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1386
 * cdef s clanhb(char *norm, char *uplo, int *n, int *k, c *ab, int *ldab, s *work) nogil:
 *     cdef s out
 *     _fortran_clanhb(&out, norm, uplo, n, k, <npy_complex64*>ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clanhbwrp, CLANHBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1387
 *     cdef s out
 *     _fortran_clanhb(&out, norm, uplo, n, k, <npy_complex64*>ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1384
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanhb "F_FUNC(clanhbwrp, CLANHBWRP)"(s *out, char *norm, char *uplo, int *n, int *k, npy_complex64 *ab, int *ldab, s *work) nogil
 * cdef s clanhb(char *norm, char *uplo, int *n, int *k, c *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanhb(&out, norm, uplo, n, k, <npy_complex64*>ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1392
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanhe "F_FUNC(clanhewrp, CLANHEWRP)"(s *out, char *norm, char *uplo, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clanhe(char *norm, char *uplo, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanhe(&out, norm, uplo, n, <npy_complex64*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clanhe(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1394
 * cdef s clanhe(char *norm, char *uplo, int *n, c *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_clanhe(&out, norm, uplo, n, <npy_complex64*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clanhewrp, CLANHEWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1395
 *     cdef s out
 *     _fortran_clanhe(&out, norm, uplo, n, <npy_complex64*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1392
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanhe "F_FUNC(clanhewrp, CLANHEWRP)"(s *out, char *norm, char *uplo, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clanhe(char *norm, char *uplo, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanhe(&out, norm, uplo, n, <npy_complex64*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1400
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanhp "F_FUNC(clanhpwrp, CLANHPWRP)"(s *out, char *norm, char *uplo, int *n, npy_complex64 *ap, s *work) nogil
 * cdef s clanhp(char *norm, char *uplo, int *n, c *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanhp(&out, norm, uplo, n, <npy_complex64*>ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clanhp(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1402
 * cdef s clanhp(char *norm, char *uplo, int *n, c *ap, s *work) nogil:
 *     cdef s out
 *     _fortran_clanhp(&out, norm, uplo, n, <npy_complex64*>ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clanhpwrp, CLANHPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1403
 *     cdef s out
 *     _fortran_clanhp(&out, norm, uplo, n, <npy_complex64*>ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1400
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanhp "F_FUNC(clanhpwrp, CLANHPWRP)"(s *out, char *norm, char *uplo, int *n, npy_complex64 *ap, s *work) nogil
 * cdef s clanhp(char *norm, char *uplo, int *n, c *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanhp(&out, norm, uplo, n, <npy_complex64*>ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1408
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanhs "F_FUNC(clanhswrp, CLANHSWRP)"(s *out, char *norm, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clanhs(char *norm, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanhs(&out, norm, n, <npy_complex64*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clanhs(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1410
 * cdef s clanhs(char *norm, int *n, c *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_clanhs(&out, norm, n, <npy_complex64*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clanhswrp, CLANHSWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1411
 *     cdef s out
 *     _fortran_clanhs(&out, norm, n, <npy_complex64*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1408
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanhs "F_FUNC(clanhswrp, CLANHSWRP)"(s *out, char *norm, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clanhs(char *norm, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanhs(&out, norm, n, <npy_complex64*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanht "F_FUNC(clanhtwrp, CLANHTWRP)"(s *out, char *norm, int *n, s *d, npy_complex64 *e) nogil
 * cdef s clanht(char *norm, int *n, s *d, c *e) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanht(&out, norm, n, d, <npy_complex64*>e)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clanht(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_e) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1418
 * cdef s clanht(char *norm, int *n, s *d, c *e) nogil:
 *     cdef s out
 *     _fortran_clanht(&out, norm, n, d, <npy_complex64*>e)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clanhtwrp, CLANHTWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_d, ((npy_complex64 *)__pyx_v_e));

  /* "scipy/linalg/cython_lapack.pyx":1419
 *     cdef s out
 *     _fortran_clanht(&out, norm, n, d, <npy_complex64*>e)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clanht "F_FUNC(clanhtwrp, CLANHTWRP)"(s *out, char *norm, int *n, s *d, npy_complex64 *e) nogil
 * cdef s clanht(char *norm, int *n, s *d, c *e) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clanht(&out, norm, n, d, <npy_complex64*>e)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1424
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clansb "F_FUNC(clansbwrp, CLANSBWRP)"(s *out, char *norm, char *uplo, int *n, int *k, npy_complex64 *ab, int *ldab, s *work) nogil
 * cdef s clansb(char *norm, char *uplo, int *n, int *k, c *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clansb(&out, norm, uplo, n, k, <npy_complex64*>ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clansb(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1426
 * cdef s clansb(char *norm, char *uplo, int *n, int *k, c *ab, int *ldab, s *work) nogil:
 *     cdef s out
 *     _fortran_clansb(&out, norm, uplo, n, k, <npy_complex64*>ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clansbwrp, CLANSBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1427
 *     cdef s out
 *     _fortran_clansb(&out, norm, uplo, n, k, <npy_complex64*>ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1424
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clansb "F_FUNC(clansbwrp, CLANSBWRP)"(s *out, char *norm, char *uplo, int *n, int *k, npy_complex64 *ab, int *ldab, s *work) nogil
 * cdef s clansb(char *norm, char *uplo, int *n, int *k, c *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clansb(&out, norm, uplo, n, k, <npy_complex64*>ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1432
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clansp "F_FUNC(clanspwrp, CLANSPWRP)"(s *out, char *norm, char *uplo, int *n, npy_complex64 *ap, s *work) nogil
 * cdef s clansp(char *norm, char *uplo, int *n, c *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clansp(&out, norm, uplo, n, <npy_complex64*>ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clansp(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1434
 * cdef s clansp(char *norm, char *uplo, int *n, c *ap, s *work) nogil:
 *     cdef s out
 *     _fortran_clansp(&out, norm, uplo, n, <npy_complex64*>ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clanspwrp, CLANSPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1435
 *     cdef s out
 *     _fortran_clansp(&out, norm, uplo, n, <npy_complex64*>ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1432
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clansp "F_FUNC(clanspwrp, CLANSPWRP)"(s *out, char *norm, char *uplo, int *n, npy_complex64 *ap, s *work) nogil
 * cdef s clansp(char *norm, char *uplo, int *n, c *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clansp(&out, norm, uplo, n, <npy_complex64*>ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1440
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clansy "F_FUNC(clansywrp, CLANSYWRP)"(s *out, char *norm, char *uplo, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clansy(char *norm, char *uplo, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clansy(&out, norm, uplo, n, <npy_complex64*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clansy(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1442
 * cdef s clansy(char *norm, char *uplo, int *n, c *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_clansy(&out, norm, uplo, n, <npy_complex64*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clansywrp, CLANSYWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1443
 *     cdef s out
 *     _fortran_clansy(&out, norm, uplo, n, <npy_complex64*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1440
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clansy "F_FUNC(clansywrp, CLANSYWRP)"(s *out, char *norm, char *uplo, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clansy(char *norm, char *uplo, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clansy(&out, norm, uplo, n, <npy_complex64*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1448
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clantb "F_FUNC(clantbwrp, CLANTBWRP)"(s *out, char *norm, char *uplo, char *diag, int *n, int *k, npy_complex64 *ab, int *ldab, s *work) nogil
 * cdef s clantb(char *norm, char *uplo, char *diag, int *n, int *k, c *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clantb(&out, norm, uplo, diag, n, k, <npy_complex64*>ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clantb(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1450
 * cdef s clantb(char *norm, char *uplo, char *diag, int *n, int *k, c *ab, int *ldab, s *work) nogil:
 *     cdef s out
 *     _fortran_clantb(&out, norm, uplo, diag, n, k, <npy_complex64*>ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clantbwrp, CLANTBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1451
 *     cdef s out
 *     _fortran_clantb(&out, norm, uplo, diag, n, k, <npy_complex64*>ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1448
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clantb "F_FUNC(clantbwrp, CLANTBWRP)"(s *out, char *norm, char *uplo, char *diag, int *n, int *k, npy_complex64 *ab, int *ldab, s *work) nogil
 * cdef s clantb(char *norm, char *uplo, char *diag, int *n, int *k, c *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clantb(&out, norm, uplo, diag, n, k, <npy_complex64*>ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clantp "F_FUNC(clantpwrp, CLANTPWRP)"(s *out, char *norm, char *uplo, char *diag, int *n, npy_complex64 *ap, s *work) nogil
 * cdef s clantp(char *norm, char *uplo, char *diag, int *n, c *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clantp(&out, norm, uplo, diag, n, <npy_complex64*>ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clantp(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1458
 * cdef s clantp(char *norm, char *uplo, char *diag, int *n, c *ap, s *work) nogil:
 *     cdef s out
 *     _fortran_clantp(&out, norm, uplo, diag, n, <npy_complex64*>ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clantpwrp, CLANTPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1459
 *     cdef s out
 *     _fortran_clantp(&out, norm, uplo, diag, n, <npy_complex64*>ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clantp "F_FUNC(clantpwrp, CLANTPWRP)"(s *out, char *norm, char *uplo, char *diag, int *n, npy_complex64 *ap, s *work) nogil
 * cdef s clantp(char *norm, char *uplo, char *diag, int *n, c *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clantp(&out, norm, uplo, diag, n, <npy_complex64*>ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1464
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clantr "F_FUNC(clantrwrp, CLANTRWRP)"(s *out, char *norm, char *uplo, char *diag, int *m, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clantr(char *norm, char *uplo, char *diag, int *m, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clantr(&out, norm, uplo, diag, m, n, <npy_complex64*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_clantr(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1466
 * cdef s clantr(char *norm, char *uplo, char *diag, int *m, int *n, c *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_clantr(&out, norm, uplo, diag, m, n, <npy_complex64*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(clantrwrp, CLANTRWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1467
 *     cdef s out
 *     _fortran_clantr(&out, norm, uplo, diag, m, n, <npy_complex64*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1464
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clantr "F_FUNC(clantrwrp, CLANTRWRP)"(s *out, char *norm, char *uplo, char *diag, int *m, int *n, npy_complex64 *a, int *lda, s *work) nogil
 * cdef s clantr(char *norm, char *uplo, char *diag, int *m, int *n, c *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_clantr(&out, norm, uplo, diag, m, n, <npy_complex64*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1472
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_disnan "F_FUNC(disnanwrp, DISNANWRP)"(bint *out, d *din) nogil
 * cdef bint disnan(d *din) nogil:             # <<<<<<<<<<<<<<
 *     cdef bint out
 *     _fortran_disnan(&out, din)
 */

static int __pyx_f_5scipy_6linalg_13cython_lapack_disnan(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_din) {
  int __pyx_v_out;
  int __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1474
 * cdef bint disnan(d *din) nogil:
 *     cdef bint out
 *     _fortran_disnan(&out, din)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(disnanwrp, DISNANWRP)((&__pyx_v_out), __pyx_v_din);

  /* "scipy/linalg/cython_lapack.pyx":1475
 *     cdef bint out
 *     _fortran_disnan(&out, din)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1472
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_disnan "F_FUNC(disnanwrp, DISNANWRP)"(bint *out, d *din) nogil
 * cdef bint disnan(d *din) nogil:             # <<<<<<<<<<<<<<
 *     cdef bint out
 *     _fortran_disnan(&out, din)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1480
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlamch "F_FUNC(dlamchwrp, DLAMCHWRP)"(d *out, char *cmach) nogil
 * cdef d dlamch(char *cmach) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlamch(&out, cmach)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlamch(char *__pyx_v_cmach) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1482
 * cdef d dlamch(char *cmach) nogil:
 *     cdef d out
 *     _fortran_dlamch(&out, cmach)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlamchwrp, DLAMCHWRP)((&__pyx_v_out), __pyx_v_cmach);

  /* "scipy/linalg/cython_lapack.pyx":1483
 *     cdef d out
 *     _fortran_dlamch(&out, cmach)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1480
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlamch "F_FUNC(dlamchwrp, DLAMCHWRP)"(d *out, char *cmach) nogil
 * cdef d dlamch(char *cmach) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlamch(&out, cmach)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1488
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaneg "F_FUNC(dlanegwrp, DLANEGWRP)"(int *out, int *n, d *d, d *lld, d *sigma, d *pivmin, int *r) nogil
 * cdef int dlaneg(int *n, d *d, d *lld, d *sigma, d *pivmin, int *r) nogil:             # <<<<<<<<<<<<<<
 *     cdef int out
 *     _fortran_dlaneg(&out, n, d, lld, sigma, pivmin, r)
 */

static int __pyx_f_5scipy_6linalg_13cython_lapack_dlaneg(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lld, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sigma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, int *__pyx_v_r) {
  int __pyx_v_out;
  int __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1490
 * cdef int dlaneg(int *n, d *d, d *lld, d *sigma, d *pivmin, int *r) nogil:
 *     cdef int out
 *     _fortran_dlaneg(&out, n, d, lld, sigma, pivmin, r)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlanegwrp, DLANEGWRP)((&__pyx_v_out), __pyx_v_n, __pyx_v_d, __pyx_v_lld, __pyx_v_sigma, __pyx_v_pivmin, __pyx_v_r);

  /* "scipy/linalg/cython_lapack.pyx":1491
 *     cdef int out
 *     _fortran_dlaneg(&out, n, d, lld, sigma, pivmin, r)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1488
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaneg "F_FUNC(dlanegwrp, DLANEGWRP)"(int *out, int *n, d *d, d *lld, d *sigma, d *pivmin, int *r) nogil
 * cdef int dlaneg(int *n, d *d, d *lld, d *sigma, d *pivmin, int *r) nogil:             # <<<<<<<<<<<<<<
 *     cdef int out
 *     _fortran_dlaneg(&out, n, d, lld, sigma, pivmin, r)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlangb "F_FUNC(dlangbwrp, DLANGBWRP)"(d *out, char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, d *work) nogil
 * cdef d dlangb(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlangb(&out, norm, n, kl, ku, ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlangb(char *__pyx_v_norm, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1498
 * cdef d dlangb(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, d *work) nogil:
 *     cdef d out
 *     _fortran_dlangb(&out, norm, n, kl, ku, ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlangbwrp, DLANGBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1499
 *     cdef d out
 *     _fortran_dlangb(&out, norm, n, kl, ku, ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlangb "F_FUNC(dlangbwrp, DLANGBWRP)"(d *out, char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, d *work) nogil
 * cdef d dlangb(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlangb(&out, norm, n, kl, ku, ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1504
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlange "F_FUNC(dlangewrp, DLANGEWRP)"(d *out, char *norm, int *m, int *n, d *a, int *lda, d *work) nogil
 * cdef d dlange(char *norm, int *m, int *n, d *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlange(&out, norm, m, n, a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlange(char *__pyx_v_norm, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1506
 * cdef d dlange(char *norm, int *m, int *n, d *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_dlange(&out, norm, m, n, a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlangewrp, DLANGEWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1507
 *     cdef d out
 *     _fortran_dlange(&out, norm, m, n, a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1504
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlange "F_FUNC(dlangewrp, DLANGEWRP)"(d *out, char *norm, int *m, int *n, d *a, int *lda, d *work) nogil
 * cdef d dlange(char *norm, int *m, int *n, d *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlange(&out, norm, m, n, a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1512
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlangt "F_FUNC(dlangtwrp, DLANGTWRP)"(d *out, char *norm, int *n, d *dl, d *d_, d *du) nogil
 * cdef d dlangt(char *norm, int *n, d *dl, d *d_, d *du) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlangt(&out, norm, n, dl, d_, du)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlangt(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d_, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1514
 * cdef d dlangt(char *norm, int *n, d *dl, d *d_, d *du) nogil:
 *     cdef d out
 *     _fortran_dlangt(&out, norm, n, dl, d_, du)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlangtwrp, DLANGTWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_dl, __pyx_v_d_, __pyx_v_du);

  /* "scipy/linalg/cython_lapack.pyx":1515
 *     cdef d out
 *     _fortran_dlangt(&out, norm, n, dl, d_, du)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1512
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlangt "F_FUNC(dlangtwrp, DLANGTWRP)"(d *out, char *norm, int *n, d *dl, d *d_, d *du) nogil
 * cdef d dlangt(char *norm, int *n, d *dl, d *d_, d *du) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlangt(&out, norm, n, dl, d_, du)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1520
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlanhs "F_FUNC(dlanhswrp, DLANHSWRP)"(d *out, char *norm, int *n, d *a, int *lda, d *work) nogil
 * cdef d dlanhs(char *norm, int *n, d *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlanhs(&out, norm, n, a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlanhs(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1522
 * cdef d dlanhs(char *norm, int *n, d *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_dlanhs(&out, norm, n, a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlanhswrp, DLANHSWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1523
 *     cdef d out
 *     _fortran_dlanhs(&out, norm, n, a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1520
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlanhs "F_FUNC(dlanhswrp, DLANHSWRP)"(d *out, char *norm, int *n, d *a, int *lda, d *work) nogil
 * cdef d dlanhs(char *norm, int *n, d *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlanhs(&out, norm, n, a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1528
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlansb "F_FUNC(dlansbwrp, DLANSBWRP)"(d *out, char *norm, char *uplo, int *n, int *k, d *ab, int *ldab, d *work) nogil
 * cdef d dlansb(char *norm, char *uplo, int *n, int *k, d *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlansb(&out, norm, uplo, n, k, ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlansb(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1530
 * cdef d dlansb(char *norm, char *uplo, int *n, int *k, d *ab, int *ldab, d *work) nogil:
 *     cdef d out
 *     _fortran_dlansb(&out, norm, uplo, n, k, ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlansbwrp, DLANSBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_k, __pyx_v_ab, __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1531
 *     cdef d out
 *     _fortran_dlansb(&out, norm, uplo, n, k, ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1528
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlansb "F_FUNC(dlansbwrp, DLANSBWRP)"(d *out, char *norm, char *uplo, int *n, int *k, d *ab, int *ldab, d *work) nogil
 * cdef d dlansb(char *norm, char *uplo, int *n, int *k, d *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlansb(&out, norm, uplo, n, k, ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlansp "F_FUNC(dlanspwrp, DLANSPWRP)"(d *out, char *norm, char *uplo, int *n, d *ap, d *work) nogil
 * cdef d dlansp(char *norm, char *uplo, int *n, d *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlansp(&out, norm, uplo, n, ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlansp(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1538
 * cdef d dlansp(char *norm, char *uplo, int *n, d *ap, d *work) nogil:
 *     cdef d out
 *     _fortran_dlansp(&out, norm, uplo, n, ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlanspwrp, DLANSPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1539
 *     cdef d out
 *     _fortran_dlansp(&out, norm, uplo, n, ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlansp "F_FUNC(dlanspwrp, DLANSPWRP)"(d *out, char *norm, char *uplo, int *n, d *ap, d *work) nogil
 * cdef d dlansp(char *norm, char *uplo, int *n, d *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlansp(&out, norm, uplo, n, ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1544
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlanst "F_FUNC(dlanstwrp, DLANSTWRP)"(d *out, char *norm, int *n, d *d_, d *e) nogil
 * cdef d dlanst(char *norm, int *n, d *d_, d *e) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlanst(&out, norm, n, d_, e)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlanst(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d_, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1546
 * cdef d dlanst(char *norm, int *n, d *d_, d *e) nogil:
 *     cdef d out
 *     _fortran_dlanst(&out, norm, n, d_, e)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlanstwrp, DLANSTWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_d_, __pyx_v_e);

  /* "scipy/linalg/cython_lapack.pyx":1547
 *     cdef d out
 *     _fortran_dlanst(&out, norm, n, d_, e)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1544
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlanst "F_FUNC(dlanstwrp, DLANSTWRP)"(d *out, char *norm, int *n, d *d_, d *e) nogil
 * cdef d dlanst(char *norm, int *n, d *d_, d *e) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlanst(&out, norm, n, d_, e)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1552
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlansy "F_FUNC(dlansywrp, DLANSYWRP)"(d *out, char *norm, char *uplo, int *n, d *a, int *lda, d *work) nogil
 * cdef d dlansy(char *norm, char *uplo, int *n, d *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlansy(&out, norm, uplo, n, a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlansy(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1554
 * cdef d dlansy(char *norm, char *uplo, int *n, d *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_dlansy(&out, norm, uplo, n, a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlansywrp, DLANSYWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1555
 *     cdef d out
 *     _fortran_dlansy(&out, norm, uplo, n, a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1552
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlansy "F_FUNC(dlansywrp, DLANSYWRP)"(d *out, char *norm, char *uplo, int *n, d *a, int *lda, d *work) nogil
 * cdef d dlansy(char *norm, char *uplo, int *n, d *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlansy(&out, norm, uplo, n, a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1560
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlantb "F_FUNC(dlantbwrp, DLANTBWRP)"(d *out, char *norm, char *uplo, char *diag, int *n, int *k, d *ab, int *ldab, d *work) nogil
 * cdef d dlantb(char *norm, char *uplo, char *diag, int *n, int *k, d *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlantb(&out, norm, uplo, diag, n, k, ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlantb(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1562
 * cdef d dlantb(char *norm, char *uplo, char *diag, int *n, int *k, d *ab, int *ldab, d *work) nogil:
 *     cdef d out
 *     _fortran_dlantb(&out, norm, uplo, diag, n, k, ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlantbwrp, DLANTBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_k, __pyx_v_ab, __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1563
 *     cdef d out
 *     _fortran_dlantb(&out, norm, uplo, diag, n, k, ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1560
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlantb "F_FUNC(dlantbwrp, DLANTBWRP)"(d *out, char *norm, char *uplo, char *diag, int *n, int *k, d *ab, int *ldab, d *work) nogil
 * cdef d dlantb(char *norm, char *uplo, char *diag, int *n, int *k, d *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlantb(&out, norm, uplo, diag, n, k, ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1568
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlantp "F_FUNC(dlantpwrp, DLANTPWRP)"(d *out, char *norm, char *uplo, char *diag, int *n, d *ap, d *work) nogil
 * cdef d dlantp(char *norm, char *uplo, char *diag, int *n, d *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlantp(&out, norm, uplo, diag, n, ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlantp(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1570
 * cdef d dlantp(char *norm, char *uplo, char *diag, int *n, d *ap, d *work) nogil:
 *     cdef d out
 *     _fortran_dlantp(&out, norm, uplo, diag, n, ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlantpwrp, DLANTPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_ap, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1571
 *     cdef d out
 *     _fortran_dlantp(&out, norm, uplo, diag, n, ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1568
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlantp "F_FUNC(dlantpwrp, DLANTPWRP)"(d *out, char *norm, char *uplo, char *diag, int *n, d *ap, d *work) nogil
 * cdef d dlantp(char *norm, char *uplo, char *diag, int *n, d *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlantp(&out, norm, uplo, diag, n, ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlantr "F_FUNC(dlantrwrp, DLANTRWRP)"(d *out, char *norm, char *uplo, char *diag, int *m, int *n, d *a, int *lda, d *work) nogil
 * cdef d dlantr(char *norm, char *uplo, char *diag, int *m, int *n, d *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlantr(&out, norm, uplo, diag, m, n, a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlantr(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1578
 * cdef d dlantr(char *norm, char *uplo, char *diag, int *m, int *n, d *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_dlantr(&out, norm, uplo, diag, m, n, a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlantrwrp, DLANTRWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1579
 *     cdef d out
 *     _fortran_dlantr(&out, norm, uplo, diag, m, n, a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlantr "F_FUNC(dlantrwrp, DLANTRWRP)"(d *out, char *norm, char *uplo, char *diag, int *m, int *n, d *a, int *lda, d *work) nogil
 * cdef d dlantr(char *norm, char *uplo, char *diag, int *m, int *n, d *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlantr(&out, norm, uplo, diag, m, n, a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1584
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlapy2 "F_FUNC(dlapy2wrp, DLAPY2WRP)"(d *out, d *x, d *y) nogil
 * cdef d dlapy2(d *x, d *y) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlapy2(&out, x, y)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlapy2(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1586
 * cdef d dlapy2(d *x, d *y) nogil:
 *     cdef d out
 *     _fortran_dlapy2(&out, x, y)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlapy2wrp, DLAPY2WRP)((&__pyx_v_out), __pyx_v_x, __pyx_v_y);

  /* "scipy/linalg/cython_lapack.pyx":1587
 *     cdef d out
 *     _fortran_dlapy2(&out, x, y)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1584
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlapy2 "F_FUNC(dlapy2wrp, DLAPY2WRP)"(d *out, d *x, d *y) nogil
 * cdef d dlapy2(d *x, d *y) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlapy2(&out, x, y)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1592
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlapy3 "F_FUNC(dlapy3wrp, DLAPY3WRP)"(d *out, d *x, d *y, d *z) nogil
 * cdef d dlapy3(d *x, d *y, d *z) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlapy3(&out, x, y, z)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dlapy3(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1594
 * cdef d dlapy3(d *x, d *y, d *z) nogil:
 *     cdef d out
 *     _fortran_dlapy3(&out, x, y, z)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dlapy3wrp, DLAPY3WRP)((&__pyx_v_out), __pyx_v_x, __pyx_v_y, __pyx_v_z);

  /* "scipy/linalg/cython_lapack.pyx":1595
 *     cdef d out
 *     _fortran_dlapy3(&out, x, y, z)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1592
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlapy3 "F_FUNC(dlapy3wrp, DLAPY3WRP)"(d *out, d *x, d *y, d *z) nogil
 * cdef d dlapy3(d *x, d *y, d *z) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dlapy3(&out, x, y, z)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1600
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dzsum1 "F_FUNC(dzsum1wrp, DZSUM1WRP)"(d *out, int *n, npy_complex128 *cx, int *incx) nogil
 * cdef d dzsum1(int *n, z *cx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dzsum1(&out, n, <npy_complex128*>cx, incx)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_dzsum1(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_cx, int *__pyx_v_incx) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1602
 * cdef d dzsum1(int *n, z *cx, int *incx) nogil:
 *     cdef d out
 *     _fortran_dzsum1(&out, n, <npy_complex128*>cx, incx)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(dzsum1wrp, DZSUM1WRP)((&__pyx_v_out), __pyx_v_n, ((npy_complex128 *)__pyx_v_cx), __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":1603
 *     cdef d out
 *     _fortran_dzsum1(&out, n, <npy_complex128*>cx, incx)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1600
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dzsum1 "F_FUNC(dzsum1wrp, DZSUM1WRP)"(d *out, int *n, npy_complex128 *cx, int *incx) nogil
 * cdef d dzsum1(int *n, z *cx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_dzsum1(&out, n, <npy_complex128*>cx, incx)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1608
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_icmax1 "F_FUNC(icmax1wrp, ICMAX1WRP)"(int *out, int *n, npy_complex64 *cx, int *incx) nogil
 * cdef int icmax1(int *n, c *cx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     cdef int out
 *     _fortran_icmax1(&out, n, <npy_complex64*>cx, incx)
 */

static int __pyx_f_5scipy_6linalg_13cython_lapack_icmax1(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_cx, int *__pyx_v_incx) {
  int __pyx_v_out;
  int __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1610
 * cdef int icmax1(int *n, c *cx, int *incx) nogil:
 *     cdef int out
 *     _fortran_icmax1(&out, n, <npy_complex64*>cx, incx)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(icmax1wrp, ICMAX1WRP)((&__pyx_v_out), __pyx_v_n, ((npy_complex64 *)__pyx_v_cx), __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":1611
 *     cdef int out
 *     _fortran_icmax1(&out, n, <npy_complex64*>cx, incx)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1608
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_icmax1 "F_FUNC(icmax1wrp, ICMAX1WRP)"(int *out, int *n, npy_complex64 *cx, int *incx) nogil
 * cdef int icmax1(int *n, c *cx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     cdef int out
 *     _fortran_icmax1(&out, n, <npy_complex64*>cx, incx)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ieeeck "F_FUNC(ieeeckwrp, IEEECKWRP)"(int *out, int *ispec, s *zero, s *one) nogil
 * cdef int ieeeck(int *ispec, s *zero, s *one) nogil:             # <<<<<<<<<<<<<<
 *     cdef int out
 *     _fortran_ieeeck(&out, ispec, zero, one)
 */

static int __pyx_f_5scipy_6linalg_13cython_lapack_ieeeck(int *__pyx_v_ispec, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_zero, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_one) {
  int __pyx_v_out;
  int __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1618
 * cdef int ieeeck(int *ispec, s *zero, s *one) nogil:
 *     cdef int out
 *     _fortran_ieeeck(&out, ispec, zero, one)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(ieeeckwrp, IEEECKWRP)((&__pyx_v_out), __pyx_v_ispec, __pyx_v_zero, __pyx_v_one);

  /* "scipy/linalg/cython_lapack.pyx":1619
 *     cdef int out
 *     _fortran_ieeeck(&out, ispec, zero, one)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ieeeck "F_FUNC(ieeeckwrp, IEEECKWRP)"(int *out, int *ispec, s *zero, s *one) nogil
 * cdef int ieeeck(int *ispec, s *zero, s *one) nogil:             # <<<<<<<<<<<<<<
 *     cdef int out
 *     _fortran_ieeeck(&out, ispec, zero, one)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1624
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_izmax1 "F_FUNC(izmax1wrp, IZMAX1WRP)"(int *out, int *n, npy_complex128 *cx, int *incx) nogil
 * cdef int izmax1(int *n, z *cx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     cdef int out
 *     _fortran_izmax1(&out, n, <npy_complex128*>cx, incx)
 */

static int __pyx_f_5scipy_6linalg_13cython_lapack_izmax1(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_cx, int *__pyx_v_incx) {
  int __pyx_v_out;
  int __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1626
 * cdef int izmax1(int *n, z *cx, int *incx) nogil:
 *     cdef int out
 *     _fortran_izmax1(&out, n, <npy_complex128*>cx, incx)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(izmax1wrp, IZMAX1WRP)((&__pyx_v_out), __pyx_v_n, ((npy_complex128 *)__pyx_v_cx), __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":1627
 *     cdef int out
 *     _fortran_izmax1(&out, n, <npy_complex128*>cx, incx)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1624
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_izmax1 "F_FUNC(izmax1wrp, IZMAX1WRP)"(int *out, int *n, npy_complex128 *cx, int *incx) nogil
 * cdef int izmax1(int *n, z *cx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     cdef int out
 *     _fortran_izmax1(&out, n, <npy_complex128*>cx, incx)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1632
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_scsum1 "F_FUNC(scsum1wrp, SCSUM1WRP)"(s *out, int *n, npy_complex64 *cx, int *incx) nogil
 * cdef s scsum1(int *n, c *cx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_scsum1(&out, n, <npy_complex64*>cx, incx)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_scsum1(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_cx, int *__pyx_v_incx) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1634
 * cdef s scsum1(int *n, c *cx, int *incx) nogil:
 *     cdef s out
 *     _fortran_scsum1(&out, n, <npy_complex64*>cx, incx)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(scsum1wrp, SCSUM1WRP)((&__pyx_v_out), __pyx_v_n, ((npy_complex64 *)__pyx_v_cx), __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":1635
 *     cdef s out
 *     _fortran_scsum1(&out, n, <npy_complex64*>cx, incx)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1632
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_scsum1 "F_FUNC(scsum1wrp, SCSUM1WRP)"(s *out, int *n, npy_complex64 *cx, int *incx) nogil
 * cdef s scsum1(int *n, c *cx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_scsum1(&out, n, <npy_complex64*>cx, incx)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1640
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slamch "F_FUNC(slamchwrp, SLAMCHWRP)"(s *out, char *cmach) nogil
 * cdef s slamch(char *cmach) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slamch(&out, cmach)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slamch(char *__pyx_v_cmach) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1642
 * cdef s slamch(char *cmach) nogil:
 *     cdef s out
 *     _fortran_slamch(&out, cmach)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slamchwrp, SLAMCHWRP)((&__pyx_v_out), __pyx_v_cmach);

  /* "scipy/linalg/cython_lapack.pyx":1643
 *     cdef s out
 *     _fortran_slamch(&out, cmach)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1640
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slamch "F_FUNC(slamchwrp, SLAMCHWRP)"(s *out, char *cmach) nogil
 * cdef s slamch(char *cmach) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slamch(&out, cmach)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1648
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slangb "F_FUNC(slangbwrp, SLANGBWRP)"(s *out, char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, s *work) nogil
 * cdef s slangb(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slangb(&out, norm, n, kl, ku, ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slangb(char *__pyx_v_norm, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1650
 * cdef s slangb(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, s *work) nogil:
 *     cdef s out
 *     _fortran_slangb(&out, norm, n, kl, ku, ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slangbwrp, SLANGBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1651
 *     cdef s out
 *     _fortran_slangb(&out, norm, n, kl, ku, ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1648
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slangb "F_FUNC(slangbwrp, SLANGBWRP)"(s *out, char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, s *work) nogil
 * cdef s slangb(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slangb(&out, norm, n, kl, ku, ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slange "F_FUNC(slangewrp, SLANGEWRP)"(s *out, char *norm, int *m, int *n, s *a, int *lda, s *work) nogil
 * cdef s slange(char *norm, int *m, int *n, s *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slange(&out, norm, m, n, a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slange(char *__pyx_v_norm, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1658
 * cdef s slange(char *norm, int *m, int *n, s *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_slange(&out, norm, m, n, a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slangewrp, SLANGEWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1659
 *     cdef s out
 *     _fortran_slange(&out, norm, m, n, a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slange "F_FUNC(slangewrp, SLANGEWRP)"(s *out, char *norm, int *m, int *n, s *a, int *lda, s *work) nogil
 * cdef s slange(char *norm, int *m, int *n, s *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slange(&out, norm, m, n, a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1664
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slangt "F_FUNC(slangtwrp, SLANGTWRP)"(s *out, char *norm, int *n, s *dl, s *d, s *du) nogil
 * cdef s slangt(char *norm, int *n, s *dl, s *d, s *du) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slangt(&out, norm, n, dl, d, du)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slangt(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1666
 * cdef s slangt(char *norm, int *n, s *dl, s *d, s *du) nogil:
 *     cdef s out
 *     _fortran_slangt(&out, norm, n, dl, d, du)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slangtwrp, SLANGTWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_dl, __pyx_v_d, __pyx_v_du);

  /* "scipy/linalg/cython_lapack.pyx":1667
 *     cdef s out
 *     _fortran_slangt(&out, norm, n, dl, d, du)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1664
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slangt "F_FUNC(slangtwrp, SLANGTWRP)"(s *out, char *norm, int *n, s *dl, s *d, s *du) nogil
 * cdef s slangt(char *norm, int *n, s *dl, s *d, s *du) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slangt(&out, norm, n, dl, d, du)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1672
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slanhs "F_FUNC(slanhswrp, SLANHSWRP)"(s *out, char *norm, int *n, s *a, int *lda, s *work) nogil
 * cdef s slanhs(char *norm, int *n, s *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slanhs(&out, norm, n, a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slanhs(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1674
 * cdef s slanhs(char *norm, int *n, s *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_slanhs(&out, norm, n, a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slanhswrp, SLANHSWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1675
 *     cdef s out
 *     _fortran_slanhs(&out, norm, n, a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1672
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slanhs "F_FUNC(slanhswrp, SLANHSWRP)"(s *out, char *norm, int *n, s *a, int *lda, s *work) nogil
 * cdef s slanhs(char *norm, int *n, s *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slanhs(&out, norm, n, a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1680
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slansb "F_FUNC(slansbwrp, SLANSBWRP)"(s *out, char *norm, char *uplo, int *n, int *k, s *ab, int *ldab, s *work) nogil
 * cdef s slansb(char *norm, char *uplo, int *n, int *k, s *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slansb(&out, norm, uplo, n, k, ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slansb(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1682
 * cdef s slansb(char *norm, char *uplo, int *n, int *k, s *ab, int *ldab, s *work) nogil:
 *     cdef s out
 *     _fortran_slansb(&out, norm, uplo, n, k, ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slansbwrp, SLANSBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_k, __pyx_v_ab, __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1683
 *     cdef s out
 *     _fortran_slansb(&out, norm, uplo, n, k, ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1680
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slansb "F_FUNC(slansbwrp, SLANSBWRP)"(s *out, char *norm, char *uplo, int *n, int *k, s *ab, int *ldab, s *work) nogil
 * cdef s slansb(char *norm, char *uplo, int *n, int *k, s *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slansb(&out, norm, uplo, n, k, ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1688
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slansp "F_FUNC(slanspwrp, SLANSPWRP)"(s *out, char *norm, char *uplo, int *n, s *ap, s *work) nogil
 * cdef s slansp(char *norm, char *uplo, int *n, s *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slansp(&out, norm, uplo, n, ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slansp(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1690
 * cdef s slansp(char *norm, char *uplo, int *n, s *ap, s *work) nogil:
 *     cdef s out
 *     _fortran_slansp(&out, norm, uplo, n, ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slanspwrp, SLANSPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1691
 *     cdef s out
 *     _fortran_slansp(&out, norm, uplo, n, ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1688
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slansp "F_FUNC(slanspwrp, SLANSPWRP)"(s *out, char *norm, char *uplo, int *n, s *ap, s *work) nogil
 * cdef s slansp(char *norm, char *uplo, int *n, s *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slansp(&out, norm, uplo, n, ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slanst "F_FUNC(slanstwrp, SLANSTWRP)"(s *out, char *norm, int *n, s *d, s *e) nogil
 * cdef s slanst(char *norm, int *n, s *d, s *e) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slanst(&out, norm, n, d, e)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slanst(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1698
 * cdef s slanst(char *norm, int *n, s *d, s *e) nogil:
 *     cdef s out
 *     _fortran_slanst(&out, norm, n, d, e)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slanstwrp, SLANSTWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_d, __pyx_v_e);

  /* "scipy/linalg/cython_lapack.pyx":1699
 *     cdef s out
 *     _fortran_slanst(&out, norm, n, d, e)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slanst "F_FUNC(slanstwrp, SLANSTWRP)"(s *out, char *norm, int *n, s *d, s *e) nogil
 * cdef s slanst(char *norm, int *n, s *d, s *e) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slanst(&out, norm, n, d, e)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1704
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slansy "F_FUNC(slansywrp, SLANSYWRP)"(s *out, char *norm, char *uplo, int *n, s *a, int *lda, s *work) nogil
 * cdef s slansy(char *norm, char *uplo, int *n, s *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slansy(&out, norm, uplo, n, a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slansy(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1706
 * cdef s slansy(char *norm, char *uplo, int *n, s *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_slansy(&out, norm, uplo, n, a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slansywrp, SLANSYWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1707
 *     cdef s out
 *     _fortran_slansy(&out, norm, uplo, n, a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1704
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slansy "F_FUNC(slansywrp, SLANSYWRP)"(s *out, char *norm, char *uplo, int *n, s *a, int *lda, s *work) nogil
 * cdef s slansy(char *norm, char *uplo, int *n, s *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slansy(&out, norm, uplo, n, a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1712
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slantb "F_FUNC(slantbwrp, SLANTBWRP)"(s *out, char *norm, char *uplo, char *diag, int *n, int *k, s *ab, int *ldab, s *work) nogil
 * cdef s slantb(char *norm, char *uplo, char *diag, int *n, int *k, s *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slantb(&out, norm, uplo, diag, n, k, ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slantb(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1714
 * cdef s slantb(char *norm, char *uplo, char *diag, int *n, int *k, s *ab, int *ldab, s *work) nogil:
 *     cdef s out
 *     _fortran_slantb(&out, norm, uplo, diag, n, k, ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slantbwrp, SLANTBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_k, __pyx_v_ab, __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1715
 *     cdef s out
 *     _fortran_slantb(&out, norm, uplo, diag, n, k, ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1712
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slantb "F_FUNC(slantbwrp, SLANTBWRP)"(s *out, char *norm, char *uplo, char *diag, int *n, int *k, s *ab, int *ldab, s *work) nogil
 * cdef s slantb(char *norm, char *uplo, char *diag, int *n, int *k, s *ab, int *ldab, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slantb(&out, norm, uplo, diag, n, k, ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1720
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slantp "F_FUNC(slantpwrp, SLANTPWRP)"(s *out, char *norm, char *uplo, char *diag, int *n, s *ap, s *work) nogil
 * cdef s slantp(char *norm, char *uplo, char *diag, int *n, s *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slantp(&out, norm, uplo, diag, n, ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slantp(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1722
 * cdef s slantp(char *norm, char *uplo, char *diag, int *n, s *ap, s *work) nogil:
 *     cdef s out
 *     _fortran_slantp(&out, norm, uplo, diag, n, ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slantpwrp, SLANTPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_ap, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1723
 *     cdef s out
 *     _fortran_slantp(&out, norm, uplo, diag, n, ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1720
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slantp "F_FUNC(slantpwrp, SLANTPWRP)"(s *out, char *norm, char *uplo, char *diag, int *n, s *ap, s *work) nogil
 * cdef s slantp(char *norm, char *uplo, char *diag, int *n, s *ap, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slantp(&out, norm, uplo, diag, n, ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1728
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slantr "F_FUNC(slantrwrp, SLANTRWRP)"(s *out, char *norm, char *uplo, char *diag, int *m, int *n, s *a, int *lda, s *work) nogil
 * cdef s slantr(char *norm, char *uplo, char *diag, int *m, int *n, s *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slantr(&out, norm, uplo, diag, m, n, a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slantr(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1730
 * cdef s slantr(char *norm, char *uplo, char *diag, int *m, int *n, s *a, int *lda, s *work) nogil:
 *     cdef s out
 *     _fortran_slantr(&out, norm, uplo, diag, m, n, a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slantrwrp, SLANTRWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1731
 *     cdef s out
 *     _fortran_slantr(&out, norm, uplo, diag, m, n, a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1728
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slantr "F_FUNC(slantrwrp, SLANTRWRP)"(s *out, char *norm, char *uplo, char *diag, int *m, int *n, s *a, int *lda, s *work) nogil
 * cdef s slantr(char *norm, char *uplo, char *diag, int *m, int *n, s *a, int *lda, s *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slantr(&out, norm, uplo, diag, m, n, a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slapy2 "F_FUNC(slapy2wrp, SLAPY2WRP)"(s *out, s *x, s *y) nogil
 * cdef s slapy2(s *x, s *y) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slapy2(&out, x, y)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slapy2(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1738
 * cdef s slapy2(s *x, s *y) nogil:
 *     cdef s out
 *     _fortran_slapy2(&out, x, y)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slapy2wrp, SLAPY2WRP)((&__pyx_v_out), __pyx_v_x, __pyx_v_y);

  /* "scipy/linalg/cython_lapack.pyx":1739
 *     cdef s out
 *     _fortran_slapy2(&out, x, y)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slapy2 "F_FUNC(slapy2wrp, SLAPY2WRP)"(s *out, s *x, s *y) nogil
 * cdef s slapy2(s *x, s *y) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slapy2(&out, x, y)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1744
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slapy3 "F_FUNC(slapy3wrp, SLAPY3WRP)"(s *out, s *x, s *y, s *z) nogil
 * cdef s slapy3(s *x, s *y, s *z) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slapy3(&out, x, y, z)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_f_5scipy_6linalg_13cython_lapack_slapy3(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z) {
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_s __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1746
 * cdef s slapy3(s *x, s *y, s *z) nogil:
 *     cdef s out
 *     _fortran_slapy3(&out, x, y, z)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(slapy3wrp, SLAPY3WRP)((&__pyx_v_out), __pyx_v_x, __pyx_v_y, __pyx_v_z);

  /* "scipy/linalg/cython_lapack.pyx":1747
 *     cdef s out
 *     _fortran_slapy3(&out, x, y, z)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1744
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slapy3 "F_FUNC(slapy3wrp, SLAPY3WRP)"(s *out, s *x, s *y, s *z) nogil
 * cdef s slapy3(s *x, s *y, s *z) nogil:             # <<<<<<<<<<<<<<
 *     cdef s out
 *     _fortran_slapy3(&out, x, y, z)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1752
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zladiv "F_FUNC(zladivwrp, ZLADIVWRP)"(z *out, npy_complex128 *x, npy_complex128 *y) nogil
 * cdef z zladiv(z *x, z *y) nogil:             # <<<<<<<<<<<<<<
 *     cdef z out
 *     _fortran_zladiv(&out, <npy_complex128*>x, <npy_complex128*>y)
 */

static __pyx_t_double_complex __pyx_f_5scipy_6linalg_13cython_lapack_zladiv(__pyx_t_double_complex *__pyx_v_x, __pyx_t_double_complex *__pyx_v_y) {
  __pyx_t_double_complex __pyx_v_out;
  __pyx_t_double_complex __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1754
 * cdef z zladiv(z *x, z *y) nogil:
 *     cdef z out
 *     _fortran_zladiv(&out, <npy_complex128*>x, <npy_complex128*>y)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zladivwrp, ZLADIVWRP)((&__pyx_v_out), ((npy_complex128 *)__pyx_v_x), ((npy_complex128 *)__pyx_v_y));

  /* "scipy/linalg/cython_lapack.pyx":1755
 *     cdef z out
 *     _fortran_zladiv(&out, <npy_complex128*>x, <npy_complex128*>y)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1752
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zladiv "F_FUNC(zladivwrp, ZLADIVWRP)"(z *out, npy_complex128 *x, npy_complex128 *y) nogil
 * cdef z zladiv(z *x, z *y) nogil:             # <<<<<<<<<<<<<<
 *     cdef z out
 *     _fortran_zladiv(&out, <npy_complex128*>x, <npy_complex128*>y)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1760
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlangb "F_FUNC(zlangbwrp, ZLANGBWRP)"(d *out, char *norm, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *work) nogil
 * cdef d zlangb(char *norm, int *n, int *kl, int *ku, z *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlangb(&out, norm, n, kl, ku, <npy_complex128*>ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlangb(char *__pyx_v_norm, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1762
 * cdef d zlangb(char *norm, int *n, int *kl, int *ku, z *ab, int *ldab, d *work) nogil:
 *     cdef d out
 *     _fortran_zlangb(&out, norm, n, kl, ku, <npy_complex128*>ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlangbwrp, ZLANGBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1763
 *     cdef d out
 *     _fortran_zlangb(&out, norm, n, kl, ku, <npy_complex128*>ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1760
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlangb "F_FUNC(zlangbwrp, ZLANGBWRP)"(d *out, char *norm, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *work) nogil
 * cdef d zlangb(char *norm, int *n, int *kl, int *ku, z *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlangb(&out, norm, n, kl, ku, <npy_complex128*>ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1768
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlange "F_FUNC(zlangewrp, ZLANGEWRP)"(d *out, char *norm, int *m, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlange(char *norm, int *m, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlange(&out, norm, m, n, <npy_complex128*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlange(char *__pyx_v_norm, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1770
 * cdef d zlange(char *norm, int *m, int *n, z *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_zlange(&out, norm, m, n, <npy_complex128*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlangewrp, ZLANGEWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1771
 *     cdef d out
 *     _fortran_zlange(&out, norm, m, n, <npy_complex128*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1768
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlange "F_FUNC(zlangewrp, ZLANGEWRP)"(d *out, char *norm, int *m, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlange(char *norm, int *m, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlange(&out, norm, m, n, <npy_complex128*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlangt "F_FUNC(zlangtwrp, ZLANGTWRP)"(d *out, char *norm, int *n, npy_complex128 *dl, npy_complex128 *d_, npy_complex128 *du) nogil
 * cdef d zlangt(char *norm, int *n, z *dl, z *d_, z *du) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlangt(&out, norm, n, <npy_complex128*>dl, <npy_complex128*>d_, <npy_complex128*>du)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlangt(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d_, __pyx_t_double_complex *__pyx_v_du) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1778
 * cdef d zlangt(char *norm, int *n, z *dl, z *d_, z *du) nogil:
 *     cdef d out
 *     _fortran_zlangt(&out, norm, n, <npy_complex128*>dl, <npy_complex128*>d_, <npy_complex128*>du)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlangtwrp, ZLANGTWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d_), ((npy_complex128 *)__pyx_v_du));

  /* "scipy/linalg/cython_lapack.pyx":1779
 *     cdef d out
 *     _fortran_zlangt(&out, norm, n, <npy_complex128*>dl, <npy_complex128*>d_, <npy_complex128*>du)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlangt "F_FUNC(zlangtwrp, ZLANGTWRP)"(d *out, char *norm, int *n, npy_complex128 *dl, npy_complex128 *d_, npy_complex128 *du) nogil
 * cdef d zlangt(char *norm, int *n, z *dl, z *d_, z *du) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlangt(&out, norm, n, <npy_complex128*>dl, <npy_complex128*>d_, <npy_complex128*>du)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1784
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanhb "F_FUNC(zlanhbwrp, ZLANHBWRP)"(d *out, char *norm, char *uplo, int *n, int *k, npy_complex128 *ab, int *ldab, d *work) nogil
 * cdef d zlanhb(char *norm, char *uplo, int *n, int *k, z *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanhb(&out, norm, uplo, n, k, <npy_complex128*>ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlanhb(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1786
 * cdef d zlanhb(char *norm, char *uplo, int *n, int *k, z *ab, int *ldab, d *work) nogil:
 *     cdef d out
 *     _fortran_zlanhb(&out, norm, uplo, n, k, <npy_complex128*>ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlanhbwrp, ZLANHBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1787
 *     cdef d out
 *     _fortran_zlanhb(&out, norm, uplo, n, k, <npy_complex128*>ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1784
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanhb "F_FUNC(zlanhbwrp, ZLANHBWRP)"(d *out, char *norm, char *uplo, int *n, int *k, npy_complex128 *ab, int *ldab, d *work) nogil
 * cdef d zlanhb(char *norm, char *uplo, int *n, int *k, z *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanhb(&out, norm, uplo, n, k, <npy_complex128*>ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1792
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanhe "F_FUNC(zlanhewrp, ZLANHEWRP)"(d *out, char *norm, char *uplo, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlanhe(char *norm, char *uplo, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanhe(&out, norm, uplo, n, <npy_complex128*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlanhe(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1794
 * cdef d zlanhe(char *norm, char *uplo, int *n, z *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_zlanhe(&out, norm, uplo, n, <npy_complex128*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlanhewrp, ZLANHEWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1795
 *     cdef d out
 *     _fortran_zlanhe(&out, norm, uplo, n, <npy_complex128*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1792
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanhe "F_FUNC(zlanhewrp, ZLANHEWRP)"(d *out, char *norm, char *uplo, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlanhe(char *norm, char *uplo, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanhe(&out, norm, uplo, n, <npy_complex128*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1800
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanhp "F_FUNC(zlanhpwrp, ZLANHPWRP)"(d *out, char *norm, char *uplo, int *n, npy_complex128 *ap, d *work) nogil
 * cdef d zlanhp(char *norm, char *uplo, int *n, z *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanhp(&out, norm, uplo, n, <npy_complex128*>ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlanhp(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1802
 * cdef d zlanhp(char *norm, char *uplo, int *n, z *ap, d *work) nogil:
 *     cdef d out
 *     _fortran_zlanhp(&out, norm, uplo, n, <npy_complex128*>ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlanhpwrp, ZLANHPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1803
 *     cdef d out
 *     _fortran_zlanhp(&out, norm, uplo, n, <npy_complex128*>ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1800
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanhp "F_FUNC(zlanhpwrp, ZLANHPWRP)"(d *out, char *norm, char *uplo, int *n, npy_complex128 *ap, d *work) nogil
 * cdef d zlanhp(char *norm, char *uplo, int *n, z *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanhp(&out, norm, uplo, n, <npy_complex128*>ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1808
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanhs "F_FUNC(zlanhswrp, ZLANHSWRP)"(d *out, char *norm, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlanhs(char *norm, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanhs(&out, norm, n, <npy_complex128*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlanhs(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1810
 * cdef d zlanhs(char *norm, int *n, z *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_zlanhs(&out, norm, n, <npy_complex128*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlanhswrp, ZLANHSWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1811
 *     cdef d out
 *     _fortran_zlanhs(&out, norm, n, <npy_complex128*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1808
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanhs "F_FUNC(zlanhswrp, ZLANHSWRP)"(d *out, char *norm, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlanhs(char *norm, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanhs(&out, norm, n, <npy_complex128*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanht "F_FUNC(zlanhtwrp, ZLANHTWRP)"(d *out, char *norm, int *n, d *d_, npy_complex128 *e) nogil
 * cdef d zlanht(char *norm, int *n, d *d_, z *e) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanht(&out, norm, n, d_, <npy_complex128*>e)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlanht(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d_, __pyx_t_double_complex *__pyx_v_e) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1818
 * cdef d zlanht(char *norm, int *n, d *d_, z *e) nogil:
 *     cdef d out
 *     _fortran_zlanht(&out, norm, n, d_, <npy_complex128*>e)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlanhtwrp, ZLANHTWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_n, __pyx_v_d_, ((npy_complex128 *)__pyx_v_e));

  /* "scipy/linalg/cython_lapack.pyx":1819
 *     cdef d out
 *     _fortran_zlanht(&out, norm, n, d_, <npy_complex128*>e)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlanht "F_FUNC(zlanhtwrp, ZLANHTWRP)"(d *out, char *norm, int *n, d *d_, npy_complex128 *e) nogil
 * cdef d zlanht(char *norm, int *n, d *d_, z *e) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlanht(&out, norm, n, d_, <npy_complex128*>e)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1824
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlansb "F_FUNC(zlansbwrp, ZLANSBWRP)"(d *out, char *norm, char *uplo, int *n, int *k, npy_complex128 *ab, int *ldab, d *work) nogil
 * cdef d zlansb(char *norm, char *uplo, int *n, int *k, z *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlansb(&out, norm, uplo, n, k, <npy_complex128*>ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlansb(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1826
 * cdef d zlansb(char *norm, char *uplo, int *n, int *k, z *ab, int *ldab, d *work) nogil:
 *     cdef d out
 *     _fortran_zlansb(&out, norm, uplo, n, k, <npy_complex128*>ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlansbwrp, ZLANSBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1827
 *     cdef d out
 *     _fortran_zlansb(&out, norm, uplo, n, k, <npy_complex128*>ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1824
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlansb "F_FUNC(zlansbwrp, ZLANSBWRP)"(d *out, char *norm, char *uplo, int *n, int *k, npy_complex128 *ab, int *ldab, d *work) nogil
 * cdef d zlansb(char *norm, char *uplo, int *n, int *k, z *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlansb(&out, norm, uplo, n, k, <npy_complex128*>ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1832
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlansp "F_FUNC(zlanspwrp, ZLANSPWRP)"(d *out, char *norm, char *uplo, int *n, npy_complex128 *ap, d *work) nogil
 * cdef d zlansp(char *norm, char *uplo, int *n, z *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlansp(&out, norm, uplo, n, <npy_complex128*>ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlansp(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1834
 * cdef d zlansp(char *norm, char *uplo, int *n, z *ap, d *work) nogil:
 *     cdef d out
 *     _fortran_zlansp(&out, norm, uplo, n, <npy_complex128*>ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlanspwrp, ZLANSPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1835
 *     cdef d out
 *     _fortran_zlansp(&out, norm, uplo, n, <npy_complex128*>ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1832
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlansp "F_FUNC(zlanspwrp, ZLANSPWRP)"(d *out, char *norm, char *uplo, int *n, npy_complex128 *ap, d *work) nogil
 * cdef d zlansp(char *norm, char *uplo, int *n, z *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlansp(&out, norm, uplo, n, <npy_complex128*>ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1840
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlansy "F_FUNC(zlansywrp, ZLANSYWRP)"(d *out, char *norm, char *uplo, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlansy(char *norm, char *uplo, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlansy(&out, norm, uplo, n, <npy_complex128*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlansy(char *__pyx_v_norm, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1842
 * cdef d zlansy(char *norm, char *uplo, int *n, z *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_zlansy(&out, norm, uplo, n, <npy_complex128*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlansywrp, ZLANSYWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1843
 *     cdef d out
 *     _fortran_zlansy(&out, norm, uplo, n, <npy_complex128*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1840
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlansy "F_FUNC(zlansywrp, ZLANSYWRP)"(d *out, char *norm, char *uplo, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlansy(char *norm, char *uplo, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlansy(&out, norm, uplo, n, <npy_complex128*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1848
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlantb "F_FUNC(zlantbwrp, ZLANTBWRP)"(d *out, char *norm, char *uplo, char *diag, int *n, int *k, npy_complex128 *ab, int *ldab, d *work) nogil
 * cdef d zlantb(char *norm, char *uplo, char *diag, int *n, int *k, z *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlantb(&out, norm, uplo, diag, n, k, <npy_complex128*>ab, ldab, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlantb(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1850
 * cdef d zlantb(char *norm, char *uplo, char *diag, int *n, int *k, z *ab, int *ldab, d *work) nogil:
 *     cdef d out
 *     _fortran_zlantb(&out, norm, uplo, diag, n, k, <npy_complex128*>ab, ldab, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlantbwrp, ZLANTBWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1851
 *     cdef d out
 *     _fortran_zlantb(&out, norm, uplo, diag, n, k, <npy_complex128*>ab, ldab, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1848
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlantb "F_FUNC(zlantbwrp, ZLANTBWRP)"(d *out, char *norm, char *uplo, char *diag, int *n, int *k, npy_complex128 *ab, int *ldab, d *work) nogil
 * cdef d zlantb(char *norm, char *uplo, char *diag, int *n, int *k, z *ab, int *ldab, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlantb(&out, norm, uplo, diag, n, k, <npy_complex128*>ab, ldab, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlantp "F_FUNC(zlantpwrp, ZLANTPWRP)"(d *out, char *norm, char *uplo, char *diag, int *n, npy_complex128 *ap, d *work) nogil
 * cdef d zlantp(char *norm, char *uplo, char *diag, int *n, z *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlantp(&out, norm, uplo, diag, n, <npy_complex128*>ap, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlantp(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1858
 * cdef d zlantp(char *norm, char *uplo, char *diag, int *n, z *ap, d *work) nogil:
 *     cdef d out
 *     _fortran_zlantp(&out, norm, uplo, diag, n, <npy_complex128*>ap, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlantpwrp, ZLANTPWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1859
 *     cdef d out
 *     _fortran_zlantp(&out, norm, uplo, diag, n, <npy_complex128*>ap, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlantp "F_FUNC(zlantpwrp, ZLANTPWRP)"(d *out, char *norm, char *uplo, char *diag, int *n, npy_complex128 *ap, d *work) nogil
 * cdef d zlantp(char *norm, char *uplo, char *diag, int *n, z *ap, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlantp(&out, norm, uplo, diag, n, <npy_complex128*>ap, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1864
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlantr "F_FUNC(zlantrwrp, ZLANTRWRP)"(d *out, char *norm, char *uplo, char *diag, int *m, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlantr(char *norm, char *uplo, char *diag, int *m, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlantr(&out, norm, uplo, diag, m, n, <npy_complex128*>a, lda, work)
 */

static __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_f_5scipy_6linalg_13cython_lapack_zlantr(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_v_out;
  __pyx_t_5scipy_6linalg_13cython_lapack_d __pyx_r;

  /* "scipy/linalg/cython_lapack.pyx":1866
 * cdef d zlantr(char *norm, char *uplo, char *diag, int *m, int *n, z *a, int *lda, d *work) nogil:
 *     cdef d out
 *     _fortran_zlantr(&out, norm, uplo, diag, m, n, <npy_complex128*>a, lda, work)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
  F_FUNC(zlantrwrp, ZLANTRWRP)((&__pyx_v_out), __pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":1867
 *     cdef d out
 *     _fortran_zlantr(&out, norm, uplo, diag, m, n, <npy_complex128*>a, lda, work)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":1864
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlantr "F_FUNC(zlantrwrp, ZLANTRWRP)"(d *out, char *norm, char *uplo, char *diag, int *m, int *n, npy_complex128 *a, int *lda, d *work) nogil
 * cdef d zlantr(char *norm, char *uplo, char *diag, int *m, int *n, z *a, int *lda, d *work) nogil:             # <<<<<<<<<<<<<<
 *     cdef d out
 *     _fortran_zlantr(&out, norm, uplo, diag, m, n, <npy_complex128*>a, lda, work)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":1871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cbdsqr "F_FUNC(cbdsqr,CBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, npy_complex64 *vt, int *ldvt, npy_complex64 *u, int *ldu, npy_complex64 *c, int *ldc, s *rwork, int *info) nogil
 * cdef void cbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, c *vt, int *ldvt, c *u, int *ldu, c *c, int *ldc, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cbdsqr(uplo, n, ncvt, nru, ncc, d, e, <npy_complex64*>vt, ldvt, <npy_complex64*>u, ldu, <npy_complex64*>c, ldc, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cbdsqr(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ncvt, int *__pyx_v_nru, int *__pyx_v_ncc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1872
 *     void _fortran_cbdsqr "F_FUNC(cbdsqr,CBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, npy_complex64 *vt, int *ldvt, npy_complex64 *u, int *ldu, npy_complex64 *c, int *ldc, s *rwork, int *info) nogil
 * cdef void cbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, c *vt, int *ldvt, c *u, int *ldu, c *c, int *ldc, s *rwork, int *info) nogil:
 *     _fortran_cbdsqr(uplo, n, ncvt, nru, ncc, d, e, <npy_complex64*>vt, ldvt, <npy_complex64*>u, ldu, <npy_complex64*>c, ldc, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cbdsqr,CBDSQR)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ncvt, __pyx_v_nru, __pyx_v_ncc, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_vt), __pyx_v_ldvt, ((npy_complex64 *)__pyx_v_u), __pyx_v_ldu, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cbdsqr "F_FUNC(cbdsqr,CBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, npy_complex64 *vt, int *ldvt, npy_complex64 *u, int *ldu, npy_complex64 *c, int *ldc, s *rwork, int *info) nogil
 * cdef void cbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, c *vt, int *ldvt, c *u, int *ldu, c *c, int *ldc, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cbdsqr(uplo, n, ncvt, nru, ncc, d, e, <npy_complex64*>vt, ldvt, <npy_complex64*>u, ldu, <npy_complex64*>c, ldc, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbbrd "F_FUNC(cgbbrd,CGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *pt, int *ldpt, npy_complex64 *c, int *ldc, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, c *ab, int *ldab, s *d, s *e, c *q, int *ldq, c *pt, int *ldpt, c *c, int *ldc, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbbrd(vect, m, n, ncc, kl, ku, <npy_complex64*>ab, ldab, d, e, <npy_complex64*>q, ldq, <npy_complex64*>pt, ldpt, <npy_complex64*>c, ldc, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbbrd(char *__pyx_v_vect, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_ncc, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_pt, int *__pyx_v_ldpt, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1877
 *     void _fortran_cgbbrd "F_FUNC(cgbbrd,CGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *pt, int *ldpt, npy_complex64 *c, int *ldc, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, c *ab, int *ldab, s *d, s *e, c *q, int *ldq, c *pt, int *ldpt, c *c, int *ldc, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgbbrd(vect, m, n, ncc, kl, ku, <npy_complex64*>ab, ldab, d, e, <npy_complex64*>q, ldq, <npy_complex64*>pt, ldpt, <npy_complex64*>c, ldc, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbbrd,CGBBRD)(__pyx_v_vect, __pyx_v_m, __pyx_v_n, __pyx_v_ncc, __pyx_v_kl, __pyx_v_ku, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_pt), __pyx_v_ldpt, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbbrd "F_FUNC(cgbbrd,CGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *pt, int *ldpt, npy_complex64 *c, int *ldc, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, c *ab, int *ldab, s *d, s *e, c *q, int *ldq, c *pt, int *ldpt, c *c, int *ldc, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbbrd(vect, m, n, ncc, kl, ku, <npy_complex64*>ab, ldab, d, e, <npy_complex64*>q, ldq, <npy_complex64*>pt, ldpt, <npy_complex64*>c, ldc, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbcon "F_FUNC(cgbcon,CGBCON)"(char *norm, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbcon(char *norm, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbcon(norm, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbcon(char *__pyx_v_norm, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1882
 *     void _fortran_cgbcon "F_FUNC(cgbcon,CGBCON)"(char *norm, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbcon(char *norm, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgbcon(norm, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, anorm, rcond, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbcon,CGBCON)(__pyx_v_norm, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbcon "F_FUNC(cgbcon,CGBCON)"(char *norm, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbcon(char *norm, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbcon(norm, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbequ "F_FUNC(cgbequ,CGBEQU)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void cgbequ(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbequ(m, n, kl, ku, <npy_complex64*>ab, ldab, r, c, rowcnd, colcnd, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbequ(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1887
 *     void _fortran_cgbequ "F_FUNC(cgbequ,CGBEQU)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void cgbequ(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:
 *     _fortran_cgbequ(m, n, kl, ku, <npy_complex64*>ab, ldab, r, c, rowcnd, colcnd, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbequ,CGBEQU)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbequ "F_FUNC(cgbequ,CGBEQU)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void cgbequ(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbequ(m, n, kl, ku, <npy_complex64*>ab, ldab, r, c, rowcnd, colcnd, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbrfs "F_FUNC(cgbrfs,CGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbrfs(trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbrfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_afb, int *__pyx_v_ldafb, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1892
 *     void _fortran_cgbrfs "F_FUNC(cgbrfs,CGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgbrfs(trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbrfs,CGBRFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_afb), __pyx_v_ldafb, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbrfs "F_FUNC(cgbrfs,CGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbrfs(trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbsv "F_FUNC(cgbsv,CGBSV)"(int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgbsv(int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbsv(n, kl, ku, nrhs, <npy_complex64*>ab, ldab, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbsv(int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1897
 *     void _fortran_cgbsv "F_FUNC(cgbsv,CGBSV)"(int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgbsv(int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_cgbsv(n, kl, ku, nrhs, <npy_complex64*>ab, ldab, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbsv,CGBSV)(__pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbsv "F_FUNC(cgbsv,CGBSV)"(int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgbsv(int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbsv(n, kl, ku, nrhs, <npy_complex64*>ab, ldab, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbsvx "F_FUNC(cgbsvx,CGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbsvx(fact, trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, ipiv, equed, r, c, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbsvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_afb, int *__pyx_v_ldafb, int *__pyx_v_ipiv, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1902
 *     void _fortran_cgbsvx "F_FUNC(cgbsvx,CGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgbsvx(fact, trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, ipiv, equed, r, c, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbsvx,CGBSVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_afb), __pyx_v_ldafb, __pyx_v_ipiv, __pyx_v_equed, __pyx_v_r, __pyx_v_c, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbsvx "F_FUNC(cgbsvx,CGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbsvx(fact, trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, ipiv, equed, r, c, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbtf2 "F_FUNC(cgbtf2,CGBTF2)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void cgbtf2(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbtf2(m, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbtf2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1907
 *     void _fortran_cgbtf2 "F_FUNC(cgbtf2,CGBTF2)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void cgbtf2(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, int *info) nogil:
 *     _fortran_cgbtf2(m, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbtf2,CGBTF2)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbtf2 "F_FUNC(cgbtf2,CGBTF2)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void cgbtf2(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbtf2(m, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbtrf "F_FUNC(cgbtrf,CGBTRF)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void cgbtrf(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbtrf(m, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbtrf(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1912
 *     void _fortran_cgbtrf "F_FUNC(cgbtrf,CGBTRF)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void cgbtrf(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, int *info) nogil:
 *     _fortran_cgbtrf(m, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbtrf,CGBTRF)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbtrf "F_FUNC(cgbtrf,CGBTRF)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void cgbtrf(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbtrf(m, n, kl, ku, <npy_complex64*>ab, ldab, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbtrs "F_FUNC(cgbtrs,CGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbtrs(trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgbtrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1917
 *     void _fortran_cgbtrs "F_FUNC(cgbtrs,CGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_cgbtrs(trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgbtrs,CGBTRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgbtrs "F_FUNC(cgbtrs,CGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex64 *ab, int *ldab, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, c *ab, int *ldab, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgbtrs(trans, n, kl, ku, nrhs, <npy_complex64*>ab, ldab, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgebak "F_FUNC(cgebak,CGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, npy_complex64 *v, int *ldv, int *info) nogil
 * cdef void cgebak(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, c *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgebak(job, side, n, ilo, ihi, scale, m, <npy_complex64*>v, ldv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgebak(char *__pyx_v_job, char *__pyx_v_side, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, int *__pyx_v_m, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1922
 *     void _fortran_cgebak "F_FUNC(cgebak,CGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, npy_complex64 *v, int *ldv, int *info) nogil
 * cdef void cgebak(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, c *v, int *ldv, int *info) nogil:
 *     _fortran_cgebak(job, side, n, ilo, ihi, scale, m, <npy_complex64*>v, ldv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgebak,CGEBAK)(__pyx_v_job, __pyx_v_side, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_m, ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgebak "F_FUNC(cgebak,CGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, npy_complex64 *v, int *ldv, int *info) nogil
 * cdef void cgebak(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, c *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgebak(job, side, n, ilo, ihi, scale, m, <npy_complex64*>v, ldv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgebal "F_FUNC(cgebal,CGEBAL)"(char *job, int *n, npy_complex64 *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil
 * cdef void cgebal(char *job, int *n, c *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgebal(job, n, <npy_complex64*>a, lda, ilo, ihi, scale, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgebal(char *__pyx_v_job, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1927
 *     void _fortran_cgebal "F_FUNC(cgebal,CGEBAL)"(char *job, int *n, npy_complex64 *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil
 * cdef void cgebal(char *job, int *n, c *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil:
 *     _fortran_cgebal(job, n, <npy_complex64*>a, lda, ilo, ihi, scale, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgebal,CGEBAL)(__pyx_v_job, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgebal "F_FUNC(cgebal,CGEBAL)"(char *job, int *n, npy_complex64 *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil
 * cdef void cgebal(char *job, int *n, c *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgebal(job, n, <npy_complex64*>a, lda, ilo, ihi, scale, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgebd2 "F_FUNC(cgebd2,CGEBD2)"(int *m, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *work, int *info) nogil
 * cdef void cgebd2(int *m, int *n, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgebd2(m, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgebd2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_tauq, __pyx_t_float_complex *__pyx_v_taup, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1932
 *     void _fortran_cgebd2 "F_FUNC(cgebd2,CGEBD2)"(int *m, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *work, int *info) nogil
 * cdef void cgebd2(int *m, int *n, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *work, int *info) nogil:
 *     _fortran_cgebd2(m, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgebd2,CGEBD2)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_tauq), ((npy_complex64 *)__pyx_v_taup), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgebd2 "F_FUNC(cgebd2,CGEBD2)"(int *m, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *work, int *info) nogil
 * cdef void cgebd2(int *m, int *n, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgebd2(m, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgebrd "F_FUNC(cgebrd,CGEBRD)"(int *m, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgebrd(int *m, int *n, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgebrd(m, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgebrd(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_tauq, __pyx_t_float_complex *__pyx_v_taup, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1937
 *     void _fortran_cgebrd "F_FUNC(cgebrd,CGEBRD)"(int *m, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgebrd(int *m, int *n, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgebrd(m, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgebrd,CGEBRD)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_tauq), ((npy_complex64 *)__pyx_v_taup), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgebrd "F_FUNC(cgebrd,CGEBRD)"(int *m, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgebrd(int *m, int *n, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgebrd(m, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgecon "F_FUNC(cgecon,CGECON)"(char *norm, int *n, npy_complex64 *a, int *lda, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgecon(char *norm, int *n, c *a, int *lda, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgecon(norm, n, <npy_complex64*>a, lda, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgecon(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1942
 *     void _fortran_cgecon "F_FUNC(cgecon,CGECON)"(char *norm, int *n, npy_complex64 *a, int *lda, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgecon(char *norm, int *n, c *a, int *lda, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgecon(norm, n, <npy_complex64*>a, lda, anorm, rcond, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgecon,CGECON)(__pyx_v_norm, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgecon "F_FUNC(cgecon,CGECON)"(char *norm, int *n, npy_complex64 *a, int *lda, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgecon(char *norm, int *n, c *a, int *lda, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgecon(norm, n, <npy_complex64*>a, lda, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeequ "F_FUNC(cgeequ,CGEEQU)"(int *m, int *n, npy_complex64 *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void cgeequ(int *m, int *n, c *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeequ(m, n, <npy_complex64*>a, lda, r, c, rowcnd, colcnd, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeequ(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1947
 *     void _fortran_cgeequ "F_FUNC(cgeequ,CGEEQU)"(int *m, int *n, npy_complex64 *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void cgeequ(int *m, int *n, c *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:
 *     _fortran_cgeequ(m, n, <npy_complex64*>a, lda, r, c, rowcnd, colcnd, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeequ,CGEEQU)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeequ "F_FUNC(cgeequ,CGEEQU)"(int *m, int *n, npy_complex64 *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void cgeequ(int *m, int *n, c *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeequ(m, n, <npy_complex64*>a, lda, r, c, rowcnd, colcnd, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgees "F_FUNC(cgees,CGEES)"(char *jobvs, char *sort, _cselect1 *select, int *n, npy_complex64 *a, int *lda, int *sdim, npy_complex64 *w, npy_complex64 *vs, int *ldvs, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgees(char *jobvs, char *sort, cselect1 *select, int *n, c *a, int *lda, int *sdim, c *w, c *vs, int *ldvs, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgees(jobvs, sort, <_cselect1*>select, n, <npy_complex64*>a, lda, sdim, <npy_complex64*>w, <npy_complex64*>vs, ldvs, <npy_complex64*>work, lwork, rwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgees(char *__pyx_v_jobvs, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_cselect1 *__pyx_v_select, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_sdim, __pyx_t_float_complex *__pyx_v_w, __pyx_t_float_complex *__pyx_v_vs, int *__pyx_v_ldvs, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1952
 *     void _fortran_cgees "F_FUNC(cgees,CGEES)"(char *jobvs, char *sort, _cselect1 *select, int *n, npy_complex64 *a, int *lda, int *sdim, npy_complex64 *w, npy_complex64 *vs, int *ldvs, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgees(char *jobvs, char *sort, cselect1 *select, int *n, c *a, int *lda, int *sdim, c *w, c *vs, int *ldvs, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:
 *     _fortran_cgees(jobvs, sort, <_cselect1*>select, n, <npy_complex64*>a, lda, sdim, <npy_complex64*>w, <npy_complex64*>vs, ldvs, <npy_complex64*>work, lwork, rwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgees,CGEES)(__pyx_v_jobvs, __pyx_v_sort, ((_cselect1 *)__pyx_v_select), __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_sdim, ((npy_complex64 *)__pyx_v_w), ((npy_complex64 *)__pyx_v_vs), __pyx_v_ldvs, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgees "F_FUNC(cgees,CGEES)"(char *jobvs, char *sort, _cselect1 *select, int *n, npy_complex64 *a, int *lda, int *sdim, npy_complex64 *w, npy_complex64 *vs, int *ldvs, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgees(char *jobvs, char *sort, cselect1 *select, int *n, c *a, int *lda, int *sdim, c *w, c *vs, int *ldvs, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgees(jobvs, sort, <_cselect1*>select, n, <npy_complex64*>a, lda, sdim, <npy_complex64*>w, <npy_complex64*>vs, ldvs, <npy_complex64*>work, lwork, rwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeesx "F_FUNC(cgeesx,CGEESX)"(char *jobvs, char *sort, _cselect1 *select, char *sense, int *n, npy_complex64 *a, int *lda, int *sdim, npy_complex64 *w, npy_complex64 *vs, int *ldvs, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgeesx(char *jobvs, char *sort, cselect1 *select, char *sense, int *n, c *a, int *lda, int *sdim, c *w, c *vs, int *ldvs, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeesx(jobvs, sort, <_cselect1*>select, sense, n, <npy_complex64*>a, lda, sdim, <npy_complex64*>w, <npy_complex64*>vs, ldvs, rconde, rcondv, <npy_complex64*>work, lwork, rwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeesx(char *__pyx_v_jobvs, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_cselect1 *__pyx_v_select, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_sdim, __pyx_t_float_complex *__pyx_v_w, __pyx_t_float_complex *__pyx_v_vs, int *__pyx_v_ldvs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcondv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1957
 *     void _fortran_cgeesx "F_FUNC(cgeesx,CGEESX)"(char *jobvs, char *sort, _cselect1 *select, char *sense, int *n, npy_complex64 *a, int *lda, int *sdim, npy_complex64 *w, npy_complex64 *vs, int *ldvs, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgeesx(char *jobvs, char *sort, cselect1 *select, char *sense, int *n, c *a, int *lda, int *sdim, c *w, c *vs, int *ldvs, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:
 *     _fortran_cgeesx(jobvs, sort, <_cselect1*>select, sense, n, <npy_complex64*>a, lda, sdim, <npy_complex64*>w, <npy_complex64*>vs, ldvs, rconde, rcondv, <npy_complex64*>work, lwork, rwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeesx,CGEESX)(__pyx_v_jobvs, __pyx_v_sort, ((_cselect1 *)__pyx_v_select), __pyx_v_sense, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_sdim, ((npy_complex64 *)__pyx_v_w), ((npy_complex64 *)__pyx_v_vs), __pyx_v_ldvs, __pyx_v_rconde, __pyx_v_rcondv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeesx "F_FUNC(cgeesx,CGEESX)"(char *jobvs, char *sort, _cselect1 *select, char *sense, int *n, npy_complex64 *a, int *lda, int *sdim, npy_complex64 *w, npy_complex64 *vs, int *ldvs, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgeesx(char *jobvs, char *sort, cselect1 *select, char *sense, int *n, c *a, int *lda, int *sdim, c *w, c *vs, int *ldvs, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeesx(jobvs, sort, <_cselect1*>select, sense, n, <npy_complex64*>a, lda, sdim, <npy_complex64*>w, <npy_complex64*>vs, ldvs, rconde, rcondv, <npy_complex64*>work, lwork, rwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeev "F_FUNC(cgeev,CGEEV)"(char *jobvl, char *jobvr, int *n, npy_complex64 *a, int *lda, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeev(char *jobvl, char *jobvr, int *n, c *a, int *lda, c *w, c *vl, int *ldvl, c *vr, int *ldvr, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeev(jobvl, jobvr, n, <npy_complex64*>a, lda, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeev(char *__pyx_v_jobvl, char *__pyx_v_jobvr, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_w, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1962
 *     void _fortran_cgeev "F_FUNC(cgeev,CGEEV)"(char *jobvl, char *jobvr, int *n, npy_complex64 *a, int *lda, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeev(char *jobvl, char *jobvr, int *n, c *a, int *lda, c *w, c *vl, int *ldvl, c *vr, int *ldvr, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_cgeev(jobvl, jobvr, n, <npy_complex64*>a, lda, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeev,CGEEV)(__pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_w), ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeev "F_FUNC(cgeev,CGEEV)"(char *jobvl, char *jobvr, int *n, npy_complex64 *a, int *lda, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeev(char *jobvl, char *jobvr, int *n, c *a, int *lda, c *w, c *vl, int *ldvl, c *vr, int *ldvr, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeev(jobvl, jobvr, n, <npy_complex64*>a, lda, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeevx "F_FUNC(cgeevx,CGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, c *a, int *lda, c *w, c *vl, int *ldvl, c *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeevx(balanc, jobvl, jobvr, sense, n, <npy_complex64*>a, lda, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeevx(char *__pyx_v_balanc, char *__pyx_v_jobvl, char *__pyx_v_jobvr, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_w, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abnrm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcondv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1967
 *     void _fortran_cgeevx "F_FUNC(cgeevx,CGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, c *a, int *lda, c *w, c *vl, int *ldvl, c *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_cgeevx(balanc, jobvl, jobvr, sense, n, <npy_complex64*>a, lda, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeevx,CGEEVX)(__pyx_v_balanc, __pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_sense, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_w), ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_abnrm, __pyx_v_rconde, __pyx_v_rcondv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeevx "F_FUNC(cgeevx,CGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, c *a, int *lda, c *w, c *vl, int *ldvl, c *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeevx(balanc, jobvl, jobvr, sense, n, <npy_complex64*>a, lda, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgehd2 "F_FUNC(cgehd2,CGEHD2)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgehd2(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgehd2(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgehd2(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1972
 *     void _fortran_cgehd2 "F_FUNC(cgehd2,CGEHD2)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgehd2(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cgehd2(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgehd2,CGEHD2)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgehd2 "F_FUNC(cgehd2,CGEHD2)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgehd2(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgehd2(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgehrd "F_FUNC(cgehrd,CGEHRD)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgehrd(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgehrd(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgehrd(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1977
 *     void _fortran_cgehrd "F_FUNC(cgehrd,CGEHRD)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgehrd(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgehrd(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgehrd,CGEHRD)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgehrd "F_FUNC(cgehrd,CGEHRD)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgehrd(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgehrd(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelq2 "F_FUNC(cgelq2,CGELQ2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgelq2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelq2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgelq2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1982
 *     void _fortran_cgelq2 "F_FUNC(cgelq2,CGELQ2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgelq2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cgelq2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgelq2,CGELQ2)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelq2 "F_FUNC(cgelq2,CGELQ2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgelq2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelq2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelqf "F_FUNC(cgelqf,CGELQF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgelqf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelqf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgelqf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1987
 *     void _fortran_cgelqf "F_FUNC(cgelqf,CGELQF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgelqf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgelqf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgelqf,CGELQF)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelqf "F_FUNC(cgelqf,CGELQF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgelqf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelqf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgels "F_FUNC(cgels,CGELS)"(char *trans, int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgels(char *trans, int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgels(trans, m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgels(char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1992
 *     void _fortran_cgels "F_FUNC(cgels,CGELS)"(char *trans, int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgels(char *trans, int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgels(trans, m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgels,CGELS)(__pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgels "F_FUNC(cgels,CGELS)"(char *trans, int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgels(char *trans, int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgels(trans, m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":1996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelsd "F_FUNC(cgelsd,CGELSD)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *s, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *info) nogil
 * cdef void cgelsd(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, s *s, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelsd(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, s, rcond, rank, <npy_complex64*>work, lwork, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgelsd(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":1997
 *     void _fortran_cgelsd "F_FUNC(cgelsd,CGELSD)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *s, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *info) nogil
 * cdef void cgelsd(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, s *s, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *iwork, int *info) nogil:
 *     _fortran_cgelsd(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, s, rcond, rank, <npy_complex64*>work, lwork, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgelsd,CGELSD)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_s, __pyx_v_rcond, __pyx_v_rank, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":1996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelsd "F_FUNC(cgelsd,CGELSD)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *s, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *info) nogil
 * cdef void cgelsd(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, s *s, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelsd(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, s, rcond, rank, <npy_complex64*>work, lwork, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelss "F_FUNC(cgelss,CGELSS)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *s, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgelss(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, s *s, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelss(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, s, rcond, rank, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgelss(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2002
 *     void _fortran_cgelss "F_FUNC(cgelss,CGELSS)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *s, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgelss(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, s *s, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_cgelss(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, s, rcond, rank, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgelss,CGELSS)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_s, __pyx_v_rcond, __pyx_v_rank, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelss "F_FUNC(cgelss,CGELSS)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *s, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgelss(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, s *s, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelss(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, s, rcond, rank, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelsy "F_FUNC(cgelsy,CGELSY)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *jpvt, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgelsy(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *jpvt, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelsy(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, jpvt, rcond, rank, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgelsy(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2007
 *     void _fortran_cgelsy "F_FUNC(cgelsy,CGELSY)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *jpvt, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgelsy(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *jpvt, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_cgelsy(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, jpvt, rcond, rank, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgelsy,CGELSY)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_jpvt, __pyx_v_rcond, __pyx_v_rank, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgelsy "F_FUNC(cgelsy,CGELSY)"(int *m, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *jpvt, s *rcond, int *rank, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgelsy(int *m, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *jpvt, s *rcond, int *rank, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgelsy(m, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, jpvt, rcond, rank, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeql2 "F_FUNC(cgeql2,CGEQL2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgeql2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeql2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeql2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2012
 *     void _fortran_cgeql2 "F_FUNC(cgeql2,CGEQL2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgeql2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cgeql2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeql2,CGEQL2)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeql2 "F_FUNC(cgeql2,CGEQL2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgeql2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeql2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeqlf "F_FUNC(cgeqlf,CGEQLF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgeqlf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeqlf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeqlf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2017
 *     void _fortran_cgeqlf "F_FUNC(cgeqlf,CGEQLF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgeqlf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgeqlf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeqlf,CGEQLF)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeqlf "F_FUNC(cgeqlf,CGEQLF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgeqlf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeqlf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeqp3 "F_FUNC(cgeqp3,CGEQP3)"(int *m, int *n, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeqp3(int *m, int *n, c *a, int *lda, int *jpvt, c *tau, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeqp3(m, n, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeqp3(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2022
 *     void _fortran_cgeqp3 "F_FUNC(cgeqp3,CGEQP3)"(int *m, int *n, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeqp3(int *m, int *n, c *a, int *lda, int *jpvt, c *tau, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_cgeqp3(m, n, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeqp3,CGEQP3)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_jpvt, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeqp3 "F_FUNC(cgeqp3,CGEQP3)"(int *m, int *n, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgeqp3(int *m, int *n, c *a, int *lda, int *jpvt, c *tau, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeqp3(m, n, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeqr2 "F_FUNC(cgeqr2,CGEQR2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgeqr2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeqr2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeqr2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2027
 *     void _fortran_cgeqr2 "F_FUNC(cgeqr2,CGEQR2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgeqr2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cgeqr2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeqr2,CGEQR2)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeqr2 "F_FUNC(cgeqr2,CGEQR2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgeqr2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeqr2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeqrf "F_FUNC(cgeqrf,CGEQRF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgeqrf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeqrf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgeqrf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2032
 *     void _fortran_cgeqrf "F_FUNC(cgeqrf,CGEQRF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgeqrf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgeqrf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgeqrf,CGEQRF)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgeqrf "F_FUNC(cgeqrf,CGEQRF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgeqrf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgeqrf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgerfs "F_FUNC(cgerfs,CGERFS)"(char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgerfs(char *trans, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgerfs(trans, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgerfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2037
 *     void _fortran_cgerfs "F_FUNC(cgerfs,CGERFS)"(char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgerfs(char *trans, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgerfs(trans, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgerfs,CGERFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgerfs "F_FUNC(cgerfs,CGERFS)"(char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgerfs(char *trans, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgerfs(trans, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgerq2 "F_FUNC(cgerq2,CGERQ2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgerq2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgerq2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgerq2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2042
 *     void _fortran_cgerq2 "F_FUNC(cgerq2,CGERQ2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgerq2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cgerq2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgerq2,CGERQ2)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgerq2 "F_FUNC(cgerq2,CGERQ2)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cgerq2(int *m, int *n, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgerq2(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgerqf "F_FUNC(cgerqf,CGERQF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgerqf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgerqf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgerqf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2047
 *     void _fortran_cgerqf "F_FUNC(cgerqf,CGERQF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgerqf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgerqf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgerqf,CGERQF)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgerqf "F_FUNC(cgerqf,CGERQF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgerqf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgerqf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesc2 "F_FUNC(cgesc2,CGESC2)"(int *n, npy_complex64 *a, int *lda, npy_complex64 *rhs, int *ipiv, int *jpiv, s *scale) nogil
 * cdef void cgesc2(int *n, c *a, int *lda, c *rhs, int *ipiv, int *jpiv, s *scale) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesc2(n, <npy_complex64*>a, lda, <npy_complex64*>rhs, ipiv, jpiv, scale)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgesc2(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_rhs, int *__pyx_v_ipiv, int *__pyx_v_jpiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale) {

  /* "scipy/linalg/cython_lapack.pyx":2052
 *     void _fortran_cgesc2 "F_FUNC(cgesc2,CGESC2)"(int *n, npy_complex64 *a, int *lda, npy_complex64 *rhs, int *ipiv, int *jpiv, s *scale) nogil
 * cdef void cgesc2(int *n, c *a, int *lda, c *rhs, int *ipiv, int *jpiv, s *scale) nogil:
 *     _fortran_cgesc2(n, <npy_complex64*>a, lda, <npy_complex64*>rhs, ipiv, jpiv, scale)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgesc2,CGESC2)(__pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_rhs), __pyx_v_ipiv, __pyx_v_jpiv, __pyx_v_scale);

  /* "scipy/linalg/cython_lapack.pyx":2051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesc2 "F_FUNC(cgesc2,CGESC2)"(int *n, npy_complex64 *a, int *lda, npy_complex64 *rhs, int *ipiv, int *jpiv, s *scale) nogil
 * cdef void cgesc2(int *n, c *a, int *lda, c *rhs, int *ipiv, int *jpiv, s *scale) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesc2(n, <npy_complex64*>a, lda, <npy_complex64*>rhs, ipiv, jpiv, scale)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesdd "F_FUNC(cgesdd,CGESDD)"(char *jobz, int *m, int *n, npy_complex64 *a, int *lda, s *s, npy_complex64 *u, int *ldu, npy_complex64 *vt, int *ldvt, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *info) nogil
 * cdef void cgesdd(char *jobz, int *m, int *n, c *a, int *lda, s *s, c *u, int *ldu, c *vt, int *ldvt, c *work, int *lwork, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesdd(jobz, m, n, <npy_complex64*>a, lda, s, <npy_complex64*>u, ldu, <npy_complex64*>vt, ldvt, <npy_complex64*>work, lwork, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgesdd(char *__pyx_v_jobz, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_float_complex *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2057
 *     void _fortran_cgesdd "F_FUNC(cgesdd,CGESDD)"(char *jobz, int *m, int *n, npy_complex64 *a, int *lda, s *s, npy_complex64 *u, int *ldu, npy_complex64 *vt, int *ldvt, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *info) nogil
 * cdef void cgesdd(char *jobz, int *m, int *n, c *a, int *lda, s *s, c *u, int *ldu, c *vt, int *ldvt, c *work, int *lwork, s *rwork, int *iwork, int *info) nogil:
 *     _fortran_cgesdd(jobz, m, n, <npy_complex64*>a, lda, s, <npy_complex64*>u, ldu, <npy_complex64*>vt, ldvt, <npy_complex64*>work, lwork, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgesdd,CGESDD)(__pyx_v_jobz, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, ((npy_complex64 *)__pyx_v_u), __pyx_v_ldu, ((npy_complex64 *)__pyx_v_vt), __pyx_v_ldvt, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesdd "F_FUNC(cgesdd,CGESDD)"(char *jobz, int *m, int *n, npy_complex64 *a, int *lda, s *s, npy_complex64 *u, int *ldu, npy_complex64 *vt, int *ldvt, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *info) nogil
 * cdef void cgesdd(char *jobz, int *m, int *n, c *a, int *lda, s *s, c *u, int *ldu, c *vt, int *ldvt, c *work, int *lwork, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesdd(jobz, m, n, <npy_complex64*>a, lda, s, <npy_complex64*>u, ldu, <npy_complex64*>vt, ldvt, <npy_complex64*>work, lwork, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesv "F_FUNC(cgesv,CGESV)"(int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgesv(int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesv(n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgesv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2062
 *     void _fortran_cgesv "F_FUNC(cgesv,CGESV)"(int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgesv(int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_cgesv(n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgesv,CGESV)(__pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesv "F_FUNC(cgesv,CGESV)"(int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgesv(int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesv(n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesvd "F_FUNC(cgesvd,CGESVD)"(char *jobu, char *jobvt, int *m, int *n, npy_complex64 *a, int *lda, s *s, npy_complex64 *u, int *ldu, npy_complex64 *vt, int *ldvt, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgesvd(char *jobu, char *jobvt, int *m, int *n, c *a, int *lda, s *s, c *u, int *ldu, c *vt, int *ldvt, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesvd(jobu, jobvt, m, n, <npy_complex64*>a, lda, s, <npy_complex64*>u, ldu, <npy_complex64*>vt, ldvt, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgesvd(char *__pyx_v_jobu, char *__pyx_v_jobvt, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_float_complex *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2067
 *     void _fortran_cgesvd "F_FUNC(cgesvd,CGESVD)"(char *jobu, char *jobvt, int *m, int *n, npy_complex64 *a, int *lda, s *s, npy_complex64 *u, int *ldu, npy_complex64 *vt, int *ldvt, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgesvd(char *jobu, char *jobvt, int *m, int *n, c *a, int *lda, s *s, c *u, int *ldu, c *vt, int *ldvt, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_cgesvd(jobu, jobvt, m, n, <npy_complex64*>a, lda, s, <npy_complex64*>u, ldu, <npy_complex64*>vt, ldvt, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgesvd,CGESVD)(__pyx_v_jobu, __pyx_v_jobvt, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, ((npy_complex64 *)__pyx_v_u), __pyx_v_ldu, ((npy_complex64 *)__pyx_v_vt), __pyx_v_ldvt, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesvd "F_FUNC(cgesvd,CGESVD)"(char *jobu, char *jobvt, int *m, int *n, npy_complex64 *a, int *lda, s *s, npy_complex64 *u, int *ldu, npy_complex64 *vt, int *ldvt, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cgesvd(char *jobu, char *jobvt, int *m, int *n, c *a, int *lda, s *s, c *u, int *ldu, c *vt, int *ldvt, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesvd(jobu, jobvt, m, n, <npy_complex64*>a, lda, s, <npy_complex64*>u, ldu, <npy_complex64*>vt, ldvt, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesvx "F_FUNC(cgesvx,CGESVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgesvx(char *fact, char *trans, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesvx(fact, trans, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, equed, r, c, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgesvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2072
 *     void _fortran_cgesvx "F_FUNC(cgesvx,CGESVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgesvx(char *fact, char *trans, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgesvx(fact, trans, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, equed, r, c, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgesvx,CGESVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_equed, __pyx_v_r, __pyx_v_c, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgesvx "F_FUNC(cgesvx,CGESVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgesvx(char *fact, char *trans, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgesvx(fact, trans, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, equed, r, c, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetc2 "F_FUNC(cgetc2,CGETC2)"(int *n, npy_complex64 *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void cgetc2(int *n, c *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetc2(n, <npy_complex64*>a, lda, ipiv, jpiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgetc2(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_jpiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2077
 *     void _fortran_cgetc2 "F_FUNC(cgetc2,CGETC2)"(int *n, npy_complex64 *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void cgetc2(int *n, c *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:
 *     _fortran_cgetc2(n, <npy_complex64*>a, lda, ipiv, jpiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgetc2,CGETC2)(__pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_jpiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetc2 "F_FUNC(cgetc2,CGETC2)"(int *n, npy_complex64 *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void cgetc2(int *n, c *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetc2(n, <npy_complex64*>a, lda, ipiv, jpiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetf2 "F_FUNC(cgetf2,CGETF2)"(int *m, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void cgetf2(int *m, int *n, c *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetf2(m, n, <npy_complex64*>a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgetf2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2082
 *     void _fortran_cgetf2 "F_FUNC(cgetf2,CGETF2)"(int *m, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void cgetf2(int *m, int *n, c *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_cgetf2(m, n, <npy_complex64*>a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgetf2,CGETF2)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetf2 "F_FUNC(cgetf2,CGETF2)"(int *m, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void cgetf2(int *m, int *n, c *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetf2(m, n, <npy_complex64*>a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetrf "F_FUNC(cgetrf,CGETRF)"(int *m, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void cgetrf(int *m, int *n, c *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetrf(m, n, <npy_complex64*>a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgetrf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2087
 *     void _fortran_cgetrf "F_FUNC(cgetrf,CGETRF)"(int *m, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void cgetrf(int *m, int *n, c *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_cgetrf(m, n, <npy_complex64*>a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgetrf,CGETRF)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetrf "F_FUNC(cgetrf,CGETRF)"(int *m, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void cgetrf(int *m, int *n, c *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetrf(m, n, <npy_complex64*>a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetri "F_FUNC(cgetri,CGETRI)"(int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgetri(int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetri(n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgetri(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2092
 *     void _fortran_cgetri "F_FUNC(cgetri,CGETRI)"(int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgetri(int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgetri(n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgetri,CGETRI)(__pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetri "F_FUNC(cgetri,CGETRI)"(int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgetri(int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetri(n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetrs "F_FUNC(cgetrs,CGETRS)"(char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgetrs(char *trans, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetrs(trans, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgetrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2097
 *     void _fortran_cgetrs "F_FUNC(cgetrs,CGETRS)"(char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgetrs(char *trans, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_cgetrs(trans, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgetrs,CGETRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgetrs "F_FUNC(cgetrs,CGETRS)"(char *trans, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgetrs(char *trans, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgetrs(trans, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggbak "F_FUNC(cggbak,CGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, npy_complex64 *v, int *ldv, int *info) nogil
 * cdef void cggbak(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, c *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggbak(job, side, n, ilo, ihi, lscale, rscale, m, <npy_complex64*>v, ldv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cggbak(char *__pyx_v_job, char *__pyx_v_side, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rscale, int *__pyx_v_m, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2102
 *     void _fortran_cggbak "F_FUNC(cggbak,CGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, npy_complex64 *v, int *ldv, int *info) nogil
 * cdef void cggbak(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, c *v, int *ldv, int *info) nogil:
 *     _fortran_cggbak(job, side, n, ilo, ihi, lscale, rscale, m, <npy_complex64*>v, ldv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cggbak,CGGBAK)(__pyx_v_job, __pyx_v_side, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_m, ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggbak "F_FUNC(cggbak,CGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, npy_complex64 *v, int *ldv, int *info) nogil
 * cdef void cggbak(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, c *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggbak(job, side, n, ilo, ihi, lscale, rscale, m, <npy_complex64*>v, ldv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggbal "F_FUNC(cggbal,CGGBAL)"(char *job, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil
 * cdef void cggbal(char *job, int *n, c *a, int *lda, c *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggbal(job, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, ilo, ihi, lscale, rscale, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cggbal(char *__pyx_v_job, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2107
 *     void _fortran_cggbal "F_FUNC(cggbal,CGGBAL)"(char *job, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil
 * cdef void cggbal(char *job, int *n, c *a, int *lda, c *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil:
 *     _fortran_cggbal(job, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, ilo, ihi, lscale, rscale, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cggbal,CGGBAL)(__pyx_v_job, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggbal "F_FUNC(cggbal,CGGBAL)"(char *job, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil
 * cdef void cggbal(char *job, int *n, c *a, int *lda, c *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggbal(job, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, ilo, ihi, lscale, rscale, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgges "F_FUNC(cgges,CGGES)"(char *jobvsl, char *jobvsr, char *sort, _cselect2 *selctg, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *sdim, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vsl, int *ldvsl, npy_complex64 *vsr, int *ldvsr, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgges(char *jobvsl, char *jobvsr, char *sort, cselect2 *selctg, int *n, c *a, int *lda, c *b, int *ldb, int *sdim, c *alpha, c *beta, c *vsl, int *ldvsl, c *vsr, int *ldvsr, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgges(jobvsl, jobvsr, sort, <_cselect2*>selctg, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, sdim, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vsl, ldvsl, <npy_complex64*>vsr, ldvsr, <npy_complex64*>work, lwork, rwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgges(char *__pyx_v_jobvsl, char *__pyx_v_jobvsr, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_cselect2 *__pyx_v_selctg, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_sdim, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_vsl, int *__pyx_v_ldvsl, __pyx_t_float_complex *__pyx_v_vsr, int *__pyx_v_ldvsr, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2112
 *     void _fortran_cgges "F_FUNC(cgges,CGGES)"(char *jobvsl, char *jobvsr, char *sort, _cselect2 *selctg, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *sdim, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vsl, int *ldvsl, npy_complex64 *vsr, int *ldvsr, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgges(char *jobvsl, char *jobvsr, char *sort, cselect2 *selctg, int *n, c *a, int *lda, c *b, int *ldb, int *sdim, c *alpha, c *beta, c *vsl, int *ldvsl, c *vsr, int *ldvsr, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:
 *     _fortran_cgges(jobvsl, jobvsr, sort, <_cselect2*>selctg, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, sdim, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vsl, ldvsl, <npy_complex64*>vsr, ldvsr, <npy_complex64*>work, lwork, rwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgges,CGGES)(__pyx_v_jobvsl, __pyx_v_jobvsr, __pyx_v_sort, ((_cselect2 *)__pyx_v_selctg), __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_sdim, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_vsl), __pyx_v_ldvsl, ((npy_complex64 *)__pyx_v_vsr), __pyx_v_ldvsr, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgges "F_FUNC(cgges,CGGES)"(char *jobvsl, char *jobvsr, char *sort, _cselect2 *selctg, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *sdim, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vsl, int *ldvsl, npy_complex64 *vsr, int *ldvsr, npy_complex64 *work, int *lwork, s *rwork, bint *bwork, int *info) nogil
 * cdef void cgges(char *jobvsl, char *jobvsr, char *sort, cselect2 *selctg, int *n, c *a, int *lda, c *b, int *ldb, int *sdim, c *alpha, c *beta, c *vsl, int *ldvsl, c *vsr, int *ldvsr, c *work, int *lwork, s *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgges(jobvsl, jobvsr, sort, <_cselect2*>selctg, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, sdim, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vsl, ldvsl, <npy_complex64*>vsr, ldvsr, <npy_complex64*>work, lwork, rwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggesx "F_FUNC(cggesx,CGGESX)"(char *jobvsl, char *jobvsr, char *sort, _cselect2 *selctg, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *sdim, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vsl, int *ldvsl, npy_complex64 *vsr, int *ldvsr, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void cggesx(char *jobvsl, char *jobvsr, char *sort, cselect2 *selctg, char *sense, int *n, c *a, int *lda, c *b, int *ldb, int *sdim, c *alpha, c *beta, c *vsl, int *ldvsl, c *vsr, int *ldvsr, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggesx(jobvsl, jobvsr, sort, <_cselect2*>selctg, sense, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, sdim, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vsl, ldvsl, <npy_complex64*>vsr, ldvsr, rconde, rcondv, <npy_complex64*>work, lwork, rwork, iwork, liwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cggesx(char *__pyx_v_jobvsl, char *__pyx_v_jobvsr, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_cselect2 *__pyx_v_selctg, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_sdim, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_vsl, int *__pyx_v_ldvsl, __pyx_t_float_complex *__pyx_v_vsr, int *__pyx_v_ldvsr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcondv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2117
 *     void _fortran_cggesx "F_FUNC(cggesx,CGGESX)"(char *jobvsl, char *jobvsr, char *sort, _cselect2 *selctg, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *sdim, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vsl, int *ldvsl, npy_complex64 *vsr, int *ldvsr, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void cggesx(char *jobvsl, char *jobvsr, char *sort, cselect2 *selctg, char *sense, int *n, c *a, int *lda, c *b, int *ldb, int *sdim, c *alpha, c *beta, c *vsl, int *ldvsl, c *vsr, int *ldvsr, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:
 *     _fortran_cggesx(jobvsl, jobvsr, sort, <_cselect2*>selctg, sense, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, sdim, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vsl, ldvsl, <npy_complex64*>vsr, ldvsr, rconde, rcondv, <npy_complex64*>work, lwork, rwork, iwork, liwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cggesx,CGGESX)(__pyx_v_jobvsl, __pyx_v_jobvsr, __pyx_v_sort, ((_cselect2 *)__pyx_v_selctg), __pyx_v_sense, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_sdim, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_vsl), __pyx_v_ldvsl, ((npy_complex64 *)__pyx_v_vsr), __pyx_v_ldvsr, __pyx_v_rconde, __pyx_v_rcondv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggesx "F_FUNC(cggesx,CGGESX)"(char *jobvsl, char *jobvsr, char *sort, _cselect2 *selctg, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *sdim, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vsl, int *ldvsl, npy_complex64 *vsr, int *ldvsr, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void cggesx(char *jobvsl, char *jobvsr, char *sort, cselect2 *selctg, char *sense, int *n, c *a, int *lda, c *b, int *ldb, int *sdim, c *alpha, c *beta, c *vsl, int *ldvsl, c *vsr, int *ldvsr, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggesx(jobvsl, jobvsr, sort, <_cselect2*>selctg, sense, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, sdim, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vsl, ldvsl, <npy_complex64*>vsr, ldvsr, rconde, rcondv, <npy_complex64*>work, lwork, rwork, iwork, liwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggev "F_FUNC(cggev,CGGEV)"(char *jobvl, char *jobvr, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cggev(char *jobvl, char *jobvr, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *vl, int *ldvl, c *vr, int *ldvr, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggev(jobvl, jobvr, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cggev(char *__pyx_v_jobvl, char *__pyx_v_jobvr, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2122
 *     void _fortran_cggev "F_FUNC(cggev,CGGEV)"(char *jobvl, char *jobvr, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cggev(char *jobvl, char *jobvr, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *vl, int *ldvl, c *vr, int *ldvr, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_cggev(jobvl, jobvr, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cggev,CGGEV)(__pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggev "F_FUNC(cggev,CGGEV)"(char *jobvl, char *jobvr, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cggev(char *jobvl, char *jobvr, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *vl, int *ldvl, c *vr, int *ldvr, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggev(jobvl, jobvr, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggevx "F_FUNC(cggevx,CGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void cggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *vl, int *ldvl, c *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *iwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggevx(balanc, jobvl, jobvr, sense, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, <npy_complex64*>work, lwork, rwork, iwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cggevx(char *__pyx_v_balanc, char *__pyx_v_jobvl, char *__pyx_v_jobvr, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abnrm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bbnrm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcondv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2127
 *     void _fortran_cggevx "F_FUNC(cggevx,CGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void cggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *vl, int *ldvl, c *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *iwork, bint *bwork, int *info) nogil:
 *     _fortran_cggevx(balanc, jobvl, jobvr, sense, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, <npy_complex64*>work, lwork, rwork, iwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cggevx,CGGEVX)(__pyx_v_balanc, __pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_sense, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_abnrm, __pyx_v_bbnrm, __pyx_v_rconde, __pyx_v_rcondv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggevx "F_FUNC(cggevx,CGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, npy_complex64 *work, int *lwork, s *rwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void cggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *vl, int *ldvl, c *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, c *work, int *lwork, s *rwork, int *iwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggevx(balanc, jobvl, jobvr, sense, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, <npy_complex64*>work, lwork, rwork, iwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggglm "F_FUNC(cggglm,CGGGLM)"(int *n, int *m, int *p, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *d, npy_complex64 *x, npy_complex64 *y, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggglm(int *n, int *m, int *p, c *a, int *lda, c *b, int *ldb, c *d, c *x, c *y, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggglm(n, m, p, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>d, <npy_complex64*>x, <npy_complex64*>y, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cggglm(int *__pyx_v_n, int *__pyx_v_m, int *__pyx_v_p, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_x, __pyx_t_float_complex *__pyx_v_y, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2132
 *     void _fortran_cggglm "F_FUNC(cggglm,CGGGLM)"(int *n, int *m, int *p, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *d, npy_complex64 *x, npy_complex64 *y, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggglm(int *n, int *m, int *p, c *a, int *lda, c *b, int *ldb, c *d, c *x, c *y, c *work, int *lwork, int *info) nogil:
 *     _fortran_cggglm(n, m, p, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>d, <npy_complex64*>x, <npy_complex64*>y, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cggglm,CGGGLM)(__pyx_v_n, __pyx_v_m, __pyx_v_p, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_x), ((npy_complex64 *)__pyx_v_y), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggglm "F_FUNC(cggglm,CGGGLM)"(int *n, int *m, int *p, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *d, npy_complex64 *x, npy_complex64 *y, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggglm(int *n, int *m, int *p, c *a, int *lda, c *b, int *ldb, c *d, c *x, c *y, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggglm(n, m, p, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>d, <npy_complex64*>x, <npy_complex64*>y, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgghrd "F_FUNC(cgghrd,CGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *info) nogil
 * cdef void cgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgghrd(compq, compz, n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgghrd(char *__pyx_v_compq, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2137
 *     void _fortran_cgghrd "F_FUNC(cgghrd,CGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *info) nogil
 * cdef void cgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *info) nogil:
 *     _fortran_cgghrd(compq, compz, n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgghrd,CGGHRD)(__pyx_v_compq, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgghrd "F_FUNC(cgghrd,CGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *info) nogil
 * cdef void cgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgghrd(compq, compz, n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgglse "F_FUNC(cgglse,CGGLSE)"(int *m, int *n, int *p, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, npy_complex64 *d, npy_complex64 *x, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgglse(int *m, int *n, int *p, c *a, int *lda, c *b, int *ldb, c *c, c *d, c *x, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgglse(m, n, p, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, <npy_complex64*>d, <npy_complex64*>x, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgglse(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_p, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_c, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_x, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2142
 *     void _fortran_cgglse "F_FUNC(cgglse,CGGLSE)"(int *m, int *n, int *p, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, npy_complex64 *d, npy_complex64 *x, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgglse(int *m, int *n, int *p, c *a, int *lda, c *b, int *ldb, c *c, c *d, c *x, c *work, int *lwork, int *info) nogil:
 *     _fortran_cgglse(m, n, p, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, <npy_complex64*>d, <npy_complex64*>x, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgglse,CGGLSE)(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_c), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_x), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgglse "F_FUNC(cgglse,CGGLSE)"(int *m, int *n, int *p, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, npy_complex64 *d, npy_complex64 *x, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cgglse(int *m, int *n, int *p, c *a, int *lda, c *b, int *ldb, c *c, c *d, c *x, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgglse(m, n, p, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, <npy_complex64*>d, <npy_complex64*>x, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggqrf "F_FUNC(cggqrf,CGGQRF)"(int *n, int *m, int *p, npy_complex64 *a, int *lda, npy_complex64 *taua, npy_complex64 *b, int *ldb, npy_complex64 *taub, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggqrf(int *n, int *m, int *p, c *a, int *lda, c *taua, c *b, int *ldb, c *taub, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggqrf(n, m, p, <npy_complex64*>a, lda, <npy_complex64*>taua, <npy_complex64*>b, ldb, <npy_complex64*>taub, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cggqrf(int *__pyx_v_n, int *__pyx_v_m, int *__pyx_v_p, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_taua, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_taub, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2147
 *     void _fortran_cggqrf "F_FUNC(cggqrf,CGGQRF)"(int *n, int *m, int *p, npy_complex64 *a, int *lda, npy_complex64 *taua, npy_complex64 *b, int *ldb, npy_complex64 *taub, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggqrf(int *n, int *m, int *p, c *a, int *lda, c *taua, c *b, int *ldb, c *taub, c *work, int *lwork, int *info) nogil:
 *     _fortran_cggqrf(n, m, p, <npy_complex64*>a, lda, <npy_complex64*>taua, <npy_complex64*>b, ldb, <npy_complex64*>taub, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cggqrf,CGGQRF)(__pyx_v_n, __pyx_v_m, __pyx_v_p, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_taua), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_taub), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggqrf "F_FUNC(cggqrf,CGGQRF)"(int *n, int *m, int *p, npy_complex64 *a, int *lda, npy_complex64 *taua, npy_complex64 *b, int *ldb, npy_complex64 *taub, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggqrf(int *n, int *m, int *p, c *a, int *lda, c *taua, c *b, int *ldb, c *taub, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggqrf(n, m, p, <npy_complex64*>a, lda, <npy_complex64*>taua, <npy_complex64*>b, ldb, <npy_complex64*>taub, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggrqf "F_FUNC(cggrqf,CGGRQF)"(int *m, int *p, int *n, npy_complex64 *a, int *lda, npy_complex64 *taua, npy_complex64 *b, int *ldb, npy_complex64 *taub, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggrqf(int *m, int *p, int *n, c *a, int *lda, c *taua, c *b, int *ldb, c *taub, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggrqf(m, p, n, <npy_complex64*>a, lda, <npy_complex64*>taua, <npy_complex64*>b, ldb, <npy_complex64*>taub, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cggrqf(int *__pyx_v_m, int *__pyx_v_p, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_taua, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_taub, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2152
 *     void _fortran_cggrqf "F_FUNC(cggrqf,CGGRQF)"(int *m, int *p, int *n, npy_complex64 *a, int *lda, npy_complex64 *taua, npy_complex64 *b, int *ldb, npy_complex64 *taub, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggrqf(int *m, int *p, int *n, c *a, int *lda, c *taua, c *b, int *ldb, c *taub, c *work, int *lwork, int *info) nogil:
 *     _fortran_cggrqf(m, p, n, <npy_complex64*>a, lda, <npy_complex64*>taua, <npy_complex64*>b, ldb, <npy_complex64*>taub, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cggrqf,CGGRQF)(__pyx_v_m, __pyx_v_p, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_taua), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_taub), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cggrqf "F_FUNC(cggrqf,CGGRQF)"(int *m, int *p, int *n, npy_complex64 *a, int *lda, npy_complex64 *taua, npy_complex64 *b, int *ldb, npy_complex64 *taub, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cggrqf(int *m, int *p, int *n, c *a, int *lda, c *taua, c *b, int *ldb, c *taub, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cggrqf(m, p, n, <npy_complex64*>a, lda, <npy_complex64*>taua, <npy_complex64*>b, ldb, <npy_complex64*>taub, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtcon "F_FUNC(cgtcon,CGTCON)"(char *norm, int *n, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void cgtcon(char *norm, int *n, c *dl, c *d, c *du, c *du2, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtcon(norm, n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgtcon(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du, __pyx_t_float_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2157
 *     void _fortran_cgtcon "F_FUNC(cgtcon,CGTCON)"(char *norm, int *n, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void cgtcon(char *norm, int *n, c *dl, c *d, c *du, c *du2, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:
 *     _fortran_cgtcon(norm, n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, anorm, rcond, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgtcon,CGTCON)(__pyx_v_norm, __pyx_v_n, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du), ((npy_complex64 *)__pyx_v_du2), __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtcon "F_FUNC(cgtcon,CGTCON)"(char *norm, int *n, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void cgtcon(char *norm, int *n, c *dl, c *d, c *du, c *du2, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtcon(norm, n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtrfs "F_FUNC(cgtrfs,CGTRFS)"(char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *dlf, npy_complex64 *df, npy_complex64 *duf, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgtrfs(char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *dlf, c *df, c *duf, c *du2, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtrfs(trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>dlf, <npy_complex64*>df, <npy_complex64*>duf, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgtrfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du, __pyx_t_float_complex *__pyx_v_dlf, __pyx_t_float_complex *__pyx_v_df, __pyx_t_float_complex *__pyx_v_duf, __pyx_t_float_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2162
 *     void _fortran_cgtrfs "F_FUNC(cgtrfs,CGTRFS)"(char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *dlf, npy_complex64 *df, npy_complex64 *duf, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgtrfs(char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *dlf, c *df, c *duf, c *du2, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgtrfs(trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>dlf, <npy_complex64*>df, <npy_complex64*>duf, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgtrfs,CGTRFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du), ((npy_complex64 *)__pyx_v_dlf), ((npy_complex64 *)__pyx_v_df), ((npy_complex64 *)__pyx_v_duf), ((npy_complex64 *)__pyx_v_du2), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtrfs "F_FUNC(cgtrfs,CGTRFS)"(char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *dlf, npy_complex64 *df, npy_complex64 *duf, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgtrfs(char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *dlf, c *df, c *duf, c *du2, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtrfs(trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>dlf, <npy_complex64*>df, <npy_complex64*>duf, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtsv "F_FUNC(cgtsv,CGTSV)"(int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgtsv(int *n, int *nrhs, c *dl, c *d, c *du, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtsv(n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgtsv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2167
 *     void _fortran_cgtsv "F_FUNC(cgtsv,CGTSV)"(int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgtsv(int *n, int *nrhs, c *dl, c *d, c *du, c *b, int *ldb, int *info) nogil:
 *     _fortran_cgtsv(n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgtsv,CGTSV)(__pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtsv "F_FUNC(cgtsv,CGTSV)"(int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgtsv(int *n, int *nrhs, c *dl, c *d, c *du, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtsv(n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtsvx "F_FUNC(cgtsvx,CGTSVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *dlf, npy_complex64 *df, npy_complex64 *duf, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgtsvx(char *fact, char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *dlf, c *df, c *duf, c *du2, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtsvx(fact, trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>dlf, <npy_complex64*>df, <npy_complex64*>duf, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgtsvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du, __pyx_t_float_complex *__pyx_v_dlf, __pyx_t_float_complex *__pyx_v_df, __pyx_t_float_complex *__pyx_v_duf, __pyx_t_float_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2172
 *     void _fortran_cgtsvx "F_FUNC(cgtsvx,CGTSVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *dlf, npy_complex64 *df, npy_complex64 *duf, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgtsvx(char *fact, char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *dlf, c *df, c *duf, c *du2, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cgtsvx(fact, trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>dlf, <npy_complex64*>df, <npy_complex64*>duf, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgtsvx,CGTSVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du), ((npy_complex64 *)__pyx_v_dlf), ((npy_complex64 *)__pyx_v_df), ((npy_complex64 *)__pyx_v_duf), ((npy_complex64 *)__pyx_v_du2), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtsvx "F_FUNC(cgtsvx,CGTSVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *dlf, npy_complex64 *df, npy_complex64 *duf, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cgtsvx(char *fact, char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *dlf, c *df, c *duf, c *du2, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtsvx(fact, trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>dlf, <npy_complex64*>df, <npy_complex64*>duf, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgttrf "F_FUNC(cgttrf,CGTTRF)"(int *n, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, int *info) nogil
 * cdef void cgttrf(int *n, c *dl, c *d, c *du, c *du2, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgttrf(n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgttrf(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du, __pyx_t_float_complex *__pyx_v_du2, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2177
 *     void _fortran_cgttrf "F_FUNC(cgttrf,CGTTRF)"(int *n, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, int *info) nogil
 * cdef void cgttrf(int *n, c *dl, c *d, c *du, c *du2, int *ipiv, int *info) nogil:
 *     _fortran_cgttrf(n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgttrf,CGTTRF)(__pyx_v_n, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du), ((npy_complex64 *)__pyx_v_du2), __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgttrf "F_FUNC(cgttrf,CGTTRF)"(int *n, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, int *info) nogil
 * cdef void cgttrf(int *n, c *dl, c *d, c *du, c *du2, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgttrf(n, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgttrs "F_FUNC(cgttrs,CGTTRS)"(char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgttrs(char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *du2, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgttrs(trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgttrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du, __pyx_t_float_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2182
 *     void _fortran_cgttrs "F_FUNC(cgttrs,CGTTRS)"(char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgttrs(char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *du2, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_cgttrs(trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgttrs,CGTTRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du), ((npy_complex64 *)__pyx_v_du2), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgttrs "F_FUNC(cgttrs,CGTTRS)"(char *trans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cgttrs(char *trans, int *n, int *nrhs, c *dl, c *d, c *du, c *du2, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgttrs(trans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtts2 "F_FUNC(cgtts2,CGTTS2)"(int *itrans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb) nogil
 * cdef void cgtts2(int *itrans, int *n, int *nrhs, c *dl, c *d, c *du, c *du2, int *ipiv, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtts2(itrans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cgtts2(int *__pyx_v_itrans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du, __pyx_t_float_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":2187
 *     void _fortran_cgtts2 "F_FUNC(cgtts2,CGTTS2)"(int *itrans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb) nogil
 * cdef void cgtts2(int *itrans, int *n, int *nrhs, c *dl, c *d, c *du, c *du2, int *ipiv, c *b, int *ldb) nogil:
 *     _fortran_cgtts2(itrans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cgtts2,CGTTS2)(__pyx_v_itrans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du), ((npy_complex64 *)__pyx_v_du2), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":2186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cgtts2 "F_FUNC(cgtts2,CGTTS2)"(int *itrans, int *n, int *nrhs, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *du2, int *ipiv, npy_complex64 *b, int *ldb) nogil
 * cdef void cgtts2(int *itrans, int *n, int *nrhs, c *dl, c *d, c *du, c *du2, int *ipiv, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cgtts2(itrans, n, nrhs, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>du2, ipiv, <npy_complex64*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbev "F_FUNC(chbev,CHBEV)"(char *jobz, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbev(char *jobz, char *uplo, int *n, int *kd, c *ab, int *ldab, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbev(jobz, uplo, n, kd, <npy_complex64*>ab, ldab, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chbev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2192
 *     void _fortran_chbev "F_FUNC(chbev,CHBEV)"(char *jobz, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbev(char *jobz, char *uplo, int *n, int *kd, c *ab, int *ldab, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:
 *     _fortran_chbev(jobz, uplo, n, kd, <npy_complex64*>ab, ldab, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chbev,CHBEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbev "F_FUNC(chbev,CHBEV)"(char *jobz, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbev(char *jobz, char *uplo, int *n, int *kd, c *ab, int *ldab, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbev(jobz, uplo, n, kd, <npy_complex64*>ab, ldab, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbevd "F_FUNC(chbevd,CHBEVD)"(char *jobz, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chbevd(char *jobz, char *uplo, int *n, int *kd, c *ab, int *ldab, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbevd(jobz, uplo, n, kd, <npy_complex64*>ab, ldab, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chbevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2197
 *     void _fortran_chbevd "F_FUNC(chbevd,CHBEVD)"(char *jobz, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chbevd(char *jobz, char *uplo, int *n, int *kd, c *ab, int *ldab, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_chbevd(jobz, uplo, n, kd, <npy_complex64*>ab, ldab, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chbevd,CHBEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbevd "F_FUNC(chbevd,CHBEVD)"(char *jobz, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chbevd(char *jobz, char *uplo, int *n, int *kd, c *ab, int *ldab, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbevd(jobz, uplo, n, kd, <npy_complex64*>ab, ldab, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbevx "F_FUNC(chbevx,CHBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, npy_complex64 *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chbevx(char *jobz, char *range, char *uplo, int *n, int *kd, c *ab, int *ldab, c *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbevx(jobz, range, uplo, n, kd, <npy_complex64*>ab, ldab, <npy_complex64*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chbevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2202
 *     void _fortran_chbevx "F_FUNC(chbevx,CHBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, npy_complex64 *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chbevx(char *jobz, char *range, char *uplo, int *n, int *kd, c *ab, int *ldab, c *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_chbevx(jobz, range, uplo, n, kd, <npy_complex64*>ab, ldab, <npy_complex64*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chbevx,CHBEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbevx "F_FUNC(chbevx,CHBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, npy_complex64 *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chbevx(char *jobz, char *range, char *uplo, int *n, int *kd, c *ab, int *ldab, c *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbevx(jobz, range, uplo, n, kd, <npy_complex64*>ab, ldab, <npy_complex64*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbgst "F_FUNC(chbgst,CHBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, npy_complex64 *x, int *ldx, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbgst(char *vect, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, c *x, int *ldx, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbgst(vect, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, <npy_complex64*>x, ldx, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chbgst(char *__pyx_v_vect, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2207
 *     void _fortran_chbgst "F_FUNC(chbgst,CHBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, npy_complex64 *x, int *ldx, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbgst(char *vect, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, c *x, int *ldx, c *work, s *rwork, int *info) nogil:
 *     _fortran_chbgst(vect, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, <npy_complex64*>x, ldx, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chbgst,CHBGST)(__pyx_v_vect, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_bb), __pyx_v_ldbb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbgst "F_FUNC(chbgst,CHBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, npy_complex64 *x, int *ldx, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbgst(char *vect, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, c *x, int *ldx, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbgst(vect, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, <npy_complex64*>x, ldx, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbgv "F_FUNC(chbgv,CHBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbgv(jobz, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chbgv(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2212
 *     void _fortran_chbgv "F_FUNC(chbgv,CHBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:
 *     _fortran_chbgv(jobz, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chbgv,CHBGV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_bb), __pyx_v_ldbb, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbgv "F_FUNC(chbgv,CHBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbgv(jobz, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbgvd "F_FUNC(chbgvd,CHBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbgvd(jobz, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chbgvd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2217
 *     void _fortran_chbgvd "F_FUNC(chbgvd,CHBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_chbgvd(jobz, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chbgvd,CHBGVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_bb), __pyx_v_ldbb, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbgvd "F_FUNC(chbgvd,CHBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbgvd(jobz, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbgvx "F_FUNC(chbgvx,CHBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, npy_complex64 *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, c *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbgvx(jobz, range, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, <npy_complex64*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chbgvx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2222
 *     void _fortran_chbgvx "F_FUNC(chbgvx,CHBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, npy_complex64 *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, c *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_chbgvx(jobz, range, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, <npy_complex64*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chbgvx,CHBGVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_bb), __pyx_v_ldbb, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbgvx "F_FUNC(chbgvx,CHBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, npy_complex64 *ab, int *ldab, npy_complex64 *bb, int *ldbb, npy_complex64 *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, c *ab, int *ldab, c *bb, int *ldbb, c *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbgvx(jobz, range, uplo, n, ka, kb, <npy_complex64*>ab, ldab, <npy_complex64*>bb, ldbb, <npy_complex64*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbtrd "F_FUNC(chbtrd,CHBTRD)"(char *vect, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *work, int *info) nogil
 * cdef void chbtrd(char *vect, char *uplo, int *n, int *kd, c *ab, int *ldab, s *d, s *e, c *q, int *ldq, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbtrd(vect, uplo, n, kd, <npy_complex64*>ab, ldab, d, e, <npy_complex64*>q, ldq, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chbtrd(char *__pyx_v_vect, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2227
 *     void _fortran_chbtrd "F_FUNC(chbtrd,CHBTRD)"(char *vect, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *work, int *info) nogil
 * cdef void chbtrd(char *vect, char *uplo, int *n, int *kd, c *ab, int *ldab, s *d, s *e, c *q, int *ldq, c *work, int *info) nogil:
 *     _fortran_chbtrd(vect, uplo, n, kd, <npy_complex64*>ab, ldab, d, e, <npy_complex64*>q, ldq, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chbtrd,CHBTRD)(__pyx_v_vect, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chbtrd "F_FUNC(chbtrd,CHBTRD)"(char *vect, char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *work, int *info) nogil
 * cdef void chbtrd(char *vect, char *uplo, int *n, int *kd, c *ab, int *ldab, s *d, s *e, c *q, int *ldq, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chbtrd(vect, uplo, n, kd, <npy_complex64*>ab, ldab, d, e, <npy_complex64*>q, ldq, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_checon "F_FUNC(checon,CHECON)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void checon(char *uplo, int *n, c *a, int *lda, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_checon(uplo, n, <npy_complex64*>a, lda, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_checon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2232
 *     void _fortran_checon "F_FUNC(checon,CHECON)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void checon(char *uplo, int *n, c *a, int *lda, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:
 *     _fortran_checon(uplo, n, <npy_complex64*>a, lda, ipiv, anorm, rcond, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(checon,CHECON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_checon "F_FUNC(checon,CHECON)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void checon(char *uplo, int *n, c *a, int *lda, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_checon(uplo, n, <npy_complex64*>a, lda, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cheev "F_FUNC(cheev,CHEEV)"(char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, s *w, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cheev(char *jobz, char *uplo, int *n, c *a, int *lda, s *w, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cheev(jobz, uplo, n, <npy_complex64*>a, lda, w, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cheev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2237
 *     void _fortran_cheev "F_FUNC(cheev,CHEEV)"(char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, s *w, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cheev(char *jobz, char *uplo, int *n, c *a, int *lda, s *w, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_cheev(jobz, uplo, n, <npy_complex64*>a, lda, w, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cheev,CHEEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_w, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cheev "F_FUNC(cheev,CHEEV)"(char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, s *w, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void cheev(char *jobz, char *uplo, int *n, c *a, int *lda, s *w, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cheev(jobz, uplo, n, <npy_complex64*>a, lda, w, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cheevd "F_FUNC(cheevd,CHEEVD)"(char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, s *w, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cheevd(char *jobz, char *uplo, int *n, c *a, int *lda, s *w, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cheevd(jobz, uplo, n, <npy_complex64*>a, lda, w, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cheevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2242
 *     void _fortran_cheevd "F_FUNC(cheevd,CHEEVD)"(char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, s *w, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cheevd(char *jobz, char *uplo, int *n, c *a, int *lda, s *w, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_cheevd(jobz, uplo, n, <npy_complex64*>a, lda, w, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cheevd,CHEEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_w, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cheevd "F_FUNC(cheevd,CHEEVD)"(char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, s *w, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cheevd(char *jobz, char *uplo, int *n, c *a, int *lda, s *w, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cheevd(jobz, uplo, n, <npy_complex64*>a, lda, w, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cheevr "F_FUNC(cheevr,CHEEVR)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, int *isuppz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cheevr(char *jobz, char *range, char *uplo, int *n, c *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, int *isuppz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cheevr(jobz, range, uplo, n, <npy_complex64*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, isuppz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cheevr(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2247
 *     void _fortran_cheevr "F_FUNC(cheevr,CHEEVR)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, int *isuppz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cheevr(char *jobz, char *range, char *uplo, int *n, c *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, int *isuppz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_cheevr(jobz, range, uplo, n, <npy_complex64*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, isuppz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cheevr,CHEEVR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_isuppz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cheevr "F_FUNC(cheevr,CHEEVR)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, int *isuppz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cheevr(char *jobz, char *range, char *uplo, int *n, c *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, int *isuppz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cheevr(jobz, range, uplo, n, <npy_complex64*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, isuppz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cheevx "F_FUNC(cheevx,CHEEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void cheevx(char *jobz, char *range, char *uplo, int *n, c *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cheevx(jobz, range, uplo, n, <npy_complex64*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cheevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2252
 *     void _fortran_cheevx "F_FUNC(cheevx,CHEEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void cheevx(char *jobz, char *range, char *uplo, int *n, c *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_cheevx(jobz, range, uplo, n, <npy_complex64*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cheevx,CHEEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cheevx "F_FUNC(cheevx,CHEEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void cheevx(char *jobz, char *range, char *uplo, int *n, c *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cheevx(jobz, range, uplo, n, <npy_complex64*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegs2 "F_FUNC(chegs2,CHEGS2)"(int *itype, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chegs2(int *itype, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegs2(itype, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chegs2(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2257
 *     void _fortran_chegs2 "F_FUNC(chegs2,CHEGS2)"(int *itype, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chegs2(int *itype, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, int *info) nogil:
 *     _fortran_chegs2(itype, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chegs2,CHEGS2)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegs2 "F_FUNC(chegs2,CHEGS2)"(int *itype, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chegs2(int *itype, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegs2(itype, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegst "F_FUNC(chegst,CHEGST)"(int *itype, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chegst(int *itype, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegst(itype, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chegst(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2262
 *     void _fortran_chegst "F_FUNC(chegst,CHEGST)"(int *itype, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chegst(int *itype, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, int *info) nogil:
 *     _fortran_chegst(itype, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chegst,CHEGST)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegst "F_FUNC(chegst,CHEGST)"(int *itype, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chegst(int *itype, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegst(itype, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegv "F_FUNC(chegv,CHEGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *w, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chegv(int *itype, char *jobz, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *w, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegv(itype, jobz, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, w, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chegv(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2267
 *     void _fortran_chegv "F_FUNC(chegv,CHEGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *w, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chegv(int *itype, char *jobz, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *w, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_chegv(itype, jobz, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, w, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chegv,CHEGV)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_w, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegv "F_FUNC(chegv,CHEGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *w, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chegv(int *itype, char *jobz, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *w, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegv(itype, jobz, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, w, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegvd "F_FUNC(chegvd,CHEGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *w, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chegvd(int *itype, char *jobz, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *w, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegvd(itype, jobz, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, w, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chegvd(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2272
 *     void _fortran_chegvd "F_FUNC(chegvd,CHEGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *w, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chegvd(int *itype, char *jobz, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *w, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_chegvd(itype, jobz, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, w, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chegvd,CHEGVD)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_w, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegvd "F_FUNC(chegvd,CHEGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *w, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chegvd(int *itype, char *jobz, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *w, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegvd(itype, jobz, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, w, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegvx "F_FUNC(chegvx,CHEGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chegvx(int *itype, char *jobz, char *range, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegvx(itype, jobz, range, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chegvx(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2277
 *     void _fortran_chegvx "F_FUNC(chegvx,CHEGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chegvx(int *itype, char *jobz, char *range, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_chegvx(itype, jobz, range, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chegvx,CHEGVX)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chegvx "F_FUNC(chegvx,CHEGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chegvx(int *itype, char *jobz, char *range, char *uplo, int *n, c *a, int *lda, c *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chegvx(itype, jobz, range, uplo, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cherfs "F_FUNC(cherfs,CHERFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cherfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cherfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cherfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2282
 *     void _fortran_cherfs "F_FUNC(cherfs,CHERFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cherfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cherfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cherfs,CHERFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cherfs "F_FUNC(cherfs,CHERFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cherfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cherfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chesv "F_FUNC(chesv,CHESV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chesv(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chesv(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chesv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2287
 *     void _fortran_chesv "F_FUNC(chesv,CHESV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chesv(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, c *work, int *lwork, int *info) nogil:
 *     _fortran_chesv(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chesv,CHESV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chesv "F_FUNC(chesv,CHESV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chesv(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chesv(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chesvx "F_FUNC(chesvx,CHESVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chesvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chesvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chesvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2292
 *     void _fortran_chesvx "F_FUNC(chesvx,CHESVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chesvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_chesvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chesvx,CHESVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chesvx "F_FUNC(chesvx,CHESVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chesvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chesvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetd2 "F_FUNC(chetd2,CHETD2)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tau, int *info) nogil
 * cdef void chetd2(char *uplo, int *n, c *a, int *lda, s *d, s *e, c *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetd2(uplo, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chetd2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2297
 *     void _fortran_chetd2 "F_FUNC(chetd2,CHETD2)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tau, int *info) nogil
 * cdef void chetd2(char *uplo, int *n, c *a, int *lda, s *d, s *e, c *tau, int *info) nogil:
 *     _fortran_chetd2(uplo, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chetd2,CHETD2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_tau), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetd2 "F_FUNC(chetd2,CHETD2)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tau, int *info) nogil
 * cdef void chetd2(char *uplo, int *n, c *a, int *lda, s *d, s *e, c *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetd2(uplo, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetf2 "F_FUNC(chetf2,CHETF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void chetf2(char *uplo, int *n, c *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetf2(uplo, n, <npy_complex64*>a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chetf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2302
 *     void _fortran_chetf2 "F_FUNC(chetf2,CHETF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void chetf2(char *uplo, int *n, c *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_chetf2(uplo, n, <npy_complex64*>a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chetf2,CHETF2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetf2 "F_FUNC(chetf2,CHETF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void chetf2(char *uplo, int *n, c *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetf2(uplo, n, <npy_complex64*>a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetrd "F_FUNC(chetrd,CHETRD)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chetrd(char *uplo, int *n, c *a, int *lda, s *d, s *e, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetrd(uplo, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chetrd(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2307
 *     void _fortran_chetrd "F_FUNC(chetrd,CHETRD)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chetrd(char *uplo, int *n, c *a, int *lda, s *d, s *e, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_chetrd(uplo, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chetrd,CHETRD)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetrd "F_FUNC(chetrd,CHETRD)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chetrd(char *uplo, int *n, c *a, int *lda, s *d, s *e, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetrd(uplo, n, <npy_complex64*>a, lda, d, e, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetrf "F_FUNC(chetrf,CHETRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chetrf(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetrf(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chetrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2312
 *     void _fortran_chetrf "F_FUNC(chetrf,CHETRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chetrf(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:
 *     _fortran_chetrf(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chetrf,CHETRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetrf "F_FUNC(chetrf,CHETRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chetrf(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetrf(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetri "F_FUNC(chetri,CHETRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void chetri(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetri(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chetri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2317
 *     void _fortran_chetri "F_FUNC(chetri,CHETRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void chetri(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *info) nogil:
 *     _fortran_chetri(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chetri,CHETRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetri "F_FUNC(chetri,CHETRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void chetri(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetri(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetrs "F_FUNC(chetrs,CHETRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chetrs(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetrs(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chetrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2322
 *     void _fortran_chetrs "F_FUNC(chetrs,CHETRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chetrs(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_chetrs(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chetrs,CHETRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chetrs "F_FUNC(chetrs,CHETRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chetrs(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chetrs(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chgeqz "F_FUNC(chgeqz,CHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *t, int *ldt, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *t, int *ldt, c *alpha, c *beta, c *q, int *ldq, c *z, int *ldz, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chgeqz(job, compq, compz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>t, ldt, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chgeqz(char *__pyx_v_job, char *__pyx_v_compq, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2327
 *     void _fortran_chgeqz "F_FUNC(chgeqz,CHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *t, int *ldt, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *t, int *ldt, c *alpha, c *beta, c *q, int *ldq, c *z, int *ldz, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_chgeqz(job, compq, compz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>t, ldt, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chgeqz,CHGEQZ)(__pyx_v_job, __pyx_v_compq, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chgeqz "F_FUNC(chgeqz,CHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *t, int *ldt, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void chgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *t, int *ldt, c *alpha, c *beta, c *q, int *ldq, c *z, int *ldz, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chgeqz(job, compq, compz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>t, ldt, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpcon "F_FUNC(chpcon,CHPCON)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void chpcon(char *uplo, int *n, c *ap, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpcon(uplo, n, <npy_complex64*>ap, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2332
 *     void _fortran_chpcon "F_FUNC(chpcon,CHPCON)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void chpcon(char *uplo, int *n, c *ap, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:
 *     _fortran_chpcon(uplo, n, <npy_complex64*>ap, ipiv, anorm, rcond, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpcon,CHPCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpcon "F_FUNC(chpcon,CHPCON)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void chpcon(char *uplo, int *n, c *ap, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpcon(uplo, n, <npy_complex64*>ap, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpev "F_FUNC(chpev,CHPEV)"(char *jobz, char *uplo, int *n, npy_complex64 *ap, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpev(char *jobz, char *uplo, int *n, c *ap, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpev(jobz, uplo, n, <npy_complex64*>ap, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2337
 *     void _fortran_chpev "F_FUNC(chpev,CHPEV)"(char *jobz, char *uplo, int *n, npy_complex64 *ap, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpev(char *jobz, char *uplo, int *n, c *ap, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:
 *     _fortran_chpev(jobz, uplo, n, <npy_complex64*>ap, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpev,CHPEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpev "F_FUNC(chpev,CHPEV)"(char *jobz, char *uplo, int *n, npy_complex64 *ap, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpev(char *jobz, char *uplo, int *n, c *ap, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpev(jobz, uplo, n, <npy_complex64*>ap, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpevd "F_FUNC(chpevd,CHPEVD)"(char *jobz, char *uplo, int *n, npy_complex64 *ap, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chpevd(char *jobz, char *uplo, int *n, c *ap, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpevd(jobz, uplo, n, <npy_complex64*>ap, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2342
 *     void _fortran_chpevd "F_FUNC(chpevd,CHPEVD)"(char *jobz, char *uplo, int *n, npy_complex64 *ap, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chpevd(char *jobz, char *uplo, int *n, c *ap, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_chpevd(jobz, uplo, n, <npy_complex64*>ap, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpevd,CHPEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpevd "F_FUNC(chpevd,CHPEVD)"(char *jobz, char *uplo, int *n, npy_complex64 *ap, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chpevd(char *jobz, char *uplo, int *n, c *ap, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpevd(jobz, uplo, n, <npy_complex64*>ap, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpevx "F_FUNC(chpevx,CHPEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chpevx(char *jobz, char *range, char *uplo, int *n, c *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpevx(jobz, range, uplo, n, <npy_complex64*>ap, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2347
 *     void _fortran_chpevx "F_FUNC(chpevx,CHPEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chpevx(char *jobz, char *range, char *uplo, int *n, c *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_chpevx(jobz, range, uplo, n, <npy_complex64*>ap, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpevx,CHPEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpevx "F_FUNC(chpevx,CHPEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex64 *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chpevx(char *jobz, char *range, char *uplo, int *n, c *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpevx(jobz, range, uplo, n, <npy_complex64*>ap, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpgst "F_FUNC(chpgst,CHPGST)"(int *itype, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, int *info) nogil
 * cdef void chpgst(int *itype, char *uplo, int *n, c *ap, c *bp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpgst(itype, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpgst(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_bp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2352
 *     void _fortran_chpgst "F_FUNC(chpgst,CHPGST)"(int *itype, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, int *info) nogil
 * cdef void chpgst(int *itype, char *uplo, int *n, c *ap, c *bp, int *info) nogil:
 *     _fortran_chpgst(itype, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpgst,CHPGST)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_bp), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpgst "F_FUNC(chpgst,CHPGST)"(int *itype, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, int *info) nogil
 * cdef void chpgst(int *itype, char *uplo, int *n, c *ap, c *bp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpgst(itype, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpgv "F_FUNC(chpgv,CHPGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpgv(int *itype, char *jobz, char *uplo, int *n, c *ap, c *bp, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpgv(itype, jobz, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpgv(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2357
 *     void _fortran_chpgv "F_FUNC(chpgv,CHPGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpgv(int *itype, char *jobz, char *uplo, int *n, c *ap, c *bp, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:
 *     _fortran_chpgv(itype, jobz, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpgv,CHPGV)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_bp), __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpgv "F_FUNC(chpgv,CHPGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpgv(int *itype, char *jobz, char *uplo, int *n, c *ap, c *bp, s *w, c *z, int *ldz, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpgv(itype, jobz, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpgvd "F_FUNC(chpgvd,CHPGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chpgvd(int *itype, char *jobz, char *uplo, int *n, c *ap, c *bp, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpgvd(itype, jobz, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpgvd(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2362
 *     void _fortran_chpgvd "F_FUNC(chpgvd,CHPGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chpgvd(int *itype, char *jobz, char *uplo, int *n, c *ap, c *bp, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_chpgvd(itype, jobz, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpgvd,CHPGVD)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_bp), __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpgvd "F_FUNC(chpgvd,CHPGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void chpgvd(int *itype, char *jobz, char *uplo, int *n, c *ap, c *bp, s *w, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpgvd(itype, jobz, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpgvx "F_FUNC(chpgvx,CHPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chpgvx(int *itype, char *jobz, char *range, char *uplo, int *n, c *ap, c *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpgvx(itype, jobz, range, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpgvx(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2367
 *     void _fortran_chpgvx "F_FUNC(chpgvx,CHPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chpgvx(int *itype, char *jobz, char *range, char *uplo, int *n, c *ap, c *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_chpgvx(itype, jobz, range, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpgvx,CHPGVX)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_bp), __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpgvx "F_FUNC(chpgvx,CHPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex64 *ap, npy_complex64 *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, npy_complex64 *work, s *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void chpgvx(int *itype, char *jobz, char *range, char *uplo, int *n, c *ap, c *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, c *work, s *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpgvx(itype, jobz, range, uplo, n, <npy_complex64*>ap, <npy_complex64*>bp, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, <npy_complex64*>work, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chprfs "F_FUNC(chprfs,CHPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2372
 *     void _fortran_chprfs "F_FUNC(chprfs,CHPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_chprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chprfs,CHPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_afp), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chprfs "F_FUNC(chprfs,CHPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpsv "F_FUNC(chpsv,CHPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chpsv(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpsv(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2377
 *     void _fortran_chpsv "F_FUNC(chpsv,CHPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chpsv(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_chpsv(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpsv,CHPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpsv "F_FUNC(chpsv,CHPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chpsv(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpsv(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpsvx "F_FUNC(chpsvx,CHPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chpsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2382
 *     void _fortran_chpsvx "F_FUNC(chpsvx,CHPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_chpsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chpsvx,CHPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_afp), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chpsvx "F_FUNC(chpsvx,CHPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void chpsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chpsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chptrd "F_FUNC(chptrd,CHPTRD)"(char *uplo, int *n, npy_complex64 *ap, s *d, s *e, npy_complex64 *tau, int *info) nogil
 * cdef void chptrd(char *uplo, int *n, c *ap, s *d, s *e, c *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chptrd(uplo, n, <npy_complex64*>ap, d, e, <npy_complex64*>tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chptrd(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2387
 *     void _fortran_chptrd "F_FUNC(chptrd,CHPTRD)"(char *uplo, int *n, npy_complex64 *ap, s *d, s *e, npy_complex64 *tau, int *info) nogil
 * cdef void chptrd(char *uplo, int *n, c *ap, s *d, s *e, c *tau, int *info) nogil:
 *     _fortran_chptrd(uplo, n, <npy_complex64*>ap, d, e, <npy_complex64*>tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chptrd,CHPTRD)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_tau), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chptrd "F_FUNC(chptrd,CHPTRD)"(char *uplo, int *n, npy_complex64 *ap, s *d, s *e, npy_complex64 *tau, int *info) nogil
 * cdef void chptrd(char *uplo, int *n, c *ap, s *d, s *e, c *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chptrd(uplo, n, <npy_complex64*>ap, d, e, <npy_complex64*>tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chptrf "F_FUNC(chptrf,CHPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, int *info) nogil
 * cdef void chptrf(char *uplo, int *n, c *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chptrf(uplo, n, <npy_complex64*>ap, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2392
 *     void _fortran_chptrf "F_FUNC(chptrf,CHPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, int *info) nogil
 * cdef void chptrf(char *uplo, int *n, c *ap, int *ipiv, int *info) nogil:
 *     _fortran_chptrf(uplo, n, <npy_complex64*>ap, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chptrf,CHPTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chptrf "F_FUNC(chptrf,CHPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, int *info) nogil
 * cdef void chptrf(char *uplo, int *n, c *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chptrf(uplo, n, <npy_complex64*>ap, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chptri "F_FUNC(chptri,CHPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void chptri(char *uplo, int *n, c *ap, int *ipiv, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chptri(uplo, n, <npy_complex64*>ap, ipiv, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2397
 *     void _fortran_chptri "F_FUNC(chptri,CHPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void chptri(char *uplo, int *n, c *ap, int *ipiv, c *work, int *info) nogil:
 *     _fortran_chptri(uplo, n, <npy_complex64*>ap, ipiv, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chptri,CHPTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chptri "F_FUNC(chptri,CHPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void chptri(char *uplo, int *n, c *ap, int *ipiv, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chptri(uplo, n, <npy_complex64*>ap, ipiv, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chptrs "F_FUNC(chptrs,CHPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chptrs(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chptrs(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2402
 *     void _fortran_chptrs "F_FUNC(chptrs,CHPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chptrs(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_chptrs(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chptrs,CHPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chptrs "F_FUNC(chptrs,CHPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void chptrs(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chptrs(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chsein "F_FUNC(chsein,CHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *ifaill, int *ifailr, int *info) nogil
 * cdef void chsein(char *side, char *eigsrc, char *initv, bint *select, int *n, c *h, int *ldh, c *w, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *ifaill, int *ifailr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chsein(side, eigsrc, initv, select, n, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, ifaill, ifailr, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chsein(char *__pyx_v_side, char *__pyx_v_eigsrc, char *__pyx_v_initv, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_float_complex *__pyx_v_w, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_ifaill, int *__pyx_v_ifailr, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2407
 *     void _fortran_chsein "F_FUNC(chsein,CHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *ifaill, int *ifailr, int *info) nogil
 * cdef void chsein(char *side, char *eigsrc, char *initv, bint *select, int *n, c *h, int *ldh, c *w, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *ifaill, int *ifailr, int *info) nogil:
 *     _fortran_chsein(side, eigsrc, initv, select, n, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, ifaill, ifailr, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chsein,CHSEIN)(__pyx_v_side, __pyx_v_eigsrc, __pyx_v_initv, __pyx_v_select, __pyx_v_n, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex64 *)__pyx_v_w), ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_ifaill, __pyx_v_ifailr, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chsein "F_FUNC(chsein,CHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *ifaill, int *ifailr, int *info) nogil
 * cdef void chsein(char *side, char *eigsrc, char *initv, bint *select, int *n, c *h, int *ldh, c *w, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *ifaill, int *ifailr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chsein(side, eigsrc, initv, select, n, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, ifaill, ifailr, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chseqr "F_FUNC(chseqr,CHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chseqr(char *job, char *compz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, c *z, int *ldz, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chseqr(job, compz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_chseqr(char *__pyx_v_job, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_float_complex *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2412
 *     void _fortran_chseqr "F_FUNC(chseqr,CHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chseqr(char *job, char *compz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, c *z, int *ldz, c *work, int *lwork, int *info) nogil:
 *     _fortran_chseqr(job, compz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(chseqr,CHSEQR)(__pyx_v_job, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex64 *)__pyx_v_w), ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_chseqr "F_FUNC(chseqr,CHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void chseqr(char *job, char *compz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, c *z, int *ldz, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_chseqr(job, compz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clabrd "F_FUNC(clabrd,CLABRD)"(int *m, int *n, int *nb, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *x, int *ldx, npy_complex64 *y, int *ldy) nogil
 * cdef void clabrd(int *m, int *n, int *nb, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *x, int *ldx, c *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clabrd(m, n, nb, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>x, ldx, <npy_complex64*>y, ldy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clabrd(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nb, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_tauq, __pyx_t_float_complex *__pyx_v_taup, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_float_complex *__pyx_v_y, int *__pyx_v_ldy) {

  /* "scipy/linalg/cython_lapack.pyx":2417
 *     void _fortran_clabrd "F_FUNC(clabrd,CLABRD)"(int *m, int *n, int *nb, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *x, int *ldx, npy_complex64 *y, int *ldy) nogil
 * cdef void clabrd(int *m, int *n, int *nb, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *x, int *ldx, c *y, int *ldy) nogil:
 *     _fortran_clabrd(m, n, nb, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>x, ldx, <npy_complex64*>y, ldy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clabrd,CLABRD)(__pyx_v_m, __pyx_v_n, __pyx_v_nb, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_tauq), ((npy_complex64 *)__pyx_v_taup), ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, ((npy_complex64 *)__pyx_v_y), __pyx_v_ldy);

  /* "scipy/linalg/cython_lapack.pyx":2416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clabrd "F_FUNC(clabrd,CLABRD)"(int *m, int *n, int *nb, npy_complex64 *a, int *lda, s *d, s *e, npy_complex64 *tauq, npy_complex64 *taup, npy_complex64 *x, int *ldx, npy_complex64 *y, int *ldy) nogil
 * cdef void clabrd(int *m, int *n, int *nb, c *a, int *lda, s *d, s *e, c *tauq, c *taup, c *x, int *ldx, c *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clabrd(m, n, nb, <npy_complex64*>a, lda, d, e, <npy_complex64*>tauq, <npy_complex64*>taup, <npy_complex64*>x, ldx, <npy_complex64*>y, ldy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacgv "F_FUNC(clacgv,CLACGV)"(int *n, npy_complex64 *x, int *incx) nogil
 * cdef void clacgv(int *n, c *x, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacgv(n, <npy_complex64*>x, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clacgv(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":2422
 *     void _fortran_clacgv "F_FUNC(clacgv,CLACGV)"(int *n, npy_complex64 *x, int *incx) nogil
 * cdef void clacgv(int *n, c *x, int *incx) nogil:
 *     _fortran_clacgv(n, <npy_complex64*>x, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clacgv,CLACGV)(__pyx_v_n, ((npy_complex64 *)__pyx_v_x), __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":2421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacgv "F_FUNC(clacgv,CLACGV)"(int *n, npy_complex64 *x, int *incx) nogil
 * cdef void clacgv(int *n, c *x, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacgv(n, <npy_complex64*>x, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacn2 "F_FUNC(clacn2,CLACN2)"(int *n, npy_complex64 *v, npy_complex64 *x, s *est, int *kase, int *isave) nogil
 * cdef void clacn2(int *n, c *v, c *x, s *est, int *kase, int *isave) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacn2(n, <npy_complex64*>v, <npy_complex64*>x, est, kase, isave)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clacn2(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_v, __pyx_t_float_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_est, int *__pyx_v_kase, int *__pyx_v_isave) {

  /* "scipy/linalg/cython_lapack.pyx":2427
 *     void _fortran_clacn2 "F_FUNC(clacn2,CLACN2)"(int *n, npy_complex64 *v, npy_complex64 *x, s *est, int *kase, int *isave) nogil
 * cdef void clacn2(int *n, c *v, c *x, s *est, int *kase, int *isave) nogil:
 *     _fortran_clacn2(n, <npy_complex64*>v, <npy_complex64*>x, est, kase, isave)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clacn2,CLACN2)(__pyx_v_n, ((npy_complex64 *)__pyx_v_v), ((npy_complex64 *)__pyx_v_x), __pyx_v_est, __pyx_v_kase, __pyx_v_isave);

  /* "scipy/linalg/cython_lapack.pyx":2426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacn2 "F_FUNC(clacn2,CLACN2)"(int *n, npy_complex64 *v, npy_complex64 *x, s *est, int *kase, int *isave) nogil
 * cdef void clacn2(int *n, c *v, c *x, s *est, int *kase, int *isave) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacn2(n, <npy_complex64*>v, <npy_complex64*>x, est, kase, isave)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacon "F_FUNC(clacon,CLACON)"(int *n, npy_complex64 *v, npy_complex64 *x, s *est, int *kase) nogil
 * cdef void clacon(int *n, c *v, c *x, s *est, int *kase) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacon(n, <npy_complex64*>v, <npy_complex64*>x, est, kase)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clacon(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_v, __pyx_t_float_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_est, int *__pyx_v_kase) {

  /* "scipy/linalg/cython_lapack.pyx":2432
 *     void _fortran_clacon "F_FUNC(clacon,CLACON)"(int *n, npy_complex64 *v, npy_complex64 *x, s *est, int *kase) nogil
 * cdef void clacon(int *n, c *v, c *x, s *est, int *kase) nogil:
 *     _fortran_clacon(n, <npy_complex64*>v, <npy_complex64*>x, est, kase)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clacon,CLACON)(__pyx_v_n, ((npy_complex64 *)__pyx_v_v), ((npy_complex64 *)__pyx_v_x), __pyx_v_est, __pyx_v_kase);

  /* "scipy/linalg/cython_lapack.pyx":2431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacon "F_FUNC(clacon,CLACON)"(int *n, npy_complex64 *v, npy_complex64 *x, s *est, int *kase) nogil
 * cdef void clacon(int *n, c *v, c *x, s *est, int *kase) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacon(n, <npy_complex64*>v, <npy_complex64*>x, est, kase)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacp2 "F_FUNC(clacp2,CLACP2)"(char *uplo, int *m, int *n, s *a, int *lda, npy_complex64 *b, int *ldb) nogil
 * cdef void clacp2(char *uplo, int *m, int *n, s *a, int *lda, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacp2(uplo, m, n, a, lda, <npy_complex64*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clacp2(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":2437
 *     void _fortran_clacp2 "F_FUNC(clacp2,CLACP2)"(char *uplo, int *m, int *n, s *a, int *lda, npy_complex64 *b, int *ldb) nogil
 * cdef void clacp2(char *uplo, int *m, int *n, s *a, int *lda, c *b, int *ldb) nogil:
 *     _fortran_clacp2(uplo, m, n, a, lda, <npy_complex64*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clacp2,CLACP2)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":2436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacp2 "F_FUNC(clacp2,CLACP2)"(char *uplo, int *m, int *n, s *a, int *lda, npy_complex64 *b, int *ldb) nogil
 * cdef void clacp2(char *uplo, int *m, int *n, s *a, int *lda, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacp2(uplo, m, n, a, lda, <npy_complex64*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacpy "F_FUNC(clacpy,CLACPY)"(char *uplo, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb) nogil
 * cdef void clacpy(char *uplo, int *m, int *n, c *a, int *lda, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacpy(uplo, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clacpy(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":2442
 *     void _fortran_clacpy "F_FUNC(clacpy,CLACPY)"(char *uplo, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb) nogil
 * cdef void clacpy(char *uplo, int *m, int *n, c *a, int *lda, c *b, int *ldb) nogil:
 *     _fortran_clacpy(uplo, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clacpy,CLACPY)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":2441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacpy "F_FUNC(clacpy,CLACPY)"(char *uplo, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb) nogil
 * cdef void clacpy(char *uplo, int *m, int *n, c *a, int *lda, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacpy(uplo, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacrm "F_FUNC(clacrm,CLACRM)"(int *m, int *n, npy_complex64 *a, int *lda, s *b, int *ldb, npy_complex64 *c, int *ldc, s *rwork) nogil
 * cdef void clacrm(int *m, int *n, c *a, int *lda, s *b, int *ldb, c *c, int *ldc, s *rwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacrm(m, n, <npy_complex64*>a, lda, b, ldb, <npy_complex64*>c, ldc, rwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clacrm(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork) {

  /* "scipy/linalg/cython_lapack.pyx":2447
 *     void _fortran_clacrm "F_FUNC(clacrm,CLACRM)"(int *m, int *n, npy_complex64 *a, int *lda, s *b, int *ldb, npy_complex64 *c, int *ldc, s *rwork) nogil
 * cdef void clacrm(int *m, int *n, c *a, int *lda, s *b, int *ldb, c *c, int *ldc, s *rwork) nogil:
 *     _fortran_clacrm(m, n, <npy_complex64*>a, lda, b, ldb, <npy_complex64*>c, ldc, rwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clacrm,CLACRM)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, __pyx_v_rwork);

  /* "scipy/linalg/cython_lapack.pyx":2446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacrm "F_FUNC(clacrm,CLACRM)"(int *m, int *n, npy_complex64 *a, int *lda, s *b, int *ldb, npy_complex64 *c, int *ldc, s *rwork) nogil
 * cdef void clacrm(int *m, int *n, c *a, int *lda, s *b, int *ldb, c *c, int *ldc, s *rwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacrm(m, n, <npy_complex64*>a, lda, b, ldb, <npy_complex64*>c, ldc, rwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacrt "F_FUNC(clacrt,CLACRT)"(int *n, npy_complex64 *cx, int *incx, npy_complex64 *cy, int *incy, npy_complex64 *c, npy_complex64 *s) nogil
 * cdef void clacrt(int *n, c *cx, int *incx, c *cy, int *incy, c *c, c *s) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacrt(n, <npy_complex64*>cx, incx, <npy_complex64*>cy, incy, <npy_complex64*>c, <npy_complex64*>s)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clacrt(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_cx, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_cy, int *__pyx_v_incy, __pyx_t_float_complex *__pyx_v_c, __pyx_t_float_complex *__pyx_v_s) {

  /* "scipy/linalg/cython_lapack.pyx":2452
 *     void _fortran_clacrt "F_FUNC(clacrt,CLACRT)"(int *n, npy_complex64 *cx, int *incx, npy_complex64 *cy, int *incy, npy_complex64 *c, npy_complex64 *s) nogil
 * cdef void clacrt(int *n, c *cx, int *incx, c *cy, int *incy, c *c, c *s) nogil:
 *     _fortran_clacrt(n, <npy_complex64*>cx, incx, <npy_complex64*>cy, incy, <npy_complex64*>c, <npy_complex64*>s)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clacrt,CLACRT)(__pyx_v_n, ((npy_complex64 *)__pyx_v_cx), __pyx_v_incx, ((npy_complex64 *)__pyx_v_cy), __pyx_v_incy, ((npy_complex64 *)__pyx_v_c), ((npy_complex64 *)__pyx_v_s));

  /* "scipy/linalg/cython_lapack.pyx":2451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clacrt "F_FUNC(clacrt,CLACRT)"(int *n, npy_complex64 *cx, int *incx, npy_complex64 *cy, int *incy, npy_complex64 *c, npy_complex64 *s) nogil
 * cdef void clacrt(int *n, c *cx, int *incx, c *cy, int *incy, c *c, c *s) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clacrt(n, <npy_complex64*>cx, incx, <npy_complex64*>cy, incy, <npy_complex64*>c, <npy_complex64*>s)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claed0 "F_FUNC(claed0,CLAED0)"(int *qsiz, int *n, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *qstore, int *ldqs, s *rwork, int *iwork, int *info) nogil
 * cdef void claed0(int *qsiz, int *n, s *d, s *e, c *q, int *ldq, c *qstore, int *ldqs, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claed0(qsiz, n, d, e, <npy_complex64*>q, ldq, <npy_complex64*>qstore, ldqs, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claed0(int *__pyx_v_qsiz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_qstore, int *__pyx_v_ldqs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2457
 *     void _fortran_claed0 "F_FUNC(claed0,CLAED0)"(int *qsiz, int *n, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *qstore, int *ldqs, s *rwork, int *iwork, int *info) nogil
 * cdef void claed0(int *qsiz, int *n, s *d, s *e, c *q, int *ldq, c *qstore, int *ldqs, s *rwork, int *iwork, int *info) nogil:
 *     _fortran_claed0(qsiz, n, d, e, <npy_complex64*>q, ldq, <npy_complex64*>qstore, ldqs, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claed0,CLAED0)(__pyx_v_qsiz, __pyx_v_n, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_qstore), __pyx_v_ldqs, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claed0 "F_FUNC(claed0,CLAED0)"(int *qsiz, int *n, s *d, s *e, npy_complex64 *q, int *ldq, npy_complex64 *qstore, int *ldqs, s *rwork, int *iwork, int *info) nogil
 * cdef void claed0(int *qsiz, int *n, s *d, s *e, c *q, int *ldq, c *qstore, int *ldqs, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claed0(qsiz, n, d, e, <npy_complex64*>q, ldq, <npy_complex64*>qstore, ldqs, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claed7 "F_FUNC(claed7,CLAED7)"(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, npy_complex64 *q, int *ldq, s *rho, int *indxq, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, npy_complex64 *work, s *rwork, int *iwork, int *info) nogil
 * cdef void claed7(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, c *q, int *ldq, s *rho, int *indxq, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, c *work, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claed7(n, cutpnt, qsiz, tlvls, curlvl, curpbm, d, <npy_complex64*>q, ldq, rho, indxq, qstore, qptr, prmptr, perm, givptr, givcol, givnum, <npy_complex64*>work, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claed7(int *__pyx_v_n, int *__pyx_v_cutpnt, int *__pyx_v_qsiz, int *__pyx_v_tlvls, int *__pyx_v_curlvl, int *__pyx_v_curpbm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_qstore, int *__pyx_v_qptr, int *__pyx_v_prmptr, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2462
 *     void _fortran_claed7 "F_FUNC(claed7,CLAED7)"(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, npy_complex64 *q, int *ldq, s *rho, int *indxq, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, npy_complex64 *work, s *rwork, int *iwork, int *info) nogil
 * cdef void claed7(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, c *q, int *ldq, s *rho, int *indxq, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, c *work, s *rwork, int *iwork, int *info) nogil:
 *     _fortran_claed7(n, cutpnt, qsiz, tlvls, curlvl, curpbm, d, <npy_complex64*>q, ldq, rho, indxq, qstore, qptr, prmptr, perm, givptr, givcol, givnum, <npy_complex64*>work, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claed7,CLAED7)(__pyx_v_n, __pyx_v_cutpnt, __pyx_v_qsiz, __pyx_v_tlvls, __pyx_v_curlvl, __pyx_v_curpbm, __pyx_v_d, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_rho, __pyx_v_indxq, __pyx_v_qstore, __pyx_v_qptr, __pyx_v_prmptr, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claed7 "F_FUNC(claed7,CLAED7)"(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, npy_complex64 *q, int *ldq, s *rho, int *indxq, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, npy_complex64 *work, s *rwork, int *iwork, int *info) nogil
 * cdef void claed7(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, c *q, int *ldq, s *rho, int *indxq, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, c *work, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claed7(n, cutpnt, qsiz, tlvls, curlvl, curpbm, d, <npy_complex64*>q, ldq, rho, indxq, qstore, qptr, prmptr, perm, givptr, givcol, givnum, <npy_complex64*>work, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claed8 "F_FUNC(claed8,CLAED8)"(int *k, int *n, int *qsiz, npy_complex64 *q, int *ldq, s *d, s *rho, int *cutpnt, s *z, s *dlamda, npy_complex64 *q2, int *ldq2, s *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, s *givnum, int *info) nogil
 * cdef void claed8(int *k, int *n, int *qsiz, c *q, int *ldq, s *d, s *rho, int *cutpnt, s *z, s *dlamda, c *q2, int *ldq2, s *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, s *givnum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claed8(k, n, qsiz, <npy_complex64*>q, ldq, d, rho, cutpnt, z, dlamda, <npy_complex64*>q2, ldq2, w, indxp, indx, indxq, perm, givptr, givcol, givnum, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claed8(int *__pyx_v_k, int *__pyx_v_n, int *__pyx_v_qsiz, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, int *__pyx_v_cutpnt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlamda, __pyx_t_float_complex *__pyx_v_q2, int *__pyx_v_ldq2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, int *__pyx_v_indxp, int *__pyx_v_indx, int *__pyx_v_indxq, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2467
 *     void _fortran_claed8 "F_FUNC(claed8,CLAED8)"(int *k, int *n, int *qsiz, npy_complex64 *q, int *ldq, s *d, s *rho, int *cutpnt, s *z, s *dlamda, npy_complex64 *q2, int *ldq2, s *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, s *givnum, int *info) nogil
 * cdef void claed8(int *k, int *n, int *qsiz, c *q, int *ldq, s *d, s *rho, int *cutpnt, s *z, s *dlamda, c *q2, int *ldq2, s *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, s *givnum, int *info) nogil:
 *     _fortran_claed8(k, n, qsiz, <npy_complex64*>q, ldq, d, rho, cutpnt, z, dlamda, <npy_complex64*>q2, ldq2, w, indxp, indx, indxq, perm, givptr, givcol, givnum, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claed8,CLAED8)(__pyx_v_k, __pyx_v_n, __pyx_v_qsiz, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_d, __pyx_v_rho, __pyx_v_cutpnt, __pyx_v_z, __pyx_v_dlamda, ((npy_complex64 *)__pyx_v_q2), __pyx_v_ldq2, __pyx_v_w, __pyx_v_indxp, __pyx_v_indx, __pyx_v_indxq, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claed8 "F_FUNC(claed8,CLAED8)"(int *k, int *n, int *qsiz, npy_complex64 *q, int *ldq, s *d, s *rho, int *cutpnt, s *z, s *dlamda, npy_complex64 *q2, int *ldq2, s *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, s *givnum, int *info) nogil
 * cdef void claed8(int *k, int *n, int *qsiz, c *q, int *ldq, s *d, s *rho, int *cutpnt, s *z, s *dlamda, c *q2, int *ldq2, s *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, s *givnum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claed8(k, n, qsiz, <npy_complex64*>q, ldq, d, rho, cutpnt, z, dlamda, <npy_complex64*>q2, ldq2, w, indxp, indx, indxq, perm, givptr, givcol, givnum, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claein "F_FUNC(claein,CLAEIN)"(bint *rightv, bint *noinit, int *n, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *v, npy_complex64 *b, int *ldb, s *rwork, s *eps3, s *smlnum, int *info) nogil
 * cdef void claein(bint *rightv, bint *noinit, int *n, c *h, int *ldh, c *w, c *v, c *b, int *ldb, s *rwork, s *eps3, s *smlnum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claein(rightv, noinit, n, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>v, <npy_complex64*>b, ldb, rwork, eps3, smlnum, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claein(int *__pyx_v_rightv, int *__pyx_v_noinit, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_float_complex *__pyx_v_w, __pyx_t_float_complex *__pyx_v_v, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_eps3, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_smlnum, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2472
 *     void _fortran_claein "F_FUNC(claein,CLAEIN)"(bint *rightv, bint *noinit, int *n, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *v, npy_complex64 *b, int *ldb, s *rwork, s *eps3, s *smlnum, int *info) nogil
 * cdef void claein(bint *rightv, bint *noinit, int *n, c *h, int *ldh, c *w, c *v, c *b, int *ldb, s *rwork, s *eps3, s *smlnum, int *info) nogil:
 *     _fortran_claein(rightv, noinit, n, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>v, <npy_complex64*>b, ldb, rwork, eps3, smlnum, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claein,CLAEIN)(__pyx_v_rightv, __pyx_v_noinit, __pyx_v_n, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex64 *)__pyx_v_w), ((npy_complex64 *)__pyx_v_v), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_rwork, __pyx_v_eps3, __pyx_v_smlnum, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claein "F_FUNC(claein,CLAEIN)"(bint *rightv, bint *noinit, int *n, npy_complex64 *h, int *ldh, npy_complex64 *w, npy_complex64 *v, npy_complex64 *b, int *ldb, s *rwork, s *eps3, s *smlnum, int *info) nogil
 * cdef void claein(bint *rightv, bint *noinit, int *n, c *h, int *ldh, c *w, c *v, c *b, int *ldb, s *rwork, s *eps3, s *smlnum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claein(rightv, noinit, n, <npy_complex64*>h, ldh, <npy_complex64*>w, <npy_complex64*>v, <npy_complex64*>b, ldb, rwork, eps3, smlnum, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claesy "F_FUNC(claesy,CLAESY)"(npy_complex64 *a, npy_complex64 *b, npy_complex64 *c, npy_complex64 *rt1, npy_complex64 *rt2, npy_complex64 *evscal, npy_complex64 *cs1, npy_complex64 *sn1) nogil
 * cdef void claesy(c *a, c *b, c *c, c *rt1, c *rt2, c *evscal, c *cs1, c *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claesy(<npy_complex64*>a, <npy_complex64*>b, <npy_complex64*>c, <npy_complex64*>rt1, <npy_complex64*>rt2, <npy_complex64*>evscal, <npy_complex64*>cs1, <npy_complex64*>sn1)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claesy(__pyx_t_float_complex *__pyx_v_a, __pyx_t_float_complex *__pyx_v_b, __pyx_t_float_complex *__pyx_v_c, __pyx_t_float_complex *__pyx_v_rt1, __pyx_t_float_complex *__pyx_v_rt2, __pyx_t_float_complex *__pyx_v_evscal, __pyx_t_float_complex *__pyx_v_cs1, __pyx_t_float_complex *__pyx_v_sn1) {

  /* "scipy/linalg/cython_lapack.pyx":2477
 *     void _fortran_claesy "F_FUNC(claesy,CLAESY)"(npy_complex64 *a, npy_complex64 *b, npy_complex64 *c, npy_complex64 *rt1, npy_complex64 *rt2, npy_complex64 *evscal, npy_complex64 *cs1, npy_complex64 *sn1) nogil
 * cdef void claesy(c *a, c *b, c *c, c *rt1, c *rt2, c *evscal, c *cs1, c *sn1) nogil:
 *     _fortran_claesy(<npy_complex64*>a, <npy_complex64*>b, <npy_complex64*>c, <npy_complex64*>rt1, <npy_complex64*>rt2, <npy_complex64*>evscal, <npy_complex64*>cs1, <npy_complex64*>sn1)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claesy,CLAESY)(((npy_complex64 *)__pyx_v_a), ((npy_complex64 *)__pyx_v_b), ((npy_complex64 *)__pyx_v_c), ((npy_complex64 *)__pyx_v_rt1), ((npy_complex64 *)__pyx_v_rt2), ((npy_complex64 *)__pyx_v_evscal), ((npy_complex64 *)__pyx_v_cs1), ((npy_complex64 *)__pyx_v_sn1));

  /* "scipy/linalg/cython_lapack.pyx":2476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claesy "F_FUNC(claesy,CLAESY)"(npy_complex64 *a, npy_complex64 *b, npy_complex64 *c, npy_complex64 *rt1, npy_complex64 *rt2, npy_complex64 *evscal, npy_complex64 *cs1, npy_complex64 *sn1) nogil
 * cdef void claesy(c *a, c *b, c *c, c *rt1, c *rt2, c *evscal, c *cs1, c *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claesy(<npy_complex64*>a, <npy_complex64*>b, <npy_complex64*>c, <npy_complex64*>rt1, <npy_complex64*>rt2, <npy_complex64*>evscal, <npy_complex64*>cs1, <npy_complex64*>sn1)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claev2 "F_FUNC(claev2,CLAEV2)"(npy_complex64 *a, npy_complex64 *b, npy_complex64 *c, s *rt1, s *rt2, s *cs1, npy_complex64 *sn1) nogil
 * cdef void claev2(c *a, c *b, c *c, s *rt1, s *rt2, s *cs1, c *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claev2(<npy_complex64*>a, <npy_complex64*>b, <npy_complex64*>c, rt1, rt2, cs1, <npy_complex64*>sn1)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claev2(__pyx_t_float_complex *__pyx_v_a, __pyx_t_float_complex *__pyx_v_b, __pyx_t_float_complex *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cs1, __pyx_t_float_complex *__pyx_v_sn1) {

  /* "scipy/linalg/cython_lapack.pyx":2482
 *     void _fortran_claev2 "F_FUNC(claev2,CLAEV2)"(npy_complex64 *a, npy_complex64 *b, npy_complex64 *c, s *rt1, s *rt2, s *cs1, npy_complex64 *sn1) nogil
 * cdef void claev2(c *a, c *b, c *c, s *rt1, s *rt2, s *cs1, c *sn1) nogil:
 *     _fortran_claev2(<npy_complex64*>a, <npy_complex64*>b, <npy_complex64*>c, rt1, rt2, cs1, <npy_complex64*>sn1)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claev2,CLAEV2)(((npy_complex64 *)__pyx_v_a), ((npy_complex64 *)__pyx_v_b), ((npy_complex64 *)__pyx_v_c), __pyx_v_rt1, __pyx_v_rt2, __pyx_v_cs1, ((npy_complex64 *)__pyx_v_sn1));

  /* "scipy/linalg/cython_lapack.pyx":2481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claev2 "F_FUNC(claev2,CLAEV2)"(npy_complex64 *a, npy_complex64 *b, npy_complex64 *c, s *rt1, s *rt2, s *cs1, npy_complex64 *sn1) nogil
 * cdef void claev2(c *a, c *b, c *c, s *rt1, s *rt2, s *cs1, c *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claev2(<npy_complex64*>a, <npy_complex64*>b, <npy_complex64*>c, rt1, rt2, cs1, <npy_complex64*>sn1)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clag2z "F_FUNC(clag2z,CLAG2Z)"(int *m, int *n, npy_complex64 *sa, int *ldsa, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void clag2z(int *m, int *n, c *sa, int *ldsa, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clag2z(m, n, <npy_complex64*>sa, ldsa, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clag2z(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_sa, int *__pyx_v_ldsa, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2487
 *     void _fortran_clag2z "F_FUNC(clag2z,CLAG2Z)"(int *m, int *n, npy_complex64 *sa, int *ldsa, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void clag2z(int *m, int *n, c *sa, int *ldsa, z *a, int *lda, int *info) nogil:
 *     _fortran_clag2z(m, n, <npy_complex64*>sa, ldsa, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clag2z,CLAG2Z)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_sa), __pyx_v_ldsa, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clag2z "F_FUNC(clag2z,CLAG2Z)"(int *m, int *n, npy_complex64 *sa, int *ldsa, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void clag2z(int *m, int *n, c *sa, int *ldsa, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clag2z(m, n, <npy_complex64*>sa, ldsa, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clags2 "F_FUNC(clags2,CLAGS2)"(bint *upper, s *a1, npy_complex64 *a2, s *a3, s *b1, npy_complex64 *b2, s *b3, s *csu, npy_complex64 *snu, s *csv, npy_complex64 *snv, s *csq, npy_complex64 *snq) nogil
 * cdef void clags2(bint *upper, s *a1, c *a2, s *a3, s *b1, c *b2, s *b3, s *csu, c *snu, s *csv, c *snv, s *csq, c *snq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clags2(upper, a1, <npy_complex64*>a2, a3, b1, <npy_complex64*>b2, b3, csu, <npy_complex64*>snu, csv, <npy_complex64*>snv, csq, <npy_complex64*>snq)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clags2(int *__pyx_v_upper, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a1, __pyx_t_float_complex *__pyx_v_a2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a3, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b1, __pyx_t_float_complex *__pyx_v_b2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b3, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csu, __pyx_t_float_complex *__pyx_v_snu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csv, __pyx_t_float_complex *__pyx_v_snv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csq, __pyx_t_float_complex *__pyx_v_snq) {

  /* "scipy/linalg/cython_lapack.pyx":2492
 *     void _fortran_clags2 "F_FUNC(clags2,CLAGS2)"(bint *upper, s *a1, npy_complex64 *a2, s *a3, s *b1, npy_complex64 *b2, s *b3, s *csu, npy_complex64 *snu, s *csv, npy_complex64 *snv, s *csq, npy_complex64 *snq) nogil
 * cdef void clags2(bint *upper, s *a1, c *a2, s *a3, s *b1, c *b2, s *b3, s *csu, c *snu, s *csv, c *snv, s *csq, c *snq) nogil:
 *     _fortran_clags2(upper, a1, <npy_complex64*>a2, a3, b1, <npy_complex64*>b2, b3, csu, <npy_complex64*>snu, csv, <npy_complex64*>snv, csq, <npy_complex64*>snq)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clags2,CLAGS2)(__pyx_v_upper, __pyx_v_a1, ((npy_complex64 *)__pyx_v_a2), __pyx_v_a3, __pyx_v_b1, ((npy_complex64 *)__pyx_v_b2), __pyx_v_b3, __pyx_v_csu, ((npy_complex64 *)__pyx_v_snu), __pyx_v_csv, ((npy_complex64 *)__pyx_v_snv), __pyx_v_csq, ((npy_complex64 *)__pyx_v_snq));

  /* "scipy/linalg/cython_lapack.pyx":2491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clags2 "F_FUNC(clags2,CLAGS2)"(bint *upper, s *a1, npy_complex64 *a2, s *a3, s *b1, npy_complex64 *b2, s *b3, s *csu, npy_complex64 *snu, s *csv, npy_complex64 *snv, s *csq, npy_complex64 *snq) nogil
 * cdef void clags2(bint *upper, s *a1, c *a2, s *a3, s *b1, c *b2, s *b3, s *csu, c *snu, s *csv, c *snv, s *csq, c *snq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clags2(upper, a1, <npy_complex64*>a2, a3, b1, <npy_complex64*>b2, b3, csu, <npy_complex64*>snu, csv, <npy_complex64*>snv, csq, <npy_complex64*>snq)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clagtm "F_FUNC(clagtm,CLAGTM)"(char *trans, int *n, int *nrhs, s *alpha, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *x, int *ldx, s *beta, npy_complex64 *b, int *ldb) nogil
 * cdef void clagtm(char *trans, int *n, int *nrhs, s *alpha, c *dl, c *d, c *du, c *x, int *ldx, s *beta, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clagtm(trans, n, nrhs, alpha, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>x, ldx, beta, <npy_complex64*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clagtm(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_dl, __pyx_t_float_complex *__pyx_v_d, __pyx_t_float_complex *__pyx_v_du, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":2497
 *     void _fortran_clagtm "F_FUNC(clagtm,CLAGTM)"(char *trans, int *n, int *nrhs, s *alpha, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *x, int *ldx, s *beta, npy_complex64 *b, int *ldb) nogil
 * cdef void clagtm(char *trans, int *n, int *nrhs, s *alpha, c *dl, c *d, c *du, c *x, int *ldx, s *beta, c *b, int *ldb) nogil:
 *     _fortran_clagtm(trans, n, nrhs, alpha, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>x, ldx, beta, <npy_complex64*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clagtm,CLAGTM)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_alpha, ((npy_complex64 *)__pyx_v_dl), ((npy_complex64 *)__pyx_v_d), ((npy_complex64 *)__pyx_v_du), ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_beta, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":2496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clagtm "F_FUNC(clagtm,CLAGTM)"(char *trans, int *n, int *nrhs, s *alpha, npy_complex64 *dl, npy_complex64 *d, npy_complex64 *du, npy_complex64 *x, int *ldx, s *beta, npy_complex64 *b, int *ldb) nogil
 * cdef void clagtm(char *trans, int *n, int *nrhs, s *alpha, c *dl, c *d, c *du, c *x, int *ldx, s *beta, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clagtm(trans, n, nrhs, alpha, <npy_complex64*>dl, <npy_complex64*>d, <npy_complex64*>du, <npy_complex64*>x, ldx, beta, <npy_complex64*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clahef "F_FUNC(clahef,CLAHEF)"(char *uplo, int *n, int *nb, int *kb, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *w, int *ldw, int *info) nogil
 * cdef void clahef(char *uplo, int *n, int *nb, int *kb, c *a, int *lda, int *ipiv, c *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clahef(uplo, n, nb, kb, <npy_complex64*>a, lda, ipiv, <npy_complex64*>w, ldw, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clahef(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_w, int *__pyx_v_ldw, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2502
 *     void _fortran_clahef "F_FUNC(clahef,CLAHEF)"(char *uplo, int *n, int *nb, int *kb, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *w, int *ldw, int *info) nogil
 * cdef void clahef(char *uplo, int *n, int *nb, int *kb, c *a, int *lda, int *ipiv, c *w, int *ldw, int *info) nogil:
 *     _fortran_clahef(uplo, n, nb, kb, <npy_complex64*>a, lda, ipiv, <npy_complex64*>w, ldw, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clahef,CLAHEF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, __pyx_v_kb, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_w), __pyx_v_ldw, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clahef "F_FUNC(clahef,CLAHEF)"(char *uplo, int *n, int *nb, int *kb, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *w, int *ldw, int *info) nogil
 * cdef void clahef(char *uplo, int *n, int *nb, int *kb, c *a, int *lda, int *ipiv, c *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clahef(uplo, n, nb, kb, <npy_complex64*>a, lda, ipiv, <npy_complex64*>w, ldw, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clahqr "F_FUNC(clahqr,CLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *info) nogil
 * cdef void clahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clahqr(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clahqr(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_float_complex *__pyx_v_w, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2507
 *     void _fortran_clahqr "F_FUNC(clahqr,CLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *info) nogil
 * cdef void clahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, int *info) nogil:
 *     _fortran_clahqr(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clahqr,CLAHQR)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex64 *)__pyx_v_w), __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clahqr "F_FUNC(clahqr,CLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *info) nogil
 * cdef void clahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clahqr(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clahr2 "F_FUNC(clahr2,CLAHR2)"(int *n, int *k, int *nb, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *t, int *ldt, npy_complex64 *y, int *ldy) nogil
 * cdef void clahr2(int *n, int *k, int *nb, c *a, int *lda, c *tau, c *t, int *ldt, c *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clahr2(n, k, nb, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>t, ldt, <npy_complex64*>y, ldy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clahr2(int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_nb, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_float_complex *__pyx_v_y, int *__pyx_v_ldy) {

  /* "scipy/linalg/cython_lapack.pyx":2512
 *     void _fortran_clahr2 "F_FUNC(clahr2,CLAHR2)"(int *n, int *k, int *nb, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *t, int *ldt, npy_complex64 *y, int *ldy) nogil
 * cdef void clahr2(int *n, int *k, int *nb, c *a, int *lda, c *tau, c *t, int *ldt, c *y, int *ldy) nogil:
 *     _fortran_clahr2(n, k, nb, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>t, ldt, <npy_complex64*>y, ldy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clahr2,CLAHR2)(__pyx_v_n, __pyx_v_k, __pyx_v_nb, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex64 *)__pyx_v_y), __pyx_v_ldy);

  /* "scipy/linalg/cython_lapack.pyx":2511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clahr2 "F_FUNC(clahr2,CLAHR2)"(int *n, int *k, int *nb, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *t, int *ldt, npy_complex64 *y, int *ldy) nogil
 * cdef void clahr2(int *n, int *k, int *nb, c *a, int *lda, c *tau, c *t, int *ldt, c *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clahr2(n, k, nb, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>t, ldt, <npy_complex64*>y, ldy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claic1 "F_FUNC(claic1,CLAIC1)"(int *job, int *j, npy_complex64 *x, s *sest, npy_complex64 *w, npy_complex64 *gamma, s *sestpr, npy_complex64 *s, npy_complex64 *c) nogil
 * cdef void claic1(int *job, int *j, c *x, s *sest, c *w, c *gamma, s *sestpr, c *s, c *c) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claic1(job, j, <npy_complex64*>x, sest, <npy_complex64*>w, <npy_complex64*>gamma, sestpr, <npy_complex64*>s, <npy_complex64*>c)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claic1(int *__pyx_v_job, int *__pyx_v_j, __pyx_t_float_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sest, __pyx_t_float_complex *__pyx_v_w, __pyx_t_float_complex *__pyx_v_gamma, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sestpr, __pyx_t_float_complex *__pyx_v_s, __pyx_t_float_complex *__pyx_v_c) {

  /* "scipy/linalg/cython_lapack.pyx":2517
 *     void _fortran_claic1 "F_FUNC(claic1,CLAIC1)"(int *job, int *j, npy_complex64 *x, s *sest, npy_complex64 *w, npy_complex64 *gamma, s *sestpr, npy_complex64 *s, npy_complex64 *c) nogil
 * cdef void claic1(int *job, int *j, c *x, s *sest, c *w, c *gamma, s *sestpr, c *s, c *c) nogil:
 *     _fortran_claic1(job, j, <npy_complex64*>x, sest, <npy_complex64*>w, <npy_complex64*>gamma, sestpr, <npy_complex64*>s, <npy_complex64*>c)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claic1,CLAIC1)(__pyx_v_job, __pyx_v_j, ((npy_complex64 *)__pyx_v_x), __pyx_v_sest, ((npy_complex64 *)__pyx_v_w), ((npy_complex64 *)__pyx_v_gamma), __pyx_v_sestpr, ((npy_complex64 *)__pyx_v_s), ((npy_complex64 *)__pyx_v_c));

  /* "scipy/linalg/cython_lapack.pyx":2516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claic1 "F_FUNC(claic1,CLAIC1)"(int *job, int *j, npy_complex64 *x, s *sest, npy_complex64 *w, npy_complex64 *gamma, s *sestpr, npy_complex64 *s, npy_complex64 *c) nogil
 * cdef void claic1(int *job, int *j, c *x, s *sest, c *w, c *gamma, s *sestpr, c *s, c *c) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claic1(job, j, <npy_complex64*>x, sest, <npy_complex64*>w, <npy_complex64*>gamma, sestpr, <npy_complex64*>s, <npy_complex64*>c)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clals0 "F_FUNC(clals0,CLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, npy_complex64 *b, int *ldb, npy_complex64 *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *rwork, int *info) nogil
 * cdef void clals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, c *b, int *ldb, c *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clals0(icompq, nl, nr, sqre, nrhs, <npy_complex64*>b, ldb, <npy_complex64*>bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clals0(int *__pyx_v_icompq, int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_bx, int *__pyx_v_ldbx, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, int *__pyx_v_ldgnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_poles, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2522
 *     void _fortran_clals0 "F_FUNC(clals0,CLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, npy_complex64 *b, int *ldb, npy_complex64 *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *rwork, int *info) nogil
 * cdef void clals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, c *b, int *ldb, c *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *rwork, int *info) nogil:
 *     _fortran_clals0(icompq, nl, nr, sqre, nrhs, <npy_complex64*>b, ldb, <npy_complex64*>bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clals0,CLALS0)(__pyx_v_icompq, __pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_bx), __pyx_v_ldbx, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_givnum, __pyx_v_ldgnum, __pyx_v_poles, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_k, __pyx_v_c, __pyx_v_s, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clals0 "F_FUNC(clals0,CLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, npy_complex64 *b, int *ldb, npy_complex64 *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *rwork, int *info) nogil
 * cdef void clals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, c *b, int *ldb, c *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clals0(icompq, nl, nr, sqre, nrhs, <npy_complex64*>b, ldb, <npy_complex64*>bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clalsa "F_FUNC(clalsa,CLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, npy_complex64 *b, int *ldb, npy_complex64 *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *rwork, int *iwork, int *info) nogil
 * cdef void clalsa(int *icompq, int *smlsiz, int *n, int *nrhs, c *b, int *ldb, c *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clalsa(icompq, smlsiz, n, nrhs, <npy_complex64*>b, ldb, <npy_complex64*>bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clalsa(int *__pyx_v_icompq, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_bx, int *__pyx_v_ldbx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_poles, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, int *__pyx_v_perm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2527
 *     void _fortran_clalsa "F_FUNC(clalsa,CLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, npy_complex64 *b, int *ldb, npy_complex64 *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *rwork, int *iwork, int *info) nogil
 * cdef void clalsa(int *icompq, int *smlsiz, int *n, int *nrhs, c *b, int *ldb, c *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *rwork, int *iwork, int *info) nogil:
 *     _fortran_clalsa(icompq, smlsiz, n, nrhs, <npy_complex64*>b, ldb, <npy_complex64*>bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clalsa,CLALSA)(__pyx_v_icompq, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_bx), __pyx_v_ldbx, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_k, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_poles, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_perm, __pyx_v_givnum, __pyx_v_c, __pyx_v_s, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clalsa "F_FUNC(clalsa,CLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, npy_complex64 *b, int *ldb, npy_complex64 *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *rwork, int *iwork, int *info) nogil
 * cdef void clalsa(int *icompq, int *smlsiz, int *n, int *nrhs, c *b, int *ldb, c *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clalsa(icompq, smlsiz, n, nrhs, <npy_complex64*>b, ldb, <npy_complex64*>bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clalsd "F_FUNC(clalsd,CLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, npy_complex64 *b, int *ldb, s *rcond, int *rank, npy_complex64 *work, s *rwork, int *iwork, int *info) nogil
 * cdef void clalsd(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, c *b, int *ldb, s *rcond, int *rank, c *work, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clalsd(uplo, smlsiz, n, nrhs, d, e, <npy_complex64*>b, ldb, rcond, rank, <npy_complex64*>work, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clalsd(char *__pyx_v_uplo, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2532
 *     void _fortran_clalsd "F_FUNC(clalsd,CLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, npy_complex64 *b, int *ldb, s *rcond, int *rank, npy_complex64 *work, s *rwork, int *iwork, int *info) nogil
 * cdef void clalsd(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, c *b, int *ldb, s *rcond, int *rank, c *work, s *rwork, int *iwork, int *info) nogil:
 *     _fortran_clalsd(uplo, smlsiz, n, nrhs, d, e, <npy_complex64*>b, ldb, rcond, rank, <npy_complex64*>work, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clalsd,CLALSD)(__pyx_v_uplo, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_rcond, __pyx_v_rank, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clalsd "F_FUNC(clalsd,CLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, npy_complex64 *b, int *ldb, s *rcond, int *rank, npy_complex64 *work, s *rwork, int *iwork, int *info) nogil
 * cdef void clalsd(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, c *b, int *ldb, s *rcond, int *rank, c *work, s *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clalsd(uplo, smlsiz, n, nrhs, d, e, <npy_complex64*>b, ldb, rcond, rank, <npy_complex64*>work, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clapll "F_FUNC(clapll,CLAPLL)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *ssmin) nogil
 * cdef void clapll(int *n, c *x, int *incx, c *y, int *incy, s *ssmin) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clapll(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, ssmin)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clapll(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ssmin) {

  /* "scipy/linalg/cython_lapack.pyx":2537
 *     void _fortran_clapll "F_FUNC(clapll,CLAPLL)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *ssmin) nogil
 * cdef void clapll(int *n, c *x, int *incx, c *y, int *incy, s *ssmin) nogil:
 *     _fortran_clapll(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, ssmin)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clapll,CLAPLL)(__pyx_v_n, ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, ((npy_complex64 *)__pyx_v_y), __pyx_v_incy, __pyx_v_ssmin);

  /* "scipy/linalg/cython_lapack.pyx":2536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clapll "F_FUNC(clapll,CLAPLL)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *ssmin) nogil
 * cdef void clapll(int *n, c *x, int *incx, c *y, int *incy, s *ssmin) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clapll(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, ssmin)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clapmt "F_FUNC(clapmt,CLAPMT)"(bint *forwrd, int *m, int *n, npy_complex64 *x, int *ldx, int *k) nogil
 * cdef void clapmt(bint *forwrd, int *m, int *n, c *x, int *ldx, int *k) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clapmt(forwrd, m, n, <npy_complex64*>x, ldx, k)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clapmt(int *__pyx_v_forwrd, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, int *__pyx_v_k) {

  /* "scipy/linalg/cython_lapack.pyx":2542
 *     void _fortran_clapmt "F_FUNC(clapmt,CLAPMT)"(bint *forwrd, int *m, int *n, npy_complex64 *x, int *ldx, int *k) nogil
 * cdef void clapmt(bint *forwrd, int *m, int *n, c *x, int *ldx, int *k) nogil:
 *     _fortran_clapmt(forwrd, m, n, <npy_complex64*>x, ldx, k)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clapmt,CLAPMT)(__pyx_v_forwrd, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_k);

  /* "scipy/linalg/cython_lapack.pyx":2541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clapmt "F_FUNC(clapmt,CLAPMT)"(bint *forwrd, int *m, int *n, npy_complex64 *x, int *ldx, int *k) nogil
 * cdef void clapmt(bint *forwrd, int *m, int *n, c *x, int *ldx, int *k) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clapmt(forwrd, m, n, <npy_complex64*>x, ldx, k)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqgb "F_FUNC(claqgb,CLAQGB)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void claqgb(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqgb(m, n, kl, ku, <npy_complex64*>ab, ldab, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqgb(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":2547
 *     void _fortran_claqgb "F_FUNC(claqgb,CLAQGB)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void claqgb(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:
 *     _fortran_claqgb(m, n, kl, ku, <npy_complex64*>ab, ldab, r, c, rowcnd, colcnd, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqgb,CLAQGB)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":2546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqgb "F_FUNC(claqgb,CLAQGB)"(int *m, int *n, int *kl, int *ku, npy_complex64 *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void claqgb(int *m, int *n, int *kl, int *ku, c *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqgb(m, n, kl, ku, <npy_complex64*>ab, ldab, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqge "F_FUNC(claqge,CLAQGE)"(int *m, int *n, npy_complex64 *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void claqge(int *m, int *n, c *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqge(m, n, <npy_complex64*>a, lda, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqge(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":2552
 *     void _fortran_claqge "F_FUNC(claqge,CLAQGE)"(int *m, int *n, npy_complex64 *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void claqge(int *m, int *n, c *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:
 *     _fortran_claqge(m, n, <npy_complex64*>a, lda, r, c, rowcnd, colcnd, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqge,CLAQGE)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":2551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqge "F_FUNC(claqge,CLAQGE)"(int *m, int *n, npy_complex64 *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void claqge(int *m, int *n, c *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqge(m, n, <npy_complex64*>a, lda, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqhb "F_FUNC(claqhb,CLAQHB)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhb(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqhb(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqhb(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":2557
 *     void _fortran_claqhb "F_FUNC(claqhb,CLAQHB)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhb(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_claqhb(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqhb,CLAQHB)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":2556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqhb "F_FUNC(claqhb,CLAQHB)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhb(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqhb(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqhe "F_FUNC(claqhe,CLAQHE)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhe(char *uplo, int *n, c *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqhe(uplo, n, <npy_complex64*>a, lda, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqhe(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":2562
 *     void _fortran_claqhe "F_FUNC(claqhe,CLAQHE)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhe(char *uplo, int *n, c *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_claqhe(uplo, n, <npy_complex64*>a, lda, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqhe,CLAQHE)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":2561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqhe "F_FUNC(claqhe,CLAQHE)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhe(char *uplo, int *n, c *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqhe(uplo, n, <npy_complex64*>a, lda, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqhp "F_FUNC(claqhp,CLAQHP)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhp(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqhp(uplo, n, <npy_complex64*>ap, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqhp(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":2567
 *     void _fortran_claqhp "F_FUNC(claqhp,CLAQHP)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhp(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_claqhp(uplo, n, <npy_complex64*>ap, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqhp,CLAQHP)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":2566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqhp "F_FUNC(claqhp,CLAQHP)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqhp(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqhp(uplo, n, <npy_complex64*>ap, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqp2 "F_FUNC(claqp2,CLAQP2)"(int *m, int *n, int *offset, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, s *vn1, s *vn2, npy_complex64 *work) nogil
 * cdef void claqp2(int *m, int *n, int *offset, c *a, int *lda, int *jpvt, c *tau, s *vn1, s *vn2, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqp2(m, n, offset, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, vn1, vn2, <npy_complex64*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqp2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_offset, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vn1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vn2, __pyx_t_float_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":2572
 *     void _fortran_claqp2 "F_FUNC(claqp2,CLAQP2)"(int *m, int *n, int *offset, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, s *vn1, s *vn2, npy_complex64 *work) nogil
 * cdef void claqp2(int *m, int *n, int *offset, c *a, int *lda, int *jpvt, c *tau, s *vn1, s *vn2, c *work) nogil:
 *     _fortran_claqp2(m, n, offset, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, vn1, vn2, <npy_complex64*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqp2,CLAQP2)(__pyx_v_m, __pyx_v_n, __pyx_v_offset, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_jpvt, ((npy_complex64 *)__pyx_v_tau), __pyx_v_vn1, __pyx_v_vn2, ((npy_complex64 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":2571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqp2 "F_FUNC(claqp2,CLAQP2)"(int *m, int *n, int *offset, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, s *vn1, s *vn2, npy_complex64 *work) nogil
 * cdef void claqp2(int *m, int *n, int *offset, c *a, int *lda, int *jpvt, c *tau, s *vn1, s *vn2, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqp2(m, n, offset, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, vn1, vn2, <npy_complex64*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqps "F_FUNC(claqps,CLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, s *vn1, s *vn2, npy_complex64 *auxv, npy_complex64 *f, int *ldf) nogil
 * cdef void claqps(int *m, int *n, int *offset, int *nb, int *kb, c *a, int *lda, int *jpvt, c *tau, s *vn1, s *vn2, c *auxv, c *f, int *ldf) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqps(m, n, offset, nb, kb, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, vn1, vn2, <npy_complex64*>auxv, <npy_complex64*>f, ldf)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqps(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_offset, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vn1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vn2, __pyx_t_float_complex *__pyx_v_auxv, __pyx_t_float_complex *__pyx_v_f, int *__pyx_v_ldf) {

  /* "scipy/linalg/cython_lapack.pyx":2577
 *     void _fortran_claqps "F_FUNC(claqps,CLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, s *vn1, s *vn2, npy_complex64 *auxv, npy_complex64 *f, int *ldf) nogil
 * cdef void claqps(int *m, int *n, int *offset, int *nb, int *kb, c *a, int *lda, int *jpvt, c *tau, s *vn1, s *vn2, c *auxv, c *f, int *ldf) nogil:
 *     _fortran_claqps(m, n, offset, nb, kb, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, vn1, vn2, <npy_complex64*>auxv, <npy_complex64*>f, ldf)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqps,CLAQPS)(__pyx_v_m, __pyx_v_n, __pyx_v_offset, __pyx_v_nb, __pyx_v_kb, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_jpvt, ((npy_complex64 *)__pyx_v_tau), __pyx_v_vn1, __pyx_v_vn2, ((npy_complex64 *)__pyx_v_auxv), ((npy_complex64 *)__pyx_v_f), __pyx_v_ldf);

  /* "scipy/linalg/cython_lapack.pyx":2576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqps "F_FUNC(claqps,CLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, npy_complex64 *a, int *lda, int *jpvt, npy_complex64 *tau, s *vn1, s *vn2, npy_complex64 *auxv, npy_complex64 *f, int *ldf) nogil
 * cdef void claqps(int *m, int *n, int *offset, int *nb, int *kb, c *a, int *lda, int *jpvt, c *tau, s *vn1, s *vn2, c *auxv, c *f, int *ldf) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqps(m, n, offset, nb, kb, <npy_complex64*>a, lda, jpvt, <npy_complex64*>tau, vn1, vn2, <npy_complex64*>auxv, <npy_complex64*>f, ldf)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr0 "F_FUNC(claqr0,CLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void claqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr0(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqr0(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_float_complex *__pyx_v_w, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2582
 *     void _fortran_claqr0 "F_FUNC(claqr0,CLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void claqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, c *work, int *lwork, int *info) nogil:
 *     _fortran_claqr0(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqr0,CLAQR0)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex64 *)__pyx_v_w), __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr0 "F_FUNC(claqr0,CLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void claqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr0(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr1 "F_FUNC(claqr1,CLAQR1)"(int *n, npy_complex64 *h, int *ldh, npy_complex64 *s1, npy_complex64 *s2, npy_complex64 *v) nogil
 * cdef void claqr1(int *n, c *h, int *ldh, c *s1, c *s2, c *v) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr1(n, <npy_complex64*>h, ldh, <npy_complex64*>s1, <npy_complex64*>s2, <npy_complex64*>v)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqr1(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_float_complex *__pyx_v_s1, __pyx_t_float_complex *__pyx_v_s2, __pyx_t_float_complex *__pyx_v_v) {

  /* "scipy/linalg/cython_lapack.pyx":2587
 *     void _fortran_claqr1 "F_FUNC(claqr1,CLAQR1)"(int *n, npy_complex64 *h, int *ldh, npy_complex64 *s1, npy_complex64 *s2, npy_complex64 *v) nogil
 * cdef void claqr1(int *n, c *h, int *ldh, c *s1, c *s2, c *v) nogil:
 *     _fortran_claqr1(n, <npy_complex64*>h, ldh, <npy_complex64*>s1, <npy_complex64*>s2, <npy_complex64*>v)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqr1,CLAQR1)(__pyx_v_n, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex64 *)__pyx_v_s1), ((npy_complex64 *)__pyx_v_s2), ((npy_complex64 *)__pyx_v_v));

  /* "scipy/linalg/cython_lapack.pyx":2586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr1 "F_FUNC(claqr1,CLAQR1)"(int *n, npy_complex64 *h, int *ldh, npy_complex64 *s1, npy_complex64 *s2, npy_complex64 *v) nogil
 * cdef void claqr1(int *n, c *h, int *ldh, c *s1, c *s2, c *v) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr1(n, <npy_complex64*>h, ldh, <npy_complex64*>s1, <npy_complex64*>s2, <npy_complex64*>v)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr2 "F_FUNC(claqr2,CLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *ns, int *nd, npy_complex64 *sh, npy_complex64 *v, int *ldv, int *nh, npy_complex64 *t, int *ldt, int *nv, npy_complex64 *wv, int *ldwv, npy_complex64 *work, int *lwork) nogil
 * cdef void claqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, int *ns, int *nd, c *sh, c *v, int *ldv, int *nh, c *t, int *ldt, int *nv, c *wv, int *ldwv, c *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr2(wantt, wantz, n, ktop, kbot, nw, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, ns, nd, <npy_complex64*>sh, <npy_complex64*>v, ldv, nh, <npy_complex64*>t, ldt, nv, <npy_complex64*>wv, ldwv, <npy_complex64*>work, lwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqr2(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nw, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ns, int *__pyx_v_nd, __pyx_t_float_complex *__pyx_v_sh, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_nh, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, int *__pyx_v_nv, __pyx_t_float_complex *__pyx_v_wv, int *__pyx_v_ldwv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork) {

  /* "scipy/linalg/cython_lapack.pyx":2592
 *     void _fortran_claqr2 "F_FUNC(claqr2,CLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *ns, int *nd, npy_complex64 *sh, npy_complex64 *v, int *ldv, int *nh, npy_complex64 *t, int *ldt, int *nv, npy_complex64 *wv, int *ldwv, npy_complex64 *work, int *lwork) nogil
 * cdef void claqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, int *ns, int *nd, c *sh, c *v, int *ldv, int *nh, c *t, int *ldt, int *nv, c *wv, int *ldwv, c *work, int *lwork) nogil:
 *     _fortran_claqr2(wantt, wantz, n, ktop, kbot, nw, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, ns, nd, <npy_complex64*>sh, <npy_complex64*>v, ldv, nh, <npy_complex64*>t, ldt, nv, <npy_complex64*>wv, ldwv, <npy_complex64*>work, lwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqr2,CLAQR2)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nw, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_ns, __pyx_v_nd, ((npy_complex64 *)__pyx_v_sh), ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, __pyx_v_nh, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, __pyx_v_nv, ((npy_complex64 *)__pyx_v_wv), __pyx_v_ldwv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork);

  /* "scipy/linalg/cython_lapack.pyx":2591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr2 "F_FUNC(claqr2,CLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *ns, int *nd, npy_complex64 *sh, npy_complex64 *v, int *ldv, int *nh, npy_complex64 *t, int *ldt, int *nv, npy_complex64 *wv, int *ldwv, npy_complex64 *work, int *lwork) nogil
 * cdef void claqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, int *ns, int *nd, c *sh, c *v, int *ldv, int *nh, c *t, int *ldt, int *nv, c *wv, int *ldwv, c *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr2(wantt, wantz, n, ktop, kbot, nw, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, ns, nd, <npy_complex64*>sh, <npy_complex64*>v, ldv, nh, <npy_complex64*>t, ldt, nv, <npy_complex64*>wv, ldwv, <npy_complex64*>work, lwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr3 "F_FUNC(claqr3,CLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *ns, int *nd, npy_complex64 *sh, npy_complex64 *v, int *ldv, int *nh, npy_complex64 *t, int *ldt, int *nv, npy_complex64 *wv, int *ldwv, npy_complex64 *work, int *lwork) nogil
 * cdef void claqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, int *ns, int *nd, c *sh, c *v, int *ldv, int *nh, c *t, int *ldt, int *nv, c *wv, int *ldwv, c *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr3(wantt, wantz, n, ktop, kbot, nw, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, ns, nd, <npy_complex64*>sh, <npy_complex64*>v, ldv, nh, <npy_complex64*>t, ldt, nv, <npy_complex64*>wv, ldwv, <npy_complex64*>work, lwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqr3(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nw, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ns, int *__pyx_v_nd, __pyx_t_float_complex *__pyx_v_sh, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_nh, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, int *__pyx_v_nv, __pyx_t_float_complex *__pyx_v_wv, int *__pyx_v_ldwv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork) {

  /* "scipy/linalg/cython_lapack.pyx":2597
 *     void _fortran_claqr3 "F_FUNC(claqr3,CLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *ns, int *nd, npy_complex64 *sh, npy_complex64 *v, int *ldv, int *nh, npy_complex64 *t, int *ldt, int *nv, npy_complex64 *wv, int *ldwv, npy_complex64 *work, int *lwork) nogil
 * cdef void claqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, int *ns, int *nd, c *sh, c *v, int *ldv, int *nh, c *t, int *ldt, int *nv, c *wv, int *ldwv, c *work, int *lwork) nogil:
 *     _fortran_claqr3(wantt, wantz, n, ktop, kbot, nw, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, ns, nd, <npy_complex64*>sh, <npy_complex64*>v, ldv, nh, <npy_complex64*>t, ldt, nv, <npy_complex64*>wv, ldwv, <npy_complex64*>work, lwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqr3,CLAQR3)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nw, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_ns, __pyx_v_nd, ((npy_complex64 *)__pyx_v_sh), ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, __pyx_v_nh, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, __pyx_v_nv, ((npy_complex64 *)__pyx_v_wv), __pyx_v_ldwv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork);

  /* "scipy/linalg/cython_lapack.pyx":2596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr3 "F_FUNC(claqr3,CLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, int *ns, int *nd, npy_complex64 *sh, npy_complex64 *v, int *ldv, int *nh, npy_complex64 *t, int *ldt, int *nv, npy_complex64 *wv, int *ldwv, npy_complex64 *work, int *lwork) nogil
 * cdef void claqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, int *ns, int *nd, c *sh, c *v, int *ldv, int *nh, c *t, int *ldt, int *nv, c *wv, int *ldwv, c *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr3(wantt, wantz, n, ktop, kbot, nw, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, ns, nd, <npy_complex64*>sh, <npy_complex64*>v, ldv, nh, <npy_complex64*>t, ldt, nv, <npy_complex64*>wv, ldwv, <npy_complex64*>work, lwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr4 "F_FUNC(claqr4,CLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void claqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr4(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqr4(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_float_complex *__pyx_v_w, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2602
 *     void _fortran_claqr4 "F_FUNC(claqr4,CLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void claqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, c *work, int *lwork, int *info) nogil:
 *     _fortran_claqr4(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqr4,CLAQR4)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex64 *)__pyx_v_w), __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr4 "F_FUNC(claqr4,CLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex64 *h, int *ldh, npy_complex64 *w, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void claqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, c *h, int *ldh, c *w, int *iloz, int *ihiz, c *z, int *ldz, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr4(wantt, wantz, n, ilo, ihi, <npy_complex64*>h, ldh, <npy_complex64*>w, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr5 "F_FUNC(claqr5,CLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, npy_complex64 *s, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *v, int *ldv, npy_complex64 *u, int *ldu, int *nv, npy_complex64 *wv, int *ldwv, int *nh, npy_complex64 *wh, int *ldwh) nogil
 * cdef void claqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, c *s, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, c *v, int *ldv, c *u, int *ldu, int *nv, c *wv, int *ldwv, int *nh, c *wh, int *ldwh) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, <npy_complex64*>s, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>v, ldv, <npy_complex64*>u, ldu, nv, <npy_complex64*>wv, ldwv, nh, <npy_complex64*>wh, ldwh)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqr5(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_kacc22, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nshfts, __pyx_t_float_complex *__pyx_v_s, __pyx_t_float_complex *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_ldu, int *__pyx_v_nv, __pyx_t_float_complex *__pyx_v_wv, int *__pyx_v_ldwv, int *__pyx_v_nh, __pyx_t_float_complex *__pyx_v_wh, int *__pyx_v_ldwh) {

  /* "scipy/linalg/cython_lapack.pyx":2607
 *     void _fortran_claqr5 "F_FUNC(claqr5,CLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, npy_complex64 *s, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *v, int *ldv, npy_complex64 *u, int *ldu, int *nv, npy_complex64 *wv, int *ldwv, int *nh, npy_complex64 *wh, int *ldwh) nogil
 * cdef void claqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, c *s, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, c *v, int *ldv, c *u, int *ldu, int *nv, c *wv, int *ldwv, int *nh, c *wh, int *ldwh) nogil:
 *     _fortran_claqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, <npy_complex64*>s, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>v, ldv, <npy_complex64*>u, ldu, nv, <npy_complex64*>wv, ldwv, nh, <npy_complex64*>wh, ldwh)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqr5,CLAQR5)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_kacc22, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nshfts, ((npy_complex64 *)__pyx_v_s), ((npy_complex64 *)__pyx_v_h), __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex64 *)__pyx_v_u), __pyx_v_ldu, __pyx_v_nv, ((npy_complex64 *)__pyx_v_wv), __pyx_v_ldwv, __pyx_v_nh, ((npy_complex64 *)__pyx_v_wh), __pyx_v_ldwh);

  /* "scipy/linalg/cython_lapack.pyx":2606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqr5 "F_FUNC(claqr5,CLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, npy_complex64 *s, npy_complex64 *h, int *ldh, int *iloz, int *ihiz, npy_complex64 *z, int *ldz, npy_complex64 *v, int *ldv, npy_complex64 *u, int *ldu, int *nv, npy_complex64 *wv, int *ldwv, int *nh, npy_complex64 *wh, int *ldwh) nogil
 * cdef void claqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, c *s, c *h, int *ldh, int *iloz, int *ihiz, c *z, int *ldz, c *v, int *ldv, c *u, int *ldu, int *nv, c *wv, int *ldwv, int *nh, c *wh, int *ldwh) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, <npy_complex64*>s, <npy_complex64*>h, ldh, iloz, ihiz, <npy_complex64*>z, ldz, <npy_complex64*>v, ldv, <npy_complex64*>u, ldu, nv, <npy_complex64*>wv, ldwv, nh, <npy_complex64*>wh, ldwh)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqsb "F_FUNC(claqsb,CLAQSB)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsb(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqsb(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqsb(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":2612
 *     void _fortran_claqsb "F_FUNC(claqsb,CLAQSB)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsb(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_claqsb(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqsb,CLAQSB)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":2611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqsb "F_FUNC(claqsb,CLAQSB)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsb(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqsb(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqsp "F_FUNC(claqsp,CLAQSP)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsp(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqsp(uplo, n, <npy_complex64*>ap, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqsp(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":2617
 *     void _fortran_claqsp "F_FUNC(claqsp,CLAQSP)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsp(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_claqsp(uplo, n, <npy_complex64*>ap, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqsp,CLAQSP)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":2616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqsp "F_FUNC(claqsp,CLAQSP)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsp(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqsp(uplo, n, <npy_complex64*>ap, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqsy "F_FUNC(claqsy,CLAQSY)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsy(char *uplo, int *n, c *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqsy(uplo, n, <npy_complex64*>a, lda, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claqsy(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":2622
 *     void _fortran_claqsy "F_FUNC(claqsy,CLAQSY)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsy(char *uplo, int *n, c *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_claqsy(uplo, n, <npy_complex64*>a, lda, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claqsy,CLAQSY)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":2621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claqsy "F_FUNC(claqsy,CLAQSY)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void claqsy(char *uplo, int *n, c *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claqsy(uplo, n, <npy_complex64*>a, lda, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clar1v "F_FUNC(clar1v,CLAR1V)"(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, npy_complex64 *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil
 * cdef void clar1v(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, c *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, <npy_complex64*>z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clar1v(int *__pyx_v_n, int *__pyx_v_b1, int *__pyx_v_bn, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lambda_, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ld, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lld, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gaptol, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_wantnc, int *__pyx_v_negcnt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ztz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_mingma, int *__pyx_v_r, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_nrminv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_resid, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rqcorr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":2627
 *     void _fortran_clar1v "F_FUNC(clar1v,CLAR1V)"(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, npy_complex64 *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil
 * cdef void clar1v(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, c *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil:
 *     _fortran_clar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, <npy_complex64*>z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clar1v,CLAR1V)(__pyx_v_n, __pyx_v_b1, __pyx_v_bn, __pyx_v_lambda_, __pyx_v_d, __pyx_v_l, __pyx_v_ld, __pyx_v_lld, __pyx_v_pivmin, __pyx_v_gaptol, ((npy_complex64 *)__pyx_v_z), __pyx_v_wantnc, __pyx_v_negcnt, __pyx_v_ztz, __pyx_v_mingma, __pyx_v_r, __pyx_v_isuppz, __pyx_v_nrminv, __pyx_v_resid, __pyx_v_rqcorr, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":2626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clar1v "F_FUNC(clar1v,CLAR1V)"(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, npy_complex64 *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil
 * cdef void clar1v(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, c *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, <npy_complex64*>z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clar2v "F_FUNC(clar2v,CLAR2V)"(int *n, npy_complex64 *x, npy_complex64 *y, npy_complex64 *z, int *incx, s *c, npy_complex64 *s, int *incc) nogil
 * cdef void clar2v(int *n, c *x, c *y, c *z, int *incx, s *c, c *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clar2v(n, <npy_complex64*>x, <npy_complex64*>y, <npy_complex64*>z, incx, c, <npy_complex64*>s, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clar2v(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_x, __pyx_t_float_complex *__pyx_v_y, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_float_complex *__pyx_v_s, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":2632
 *     void _fortran_clar2v "F_FUNC(clar2v,CLAR2V)"(int *n, npy_complex64 *x, npy_complex64 *y, npy_complex64 *z, int *incx, s *c, npy_complex64 *s, int *incc) nogil
 * cdef void clar2v(int *n, c *x, c *y, c *z, int *incx, s *c, c *s, int *incc) nogil:
 *     _fortran_clar2v(n, <npy_complex64*>x, <npy_complex64*>y, <npy_complex64*>z, incx, c, <npy_complex64*>s, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clar2v,CLAR2V)(__pyx_v_n, ((npy_complex64 *)__pyx_v_x), ((npy_complex64 *)__pyx_v_y), ((npy_complex64 *)__pyx_v_z), __pyx_v_incx, __pyx_v_c, ((npy_complex64 *)__pyx_v_s), __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":2631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clar2v "F_FUNC(clar2v,CLAR2V)"(int *n, npy_complex64 *x, npy_complex64 *y, npy_complex64 *z, int *incx, s *c, npy_complex64 *s, int *incc) nogil
 * cdef void clar2v(int *n, c *x, c *y, c *z, int *incx, s *c, c *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clar2v(n, <npy_complex64*>x, <npy_complex64*>y, <npy_complex64*>z, incx, c, <npy_complex64*>s, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarcm "F_FUNC(clarcm,CLARCM)"(int *m, int *n, s *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, s *rwork) nogil
 * cdef void clarcm(int *m, int *n, s *a, int *lda, c *b, int *ldb, c *c, int *ldc, s *rwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarcm(m, n, a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, rwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarcm(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork) {

  /* "scipy/linalg/cython_lapack.pyx":2637
 *     void _fortran_clarcm "F_FUNC(clarcm,CLARCM)"(int *m, int *n, s *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, s *rwork) nogil
 * cdef void clarcm(int *m, int *n, s *a, int *lda, c *b, int *ldb, c *c, int *ldc, s *rwork) nogil:
 *     _fortran_clarcm(m, n, a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, rwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarcm,CLARCM)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, __pyx_v_rwork);

  /* "scipy/linalg/cython_lapack.pyx":2636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarcm "F_FUNC(clarcm,CLARCM)"(int *m, int *n, s *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, s *rwork) nogil
 * cdef void clarcm(int *m, int *n, s *a, int *lda, c *b, int *ldb, c *c, int *ldc, s *rwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarcm(m, n, a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, rwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarf "F_FUNC(clarf,CLARF)"(char *side, int *m, int *n, npy_complex64 *v, int *incv, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarf(char *side, int *m, int *n, c *v, int *incv, c *tau, c *c, int *ldc, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarf(side, m, n, <npy_complex64*>v, incv, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarf(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_incv, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":2642
 *     void _fortran_clarf "F_FUNC(clarf,CLARF)"(char *side, int *m, int *n, npy_complex64 *v, int *incv, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarf(char *side, int *m, int *n, c *v, int *incv, c *tau, c *c, int *ldc, c *work) nogil:
 *     _fortran_clarf(side, m, n, <npy_complex64*>v, incv, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarf,CLARF)(__pyx_v_side, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_v), __pyx_v_incv, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":2641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarf "F_FUNC(clarf,CLARF)"(char *side, int *m, int *n, npy_complex64 *v, int *incv, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarf(char *side, int *m, int *n, c *v, int *incv, c *tau, c *c, int *ldc, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarf(side, m, n, <npy_complex64*>v, incv, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarfb "F_FUNC(clarfb,CLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *t, int *ldt, npy_complex64 *c, int *ldc, npy_complex64 *work, int *ldwork) nogil
 * cdef void clarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, c *v, int *ldv, c *t, int *ldt, c *c, int *ldc, c *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarfb(side, trans, direct, storev, m, n, k, <npy_complex64*>v, ldv, <npy_complex64*>t, ldt, <npy_complex64*>c, ldc, <npy_complex64*>work, ldwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarfb(char *__pyx_v_side, char *__pyx_v_trans, char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_ldwork) {

  /* "scipy/linalg/cython_lapack.pyx":2647
 *     void _fortran_clarfb "F_FUNC(clarfb,CLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *t, int *ldt, npy_complex64 *c, int *ldc, npy_complex64 *work, int *ldwork) nogil
 * cdef void clarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, c *v, int *ldv, c *t, int *ldt, c *c, int *ldc, c *work, int *ldwork) nogil:
 *     _fortran_clarfb(side, trans, direct, storev, m, n, k, <npy_complex64*>v, ldv, <npy_complex64*>t, ldt, <npy_complex64*>c, ldc, <npy_complex64*>work, ldwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarfb,CLARFB)(__pyx_v_side, __pyx_v_trans, __pyx_v_direct, __pyx_v_storev, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_ldwork);

  /* "scipy/linalg/cython_lapack.pyx":2646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarfb "F_FUNC(clarfb,CLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *t, int *ldt, npy_complex64 *c, int *ldc, npy_complex64 *work, int *ldwork) nogil
 * cdef void clarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, c *v, int *ldv, c *t, int *ldt, c *c, int *ldc, c *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarfb(side, trans, direct, storev, m, n, k, <npy_complex64*>v, ldv, <npy_complex64*>t, ldt, <npy_complex64*>c, ldc, <npy_complex64*>work, ldwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarfg "F_FUNC(clarfg,CLARFG)"(int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *tau) nogil
 * cdef void clarfg(int *n, c *alpha, c *x, int *incx, c *tau) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarfg(n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>tau)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarfg(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_tau) {

  /* "scipy/linalg/cython_lapack.pyx":2652
 *     void _fortran_clarfg "F_FUNC(clarfg,CLARFG)"(int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *tau) nogil
 * cdef void clarfg(int *n, c *alpha, c *x, int *incx, c *tau) nogil:
 *     _fortran_clarfg(n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>tau)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarfg,CLARFG)(__pyx_v_n, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, ((npy_complex64 *)__pyx_v_tau));

  /* "scipy/linalg/cython_lapack.pyx":2651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarfg "F_FUNC(clarfg,CLARFG)"(int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *tau) nogil
 * cdef void clarfg(int *n, c *alpha, c *x, int *incx, c *tau) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarfg(n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>tau)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarft "F_FUNC(clarft,CLARFT)"(char *direct, char *storev, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *tau, npy_complex64 *t, int *ldt) nogil
 * cdef void clarft(char *direct, char *storev, int *n, int *k, c *v, int *ldv, c *tau, c *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarft(direct, storev, n, k, <npy_complex64*>v, ldv, <npy_complex64*>tau, <npy_complex64*>t, ldt)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarft(char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt) {

  /* "scipy/linalg/cython_lapack.pyx":2657
 *     void _fortran_clarft "F_FUNC(clarft,CLARFT)"(char *direct, char *storev, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *tau, npy_complex64 *t, int *ldt) nogil
 * cdef void clarft(char *direct, char *storev, int *n, int *k, c *v, int *ldv, c *tau, c *t, int *ldt) nogil:
 *     _fortran_clarft(direct, storev, n, k, <npy_complex64*>v, ldv, <npy_complex64*>tau, <npy_complex64*>t, ldt)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarft,CLARFT)(__pyx_v_direct, __pyx_v_storev, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt);

  /* "scipy/linalg/cython_lapack.pyx":2656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarft "F_FUNC(clarft,CLARFT)"(char *direct, char *storev, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *tau, npy_complex64 *t, int *ldt) nogil
 * cdef void clarft(char *direct, char *storev, int *n, int *k, c *v, int *ldv, c *tau, c *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarft(direct, storev, n, k, <npy_complex64*>v, ldv, <npy_complex64*>tau, <npy_complex64*>t, ldt)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarfx "F_FUNC(clarfx,CLARFX)"(char *side, int *m, int *n, npy_complex64 *v, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarfx(char *side, int *m, int *n, c *v, c *tau, c *c, int *ldc, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarfx(side, m, n, <npy_complex64*>v, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarfx(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_v, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":2662
 *     void _fortran_clarfx "F_FUNC(clarfx,CLARFX)"(char *side, int *m, int *n, npy_complex64 *v, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarfx(char *side, int *m, int *n, c *v, c *tau, c *c, int *ldc, c *work) nogil:
 *     _fortran_clarfx(side, m, n, <npy_complex64*>v, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarfx,CLARFX)(__pyx_v_side, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_v), ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":2661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarfx "F_FUNC(clarfx,CLARFX)"(char *side, int *m, int *n, npy_complex64 *v, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarfx(char *side, int *m, int *n, c *v, c *tau, c *c, int *ldc, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarfx(side, m, n, <npy_complex64*>v, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clargv "F_FUNC(clargv,CLARGV)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *c, int *incc) nogil
 * cdef void clargv(int *n, c *x, int *incx, c *y, int *incy, s *c, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clargv(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, c, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clargv(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":2667
 *     void _fortran_clargv "F_FUNC(clargv,CLARGV)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *c, int *incc) nogil
 * cdef void clargv(int *n, c *x, int *incx, c *y, int *incy, s *c, int *incc) nogil:
 *     _fortran_clargv(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, c, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clargv,CLARGV)(__pyx_v_n, ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, ((npy_complex64 *)__pyx_v_y), __pyx_v_incy, __pyx_v_c, __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":2666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clargv "F_FUNC(clargv,CLARGV)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *c, int *incc) nogil
 * cdef void clargv(int *n, c *x, int *incx, c *y, int *incy, s *c, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clargv(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, c, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarnv "F_FUNC(clarnv,CLARNV)"(int *idist, int *iseed, int *n, npy_complex64 *x) nogil
 * cdef void clarnv(int *idist, int *iseed, int *n, c *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarnv(idist, iseed, n, <npy_complex64*>x)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarnv(int *__pyx_v_idist, int *__pyx_v_iseed, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_x) {

  /* "scipy/linalg/cython_lapack.pyx":2672
 *     void _fortran_clarnv "F_FUNC(clarnv,CLARNV)"(int *idist, int *iseed, int *n, npy_complex64 *x) nogil
 * cdef void clarnv(int *idist, int *iseed, int *n, c *x) nogil:
 *     _fortran_clarnv(idist, iseed, n, <npy_complex64*>x)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarnv,CLARNV)(__pyx_v_idist, __pyx_v_iseed, __pyx_v_n, ((npy_complex64 *)__pyx_v_x));

  /* "scipy/linalg/cython_lapack.pyx":2671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarnv "F_FUNC(clarnv,CLARNV)"(int *idist, int *iseed, int *n, npy_complex64 *x) nogil
 * cdef void clarnv(int *idist, int *iseed, int *n, c *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarnv(idist, iseed, n, <npy_complex64*>x)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarrv "F_FUNC(clarrv,CLARRV)"(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, npy_complex64 *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil
 * cdef void clarrv(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, c *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, <npy_complex64*>z, ldz, isuppz, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarrv(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, int *__pyx_v_isplit, int *__pyx_v_m, int *__pyx_v_dol, int *__pyx_v_dou, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_minrgp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wgap, int *__pyx_v_iblock, int *__pyx_v_indexw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gers, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2677
 *     void _fortran_clarrv "F_FUNC(clarrv,CLARRV)"(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, npy_complex64 *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil
 * cdef void clarrv(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, c *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil:
 *     _fortran_clarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, <npy_complex64*>z, ldz, isuppz, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarrv,CLARRV)(__pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_d, __pyx_v_l, __pyx_v_pivmin, __pyx_v_isplit, __pyx_v_m, __pyx_v_dol, __pyx_v_dou, __pyx_v_minrgp, __pyx_v_rtol1, __pyx_v_rtol2, __pyx_v_w, __pyx_v_werr, __pyx_v_wgap, __pyx_v_iblock, __pyx_v_indexw, __pyx_v_gers, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarrv "F_FUNC(clarrv,CLARRV)"(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, npy_complex64 *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil
 * cdef void clarrv(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, c *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, <npy_complex64*>z, ldz, isuppz, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clartg "F_FUNC(clartg,CLARTG)"(npy_complex64 *f, npy_complex64 *g, s *cs, npy_complex64 *sn, npy_complex64 *r) nogil
 * cdef void clartg(c *f, c *g, s *cs, c *sn, c *r) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clartg(<npy_complex64*>f, <npy_complex64*>g, cs, <npy_complex64*>sn, <npy_complex64*>r)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clartg(__pyx_t_float_complex *__pyx_v_f, __pyx_t_float_complex *__pyx_v_g, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cs, __pyx_t_float_complex *__pyx_v_sn, __pyx_t_float_complex *__pyx_v_r) {

  /* "scipy/linalg/cython_lapack.pyx":2682
 *     void _fortran_clartg "F_FUNC(clartg,CLARTG)"(npy_complex64 *f, npy_complex64 *g, s *cs, npy_complex64 *sn, npy_complex64 *r) nogil
 * cdef void clartg(c *f, c *g, s *cs, c *sn, c *r) nogil:
 *     _fortran_clartg(<npy_complex64*>f, <npy_complex64*>g, cs, <npy_complex64*>sn, <npy_complex64*>r)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clartg,CLARTG)(((npy_complex64 *)__pyx_v_f), ((npy_complex64 *)__pyx_v_g), __pyx_v_cs, ((npy_complex64 *)__pyx_v_sn), ((npy_complex64 *)__pyx_v_r));

  /* "scipy/linalg/cython_lapack.pyx":2681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clartg "F_FUNC(clartg,CLARTG)"(npy_complex64 *f, npy_complex64 *g, s *cs, npy_complex64 *sn, npy_complex64 *r) nogil
 * cdef void clartg(c *f, c *g, s *cs, c *sn, c *r) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clartg(<npy_complex64*>f, <npy_complex64*>g, cs, <npy_complex64*>sn, <npy_complex64*>r)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clartv "F_FUNC(clartv,CLARTV)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *c, npy_complex64 *s, int *incc) nogil
 * cdef void clartv(int *n, c *x, int *incx, c *y, int *incy, s *c, c *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clartv(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, c, <npy_complex64*>s, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clartv(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_float_complex *__pyx_v_s, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":2687
 *     void _fortran_clartv "F_FUNC(clartv,CLARTV)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *c, npy_complex64 *s, int *incc) nogil
 * cdef void clartv(int *n, c *x, int *incx, c *y, int *incy, s *c, c *s, int *incc) nogil:
 *     _fortran_clartv(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, c, <npy_complex64*>s, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clartv,CLARTV)(__pyx_v_n, ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, ((npy_complex64 *)__pyx_v_y), __pyx_v_incy, __pyx_v_c, ((npy_complex64 *)__pyx_v_s), __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":2686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clartv "F_FUNC(clartv,CLARTV)"(int *n, npy_complex64 *x, int *incx, npy_complex64 *y, int *incy, s *c, npy_complex64 *s, int *incc) nogil
 * cdef void clartv(int *n, c *x, int *incx, c *y, int *incy, s *c, c *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clartv(n, <npy_complex64*>x, incx, <npy_complex64*>y, incy, c, <npy_complex64*>s, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarz "F_FUNC(clarz,CLARZ)"(char *side, int *m, int *n, int *l, npy_complex64 *v, int *incv, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarz(char *side, int *m, int *n, int *l, c *v, int *incv, c *tau, c *c, int *ldc, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarz(side, m, n, l, <npy_complex64*>v, incv, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarz(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_l, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_incv, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":2692
 *     void _fortran_clarz "F_FUNC(clarz,CLARZ)"(char *side, int *m, int *n, int *l, npy_complex64 *v, int *incv, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarz(char *side, int *m, int *n, int *l, c *v, int *incv, c *tau, c *c, int *ldc, c *work) nogil:
 *     _fortran_clarz(side, m, n, l, <npy_complex64*>v, incv, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarz,CLARZ)(__pyx_v_side, __pyx_v_m, __pyx_v_n, __pyx_v_l, ((npy_complex64 *)__pyx_v_v), __pyx_v_incv, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":2691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarz "F_FUNC(clarz,CLARZ)"(char *side, int *m, int *n, int *l, npy_complex64 *v, int *incv, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work) nogil
 * cdef void clarz(char *side, int *m, int *n, int *l, c *v, int *incv, c *tau, c *c, int *ldc, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarz(side, m, n, l, <npy_complex64*>v, incv, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarzb "F_FUNC(clarzb,CLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, npy_complex64 *v, int *ldv, npy_complex64 *t, int *ldt, npy_complex64 *c, int *ldc, npy_complex64 *work, int *ldwork) nogil
 * cdef void clarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, c *v, int *ldv, c *t, int *ldt, c *c, int *ldc, c *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarzb(side, trans, direct, storev, m, n, k, l, <npy_complex64*>v, ldv, <npy_complex64*>t, ldt, <npy_complex64*>c, ldc, <npy_complex64*>work, ldwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarzb(char *__pyx_v_side, char *__pyx_v_trans, char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_ldwork) {

  /* "scipy/linalg/cython_lapack.pyx":2697
 *     void _fortran_clarzb "F_FUNC(clarzb,CLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, npy_complex64 *v, int *ldv, npy_complex64 *t, int *ldt, npy_complex64 *c, int *ldc, npy_complex64 *work, int *ldwork) nogil
 * cdef void clarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, c *v, int *ldv, c *t, int *ldt, c *c, int *ldc, c *work, int *ldwork) nogil:
 *     _fortran_clarzb(side, trans, direct, storev, m, n, k, l, <npy_complex64*>v, ldv, <npy_complex64*>t, ldt, <npy_complex64*>c, ldc, <npy_complex64*>work, ldwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarzb,CLARZB)(__pyx_v_side, __pyx_v_trans, __pyx_v_direct, __pyx_v_storev, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_ldwork);

  /* "scipy/linalg/cython_lapack.pyx":2696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarzb "F_FUNC(clarzb,CLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, npy_complex64 *v, int *ldv, npy_complex64 *t, int *ldt, npy_complex64 *c, int *ldc, npy_complex64 *work, int *ldwork) nogil
 * cdef void clarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, c *v, int *ldv, c *t, int *ldt, c *c, int *ldc, c *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarzb(side, trans, direct, storev, m, n, k, l, <npy_complex64*>v, ldv, <npy_complex64*>t, ldt, <npy_complex64*>c, ldc, <npy_complex64*>work, ldwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarzt "F_FUNC(clarzt,CLARZT)"(char *direct, char *storev, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *tau, npy_complex64 *t, int *ldt) nogil
 * cdef void clarzt(char *direct, char *storev, int *n, int *k, c *v, int *ldv, c *tau, c *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarzt(direct, storev, n, k, <npy_complex64*>v, ldv, <npy_complex64*>tau, <npy_complex64*>t, ldt)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clarzt(char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt) {

  /* "scipy/linalg/cython_lapack.pyx":2702
 *     void _fortran_clarzt "F_FUNC(clarzt,CLARZT)"(char *direct, char *storev, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *tau, npy_complex64 *t, int *ldt) nogil
 * cdef void clarzt(char *direct, char *storev, int *n, int *k, c *v, int *ldv, c *tau, c *t, int *ldt) nogil:
 *     _fortran_clarzt(direct, storev, n, k, <npy_complex64*>v, ldv, <npy_complex64*>tau, <npy_complex64*>t, ldt)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clarzt,CLARZT)(__pyx_v_direct, __pyx_v_storev, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt);

  /* "scipy/linalg/cython_lapack.pyx":2701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clarzt "F_FUNC(clarzt,CLARZT)"(char *direct, char *storev, int *n, int *k, npy_complex64 *v, int *ldv, npy_complex64 *tau, npy_complex64 *t, int *ldt) nogil
 * cdef void clarzt(char *direct, char *storev, int *n, int *k, c *v, int *ldv, c *tau, c *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clarzt(direct, storev, n, k, <npy_complex64*>v, ldv, <npy_complex64*>tau, <npy_complex64*>t, ldt)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clascl "F_FUNC(clascl,CLASCL)"(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clascl(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clascl(type_bn, kl, ku, cfrom, cto, m, n, <npy_complex64*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clascl(char *__pyx_v_type_bn, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cfrom, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cto, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2707
 *     void _fortran_clascl "F_FUNC(clascl,CLASCL)"(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clascl(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, c *a, int *lda, int *info) nogil:
 *     _fortran_clascl(type_bn, kl, ku, cfrom, cto, m, n, <npy_complex64*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clascl,CLASCL)(__pyx_v_type_bn, __pyx_v_kl, __pyx_v_ku, __pyx_v_cfrom, __pyx_v_cto, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clascl "F_FUNC(clascl,CLASCL)"(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clascl(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clascl(type_bn, kl, ku, cfrom, cto, m, n, <npy_complex64*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claset "F_FUNC(claset,CLASET)"(char *uplo, int *m, int *n, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *a, int *lda) nogil
 * cdef void claset(char *uplo, int *m, int *n, c *alpha, c *beta, c *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claset(uplo, m, n, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claset(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":2712
 *     void _fortran_claset "F_FUNC(claset,CLASET)"(char *uplo, int *m, int *n, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *a, int *lda) nogil
 * cdef void claset(char *uplo, int *m, int *n, c *alpha, c *beta, c *a, int *lda) nogil:
 *     _fortran_claset(uplo, m, n, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claset,CLASET)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_a), __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":2711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claset "F_FUNC(claset,CLASET)"(char *uplo, int *m, int *n, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *a, int *lda) nogil
 * cdef void claset(char *uplo, int *m, int *n, c *alpha, c *beta, c *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claset(uplo, m, n, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clasr "F_FUNC(clasr,CLASR)"(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, npy_complex64 *a, int *lda) nogil
 * cdef void clasr(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, c *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clasr(side, pivot, direct, m, n, c, s, <npy_complex64*>a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clasr(char *__pyx_v_side, char *__pyx_v_pivot, char *__pyx_v_direct, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":2717
 *     void _fortran_clasr "F_FUNC(clasr,CLASR)"(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, npy_complex64 *a, int *lda) nogil
 * cdef void clasr(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, c *a, int *lda) nogil:
 *     _fortran_clasr(side, pivot, direct, m, n, c, s, <npy_complex64*>a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clasr,CLASR)(__pyx_v_side, __pyx_v_pivot, __pyx_v_direct, __pyx_v_m, __pyx_v_n, __pyx_v_c, __pyx_v_s, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":2716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clasr "F_FUNC(clasr,CLASR)"(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, npy_complex64 *a, int *lda) nogil
 * cdef void clasr(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, c *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clasr(side, pivot, direct, m, n, c, s, <npy_complex64*>a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_classq "F_FUNC(classq,CLASSQ)"(int *n, npy_complex64 *x, int *incx, s *scale, s *sumsq) nogil
 * cdef void classq(int *n, c *x, int *incx, s *scale, s *sumsq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_classq(n, <npy_complex64*>x, incx, scale, sumsq)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_classq(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sumsq) {

  /* "scipy/linalg/cython_lapack.pyx":2722
 *     void _fortran_classq "F_FUNC(classq,CLASSQ)"(int *n, npy_complex64 *x, int *incx, s *scale, s *sumsq) nogil
 * cdef void classq(int *n, c *x, int *incx, s *scale, s *sumsq) nogil:
 *     _fortran_classq(n, <npy_complex64*>x, incx, scale, sumsq)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(classq,CLASSQ)(__pyx_v_n, ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, __pyx_v_scale, __pyx_v_sumsq);

  /* "scipy/linalg/cython_lapack.pyx":2721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_classq "F_FUNC(classq,CLASSQ)"(int *n, npy_complex64 *x, int *incx, s *scale, s *sumsq) nogil
 * cdef void classq(int *n, c *x, int *incx, s *scale, s *sumsq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_classq(n, <npy_complex64*>x, incx, scale, sumsq)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claswp "F_FUNC(claswp,CLASWP)"(int *n, npy_complex64 *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void claswp(int *n, c *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claswp(n, <npy_complex64*>a, lda, k1, k2, ipiv, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_claswp(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_k1, int *__pyx_v_k2, int *__pyx_v_ipiv, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":2727
 *     void _fortran_claswp "F_FUNC(claswp,CLASWP)"(int *n, npy_complex64 *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void claswp(int *n, c *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:
 *     _fortran_claswp(n, <npy_complex64*>a, lda, k1, k2, ipiv, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(claswp,CLASWP)(__pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_k1, __pyx_v_k2, __pyx_v_ipiv, __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":2726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_claswp "F_FUNC(claswp,CLASWP)"(int *n, npy_complex64 *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void claswp(int *n, c *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_claswp(n, <npy_complex64*>a, lda, k1, k2, ipiv, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clasyf "F_FUNC(clasyf,CLASYF)"(char *uplo, int *n, int *nb, int *kb, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *w, int *ldw, int *info) nogil
 * cdef void clasyf(char *uplo, int *n, int *nb, int *kb, c *a, int *lda, int *ipiv, c *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clasyf(uplo, n, nb, kb, <npy_complex64*>a, lda, ipiv, <npy_complex64*>w, ldw, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clasyf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_w, int *__pyx_v_ldw, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2732
 *     void _fortran_clasyf "F_FUNC(clasyf,CLASYF)"(char *uplo, int *n, int *nb, int *kb, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *w, int *ldw, int *info) nogil
 * cdef void clasyf(char *uplo, int *n, int *nb, int *kb, c *a, int *lda, int *ipiv, c *w, int *ldw, int *info) nogil:
 *     _fortran_clasyf(uplo, n, nb, kb, <npy_complex64*>a, lda, ipiv, <npy_complex64*>w, ldw, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clasyf,CLASYF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, __pyx_v_kb, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_w), __pyx_v_ldw, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clasyf "F_FUNC(clasyf,CLASYF)"(char *uplo, int *n, int *nb, int *kb, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *w, int *ldw, int *info) nogil
 * cdef void clasyf(char *uplo, int *n, int *nb, int *kb, c *a, int *lda, int *ipiv, c *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clasyf(uplo, n, nb, kb, <npy_complex64*>a, lda, ipiv, <npy_complex64*>w, ldw, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatbs "F_FUNC(clatbs,CLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, npy_complex64 *ab, int *ldab, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, c *ab, int *ldab, c *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatbs(uplo, trans, diag, normin, n, kd, <npy_complex64*>ab, ldab, <npy_complex64*>x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clatbs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2737
 *     void _fortran_clatbs "F_FUNC(clatbs,CLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, npy_complex64 *ab, int *ldab, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, c *ab, int *ldab, c *x, s *scale, s *cnorm, int *info) nogil:
 *     _fortran_clatbs(uplo, trans, diag, normin, n, kd, <npy_complex64*>ab, ldab, <npy_complex64*>x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clatbs,CLATBS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_x), __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatbs "F_FUNC(clatbs,CLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, npy_complex64 *ab, int *ldab, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, c *ab, int *ldab, c *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatbs(uplo, trans, diag, normin, n, kd, <npy_complex64*>ab, ldab, <npy_complex64*>x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatdf "F_FUNC(clatdf,CLATDF)"(int *ijob, int *n, npy_complex64 *z, int *ldz, npy_complex64 *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void clatdf(int *ijob, int *n, c *z, int *ldz, c *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatdf(ijob, n, <npy_complex64*>z, ldz, <npy_complex64*>rhs, rdsum, rdscal, ipiv, jpiv)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clatdf(int *__pyx_v_ijob, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_rhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rdsum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rdscal, int *__pyx_v_ipiv, int *__pyx_v_jpiv) {

  /* "scipy/linalg/cython_lapack.pyx":2742
 *     void _fortran_clatdf "F_FUNC(clatdf,CLATDF)"(int *ijob, int *n, npy_complex64 *z, int *ldz, npy_complex64 *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void clatdf(int *ijob, int *n, c *z, int *ldz, c *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil:
 *     _fortran_clatdf(ijob, n, <npy_complex64*>z, ldz, <npy_complex64*>rhs, rdsum, rdscal, ipiv, jpiv)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clatdf,CLATDF)(__pyx_v_ijob, __pyx_v_n, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_rhs), __pyx_v_rdsum, __pyx_v_rdscal, __pyx_v_ipiv, __pyx_v_jpiv);

  /* "scipy/linalg/cython_lapack.pyx":2741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatdf "F_FUNC(clatdf,CLATDF)"(int *ijob, int *n, npy_complex64 *z, int *ldz, npy_complex64 *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void clatdf(int *ijob, int *n, c *z, int *ldz, c *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatdf(ijob, n, <npy_complex64*>z, ldz, <npy_complex64*>rhs, rdsum, rdscal, ipiv, jpiv)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatps "F_FUNC(clatps,CLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex64 *ap, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatps(char *uplo, char *trans, char *diag, char *normin, int *n, c *ap, c *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatps(uplo, trans, diag, normin, n, <npy_complex64*>ap, <npy_complex64*>x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clatps(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2747
 *     void _fortran_clatps "F_FUNC(clatps,CLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex64 *ap, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatps(char *uplo, char *trans, char *diag, char *normin, int *n, c *ap, c *x, s *scale, s *cnorm, int *info) nogil:
 *     _fortran_clatps(uplo, trans, diag, normin, n, <npy_complex64*>ap, <npy_complex64*>x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clatps,CLATPS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_x), __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatps "F_FUNC(clatps,CLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex64 *ap, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatps(char *uplo, char *trans, char *diag, char *normin, int *n, c *ap, c *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatps(uplo, trans, diag, normin, n, <npy_complex64*>ap, <npy_complex64*>x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatrd "F_FUNC(clatrd,CLATRD)"(char *uplo, int *n, int *nb, npy_complex64 *a, int *lda, s *e, npy_complex64 *tau, npy_complex64 *w, int *ldw) nogil
 * cdef void clatrd(char *uplo, int *n, int *nb, c *a, int *lda, s *e, c *tau, c *w, int *ldw) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatrd(uplo, n, nb, <npy_complex64*>a, lda, e, <npy_complex64*>tau, <npy_complex64*>w, ldw)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clatrd(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_w, int *__pyx_v_ldw) {

  /* "scipy/linalg/cython_lapack.pyx":2752
 *     void _fortran_clatrd "F_FUNC(clatrd,CLATRD)"(char *uplo, int *n, int *nb, npy_complex64 *a, int *lda, s *e, npy_complex64 *tau, npy_complex64 *w, int *ldw) nogil
 * cdef void clatrd(char *uplo, int *n, int *nb, c *a, int *lda, s *e, c *tau, c *w, int *ldw) nogil:
 *     _fortran_clatrd(uplo, n, nb, <npy_complex64*>a, lda, e, <npy_complex64*>tau, <npy_complex64*>w, ldw)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clatrd,CLATRD)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_e, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_w), __pyx_v_ldw);

  /* "scipy/linalg/cython_lapack.pyx":2751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatrd "F_FUNC(clatrd,CLATRD)"(char *uplo, int *n, int *nb, npy_complex64 *a, int *lda, s *e, npy_complex64 *tau, npy_complex64 *w, int *ldw) nogil
 * cdef void clatrd(char *uplo, int *n, int *nb, c *a, int *lda, s *e, c *tau, c *w, int *ldw) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatrd(uplo, n, nb, <npy_complex64*>a, lda, e, <npy_complex64*>tau, <npy_complex64*>w, ldw)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatrs "F_FUNC(clatrs,CLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex64 *a, int *lda, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatrs(char *uplo, char *trans, char *diag, char *normin, int *n, c *a, int *lda, c *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatrs(uplo, trans, diag, normin, n, <npy_complex64*>a, lda, <npy_complex64*>x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clatrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2757
 *     void _fortran_clatrs "F_FUNC(clatrs,CLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex64 *a, int *lda, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatrs(char *uplo, char *trans, char *diag, char *normin, int *n, c *a, int *lda, c *x, s *scale, s *cnorm, int *info) nogil:
 *     _fortran_clatrs(uplo, trans, diag, normin, n, <npy_complex64*>a, lda, <npy_complex64*>x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clatrs,CLATRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_x), __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatrs "F_FUNC(clatrs,CLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex64 *a, int *lda, npy_complex64 *x, s *scale, s *cnorm, int *info) nogil
 * cdef void clatrs(char *uplo, char *trans, char *diag, char *normin, int *n, c *a, int *lda, c *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatrs(uplo, trans, diag, normin, n, <npy_complex64*>a, lda, <npy_complex64*>x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatrz "F_FUNC(clatrz,CLATRZ)"(int *m, int *n, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work) nogil
 * cdef void clatrz(int *m, int *n, int *l, c *a, int *lda, c *tau, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatrz(m, n, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clatrz(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_l, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":2762
 *     void _fortran_clatrz "F_FUNC(clatrz,CLATRZ)"(int *m, int *n, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work) nogil
 * cdef void clatrz(int *m, int *n, int *l, c *a, int *lda, c *tau, c *work) nogil:
 *     _fortran_clatrz(m, n, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clatrz,CLATRZ)(__pyx_v_m, __pyx_v_n, __pyx_v_l, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":2761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clatrz "F_FUNC(clatrz,CLATRZ)"(int *m, int *n, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work) nogil
 * cdef void clatrz(int *m, int *n, int *l, c *a, int *lda, c *tau, c *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clatrz(m, n, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clauu2 "F_FUNC(clauu2,CLAUU2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clauu2(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clauu2(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clauu2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2767
 *     void _fortran_clauu2 "F_FUNC(clauu2,CLAUU2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clauu2(char *uplo, int *n, c *a, int *lda, int *info) nogil:
 *     _fortran_clauu2(uplo, n, <npy_complex64*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clauu2,CLAUU2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clauu2 "F_FUNC(clauu2,CLAUU2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clauu2(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clauu2(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clauum "F_FUNC(clauum,CLAUUM)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clauum(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clauum(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_clauum(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2772
 *     void _fortran_clauum "F_FUNC(clauum,CLAUUM)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clauum(char *uplo, int *n, c *a, int *lda, int *info) nogil:
 *     _fortran_clauum(uplo, n, <npy_complex64*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(clauum,CLAUUM)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_clauum "F_FUNC(clauum,CLAUUM)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void clauum(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_clauum(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbcon "F_FUNC(cpbcon,CPBCON)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbcon(char *uplo, int *n, int *kd, c *ab, int *ldab, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbcon(uplo, n, kd, <npy_complex64*>ab, ldab, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbcon(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2777
 *     void _fortran_cpbcon "F_FUNC(cpbcon,CPBCON)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbcon(char *uplo, int *n, int *kd, c *ab, int *ldab, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:
 *     _fortran_cpbcon(uplo, n, kd, <npy_complex64*>ab, ldab, anorm, rcond, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbcon,CPBCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbcon "F_FUNC(cpbcon,CPBCON)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbcon(char *uplo, int *n, int *kd, c *ab, int *ldab, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbcon(uplo, n, kd, <npy_complex64*>ab, ldab, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbequ "F_FUNC(cpbequ,CPBEQU)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cpbequ(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbequ(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbequ(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2782
 *     void _fortran_cpbequ "F_FUNC(cpbequ,CPBEQU)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cpbequ(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil:
 *     _fortran_cpbequ(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbequ,CPBEQU)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbequ "F_FUNC(cpbequ,CPBEQU)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cpbequ(char *uplo, int *n, int *kd, c *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbequ(uplo, n, kd, <npy_complex64*>ab, ldab, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbrfs "F_FUNC(cpbrfs,CPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbrfs(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbrfs(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_afb, int *__pyx_v_ldafb, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2787
 *     void _fortran_cpbrfs "F_FUNC(cpbrfs,CPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbrfs(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cpbrfs(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbrfs,CPBRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_afb), __pyx_v_ldafb, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbrfs "F_FUNC(cpbrfs,CPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbrfs(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbrfs(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbstf "F_FUNC(cpbstf,CPBSTF)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbstf(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbstf(uplo, n, kd, <npy_complex64*>ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbstf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2792
 *     void _fortran_cpbstf "F_FUNC(cpbstf,CPBSTF)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbstf(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:
 *     _fortran_cpbstf(uplo, n, kd, <npy_complex64*>ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbstf,CPBSTF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbstf "F_FUNC(cpbstf,CPBSTF)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbstf(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbstf(uplo, n, kd, <npy_complex64*>ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbsv "F_FUNC(cpbsv,CPBSV)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpbsv(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbsv(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2797
 *     void _fortran_cpbsv "F_FUNC(cpbsv,CPBSV)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpbsv(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:
 *     _fortran_cpbsv(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbsv,CPBSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbsv "F_FUNC(cpbsv,CPBSV)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpbsv(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbsv(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbsvx "F_FUNC(cpbsvx,CPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbsvx(fact, uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_afb, int *__pyx_v_ldafb, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2802
 *     void _fortran_cpbsvx "F_FUNC(cpbsvx,CPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cpbsvx(fact, uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbsvx,CPBSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_afb), __pyx_v_ldafb, __pyx_v_equed, __pyx_v_s, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbsvx "F_FUNC(cpbsvx,CPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *afb, int *ldafb, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *afb, int *ldafb, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbsvx(fact, uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>afb, ldafb, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbtf2 "F_FUNC(cpbtf2,CPBTF2)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbtf2(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbtf2(uplo, n, kd, <npy_complex64*>ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbtf2(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2807
 *     void _fortran_cpbtf2 "F_FUNC(cpbtf2,CPBTF2)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbtf2(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:
 *     _fortran_cpbtf2(uplo, n, kd, <npy_complex64*>ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbtf2,CPBTF2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbtf2 "F_FUNC(cpbtf2,CPBTF2)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbtf2(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbtf2(uplo, n, kd, <npy_complex64*>ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbtrf "F_FUNC(cpbtrf,CPBTRF)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbtrf(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbtrf(uplo, n, kd, <npy_complex64*>ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbtrf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2812
 *     void _fortran_cpbtrf "F_FUNC(cpbtrf,CPBTRF)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbtrf(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:
 *     _fortran_cpbtrf(uplo, n, kd, <npy_complex64*>ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbtrf,CPBTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbtrf "F_FUNC(cpbtrf,CPBTRF)"(char *uplo, int *n, int *kd, npy_complex64 *ab, int *ldab, int *info) nogil
 * cdef void cpbtrf(char *uplo, int *n, int *kd, c *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbtrf(uplo, n, kd, <npy_complex64*>ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbtrs "F_FUNC(cpbtrs,CPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpbtrs(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbtrs(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpbtrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2817
 *     void _fortran_cpbtrs "F_FUNC(cpbtrs,CPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpbtrs(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:
 *     _fortran_cpbtrs(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpbtrs,CPBTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpbtrs "F_FUNC(cpbtrs,CPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpbtrs(char *uplo, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpbtrs(uplo, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpocon "F_FUNC(cpocon,CPOCON)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpocon(char *uplo, int *n, c *a, int *lda, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpocon(uplo, n, <npy_complex64*>a, lda, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpocon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2822
 *     void _fortran_cpocon "F_FUNC(cpocon,CPOCON)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpocon(char *uplo, int *n, c *a, int *lda, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:
 *     _fortran_cpocon(uplo, n, <npy_complex64*>a, lda, anorm, rcond, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpocon,CPOCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpocon "F_FUNC(cpocon,CPOCON)"(char *uplo, int *n, npy_complex64 *a, int *lda, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpocon(char *uplo, int *n, c *a, int *lda, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpocon(uplo, n, <npy_complex64*>a, lda, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpoequ "F_FUNC(cpoequ,CPOEQU)"(int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cpoequ(int *n, c *a, int *lda, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpoequ(n, <npy_complex64*>a, lda, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpoequ(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2827
 *     void _fortran_cpoequ "F_FUNC(cpoequ,CPOEQU)"(int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cpoequ(int *n, c *a, int *lda, s *s, s *scond, s *amax, int *info) nogil:
 *     _fortran_cpoequ(n, <npy_complex64*>a, lda, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpoequ,CPOEQU)(__pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpoequ "F_FUNC(cpoequ,CPOEQU)"(int *n, npy_complex64 *a, int *lda, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cpoequ(int *n, c *a, int *lda, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpoequ(n, <npy_complex64*>a, lda, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cporfs "F_FUNC(cporfs,CPORFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cporfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cporfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cporfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_af, int *__pyx_v_ldaf, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2832
 *     void _fortran_cporfs "F_FUNC(cporfs,CPORFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cporfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cporfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cporfs,CPORFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_af), __pyx_v_ldaf, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cporfs "F_FUNC(cporfs,CPORFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cporfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cporfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cposv "F_FUNC(cposv,CPOSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cposv(char *uplo, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cposv(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cposv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2837
 *     void _fortran_cposv "F_FUNC(cposv,CPOSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cposv(char *uplo, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:
 *     _fortran_cposv(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cposv,CPOSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cposv "F_FUNC(cposv,CPOSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cposv(char *uplo, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cposv(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cposvx "F_FUNC(cposvx,CPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cposvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cposvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cposvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_af, int *__pyx_v_ldaf, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2842
 *     void _fortran_cposvx "F_FUNC(cposvx,CPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cposvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cposvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cposvx,CPOSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_equed, __pyx_v_s, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cposvx "F_FUNC(cposvx,CPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cposvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cposvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpotf2 "F_FUNC(cpotf2,CPOTF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotf2(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpotf2(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpotf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2847
 *     void _fortran_cpotf2 "F_FUNC(cpotf2,CPOTF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotf2(char *uplo, int *n, c *a, int *lda, int *info) nogil:
 *     _fortran_cpotf2(uplo, n, <npy_complex64*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpotf2,CPOTF2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpotf2 "F_FUNC(cpotf2,CPOTF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotf2(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpotf2(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpotrf "F_FUNC(cpotrf,CPOTRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotrf(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpotrf(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpotrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2852
 *     void _fortran_cpotrf "F_FUNC(cpotrf,CPOTRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotrf(char *uplo, int *n, c *a, int *lda, int *info) nogil:
 *     _fortran_cpotrf(uplo, n, <npy_complex64*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpotrf,CPOTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpotrf "F_FUNC(cpotrf,CPOTRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotrf(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpotrf(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpotri "F_FUNC(cpotri,CPOTRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotri(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpotri(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpotri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2857
 *     void _fortran_cpotri "F_FUNC(cpotri,CPOTRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotri(char *uplo, int *n, c *a, int *lda, int *info) nogil:
 *     _fortran_cpotri(uplo, n, <npy_complex64*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpotri,CPOTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpotri "F_FUNC(cpotri,CPOTRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void cpotri(char *uplo, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpotri(uplo, n, <npy_complex64*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpotrs "F_FUNC(cpotrs,CPOTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpotrs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpotrs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpotrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2862
 *     void _fortran_cpotrs "F_FUNC(cpotrs,CPOTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpotrs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:
 *     _fortran_cpotrs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpotrs,CPOTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpotrs "F_FUNC(cpotrs,CPOTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpotrs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpotrs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cppcon "F_FUNC(cppcon,CPPCON)"(char *uplo, int *n, npy_complex64 *ap, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cppcon(char *uplo, int *n, c *ap, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cppcon(uplo, n, <npy_complex64*>ap, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cppcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2867
 *     void _fortran_cppcon "F_FUNC(cppcon,CPPCON)"(char *uplo, int *n, npy_complex64 *ap, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cppcon(char *uplo, int *n, c *ap, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:
 *     _fortran_cppcon(uplo, n, <npy_complex64*>ap, anorm, rcond, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cppcon,CPPCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cppcon "F_FUNC(cppcon,CPPCON)"(char *uplo, int *n, npy_complex64 *ap, s *anorm, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cppcon(char *uplo, int *n, c *ap, s *anorm, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cppcon(uplo, n, <npy_complex64*>ap, anorm, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cppequ "F_FUNC(cppequ,CPPEQU)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cppequ(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cppequ(uplo, n, <npy_complex64*>ap, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cppequ(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2872
 *     void _fortran_cppequ "F_FUNC(cppequ,CPPEQU)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cppequ(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, int *info) nogil:
 *     _fortran_cppequ(uplo, n, <npy_complex64*>ap, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cppequ,CPPEQU)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cppequ "F_FUNC(cppequ,CPPEQU)"(char *uplo, int *n, npy_complex64 *ap, s *s, s *scond, s *amax, int *info) nogil
 * cdef void cppequ(char *uplo, int *n, c *ap, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cppequ(uplo, n, <npy_complex64*>ap, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpprfs "F_FUNC(cpprfs,CPPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_afp, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2877
 *     void _fortran_cpprfs "F_FUNC(cpprfs,CPPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cpprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpprfs,CPPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_afp), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpprfs "F_FUNC(cpprfs,CPPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cpprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cppsv "F_FUNC(cppsv,CPPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cppsv(char *uplo, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cppsv(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cppsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2882
 *     void _fortran_cppsv "F_FUNC(cppsv,CPPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cppsv(char *uplo, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:
 *     _fortran_cppsv(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cppsv,CPPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cppsv "F_FUNC(cppsv,CPPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cppsv(char *uplo, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cppsv(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cppsvx "F_FUNC(cppsvx,CPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cppsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cppsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cppsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_afp, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2887
 *     void _fortran_cppsvx "F_FUNC(cppsvx,CPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cppsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cppsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cppsvx,CPPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_afp), __pyx_v_equed, __pyx_v_s, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cppsvx "F_FUNC(cppsvx,CPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, char *equed, s *s, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cppsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, char *equed, s *s, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cppsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, equed, s, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpptrf "F_FUNC(cpptrf,CPPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void cpptrf(char *uplo, int *n, c *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpptrf(uplo, n, <npy_complex64*>ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2892
 *     void _fortran_cpptrf "F_FUNC(cpptrf,CPPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void cpptrf(char *uplo, int *n, c *ap, int *info) nogil:
 *     _fortran_cpptrf(uplo, n, <npy_complex64*>ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpptrf,CPPTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpptrf "F_FUNC(cpptrf,CPPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void cpptrf(char *uplo, int *n, c *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpptrf(uplo, n, <npy_complex64*>ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpptri "F_FUNC(cpptri,CPPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void cpptri(char *uplo, int *n, c *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpptri(uplo, n, <npy_complex64*>ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2897
 *     void _fortran_cpptri "F_FUNC(cpptri,CPPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void cpptri(char *uplo, int *n, c *ap, int *info) nogil:
 *     _fortran_cpptri(uplo, n, <npy_complex64*>ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpptri,CPPTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpptri "F_FUNC(cpptri,CPPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void cpptri(char *uplo, int *n, c *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpptri(uplo, n, <npy_complex64*>ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpptrs "F_FUNC(cpptrs,CPPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpptrs(char *uplo, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpptrs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2902
 *     void _fortran_cpptrs "F_FUNC(cpptrs,CPPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpptrs(char *uplo, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:
 *     _fortran_cpptrs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpptrs,CPPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpptrs "F_FUNC(cpptrs,CPPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpptrs(char *uplo, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpptrs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptcon "F_FUNC(cptcon,CPTCON)"(int *n, s *d, npy_complex64 *e, s *anorm, s *rcond, s *rwork, int *info) nogil
 * cdef void cptcon(int *n, s *d, c *e, s *anorm, s *rcond, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptcon(n, d, <npy_complex64*>e, anorm, rcond, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cptcon(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2907
 *     void _fortran_cptcon "F_FUNC(cptcon,CPTCON)"(int *n, s *d, npy_complex64 *e, s *anorm, s *rcond, s *rwork, int *info) nogil
 * cdef void cptcon(int *n, s *d, c *e, s *anorm, s *rcond, s *rwork, int *info) nogil:
 *     _fortran_cptcon(n, d, <npy_complex64*>e, anorm, rcond, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cptcon,CPTCON)(__pyx_v_n, __pyx_v_d, ((npy_complex64 *)__pyx_v_e), __pyx_v_anorm, __pyx_v_rcond, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptcon "F_FUNC(cptcon,CPTCON)"(int *n, s *d, npy_complex64 *e, s *anorm, s *rcond, s *rwork, int *info) nogil
 * cdef void cptcon(int *n, s *d, c *e, s *anorm, s *rcond, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptcon(n, d, <npy_complex64*>e, anorm, rcond, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpteqr "F_FUNC(cpteqr,CPTEQR)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, s *work, int *info) nogil
 * cdef void cpteqr(char *compz, int *n, s *d, s *e, c *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpteqr(compz, n, d, e, <npy_complex64*>z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpteqr(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2912
 *     void _fortran_cpteqr "F_FUNC(cpteqr,CPTEQR)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, s *work, int *info) nogil
 * cdef void cpteqr(char *compz, int *n, s *d, s *e, c *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_cpteqr(compz, n, d, e, <npy_complex64*>z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpteqr,CPTEQR)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpteqr "F_FUNC(cpteqr,CPTEQR)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, s *work, int *info) nogil
 * cdef void cpteqr(char *compz, int *n, s *d, s *e, c *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpteqr(compz, n, d, e, <npy_complex64*>z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptrfs "F_FUNC(cptrfs,CPTRFS)"(char *uplo, int *n, int *nrhs, s *d, npy_complex64 *e, s *df, npy_complex64 *ef, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cptrfs(char *uplo, int *n, int *nrhs, s *d, c *e, s *df, c *ef, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptrfs(uplo, n, nrhs, d, <npy_complex64*>e, df, <npy_complex64*>ef, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cptrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_df, __pyx_t_float_complex *__pyx_v_ef, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2917
 *     void _fortran_cptrfs "F_FUNC(cptrfs,CPTRFS)"(char *uplo, int *n, int *nrhs, s *d, npy_complex64 *e, s *df, npy_complex64 *ef, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cptrfs(char *uplo, int *n, int *nrhs, s *d, c *e, s *df, c *ef, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cptrfs(uplo, n, nrhs, d, <npy_complex64*>e, df, <npy_complex64*>ef, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cptrfs,CPTRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex64 *)__pyx_v_e), __pyx_v_df, ((npy_complex64 *)__pyx_v_ef), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptrfs "F_FUNC(cptrfs,CPTRFS)"(char *uplo, int *n, int *nrhs, s *d, npy_complex64 *e, s *df, npy_complex64 *ef, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cptrfs(char *uplo, int *n, int *nrhs, s *d, c *e, s *df, c *ef, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptrfs(uplo, n, nrhs, d, <npy_complex64*>e, df, <npy_complex64*>ef, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptsv "F_FUNC(cptsv,CPTSV)"(int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cptsv(int *n, int *nrhs, s *d, c *e, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptsv(n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cptsv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_e, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2922
 *     void _fortran_cptsv "F_FUNC(cptsv,CPTSV)"(int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cptsv(int *n, int *nrhs, s *d, c *e, c *b, int *ldb, int *info) nogil:
 *     _fortran_cptsv(n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cptsv,CPTSV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex64 *)__pyx_v_e), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptsv "F_FUNC(cptsv,CPTSV)"(int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cptsv(int *n, int *nrhs, s *d, c *e, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptsv(n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptsvx "F_FUNC(cptsvx,CPTSVX)"(char *fact, int *n, int *nrhs, s *d, npy_complex64 *e, s *df, npy_complex64 *ef, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cptsvx(char *fact, int *n, int *nrhs, s *d, c *e, s *df, c *ef, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptsvx(fact, n, nrhs, d, <npy_complex64*>e, df, <npy_complex64*>ef, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cptsvx(char *__pyx_v_fact, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_df, __pyx_t_float_complex *__pyx_v_ef, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2927
 *     void _fortran_cptsvx "F_FUNC(cptsvx,CPTSVX)"(char *fact, int *n, int *nrhs, s *d, npy_complex64 *e, s *df, npy_complex64 *ef, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cptsvx(char *fact, int *n, int *nrhs, s *d, c *e, s *df, c *ef, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cptsvx(fact, n, nrhs, d, <npy_complex64*>e, df, <npy_complex64*>ef, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cptsvx,CPTSVX)(__pyx_v_fact, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex64 *)__pyx_v_e), __pyx_v_df, ((npy_complex64 *)__pyx_v_ef), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptsvx "F_FUNC(cptsvx,CPTSVX)"(char *fact, int *n, int *nrhs, s *d, npy_complex64 *e, s *df, npy_complex64 *ef, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cptsvx(char *fact, int *n, int *nrhs, s *d, c *e, s *df, c *ef, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptsvx(fact, n, nrhs, d, <npy_complex64*>e, df, <npy_complex64*>ef, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpttrf "F_FUNC(cpttrf,CPTTRF)"(int *n, s *d, npy_complex64 *e, int *info) nogil
 * cdef void cpttrf(int *n, s *d, c *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpttrf(n, d, <npy_complex64*>e, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpttrf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_e, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2932
 *     void _fortran_cpttrf "F_FUNC(cpttrf,CPTTRF)"(int *n, s *d, npy_complex64 *e, int *info) nogil
 * cdef void cpttrf(int *n, s *d, c *e, int *info) nogil:
 *     _fortran_cpttrf(n, d, <npy_complex64*>e, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpttrf,CPTTRF)(__pyx_v_n, __pyx_v_d, ((npy_complex64 *)__pyx_v_e), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpttrf "F_FUNC(cpttrf,CPTTRF)"(int *n, s *d, npy_complex64 *e, int *info) nogil
 * cdef void cpttrf(int *n, s *d, c *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpttrf(n, d, <npy_complex64*>e, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpttrs "F_FUNC(cpttrs,CPTTRS)"(char *uplo, int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpttrs(char *uplo, int *n, int *nrhs, s *d, c *e, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpttrs(uplo, n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cpttrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_e, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2937
 *     void _fortran_cpttrs "F_FUNC(cpttrs,CPTTRS)"(char *uplo, int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpttrs(char *uplo, int *n, int *nrhs, s *d, c *e, c *b, int *ldb, int *info) nogil:
 *     _fortran_cpttrs(uplo, n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cpttrs,CPTTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex64 *)__pyx_v_e), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cpttrs "F_FUNC(cpttrs,CPTTRS)"(char *uplo, int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cpttrs(char *uplo, int *n, int *nrhs, s *d, c *e, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cpttrs(uplo, n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptts2 "F_FUNC(cptts2,CPTTS2)"(int *iuplo, int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb) nogil
 * cdef void cptts2(int *iuplo, int *n, int *nrhs, s *d, c *e, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptts2(iuplo, n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cptts2(int *__pyx_v_iuplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_float_complex *__pyx_v_e, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":2942
 *     void _fortran_cptts2 "F_FUNC(cptts2,CPTTS2)"(int *iuplo, int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb) nogil
 * cdef void cptts2(int *iuplo, int *n, int *nrhs, s *d, c *e, c *b, int *ldb) nogil:
 *     _fortran_cptts2(iuplo, n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cptts2,CPTTS2)(__pyx_v_iuplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex64 *)__pyx_v_e), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":2941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cptts2 "F_FUNC(cptts2,CPTTS2)"(int *iuplo, int *n, int *nrhs, s *d, npy_complex64 *e, npy_complex64 *b, int *ldb) nogil
 * cdef void cptts2(int *iuplo, int *n, int *nrhs, s *d, c *e, c *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cptts2(iuplo, n, nrhs, d, <npy_complex64*>e, <npy_complex64*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_crot "F_FUNC(crot,CROT)"(int *n, npy_complex64 *cx, int *incx, npy_complex64 *cy, int *incy, s *c, npy_complex64 *s) nogil
 * cdef void crot(int *n, c *cx, int *incx, c *cy, int *incy, s *c, c *s) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_crot(n, <npy_complex64*>cx, incx, <npy_complex64*>cy, incy, c, <npy_complex64*>s)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_crot(int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_cx, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_cy, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_float_complex *__pyx_v_s) {

  /* "scipy/linalg/cython_lapack.pyx":2947
 *     void _fortran_crot "F_FUNC(crot,CROT)"(int *n, npy_complex64 *cx, int *incx, npy_complex64 *cy, int *incy, s *c, npy_complex64 *s) nogil
 * cdef void crot(int *n, c *cx, int *incx, c *cy, int *incy, s *c, c *s) nogil:
 *     _fortran_crot(n, <npy_complex64*>cx, incx, <npy_complex64*>cy, incy, c, <npy_complex64*>s)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(crot,CROT)(__pyx_v_n, ((npy_complex64 *)__pyx_v_cx), __pyx_v_incx, ((npy_complex64 *)__pyx_v_cy), __pyx_v_incy, __pyx_v_c, ((npy_complex64 *)__pyx_v_s));

  /* "scipy/linalg/cython_lapack.pyx":2946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_crot "F_FUNC(crot,CROT)"(int *n, npy_complex64 *cx, int *incx, npy_complex64 *cy, int *incy, s *c, npy_complex64 *s) nogil
 * cdef void crot(int *n, c *cx, int *incx, c *cy, int *incy, s *c, c *s) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_crot(n, <npy_complex64*>cx, incx, <npy_complex64*>cy, incy, c, <npy_complex64*>s)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspcon "F_FUNC(cspcon,CSPCON)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void cspcon(char *uplo, int *n, c *ap, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspcon(uplo, n, <npy_complex64*>ap, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cspcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2952
 *     void _fortran_cspcon "F_FUNC(cspcon,CSPCON)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void cspcon(char *uplo, int *n, c *ap, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:
 *     _fortran_cspcon(uplo, n, <npy_complex64*>ap, ipiv, anorm, rcond, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cspcon,CSPCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspcon "F_FUNC(cspcon,CSPCON)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void cspcon(char *uplo, int *n, c *ap, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspcon(uplo, n, <npy_complex64*>ap, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspmv "F_FUNC(cspmv,CSPMV)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *ap, npy_complex64 *x, int *incx, npy_complex64 *beta, npy_complex64 *y, int *incy) nogil
 * cdef void cspmv(char *uplo, int *n, c *alpha, c *ap, c *x, int *incx, c *beta, c *y, int *incy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspmv(uplo, n, <npy_complex64*>alpha, <npy_complex64*>ap, <npy_complex64*>x, incx, <npy_complex64*>beta, <npy_complex64*>y, incy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cspmv(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_y, int *__pyx_v_incy) {

  /* "scipy/linalg/cython_lapack.pyx":2957
 *     void _fortran_cspmv "F_FUNC(cspmv,CSPMV)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *ap, npy_complex64 *x, int *incx, npy_complex64 *beta, npy_complex64 *y, int *incy) nogil
 * cdef void cspmv(char *uplo, int *n, c *alpha, c *ap, c *x, int *incx, c *beta, c *y, int *incy) nogil:
 *     _fortran_cspmv(uplo, n, <npy_complex64*>alpha, <npy_complex64*>ap, <npy_complex64*>x, incx, <npy_complex64*>beta, <npy_complex64*>y, incy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cspmv,CSPMV)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_y), __pyx_v_incy);

  /* "scipy/linalg/cython_lapack.pyx":2956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspmv "F_FUNC(cspmv,CSPMV)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *ap, npy_complex64 *x, int *incx, npy_complex64 *beta, npy_complex64 *y, int *incy) nogil
 * cdef void cspmv(char *uplo, int *n, c *alpha, c *ap, c *x, int *incx, c *beta, c *y, int *incy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspmv(uplo, n, <npy_complex64*>alpha, <npy_complex64*>ap, <npy_complex64*>x, incx, <npy_complex64*>beta, <npy_complex64*>y, incy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspr "F_FUNC(cspr,CSPR)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *ap) nogil
 * cdef void cspr(char *uplo, int *n, c *alpha, c *x, int *incx, c *ap) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspr(uplo, n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>ap)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cspr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_ap) {

  /* "scipy/linalg/cython_lapack.pyx":2962
 *     void _fortran_cspr "F_FUNC(cspr,CSPR)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *ap) nogil
 * cdef void cspr(char *uplo, int *n, c *alpha, c *x, int *incx, c *ap) nogil:
 *     _fortran_cspr(uplo, n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>ap)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cspr,CSPR)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, ((npy_complex64 *)__pyx_v_ap));

  /* "scipy/linalg/cython_lapack.pyx":2961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspr "F_FUNC(cspr,CSPR)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *ap) nogil
 * cdef void cspr(char *uplo, int *n, c *alpha, c *x, int *incx, c *ap) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspr(uplo, n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>ap)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csprfs "F_FUNC(csprfs,CSPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void csprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2967
 *     void _fortran_csprfs "F_FUNC(csprfs,CSPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void csprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_csprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csprfs,CSPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_afp), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csprfs "F_FUNC(csprfs,CSPRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void csprfs(char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csprfs(uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspsv "F_FUNC(cspsv,CSPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cspsv(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspsv(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cspsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2972
 *     void _fortran_cspsv "F_FUNC(cspsv,CSPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cspsv(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_cspsv(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cspsv,CSPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspsv "F_FUNC(cspsv,CSPSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void cspsv(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspsv(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspsvx "F_FUNC(cspsvx,CSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cspsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cspsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2977
 *     void _fortran_cspsvx "F_FUNC(cspsvx,CSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cspsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_cspsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cspsvx,CSPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_afp), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cspsvx "F_FUNC(cspsvx,CSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *afp, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void cspsvx(char *fact, char *uplo, int *n, int *nrhs, c *ap, c *afp, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cspsvx(fact, uplo, n, nrhs, <npy_complex64*>ap, <npy_complex64*>afp, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csptrf "F_FUNC(csptrf,CSPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, int *info) nogil
 * cdef void csptrf(char *uplo, int *n, c *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csptrf(uplo, n, <npy_complex64*>ap, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2982
 *     void _fortran_csptrf "F_FUNC(csptrf,CSPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, int *info) nogil
 * cdef void csptrf(char *uplo, int *n, c *ap, int *ipiv, int *info) nogil:
 *     _fortran_csptrf(uplo, n, <npy_complex64*>ap, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csptrf,CSPTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csptrf "F_FUNC(csptrf,CSPTRF)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, int *info) nogil
 * cdef void csptrf(char *uplo, int *n, c *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csptrf(uplo, n, <npy_complex64*>ap, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csptri "F_FUNC(csptri,CSPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void csptri(char *uplo, int *n, c *ap, int *ipiv, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csptri(uplo, n, <npy_complex64*>ap, ipiv, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2987
 *     void _fortran_csptri "F_FUNC(csptri,CSPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void csptri(char *uplo, int *n, c *ap, int *ipiv, c *work, int *info) nogil:
 *     _fortran_csptri(uplo, n, <npy_complex64*>ap, ipiv, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csptri,CSPTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csptri "F_FUNC(csptri,CSPTRI)"(char *uplo, int *n, npy_complex64 *ap, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void csptri(char *uplo, int *n, c *ap, int *ipiv, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csptri(uplo, n, <npy_complex64*>ap, ipiv, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csptrs "F_FUNC(csptrs,CSPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void csptrs(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csptrs(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":2992
 *     void _fortran_csptrs "F_FUNC(csptrs,CSPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void csptrs(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_csptrs(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csptrs,CSPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":2991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csptrs "F_FUNC(csptrs,CSPTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *ap, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void csptrs(char *uplo, int *n, int *nrhs, c *ap, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csptrs(uplo, n, nrhs, <npy_complex64*>ap, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":2996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csrscl "F_FUNC(csrscl,CSRSCL)"(int *n, s *sa, npy_complex64 *sx, int *incx) nogil
 * cdef void csrscl(int *n, s *sa, c *sx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csrscl(n, sa, <npy_complex64*>sx, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csrscl(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sa, __pyx_t_float_complex *__pyx_v_sx, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":2997
 *     void _fortran_csrscl "F_FUNC(csrscl,CSRSCL)"(int *n, s *sa, npy_complex64 *sx, int *incx) nogil
 * cdef void csrscl(int *n, s *sa, c *sx, int *incx) nogil:
 *     _fortran_csrscl(n, sa, <npy_complex64*>sx, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csrscl,CSRSCL)(__pyx_v_n, __pyx_v_sa, ((npy_complex64 *)__pyx_v_sx), __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":2996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csrscl "F_FUNC(csrscl,CSRSCL)"(int *n, s *sa, npy_complex64 *sx, int *incx) nogil
 * cdef void csrscl(int *n, s *sa, c *sx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csrscl(n, sa, <npy_complex64*>sx, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cstedc "F_FUNC(cstedc,CSTEDC)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstedc(char *compz, int *n, s *d, s *e, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cstedc(compz, n, d, e, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cstedc(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3002
 *     void _fortran_cstedc "F_FUNC(cstedc,CSTEDC)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstedc(char *compz, int *n, s *d, s *e, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_cstedc(compz, n, d, e, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cstedc,CSTEDC)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cstedc "F_FUNC(cstedc,CSTEDC)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, npy_complex64 *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstedc(char *compz, int *n, s *d, s *e, c *z, int *ldz, c *work, int *lwork, s *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cstedc(compz, n, d, e, <npy_complex64*>z, ldz, <npy_complex64*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cstegr "F_FUNC(cstegr,CSTEGR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstegr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cstegr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3007
 *     void _fortran_cstegr "F_FUNC(cstegr,CSTEGR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstegr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_cstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, isuppz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cstegr,CSTEGR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cstegr "F_FUNC(cstegr,CSTEGR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, npy_complex64 *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstegr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, c *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, <npy_complex64*>z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cstein "F_FUNC(cstein,CSTEIN)"(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, npy_complex64 *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void cstein(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, c *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cstein(n, d, e, m, w, iblock, isplit, <npy_complex64*>z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cstein(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, int *__pyx_v_iblock, int *__pyx_v_isplit, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3012
 *     void _fortran_cstein "F_FUNC(cstein,CSTEIN)"(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, npy_complex64 *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void cstein(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, c *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_cstein(n, d, e, m, w, iblock, isplit, <npy_complex64*>z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cstein,CSTEIN)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_m, __pyx_v_w, __pyx_v_iblock, __pyx_v_isplit, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cstein "F_FUNC(cstein,CSTEIN)"(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, npy_complex64 *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void cstein(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, c *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cstein(n, d, e, m, w, iblock, isplit, <npy_complex64*>z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cstemr "F_FUNC(cstemr,CSTEMR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, npy_complex64 *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstemr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, c *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, <npy_complex64*>z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cstemr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_nzc, int *__pyx_v_isuppz, int *__pyx_v_tryrac, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3017
 *     void _fortran_cstemr "F_FUNC(cstemr,CSTEMR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, npy_complex64 *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstemr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, c *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_cstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, <npy_complex64*>z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cstemr,CSTEMR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_m, __pyx_v_w, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_nzc, __pyx_v_isuppz, __pyx_v_tryrac, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cstemr "F_FUNC(cstemr,CSTEMR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, npy_complex64 *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void cstemr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, c *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, <npy_complex64*>z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csteqr "F_FUNC(csteqr,CSTEQR)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, s *work, int *info) nogil
 * cdef void csteqr(char *compz, int *n, s *d, s *e, c *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csteqr(compz, n, d, e, <npy_complex64*>z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csteqr(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3022
 *     void _fortran_csteqr "F_FUNC(csteqr,CSTEQR)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, s *work, int *info) nogil
 * cdef void csteqr(char *compz, int *n, s *d, s *e, c *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_csteqr(compz, n, d, e, <npy_complex64*>z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csteqr,CSTEQR)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csteqr "F_FUNC(csteqr,CSTEQR)"(char *compz, int *n, s *d, s *e, npy_complex64 *z, int *ldz, s *work, int *info) nogil
 * cdef void csteqr(char *compz, int *n, s *d, s *e, c *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csteqr(compz, n, d, e, <npy_complex64*>z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csycon "F_FUNC(csycon,CSYCON)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void csycon(char *uplo, int *n, c *a, int *lda, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csycon(uplo, n, <npy_complex64*>a, lda, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csycon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3027
 *     void _fortran_csycon "F_FUNC(csycon,CSYCON)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void csycon(char *uplo, int *n, c *a, int *lda, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:
 *     _fortran_csycon(uplo, n, <npy_complex64*>a, lda, ipiv, anorm, rcond, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csycon,CSYCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csycon "F_FUNC(csycon,CSYCON)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, s *anorm, s *rcond, npy_complex64 *work, int *info) nogil
 * cdef void csycon(char *uplo, int *n, c *a, int *lda, int *ipiv, s *anorm, s *rcond, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csycon(uplo, n, <npy_complex64*>a, lda, ipiv, anorm, rcond, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csymv "F_FUNC(csymv,CSYMV)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *a, int *lda, npy_complex64 *x, int *incx, npy_complex64 *beta, npy_complex64 *y, int *incy) nogil
 * cdef void csymv(char *uplo, int *n, c *alpha, c *a, int *lda, c *x, int *incx, c *beta, c *y, int *incy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csymv(uplo, n, <npy_complex64*>alpha, <npy_complex64*>a, lda, <npy_complex64*>x, incx, <npy_complex64*>beta, <npy_complex64*>y, incy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csymv(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_y, int *__pyx_v_incy) {

  /* "scipy/linalg/cython_lapack.pyx":3032
 *     void _fortran_csymv "F_FUNC(csymv,CSYMV)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *a, int *lda, npy_complex64 *x, int *incx, npy_complex64 *beta, npy_complex64 *y, int *incy) nogil
 * cdef void csymv(char *uplo, int *n, c *alpha, c *a, int *lda, c *x, int *incx, c *beta, c *y, int *incy) nogil:
 *     _fortran_csymv(uplo, n, <npy_complex64*>alpha, <npy_complex64*>a, lda, <npy_complex64*>x, incx, <npy_complex64*>beta, <npy_complex64*>y, incy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csymv,CSYMV)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_y), __pyx_v_incy);

  /* "scipy/linalg/cython_lapack.pyx":3031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csymv "F_FUNC(csymv,CSYMV)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *a, int *lda, npy_complex64 *x, int *incx, npy_complex64 *beta, npy_complex64 *y, int *incy) nogil
 * cdef void csymv(char *uplo, int *n, c *alpha, c *a, int *lda, c *x, int *incx, c *beta, c *y, int *incy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csymv(uplo, n, <npy_complex64*>alpha, <npy_complex64*>a, lda, <npy_complex64*>x, incx, <npy_complex64*>beta, <npy_complex64*>y, incy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csyr "F_FUNC(csyr,CSYR)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *a, int *lda) nogil
 * cdef void csyr(char *uplo, int *n, c *alpha, c *x, int *incx, c *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csyr(uplo, n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csyr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":3037
 *     void _fortran_csyr "F_FUNC(csyr,CSYR)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *a, int *lda) nogil
 * cdef void csyr(char *uplo, int *n, c *alpha, c *x, int *incx, c *a, int *lda) nogil:
 *     _fortran_csyr(uplo, n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csyr,CSYR)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_x), __pyx_v_incx, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":3036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csyr "F_FUNC(csyr,CSYR)"(char *uplo, int *n, npy_complex64 *alpha, npy_complex64 *x, int *incx, npy_complex64 *a, int *lda) nogil
 * cdef void csyr(char *uplo, int *n, c *alpha, c *x, int *incx, c *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csyr(uplo, n, <npy_complex64*>alpha, <npy_complex64*>x, incx, <npy_complex64*>a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csyrfs "F_FUNC(csyrfs,CSYRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void csyrfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csyrfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csyrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3042
 *     void _fortran_csyrfs "F_FUNC(csyrfs,CSYRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void csyrfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_csyrfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csyrfs,CSYRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csyrfs "F_FUNC(csyrfs,CSYRFS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void csyrfs(char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csyrfs(uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csysv "F_FUNC(csysv,CSYSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void csysv(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csysv(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csysv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3047
 *     void _fortran_csysv "F_FUNC(csysv,CSYSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void csysv(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, c *work, int *lwork, int *info) nogil:
 *     _fortran_csysv(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csysv,CSYSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csysv "F_FUNC(csysv,CSYSV)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void csysv(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csysv(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csysvx "F_FUNC(csysvx,CSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void csysvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csysvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csysvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3052
 *     void _fortran_csysvx "F_FUNC(csysvx,CSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void csysvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, int *lwork, s *rwork, int *info) nogil:
 *     _fortran_csysvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csysvx,CSYSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csysvx "F_FUNC(csysvx,CSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *af, int *ldaf, int *ipiv, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *rcond, s *ferr, s *berr, npy_complex64 *work, int *lwork, s *rwork, int *info) nogil
 * cdef void csysvx(char *fact, char *uplo, int *n, int *nrhs, c *a, int *lda, c *af, int *ldaf, int *ipiv, c *b, int *ldb, c *x, int *ldx, s *rcond, s *ferr, s *berr, c *work, int *lwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csysvx(fact, uplo, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>af, ldaf, ipiv, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, rcond, ferr, berr, <npy_complex64*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csytf2 "F_FUNC(csytf2,CSYTF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void csytf2(char *uplo, int *n, c *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csytf2(uplo, n, <npy_complex64*>a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csytf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3057
 *     void _fortran_csytf2 "F_FUNC(csytf2,CSYTF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void csytf2(char *uplo, int *n, c *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_csytf2(uplo, n, <npy_complex64*>a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csytf2,CSYTF2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csytf2 "F_FUNC(csytf2,CSYTF2)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void csytf2(char *uplo, int *n, c *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csytf2(uplo, n, <npy_complex64*>a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csytrf "F_FUNC(csytrf,CSYTRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void csytrf(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csytrf(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csytrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3062
 *     void _fortran_csytrf "F_FUNC(csytrf,CSYTRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void csytrf(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:
 *     _fortran_csytrf(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csytrf,CSYTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csytrf "F_FUNC(csytrf,CSYTRF)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void csytrf(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csytrf(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csytri "F_FUNC(csytri,CSYTRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void csytri(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csytri(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csytri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3067
 *     void _fortran_csytri "F_FUNC(csytri,CSYTRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void csytri(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *info) nogil:
 *     _fortran_csytri(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csytri,CSYTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csytri "F_FUNC(csytri,CSYTRI)"(char *uplo, int *n, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *work, int *info) nogil
 * cdef void csytri(char *uplo, int *n, c *a, int *lda, int *ipiv, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csytri(uplo, n, <npy_complex64*>a, lda, ipiv, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csytrs "F_FUNC(csytrs,CSYTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void csytrs(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csytrs(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_csytrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3072
 *     void _fortran_csytrs "F_FUNC(csytrs,CSYTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void csytrs(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:
 *     _fortran_csytrs(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(csytrs,CSYTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_csytrs "F_FUNC(csytrs,CSYTRS)"(char *uplo, int *n, int *nrhs, npy_complex64 *a, int *lda, int *ipiv, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void csytrs(char *uplo, int *n, int *nrhs, c *a, int *lda, int *ipiv, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_csytrs(uplo, n, nrhs, <npy_complex64*>a, lda, ipiv, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctbcon "F_FUNC(ctbcon,CTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, npy_complex64 *ab, int *ldab, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctbcon(char *norm, char *uplo, char *diag, int *n, int *kd, c *ab, int *ldab, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctbcon(norm, uplo, diag, n, kd, <npy_complex64*>ab, ldab, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctbcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3077
 *     void _fortran_ctbcon "F_FUNC(ctbcon,CTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, npy_complex64 *ab, int *ldab, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctbcon(char *norm, char *uplo, char *diag, int *n, int *kd, c *ab, int *ldab, s *rcond, c *work, s *rwork, int *info) nogil:
 *     _fortran_ctbcon(norm, uplo, diag, n, kd, <npy_complex64*>ab, ldab, rcond, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctbcon,CTBCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctbcon "F_FUNC(ctbcon,CTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, npy_complex64 *ab, int *ldab, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctbcon(char *norm, char *uplo, char *diag, int *n, int *kd, c *ab, int *ldab, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctbcon(norm, uplo, diag, n, kd, <npy_complex64*>ab, ldab, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctbrfs "F_FUNC(ctbrfs,CTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctbrfs(uplo, trans, diag, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctbrfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3082
 *     void _fortran_ctbrfs "F_FUNC(ctbrfs,CTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_ctbrfs(uplo, trans, diag, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctbrfs,CTBRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctbrfs "F_FUNC(ctbrfs,CTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctbrfs(uplo, trans, diag, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctbtrs "F_FUNC(ctbtrs,CTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctbtrs(uplo, trans, diag, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctbtrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3087
 *     void _fortran_ctbtrs "F_FUNC(ctbtrs,CTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:
 *     _fortran_ctbtrs(uplo, trans, diag, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctbtrs,CTBTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctbtrs "F_FUNC(ctbtrs,CTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex64 *ab, int *ldab, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, c *ab, int *ldab, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctbtrs(uplo, trans, diag, n, kd, nrhs, <npy_complex64*>ab, ldab, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgevc "F_FUNC(ctgevc,CTGEVC)"(char *side, char *howmny, bint *select, int *n, npy_complex64 *s, int *lds, npy_complex64 *p, int *ldp, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctgevc(char *side, char *howmny, bint *select, int *n, c *s, int *lds, c *p, int *ldp, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgevc(side, howmny, select, n, <npy_complex64*>s, lds, <npy_complex64*>p, ldp, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctgevc(char *__pyx_v_side, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_s, int *__pyx_v_lds, __pyx_t_float_complex *__pyx_v_p, int *__pyx_v_ldp, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3092
 *     void _fortran_ctgevc "F_FUNC(ctgevc,CTGEVC)"(char *side, char *howmny, bint *select, int *n, npy_complex64 *s, int *lds, npy_complex64 *p, int *ldp, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctgevc(char *side, char *howmny, bint *select, int *n, c *s, int *lds, c *p, int *ldp, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *info) nogil:
 *     _fortran_ctgevc(side, howmny, select, n, <npy_complex64*>s, lds, <npy_complex64*>p, ldp, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctgevc,CTGEVC)(__pyx_v_side, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, ((npy_complex64 *)__pyx_v_s), __pyx_v_lds, ((npy_complex64 *)__pyx_v_p), __pyx_v_ldp, ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgevc "F_FUNC(ctgevc,CTGEVC)"(char *side, char *howmny, bint *select, int *n, npy_complex64 *s, int *lds, npy_complex64 *p, int *ldp, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctgevc(char *side, char *howmny, bint *select, int *n, c *s, int *lds, c *p, int *ldp, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgevc(side, howmny, select, n, <npy_complex64*>s, lds, <npy_complex64*>p, ldp, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgex2 "F_FUNC(ctgex2,CTGEX2)"(bint *wantq, bint *wantz, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *j1, int *info) nogil
 * cdef void ctgex2(bint *wantq, bint *wantz, int *n, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *j1, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgex2(wantq, wantz, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, j1, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctgex2(int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_j1, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3097
 *     void _fortran_ctgex2 "F_FUNC(ctgex2,CTGEX2)"(bint *wantq, bint *wantz, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *j1, int *info) nogil
 * cdef void ctgex2(bint *wantq, bint *wantz, int *n, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *j1, int *info) nogil:
 *     _fortran_ctgex2(wantq, wantz, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, j1, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctgex2,CTGEX2)(__pyx_v_wantq, __pyx_v_wantz, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_j1, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgex2 "F_FUNC(ctgex2,CTGEX2)"(bint *wantq, bint *wantz, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *j1, int *info) nogil
 * cdef void ctgex2(bint *wantq, bint *wantz, int *n, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *j1, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgex2(wantq, wantz, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, j1, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgexc "F_FUNC(ctgexc,CTGEXC)"(bint *wantq, bint *wantz, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *ifst, int *ilst, int *info) nogil
 * cdef void ctgexc(bint *wantq, bint *wantz, int *n, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *ifst, int *ilst, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgexc(wantq, wantz, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, ifst, ilst, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctgexc(int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ifst, int *__pyx_v_ilst, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3102
 *     void _fortran_ctgexc "F_FUNC(ctgexc,CTGEXC)"(bint *wantq, bint *wantz, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *ifst, int *ilst, int *info) nogil
 * cdef void ctgexc(bint *wantq, bint *wantz, int *n, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *ifst, int *ilst, int *info) nogil:
 *     _fortran_ctgexc(wantq, wantz, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, ifst, ilst, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctgexc,CTGEXC)(__pyx_v_wantq, __pyx_v_wantz, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_ifst, __pyx_v_ilst, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgexc "F_FUNC(ctgexc,CTGEXC)"(bint *wantq, bint *wantz, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *ifst, int *ilst, int *info) nogil
 * cdef void ctgexc(bint *wantq, bint *wantz, int *n, c *a, int *lda, c *b, int *ldb, c *q, int *ldq, c *z, int *ldz, int *ifst, int *ilst, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgexc(wantq, wantz, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, ifst, ilst, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsen "F_FUNC(ctgsen,CTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *m, s *pl, s *pr, s *dif, npy_complex64 *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ctgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *q, int *ldq, c *z, int *ldz, int *m, s *pl, s *pr, s *dif, c *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsen(ijob, wantq, wantz, select, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, m, pl, pr, dif, <npy_complex64*>work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctgsen(int *__pyx_v_ijob, int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dif, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3107
 *     void _fortran_ctgsen "F_FUNC(ctgsen,CTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *m, s *pl, s *pr, s *dif, npy_complex64 *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ctgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *q, int *ldq, c *z, int *ldz, int *m, s *pl, s *pr, s *dif, c *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_ctgsen(ijob, wantq, wantz, select, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, m, pl, pr, dif, <npy_complex64*>work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctgsen,CTGSEN)(__pyx_v_ijob, __pyx_v_wantq, __pyx_v_wantz, __pyx_v_select, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_alpha), ((npy_complex64 *)__pyx_v_beta), ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_m, __pyx_v_pl, __pyx_v_pr, __pyx_v_dif, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsen "F_FUNC(ctgsen,CTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *alpha, npy_complex64 *beta, npy_complex64 *q, int *ldq, npy_complex64 *z, int *ldz, int *m, s *pl, s *pr, s *dif, npy_complex64 *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ctgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, c *a, int *lda, c *b, int *ldb, c *alpha, c *beta, c *q, int *ldq, c *z, int *ldz, int *m, s *pl, s *pr, s *dif, c *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsen(ijob, wantq, wantz, select, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>alpha, <npy_complex64*>beta, <npy_complex64*>q, ldq, <npy_complex64*>z, ldz, m, pl, pr, dif, <npy_complex64*>work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsja "F_FUNC(ctgsja,CTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, npy_complex64 *u, int *ldu, npy_complex64 *v, int *ldv, npy_complex64 *q, int *ldq, npy_complex64 *work, int *ncycle, int *info) nogil
 * cdef void ctgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, c *a, int *lda, c *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, c *u, int *ldu, c *v, int *ldv, c *q, int *ldq, c *work, int *ncycle, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsja(jobu, jobv, jobq, m, p, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, tola, tolb, alpha, beta, <npy_complex64*>u, ldu, <npy_complex64*>v, ldv, <npy_complex64*>q, ldq, <npy_complex64*>work, ncycle, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctgsja(char *__pyx_v_jobu, char *__pyx_v_jobv, char *__pyx_v_jobq, int *__pyx_v_m, int *__pyx_v_p, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tola, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tolb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_ncycle, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3112
 *     void _fortran_ctgsja "F_FUNC(ctgsja,CTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, npy_complex64 *u, int *ldu, npy_complex64 *v, int *ldv, npy_complex64 *q, int *ldq, npy_complex64 *work, int *ncycle, int *info) nogil
 * cdef void ctgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, c *a, int *lda, c *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, c *u, int *ldu, c *v, int *ldv, c *q, int *ldq, c *work, int *ncycle, int *info) nogil:
 *     _fortran_ctgsja(jobu, jobv, jobq, m, p, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, tola, tolb, alpha, beta, <npy_complex64*>u, ldu, <npy_complex64*>v, ldv, <npy_complex64*>q, ldq, <npy_complex64*>work, ncycle, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctgsja,CTGSJA)(__pyx_v_jobu, __pyx_v_jobv, __pyx_v_jobq, __pyx_v_m, __pyx_v_p, __pyx_v_n, __pyx_v_k, __pyx_v_l, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_tola, __pyx_v_tolb, __pyx_v_alpha, __pyx_v_beta, ((npy_complex64 *)__pyx_v_u), __pyx_v_ldu, ((npy_complex64 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_work), __pyx_v_ncycle, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsja "F_FUNC(ctgsja,CTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, npy_complex64 *u, int *ldu, npy_complex64 *v, int *ldv, npy_complex64 *q, int *ldq, npy_complex64 *work, int *ncycle, int *info) nogil
 * cdef void ctgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, c *a, int *lda, c *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, c *u, int *ldu, c *v, int *ldv, c *q, int *ldq, c *work, int *ncycle, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsja(jobu, jobv, jobq, m, p, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, tola, tolb, alpha, beta, <npy_complex64*>u, ldu, <npy_complex64*>v, ldv, <npy_complex64*>q, ldq, <npy_complex64*>work, ncycle, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsna "F_FUNC(ctgsna,CTGSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, s *s, s *dif, int *mm, int *m, npy_complex64 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ctgsna(char *job, char *howmny, bint *select, int *n, c *a, int *lda, c *b, int *ldb, c *vl, int *ldvl, c *vr, int *ldvr, s *s, s *dif, int *mm, int *m, c *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsna(job, howmny, select, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, s, dif, mm, m, <npy_complex64*>work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctgsna(char *__pyx_v_job, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dif, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3117
 *     void _fortran_ctgsna "F_FUNC(ctgsna,CTGSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, s *s, s *dif, int *mm, int *m, npy_complex64 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ctgsna(char *job, char *howmny, bint *select, int *n, c *a, int *lda, c *b, int *ldb, c *vl, int *ldvl, c *vr, int *ldvr, s *s, s *dif, int *mm, int *m, c *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_ctgsna(job, howmny, select, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, s, dif, mm, m, <npy_complex64*>work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctgsna,CTGSNA)(__pyx_v_job, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_s, __pyx_v_dif, __pyx_v_mm, __pyx_v_m, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsna "F_FUNC(ctgsna,CTGSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, s *s, s *dif, int *mm, int *m, npy_complex64 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ctgsna(char *job, char *howmny, bint *select, int *n, c *a, int *lda, c *b, int *ldb, c *vl, int *ldvl, c *vr, int *ldvr, s *s, s *dif, int *mm, int *m, c *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsna(job, howmny, select, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, s, dif, mm, m, <npy_complex64*>work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsy2 "F_FUNC(ctgsy2,CTGSY2)"(char *trans, int *ijob, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, npy_complex64 *d, int *ldd, npy_complex64 *e, int *lde, npy_complex64 *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *info) nogil
 * cdef void ctgsy2(char *trans, int *ijob, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, c *d, int *ldd, c *e, int *lde, c *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsy2(trans, ijob, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, <npy_complex64*>d, ldd, <npy_complex64*>e, lde, <npy_complex64*>f, ldf, scale, rdsum, rdscal, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctgsy2(char *__pyx_v_trans, int *__pyx_v_ijob, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_d, int *__pyx_v_ldd, __pyx_t_float_complex *__pyx_v_e, int *__pyx_v_lde, __pyx_t_float_complex *__pyx_v_f, int *__pyx_v_ldf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rdsum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rdscal, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3122
 *     void _fortran_ctgsy2 "F_FUNC(ctgsy2,CTGSY2)"(char *trans, int *ijob, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, npy_complex64 *d, int *ldd, npy_complex64 *e, int *lde, npy_complex64 *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *info) nogil
 * cdef void ctgsy2(char *trans, int *ijob, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, c *d, int *ldd, c *e, int *lde, c *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *info) nogil:
 *     _fortran_ctgsy2(trans, ijob, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, <npy_complex64*>d, ldd, <npy_complex64*>e, lde, <npy_complex64*>f, ldf, scale, rdsum, rdscal, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctgsy2,CTGSY2)(__pyx_v_trans, __pyx_v_ijob, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_d), __pyx_v_ldd, ((npy_complex64 *)__pyx_v_e), __pyx_v_lde, ((npy_complex64 *)__pyx_v_f), __pyx_v_ldf, __pyx_v_scale, __pyx_v_rdsum, __pyx_v_rdscal, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsy2 "F_FUNC(ctgsy2,CTGSY2)"(char *trans, int *ijob, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, npy_complex64 *d, int *ldd, npy_complex64 *e, int *lde, npy_complex64 *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *info) nogil
 * cdef void ctgsy2(char *trans, int *ijob, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, c *d, int *ldd, c *e, int *lde, c *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsy2(trans, ijob, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, <npy_complex64*>d, ldd, <npy_complex64*>e, lde, <npy_complex64*>f, ldf, scale, rdsum, rdscal, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsyl "F_FUNC(ctgsyl,CTGSYL)"(char *trans, int *ijob, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, npy_complex64 *d, int *ldd, npy_complex64 *e, int *lde, npy_complex64 *f, int *ldf, s *scale, s *dif, npy_complex64 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ctgsyl(char *trans, int *ijob, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, c *d, int *ldd, c *e, int *lde, c *f, int *ldf, s *scale, s *dif, c *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsyl(trans, ijob, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, <npy_complex64*>d, ldd, <npy_complex64*>e, lde, <npy_complex64*>f, ldf, scale, dif, <npy_complex64*>work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctgsyl(char *__pyx_v_trans, int *__pyx_v_ijob, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_d, int *__pyx_v_ldd, __pyx_t_float_complex *__pyx_v_e, int *__pyx_v_lde, __pyx_t_float_complex *__pyx_v_f, int *__pyx_v_ldf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dif, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3127
 *     void _fortran_ctgsyl "F_FUNC(ctgsyl,CTGSYL)"(char *trans, int *ijob, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, npy_complex64 *d, int *ldd, npy_complex64 *e, int *lde, npy_complex64 *f, int *ldf, s *scale, s *dif, npy_complex64 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ctgsyl(char *trans, int *ijob, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, c *d, int *ldd, c *e, int *lde, c *f, int *ldf, s *scale, s *dif, c *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_ctgsyl(trans, ijob, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, <npy_complex64*>d, ldd, <npy_complex64*>e, lde, <npy_complex64*>f, ldf, scale, dif, <npy_complex64*>work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctgsyl,CTGSYL)(__pyx_v_trans, __pyx_v_ijob, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_d), __pyx_v_ldd, ((npy_complex64 *)__pyx_v_e), __pyx_v_lde, ((npy_complex64 *)__pyx_v_f), __pyx_v_ldf, __pyx_v_scale, __pyx_v_dif, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctgsyl "F_FUNC(ctgsyl,CTGSYL)"(char *trans, int *ijob, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, npy_complex64 *d, int *ldd, npy_complex64 *e, int *lde, npy_complex64 *f, int *ldf, s *scale, s *dif, npy_complex64 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ctgsyl(char *trans, int *ijob, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, c *d, int *ldd, c *e, int *lde, c *f, int *ldf, s *scale, s *dif, c *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctgsyl(trans, ijob, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, <npy_complex64*>d, ldd, <npy_complex64*>e, lde, <npy_complex64*>f, ldf, scale, dif, <npy_complex64*>work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctpcon "F_FUNC(ctpcon,CTPCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex64 *ap, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctpcon(char *norm, char *uplo, char *diag, int *n, c *ap, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctpcon(norm, uplo, diag, n, <npy_complex64*>ap, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctpcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3132
 *     void _fortran_ctpcon "F_FUNC(ctpcon,CTPCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex64 *ap, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctpcon(char *norm, char *uplo, char *diag, int *n, c *ap, s *rcond, c *work, s *rwork, int *info) nogil:
 *     _fortran_ctpcon(norm, uplo, diag, n, <npy_complex64*>ap, rcond, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctpcon,CTPCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctpcon "F_FUNC(ctpcon,CTPCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex64 *ap, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctpcon(char *norm, char *uplo, char *diag, int *n, c *ap, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctpcon(norm, uplo, diag, n, <npy_complex64*>ap, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctprfs "F_FUNC(ctprfs,CTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *ap, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctprfs(uplo, trans, diag, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctprfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3137
 *     void _fortran_ctprfs "F_FUNC(ctprfs,CTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *ap, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_ctprfs(uplo, trans, diag, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctprfs,CTPRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctprfs "F_FUNC(ctprfs,CTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *ap, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctprfs(uplo, trans, diag, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctptri "F_FUNC(ctptri,CTPTRI)"(char *uplo, char *diag, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void ctptri(char *uplo, char *diag, int *n, c *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctptri(uplo, diag, n, <npy_complex64*>ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctptri(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3142
 *     void _fortran_ctptri "F_FUNC(ctptri,CTPTRI)"(char *uplo, char *diag, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void ctptri(char *uplo, char *diag, int *n, c *ap, int *info) nogil:
 *     _fortran_ctptri(uplo, diag, n, <npy_complex64*>ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctptri,CTPTRI)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctptri "F_FUNC(ctptri,CTPTRI)"(char *uplo, char *diag, int *n, npy_complex64 *ap, int *info) nogil
 * cdef void ctptri(char *uplo, char *diag, int *n, c *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctptri(uplo, diag, n, <npy_complex64*>ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctptrs "F_FUNC(ctptrs,CTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctptrs(uplo, trans, diag, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctptrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3147
 *     void _fortran_ctptrs "F_FUNC(ctptrs,CTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:
 *     _fortran_ctptrs(uplo, trans, diag, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctptrs,CTPTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctptrs "F_FUNC(ctptrs,CTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *ap, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *ap, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctptrs(uplo, trans, diag, n, nrhs, <npy_complex64*>ap, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrcon "F_FUNC(ctrcon,CTRCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrcon(char *norm, char *uplo, char *diag, int *n, c *a, int *lda, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrcon(norm, uplo, diag, n, <npy_complex64*>a, lda, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3152
 *     void _fortran_ctrcon "F_FUNC(ctrcon,CTRCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrcon(char *norm, char *uplo, char *diag, int *n, c *a, int *lda, s *rcond, c *work, s *rwork, int *info) nogil:
 *     _fortran_ctrcon(norm, uplo, diag, n, <npy_complex64*>a, lda, rcond, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrcon,CTRCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_rcond, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrcon "F_FUNC(ctrcon,CTRCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, s *rcond, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrcon(char *norm, char *uplo, char *diag, int *n, c *a, int *lda, s *rcond, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrcon(norm, uplo, diag, n, <npy_complex64*>a, lda, rcond, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrevc "F_FUNC(ctrevc,CTREVC)"(char *side, char *howmny, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrevc(char *side, char *howmny, bint *select, int *n, c *t, int *ldt, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrevc(side, howmny, select, n, <npy_complex64*>t, ldt, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrevc(char *__pyx_v_side, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3157
 *     void _fortran_ctrevc "F_FUNC(ctrevc,CTREVC)"(char *side, char *howmny, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrevc(char *side, char *howmny, bint *select, int *n, c *t, int *ldt, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *info) nogil:
 *     _fortran_ctrevc(side, howmny, select, n, <npy_complex64*>t, ldt, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrevc,CTREVC)(__pyx_v_side, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrevc "F_FUNC(ctrevc,CTREVC)"(char *side, char *howmny, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, int *mm, int *m, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrevc(char *side, char *howmny, bint *select, int *n, c *t, int *ldt, c *vl, int *ldvl, c *vr, int *ldvr, int *mm, int *m, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrevc(side, howmny, select, n, <npy_complex64*>t, ldt, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, mm, m, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrexc "F_FUNC(ctrexc,CTREXC)"(char *compq, int *n, npy_complex64 *t, int *ldt, npy_complex64 *q, int *ldq, int *ifst, int *ilst, int *info) nogil
 * cdef void ctrexc(char *compq, int *n, c *t, int *ldt, c *q, int *ldq, int *ifst, int *ilst, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrexc(compq, n, <npy_complex64*>t, ldt, <npy_complex64*>q, ldq, ifst, ilst, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrexc(char *__pyx_v_compq, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_ifst, int *__pyx_v_ilst, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3162
 *     void _fortran_ctrexc "F_FUNC(ctrexc,CTREXC)"(char *compq, int *n, npy_complex64 *t, int *ldt, npy_complex64 *q, int *ldq, int *ifst, int *ilst, int *info) nogil
 * cdef void ctrexc(char *compq, int *n, c *t, int *ldt, c *q, int *ldq, int *ifst, int *ilst, int *info) nogil:
 *     _fortran_ctrexc(compq, n, <npy_complex64*>t, ldt, <npy_complex64*>q, ldq, ifst, ilst, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrexc,CTREXC)(__pyx_v_compq, __pyx_v_n, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_ifst, __pyx_v_ilst, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrexc "F_FUNC(ctrexc,CTREXC)"(char *compq, int *n, npy_complex64 *t, int *ldt, npy_complex64 *q, int *ldq, int *ifst, int *ilst, int *info) nogil
 * cdef void ctrexc(char *compq, int *n, c *t, int *ldt, c *q, int *ldq, int *ifst, int *ilst, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrexc(compq, n, <npy_complex64*>t, ldt, <npy_complex64*>q, ldq, ifst, ilst, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrrfs "F_FUNC(ctrrfs,CTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrrfs(uplo, trans, diag, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrrfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_float_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3167
 *     void _fortran_ctrrfs "F_FUNC(ctrrfs,CTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:
 *     _fortran_ctrrfs(uplo, trans, diag, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrrfs,CTRRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex64 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrrfs "F_FUNC(ctrrfs,CTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *x, int *ldx, s *ferr, s *berr, npy_complex64 *work, s *rwork, int *info) nogil
 * cdef void ctrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, c *x, int *ldx, s *ferr, s *berr, c *work, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrrfs(uplo, trans, diag, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>x, ldx, ferr, berr, <npy_complex64*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrsen "F_FUNC(ctrsen,CTRSEN)"(char *job, char *compq, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *q, int *ldq, npy_complex64 *w, int *m, s *s, s *sep, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void ctrsen(char *job, char *compq, bint *select, int *n, c *t, int *ldt, c *q, int *ldq, c *w, int *m, s *s, s *sep, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrsen(job, compq, select, n, <npy_complex64*>t, ldt, <npy_complex64*>q, ldq, <npy_complex64*>w, m, s, sep, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrsen(char *__pyx_v_job, char *__pyx_v_compq, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_w, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sep, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3172
 *     void _fortran_ctrsen "F_FUNC(ctrsen,CTRSEN)"(char *job, char *compq, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *q, int *ldq, npy_complex64 *w, int *m, s *s, s *sep, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void ctrsen(char *job, char *compq, bint *select, int *n, c *t, int *ldt, c *q, int *ldq, c *w, int *m, s *s, s *sep, c *work, int *lwork, int *info) nogil:
 *     _fortran_ctrsen(job, compq, select, n, <npy_complex64*>t, ldt, <npy_complex64*>q, ldq, <npy_complex64*>w, m, s, sep, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrsen,CTRSEN)(__pyx_v_job, __pyx_v_compq, __pyx_v_select, __pyx_v_n, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_w), __pyx_v_m, __pyx_v_s, __pyx_v_sep, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrsen "F_FUNC(ctrsen,CTRSEN)"(char *job, char *compq, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *q, int *ldq, npy_complex64 *w, int *m, s *s, s *sep, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void ctrsen(char *job, char *compq, bint *select, int *n, c *t, int *ldt, c *q, int *ldq, c *w, int *m, s *s, s *sep, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrsen(job, compq, select, n, <npy_complex64*>t, ldt, <npy_complex64*>q, ldq, <npy_complex64*>w, m, s, sep, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrsna "F_FUNC(ctrsna,CTRSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, s *s, s *sep, int *mm, int *m, npy_complex64 *work, int *ldwork, s *rwork, int *info) nogil
 * cdef void ctrsna(char *job, char *howmny, bint *select, int *n, c *t, int *ldt, c *vl, int *ldvl, c *vr, int *ldvr, s *s, s *sep, int *mm, int *m, c *work, int *ldwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrsna(job, howmny, select, n, <npy_complex64*>t, ldt, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, s, sep, mm, m, <npy_complex64*>work, ldwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrsna(char *__pyx_v_job, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_float_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_float_complex *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sep, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_ldwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3177
 *     void _fortran_ctrsna "F_FUNC(ctrsna,CTRSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, s *s, s *sep, int *mm, int *m, npy_complex64 *work, int *ldwork, s *rwork, int *info) nogil
 * cdef void ctrsna(char *job, char *howmny, bint *select, int *n, c *t, int *ldt, c *vl, int *ldvl, c *vr, int *ldvr, s *s, s *sep, int *mm, int *m, c *work, int *ldwork, s *rwork, int *info) nogil:
 *     _fortran_ctrsna(job, howmny, select, n, <npy_complex64*>t, ldt, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, s, sep, mm, m, <npy_complex64*>work, ldwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrsna,CTRSNA)(__pyx_v_job, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, ((npy_complex64 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex64 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex64 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_s, __pyx_v_sep, __pyx_v_mm, __pyx_v_m, ((npy_complex64 *)__pyx_v_work), __pyx_v_ldwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrsna "F_FUNC(ctrsna,CTRSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex64 *t, int *ldt, npy_complex64 *vl, int *ldvl, npy_complex64 *vr, int *ldvr, s *s, s *sep, int *mm, int *m, npy_complex64 *work, int *ldwork, s *rwork, int *info) nogil
 * cdef void ctrsna(char *job, char *howmny, bint *select, int *n, c *t, int *ldt, c *vl, int *ldvl, c *vr, int *ldvr, s *s, s *sep, int *mm, int *m, c *work, int *ldwork, s *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrsna(job, howmny, select, n, <npy_complex64*>t, ldt, <npy_complex64*>vl, ldvl, <npy_complex64*>vr, ldvr, s, sep, mm, m, <npy_complex64*>work, ldwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrsyl "F_FUNC(ctrsyl,CTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, s *scale, int *info) nogil
 * cdef void ctrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, s *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrsyl(trana, tranb, isgn, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, scale, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrsyl(char *__pyx_v_trana, char *__pyx_v_tranb, int *__pyx_v_isgn, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3182
 *     void _fortran_ctrsyl "F_FUNC(ctrsyl,CTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, s *scale, int *info) nogil
 * cdef void ctrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, s *scale, int *info) nogil:
 *     _fortran_ctrsyl(trana, tranb, isgn, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, scale, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrsyl,CTRSYL)(__pyx_v_trana, __pyx_v_tranb, __pyx_v_isgn, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, __pyx_v_scale, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrsyl "F_FUNC(ctrsyl,CTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, npy_complex64 *c, int *ldc, s *scale, int *info) nogil
 * cdef void ctrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, c *a, int *lda, c *b, int *ldb, c *c, int *ldc, s *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrsyl(trana, tranb, isgn, m, n, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, <npy_complex64*>c, ldc, scale, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrti2 "F_FUNC(ctrti2,CTRTI2)"(char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void ctrti2(char *uplo, char *diag, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrti2(uplo, diag, n, <npy_complex64*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrti2(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3187
 *     void _fortran_ctrti2 "F_FUNC(ctrti2,CTRTI2)"(char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void ctrti2(char *uplo, char *diag, int *n, c *a, int *lda, int *info) nogil:
 *     _fortran_ctrti2(uplo, diag, n, <npy_complex64*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrti2,CTRTI2)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrti2 "F_FUNC(ctrti2,CTRTI2)"(char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void ctrti2(char *uplo, char *diag, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrti2(uplo, diag, n, <npy_complex64*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrtri "F_FUNC(ctrtri,CTRTRI)"(char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void ctrtri(char *uplo, char *diag, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrtri(uplo, diag, n, <npy_complex64*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrtri(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3192
 *     void _fortran_ctrtri "F_FUNC(ctrtri,CTRTRI)"(char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void ctrtri(char *uplo, char *diag, int *n, c *a, int *lda, int *info) nogil:
 *     _fortran_ctrtri(uplo, diag, n, <npy_complex64*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrtri,CTRTRI)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrtri "F_FUNC(ctrtri,CTRTRI)"(char *uplo, char *diag, int *n, npy_complex64 *a, int *lda, int *info) nogil
 * cdef void ctrtri(char *uplo, char *diag, int *n, c *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrtri(uplo, diag, n, <npy_complex64*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrtrs "F_FUNC(ctrtrs,CTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrtrs(uplo, trans, diag, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctrtrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3197
 *     void _fortran_ctrtrs "F_FUNC(ctrtrs,CTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:
 *     _fortran_ctrtrs(uplo, trans, diag, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctrtrs,CTRTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctrtrs "F_FUNC(ctrtrs,CTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex64 *a, int *lda, npy_complex64 *b, int *ldb, int *info) nogil
 * cdef void ctrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, c *a, int *lda, c *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctrtrs(uplo, trans, diag, n, nrhs, <npy_complex64*>a, lda, <npy_complex64*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctzrzf "F_FUNC(ctzrzf,CTZRZF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void ctzrzf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctzrzf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ctzrzf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3202
 *     void _fortran_ctzrzf "F_FUNC(ctzrzf,CTZRZF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void ctzrzf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_ctzrzf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ctzrzf,CTZRZF)(__pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ctzrzf "F_FUNC(ctzrzf,CTZRZF)"(int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void ctzrzf(int *m, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ctzrzf(m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cung2l "F_FUNC(cung2l,CUNG2L)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cung2l(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cung2l(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cung2l(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3207
 *     void _fortran_cung2l "F_FUNC(cung2l,CUNG2L)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cung2l(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cung2l(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cung2l,CUNG2L)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cung2l "F_FUNC(cung2l,CUNG2L)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cung2l(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cung2l(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cung2r "F_FUNC(cung2r,CUNG2R)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cung2r(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cung2r(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cung2r(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3212
 *     void _fortran_cung2r "F_FUNC(cung2r,CUNG2R)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cung2r(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cung2r(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cung2r,CUNG2R)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cung2r "F_FUNC(cung2r,CUNG2R)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cung2r(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cung2r(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungbr "F_FUNC(cungbr,CUNGBR)"(char *vect, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungbr(char *vect, int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungbr(vect, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cungbr(char *__pyx_v_vect, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3217
 *     void _fortran_cungbr "F_FUNC(cungbr,CUNGBR)"(char *vect, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungbr(char *vect, int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cungbr(vect, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cungbr,CUNGBR)(__pyx_v_vect, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungbr "F_FUNC(cungbr,CUNGBR)"(char *vect, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungbr(char *vect, int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungbr(vect, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunghr "F_FUNC(cunghr,CUNGHR)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunghr(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunghr(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunghr(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3222
 *     void _fortran_cunghr "F_FUNC(cunghr,CUNGHR)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunghr(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunghr(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunghr,CUNGHR)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunghr "F_FUNC(cunghr,CUNGHR)"(int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunghr(int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunghr(n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungl2 "F_FUNC(cungl2,CUNGL2)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cungl2(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungl2(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cungl2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3227
 *     void _fortran_cungl2 "F_FUNC(cungl2,CUNGL2)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cungl2(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cungl2(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cungl2,CUNGL2)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungl2 "F_FUNC(cungl2,CUNGL2)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cungl2(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungl2(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunglq "F_FUNC(cunglq,CUNGLQ)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunglq(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunglq(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunglq(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3232
 *     void _fortran_cunglq "F_FUNC(cunglq,CUNGLQ)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunglq(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunglq(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunglq,CUNGLQ)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunglq "F_FUNC(cunglq,CUNGLQ)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunglq(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunglq(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungql "F_FUNC(cungql,CUNGQL)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungql(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungql(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cungql(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3237
 *     void _fortran_cungql "F_FUNC(cungql,CUNGQL)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungql(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cungql(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cungql,CUNGQL)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungql "F_FUNC(cungql,CUNGQL)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungql(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungql(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungqr "F_FUNC(cungqr,CUNGQR)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungqr(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungqr(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cungqr(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3242
 *     void _fortran_cungqr "F_FUNC(cungqr,CUNGQR)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungqr(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cungqr(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cungqr,CUNGQR)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungqr "F_FUNC(cungqr,CUNGQR)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungqr(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungqr(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungr2 "F_FUNC(cungr2,CUNGR2)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cungr2(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungr2(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cungr2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3247
 *     void _fortran_cungr2 "F_FUNC(cungr2,CUNGR2)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cungr2(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:
 *     _fortran_cungr2(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cungr2,CUNGR2)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungr2 "F_FUNC(cungr2,CUNGR2)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *info) nogil
 * cdef void cungr2(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungr2(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungrq "F_FUNC(cungrq,CUNGRQ)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungrq(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungrq(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cungrq(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3252
 *     void _fortran_cungrq "F_FUNC(cungrq,CUNGRQ)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungrq(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cungrq(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cungrq,CUNGRQ)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungrq "F_FUNC(cungrq,CUNGRQ)"(int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungrq(int *m, int *n, int *k, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungrq(m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungtr "F_FUNC(cungtr,CUNGTR)"(char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungtr(char *uplo, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungtr(uplo, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cungtr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3257
 *     void _fortran_cungtr "F_FUNC(cungtr,CUNGTR)"(char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungtr(char *uplo, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:
 *     _fortran_cungtr(uplo, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cungtr,CUNGTR)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cungtr "F_FUNC(cungtr,CUNGTR)"(char *uplo, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cungtr(char *uplo, int *n, c *a, int *lda, c *tau, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cungtr(uplo, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunm2l "F_FUNC(cunm2l,CUNM2L)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunm2l(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunm2l(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunm2l(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3262
 *     void _fortran_cunm2l "F_FUNC(cunm2l,CUNM2L)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunm2l(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:
 *     _fortran_cunm2l(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunm2l,CUNM2L)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunm2l "F_FUNC(cunm2l,CUNM2L)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunm2l(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunm2l(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunm2r "F_FUNC(cunm2r,CUNM2R)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunm2r(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunm2r(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunm2r(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3267
 *     void _fortran_cunm2r "F_FUNC(cunm2r,CUNM2R)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunm2r(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:
 *     _fortran_cunm2r(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunm2r,CUNM2R)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunm2r "F_FUNC(cunm2r,CUNM2R)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunm2r(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunm2r(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmbr "F_FUNC(cunmbr,CUNMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmbr(char *vect, char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmbr(vect, side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmbr(char *__pyx_v_vect, char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3272
 *     void _fortran_cunmbr "F_FUNC(cunmbr,CUNMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmbr(char *vect, char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunmbr(vect, side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmbr,CUNMBR)(__pyx_v_vect, __pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmbr "F_FUNC(cunmbr,CUNMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmbr(char *vect, char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmbr(vect, side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmhr "F_FUNC(cunmhr,CUNMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmhr(side, trans, m, n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmhr(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3277
 *     void _fortran_cunmhr "F_FUNC(cunmhr,CUNMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunmhr(side, trans, m, n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmhr,CUNMHR)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmhr "F_FUNC(cunmhr,CUNMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmhr(side, trans, m, n, ilo, ihi, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunml2 "F_FUNC(cunml2,CUNML2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunml2(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunml2(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunml2(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3282
 *     void _fortran_cunml2 "F_FUNC(cunml2,CUNML2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunml2(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:
 *     _fortran_cunml2(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunml2,CUNML2)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunml2 "F_FUNC(cunml2,CUNML2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunml2(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunml2(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmlq "F_FUNC(cunmlq,CUNMLQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmlq(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmlq(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmlq(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3287
 *     void _fortran_cunmlq "F_FUNC(cunmlq,CUNMLQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmlq(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunmlq(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmlq,CUNMLQ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmlq "F_FUNC(cunmlq,CUNMLQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmlq(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmlq(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmql "F_FUNC(cunmql,CUNMQL)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmql(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmql(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmql(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3292
 *     void _fortran_cunmql "F_FUNC(cunmql,CUNMQL)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmql(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunmql(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmql,CUNMQL)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmql "F_FUNC(cunmql,CUNMQL)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmql(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmql(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmqr "F_FUNC(cunmqr,CUNMQR)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmqr(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmqr(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmqr(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3297
 *     void _fortran_cunmqr "F_FUNC(cunmqr,CUNMQR)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmqr(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunmqr(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmqr,CUNMQR)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmqr "F_FUNC(cunmqr,CUNMQR)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmqr(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmqr(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmr2 "F_FUNC(cunmr2,CUNMR2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunmr2(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmr2(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmr2(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3302
 *     void _fortran_cunmr2 "F_FUNC(cunmr2,CUNMR2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunmr2(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:
 *     _fortran_cunmr2(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmr2,CUNMR2)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmr2 "F_FUNC(cunmr2,CUNMR2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunmr2(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmr2(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmr3 "F_FUNC(cunmr3,CUNMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunmr3(char *side, char *trans, int *m, int *n, int *k, int *l, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmr3(side, trans, m, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmr3(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3307
 *     void _fortran_cunmr3 "F_FUNC(cunmr3,CUNMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunmr3(char *side, char *trans, int *m, int *n, int *k, int *l, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:
 *     _fortran_cunmr3(side, trans, m, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmr3,CUNMR3)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmr3 "F_FUNC(cunmr3,CUNMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cunmr3(char *side, char *trans, int *m, int *n, int *k, int *l, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmr3(side, trans, m, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmrq "F_FUNC(cunmrq,CUNMRQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmrq(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmrq(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmrq(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3312
 *     void _fortran_cunmrq "F_FUNC(cunmrq,CUNMRQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmrq(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunmrq(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmrq,CUNMRQ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmrq "F_FUNC(cunmrq,CUNMRQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmrq(char *side, char *trans, int *m, int *n, int *k, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmrq(side, trans, m, n, k, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmrz "F_FUNC(cunmrz,CUNMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmrz(char *side, char *trans, int *m, int *n, int *k, int *l, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmrz(side, trans, m, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmrz(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3317
 *     void _fortran_cunmrz "F_FUNC(cunmrz,CUNMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmrz(char *side, char *trans, int *m, int *n, int *k, int *l, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunmrz(side, trans, m, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmrz,CUNMRZ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmrz "F_FUNC(cunmrz,CUNMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmrz(char *side, char *trans, int *m, int *n, int *k, int *l, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmrz(side, trans, m, n, k, l, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmtr "F_FUNC(cunmtr,CUNMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmtr(char *side, char *uplo, char *trans, int *m, int *n, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmtr(side, uplo, trans, m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cunmtr(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3322
 *     void _fortran_cunmtr "F_FUNC(cunmtr,CUNMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmtr(char *side, char *uplo, char *trans, int *m, int *n, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:
 *     _fortran_cunmtr(side, uplo, trans, m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cunmtr,CUNMTR)(__pyx_v_side, __pyx_v_uplo, __pyx_v_trans, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cunmtr "F_FUNC(cunmtr,CUNMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex64 *a, int *lda, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *lwork, int *info) nogil
 * cdef void cunmtr(char *side, char *uplo, char *trans, int *m, int *n, c *a, int *lda, c *tau, c *c, int *ldc, c *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cunmtr(side, uplo, trans, m, n, <npy_complex64*>a, lda, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cupgtr "F_FUNC(cupgtr,CUPGTR)"(char *uplo, int *n, npy_complex64 *ap, npy_complex64 *tau, npy_complex64 *q, int *ldq, npy_complex64 *work, int *info) nogil
 * cdef void cupgtr(char *uplo, int *n, c *ap, c *tau, c *q, int *ldq, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cupgtr(uplo, n, <npy_complex64*>ap, <npy_complex64*>tau, <npy_complex64*>q, ldq, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cupgtr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3327
 *     void _fortran_cupgtr "F_FUNC(cupgtr,CUPGTR)"(char *uplo, int *n, npy_complex64 *ap, npy_complex64 *tau, npy_complex64 *q, int *ldq, npy_complex64 *work, int *info) nogil
 * cdef void cupgtr(char *uplo, int *n, c *ap, c *tau, c *q, int *ldq, c *work, int *info) nogil:
 *     _fortran_cupgtr(uplo, n, <npy_complex64*>ap, <npy_complex64*>tau, <npy_complex64*>q, ldq, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cupgtr,CUPGTR)(__pyx_v_uplo, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cupgtr "F_FUNC(cupgtr,CUPGTR)"(char *uplo, int *n, npy_complex64 *ap, npy_complex64 *tau, npy_complex64 *q, int *ldq, npy_complex64 *work, int *info) nogil
 * cdef void cupgtr(char *uplo, int *n, c *ap, c *tau, c *q, int *ldq, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cupgtr(uplo, n, <npy_complex64*>ap, <npy_complex64*>tau, <npy_complex64*>q, ldq, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cupmtr "F_FUNC(cupmtr,CUPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex64 *ap, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cupmtr(char *side, char *uplo, char *trans, int *m, int *n, c *ap, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cupmtr(side, uplo, trans, m, n, <npy_complex64*>ap, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_cupmtr(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_float_complex *__pyx_v_ap, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3332
 *     void _fortran_cupmtr "F_FUNC(cupmtr,CUPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex64 *ap, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cupmtr(char *side, char *uplo, char *trans, int *m, int *n, c *ap, c *tau, c *c, int *ldc, c *work, int *info) nogil:
 *     _fortran_cupmtr(side, uplo, trans, m, n, <npy_complex64*>ap, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(cupmtr,CUPMTR)(__pyx_v_side, __pyx_v_uplo, __pyx_v_trans, __pyx_v_m, __pyx_v_n, ((npy_complex64 *)__pyx_v_ap), ((npy_complex64 *)__pyx_v_tau), ((npy_complex64 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex64 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_cupmtr "F_FUNC(cupmtr,CUPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex64 *ap, npy_complex64 *tau, npy_complex64 *c, int *ldc, npy_complex64 *work, int *info) nogil
 * cdef void cupmtr(char *side, char *uplo, char *trans, int *m, int *n, c *ap, c *tau, c *c, int *ldc, c *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_cupmtr(side, uplo, trans, m, n, <npy_complex64*>ap, <npy_complex64*>tau, <npy_complex64*>c, ldc, <npy_complex64*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dbdsdc "F_FUNC(dbdsdc,DBDSDC)"(char *uplo, char *compq, int *n, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, d *q, int *iq, d *work, int *iwork, int *info) nogil
 * cdef void dbdsdc(char *uplo, char *compq, int *n, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, d *q, int *iq, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dbdsdc(uplo, compq, n, d, e, u, ldu, vt, ldvt, q, iq, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dbdsdc(char *__pyx_v_uplo, char *__pyx_v_compq, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_iq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3337
 *     void _fortran_dbdsdc "F_FUNC(dbdsdc,DBDSDC)"(char *uplo, char *compq, int *n, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, d *q, int *iq, d *work, int *iwork, int *info) nogil
 * cdef void dbdsdc(char *uplo, char *compq, int *n, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, d *q, int *iq, d *work, int *iwork, int *info) nogil:
 *     _fortran_dbdsdc(uplo, compq, n, d, e, u, ldu, vt, ldvt, q, iq, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dbdsdc,DBDSDC)(__pyx_v_uplo, __pyx_v_compq, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_q, __pyx_v_iq, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dbdsdc "F_FUNC(dbdsdc,DBDSDC)"(char *uplo, char *compq, int *n, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, d *q, int *iq, d *work, int *iwork, int *info) nogil
 * cdef void dbdsdc(char *uplo, char *compq, int *n, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, d *q, int *iq, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dbdsdc(uplo, compq, n, d, e, u, ldu, vt, ldvt, q, iq, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dbdsqr "F_FUNC(dbdsqr,DBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dbdsqr(uplo, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dbdsqr(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ncvt, int *__pyx_v_nru, int *__pyx_v_ncc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3342
 *     void _fortran_dbdsqr "F_FUNC(dbdsqr,DBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dbdsqr(uplo, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dbdsqr,DBDSQR)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ncvt, __pyx_v_nru, __pyx_v_ncc, __pyx_v_d, __pyx_v_e, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_u, __pyx_v_ldu, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dbdsqr "F_FUNC(dbdsqr,DBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dbdsqr(uplo, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ddisna "F_FUNC(ddisna,DDISNA)"(char *job, int *m, int *n, d *d, d *sep, int *info) nogil
 * cdef void ddisna(char *job, int *m, int *n, d *d, d *sep, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ddisna(job, m, n, d, sep, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ddisna(char *__pyx_v_job, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sep, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3347
 *     void _fortran_ddisna "F_FUNC(ddisna,DDISNA)"(char *job, int *m, int *n, d *d, d *sep, int *info) nogil
 * cdef void ddisna(char *job, int *m, int *n, d *d, d *sep, int *info) nogil:
 *     _fortran_ddisna(job, m, n, d, sep, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ddisna,DDISNA)(__pyx_v_job, __pyx_v_m, __pyx_v_n, __pyx_v_d, __pyx_v_sep, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ddisna "F_FUNC(ddisna,DDISNA)"(char *job, int *m, int *n, d *d, d *sep, int *info) nogil
 * cdef void ddisna(char *job, int *m, int *n, d *d, d *sep, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ddisna(job, m, n, d, sep, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbbrd "F_FUNC(dgbbrd,DGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *pt, int *ldpt, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *pt, int *ldpt, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbbrd(vect, m, n, ncc, kl, ku, ab, ldab, d, e, q, ldq, pt, ldpt, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbbrd(char *__pyx_v_vect, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_ncc, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pt, int *__pyx_v_ldpt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3352
 *     void _fortran_dgbbrd "F_FUNC(dgbbrd,DGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *pt, int *ldpt, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *pt, int *ldpt, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dgbbrd(vect, m, n, ncc, kl, ku, ab, ldab, d, e, q, ldq, pt, ldpt, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbbrd,DGBBRD)(__pyx_v_vect, __pyx_v_m, __pyx_v_n, __pyx_v_ncc, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_d, __pyx_v_e, __pyx_v_q, __pyx_v_ldq, __pyx_v_pt, __pyx_v_ldpt, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbbrd "F_FUNC(dgbbrd,DGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *pt, int *ldpt, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *pt, int *ldpt, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbbrd(vect, m, n, ncc, kl, ku, ab, ldab, d, e, q, ldq, pt, ldpt, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbcon "F_FUNC(dgbcon,DGBCON)"(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgbcon(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbcon(norm, n, kl, ku, ab, ldab, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbcon(char *__pyx_v_norm, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3357
 *     void _fortran_dgbcon "F_FUNC(dgbcon,DGBCON)"(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgbcon(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgbcon(norm, n, kl, ku, ab, ldab, ipiv, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbcon,DGBCON)(__pyx_v_norm, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbcon "F_FUNC(dgbcon,DGBCON)"(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgbcon(char *norm, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbcon(norm, n, kl, ku, ab, ldab, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbequ "F_FUNC(dgbequ,DGBEQU)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void dgbequ(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbequ(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbequ(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3362
 *     void _fortran_dgbequ "F_FUNC(dgbequ,DGBEQU)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void dgbequ(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:
 *     _fortran_dgbequ(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbequ,DGBEQU)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbequ "F_FUNC(dgbequ,DGBEQU)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void dgbequ(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbequ(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbrfs "F_FUNC(dgbrfs,DGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbrfs(trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbrfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_afb, int *__pyx_v_ldafb, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3367
 *     void _fortran_dgbrfs "F_FUNC(dgbrfs,DGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgbrfs(trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbrfs,DGBRFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_afb, __pyx_v_ldafb, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbrfs "F_FUNC(dgbrfs,DGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbrfs(trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbsv "F_FUNC(dgbsv,DGBSV)"(int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgbsv(int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbsv(n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbsv(int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3372
 *     void _fortran_dgbsv "F_FUNC(dgbsv,DGBSV)"(int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgbsv(int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil:
 *     _fortran_dgbsv(n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbsv,DGBSV)(__pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbsv "F_FUNC(dgbsv,DGBSV)"(int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgbsv(int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbsv(n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbsvx "F_FUNC(dgbsvx,DGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbsvx(fact, trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbsvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_afb, int *__pyx_v_ldafb, int *__pyx_v_ipiv, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3377
 *     void _fortran_dgbsvx "F_FUNC(dgbsvx,DGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgbsvx(fact, trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbsvx,DGBSVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_afb, __pyx_v_ldafb, __pyx_v_ipiv, __pyx_v_equed, __pyx_v_r, __pyx_v_c, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbsvx "F_FUNC(dgbsvx,DGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbsvx(fact, trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbtf2 "F_FUNC(dgbtf2,DGBTF2)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void dgbtf2(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbtf2(m, n, kl, ku, ab, ldab, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbtf2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3382
 *     void _fortran_dgbtf2 "F_FUNC(dgbtf2,DGBTF2)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void dgbtf2(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil:
 *     _fortran_dgbtf2(m, n, kl, ku, ab, ldab, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbtf2,DGBTF2)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbtf2 "F_FUNC(dgbtf2,DGBTF2)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void dgbtf2(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbtf2(m, n, kl, ku, ab, ldab, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbtrf "F_FUNC(dgbtrf,DGBTRF)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void dgbtrf(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbtrf(m, n, kl, ku, ab, ldab, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbtrf(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3387
 *     void _fortran_dgbtrf "F_FUNC(dgbtrf,DGBTRF)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void dgbtrf(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil:
 *     _fortran_dgbtrf(m, n, kl, ku, ab, ldab, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbtrf,DGBTRF)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbtrf "F_FUNC(dgbtrf,DGBTRF)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void dgbtrf(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbtrf(m, n, kl, ku, ab, ldab, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbtrs "F_FUNC(dgbtrs,DGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbtrs(trans, n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgbtrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3392
 *     void _fortran_dgbtrs "F_FUNC(dgbtrs,DGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil:
 *     _fortran_dgbtrs(trans, n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgbtrs,DGBTRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgbtrs "F_FUNC(dgbtrs,DGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, d *ab, int *ldab, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgbtrs(trans, n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgebak "F_FUNC(dgebak,DGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, d *v, int *ldv, int *info) nogil
 * cdef void dgebak(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, d *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgebak(job, side, n, ilo, ihi, scale, m, v, ldv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgebak(char *__pyx_v_job, char *__pyx_v_side, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3397
 *     void _fortran_dgebak "F_FUNC(dgebak,DGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, d *v, int *ldv, int *info) nogil
 * cdef void dgebak(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, d *v, int *ldv, int *info) nogil:
 *     _fortran_dgebak(job, side, n, ilo, ihi, scale, m, v, ldv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgebak,DGEBAK)(__pyx_v_job, __pyx_v_side, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_m, __pyx_v_v, __pyx_v_ldv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgebak "F_FUNC(dgebak,DGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, d *v, int *ldv, int *info) nogil
 * cdef void dgebak(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, d *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgebak(job, side, n, ilo, ihi, scale, m, v, ldv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgebal "F_FUNC(dgebal,DGEBAL)"(char *job, int *n, d *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil
 * cdef void dgebal(char *job, int *n, d *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgebal(job, n, a, lda, ilo, ihi, scale, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgebal(char *__pyx_v_job, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3402
 *     void _fortran_dgebal "F_FUNC(dgebal,DGEBAL)"(char *job, int *n, d *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil
 * cdef void dgebal(char *job, int *n, d *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil:
 *     _fortran_dgebal(job, n, a, lda, ilo, ihi, scale, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgebal,DGEBAL)(__pyx_v_job, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgebal "F_FUNC(dgebal,DGEBAL)"(char *job, int *n, d *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil
 * cdef void dgebal(char *job, int *n, d *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgebal(job, n, a, lda, ilo, ihi, scale, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgebd2 "F_FUNC(dgebd2,DGEBD2)"(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *info) nogil
 * cdef void dgebd2(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgebd2(m, n, a, lda, d, e, tauq, taup, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgebd2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tauq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_taup, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3407
 *     void _fortran_dgebd2 "F_FUNC(dgebd2,DGEBD2)"(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *info) nogil
 * cdef void dgebd2(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *info) nogil:
 *     _fortran_dgebd2(m, n, a, lda, d, e, tauq, taup, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgebd2,DGEBD2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tauq, __pyx_v_taup, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgebd2 "F_FUNC(dgebd2,DGEBD2)"(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *info) nogil
 * cdef void dgebd2(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgebd2(m, n, a, lda, d, e, tauq, taup, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgebrd "F_FUNC(dgebrd,DGEBRD)"(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *lwork, int *info) nogil
 * cdef void dgebrd(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgebrd(m, n, a, lda, d, e, tauq, taup, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgebrd(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tauq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_taup, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3412
 *     void _fortran_dgebrd "F_FUNC(dgebrd,DGEBRD)"(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *lwork, int *info) nogil
 * cdef void dgebrd(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgebrd(m, n, a, lda, d, e, tauq, taup, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgebrd,DGEBRD)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tauq, __pyx_v_taup, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgebrd "F_FUNC(dgebrd,DGEBRD)"(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *lwork, int *info) nogil
 * cdef void dgebrd(int *m, int *n, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgebrd(m, n, a, lda, d, e, tauq, taup, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgecon "F_FUNC(dgecon,DGECON)"(char *norm, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgecon(char *norm, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgecon(norm, n, a, lda, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgecon(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3417
 *     void _fortran_dgecon "F_FUNC(dgecon,DGECON)"(char *norm, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgecon(char *norm, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgecon(norm, n, a, lda, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgecon,DGECON)(__pyx_v_norm, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgecon "F_FUNC(dgecon,DGECON)"(char *norm, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgecon(char *norm, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgecon(norm, n, a, lda, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeequ "F_FUNC(dgeequ,DGEEQU)"(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void dgeequ(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeequ(m, n, a, lda, r, c, rowcnd, colcnd, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeequ(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3422
 *     void _fortran_dgeequ "F_FUNC(dgeequ,DGEEQU)"(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void dgeequ(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:
 *     _fortran_dgeequ(m, n, a, lda, r, c, rowcnd, colcnd, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeequ,DGEEQU)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeequ "F_FUNC(dgeequ,DGEEQU)"(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void dgeequ(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeequ(m, n, a, lda, r, c, rowcnd, colcnd, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgees "F_FUNC(dgees,DGEES)"(char *jobvs, char *sort, _dselect2 *select, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void dgees(char *jobvs, char *sort, dselect2 *select, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *work, int *lwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgees(jobvs, sort, <_dselect2*>select, n, a, lda, sdim, wr, wi, vs, ldvs, work, lwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgees(char *__pyx_v_jobvs, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_dselect2 *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_sdim, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vs, int *__pyx_v_ldvs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3427
 *     void _fortran_dgees "F_FUNC(dgees,DGEES)"(char *jobvs, char *sort, _dselect2 *select, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void dgees(char *jobvs, char *sort, dselect2 *select, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *work, int *lwork, bint *bwork, int *info) nogil:
 *     _fortran_dgees(jobvs, sort, <_dselect2*>select, n, a, lda, sdim, wr, wi, vs, ldvs, work, lwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgees,DGEES)(__pyx_v_jobvs, __pyx_v_sort, ((_dselect2 *)__pyx_v_select), __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_sdim, __pyx_v_wr, __pyx_v_wi, __pyx_v_vs, __pyx_v_ldvs, __pyx_v_work, __pyx_v_lwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgees "F_FUNC(dgees,DGEES)"(char *jobvs, char *sort, _dselect2 *select, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void dgees(char *jobvs, char *sort, dselect2 *select, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *work, int *lwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgees(jobvs, sort, <_dselect2*>select, n, a, lda, sdim, wr, wi, vs, ldvs, work, lwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeesx "F_FUNC(dgeesx,DGEESX)"(char *jobvs, char *sort, _dselect2 *select, char *sense, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void dgeesx(char *jobvs, char *sort, dselect2 *select, char *sense, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeesx(jobvs, sort, <_dselect2*>select, sense, n, a, lda, sdim, wr, wi, vs, ldvs, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeesx(char *__pyx_v_jobvs, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_dselect2 *__pyx_v_select, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_sdim, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vs, int *__pyx_v_ldvs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcondv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3432
 *     void _fortran_dgeesx "F_FUNC(dgeesx,DGEESX)"(char *jobvs, char *sort, _dselect2 *select, char *sense, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void dgeesx(char *jobvs, char *sort, dselect2 *select, char *sense, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:
 *     _fortran_dgeesx(jobvs, sort, <_dselect2*>select, sense, n, a, lda, sdim, wr, wi, vs, ldvs, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeesx,DGEESX)(__pyx_v_jobvs, __pyx_v_sort, ((_dselect2 *)__pyx_v_select), __pyx_v_sense, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_sdim, __pyx_v_wr, __pyx_v_wi, __pyx_v_vs, __pyx_v_ldvs, __pyx_v_rconde, __pyx_v_rcondv, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeesx "F_FUNC(dgeesx,DGEESX)"(char *jobvs, char *sort, _dselect2 *select, char *sense, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void dgeesx(char *jobvs, char *sort, dselect2 *select, char *sense, int *n, d *a, int *lda, int *sdim, d *wr, d *wi, d *vs, int *ldvs, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeesx(jobvs, sort, <_dselect2*>select, sense, n, a, lda, sdim, wr, wi, vs, ldvs, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeev "F_FUNC(dgeev,DGEEV)"(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil
 * cdef void dgeev(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeev(jobvl, jobvr, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeev(char *__pyx_v_jobvl, char *__pyx_v_jobvr, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3437
 *     void _fortran_dgeev "F_FUNC(dgeev,DGEEV)"(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil
 * cdef void dgeev(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgeev(jobvl, jobvr, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeev,DGEEV)(__pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_wr, __pyx_v_wi, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeev "F_FUNC(dgeev,DGEEV)"(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil
 * cdef void dgeev(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeev(jobvl, jobvr, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeevx "F_FUNC(dgeevx,DGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeevx(balanc, jobvl, jobvr, sense, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeevx(char *__pyx_v_balanc, char *__pyx_v_jobvl, char *__pyx_v_jobvr, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abnrm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcondv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3442
 *     void _fortran_dgeevx "F_FUNC(dgeevx,DGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_dgeevx(balanc, jobvl, jobvr, sense, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeevx,DGEEVX)(__pyx_v_balanc, __pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_sense, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_wr, __pyx_v_wi, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_abnrm, __pyx_v_rconde, __pyx_v_rcondv, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeevx "F_FUNC(dgeevx,DGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeevx(balanc, jobvl, jobvr, sense, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgehd2 "F_FUNC(dgehd2,DGEHD2)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgehd2(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgehd2(n, ilo, ihi, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgehd2(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3447
 *     void _fortran_dgehd2 "F_FUNC(dgehd2,DGEHD2)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgehd2(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dgehd2(n, ilo, ihi, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgehd2,DGEHD2)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgehd2 "F_FUNC(dgehd2,DGEHD2)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgehd2(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgehd2(n, ilo, ihi, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgehrd "F_FUNC(dgehrd,DGEHRD)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgehrd(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgehrd(n, ilo, ihi, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgehrd(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3452
 *     void _fortran_dgehrd "F_FUNC(dgehrd,DGEHRD)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgehrd(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgehrd(n, ilo, ihi, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgehrd,DGEHRD)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgehrd "F_FUNC(dgehrd,DGEHRD)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgehrd(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgehrd(n, ilo, ihi, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelq2 "F_FUNC(dgelq2,DGELQ2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgelq2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelq2(m, n, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgelq2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3457
 *     void _fortran_dgelq2 "F_FUNC(dgelq2,DGELQ2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgelq2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dgelq2(m, n, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgelq2,DGELQ2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelq2 "F_FUNC(dgelq2,DGELQ2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgelq2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelq2(m, n, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelqf "F_FUNC(dgelqf,DGELQF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgelqf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelqf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgelqf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3462
 *     void _fortran_dgelqf "F_FUNC(dgelqf,DGELQF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgelqf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgelqf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgelqf,DGELQF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelqf "F_FUNC(dgelqf,DGELQF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgelqf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelqf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgels "F_FUNC(dgels,DGELS)"(char *trans, int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *work, int *lwork, int *info) nogil
 * cdef void dgels(char *trans, int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgels(trans, m, n, nrhs, a, lda, b, ldb, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgels(char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3467
 *     void _fortran_dgels "F_FUNC(dgels,DGELS)"(char *trans, int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *work, int *lwork, int *info) nogil
 * cdef void dgels(char *trans, int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgels(trans, m, n, nrhs, a, lda, b, ldb, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgels,DGELS)(__pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgels "F_FUNC(dgels,DGELS)"(char *trans, int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *work, int *lwork, int *info) nogil
 * cdef void dgels(char *trans, int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgels(trans, m, n, nrhs, a, lda, b, ldb, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelsd "F_FUNC(dgelsd,DGELSD)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgelsd(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelsd(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgelsd(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3472
 *     void _fortran_dgelsd "F_FUNC(dgelsd,DGELSD)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgelsd(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_dgelsd(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgelsd,DGELSD)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_s, __pyx_v_rcond, __pyx_v_rank, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelsd "F_FUNC(dgelsd,DGELSD)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgelsd(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelsd(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelss "F_FUNC(dgelss,DGELSS)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *info) nogil
 * cdef void dgelss(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelss(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgelss(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3477
 *     void _fortran_dgelss "F_FUNC(dgelss,DGELSS)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *info) nogil
 * cdef void dgelss(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgelss(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgelss,DGELSS)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_s, __pyx_v_rcond, __pyx_v_rank, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelss "F_FUNC(dgelss,DGELSS)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *info) nogil
 * cdef void dgelss(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *s, d *rcond, int *rank, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelss(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelsy "F_FUNC(dgelsy,DGELSY)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *jpvt, d *rcond, int *rank, d *work, int *lwork, int *info) nogil
 * cdef void dgelsy(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *jpvt, d *rcond, int *rank, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelsy(m, n, nrhs, a, lda, b, ldb, jpvt, rcond, rank, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgelsy(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3482
 *     void _fortran_dgelsy "F_FUNC(dgelsy,DGELSY)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *jpvt, d *rcond, int *rank, d *work, int *lwork, int *info) nogil
 * cdef void dgelsy(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *jpvt, d *rcond, int *rank, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgelsy(m, n, nrhs, a, lda, b, ldb, jpvt, rcond, rank, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgelsy,DGELSY)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_jpvt, __pyx_v_rcond, __pyx_v_rank, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgelsy "F_FUNC(dgelsy,DGELSY)"(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *jpvt, d *rcond, int *rank, d *work, int *lwork, int *info) nogil
 * cdef void dgelsy(int *m, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *jpvt, d *rcond, int *rank, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgelsy(m, n, nrhs, a, lda, b, ldb, jpvt, rcond, rank, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeql2 "F_FUNC(dgeql2,DGEQL2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgeql2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeql2(m, n, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeql2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3487
 *     void _fortran_dgeql2 "F_FUNC(dgeql2,DGEQL2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgeql2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dgeql2(m, n, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeql2,DGEQL2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeql2 "F_FUNC(dgeql2,DGEQL2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgeql2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeql2(m, n, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeqlf "F_FUNC(dgeqlf,DGEQLF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqlf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeqlf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeqlf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3492
 *     void _fortran_dgeqlf "F_FUNC(dgeqlf,DGEQLF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqlf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgeqlf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeqlf,DGEQLF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeqlf "F_FUNC(dgeqlf,DGEQLF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqlf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeqlf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeqp3 "F_FUNC(dgeqp3,DGEQP3)"(int *m, int *n, d *a, int *lda, int *jpvt, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqp3(int *m, int *n, d *a, int *lda, int *jpvt, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeqp3(m, n, a, lda, jpvt, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeqp3(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3497
 *     void _fortran_dgeqp3 "F_FUNC(dgeqp3,DGEQP3)"(int *m, int *n, d *a, int *lda, int *jpvt, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqp3(int *m, int *n, d *a, int *lda, int *jpvt, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgeqp3(m, n, a, lda, jpvt, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeqp3,DGEQP3)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_jpvt, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeqp3 "F_FUNC(dgeqp3,DGEQP3)"(int *m, int *n, d *a, int *lda, int *jpvt, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqp3(int *m, int *n, d *a, int *lda, int *jpvt, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeqp3(m, n, a, lda, jpvt, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeqr2 "F_FUNC(dgeqr2,DGEQR2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgeqr2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeqr2(m, n, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeqr2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3502
 *     void _fortran_dgeqr2 "F_FUNC(dgeqr2,DGEQR2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgeqr2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dgeqr2(m, n, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeqr2,DGEQR2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeqr2 "F_FUNC(dgeqr2,DGEQR2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgeqr2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeqr2(m, n, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeqrf "F_FUNC(dgeqrf,DGEQRF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqrf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeqrf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgeqrf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3507
 *     void _fortran_dgeqrf "F_FUNC(dgeqrf,DGEQRF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqrf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgeqrf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgeqrf,DGEQRF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgeqrf "F_FUNC(dgeqrf,DGEQRF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgeqrf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgeqrf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgerfs "F_FUNC(dgerfs,DGERFS)"(char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgerfs(char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgerfs(trans, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgerfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3512
 *     void _fortran_dgerfs "F_FUNC(dgerfs,DGERFS)"(char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgerfs(char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgerfs(trans, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgerfs,DGERFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgerfs "F_FUNC(dgerfs,DGERFS)"(char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgerfs(char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgerfs(trans, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgerq2 "F_FUNC(dgerq2,DGERQ2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgerq2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgerq2(m, n, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgerq2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3517
 *     void _fortran_dgerq2 "F_FUNC(dgerq2,DGERQ2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgerq2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dgerq2(m, n, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgerq2,DGERQ2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgerq2 "F_FUNC(dgerq2,DGERQ2)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dgerq2(int *m, int *n, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgerq2(m, n, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgerqf "F_FUNC(dgerqf,DGERQF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgerqf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgerqf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgerqf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3522
 *     void _fortran_dgerqf "F_FUNC(dgerqf,DGERQF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgerqf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgerqf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgerqf,DGERQF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgerqf "F_FUNC(dgerqf,DGERQF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dgerqf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgerqf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesc2 "F_FUNC(dgesc2,DGESC2)"(int *n, d *a, int *lda, d *rhs, int *ipiv, int *jpiv, d *scale) nogil
 * cdef void dgesc2(int *n, d *a, int *lda, d *rhs, int *ipiv, int *jpiv, d *scale) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesc2(n, a, lda, rhs, ipiv, jpiv, scale)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgesc2(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rhs, int *__pyx_v_ipiv, int *__pyx_v_jpiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale) {

  /* "scipy/linalg/cython_lapack.pyx":3527
 *     void _fortran_dgesc2 "F_FUNC(dgesc2,DGESC2)"(int *n, d *a, int *lda, d *rhs, int *ipiv, int *jpiv, d *scale) nogil
 * cdef void dgesc2(int *n, d *a, int *lda, d *rhs, int *ipiv, int *jpiv, d *scale) nogil:
 *     _fortran_dgesc2(n, a, lda, rhs, ipiv, jpiv, scale)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgesc2,DGESC2)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_rhs, __pyx_v_ipiv, __pyx_v_jpiv, __pyx_v_scale);

  /* "scipy/linalg/cython_lapack.pyx":3526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesc2 "F_FUNC(dgesc2,DGESC2)"(int *n, d *a, int *lda, d *rhs, int *ipiv, int *jpiv, d *scale) nogil
 * cdef void dgesc2(int *n, d *a, int *lda, d *rhs, int *ipiv, int *jpiv, d *scale) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesc2(n, a, lda, rhs, ipiv, jpiv, scale)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesdd "F_FUNC(dgesdd,DGESDD)"(char *jobz, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgesdd(char *jobz, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesdd(jobz, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgesdd(char *__pyx_v_jobz, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3532
 *     void _fortran_dgesdd "F_FUNC(dgesdd,DGESDD)"(char *jobz, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgesdd(char *jobz, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_dgesdd(jobz, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgesdd,DGESDD)(__pyx_v_jobz, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_s, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesdd "F_FUNC(dgesdd,DGESDD)"(char *jobz, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dgesdd(char *jobz, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesdd(jobz, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesv "F_FUNC(dgesv,DGESV)"(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgesv(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesv(n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgesv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3537
 *     void _fortran_dgesv "F_FUNC(dgesv,DGESV)"(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgesv(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:
 *     _fortran_dgesv(n, nrhs, a, lda, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgesv,DGESV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesv "F_FUNC(dgesv,DGESV)"(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgesv(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesv(n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesvd "F_FUNC(dgesvd,DGESVD)"(char *jobu, char *jobvt, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *info) nogil
 * cdef void dgesvd(char *jobu, char *jobvt, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesvd(jobu, jobvt, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgesvd(char *__pyx_v_jobu, char *__pyx_v_jobvt, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3542
 *     void _fortran_dgesvd "F_FUNC(dgesvd,DGESVD)"(char *jobu, char *jobvt, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *info) nogil
 * cdef void dgesvd(char *jobu, char *jobvt, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgesvd(jobu, jobvt, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgesvd,DGESVD)(__pyx_v_jobu, __pyx_v_jobvt, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_s, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesvd "F_FUNC(dgesvd,DGESVD)"(char *jobu, char *jobvt, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *info) nogil
 * cdef void dgesvd(char *jobu, char *jobvt, int *m, int *n, d *a, int *lda, d *s, d *u, int *ldu, d *vt, int *ldvt, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesvd(jobu, jobvt, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesvx "F_FUNC(dgesvx,DGESVX)"(char *fact, char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgesvx(char *fact, char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesvx(fact, trans, n, nrhs, a, lda, af, ldaf, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgesvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3547
 *     void _fortran_dgesvx "F_FUNC(dgesvx,DGESVX)"(char *fact, char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgesvx(char *fact, char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgesvx(fact, trans, n, nrhs, a, lda, af, ldaf, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgesvx,DGESVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_equed, __pyx_v_r, __pyx_v_c, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgesvx "F_FUNC(dgesvx,DGESVX)"(char *fact, char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgesvx(char *fact, char *trans, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgesvx(fact, trans, n, nrhs, a, lda, af, ldaf, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetc2 "F_FUNC(dgetc2,DGETC2)"(int *n, d *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void dgetc2(int *n, d *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetc2(n, a, lda, ipiv, jpiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgetc2(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_jpiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3552
 *     void _fortran_dgetc2 "F_FUNC(dgetc2,DGETC2)"(int *n, d *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void dgetc2(int *n, d *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:
 *     _fortran_dgetc2(n, a, lda, ipiv, jpiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgetc2,DGETC2)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_jpiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetc2 "F_FUNC(dgetc2,DGETC2)"(int *n, d *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void dgetc2(int *n, d *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetc2(n, a, lda, ipiv, jpiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetf2 "F_FUNC(dgetf2,DGETF2)"(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dgetf2(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetf2(m, n, a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgetf2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3557
 *     void _fortran_dgetf2 "F_FUNC(dgetf2,DGETF2)"(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dgetf2(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_dgetf2(m, n, a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgetf2,DGETF2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetf2 "F_FUNC(dgetf2,DGETF2)"(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dgetf2(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetf2(m, n, a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetrf "F_FUNC(dgetrf,DGETRF)"(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dgetrf(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetrf(m, n, a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgetrf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3562
 *     void _fortran_dgetrf "F_FUNC(dgetrf,DGETRF)"(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dgetrf(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_dgetrf(m, n, a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgetrf,DGETRF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetrf "F_FUNC(dgetrf,DGETRF)"(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dgetrf(int *m, int *n, d *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetrf(m, n, a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetri "F_FUNC(dgetri,DGETRI)"(int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil
 * cdef void dgetri(int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetri(n, a, lda, ipiv, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgetri(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3567
 *     void _fortran_dgetri "F_FUNC(dgetri,DGETRI)"(int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil
 * cdef void dgetri(int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgetri(n, a, lda, ipiv, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgetri,DGETRI)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetri "F_FUNC(dgetri,DGETRI)"(int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil
 * cdef void dgetri(int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetri(n, a, lda, ipiv, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetrs "F_FUNC(dgetrs,DGETRS)"(char *trans, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgetrs(char *trans, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetrs(trans, n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgetrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3572
 *     void _fortran_dgetrs "F_FUNC(dgetrs,DGETRS)"(char *trans, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgetrs(char *trans, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:
 *     _fortran_dgetrs(trans, n, nrhs, a, lda, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgetrs,DGETRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgetrs "F_FUNC(dgetrs,DGETRS)"(char *trans, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgetrs(char *trans, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgetrs(trans, n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggbak "F_FUNC(dggbak,DGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, d *v, int *ldv, int *info) nogil
 * cdef void dggbak(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, d *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggbak(job, side, n, ilo, ihi, lscale, rscale, m, v, ldv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dggbak(char *__pyx_v_job, char *__pyx_v_side, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rscale, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3577
 *     void _fortran_dggbak "F_FUNC(dggbak,DGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, d *v, int *ldv, int *info) nogil
 * cdef void dggbak(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, d *v, int *ldv, int *info) nogil:
 *     _fortran_dggbak(job, side, n, ilo, ihi, lscale, rscale, m, v, ldv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dggbak,DGGBAK)(__pyx_v_job, __pyx_v_side, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_m, __pyx_v_v, __pyx_v_ldv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggbak "F_FUNC(dggbak,DGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, d *v, int *ldv, int *info) nogil
 * cdef void dggbak(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, d *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggbak(job, side, n, ilo, ihi, lscale, rscale, m, v, ldv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggbal "F_FUNC(dggbal,DGGBAL)"(char *job, int *n, d *a, int *lda, d *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil
 * cdef void dggbal(char *job, int *n, d *a, int *lda, d *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggbal(job, n, a, lda, b, ldb, ilo, ihi, lscale, rscale, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dggbal(char *__pyx_v_job, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3582
 *     void _fortran_dggbal "F_FUNC(dggbal,DGGBAL)"(char *job, int *n, d *a, int *lda, d *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil
 * cdef void dggbal(char *job, int *n, d *a, int *lda, d *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil:
 *     _fortran_dggbal(job, n, a, lda, b, ldb, ilo, ihi, lscale, rscale, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dggbal,DGGBAL)(__pyx_v_job, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggbal "F_FUNC(dggbal,DGGBAL)"(char *job, int *n, d *a, int *lda, d *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil
 * cdef void dggbal(char *job, int *n, d *a, int *lda, d *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggbal(job, n, a, lda, b, ldb, ilo, ihi, lscale, rscale, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgges "F_FUNC(dgges,DGGES)"(char *jobvsl, char *jobvsr, char *sort, _dselect3 *selctg, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void dgges(char *jobvsl, char *jobvsr, char *sort, dselect3 *selctg, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *work, int *lwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgges(jobvsl, jobvsr, sort, <_dselect3*>selctg, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, work, lwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgges(char *__pyx_v_jobvsl, char *__pyx_v_jobvsr, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_dselect3 *__pyx_v_selctg, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_sdim, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vsl, int *__pyx_v_ldvsl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vsr, int *__pyx_v_ldvsr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3587
 *     void _fortran_dgges "F_FUNC(dgges,DGGES)"(char *jobvsl, char *jobvsr, char *sort, _dselect3 *selctg, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void dgges(char *jobvsl, char *jobvsr, char *sort, dselect3 *selctg, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *work, int *lwork, bint *bwork, int *info) nogil:
 *     _fortran_dgges(jobvsl, jobvsr, sort, <_dselect3*>selctg, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, work, lwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgges,DGGES)(__pyx_v_jobvsl, __pyx_v_jobvsr, __pyx_v_sort, ((_dselect3 *)__pyx_v_selctg), __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_sdim, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_vsl, __pyx_v_ldvsl, __pyx_v_vsr, __pyx_v_ldvsr, __pyx_v_work, __pyx_v_lwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgges "F_FUNC(dgges,DGGES)"(char *jobvsl, char *jobvsr, char *sort, _dselect3 *selctg, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void dgges(char *jobvsl, char *jobvsr, char *sort, dselect3 *selctg, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *work, int *lwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgges(jobvsl, jobvsr, sort, <_dselect3*>selctg, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, work, lwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggesx "F_FUNC(dggesx,DGGESX)"(char *jobvsl, char *jobvsr, char *sort, _dselect3 *selctg, char *sense, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void dggesx(char *jobvsl, char *jobvsr, char *sort, dselect3 *selctg, char *sense, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggesx(jobvsl, jobvsr, sort, <_dselect3*>selctg, sense, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dggesx(char *__pyx_v_jobvsl, char *__pyx_v_jobvsr, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_dselect3 *__pyx_v_selctg, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_sdim, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vsl, int *__pyx_v_ldvsl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vsr, int *__pyx_v_ldvsr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcondv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3592
 *     void _fortran_dggesx "F_FUNC(dggesx,DGGESX)"(char *jobvsl, char *jobvsr, char *sort, _dselect3 *selctg, char *sense, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void dggesx(char *jobvsl, char *jobvsr, char *sort, dselect3 *selctg, char *sense, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:
 *     _fortran_dggesx(jobvsl, jobvsr, sort, <_dselect3*>selctg, sense, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dggesx,DGGESX)(__pyx_v_jobvsl, __pyx_v_jobvsr, __pyx_v_sort, ((_dselect3 *)__pyx_v_selctg), __pyx_v_sense, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_sdim, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_vsl, __pyx_v_ldvsl, __pyx_v_vsr, __pyx_v_ldvsr, __pyx_v_rconde, __pyx_v_rcondv, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggesx "F_FUNC(dggesx,DGGESX)"(char *jobvsl, char *jobvsr, char *sort, _dselect3 *selctg, char *sense, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void dggesx(char *jobvsl, char *jobvsr, char *sort, dselect3 *selctg, char *sense, int *n, d *a, int *lda, d *b, int *ldb, int *sdim, d *alphar, d *alphai, d *beta, d *vsl, int *ldvsl, d *vsr, int *ldvsr, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggesx(jobvsl, jobvsr, sort, <_dselect3*>selctg, sense, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggev "F_FUNC(dggev,DGGEV)"(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil
 * cdef void dggev(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggev(jobvl, jobvr, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dggev(char *__pyx_v_jobvl, char *__pyx_v_jobvr, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3597
 *     void _fortran_dggev "F_FUNC(dggev,DGGEV)"(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil
 * cdef void dggev(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil:
 *     _fortran_dggev(jobvl, jobvr, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dggev,DGGEV)(__pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggev "F_FUNC(dggev,DGGEV)"(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil
 * cdef void dggev(char *jobvl, char *jobvr, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggev(jobvl, jobvr, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggevx "F_FUNC(dggevx,DGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void dggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggevx(balanc, jobvl, jobvr, sense, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, work, lwork, iwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dggevx(char *__pyx_v_balanc, char *__pyx_v_jobvl, char *__pyx_v_jobvr, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abnrm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bbnrm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcondv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3602
 *     void _fortran_dggevx "F_FUNC(dggevx,DGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void dggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, bint *bwork, int *info) nogil:
 *     _fortran_dggevx(balanc, jobvl, jobvr, sense, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, work, lwork, iwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dggevx,DGGEVX)(__pyx_v_balanc, __pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_sense, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_abnrm, __pyx_v_bbnrm, __pyx_v_rconde, __pyx_v_rcondv, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggevx "F_FUNC(dggevx,DGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void dggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *vl, int *ldvl, d *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, d *work, int *lwork, int *iwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggevx(balanc, jobvl, jobvr, sense, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, work, lwork, iwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggglm "F_FUNC(dggglm,DGGGLM)"(int *n, int *m, int *p, d *a, int *lda, d *b, int *ldb, d *d, d *x, d *y, d *work, int *lwork, int *info) nogil
 * cdef void dggglm(int *n, int *m, int *p, d *a, int *lda, d *b, int *ldb, d *d, d *x, d *y, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggglm(n, m, p, a, lda, b, ldb, d, x, y, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dggglm(int *__pyx_v_n, int *__pyx_v_m, int *__pyx_v_p, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3607
 *     void _fortran_dggglm "F_FUNC(dggglm,DGGGLM)"(int *n, int *m, int *p, d *a, int *lda, d *b, int *ldb, d *d, d *x, d *y, d *work, int *lwork, int *info) nogil
 * cdef void dggglm(int *n, int *m, int *p, d *a, int *lda, d *b, int *ldb, d *d, d *x, d *y, d *work, int *lwork, int *info) nogil:
 *     _fortran_dggglm(n, m, p, a, lda, b, ldb, d, x, y, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dggglm,DGGGLM)(__pyx_v_n, __pyx_v_m, __pyx_v_p, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_d, __pyx_v_x, __pyx_v_y, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggglm "F_FUNC(dggglm,DGGGLM)"(int *n, int *m, int *p, d *a, int *lda, d *b, int *ldb, d *d, d *x, d *y, d *work, int *lwork, int *info) nogil
 * cdef void dggglm(int *n, int *m, int *p, d *a, int *lda, d *b, int *ldb, d *d, d *x, d *y, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggglm(n, m, p, a, lda, b, ldb, d, x, y, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgghrd "F_FUNC(dgghrd,DGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *info) nogil
 * cdef void dgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgghrd(compq, compz, n, ilo, ihi, a, lda, b, ldb, q, ldq, z, ldz, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgghrd(char *__pyx_v_compq, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3612
 *     void _fortran_dgghrd "F_FUNC(dgghrd,DGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *info) nogil
 * cdef void dgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *info) nogil:
 *     _fortran_dgghrd(compq, compz, n, ilo, ihi, a, lda, b, ldb, q, ldq, z, ldz, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgghrd,DGGHRD)(__pyx_v_compq, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgghrd "F_FUNC(dgghrd,DGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *info) nogil
 * cdef void dgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgghrd(compq, compz, n, ilo, ihi, a, lda, b, ldb, q, ldq, z, ldz, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgglse "F_FUNC(dgglse,DGGLSE)"(int *m, int *n, int *p, d *a, int *lda, d *b, int *ldb, d *c, d *d, d *x, d *work, int *lwork, int *info) nogil
 * cdef void dgglse(int *m, int *n, int *p, d *a, int *lda, d *b, int *ldb, d *c, d *d, d *x, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgglse(m, n, p, a, lda, b, ldb, c, d, x, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgglse(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_p, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3617
 *     void _fortran_dgglse "F_FUNC(dgglse,DGGLSE)"(int *m, int *n, int *p, d *a, int *lda, d *b, int *ldb, d *c, d *d, d *x, d *work, int *lwork, int *info) nogil
 * cdef void dgglse(int *m, int *n, int *p, d *a, int *lda, d *b, int *ldb, d *c, d *d, d *x, d *work, int *lwork, int *info) nogil:
 *     _fortran_dgglse(m, n, p, a, lda, b, ldb, c, d, x, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgglse,DGGLSE)(__pyx_v_m, __pyx_v_n, __pyx_v_p, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_c, __pyx_v_d, __pyx_v_x, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgglse "F_FUNC(dgglse,DGGLSE)"(int *m, int *n, int *p, d *a, int *lda, d *b, int *ldb, d *c, d *d, d *x, d *work, int *lwork, int *info) nogil
 * cdef void dgglse(int *m, int *n, int *p, d *a, int *lda, d *b, int *ldb, d *c, d *d, d *x, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgglse(m, n, p, a, lda, b, ldb, c, d, x, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggqrf "F_FUNC(dggqrf,DGGQRF)"(int *n, int *m, int *p, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil
 * cdef void dggqrf(int *n, int *m, int *p, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggqrf(n, m, p, a, lda, taua, b, ldb, taub, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dggqrf(int *__pyx_v_n, int *__pyx_v_m, int *__pyx_v_p, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_taua, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_taub, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3622
 *     void _fortran_dggqrf "F_FUNC(dggqrf,DGGQRF)"(int *n, int *m, int *p, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil
 * cdef void dggqrf(int *n, int *m, int *p, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil:
 *     _fortran_dggqrf(n, m, p, a, lda, taua, b, ldb, taub, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dggqrf,DGGQRF)(__pyx_v_n, __pyx_v_m, __pyx_v_p, __pyx_v_a, __pyx_v_lda, __pyx_v_taua, __pyx_v_b, __pyx_v_ldb, __pyx_v_taub, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggqrf "F_FUNC(dggqrf,DGGQRF)"(int *n, int *m, int *p, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil
 * cdef void dggqrf(int *n, int *m, int *p, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggqrf(n, m, p, a, lda, taua, b, ldb, taub, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggrqf "F_FUNC(dggrqf,DGGRQF)"(int *m, int *p, int *n, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil
 * cdef void dggrqf(int *m, int *p, int *n, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggrqf(m, p, n, a, lda, taua, b, ldb, taub, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dggrqf(int *__pyx_v_m, int *__pyx_v_p, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_taua, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_taub, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3627
 *     void _fortran_dggrqf "F_FUNC(dggrqf,DGGRQF)"(int *m, int *p, int *n, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil
 * cdef void dggrqf(int *m, int *p, int *n, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil:
 *     _fortran_dggrqf(m, p, n, a, lda, taua, b, ldb, taub, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dggrqf,DGGRQF)(__pyx_v_m, __pyx_v_p, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_taua, __pyx_v_b, __pyx_v_ldb, __pyx_v_taub, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dggrqf "F_FUNC(dggrqf,DGGRQF)"(int *m, int *p, int *n, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil
 * cdef void dggrqf(int *m, int *p, int *n, d *a, int *lda, d *taua, d *b, int *ldb, d *taub, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dggrqf(m, p, n, a, lda, taua, b, ldb, taub, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtcon "F_FUNC(dgtcon,DGTCON)"(char *norm, int *n, d *dl, d *d, d *du, d *du2, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgtcon(char *norm, int *n, d *dl, d *d, d *du, d *du2, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtcon(norm, n, dl, d, du, du2, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgtcon(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3632
 *     void _fortran_dgtcon "F_FUNC(dgtcon,DGTCON)"(char *norm, int *n, d *dl, d *d, d *du, d *du2, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgtcon(char *norm, int *n, d *dl, d *d, d *du, d *du2, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgtcon(norm, n, dl, d, du, du2, ipiv, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgtcon,DGTCON)(__pyx_v_norm, __pyx_v_n, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtcon "F_FUNC(dgtcon,DGTCON)"(char *norm, int *n, d *dl, d *d, d *du, d *du2, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dgtcon(char *norm, int *n, d *dl, d *d, d *du, d *du2, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtcon(norm, n, dl, d, du, du2, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtrfs "F_FUNC(dgtrfs,DGTRFS)"(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgtrfs(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtrfs(trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgtrfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_df, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_duf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3637
 *     void _fortran_dgtrfs "F_FUNC(dgtrfs,DGTRFS)"(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgtrfs(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgtrfs(trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgtrfs,DGTRFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_dlf, __pyx_v_df, __pyx_v_duf, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtrfs "F_FUNC(dgtrfs,DGTRFS)"(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgtrfs(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtrfs(trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtsv "F_FUNC(dgtsv,DGTSV)"(int *n, int *nrhs, d *dl, d *d, d *du, d *b, int *ldb, int *info) nogil
 * cdef void dgtsv(int *n, int *nrhs, d *dl, d *d, d *du, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtsv(n, nrhs, dl, d, du, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgtsv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3642
 *     void _fortran_dgtsv "F_FUNC(dgtsv,DGTSV)"(int *n, int *nrhs, d *dl, d *d, d *du, d *b, int *ldb, int *info) nogil
 * cdef void dgtsv(int *n, int *nrhs, d *dl, d *d, d *du, d *b, int *ldb, int *info) nogil:
 *     _fortran_dgtsv(n, nrhs, dl, d, du, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgtsv,DGTSV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtsv "F_FUNC(dgtsv,DGTSV)"(int *n, int *nrhs, d *dl, d *d, d *du, d *b, int *ldb, int *info) nogil
 * cdef void dgtsv(int *n, int *nrhs, d *dl, d *d, d *du, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtsv(n, nrhs, dl, d, du, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtsvx "F_FUNC(dgtsvx,DGTSVX)"(char *fact, char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgtsvx(char *fact, char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtsvx(fact, trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgtsvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_df, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_duf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3647
 *     void _fortran_dgtsvx "F_FUNC(dgtsvx,DGTSVX)"(char *fact, char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgtsvx(char *fact, char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dgtsvx(fact, trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgtsvx,DGTSVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_dlf, __pyx_v_df, __pyx_v_duf, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtsvx "F_FUNC(dgtsvx,DGTSVX)"(char *fact, char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dgtsvx(char *fact, char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *dlf, d *df, d *duf, d *du2, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtsvx(fact, trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgttrf "F_FUNC(dgttrf,DGTTRF)"(int *n, d *dl, d *d, d *du, d *du2, int *ipiv, int *info) nogil
 * cdef void dgttrf(int *n, d *dl, d *d, d *du, d *du2, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgttrf(n, dl, d, du, du2, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgttrf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du2, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3652
 *     void _fortran_dgttrf "F_FUNC(dgttrf,DGTTRF)"(int *n, d *dl, d *d, d *du, d *du2, int *ipiv, int *info) nogil
 * cdef void dgttrf(int *n, d *dl, d *d, d *du, d *du2, int *ipiv, int *info) nogil:
 *     _fortran_dgttrf(n, dl, d, du, du2, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgttrf,DGTTRF)(__pyx_v_n, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgttrf "F_FUNC(dgttrf,DGTTRF)"(int *n, d *dl, d *d, d *du, d *du2, int *ipiv, int *info) nogil
 * cdef void dgttrf(int *n, d *dl, d *d, d *du, d *du2, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgttrf(n, dl, d, du, du2, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgttrs "F_FUNC(dgttrs,DGTTRS)"(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgttrs(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgttrs(trans, n, nrhs, dl, d, du, du2, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgttrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3657
 *     void _fortran_dgttrs "F_FUNC(dgttrs,DGTTRS)"(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgttrs(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb, int *info) nogil:
 *     _fortran_dgttrs(trans, n, nrhs, dl, d, du, du2, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgttrs,DGTTRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgttrs "F_FUNC(dgttrs,DGTTRS)"(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dgttrs(char *trans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgttrs(trans, n, nrhs, dl, d, du, du2, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtts2 "F_FUNC(dgtts2,DGTTS2)"(int *itrans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb) nogil
 * cdef void dgtts2(int *itrans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtts2(itrans, n, nrhs, dl, d, du, du2, ipiv, b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dgtts2(int *__pyx_v_itrans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":3662
 *     void _fortran_dgtts2 "F_FUNC(dgtts2,DGTTS2)"(int *itrans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb) nogil
 * cdef void dgtts2(int *itrans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb) nogil:
 *     _fortran_dgtts2(itrans, n, nrhs, dl, d, du, du2, ipiv, b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dgtts2,DGTTS2)(__pyx_v_itrans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":3661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dgtts2 "F_FUNC(dgtts2,DGTTS2)"(int *itrans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb) nogil
 * cdef void dgtts2(int *itrans, int *n, int *nrhs, d *dl, d *d, d *du, d *du2, int *ipiv, d *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dgtts2(itrans, n, nrhs, dl, d, du, du2, ipiv, b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dhgeqz "F_FUNC(dhgeqz,DHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *t, int *ldt, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dhgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *t, int *ldt, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dhgeqz(job, compq, compz, n, ilo, ihi, h, ldh, t, ldt, alphar, alphai, beta, q, ldq, z, ldz, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dhgeqz(char *__pyx_v_job, char *__pyx_v_compq, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3667
 *     void _fortran_dhgeqz "F_FUNC(dhgeqz,DHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *t, int *ldt, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dhgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *t, int *ldt, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, d *work, int *lwork, int *info) nogil:
 *     _fortran_dhgeqz(job, compq, compz, n, ilo, ihi, h, ldh, t, ldt, alphar, alphai, beta, q, ldq, z, ldz, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dhgeqz,DHGEQZ)(__pyx_v_job, __pyx_v_compq, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_t, __pyx_v_ldt, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dhgeqz "F_FUNC(dhgeqz,DHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *t, int *ldt, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dhgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *t, int *ldt, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dhgeqz(job, compq, compz, n, ilo, ihi, h, ldh, t, ldt, alphar, alphai, beta, q, ldq, z, ldz, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dhsein "F_FUNC(dhsein,DHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, d *h, int *ldh, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *ifaill, int *ifailr, int *info) nogil
 * cdef void dhsein(char *side, char *eigsrc, char *initv, bint *select, int *n, d *h, int *ldh, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *ifaill, int *ifailr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dhsein(side, eigsrc, initv, select, n, h, ldh, wr, wi, vl, ldvl, vr, ldvr, mm, m, work, ifaill, ifailr, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dhsein(char *__pyx_v_side, char *__pyx_v_eigsrc, char *__pyx_v_initv, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_ifaill, int *__pyx_v_ifailr, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3672
 *     void _fortran_dhsein "F_FUNC(dhsein,DHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, d *h, int *ldh, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *ifaill, int *ifailr, int *info) nogil
 * cdef void dhsein(char *side, char *eigsrc, char *initv, bint *select, int *n, d *h, int *ldh, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *ifaill, int *ifailr, int *info) nogil:
 *     _fortran_dhsein(side, eigsrc, initv, select, n, h, ldh, wr, wi, vl, ldvl, vr, ldvr, mm, m, work, ifaill, ifailr, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dhsein,DHSEIN)(__pyx_v_side, __pyx_v_eigsrc, __pyx_v_initv, __pyx_v_select, __pyx_v_n, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_ifaill, __pyx_v_ifailr, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dhsein "F_FUNC(dhsein,DHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, d *h, int *ldh, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *ifaill, int *ifailr, int *info) nogil
 * cdef void dhsein(char *side, char *eigsrc, char *initv, bint *select, int *n, d *h, int *ldh, d *wr, d *wi, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *ifaill, int *ifailr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dhsein(side, eigsrc, initv, select, n, h, ldh, wr, wi, vl, ldvl, vr, ldvr, mm, m, work, ifaill, ifailr, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dhseqr "F_FUNC(dhseqr,DHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dhseqr(char *job, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, d *z, int *ldz, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dhseqr(job, compz, n, ilo, ihi, h, ldh, wr, wi, z, ldz, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dhseqr(char *__pyx_v_job, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3677
 *     void _fortran_dhseqr "F_FUNC(dhseqr,DHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dhseqr(char *job, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, d *z, int *ldz, d *work, int *lwork, int *info) nogil:
 *     _fortran_dhseqr(job, compz, n, ilo, ihi, h, ldh, wr, wi, z, ldz, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dhseqr,DHSEQR)(__pyx_v_job, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dhseqr "F_FUNC(dhseqr,DHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dhseqr(char *job, char *compz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, d *z, int *ldz, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dhseqr(job, compz, n, ilo, ihi, h, ldh, wr, wi, z, ldz, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlabad "F_FUNC(dlabad,DLABAD)"(d *small, d *large) nogil
 * cdef void dlabad(d *small, d *large) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlabad(small, large)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlabad(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_small, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_large) {

  /* "scipy/linalg/cython_lapack.pyx":3682
 *     void _fortran_dlabad "F_FUNC(dlabad,DLABAD)"(d *small, d *large) nogil
 * cdef void dlabad(d *small, d *large) nogil:
 *     _fortran_dlabad(small, large)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlabad,DLABAD)(__pyx_v_small, __pyx_v_large);

  /* "scipy/linalg/cython_lapack.pyx":3681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlabad "F_FUNC(dlabad,DLABAD)"(d *small, d *large) nogil
 * cdef void dlabad(d *small, d *large) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlabad(small, large)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlabrd "F_FUNC(dlabrd,DLABRD)"(int *m, int *n, int *nb, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *x, int *ldx, d *y, int *ldy) nogil
 * cdef void dlabrd(int *m, int *n, int *nb, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *x, int *ldx, d *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlabrd(m, n, nb, a, lda, d, e, tauq, taup, x, ldx, y, ldy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlabrd(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tauq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_taup, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, int *__pyx_v_ldy) {

  /* "scipy/linalg/cython_lapack.pyx":3687
 *     void _fortran_dlabrd "F_FUNC(dlabrd,DLABRD)"(int *m, int *n, int *nb, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *x, int *ldx, d *y, int *ldy) nogil
 * cdef void dlabrd(int *m, int *n, int *nb, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *x, int *ldx, d *y, int *ldy) nogil:
 *     _fortran_dlabrd(m, n, nb, a, lda, d, e, tauq, taup, x, ldx, y, ldy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlabrd,DLABRD)(__pyx_v_m, __pyx_v_n, __pyx_v_nb, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tauq, __pyx_v_taup, __pyx_v_x, __pyx_v_ldx, __pyx_v_y, __pyx_v_ldy);

  /* "scipy/linalg/cython_lapack.pyx":3686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlabrd "F_FUNC(dlabrd,DLABRD)"(int *m, int *n, int *nb, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *x, int *ldx, d *y, int *ldy) nogil
 * cdef void dlabrd(int *m, int *n, int *nb, d *a, int *lda, d *d, d *e, d *tauq, d *taup, d *x, int *ldx, d *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlabrd(m, n, nb, a, lda, d, e, tauq, taup, x, ldx, y, ldy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlacn2 "F_FUNC(dlacn2,DLACN2)"(int *n, d *v, d *x, int *isgn, d *est, int *kase, int *isave) nogil
 * cdef void dlacn2(int *n, d *v, d *x, int *isgn, d *est, int *kase, int *isave) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlacn2(n, v, x, isgn, est, kase, isave)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlacn2(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_isgn, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_est, int *__pyx_v_kase, int *__pyx_v_isave) {

  /* "scipy/linalg/cython_lapack.pyx":3692
 *     void _fortran_dlacn2 "F_FUNC(dlacn2,DLACN2)"(int *n, d *v, d *x, int *isgn, d *est, int *kase, int *isave) nogil
 * cdef void dlacn2(int *n, d *v, d *x, int *isgn, d *est, int *kase, int *isave) nogil:
 *     _fortran_dlacn2(n, v, x, isgn, est, kase, isave)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlacn2,DLACN2)(__pyx_v_n, __pyx_v_v, __pyx_v_x, __pyx_v_isgn, __pyx_v_est, __pyx_v_kase, __pyx_v_isave);

  /* "scipy/linalg/cython_lapack.pyx":3691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlacn2 "F_FUNC(dlacn2,DLACN2)"(int *n, d *v, d *x, int *isgn, d *est, int *kase, int *isave) nogil
 * cdef void dlacn2(int *n, d *v, d *x, int *isgn, d *est, int *kase, int *isave) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlacn2(n, v, x, isgn, est, kase, isave)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlacon "F_FUNC(dlacon,DLACON)"(int *n, d *v, d *x, int *isgn, d *est, int *kase) nogil
 * cdef void dlacon(int *n, d *v, d *x, int *isgn, d *est, int *kase) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlacon(n, v, x, isgn, est, kase)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlacon(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_isgn, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_est, int *__pyx_v_kase) {

  /* "scipy/linalg/cython_lapack.pyx":3697
 *     void _fortran_dlacon "F_FUNC(dlacon,DLACON)"(int *n, d *v, d *x, int *isgn, d *est, int *kase) nogil
 * cdef void dlacon(int *n, d *v, d *x, int *isgn, d *est, int *kase) nogil:
 *     _fortran_dlacon(n, v, x, isgn, est, kase)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlacon,DLACON)(__pyx_v_n, __pyx_v_v, __pyx_v_x, __pyx_v_isgn, __pyx_v_est, __pyx_v_kase);

  /* "scipy/linalg/cython_lapack.pyx":3696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlacon "F_FUNC(dlacon,DLACON)"(int *n, d *v, d *x, int *isgn, d *est, int *kase) nogil
 * cdef void dlacon(int *n, d *v, d *x, int *isgn, d *est, int *kase) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlacon(n, v, x, isgn, est, kase)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlacpy "F_FUNC(dlacpy,DLACPY)"(char *uplo, int *m, int *n, d *a, int *lda, d *b, int *ldb) nogil
 * cdef void dlacpy(char *uplo, int *m, int *n, d *a, int *lda, d *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlacpy(uplo, m, n, a, lda, b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlacpy(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":3702
 *     void _fortran_dlacpy "F_FUNC(dlacpy,DLACPY)"(char *uplo, int *m, int *n, d *a, int *lda, d *b, int *ldb) nogil
 * cdef void dlacpy(char *uplo, int *m, int *n, d *a, int *lda, d *b, int *ldb) nogil:
 *     _fortran_dlacpy(uplo, m, n, a, lda, b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlacpy,DLACPY)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":3701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlacpy "F_FUNC(dlacpy,DLACPY)"(char *uplo, int *m, int *n, d *a, int *lda, d *b, int *ldb) nogil
 * cdef void dlacpy(char *uplo, int *m, int *n, d *a, int *lda, d *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlacpy(uplo, m, n, a, lda, b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dladiv "F_FUNC(dladiv,DLADIV)"(d *a, d *b, d *c, d *d, d *p, d *q) nogil
 * cdef void dladiv(d *a, d *b, d *c, d *d, d *p, d *q) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dladiv(a, b, c, d, p, q)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dladiv(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_p, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q) {

  /* "scipy/linalg/cython_lapack.pyx":3707
 *     void _fortran_dladiv "F_FUNC(dladiv,DLADIV)"(d *a, d *b, d *c, d *d, d *p, d *q) nogil
 * cdef void dladiv(d *a, d *b, d *c, d *d, d *p, d *q) nogil:
 *     _fortran_dladiv(a, b, c, d, p, q)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dladiv,DLADIV)(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_d, __pyx_v_p, __pyx_v_q);

  /* "scipy/linalg/cython_lapack.pyx":3706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dladiv "F_FUNC(dladiv,DLADIV)"(d *a, d *b, d *c, d *d, d *p, d *q) nogil
 * cdef void dladiv(d *a, d *b, d *c, d *d, d *p, d *q) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dladiv(a, b, c, d, p, q)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlae2 "F_FUNC(dlae2,DLAE2)"(d *a, d *b, d *c, d *rt1, d *rt2) nogil
 * cdef void dlae2(d *a, d *b, d *c, d *rt1, d *rt2) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlae2(a, b, c, rt1, rt2)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlae2(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt2) {

  /* "scipy/linalg/cython_lapack.pyx":3712
 *     void _fortran_dlae2 "F_FUNC(dlae2,DLAE2)"(d *a, d *b, d *c, d *rt1, d *rt2) nogil
 * cdef void dlae2(d *a, d *b, d *c, d *rt1, d *rt2) nogil:
 *     _fortran_dlae2(a, b, c, rt1, rt2)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlae2,DLAE2)(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_rt1, __pyx_v_rt2);

  /* "scipy/linalg/cython_lapack.pyx":3711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlae2 "F_FUNC(dlae2,DLAE2)"(d *a, d *b, d *c, d *rt1, d *rt2) nogil
 * cdef void dlae2(d *a, d *b, d *c, d *rt1, d *rt2) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlae2(a, b, c, rt1, rt2)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaebz "F_FUNC(dlaebz,DLAEBZ)"(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, d *abstol, d *reltol, d *pivmin, d *d, d *e, d *e2, int *nval, d *ab, d *c, int *mout, int *nab, d *work, int *iwork, int *info) nogil
 * cdef void dlaebz(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, d *abstol, d *reltol, d *pivmin, d *d, d *e, d *e2, int *nval, d *ab, d *c, int *mout, int *nab, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaebz(ijob, nitmax, n, mmax, minp, nbmin, abstol, reltol, pivmin, d, e, e2, nval, ab, c, mout, nab, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaebz(int *__pyx_v_ijob, int *__pyx_v_nitmax, int *__pyx_v_n, int *__pyx_v_mmax, int *__pyx_v_minp, int *__pyx_v_nbmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_reltol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e2, int *__pyx_v_nval, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_mout, int *__pyx_v_nab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3717
 *     void _fortran_dlaebz "F_FUNC(dlaebz,DLAEBZ)"(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, d *abstol, d *reltol, d *pivmin, d *d, d *e, d *e2, int *nval, d *ab, d *c, int *mout, int *nab, d *work, int *iwork, int *info) nogil
 * cdef void dlaebz(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, d *abstol, d *reltol, d *pivmin, d *d, d *e, d *e2, int *nval, d *ab, d *c, int *mout, int *nab, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlaebz(ijob, nitmax, n, mmax, minp, nbmin, abstol, reltol, pivmin, d, e, e2, nval, ab, c, mout, nab, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaebz,DLAEBZ)(__pyx_v_ijob, __pyx_v_nitmax, __pyx_v_n, __pyx_v_mmax, __pyx_v_minp, __pyx_v_nbmin, __pyx_v_abstol, __pyx_v_reltol, __pyx_v_pivmin, __pyx_v_d, __pyx_v_e, __pyx_v_e2, __pyx_v_nval, __pyx_v_ab, __pyx_v_c, __pyx_v_mout, __pyx_v_nab, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaebz "F_FUNC(dlaebz,DLAEBZ)"(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, d *abstol, d *reltol, d *pivmin, d *d, d *e, d *e2, int *nval, d *ab, d *c, int *mout, int *nab, d *work, int *iwork, int *info) nogil
 * cdef void dlaebz(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, d *abstol, d *reltol, d *pivmin, d *d, d *e, d *e2, int *nval, d *ab, d *c, int *mout, int *nab, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaebz(ijob, nitmax, n, mmax, minp, nbmin, abstol, reltol, pivmin, d, e, e2, nval, ab, c, mout, nab, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed0 "F_FUNC(dlaed0,DLAED0)"(int *icompq, int *qsiz, int *n, d *d, d *e, d *q, int *ldq, d *qstore, int *ldqs, d *work, int *iwork, int *info) nogil
 * cdef void dlaed0(int *icompq, int *qsiz, int *n, d *d, d *e, d *q, int *ldq, d *qstore, int *ldqs, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed0(icompq, qsiz, n, d, e, q, ldq, qstore, ldqs, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed0(int *__pyx_v_icompq, int *__pyx_v_qsiz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_qstore, int *__pyx_v_ldqs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3722
 *     void _fortran_dlaed0 "F_FUNC(dlaed0,DLAED0)"(int *icompq, int *qsiz, int *n, d *d, d *e, d *q, int *ldq, d *qstore, int *ldqs, d *work, int *iwork, int *info) nogil
 * cdef void dlaed0(int *icompq, int *qsiz, int *n, d *d, d *e, d *q, int *ldq, d *qstore, int *ldqs, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlaed0(icompq, qsiz, n, d, e, q, ldq, qstore, ldqs, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed0,DLAED0)(__pyx_v_icompq, __pyx_v_qsiz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_q, __pyx_v_ldq, __pyx_v_qstore, __pyx_v_ldqs, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed0 "F_FUNC(dlaed0,DLAED0)"(int *icompq, int *qsiz, int *n, d *d, d *e, d *q, int *ldq, d *qstore, int *ldqs, d *work, int *iwork, int *info) nogil
 * cdef void dlaed0(int *icompq, int *qsiz, int *n, d *d, d *e, d *q, int *ldq, d *qstore, int *ldqs, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed0(icompq, qsiz, n, d, e, q, ldq, qstore, ldqs, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed1 "F_FUNC(dlaed1,DLAED1)"(int *n, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *work, int *iwork, int *info) nogil
 * cdef void dlaed1(int *n, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed1(n, d, q, ldq, indxq, rho, cutpnt, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed1(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, int *__pyx_v_cutpnt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3727
 *     void _fortran_dlaed1 "F_FUNC(dlaed1,DLAED1)"(int *n, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *work, int *iwork, int *info) nogil
 * cdef void dlaed1(int *n, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlaed1(n, d, q, ldq, indxq, rho, cutpnt, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed1,DLAED1)(__pyx_v_n, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_indxq, __pyx_v_rho, __pyx_v_cutpnt, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed1 "F_FUNC(dlaed1,DLAED1)"(int *n, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *work, int *iwork, int *info) nogil
 * cdef void dlaed1(int *n, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed1(n, d, q, ldq, indxq, rho, cutpnt, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed2 "F_FUNC(dlaed2,DLAED2)"(int *k, int *n, int *n1, d *d, d *q, int *ldq, int *indxq, d *rho, d *z, d *dlamda, d *w, d *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil
 * cdef void dlaed2(int *k, int *n, int *n1, d *d, d *q, int *ldq, int *indxq, d *rho, d *z, d *dlamda, d *w, d *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed2(k, n, n1, d, q, ldq, indxq, rho, z, dlamda, w, q2, indx, indxc, indxp, coltyp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed2(int *__pyx_v_k, int *__pyx_v_n, int *__pyx_v_n1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlamda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q2, int *__pyx_v_indx, int *__pyx_v_indxc, int *__pyx_v_indxp, int *__pyx_v_coltyp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3732
 *     void _fortran_dlaed2 "F_FUNC(dlaed2,DLAED2)"(int *k, int *n, int *n1, d *d, d *q, int *ldq, int *indxq, d *rho, d *z, d *dlamda, d *w, d *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil
 * cdef void dlaed2(int *k, int *n, int *n1, d *d, d *q, int *ldq, int *indxq, d *rho, d *z, d *dlamda, d *w, d *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil:
 *     _fortran_dlaed2(k, n, n1, d, q, ldq, indxq, rho, z, dlamda, w, q2, indx, indxc, indxp, coltyp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed2,DLAED2)(__pyx_v_k, __pyx_v_n, __pyx_v_n1, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_indxq, __pyx_v_rho, __pyx_v_z, __pyx_v_dlamda, __pyx_v_w, __pyx_v_q2, __pyx_v_indx, __pyx_v_indxc, __pyx_v_indxp, __pyx_v_coltyp, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed2 "F_FUNC(dlaed2,DLAED2)"(int *k, int *n, int *n1, d *d, d *q, int *ldq, int *indxq, d *rho, d *z, d *dlamda, d *w, d *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil
 * cdef void dlaed2(int *k, int *n, int *n1, d *d, d *q, int *ldq, int *indxq, d *rho, d *z, d *dlamda, d *w, d *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed2(k, n, n1, d, q, ldq, indxq, rho, z, dlamda, w, q2, indx, indxc, indxp, coltyp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed3 "F_FUNC(dlaed3,DLAED3)"(int *k, int *n, int *n1, d *d, d *q, int *ldq, d *rho, d *dlamda, d *q2, int *indx, int *ctot, d *w, d *s, int *info) nogil
 * cdef void dlaed3(int *k, int *n, int *n1, d *d, d *q, int *ldq, d *rho, d *dlamda, d *q2, int *indx, int *ctot, d *w, d *s, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed3(k, n, n1, d, q, ldq, rho, dlamda, q2, indx, ctot, w, s, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed3(int *__pyx_v_k, int *__pyx_v_n, int *__pyx_v_n1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlamda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q2, int *__pyx_v_indx, int *__pyx_v_ctot, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3737
 *     void _fortran_dlaed3 "F_FUNC(dlaed3,DLAED3)"(int *k, int *n, int *n1, d *d, d *q, int *ldq, d *rho, d *dlamda, d *q2, int *indx, int *ctot, d *w, d *s, int *info) nogil
 * cdef void dlaed3(int *k, int *n, int *n1, d *d, d *q, int *ldq, d *rho, d *dlamda, d *q2, int *indx, int *ctot, d *w, d *s, int *info) nogil:
 *     _fortran_dlaed3(k, n, n1, d, q, ldq, rho, dlamda, q2, indx, ctot, w, s, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed3,DLAED3)(__pyx_v_k, __pyx_v_n, __pyx_v_n1, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_rho, __pyx_v_dlamda, __pyx_v_q2, __pyx_v_indx, __pyx_v_ctot, __pyx_v_w, __pyx_v_s, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed3 "F_FUNC(dlaed3,DLAED3)"(int *k, int *n, int *n1, d *d, d *q, int *ldq, d *rho, d *dlamda, d *q2, int *indx, int *ctot, d *w, d *s, int *info) nogil
 * cdef void dlaed3(int *k, int *n, int *n1, d *d, d *q, int *ldq, d *rho, d *dlamda, d *q2, int *indx, int *ctot, d *w, d *s, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed3(k, n, n1, d, q, ldq, rho, dlamda, q2, indx, ctot, w, s, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed4 "F_FUNC(dlaed4,DLAED4)"(int *n, int *i, d *d, d *z, d *delta, d *rho, d *dlam, int *info) nogil
 * cdef void dlaed4(int *n, int *i, d *d, d *z, d *delta, d *rho, d *dlam, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed4(n, i, d, z, delta, rho, dlam, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed4(int *__pyx_v_n, int *__pyx_v_i, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_delta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlam, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3742
 *     void _fortran_dlaed4 "F_FUNC(dlaed4,DLAED4)"(int *n, int *i, d *d, d *z, d *delta, d *rho, d *dlam, int *info) nogil
 * cdef void dlaed4(int *n, int *i, d *d, d *z, d *delta, d *rho, d *dlam, int *info) nogil:
 *     _fortran_dlaed4(n, i, d, z, delta, rho, dlam, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed4,DLAED4)(__pyx_v_n, __pyx_v_i, __pyx_v_d, __pyx_v_z, __pyx_v_delta, __pyx_v_rho, __pyx_v_dlam, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed4 "F_FUNC(dlaed4,DLAED4)"(int *n, int *i, d *d, d *z, d *delta, d *rho, d *dlam, int *info) nogil
 * cdef void dlaed4(int *n, int *i, d *d, d *z, d *delta, d *rho, d *dlam, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed4(n, i, d, z, delta, rho, dlam, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed5 "F_FUNC(dlaed5,DLAED5)"(int *i, d *d, d *z, d *delta, d *rho, d *dlam) nogil
 * cdef void dlaed5(int *i, d *d, d *z, d *delta, d *rho, d *dlam) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed5(i, d, z, delta, rho, dlam)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed5(int *__pyx_v_i, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_delta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlam) {

  /* "scipy/linalg/cython_lapack.pyx":3747
 *     void _fortran_dlaed5 "F_FUNC(dlaed5,DLAED5)"(int *i, d *d, d *z, d *delta, d *rho, d *dlam) nogil
 * cdef void dlaed5(int *i, d *d, d *z, d *delta, d *rho, d *dlam) nogil:
 *     _fortran_dlaed5(i, d, z, delta, rho, dlam)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed5,DLAED5)(__pyx_v_i, __pyx_v_d, __pyx_v_z, __pyx_v_delta, __pyx_v_rho, __pyx_v_dlam);

  /* "scipy/linalg/cython_lapack.pyx":3746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed5 "F_FUNC(dlaed5,DLAED5)"(int *i, d *d, d *z, d *delta, d *rho, d *dlam) nogil
 * cdef void dlaed5(int *i, d *d, d *z, d *delta, d *rho, d *dlam) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed5(i, d, z, delta, rho, dlam)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed6 "F_FUNC(dlaed6,DLAED6)"(int *kniter, bint *orgati, d *rho, d *d, d *z, d *finit, d *tau, int *info) nogil
 * cdef void dlaed6(int *kniter, bint *orgati, d *rho, d *d, d *z, d *finit, d *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed6(kniter, orgati, rho, d, z, finit, tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed6(int *__pyx_v_kniter, int *__pyx_v_orgati, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_finit, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3752
 *     void _fortran_dlaed6 "F_FUNC(dlaed6,DLAED6)"(int *kniter, bint *orgati, d *rho, d *d, d *z, d *finit, d *tau, int *info) nogil
 * cdef void dlaed6(int *kniter, bint *orgati, d *rho, d *d, d *z, d *finit, d *tau, int *info) nogil:
 *     _fortran_dlaed6(kniter, orgati, rho, d, z, finit, tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed6,DLAED6)(__pyx_v_kniter, __pyx_v_orgati, __pyx_v_rho, __pyx_v_d, __pyx_v_z, __pyx_v_finit, __pyx_v_tau, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed6 "F_FUNC(dlaed6,DLAED6)"(int *kniter, bint *orgati, d *rho, d *d, d *z, d *finit, d *tau, int *info) nogil
 * cdef void dlaed6(int *kniter, bint *orgati, d *rho, d *d, d *z, d *finit, d *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed6(kniter, orgati, rho, d, z, finit, tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed7 "F_FUNC(dlaed7,DLAED7)"(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *work, int *iwork, int *info) nogil
 * cdef void dlaed7(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed7(icompq, n, qsiz, tlvls, curlvl, curpbm, d, q, ldq, indxq, rho, cutpnt, qstore, qptr, prmptr, perm, givptr, givcol, givnum, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed7(int *__pyx_v_icompq, int *__pyx_v_n, int *__pyx_v_qsiz, int *__pyx_v_tlvls, int *__pyx_v_curlvl, int *__pyx_v_curpbm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, int *__pyx_v_cutpnt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_qstore, int *__pyx_v_qptr, int *__pyx_v_prmptr, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3757
 *     void _fortran_dlaed7 "F_FUNC(dlaed7,DLAED7)"(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *work, int *iwork, int *info) nogil
 * cdef void dlaed7(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlaed7(icompq, n, qsiz, tlvls, curlvl, curpbm, d, q, ldq, indxq, rho, cutpnt, qstore, qptr, prmptr, perm, givptr, givcol, givnum, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed7,DLAED7)(__pyx_v_icompq, __pyx_v_n, __pyx_v_qsiz, __pyx_v_tlvls, __pyx_v_curlvl, __pyx_v_curpbm, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_indxq, __pyx_v_rho, __pyx_v_cutpnt, __pyx_v_qstore, __pyx_v_qptr, __pyx_v_prmptr, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed7 "F_FUNC(dlaed7,DLAED7)"(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *work, int *iwork, int *info) nogil
 * cdef void dlaed7(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed7(icompq, n, qsiz, tlvls, curlvl, curpbm, d, q, ldq, indxq, rho, cutpnt, qstore, qptr, prmptr, perm, givptr, givcol, givnum, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed8 "F_FUNC(dlaed8,DLAED8)"(int *icompq, int *k, int *n, int *qsiz, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *z, d *dlamda, d *q2, int *ldq2, d *w, int *perm, int *givptr, int *givcol, d *givnum, int *indxp, int *indx, int *info) nogil
 * cdef void dlaed8(int *icompq, int *k, int *n, int *qsiz, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *z, d *dlamda, d *q2, int *ldq2, d *w, int *perm, int *givptr, int *givcol, d *givnum, int *indxp, int *indx, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed8(icompq, k, n, qsiz, d, q, ldq, indxq, rho, cutpnt, z, dlamda, q2, ldq2, w, perm, givptr, givcol, givnum, indxp, indx, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed8(int *__pyx_v_icompq, int *__pyx_v_k, int *__pyx_v_n, int *__pyx_v_qsiz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, int *__pyx_v_cutpnt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlamda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q2, int *__pyx_v_ldq2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, int *__pyx_v_indxp, int *__pyx_v_indx, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3762
 *     void _fortran_dlaed8 "F_FUNC(dlaed8,DLAED8)"(int *icompq, int *k, int *n, int *qsiz, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *z, d *dlamda, d *q2, int *ldq2, d *w, int *perm, int *givptr, int *givcol, d *givnum, int *indxp, int *indx, int *info) nogil
 * cdef void dlaed8(int *icompq, int *k, int *n, int *qsiz, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *z, d *dlamda, d *q2, int *ldq2, d *w, int *perm, int *givptr, int *givcol, d *givnum, int *indxp, int *indx, int *info) nogil:
 *     _fortran_dlaed8(icompq, k, n, qsiz, d, q, ldq, indxq, rho, cutpnt, z, dlamda, q2, ldq2, w, perm, givptr, givcol, givnum, indxp, indx, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed8,DLAED8)(__pyx_v_icompq, __pyx_v_k, __pyx_v_n, __pyx_v_qsiz, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_indxq, __pyx_v_rho, __pyx_v_cutpnt, __pyx_v_z, __pyx_v_dlamda, __pyx_v_q2, __pyx_v_ldq2, __pyx_v_w, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, __pyx_v_indxp, __pyx_v_indx, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed8 "F_FUNC(dlaed8,DLAED8)"(int *icompq, int *k, int *n, int *qsiz, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *z, d *dlamda, d *q2, int *ldq2, d *w, int *perm, int *givptr, int *givcol, d *givnum, int *indxp, int *indx, int *info) nogil
 * cdef void dlaed8(int *icompq, int *k, int *n, int *qsiz, d *d, d *q, int *ldq, int *indxq, d *rho, int *cutpnt, d *z, d *dlamda, d *q2, int *ldq2, d *w, int *perm, int *givptr, int *givcol, d *givnum, int *indxp, int *indx, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed8(icompq, k, n, qsiz, d, q, ldq, indxq, rho, cutpnt, z, dlamda, q2, ldq2, w, perm, givptr, givcol, givnum, indxp, indx, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed9 "F_FUNC(dlaed9,DLAED9)"(int *k, int *kstart, int *kstop, int *n, d *d, d *q, int *ldq, d *rho, d *dlamda, d *w, d *s, int *lds, int *info) nogil
 * cdef void dlaed9(int *k, int *kstart, int *kstop, int *n, d *d, d *q, int *ldq, d *rho, d *dlamda, d *w, d *s, int *lds, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed9(k, kstart, kstop, n, d, q, ldq, rho, dlamda, w, s, lds, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaed9(int *__pyx_v_k, int *__pyx_v_kstart, int *__pyx_v_kstop, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlamda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, int *__pyx_v_lds, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3767
 *     void _fortran_dlaed9 "F_FUNC(dlaed9,DLAED9)"(int *k, int *kstart, int *kstop, int *n, d *d, d *q, int *ldq, d *rho, d *dlamda, d *w, d *s, int *lds, int *info) nogil
 * cdef void dlaed9(int *k, int *kstart, int *kstop, int *n, d *d, d *q, int *ldq, d *rho, d *dlamda, d *w, d *s, int *lds, int *info) nogil:
 *     _fortran_dlaed9(k, kstart, kstop, n, d, q, ldq, rho, dlamda, w, s, lds, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaed9,DLAED9)(__pyx_v_k, __pyx_v_kstart, __pyx_v_kstop, __pyx_v_n, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_rho, __pyx_v_dlamda, __pyx_v_w, __pyx_v_s, __pyx_v_lds, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaed9 "F_FUNC(dlaed9,DLAED9)"(int *k, int *kstart, int *kstop, int *n, d *d, d *q, int *ldq, d *rho, d *dlamda, d *w, d *s, int *lds, int *info) nogil
 * cdef void dlaed9(int *k, int *kstart, int *kstop, int *n, d *d, d *q, int *ldq, d *rho, d *dlamda, d *w, d *s, int *lds, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaed9(k, kstart, kstop, n, d, q, ldq, rho, dlamda, w, s, lds, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaeda "F_FUNC(dlaeda,DLAEDA)"(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *q, int *qptr, d *z, d *ztemp, int *info) nogil
 * cdef void dlaeda(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *q, int *qptr, d *z, d *ztemp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaeda(n, tlvls, curlvl, curpbm, prmptr, perm, givptr, givcol, givnum, q, qptr, z, ztemp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaeda(int *__pyx_v_n, int *__pyx_v_tlvls, int *__pyx_v_curlvl, int *__pyx_v_curpbm, int *__pyx_v_prmptr, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_qptr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ztemp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3772
 *     void _fortran_dlaeda "F_FUNC(dlaeda,DLAEDA)"(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *q, int *qptr, d *z, d *ztemp, int *info) nogil
 * cdef void dlaeda(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *q, int *qptr, d *z, d *ztemp, int *info) nogil:
 *     _fortran_dlaeda(n, tlvls, curlvl, curpbm, prmptr, perm, givptr, givcol, givnum, q, qptr, z, ztemp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaeda,DLAEDA)(__pyx_v_n, __pyx_v_tlvls, __pyx_v_curlvl, __pyx_v_curpbm, __pyx_v_prmptr, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, __pyx_v_q, __pyx_v_qptr, __pyx_v_z, __pyx_v_ztemp, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaeda "F_FUNC(dlaeda,DLAEDA)"(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *q, int *qptr, d *z, d *ztemp, int *info) nogil
 * cdef void dlaeda(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, d *q, int *qptr, d *z, d *ztemp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaeda(n, tlvls, curlvl, curpbm, prmptr, perm, givptr, givcol, givnum, q, qptr, z, ztemp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaein "F_FUNC(dlaein,DLAEIN)"(bint *rightv, bint *noinit, int *n, d *h, int *ldh, d *wr, d *wi, d *vr, d *vi, d *b, int *ldb, d *work, d *eps3, d *smlnum, d *bignum, int *info) nogil
 * cdef void dlaein(bint *rightv, bint *noinit, int *n, d *h, int *ldh, d *wr, d *wi, d *vr, d *vi, d *b, int *ldb, d *work, d *eps3, d *smlnum, d *bignum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaein(rightv, noinit, n, h, ldh, wr, wi, vr, vi, b, ldb, work, eps3, smlnum, bignum, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaein(int *__pyx_v_rightv, int *__pyx_v_noinit, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_eps3, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_smlnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bignum, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3777
 *     void _fortran_dlaein "F_FUNC(dlaein,DLAEIN)"(bint *rightv, bint *noinit, int *n, d *h, int *ldh, d *wr, d *wi, d *vr, d *vi, d *b, int *ldb, d *work, d *eps3, d *smlnum, d *bignum, int *info) nogil
 * cdef void dlaein(bint *rightv, bint *noinit, int *n, d *h, int *ldh, d *wr, d *wi, d *vr, d *vi, d *b, int *ldb, d *work, d *eps3, d *smlnum, d *bignum, int *info) nogil:
 *     _fortran_dlaein(rightv, noinit, n, h, ldh, wr, wi, vr, vi, b, ldb, work, eps3, smlnum, bignum, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaein,DLAEIN)(__pyx_v_rightv, __pyx_v_noinit, __pyx_v_n, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_vr, __pyx_v_vi, __pyx_v_b, __pyx_v_ldb, __pyx_v_work, __pyx_v_eps3, __pyx_v_smlnum, __pyx_v_bignum, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaein "F_FUNC(dlaein,DLAEIN)"(bint *rightv, bint *noinit, int *n, d *h, int *ldh, d *wr, d *wi, d *vr, d *vi, d *b, int *ldb, d *work, d *eps3, d *smlnum, d *bignum, int *info) nogil
 * cdef void dlaein(bint *rightv, bint *noinit, int *n, d *h, int *ldh, d *wr, d *wi, d *vr, d *vi, d *b, int *ldb, d *work, d *eps3, d *smlnum, d *bignum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaein(rightv, noinit, n, h, ldh, wr, wi, vr, vi, b, ldb, work, eps3, smlnum, bignum, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaev2 "F_FUNC(dlaev2,DLAEV2)"(d *a, d *b, d *c, d *rt1, d *rt2, d *cs1, d *sn1) nogil
 * cdef void dlaev2(d *a, d *b, d *c, d *rt1, d *rt2, d *cs1, d *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaev2(a, b, c, rt1, rt2, cs1, sn1)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaev2(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cs1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sn1) {

  /* "scipy/linalg/cython_lapack.pyx":3782
 *     void _fortran_dlaev2 "F_FUNC(dlaev2,DLAEV2)"(d *a, d *b, d *c, d *rt1, d *rt2, d *cs1, d *sn1) nogil
 * cdef void dlaev2(d *a, d *b, d *c, d *rt1, d *rt2, d *cs1, d *sn1) nogil:
 *     _fortran_dlaev2(a, b, c, rt1, rt2, cs1, sn1)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaev2,DLAEV2)(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_rt1, __pyx_v_rt2, __pyx_v_cs1, __pyx_v_sn1);

  /* "scipy/linalg/cython_lapack.pyx":3781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaev2 "F_FUNC(dlaev2,DLAEV2)"(d *a, d *b, d *c, d *rt1, d *rt2, d *cs1, d *sn1) nogil
 * cdef void dlaev2(d *a, d *b, d *c, d *rt1, d *rt2, d *cs1, d *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaev2(a, b, c, rt1, rt2, cs1, sn1)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaexc "F_FUNC(dlaexc,DLAEXC)"(bint *wantq, int *n, d *t, int *ldt, d *q, int *ldq, int *j1, int *n1, int *n2, d *work, int *info) nogil
 * cdef void dlaexc(bint *wantq, int *n, d *t, int *ldt, d *q, int *ldq, int *j1, int *n1, int *n2, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaexc(wantq, n, t, ldt, q, ldq, j1, n1, n2, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaexc(int *__pyx_v_wantq, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_j1, int *__pyx_v_n1, int *__pyx_v_n2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3787
 *     void _fortran_dlaexc "F_FUNC(dlaexc,DLAEXC)"(bint *wantq, int *n, d *t, int *ldt, d *q, int *ldq, int *j1, int *n1, int *n2, d *work, int *info) nogil
 * cdef void dlaexc(bint *wantq, int *n, d *t, int *ldt, d *q, int *ldq, int *j1, int *n1, int *n2, d *work, int *info) nogil:
 *     _fortran_dlaexc(wantq, n, t, ldt, q, ldq, j1, n1, n2, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaexc,DLAEXC)(__pyx_v_wantq, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_q, __pyx_v_ldq, __pyx_v_j1, __pyx_v_n1, __pyx_v_n2, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaexc "F_FUNC(dlaexc,DLAEXC)"(bint *wantq, int *n, d *t, int *ldt, d *q, int *ldq, int *j1, int *n1, int *n2, d *work, int *info) nogil
 * cdef void dlaexc(bint *wantq, int *n, d *t, int *ldt, d *q, int *ldq, int *j1, int *n1, int *n2, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaexc(wantq, n, t, ldt, q, ldq, j1, n1, n2, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlag2 "F_FUNC(dlag2,DLAG2)"(d *a, int *lda, d *b, int *ldb, d *safmin, d *scale1, d *scale2, d *wr1, d *wr2, d *wi) nogil
 * cdef void dlag2(d *a, int *lda, d *b, int *ldb, d *safmin, d *scale1, d *scale2, d *wr1, d *wr2, d *wi) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlag2(a, lda, b, ldb, safmin, scale1, scale2, wr1, wr2, wi)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlag2(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_safmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi) {

  /* "scipy/linalg/cython_lapack.pyx":3792
 *     void _fortran_dlag2 "F_FUNC(dlag2,DLAG2)"(d *a, int *lda, d *b, int *ldb, d *safmin, d *scale1, d *scale2, d *wr1, d *wr2, d *wi) nogil
 * cdef void dlag2(d *a, int *lda, d *b, int *ldb, d *safmin, d *scale1, d *scale2, d *wr1, d *wr2, d *wi) nogil:
 *     _fortran_dlag2(a, lda, b, ldb, safmin, scale1, scale2, wr1, wr2, wi)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlag2,DLAG2)(__pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_safmin, __pyx_v_scale1, __pyx_v_scale2, __pyx_v_wr1, __pyx_v_wr2, __pyx_v_wi);

  /* "scipy/linalg/cython_lapack.pyx":3791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlag2 "F_FUNC(dlag2,DLAG2)"(d *a, int *lda, d *b, int *ldb, d *safmin, d *scale1, d *scale2, d *wr1, d *wr2, d *wi) nogil
 * cdef void dlag2(d *a, int *lda, d *b, int *ldb, d *safmin, d *scale1, d *scale2, d *wr1, d *wr2, d *wi) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlag2(a, lda, b, ldb, safmin, scale1, scale2, wr1, wr2, wi)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlag2s "F_FUNC(dlag2s,DLAG2S)"(int *m, int *n, d *a, int *lda, s *sa, int *ldsa, int *info) nogil
 * cdef void dlag2s(int *m, int *n, d *a, int *lda, s *sa, int *ldsa, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlag2s(m, n, a, lda, sa, ldsa, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlag2s(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sa, int *__pyx_v_ldsa, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3797
 *     void _fortran_dlag2s "F_FUNC(dlag2s,DLAG2S)"(int *m, int *n, d *a, int *lda, s *sa, int *ldsa, int *info) nogil
 * cdef void dlag2s(int *m, int *n, d *a, int *lda, s *sa, int *ldsa, int *info) nogil:
 *     _fortran_dlag2s(m, n, a, lda, sa, ldsa, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlag2s,DLAG2S)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_sa, __pyx_v_ldsa, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlag2s "F_FUNC(dlag2s,DLAG2S)"(int *m, int *n, d *a, int *lda, s *sa, int *ldsa, int *info) nogil
 * cdef void dlag2s(int *m, int *n, d *a, int *lda, s *sa, int *ldsa, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlag2s(m, n, a, lda, sa, ldsa, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlags2 "F_FUNC(dlags2,DLAGS2)"(bint *upper, d *a1, d *a2, d *a3, d *b1, d *b2, d *b3, d *csu, d *snu, d *csv, d *snv, d *csq, d *snq) nogil
 * cdef void dlags2(bint *upper, d *a1, d *a2, d *a3, d *b1, d *b2, d *b3, d *csu, d *snu, d *csv, d *snv, d *csq, d *snq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlags2(upper, a1, a2, a3, b1, b2, b3, csu, snu, csv, snv, csq, snq)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlags2(int *__pyx_v_upper, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a3, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b3, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_snu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_snv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_snq) {

  /* "scipy/linalg/cython_lapack.pyx":3802
 *     void _fortran_dlags2 "F_FUNC(dlags2,DLAGS2)"(bint *upper, d *a1, d *a2, d *a3, d *b1, d *b2, d *b3, d *csu, d *snu, d *csv, d *snv, d *csq, d *snq) nogil
 * cdef void dlags2(bint *upper, d *a1, d *a2, d *a3, d *b1, d *b2, d *b3, d *csu, d *snu, d *csv, d *snv, d *csq, d *snq) nogil:
 *     _fortran_dlags2(upper, a1, a2, a3, b1, b2, b3, csu, snu, csv, snv, csq, snq)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlags2,DLAGS2)(__pyx_v_upper, __pyx_v_a1, __pyx_v_a2, __pyx_v_a3, __pyx_v_b1, __pyx_v_b2, __pyx_v_b3, __pyx_v_csu, __pyx_v_snu, __pyx_v_csv, __pyx_v_snv, __pyx_v_csq, __pyx_v_snq);

  /* "scipy/linalg/cython_lapack.pyx":3801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlags2 "F_FUNC(dlags2,DLAGS2)"(bint *upper, d *a1, d *a2, d *a3, d *b1, d *b2, d *b3, d *csu, d *snu, d *csv, d *snv, d *csq, d *snq) nogil
 * cdef void dlags2(bint *upper, d *a1, d *a2, d *a3, d *b1, d *b2, d *b3, d *csu, d *snu, d *csv, d *snv, d *csq, d *snq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlags2(upper, a1, a2, a3, b1, b2, b3, csu, snu, csv, snv, csq, snq)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlagtf "F_FUNC(dlagtf,DLAGTF)"(int *n, d *a, d *lambda_, d *b, d *c, d *tol, d *d, int *in_, int *info) nogil
 * cdef void dlagtf(int *n, d *a, d *lambda_, d *b, d *c, d *tol, d *d, int *in_, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlagtf(n, a, lambda_, b, c, tol, d, in_, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlagtf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lambda_, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, int *__pyx_v_in_, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3807
 *     void _fortran_dlagtf "F_FUNC(dlagtf,DLAGTF)"(int *n, d *a, d *lambda_, d *b, d *c, d *tol, d *d, int *in_, int *info) nogil
 * cdef void dlagtf(int *n, d *a, d *lambda_, d *b, d *c, d *tol, d *d, int *in_, int *info) nogil:
 *     _fortran_dlagtf(n, a, lambda_, b, c, tol, d, in_, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlagtf,DLAGTF)(__pyx_v_n, __pyx_v_a, __pyx_v_lambda_, __pyx_v_b, __pyx_v_c, __pyx_v_tol, __pyx_v_d, __pyx_v_in_, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlagtf "F_FUNC(dlagtf,DLAGTF)"(int *n, d *a, d *lambda_, d *b, d *c, d *tol, d *d, int *in_, int *info) nogil
 * cdef void dlagtf(int *n, d *a, d *lambda_, d *b, d *c, d *tol, d *d, int *in_, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlagtf(n, a, lambda_, b, c, tol, d, in_, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlagtm "F_FUNC(dlagtm,DLAGTM)"(char *trans, int *n, int *nrhs, d *alpha, d *dl, d *d, d *du, d *x, int *ldx, d *beta, d *b, int *ldb) nogil
 * cdef void dlagtm(char *trans, int *n, int *nrhs, d *alpha, d *dl, d *d, d *du, d *x, int *ldx, d *beta, d *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlagtm(trans, n, nrhs, alpha, dl, d, du, x, ldx, beta, b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlagtm(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":3812
 *     void _fortran_dlagtm "F_FUNC(dlagtm,DLAGTM)"(char *trans, int *n, int *nrhs, d *alpha, d *dl, d *d, d *du, d *x, int *ldx, d *beta, d *b, int *ldb) nogil
 * cdef void dlagtm(char *trans, int *n, int *nrhs, d *alpha, d *dl, d *d, d *du, d *x, int *ldx, d *beta, d *b, int *ldb) nogil:
 *     _fortran_dlagtm(trans, n, nrhs, alpha, dl, d, du, x, ldx, beta, b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlagtm,DLAGTM)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_alpha, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_x, __pyx_v_ldx, __pyx_v_beta, __pyx_v_b, __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":3811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlagtm "F_FUNC(dlagtm,DLAGTM)"(char *trans, int *n, int *nrhs, d *alpha, d *dl, d *d, d *du, d *x, int *ldx, d *beta, d *b, int *ldb) nogil
 * cdef void dlagtm(char *trans, int *n, int *nrhs, d *alpha, d *dl, d *d, d *du, d *x, int *ldx, d *beta, d *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlagtm(trans, n, nrhs, alpha, dl, d, du, x, ldx, beta, b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlagts "F_FUNC(dlagts,DLAGTS)"(int *job, int *n, d *a, d *b, d *c, d *d, int *in_, d *y, d *tol, int *info) nogil
 * cdef void dlagts(int *job, int *n, d *a, d *b, d *c, d *d, int *in_, d *y, d *tol, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlagts(job, n, a, b, c, d, in_, y, tol, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlagts(int *__pyx_v_job, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, int *__pyx_v_in_, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tol, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3817
 *     void _fortran_dlagts "F_FUNC(dlagts,DLAGTS)"(int *job, int *n, d *a, d *b, d *c, d *d, int *in_, d *y, d *tol, int *info) nogil
 * cdef void dlagts(int *job, int *n, d *a, d *b, d *c, d *d, int *in_, d *y, d *tol, int *info) nogil:
 *     _fortran_dlagts(job, n, a, b, c, d, in_, y, tol, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlagts,DLAGTS)(__pyx_v_job, __pyx_v_n, __pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_d, __pyx_v_in_, __pyx_v_y, __pyx_v_tol, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlagts "F_FUNC(dlagts,DLAGTS)"(int *job, int *n, d *a, d *b, d *c, d *d, int *in_, d *y, d *tol, int *info) nogil
 * cdef void dlagts(int *job, int *n, d *a, d *b, d *c, d *d, int *in_, d *y, d *tol, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlagts(job, n, a, b, c, d, in_, y, tol, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlagv2 "F_FUNC(dlagv2,DLAGV2)"(d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *csl, d *snl, d *csr, d *snr) nogil
 * cdef void dlagv2(d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *csl, d *snl, d *csr, d *snr) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlagv2(a, lda, b, ldb, alphar, alphai, beta, csl, snl, csr, snr)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlagv2(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_snl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_snr) {

  /* "scipy/linalg/cython_lapack.pyx":3822
 *     void _fortran_dlagv2 "F_FUNC(dlagv2,DLAGV2)"(d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *csl, d *snl, d *csr, d *snr) nogil
 * cdef void dlagv2(d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *csl, d *snl, d *csr, d *snr) nogil:
 *     _fortran_dlagv2(a, lda, b, ldb, alphar, alphai, beta, csl, snl, csr, snr)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlagv2,DLAGV2)(__pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_csl, __pyx_v_snl, __pyx_v_csr, __pyx_v_snr);

  /* "scipy/linalg/cython_lapack.pyx":3821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlagv2 "F_FUNC(dlagv2,DLAGV2)"(d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *csl, d *snl, d *csr, d *snr) nogil
 * cdef void dlagv2(d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *csl, d *snl, d *csr, d *snr) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlagv2(a, lda, b, ldb, alphar, alphai, beta, csl, snl, csr, snr)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlahqr "F_FUNC(dlahqr,DLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, int *info) nogil
 * cdef void dlahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlahqr(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlahqr(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3827
 *     void _fortran_dlahqr "F_FUNC(dlahqr,DLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, int *info) nogil
 * cdef void dlahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, int *info) nogil:
 *     _fortran_dlahqr(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlahqr,DLAHQR)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlahqr "F_FUNC(dlahqr,DLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, int *info) nogil
 * cdef void dlahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlahqr(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlahr2 "F_FUNC(dlahr2,DLAHR2)"(int *n, int *k, int *nb, d *a, int *lda, d *tau, d *t, int *ldt, d *y, int *ldy) nogil
 * cdef void dlahr2(int *n, int *k, int *nb, d *a, int *lda, d *tau, d *t, int *ldt, d *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlahr2(n, k, nb, a, lda, tau, t, ldt, y, ldy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlahr2(int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_nb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, int *__pyx_v_ldy) {

  /* "scipy/linalg/cython_lapack.pyx":3832
 *     void _fortran_dlahr2 "F_FUNC(dlahr2,DLAHR2)"(int *n, int *k, int *nb, d *a, int *lda, d *tau, d *t, int *ldt, d *y, int *ldy) nogil
 * cdef void dlahr2(int *n, int *k, int *nb, d *a, int *lda, d *tau, d *t, int *ldt, d *y, int *ldy) nogil:
 *     _fortran_dlahr2(n, k, nb, a, lda, tau, t, ldt, y, ldy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlahr2,DLAHR2)(__pyx_v_n, __pyx_v_k, __pyx_v_nb, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_t, __pyx_v_ldt, __pyx_v_y, __pyx_v_ldy);

  /* "scipy/linalg/cython_lapack.pyx":3831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlahr2 "F_FUNC(dlahr2,DLAHR2)"(int *n, int *k, int *nb, d *a, int *lda, d *tau, d *t, int *ldt, d *y, int *ldy) nogil
 * cdef void dlahr2(int *n, int *k, int *nb, d *a, int *lda, d *tau, d *t, int *ldt, d *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlahr2(n, k, nb, a, lda, tau, t, ldt, y, ldy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaic1 "F_FUNC(dlaic1,DLAIC1)"(int *job, int *j, d *x, d *sest, d *w, d *gamma, d *sestpr, d *s, d *c) nogil
 * cdef void dlaic1(int *job, int *j, d *x, d *sest, d *w, d *gamma, d *sestpr, d *s, d *c) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaic1(job, j, x, sest, w, gamma, sestpr, s, c)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaic1(int *__pyx_v_job, int *__pyx_v_j, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sest, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gamma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sestpr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c) {

  /* "scipy/linalg/cython_lapack.pyx":3837
 *     void _fortran_dlaic1 "F_FUNC(dlaic1,DLAIC1)"(int *job, int *j, d *x, d *sest, d *w, d *gamma, d *sestpr, d *s, d *c) nogil
 * cdef void dlaic1(int *job, int *j, d *x, d *sest, d *w, d *gamma, d *sestpr, d *s, d *c) nogil:
 *     _fortran_dlaic1(job, j, x, sest, w, gamma, sestpr, s, c)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaic1,DLAIC1)(__pyx_v_job, __pyx_v_j, __pyx_v_x, __pyx_v_sest, __pyx_v_w, __pyx_v_gamma, __pyx_v_sestpr, __pyx_v_s, __pyx_v_c);

  /* "scipy/linalg/cython_lapack.pyx":3836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaic1 "F_FUNC(dlaic1,DLAIC1)"(int *job, int *j, d *x, d *sest, d *w, d *gamma, d *sestpr, d *s, d *c) nogil
 * cdef void dlaic1(int *job, int *j, d *x, d *sest, d *w, d *gamma, d *sestpr, d *s, d *c) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaic1(job, j, x, sest, w, gamma, sestpr, s, c)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaln2 "F_FUNC(dlaln2,DLALN2)"(bint *ltrans, int *na, int *nw, d *smin, d *ca, d *a, int *lda, d *d1, d *d2, d *b, int *ldb, d *wr, d *wi, d *x, int *ldx, d *scale, d *xnorm, int *info) nogil
 * cdef void dlaln2(bint *ltrans, int *na, int *nw, d *smin, d *ca, d *a, int *lda, d *d1, d *d2, d *b, int *ldb, d *wr, d *wi, d *x, int *ldx, d *scale, d *xnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaln2(ltrans, na, nw, smin, ca, a, lda, d1, d2, b, ldb, wr, wi, x, ldx, scale, xnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaln2(int *__pyx_v_ltrans, int *__pyx_v_na, int *__pyx_v_nw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_smin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ca, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_xnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3842
 *     void _fortran_dlaln2 "F_FUNC(dlaln2,DLALN2)"(bint *ltrans, int *na, int *nw, d *smin, d *ca, d *a, int *lda, d *d1, d *d2, d *b, int *ldb, d *wr, d *wi, d *x, int *ldx, d *scale, d *xnorm, int *info) nogil
 * cdef void dlaln2(bint *ltrans, int *na, int *nw, d *smin, d *ca, d *a, int *lda, d *d1, d *d2, d *b, int *ldb, d *wr, d *wi, d *x, int *ldx, d *scale, d *xnorm, int *info) nogil:
 *     _fortran_dlaln2(ltrans, na, nw, smin, ca, a, lda, d1, d2, b, ldb, wr, wi, x, ldx, scale, xnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaln2,DLALN2)(__pyx_v_ltrans, __pyx_v_na, __pyx_v_nw, __pyx_v_smin, __pyx_v_ca, __pyx_v_a, __pyx_v_lda, __pyx_v_d1, __pyx_v_d2, __pyx_v_b, __pyx_v_ldb, __pyx_v_wr, __pyx_v_wi, __pyx_v_x, __pyx_v_ldx, __pyx_v_scale, __pyx_v_xnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaln2 "F_FUNC(dlaln2,DLALN2)"(bint *ltrans, int *na, int *nw, d *smin, d *ca, d *a, int *lda, d *d1, d *d2, d *b, int *ldb, d *wr, d *wi, d *x, int *ldx, d *scale, d *xnorm, int *info) nogil
 * cdef void dlaln2(bint *ltrans, int *na, int *nw, d *smin, d *ca, d *a, int *lda, d *d1, d *d2, d *b, int *ldb, d *wr, d *wi, d *x, int *ldx, d *scale, d *xnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaln2(ltrans, na, nw, smin, ca, a, lda, d1, d2, b, ldb, wr, wi, x, ldx, scale, xnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlals0 "F_FUNC(dlals0,DLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *info) nogil
 * cdef void dlals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlals0(icompq, nl, nr, sqre, nrhs, b, ldb, bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlals0(int *__pyx_v_icompq, int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bx, int *__pyx_v_ldbx, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, int *__pyx_v_ldgnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_poles, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3847
 *     void _fortran_dlals0 "F_FUNC(dlals0,DLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *info) nogil
 * cdef void dlals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *info) nogil:
 *     _fortran_dlals0(icompq, nl, nr, sqre, nrhs, b, ldb, bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlals0,DLALS0)(__pyx_v_icompq, __pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_nrhs, __pyx_v_b, __pyx_v_ldb, __pyx_v_bx, __pyx_v_ldbx, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_givnum, __pyx_v_ldgnum, __pyx_v_poles, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_k, __pyx_v_c, __pyx_v_s, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlals0 "F_FUNC(dlals0,DLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *info) nogil
 * cdef void dlals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlals0(icompq, nl, nr, sqre, nrhs, b, ldb, bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlalsa "F_FUNC(dlalsa,DLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlalsa(int *icompq, int *smlsiz, int *n, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlalsa(icompq, smlsiz, n, nrhs, b, ldb, bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlalsa(int *__pyx_v_icompq, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bx, int *__pyx_v_ldbx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_poles, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, int *__pyx_v_perm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3852
 *     void _fortran_dlalsa "F_FUNC(dlalsa,DLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlalsa(int *icompq, int *smlsiz, int *n, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlalsa(icompq, smlsiz, n, nrhs, b, ldb, bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlalsa,DLALSA)(__pyx_v_icompq, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_nrhs, __pyx_v_b, __pyx_v_ldb, __pyx_v_bx, __pyx_v_ldbx, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_k, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_poles, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_perm, __pyx_v_givnum, __pyx_v_c, __pyx_v_s, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlalsa "F_FUNC(dlalsa,DLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlalsa(int *icompq, int *smlsiz, int *n, int *nrhs, d *b, int *ldb, d *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlalsa(icompq, smlsiz, n, nrhs, b, ldb, bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlalsd "F_FUNC(dlalsd,DLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, d *b, int *ldb, d *rcond, int *rank, d *work, int *iwork, int *info) nogil
 * cdef void dlalsd(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, d *b, int *ldb, d *rcond, int *rank, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlalsd(uplo, smlsiz, n, nrhs, d, e, b, ldb, rcond, rank, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlalsd(char *__pyx_v_uplo, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3857
 *     void _fortran_dlalsd "F_FUNC(dlalsd,DLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, d *b, int *ldb, d *rcond, int *rank, d *work, int *iwork, int *info) nogil
 * cdef void dlalsd(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, d *b, int *ldb, d *rcond, int *rank, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlalsd(uplo, smlsiz, n, nrhs, d, e, b, ldb, rcond, rank, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlalsd,DLALSD)(__pyx_v_uplo, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_b, __pyx_v_ldb, __pyx_v_rcond, __pyx_v_rank, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlalsd "F_FUNC(dlalsd,DLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, d *b, int *ldb, d *rcond, int *rank, d *work, int *iwork, int *info) nogil
 * cdef void dlalsd(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, d *b, int *ldb, d *rcond, int *rank, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlalsd(uplo, smlsiz, n, nrhs, d, e, b, ldb, rcond, rank, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlamrg "F_FUNC(dlamrg,DLAMRG)"(int *n1, int *n2, d *a, int *dtrd1, int *dtrd2, int *index_bn) nogil
 * cdef void dlamrg(int *n1, int *n2, d *a, int *dtrd1, int *dtrd2, int *index_bn) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlamrg(n1, n2, a, dtrd1, dtrd2, index_bn)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlamrg(int *__pyx_v_n1, int *__pyx_v_n2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_dtrd1, int *__pyx_v_dtrd2, int *__pyx_v_index_bn) {

  /* "scipy/linalg/cython_lapack.pyx":3862
 *     void _fortran_dlamrg "F_FUNC(dlamrg,DLAMRG)"(int *n1, int *n2, d *a, int *dtrd1, int *dtrd2, int *index_bn) nogil
 * cdef void dlamrg(int *n1, int *n2, d *a, int *dtrd1, int *dtrd2, int *index_bn) nogil:
 *     _fortran_dlamrg(n1, n2, a, dtrd1, dtrd2, index_bn)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlamrg,DLAMRG)(__pyx_v_n1, __pyx_v_n2, __pyx_v_a, __pyx_v_dtrd1, __pyx_v_dtrd2, __pyx_v_index_bn);

  /* "scipy/linalg/cython_lapack.pyx":3861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlamrg "F_FUNC(dlamrg,DLAMRG)"(int *n1, int *n2, d *a, int *dtrd1, int *dtrd2, int *index_bn) nogil
 * cdef void dlamrg(int *n1, int *n2, d *a, int *dtrd1, int *dtrd2, int *index_bn) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlamrg(n1, n2, a, dtrd1, dtrd2, index_bn)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlanv2 "F_FUNC(dlanv2,DLANV2)"(d *a, d *b, d *c, d *d, d *rt1r, d *rt1i, d *rt2r, d *rt2i, d *cs, d *sn) nogil
 * cdef void dlanv2(d *a, d *b, d *c, d *d, d *rt1r, d *rt1i, d *rt2r, d *rt2i, d *cs, d *sn) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlanv2(a, b, c, d, rt1r, rt1i, rt2r, rt2i, cs, sn)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlanv2(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt1r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt1i, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt2r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt2i, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sn) {

  /* "scipy/linalg/cython_lapack.pyx":3867
 *     void _fortran_dlanv2 "F_FUNC(dlanv2,DLANV2)"(d *a, d *b, d *c, d *d, d *rt1r, d *rt1i, d *rt2r, d *rt2i, d *cs, d *sn) nogil
 * cdef void dlanv2(d *a, d *b, d *c, d *d, d *rt1r, d *rt1i, d *rt2r, d *rt2i, d *cs, d *sn) nogil:
 *     _fortran_dlanv2(a, b, c, d, rt1r, rt1i, rt2r, rt2i, cs, sn)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlanv2,DLANV2)(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_d, __pyx_v_rt1r, __pyx_v_rt1i, __pyx_v_rt2r, __pyx_v_rt2i, __pyx_v_cs, __pyx_v_sn);

  /* "scipy/linalg/cython_lapack.pyx":3866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlanv2 "F_FUNC(dlanv2,DLANV2)"(d *a, d *b, d *c, d *d, d *rt1r, d *rt1i, d *rt2r, d *rt2i, d *cs, d *sn) nogil
 * cdef void dlanv2(d *a, d *b, d *c, d *d, d *rt1r, d *rt1i, d *rt2r, d *rt2i, d *cs, d *sn) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlanv2(a, b, c, d, rt1r, rt1i, rt2r, rt2i, cs, sn)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlapll "F_FUNC(dlapll,DLAPLL)"(int *n, d *x, int *incx, d *y, int *incy, d *ssmin) nogil
 * cdef void dlapll(int *n, d *x, int *incx, d *y, int *incy, d *ssmin) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlapll(n, x, incx, y, incy, ssmin)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlapll(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ssmin) {

  /* "scipy/linalg/cython_lapack.pyx":3872
 *     void _fortran_dlapll "F_FUNC(dlapll,DLAPLL)"(int *n, d *x, int *incx, d *y, int *incy, d *ssmin) nogil
 * cdef void dlapll(int *n, d *x, int *incx, d *y, int *incy, d *ssmin) nogil:
 *     _fortran_dlapll(n, x, incx, y, incy, ssmin)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlapll,DLAPLL)(__pyx_v_n, __pyx_v_x, __pyx_v_incx, __pyx_v_y, __pyx_v_incy, __pyx_v_ssmin);

  /* "scipy/linalg/cython_lapack.pyx":3871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlapll "F_FUNC(dlapll,DLAPLL)"(int *n, d *x, int *incx, d *y, int *incy, d *ssmin) nogil
 * cdef void dlapll(int *n, d *x, int *incx, d *y, int *incy, d *ssmin) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlapll(n, x, incx, y, incy, ssmin)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlapmt "F_FUNC(dlapmt,DLAPMT)"(bint *forwrd, int *m, int *n, d *x, int *ldx, int *k) nogil
 * cdef void dlapmt(bint *forwrd, int *m, int *n, d *x, int *ldx, int *k) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlapmt(forwrd, m, n, x, ldx, k)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlapmt(int *__pyx_v_forwrd, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, int *__pyx_v_k) {

  /* "scipy/linalg/cython_lapack.pyx":3877
 *     void _fortran_dlapmt "F_FUNC(dlapmt,DLAPMT)"(bint *forwrd, int *m, int *n, d *x, int *ldx, int *k) nogil
 * cdef void dlapmt(bint *forwrd, int *m, int *n, d *x, int *ldx, int *k) nogil:
 *     _fortran_dlapmt(forwrd, m, n, x, ldx, k)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlapmt,DLAPMT)(__pyx_v_forwrd, __pyx_v_m, __pyx_v_n, __pyx_v_x, __pyx_v_ldx, __pyx_v_k);

  /* "scipy/linalg/cython_lapack.pyx":3876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlapmt "F_FUNC(dlapmt,DLAPMT)"(bint *forwrd, int *m, int *n, d *x, int *ldx, int *k) nogil
 * cdef void dlapmt(bint *forwrd, int *m, int *n, d *x, int *ldx, int *k) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlapmt(forwrd, m, n, x, ldx, k)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqgb "F_FUNC(dlaqgb,DLAQGB)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void dlaqgb(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqgb(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqgb(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":3882
 *     void _fortran_dlaqgb "F_FUNC(dlaqgb,DLAQGB)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void dlaqgb(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:
 *     _fortran_dlaqgb(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqgb,DLAQGB)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":3881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqgb "F_FUNC(dlaqgb,DLAQGB)"(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void dlaqgb(int *m, int *n, int *kl, int *ku, d *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqgb(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqge "F_FUNC(dlaqge,DLAQGE)"(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void dlaqge(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqge(m, n, a, lda, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqge(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":3887
 *     void _fortran_dlaqge "F_FUNC(dlaqge,DLAQGE)"(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void dlaqge(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:
 *     _fortran_dlaqge(m, n, a, lda, r, c, rowcnd, colcnd, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqge,DLAQGE)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":3886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqge "F_FUNC(dlaqge,DLAQGE)"(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void dlaqge(int *m, int *n, d *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqge(m, n, a, lda, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqp2 "F_FUNC(dlaqp2,DLAQP2)"(int *m, int *n, int *offset, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *work) nogil
 * cdef void dlaqp2(int *m, int *n, int *offset, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqp2(m, n, offset, a, lda, jpvt, tau, vn1, vn2, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqp2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_offset, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vn1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vn2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":3892
 *     void _fortran_dlaqp2 "F_FUNC(dlaqp2,DLAQP2)"(int *m, int *n, int *offset, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *work) nogil
 * cdef void dlaqp2(int *m, int *n, int *offset, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *work) nogil:
 *     _fortran_dlaqp2(m, n, offset, a, lda, jpvt, tau, vn1, vn2, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqp2,DLAQP2)(__pyx_v_m, __pyx_v_n, __pyx_v_offset, __pyx_v_a, __pyx_v_lda, __pyx_v_jpvt, __pyx_v_tau, __pyx_v_vn1, __pyx_v_vn2, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":3891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqp2 "F_FUNC(dlaqp2,DLAQP2)"(int *m, int *n, int *offset, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *work) nogil
 * cdef void dlaqp2(int *m, int *n, int *offset, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqp2(m, n, offset, a, lda, jpvt, tau, vn1, vn2, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqps "F_FUNC(dlaqps,DLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *auxv, d *f, int *ldf) nogil
 * cdef void dlaqps(int *m, int *n, int *offset, int *nb, int *kb, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *auxv, d *f, int *ldf) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqps(m, n, offset, nb, kb, a, lda, jpvt, tau, vn1, vn2, auxv, f, ldf)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqps(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_offset, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vn1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vn2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_auxv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_f, int *__pyx_v_ldf) {

  /* "scipy/linalg/cython_lapack.pyx":3897
 *     void _fortran_dlaqps "F_FUNC(dlaqps,DLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *auxv, d *f, int *ldf) nogil
 * cdef void dlaqps(int *m, int *n, int *offset, int *nb, int *kb, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *auxv, d *f, int *ldf) nogil:
 *     _fortran_dlaqps(m, n, offset, nb, kb, a, lda, jpvt, tau, vn1, vn2, auxv, f, ldf)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqps,DLAQPS)(__pyx_v_m, __pyx_v_n, __pyx_v_offset, __pyx_v_nb, __pyx_v_kb, __pyx_v_a, __pyx_v_lda, __pyx_v_jpvt, __pyx_v_tau, __pyx_v_vn1, __pyx_v_vn2, __pyx_v_auxv, __pyx_v_f, __pyx_v_ldf);

  /* "scipy/linalg/cython_lapack.pyx":3896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqps "F_FUNC(dlaqps,DLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *auxv, d *f, int *ldf) nogil
 * cdef void dlaqps(int *m, int *n, int *offset, int *nb, int *kb, d *a, int *lda, int *jpvt, d *tau, d *vn1, d *vn2, d *auxv, d *f, int *ldf) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqps(m, n, offset, nb, kb, a, lda, jpvt, tau, vn1, vn2, auxv, f, ldf)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr0 "F_FUNC(dlaqr0,DLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dlaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr0(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqr0(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3902
 *     void _fortran_dlaqr0 "F_FUNC(dlaqr0,DLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dlaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil:
 *     _fortran_dlaqr0(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqr0,DLAQR0)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr0 "F_FUNC(dlaqr0,DLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dlaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr0(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr1 "F_FUNC(dlaqr1,DLAQR1)"(int *n, d *h, int *ldh, d *sr1, d *si1, d *sr2, d *si2, d *v) nogil
 * cdef void dlaqr1(int *n, d *h, int *ldh, d *sr1, d *si1, d *sr2, d *si2, d *v) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr1(n, h, ldh, sr1, si1, sr2, si2, v)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqr1(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sr1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_si1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sr2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_si2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v) {

  /* "scipy/linalg/cython_lapack.pyx":3907
 *     void _fortran_dlaqr1 "F_FUNC(dlaqr1,DLAQR1)"(int *n, d *h, int *ldh, d *sr1, d *si1, d *sr2, d *si2, d *v) nogil
 * cdef void dlaqr1(int *n, d *h, int *ldh, d *sr1, d *si1, d *sr2, d *si2, d *v) nogil:
 *     _fortran_dlaqr1(n, h, ldh, sr1, si1, sr2, si2, v)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqr1,DLAQR1)(__pyx_v_n, __pyx_v_h, __pyx_v_ldh, __pyx_v_sr1, __pyx_v_si1, __pyx_v_sr2, __pyx_v_si2, __pyx_v_v);

  /* "scipy/linalg/cython_lapack.pyx":3906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr1 "F_FUNC(dlaqr1,DLAQR1)"(int *n, d *h, int *ldh, d *sr1, d *si1, d *sr2, d *si2, d *v) nogil
 * cdef void dlaqr1(int *n, d *h, int *ldh, d *sr1, d *si1, d *sr2, d *si2, d *v) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr1(n, h, ldh, sr1, si1, sr2, si2, v)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr2 "F_FUNC(dlaqr2,DLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil
 * cdef void dlaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr2(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqr2(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ns, int *__pyx_v_nd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_si, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_nh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, int *__pyx_v_nv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wv, int *__pyx_v_ldwv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork) {

  /* "scipy/linalg/cython_lapack.pyx":3912
 *     void _fortran_dlaqr2 "F_FUNC(dlaqr2,DLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil
 * cdef void dlaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil:
 *     _fortran_dlaqr2(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqr2,DLAQR2)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nw, __pyx_v_h, __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_ns, __pyx_v_nd, __pyx_v_sr, __pyx_v_si, __pyx_v_v, __pyx_v_ldv, __pyx_v_nh, __pyx_v_t, __pyx_v_ldt, __pyx_v_nv, __pyx_v_wv, __pyx_v_ldwv, __pyx_v_work, __pyx_v_lwork);

  /* "scipy/linalg/cython_lapack.pyx":3911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr2 "F_FUNC(dlaqr2,DLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil
 * cdef void dlaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr2(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr3 "F_FUNC(dlaqr3,DLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil
 * cdef void dlaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr3(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqr3(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ns, int *__pyx_v_nd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_si, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_nh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, int *__pyx_v_nv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wv, int *__pyx_v_ldwv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork) {

  /* "scipy/linalg/cython_lapack.pyx":3917
 *     void _fortran_dlaqr3 "F_FUNC(dlaqr3,DLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil
 * cdef void dlaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil:
 *     _fortran_dlaqr3(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqr3,DLAQR3)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nw, __pyx_v_h, __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_ns, __pyx_v_nd, __pyx_v_sr, __pyx_v_si, __pyx_v_v, __pyx_v_ldv, __pyx_v_nh, __pyx_v_t, __pyx_v_ldt, __pyx_v_nv, __pyx_v_wv, __pyx_v_ldwv, __pyx_v_work, __pyx_v_lwork);

  /* "scipy/linalg/cython_lapack.pyx":3916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr3 "F_FUNC(dlaqr3,DLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil
 * cdef void dlaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, int *ns, int *nd, d *sr, d *si, d *v, int *ldv, int *nh, d *t, int *ldt, int *nv, d *wv, int *ldwv, d *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr3(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr4 "F_FUNC(dlaqr4,DLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dlaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr4(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqr4(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3922
 *     void _fortran_dlaqr4 "F_FUNC(dlaqr4,DLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dlaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil:
 *     _fortran_dlaqr4(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqr4,DLAQR4)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr4 "F_FUNC(dlaqr4,DLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil
 * cdef void dlaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, d *h, int *ldh, d *wr, d *wi, int *iloz, int *ihiz, d *z, int *ldz, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr4(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr5 "F_FUNC(dlaqr5,DLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, d *sr, d *si, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, d *v, int *ldv, d *u, int *ldu, int *nv, d *wv, int *ldwv, int *nh, d *wh, int *ldwh) nogil
 * cdef void dlaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, d *sr, d *si, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, d *v, int *ldv, d *u, int *ldu, int *nv, d *wv, int *ldwv, int *nh, d *wh, int *ldwh) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, sr, si, h, ldh, iloz, ihiz, z, ldz, v, ldv, u, ldu, nv, wv, ldwv, nh, wh, ldwh)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqr5(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_kacc22, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nshfts, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_si, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, int *__pyx_v_nv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wv, int *__pyx_v_ldwv, int *__pyx_v_nh, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wh, int *__pyx_v_ldwh) {

  /* "scipy/linalg/cython_lapack.pyx":3927
 *     void _fortran_dlaqr5 "F_FUNC(dlaqr5,DLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, d *sr, d *si, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, d *v, int *ldv, d *u, int *ldu, int *nv, d *wv, int *ldwv, int *nh, d *wh, int *ldwh) nogil
 * cdef void dlaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, d *sr, d *si, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, d *v, int *ldv, d *u, int *ldu, int *nv, d *wv, int *ldwv, int *nh, d *wh, int *ldwh) nogil:
 *     _fortran_dlaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, sr, si, h, ldh, iloz, ihiz, z, ldz, v, ldv, u, ldu, nv, wv, ldwv, nh, wh, ldwh)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqr5,DLAQR5)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_kacc22, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nshfts, __pyx_v_sr, __pyx_v_si, __pyx_v_h, __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_v, __pyx_v_ldv, __pyx_v_u, __pyx_v_ldu, __pyx_v_nv, __pyx_v_wv, __pyx_v_ldwv, __pyx_v_nh, __pyx_v_wh, __pyx_v_ldwh);

  /* "scipy/linalg/cython_lapack.pyx":3926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqr5 "F_FUNC(dlaqr5,DLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, d *sr, d *si, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, d *v, int *ldv, d *u, int *ldu, int *nv, d *wv, int *ldwv, int *nh, d *wh, int *ldwh) nogil
 * cdef void dlaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, d *sr, d *si, d *h, int *ldh, int *iloz, int *ihiz, d *z, int *ldz, d *v, int *ldv, d *u, int *ldu, int *nv, d *wv, int *ldwv, int *nh, d *wh, int *ldwh) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, sr, si, h, ldh, iloz, ihiz, z, ldz, v, ldv, u, ldu, nv, wv, ldwv, nh, wh, ldwh)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqsb "F_FUNC(dlaqsb,DLAQSB)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsb(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqsb(uplo, n, kd, ab, ldab, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqsb(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":3932
 *     void _fortran_dlaqsb "F_FUNC(dlaqsb,DLAQSB)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsb(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_dlaqsb(uplo, n, kd, ab, ldab, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqsb,DLAQSB)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":3931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqsb "F_FUNC(dlaqsb,DLAQSB)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsb(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqsb(uplo, n, kd, ab, ldab, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqsp "F_FUNC(dlaqsp,DLAQSP)"(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsp(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqsp(uplo, n, ap, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqsp(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":3937
 *     void _fortran_dlaqsp "F_FUNC(dlaqsp,DLAQSP)"(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsp(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_dlaqsp(uplo, n, ap, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqsp,DLAQSP)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":3936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqsp "F_FUNC(dlaqsp,DLAQSP)"(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsp(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqsp(uplo, n, ap, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqsy "F_FUNC(dlaqsy,DLAQSY)"(char *uplo, int *n, d *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsy(char *uplo, int *n, d *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqsy(uplo, n, a, lda, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqsy(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":3942
 *     void _fortran_dlaqsy "F_FUNC(dlaqsy,DLAQSY)"(char *uplo, int *n, d *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsy(char *uplo, int *n, d *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_dlaqsy(uplo, n, a, lda, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqsy,DLAQSY)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":3941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqsy "F_FUNC(dlaqsy,DLAQSY)"(char *uplo, int *n, d *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void dlaqsy(char *uplo, int *n, d *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqsy(uplo, n, a, lda, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqtr "F_FUNC(dlaqtr,DLAQTR)"(bint *ltran, bint *lreal, int *n, d *t, int *ldt, d *b, d *w, d *scale, d *x, d *work, int *info) nogil
 * cdef void dlaqtr(bint *ltran, bint *lreal, int *n, d *t, int *ldt, d *b, d *w, d *scale, d *x, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqtr(ltran, lreal, n, t, ldt, b, w, scale, x, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaqtr(int *__pyx_v_ltran, int *__pyx_v_lreal, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3947
 *     void _fortran_dlaqtr "F_FUNC(dlaqtr,DLAQTR)"(bint *ltran, bint *lreal, int *n, d *t, int *ldt, d *b, d *w, d *scale, d *x, d *work, int *info) nogil
 * cdef void dlaqtr(bint *ltran, bint *lreal, int *n, d *t, int *ldt, d *b, d *w, d *scale, d *x, d *work, int *info) nogil:
 *     _fortran_dlaqtr(ltran, lreal, n, t, ldt, b, w, scale, x, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaqtr,DLAQTR)(__pyx_v_ltran, __pyx_v_lreal, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_b, __pyx_v_w, __pyx_v_scale, __pyx_v_x, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaqtr "F_FUNC(dlaqtr,DLAQTR)"(bint *ltran, bint *lreal, int *n, d *t, int *ldt, d *b, d *w, d *scale, d *x, d *work, int *info) nogil
 * cdef void dlaqtr(bint *ltran, bint *lreal, int *n, d *t, int *ldt, d *b, d *w, d *scale, d *x, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaqtr(ltran, lreal, n, t, ldt, b, w, scale, x, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlar1v "F_FUNC(dlar1v,DLAR1V)"(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, d *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil
 * cdef void dlar1v(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, d *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlar1v(int *__pyx_v_n, int *__pyx_v_b1, int *__pyx_v_bn, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lambda_, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ld, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lld, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gaptol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_wantnc, int *__pyx_v_negcnt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ztz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_mingma, int *__pyx_v_r, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_nrminv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_resid, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rqcorr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":3952
 *     void _fortran_dlar1v "F_FUNC(dlar1v,DLAR1V)"(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, d *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil
 * cdef void dlar1v(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, d *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil:
 *     _fortran_dlar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlar1v,DLAR1V)(__pyx_v_n, __pyx_v_b1, __pyx_v_bn, __pyx_v_lambda_, __pyx_v_d, __pyx_v_l, __pyx_v_ld, __pyx_v_lld, __pyx_v_pivmin, __pyx_v_gaptol, __pyx_v_z, __pyx_v_wantnc, __pyx_v_negcnt, __pyx_v_ztz, __pyx_v_mingma, __pyx_v_r, __pyx_v_isuppz, __pyx_v_nrminv, __pyx_v_resid, __pyx_v_rqcorr, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":3951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlar1v "F_FUNC(dlar1v,DLAR1V)"(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, d *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil
 * cdef void dlar1v(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, d *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlar2v "F_FUNC(dlar2v,DLAR2V)"(int *n, d *x, d *y, d *z, int *incx, d *c, d *s, int *incc) nogil
 * cdef void dlar2v(int *n, d *x, d *y, d *z, int *incx, d *c, d *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlar2v(n, x, y, z, incx, c, s, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlar2v(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":3957
 *     void _fortran_dlar2v "F_FUNC(dlar2v,DLAR2V)"(int *n, d *x, d *y, d *z, int *incx, d *c, d *s, int *incc) nogil
 * cdef void dlar2v(int *n, d *x, d *y, d *z, int *incx, d *c, d *s, int *incc) nogil:
 *     _fortran_dlar2v(n, x, y, z, incx, c, s, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlar2v,DLAR2V)(__pyx_v_n, __pyx_v_x, __pyx_v_y, __pyx_v_z, __pyx_v_incx, __pyx_v_c, __pyx_v_s, __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":3956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlar2v "F_FUNC(dlar2v,DLAR2V)"(int *n, d *x, d *y, d *z, int *incx, d *c, d *s, int *incc) nogil
 * cdef void dlar2v(int *n, d *x, d *y, d *z, int *incx, d *c, d *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlar2v(n, x, y, z, incx, c, s, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarf "F_FUNC(dlarf,DLARF)"(char *side, int *m, int *n, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarf(char *side, int *m, int *n, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarf(side, m, n, v, incv, tau, c, ldc, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarf(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_incv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":3962
 *     void _fortran_dlarf "F_FUNC(dlarf,DLARF)"(char *side, int *m, int *n, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarf(char *side, int *m, int *n, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil:
 *     _fortran_dlarf(side, m, n, v, incv, tau, c, ldc, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarf,DLARF)(__pyx_v_side, __pyx_v_m, __pyx_v_n, __pyx_v_v, __pyx_v_incv, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":3961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarf "F_FUNC(dlarf,DLARF)"(char *side, int *m, int *n, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarf(char *side, int *m, int *n, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarf(side, m, n, v, incv, tau, c, ldc, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarfb "F_FUNC(dlarfb,DLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil
 * cdef void dlarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarfb(side, trans, direct, storev, m, n, k, v, ldv, t, ldt, c, ldc, work, ldwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarfb(char *__pyx_v_side, char *__pyx_v_trans, char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_ldwork) {

  /* "scipy/linalg/cython_lapack.pyx":3967
 *     void _fortran_dlarfb "F_FUNC(dlarfb,DLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil
 * cdef void dlarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil:
 *     _fortran_dlarfb(side, trans, direct, storev, m, n, k, v, ldv, t, ldt, c, ldc, work, ldwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarfb,DLARFB)(__pyx_v_side, __pyx_v_trans, __pyx_v_direct, __pyx_v_storev, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_v, __pyx_v_ldv, __pyx_v_t, __pyx_v_ldt, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_ldwork);

  /* "scipy/linalg/cython_lapack.pyx":3966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarfb "F_FUNC(dlarfb,DLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil
 * cdef void dlarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarfb(side, trans, direct, storev, m, n, k, v, ldv, t, ldt, c, ldc, work, ldwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarfg "F_FUNC(dlarfg,DLARFG)"(int *n, d *alpha, d *x, int *incx, d *tau) nogil
 * cdef void dlarfg(int *n, d *alpha, d *x, int *incx, d *tau) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarfg(n, alpha, x, incx, tau)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarfg(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau) {

  /* "scipy/linalg/cython_lapack.pyx":3972
 *     void _fortran_dlarfg "F_FUNC(dlarfg,DLARFG)"(int *n, d *alpha, d *x, int *incx, d *tau) nogil
 * cdef void dlarfg(int *n, d *alpha, d *x, int *incx, d *tau) nogil:
 *     _fortran_dlarfg(n, alpha, x, incx, tau)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarfg,DLARFG)(__pyx_v_n, __pyx_v_alpha, __pyx_v_x, __pyx_v_incx, __pyx_v_tau);

  /* "scipy/linalg/cython_lapack.pyx":3971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarfg "F_FUNC(dlarfg,DLARFG)"(int *n, d *alpha, d *x, int *incx, d *tau) nogil
 * cdef void dlarfg(int *n, d *alpha, d *x, int *incx, d *tau) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarfg(n, alpha, x, incx, tau)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarft "F_FUNC(dlarft,DLARFT)"(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil
 * cdef void dlarft(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarft(direct, storev, n, k, v, ldv, tau, t, ldt)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarft(char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt) {

  /* "scipy/linalg/cython_lapack.pyx":3977
 *     void _fortran_dlarft "F_FUNC(dlarft,DLARFT)"(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil
 * cdef void dlarft(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil:
 *     _fortran_dlarft(direct, storev, n, k, v, ldv, tau, t, ldt)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarft,DLARFT)(__pyx_v_direct, __pyx_v_storev, __pyx_v_n, __pyx_v_k, __pyx_v_v, __pyx_v_ldv, __pyx_v_tau, __pyx_v_t, __pyx_v_ldt);

  /* "scipy/linalg/cython_lapack.pyx":3976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarft "F_FUNC(dlarft,DLARFT)"(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil
 * cdef void dlarft(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarft(direct, storev, n, k, v, ldv, tau, t, ldt)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarfx "F_FUNC(dlarfx,DLARFX)"(char *side, int *m, int *n, d *v, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarfx(char *side, int *m, int *n, d *v, d *tau, d *c, int *ldc, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarfx(side, m, n, v, tau, c, ldc, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarfx(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":3982
 *     void _fortran_dlarfx "F_FUNC(dlarfx,DLARFX)"(char *side, int *m, int *n, d *v, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarfx(char *side, int *m, int *n, d *v, d *tau, d *c, int *ldc, d *work) nogil:
 *     _fortran_dlarfx(side, m, n, v, tau, c, ldc, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarfx,DLARFX)(__pyx_v_side, __pyx_v_m, __pyx_v_n, __pyx_v_v, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":3981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarfx "F_FUNC(dlarfx,DLARFX)"(char *side, int *m, int *n, d *v, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarfx(char *side, int *m, int *n, d *v, d *tau, d *c, int *ldc, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarfx(side, m, n, v, tau, c, ldc, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlargv "F_FUNC(dlargv,DLARGV)"(int *n, d *x, int *incx, d *y, int *incy, d *c, int *incc) nogil
 * cdef void dlargv(int *n, d *x, int *incx, d *y, int *incy, d *c, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlargv(n, x, incx, y, incy, c, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlargv(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":3987
 *     void _fortran_dlargv "F_FUNC(dlargv,DLARGV)"(int *n, d *x, int *incx, d *y, int *incy, d *c, int *incc) nogil
 * cdef void dlargv(int *n, d *x, int *incx, d *y, int *incy, d *c, int *incc) nogil:
 *     _fortran_dlargv(n, x, incx, y, incy, c, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlargv,DLARGV)(__pyx_v_n, __pyx_v_x, __pyx_v_incx, __pyx_v_y, __pyx_v_incy, __pyx_v_c, __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":3986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlargv "F_FUNC(dlargv,DLARGV)"(int *n, d *x, int *incx, d *y, int *incy, d *c, int *incc) nogil
 * cdef void dlargv(int *n, d *x, int *incx, d *y, int *incy, d *c, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlargv(n, x, incx, y, incy, c, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarnv "F_FUNC(dlarnv,DLARNV)"(int *idist, int *iseed, int *n, d *x) nogil
 * cdef void dlarnv(int *idist, int *iseed, int *n, d *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarnv(idist, iseed, n, x)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarnv(int *__pyx_v_idist, int *__pyx_v_iseed, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x) {

  /* "scipy/linalg/cython_lapack.pyx":3992
 *     void _fortran_dlarnv "F_FUNC(dlarnv,DLARNV)"(int *idist, int *iseed, int *n, d *x) nogil
 * cdef void dlarnv(int *idist, int *iseed, int *n, d *x) nogil:
 *     _fortran_dlarnv(idist, iseed, n, x)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarnv,DLARNV)(__pyx_v_idist, __pyx_v_iseed, __pyx_v_n, __pyx_v_x);

  /* "scipy/linalg/cython_lapack.pyx":3991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarnv "F_FUNC(dlarnv,DLARNV)"(int *idist, int *iseed, int *n, d *x) nogil
 * cdef void dlarnv(int *idist, int *iseed, int *n, d *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarnv(idist, iseed, n, x)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":3996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarra "F_FUNC(dlarra,DLARRA)"(int *n, d *d, d *e, d *e2, d *spltol, d *tnrm, int *nsplit, int *isplit, int *info) nogil
 * cdef void dlarra(int *n, d *d, d *e, d *e2, d *spltol, d *tnrm, int *nsplit, int *isplit, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarra(n, d, e, e2, spltol, tnrm, nsplit, isplit, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarra(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_spltol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tnrm, int *__pyx_v_nsplit, int *__pyx_v_isplit, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":3997
 *     void _fortran_dlarra "F_FUNC(dlarra,DLARRA)"(int *n, d *d, d *e, d *e2, d *spltol, d *tnrm, int *nsplit, int *isplit, int *info) nogil
 * cdef void dlarra(int *n, d *d, d *e, d *e2, d *spltol, d *tnrm, int *nsplit, int *isplit, int *info) nogil:
 *     _fortran_dlarra(n, d, e, e2, spltol, tnrm, nsplit, isplit, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarra,DLARRA)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_e2, __pyx_v_spltol, __pyx_v_tnrm, __pyx_v_nsplit, __pyx_v_isplit, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":3996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarra "F_FUNC(dlarra,DLARRA)"(int *n, d *d, d *e, d *e2, d *spltol, d *tnrm, int *nsplit, int *isplit, int *info) nogil
 * cdef void dlarra(int *n, d *d, d *e, d *e2, d *spltol, d *tnrm, int *nsplit, int *isplit, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarra(n, d, e, e2, spltol, tnrm, nsplit, isplit, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrb "F_FUNC(dlarrb,DLARRB)"(int *n, d *d, d *lld, int *ifirst, int *ilast, d *rtol1, d *rtol2, int *offset, d *w, d *wgap, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *twist, int *info) nogil
 * cdef void dlarrb(int *n, d *d, d *lld, int *ifirst, int *ilast, d *rtol1, d *rtol2, int *offset, d *w, d *wgap, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *twist, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrb(n, d, lld, ifirst, ilast, rtol1, rtol2, offset, w, wgap, werr, work, iwork, pivmin, spdiam, twist, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarrb(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lld, int *__pyx_v_ifirst, int *__pyx_v_ilast, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol2, int *__pyx_v_offset, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wgap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_spdiam, int *__pyx_v_twist, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4002
 *     void _fortran_dlarrb "F_FUNC(dlarrb,DLARRB)"(int *n, d *d, d *lld, int *ifirst, int *ilast, d *rtol1, d *rtol2, int *offset, d *w, d *wgap, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *twist, int *info) nogil
 * cdef void dlarrb(int *n, d *d, d *lld, int *ifirst, int *ilast, d *rtol1, d *rtol2, int *offset, d *w, d *wgap, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *twist, int *info) nogil:
 *     _fortran_dlarrb(n, d, lld, ifirst, ilast, rtol1, rtol2, offset, w, wgap, werr, work, iwork, pivmin, spdiam, twist, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarrb,DLARRB)(__pyx_v_n, __pyx_v_d, __pyx_v_lld, __pyx_v_ifirst, __pyx_v_ilast, __pyx_v_rtol1, __pyx_v_rtol2, __pyx_v_offset, __pyx_v_w, __pyx_v_wgap, __pyx_v_werr, __pyx_v_work, __pyx_v_iwork, __pyx_v_pivmin, __pyx_v_spdiam, __pyx_v_twist, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrb "F_FUNC(dlarrb,DLARRB)"(int *n, d *d, d *lld, int *ifirst, int *ilast, d *rtol1, d *rtol2, int *offset, d *w, d *wgap, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *twist, int *info) nogil
 * cdef void dlarrb(int *n, d *d, d *lld, int *ifirst, int *ilast, d *rtol1, d *rtol2, int *offset, d *w, d *wgap, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *twist, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrb(n, d, lld, ifirst, ilast, rtol1, rtol2, offset, w, wgap, werr, work, iwork, pivmin, spdiam, twist, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrc "F_FUNC(dlarrc,DLARRC)"(char *jobt, int *n, d *vl, d *vu, d *d, d *e, d *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil
 * cdef void dlarrc(char *jobt, int *n, d *vl, d *vu, d *d, d *e, d *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrc(jobt, n, vl, vu, d, e, pivmin, eigcnt, lcnt, rcnt, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarrc(char *__pyx_v_jobt, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, int *__pyx_v_eigcnt, int *__pyx_v_lcnt, int *__pyx_v_rcnt, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4007
 *     void _fortran_dlarrc "F_FUNC(dlarrc,DLARRC)"(char *jobt, int *n, d *vl, d *vu, d *d, d *e, d *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil
 * cdef void dlarrc(char *jobt, int *n, d *vl, d *vu, d *d, d *e, d *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil:
 *     _fortran_dlarrc(jobt, n, vl, vu, d, e, pivmin, eigcnt, lcnt, rcnt, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarrc,DLARRC)(__pyx_v_jobt, __pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_d, __pyx_v_e, __pyx_v_pivmin, __pyx_v_eigcnt, __pyx_v_lcnt, __pyx_v_rcnt, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrc "F_FUNC(dlarrc,DLARRC)"(char *jobt, int *n, d *vl, d *vu, d *d, d *e, d *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil
 * cdef void dlarrc(char *jobt, int *n, d *vl, d *vu, d *d, d *e, d *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrc(jobt, n, vl, vu, d, e, pivmin, eigcnt, lcnt, rcnt, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrd "F_FUNC(dlarrd,DLARRD)"(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *gers, d *reltol, d *d, d *e, d *e2, d *pivmin, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wl, d *wu, int *iblock, int *indexw, d *work, int *iwork, int *info) nogil
 * cdef void dlarrd(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *gers, d *reltol, d *d, d *e, d *e2, d *pivmin, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wl, d *wu, int *iblock, int *indexw, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrd(range, order, n, vl, vu, il, iu, gers, reltol, d, e, e2, pivmin, nsplit, isplit, m, w, werr, wl, wu, iblock, indexw, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarrd(char *__pyx_v_range, char *__pyx_v_order, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gers, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_reltol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, int *__pyx_v_nsplit, int *__pyx_v_isplit, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wu, int *__pyx_v_iblock, int *__pyx_v_indexw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4012
 *     void _fortran_dlarrd "F_FUNC(dlarrd,DLARRD)"(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *gers, d *reltol, d *d, d *e, d *e2, d *pivmin, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wl, d *wu, int *iblock, int *indexw, d *work, int *iwork, int *info) nogil
 * cdef void dlarrd(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *gers, d *reltol, d *d, d *e, d *e2, d *pivmin, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wl, d *wu, int *iblock, int *indexw, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlarrd(range, order, n, vl, vu, il, iu, gers, reltol, d, e, e2, pivmin, nsplit, isplit, m, w, werr, wl, wu, iblock, indexw, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarrd,DLARRD)(__pyx_v_range, __pyx_v_order, __pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_gers, __pyx_v_reltol, __pyx_v_d, __pyx_v_e, __pyx_v_e2, __pyx_v_pivmin, __pyx_v_nsplit, __pyx_v_isplit, __pyx_v_m, __pyx_v_w, __pyx_v_werr, __pyx_v_wl, __pyx_v_wu, __pyx_v_iblock, __pyx_v_indexw, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrd "F_FUNC(dlarrd,DLARRD)"(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *gers, d *reltol, d *d, d *e, d *e2, d *pivmin, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wl, d *wu, int *iblock, int *indexw, d *work, int *iwork, int *info) nogil
 * cdef void dlarrd(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *gers, d *reltol, d *d, d *e, d *e2, d *pivmin, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wl, d *wu, int *iblock, int *indexw, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrd(range, order, n, vl, vu, il, iu, gers, reltol, d, e, e2, pivmin, nsplit, isplit, m, w, werr, wl, wu, iblock, indexw, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarre "F_FUNC(dlarre,DLARRE)"(char *range, int *n, d *vl, d *vu, int *il, int *iu, d *d, d *e, d *e2, d *rtol1, d *rtol2, d *spltol, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *pivmin, d *work, int *iwork, int *info) nogil
 * cdef void dlarre(char *range, int *n, d *vl, d *vu, int *il, int *iu, d *d, d *e, d *e2, d *rtol1, d *rtol2, d *spltol, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *pivmin, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarre(range, n, vl, vu, il, iu, d, e, e2, rtol1, rtol2, spltol, nsplit, isplit, m, w, werr, wgap, iblock, indexw, gers, pivmin, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarre(char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_spltol, int *__pyx_v_nsplit, int *__pyx_v_isplit, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wgap, int *__pyx_v_iblock, int *__pyx_v_indexw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gers, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4017
 *     void _fortran_dlarre "F_FUNC(dlarre,DLARRE)"(char *range, int *n, d *vl, d *vu, int *il, int *iu, d *d, d *e, d *e2, d *rtol1, d *rtol2, d *spltol, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *pivmin, d *work, int *iwork, int *info) nogil
 * cdef void dlarre(char *range, int *n, d *vl, d *vu, int *il, int *iu, d *d, d *e, d *e2, d *rtol1, d *rtol2, d *spltol, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *pivmin, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlarre(range, n, vl, vu, il, iu, d, e, e2, rtol1, rtol2, spltol, nsplit, isplit, m, w, werr, wgap, iblock, indexw, gers, pivmin, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarre,DLARRE)(__pyx_v_range, __pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_d, __pyx_v_e, __pyx_v_e2, __pyx_v_rtol1, __pyx_v_rtol2, __pyx_v_spltol, __pyx_v_nsplit, __pyx_v_isplit, __pyx_v_m, __pyx_v_w, __pyx_v_werr, __pyx_v_wgap, __pyx_v_iblock, __pyx_v_indexw, __pyx_v_gers, __pyx_v_pivmin, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarre "F_FUNC(dlarre,DLARRE)"(char *range, int *n, d *vl, d *vu, int *il, int *iu, d *d, d *e, d *e2, d *rtol1, d *rtol2, d *spltol, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *pivmin, d *work, int *iwork, int *info) nogil
 * cdef void dlarre(char *range, int *n, d *vl, d *vu, int *il, int *iu, d *d, d *e, d *e2, d *rtol1, d *rtol2, d *spltol, int *nsplit, int *isplit, int *m, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *pivmin, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarre(range, n, vl, vu, il, iu, d, e, e2, rtol1, rtol2, spltol, nsplit, isplit, m, w, werr, wgap, iblock, indexw, gers, pivmin, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrf "F_FUNC(dlarrf,DLARRF)"(int *n, d *d, d *l, d *ld, int *clstrt, int *clend, d *w, d *wgap, d *werr, d *spdiam, d *clgapl, d *clgapr, d *pivmin, d *sigma, d *dplus, d *lplus, d *work, int *info) nogil
 * cdef void dlarrf(int *n, d *d, d *l, d *ld, int *clstrt, int *clend, d *w, d *wgap, d *werr, d *spdiam, d *clgapl, d *clgapr, d *pivmin, d *sigma, d *dplus, d *lplus, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrf(n, d, l, ld, clstrt, clend, w, wgap, werr, spdiam, clgapl, clgapr, pivmin, sigma, dplus, lplus, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarrf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ld, int *__pyx_v_clstrt, int *__pyx_v_clend, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wgap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_spdiam, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_clgapl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_clgapr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sigma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dplus, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lplus, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4022
 *     void _fortran_dlarrf "F_FUNC(dlarrf,DLARRF)"(int *n, d *d, d *l, d *ld, int *clstrt, int *clend, d *w, d *wgap, d *werr, d *spdiam, d *clgapl, d *clgapr, d *pivmin, d *sigma, d *dplus, d *lplus, d *work, int *info) nogil
 * cdef void dlarrf(int *n, d *d, d *l, d *ld, int *clstrt, int *clend, d *w, d *wgap, d *werr, d *spdiam, d *clgapl, d *clgapr, d *pivmin, d *sigma, d *dplus, d *lplus, d *work, int *info) nogil:
 *     _fortran_dlarrf(n, d, l, ld, clstrt, clend, w, wgap, werr, spdiam, clgapl, clgapr, pivmin, sigma, dplus, lplus, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarrf,DLARRF)(__pyx_v_n, __pyx_v_d, __pyx_v_l, __pyx_v_ld, __pyx_v_clstrt, __pyx_v_clend, __pyx_v_w, __pyx_v_wgap, __pyx_v_werr, __pyx_v_spdiam, __pyx_v_clgapl, __pyx_v_clgapr, __pyx_v_pivmin, __pyx_v_sigma, __pyx_v_dplus, __pyx_v_lplus, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrf "F_FUNC(dlarrf,DLARRF)"(int *n, d *d, d *l, d *ld, int *clstrt, int *clend, d *w, d *wgap, d *werr, d *spdiam, d *clgapl, d *clgapr, d *pivmin, d *sigma, d *dplus, d *lplus, d *work, int *info) nogil
 * cdef void dlarrf(int *n, d *d, d *l, d *ld, int *clstrt, int *clend, d *w, d *wgap, d *werr, d *spdiam, d *clgapl, d *clgapr, d *pivmin, d *sigma, d *dplus, d *lplus, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrf(n, d, l, ld, clstrt, clend, w, wgap, werr, spdiam, clgapl, clgapr, pivmin, sigma, dplus, lplus, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrj "F_FUNC(dlarrj,DLARRJ)"(int *n, d *d, d *e2, int *ifirst, int *ilast, d *rtol, int *offset, d *w, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *info) nogil
 * cdef void dlarrj(int *n, d *d, d *e2, int *ifirst, int *ilast, d *rtol, int *offset, d *w, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrj(n, d, e2, ifirst, ilast, rtol, offset, w, werr, work, iwork, pivmin, spdiam, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarrj(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e2, int *__pyx_v_ifirst, int *__pyx_v_ilast, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol, int *__pyx_v_offset, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_spdiam, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4027
 *     void _fortran_dlarrj "F_FUNC(dlarrj,DLARRJ)"(int *n, d *d, d *e2, int *ifirst, int *ilast, d *rtol, int *offset, d *w, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *info) nogil
 * cdef void dlarrj(int *n, d *d, d *e2, int *ifirst, int *ilast, d *rtol, int *offset, d *w, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *info) nogil:
 *     _fortran_dlarrj(n, d, e2, ifirst, ilast, rtol, offset, w, werr, work, iwork, pivmin, spdiam, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarrj,DLARRJ)(__pyx_v_n, __pyx_v_d, __pyx_v_e2, __pyx_v_ifirst, __pyx_v_ilast, __pyx_v_rtol, __pyx_v_offset, __pyx_v_w, __pyx_v_werr, __pyx_v_work, __pyx_v_iwork, __pyx_v_pivmin, __pyx_v_spdiam, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrj "F_FUNC(dlarrj,DLARRJ)"(int *n, d *d, d *e2, int *ifirst, int *ilast, d *rtol, int *offset, d *w, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *info) nogil
 * cdef void dlarrj(int *n, d *d, d *e2, int *ifirst, int *ilast, d *rtol, int *offset, d *w, d *werr, d *work, int *iwork, d *pivmin, d *spdiam, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrj(n, d, e2, ifirst, ilast, rtol, offset, w, werr, work, iwork, pivmin, spdiam, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrk "F_FUNC(dlarrk,DLARRK)"(int *n, int *iw, d *gl, d *gu, d *d, d *e2, d *pivmin, d *reltol, d *w, d *werr, int *info) nogil
 * cdef void dlarrk(int *n, int *iw, d *gl, d *gu, d *d, d *e2, d *pivmin, d *reltol, d *w, d *werr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrk(n, iw, gl, gu, d, e2, pivmin, reltol, w, werr, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarrk(int *__pyx_v_n, int *__pyx_v_iw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_reltol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_werr, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4032
 *     void _fortran_dlarrk "F_FUNC(dlarrk,DLARRK)"(int *n, int *iw, d *gl, d *gu, d *d, d *e2, d *pivmin, d *reltol, d *w, d *werr, int *info) nogil
 * cdef void dlarrk(int *n, int *iw, d *gl, d *gu, d *d, d *e2, d *pivmin, d *reltol, d *w, d *werr, int *info) nogil:
 *     _fortran_dlarrk(n, iw, gl, gu, d, e2, pivmin, reltol, w, werr, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarrk,DLARRK)(__pyx_v_n, __pyx_v_iw, __pyx_v_gl, __pyx_v_gu, __pyx_v_d, __pyx_v_e2, __pyx_v_pivmin, __pyx_v_reltol, __pyx_v_w, __pyx_v_werr, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrk "F_FUNC(dlarrk,DLARRK)"(int *n, int *iw, d *gl, d *gu, d *d, d *e2, d *pivmin, d *reltol, d *w, d *werr, int *info) nogil
 * cdef void dlarrk(int *n, int *iw, d *gl, d *gu, d *d, d *e2, d *pivmin, d *reltol, d *w, d *werr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrk(n, iw, gl, gu, d, e2, pivmin, reltol, w, werr, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrr "F_FUNC(dlarrr,DLARRR)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dlarrr(int *n, d *d, d *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrr(n, d, e, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarrr(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4037
 *     void _fortran_dlarrr "F_FUNC(dlarrr,DLARRR)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dlarrr(int *n, d *d, d *e, int *info) nogil:
 *     _fortran_dlarrr(n, d, e, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarrr,DLARRR)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrr "F_FUNC(dlarrr,DLARRR)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dlarrr(int *n, d *d, d *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrr(n, d, e, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrv "F_FUNC(dlarrv,DLARRV)"(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil
 * cdef void dlarrv(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, z, ldz, isuppz, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarrv(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, int *__pyx_v_isplit, int *__pyx_v_m, int *__pyx_v_dol, int *__pyx_v_dou, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_minrgp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wgap, int *__pyx_v_iblock, int *__pyx_v_indexw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gers, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4042
 *     void _fortran_dlarrv "F_FUNC(dlarrv,DLARRV)"(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil
 * cdef void dlarrv(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, z, ldz, isuppz, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarrv,DLARRV)(__pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_d, __pyx_v_l, __pyx_v_pivmin, __pyx_v_isplit, __pyx_v_m, __pyx_v_dol, __pyx_v_dou, __pyx_v_minrgp, __pyx_v_rtol1, __pyx_v_rtol2, __pyx_v_w, __pyx_v_werr, __pyx_v_wgap, __pyx_v_iblock, __pyx_v_indexw, __pyx_v_gers, __pyx_v_z, __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarrv "F_FUNC(dlarrv,DLARRV)"(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil
 * cdef void dlarrv(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, d *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, z, ldz, isuppz, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlartg "F_FUNC(dlartg,DLARTG)"(d *f, d *g, d *cs, d *sn, d *r) nogil
 * cdef void dlartg(d *f, d *g, d *cs, d *sn, d *r) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlartg(f, g, cs, sn, r)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlartg(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_f, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_g, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sn, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r) {

  /* "scipy/linalg/cython_lapack.pyx":4047
 *     void _fortran_dlartg "F_FUNC(dlartg,DLARTG)"(d *f, d *g, d *cs, d *sn, d *r) nogil
 * cdef void dlartg(d *f, d *g, d *cs, d *sn, d *r) nogil:
 *     _fortran_dlartg(f, g, cs, sn, r)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlartg,DLARTG)(__pyx_v_f, __pyx_v_g, __pyx_v_cs, __pyx_v_sn, __pyx_v_r);

  /* "scipy/linalg/cython_lapack.pyx":4046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlartg "F_FUNC(dlartg,DLARTG)"(d *f, d *g, d *cs, d *sn, d *r) nogil
 * cdef void dlartg(d *f, d *g, d *cs, d *sn, d *r) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlartg(f, g, cs, sn, r)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlartv "F_FUNC(dlartv,DLARTV)"(int *n, d *x, int *incx, d *y, int *incy, d *c, d *s, int *incc) nogil
 * cdef void dlartv(int *n, d *x, int *incx, d *y, int *incy, d *c, d *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlartv(n, x, incx, y, incy, c, s, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlartv(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":4052
 *     void _fortran_dlartv "F_FUNC(dlartv,DLARTV)"(int *n, d *x, int *incx, d *y, int *incy, d *c, d *s, int *incc) nogil
 * cdef void dlartv(int *n, d *x, int *incx, d *y, int *incy, d *c, d *s, int *incc) nogil:
 *     _fortran_dlartv(n, x, incx, y, incy, c, s, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlartv,DLARTV)(__pyx_v_n, __pyx_v_x, __pyx_v_incx, __pyx_v_y, __pyx_v_incy, __pyx_v_c, __pyx_v_s, __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":4051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlartv "F_FUNC(dlartv,DLARTV)"(int *n, d *x, int *incx, d *y, int *incy, d *c, d *s, int *incc) nogil
 * cdef void dlartv(int *n, d *x, int *incx, d *y, int *incy, d *c, d *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlartv(n, x, incx, y, incy, c, s, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaruv "F_FUNC(dlaruv,DLARUV)"(int *iseed, int *n, d *x) nogil
 * cdef void dlaruv(int *iseed, int *n, d *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaruv(iseed, n, x)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaruv(int *__pyx_v_iseed, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x) {

  /* "scipy/linalg/cython_lapack.pyx":4057
 *     void _fortran_dlaruv "F_FUNC(dlaruv,DLARUV)"(int *iseed, int *n, d *x) nogil
 * cdef void dlaruv(int *iseed, int *n, d *x) nogil:
 *     _fortran_dlaruv(iseed, n, x)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaruv,DLARUV)(__pyx_v_iseed, __pyx_v_n, __pyx_v_x);

  /* "scipy/linalg/cython_lapack.pyx":4056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaruv "F_FUNC(dlaruv,DLARUV)"(int *iseed, int *n, d *x) nogil
 * cdef void dlaruv(int *iseed, int *n, d *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaruv(iseed, n, x)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarz "F_FUNC(dlarz,DLARZ)"(char *side, int *m, int *n, int *l, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarz(char *side, int *m, int *n, int *l, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarz(side, m, n, l, v, incv, tau, c, ldc, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarz(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_incv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":4062
 *     void _fortran_dlarz "F_FUNC(dlarz,DLARZ)"(char *side, int *m, int *n, int *l, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarz(char *side, int *m, int *n, int *l, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil:
 *     _fortran_dlarz(side, m, n, l, v, incv, tau, c, ldc, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarz,DLARZ)(__pyx_v_side, __pyx_v_m, __pyx_v_n, __pyx_v_l, __pyx_v_v, __pyx_v_incv, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":4061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarz "F_FUNC(dlarz,DLARZ)"(char *side, int *m, int *n, int *l, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil
 * cdef void dlarz(char *side, int *m, int *n, int *l, d *v, int *incv, d *tau, d *c, int *ldc, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarz(side, m, n, l, v, incv, tau, c, ldc, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarzb "F_FUNC(dlarzb,DLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil
 * cdef void dlarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarzb(side, trans, direct, storev, m, n, k, l, v, ldv, t, ldt, c, ldc, work, ldwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarzb(char *__pyx_v_side, char *__pyx_v_trans, char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_ldwork) {

  /* "scipy/linalg/cython_lapack.pyx":4067
 *     void _fortran_dlarzb "F_FUNC(dlarzb,DLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil
 * cdef void dlarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil:
 *     _fortran_dlarzb(side, trans, direct, storev, m, n, k, l, v, ldv, t, ldt, c, ldc, work, ldwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarzb,DLARZB)(__pyx_v_side, __pyx_v_trans, __pyx_v_direct, __pyx_v_storev, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, __pyx_v_v, __pyx_v_ldv, __pyx_v_t, __pyx_v_ldt, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_ldwork);

  /* "scipy/linalg/cython_lapack.pyx":4066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarzb "F_FUNC(dlarzb,DLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil
 * cdef void dlarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, d *v, int *ldv, d *t, int *ldt, d *c, int *ldc, d *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarzb(side, trans, direct, storev, m, n, k, l, v, ldv, t, ldt, c, ldc, work, ldwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarzt "F_FUNC(dlarzt,DLARZT)"(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil
 * cdef void dlarzt(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarzt(direct, storev, n, k, v, ldv, tau, t, ldt)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlarzt(char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt) {

  /* "scipy/linalg/cython_lapack.pyx":4072
 *     void _fortran_dlarzt "F_FUNC(dlarzt,DLARZT)"(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil
 * cdef void dlarzt(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil:
 *     _fortran_dlarzt(direct, storev, n, k, v, ldv, tau, t, ldt)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlarzt,DLARZT)(__pyx_v_direct, __pyx_v_storev, __pyx_v_n, __pyx_v_k, __pyx_v_v, __pyx_v_ldv, __pyx_v_tau, __pyx_v_t, __pyx_v_ldt);

  /* "scipy/linalg/cython_lapack.pyx":4071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlarzt "F_FUNC(dlarzt,DLARZT)"(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil
 * cdef void dlarzt(char *direct, char *storev, int *n, int *k, d *v, int *ldv, d *tau, d *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlarzt(direct, storev, n, k, v, ldv, tau, t, ldt)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlas2 "F_FUNC(dlas2,DLAS2)"(d *f, d *g, d *h, d *ssmin, d *ssmax) nogil
 * cdef void dlas2(d *f, d *g, d *h, d *ssmin, d *ssmax) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlas2(f, g, h, ssmin, ssmax)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlas2(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_f, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_g, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ssmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ssmax) {

  /* "scipy/linalg/cython_lapack.pyx":4077
 *     void _fortran_dlas2 "F_FUNC(dlas2,DLAS2)"(d *f, d *g, d *h, d *ssmin, d *ssmax) nogil
 * cdef void dlas2(d *f, d *g, d *h, d *ssmin, d *ssmax) nogil:
 *     _fortran_dlas2(f, g, h, ssmin, ssmax)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlas2,DLAS2)(__pyx_v_f, __pyx_v_g, __pyx_v_h, __pyx_v_ssmin, __pyx_v_ssmax);

  /* "scipy/linalg/cython_lapack.pyx":4076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlas2 "F_FUNC(dlas2,DLAS2)"(d *f, d *g, d *h, d *ssmin, d *ssmax) nogil
 * cdef void dlas2(d *f, d *g, d *h, d *ssmin, d *ssmax) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlas2(f, g, h, ssmin, ssmax)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlascl "F_FUNC(dlascl,DLASCL)"(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlascl(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlascl(type_bn, kl, ku, cfrom, cto, m, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlascl(char *__pyx_v_type_bn, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cfrom, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cto, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4082
 *     void _fortran_dlascl "F_FUNC(dlascl,DLASCL)"(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlascl(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, d *a, int *lda, int *info) nogil:
 *     _fortran_dlascl(type_bn, kl, ku, cfrom, cto, m, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlascl,DLASCL)(__pyx_v_type_bn, __pyx_v_kl, __pyx_v_ku, __pyx_v_cfrom, __pyx_v_cto, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlascl "F_FUNC(dlascl,DLASCL)"(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlascl(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlascl(type_bn, kl, ku, cfrom, cto, m, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd0 "F_FUNC(dlasd0,DLASD0)"(int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, int *smlsiz, int *iwork, d *work, int *info) nogil
 * cdef void dlasd0(int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, int *smlsiz, int *iwork, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd0(n, sqre, d, e, u, ldu, vt, ldvt, smlsiz, iwork, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd0(int *__pyx_v_n, int *__pyx_v_sqre, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, int *__pyx_v_smlsiz, int *__pyx_v_iwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4087
 *     void _fortran_dlasd0 "F_FUNC(dlasd0,DLASD0)"(int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, int *smlsiz, int *iwork, d *work, int *info) nogil
 * cdef void dlasd0(int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, int *smlsiz, int *iwork, d *work, int *info) nogil:
 *     _fortran_dlasd0(n, sqre, d, e, u, ldu, vt, ldvt, smlsiz, iwork, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd0,DLASD0)(__pyx_v_n, __pyx_v_sqre, __pyx_v_d, __pyx_v_e, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_smlsiz, __pyx_v_iwork, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd0 "F_FUNC(dlasd0,DLASD0)"(int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, int *smlsiz, int *iwork, d *work, int *info) nogil
 * cdef void dlasd0(int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *ldvt, int *smlsiz, int *iwork, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd0(n, sqre, d, e, u, ldu, vt, ldvt, smlsiz, iwork, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd1 "F_FUNC(dlasd1,DLASD1)"(int *nl, int *nr, int *sqre, d *d, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, int *idxq, int *iwork, d *work, int *info) nogil
 * cdef void dlasd1(int *nl, int *nr, int *sqre, d *d, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, int *idxq, int *iwork, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd1(nl, nr, sqre, d, alpha, beta, u, ldu, vt, ldvt, idxq, iwork, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd1(int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, int *__pyx_v_idxq, int *__pyx_v_iwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4092
 *     void _fortran_dlasd1 "F_FUNC(dlasd1,DLASD1)"(int *nl, int *nr, int *sqre, d *d, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, int *idxq, int *iwork, d *work, int *info) nogil
 * cdef void dlasd1(int *nl, int *nr, int *sqre, d *d, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, int *idxq, int *iwork, d *work, int *info) nogil:
 *     _fortran_dlasd1(nl, nr, sqre, d, alpha, beta, u, ldu, vt, ldvt, idxq, iwork, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd1,DLASD1)(__pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_d, __pyx_v_alpha, __pyx_v_beta, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_idxq, __pyx_v_iwork, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd1 "F_FUNC(dlasd1,DLASD1)"(int *nl, int *nr, int *sqre, d *d, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, int *idxq, int *iwork, d *work, int *info) nogil
 * cdef void dlasd1(int *nl, int *nr, int *sqre, d *d, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, int *idxq, int *iwork, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd1(nl, nr, sqre, d, alpha, beta, u, ldu, vt, ldvt, idxq, iwork, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd2 "F_FUNC(dlasd2,DLASD2)"(int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, d *dsigma, d *u2, int *ldu2, d *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil
 * cdef void dlasd2(int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, d *dsigma, d *u2, int *ldu2, d *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd2(nl, nr, sqre, k, d, z, alpha, beta, u, ldu, vt, ldvt, dsigma, u2, ldu2, vt2, ldvt2, idxp, idx, idxc, idxq, coltyp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd2(int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dsigma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u2, int *__pyx_v_ldu2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt2, int *__pyx_v_ldvt2, int *__pyx_v_idxp, int *__pyx_v_idx, int *__pyx_v_idxc, int *__pyx_v_idxq, int *__pyx_v_coltyp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4097
 *     void _fortran_dlasd2 "F_FUNC(dlasd2,DLASD2)"(int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, d *dsigma, d *u2, int *ldu2, d *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil
 * cdef void dlasd2(int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, d *dsigma, d *u2, int *ldu2, d *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil:
 *     _fortran_dlasd2(nl, nr, sqre, k, d, z, alpha, beta, u, ldu, vt, ldvt, dsigma, u2, ldu2, vt2, ldvt2, idxp, idx, idxc, idxq, coltyp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd2,DLASD2)(__pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_k, __pyx_v_d, __pyx_v_z, __pyx_v_alpha, __pyx_v_beta, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_dsigma, __pyx_v_u2, __pyx_v_ldu2, __pyx_v_vt2, __pyx_v_ldvt2, __pyx_v_idxp, __pyx_v_idx, __pyx_v_idxc, __pyx_v_idxq, __pyx_v_coltyp, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd2 "F_FUNC(dlasd2,DLASD2)"(int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, d *dsigma, d *u2, int *ldu2, d *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil
 * cdef void dlasd2(int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *alpha, d *beta, d *u, int *ldu, d *vt, int *ldvt, d *dsigma, d *u2, int *ldu2, d *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd2(nl, nr, sqre, k, d, z, alpha, beta, u, ldu, vt, ldvt, dsigma, u2, ldu2, vt2, ldvt2, idxp, idx, idxc, idxq, coltyp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd3 "F_FUNC(dlasd3,DLASD3)"(int *nl, int *nr, int *sqre, int *k, d *d, d *q, int *ldq, d *dsigma, d *u, int *ldu, d *u2, int *ldu2, d *vt, int *ldvt, d *vt2, int *ldvt2, int *idxc, int *ctot, d *z, int *info) nogil
 * cdef void dlasd3(int *nl, int *nr, int *sqre, int *k, d *d, d *q, int *ldq, d *dsigma, d *u, int *ldu, d *u2, int *ldu2, d *vt, int *ldvt, d *vt2, int *ldvt2, int *idxc, int *ctot, d *z, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd3(nl, nr, sqre, k, d, q, ldq, dsigma, u, ldu, u2, ldu2, vt, ldvt, vt2, ldvt2, idxc, ctot, z, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd3(int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dsigma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u2, int *__pyx_v_ldu2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt2, int *__pyx_v_ldvt2, int *__pyx_v_idxc, int *__pyx_v_ctot, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4102
 *     void _fortran_dlasd3 "F_FUNC(dlasd3,DLASD3)"(int *nl, int *nr, int *sqre, int *k, d *d, d *q, int *ldq, d *dsigma, d *u, int *ldu, d *u2, int *ldu2, d *vt, int *ldvt, d *vt2, int *ldvt2, int *idxc, int *ctot, d *z, int *info) nogil
 * cdef void dlasd3(int *nl, int *nr, int *sqre, int *k, d *d, d *q, int *ldq, d *dsigma, d *u, int *ldu, d *u2, int *ldu2, d *vt, int *ldvt, d *vt2, int *ldvt2, int *idxc, int *ctot, d *z, int *info) nogil:
 *     _fortran_dlasd3(nl, nr, sqre, k, d, q, ldq, dsigma, u, ldu, u2, ldu2, vt, ldvt, vt2, ldvt2, idxc, ctot, z, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd3,DLASD3)(__pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_k, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_dsigma, __pyx_v_u, __pyx_v_ldu, __pyx_v_u2, __pyx_v_ldu2, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_vt2, __pyx_v_ldvt2, __pyx_v_idxc, __pyx_v_ctot, __pyx_v_z, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd3 "F_FUNC(dlasd3,DLASD3)"(int *nl, int *nr, int *sqre, int *k, d *d, d *q, int *ldq, d *dsigma, d *u, int *ldu, d *u2, int *ldu2, d *vt, int *ldvt, d *vt2, int *ldvt2, int *idxc, int *ctot, d *z, int *info) nogil
 * cdef void dlasd3(int *nl, int *nr, int *sqre, int *k, d *d, d *q, int *ldq, d *dsigma, d *u, int *ldu, d *u2, int *ldu2, d *vt, int *ldvt, d *vt2, int *ldvt2, int *idxc, int *ctot, d *z, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd3(nl, nr, sqre, k, d, q, ldq, dsigma, u, ldu, u2, ldu2, vt, ldvt, vt2, ldvt2, idxc, ctot, z, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd4 "F_FUNC(dlasd4,DLASD4)"(int *n, int *i, d *d, d *z, d *delta, d *rho, d *sigma, d *work, int *info) nogil
 * cdef void dlasd4(int *n, int *i, d *d, d *z, d *delta, d *rho, d *sigma, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd4(n, i, d, z, delta, rho, sigma, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd4(int *__pyx_v_n, int *__pyx_v_i, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_delta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sigma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4107
 *     void _fortran_dlasd4 "F_FUNC(dlasd4,DLASD4)"(int *n, int *i, d *d, d *z, d *delta, d *rho, d *sigma, d *work, int *info) nogil
 * cdef void dlasd4(int *n, int *i, d *d, d *z, d *delta, d *rho, d *sigma, d *work, int *info) nogil:
 *     _fortran_dlasd4(n, i, d, z, delta, rho, sigma, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd4,DLASD4)(__pyx_v_n, __pyx_v_i, __pyx_v_d, __pyx_v_z, __pyx_v_delta, __pyx_v_rho, __pyx_v_sigma, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd4 "F_FUNC(dlasd4,DLASD4)"(int *n, int *i, d *d, d *z, d *delta, d *rho, d *sigma, d *work, int *info) nogil
 * cdef void dlasd4(int *n, int *i, d *d, d *z, d *delta, d *rho, d *sigma, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd4(n, i, d, z, delta, rho, sigma, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd5 "F_FUNC(dlasd5,DLASD5)"(int *i, d *d, d *z, d *delta, d *rho, d *dsigma, d *work) nogil
 * cdef void dlasd5(int *i, d *d, d *z, d *delta, d *rho, d *dsigma, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd5(i, d, z, delta, rho, dsigma, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd5(int *__pyx_v_i, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_delta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dsigma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":4112
 *     void _fortran_dlasd5 "F_FUNC(dlasd5,DLASD5)"(int *i, d *d, d *z, d *delta, d *rho, d *dsigma, d *work) nogil
 * cdef void dlasd5(int *i, d *d, d *z, d *delta, d *rho, d *dsigma, d *work) nogil:
 *     _fortran_dlasd5(i, d, z, delta, rho, dsigma, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd5,DLASD5)(__pyx_v_i, __pyx_v_d, __pyx_v_z, __pyx_v_delta, __pyx_v_rho, __pyx_v_dsigma, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":4111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd5 "F_FUNC(dlasd5,DLASD5)"(int *i, d *d, d *z, d *delta, d *rho, d *dsigma, d *work) nogil
 * cdef void dlasd5(int *i, d *d, d *z, d *delta, d *rho, d *dsigma, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd5(i, d, z, delta, rho, dsigma, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd6 "F_FUNC(dlasd6,DLASD6)"(int *icompq, int *nl, int *nr, int *sqre, d *d, d *vf, d *vl, d *alpha, d *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlasd6(int *icompq, int *nl, int *nr, int *sqre, d *d, d *vf, d *vl, d *alpha, d *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd6(icompq, nl, nr, sqre, d, vf, vl, alpha, beta, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd6(int *__pyx_v_icompq, int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, int *__pyx_v_idxq, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, int *__pyx_v_ldgnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_poles, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4117
 *     void _fortran_dlasd6 "F_FUNC(dlasd6,DLASD6)"(int *icompq, int *nl, int *nr, int *sqre, d *d, d *vf, d *vl, d *alpha, d *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlasd6(int *icompq, int *nl, int *nr, int *sqre, d *d, d *vf, d *vl, d *alpha, d *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlasd6(icompq, nl, nr, sqre, d, vf, vl, alpha, beta, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd6,DLASD6)(__pyx_v_icompq, __pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_d, __pyx_v_vf, __pyx_v_vl, __pyx_v_alpha, __pyx_v_beta, __pyx_v_idxq, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_givnum, __pyx_v_ldgnum, __pyx_v_poles, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_k, __pyx_v_c, __pyx_v_s, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd6 "F_FUNC(dlasd6,DLASD6)"(int *icompq, int *nl, int *nr, int *sqre, d *d, d *vf, d *vl, d *alpha, d *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlasd6(int *icompq, int *nl, int *nr, int *sqre, d *d, d *vf, d *vl, d *alpha, d *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd6(icompq, nl, nr, sqre, d, vf, vl, alpha, beta, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd7 "F_FUNC(dlasd7,DLASD7)"(int *icompq, int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *zw, d *vf, d *vfw, d *vl, d *vlw, d *alpha, d *beta, d *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *c, d *s, int *info) nogil
 * cdef void dlasd7(int *icompq, int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *zw, d *vf, d *vfw, d *vl, d *vlw, d *alpha, d *beta, d *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *c, d *s, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd7(icompq, nl, nr, sqre, k, d, z, zw, vf, vfw, vl, vlw, alpha, beta, dsigma, idx, idxp, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, c, s, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd7(int *__pyx_v_icompq, int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_zw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vfw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vlw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dsigma, int *__pyx_v_idx, int *__pyx_v_idxp, int *__pyx_v_idxq, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, int *__pyx_v_ldgnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4122
 *     void _fortran_dlasd7 "F_FUNC(dlasd7,DLASD7)"(int *icompq, int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *zw, d *vf, d *vfw, d *vl, d *vlw, d *alpha, d *beta, d *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *c, d *s, int *info) nogil
 * cdef void dlasd7(int *icompq, int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *zw, d *vf, d *vfw, d *vl, d *vlw, d *alpha, d *beta, d *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *c, d *s, int *info) nogil:
 *     _fortran_dlasd7(icompq, nl, nr, sqre, k, d, z, zw, vf, vfw, vl, vlw, alpha, beta, dsigma, idx, idxp, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, c, s, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd7,DLASD7)(__pyx_v_icompq, __pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_k, __pyx_v_d, __pyx_v_z, __pyx_v_zw, __pyx_v_vf, __pyx_v_vfw, __pyx_v_vl, __pyx_v_vlw, __pyx_v_alpha, __pyx_v_beta, __pyx_v_dsigma, __pyx_v_idx, __pyx_v_idxp, __pyx_v_idxq, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_givnum, __pyx_v_ldgnum, __pyx_v_c, __pyx_v_s, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd7 "F_FUNC(dlasd7,DLASD7)"(int *icompq, int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *zw, d *vf, d *vfw, d *vl, d *vlw, d *alpha, d *beta, d *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *c, d *s, int *info) nogil
 * cdef void dlasd7(int *icompq, int *nl, int *nr, int *sqre, int *k, d *d, d *z, d *zw, d *vf, d *vfw, d *vl, d *vlw, d *alpha, d *beta, d *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *c, d *s, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd7(icompq, nl, nr, sqre, k, d, z, zw, vf, vfw, vl, vlw, alpha, beta, dsigma, idx, idxp, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, c, s, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd8 "F_FUNC(dlasd8,DLASD8)"(int *icompq, int *k, d *d, d *z, d *vf, d *vl, d *difl, d *difr, int *lddifr, d *dsigma, d *work, int *info) nogil
 * cdef void dlasd8(int *icompq, int *k, d *d, d *z, d *vf, d *vl, d *difl, d *difr, int *lddifr, d *dsigma, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd8(icompq, k, d, z, vf, vl, difl, difr, lddifr, dsigma, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasd8(int *__pyx_v_icompq, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difr, int *__pyx_v_lddifr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dsigma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4127
 *     void _fortran_dlasd8 "F_FUNC(dlasd8,DLASD8)"(int *icompq, int *k, d *d, d *z, d *vf, d *vl, d *difl, d *difr, int *lddifr, d *dsigma, d *work, int *info) nogil
 * cdef void dlasd8(int *icompq, int *k, d *d, d *z, d *vf, d *vl, d *difl, d *difr, int *lddifr, d *dsigma, d *work, int *info) nogil:
 *     _fortran_dlasd8(icompq, k, d, z, vf, vl, difl, difr, lddifr, dsigma, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasd8,DLASD8)(__pyx_v_icompq, __pyx_v_k, __pyx_v_d, __pyx_v_z, __pyx_v_vf, __pyx_v_vl, __pyx_v_difl, __pyx_v_difr, __pyx_v_lddifr, __pyx_v_dsigma, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasd8 "F_FUNC(dlasd8,DLASD8)"(int *icompq, int *k, d *d, d *z, d *vf, d *vl, d *difl, d *difr, int *lddifr, d *dsigma, d *work, int *info) nogil
 * cdef void dlasd8(int *icompq, int *k, d *d, d *z, d *vf, d *vl, d *difl, d *difr, int *lddifr, d *dsigma, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasd8(icompq, k, d, z, vf, vl, difl, difr, lddifr, dsigma, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasda "F_FUNC(dlasda,DLASDA)"(int *icompq, int *smlsiz, int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlasda(int *icompq, int *smlsiz, int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasda(icompq, smlsiz, n, sqre, d, e, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasda(int *__pyx_v_icompq, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_sqre, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_poles, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, int *__pyx_v_perm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4132
 *     void _fortran_dlasda "F_FUNC(dlasda,DLASDA)"(int *icompq, int *smlsiz, int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlasda(int *icompq, int *smlsiz, int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil:
 *     _fortran_dlasda(icompq, smlsiz, n, sqre, d, e, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasda,DLASDA)(__pyx_v_icompq, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_sqre, __pyx_v_d, __pyx_v_e, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_k, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_poles, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_perm, __pyx_v_givnum, __pyx_v_c, __pyx_v_s, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasda "F_FUNC(dlasda,DLASDA)"(int *icompq, int *smlsiz, int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil
 * cdef void dlasda(int *icompq, int *smlsiz, int *n, int *sqre, d *d, d *e, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasda(icompq, smlsiz, n, sqre, d, e, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasdq "F_FUNC(dlasdq,DLASDQ)"(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dlasdq(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasdq(uplo, sqre, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasdq(char *__pyx_v_uplo, int *__pyx_v_sqre, int *__pyx_v_n, int *__pyx_v_ncvt, int *__pyx_v_nru, int *__pyx_v_ncc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4137
 *     void _fortran_dlasdq "F_FUNC(dlasdq,DLASDQ)"(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dlasdq(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dlasdq(uplo, sqre, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasdq,DLASDQ)(__pyx_v_uplo, __pyx_v_sqre, __pyx_v_n, __pyx_v_ncvt, __pyx_v_nru, __pyx_v_ncc, __pyx_v_d, __pyx_v_e, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_u, __pyx_v_ldu, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasdq "F_FUNC(dlasdq,DLASDQ)"(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dlasdq(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, d *vt, int *ldvt, d *u, int *ldu, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasdq(uplo, sqre, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasdt "F_FUNC(dlasdt,DLASDT)"(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil
 * cdef void dlasdt(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasdt(n, lvl, nd, inode, ndiml, ndimr, msub)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasdt(int *__pyx_v_n, int *__pyx_v_lvl, int *__pyx_v_nd, int *__pyx_v_inode, int *__pyx_v_ndiml, int *__pyx_v_ndimr, int *__pyx_v_msub) {

  /* "scipy/linalg/cython_lapack.pyx":4142
 *     void _fortran_dlasdt "F_FUNC(dlasdt,DLASDT)"(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil
 * cdef void dlasdt(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil:
 *     _fortran_dlasdt(n, lvl, nd, inode, ndiml, ndimr, msub)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasdt,DLASDT)(__pyx_v_n, __pyx_v_lvl, __pyx_v_nd, __pyx_v_inode, __pyx_v_ndiml, __pyx_v_ndimr, __pyx_v_msub);

  /* "scipy/linalg/cython_lapack.pyx":4141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasdt "F_FUNC(dlasdt,DLASDT)"(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil
 * cdef void dlasdt(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasdt(n, lvl, nd, inode, ndiml, ndimr, msub)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaset "F_FUNC(dlaset,DLASET)"(char *uplo, int *m, int *n, d *alpha, d *beta, d *a, int *lda) nogil
 * cdef void dlaset(char *uplo, int *m, int *n, d *alpha, d *beta, d *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaset(uplo, m, n, alpha, beta, a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaset(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":4147
 *     void _fortran_dlaset "F_FUNC(dlaset,DLASET)"(char *uplo, int *m, int *n, d *alpha, d *beta, d *a, int *lda) nogil
 * cdef void dlaset(char *uplo, int *m, int *n, d *alpha, d *beta, d *a, int *lda) nogil:
 *     _fortran_dlaset(uplo, m, n, alpha, beta, a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaset,DLASET)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, __pyx_v_alpha, __pyx_v_beta, __pyx_v_a, __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":4146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaset "F_FUNC(dlaset,DLASET)"(char *uplo, int *m, int *n, d *alpha, d *beta, d *a, int *lda) nogil
 * cdef void dlaset(char *uplo, int *m, int *n, d *alpha, d *beta, d *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaset(uplo, m, n, alpha, beta, a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasq1 "F_FUNC(dlasq1,DLASQ1)"(int *n, d *d, d *e, d *work, int *info) nogil
 * cdef void dlasq1(int *n, d *d, d *e, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasq1(n, d, e, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasq1(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4152
 *     void _fortran_dlasq1 "F_FUNC(dlasq1,DLASQ1)"(int *n, d *d, d *e, d *work, int *info) nogil
 * cdef void dlasq1(int *n, d *d, d *e, d *work, int *info) nogil:
 *     _fortran_dlasq1(n, d, e, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasq1,DLASQ1)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasq1 "F_FUNC(dlasq1,DLASQ1)"(int *n, d *d, d *e, d *work, int *info) nogil
 * cdef void dlasq1(int *n, d *d, d *e, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasq1(n, d, e, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasq2 "F_FUNC(dlasq2,DLASQ2)"(int *n, d *z, int *info) nogil
 * cdef void dlasq2(int *n, d *z, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasq2(n, z, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasq2(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4157
 *     void _fortran_dlasq2 "F_FUNC(dlasq2,DLASQ2)"(int *n, d *z, int *info) nogil
 * cdef void dlasq2(int *n, d *z, int *info) nogil:
 *     _fortran_dlasq2(n, z, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasq2,DLASQ2)(__pyx_v_n, __pyx_v_z, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasq2 "F_FUNC(dlasq2,DLASQ2)"(int *n, d *z, int *info) nogil
 * cdef void dlasq2(int *n, d *z, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasq2(n, z, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasq6 "F_FUNC(dlasq6,DLASQ6)"(int *i0, int *n0, d *z, int *pp, d *dmin, d *dmin1, d *dmin2, d *dn, d *dnm1, d *dnm2) nogil
 * cdef void dlasq6(int *i0, int *n0, d *z, int *pp, d *dmin, d *dmin1, d *dmin2, d *dn, d *dnm1, d *dnm2) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasq6(i0, n0, z, pp, dmin, dmin1, dmin2, dn, dnm1, dnm2)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasq6(int *__pyx_v_i0, int *__pyx_v_n0, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_pp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dmin1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dmin2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dn, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dnm1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dnm2) {

  /* "scipy/linalg/cython_lapack.pyx":4162
 *     void _fortran_dlasq6 "F_FUNC(dlasq6,DLASQ6)"(int *i0, int *n0, d *z, int *pp, d *dmin, d *dmin1, d *dmin2, d *dn, d *dnm1, d *dnm2) nogil
 * cdef void dlasq6(int *i0, int *n0, d *z, int *pp, d *dmin, d *dmin1, d *dmin2, d *dn, d *dnm1, d *dnm2) nogil:
 *     _fortran_dlasq6(i0, n0, z, pp, dmin, dmin1, dmin2, dn, dnm1, dnm2)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasq6,DLASQ6)(__pyx_v_i0, __pyx_v_n0, __pyx_v_z, __pyx_v_pp, __pyx_v_dmin, __pyx_v_dmin1, __pyx_v_dmin2, __pyx_v_dn, __pyx_v_dnm1, __pyx_v_dnm2);

  /* "scipy/linalg/cython_lapack.pyx":4161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasq6 "F_FUNC(dlasq6,DLASQ6)"(int *i0, int *n0, d *z, int *pp, d *dmin, d *dmin1, d *dmin2, d *dn, d *dnm1, d *dnm2) nogil
 * cdef void dlasq6(int *i0, int *n0, d *z, int *pp, d *dmin, d *dmin1, d *dmin2, d *dn, d *dnm1, d *dnm2) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasq6(i0, n0, z, pp, dmin, dmin1, dmin2, dn, dnm1, dnm2)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasr "F_FUNC(dlasr,DLASR)"(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, d *a, int *lda) nogil
 * cdef void dlasr(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, d *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasr(side, pivot, direct, m, n, c, s, a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasr(char *__pyx_v_side, char *__pyx_v_pivot, char *__pyx_v_direct, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":4167
 *     void _fortran_dlasr "F_FUNC(dlasr,DLASR)"(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, d *a, int *lda) nogil
 * cdef void dlasr(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, d *a, int *lda) nogil:
 *     _fortran_dlasr(side, pivot, direct, m, n, c, s, a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasr,DLASR)(__pyx_v_side, __pyx_v_pivot, __pyx_v_direct, __pyx_v_m, __pyx_v_n, __pyx_v_c, __pyx_v_s, __pyx_v_a, __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":4166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasr "F_FUNC(dlasr,DLASR)"(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, d *a, int *lda) nogil
 * cdef void dlasr(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, d *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasr(side, pivot, direct, m, n, c, s, a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasrt "F_FUNC(dlasrt,DLASRT)"(char *id, int *n, d *d, int *info) nogil
 * cdef void dlasrt(char *id, int *n, d *d, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasrt(id, n, d, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasrt(char *__pyx_v_id, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4172
 *     void _fortran_dlasrt "F_FUNC(dlasrt,DLASRT)"(char *id, int *n, d *d, int *info) nogil
 * cdef void dlasrt(char *id, int *n, d *d, int *info) nogil:
 *     _fortran_dlasrt(id, n, d, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasrt,DLASRT)(__pyx_v_id, __pyx_v_n, __pyx_v_d, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasrt "F_FUNC(dlasrt,DLASRT)"(char *id, int *n, d *d, int *info) nogil
 * cdef void dlasrt(char *id, int *n, d *d, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasrt(id, n, d, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlassq "F_FUNC(dlassq,DLASSQ)"(int *n, d *x, int *incx, d *scale, d *sumsq) nogil
 * cdef void dlassq(int *n, d *x, int *incx, d *scale, d *sumsq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlassq(n, x, incx, scale, sumsq)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlassq(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sumsq) {

  /* "scipy/linalg/cython_lapack.pyx":4177
 *     void _fortran_dlassq "F_FUNC(dlassq,DLASSQ)"(int *n, d *x, int *incx, d *scale, d *sumsq) nogil
 * cdef void dlassq(int *n, d *x, int *incx, d *scale, d *sumsq) nogil:
 *     _fortran_dlassq(n, x, incx, scale, sumsq)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlassq,DLASSQ)(__pyx_v_n, __pyx_v_x, __pyx_v_incx, __pyx_v_scale, __pyx_v_sumsq);

  /* "scipy/linalg/cython_lapack.pyx":4176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlassq "F_FUNC(dlassq,DLASSQ)"(int *n, d *x, int *incx, d *scale, d *sumsq) nogil
 * cdef void dlassq(int *n, d *x, int *incx, d *scale, d *sumsq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlassq(n, x, incx, scale, sumsq)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasv2 "F_FUNC(dlasv2,DLASV2)"(d *f, d *g, d *h, d *ssmin, d *ssmax, d *snr, d *csr, d *snl, d *csl) nogil
 * cdef void dlasv2(d *f, d *g, d *h, d *ssmin, d *ssmax, d *snr, d *csr, d *snl, d *csl) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasv2(f, g, h, ssmin, ssmax, snr, csr, snl, csl)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasv2(__pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_f, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_g, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_h, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ssmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ssmax, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_snr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_snl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csl) {

  /* "scipy/linalg/cython_lapack.pyx":4182
 *     void _fortran_dlasv2 "F_FUNC(dlasv2,DLASV2)"(d *f, d *g, d *h, d *ssmin, d *ssmax, d *snr, d *csr, d *snl, d *csl) nogil
 * cdef void dlasv2(d *f, d *g, d *h, d *ssmin, d *ssmax, d *snr, d *csr, d *snl, d *csl) nogil:
 *     _fortran_dlasv2(f, g, h, ssmin, ssmax, snr, csr, snl, csl)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasv2,DLASV2)(__pyx_v_f, __pyx_v_g, __pyx_v_h, __pyx_v_ssmin, __pyx_v_ssmax, __pyx_v_snr, __pyx_v_csr, __pyx_v_snl, __pyx_v_csl);

  /* "scipy/linalg/cython_lapack.pyx":4181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasv2 "F_FUNC(dlasv2,DLASV2)"(d *f, d *g, d *h, d *ssmin, d *ssmax, d *snr, d *csr, d *snl, d *csl) nogil
 * cdef void dlasv2(d *f, d *g, d *h, d *ssmin, d *ssmax, d *snr, d *csr, d *snl, d *csl) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasv2(f, g, h, ssmin, ssmax, snr, csr, snl, csl)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaswp "F_FUNC(dlaswp,DLASWP)"(int *n, d *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void dlaswp(int *n, d *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaswp(n, a, lda, k1, k2, ipiv, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlaswp(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_k1, int *__pyx_v_k2, int *__pyx_v_ipiv, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":4187
 *     void _fortran_dlaswp "F_FUNC(dlaswp,DLASWP)"(int *n, d *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void dlaswp(int *n, d *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:
 *     _fortran_dlaswp(n, a, lda, k1, k2, ipiv, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlaswp,DLASWP)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_k1, __pyx_v_k2, __pyx_v_ipiv, __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":4186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlaswp "F_FUNC(dlaswp,DLASWP)"(int *n, d *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void dlaswp(int *n, d *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlaswp(n, a, lda, k1, k2, ipiv, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasy2 "F_FUNC(dlasy2,DLASY2)"(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, d *tl, int *ldtl, d *tr, int *ldtr, d *b, int *ldb, d *scale, d *x, int *ldx, d *xnorm, int *info) nogil
 * cdef void dlasy2(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, d *tl, int *ldtl, d *tr, int *ldtr, d *b, int *ldb, d *scale, d *x, int *ldx, d *xnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasy2(ltranl, ltranr, isgn, n1, n2, tl, ldtl, tr, ldtr, b, ldb, scale, x, ldx, xnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasy2(int *__pyx_v_ltranl, int *__pyx_v_ltranr, int *__pyx_v_isgn, int *__pyx_v_n1, int *__pyx_v_n2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tl, int *__pyx_v_ldtl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tr, int *__pyx_v_ldtr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_xnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4192
 *     void _fortran_dlasy2 "F_FUNC(dlasy2,DLASY2)"(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, d *tl, int *ldtl, d *tr, int *ldtr, d *b, int *ldb, d *scale, d *x, int *ldx, d *xnorm, int *info) nogil
 * cdef void dlasy2(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, d *tl, int *ldtl, d *tr, int *ldtr, d *b, int *ldb, d *scale, d *x, int *ldx, d *xnorm, int *info) nogil:
 *     _fortran_dlasy2(ltranl, ltranr, isgn, n1, n2, tl, ldtl, tr, ldtr, b, ldb, scale, x, ldx, xnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasy2,DLASY2)(__pyx_v_ltranl, __pyx_v_ltranr, __pyx_v_isgn, __pyx_v_n1, __pyx_v_n2, __pyx_v_tl, __pyx_v_ldtl, __pyx_v_tr, __pyx_v_ldtr, __pyx_v_b, __pyx_v_ldb, __pyx_v_scale, __pyx_v_x, __pyx_v_ldx, __pyx_v_xnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasy2 "F_FUNC(dlasy2,DLASY2)"(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, d *tl, int *ldtl, d *tr, int *ldtr, d *b, int *ldb, d *scale, d *x, int *ldx, d *xnorm, int *info) nogil
 * cdef void dlasy2(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, d *tl, int *ldtl, d *tr, int *ldtr, d *b, int *ldb, d *scale, d *x, int *ldx, d *xnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasy2(ltranl, ltranr, isgn, n1, n2, tl, ldtl, tr, ldtr, b, ldb, scale, x, ldx, xnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasyf "F_FUNC(dlasyf,DLASYF)"(char *uplo, int *n, int *nb, int *kb, d *a, int *lda, int *ipiv, d *w, int *ldw, int *info) nogil
 * cdef void dlasyf(char *uplo, int *n, int *nb, int *kb, d *a, int *lda, int *ipiv, d *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasyf(uplo, n, nb, kb, a, lda, ipiv, w, ldw, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlasyf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, int *__pyx_v_ldw, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4197
 *     void _fortran_dlasyf "F_FUNC(dlasyf,DLASYF)"(char *uplo, int *n, int *nb, int *kb, d *a, int *lda, int *ipiv, d *w, int *ldw, int *info) nogil
 * cdef void dlasyf(char *uplo, int *n, int *nb, int *kb, d *a, int *lda, int *ipiv, d *w, int *ldw, int *info) nogil:
 *     _fortran_dlasyf(uplo, n, nb, kb, a, lda, ipiv, w, ldw, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlasyf,DLASYF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, __pyx_v_kb, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_w, __pyx_v_ldw, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlasyf "F_FUNC(dlasyf,DLASYF)"(char *uplo, int *n, int *nb, int *kb, d *a, int *lda, int *ipiv, d *w, int *ldw, int *info) nogil
 * cdef void dlasyf(char *uplo, int *n, int *nb, int *kb, d *a, int *lda, int *ipiv, d *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlasyf(uplo, n, nb, kb, a, lda, ipiv, w, ldw, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatbs "F_FUNC(dlatbs,DLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, d *ab, int *ldab, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, d *ab, int *ldab, d *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatbs(uplo, trans, diag, normin, n, kd, ab, ldab, x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlatbs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4202
 *     void _fortran_dlatbs "F_FUNC(dlatbs,DLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, d *ab, int *ldab, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, d *ab, int *ldab, d *x, d *scale, d *cnorm, int *info) nogil:
 *     _fortran_dlatbs(uplo, trans, diag, normin, n, kd, ab, ldab, x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlatbs,DLATBS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_x, __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatbs "F_FUNC(dlatbs,DLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, d *ab, int *ldab, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, d *ab, int *ldab, d *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatbs(uplo, trans, diag, normin, n, kd, ab, ldab, x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatdf "F_FUNC(dlatdf,DLATDF)"(int *ijob, int *n, d *z, int *ldz, d *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void dlatdf(int *ijob, int *n, d *z, int *ldz, d *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatdf(ijob, n, z, ldz, rhs, rdsum, rdscal, ipiv, jpiv)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlatdf(int *__pyx_v_ijob, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rdsum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rdscal, int *__pyx_v_ipiv, int *__pyx_v_jpiv) {

  /* "scipy/linalg/cython_lapack.pyx":4207
 *     void _fortran_dlatdf "F_FUNC(dlatdf,DLATDF)"(int *ijob, int *n, d *z, int *ldz, d *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void dlatdf(int *ijob, int *n, d *z, int *ldz, d *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil:
 *     _fortran_dlatdf(ijob, n, z, ldz, rhs, rdsum, rdscal, ipiv, jpiv)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlatdf,DLATDF)(__pyx_v_ijob, __pyx_v_n, __pyx_v_z, __pyx_v_ldz, __pyx_v_rhs, __pyx_v_rdsum, __pyx_v_rdscal, __pyx_v_ipiv, __pyx_v_jpiv);

  /* "scipy/linalg/cython_lapack.pyx":4206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatdf "F_FUNC(dlatdf,DLATDF)"(int *ijob, int *n, d *z, int *ldz, d *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void dlatdf(int *ijob, int *n, d *z, int *ldz, d *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatdf(ijob, n, z, ldz, rhs, rdsum, rdscal, ipiv, jpiv)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatps "F_FUNC(dlatps,DLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, d *ap, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatps(char *uplo, char *trans, char *diag, char *normin, int *n, d *ap, d *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatps(uplo, trans, diag, normin, n, ap, x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlatps(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4212
 *     void _fortran_dlatps "F_FUNC(dlatps,DLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, d *ap, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatps(char *uplo, char *trans, char *diag, char *normin, int *n, d *ap, d *x, d *scale, d *cnorm, int *info) nogil:
 *     _fortran_dlatps(uplo, trans, diag, normin, n, ap, x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlatps,DLATPS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, __pyx_v_ap, __pyx_v_x, __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatps "F_FUNC(dlatps,DLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, d *ap, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatps(char *uplo, char *trans, char *diag, char *normin, int *n, d *ap, d *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatps(uplo, trans, diag, normin, n, ap, x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatrd "F_FUNC(dlatrd,DLATRD)"(char *uplo, int *n, int *nb, d *a, int *lda, d *e, d *tau, d *w, int *ldw) nogil
 * cdef void dlatrd(char *uplo, int *n, int *nb, d *a, int *lda, d *e, d *tau, d *w, int *ldw) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatrd(uplo, n, nb, a, lda, e, tau, w, ldw)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlatrd(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, int *__pyx_v_ldw) {

  /* "scipy/linalg/cython_lapack.pyx":4217
 *     void _fortran_dlatrd "F_FUNC(dlatrd,DLATRD)"(char *uplo, int *n, int *nb, d *a, int *lda, d *e, d *tau, d *w, int *ldw) nogil
 * cdef void dlatrd(char *uplo, int *n, int *nb, d *a, int *lda, d *e, d *tau, d *w, int *ldw) nogil:
 *     _fortran_dlatrd(uplo, n, nb, a, lda, e, tau, w, ldw)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlatrd,DLATRD)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, __pyx_v_a, __pyx_v_lda, __pyx_v_e, __pyx_v_tau, __pyx_v_w, __pyx_v_ldw);

  /* "scipy/linalg/cython_lapack.pyx":4216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatrd "F_FUNC(dlatrd,DLATRD)"(char *uplo, int *n, int *nb, d *a, int *lda, d *e, d *tau, d *w, int *ldw) nogil
 * cdef void dlatrd(char *uplo, int *n, int *nb, d *a, int *lda, d *e, d *tau, d *w, int *ldw) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatrd(uplo, n, nb, a, lda, e, tau, w, ldw)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatrs "F_FUNC(dlatrs,DLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, d *a, int *lda, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatrs(char *uplo, char *trans, char *diag, char *normin, int *n, d *a, int *lda, d *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatrs(uplo, trans, diag, normin, n, a, lda, x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlatrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4222
 *     void _fortran_dlatrs "F_FUNC(dlatrs,DLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, d *a, int *lda, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatrs(char *uplo, char *trans, char *diag, char *normin, int *n, d *a, int *lda, d *x, d *scale, d *cnorm, int *info) nogil:
 *     _fortran_dlatrs(uplo, trans, diag, normin, n, a, lda, x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlatrs,DLATRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_x, __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatrs "F_FUNC(dlatrs,DLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, d *a, int *lda, d *x, d *scale, d *cnorm, int *info) nogil
 * cdef void dlatrs(char *uplo, char *trans, char *diag, char *normin, int *n, d *a, int *lda, d *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatrs(uplo, trans, diag, normin, n, a, lda, x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatrz "F_FUNC(dlatrz,DLATRZ)"(int *m, int *n, int *l, d *a, int *lda, d *tau, d *work) nogil
 * cdef void dlatrz(int *m, int *n, int *l, d *a, int *lda, d *tau, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatrz(m, n, l, a, lda, tau, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlatrz(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":4227
 *     void _fortran_dlatrz "F_FUNC(dlatrz,DLATRZ)"(int *m, int *n, int *l, d *a, int *lda, d *tau, d *work) nogil
 * cdef void dlatrz(int *m, int *n, int *l, d *a, int *lda, d *tau, d *work) nogil:
 *     _fortran_dlatrz(m, n, l, a, lda, tau, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlatrz,DLATRZ)(__pyx_v_m, __pyx_v_n, __pyx_v_l, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":4226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlatrz "F_FUNC(dlatrz,DLATRZ)"(int *m, int *n, int *l, d *a, int *lda, d *tau, d *work) nogil
 * cdef void dlatrz(int *m, int *n, int *l, d *a, int *lda, d *tau, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlatrz(m, n, l, a, lda, tau, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlauu2 "F_FUNC(dlauu2,DLAUU2)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlauu2(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlauu2(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlauu2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4232
 *     void _fortran_dlauu2 "F_FUNC(dlauu2,DLAUU2)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlauu2(char *uplo, int *n, d *a, int *lda, int *info) nogil:
 *     _fortran_dlauu2(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlauu2,DLAUU2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlauu2 "F_FUNC(dlauu2,DLAUU2)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlauu2(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlauu2(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlauum "F_FUNC(dlauum,DLAUUM)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlauum(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlauum(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dlauum(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4237
 *     void _fortran_dlauum "F_FUNC(dlauum,DLAUUM)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlauum(char *uplo, int *n, d *a, int *lda, int *info) nogil:
 *     _fortran_dlauum(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dlauum,DLAUUM)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dlauum "F_FUNC(dlauum,DLAUUM)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dlauum(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dlauum(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dopgtr "F_FUNC(dopgtr,DOPGTR)"(char *uplo, int *n, d *ap, d *tau, d *q, int *ldq, d *work, int *info) nogil
 * cdef void dopgtr(char *uplo, int *n, d *ap, d *tau, d *q, int *ldq, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dopgtr(uplo, n, ap, tau, q, ldq, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dopgtr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4242
 *     void _fortran_dopgtr "F_FUNC(dopgtr,DOPGTR)"(char *uplo, int *n, d *ap, d *tau, d *q, int *ldq, d *work, int *info) nogil
 * cdef void dopgtr(char *uplo, int *n, d *ap, d *tau, d *q, int *ldq, d *work, int *info) nogil:
 *     _fortran_dopgtr(uplo, n, ap, tau, q, ldq, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dopgtr,DOPGTR)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_tau, __pyx_v_q, __pyx_v_ldq, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dopgtr "F_FUNC(dopgtr,DOPGTR)"(char *uplo, int *n, d *ap, d *tau, d *q, int *ldq, d *work, int *info) nogil
 * cdef void dopgtr(char *uplo, int *n, d *ap, d *tau, d *q, int *ldq, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dopgtr(uplo, n, ap, tau, q, ldq, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dopmtr "F_FUNC(dopmtr,DOPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, d *ap, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dopmtr(char *side, char *uplo, char *trans, int *m, int *n, d *ap, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dopmtr(side, uplo, trans, m, n, ap, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dopmtr(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4247
 *     void _fortran_dopmtr "F_FUNC(dopmtr,DOPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, d *ap, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dopmtr(char *side, char *uplo, char *trans, int *m, int *n, d *ap, d *tau, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dopmtr(side, uplo, trans, m, n, ap, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dopmtr,DOPMTR)(__pyx_v_side, __pyx_v_uplo, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_ap, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dopmtr "F_FUNC(dopmtr,DOPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, d *ap, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dopmtr(char *side, char *uplo, char *trans, int *m, int *n, d *ap, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dopmtr(side, uplo, trans, m, n, ap, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorg2l "F_FUNC(dorg2l,DORG2L)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorg2l(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorg2l(m, n, k, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorg2l(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4252
 *     void _fortran_dorg2l "F_FUNC(dorg2l,DORG2L)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorg2l(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dorg2l(m, n, k, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorg2l,DORG2L)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorg2l "F_FUNC(dorg2l,DORG2L)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorg2l(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorg2l(m, n, k, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorg2r "F_FUNC(dorg2r,DORG2R)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorg2r(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorg2r(m, n, k, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorg2r(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4257
 *     void _fortran_dorg2r "F_FUNC(dorg2r,DORG2R)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorg2r(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dorg2r(m, n, k, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorg2r,DORG2R)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorg2r "F_FUNC(dorg2r,DORG2R)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorg2r(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorg2r(m, n, k, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgbr "F_FUNC(dorgbr,DORGBR)"(char *vect, int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgbr(char *vect, int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgbr(vect, m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorgbr(char *__pyx_v_vect, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4262
 *     void _fortran_dorgbr "F_FUNC(dorgbr,DORGBR)"(char *vect, int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgbr(char *vect, int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dorgbr(vect, m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorgbr,DORGBR)(__pyx_v_vect, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgbr "F_FUNC(dorgbr,DORGBR)"(char *vect, int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgbr(char *vect, int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgbr(vect, m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorghr "F_FUNC(dorghr,DORGHR)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorghr(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorghr(n, ilo, ihi, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorghr(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4267
 *     void _fortran_dorghr "F_FUNC(dorghr,DORGHR)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorghr(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dorghr(n, ilo, ihi, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorghr,DORGHR)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorghr "F_FUNC(dorghr,DORGHR)"(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorghr(int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorghr(n, ilo, ihi, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgl2 "F_FUNC(dorgl2,DORGL2)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorgl2(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgl2(m, n, k, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorgl2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4272
 *     void _fortran_dorgl2 "F_FUNC(dorgl2,DORGL2)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorgl2(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dorgl2(m, n, k, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorgl2,DORGL2)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgl2 "F_FUNC(dorgl2,DORGL2)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorgl2(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgl2(m, n, k, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorglq "F_FUNC(dorglq,DORGLQ)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorglq(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorglq(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorglq(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4277
 *     void _fortran_dorglq "F_FUNC(dorglq,DORGLQ)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorglq(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dorglq(m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorglq,DORGLQ)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorglq "F_FUNC(dorglq,DORGLQ)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorglq(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorglq(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgql "F_FUNC(dorgql,DORGQL)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgql(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgql(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorgql(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4282
 *     void _fortran_dorgql "F_FUNC(dorgql,DORGQL)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgql(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dorgql(m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorgql,DORGQL)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgql "F_FUNC(dorgql,DORGQL)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgql(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgql(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgqr "F_FUNC(dorgqr,DORGQR)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgqr(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgqr(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorgqr(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4287
 *     void _fortran_dorgqr "F_FUNC(dorgqr,DORGQR)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgqr(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dorgqr(m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorgqr,DORGQR)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgqr "F_FUNC(dorgqr,DORGQR)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgqr(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgqr(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgr2 "F_FUNC(dorgr2,DORGR2)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorgr2(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgr2(m, n, k, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorgr2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4292
 *     void _fortran_dorgr2 "F_FUNC(dorgr2,DORGR2)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorgr2(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:
 *     _fortran_dorgr2(m, n, k, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorgr2,DORGR2)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgr2 "F_FUNC(dorgr2,DORGR2)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil
 * cdef void dorgr2(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgr2(m, n, k, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgrq "F_FUNC(dorgrq,DORGRQ)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgrq(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgrq(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorgrq(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4297
 *     void _fortran_dorgrq "F_FUNC(dorgrq,DORGRQ)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgrq(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dorgrq(m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorgrq,DORGRQ)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgrq "F_FUNC(dorgrq,DORGRQ)"(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgrq(int *m, int *n, int *k, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgrq(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgtr "F_FUNC(dorgtr,DORGTR)"(char *uplo, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgtr(char *uplo, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgtr(uplo, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorgtr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4302
 *     void _fortran_dorgtr "F_FUNC(dorgtr,DORGTR)"(char *uplo, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgtr(char *uplo, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dorgtr(uplo, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorgtr,DORGTR)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorgtr "F_FUNC(dorgtr,DORGTR)"(char *uplo, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dorgtr(char *uplo, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorgtr(uplo, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorm2l "F_FUNC(dorm2l,DORM2L)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorm2l(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorm2l(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorm2l(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4307
 *     void _fortran_dorm2l "F_FUNC(dorm2l,DORM2L)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorm2l(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dorm2l(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorm2l,DORM2L)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorm2l "F_FUNC(dorm2l,DORM2L)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorm2l(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorm2l(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorm2r "F_FUNC(dorm2r,DORM2R)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorm2r(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorm2r(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorm2r(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4312
 *     void _fortran_dorm2r "F_FUNC(dorm2r,DORM2R)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorm2r(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dorm2r(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorm2r,DORM2R)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorm2r "F_FUNC(dorm2r,DORM2R)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorm2r(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorm2r(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormbr "F_FUNC(dormbr,DORMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormbr(char *vect, char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormbr(vect, side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormbr(char *__pyx_v_vect, char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4317
 *     void _fortran_dormbr "F_FUNC(dormbr,DORMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormbr(char *vect, char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:
 *     _fortran_dormbr(vect, side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormbr,DORMBR)(__pyx_v_vect, __pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormbr "F_FUNC(dormbr,DORMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormbr(char *vect, char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormbr(vect, side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormhr "F_FUNC(dormhr,DORMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormhr(side, trans, m, n, ilo, ihi, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormhr(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4322
 *     void _fortran_dormhr "F_FUNC(dormhr,DORMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:
 *     _fortran_dormhr(side, trans, m, n, ilo, ihi, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormhr,DORMHR)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormhr "F_FUNC(dormhr,DORMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormhr(side, trans, m, n, ilo, ihi, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorml2 "F_FUNC(dorml2,DORML2)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorml2(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorml2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dorml2(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4327
 *     void _fortran_dorml2 "F_FUNC(dorml2,DORML2)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorml2(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dorml2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dorml2,DORML2)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dorml2 "F_FUNC(dorml2,DORML2)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dorml2(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dorml2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormlq "F_FUNC(dormlq,DORMLQ)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormlq(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormlq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormlq(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4332
 *     void _fortran_dormlq "F_FUNC(dormlq,DORMLQ)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormlq(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:
 *     _fortran_dormlq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormlq,DORMLQ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormlq "F_FUNC(dormlq,DORMLQ)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormlq(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormlq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormql "F_FUNC(dormql,DORMQL)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormql(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormql(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormql(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4337
 *     void _fortran_dormql "F_FUNC(dormql,DORMQL)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormql(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:
 *     _fortran_dormql(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormql,DORMQL)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormql "F_FUNC(dormql,DORMQL)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormql(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormql(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormqr "F_FUNC(dormqr,DORMQR)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormqr(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormqr(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormqr(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4342
 *     void _fortran_dormqr "F_FUNC(dormqr,DORMQR)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormqr(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:
 *     _fortran_dormqr(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormqr,DORMQR)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormqr "F_FUNC(dormqr,DORMQR)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormqr(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormqr(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormr2 "F_FUNC(dormr2,DORMR2)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dormr2(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormr2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormr2(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4347
 *     void _fortran_dormr2 "F_FUNC(dormr2,DORMR2)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dormr2(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dormr2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormr2,DORMR2)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormr2 "F_FUNC(dormr2,DORMR2)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dormr2(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormr2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormr3 "F_FUNC(dormr3,DORMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dormr3(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormr3(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormr3(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4352
 *     void _fortran_dormr3 "F_FUNC(dormr3,DORMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dormr3(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:
 *     _fortran_dormr3(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormr3,DORMR3)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormr3 "F_FUNC(dormr3,DORMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil
 * cdef void dormr3(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormr3(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormrq "F_FUNC(dormrq,DORMRQ)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormrq(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormrq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormrq(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4357
 *     void _fortran_dormrq "F_FUNC(dormrq,DORMRQ)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormrq(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:
 *     _fortran_dormrq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormrq,DORMRQ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormrq "F_FUNC(dormrq,DORMRQ)"(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormrq(char *side, char *trans, int *m, int *n, int *k, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormrq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormrz "F_FUNC(dormrz,DORMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormrz(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormrz(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormrz(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4362
 *     void _fortran_dormrz "F_FUNC(dormrz,DORMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormrz(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:
 *     _fortran_dormrz(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormrz,DORMRZ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormrz "F_FUNC(dormrz,DORMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormrz(char *side, char *trans, int *m, int *n, int *k, int *l, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormrz(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormtr "F_FUNC(dormtr,DORMTR)"(char *side, char *uplo, char *trans, int *m, int *n, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormtr(char *side, char *uplo, char *trans, int *m, int *n, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormtr(side, uplo, trans, m, n, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dormtr(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4367
 *     void _fortran_dormtr "F_FUNC(dormtr,DORMTR)"(char *side, char *uplo, char *trans, int *m, int *n, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormtr(char *side, char *uplo, char *trans, int *m, int *n, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:
 *     _fortran_dormtr(side, uplo, trans, m, n, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dormtr,DORMTR)(__pyx_v_side, __pyx_v_uplo, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dormtr "F_FUNC(dormtr,DORMTR)"(char *side, char *uplo, char *trans, int *m, int *n, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil
 * cdef void dormtr(char *side, char *uplo, char *trans, int *m, int *n, d *a, int *lda, d *tau, d *c, int *ldc, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dormtr(side, uplo, trans, m, n, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbcon "F_FUNC(dpbcon,DPBCON)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dpbcon(char *uplo, int *n, int *kd, d *ab, int *ldab, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbcon(uplo, n, kd, ab, ldab, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbcon(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4372
 *     void _fortran_dpbcon "F_FUNC(dpbcon,DPBCON)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dpbcon(char *uplo, int *n, int *kd, d *ab, int *ldab, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dpbcon(uplo, n, kd, ab, ldab, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbcon,DPBCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbcon "F_FUNC(dpbcon,DPBCON)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dpbcon(char *uplo, int *n, int *kd, d *ab, int *ldab, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbcon(uplo, n, kd, ab, ldab, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbequ "F_FUNC(dpbequ,DPBEQU)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dpbequ(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbequ(uplo, n, kd, ab, ldab, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbequ(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4377
 *     void _fortran_dpbequ "F_FUNC(dpbequ,DPBEQU)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dpbequ(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil:
 *     _fortran_dpbequ(uplo, n, kd, ab, ldab, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbequ,DPBEQU)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbequ "F_FUNC(dpbequ,DPBEQU)"(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dpbequ(char *uplo, int *n, int *kd, d *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbequ(uplo, n, kd, ab, ldab, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbrfs "F_FUNC(dpbrfs,DPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpbrfs(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbrfs(uplo, n, kd, nrhs, ab, ldab, afb, ldafb, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_afb, int *__pyx_v_ldafb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4382
 *     void _fortran_dpbrfs "F_FUNC(dpbrfs,DPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpbrfs(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dpbrfs(uplo, n, kd, nrhs, ab, ldab, afb, ldafb, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbrfs,DPBRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_afb, __pyx_v_ldafb, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbrfs "F_FUNC(dpbrfs,DPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpbrfs(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbrfs(uplo, n, kd, nrhs, ab, ldab, afb, ldafb, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbstf "F_FUNC(dpbstf,DPBSTF)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbstf(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbstf(uplo, n, kd, ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbstf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4387
 *     void _fortran_dpbstf "F_FUNC(dpbstf,DPBSTF)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbstf(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:
 *     _fortran_dpbstf(uplo, n, kd, ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbstf,DPBSTF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbstf "F_FUNC(dpbstf,DPBSTF)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbstf(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbstf(uplo, n, kd, ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbsv "F_FUNC(dpbsv,DPBSV)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dpbsv(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbsv(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4392
 *     void _fortran_dpbsv "F_FUNC(dpbsv,DPBSV)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dpbsv(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:
 *     _fortran_dpbsv(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbsv,DPBSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbsv "F_FUNC(dpbsv,DPBSV)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dpbsv(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbsv(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbsvx "F_FUNC(dpbsvx,DPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbsvx(fact, uplo, n, kd, nrhs, ab, ldab, afb, ldafb, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_afb, int *__pyx_v_ldafb, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4397
 *     void _fortran_dpbsvx "F_FUNC(dpbsvx,DPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dpbsvx(fact, uplo, n, kd, nrhs, ab, ldab, afb, ldafb, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbsvx,DPBSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_afb, __pyx_v_ldafb, __pyx_v_equed, __pyx_v_s, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbsvx "F_FUNC(dpbsvx,DPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *afb, int *ldafb, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbsvx(fact, uplo, n, kd, nrhs, ab, ldab, afb, ldafb, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbtf2 "F_FUNC(dpbtf2,DPBTF2)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbtf2(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbtf2(uplo, n, kd, ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbtf2(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4402
 *     void _fortran_dpbtf2 "F_FUNC(dpbtf2,DPBTF2)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbtf2(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:
 *     _fortran_dpbtf2(uplo, n, kd, ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbtf2,DPBTF2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbtf2 "F_FUNC(dpbtf2,DPBTF2)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbtf2(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbtf2(uplo, n, kd, ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbtrf "F_FUNC(dpbtrf,DPBTRF)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbtrf(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbtrf(uplo, n, kd, ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbtrf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4407
 *     void _fortran_dpbtrf "F_FUNC(dpbtrf,DPBTRF)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbtrf(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:
 *     _fortran_dpbtrf(uplo, n, kd, ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbtrf,DPBTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbtrf "F_FUNC(dpbtrf,DPBTRF)"(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil
 * cdef void dpbtrf(char *uplo, int *n, int *kd, d *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbtrf(uplo, n, kd, ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbtrs "F_FUNC(dpbtrs,DPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dpbtrs(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbtrs(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpbtrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4412
 *     void _fortran_dpbtrs "F_FUNC(dpbtrs,DPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dpbtrs(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:
 *     _fortran_dpbtrs(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpbtrs,DPBTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpbtrs "F_FUNC(dpbtrs,DPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dpbtrs(char *uplo, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpbtrs(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpocon "F_FUNC(dpocon,DPOCON)"(char *uplo, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dpocon(char *uplo, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpocon(uplo, n, a, lda, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpocon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4417
 *     void _fortran_dpocon "F_FUNC(dpocon,DPOCON)"(char *uplo, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dpocon(char *uplo, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dpocon(uplo, n, a, lda, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpocon,DPOCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpocon "F_FUNC(dpocon,DPOCON)"(char *uplo, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dpocon(char *uplo, int *n, d *a, int *lda, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpocon(uplo, n, a, lda, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpoequ "F_FUNC(dpoequ,DPOEQU)"(int *n, d *a, int *lda, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dpoequ(int *n, d *a, int *lda, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpoequ(n, a, lda, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpoequ(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4422
 *     void _fortran_dpoequ "F_FUNC(dpoequ,DPOEQU)"(int *n, d *a, int *lda, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dpoequ(int *n, d *a, int *lda, d *s, d *scond, d *amax, int *info) nogil:
 *     _fortran_dpoequ(n, a, lda, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpoequ,DPOEQU)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpoequ "F_FUNC(dpoequ,DPOEQU)"(int *n, d *a, int *lda, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dpoequ(int *n, d *a, int *lda, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpoequ(n, a, lda, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dporfs "F_FUNC(dporfs,DPORFS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dporfs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dporfs(uplo, n, nrhs, a, lda, af, ldaf, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dporfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_af, int *__pyx_v_ldaf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4427
 *     void _fortran_dporfs "F_FUNC(dporfs,DPORFS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dporfs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dporfs(uplo, n, nrhs, a, lda, af, ldaf, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dporfs,DPORFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dporfs "F_FUNC(dporfs,DPORFS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dporfs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dporfs(uplo, n, nrhs, a, lda, af, ldaf, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dposv "F_FUNC(dposv,DPOSV)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dposv(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dposv(uplo, n, nrhs, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dposv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4432
 *     void _fortran_dposv "F_FUNC(dposv,DPOSV)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dposv(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:
 *     _fortran_dposv(uplo, n, nrhs, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dposv,DPOSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dposv "F_FUNC(dposv,DPOSV)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dposv(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dposv(uplo, n, nrhs, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dposvx "F_FUNC(dposvx,DPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dposvx(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dposvx(fact, uplo, n, nrhs, a, lda, af, ldaf, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dposvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_af, int *__pyx_v_ldaf, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4437
 *     void _fortran_dposvx "F_FUNC(dposvx,DPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dposvx(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dposvx(fact, uplo, n, nrhs, a, lda, af, ldaf, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dposvx,DPOSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_equed, __pyx_v_s, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dposvx "F_FUNC(dposvx,DPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dposvx(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dposvx(fact, uplo, n, nrhs, a, lda, af, ldaf, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpotf2 "F_FUNC(dpotf2,DPOTF2)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotf2(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpotf2(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpotf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4442
 *     void _fortran_dpotf2 "F_FUNC(dpotf2,DPOTF2)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotf2(char *uplo, int *n, d *a, int *lda, int *info) nogil:
 *     _fortran_dpotf2(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpotf2,DPOTF2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpotf2 "F_FUNC(dpotf2,DPOTF2)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotf2(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpotf2(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpotrf "F_FUNC(dpotrf,DPOTRF)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotrf(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpotrf(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpotrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4447
 *     void _fortran_dpotrf "F_FUNC(dpotrf,DPOTRF)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotrf(char *uplo, int *n, d *a, int *lda, int *info) nogil:
 *     _fortran_dpotrf(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpotrf,DPOTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpotrf "F_FUNC(dpotrf,DPOTRF)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotrf(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpotrf(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpotri "F_FUNC(dpotri,DPOTRI)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotri(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpotri(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpotri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4452
 *     void _fortran_dpotri "F_FUNC(dpotri,DPOTRI)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotri(char *uplo, int *n, d *a, int *lda, int *info) nogil:
 *     _fortran_dpotri(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpotri,DPOTRI)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpotri "F_FUNC(dpotri,DPOTRI)"(char *uplo, int *n, d *a, int *lda, int *info) nogil
 * cdef void dpotri(char *uplo, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpotri(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpotrs "F_FUNC(dpotrs,DPOTRS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dpotrs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpotrs(uplo, n, nrhs, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpotrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4457
 *     void _fortran_dpotrs "F_FUNC(dpotrs,DPOTRS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dpotrs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:
 *     _fortran_dpotrs(uplo, n, nrhs, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpotrs,DPOTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpotrs "F_FUNC(dpotrs,DPOTRS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dpotrs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpotrs(uplo, n, nrhs, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dppcon "F_FUNC(dppcon,DPPCON)"(char *uplo, int *n, d *ap, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dppcon(char *uplo, int *n, d *ap, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dppcon(uplo, n, ap, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dppcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4462
 *     void _fortran_dppcon "F_FUNC(dppcon,DPPCON)"(char *uplo, int *n, d *ap, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dppcon(char *uplo, int *n, d *ap, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dppcon(uplo, n, ap, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dppcon,DPPCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dppcon "F_FUNC(dppcon,DPPCON)"(char *uplo, int *n, d *ap, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dppcon(char *uplo, int *n, d *ap, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dppcon(uplo, n, ap, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dppequ "F_FUNC(dppequ,DPPEQU)"(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dppequ(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dppequ(uplo, n, ap, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dppequ(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4467
 *     void _fortran_dppequ "F_FUNC(dppequ,DPPEQU)"(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dppequ(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, int *info) nogil:
 *     _fortran_dppequ(uplo, n, ap, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dppequ,DPPEQU)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dppequ "F_FUNC(dppequ,DPPEQU)"(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, int *info) nogil
 * cdef void dppequ(char *uplo, int *n, d *ap, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dppequ(uplo, n, ap, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpprfs "F_FUNC(dpprfs,DPPRFS)"(char *uplo, int *n, int *nrhs, d *ap, d *afp, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpprfs(char *uplo, int *n, int *nrhs, d *ap, d *afp, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpprfs(uplo, n, nrhs, ap, afp, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_afp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4472
 *     void _fortran_dpprfs "F_FUNC(dpprfs,DPPRFS)"(char *uplo, int *n, int *nrhs, d *ap, d *afp, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpprfs(char *uplo, int *n, int *nrhs, d *ap, d *afp, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dpprfs(uplo, n, nrhs, ap, afp, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpprfs,DPPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_afp, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpprfs "F_FUNC(dpprfs,DPPRFS)"(char *uplo, int *n, int *nrhs, d *ap, d *afp, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dpprfs(char *uplo, int *n, int *nrhs, d *ap, d *afp, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpprfs(uplo, n, nrhs, ap, afp, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dppsv "F_FUNC(dppsv,DPPSV)"(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dppsv(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dppsv(uplo, n, nrhs, ap, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dppsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4477
 *     void _fortran_dppsv "F_FUNC(dppsv,DPPSV)"(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dppsv(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:
 *     _fortran_dppsv(uplo, n, nrhs, ap, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dppsv,DPPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dppsv "F_FUNC(dppsv,DPPSV)"(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dppsv(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dppsv(uplo, n, nrhs, ap, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dppsvx "F_FUNC(dppsvx,DPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dppsvx(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dppsvx(fact, uplo, n, nrhs, ap, afp, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dppsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_afp, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4482
 *     void _fortran_dppsvx "F_FUNC(dppsvx,DPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dppsvx(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dppsvx(fact, uplo, n, nrhs, ap, afp, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dppsvx,DPPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_afp, __pyx_v_equed, __pyx_v_s, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dppsvx "F_FUNC(dppsvx,DPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dppsvx(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, char *equed, d *s, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dppsvx(fact, uplo, n, nrhs, ap, afp, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpptrf "F_FUNC(dpptrf,DPPTRF)"(char *uplo, int *n, d *ap, int *info) nogil
 * cdef void dpptrf(char *uplo, int *n, d *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpptrf(uplo, n, ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4487
 *     void _fortran_dpptrf "F_FUNC(dpptrf,DPPTRF)"(char *uplo, int *n, d *ap, int *info) nogil
 * cdef void dpptrf(char *uplo, int *n, d *ap, int *info) nogil:
 *     _fortran_dpptrf(uplo, n, ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpptrf,DPPTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpptrf "F_FUNC(dpptrf,DPPTRF)"(char *uplo, int *n, d *ap, int *info) nogil
 * cdef void dpptrf(char *uplo, int *n, d *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpptrf(uplo, n, ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpptri "F_FUNC(dpptri,DPPTRI)"(char *uplo, int *n, d *ap, int *info) nogil
 * cdef void dpptri(char *uplo, int *n, d *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpptri(uplo, n, ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4492
 *     void _fortran_dpptri "F_FUNC(dpptri,DPPTRI)"(char *uplo, int *n, d *ap, int *info) nogil
 * cdef void dpptri(char *uplo, int *n, d *ap, int *info) nogil:
 *     _fortran_dpptri(uplo, n, ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpptri,DPPTRI)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpptri "F_FUNC(dpptri,DPPTRI)"(char *uplo, int *n, d *ap, int *info) nogil
 * cdef void dpptri(char *uplo, int *n, d *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpptri(uplo, n, ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpptrs "F_FUNC(dpptrs,DPPTRS)"(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dpptrs(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpptrs(uplo, n, nrhs, ap, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4497
 *     void _fortran_dpptrs "F_FUNC(dpptrs,DPPTRS)"(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dpptrs(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:
 *     _fortran_dpptrs(uplo, n, nrhs, ap, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpptrs,DPPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpptrs "F_FUNC(dpptrs,DPPTRS)"(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dpptrs(char *uplo, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpptrs(uplo, n, nrhs, ap, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptcon "F_FUNC(dptcon,DPTCON)"(int *n, d *d, d *e, d *anorm, d *rcond, d *work, int *info) nogil
 * cdef void dptcon(int *n, d *d, d *e, d *anorm, d *rcond, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptcon(n, d, e, anorm, rcond, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dptcon(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4502
 *     void _fortran_dptcon "F_FUNC(dptcon,DPTCON)"(int *n, d *d, d *e, d *anorm, d *rcond, d *work, int *info) nogil
 * cdef void dptcon(int *n, d *d, d *e, d *anorm, d *rcond, d *work, int *info) nogil:
 *     _fortran_dptcon(n, d, e, anorm, rcond, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dptcon,DPTCON)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptcon "F_FUNC(dptcon,DPTCON)"(int *n, d *d, d *e, d *anorm, d *rcond, d *work, int *info) nogil
 * cdef void dptcon(int *n, d *d, d *e, d *anorm, d *rcond, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptcon(n, d, e, anorm, rcond, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpteqr "F_FUNC(dpteqr,DPTEQR)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dpteqr(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpteqr(compz, n, d, e, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpteqr(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4507
 *     void _fortran_dpteqr "F_FUNC(dpteqr,DPTEQR)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dpteqr(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_dpteqr(compz, n, d, e, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpteqr,DPTEQR)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpteqr "F_FUNC(dpteqr,DPTEQR)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dpteqr(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpteqr(compz, n, d, e, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptrfs "F_FUNC(dptrfs,DPTRFS)"(int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *info) nogil
 * cdef void dptrfs(int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptrfs(n, nrhs, d, e, df, ef, b, ldb, x, ldx, ferr, berr, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dptrfs(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_df, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ef, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4512
 *     void _fortran_dptrfs "F_FUNC(dptrfs,DPTRFS)"(int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *info) nogil
 * cdef void dptrfs(int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *info) nogil:
 *     _fortran_dptrfs(n, nrhs, d, e, df, ef, b, ldb, x, ldx, ferr, berr, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dptrfs,DPTRFS)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_df, __pyx_v_ef, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptrfs "F_FUNC(dptrfs,DPTRFS)"(int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *info) nogil
 * cdef void dptrfs(int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptrfs(n, nrhs, d, e, df, ef, b, ldb, x, ldx, ferr, berr, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptsv "F_FUNC(dptsv,DPTSV)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil
 * cdef void dptsv(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptsv(n, nrhs, d, e, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dptsv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4517
 *     void _fortran_dptsv "F_FUNC(dptsv,DPTSV)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil
 * cdef void dptsv(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil:
 *     _fortran_dptsv(n, nrhs, d, e, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dptsv,DPTSV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptsv "F_FUNC(dptsv,DPTSV)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil
 * cdef void dptsv(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptsv(n, nrhs, d, e, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptsvx "F_FUNC(dptsvx,DPTSVX)"(char *fact, int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *info) nogil
 * cdef void dptsvx(char *fact, int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptsvx(fact, n, nrhs, d, e, df, ef, b, ldb, x, ldx, rcond, ferr, berr, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dptsvx(char *__pyx_v_fact, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_df, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ef, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4522
 *     void _fortran_dptsvx "F_FUNC(dptsvx,DPTSVX)"(char *fact, int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *info) nogil
 * cdef void dptsvx(char *fact, int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *info) nogil:
 *     _fortran_dptsvx(fact, n, nrhs, d, e, df, ef, b, ldb, x, ldx, rcond, ferr, berr, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dptsvx,DPTSVX)(__pyx_v_fact, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_df, __pyx_v_ef, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptsvx "F_FUNC(dptsvx,DPTSVX)"(char *fact, int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *info) nogil
 * cdef void dptsvx(char *fact, int *n, int *nrhs, d *d, d *e, d *df, d *ef, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptsvx(fact, n, nrhs, d, e, df, ef, b, ldb, x, ldx, rcond, ferr, berr, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpttrf "F_FUNC(dpttrf,DPTTRF)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dpttrf(int *n, d *d, d *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpttrf(n, d, e, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpttrf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4527
 *     void _fortran_dpttrf "F_FUNC(dpttrf,DPTTRF)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dpttrf(int *n, d *d, d *e, int *info) nogil:
 *     _fortran_dpttrf(n, d, e, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpttrf,DPTTRF)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpttrf "F_FUNC(dpttrf,DPTTRF)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dpttrf(int *n, d *d, d *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpttrf(n, d, e, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpttrs "F_FUNC(dpttrs,DPTTRS)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil
 * cdef void dpttrs(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpttrs(n, nrhs, d, e, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dpttrs(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4532
 *     void _fortran_dpttrs "F_FUNC(dpttrs,DPTTRS)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil
 * cdef void dpttrs(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil:
 *     _fortran_dpttrs(n, nrhs, d, e, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dpttrs,DPTTRS)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dpttrs "F_FUNC(dpttrs,DPTTRS)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil
 * cdef void dpttrs(int *n, int *nrhs, d *d, d *e, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dpttrs(n, nrhs, d, e, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptts2 "F_FUNC(dptts2,DPTTS2)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb) nogil
 * cdef void dptts2(int *n, int *nrhs, d *d, d *e, d *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptts2(n, nrhs, d, e, b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dptts2(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":4537
 *     void _fortran_dptts2 "F_FUNC(dptts2,DPTTS2)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb) nogil
 * cdef void dptts2(int *n, int *nrhs, d *d, d *e, d *b, int *ldb) nogil:
 *     _fortran_dptts2(n, nrhs, d, e, b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dptts2,DPTTS2)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_b, __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":4536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dptts2 "F_FUNC(dptts2,DPTTS2)"(int *n, int *nrhs, d *d, d *e, d *b, int *ldb) nogil
 * cdef void dptts2(int *n, int *nrhs, d *d, d *e, d *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dptts2(n, nrhs, d, e, b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_drscl "F_FUNC(drscl,DRSCL)"(int *n, d *sa, d *sx, int *incx) nogil
 * cdef void drscl(int *n, d *sa, d *sx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_drscl(n, sa, sx, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_drscl(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sa, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sx, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":4542
 *     void _fortran_drscl "F_FUNC(drscl,DRSCL)"(int *n, d *sa, d *sx, int *incx) nogil
 * cdef void drscl(int *n, d *sa, d *sx, int *incx) nogil:
 *     _fortran_drscl(n, sa, sx, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(drscl,DRSCL)(__pyx_v_n, __pyx_v_sa, __pyx_v_sx, __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":4541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_drscl "F_FUNC(drscl,DRSCL)"(int *n, d *sa, d *sx, int *incx) nogil
 * cdef void drscl(int *n, d *sa, d *sx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_drscl(n, sa, sx, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbev "F_FUNC(dsbev,DSBEV)"(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsbev(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbev(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsbev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4547
 *     void _fortran_dsbev "F_FUNC(dsbev,DSBEV)"(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsbev(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_dsbev(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsbev,DSBEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbev "F_FUNC(dsbev,DSBEV)"(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsbev(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbev(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbevd "F_FUNC(dsbevd,DSBEVD)"(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsbevd(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbevd(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsbevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4552
 *     void _fortran_dsbevd "F_FUNC(dsbevd,DSBEVD)"(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsbevd(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dsbevd(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsbevd,DSBEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbevd "F_FUNC(dsbevd,DSBEVD)"(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsbevd(char *jobz, char *uplo, int *n, int *kd, d *ab, int *ldab, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbevd(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbevx "F_FUNC(dsbevx,DSBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, d *ab, int *ldab, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dsbevx(char *jobz, char *range, char *uplo, int *n, int *kd, d *ab, int *ldab, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbevx(jobz, range, uplo, n, kd, ab, ldab, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsbevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4557
 *     void _fortran_dsbevx "F_FUNC(dsbevx,DSBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, d *ab, int *ldab, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dsbevx(char *jobz, char *range, char *uplo, int *n, int *kd, d *ab, int *ldab, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_dsbevx(jobz, range, uplo, n, kd, ab, ldab, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsbevx,DSBEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_q, __pyx_v_ldq, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbevx "F_FUNC(dsbevx,DSBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, d *ab, int *ldab, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dsbevx(char *jobz, char *range, char *uplo, int *n, int *kd, d *ab, int *ldab, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbevx(jobz, range, uplo, n, kd, ab, ldab, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbgst "F_FUNC(dsbgst,DSBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *x, int *ldx, d *work, int *info) nogil
 * cdef void dsbgst(char *vect, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *x, int *ldx, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbgst(vect, uplo, n, ka, kb, ab, ldab, bb, ldbb, x, ldx, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsbgst(char *__pyx_v_vect, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4562
 *     void _fortran_dsbgst "F_FUNC(dsbgst,DSBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *x, int *ldx, d *work, int *info) nogil
 * cdef void dsbgst(char *vect, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *x, int *ldx, d *work, int *info) nogil:
 *     _fortran_dsbgst(vect, uplo, n, ka, kb, ab, ldab, bb, ldbb, x, ldx, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsbgst,DSBGST)(__pyx_v_vect, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, __pyx_v_ab, __pyx_v_ldab, __pyx_v_bb, __pyx_v_ldbb, __pyx_v_x, __pyx_v_ldx, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbgst "F_FUNC(dsbgst,DSBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *x, int *ldx, d *work, int *info) nogil
 * cdef void dsbgst(char *vect, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *x, int *ldx, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbgst(vect, uplo, n, ka, kb, ab, ldab, bb, ldbb, x, ldx, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbgv "F_FUNC(dsbgv,DSBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbgv(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsbgv(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4567
 *     void _fortran_dsbgv "F_FUNC(dsbgv,DSBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_dsbgv(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsbgv,DSBGV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, __pyx_v_ab, __pyx_v_ldab, __pyx_v_bb, __pyx_v_ldbb, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbgv "F_FUNC(dsbgv,DSBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbgv(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbgvd "F_FUNC(dsbgvd,DSBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbgvd(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsbgvd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4572
 *     void _fortran_dsbgvd "F_FUNC(dsbgvd,DSBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dsbgvd(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsbgvd,DSBGVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, __pyx_v_ab, __pyx_v_ldab, __pyx_v_bb, __pyx_v_ldbb, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbgvd "F_FUNC(dsbgvd,DSBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbgvd(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbgvx "F_FUNC(dsbgvx,DSBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dsbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbgvx(jobz, range, uplo, n, ka, kb, ab, ldab, bb, ldbb, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsbgvx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4577
 *     void _fortran_dsbgvx "F_FUNC(dsbgvx,DSBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dsbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_dsbgvx(jobz, range, uplo, n, ka, kb, ab, ldab, bb, ldbb, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsbgvx,DSBGVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, __pyx_v_ab, __pyx_v_ldab, __pyx_v_bb, __pyx_v_ldbb, __pyx_v_q, __pyx_v_ldq, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbgvx "F_FUNC(dsbgvx,DSBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dsbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, d *ab, int *ldab, d *bb, int *ldbb, d *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbgvx(jobz, range, uplo, n, ka, kb, ab, ldab, bb, ldbb, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbtrd "F_FUNC(dsbtrd,DSBTRD)"(char *vect, char *uplo, int *n, int *kd, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *work, int *info) nogil
 * cdef void dsbtrd(char *vect, char *uplo, int *n, int *kd, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbtrd(vect, uplo, n, kd, ab, ldab, d, e, q, ldq, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsbtrd(char *__pyx_v_vect, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4582
 *     void _fortran_dsbtrd "F_FUNC(dsbtrd,DSBTRD)"(char *vect, char *uplo, int *n, int *kd, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *work, int *info) nogil
 * cdef void dsbtrd(char *vect, char *uplo, int *n, int *kd, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *work, int *info) nogil:
 *     _fortran_dsbtrd(vect, uplo, n, kd, ab, ldab, d, e, q, ldq, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsbtrd,DSBTRD)(__pyx_v_vect, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_d, __pyx_v_e, __pyx_v_q, __pyx_v_ldq, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsbtrd "F_FUNC(dsbtrd,DSBTRD)"(char *vect, char *uplo, int *n, int *kd, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *work, int *info) nogil
 * cdef void dsbtrd(char *vect, char *uplo, int *n, int *kd, d *ab, int *ldab, d *d, d *e, d *q, int *ldq, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsbtrd(vect, uplo, n, kd, ab, ldab, d, e, q, ldq, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsgesv "F_FUNC(dsgesv,DSGESV)"(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *work, s *swork, int *iter, int *info) nogil
 * cdef void dsgesv(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *work, s *swork, int *iter, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsgesv(n, nrhs, a, lda, ipiv, b, ldb, x, ldx, work, swork, iter, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsgesv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_swork, int *__pyx_v_iter, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4587
 *     void _fortran_dsgesv "F_FUNC(dsgesv,DSGESV)"(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *work, s *swork, int *iter, int *info) nogil
 * cdef void dsgesv(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *work, s *swork, int *iter, int *info) nogil:
 *     _fortran_dsgesv(n, nrhs, a, lda, ipiv, b, ldb, x, ldx, work, swork, iter, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsgesv,DSGESV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_work, __pyx_v_swork, __pyx_v_iter, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsgesv "F_FUNC(dsgesv,DSGESV)"(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *work, s *swork, int *iter, int *info) nogil
 * cdef void dsgesv(int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *work, s *swork, int *iter, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsgesv(n, nrhs, a, lda, ipiv, b, ldb, x, ldx, work, swork, iter, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspcon "F_FUNC(dspcon,DSPCON)"(char *uplo, int *n, d *ap, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dspcon(char *uplo, int *n, d *ap, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspcon(uplo, n, ap, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4592
 *     void _fortran_dspcon "F_FUNC(dspcon,DSPCON)"(char *uplo, int *n, d *ap, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dspcon(char *uplo, int *n, d *ap, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dspcon(uplo, n, ap, ipiv, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspcon,DSPCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspcon "F_FUNC(dspcon,DSPCON)"(char *uplo, int *n, d *ap, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dspcon(char *uplo, int *n, d *ap, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspcon(uplo, n, ap, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspev "F_FUNC(dspev,DSPEV)"(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dspev(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspev(jobz, uplo, n, ap, w, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4597
 *     void _fortran_dspev "F_FUNC(dspev,DSPEV)"(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dspev(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_dspev(jobz, uplo, n, ap, w, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspev,DSPEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspev "F_FUNC(dspev,DSPEV)"(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dspev(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspev(jobz, uplo, n, ap, w, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspevd "F_FUNC(dspevd,DSPEVD)"(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dspevd(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspevd(jobz, uplo, n, ap, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4602
 *     void _fortran_dspevd "F_FUNC(dspevd,DSPEVD)"(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dspevd(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dspevd(jobz, uplo, n, ap, w, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspevd,DSPEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspevd "F_FUNC(dspevd,DSPEVD)"(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dspevd(char *jobz, char *uplo, int *n, d *ap, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspevd(jobz, uplo, n, ap, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspevx "F_FUNC(dspevx,DSPEVX)"(char *jobz, char *range, char *uplo, int *n, d *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dspevx(char *jobz, char *range, char *uplo, int *n, d *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspevx(jobz, range, uplo, n, ap, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4607
 *     void _fortran_dspevx "F_FUNC(dspevx,DSPEVX)"(char *jobz, char *range, char *uplo, int *n, d *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dspevx(char *jobz, char *range, char *uplo, int *n, d *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_dspevx(jobz, range, uplo, n, ap, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspevx,DSPEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspevx "F_FUNC(dspevx,DSPEVX)"(char *jobz, char *range, char *uplo, int *n, d *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dspevx(char *jobz, char *range, char *uplo, int *n, d *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspevx(jobz, range, uplo, n, ap, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspgst "F_FUNC(dspgst,DSPGST)"(int *itype, char *uplo, int *n, d *ap, d *bp, int *info) nogil
 * cdef void dspgst(int *itype, char *uplo, int *n, d *ap, d *bp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspgst(itype, uplo, n, ap, bp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspgst(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4612
 *     void _fortran_dspgst "F_FUNC(dspgst,DSPGST)"(int *itype, char *uplo, int *n, d *ap, d *bp, int *info) nogil
 * cdef void dspgst(int *itype, char *uplo, int *n, d *ap, d *bp, int *info) nogil:
 *     _fortran_dspgst(itype, uplo, n, ap, bp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspgst,DSPGST)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_bp, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspgst "F_FUNC(dspgst,DSPGST)"(int *itype, char *uplo, int *n, d *ap, d *bp, int *info) nogil
 * cdef void dspgst(int *itype, char *uplo, int *n, d *ap, d *bp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspgst(itype, uplo, n, ap, bp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspgv "F_FUNC(dspgv,DSPGV)"(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dspgv(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspgv(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspgv(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4617
 *     void _fortran_dspgv "F_FUNC(dspgv,DSPGV)"(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dspgv(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_dspgv(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspgv,DSPGV)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_bp, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspgv "F_FUNC(dspgv,DSPGV)"(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dspgv(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspgv(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspgvd "F_FUNC(dspgvd,DSPGVD)"(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dspgvd(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspgvd(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspgvd(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4622
 *     void _fortran_dspgvd "F_FUNC(dspgvd,DSPGVD)"(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dspgvd(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dspgvd(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspgvd,DSPGVD)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_bp, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspgvd "F_FUNC(dspgvd,DSPGVD)"(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dspgvd(int *itype, char *jobz, char *uplo, int *n, d *ap, d *bp, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspgvd(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspgvx "F_FUNC(dspgvx,DSPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, d *ap, d *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dspgvx(int *itype, char *jobz, char *range, char *uplo, int *n, d *ap, d *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspgvx(itype, jobz, range, uplo, n, ap, bp, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspgvx(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4627
 *     void _fortran_dspgvx "F_FUNC(dspgvx,DSPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, d *ap, d *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dspgvx(int *itype, char *jobz, char *range, char *uplo, int *n, d *ap, d *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_dspgvx(itype, jobz, range, uplo, n, ap, bp, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspgvx,DSPGVX)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_bp, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspgvx "F_FUNC(dspgvx,DSPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, d *ap, d *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dspgvx(int *itype, char *jobz, char *range, char *uplo, int *n, d *ap, d *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspgvx(itype, jobz, range, uplo, n, ap, bp, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsprfs "F_FUNC(dsprfs,DSPRFS)"(char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dsprfs(char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsprfs(uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4632
 *     void _fortran_dsprfs "F_FUNC(dsprfs,DSPRFS)"(char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dsprfs(char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dsprfs(uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsprfs,DSPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_afp, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsprfs "F_FUNC(dsprfs,DSPRFS)"(char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dsprfs(char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsprfs(uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspsv "F_FUNC(dspsv,DSPSV)"(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dspsv(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspsv(uplo, n, nrhs, ap, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4637
 *     void _fortran_dspsv "F_FUNC(dspsv,DSPSV)"(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dspsv(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil:
 *     _fortran_dspsv(uplo, n, nrhs, ap, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspsv,DSPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspsv "F_FUNC(dspsv,DSPSV)"(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dspsv(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspsv(uplo, n, nrhs, ap, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspsvx "F_FUNC(dspsvx,DSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dspsvx(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dspsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4642
 *     void _fortran_dspsvx "F_FUNC(dspsvx,DSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dspsvx(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dspsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dspsvx,DSPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_afp, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dspsvx "F_FUNC(dspsvx,DSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dspsvx(char *fact, char *uplo, int *n, int *nrhs, d *ap, d *afp, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dspsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsptrd "F_FUNC(dsptrd,DSPTRD)"(char *uplo, int *n, d *ap, d *d, d *e, d *tau, int *info) nogil
 * cdef void dsptrd(char *uplo, int *n, d *ap, d *d, d *e, d *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsptrd(uplo, n, ap, d, e, tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsptrd(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4647
 *     void _fortran_dsptrd "F_FUNC(dsptrd,DSPTRD)"(char *uplo, int *n, d *ap, d *d, d *e, d *tau, int *info) nogil
 * cdef void dsptrd(char *uplo, int *n, d *ap, d *d, d *e, d *tau, int *info) nogil:
 *     _fortran_dsptrd(uplo, n, ap, d, e, tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsptrd,DSPTRD)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_d, __pyx_v_e, __pyx_v_tau, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsptrd "F_FUNC(dsptrd,DSPTRD)"(char *uplo, int *n, d *ap, d *d, d *e, d *tau, int *info) nogil
 * cdef void dsptrd(char *uplo, int *n, d *ap, d *d, d *e, d *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsptrd(uplo, n, ap, d, e, tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsptrf "F_FUNC(dsptrf,DSPTRF)"(char *uplo, int *n, d *ap, int *ipiv, int *info) nogil
 * cdef void dsptrf(char *uplo, int *n, d *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsptrf(uplo, n, ap, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4652
 *     void _fortran_dsptrf "F_FUNC(dsptrf,DSPTRF)"(char *uplo, int *n, d *ap, int *ipiv, int *info) nogil
 * cdef void dsptrf(char *uplo, int *n, d *ap, int *ipiv, int *info) nogil:
 *     _fortran_dsptrf(uplo, n, ap, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsptrf,DSPTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsptrf "F_FUNC(dsptrf,DSPTRF)"(char *uplo, int *n, d *ap, int *ipiv, int *info) nogil
 * cdef void dsptrf(char *uplo, int *n, d *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsptrf(uplo, n, ap, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsptri "F_FUNC(dsptri,DSPTRI)"(char *uplo, int *n, d *ap, int *ipiv, d *work, int *info) nogil
 * cdef void dsptri(char *uplo, int *n, d *ap, int *ipiv, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsptri(uplo, n, ap, ipiv, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4657
 *     void _fortran_dsptri "F_FUNC(dsptri,DSPTRI)"(char *uplo, int *n, d *ap, int *ipiv, d *work, int *info) nogil
 * cdef void dsptri(char *uplo, int *n, d *ap, int *ipiv, d *work, int *info) nogil:
 *     _fortran_dsptri(uplo, n, ap, ipiv, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsptri,DSPTRI)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsptri "F_FUNC(dsptri,DSPTRI)"(char *uplo, int *n, d *ap, int *ipiv, d *work, int *info) nogil
 * cdef void dsptri(char *uplo, int *n, d *ap, int *ipiv, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsptri(uplo, n, ap, ipiv, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsptrs "F_FUNC(dsptrs,DSPTRS)"(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dsptrs(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsptrs(uplo, n, nrhs, ap, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4662
 *     void _fortran_dsptrs "F_FUNC(dsptrs,DSPTRS)"(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dsptrs(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil:
 *     _fortran_dsptrs(uplo, n, nrhs, ap, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsptrs,DSPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsptrs "F_FUNC(dsptrs,DSPTRS)"(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dsptrs(char *uplo, int *n, int *nrhs, d *ap, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsptrs(uplo, n, nrhs, ap, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstebz "F_FUNC(dstebz,DSTEBZ)"(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *abstol, d *d, d *e, int *m, int *nsplit, d *w, int *iblock, int *isplit, d *work, int *iwork, int *info) nogil
 * cdef void dstebz(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *abstol, d *d, d *e, int *m, int *nsplit, d *w, int *iblock, int *isplit, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstebz(range, order, n, vl, vu, il, iu, abstol, d, e, m, nsplit, w, iblock, isplit, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstebz(char *__pyx_v_range, char *__pyx_v_order, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, int *__pyx_v_m, int *__pyx_v_nsplit, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, int *__pyx_v_iblock, int *__pyx_v_isplit, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4667
 *     void _fortran_dstebz "F_FUNC(dstebz,DSTEBZ)"(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *abstol, d *d, d *e, int *m, int *nsplit, d *w, int *iblock, int *isplit, d *work, int *iwork, int *info) nogil
 * cdef void dstebz(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *abstol, d *d, d *e, int *m, int *nsplit, d *w, int *iblock, int *isplit, d *work, int *iwork, int *info) nogil:
 *     _fortran_dstebz(range, order, n, vl, vu, il, iu, abstol, d, e, m, nsplit, w, iblock, isplit, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstebz,DSTEBZ)(__pyx_v_range, __pyx_v_order, __pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_d, __pyx_v_e, __pyx_v_m, __pyx_v_nsplit, __pyx_v_w, __pyx_v_iblock, __pyx_v_isplit, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstebz "F_FUNC(dstebz,DSTEBZ)"(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *abstol, d *d, d *e, int *m, int *nsplit, d *w, int *iblock, int *isplit, d *work, int *iwork, int *info) nogil
 * cdef void dstebz(char *range, char *order, int *n, d *vl, d *vu, int *il, int *iu, d *abstol, d *d, d *e, int *m, int *nsplit, d *w, int *iblock, int *isplit, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstebz(range, order, n, vl, vu, il, iu, abstol, d, e, m, nsplit, w, iblock, isplit, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstedc "F_FUNC(dstedc,DSTEDC)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstedc(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstedc(compz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstedc(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4672
 *     void _fortran_dstedc "F_FUNC(dstedc,DSTEDC)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstedc(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dstedc(compz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstedc,DSTEDC)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstedc "F_FUNC(dstedc,DSTEDC)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstedc(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstedc(compz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstegr "F_FUNC(dstegr,DSTEGR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstegr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstegr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4677
 *     void _fortran_dstegr "F_FUNC(dstegr,DSTEGR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstegr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstegr,DSTEGR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstegr "F_FUNC(dstegr,DSTEGR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstegr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstein "F_FUNC(dstein,DSTEIN)"(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dstein(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstein(n, d, e, m, w, iblock, isplit, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstein(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, int *__pyx_v_iblock, int *__pyx_v_isplit, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4682
 *     void _fortran_dstein "F_FUNC(dstein,DSTEIN)"(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dstein(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_dstein(n, d, e, m, w, iblock, isplit, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstein,DSTEIN)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_m, __pyx_v_w, __pyx_v_iblock, __pyx_v_isplit, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstein "F_FUNC(dstein,DSTEIN)"(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dstein(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstein(n, d, e, m, w, iblock, isplit, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstemr "F_FUNC(dstemr,DSTEMR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, d *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstemr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, d *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstemr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_nzc, int *__pyx_v_isuppz, int *__pyx_v_tryrac, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4687
 *     void _fortran_dstemr "F_FUNC(dstemr,DSTEMR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, d *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstemr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, d *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstemr,DSTEMR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_nzc, __pyx_v_isuppz, __pyx_v_tryrac, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstemr "F_FUNC(dstemr,DSTEMR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, d *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstemr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, d *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsteqr "F_FUNC(dsteqr,DSTEQR)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsteqr(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsteqr(compz, n, d, e, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsteqr(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4692
 *     void _fortran_dsteqr "F_FUNC(dsteqr,DSTEQR)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsteqr(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_dsteqr(compz, n, d, e, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsteqr,DSTEQR)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsteqr "F_FUNC(dsteqr,DSTEQR)"(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dsteqr(char *compz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsteqr(compz, n, d, e, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsterf "F_FUNC(dsterf,DSTERF)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dsterf(int *n, d *d, d *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsterf(n, d, e, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsterf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4697
 *     void _fortran_dsterf "F_FUNC(dsterf,DSTERF)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dsterf(int *n, d *d, d *e, int *info) nogil:
 *     _fortran_dsterf(n, d, e, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsterf,DSTERF)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsterf "F_FUNC(dsterf,DSTERF)"(int *n, d *d, d *e, int *info) nogil
 * cdef void dsterf(int *n, d *d, d *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsterf(n, d, e, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstev "F_FUNC(dstev,DSTEV)"(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dstev(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstev(jobz, n, d, e, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstev(char *__pyx_v_jobz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4702
 *     void _fortran_dstev "F_FUNC(dstev,DSTEV)"(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dstev(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_dstev(jobz, n, d, e, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstev,DSTEV)(__pyx_v_jobz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstev "F_FUNC(dstev,DSTEV)"(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil
 * cdef void dstev(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstev(jobz, n, d, e, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstevd "F_FUNC(dstevd,DSTEVD)"(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstevd(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstevd(jobz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstevd(char *__pyx_v_jobz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4707
 *     void _fortran_dstevd "F_FUNC(dstevd,DSTEVD)"(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstevd(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dstevd(jobz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstevd,DSTEVD)(__pyx_v_jobz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstevd "F_FUNC(dstevd,DSTEVD)"(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstevd(char *jobz, int *n, d *d, d *e, d *z, int *ldz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstevd(jobz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstevr "F_FUNC(dstevr,DSTEVR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstevr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstevr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstevr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4712
 *     void _fortran_dstevr "F_FUNC(dstevr,DSTEVR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstevr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dstevr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstevr,DSTEVR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstevr "F_FUNC(dstevr,DSTEVR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dstevr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstevr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstevx "F_FUNC(dstevx,DSTEVX)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dstevx(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstevx(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dstevx(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4717
 *     void _fortran_dstevx "F_FUNC(dstevx,DSTEVX)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dstevx(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_dstevx(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dstevx,DSTEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dstevx "F_FUNC(dstevx,DSTEVX)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void dstevx(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dstevx(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsycon "F_FUNC(dsycon,DSYCON)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dsycon(char *uplo, int *n, d *a, int *lda, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsycon(uplo, n, a, lda, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsycon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4722
 *     void _fortran_dsycon "F_FUNC(dsycon,DSYCON)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dsycon(char *uplo, int *n, d *a, int *lda, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dsycon(uplo, n, a, lda, ipiv, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsycon,DSYCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsycon "F_FUNC(dsycon,DSYCON)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dsycon(char *uplo, int *n, d *a, int *lda, int *ipiv, d *anorm, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsycon(uplo, n, a, lda, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyev "F_FUNC(dsyev,DSYEV)"(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *info) nogil
 * cdef void dsyev(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyev(jobz, uplo, n, a, lda, w, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsyev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4727
 *     void _fortran_dsyev "F_FUNC(dsyev,DSYEV)"(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *info) nogil
 * cdef void dsyev(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *info) nogil:
 *     _fortran_dsyev(jobz, uplo, n, a, lda, w, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsyev,DSYEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_w, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyev "F_FUNC(dsyev,DSYEV)"(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *info) nogil
 * cdef void dsyev(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyev(jobz, uplo, n, a, lda, w, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyevd "F_FUNC(dsyevd,DSYEVD)"(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsyevd(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyevd(jobz, uplo, n, a, lda, w, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsyevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4732
 *     void _fortran_dsyevd "F_FUNC(dsyevd,DSYEVD)"(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsyevd(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dsyevd(jobz, uplo, n, a, lda, w, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsyevd,DSYEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_w, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyevd "F_FUNC(dsyevd,DSYEVD)"(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsyevd(char *jobz, char *uplo, int *n, d *a, int *lda, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyevd(jobz, uplo, n, a, lda, w, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyevr "F_FUNC(dsyevr,DSYEVR)"(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsyevr(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyevr(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsyevr(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4737
 *     void _fortran_dsyevr "F_FUNC(dsyevr,DSYEVR)"(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsyevr(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dsyevr(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsyevr,DSYEVR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyevr "F_FUNC(dsyevr,DSYEVR)"(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsyevr(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyevr(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyevx "F_FUNC(dsyevx,DSYEVX)"(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void dsyevx(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyevx(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsyevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4742
 *     void _fortran_dsyevx "F_FUNC(dsyevx,DSYEVX)"(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void dsyevx(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_dsyevx(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsyevx,DSYEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyevx "F_FUNC(dsyevx,DSYEVX)"(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void dsyevx(char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyevx(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygs2 "F_FUNC(dsygs2,DSYGS2)"(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dsygs2(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygs2(itype, uplo, n, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsygs2(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4747
 *     void _fortran_dsygs2 "F_FUNC(dsygs2,DSYGS2)"(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dsygs2(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil:
 *     _fortran_dsygs2(itype, uplo, n, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsygs2,DSYGS2)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygs2 "F_FUNC(dsygs2,DSYGS2)"(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dsygs2(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygs2(itype, uplo, n, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygst "F_FUNC(dsygst,DSYGST)"(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dsygst(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygst(itype, uplo, n, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsygst(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4752
 *     void _fortran_dsygst "F_FUNC(dsygst,DSYGST)"(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dsygst(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil:
 *     _fortran_dsygst(itype, uplo, n, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsygst,DSYGST)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygst "F_FUNC(dsygst,DSYGST)"(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dsygst(int *itype, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygst(itype, uplo, n, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygv "F_FUNC(dsygv,DSYGV)"(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *info) nogil
 * cdef void dsygv(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygv(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsygv(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4757
 *     void _fortran_dsygv "F_FUNC(dsygv,DSYGV)"(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *info) nogil
 * cdef void dsygv(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *info) nogil:
 *     _fortran_dsygv(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsygv,DSYGV)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_w, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygv "F_FUNC(dsygv,DSYGV)"(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *info) nogil
 * cdef void dsygv(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygv(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygvd "F_FUNC(dsygvd,DSYGVD)"(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsygvd(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygvd(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsygvd(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4762
 *     void _fortran_dsygvd "F_FUNC(dsygvd,DSYGVD)"(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsygvd(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dsygvd(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsygvd,DSYGVD)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_w, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygvd "F_FUNC(dsygvd,DSYGVD)"(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dsygvd(int *itype, char *jobz, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *w, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygvd(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygvx "F_FUNC(dsygvx,DSYGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void dsygvx(int *itype, char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygvx(itype, jobz, range, uplo, n, a, lda, b, ldb, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsygvx(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4767
 *     void _fortran_dsygvx "F_FUNC(dsygvx,DSYGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void dsygvx(int *itype, char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_dsygvx(itype, jobz, range, uplo, n, a, lda, b, ldb, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsygvx,DSYGVX)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsygvx "F_FUNC(dsygvx,DSYGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void dsygvx(int *itype, char *jobz, char *range, char *uplo, int *n, d *a, int *lda, d *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, d *z, int *ldz, d *work, int *lwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsygvx(itype, jobz, range, uplo, n, a, lda, b, ldb, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyrfs "F_FUNC(dsyrfs,DSYRFS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dsyrfs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyrfs(uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsyrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4772
 *     void _fortran_dsyrfs "F_FUNC(dsyrfs,DSYRFS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dsyrfs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dsyrfs(uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsyrfs,DSYRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsyrfs "F_FUNC(dsyrfs,DSYRFS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dsyrfs(char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsyrfs(uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsysv "F_FUNC(dsysv,DSYSV)"(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *work, int *lwork, int *info) nogil
 * cdef void dsysv(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsysv(uplo, n, nrhs, a, lda, ipiv, b, ldb, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsysv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4777
 *     void _fortran_dsysv "F_FUNC(dsysv,DSYSV)"(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *work, int *lwork, int *info) nogil
 * cdef void dsysv(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *work, int *lwork, int *info) nogil:
 *     _fortran_dsysv(uplo, n, nrhs, a, lda, ipiv, b, ldb, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsysv,DSYSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsysv "F_FUNC(dsysv,DSYSV)"(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *work, int *lwork, int *info) nogil
 * cdef void dsysv(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsysv(uplo, n, nrhs, a, lda, ipiv, b, ldb, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsysvx "F_FUNC(dsysvx,DSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dsysvx(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsysvx(fact, uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsysvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4782
 *     void _fortran_dsysvx "F_FUNC(dsysvx,DSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dsysvx(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_dsysvx(fact, uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsysvx,DSYSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsysvx "F_FUNC(dsysvx,DSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dsysvx(char *fact, char *uplo, int *n, int *nrhs, d *a, int *lda, d *af, int *ldaf, int *ipiv, d *b, int *ldb, d *x, int *ldx, d *rcond, d *ferr, d *berr, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsysvx(fact, uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytd2 "F_FUNC(dsytd2,DSYTD2)"(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, int *info) nogil
 * cdef void dsytd2(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytd2(uplo, n, a, lda, d, e, tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsytd2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4787
 *     void _fortran_dsytd2 "F_FUNC(dsytd2,DSYTD2)"(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, int *info) nogil
 * cdef void dsytd2(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, int *info) nogil:
 *     _fortran_dsytd2(uplo, n, a, lda, d, e, tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsytd2,DSYTD2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tau, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytd2 "F_FUNC(dsytd2,DSYTD2)"(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, int *info) nogil
 * cdef void dsytd2(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytd2(uplo, n, a, lda, d, e, tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytf2 "F_FUNC(dsytf2,DSYTF2)"(char *uplo, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dsytf2(char *uplo, int *n, d *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytf2(uplo, n, a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsytf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4792
 *     void _fortran_dsytf2 "F_FUNC(dsytf2,DSYTF2)"(char *uplo, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dsytf2(char *uplo, int *n, d *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_dsytf2(uplo, n, a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsytf2,DSYTF2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytf2 "F_FUNC(dsytf2,DSYTF2)"(char *uplo, int *n, d *a, int *lda, int *ipiv, int *info) nogil
 * cdef void dsytf2(char *uplo, int *n, d *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytf2(uplo, n, a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytrd "F_FUNC(dsytrd,DSYTRD)"(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dsytrd(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytrd(uplo, n, a, lda, d, e, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsytrd(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4797
 *     void _fortran_dsytrd "F_FUNC(dsytrd,DSYTRD)"(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dsytrd(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dsytrd(uplo, n, a, lda, d, e, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsytrd,DSYTRD)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytrd "F_FUNC(dsytrd,DSYTRD)"(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dsytrd(char *uplo, int *n, d *a, int *lda, d *d, d *e, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytrd(uplo, n, a, lda, d, e, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytrf "F_FUNC(dsytrf,DSYTRF)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil
 * cdef void dsytrf(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytrf(uplo, n, a, lda, ipiv, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsytrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4802
 *     void _fortran_dsytrf "F_FUNC(dsytrf,DSYTRF)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil
 * cdef void dsytrf(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil:
 *     _fortran_dsytrf(uplo, n, a, lda, ipiv, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsytrf,DSYTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytrf "F_FUNC(dsytrf,DSYTRF)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil
 * cdef void dsytrf(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytrf(uplo, n, a, lda, ipiv, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytri "F_FUNC(dsytri,DSYTRI)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *info) nogil
 * cdef void dsytri(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytri(uplo, n, a, lda, ipiv, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsytri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4807
 *     void _fortran_dsytri "F_FUNC(dsytri,DSYTRI)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *info) nogil
 * cdef void dsytri(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *info) nogil:
 *     _fortran_dsytri(uplo, n, a, lda, ipiv, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsytri,DSYTRI)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytri "F_FUNC(dsytri,DSYTRI)"(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *info) nogil
 * cdef void dsytri(char *uplo, int *n, d *a, int *lda, int *ipiv, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytri(uplo, n, a, lda, ipiv, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytrs "F_FUNC(dsytrs,DSYTRS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dsytrs(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytrs(uplo, n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dsytrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4812
 *     void _fortran_dsytrs "F_FUNC(dsytrs,DSYTRS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dsytrs(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:
 *     _fortran_dsytrs(uplo, n, nrhs, a, lda, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dsytrs,DSYTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dsytrs "F_FUNC(dsytrs,DSYTRS)"(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil
 * cdef void dsytrs(char *uplo, int *n, int *nrhs, d *a, int *lda, int *ipiv, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dsytrs(uplo, n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtbcon "F_FUNC(dtbcon,DTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, d *ab, int *ldab, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtbcon(char *norm, char *uplo, char *diag, int *n, int *kd, d *ab, int *ldab, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtbcon(norm, uplo, diag, n, kd, ab, ldab, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtbcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4817
 *     void _fortran_dtbcon "F_FUNC(dtbcon,DTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, d *ab, int *ldab, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtbcon(char *norm, char *uplo, char *diag, int *n, int *kd, d *ab, int *ldab, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dtbcon(norm, uplo, diag, n, kd, ab, ldab, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtbcon,DTBCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtbcon "F_FUNC(dtbcon,DTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, d *ab, int *ldab, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtbcon(char *norm, char *uplo, char *diag, int *n, int *kd, d *ab, int *ldab, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtbcon(norm, uplo, diag, n, kd, ab, ldab, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtbrfs "F_FUNC(dtbrfs,DTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtbrfs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtbrfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4822
 *     void _fortran_dtbrfs "F_FUNC(dtbrfs,DTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dtbrfs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtbrfs,DTBRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtbrfs "F_FUNC(dtbrfs,DTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtbrfs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtbtrs "F_FUNC(dtbtrs,DTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dtbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtbtrs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtbtrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4827
 *     void _fortran_dtbtrs "F_FUNC(dtbtrs,DTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dtbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:
 *     _fortran_dtbtrs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtbtrs,DTBTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtbtrs "F_FUNC(dtbtrs,DTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil
 * cdef void dtbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, d *ab, int *ldab, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtbtrs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgevc "F_FUNC(dtgevc,DTGEVC)"(char *side, char *howmny, bint *select, int *n, d *s, int *lds, d *p, int *ldp, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil
 * cdef void dtgevc(char *side, char *howmny, bint *select, int *n, d *s, int *lds, d *p, int *ldp, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgevc(side, howmny, select, n, s, lds, p, ldp, vl, ldvl, vr, ldvr, mm, m, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtgevc(char *__pyx_v_side, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, int *__pyx_v_lds, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_p, int *__pyx_v_ldp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4832
 *     void _fortran_dtgevc "F_FUNC(dtgevc,DTGEVC)"(char *side, char *howmny, bint *select, int *n, d *s, int *lds, d *p, int *ldp, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil
 * cdef void dtgevc(char *side, char *howmny, bint *select, int *n, d *s, int *lds, d *p, int *ldp, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil:
 *     _fortran_dtgevc(side, howmny, select, n, s, lds, p, ldp, vl, ldvl, vr, ldvr, mm, m, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtgevc,DTGEVC)(__pyx_v_side, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, __pyx_v_s, __pyx_v_lds, __pyx_v_p, __pyx_v_ldp, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgevc "F_FUNC(dtgevc,DTGEVC)"(char *side, char *howmny, bint *select, int *n, d *s, int *lds, d *p, int *ldp, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil
 * cdef void dtgevc(char *side, char *howmny, bint *select, int *n, d *s, int *lds, d *p, int *ldp, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgevc(side, howmny, select, n, s, lds, p, ldp, vl, ldvl, vr, ldvr, mm, m, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgex2 "F_FUNC(dtgex2,DTGEX2)"(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *j1, int *n1, int *n2, d *work, int *lwork, int *info) nogil
 * cdef void dtgex2(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *j1, int *n1, int *n2, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgex2(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, j1, n1, n2, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtgex2(int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_j1, int *__pyx_v_n1, int *__pyx_v_n2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4837
 *     void _fortran_dtgex2 "F_FUNC(dtgex2,DTGEX2)"(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *j1, int *n1, int *n2, d *work, int *lwork, int *info) nogil
 * cdef void dtgex2(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *j1, int *n1, int *n2, d *work, int *lwork, int *info) nogil:
 *     _fortran_dtgex2(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, j1, n1, n2, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtgex2,DTGEX2)(__pyx_v_wantq, __pyx_v_wantz, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_j1, __pyx_v_n1, __pyx_v_n2, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgex2 "F_FUNC(dtgex2,DTGEX2)"(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *j1, int *n1, int *n2, d *work, int *lwork, int *info) nogil
 * cdef void dtgex2(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *j1, int *n1, int *n2, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgex2(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, j1, n1, n2, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgexc "F_FUNC(dtgexc,DTGEXC)"(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *ifst, int *ilst, d *work, int *lwork, int *info) nogil
 * cdef void dtgexc(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *ifst, int *ilst, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgexc(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, ifst, ilst, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtgexc(int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ifst, int *__pyx_v_ilst, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4842
 *     void _fortran_dtgexc "F_FUNC(dtgexc,DTGEXC)"(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *ifst, int *ilst, d *work, int *lwork, int *info) nogil
 * cdef void dtgexc(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *ifst, int *ilst, d *work, int *lwork, int *info) nogil:
 *     _fortran_dtgexc(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, ifst, ilst, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtgexc,DTGEXC)(__pyx_v_wantq, __pyx_v_wantz, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_ifst, __pyx_v_ilst, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgexc "F_FUNC(dtgexc,DTGEXC)"(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *ifst, int *ilst, d *work, int *lwork, int *info) nogil
 * cdef void dtgexc(bint *wantq, bint *wantz, int *n, d *a, int *lda, d *b, int *ldb, d *q, int *ldq, d *z, int *ldz, int *ifst, int *ilst, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgexc(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, ifst, ilst, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsen "F_FUNC(dtgsen,DTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, int *m, d *pl, d *pr, d *dif, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dtgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, int *m, d *pl, d *pr, d *dif, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsen(ijob, wantq, wantz, select, n, a, lda, b, ldb, alphar, alphai, beta, q, ldq, z, ldz, m, pl, pr, dif, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtgsen(int *__pyx_v_ijob, int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dif, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4847
 *     void _fortran_dtgsen "F_FUNC(dtgsen,DTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, int *m, d *pl, d *pr, d *dif, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dtgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, int *m, d *pl, d *pr, d *dif, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dtgsen(ijob, wantq, wantz, select, n, a, lda, b, ldb, alphar, alphai, beta, q, ldq, z, ldz, m, pl, pr, dif, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtgsen,DTGSEN)(__pyx_v_ijob, __pyx_v_wantq, __pyx_v_wantz, __pyx_v_select, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_m, __pyx_v_pl, __pyx_v_pr, __pyx_v_dif, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsen "F_FUNC(dtgsen,DTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, int *m, d *pl, d *pr, d *dif, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dtgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *alphar, d *alphai, d *beta, d *q, int *ldq, d *z, int *ldz, int *m, d *pl, d *pr, d *dif, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsen(ijob, wantq, wantz, select, n, a, lda, b, ldb, alphar, alphai, beta, q, ldq, z, ldz, m, pl, pr, dif, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsja "F_FUNC(dtgsja,DTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, d *a, int *lda, d *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, d *u, int *ldu, d *v, int *ldv, d *q, int *ldq, d *work, int *ncycle, int *info) nogil
 * cdef void dtgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, d *a, int *lda, d *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, d *u, int *ldu, d *v, int *ldv, d *q, int *ldq, d *work, int *ncycle, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsja(jobu, jobv, jobq, m, p, n, k, l, a, lda, b, ldb, tola, tolb, alpha, beta, u, ldu, v, ldv, q, ldq, work, ncycle, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtgsja(char *__pyx_v_jobu, char *__pyx_v_jobv, char *__pyx_v_jobq, int *__pyx_v_m, int *__pyx_v_p, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tola, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tolb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_ncycle, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4852
 *     void _fortran_dtgsja "F_FUNC(dtgsja,DTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, d *a, int *lda, d *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, d *u, int *ldu, d *v, int *ldv, d *q, int *ldq, d *work, int *ncycle, int *info) nogil
 * cdef void dtgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, d *a, int *lda, d *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, d *u, int *ldu, d *v, int *ldv, d *q, int *ldq, d *work, int *ncycle, int *info) nogil:
 *     _fortran_dtgsja(jobu, jobv, jobq, m, p, n, k, l, a, lda, b, ldb, tola, tolb, alpha, beta, u, ldu, v, ldv, q, ldq, work, ncycle, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtgsja,DTGSJA)(__pyx_v_jobu, __pyx_v_jobv, __pyx_v_jobq, __pyx_v_m, __pyx_v_p, __pyx_v_n, __pyx_v_k, __pyx_v_l, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_tola, __pyx_v_tolb, __pyx_v_alpha, __pyx_v_beta, __pyx_v_u, __pyx_v_ldu, __pyx_v_v, __pyx_v_ldv, __pyx_v_q, __pyx_v_ldq, __pyx_v_work, __pyx_v_ncycle, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsja "F_FUNC(dtgsja,DTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, d *a, int *lda, d *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, d *u, int *ldu, d *v, int *ldv, d *q, int *ldq, d *work, int *ncycle, int *info) nogil
 * cdef void dtgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, d *a, int *lda, d *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, d *u, int *ldu, d *v, int *ldv, d *q, int *ldq, d *work, int *ncycle, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsja(jobu, jobv, jobq, m, p, n, k, l, a, lda, b, ldb, tola, tolb, alpha, beta, u, ldu, v, ldv, q, ldq, work, ncycle, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsna "F_FUNC(dtgsna,DTGSNA)"(char *job, char *howmny, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *dif, int *mm, int *m, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dtgsna(char *job, char *howmny, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *dif, int *mm, int *m, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsna(job, howmny, select, n, a, lda, b, ldb, vl, ldvl, vr, ldvr, s, dif, mm, m, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtgsna(char *__pyx_v_job, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dif, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4857
 *     void _fortran_dtgsna "F_FUNC(dtgsna,DTGSNA)"(char *job, char *howmny, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *dif, int *mm, int *m, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dtgsna(char *job, char *howmny, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *dif, int *mm, int *m, d *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_dtgsna(job, howmny, select, n, a, lda, b, ldb, vl, ldvl, vr, ldvr, s, dif, mm, m, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtgsna,DTGSNA)(__pyx_v_job, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_s, __pyx_v_dif, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsna "F_FUNC(dtgsna,DTGSNA)"(char *job, char *howmny, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *dif, int *mm, int *m, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dtgsna(char *job, char *howmny, bint *select, int *n, d *a, int *lda, d *b, int *ldb, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *dif, int *mm, int *m, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsna(job, howmny, select, n, a, lda, b, ldb, vl, ldvl, vr, ldvr, s, dif, mm, m, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsy2 "F_FUNC(dtgsy2,DTGSY2)"(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *iwork, int *pq, int *info) nogil
 * cdef void dtgsy2(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *iwork, int *pq, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsy2(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, rdsum, rdscal, iwork, pq, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtgsy2(char *__pyx_v_trans, int *__pyx_v_ijob, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, int *__pyx_v_ldd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, int *__pyx_v_lde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_f, int *__pyx_v_ldf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rdsum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rdscal, int *__pyx_v_iwork, int *__pyx_v_pq, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4862
 *     void _fortran_dtgsy2 "F_FUNC(dtgsy2,DTGSY2)"(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *iwork, int *pq, int *info) nogil
 * cdef void dtgsy2(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *iwork, int *pq, int *info) nogil:
 *     _fortran_dtgsy2(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, rdsum, rdscal, iwork, pq, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtgsy2,DTGSY2)(__pyx_v_trans, __pyx_v_ijob, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_c, __pyx_v_ldc, __pyx_v_d, __pyx_v_ldd, __pyx_v_e, __pyx_v_lde, __pyx_v_f, __pyx_v_ldf, __pyx_v_scale, __pyx_v_rdsum, __pyx_v_rdscal, __pyx_v_iwork, __pyx_v_pq, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsy2 "F_FUNC(dtgsy2,DTGSY2)"(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *iwork, int *pq, int *info) nogil
 * cdef void dtgsy2(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *iwork, int *pq, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsy2(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, rdsum, rdscal, iwork, pq, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsyl "F_FUNC(dtgsyl,DTGSYL)"(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *dif, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dtgsyl(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *dif, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsyl(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, dif, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtgsyl(char *__pyx_v_trans, int *__pyx_v_ijob, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, int *__pyx_v_ldd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, int *__pyx_v_lde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_f, int *__pyx_v_ldf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dif, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4867
 *     void _fortran_dtgsyl "F_FUNC(dtgsyl,DTGSYL)"(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *dif, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dtgsyl(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *dif, d *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_dtgsyl(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, dif, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtgsyl,DTGSYL)(__pyx_v_trans, __pyx_v_ijob, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_c, __pyx_v_ldc, __pyx_v_d, __pyx_v_ldd, __pyx_v_e, __pyx_v_lde, __pyx_v_f, __pyx_v_ldf, __pyx_v_scale, __pyx_v_dif, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtgsyl "F_FUNC(dtgsyl,DTGSYL)"(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *dif, d *work, int *lwork, int *iwork, int *info) nogil
 * cdef void dtgsyl(char *trans, int *ijob, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *d, int *ldd, d *e, int *lde, d *f, int *ldf, d *scale, d *dif, d *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtgsyl(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, dif, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtpcon "F_FUNC(dtpcon,DTPCON)"(char *norm, char *uplo, char *diag, int *n, d *ap, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtpcon(char *norm, char *uplo, char *diag, int *n, d *ap, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtpcon(norm, uplo, diag, n, ap, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtpcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4872
 *     void _fortran_dtpcon "F_FUNC(dtpcon,DTPCON)"(char *norm, char *uplo, char *diag, int *n, d *ap, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtpcon(char *norm, char *uplo, char *diag, int *n, d *ap, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dtpcon(norm, uplo, diag, n, ap, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtpcon,DTPCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_ap, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtpcon "F_FUNC(dtpcon,DTPCON)"(char *norm, char *uplo, char *diag, int *n, d *ap, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtpcon(char *norm, char *uplo, char *diag, int *n, d *ap, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtpcon(norm, uplo, diag, n, ap, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtprfs "F_FUNC(dtprfs,DTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtprfs(uplo, trans, diag, n, nrhs, ap, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtprfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4877
 *     void _fortran_dtprfs "F_FUNC(dtprfs,DTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dtprfs(uplo, trans, diag, n, nrhs, ap, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtprfs,DTPRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtprfs "F_FUNC(dtprfs,DTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtprfs(uplo, trans, diag, n, nrhs, ap, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtptri "F_FUNC(dtptri,DTPTRI)"(char *uplo, char *diag, int *n, d *ap, int *info) nogil
 * cdef void dtptri(char *uplo, char *diag, int *n, d *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtptri(uplo, diag, n, ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtptri(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4882
 *     void _fortran_dtptri "F_FUNC(dtptri,DTPTRI)"(char *uplo, char *diag, int *n, d *ap, int *info) nogil
 * cdef void dtptri(char *uplo, char *diag, int *n, d *ap, int *info) nogil:
 *     _fortran_dtptri(uplo, diag, n, ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtptri,DTPTRI)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_ap, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtptri "F_FUNC(dtptri,DTPTRI)"(char *uplo, char *diag, int *n, d *ap, int *info) nogil
 * cdef void dtptri(char *uplo, char *diag, int *n, d *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtptri(uplo, diag, n, ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtptrs "F_FUNC(dtptrs,DTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dtptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtptrs(uplo, trans, diag, n, nrhs, ap, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtptrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4887
 *     void _fortran_dtptrs "F_FUNC(dtptrs,DTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dtptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:
 *     _fortran_dtptrs(uplo, trans, diag, n, nrhs, ap, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtptrs,DTPTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtptrs "F_FUNC(dtptrs,DTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil
 * cdef void dtptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *ap, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtptrs(uplo, trans, diag, n, nrhs, ap, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrcon "F_FUNC(dtrcon,DTRCON)"(char *norm, char *uplo, char *diag, int *n, d *a, int *lda, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtrcon(char *norm, char *uplo, char *diag, int *n, d *a, int *lda, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrcon(norm, uplo, diag, n, a, lda, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4892
 *     void _fortran_dtrcon "F_FUNC(dtrcon,DTRCON)"(char *norm, char *uplo, char *diag, int *n, d *a, int *lda, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtrcon(char *norm, char *uplo, char *diag, int *n, d *a, int *lda, d *rcond, d *work, int *iwork, int *info) nogil:
 *     _fortran_dtrcon(norm, uplo, diag, n, a, lda, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrcon,DTRCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrcon "F_FUNC(dtrcon,DTRCON)"(char *norm, char *uplo, char *diag, int *n, d *a, int *lda, d *rcond, d *work, int *iwork, int *info) nogil
 * cdef void dtrcon(char *norm, char *uplo, char *diag, int *n, d *a, int *lda, d *rcond, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrcon(norm, uplo, diag, n, a, lda, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrevc "F_FUNC(dtrevc,DTREVC)"(char *side, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil
 * cdef void dtrevc(char *side, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrevc(side, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, mm, m, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrevc(char *__pyx_v_side, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4897
 *     void _fortran_dtrevc "F_FUNC(dtrevc,DTREVC)"(char *side, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil
 * cdef void dtrevc(char *side, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil:
 *     _fortran_dtrevc(side, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, mm, m, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrevc,DTREVC)(__pyx_v_side, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrevc "F_FUNC(dtrevc,DTREVC)"(char *side, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil
 * cdef void dtrevc(char *side, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, int *mm, int *m, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrevc(side, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, mm, m, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrexc "F_FUNC(dtrexc,DTREXC)"(char *compq, int *n, d *t, int *ldt, d *q, int *ldq, int *ifst, int *ilst, d *work, int *info) nogil
 * cdef void dtrexc(char *compq, int *n, d *t, int *ldt, d *q, int *ldq, int *ifst, int *ilst, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrexc(compq, n, t, ldt, q, ldq, ifst, ilst, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrexc(char *__pyx_v_compq, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_ifst, int *__pyx_v_ilst, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4902
 *     void _fortran_dtrexc "F_FUNC(dtrexc,DTREXC)"(char *compq, int *n, d *t, int *ldt, d *q, int *ldq, int *ifst, int *ilst, d *work, int *info) nogil
 * cdef void dtrexc(char *compq, int *n, d *t, int *ldt, d *q, int *ldq, int *ifst, int *ilst, d *work, int *info) nogil:
 *     _fortran_dtrexc(compq, n, t, ldt, q, ldq, ifst, ilst, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrexc,DTREXC)(__pyx_v_compq, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_q, __pyx_v_ldq, __pyx_v_ifst, __pyx_v_ilst, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrexc "F_FUNC(dtrexc,DTREXC)"(char *compq, int *n, d *t, int *ldt, d *q, int *ldq, int *ifst, int *ilst, d *work, int *info) nogil
 * cdef void dtrexc(char *compq, int *n, d *t, int *ldt, d *q, int *ldq, int *ifst, int *ilst, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrexc(compq, n, t, ldt, q, ldq, ifst, ilst, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrrfs "F_FUNC(dtrrfs,DTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrrfs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrrfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4907
 *     void _fortran_dtrrfs "F_FUNC(dtrrfs,DTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:
 *     _fortran_dtrrfs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrrfs,DTRRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrrfs "F_FUNC(dtrrfs,DTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil
 * cdef void dtrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, d *x, int *ldx, d *ferr, d *berr, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrrfs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrsen "F_FUNC(dtrsen,DTRSEN)"(char *job, char *compq, bint *select, int *n, d *t, int *ldt, d *q, int *ldq, d *wr, d *wi, int *m, d *s, d *sep, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dtrsen(char *job, char *compq, bint *select, int *n, d *t, int *ldt, d *q, int *ldq, d *wr, d *wi, int *m, d *s, d *sep, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrsen(job, compq, select, n, t, ldt, q, ldq, wr, wi, m, s, sep, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrsen(char *__pyx_v_job, char *__pyx_v_compq, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wi, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sep, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4912
 *     void _fortran_dtrsen "F_FUNC(dtrsen,DTRSEN)"(char *job, char *compq, bint *select, int *n, d *t, int *ldt, d *q, int *ldq, d *wr, d *wi, int *m, d *s, d *sep, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dtrsen(char *job, char *compq, bint *select, int *n, d *t, int *ldt, d *q, int *ldq, d *wr, d *wi, int *m, d *s, d *sep, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_dtrsen(job, compq, select, n, t, ldt, q, ldq, wr, wi, m, s, sep, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrsen,DTRSEN)(__pyx_v_job, __pyx_v_compq, __pyx_v_select, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_q, __pyx_v_ldq, __pyx_v_wr, __pyx_v_wi, __pyx_v_m, __pyx_v_s, __pyx_v_sep, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrsen "F_FUNC(dtrsen,DTRSEN)"(char *job, char *compq, bint *select, int *n, d *t, int *ldt, d *q, int *ldq, d *wr, d *wi, int *m, d *s, d *sep, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void dtrsen(char *job, char *compq, bint *select, int *n, d *t, int *ldt, d *q, int *ldq, d *wr, d *wi, int *m, d *s, d *sep, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrsen(job, compq, select, n, t, ldt, q, ldq, wr, wi, m, s, sep, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrsna "F_FUNC(dtrsna,DTRSNA)"(char *job, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *sep, int *mm, int *m, d *work, int *ldwork, int *iwork, int *info) nogil
 * cdef void dtrsna(char *job, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *sep, int *mm, int *m, d *work, int *ldwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrsna(job, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, s, sep, mm, m, work, ldwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrsna(char *__pyx_v_job, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sep, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_ldwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4917
 *     void _fortran_dtrsna "F_FUNC(dtrsna,DTRSNA)"(char *job, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *sep, int *mm, int *m, d *work, int *ldwork, int *iwork, int *info) nogil
 * cdef void dtrsna(char *job, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *sep, int *mm, int *m, d *work, int *ldwork, int *iwork, int *info) nogil:
 *     _fortran_dtrsna(job, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, s, sep, mm, m, work, ldwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrsna,DTRSNA)(__pyx_v_job, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_s, __pyx_v_sep, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_ldwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrsna "F_FUNC(dtrsna,DTRSNA)"(char *job, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *sep, int *mm, int *m, d *work, int *ldwork, int *iwork, int *info) nogil
 * cdef void dtrsna(char *job, char *howmny, bint *select, int *n, d *t, int *ldt, d *vl, int *ldvl, d *vr, int *ldvr, d *s, d *sep, int *mm, int *m, d *work, int *ldwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrsna(job, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, s, sep, mm, m, work, ldwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrsyl "F_FUNC(dtrsyl,DTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *scale, int *info) nogil
 * cdef void dtrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrsyl(trana, tranb, isgn, m, n, a, lda, b, ldb, c, ldc, scale, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrsyl(char *__pyx_v_trana, char *__pyx_v_tranb, int *__pyx_v_isgn, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4922
 *     void _fortran_dtrsyl "F_FUNC(dtrsyl,DTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *scale, int *info) nogil
 * cdef void dtrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *scale, int *info) nogil:
 *     _fortran_dtrsyl(trana, tranb, isgn, m, n, a, lda, b, ldb, c, ldc, scale, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrsyl,DTRSYL)(__pyx_v_trana, __pyx_v_tranb, __pyx_v_isgn, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_c, __pyx_v_ldc, __pyx_v_scale, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrsyl "F_FUNC(dtrsyl,DTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *scale, int *info) nogil
 * cdef void dtrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, d *a, int *lda, d *b, int *ldb, d *c, int *ldc, d *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrsyl(trana, tranb, isgn, m, n, a, lda, b, ldb, c, ldc, scale, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrti2 "F_FUNC(dtrti2,DTRTI2)"(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil
 * cdef void dtrti2(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrti2(uplo, diag, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrti2(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4927
 *     void _fortran_dtrti2 "F_FUNC(dtrti2,DTRTI2)"(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil
 * cdef void dtrti2(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil:
 *     _fortran_dtrti2(uplo, diag, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrti2,DTRTI2)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrti2 "F_FUNC(dtrti2,DTRTI2)"(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil
 * cdef void dtrti2(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrti2(uplo, diag, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrtri "F_FUNC(dtrtri,DTRTRI)"(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil
 * cdef void dtrtri(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrtri(uplo, diag, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrtri(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4932
 *     void _fortran_dtrtri "F_FUNC(dtrtri,DTRTRI)"(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil
 * cdef void dtrtri(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil:
 *     _fortran_dtrtri(uplo, diag, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrtri,DTRTRI)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrtri "F_FUNC(dtrtri,DTRTRI)"(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil
 * cdef void dtrtri(char *uplo, char *diag, int *n, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrtri(uplo, diag, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrtrs "F_FUNC(dtrtrs,DTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dtrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrtrs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtrtrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4937
 *     void _fortran_dtrtrs "F_FUNC(dtrtrs,DTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dtrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:
 *     _fortran_dtrtrs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtrtrs,DTRTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtrtrs "F_FUNC(dtrtrs,DTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil
 * cdef void dtrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, d *a, int *lda, d *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtrtrs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtzrzf "F_FUNC(dtzrzf,DTZRZF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dtzrzf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtzrzf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_dtzrzf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4942
 *     void _fortran_dtzrzf "F_FUNC(dtzrzf,DTZRZF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dtzrzf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:
 *     _fortran_dtzrzf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(dtzrzf,DTZRZF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_dtzrzf "F_FUNC(dtzrzf,DTZRZF)"(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil
 * cdef void dtzrzf(int *m, int *n, d *a, int *lda, d *tau, d *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_dtzrzf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ilaver "F_FUNC(ilaver,ILAVER)"(int *vers_major, int *vers_minor, int *vers_patch) nogil
 * cdef void ilaver(int *vers_major, int *vers_minor, int *vers_patch) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ilaver(vers_major, vers_minor, vers_patch)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ilaver(int *__pyx_v_vers_major, int *__pyx_v_vers_minor, int *__pyx_v_vers_patch) {

  /* "scipy/linalg/cython_lapack.pyx":4947
 *     void _fortran_ilaver "F_FUNC(ilaver,ILAVER)"(int *vers_major, int *vers_minor, int *vers_patch) nogil
 * cdef void ilaver(int *vers_major, int *vers_minor, int *vers_patch) nogil:
 *     _fortran_ilaver(vers_major, vers_minor, vers_patch)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ilaver,ILAVER)(__pyx_v_vers_major, __pyx_v_vers_minor, __pyx_v_vers_patch);

  /* "scipy/linalg/cython_lapack.pyx":4946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ilaver "F_FUNC(ilaver,ILAVER)"(int *vers_major, int *vers_minor, int *vers_patch) nogil
 * cdef void ilaver(int *vers_major, int *vers_minor, int *vers_patch) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ilaver(vers_major, vers_minor, vers_patch)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sbdsdc "F_FUNC(sbdsdc,SBDSDC)"(char *uplo, char *compq, int *n, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, s *q, int *iq, s *work, int *iwork, int *info) nogil
 * cdef void sbdsdc(char *uplo, char *compq, int *n, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, s *q, int *iq, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sbdsdc(uplo, compq, n, d, e, u, ldu, vt, ldvt, q, iq, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sbdsdc(char *__pyx_v_uplo, char *__pyx_v_compq, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_iq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4952
 *     void _fortran_sbdsdc "F_FUNC(sbdsdc,SBDSDC)"(char *uplo, char *compq, int *n, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, s *q, int *iq, s *work, int *iwork, int *info) nogil
 * cdef void sbdsdc(char *uplo, char *compq, int *n, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, s *q, int *iq, s *work, int *iwork, int *info) nogil:
 *     _fortran_sbdsdc(uplo, compq, n, d, e, u, ldu, vt, ldvt, q, iq, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sbdsdc,SBDSDC)(__pyx_v_uplo, __pyx_v_compq, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_q, __pyx_v_iq, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sbdsdc "F_FUNC(sbdsdc,SBDSDC)"(char *uplo, char *compq, int *n, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, s *q, int *iq, s *work, int *iwork, int *info) nogil
 * cdef void sbdsdc(char *uplo, char *compq, int *n, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, s *q, int *iq, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sbdsdc(uplo, compq, n, d, e, u, ldu, vt, ldvt, q, iq, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sbdsqr "F_FUNC(sbdsqr,SBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sbdsqr(uplo, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sbdsqr(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ncvt, int *__pyx_v_nru, int *__pyx_v_ncc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4957
 *     void _fortran_sbdsqr "F_FUNC(sbdsqr,SBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_sbdsqr(uplo, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sbdsqr,SBDSQR)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ncvt, __pyx_v_nru, __pyx_v_ncc, __pyx_v_d, __pyx_v_e, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_u, __pyx_v_ldu, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sbdsqr "F_FUNC(sbdsqr,SBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sbdsqr(uplo, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sdisna "F_FUNC(sdisna,SDISNA)"(char *job, int *m, int *n, s *d, s *sep, int *info) nogil
 * cdef void sdisna(char *job, int *m, int *n, s *d, s *sep, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sdisna(job, m, n, d, sep, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sdisna(char *__pyx_v_job, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sep, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4962
 *     void _fortran_sdisna "F_FUNC(sdisna,SDISNA)"(char *job, int *m, int *n, s *d, s *sep, int *info) nogil
 * cdef void sdisna(char *job, int *m, int *n, s *d, s *sep, int *info) nogil:
 *     _fortran_sdisna(job, m, n, d, sep, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sdisna,SDISNA)(__pyx_v_job, __pyx_v_m, __pyx_v_n, __pyx_v_d, __pyx_v_sep, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sdisna "F_FUNC(sdisna,SDISNA)"(char *job, int *m, int *n, s *d, s *sep, int *info) nogil
 * cdef void sdisna(char *job, int *m, int *n, s *d, s *sep, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sdisna(job, m, n, d, sep, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbbrd "F_FUNC(sgbbrd,SGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *pt, int *ldpt, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *pt, int *ldpt, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbbrd(vect, m, n, ncc, kl, ku, ab, ldab, d, e, q, ldq, pt, ldpt, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbbrd(char *__pyx_v_vect, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_ncc, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pt, int *__pyx_v_ldpt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4967
 *     void _fortran_sgbbrd "F_FUNC(sgbbrd,SGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *pt, int *ldpt, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *pt, int *ldpt, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_sgbbrd(vect, m, n, ncc, kl, ku, ab, ldab, d, e, q, ldq, pt, ldpt, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbbrd,SGBBRD)(__pyx_v_vect, __pyx_v_m, __pyx_v_n, __pyx_v_ncc, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_d, __pyx_v_e, __pyx_v_q, __pyx_v_ldq, __pyx_v_pt, __pyx_v_ldpt, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbbrd "F_FUNC(sgbbrd,SGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *pt, int *ldpt, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *pt, int *ldpt, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbbrd(vect, m, n, ncc, kl, ku, ab, ldab, d, e, q, ldq, pt, ldpt, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbcon "F_FUNC(sgbcon,SGBCON)"(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgbcon(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbcon(norm, n, kl, ku, ab, ldab, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbcon(char *__pyx_v_norm, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4972
 *     void _fortran_sgbcon "F_FUNC(sgbcon,SGBCON)"(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgbcon(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgbcon(norm, n, kl, ku, ab, ldab, ipiv, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbcon,SGBCON)(__pyx_v_norm, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbcon "F_FUNC(sgbcon,SGBCON)"(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgbcon(char *norm, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbcon(norm, n, kl, ku, ab, ldab, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbequ "F_FUNC(sgbequ,SGBEQU)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void sgbequ(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbequ(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbequ(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4977
 *     void _fortran_sgbequ "F_FUNC(sgbequ,SGBEQU)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void sgbequ(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:
 *     _fortran_sgbequ(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbequ,SGBEQU)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbequ "F_FUNC(sgbequ,SGBEQU)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void sgbequ(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbequ(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbrfs "F_FUNC(sgbrfs,SGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbrfs(trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbrfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_afb, int *__pyx_v_ldafb, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4982
 *     void _fortran_sgbrfs "F_FUNC(sgbrfs,SGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgbrfs(trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbrfs,SGBRFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_afb, __pyx_v_ldafb, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbrfs "F_FUNC(sgbrfs,SGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbrfs(trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbsv "F_FUNC(sgbsv,SGBSV)"(int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgbsv(int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbsv(n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbsv(int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4987
 *     void _fortran_sgbsv "F_FUNC(sgbsv,SGBSV)"(int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgbsv(int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil:
 *     _fortran_sgbsv(n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbsv,SGBSV)(__pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbsv "F_FUNC(sgbsv,SGBSV)"(int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgbsv(int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbsv(n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbsvx "F_FUNC(sgbsvx,SGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbsvx(fact, trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbsvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_afb, int *__pyx_v_ldafb, int *__pyx_v_ipiv, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4992
 *     void _fortran_sgbsvx "F_FUNC(sgbsvx,SGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgbsvx(fact, trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbsvx,SGBSVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_afb, __pyx_v_ldafb, __pyx_v_ipiv, __pyx_v_equed, __pyx_v_r, __pyx_v_c, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbsvx "F_FUNC(sgbsvx,SGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbsvx(fact, trans, n, kl, ku, nrhs, ab, ldab, afb, ldafb, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":4996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbtf2 "F_FUNC(sgbtf2,SGBTF2)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void sgbtf2(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbtf2(m, n, kl, ku, ab, ldab, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbtf2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":4997
 *     void _fortran_sgbtf2 "F_FUNC(sgbtf2,SGBTF2)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void sgbtf2(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil:
 *     _fortran_sgbtf2(m, n, kl, ku, ab, ldab, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbtf2,SGBTF2)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":4996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbtf2 "F_FUNC(sgbtf2,SGBTF2)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void sgbtf2(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbtf2(m, n, kl, ku, ab, ldab, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbtrf "F_FUNC(sgbtrf,SGBTRF)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void sgbtrf(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbtrf(m, n, kl, ku, ab, ldab, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbtrf(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5002
 *     void _fortran_sgbtrf "F_FUNC(sgbtrf,SGBTRF)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void sgbtrf(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil:
 *     _fortran_sgbtrf(m, n, kl, ku, ab, ldab, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbtrf,SGBTRF)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbtrf "F_FUNC(sgbtrf,SGBTRF)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void sgbtrf(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbtrf(m, n, kl, ku, ab, ldab, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbtrs "F_FUNC(sgbtrs,SGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbtrs(trans, n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgbtrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5007
 *     void _fortran_sgbtrs "F_FUNC(sgbtrs,SGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil:
 *     _fortran_sgbtrs(trans, n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgbtrs,SGBTRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgbtrs "F_FUNC(sgbtrs,SGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, s *ab, int *ldab, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgbtrs(trans, n, kl, ku, nrhs, ab, ldab, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgebak "F_FUNC(sgebak,SGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, s *v, int *ldv, int *info) nogil
 * cdef void sgebak(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, s *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgebak(job, side, n, ilo, ihi, scale, m, v, ldv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgebak(char *__pyx_v_job, char *__pyx_v_side, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5012
 *     void _fortran_sgebak "F_FUNC(sgebak,SGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, s *v, int *ldv, int *info) nogil
 * cdef void sgebak(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, s *v, int *ldv, int *info) nogil:
 *     _fortran_sgebak(job, side, n, ilo, ihi, scale, m, v, ldv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgebak,SGEBAK)(__pyx_v_job, __pyx_v_side, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_m, __pyx_v_v, __pyx_v_ldv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgebak "F_FUNC(sgebak,SGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, s *v, int *ldv, int *info) nogil
 * cdef void sgebak(char *job, char *side, int *n, int *ilo, int *ihi, s *scale, int *m, s *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgebak(job, side, n, ilo, ihi, scale, m, v, ldv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgebal "F_FUNC(sgebal,SGEBAL)"(char *job, int *n, s *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil
 * cdef void sgebal(char *job, int *n, s *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgebal(job, n, a, lda, ilo, ihi, scale, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgebal(char *__pyx_v_job, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5017
 *     void _fortran_sgebal "F_FUNC(sgebal,SGEBAL)"(char *job, int *n, s *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil
 * cdef void sgebal(char *job, int *n, s *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil:
 *     _fortran_sgebal(job, n, a, lda, ilo, ihi, scale, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgebal,SGEBAL)(__pyx_v_job, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgebal "F_FUNC(sgebal,SGEBAL)"(char *job, int *n, s *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil
 * cdef void sgebal(char *job, int *n, s *a, int *lda, int *ilo, int *ihi, s *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgebal(job, n, a, lda, ilo, ihi, scale, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgebd2 "F_FUNC(sgebd2,SGEBD2)"(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *info) nogil
 * cdef void sgebd2(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgebd2(m, n, a, lda, d, e, tauq, taup, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgebd2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tauq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_taup, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5022
 *     void _fortran_sgebd2 "F_FUNC(sgebd2,SGEBD2)"(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *info) nogil
 * cdef void sgebd2(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *info) nogil:
 *     _fortran_sgebd2(m, n, a, lda, d, e, tauq, taup, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgebd2,SGEBD2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tauq, __pyx_v_taup, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgebd2 "F_FUNC(sgebd2,SGEBD2)"(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *info) nogil
 * cdef void sgebd2(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgebd2(m, n, a, lda, d, e, tauq, taup, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgebrd "F_FUNC(sgebrd,SGEBRD)"(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *lwork, int *info) nogil
 * cdef void sgebrd(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgebrd(m, n, a, lda, d, e, tauq, taup, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgebrd(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tauq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_taup, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5027
 *     void _fortran_sgebrd "F_FUNC(sgebrd,SGEBRD)"(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *lwork, int *info) nogil
 * cdef void sgebrd(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgebrd(m, n, a, lda, d, e, tauq, taup, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgebrd,SGEBRD)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tauq, __pyx_v_taup, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgebrd "F_FUNC(sgebrd,SGEBRD)"(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *lwork, int *info) nogil
 * cdef void sgebrd(int *m, int *n, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgebrd(m, n, a, lda, d, e, tauq, taup, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgecon "F_FUNC(sgecon,SGECON)"(char *norm, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgecon(char *norm, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgecon(norm, n, a, lda, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgecon(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5032
 *     void _fortran_sgecon "F_FUNC(sgecon,SGECON)"(char *norm, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgecon(char *norm, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgecon(norm, n, a, lda, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgecon,SGECON)(__pyx_v_norm, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgecon "F_FUNC(sgecon,SGECON)"(char *norm, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgecon(char *norm, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgecon(norm, n, a, lda, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeequ "F_FUNC(sgeequ,SGEEQU)"(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void sgeequ(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeequ(m, n, a, lda, r, c, rowcnd, colcnd, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeequ(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5037
 *     void _fortran_sgeequ "F_FUNC(sgeequ,SGEEQU)"(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void sgeequ(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:
 *     _fortran_sgeequ(m, n, a, lda, r, c, rowcnd, colcnd, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeequ,SGEEQU)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeequ "F_FUNC(sgeequ,SGEEQU)"(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil
 * cdef void sgeequ(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeequ(m, n, a, lda, r, c, rowcnd, colcnd, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgees "F_FUNC(sgees,SGEES)"(char *jobvs, char *sort, _sselect2 *select, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void sgees(char *jobvs, char *sort, sselect2 *select, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *work, int *lwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgees(jobvs, sort, <_sselect2*>select, n, a, lda, sdim, wr, wi, vs, ldvs, work, lwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgees(char *__pyx_v_jobvs, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_sselect2 *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_sdim, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vs, int *__pyx_v_ldvs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5042
 *     void _fortran_sgees "F_FUNC(sgees,SGEES)"(char *jobvs, char *sort, _sselect2 *select, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void sgees(char *jobvs, char *sort, sselect2 *select, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *work, int *lwork, bint *bwork, int *info) nogil:
 *     _fortran_sgees(jobvs, sort, <_sselect2*>select, n, a, lda, sdim, wr, wi, vs, ldvs, work, lwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgees,SGEES)(__pyx_v_jobvs, __pyx_v_sort, ((_sselect2 *)__pyx_v_select), __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_sdim, __pyx_v_wr, __pyx_v_wi, __pyx_v_vs, __pyx_v_ldvs, __pyx_v_work, __pyx_v_lwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgees "F_FUNC(sgees,SGEES)"(char *jobvs, char *sort, _sselect2 *select, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void sgees(char *jobvs, char *sort, sselect2 *select, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *work, int *lwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgees(jobvs, sort, <_sselect2*>select, n, a, lda, sdim, wr, wi, vs, ldvs, work, lwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeesx "F_FUNC(sgeesx,SGEESX)"(char *jobvs, char *sort, _sselect2 *select, char *sense, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void sgeesx(char *jobvs, char *sort, sselect2 *select, char *sense, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeesx(jobvs, sort, <_sselect2*>select, sense, n, a, lda, sdim, wr, wi, vs, ldvs, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeesx(char *__pyx_v_jobvs, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_sselect2 *__pyx_v_select, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_sdim, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vs, int *__pyx_v_ldvs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcondv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5047
 *     void _fortran_sgeesx "F_FUNC(sgeesx,SGEESX)"(char *jobvs, char *sort, _sselect2 *select, char *sense, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void sgeesx(char *jobvs, char *sort, sselect2 *select, char *sense, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:
 *     _fortran_sgeesx(jobvs, sort, <_sselect2*>select, sense, n, a, lda, sdim, wr, wi, vs, ldvs, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeesx,SGEESX)(__pyx_v_jobvs, __pyx_v_sort, ((_sselect2 *)__pyx_v_select), __pyx_v_sense, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_sdim, __pyx_v_wr, __pyx_v_wi, __pyx_v_vs, __pyx_v_ldvs, __pyx_v_rconde, __pyx_v_rcondv, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeesx "F_FUNC(sgeesx,SGEESX)"(char *jobvs, char *sort, _sselect2 *select, char *sense, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void sgeesx(char *jobvs, char *sort, sselect2 *select, char *sense, int *n, s *a, int *lda, int *sdim, s *wr, s *wi, s *vs, int *ldvs, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeesx(jobvs, sort, <_sselect2*>select, sense, n, a, lda, sdim, wr, wi, vs, ldvs, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeev "F_FUNC(sgeev,SGEEV)"(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil
 * cdef void sgeev(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeev(jobvl, jobvr, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeev(char *__pyx_v_jobvl, char *__pyx_v_jobvr, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5052
 *     void _fortran_sgeev "F_FUNC(sgeev,SGEEV)"(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil
 * cdef void sgeev(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgeev(jobvl, jobvr, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeev,SGEEV)(__pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_wr, __pyx_v_wi, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeev "F_FUNC(sgeev,SGEEV)"(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil
 * cdef void sgeev(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeev(jobvl, jobvr, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeevx "F_FUNC(sgeevx,SGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeevx(balanc, jobvl, jobvr, sense, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeevx(char *__pyx_v_balanc, char *__pyx_v_jobvl, char *__pyx_v_jobvr, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abnrm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcondv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5057
 *     void _fortran_sgeevx "F_FUNC(sgeevx,SGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_sgeevx(balanc, jobvl, jobvr, sense, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeevx,SGEEVX)(__pyx_v_balanc, __pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_sense, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_wr, __pyx_v_wi, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_abnrm, __pyx_v_rconde, __pyx_v_rcondv, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeevx "F_FUNC(sgeevx,SGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *scale, s *abnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeevx(balanc, jobvl, jobvr, sense, n, a, lda, wr, wi, vl, ldvl, vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgehd2 "F_FUNC(sgehd2,SGEHD2)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgehd2(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgehd2(n, ilo, ihi, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgehd2(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5062
 *     void _fortran_sgehd2 "F_FUNC(sgehd2,SGEHD2)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgehd2(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sgehd2(n, ilo, ihi, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgehd2,SGEHD2)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgehd2 "F_FUNC(sgehd2,SGEHD2)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgehd2(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgehd2(n, ilo, ihi, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgehrd "F_FUNC(sgehrd,SGEHRD)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgehrd(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgehrd(n, ilo, ihi, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgehrd(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5067
 *     void _fortran_sgehrd "F_FUNC(sgehrd,SGEHRD)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgehrd(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgehrd(n, ilo, ihi, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgehrd,SGEHRD)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgehrd "F_FUNC(sgehrd,SGEHRD)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgehrd(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgehrd(n, ilo, ihi, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelq2 "F_FUNC(sgelq2,SGELQ2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgelq2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelq2(m, n, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgelq2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5072
 *     void _fortran_sgelq2 "F_FUNC(sgelq2,SGELQ2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgelq2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sgelq2(m, n, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgelq2,SGELQ2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelq2 "F_FUNC(sgelq2,SGELQ2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgelq2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelq2(m, n, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelqf "F_FUNC(sgelqf,SGELQF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgelqf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelqf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgelqf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5077
 *     void _fortran_sgelqf "F_FUNC(sgelqf,SGELQF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgelqf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgelqf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgelqf,SGELQF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelqf "F_FUNC(sgelqf,SGELQF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgelqf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelqf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgels "F_FUNC(sgels,SGELS)"(char *trans, int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *work, int *lwork, int *info) nogil
 * cdef void sgels(char *trans, int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgels(trans, m, n, nrhs, a, lda, b, ldb, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgels(char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5082
 *     void _fortran_sgels "F_FUNC(sgels,SGELS)"(char *trans, int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *work, int *lwork, int *info) nogil
 * cdef void sgels(char *trans, int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgels(trans, m, n, nrhs, a, lda, b, ldb, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgels,SGELS)(__pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgels "F_FUNC(sgels,SGELS)"(char *trans, int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *work, int *lwork, int *info) nogil
 * cdef void sgels(char *trans, int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgels(trans, m, n, nrhs, a, lda, b, ldb, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelsd "F_FUNC(sgelsd,SGELSD)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgelsd(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelsd(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgelsd(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5087
 *     void _fortran_sgelsd "F_FUNC(sgelsd,SGELSD)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgelsd(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_sgelsd(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgelsd,SGELSD)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_s, __pyx_v_rcond, __pyx_v_rank, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelsd "F_FUNC(sgelsd,SGELSD)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgelsd(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelsd(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelss "F_FUNC(sgelss,SGELSS)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *info) nogil
 * cdef void sgelss(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelss(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgelss(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5092
 *     void _fortran_sgelss "F_FUNC(sgelss,SGELSS)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *info) nogil
 * cdef void sgelss(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgelss(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgelss,SGELSS)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_s, __pyx_v_rcond, __pyx_v_rank, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelss "F_FUNC(sgelss,SGELSS)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *info) nogil
 * cdef void sgelss(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *s, s *rcond, int *rank, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelss(m, n, nrhs, a, lda, b, ldb, s, rcond, rank, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelsy "F_FUNC(sgelsy,SGELSY)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *jpvt, s *rcond, int *rank, s *work, int *lwork, int *info) nogil
 * cdef void sgelsy(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *jpvt, s *rcond, int *rank, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelsy(m, n, nrhs, a, lda, b, ldb, jpvt, rcond, rank, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgelsy(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5097
 *     void _fortran_sgelsy "F_FUNC(sgelsy,SGELSY)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *jpvt, s *rcond, int *rank, s *work, int *lwork, int *info) nogil
 * cdef void sgelsy(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *jpvt, s *rcond, int *rank, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgelsy(m, n, nrhs, a, lda, b, ldb, jpvt, rcond, rank, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgelsy,SGELSY)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_jpvt, __pyx_v_rcond, __pyx_v_rank, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgelsy "F_FUNC(sgelsy,SGELSY)"(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *jpvt, s *rcond, int *rank, s *work, int *lwork, int *info) nogil
 * cdef void sgelsy(int *m, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *jpvt, s *rcond, int *rank, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgelsy(m, n, nrhs, a, lda, b, ldb, jpvt, rcond, rank, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeql2 "F_FUNC(sgeql2,SGEQL2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgeql2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeql2(m, n, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeql2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5102
 *     void _fortran_sgeql2 "F_FUNC(sgeql2,SGEQL2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgeql2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sgeql2(m, n, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeql2,SGEQL2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeql2 "F_FUNC(sgeql2,SGEQL2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgeql2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeql2(m, n, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeqlf "F_FUNC(sgeqlf,SGEQLF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqlf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeqlf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeqlf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5107
 *     void _fortran_sgeqlf "F_FUNC(sgeqlf,SGEQLF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqlf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgeqlf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeqlf,SGEQLF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeqlf "F_FUNC(sgeqlf,SGEQLF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqlf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeqlf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeqp3 "F_FUNC(sgeqp3,SGEQP3)"(int *m, int *n, s *a, int *lda, int *jpvt, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqp3(int *m, int *n, s *a, int *lda, int *jpvt, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeqp3(m, n, a, lda, jpvt, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeqp3(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5112
 *     void _fortran_sgeqp3 "F_FUNC(sgeqp3,SGEQP3)"(int *m, int *n, s *a, int *lda, int *jpvt, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqp3(int *m, int *n, s *a, int *lda, int *jpvt, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgeqp3(m, n, a, lda, jpvt, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeqp3,SGEQP3)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_jpvt, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeqp3 "F_FUNC(sgeqp3,SGEQP3)"(int *m, int *n, s *a, int *lda, int *jpvt, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqp3(int *m, int *n, s *a, int *lda, int *jpvt, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeqp3(m, n, a, lda, jpvt, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeqr2 "F_FUNC(sgeqr2,SGEQR2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgeqr2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeqr2(m, n, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeqr2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5117
 *     void _fortran_sgeqr2 "F_FUNC(sgeqr2,SGEQR2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgeqr2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sgeqr2(m, n, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeqr2,SGEQR2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeqr2 "F_FUNC(sgeqr2,SGEQR2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgeqr2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeqr2(m, n, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeqrf "F_FUNC(sgeqrf,SGEQRF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqrf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeqrf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgeqrf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5122
 *     void _fortran_sgeqrf "F_FUNC(sgeqrf,SGEQRF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqrf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgeqrf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgeqrf,SGEQRF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgeqrf "F_FUNC(sgeqrf,SGEQRF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgeqrf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgeqrf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgerfs "F_FUNC(sgerfs,SGERFS)"(char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgerfs(char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgerfs(trans, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgerfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5127
 *     void _fortran_sgerfs "F_FUNC(sgerfs,SGERFS)"(char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgerfs(char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgerfs(trans, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgerfs,SGERFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgerfs "F_FUNC(sgerfs,SGERFS)"(char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgerfs(char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgerfs(trans, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgerq2 "F_FUNC(sgerq2,SGERQ2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgerq2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgerq2(m, n, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgerq2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5132
 *     void _fortran_sgerq2 "F_FUNC(sgerq2,SGERQ2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgerq2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sgerq2(m, n, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgerq2,SGERQ2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgerq2 "F_FUNC(sgerq2,SGERQ2)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sgerq2(int *m, int *n, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgerq2(m, n, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgerqf "F_FUNC(sgerqf,SGERQF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgerqf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgerqf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgerqf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5137
 *     void _fortran_sgerqf "F_FUNC(sgerqf,SGERQF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgerqf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgerqf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgerqf,SGERQF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgerqf "F_FUNC(sgerqf,SGERQF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sgerqf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgerqf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesc2 "F_FUNC(sgesc2,SGESC2)"(int *n, s *a, int *lda, s *rhs, int *ipiv, int *jpiv, s *scale) nogil
 * cdef void sgesc2(int *n, s *a, int *lda, s *rhs, int *ipiv, int *jpiv, s *scale) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesc2(n, a, lda, rhs, ipiv, jpiv, scale)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgesc2(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rhs, int *__pyx_v_ipiv, int *__pyx_v_jpiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale) {

  /* "scipy/linalg/cython_lapack.pyx":5142
 *     void _fortran_sgesc2 "F_FUNC(sgesc2,SGESC2)"(int *n, s *a, int *lda, s *rhs, int *ipiv, int *jpiv, s *scale) nogil
 * cdef void sgesc2(int *n, s *a, int *lda, s *rhs, int *ipiv, int *jpiv, s *scale) nogil:
 *     _fortran_sgesc2(n, a, lda, rhs, ipiv, jpiv, scale)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgesc2,SGESC2)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_rhs, __pyx_v_ipiv, __pyx_v_jpiv, __pyx_v_scale);

  /* "scipy/linalg/cython_lapack.pyx":5141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesc2 "F_FUNC(sgesc2,SGESC2)"(int *n, s *a, int *lda, s *rhs, int *ipiv, int *jpiv, s *scale) nogil
 * cdef void sgesc2(int *n, s *a, int *lda, s *rhs, int *ipiv, int *jpiv, s *scale) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesc2(n, a, lda, rhs, ipiv, jpiv, scale)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesdd "F_FUNC(sgesdd,SGESDD)"(char *jobz, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgesdd(char *jobz, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesdd(jobz, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgesdd(char *__pyx_v_jobz, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5147
 *     void _fortran_sgesdd "F_FUNC(sgesdd,SGESDD)"(char *jobz, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgesdd(char *jobz, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_sgesdd(jobz, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgesdd,SGESDD)(__pyx_v_jobz, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_s, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesdd "F_FUNC(sgesdd,SGESDD)"(char *jobz, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void sgesdd(char *jobz, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesdd(jobz, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesv "F_FUNC(sgesv,SGESV)"(int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgesv(int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesv(n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgesv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5152
 *     void _fortran_sgesv "F_FUNC(sgesv,SGESV)"(int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgesv(int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:
 *     _fortran_sgesv(n, nrhs, a, lda, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgesv,SGESV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesv "F_FUNC(sgesv,SGESV)"(int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgesv(int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesv(n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesvd "F_FUNC(sgesvd,SGESVD)"(char *jobu, char *jobvt, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *info) nogil
 * cdef void sgesvd(char *jobu, char *jobvt, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesvd(jobu, jobvt, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgesvd(char *__pyx_v_jobu, char *__pyx_v_jobvt, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5157
 *     void _fortran_sgesvd "F_FUNC(sgesvd,SGESVD)"(char *jobu, char *jobvt, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *info) nogil
 * cdef void sgesvd(char *jobu, char *jobvt, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgesvd(jobu, jobvt, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgesvd,SGESVD)(__pyx_v_jobu, __pyx_v_jobvt, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_s, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesvd "F_FUNC(sgesvd,SGESVD)"(char *jobu, char *jobvt, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *info) nogil
 * cdef void sgesvd(char *jobu, char *jobvt, int *m, int *n, s *a, int *lda, s *s, s *u, int *ldu, s *vt, int *ldvt, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesvd(jobu, jobvt, m, n, a, lda, s, u, ldu, vt, ldvt, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesvx "F_FUNC(sgesvx,SGESVX)"(char *fact, char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgesvx(char *fact, char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesvx(fact, trans, n, nrhs, a, lda, af, ldaf, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgesvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5162
 *     void _fortran_sgesvx "F_FUNC(sgesvx,SGESVX)"(char *fact, char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgesvx(char *fact, char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgesvx(fact, trans, n, nrhs, a, lda, af, ldaf, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgesvx,SGESVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_equed, __pyx_v_r, __pyx_v_c, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgesvx "F_FUNC(sgesvx,SGESVX)"(char *fact, char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgesvx(char *fact, char *trans, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, char *equed, s *r, s *c, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgesvx(fact, trans, n, nrhs, a, lda, af, ldaf, ipiv, equed, r, c, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetc2 "F_FUNC(sgetc2,SGETC2)"(int *n, s *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void sgetc2(int *n, s *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetc2(n, a, lda, ipiv, jpiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgetc2(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_jpiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5167
 *     void _fortran_sgetc2 "F_FUNC(sgetc2,SGETC2)"(int *n, s *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void sgetc2(int *n, s *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:
 *     _fortran_sgetc2(n, a, lda, ipiv, jpiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgetc2,SGETC2)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_jpiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetc2 "F_FUNC(sgetc2,SGETC2)"(int *n, s *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void sgetc2(int *n, s *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetc2(n, a, lda, ipiv, jpiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetf2 "F_FUNC(sgetf2,SGETF2)"(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void sgetf2(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetf2(m, n, a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgetf2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5172
 *     void _fortran_sgetf2 "F_FUNC(sgetf2,SGETF2)"(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void sgetf2(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_sgetf2(m, n, a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgetf2,SGETF2)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetf2 "F_FUNC(sgetf2,SGETF2)"(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void sgetf2(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetf2(m, n, a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetrf "F_FUNC(sgetrf,SGETRF)"(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void sgetrf(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetrf(m, n, a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgetrf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5177
 *     void _fortran_sgetrf "F_FUNC(sgetrf,SGETRF)"(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void sgetrf(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_sgetrf(m, n, a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgetrf,SGETRF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetrf "F_FUNC(sgetrf,SGETRF)"(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void sgetrf(int *m, int *n, s *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetrf(m, n, a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetri "F_FUNC(sgetri,SGETRI)"(int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil
 * cdef void sgetri(int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetri(n, a, lda, ipiv, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgetri(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5182
 *     void _fortran_sgetri "F_FUNC(sgetri,SGETRI)"(int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil
 * cdef void sgetri(int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgetri(n, a, lda, ipiv, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgetri,SGETRI)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetri "F_FUNC(sgetri,SGETRI)"(int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil
 * cdef void sgetri(int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetri(n, a, lda, ipiv, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetrs "F_FUNC(sgetrs,SGETRS)"(char *trans, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgetrs(char *trans, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetrs(trans, n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgetrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5187
 *     void _fortran_sgetrs "F_FUNC(sgetrs,SGETRS)"(char *trans, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgetrs(char *trans, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:
 *     _fortran_sgetrs(trans, n, nrhs, a, lda, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgetrs,SGETRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgetrs "F_FUNC(sgetrs,SGETRS)"(char *trans, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgetrs(char *trans, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgetrs(trans, n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggbak "F_FUNC(sggbak,SGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, s *v, int *ldv, int *info) nogil
 * cdef void sggbak(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, s *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggbak(job, side, n, ilo, ihi, lscale, rscale, m, v, ldv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sggbak(char *__pyx_v_job, char *__pyx_v_side, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rscale, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5192
 *     void _fortran_sggbak "F_FUNC(sggbak,SGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, s *v, int *ldv, int *info) nogil
 * cdef void sggbak(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, s *v, int *ldv, int *info) nogil:
 *     _fortran_sggbak(job, side, n, ilo, ihi, lscale, rscale, m, v, ldv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sggbak,SGGBAK)(__pyx_v_job, __pyx_v_side, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_m, __pyx_v_v, __pyx_v_ldv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggbak "F_FUNC(sggbak,SGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, s *v, int *ldv, int *info) nogil
 * cdef void sggbak(char *job, char *side, int *n, int *ilo, int *ihi, s *lscale, s *rscale, int *m, s *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggbak(job, side, n, ilo, ihi, lscale, rscale, m, v, ldv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggbal "F_FUNC(sggbal,SGGBAL)"(char *job, int *n, s *a, int *lda, s *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil
 * cdef void sggbal(char *job, int *n, s *a, int *lda, s *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggbal(job, n, a, lda, b, ldb, ilo, ihi, lscale, rscale, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sggbal(char *__pyx_v_job, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5197
 *     void _fortran_sggbal "F_FUNC(sggbal,SGGBAL)"(char *job, int *n, s *a, int *lda, s *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil
 * cdef void sggbal(char *job, int *n, s *a, int *lda, s *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil:
 *     _fortran_sggbal(job, n, a, lda, b, ldb, ilo, ihi, lscale, rscale, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sggbal,SGGBAL)(__pyx_v_job, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggbal "F_FUNC(sggbal,SGGBAL)"(char *job, int *n, s *a, int *lda, s *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil
 * cdef void sggbal(char *job, int *n, s *a, int *lda, s *b, int *ldb, int *ilo, int *ihi, s *lscale, s *rscale, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggbal(job, n, a, lda, b, ldb, ilo, ihi, lscale, rscale, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgges "F_FUNC(sgges,SGGES)"(char *jobvsl, char *jobvsr, char *sort, _sselect3 *selctg, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void sgges(char *jobvsl, char *jobvsr, char *sort, sselect3 *selctg, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *work, int *lwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgges(jobvsl, jobvsr, sort, <_sselect3*>selctg, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, work, lwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgges(char *__pyx_v_jobvsl, char *__pyx_v_jobvsr, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_sselect3 *__pyx_v_selctg, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_sdim, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vsl, int *__pyx_v_ldvsl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vsr, int *__pyx_v_ldvsr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5202
 *     void _fortran_sgges "F_FUNC(sgges,SGGES)"(char *jobvsl, char *jobvsr, char *sort, _sselect3 *selctg, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void sgges(char *jobvsl, char *jobvsr, char *sort, sselect3 *selctg, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *work, int *lwork, bint *bwork, int *info) nogil:
 *     _fortran_sgges(jobvsl, jobvsr, sort, <_sselect3*>selctg, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, work, lwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgges,SGGES)(__pyx_v_jobvsl, __pyx_v_jobvsr, __pyx_v_sort, ((_sselect3 *)__pyx_v_selctg), __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_sdim, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_vsl, __pyx_v_ldvsl, __pyx_v_vsr, __pyx_v_ldvsr, __pyx_v_work, __pyx_v_lwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgges "F_FUNC(sgges,SGGES)"(char *jobvsl, char *jobvsr, char *sort, _sselect3 *selctg, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *work, int *lwork, bint *bwork, int *info) nogil
 * cdef void sgges(char *jobvsl, char *jobvsr, char *sort, sselect3 *selctg, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *work, int *lwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgges(jobvsl, jobvsr, sort, <_sselect3*>selctg, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, work, lwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggesx "F_FUNC(sggesx,SGGESX)"(char *jobvsl, char *jobvsr, char *sort, _sselect3 *selctg, char *sense, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void sggesx(char *jobvsl, char *jobvsr, char *sort, sselect3 *selctg, char *sense, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggesx(jobvsl, jobvsr, sort, <_sselect3*>selctg, sense, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sggesx(char *__pyx_v_jobvsl, char *__pyx_v_jobvsr, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_sselect3 *__pyx_v_selctg, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_sdim, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vsl, int *__pyx_v_ldvsl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vsr, int *__pyx_v_ldvsr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcondv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5207
 *     void _fortran_sggesx "F_FUNC(sggesx,SGGESX)"(char *jobvsl, char *jobvsr, char *sort, _sselect3 *selctg, char *sense, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void sggesx(char *jobvsl, char *jobvsr, char *sort, sselect3 *selctg, char *sense, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:
 *     _fortran_sggesx(jobvsl, jobvsr, sort, <_sselect3*>selctg, sense, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sggesx,SGGESX)(__pyx_v_jobvsl, __pyx_v_jobvsr, __pyx_v_sort, ((_sselect3 *)__pyx_v_selctg), __pyx_v_sense, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_sdim, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_vsl, __pyx_v_ldvsl, __pyx_v_vsr, __pyx_v_ldvsr, __pyx_v_rconde, __pyx_v_rcondv, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggesx "F_FUNC(sggesx,SGGESX)"(char *jobvsl, char *jobvsr, char *sort, _sselect3 *selctg, char *sense, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void sggesx(char *jobvsl, char *jobvsr, char *sort, sselect3 *selctg, char *sense, int *n, s *a, int *lda, s *b, int *ldb, int *sdim, s *alphar, s *alphai, s *beta, s *vsl, int *ldvsl, s *vsr, int *ldvsr, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggesx(jobvsl, jobvsr, sort, <_sselect3*>selctg, sense, n, a, lda, b, ldb, sdim, alphar, alphai, beta, vsl, ldvsl, vsr, ldvsr, rconde, rcondv, work, lwork, iwork, liwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggev "F_FUNC(sggev,SGGEV)"(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil
 * cdef void sggev(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggev(jobvl, jobvr, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sggev(char *__pyx_v_jobvl, char *__pyx_v_jobvr, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5212
 *     void _fortran_sggev "F_FUNC(sggev,SGGEV)"(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil
 * cdef void sggev(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil:
 *     _fortran_sggev(jobvl, jobvr, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sggev,SGGEV)(__pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggev "F_FUNC(sggev,SGGEV)"(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil
 * cdef void sggev(char *jobvl, char *jobvr, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggev(jobvl, jobvr, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggevx "F_FUNC(sggevx,SGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void sggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggevx(balanc, jobvl, jobvr, sense, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, work, lwork, iwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sggevx(char *__pyx_v_balanc, char *__pyx_v_jobvl, char *__pyx_v_jobvr, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rscale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abnrm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bbnrm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcondv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5217
 *     void _fortran_sggevx "F_FUNC(sggevx,SGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void sggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, bint *bwork, int *info) nogil:
 *     _fortran_sggevx(balanc, jobvl, jobvr, sense, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, work, lwork, iwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sggevx,SGGEVX)(__pyx_v_balanc, __pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_sense, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_abnrm, __pyx_v_bbnrm, __pyx_v_rconde, __pyx_v_rcondv, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggevx "F_FUNC(sggevx,SGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void sggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *vl, int *ldvl, s *vr, int *ldvr, int *ilo, int *ihi, s *lscale, s *rscale, s *abnrm, s *bbnrm, s *rconde, s *rcondv, s *work, int *lwork, int *iwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggevx(balanc, jobvl, jobvr, sense, n, a, lda, b, ldb, alphar, alphai, beta, vl, ldvl, vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, work, lwork, iwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggglm "F_FUNC(sggglm,SGGGLM)"(int *n, int *m, int *p, s *a, int *lda, s *b, int *ldb, s *d, s *x, s *y, s *work, int *lwork, int *info) nogil
 * cdef void sggglm(int *n, int *m, int *p, s *a, int *lda, s *b, int *ldb, s *d, s *x, s *y, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggglm(n, m, p, a, lda, b, ldb, d, x, y, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sggglm(int *__pyx_v_n, int *__pyx_v_m, int *__pyx_v_p, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5222
 *     void _fortran_sggglm "F_FUNC(sggglm,SGGGLM)"(int *n, int *m, int *p, s *a, int *lda, s *b, int *ldb, s *d, s *x, s *y, s *work, int *lwork, int *info) nogil
 * cdef void sggglm(int *n, int *m, int *p, s *a, int *lda, s *b, int *ldb, s *d, s *x, s *y, s *work, int *lwork, int *info) nogil:
 *     _fortran_sggglm(n, m, p, a, lda, b, ldb, d, x, y, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sggglm,SGGGLM)(__pyx_v_n, __pyx_v_m, __pyx_v_p, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_d, __pyx_v_x, __pyx_v_y, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggglm "F_FUNC(sggglm,SGGGLM)"(int *n, int *m, int *p, s *a, int *lda, s *b, int *ldb, s *d, s *x, s *y, s *work, int *lwork, int *info) nogil
 * cdef void sggglm(int *n, int *m, int *p, s *a, int *lda, s *b, int *ldb, s *d, s *x, s *y, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggglm(n, m, p, a, lda, b, ldb, d, x, y, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgghrd "F_FUNC(sgghrd,SGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *info) nogil
 * cdef void sgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgghrd(compq, compz, n, ilo, ihi, a, lda, b, ldb, q, ldq, z, ldz, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgghrd(char *__pyx_v_compq, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5227
 *     void _fortran_sgghrd "F_FUNC(sgghrd,SGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *info) nogil
 * cdef void sgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *info) nogil:
 *     _fortran_sgghrd(compq, compz, n, ilo, ihi, a, lda, b, ldb, q, ldq, z, ldz, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgghrd,SGGHRD)(__pyx_v_compq, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgghrd "F_FUNC(sgghrd,SGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *info) nogil
 * cdef void sgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgghrd(compq, compz, n, ilo, ihi, a, lda, b, ldb, q, ldq, z, ldz, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgglse "F_FUNC(sgglse,SGGLSE)"(int *m, int *n, int *p, s *a, int *lda, s *b, int *ldb, s *c, s *d, s *x, s *work, int *lwork, int *info) nogil
 * cdef void sgglse(int *m, int *n, int *p, s *a, int *lda, s *b, int *ldb, s *c, s *d, s *x, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgglse(m, n, p, a, lda, b, ldb, c, d, x, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgglse(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_p, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5232
 *     void _fortran_sgglse "F_FUNC(sgglse,SGGLSE)"(int *m, int *n, int *p, s *a, int *lda, s *b, int *ldb, s *c, s *d, s *x, s *work, int *lwork, int *info) nogil
 * cdef void sgglse(int *m, int *n, int *p, s *a, int *lda, s *b, int *ldb, s *c, s *d, s *x, s *work, int *lwork, int *info) nogil:
 *     _fortran_sgglse(m, n, p, a, lda, b, ldb, c, d, x, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgglse,SGGLSE)(__pyx_v_m, __pyx_v_n, __pyx_v_p, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_c, __pyx_v_d, __pyx_v_x, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgglse "F_FUNC(sgglse,SGGLSE)"(int *m, int *n, int *p, s *a, int *lda, s *b, int *ldb, s *c, s *d, s *x, s *work, int *lwork, int *info) nogil
 * cdef void sgglse(int *m, int *n, int *p, s *a, int *lda, s *b, int *ldb, s *c, s *d, s *x, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgglse(m, n, p, a, lda, b, ldb, c, d, x, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggqrf "F_FUNC(sggqrf,SGGQRF)"(int *n, int *m, int *p, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil
 * cdef void sggqrf(int *n, int *m, int *p, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggqrf(n, m, p, a, lda, taua, b, ldb, taub, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sggqrf(int *__pyx_v_n, int *__pyx_v_m, int *__pyx_v_p, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_taua, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_taub, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5237
 *     void _fortran_sggqrf "F_FUNC(sggqrf,SGGQRF)"(int *n, int *m, int *p, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil
 * cdef void sggqrf(int *n, int *m, int *p, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil:
 *     _fortran_sggqrf(n, m, p, a, lda, taua, b, ldb, taub, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sggqrf,SGGQRF)(__pyx_v_n, __pyx_v_m, __pyx_v_p, __pyx_v_a, __pyx_v_lda, __pyx_v_taua, __pyx_v_b, __pyx_v_ldb, __pyx_v_taub, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggqrf "F_FUNC(sggqrf,SGGQRF)"(int *n, int *m, int *p, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil
 * cdef void sggqrf(int *n, int *m, int *p, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggqrf(n, m, p, a, lda, taua, b, ldb, taub, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggrqf "F_FUNC(sggrqf,SGGRQF)"(int *m, int *p, int *n, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil
 * cdef void sggrqf(int *m, int *p, int *n, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggrqf(m, p, n, a, lda, taua, b, ldb, taub, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sggrqf(int *__pyx_v_m, int *__pyx_v_p, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_taua, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_taub, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5242
 *     void _fortran_sggrqf "F_FUNC(sggrqf,SGGRQF)"(int *m, int *p, int *n, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil
 * cdef void sggrqf(int *m, int *p, int *n, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil:
 *     _fortran_sggrqf(m, p, n, a, lda, taua, b, ldb, taub, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sggrqf,SGGRQF)(__pyx_v_m, __pyx_v_p, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_taua, __pyx_v_b, __pyx_v_ldb, __pyx_v_taub, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sggrqf "F_FUNC(sggrqf,SGGRQF)"(int *m, int *p, int *n, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil
 * cdef void sggrqf(int *m, int *p, int *n, s *a, int *lda, s *taua, s *b, int *ldb, s *taub, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sggrqf(m, p, n, a, lda, taua, b, ldb, taub, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtcon "F_FUNC(sgtcon,SGTCON)"(char *norm, int *n, s *dl, s *d, s *du, s *du2, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgtcon(char *norm, int *n, s *dl, s *d, s *du, s *du2, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtcon(norm, n, dl, d, du, du2, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgtcon(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5247
 *     void _fortran_sgtcon "F_FUNC(sgtcon,SGTCON)"(char *norm, int *n, s *dl, s *d, s *du, s *du2, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgtcon(char *norm, int *n, s *dl, s *d, s *du, s *du2, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgtcon(norm, n, dl, d, du, du2, ipiv, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgtcon,SGTCON)(__pyx_v_norm, __pyx_v_n, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtcon "F_FUNC(sgtcon,SGTCON)"(char *norm, int *n, s *dl, s *d, s *du, s *du2, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sgtcon(char *norm, int *n, s *dl, s *d, s *du, s *du2, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtcon(norm, n, dl, d, du, du2, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtrfs "F_FUNC(sgtrfs,SGTRFS)"(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgtrfs(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtrfs(trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgtrfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_df, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_duf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5252
 *     void _fortran_sgtrfs "F_FUNC(sgtrfs,SGTRFS)"(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgtrfs(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgtrfs(trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgtrfs,SGTRFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_dlf, __pyx_v_df, __pyx_v_duf, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtrfs "F_FUNC(sgtrfs,SGTRFS)"(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgtrfs(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtrfs(trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtsv "F_FUNC(sgtsv,SGTSV)"(int *n, int *nrhs, s *dl, s *d, s *du, s *b, int *ldb, int *info) nogil
 * cdef void sgtsv(int *n, int *nrhs, s *dl, s *d, s *du, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtsv(n, nrhs, dl, d, du, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgtsv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5257
 *     void _fortran_sgtsv "F_FUNC(sgtsv,SGTSV)"(int *n, int *nrhs, s *dl, s *d, s *du, s *b, int *ldb, int *info) nogil
 * cdef void sgtsv(int *n, int *nrhs, s *dl, s *d, s *du, s *b, int *ldb, int *info) nogil:
 *     _fortran_sgtsv(n, nrhs, dl, d, du, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgtsv,SGTSV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtsv "F_FUNC(sgtsv,SGTSV)"(int *n, int *nrhs, s *dl, s *d, s *du, s *b, int *ldb, int *info) nogil
 * cdef void sgtsv(int *n, int *nrhs, s *dl, s *d, s *du, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtsv(n, nrhs, dl, d, du, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtsvx "F_FUNC(sgtsvx,SGTSVX)"(char *fact, char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgtsvx(char *fact, char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtsvx(fact, trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgtsvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_df, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_duf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5262
 *     void _fortran_sgtsvx "F_FUNC(sgtsvx,SGTSVX)"(char *fact, char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgtsvx(char *fact, char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sgtsvx(fact, trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgtsvx,SGTSVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_dlf, __pyx_v_df, __pyx_v_duf, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtsvx "F_FUNC(sgtsvx,SGTSVX)"(char *fact, char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sgtsvx(char *fact, char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *dlf, s *df, s *duf, s *du2, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtsvx(fact, trans, n, nrhs, dl, d, du, dlf, df, duf, du2, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgttrf "F_FUNC(sgttrf,SGTTRF)"(int *n, s *dl, s *d, s *du, s *du2, int *ipiv, int *info) nogil
 * cdef void sgttrf(int *n, s *dl, s *d, s *du, s *du2, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgttrf(n, dl, d, du, du2, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgttrf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du2, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5267
 *     void _fortran_sgttrf "F_FUNC(sgttrf,SGTTRF)"(int *n, s *dl, s *d, s *du, s *du2, int *ipiv, int *info) nogil
 * cdef void sgttrf(int *n, s *dl, s *d, s *du, s *du2, int *ipiv, int *info) nogil:
 *     _fortran_sgttrf(n, dl, d, du, du2, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgttrf,SGTTRF)(__pyx_v_n, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgttrf "F_FUNC(sgttrf,SGTTRF)"(int *n, s *dl, s *d, s *du, s *du2, int *ipiv, int *info) nogil
 * cdef void sgttrf(int *n, s *dl, s *d, s *du, s *du2, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgttrf(n, dl, d, du, du2, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgttrs "F_FUNC(sgttrs,SGTTRS)"(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgttrs(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgttrs(trans, n, nrhs, dl, d, du, du2, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgttrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5272
 *     void _fortran_sgttrs "F_FUNC(sgttrs,SGTTRS)"(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgttrs(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb, int *info) nogil:
 *     _fortran_sgttrs(trans, n, nrhs, dl, d, du, du2, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgttrs,SGTTRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgttrs "F_FUNC(sgttrs,SGTTRS)"(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sgttrs(char *trans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgttrs(trans, n, nrhs, dl, d, du, du2, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtts2 "F_FUNC(sgtts2,SGTTS2)"(int *itrans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb) nogil
 * cdef void sgtts2(int *itrans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtts2(itrans, n, nrhs, dl, d, du, du2, ipiv, b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sgtts2(int *__pyx_v_itrans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":5277
 *     void _fortran_sgtts2 "F_FUNC(sgtts2,SGTTS2)"(int *itrans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb) nogil
 * cdef void sgtts2(int *itrans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb) nogil:
 *     _fortran_sgtts2(itrans, n, nrhs, dl, d, du, du2, ipiv, b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sgtts2,SGTTS2)(__pyx_v_itrans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_du2, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":5276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sgtts2 "F_FUNC(sgtts2,SGTTS2)"(int *itrans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb) nogil
 * cdef void sgtts2(int *itrans, int *n, int *nrhs, s *dl, s *d, s *du, s *du2, int *ipiv, s *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sgtts2(itrans, n, nrhs, dl, d, du, du2, ipiv, b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_shgeqz "F_FUNC(shgeqz,SHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *t, int *ldt, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void shgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *t, int *ldt, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_shgeqz(job, compq, compz, n, ilo, ihi, h, ldh, t, ldt, alphar, alphai, beta, q, ldq, z, ldz, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_shgeqz(char *__pyx_v_job, char *__pyx_v_compq, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5282
 *     void _fortran_shgeqz "F_FUNC(shgeqz,SHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *t, int *ldt, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void shgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *t, int *ldt, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, s *work, int *lwork, int *info) nogil:
 *     _fortran_shgeqz(job, compq, compz, n, ilo, ihi, h, ldh, t, ldt, alphar, alphai, beta, q, ldq, z, ldz, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(shgeqz,SHGEQZ)(__pyx_v_job, __pyx_v_compq, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_t, __pyx_v_ldt, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_shgeqz "F_FUNC(shgeqz,SHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *t, int *ldt, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void shgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *t, int *ldt, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_shgeqz(job, compq, compz, n, ilo, ihi, h, ldh, t, ldt, alphar, alphai, beta, q, ldq, z, ldz, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_shsein "F_FUNC(shsein,SHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, s *h, int *ldh, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *ifaill, int *ifailr, int *info) nogil
 * cdef void shsein(char *side, char *eigsrc, char *initv, bint *select, int *n, s *h, int *ldh, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *ifaill, int *ifailr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_shsein(side, eigsrc, initv, select, n, h, ldh, wr, wi, vl, ldvl, vr, ldvr, mm, m, work, ifaill, ifailr, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_shsein(char *__pyx_v_side, char *__pyx_v_eigsrc, char *__pyx_v_initv, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_ifaill, int *__pyx_v_ifailr, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5287
 *     void _fortran_shsein "F_FUNC(shsein,SHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, s *h, int *ldh, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *ifaill, int *ifailr, int *info) nogil
 * cdef void shsein(char *side, char *eigsrc, char *initv, bint *select, int *n, s *h, int *ldh, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *ifaill, int *ifailr, int *info) nogil:
 *     _fortran_shsein(side, eigsrc, initv, select, n, h, ldh, wr, wi, vl, ldvl, vr, ldvr, mm, m, work, ifaill, ifailr, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(shsein,SHSEIN)(__pyx_v_side, __pyx_v_eigsrc, __pyx_v_initv, __pyx_v_select, __pyx_v_n, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_ifaill, __pyx_v_ifailr, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_shsein "F_FUNC(shsein,SHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, s *h, int *ldh, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *ifaill, int *ifailr, int *info) nogil
 * cdef void shsein(char *side, char *eigsrc, char *initv, bint *select, int *n, s *h, int *ldh, s *wr, s *wi, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *ifaill, int *ifailr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_shsein(side, eigsrc, initv, select, n, h, ldh, wr, wi, vl, ldvl, vr, ldvr, mm, m, work, ifaill, ifailr, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_shseqr "F_FUNC(shseqr,SHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void shseqr(char *job, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, s *z, int *ldz, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_shseqr(job, compz, n, ilo, ihi, h, ldh, wr, wi, z, ldz, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_shseqr(char *__pyx_v_job, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5292
 *     void _fortran_shseqr "F_FUNC(shseqr,SHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void shseqr(char *job, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, s *z, int *ldz, s *work, int *lwork, int *info) nogil:
 *     _fortran_shseqr(job, compz, n, ilo, ihi, h, ldh, wr, wi, z, ldz, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(shseqr,SHSEQR)(__pyx_v_job, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_shseqr "F_FUNC(shseqr,SHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void shseqr(char *job, char *compz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, s *z, int *ldz, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_shseqr(job, compz, n, ilo, ihi, h, ldh, wr, wi, z, ldz, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slabad "F_FUNC(slabad,SLABAD)"(s *small, s *large) nogil
 * cdef void slabad(s *small, s *large) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slabad(small, large)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slabad(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_small, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_large) {

  /* "scipy/linalg/cython_lapack.pyx":5297
 *     void _fortran_slabad "F_FUNC(slabad,SLABAD)"(s *small, s *large) nogil
 * cdef void slabad(s *small, s *large) nogil:
 *     _fortran_slabad(small, large)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slabad,SLABAD)(__pyx_v_small, __pyx_v_large);

  /* "scipy/linalg/cython_lapack.pyx":5296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slabad "F_FUNC(slabad,SLABAD)"(s *small, s *large) nogil
 * cdef void slabad(s *small, s *large) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slabad(small, large)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slabrd "F_FUNC(slabrd,SLABRD)"(int *m, int *n, int *nb, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *x, int *ldx, s *y, int *ldy) nogil
 * cdef void slabrd(int *m, int *n, int *nb, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *x, int *ldx, s *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slabrd(m, n, nb, a, lda, d, e, tauq, taup, x, ldx, y, ldy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slabrd(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tauq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_taup, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, int *__pyx_v_ldy) {

  /* "scipy/linalg/cython_lapack.pyx":5302
 *     void _fortran_slabrd "F_FUNC(slabrd,SLABRD)"(int *m, int *n, int *nb, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *x, int *ldx, s *y, int *ldy) nogil
 * cdef void slabrd(int *m, int *n, int *nb, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *x, int *ldx, s *y, int *ldy) nogil:
 *     _fortran_slabrd(m, n, nb, a, lda, d, e, tauq, taup, x, ldx, y, ldy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slabrd,SLABRD)(__pyx_v_m, __pyx_v_n, __pyx_v_nb, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tauq, __pyx_v_taup, __pyx_v_x, __pyx_v_ldx, __pyx_v_y, __pyx_v_ldy);

  /* "scipy/linalg/cython_lapack.pyx":5301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slabrd "F_FUNC(slabrd,SLABRD)"(int *m, int *n, int *nb, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *x, int *ldx, s *y, int *ldy) nogil
 * cdef void slabrd(int *m, int *n, int *nb, s *a, int *lda, s *d, s *e, s *tauq, s *taup, s *x, int *ldx, s *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slabrd(m, n, nb, a, lda, d, e, tauq, taup, x, ldx, y, ldy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slacn2 "F_FUNC(slacn2,SLACN2)"(int *n, s *v, s *x, int *isgn, s *est, int *kase, int *isave) nogil
 * cdef void slacn2(int *n, s *v, s *x, int *isgn, s *est, int *kase, int *isave) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slacn2(n, v, x, isgn, est, kase, isave)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slacn2(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_isgn, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_est, int *__pyx_v_kase, int *__pyx_v_isave) {

  /* "scipy/linalg/cython_lapack.pyx":5307
 *     void _fortran_slacn2 "F_FUNC(slacn2,SLACN2)"(int *n, s *v, s *x, int *isgn, s *est, int *kase, int *isave) nogil
 * cdef void slacn2(int *n, s *v, s *x, int *isgn, s *est, int *kase, int *isave) nogil:
 *     _fortran_slacn2(n, v, x, isgn, est, kase, isave)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slacn2,SLACN2)(__pyx_v_n, __pyx_v_v, __pyx_v_x, __pyx_v_isgn, __pyx_v_est, __pyx_v_kase, __pyx_v_isave);

  /* "scipy/linalg/cython_lapack.pyx":5306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slacn2 "F_FUNC(slacn2,SLACN2)"(int *n, s *v, s *x, int *isgn, s *est, int *kase, int *isave) nogil
 * cdef void slacn2(int *n, s *v, s *x, int *isgn, s *est, int *kase, int *isave) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slacn2(n, v, x, isgn, est, kase, isave)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slacon "F_FUNC(slacon,SLACON)"(int *n, s *v, s *x, int *isgn, s *est, int *kase) nogil
 * cdef void slacon(int *n, s *v, s *x, int *isgn, s *est, int *kase) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slacon(n, v, x, isgn, est, kase)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slacon(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_isgn, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_est, int *__pyx_v_kase) {

  /* "scipy/linalg/cython_lapack.pyx":5312
 *     void _fortran_slacon "F_FUNC(slacon,SLACON)"(int *n, s *v, s *x, int *isgn, s *est, int *kase) nogil
 * cdef void slacon(int *n, s *v, s *x, int *isgn, s *est, int *kase) nogil:
 *     _fortran_slacon(n, v, x, isgn, est, kase)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slacon,SLACON)(__pyx_v_n, __pyx_v_v, __pyx_v_x, __pyx_v_isgn, __pyx_v_est, __pyx_v_kase);

  /* "scipy/linalg/cython_lapack.pyx":5311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slacon "F_FUNC(slacon,SLACON)"(int *n, s *v, s *x, int *isgn, s *est, int *kase) nogil
 * cdef void slacon(int *n, s *v, s *x, int *isgn, s *est, int *kase) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slacon(n, v, x, isgn, est, kase)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slacpy "F_FUNC(slacpy,SLACPY)"(char *uplo, int *m, int *n, s *a, int *lda, s *b, int *ldb) nogil
 * cdef void slacpy(char *uplo, int *m, int *n, s *a, int *lda, s *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slacpy(uplo, m, n, a, lda, b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slacpy(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":5317
 *     void _fortran_slacpy "F_FUNC(slacpy,SLACPY)"(char *uplo, int *m, int *n, s *a, int *lda, s *b, int *ldb) nogil
 * cdef void slacpy(char *uplo, int *m, int *n, s *a, int *lda, s *b, int *ldb) nogil:
 *     _fortran_slacpy(uplo, m, n, a, lda, b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slacpy,SLACPY)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":5316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slacpy "F_FUNC(slacpy,SLACPY)"(char *uplo, int *m, int *n, s *a, int *lda, s *b, int *ldb) nogil
 * cdef void slacpy(char *uplo, int *m, int *n, s *a, int *lda, s *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slacpy(uplo, m, n, a, lda, b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sladiv "F_FUNC(sladiv,SLADIV)"(s *a, s *b, s *c, s *d, s *p, s *q) nogil
 * cdef void sladiv(s *a, s *b, s *c, s *d, s *p, s *q) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sladiv(a, b, c, d, p, q)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sladiv(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_p, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q) {

  /* "scipy/linalg/cython_lapack.pyx":5322
 *     void _fortran_sladiv "F_FUNC(sladiv,SLADIV)"(s *a, s *b, s *c, s *d, s *p, s *q) nogil
 * cdef void sladiv(s *a, s *b, s *c, s *d, s *p, s *q) nogil:
 *     _fortran_sladiv(a, b, c, d, p, q)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sladiv,SLADIV)(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_d, __pyx_v_p, __pyx_v_q);

  /* "scipy/linalg/cython_lapack.pyx":5321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sladiv "F_FUNC(sladiv,SLADIV)"(s *a, s *b, s *c, s *d, s *p, s *q) nogil
 * cdef void sladiv(s *a, s *b, s *c, s *d, s *p, s *q) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sladiv(a, b, c, d, p, q)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slae2 "F_FUNC(slae2,SLAE2)"(s *a, s *b, s *c, s *rt1, s *rt2) nogil
 * cdef void slae2(s *a, s *b, s *c, s *rt1, s *rt2) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slae2(a, b, c, rt1, rt2)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slae2(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt2) {

  /* "scipy/linalg/cython_lapack.pyx":5327
 *     void _fortran_slae2 "F_FUNC(slae2,SLAE2)"(s *a, s *b, s *c, s *rt1, s *rt2) nogil
 * cdef void slae2(s *a, s *b, s *c, s *rt1, s *rt2) nogil:
 *     _fortran_slae2(a, b, c, rt1, rt2)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slae2,SLAE2)(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_rt1, __pyx_v_rt2);

  /* "scipy/linalg/cython_lapack.pyx":5326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slae2 "F_FUNC(slae2,SLAE2)"(s *a, s *b, s *c, s *rt1, s *rt2) nogil
 * cdef void slae2(s *a, s *b, s *c, s *rt1, s *rt2) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slae2(a, b, c, rt1, rt2)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaebz "F_FUNC(slaebz,SLAEBZ)"(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, s *abstol, s *reltol, s *pivmin, s *d, s *e, s *e2, int *nval, s *ab, s *c, int *mout, int *nab, s *work, int *iwork, int *info) nogil
 * cdef void slaebz(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, s *abstol, s *reltol, s *pivmin, s *d, s *e, s *e2, int *nval, s *ab, s *c, int *mout, int *nab, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaebz(ijob, nitmax, n, mmax, minp, nbmin, abstol, reltol, pivmin, d, e, e2, nval, ab, c, mout, nab, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaebz(int *__pyx_v_ijob, int *__pyx_v_nitmax, int *__pyx_v_n, int *__pyx_v_mmax, int *__pyx_v_minp, int *__pyx_v_nbmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_reltol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e2, int *__pyx_v_nval, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_mout, int *__pyx_v_nab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5332
 *     void _fortran_slaebz "F_FUNC(slaebz,SLAEBZ)"(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, s *abstol, s *reltol, s *pivmin, s *d, s *e, s *e2, int *nval, s *ab, s *c, int *mout, int *nab, s *work, int *iwork, int *info) nogil
 * cdef void slaebz(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, s *abstol, s *reltol, s *pivmin, s *d, s *e, s *e2, int *nval, s *ab, s *c, int *mout, int *nab, s *work, int *iwork, int *info) nogil:
 *     _fortran_slaebz(ijob, nitmax, n, mmax, minp, nbmin, abstol, reltol, pivmin, d, e, e2, nval, ab, c, mout, nab, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaebz,SLAEBZ)(__pyx_v_ijob, __pyx_v_nitmax, __pyx_v_n, __pyx_v_mmax, __pyx_v_minp, __pyx_v_nbmin, __pyx_v_abstol, __pyx_v_reltol, __pyx_v_pivmin, __pyx_v_d, __pyx_v_e, __pyx_v_e2, __pyx_v_nval, __pyx_v_ab, __pyx_v_c, __pyx_v_mout, __pyx_v_nab, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaebz "F_FUNC(slaebz,SLAEBZ)"(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, s *abstol, s *reltol, s *pivmin, s *d, s *e, s *e2, int *nval, s *ab, s *c, int *mout, int *nab, s *work, int *iwork, int *info) nogil
 * cdef void slaebz(int *ijob, int *nitmax, int *n, int *mmax, int *minp, int *nbmin, s *abstol, s *reltol, s *pivmin, s *d, s *e, s *e2, int *nval, s *ab, s *c, int *mout, int *nab, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaebz(ijob, nitmax, n, mmax, minp, nbmin, abstol, reltol, pivmin, d, e, e2, nval, ab, c, mout, nab, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed0 "F_FUNC(slaed0,SLAED0)"(int *icompq, int *qsiz, int *n, s *d, s *e, s *q, int *ldq, s *qstore, int *ldqs, s *work, int *iwork, int *info) nogil
 * cdef void slaed0(int *icompq, int *qsiz, int *n, s *d, s *e, s *q, int *ldq, s *qstore, int *ldqs, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed0(icompq, qsiz, n, d, e, q, ldq, qstore, ldqs, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed0(int *__pyx_v_icompq, int *__pyx_v_qsiz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_qstore, int *__pyx_v_ldqs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5337
 *     void _fortran_slaed0 "F_FUNC(slaed0,SLAED0)"(int *icompq, int *qsiz, int *n, s *d, s *e, s *q, int *ldq, s *qstore, int *ldqs, s *work, int *iwork, int *info) nogil
 * cdef void slaed0(int *icompq, int *qsiz, int *n, s *d, s *e, s *q, int *ldq, s *qstore, int *ldqs, s *work, int *iwork, int *info) nogil:
 *     _fortran_slaed0(icompq, qsiz, n, d, e, q, ldq, qstore, ldqs, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed0,SLAED0)(__pyx_v_icompq, __pyx_v_qsiz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_q, __pyx_v_ldq, __pyx_v_qstore, __pyx_v_ldqs, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed0 "F_FUNC(slaed0,SLAED0)"(int *icompq, int *qsiz, int *n, s *d, s *e, s *q, int *ldq, s *qstore, int *ldqs, s *work, int *iwork, int *info) nogil
 * cdef void slaed0(int *icompq, int *qsiz, int *n, s *d, s *e, s *q, int *ldq, s *qstore, int *ldqs, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed0(icompq, qsiz, n, d, e, q, ldq, qstore, ldqs, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed1 "F_FUNC(slaed1,SLAED1)"(int *n, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *work, int *iwork, int *info) nogil
 * cdef void slaed1(int *n, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed1(n, d, q, ldq, indxq, rho, cutpnt, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed1(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, int *__pyx_v_cutpnt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5342
 *     void _fortran_slaed1 "F_FUNC(slaed1,SLAED1)"(int *n, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *work, int *iwork, int *info) nogil
 * cdef void slaed1(int *n, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *work, int *iwork, int *info) nogil:
 *     _fortran_slaed1(n, d, q, ldq, indxq, rho, cutpnt, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed1,SLAED1)(__pyx_v_n, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_indxq, __pyx_v_rho, __pyx_v_cutpnt, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed1 "F_FUNC(slaed1,SLAED1)"(int *n, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *work, int *iwork, int *info) nogil
 * cdef void slaed1(int *n, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed1(n, d, q, ldq, indxq, rho, cutpnt, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed2 "F_FUNC(slaed2,SLAED2)"(int *k, int *n, int *n1, s *d, s *q, int *ldq, int *indxq, s *rho, s *z, s *dlamda, s *w, s *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil
 * cdef void slaed2(int *k, int *n, int *n1, s *d, s *q, int *ldq, int *indxq, s *rho, s *z, s *dlamda, s *w, s *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed2(k, n, n1, d, q, ldq, indxq, rho, z, dlamda, w, q2, indx, indxc, indxp, coltyp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed2(int *__pyx_v_k, int *__pyx_v_n, int *__pyx_v_n1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlamda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q2, int *__pyx_v_indx, int *__pyx_v_indxc, int *__pyx_v_indxp, int *__pyx_v_coltyp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5347
 *     void _fortran_slaed2 "F_FUNC(slaed2,SLAED2)"(int *k, int *n, int *n1, s *d, s *q, int *ldq, int *indxq, s *rho, s *z, s *dlamda, s *w, s *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil
 * cdef void slaed2(int *k, int *n, int *n1, s *d, s *q, int *ldq, int *indxq, s *rho, s *z, s *dlamda, s *w, s *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil:
 *     _fortran_slaed2(k, n, n1, d, q, ldq, indxq, rho, z, dlamda, w, q2, indx, indxc, indxp, coltyp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed2,SLAED2)(__pyx_v_k, __pyx_v_n, __pyx_v_n1, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_indxq, __pyx_v_rho, __pyx_v_z, __pyx_v_dlamda, __pyx_v_w, __pyx_v_q2, __pyx_v_indx, __pyx_v_indxc, __pyx_v_indxp, __pyx_v_coltyp, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed2 "F_FUNC(slaed2,SLAED2)"(int *k, int *n, int *n1, s *d, s *q, int *ldq, int *indxq, s *rho, s *z, s *dlamda, s *w, s *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil
 * cdef void slaed2(int *k, int *n, int *n1, s *d, s *q, int *ldq, int *indxq, s *rho, s *z, s *dlamda, s *w, s *q2, int *indx, int *indxc, int *indxp, int *coltyp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed2(k, n, n1, d, q, ldq, indxq, rho, z, dlamda, w, q2, indx, indxc, indxp, coltyp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed3 "F_FUNC(slaed3,SLAED3)"(int *k, int *n, int *n1, s *d, s *q, int *ldq, s *rho, s *dlamda, s *q2, int *indx, int *ctot, s *w, s *s, int *info) nogil
 * cdef void slaed3(int *k, int *n, int *n1, s *d, s *q, int *ldq, s *rho, s *dlamda, s *q2, int *indx, int *ctot, s *w, s *s, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed3(k, n, n1, d, q, ldq, rho, dlamda, q2, indx, ctot, w, s, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed3(int *__pyx_v_k, int *__pyx_v_n, int *__pyx_v_n1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlamda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q2, int *__pyx_v_indx, int *__pyx_v_ctot, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5352
 *     void _fortran_slaed3 "F_FUNC(slaed3,SLAED3)"(int *k, int *n, int *n1, s *d, s *q, int *ldq, s *rho, s *dlamda, s *q2, int *indx, int *ctot, s *w, s *s, int *info) nogil
 * cdef void slaed3(int *k, int *n, int *n1, s *d, s *q, int *ldq, s *rho, s *dlamda, s *q2, int *indx, int *ctot, s *w, s *s, int *info) nogil:
 *     _fortran_slaed3(k, n, n1, d, q, ldq, rho, dlamda, q2, indx, ctot, w, s, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed3,SLAED3)(__pyx_v_k, __pyx_v_n, __pyx_v_n1, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_rho, __pyx_v_dlamda, __pyx_v_q2, __pyx_v_indx, __pyx_v_ctot, __pyx_v_w, __pyx_v_s, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed3 "F_FUNC(slaed3,SLAED3)"(int *k, int *n, int *n1, s *d, s *q, int *ldq, s *rho, s *dlamda, s *q2, int *indx, int *ctot, s *w, s *s, int *info) nogil
 * cdef void slaed3(int *k, int *n, int *n1, s *d, s *q, int *ldq, s *rho, s *dlamda, s *q2, int *indx, int *ctot, s *w, s *s, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed3(k, n, n1, d, q, ldq, rho, dlamda, q2, indx, ctot, w, s, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed4 "F_FUNC(slaed4,SLAED4)"(int *n, int *i, s *d, s *z, s *delta, s *rho, s *dlam, int *info) nogil
 * cdef void slaed4(int *n, int *i, s *d, s *z, s *delta, s *rho, s *dlam, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed4(n, i, d, z, delta, rho, dlam, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed4(int *__pyx_v_n, int *__pyx_v_i, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_delta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlam, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5357
 *     void _fortran_slaed4 "F_FUNC(slaed4,SLAED4)"(int *n, int *i, s *d, s *z, s *delta, s *rho, s *dlam, int *info) nogil
 * cdef void slaed4(int *n, int *i, s *d, s *z, s *delta, s *rho, s *dlam, int *info) nogil:
 *     _fortran_slaed4(n, i, d, z, delta, rho, dlam, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed4,SLAED4)(__pyx_v_n, __pyx_v_i, __pyx_v_d, __pyx_v_z, __pyx_v_delta, __pyx_v_rho, __pyx_v_dlam, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed4 "F_FUNC(slaed4,SLAED4)"(int *n, int *i, s *d, s *z, s *delta, s *rho, s *dlam, int *info) nogil
 * cdef void slaed4(int *n, int *i, s *d, s *z, s *delta, s *rho, s *dlam, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed4(n, i, d, z, delta, rho, dlam, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed5 "F_FUNC(slaed5,SLAED5)"(int *i, s *d, s *z, s *delta, s *rho, s *dlam) nogil
 * cdef void slaed5(int *i, s *d, s *z, s *delta, s *rho, s *dlam) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed5(i, d, z, delta, rho, dlam)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed5(int *__pyx_v_i, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_delta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlam) {

  /* "scipy/linalg/cython_lapack.pyx":5362
 *     void _fortran_slaed5 "F_FUNC(slaed5,SLAED5)"(int *i, s *d, s *z, s *delta, s *rho, s *dlam) nogil
 * cdef void slaed5(int *i, s *d, s *z, s *delta, s *rho, s *dlam) nogil:
 *     _fortran_slaed5(i, d, z, delta, rho, dlam)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed5,SLAED5)(__pyx_v_i, __pyx_v_d, __pyx_v_z, __pyx_v_delta, __pyx_v_rho, __pyx_v_dlam);

  /* "scipy/linalg/cython_lapack.pyx":5361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed5 "F_FUNC(slaed5,SLAED5)"(int *i, s *d, s *z, s *delta, s *rho, s *dlam) nogil
 * cdef void slaed5(int *i, s *d, s *z, s *delta, s *rho, s *dlam) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed5(i, d, z, delta, rho, dlam)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed6 "F_FUNC(slaed6,SLAED6)"(int *kniter, bint *orgati, s *rho, s *d, s *z, s *finit, s *tau, int *info) nogil
 * cdef void slaed6(int *kniter, bint *orgati, s *rho, s *d, s *z, s *finit, s *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed6(kniter, orgati, rho, d, z, finit, tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed6(int *__pyx_v_kniter, int *__pyx_v_orgati, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_finit, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5367
 *     void _fortran_slaed6 "F_FUNC(slaed6,SLAED6)"(int *kniter, bint *orgati, s *rho, s *d, s *z, s *finit, s *tau, int *info) nogil
 * cdef void slaed6(int *kniter, bint *orgati, s *rho, s *d, s *z, s *finit, s *tau, int *info) nogil:
 *     _fortran_slaed6(kniter, orgati, rho, d, z, finit, tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed6,SLAED6)(__pyx_v_kniter, __pyx_v_orgati, __pyx_v_rho, __pyx_v_d, __pyx_v_z, __pyx_v_finit, __pyx_v_tau, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed6 "F_FUNC(slaed6,SLAED6)"(int *kniter, bint *orgati, s *rho, s *d, s *z, s *finit, s *tau, int *info) nogil
 * cdef void slaed6(int *kniter, bint *orgati, s *rho, s *d, s *z, s *finit, s *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed6(kniter, orgati, rho, d, z, finit, tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed7 "F_FUNC(slaed7,SLAED7)"(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *work, int *iwork, int *info) nogil
 * cdef void slaed7(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed7(icompq, n, qsiz, tlvls, curlvl, curpbm, d, q, ldq, indxq, rho, cutpnt, qstore, qptr, prmptr, perm, givptr, givcol, givnum, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed7(int *__pyx_v_icompq, int *__pyx_v_n, int *__pyx_v_qsiz, int *__pyx_v_tlvls, int *__pyx_v_curlvl, int *__pyx_v_curpbm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, int *__pyx_v_cutpnt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_qstore, int *__pyx_v_qptr, int *__pyx_v_prmptr, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5372
 *     void _fortran_slaed7 "F_FUNC(slaed7,SLAED7)"(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *work, int *iwork, int *info) nogil
 * cdef void slaed7(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *work, int *iwork, int *info) nogil:
 *     _fortran_slaed7(icompq, n, qsiz, tlvls, curlvl, curpbm, d, q, ldq, indxq, rho, cutpnt, qstore, qptr, prmptr, perm, givptr, givcol, givnum, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed7,SLAED7)(__pyx_v_icompq, __pyx_v_n, __pyx_v_qsiz, __pyx_v_tlvls, __pyx_v_curlvl, __pyx_v_curpbm, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_indxq, __pyx_v_rho, __pyx_v_cutpnt, __pyx_v_qstore, __pyx_v_qptr, __pyx_v_prmptr, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed7 "F_FUNC(slaed7,SLAED7)"(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *work, int *iwork, int *info) nogil
 * cdef void slaed7(int *icompq, int *n, int *qsiz, int *tlvls, int *curlvl, int *curpbm, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed7(icompq, n, qsiz, tlvls, curlvl, curpbm, d, q, ldq, indxq, rho, cutpnt, qstore, qptr, prmptr, perm, givptr, givcol, givnum, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed8 "F_FUNC(slaed8,SLAED8)"(int *icompq, int *k, int *n, int *qsiz, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *z, s *dlamda, s *q2, int *ldq2, s *w, int *perm, int *givptr, int *givcol, s *givnum, int *indxp, int *indx, int *info) nogil
 * cdef void slaed8(int *icompq, int *k, int *n, int *qsiz, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *z, s *dlamda, s *q2, int *ldq2, s *w, int *perm, int *givptr, int *givcol, s *givnum, int *indxp, int *indx, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed8(icompq, k, n, qsiz, d, q, ldq, indxq, rho, cutpnt, z, dlamda, q2, ldq2, w, perm, givptr, givcol, givnum, indxp, indx, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed8(int *__pyx_v_icompq, int *__pyx_v_k, int *__pyx_v_n, int *__pyx_v_qsiz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, int *__pyx_v_cutpnt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlamda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q2, int *__pyx_v_ldq2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, int *__pyx_v_indxp, int *__pyx_v_indx, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5377
 *     void _fortran_slaed8 "F_FUNC(slaed8,SLAED8)"(int *icompq, int *k, int *n, int *qsiz, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *z, s *dlamda, s *q2, int *ldq2, s *w, int *perm, int *givptr, int *givcol, s *givnum, int *indxp, int *indx, int *info) nogil
 * cdef void slaed8(int *icompq, int *k, int *n, int *qsiz, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *z, s *dlamda, s *q2, int *ldq2, s *w, int *perm, int *givptr, int *givcol, s *givnum, int *indxp, int *indx, int *info) nogil:
 *     _fortran_slaed8(icompq, k, n, qsiz, d, q, ldq, indxq, rho, cutpnt, z, dlamda, q2, ldq2, w, perm, givptr, givcol, givnum, indxp, indx, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed8,SLAED8)(__pyx_v_icompq, __pyx_v_k, __pyx_v_n, __pyx_v_qsiz, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_indxq, __pyx_v_rho, __pyx_v_cutpnt, __pyx_v_z, __pyx_v_dlamda, __pyx_v_q2, __pyx_v_ldq2, __pyx_v_w, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, __pyx_v_indxp, __pyx_v_indx, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed8 "F_FUNC(slaed8,SLAED8)"(int *icompq, int *k, int *n, int *qsiz, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *z, s *dlamda, s *q2, int *ldq2, s *w, int *perm, int *givptr, int *givcol, s *givnum, int *indxp, int *indx, int *info) nogil
 * cdef void slaed8(int *icompq, int *k, int *n, int *qsiz, s *d, s *q, int *ldq, int *indxq, s *rho, int *cutpnt, s *z, s *dlamda, s *q2, int *ldq2, s *w, int *perm, int *givptr, int *givcol, s *givnum, int *indxp, int *indx, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed8(icompq, k, n, qsiz, d, q, ldq, indxq, rho, cutpnt, z, dlamda, q2, ldq2, w, perm, givptr, givcol, givnum, indxp, indx, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed9 "F_FUNC(slaed9,SLAED9)"(int *k, int *kstart, int *kstop, int *n, s *d, s *q, int *ldq, s *rho, s *dlamda, s *w, s *s, int *lds, int *info) nogil
 * cdef void slaed9(int *k, int *kstart, int *kstop, int *n, s *d, s *q, int *ldq, s *rho, s *dlamda, s *w, s *s, int *lds, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed9(k, kstart, kstop, n, d, q, ldq, rho, dlamda, w, s, lds, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaed9(int *__pyx_v_k, int *__pyx_v_kstart, int *__pyx_v_kstop, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dlamda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, int *__pyx_v_lds, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5382
 *     void _fortran_slaed9 "F_FUNC(slaed9,SLAED9)"(int *k, int *kstart, int *kstop, int *n, s *d, s *q, int *ldq, s *rho, s *dlamda, s *w, s *s, int *lds, int *info) nogil
 * cdef void slaed9(int *k, int *kstart, int *kstop, int *n, s *d, s *q, int *ldq, s *rho, s *dlamda, s *w, s *s, int *lds, int *info) nogil:
 *     _fortran_slaed9(k, kstart, kstop, n, d, q, ldq, rho, dlamda, w, s, lds, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaed9,SLAED9)(__pyx_v_k, __pyx_v_kstart, __pyx_v_kstop, __pyx_v_n, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_rho, __pyx_v_dlamda, __pyx_v_w, __pyx_v_s, __pyx_v_lds, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaed9 "F_FUNC(slaed9,SLAED9)"(int *k, int *kstart, int *kstop, int *n, s *d, s *q, int *ldq, s *rho, s *dlamda, s *w, s *s, int *lds, int *info) nogil
 * cdef void slaed9(int *k, int *kstart, int *kstop, int *n, s *d, s *q, int *ldq, s *rho, s *dlamda, s *w, s *s, int *lds, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaed9(k, kstart, kstop, n, d, q, ldq, rho, dlamda, w, s, lds, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaeda "F_FUNC(slaeda,SLAEDA)"(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *q, int *qptr, s *z, s *ztemp, int *info) nogil
 * cdef void slaeda(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *q, int *qptr, s *z, s *ztemp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaeda(n, tlvls, curlvl, curpbm, prmptr, perm, givptr, givcol, givnum, q, qptr, z, ztemp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaeda(int *__pyx_v_n, int *__pyx_v_tlvls, int *__pyx_v_curlvl, int *__pyx_v_curpbm, int *__pyx_v_prmptr, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_qptr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ztemp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5387
 *     void _fortran_slaeda "F_FUNC(slaeda,SLAEDA)"(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *q, int *qptr, s *z, s *ztemp, int *info) nogil
 * cdef void slaeda(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *q, int *qptr, s *z, s *ztemp, int *info) nogil:
 *     _fortran_slaeda(n, tlvls, curlvl, curpbm, prmptr, perm, givptr, givcol, givnum, q, qptr, z, ztemp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaeda,SLAEDA)(__pyx_v_n, __pyx_v_tlvls, __pyx_v_curlvl, __pyx_v_curpbm, __pyx_v_prmptr, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, __pyx_v_q, __pyx_v_qptr, __pyx_v_z, __pyx_v_ztemp, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaeda "F_FUNC(slaeda,SLAEDA)"(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *q, int *qptr, s *z, s *ztemp, int *info) nogil
 * cdef void slaeda(int *n, int *tlvls, int *curlvl, int *curpbm, int *prmptr, int *perm, int *givptr, int *givcol, s *givnum, s *q, int *qptr, s *z, s *ztemp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaeda(n, tlvls, curlvl, curpbm, prmptr, perm, givptr, givcol, givnum, q, qptr, z, ztemp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaein "F_FUNC(slaein,SLAEIN)"(bint *rightv, bint *noinit, int *n, s *h, int *ldh, s *wr, s *wi, s *vr, s *vi, s *b, int *ldb, s *work, s *eps3, s *smlnum, s *bignum, int *info) nogil
 * cdef void slaein(bint *rightv, bint *noinit, int *n, s *h, int *ldh, s *wr, s *wi, s *vr, s *vi, s *b, int *ldb, s *work, s *eps3, s *smlnum, s *bignum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaein(rightv, noinit, n, h, ldh, wr, wi, vr, vi, b, ldb, work, eps3, smlnum, bignum, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaein(int *__pyx_v_rightv, int *__pyx_v_noinit, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_eps3, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_smlnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bignum, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5392
 *     void _fortran_slaein "F_FUNC(slaein,SLAEIN)"(bint *rightv, bint *noinit, int *n, s *h, int *ldh, s *wr, s *wi, s *vr, s *vi, s *b, int *ldb, s *work, s *eps3, s *smlnum, s *bignum, int *info) nogil
 * cdef void slaein(bint *rightv, bint *noinit, int *n, s *h, int *ldh, s *wr, s *wi, s *vr, s *vi, s *b, int *ldb, s *work, s *eps3, s *smlnum, s *bignum, int *info) nogil:
 *     _fortran_slaein(rightv, noinit, n, h, ldh, wr, wi, vr, vi, b, ldb, work, eps3, smlnum, bignum, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaein,SLAEIN)(__pyx_v_rightv, __pyx_v_noinit, __pyx_v_n, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_vr, __pyx_v_vi, __pyx_v_b, __pyx_v_ldb, __pyx_v_work, __pyx_v_eps3, __pyx_v_smlnum, __pyx_v_bignum, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaein "F_FUNC(slaein,SLAEIN)"(bint *rightv, bint *noinit, int *n, s *h, int *ldh, s *wr, s *wi, s *vr, s *vi, s *b, int *ldb, s *work, s *eps3, s *smlnum, s *bignum, int *info) nogil
 * cdef void slaein(bint *rightv, bint *noinit, int *n, s *h, int *ldh, s *wr, s *wi, s *vr, s *vi, s *b, int *ldb, s *work, s *eps3, s *smlnum, s *bignum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaein(rightv, noinit, n, h, ldh, wr, wi, vr, vi, b, ldb, work, eps3, smlnum, bignum, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaev2 "F_FUNC(slaev2,SLAEV2)"(s *a, s *b, s *c, s *rt1, s *rt2, s *cs1, s *sn1) nogil
 * cdef void slaev2(s *a, s *b, s *c, s *rt1, s *rt2, s *cs1, s *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaev2(a, b, c, rt1, rt2, cs1, sn1)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaev2(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cs1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sn1) {

  /* "scipy/linalg/cython_lapack.pyx":5397
 *     void _fortran_slaev2 "F_FUNC(slaev2,SLAEV2)"(s *a, s *b, s *c, s *rt1, s *rt2, s *cs1, s *sn1) nogil
 * cdef void slaev2(s *a, s *b, s *c, s *rt1, s *rt2, s *cs1, s *sn1) nogil:
 *     _fortran_slaev2(a, b, c, rt1, rt2, cs1, sn1)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaev2,SLAEV2)(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_rt1, __pyx_v_rt2, __pyx_v_cs1, __pyx_v_sn1);

  /* "scipy/linalg/cython_lapack.pyx":5396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaev2 "F_FUNC(slaev2,SLAEV2)"(s *a, s *b, s *c, s *rt1, s *rt2, s *cs1, s *sn1) nogil
 * cdef void slaev2(s *a, s *b, s *c, s *rt1, s *rt2, s *cs1, s *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaev2(a, b, c, rt1, rt2, cs1, sn1)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaexc "F_FUNC(slaexc,SLAEXC)"(bint *wantq, int *n, s *t, int *ldt, s *q, int *ldq, int *j1, int *n1, int *n2, s *work, int *info) nogil
 * cdef void slaexc(bint *wantq, int *n, s *t, int *ldt, s *q, int *ldq, int *j1, int *n1, int *n2, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaexc(wantq, n, t, ldt, q, ldq, j1, n1, n2, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaexc(int *__pyx_v_wantq, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_j1, int *__pyx_v_n1, int *__pyx_v_n2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5402
 *     void _fortran_slaexc "F_FUNC(slaexc,SLAEXC)"(bint *wantq, int *n, s *t, int *ldt, s *q, int *ldq, int *j1, int *n1, int *n2, s *work, int *info) nogil
 * cdef void slaexc(bint *wantq, int *n, s *t, int *ldt, s *q, int *ldq, int *j1, int *n1, int *n2, s *work, int *info) nogil:
 *     _fortran_slaexc(wantq, n, t, ldt, q, ldq, j1, n1, n2, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaexc,SLAEXC)(__pyx_v_wantq, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_q, __pyx_v_ldq, __pyx_v_j1, __pyx_v_n1, __pyx_v_n2, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaexc "F_FUNC(slaexc,SLAEXC)"(bint *wantq, int *n, s *t, int *ldt, s *q, int *ldq, int *j1, int *n1, int *n2, s *work, int *info) nogil
 * cdef void slaexc(bint *wantq, int *n, s *t, int *ldt, s *q, int *ldq, int *j1, int *n1, int *n2, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaexc(wantq, n, t, ldt, q, ldq, j1, n1, n2, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slag2 "F_FUNC(slag2,SLAG2)"(s *a, int *lda, s *b, int *ldb, s *safmin, s *scale1, s *scale2, s *wr1, s *wr2, s *wi) nogil
 * cdef void slag2(s *a, int *lda, s *b, int *ldb, s *safmin, s *scale1, s *scale2, s *wr1, s *wr2, s *wi) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slag2(a, lda, b, ldb, safmin, scale1, scale2, wr1, wr2, wi)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slag2(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_safmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi) {

  /* "scipy/linalg/cython_lapack.pyx":5407
 *     void _fortran_slag2 "F_FUNC(slag2,SLAG2)"(s *a, int *lda, s *b, int *ldb, s *safmin, s *scale1, s *scale2, s *wr1, s *wr2, s *wi) nogil
 * cdef void slag2(s *a, int *lda, s *b, int *ldb, s *safmin, s *scale1, s *scale2, s *wr1, s *wr2, s *wi) nogil:
 *     _fortran_slag2(a, lda, b, ldb, safmin, scale1, scale2, wr1, wr2, wi)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slag2,SLAG2)(__pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_safmin, __pyx_v_scale1, __pyx_v_scale2, __pyx_v_wr1, __pyx_v_wr2, __pyx_v_wi);

  /* "scipy/linalg/cython_lapack.pyx":5406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slag2 "F_FUNC(slag2,SLAG2)"(s *a, int *lda, s *b, int *ldb, s *safmin, s *scale1, s *scale2, s *wr1, s *wr2, s *wi) nogil
 * cdef void slag2(s *a, int *lda, s *b, int *ldb, s *safmin, s *scale1, s *scale2, s *wr1, s *wr2, s *wi) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slag2(a, lda, b, ldb, safmin, scale1, scale2, wr1, wr2, wi)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slag2d "F_FUNC(slag2d,SLAG2D)"(int *m, int *n, s *sa, int *ldsa, d *a, int *lda, int *info) nogil
 * cdef void slag2d(int *m, int *n, s *sa, int *ldsa, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slag2d(m, n, sa, ldsa, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slag2d(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sa, int *__pyx_v_ldsa, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5412
 *     void _fortran_slag2d "F_FUNC(slag2d,SLAG2D)"(int *m, int *n, s *sa, int *ldsa, d *a, int *lda, int *info) nogil
 * cdef void slag2d(int *m, int *n, s *sa, int *ldsa, d *a, int *lda, int *info) nogil:
 *     _fortran_slag2d(m, n, sa, ldsa, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slag2d,SLAG2D)(__pyx_v_m, __pyx_v_n, __pyx_v_sa, __pyx_v_ldsa, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slag2d "F_FUNC(slag2d,SLAG2D)"(int *m, int *n, s *sa, int *ldsa, d *a, int *lda, int *info) nogil
 * cdef void slag2d(int *m, int *n, s *sa, int *ldsa, d *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slag2d(m, n, sa, ldsa, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slags2 "F_FUNC(slags2,SLAGS2)"(bint *upper, s *a1, s *a2, s *a3, s *b1, s *b2, s *b3, s *csu, s *snu, s *csv, s *snv, s *csq, s *snq) nogil
 * cdef void slags2(bint *upper, s *a1, s *a2, s *a3, s *b1, s *b2, s *b3, s *csu, s *snu, s *csv, s *snv, s *csq, s *snq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slags2(upper, a1, a2, a3, b1, b2, b3, csu, snu, csv, snv, csq, snq)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slags2(int *__pyx_v_upper, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a3, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b3, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_snu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_snv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_snq) {

  /* "scipy/linalg/cython_lapack.pyx":5417
 *     void _fortran_slags2 "F_FUNC(slags2,SLAGS2)"(bint *upper, s *a1, s *a2, s *a3, s *b1, s *b2, s *b3, s *csu, s *snu, s *csv, s *snv, s *csq, s *snq) nogil
 * cdef void slags2(bint *upper, s *a1, s *a2, s *a3, s *b1, s *b2, s *b3, s *csu, s *snu, s *csv, s *snv, s *csq, s *snq) nogil:
 *     _fortran_slags2(upper, a1, a2, a3, b1, b2, b3, csu, snu, csv, snv, csq, snq)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slags2,SLAGS2)(__pyx_v_upper, __pyx_v_a1, __pyx_v_a2, __pyx_v_a3, __pyx_v_b1, __pyx_v_b2, __pyx_v_b3, __pyx_v_csu, __pyx_v_snu, __pyx_v_csv, __pyx_v_snv, __pyx_v_csq, __pyx_v_snq);

  /* "scipy/linalg/cython_lapack.pyx":5416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slags2 "F_FUNC(slags2,SLAGS2)"(bint *upper, s *a1, s *a2, s *a3, s *b1, s *b2, s *b3, s *csu, s *snu, s *csv, s *snv, s *csq, s *snq) nogil
 * cdef void slags2(bint *upper, s *a1, s *a2, s *a3, s *b1, s *b2, s *b3, s *csu, s *snu, s *csv, s *snv, s *csq, s *snq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slags2(upper, a1, a2, a3, b1, b2, b3, csu, snu, csv, snv, csq, snq)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slagtf "F_FUNC(slagtf,SLAGTF)"(int *n, s *a, s *lambda_, s *b, s *c, s *tol, s *d, int *in_, int *info) nogil
 * cdef void slagtf(int *n, s *a, s *lambda_, s *b, s *c, s *tol, s *d, int *in_, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slagtf(n, a, lambda_, b, c, tol, d, in_, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slagtf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lambda_, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, int *__pyx_v_in_, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5422
 *     void _fortran_slagtf "F_FUNC(slagtf,SLAGTF)"(int *n, s *a, s *lambda_, s *b, s *c, s *tol, s *d, int *in_, int *info) nogil
 * cdef void slagtf(int *n, s *a, s *lambda_, s *b, s *c, s *tol, s *d, int *in_, int *info) nogil:
 *     _fortran_slagtf(n, a, lambda_, b, c, tol, d, in_, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slagtf,SLAGTF)(__pyx_v_n, __pyx_v_a, __pyx_v_lambda_, __pyx_v_b, __pyx_v_c, __pyx_v_tol, __pyx_v_d, __pyx_v_in_, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slagtf "F_FUNC(slagtf,SLAGTF)"(int *n, s *a, s *lambda_, s *b, s *c, s *tol, s *d, int *in_, int *info) nogil
 * cdef void slagtf(int *n, s *a, s *lambda_, s *b, s *c, s *tol, s *d, int *in_, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slagtf(n, a, lambda_, b, c, tol, d, in_, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slagtm "F_FUNC(slagtm,SLAGTM)"(char *trans, int *n, int *nrhs, s *alpha, s *dl, s *d, s *du, s *x, int *ldx, s *beta, s *b, int *ldb) nogil
 * cdef void slagtm(char *trans, int *n, int *nrhs, s *alpha, s *dl, s *d, s *du, s *x, int *ldx, s *beta, s *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slagtm(trans, n, nrhs, alpha, dl, d, du, x, ldx, beta, b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slagtm(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_du, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":5427
 *     void _fortran_slagtm "F_FUNC(slagtm,SLAGTM)"(char *trans, int *n, int *nrhs, s *alpha, s *dl, s *d, s *du, s *x, int *ldx, s *beta, s *b, int *ldb) nogil
 * cdef void slagtm(char *trans, int *n, int *nrhs, s *alpha, s *dl, s *d, s *du, s *x, int *ldx, s *beta, s *b, int *ldb) nogil:
 *     _fortran_slagtm(trans, n, nrhs, alpha, dl, d, du, x, ldx, beta, b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slagtm,SLAGTM)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_alpha, __pyx_v_dl, __pyx_v_d, __pyx_v_du, __pyx_v_x, __pyx_v_ldx, __pyx_v_beta, __pyx_v_b, __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":5426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slagtm "F_FUNC(slagtm,SLAGTM)"(char *trans, int *n, int *nrhs, s *alpha, s *dl, s *d, s *du, s *x, int *ldx, s *beta, s *b, int *ldb) nogil
 * cdef void slagtm(char *trans, int *n, int *nrhs, s *alpha, s *dl, s *d, s *du, s *x, int *ldx, s *beta, s *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slagtm(trans, n, nrhs, alpha, dl, d, du, x, ldx, beta, b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slagts "F_FUNC(slagts,SLAGTS)"(int *job, int *n, s *a, s *b, s *c, s *d, int *in_, s *y, s *tol, int *info) nogil
 * cdef void slagts(int *job, int *n, s *a, s *b, s *c, s *d, int *in_, s *y, s *tol, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slagts(job, n, a, b, c, d, in_, y, tol, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slagts(int *__pyx_v_job, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, int *__pyx_v_in_, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tol, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5432
 *     void _fortran_slagts "F_FUNC(slagts,SLAGTS)"(int *job, int *n, s *a, s *b, s *c, s *d, int *in_, s *y, s *tol, int *info) nogil
 * cdef void slagts(int *job, int *n, s *a, s *b, s *c, s *d, int *in_, s *y, s *tol, int *info) nogil:
 *     _fortran_slagts(job, n, a, b, c, d, in_, y, tol, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slagts,SLAGTS)(__pyx_v_job, __pyx_v_n, __pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_d, __pyx_v_in_, __pyx_v_y, __pyx_v_tol, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slagts "F_FUNC(slagts,SLAGTS)"(int *job, int *n, s *a, s *b, s *c, s *d, int *in_, s *y, s *tol, int *info) nogil
 * cdef void slagts(int *job, int *n, s *a, s *b, s *c, s *d, int *in_, s *y, s *tol, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slagts(job, n, a, b, c, d, in_, y, tol, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slagv2 "F_FUNC(slagv2,SLAGV2)"(s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *csl, s *snl, s *csr, s *snr) nogil
 * cdef void slagv2(s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *csl, s *snl, s *csr, s *snr) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slagv2(a, lda, b, ldb, alphar, alphai, beta, csl, snl, csr, snr)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slagv2(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_snl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_snr) {

  /* "scipy/linalg/cython_lapack.pyx":5437
 *     void _fortran_slagv2 "F_FUNC(slagv2,SLAGV2)"(s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *csl, s *snl, s *csr, s *snr) nogil
 * cdef void slagv2(s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *csl, s *snl, s *csr, s *snr) nogil:
 *     _fortran_slagv2(a, lda, b, ldb, alphar, alphai, beta, csl, snl, csr, snr)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slagv2,SLAGV2)(__pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_csl, __pyx_v_snl, __pyx_v_csr, __pyx_v_snr);

  /* "scipy/linalg/cython_lapack.pyx":5436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slagv2 "F_FUNC(slagv2,SLAGV2)"(s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *csl, s *snl, s *csr, s *snr) nogil
 * cdef void slagv2(s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *csl, s *snl, s *csr, s *snr) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slagv2(a, lda, b, ldb, alphar, alphai, beta, csl, snl, csr, snr)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slahqr "F_FUNC(slahqr,SLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, int *info) nogil
 * cdef void slahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slahqr(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slahqr(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5442
 *     void _fortran_slahqr "F_FUNC(slahqr,SLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, int *info) nogil
 * cdef void slahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, int *info) nogil:
 *     _fortran_slahqr(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slahqr,SLAHQR)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slahqr "F_FUNC(slahqr,SLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, int *info) nogil
 * cdef void slahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slahqr(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slahr2 "F_FUNC(slahr2,SLAHR2)"(int *n, int *k, int *nb, s *a, int *lda, s *tau, s *t, int *ldt, s *y, int *ldy) nogil
 * cdef void slahr2(int *n, int *k, int *nb, s *a, int *lda, s *tau, s *t, int *ldt, s *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slahr2(n, k, nb, a, lda, tau, t, ldt, y, ldy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slahr2(int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_nb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, int *__pyx_v_ldy) {

  /* "scipy/linalg/cython_lapack.pyx":5447
 *     void _fortran_slahr2 "F_FUNC(slahr2,SLAHR2)"(int *n, int *k, int *nb, s *a, int *lda, s *tau, s *t, int *ldt, s *y, int *ldy) nogil
 * cdef void slahr2(int *n, int *k, int *nb, s *a, int *lda, s *tau, s *t, int *ldt, s *y, int *ldy) nogil:
 *     _fortran_slahr2(n, k, nb, a, lda, tau, t, ldt, y, ldy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slahr2,SLAHR2)(__pyx_v_n, __pyx_v_k, __pyx_v_nb, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_t, __pyx_v_ldt, __pyx_v_y, __pyx_v_ldy);

  /* "scipy/linalg/cython_lapack.pyx":5446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slahr2 "F_FUNC(slahr2,SLAHR2)"(int *n, int *k, int *nb, s *a, int *lda, s *tau, s *t, int *ldt, s *y, int *ldy) nogil
 * cdef void slahr2(int *n, int *k, int *nb, s *a, int *lda, s *tau, s *t, int *ldt, s *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slahr2(n, k, nb, a, lda, tau, t, ldt, y, ldy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaic1 "F_FUNC(slaic1,SLAIC1)"(int *job, int *j, s *x, s *sest, s *w, s *gamma, s *sestpr, s *s, s *c) nogil
 * cdef void slaic1(int *job, int *j, s *x, s *sest, s *w, s *gamma, s *sestpr, s *s, s *c) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaic1(job, j, x, sest, w, gamma, sestpr, s, c)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaic1(int *__pyx_v_job, int *__pyx_v_j, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sest, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gamma, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sestpr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c) {

  /* "scipy/linalg/cython_lapack.pyx":5452
 *     void _fortran_slaic1 "F_FUNC(slaic1,SLAIC1)"(int *job, int *j, s *x, s *sest, s *w, s *gamma, s *sestpr, s *s, s *c) nogil
 * cdef void slaic1(int *job, int *j, s *x, s *sest, s *w, s *gamma, s *sestpr, s *s, s *c) nogil:
 *     _fortran_slaic1(job, j, x, sest, w, gamma, sestpr, s, c)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaic1,SLAIC1)(__pyx_v_job, __pyx_v_j, __pyx_v_x, __pyx_v_sest, __pyx_v_w, __pyx_v_gamma, __pyx_v_sestpr, __pyx_v_s, __pyx_v_c);

  /* "scipy/linalg/cython_lapack.pyx":5451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaic1 "F_FUNC(slaic1,SLAIC1)"(int *job, int *j, s *x, s *sest, s *w, s *gamma, s *sestpr, s *s, s *c) nogil
 * cdef void slaic1(int *job, int *j, s *x, s *sest, s *w, s *gamma, s *sestpr, s *s, s *c) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaic1(job, j, x, sest, w, gamma, sestpr, s, c)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaln2 "F_FUNC(slaln2,SLALN2)"(bint *ltrans, int *na, int *nw, s *smin, s *ca, s *a, int *lda, s *d1, s *d2, s *b, int *ldb, s *wr, s *wi, s *x, int *ldx, s *scale, s *xnorm, int *info) nogil
 * cdef void slaln2(bint *ltrans, int *na, int *nw, s *smin, s *ca, s *a, int *lda, s *d1, s *d2, s *b, int *ldb, s *wr, s *wi, s *x, int *ldx, s *scale, s *xnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaln2(ltrans, na, nw, smin, ca, a, lda, d1, d2, b, ldb, wr, wi, x, ldx, scale, xnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaln2(int *__pyx_v_ltrans, int *__pyx_v_na, int *__pyx_v_nw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_smin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ca, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_xnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5457
 *     void _fortran_slaln2 "F_FUNC(slaln2,SLALN2)"(bint *ltrans, int *na, int *nw, s *smin, s *ca, s *a, int *lda, s *d1, s *d2, s *b, int *ldb, s *wr, s *wi, s *x, int *ldx, s *scale, s *xnorm, int *info) nogil
 * cdef void slaln2(bint *ltrans, int *na, int *nw, s *smin, s *ca, s *a, int *lda, s *d1, s *d2, s *b, int *ldb, s *wr, s *wi, s *x, int *ldx, s *scale, s *xnorm, int *info) nogil:
 *     _fortran_slaln2(ltrans, na, nw, smin, ca, a, lda, d1, d2, b, ldb, wr, wi, x, ldx, scale, xnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaln2,SLALN2)(__pyx_v_ltrans, __pyx_v_na, __pyx_v_nw, __pyx_v_smin, __pyx_v_ca, __pyx_v_a, __pyx_v_lda, __pyx_v_d1, __pyx_v_d2, __pyx_v_b, __pyx_v_ldb, __pyx_v_wr, __pyx_v_wi, __pyx_v_x, __pyx_v_ldx, __pyx_v_scale, __pyx_v_xnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaln2 "F_FUNC(slaln2,SLALN2)"(bint *ltrans, int *na, int *nw, s *smin, s *ca, s *a, int *lda, s *d1, s *d2, s *b, int *ldb, s *wr, s *wi, s *x, int *ldx, s *scale, s *xnorm, int *info) nogil
 * cdef void slaln2(bint *ltrans, int *na, int *nw, s *smin, s *ca, s *a, int *lda, s *d1, s *d2, s *b, int *ldb, s *wr, s *wi, s *x, int *ldx, s *scale, s *xnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaln2(ltrans, na, nw, smin, ca, a, lda, d1, d2, b, ldb, wr, wi, x, ldx, scale, xnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slals0 "F_FUNC(slals0,SLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *info) nogil
 * cdef void slals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slals0(icompq, nl, nr, sqre, nrhs, b, ldb, bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slals0(int *__pyx_v_icompq, int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bx, int *__pyx_v_ldbx, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, int *__pyx_v_ldgnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_poles, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5462
 *     void _fortran_slals0 "F_FUNC(slals0,SLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *info) nogil
 * cdef void slals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *info) nogil:
 *     _fortran_slals0(icompq, nl, nr, sqre, nrhs, b, ldb, bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slals0,SLALS0)(__pyx_v_icompq, __pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_nrhs, __pyx_v_b, __pyx_v_ldb, __pyx_v_bx, __pyx_v_ldbx, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_givnum, __pyx_v_ldgnum, __pyx_v_poles, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_k, __pyx_v_c, __pyx_v_s, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slals0 "F_FUNC(slals0,SLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *info) nogil
 * cdef void slals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slals0(icompq, nl, nr, sqre, nrhs, b, ldb, bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slalsa "F_FUNC(slalsa,SLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slalsa(int *icompq, int *smlsiz, int *n, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slalsa(icompq, smlsiz, n, nrhs, b, ldb, bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slalsa(int *__pyx_v_icompq, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bx, int *__pyx_v_ldbx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_poles, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, int *__pyx_v_perm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5467
 *     void _fortran_slalsa "F_FUNC(slalsa,SLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slalsa(int *icompq, int *smlsiz, int *n, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil:
 *     _fortran_slalsa(icompq, smlsiz, n, nrhs, b, ldb, bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slalsa,SLALSA)(__pyx_v_icompq, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_nrhs, __pyx_v_b, __pyx_v_ldb, __pyx_v_bx, __pyx_v_ldbx, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_k, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_poles, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_perm, __pyx_v_givnum, __pyx_v_c, __pyx_v_s, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slalsa "F_FUNC(slalsa,SLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slalsa(int *icompq, int *smlsiz, int *n, int *nrhs, s *b, int *ldb, s *bx, int *ldbx, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slalsa(icompq, smlsiz, n, nrhs, b, ldb, bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slalsd "F_FUNC(slalsd,SLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, s *b, int *ldb, s *rcond, int *rank, s *work, int *iwork, int *info) nogil
 * cdef void slalsd(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, s *b, int *ldb, s *rcond, int *rank, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slalsd(uplo, smlsiz, n, nrhs, d, e, b, ldb, rcond, rank, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slalsd(char *__pyx_v_uplo, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5472
 *     void _fortran_slalsd "F_FUNC(slalsd,SLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, s *b, int *ldb, s *rcond, int *rank, s *work, int *iwork, int *info) nogil
 * cdef void slalsd(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, s *b, int *ldb, s *rcond, int *rank, s *work, int *iwork, int *info) nogil:
 *     _fortran_slalsd(uplo, smlsiz, n, nrhs, d, e, b, ldb, rcond, rank, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slalsd,SLALSD)(__pyx_v_uplo, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_b, __pyx_v_ldb, __pyx_v_rcond, __pyx_v_rank, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slalsd "F_FUNC(slalsd,SLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, s *b, int *ldb, s *rcond, int *rank, s *work, int *iwork, int *info) nogil
 * cdef void slalsd(char *uplo, int *smlsiz, int *n, int *nrhs, s *d, s *e, s *b, int *ldb, s *rcond, int *rank, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slalsd(uplo, smlsiz, n, nrhs, d, e, b, ldb, rcond, rank, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slamrg "F_FUNC(slamrg,SLAMRG)"(int *n1, int *n2, s *a, int *strd1, int *strd2, int *index_bn) nogil
 * cdef void slamrg(int *n1, int *n2, s *a, int *strd1, int *strd2, int *index_bn) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slamrg(n1, n2, a, strd1, strd2, index_bn)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slamrg(int *__pyx_v_n1, int *__pyx_v_n2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_strd1, int *__pyx_v_strd2, int *__pyx_v_index_bn) {

  /* "scipy/linalg/cython_lapack.pyx":5477
 *     void _fortran_slamrg "F_FUNC(slamrg,SLAMRG)"(int *n1, int *n2, s *a, int *strd1, int *strd2, int *index_bn) nogil
 * cdef void slamrg(int *n1, int *n2, s *a, int *strd1, int *strd2, int *index_bn) nogil:
 *     _fortran_slamrg(n1, n2, a, strd1, strd2, index_bn)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slamrg,SLAMRG)(__pyx_v_n1, __pyx_v_n2, __pyx_v_a, __pyx_v_strd1, __pyx_v_strd2, __pyx_v_index_bn);

  /* "scipy/linalg/cython_lapack.pyx":5476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slamrg "F_FUNC(slamrg,SLAMRG)"(int *n1, int *n2, s *a, int *strd1, int *strd2, int *index_bn) nogil
 * cdef void slamrg(int *n1, int *n2, s *a, int *strd1, int *strd2, int *index_bn) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slamrg(n1, n2, a, strd1, strd2, index_bn)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slanv2 "F_FUNC(slanv2,SLANV2)"(s *a, s *b, s *c, s *d, s *rt1r, s *rt1i, s *rt2r, s *rt2i, s *cs, s *sn) nogil
 * cdef void slanv2(s *a, s *b, s *c, s *d, s *rt1r, s *rt1i, s *rt2r, s *rt2i, s *cs, s *sn) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slanv2(a, b, c, d, rt1r, rt1i, rt2r, rt2i, cs, sn)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slanv2(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt1r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt1i, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt2r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rt2i, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sn) {

  /* "scipy/linalg/cython_lapack.pyx":5482
 *     void _fortran_slanv2 "F_FUNC(slanv2,SLANV2)"(s *a, s *b, s *c, s *d, s *rt1r, s *rt1i, s *rt2r, s *rt2i, s *cs, s *sn) nogil
 * cdef void slanv2(s *a, s *b, s *c, s *d, s *rt1r, s *rt1i, s *rt2r, s *rt2i, s *cs, s *sn) nogil:
 *     _fortran_slanv2(a, b, c, d, rt1r, rt1i, rt2r, rt2i, cs, sn)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slanv2,SLANV2)(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_d, __pyx_v_rt1r, __pyx_v_rt1i, __pyx_v_rt2r, __pyx_v_rt2i, __pyx_v_cs, __pyx_v_sn);

  /* "scipy/linalg/cython_lapack.pyx":5481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slanv2 "F_FUNC(slanv2,SLANV2)"(s *a, s *b, s *c, s *d, s *rt1r, s *rt1i, s *rt2r, s *rt2i, s *cs, s *sn) nogil
 * cdef void slanv2(s *a, s *b, s *c, s *d, s *rt1r, s *rt1i, s *rt2r, s *rt2i, s *cs, s *sn) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slanv2(a, b, c, d, rt1r, rt1i, rt2r, rt2i, cs, sn)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slapll "F_FUNC(slapll,SLAPLL)"(int *n, s *x, int *incx, s *y, int *incy, s *ssmin) nogil
 * cdef void slapll(int *n, s *x, int *incx, s *y, int *incy, s *ssmin) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slapll(n, x, incx, y, incy, ssmin)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slapll(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ssmin) {

  /* "scipy/linalg/cython_lapack.pyx":5487
 *     void _fortran_slapll "F_FUNC(slapll,SLAPLL)"(int *n, s *x, int *incx, s *y, int *incy, s *ssmin) nogil
 * cdef void slapll(int *n, s *x, int *incx, s *y, int *incy, s *ssmin) nogil:
 *     _fortran_slapll(n, x, incx, y, incy, ssmin)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slapll,SLAPLL)(__pyx_v_n, __pyx_v_x, __pyx_v_incx, __pyx_v_y, __pyx_v_incy, __pyx_v_ssmin);

  /* "scipy/linalg/cython_lapack.pyx":5486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slapll "F_FUNC(slapll,SLAPLL)"(int *n, s *x, int *incx, s *y, int *incy, s *ssmin) nogil
 * cdef void slapll(int *n, s *x, int *incx, s *y, int *incy, s *ssmin) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slapll(n, x, incx, y, incy, ssmin)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slapmt "F_FUNC(slapmt,SLAPMT)"(bint *forwrd, int *m, int *n, s *x, int *ldx, int *k) nogil
 * cdef void slapmt(bint *forwrd, int *m, int *n, s *x, int *ldx, int *k) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slapmt(forwrd, m, n, x, ldx, k)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slapmt(int *__pyx_v_forwrd, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, int *__pyx_v_k) {

  /* "scipy/linalg/cython_lapack.pyx":5492
 *     void _fortran_slapmt "F_FUNC(slapmt,SLAPMT)"(bint *forwrd, int *m, int *n, s *x, int *ldx, int *k) nogil
 * cdef void slapmt(bint *forwrd, int *m, int *n, s *x, int *ldx, int *k) nogil:
 *     _fortran_slapmt(forwrd, m, n, x, ldx, k)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slapmt,SLAPMT)(__pyx_v_forwrd, __pyx_v_m, __pyx_v_n, __pyx_v_x, __pyx_v_ldx, __pyx_v_k);

  /* "scipy/linalg/cython_lapack.pyx":5491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slapmt "F_FUNC(slapmt,SLAPMT)"(bint *forwrd, int *m, int *n, s *x, int *ldx, int *k) nogil
 * cdef void slapmt(bint *forwrd, int *m, int *n, s *x, int *ldx, int *k) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slapmt(forwrd, m, n, x, ldx, k)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqgb "F_FUNC(slaqgb,SLAQGB)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void slaqgb(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqgb(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqgb(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":5497
 *     void _fortran_slaqgb "F_FUNC(slaqgb,SLAQGB)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void slaqgb(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:
 *     _fortran_slaqgb(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqgb,SLAQGB)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_ab, __pyx_v_ldab, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":5496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqgb "F_FUNC(slaqgb,SLAQGB)"(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void slaqgb(int *m, int *n, int *kl, int *ku, s *ab, int *ldab, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqgb(m, n, kl, ku, ab, ldab, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqge "F_FUNC(slaqge,SLAQGE)"(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void slaqge(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqge(m, n, a, lda, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqge(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":5502
 *     void _fortran_slaqge "F_FUNC(slaqge,SLAQGE)"(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void slaqge(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:
 *     _fortran_slaqge(m, n, a, lda, r, c, rowcnd, colcnd, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqge,SLAQGE)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":5501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqge "F_FUNC(slaqge,SLAQGE)"(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil
 * cdef void slaqge(int *m, int *n, s *a, int *lda, s *r, s *c, s *rowcnd, s *colcnd, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqge(m, n, a, lda, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqp2 "F_FUNC(slaqp2,SLAQP2)"(int *m, int *n, int *offset, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *work) nogil
 * cdef void slaqp2(int *m, int *n, int *offset, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqp2(m, n, offset, a, lda, jpvt, tau, vn1, vn2, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqp2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_offset, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vn1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vn2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":5507
 *     void _fortran_slaqp2 "F_FUNC(slaqp2,SLAQP2)"(int *m, int *n, int *offset, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *work) nogil
 * cdef void slaqp2(int *m, int *n, int *offset, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *work) nogil:
 *     _fortran_slaqp2(m, n, offset, a, lda, jpvt, tau, vn1, vn2, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqp2,SLAQP2)(__pyx_v_m, __pyx_v_n, __pyx_v_offset, __pyx_v_a, __pyx_v_lda, __pyx_v_jpvt, __pyx_v_tau, __pyx_v_vn1, __pyx_v_vn2, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":5506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqp2 "F_FUNC(slaqp2,SLAQP2)"(int *m, int *n, int *offset, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *work) nogil
 * cdef void slaqp2(int *m, int *n, int *offset, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqp2(m, n, offset, a, lda, jpvt, tau, vn1, vn2, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqps "F_FUNC(slaqps,SLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *auxv, s *f, int *ldf) nogil
 * cdef void slaqps(int *m, int *n, int *offset, int *nb, int *kb, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *auxv, s *f, int *ldf) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqps(m, n, offset, nb, kb, a, lda, jpvt, tau, vn1, vn2, auxv, f, ldf)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqps(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_offset, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vn1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vn2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_auxv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_f, int *__pyx_v_ldf) {

  /* "scipy/linalg/cython_lapack.pyx":5512
 *     void _fortran_slaqps "F_FUNC(slaqps,SLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *auxv, s *f, int *ldf) nogil
 * cdef void slaqps(int *m, int *n, int *offset, int *nb, int *kb, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *auxv, s *f, int *ldf) nogil:
 *     _fortran_slaqps(m, n, offset, nb, kb, a, lda, jpvt, tau, vn1, vn2, auxv, f, ldf)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqps,SLAQPS)(__pyx_v_m, __pyx_v_n, __pyx_v_offset, __pyx_v_nb, __pyx_v_kb, __pyx_v_a, __pyx_v_lda, __pyx_v_jpvt, __pyx_v_tau, __pyx_v_vn1, __pyx_v_vn2, __pyx_v_auxv, __pyx_v_f, __pyx_v_ldf);

  /* "scipy/linalg/cython_lapack.pyx":5511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqps "F_FUNC(slaqps,SLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *auxv, s *f, int *ldf) nogil
 * cdef void slaqps(int *m, int *n, int *offset, int *nb, int *kb, s *a, int *lda, int *jpvt, s *tau, s *vn1, s *vn2, s *auxv, s *f, int *ldf) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqps(m, n, offset, nb, kb, a, lda, jpvt, tau, vn1, vn2, auxv, f, ldf)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr0 "F_FUNC(slaqr0,SLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void slaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr0(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqr0(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5517
 *     void _fortran_slaqr0 "F_FUNC(slaqr0,SLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void slaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil:
 *     _fortran_slaqr0(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqr0,SLAQR0)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr0 "F_FUNC(slaqr0,SLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void slaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr0(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr1 "F_FUNC(slaqr1,SLAQR1)"(int *n, s *h, int *ldh, s *sr1, s *si1, s *sr2, s *si2, s *v) nogil
 * cdef void slaqr1(int *n, s *h, int *ldh, s *sr1, s *si1, s *sr2, s *si2, s *v) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr1(n, h, ldh, sr1, si1, sr2, si2, v)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqr1(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sr1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_si1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sr2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_si2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v) {

  /* "scipy/linalg/cython_lapack.pyx":5522
 *     void _fortran_slaqr1 "F_FUNC(slaqr1,SLAQR1)"(int *n, s *h, int *ldh, s *sr1, s *si1, s *sr2, s *si2, s *v) nogil
 * cdef void slaqr1(int *n, s *h, int *ldh, s *sr1, s *si1, s *sr2, s *si2, s *v) nogil:
 *     _fortran_slaqr1(n, h, ldh, sr1, si1, sr2, si2, v)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqr1,SLAQR1)(__pyx_v_n, __pyx_v_h, __pyx_v_ldh, __pyx_v_sr1, __pyx_v_si1, __pyx_v_sr2, __pyx_v_si2, __pyx_v_v);

  /* "scipy/linalg/cython_lapack.pyx":5521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr1 "F_FUNC(slaqr1,SLAQR1)"(int *n, s *h, int *ldh, s *sr1, s *si1, s *sr2, s *si2, s *v) nogil
 * cdef void slaqr1(int *n, s *h, int *ldh, s *sr1, s *si1, s *sr2, s *si2, s *v) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr1(n, h, ldh, sr1, si1, sr2, si2, v)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr2 "F_FUNC(slaqr2,SLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil
 * cdef void slaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr2(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqr2(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ns, int *__pyx_v_nd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_si, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_nh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, int *__pyx_v_nv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wv, int *__pyx_v_ldwv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork) {

  /* "scipy/linalg/cython_lapack.pyx":5527
 *     void _fortran_slaqr2 "F_FUNC(slaqr2,SLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil
 * cdef void slaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil:
 *     _fortran_slaqr2(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqr2,SLAQR2)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nw, __pyx_v_h, __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_ns, __pyx_v_nd, __pyx_v_sr, __pyx_v_si, __pyx_v_v, __pyx_v_ldv, __pyx_v_nh, __pyx_v_t, __pyx_v_ldt, __pyx_v_nv, __pyx_v_wv, __pyx_v_ldwv, __pyx_v_work, __pyx_v_lwork);

  /* "scipy/linalg/cython_lapack.pyx":5526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr2 "F_FUNC(slaqr2,SLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil
 * cdef void slaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr2(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr3 "F_FUNC(slaqr3,SLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil
 * cdef void slaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr3(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqr3(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ns, int *__pyx_v_nd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_si, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_nh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, int *__pyx_v_nv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wv, int *__pyx_v_ldwv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork) {

  /* "scipy/linalg/cython_lapack.pyx":5532
 *     void _fortran_slaqr3 "F_FUNC(slaqr3,SLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil
 * cdef void slaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil:
 *     _fortran_slaqr3(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqr3,SLAQR3)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nw, __pyx_v_h, __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_ns, __pyx_v_nd, __pyx_v_sr, __pyx_v_si, __pyx_v_v, __pyx_v_ldv, __pyx_v_nh, __pyx_v_t, __pyx_v_ldt, __pyx_v_nv, __pyx_v_wv, __pyx_v_ldwv, __pyx_v_work, __pyx_v_lwork);

  /* "scipy/linalg/cython_lapack.pyx":5531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr3 "F_FUNC(slaqr3,SLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil
 * cdef void slaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, int *ns, int *nd, s *sr, s *si, s *v, int *ldv, int *nh, s *t, int *ldt, int *nv, s *wv, int *ldwv, s *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr3(wantt, wantz, n, ktop, kbot, nw, h, ldh, iloz, ihiz, z, ldz, ns, nd, sr, si, v, ldv, nh, t, ldt, nv, wv, ldwv, work, lwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr4 "F_FUNC(slaqr4,SLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void slaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr4(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqr4(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5537
 *     void _fortran_slaqr4 "F_FUNC(slaqr4,SLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void slaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil:
 *     _fortran_slaqr4(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqr4,SLAQR4)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_h, __pyx_v_ldh, __pyx_v_wr, __pyx_v_wi, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr4 "F_FUNC(slaqr4,SLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil
 * cdef void slaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, s *h, int *ldh, s *wr, s *wi, int *iloz, int *ihiz, s *z, int *ldz, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr4(wantt, wantz, n, ilo, ihi, h, ldh, wr, wi, iloz, ihiz, z, ldz, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr5 "F_FUNC(slaqr5,SLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, s *sr, s *si, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, s *v, int *ldv, s *u, int *ldu, int *nv, s *wv, int *ldwv, int *nh, s *wh, int *ldwh) nogil
 * cdef void slaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, s *sr, s *si, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, s *v, int *ldv, s *u, int *ldu, int *nv, s *wv, int *ldwv, int *nh, s *wh, int *ldwh) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, sr, si, h, ldh, iloz, ihiz, z, ldz, v, ldv, u, ldu, nv, wv, ldwv, nh, wh, ldwh)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqr5(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_kacc22, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nshfts, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_si, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, int *__pyx_v_nv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wv, int *__pyx_v_ldwv, int *__pyx_v_nh, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wh, int *__pyx_v_ldwh) {

  /* "scipy/linalg/cython_lapack.pyx":5542
 *     void _fortran_slaqr5 "F_FUNC(slaqr5,SLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, s *sr, s *si, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, s *v, int *ldv, s *u, int *ldu, int *nv, s *wv, int *ldwv, int *nh, s *wh, int *ldwh) nogil
 * cdef void slaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, s *sr, s *si, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, s *v, int *ldv, s *u, int *ldu, int *nv, s *wv, int *ldwv, int *nh, s *wh, int *ldwh) nogil:
 *     _fortran_slaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, sr, si, h, ldh, iloz, ihiz, z, ldz, v, ldv, u, ldu, nv, wv, ldwv, nh, wh, ldwh)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqr5,SLAQR5)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_kacc22, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nshfts, __pyx_v_sr, __pyx_v_si, __pyx_v_h, __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, __pyx_v_z, __pyx_v_ldz, __pyx_v_v, __pyx_v_ldv, __pyx_v_u, __pyx_v_ldu, __pyx_v_nv, __pyx_v_wv, __pyx_v_ldwv, __pyx_v_nh, __pyx_v_wh, __pyx_v_ldwh);

  /* "scipy/linalg/cython_lapack.pyx":5541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqr5 "F_FUNC(slaqr5,SLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, s *sr, s *si, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, s *v, int *ldv, s *u, int *ldu, int *nv, s *wv, int *ldwv, int *nh, s *wh, int *ldwh) nogil
 * cdef void slaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, s *sr, s *si, s *h, int *ldh, int *iloz, int *ihiz, s *z, int *ldz, s *v, int *ldv, s *u, int *ldu, int *nv, s *wv, int *ldwv, int *nh, s *wh, int *ldwh) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, sr, si, h, ldh, iloz, ihiz, z, ldz, v, ldv, u, ldu, nv, wv, ldwv, nh, wh, ldwh)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqsb "F_FUNC(slaqsb,SLAQSB)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsb(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqsb(uplo, n, kd, ab, ldab, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqsb(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":5547
 *     void _fortran_slaqsb "F_FUNC(slaqsb,SLAQSB)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsb(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_slaqsb(uplo, n, kd, ab, ldab, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqsb,SLAQSB)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":5546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqsb "F_FUNC(slaqsb,SLAQSB)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsb(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqsb(uplo, n, kd, ab, ldab, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqsp "F_FUNC(slaqsp,SLAQSP)"(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsp(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqsp(uplo, n, ap, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqsp(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":5552
 *     void _fortran_slaqsp "F_FUNC(slaqsp,SLAQSP)"(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsp(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_slaqsp(uplo, n, ap, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqsp,SLAQSP)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":5551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqsp "F_FUNC(slaqsp,SLAQSP)"(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsp(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqsp(uplo, n, ap, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqsy "F_FUNC(slaqsy,SLAQSY)"(char *uplo, int *n, s *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsy(char *uplo, int *n, s *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqsy(uplo, n, a, lda, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqsy(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":5557
 *     void _fortran_slaqsy "F_FUNC(slaqsy,SLAQSY)"(char *uplo, int *n, s *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsy(char *uplo, int *n, s *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:
 *     _fortran_slaqsy(uplo, n, a, lda, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqsy,SLAQSY)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":5556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqsy "F_FUNC(slaqsy,SLAQSY)"(char *uplo, int *n, s *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil
 * cdef void slaqsy(char *uplo, int *n, s *a, int *lda, s *s, s *scond, s *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqsy(uplo, n, a, lda, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqtr "F_FUNC(slaqtr,SLAQTR)"(bint *ltran, bint *lreal, int *n, s *t, int *ldt, s *b, s *w, s *scale, s *x, s *work, int *info) nogil
 * cdef void slaqtr(bint *ltran, bint *lreal, int *n, s *t, int *ldt, s *b, s *w, s *scale, s *x, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqtr(ltran, lreal, n, t, ldt, b, w, scale, x, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaqtr(int *__pyx_v_ltran, int *__pyx_v_lreal, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5562
 *     void _fortran_slaqtr "F_FUNC(slaqtr,SLAQTR)"(bint *ltran, bint *lreal, int *n, s *t, int *ldt, s *b, s *w, s *scale, s *x, s *work, int *info) nogil
 * cdef void slaqtr(bint *ltran, bint *lreal, int *n, s *t, int *ldt, s *b, s *w, s *scale, s *x, s *work, int *info) nogil:
 *     _fortran_slaqtr(ltran, lreal, n, t, ldt, b, w, scale, x, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaqtr,SLAQTR)(__pyx_v_ltran, __pyx_v_lreal, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_b, __pyx_v_w, __pyx_v_scale, __pyx_v_x, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaqtr "F_FUNC(slaqtr,SLAQTR)"(bint *ltran, bint *lreal, int *n, s *t, int *ldt, s *b, s *w, s *scale, s *x, s *work, int *info) nogil
 * cdef void slaqtr(bint *ltran, bint *lreal, int *n, s *t, int *ldt, s *b, s *w, s *scale, s *x, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaqtr(ltran, lreal, n, t, ldt, b, w, scale, x, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slar1v "F_FUNC(slar1v,SLAR1V)"(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, s *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil
 * cdef void slar1v(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, s *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slar1v(int *__pyx_v_n, int *__pyx_v_b1, int *__pyx_v_bn, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lambda_, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ld, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lld, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gaptol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_wantnc, int *__pyx_v_negcnt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ztz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_mingma, int *__pyx_v_r, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_nrminv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_resid, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rqcorr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":5567
 *     void _fortran_slar1v "F_FUNC(slar1v,SLAR1V)"(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, s *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil
 * cdef void slar1v(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, s *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil:
 *     _fortran_slar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slar1v,SLAR1V)(__pyx_v_n, __pyx_v_b1, __pyx_v_bn, __pyx_v_lambda_, __pyx_v_d, __pyx_v_l, __pyx_v_ld, __pyx_v_lld, __pyx_v_pivmin, __pyx_v_gaptol, __pyx_v_z, __pyx_v_wantnc, __pyx_v_negcnt, __pyx_v_ztz, __pyx_v_mingma, __pyx_v_r, __pyx_v_isuppz, __pyx_v_nrminv, __pyx_v_resid, __pyx_v_rqcorr, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":5566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slar1v "F_FUNC(slar1v,SLAR1V)"(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, s *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil
 * cdef void slar1v(int *n, int *b1, int *bn, s *lambda_, s *d, s *l, s *ld, s *lld, s *pivmin, s *gaptol, s *z, bint *wantnc, int *negcnt, s *ztz, s *mingma, int *r, int *isuppz, s *nrminv, s *resid, s *rqcorr, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slar2v "F_FUNC(slar2v,SLAR2V)"(int *n, s *x, s *y, s *z, int *incx, s *c, s *s, int *incc) nogil
 * cdef void slar2v(int *n, s *x, s *y, s *z, int *incx, s *c, s *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slar2v(n, x, y, z, incx, c, s, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slar2v(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":5572
 *     void _fortran_slar2v "F_FUNC(slar2v,SLAR2V)"(int *n, s *x, s *y, s *z, int *incx, s *c, s *s, int *incc) nogil
 * cdef void slar2v(int *n, s *x, s *y, s *z, int *incx, s *c, s *s, int *incc) nogil:
 *     _fortran_slar2v(n, x, y, z, incx, c, s, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slar2v,SLAR2V)(__pyx_v_n, __pyx_v_x, __pyx_v_y, __pyx_v_z, __pyx_v_incx, __pyx_v_c, __pyx_v_s, __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":5571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slar2v "F_FUNC(slar2v,SLAR2V)"(int *n, s *x, s *y, s *z, int *incx, s *c, s *s, int *incc) nogil
 * cdef void slar2v(int *n, s *x, s *y, s *z, int *incx, s *c, s *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slar2v(n, x, y, z, incx, c, s, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarf "F_FUNC(slarf,SLARF)"(char *side, int *m, int *n, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarf(char *side, int *m, int *n, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarf(side, m, n, v, incv, tau, c, ldc, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarf(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_incv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":5577
 *     void _fortran_slarf "F_FUNC(slarf,SLARF)"(char *side, int *m, int *n, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarf(char *side, int *m, int *n, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil:
 *     _fortran_slarf(side, m, n, v, incv, tau, c, ldc, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarf,SLARF)(__pyx_v_side, __pyx_v_m, __pyx_v_n, __pyx_v_v, __pyx_v_incv, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":5576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarf "F_FUNC(slarf,SLARF)"(char *side, int *m, int *n, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarf(char *side, int *m, int *n, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarf(side, m, n, v, incv, tau, c, ldc, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarfb "F_FUNC(slarfb,SLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil
 * cdef void slarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarfb(side, trans, direct, storev, m, n, k, v, ldv, t, ldt, c, ldc, work, ldwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarfb(char *__pyx_v_side, char *__pyx_v_trans, char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_ldwork) {

  /* "scipy/linalg/cython_lapack.pyx":5582
 *     void _fortran_slarfb "F_FUNC(slarfb,SLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil
 * cdef void slarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil:
 *     _fortran_slarfb(side, trans, direct, storev, m, n, k, v, ldv, t, ldt, c, ldc, work, ldwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarfb,SLARFB)(__pyx_v_side, __pyx_v_trans, __pyx_v_direct, __pyx_v_storev, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_v, __pyx_v_ldv, __pyx_v_t, __pyx_v_ldt, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_ldwork);

  /* "scipy/linalg/cython_lapack.pyx":5581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarfb "F_FUNC(slarfb,SLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil
 * cdef void slarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarfb(side, trans, direct, storev, m, n, k, v, ldv, t, ldt, c, ldc, work, ldwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarfg "F_FUNC(slarfg,SLARFG)"(int *n, s *alpha, s *x, int *incx, s *tau) nogil
 * cdef void slarfg(int *n, s *alpha, s *x, int *incx, s *tau) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarfg(n, alpha, x, incx, tau)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarfg(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau) {

  /* "scipy/linalg/cython_lapack.pyx":5587
 *     void _fortran_slarfg "F_FUNC(slarfg,SLARFG)"(int *n, s *alpha, s *x, int *incx, s *tau) nogil
 * cdef void slarfg(int *n, s *alpha, s *x, int *incx, s *tau) nogil:
 *     _fortran_slarfg(n, alpha, x, incx, tau)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarfg,SLARFG)(__pyx_v_n, __pyx_v_alpha, __pyx_v_x, __pyx_v_incx, __pyx_v_tau);

  /* "scipy/linalg/cython_lapack.pyx":5586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarfg "F_FUNC(slarfg,SLARFG)"(int *n, s *alpha, s *x, int *incx, s *tau) nogil
 * cdef void slarfg(int *n, s *alpha, s *x, int *incx, s *tau) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarfg(n, alpha, x, incx, tau)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarft "F_FUNC(slarft,SLARFT)"(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil
 * cdef void slarft(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarft(direct, storev, n, k, v, ldv, tau, t, ldt)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarft(char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt) {

  /* "scipy/linalg/cython_lapack.pyx":5592
 *     void _fortran_slarft "F_FUNC(slarft,SLARFT)"(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil
 * cdef void slarft(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil:
 *     _fortran_slarft(direct, storev, n, k, v, ldv, tau, t, ldt)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarft,SLARFT)(__pyx_v_direct, __pyx_v_storev, __pyx_v_n, __pyx_v_k, __pyx_v_v, __pyx_v_ldv, __pyx_v_tau, __pyx_v_t, __pyx_v_ldt);

  /* "scipy/linalg/cython_lapack.pyx":5591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarft "F_FUNC(slarft,SLARFT)"(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil
 * cdef void slarft(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarft(direct, storev, n, k, v, ldv, tau, t, ldt)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarfx "F_FUNC(slarfx,SLARFX)"(char *side, int *m, int *n, s *v, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarfx(char *side, int *m, int *n, s *v, s *tau, s *c, int *ldc, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarfx(side, m, n, v, tau, c, ldc, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarfx(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":5597
 *     void _fortran_slarfx "F_FUNC(slarfx,SLARFX)"(char *side, int *m, int *n, s *v, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarfx(char *side, int *m, int *n, s *v, s *tau, s *c, int *ldc, s *work) nogil:
 *     _fortran_slarfx(side, m, n, v, tau, c, ldc, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarfx,SLARFX)(__pyx_v_side, __pyx_v_m, __pyx_v_n, __pyx_v_v, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":5596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarfx "F_FUNC(slarfx,SLARFX)"(char *side, int *m, int *n, s *v, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarfx(char *side, int *m, int *n, s *v, s *tau, s *c, int *ldc, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarfx(side, m, n, v, tau, c, ldc, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slargv "F_FUNC(slargv,SLARGV)"(int *n, s *x, int *incx, s *y, int *incy, s *c, int *incc) nogil
 * cdef void slargv(int *n, s *x, int *incx, s *y, int *incy, s *c, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slargv(n, x, incx, y, incy, c, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slargv(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":5602
 *     void _fortran_slargv "F_FUNC(slargv,SLARGV)"(int *n, s *x, int *incx, s *y, int *incy, s *c, int *incc) nogil
 * cdef void slargv(int *n, s *x, int *incx, s *y, int *incy, s *c, int *incc) nogil:
 *     _fortran_slargv(n, x, incx, y, incy, c, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slargv,SLARGV)(__pyx_v_n, __pyx_v_x, __pyx_v_incx, __pyx_v_y, __pyx_v_incy, __pyx_v_c, __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":5601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slargv "F_FUNC(slargv,SLARGV)"(int *n, s *x, int *incx, s *y, int *incy, s *c, int *incc) nogil
 * cdef void slargv(int *n, s *x, int *incx, s *y, int *incy, s *c, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slargv(n, x, incx, y, incy, c, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarnv "F_FUNC(slarnv,SLARNV)"(int *idist, int *iseed, int *n, s *x) nogil
 * cdef void slarnv(int *idist, int *iseed, int *n, s *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarnv(idist, iseed, n, x)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarnv(int *__pyx_v_idist, int *__pyx_v_iseed, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x) {

  /* "scipy/linalg/cython_lapack.pyx":5607
 *     void _fortran_slarnv "F_FUNC(slarnv,SLARNV)"(int *idist, int *iseed, int *n, s *x) nogil
 * cdef void slarnv(int *idist, int *iseed, int *n, s *x) nogil:
 *     _fortran_slarnv(idist, iseed, n, x)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarnv,SLARNV)(__pyx_v_idist, __pyx_v_iseed, __pyx_v_n, __pyx_v_x);

  /* "scipy/linalg/cython_lapack.pyx":5606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarnv "F_FUNC(slarnv,SLARNV)"(int *idist, int *iseed, int *n, s *x) nogil
 * cdef void slarnv(int *idist, int *iseed, int *n, s *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarnv(idist, iseed, n, x)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarra "F_FUNC(slarra,SLARRA)"(int *n, s *d, s *e, s *e2, s *spltol, s *tnrm, int *nsplit, int *isplit, int *info) nogil
 * cdef void slarra(int *n, s *d, s *e, s *e2, s *spltol, s *tnrm, int *nsplit, int *isplit, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarra(n, d, e, e2, spltol, tnrm, nsplit, isplit, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarra(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_spltol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tnrm, int *__pyx_v_nsplit, int *__pyx_v_isplit, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5612
 *     void _fortran_slarra "F_FUNC(slarra,SLARRA)"(int *n, s *d, s *e, s *e2, s *spltol, s *tnrm, int *nsplit, int *isplit, int *info) nogil
 * cdef void slarra(int *n, s *d, s *e, s *e2, s *spltol, s *tnrm, int *nsplit, int *isplit, int *info) nogil:
 *     _fortran_slarra(n, d, e, e2, spltol, tnrm, nsplit, isplit, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarra,SLARRA)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_e2, __pyx_v_spltol, __pyx_v_tnrm, __pyx_v_nsplit, __pyx_v_isplit, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarra "F_FUNC(slarra,SLARRA)"(int *n, s *d, s *e, s *e2, s *spltol, s *tnrm, int *nsplit, int *isplit, int *info) nogil
 * cdef void slarra(int *n, s *d, s *e, s *e2, s *spltol, s *tnrm, int *nsplit, int *isplit, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarra(n, d, e, e2, spltol, tnrm, nsplit, isplit, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrb "F_FUNC(slarrb,SLARRB)"(int *n, s *d, s *lld, int *ifirst, int *ilast, s *rtol1, s *rtol2, int *offset, s *w, s *wgap, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *twist, int *info) nogil
 * cdef void slarrb(int *n, s *d, s *lld, int *ifirst, int *ilast, s *rtol1, s *rtol2, int *offset, s *w, s *wgap, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *twist, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrb(n, d, lld, ifirst, ilast, rtol1, rtol2, offset, w, wgap, werr, work, iwork, pivmin, spdiam, twist, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarrb(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lld, int *__pyx_v_ifirst, int *__pyx_v_ilast, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol2, int *__pyx_v_offset, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wgap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_spdiam, int *__pyx_v_twist, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5617
 *     void _fortran_slarrb "F_FUNC(slarrb,SLARRB)"(int *n, s *d, s *lld, int *ifirst, int *ilast, s *rtol1, s *rtol2, int *offset, s *w, s *wgap, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *twist, int *info) nogil
 * cdef void slarrb(int *n, s *d, s *lld, int *ifirst, int *ilast, s *rtol1, s *rtol2, int *offset, s *w, s *wgap, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *twist, int *info) nogil:
 *     _fortran_slarrb(n, d, lld, ifirst, ilast, rtol1, rtol2, offset, w, wgap, werr, work, iwork, pivmin, spdiam, twist, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarrb,SLARRB)(__pyx_v_n, __pyx_v_d, __pyx_v_lld, __pyx_v_ifirst, __pyx_v_ilast, __pyx_v_rtol1, __pyx_v_rtol2, __pyx_v_offset, __pyx_v_w, __pyx_v_wgap, __pyx_v_werr, __pyx_v_work, __pyx_v_iwork, __pyx_v_pivmin, __pyx_v_spdiam, __pyx_v_twist, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrb "F_FUNC(slarrb,SLARRB)"(int *n, s *d, s *lld, int *ifirst, int *ilast, s *rtol1, s *rtol2, int *offset, s *w, s *wgap, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *twist, int *info) nogil
 * cdef void slarrb(int *n, s *d, s *lld, int *ifirst, int *ilast, s *rtol1, s *rtol2, int *offset, s *w, s *wgap, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *twist, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrb(n, d, lld, ifirst, ilast, rtol1, rtol2, offset, w, wgap, werr, work, iwork, pivmin, spdiam, twist, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrc "F_FUNC(slarrc,SLARRC)"(char *jobt, int *n, s *vl, s *vu, s *d, s *e, s *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil
 * cdef void slarrc(char *jobt, int *n, s *vl, s *vu, s *d, s *e, s *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrc(jobt, n, vl, vu, d, e, pivmin, eigcnt, lcnt, rcnt, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarrc(char *__pyx_v_jobt, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, int *__pyx_v_eigcnt, int *__pyx_v_lcnt, int *__pyx_v_rcnt, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5622
 *     void _fortran_slarrc "F_FUNC(slarrc,SLARRC)"(char *jobt, int *n, s *vl, s *vu, s *d, s *e, s *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil
 * cdef void slarrc(char *jobt, int *n, s *vl, s *vu, s *d, s *e, s *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil:
 *     _fortran_slarrc(jobt, n, vl, vu, d, e, pivmin, eigcnt, lcnt, rcnt, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarrc,SLARRC)(__pyx_v_jobt, __pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_d, __pyx_v_e, __pyx_v_pivmin, __pyx_v_eigcnt, __pyx_v_lcnt, __pyx_v_rcnt, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrc "F_FUNC(slarrc,SLARRC)"(char *jobt, int *n, s *vl, s *vu, s *d, s *e, s *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil
 * cdef void slarrc(char *jobt, int *n, s *vl, s *vu, s *d, s *e, s *pivmin, int *eigcnt, int *lcnt, int *rcnt, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrc(jobt, n, vl, vu, d, e, pivmin, eigcnt, lcnt, rcnt, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrd "F_FUNC(slarrd,SLARRD)"(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *gers, s *reltol, s *d, s *e, s *e2, s *pivmin, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wl, s *wu, int *iblock, int *indexw, s *work, int *iwork, int *info) nogil
 * cdef void slarrd(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *gers, s *reltol, s *d, s *e, s *e2, s *pivmin, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wl, s *wu, int *iblock, int *indexw, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrd(range, order, n, vl, vu, il, iu, gers, reltol, d, e, e2, pivmin, nsplit, isplit, m, w, werr, wl, wu, iblock, indexw, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarrd(char *__pyx_v_range, char *__pyx_v_order, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gers, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_reltol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, int *__pyx_v_nsplit, int *__pyx_v_isplit, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wu, int *__pyx_v_iblock, int *__pyx_v_indexw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5627
 *     void _fortran_slarrd "F_FUNC(slarrd,SLARRD)"(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *gers, s *reltol, s *d, s *e, s *e2, s *pivmin, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wl, s *wu, int *iblock, int *indexw, s *work, int *iwork, int *info) nogil
 * cdef void slarrd(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *gers, s *reltol, s *d, s *e, s *e2, s *pivmin, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wl, s *wu, int *iblock, int *indexw, s *work, int *iwork, int *info) nogil:
 *     _fortran_slarrd(range, order, n, vl, vu, il, iu, gers, reltol, d, e, e2, pivmin, nsplit, isplit, m, w, werr, wl, wu, iblock, indexw, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarrd,SLARRD)(__pyx_v_range, __pyx_v_order, __pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_gers, __pyx_v_reltol, __pyx_v_d, __pyx_v_e, __pyx_v_e2, __pyx_v_pivmin, __pyx_v_nsplit, __pyx_v_isplit, __pyx_v_m, __pyx_v_w, __pyx_v_werr, __pyx_v_wl, __pyx_v_wu, __pyx_v_iblock, __pyx_v_indexw, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrd "F_FUNC(slarrd,SLARRD)"(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *gers, s *reltol, s *d, s *e, s *e2, s *pivmin, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wl, s *wu, int *iblock, int *indexw, s *work, int *iwork, int *info) nogil
 * cdef void slarrd(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *gers, s *reltol, s *d, s *e, s *e2, s *pivmin, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wl, s *wu, int *iblock, int *indexw, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrd(range, order, n, vl, vu, il, iu, gers, reltol, d, e, e2, pivmin, nsplit, isplit, m, w, werr, wl, wu, iblock, indexw, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarre "F_FUNC(slarre,SLARRE)"(char *range, int *n, s *vl, s *vu, int *il, int *iu, s *d, s *e, s *e2, s *rtol1, s *rtol2, s *spltol, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *pivmin, s *work, int *iwork, int *info) nogil
 * cdef void slarre(char *range, int *n, s *vl, s *vu, int *il, int *iu, s *d, s *e, s *e2, s *rtol1, s *rtol2, s *spltol, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *pivmin, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarre(range, n, vl, vu, il, iu, d, e, e2, rtol1, rtol2, spltol, nsplit, isplit, m, w, werr, wgap, iblock, indexw, gers, pivmin, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarre(char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_spltol, int *__pyx_v_nsplit, int *__pyx_v_isplit, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wgap, int *__pyx_v_iblock, int *__pyx_v_indexw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gers, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5632
 *     void _fortran_slarre "F_FUNC(slarre,SLARRE)"(char *range, int *n, s *vl, s *vu, int *il, int *iu, s *d, s *e, s *e2, s *rtol1, s *rtol2, s *spltol, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *pivmin, s *work, int *iwork, int *info) nogil
 * cdef void slarre(char *range, int *n, s *vl, s *vu, int *il, int *iu, s *d, s *e, s *e2, s *rtol1, s *rtol2, s *spltol, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *pivmin, s *work, int *iwork, int *info) nogil:
 *     _fortran_slarre(range, n, vl, vu, il, iu, d, e, e2, rtol1, rtol2, spltol, nsplit, isplit, m, w, werr, wgap, iblock, indexw, gers, pivmin, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarre,SLARRE)(__pyx_v_range, __pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_d, __pyx_v_e, __pyx_v_e2, __pyx_v_rtol1, __pyx_v_rtol2, __pyx_v_spltol, __pyx_v_nsplit, __pyx_v_isplit, __pyx_v_m, __pyx_v_w, __pyx_v_werr, __pyx_v_wgap, __pyx_v_iblock, __pyx_v_indexw, __pyx_v_gers, __pyx_v_pivmin, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarre "F_FUNC(slarre,SLARRE)"(char *range, int *n, s *vl, s *vu, int *il, int *iu, s *d, s *e, s *e2, s *rtol1, s *rtol2, s *spltol, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *pivmin, s *work, int *iwork, int *info) nogil
 * cdef void slarre(char *range, int *n, s *vl, s *vu, int *il, int *iu, s *d, s *e, s *e2, s *rtol1, s *rtol2, s *spltol, int *nsplit, int *isplit, int *m, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *pivmin, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarre(range, n, vl, vu, il, iu, d, e, e2, rtol1, rtol2, spltol, nsplit, isplit, m, w, werr, wgap, iblock, indexw, gers, pivmin, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrf "F_FUNC(slarrf,SLARRF)"(int *n, s *d, s *l, s *ld, int *clstrt, int *clend, s *w, s *wgap, s *werr, s *spdiam, s *clgapl, s *clgapr, s *pivmin, s *sigma, s *dplus, s *lplus, s *work, int *info) nogil
 * cdef void slarrf(int *n, s *d, s *l, s *ld, int *clstrt, int *clend, s *w, s *wgap, s *werr, s *spdiam, s *clgapl, s *clgapr, s *pivmin, s *sigma, s *dplus, s *lplus, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrf(n, d, l, ld, clstrt, clend, w, wgap, werr, spdiam, clgapl, clgapr, pivmin, sigma, dplus, lplus, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarrf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ld, int *__pyx_v_clstrt, int *__pyx_v_clend, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wgap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_spdiam, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_clgapl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_clgapr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sigma, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dplus, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_lplus, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5637
 *     void _fortran_slarrf "F_FUNC(slarrf,SLARRF)"(int *n, s *d, s *l, s *ld, int *clstrt, int *clend, s *w, s *wgap, s *werr, s *spdiam, s *clgapl, s *clgapr, s *pivmin, s *sigma, s *dplus, s *lplus, s *work, int *info) nogil
 * cdef void slarrf(int *n, s *d, s *l, s *ld, int *clstrt, int *clend, s *w, s *wgap, s *werr, s *spdiam, s *clgapl, s *clgapr, s *pivmin, s *sigma, s *dplus, s *lplus, s *work, int *info) nogil:
 *     _fortran_slarrf(n, d, l, ld, clstrt, clend, w, wgap, werr, spdiam, clgapl, clgapr, pivmin, sigma, dplus, lplus, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarrf,SLARRF)(__pyx_v_n, __pyx_v_d, __pyx_v_l, __pyx_v_ld, __pyx_v_clstrt, __pyx_v_clend, __pyx_v_w, __pyx_v_wgap, __pyx_v_werr, __pyx_v_spdiam, __pyx_v_clgapl, __pyx_v_clgapr, __pyx_v_pivmin, __pyx_v_sigma, __pyx_v_dplus, __pyx_v_lplus, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrf "F_FUNC(slarrf,SLARRF)"(int *n, s *d, s *l, s *ld, int *clstrt, int *clend, s *w, s *wgap, s *werr, s *spdiam, s *clgapl, s *clgapr, s *pivmin, s *sigma, s *dplus, s *lplus, s *work, int *info) nogil
 * cdef void slarrf(int *n, s *d, s *l, s *ld, int *clstrt, int *clend, s *w, s *wgap, s *werr, s *spdiam, s *clgapl, s *clgapr, s *pivmin, s *sigma, s *dplus, s *lplus, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrf(n, d, l, ld, clstrt, clend, w, wgap, werr, spdiam, clgapl, clgapr, pivmin, sigma, dplus, lplus, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrj "F_FUNC(slarrj,SLARRJ)"(int *n, s *d, s *e2, int *ifirst, int *ilast, s *rtol, int *offset, s *w, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *info) nogil
 * cdef void slarrj(int *n, s *d, s *e2, int *ifirst, int *ilast, s *rtol, int *offset, s *w, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrj(n, d, e2, ifirst, ilast, rtol, offset, w, werr, work, iwork, pivmin, spdiam, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarrj(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e2, int *__pyx_v_ifirst, int *__pyx_v_ilast, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol, int *__pyx_v_offset, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_spdiam, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5642
 *     void _fortran_slarrj "F_FUNC(slarrj,SLARRJ)"(int *n, s *d, s *e2, int *ifirst, int *ilast, s *rtol, int *offset, s *w, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *info) nogil
 * cdef void slarrj(int *n, s *d, s *e2, int *ifirst, int *ilast, s *rtol, int *offset, s *w, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *info) nogil:
 *     _fortran_slarrj(n, d, e2, ifirst, ilast, rtol, offset, w, werr, work, iwork, pivmin, spdiam, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarrj,SLARRJ)(__pyx_v_n, __pyx_v_d, __pyx_v_e2, __pyx_v_ifirst, __pyx_v_ilast, __pyx_v_rtol, __pyx_v_offset, __pyx_v_w, __pyx_v_werr, __pyx_v_work, __pyx_v_iwork, __pyx_v_pivmin, __pyx_v_spdiam, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrj "F_FUNC(slarrj,SLARRJ)"(int *n, s *d, s *e2, int *ifirst, int *ilast, s *rtol, int *offset, s *w, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *info) nogil
 * cdef void slarrj(int *n, s *d, s *e2, int *ifirst, int *ilast, s *rtol, int *offset, s *w, s *werr, s *work, int *iwork, s *pivmin, s *spdiam, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrj(n, d, e2, ifirst, ilast, rtol, offset, w, werr, work, iwork, pivmin, spdiam, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrk "F_FUNC(slarrk,SLARRK)"(int *n, int *iw, s *gl, s *gu, s *d, s *e2, s *pivmin, s *reltol, s *w, s *werr, int *info) nogil
 * cdef void slarrk(int *n, int *iw, s *gl, s *gu, s *d, s *e2, s *pivmin, s *reltol, s *w, s *werr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrk(n, iw, gl, gu, d, e2, pivmin, reltol, w, werr, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarrk(int *__pyx_v_n, int *__pyx_v_iw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_reltol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_werr, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5647
 *     void _fortran_slarrk "F_FUNC(slarrk,SLARRK)"(int *n, int *iw, s *gl, s *gu, s *d, s *e2, s *pivmin, s *reltol, s *w, s *werr, int *info) nogil
 * cdef void slarrk(int *n, int *iw, s *gl, s *gu, s *d, s *e2, s *pivmin, s *reltol, s *w, s *werr, int *info) nogil:
 *     _fortran_slarrk(n, iw, gl, gu, d, e2, pivmin, reltol, w, werr, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarrk,SLARRK)(__pyx_v_n, __pyx_v_iw, __pyx_v_gl, __pyx_v_gu, __pyx_v_d, __pyx_v_e2, __pyx_v_pivmin, __pyx_v_reltol, __pyx_v_w, __pyx_v_werr, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrk "F_FUNC(slarrk,SLARRK)"(int *n, int *iw, s *gl, s *gu, s *d, s *e2, s *pivmin, s *reltol, s *w, s *werr, int *info) nogil
 * cdef void slarrk(int *n, int *iw, s *gl, s *gu, s *d, s *e2, s *pivmin, s *reltol, s *w, s *werr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrk(n, iw, gl, gu, d, e2, pivmin, reltol, w, werr, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrr "F_FUNC(slarrr,SLARRR)"(int *n, s *d, s *e, int *info) nogil
 * cdef void slarrr(int *n, s *d, s *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrr(n, d, e, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarrr(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5652
 *     void _fortran_slarrr "F_FUNC(slarrr,SLARRR)"(int *n, s *d, s *e, int *info) nogil
 * cdef void slarrr(int *n, s *d, s *e, int *info) nogil:
 *     _fortran_slarrr(n, d, e, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarrr,SLARRR)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrr "F_FUNC(slarrr,SLARRR)"(int *n, s *d, s *e, int *info) nogil
 * cdef void slarrr(int *n, s *d, s *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrr(n, d, e, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrv "F_FUNC(slarrv,SLARRV)"(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil
 * cdef void slarrv(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, z, ldz, isuppz, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarrv(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pivmin, int *__pyx_v_isplit, int *__pyx_v_m, int *__pyx_v_dol, int *__pyx_v_dou, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_minrgp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rtol2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wgap, int *__pyx_v_iblock, int *__pyx_v_indexw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_gers, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5657
 *     void _fortran_slarrv "F_FUNC(slarrv,SLARRV)"(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil
 * cdef void slarrv(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil:
 *     _fortran_slarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, z, ldz, isuppz, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarrv,SLARRV)(__pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_d, __pyx_v_l, __pyx_v_pivmin, __pyx_v_isplit, __pyx_v_m, __pyx_v_dol, __pyx_v_dou, __pyx_v_minrgp, __pyx_v_rtol1, __pyx_v_rtol2, __pyx_v_w, __pyx_v_werr, __pyx_v_wgap, __pyx_v_iblock, __pyx_v_indexw, __pyx_v_gers, __pyx_v_z, __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarrv "F_FUNC(slarrv,SLARRV)"(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil
 * cdef void slarrv(int *n, s *vl, s *vu, s *d, s *l, s *pivmin, int *isplit, int *m, int *dol, int *dou, s *minrgp, s *rtol1, s *rtol2, s *w, s *werr, s *wgap, int *iblock, int *indexw, s *gers, s *z, int *ldz, int *isuppz, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, z, ldz, isuppz, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slartg "F_FUNC(slartg,SLARTG)"(s *f, s *g, s *cs, s *sn, s *r) nogil
 * cdef void slartg(s *f, s *g, s *cs, s *sn, s *r) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slartg(f, g, cs, sn, r)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slartg(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_f, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_g, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sn, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_r) {

  /* "scipy/linalg/cython_lapack.pyx":5662
 *     void _fortran_slartg "F_FUNC(slartg,SLARTG)"(s *f, s *g, s *cs, s *sn, s *r) nogil
 * cdef void slartg(s *f, s *g, s *cs, s *sn, s *r) nogil:
 *     _fortran_slartg(f, g, cs, sn, r)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slartg,SLARTG)(__pyx_v_f, __pyx_v_g, __pyx_v_cs, __pyx_v_sn, __pyx_v_r);

  /* "scipy/linalg/cython_lapack.pyx":5661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slartg "F_FUNC(slartg,SLARTG)"(s *f, s *g, s *cs, s *sn, s *r) nogil
 * cdef void slartg(s *f, s *g, s *cs, s *sn, s *r) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slartg(f, g, cs, sn, r)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slartv "F_FUNC(slartv,SLARTV)"(int *n, s *x, int *incx, s *y, int *incy, s *c, s *s, int *incc) nogil
 * cdef void slartv(int *n, s *x, int *incx, s *y, int *incy, s *c, s *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slartv(n, x, incx, y, incy, c, s, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slartv(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":5667
 *     void _fortran_slartv "F_FUNC(slartv,SLARTV)"(int *n, s *x, int *incx, s *y, int *incy, s *c, s *s, int *incc) nogil
 * cdef void slartv(int *n, s *x, int *incx, s *y, int *incy, s *c, s *s, int *incc) nogil:
 *     _fortran_slartv(n, x, incx, y, incy, c, s, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slartv,SLARTV)(__pyx_v_n, __pyx_v_x, __pyx_v_incx, __pyx_v_y, __pyx_v_incy, __pyx_v_c, __pyx_v_s, __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":5666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slartv "F_FUNC(slartv,SLARTV)"(int *n, s *x, int *incx, s *y, int *incy, s *c, s *s, int *incc) nogil
 * cdef void slartv(int *n, s *x, int *incx, s *y, int *incy, s *c, s *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slartv(n, x, incx, y, incy, c, s, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaruv "F_FUNC(slaruv,SLARUV)"(int *iseed, int *n, s *x) nogil
 * cdef void slaruv(int *iseed, int *n, s *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaruv(iseed, n, x)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaruv(int *__pyx_v_iseed, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x) {

  /* "scipy/linalg/cython_lapack.pyx":5672
 *     void _fortran_slaruv "F_FUNC(slaruv,SLARUV)"(int *iseed, int *n, s *x) nogil
 * cdef void slaruv(int *iseed, int *n, s *x) nogil:
 *     _fortran_slaruv(iseed, n, x)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaruv,SLARUV)(__pyx_v_iseed, __pyx_v_n, __pyx_v_x);

  /* "scipy/linalg/cython_lapack.pyx":5671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaruv "F_FUNC(slaruv,SLARUV)"(int *iseed, int *n, s *x) nogil
 * cdef void slaruv(int *iseed, int *n, s *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaruv(iseed, n, x)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarz "F_FUNC(slarz,SLARZ)"(char *side, int *m, int *n, int *l, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarz(char *side, int *m, int *n, int *l, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarz(side, m, n, l, v, incv, tau, c, ldc, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarz(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_incv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":5677
 *     void _fortran_slarz "F_FUNC(slarz,SLARZ)"(char *side, int *m, int *n, int *l, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarz(char *side, int *m, int *n, int *l, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil:
 *     _fortran_slarz(side, m, n, l, v, incv, tau, c, ldc, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarz,SLARZ)(__pyx_v_side, __pyx_v_m, __pyx_v_n, __pyx_v_l, __pyx_v_v, __pyx_v_incv, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":5676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarz "F_FUNC(slarz,SLARZ)"(char *side, int *m, int *n, int *l, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil
 * cdef void slarz(char *side, int *m, int *n, int *l, s *v, int *incv, s *tau, s *c, int *ldc, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarz(side, m, n, l, v, incv, tau, c, ldc, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarzb "F_FUNC(slarzb,SLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil
 * cdef void slarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarzb(side, trans, direct, storev, m, n, k, l, v, ldv, t, ldt, c, ldc, work, ldwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarzb(char *__pyx_v_side, char *__pyx_v_trans, char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_ldwork) {

  /* "scipy/linalg/cython_lapack.pyx":5682
 *     void _fortran_slarzb "F_FUNC(slarzb,SLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil
 * cdef void slarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil:
 *     _fortran_slarzb(side, trans, direct, storev, m, n, k, l, v, ldv, t, ldt, c, ldc, work, ldwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarzb,SLARZB)(__pyx_v_side, __pyx_v_trans, __pyx_v_direct, __pyx_v_storev, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, __pyx_v_v, __pyx_v_ldv, __pyx_v_t, __pyx_v_ldt, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_ldwork);

  /* "scipy/linalg/cython_lapack.pyx":5681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarzb "F_FUNC(slarzb,SLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil
 * cdef void slarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, s *v, int *ldv, s *t, int *ldt, s *c, int *ldc, s *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarzb(side, trans, direct, storev, m, n, k, l, v, ldv, t, ldt, c, ldc, work, ldwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarzt "F_FUNC(slarzt,SLARZT)"(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil
 * cdef void slarzt(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarzt(direct, storev, n, k, v, ldv, tau, t, ldt)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slarzt(char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt) {

  /* "scipy/linalg/cython_lapack.pyx":5687
 *     void _fortran_slarzt "F_FUNC(slarzt,SLARZT)"(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil
 * cdef void slarzt(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil:
 *     _fortran_slarzt(direct, storev, n, k, v, ldv, tau, t, ldt)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slarzt,SLARZT)(__pyx_v_direct, __pyx_v_storev, __pyx_v_n, __pyx_v_k, __pyx_v_v, __pyx_v_ldv, __pyx_v_tau, __pyx_v_t, __pyx_v_ldt);

  /* "scipy/linalg/cython_lapack.pyx":5686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slarzt "F_FUNC(slarzt,SLARZT)"(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil
 * cdef void slarzt(char *direct, char *storev, int *n, int *k, s *v, int *ldv, s *tau, s *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slarzt(direct, storev, n, k, v, ldv, tau, t, ldt)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slas2 "F_FUNC(slas2,SLAS2)"(s *f, s *g, s *h, s *ssmin, s *ssmax) nogil
 * cdef void slas2(s *f, s *g, s *h, s *ssmin, s *ssmax) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slas2(f, g, h, ssmin, ssmax)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slas2(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_f, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_g, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ssmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ssmax) {

  /* "scipy/linalg/cython_lapack.pyx":5692
 *     void _fortran_slas2 "F_FUNC(slas2,SLAS2)"(s *f, s *g, s *h, s *ssmin, s *ssmax) nogil
 * cdef void slas2(s *f, s *g, s *h, s *ssmin, s *ssmax) nogil:
 *     _fortran_slas2(f, g, h, ssmin, ssmax)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slas2,SLAS2)(__pyx_v_f, __pyx_v_g, __pyx_v_h, __pyx_v_ssmin, __pyx_v_ssmax);

  /* "scipy/linalg/cython_lapack.pyx":5691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slas2 "F_FUNC(slas2,SLAS2)"(s *f, s *g, s *h, s *ssmin, s *ssmax) nogil
 * cdef void slas2(s *f, s *g, s *h, s *ssmin, s *ssmax) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slas2(f, g, h, ssmin, ssmax)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slascl "F_FUNC(slascl,SLASCL)"(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, s *a, int *lda, int *info) nogil
 * cdef void slascl(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slascl(type_bn, kl, ku, cfrom, cto, m, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slascl(char *__pyx_v_type_bn, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cfrom, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cto, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5697
 *     void _fortran_slascl "F_FUNC(slascl,SLASCL)"(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, s *a, int *lda, int *info) nogil
 * cdef void slascl(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, s *a, int *lda, int *info) nogil:
 *     _fortran_slascl(type_bn, kl, ku, cfrom, cto, m, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slascl,SLASCL)(__pyx_v_type_bn, __pyx_v_kl, __pyx_v_ku, __pyx_v_cfrom, __pyx_v_cto, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slascl "F_FUNC(slascl,SLASCL)"(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, s *a, int *lda, int *info) nogil
 * cdef void slascl(char *type_bn, int *kl, int *ku, s *cfrom, s *cto, int *m, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slascl(type_bn, kl, ku, cfrom, cto, m, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd0 "F_FUNC(slasd0,SLASD0)"(int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, int *smlsiz, int *iwork, s *work, int *info) nogil
 * cdef void slasd0(int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, int *smlsiz, int *iwork, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd0(n, sqre, d, e, u, ldu, vt, ldvt, smlsiz, iwork, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd0(int *__pyx_v_n, int *__pyx_v_sqre, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, int *__pyx_v_smlsiz, int *__pyx_v_iwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5702
 *     void _fortran_slasd0 "F_FUNC(slasd0,SLASD0)"(int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, int *smlsiz, int *iwork, s *work, int *info) nogil
 * cdef void slasd0(int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, int *smlsiz, int *iwork, s *work, int *info) nogil:
 *     _fortran_slasd0(n, sqre, d, e, u, ldu, vt, ldvt, smlsiz, iwork, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd0,SLASD0)(__pyx_v_n, __pyx_v_sqre, __pyx_v_d, __pyx_v_e, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_smlsiz, __pyx_v_iwork, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd0 "F_FUNC(slasd0,SLASD0)"(int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, int *smlsiz, int *iwork, s *work, int *info) nogil
 * cdef void slasd0(int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *ldvt, int *smlsiz, int *iwork, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd0(n, sqre, d, e, u, ldu, vt, ldvt, smlsiz, iwork, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd1 "F_FUNC(slasd1,SLASD1)"(int *nl, int *nr, int *sqre, s *d, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, int *idxq, int *iwork, s *work, int *info) nogil
 * cdef void slasd1(int *nl, int *nr, int *sqre, s *d, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, int *idxq, int *iwork, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd1(nl, nr, sqre, d, alpha, beta, u, ldu, vt, ldvt, idxq, iwork, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd1(int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, int *__pyx_v_idxq, int *__pyx_v_iwork, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5707
 *     void _fortran_slasd1 "F_FUNC(slasd1,SLASD1)"(int *nl, int *nr, int *sqre, s *d, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, int *idxq, int *iwork, s *work, int *info) nogil
 * cdef void slasd1(int *nl, int *nr, int *sqre, s *d, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, int *idxq, int *iwork, s *work, int *info) nogil:
 *     _fortran_slasd1(nl, nr, sqre, d, alpha, beta, u, ldu, vt, ldvt, idxq, iwork, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd1,SLASD1)(__pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_d, __pyx_v_alpha, __pyx_v_beta, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_idxq, __pyx_v_iwork, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd1 "F_FUNC(slasd1,SLASD1)"(int *nl, int *nr, int *sqre, s *d, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, int *idxq, int *iwork, s *work, int *info) nogil
 * cdef void slasd1(int *nl, int *nr, int *sqre, s *d, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, int *idxq, int *iwork, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd1(nl, nr, sqre, d, alpha, beta, u, ldu, vt, ldvt, idxq, iwork, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd2 "F_FUNC(slasd2,SLASD2)"(int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, s *dsigma, s *u2, int *ldu2, s *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil
 * cdef void slasd2(int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, s *dsigma, s *u2, int *ldu2, s *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd2(nl, nr, sqre, k, d, z, alpha, beta, u, ldu, vt, ldvt, dsigma, u2, ldu2, vt2, ldvt2, idxp, idx, idxc, idxq, coltyp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd2(int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dsigma, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u2, int *__pyx_v_ldu2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt2, int *__pyx_v_ldvt2, int *__pyx_v_idxp, int *__pyx_v_idx, int *__pyx_v_idxc, int *__pyx_v_idxq, int *__pyx_v_coltyp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5712
 *     void _fortran_slasd2 "F_FUNC(slasd2,SLASD2)"(int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, s *dsigma, s *u2, int *ldu2, s *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil
 * cdef void slasd2(int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, s *dsigma, s *u2, int *ldu2, s *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil:
 *     _fortran_slasd2(nl, nr, sqre, k, d, z, alpha, beta, u, ldu, vt, ldvt, dsigma, u2, ldu2, vt2, ldvt2, idxp, idx, idxc, idxq, coltyp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd2,SLASD2)(__pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_k, __pyx_v_d, __pyx_v_z, __pyx_v_alpha, __pyx_v_beta, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_dsigma, __pyx_v_u2, __pyx_v_ldu2, __pyx_v_vt2, __pyx_v_ldvt2, __pyx_v_idxp, __pyx_v_idx, __pyx_v_idxc, __pyx_v_idxq, __pyx_v_coltyp, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd2 "F_FUNC(slasd2,SLASD2)"(int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, s *dsigma, s *u2, int *ldu2, s *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil
 * cdef void slasd2(int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *alpha, s *beta, s *u, int *ldu, s *vt, int *ldvt, s *dsigma, s *u2, int *ldu2, s *vt2, int *ldvt2, int *idxp, int *idx, int *idxc, int *idxq, int *coltyp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd2(nl, nr, sqre, k, d, z, alpha, beta, u, ldu, vt, ldvt, dsigma, u2, ldu2, vt2, ldvt2, idxp, idx, idxc, idxq, coltyp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd3 "F_FUNC(slasd3,SLASD3)"(int *nl, int *nr, int *sqre, int *k, s *d, s *q, int *ldq, s *dsigma, s *u, int *ldu, s *u2, int *ldu2, s *vt, int *ldvt, s *vt2, int *ldvt2, int *idxc, int *ctot, s *z, int *info) nogil
 * cdef void slasd3(int *nl, int *nr, int *sqre, int *k, s *d, s *q, int *ldq, s *dsigma, s *u, int *ldu, s *u2, int *ldu2, s *vt, int *ldvt, s *vt2, int *ldvt2, int *idxc, int *ctot, s *z, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd3(nl, nr, sqre, k, d, q, ldq, dsigma, u, ldu, u2, ldu2, vt, ldvt, vt2, ldvt2, idxc, ctot, z, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd3(int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dsigma, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u2, int *__pyx_v_ldu2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt2, int *__pyx_v_ldvt2, int *__pyx_v_idxc, int *__pyx_v_ctot, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5717
 *     void _fortran_slasd3 "F_FUNC(slasd3,SLASD3)"(int *nl, int *nr, int *sqre, int *k, s *d, s *q, int *ldq, s *dsigma, s *u, int *ldu, s *u2, int *ldu2, s *vt, int *ldvt, s *vt2, int *ldvt2, int *idxc, int *ctot, s *z, int *info) nogil
 * cdef void slasd3(int *nl, int *nr, int *sqre, int *k, s *d, s *q, int *ldq, s *dsigma, s *u, int *ldu, s *u2, int *ldu2, s *vt, int *ldvt, s *vt2, int *ldvt2, int *idxc, int *ctot, s *z, int *info) nogil:
 *     _fortran_slasd3(nl, nr, sqre, k, d, q, ldq, dsigma, u, ldu, u2, ldu2, vt, ldvt, vt2, ldvt2, idxc, ctot, z, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd3,SLASD3)(__pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_k, __pyx_v_d, __pyx_v_q, __pyx_v_ldq, __pyx_v_dsigma, __pyx_v_u, __pyx_v_ldu, __pyx_v_u2, __pyx_v_ldu2, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_vt2, __pyx_v_ldvt2, __pyx_v_idxc, __pyx_v_ctot, __pyx_v_z, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd3 "F_FUNC(slasd3,SLASD3)"(int *nl, int *nr, int *sqre, int *k, s *d, s *q, int *ldq, s *dsigma, s *u, int *ldu, s *u2, int *ldu2, s *vt, int *ldvt, s *vt2, int *ldvt2, int *idxc, int *ctot, s *z, int *info) nogil
 * cdef void slasd3(int *nl, int *nr, int *sqre, int *k, s *d, s *q, int *ldq, s *dsigma, s *u, int *ldu, s *u2, int *ldu2, s *vt, int *ldvt, s *vt2, int *ldvt2, int *idxc, int *ctot, s *z, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd3(nl, nr, sqre, k, d, q, ldq, dsigma, u, ldu, u2, ldu2, vt, ldvt, vt2, ldvt2, idxc, ctot, z, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd4 "F_FUNC(slasd4,SLASD4)"(int *n, int *i, s *d, s *z, s *delta, s *rho, s *sigma, s *work, int *info) nogil
 * cdef void slasd4(int *n, int *i, s *d, s *z, s *delta, s *rho, s *sigma, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd4(n, i, d, z, delta, rho, sigma, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd4(int *__pyx_v_n, int *__pyx_v_i, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_delta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sigma, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5722
 *     void _fortran_slasd4 "F_FUNC(slasd4,SLASD4)"(int *n, int *i, s *d, s *z, s *delta, s *rho, s *sigma, s *work, int *info) nogil
 * cdef void slasd4(int *n, int *i, s *d, s *z, s *delta, s *rho, s *sigma, s *work, int *info) nogil:
 *     _fortran_slasd4(n, i, d, z, delta, rho, sigma, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd4,SLASD4)(__pyx_v_n, __pyx_v_i, __pyx_v_d, __pyx_v_z, __pyx_v_delta, __pyx_v_rho, __pyx_v_sigma, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd4 "F_FUNC(slasd4,SLASD4)"(int *n, int *i, s *d, s *z, s *delta, s *rho, s *sigma, s *work, int *info) nogil
 * cdef void slasd4(int *n, int *i, s *d, s *z, s *delta, s *rho, s *sigma, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd4(n, i, d, z, delta, rho, sigma, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd5 "F_FUNC(slasd5,SLASD5)"(int *i, s *d, s *z, s *delta, s *rho, s *dsigma, s *work) nogil
 * cdef void slasd5(int *i, s *d, s *z, s *delta, s *rho, s *dsigma, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd5(i, d, z, delta, rho, dsigma, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd5(int *__pyx_v_i, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_delta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rho, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dsigma, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":5727
 *     void _fortran_slasd5 "F_FUNC(slasd5,SLASD5)"(int *i, s *d, s *z, s *delta, s *rho, s *dsigma, s *work) nogil
 * cdef void slasd5(int *i, s *d, s *z, s *delta, s *rho, s *dsigma, s *work) nogil:
 *     _fortran_slasd5(i, d, z, delta, rho, dsigma, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd5,SLASD5)(__pyx_v_i, __pyx_v_d, __pyx_v_z, __pyx_v_delta, __pyx_v_rho, __pyx_v_dsigma, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":5726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd5 "F_FUNC(slasd5,SLASD5)"(int *i, s *d, s *z, s *delta, s *rho, s *dsigma, s *work) nogil
 * cdef void slasd5(int *i, s *d, s *z, s *delta, s *rho, s *dsigma, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd5(i, d, z, delta, rho, dsigma, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd6 "F_FUNC(slasd6,SLASD6)"(int *icompq, int *nl, int *nr, int *sqre, s *d, s *vf, s *vl, s *alpha, s *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slasd6(int *icompq, int *nl, int *nr, int *sqre, s *d, s *vf, s *vl, s *alpha, s *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd6(icompq, nl, nr, sqre, d, vf, vl, alpha, beta, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd6(int *__pyx_v_icompq, int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, int *__pyx_v_idxq, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, int *__pyx_v_ldgnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_poles, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5732
 *     void _fortran_slasd6 "F_FUNC(slasd6,SLASD6)"(int *icompq, int *nl, int *nr, int *sqre, s *d, s *vf, s *vl, s *alpha, s *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slasd6(int *icompq, int *nl, int *nr, int *sqre, s *d, s *vf, s *vl, s *alpha, s *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *iwork, int *info) nogil:
 *     _fortran_slasd6(icompq, nl, nr, sqre, d, vf, vl, alpha, beta, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd6,SLASD6)(__pyx_v_icompq, __pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_d, __pyx_v_vf, __pyx_v_vl, __pyx_v_alpha, __pyx_v_beta, __pyx_v_idxq, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_givnum, __pyx_v_ldgnum, __pyx_v_poles, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_k, __pyx_v_c, __pyx_v_s, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd6 "F_FUNC(slasd6,SLASD6)"(int *icompq, int *nl, int *nr, int *sqre, s *d, s *vf, s *vl, s *alpha, s *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slasd6(int *icompq, int *nl, int *nr, int *sqre, s *d, s *vf, s *vl, s *alpha, s *beta, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *poles, s *difl, s *difr, s *z, int *k, s *c, s *s, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd6(icompq, nl, nr, sqre, d, vf, vl, alpha, beta, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd7 "F_FUNC(slasd7,SLASD7)"(int *icompq, int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *zw, s *vf, s *vfw, s *vl, s *vlw, s *alpha, s *beta, s *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *c, s *s, int *info) nogil
 * cdef void slasd7(int *icompq, int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *zw, s *vf, s *vfw, s *vl, s *vlw, s *alpha, s *beta, s *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *c, s *s, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd7(icompq, nl, nr, sqre, k, d, z, zw, vf, vfw, vl, vlw, alpha, beta, dsigma, idx, idxp, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, c, s, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd7(int *__pyx_v_icompq, int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_zw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vfw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vlw, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dsigma, int *__pyx_v_idx, int *__pyx_v_idxp, int *__pyx_v_idxq, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, int *__pyx_v_ldgnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5737
 *     void _fortran_slasd7 "F_FUNC(slasd7,SLASD7)"(int *icompq, int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *zw, s *vf, s *vfw, s *vl, s *vlw, s *alpha, s *beta, s *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *c, s *s, int *info) nogil
 * cdef void slasd7(int *icompq, int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *zw, s *vf, s *vfw, s *vl, s *vlw, s *alpha, s *beta, s *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *c, s *s, int *info) nogil:
 *     _fortran_slasd7(icompq, nl, nr, sqre, k, d, z, zw, vf, vfw, vl, vlw, alpha, beta, dsigma, idx, idxp, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, c, s, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd7,SLASD7)(__pyx_v_icompq, __pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_k, __pyx_v_d, __pyx_v_z, __pyx_v_zw, __pyx_v_vf, __pyx_v_vfw, __pyx_v_vl, __pyx_v_vlw, __pyx_v_alpha, __pyx_v_beta, __pyx_v_dsigma, __pyx_v_idx, __pyx_v_idxp, __pyx_v_idxq, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_givnum, __pyx_v_ldgnum, __pyx_v_c, __pyx_v_s, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd7 "F_FUNC(slasd7,SLASD7)"(int *icompq, int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *zw, s *vf, s *vfw, s *vl, s *vlw, s *alpha, s *beta, s *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *c, s *s, int *info) nogil
 * cdef void slasd7(int *icompq, int *nl, int *nr, int *sqre, int *k, s *d, s *z, s *zw, s *vf, s *vfw, s *vl, s *vlw, s *alpha, s *beta, s *dsigma, int *idx, int *idxp, int *idxq, int *perm, int *givptr, int *givcol, int *ldgcol, s *givnum, int *ldgnum, s *c, s *s, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd7(icompq, nl, nr, sqre, k, d, z, zw, vf, vfw, vl, vlw, alpha, beta, dsigma, idx, idxp, idxq, perm, givptr, givcol, ldgcol, givnum, ldgnum, c, s, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd8 "F_FUNC(slasd8,SLASD8)"(int *icompq, int *k, s *d, s *z, s *vf, s *vl, s *difl, s *difr, int *lddifr, s *dsigma, s *work, int *info) nogil
 * cdef void slasd8(int *icompq, int *k, s *d, s *z, s *vf, s *vl, s *difl, s *difr, int *lddifr, s *dsigma, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd8(icompq, k, d, z, vf, vl, difl, difr, lddifr, dsigma, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasd8(int *__pyx_v_icompq, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difr, int *__pyx_v_lddifr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dsigma, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5742
 *     void _fortran_slasd8 "F_FUNC(slasd8,SLASD8)"(int *icompq, int *k, s *d, s *z, s *vf, s *vl, s *difl, s *difr, int *lddifr, s *dsigma, s *work, int *info) nogil
 * cdef void slasd8(int *icompq, int *k, s *d, s *z, s *vf, s *vl, s *difl, s *difr, int *lddifr, s *dsigma, s *work, int *info) nogil:
 *     _fortran_slasd8(icompq, k, d, z, vf, vl, difl, difr, lddifr, dsigma, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasd8,SLASD8)(__pyx_v_icompq, __pyx_v_k, __pyx_v_d, __pyx_v_z, __pyx_v_vf, __pyx_v_vl, __pyx_v_difl, __pyx_v_difr, __pyx_v_lddifr, __pyx_v_dsigma, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasd8 "F_FUNC(slasd8,SLASD8)"(int *icompq, int *k, s *d, s *z, s *vf, s *vl, s *difl, s *difr, int *lddifr, s *dsigma, s *work, int *info) nogil
 * cdef void slasd8(int *icompq, int *k, s *d, s *z, s *vf, s *vl, s *difl, s *difr, int *lddifr, s *dsigma, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasd8(icompq, k, d, z, vf, vl, difl, difr, lddifr, dsigma, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasda "F_FUNC(slasda,SLASDA)"(int *icompq, int *smlsiz, int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slasda(int *icompq, int *smlsiz, int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasda(icompq, smlsiz, n, sqre, d, e, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasda(int *__pyx_v_icompq, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_sqre, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_poles, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, int *__pyx_v_perm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5747
 *     void _fortran_slasda "F_FUNC(slasda,SLASDA)"(int *icompq, int *smlsiz, int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slasda(int *icompq, int *smlsiz, int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil:
 *     _fortran_slasda(icompq, smlsiz, n, sqre, d, e, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasda,SLASDA)(__pyx_v_icompq, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_sqre, __pyx_v_d, __pyx_v_e, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_k, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_poles, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_perm, __pyx_v_givnum, __pyx_v_c, __pyx_v_s, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasda "F_FUNC(slasda,SLASDA)"(int *icompq, int *smlsiz, int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil
 * cdef void slasda(int *icompq, int *smlsiz, int *n, int *sqre, s *d, s *e, s *u, int *ldu, s *vt, int *k, s *difl, s *difr, s *z, s *poles, int *givptr, int *givcol, int *ldgcol, int *perm, s *givnum, s *c, s *s, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasda(icompq, smlsiz, n, sqre, d, e, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasdq "F_FUNC(slasdq,SLASDQ)"(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil
 * cdef void slasdq(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasdq(uplo, sqre, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasdq(char *__pyx_v_uplo, int *__pyx_v_sqre, int *__pyx_v_n, int *__pyx_v_ncvt, int *__pyx_v_nru, int *__pyx_v_ncc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5752
 *     void _fortran_slasdq "F_FUNC(slasdq,SLASDQ)"(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil
 * cdef void slasdq(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_slasdq(uplo, sqre, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasdq,SLASDQ)(__pyx_v_uplo, __pyx_v_sqre, __pyx_v_n, __pyx_v_ncvt, __pyx_v_nru, __pyx_v_ncc, __pyx_v_d, __pyx_v_e, __pyx_v_vt, __pyx_v_ldvt, __pyx_v_u, __pyx_v_ldu, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasdq "F_FUNC(slasdq,SLASDQ)"(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil
 * cdef void slasdq(char *uplo, int *sqre, int *n, int *ncvt, int *nru, int *ncc, s *d, s *e, s *vt, int *ldvt, s *u, int *ldu, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasdq(uplo, sqre, n, ncvt, nru, ncc, d, e, vt, ldvt, u, ldu, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasdt "F_FUNC(slasdt,SLASDT)"(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil
 * cdef void slasdt(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasdt(n, lvl, nd, inode, ndiml, ndimr, msub)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasdt(int *__pyx_v_n, int *__pyx_v_lvl, int *__pyx_v_nd, int *__pyx_v_inode, int *__pyx_v_ndiml, int *__pyx_v_ndimr, int *__pyx_v_msub) {

  /* "scipy/linalg/cython_lapack.pyx":5757
 *     void _fortran_slasdt "F_FUNC(slasdt,SLASDT)"(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil
 * cdef void slasdt(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil:
 *     _fortran_slasdt(n, lvl, nd, inode, ndiml, ndimr, msub)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasdt,SLASDT)(__pyx_v_n, __pyx_v_lvl, __pyx_v_nd, __pyx_v_inode, __pyx_v_ndiml, __pyx_v_ndimr, __pyx_v_msub);

  /* "scipy/linalg/cython_lapack.pyx":5756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasdt "F_FUNC(slasdt,SLASDT)"(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil
 * cdef void slasdt(int *n, int *lvl, int *nd, int *inode, int *ndiml, int *ndimr, int *msub) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasdt(n, lvl, nd, inode, ndiml, ndimr, msub)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaset "F_FUNC(slaset,SLASET)"(char *uplo, int *m, int *n, s *alpha, s *beta, s *a, int *lda) nogil
 * cdef void slaset(char *uplo, int *m, int *n, s *alpha, s *beta, s *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaset(uplo, m, n, alpha, beta, a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaset(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":5762
 *     void _fortran_slaset "F_FUNC(slaset,SLASET)"(char *uplo, int *m, int *n, s *alpha, s *beta, s *a, int *lda) nogil
 * cdef void slaset(char *uplo, int *m, int *n, s *alpha, s *beta, s *a, int *lda) nogil:
 *     _fortran_slaset(uplo, m, n, alpha, beta, a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaset,SLASET)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, __pyx_v_alpha, __pyx_v_beta, __pyx_v_a, __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":5761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaset "F_FUNC(slaset,SLASET)"(char *uplo, int *m, int *n, s *alpha, s *beta, s *a, int *lda) nogil
 * cdef void slaset(char *uplo, int *m, int *n, s *alpha, s *beta, s *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaset(uplo, m, n, alpha, beta, a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasq1 "F_FUNC(slasq1,SLASQ1)"(int *n, s *d, s *e, s *work, int *info) nogil
 * cdef void slasq1(int *n, s *d, s *e, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasq1(n, d, e, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasq1(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5767
 *     void _fortran_slasq1 "F_FUNC(slasq1,SLASQ1)"(int *n, s *d, s *e, s *work, int *info) nogil
 * cdef void slasq1(int *n, s *d, s *e, s *work, int *info) nogil:
 *     _fortran_slasq1(n, d, e, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasq1,SLASQ1)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasq1 "F_FUNC(slasq1,SLASQ1)"(int *n, s *d, s *e, s *work, int *info) nogil
 * cdef void slasq1(int *n, s *d, s *e, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasq1(n, d, e, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasq2 "F_FUNC(slasq2,SLASQ2)"(int *n, s *z, int *info) nogil
 * cdef void slasq2(int *n, s *z, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasq2(n, z, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasq2(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5772
 *     void _fortran_slasq2 "F_FUNC(slasq2,SLASQ2)"(int *n, s *z, int *info) nogil
 * cdef void slasq2(int *n, s *z, int *info) nogil:
 *     _fortran_slasq2(n, z, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasq2,SLASQ2)(__pyx_v_n, __pyx_v_z, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasq2 "F_FUNC(slasq2,SLASQ2)"(int *n, s *z, int *info) nogil
 * cdef void slasq2(int *n, s *z, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasq2(n, z, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasq6 "F_FUNC(slasq6,SLASQ6)"(int *i0, int *n0, s *z, int *pp, s *dmin, s *dmin1, s *dmin2, s *dn, s *dnm1, s *dnm2) nogil
 * cdef void slasq6(int *i0, int *n0, s *z, int *pp, s *dmin, s *dmin1, s *dmin2, s *dn, s *dnm1, s *dnm2) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasq6(i0, n0, z, pp, dmin, dmin1, dmin2, dn, dnm1, dnm2)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasq6(int *__pyx_v_i0, int *__pyx_v_n0, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_pp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dmin1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dmin2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dn, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dnm1, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dnm2) {

  /* "scipy/linalg/cython_lapack.pyx":5777
 *     void _fortran_slasq6 "F_FUNC(slasq6,SLASQ6)"(int *i0, int *n0, s *z, int *pp, s *dmin, s *dmin1, s *dmin2, s *dn, s *dnm1, s *dnm2) nogil
 * cdef void slasq6(int *i0, int *n0, s *z, int *pp, s *dmin, s *dmin1, s *dmin2, s *dn, s *dnm1, s *dnm2) nogil:
 *     _fortran_slasq6(i0, n0, z, pp, dmin, dmin1, dmin2, dn, dnm1, dnm2)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasq6,SLASQ6)(__pyx_v_i0, __pyx_v_n0, __pyx_v_z, __pyx_v_pp, __pyx_v_dmin, __pyx_v_dmin1, __pyx_v_dmin2, __pyx_v_dn, __pyx_v_dnm1, __pyx_v_dnm2);

  /* "scipy/linalg/cython_lapack.pyx":5776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasq6 "F_FUNC(slasq6,SLASQ6)"(int *i0, int *n0, s *z, int *pp, s *dmin, s *dmin1, s *dmin2, s *dn, s *dnm1, s *dnm2) nogil
 * cdef void slasq6(int *i0, int *n0, s *z, int *pp, s *dmin, s *dmin1, s *dmin2, s *dn, s *dnm1, s *dnm2) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasq6(i0, n0, z, pp, dmin, dmin1, dmin2, dn, dnm1, dnm2)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasr "F_FUNC(slasr,SLASR)"(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, s *a, int *lda) nogil
 * cdef void slasr(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, s *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasr(side, pivot, direct, m, n, c, s, a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasr(char *__pyx_v_side, char *__pyx_v_pivot, char *__pyx_v_direct, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":5782
 *     void _fortran_slasr "F_FUNC(slasr,SLASR)"(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, s *a, int *lda) nogil
 * cdef void slasr(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, s *a, int *lda) nogil:
 *     _fortran_slasr(side, pivot, direct, m, n, c, s, a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasr,SLASR)(__pyx_v_side, __pyx_v_pivot, __pyx_v_direct, __pyx_v_m, __pyx_v_n, __pyx_v_c, __pyx_v_s, __pyx_v_a, __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":5781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasr "F_FUNC(slasr,SLASR)"(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, s *a, int *lda) nogil
 * cdef void slasr(char *side, char *pivot, char *direct, int *m, int *n, s *c, s *s, s *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasr(side, pivot, direct, m, n, c, s, a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasrt "F_FUNC(slasrt,SLASRT)"(char *id, int *n, s *d, int *info) nogil
 * cdef void slasrt(char *id, int *n, s *d, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasrt(id, n, d, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasrt(char *__pyx_v_id, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5787
 *     void _fortran_slasrt "F_FUNC(slasrt,SLASRT)"(char *id, int *n, s *d, int *info) nogil
 * cdef void slasrt(char *id, int *n, s *d, int *info) nogil:
 *     _fortran_slasrt(id, n, d, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasrt,SLASRT)(__pyx_v_id, __pyx_v_n, __pyx_v_d, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasrt "F_FUNC(slasrt,SLASRT)"(char *id, int *n, s *d, int *info) nogil
 * cdef void slasrt(char *id, int *n, s *d, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasrt(id, n, d, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slassq "F_FUNC(slassq,SLASSQ)"(int *n, s *x, int *incx, s *scale, s *sumsq) nogil
 * cdef void slassq(int *n, s *x, int *incx, s *scale, s *sumsq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slassq(n, x, incx, scale, sumsq)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slassq(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sumsq) {

  /* "scipy/linalg/cython_lapack.pyx":5792
 *     void _fortran_slassq "F_FUNC(slassq,SLASSQ)"(int *n, s *x, int *incx, s *scale, s *sumsq) nogil
 * cdef void slassq(int *n, s *x, int *incx, s *scale, s *sumsq) nogil:
 *     _fortran_slassq(n, x, incx, scale, sumsq)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slassq,SLASSQ)(__pyx_v_n, __pyx_v_x, __pyx_v_incx, __pyx_v_scale, __pyx_v_sumsq);

  /* "scipy/linalg/cython_lapack.pyx":5791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slassq "F_FUNC(slassq,SLASSQ)"(int *n, s *x, int *incx, s *scale, s *sumsq) nogil
 * cdef void slassq(int *n, s *x, int *incx, s *scale, s *sumsq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slassq(n, x, incx, scale, sumsq)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasv2 "F_FUNC(slasv2,SLASV2)"(s *f, s *g, s *h, s *ssmin, s *ssmax, s *snr, s *csr, s *snl, s *csl) nogil
 * cdef void slasv2(s *f, s *g, s *h, s *ssmin, s *ssmax, s *snr, s *csr, s *snl, s *csl) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasv2(f, g, h, ssmin, ssmax, snr, csr, snl, csl)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasv2(__pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_f, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_g, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_h, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ssmin, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ssmax, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_snr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_snl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_csl) {

  /* "scipy/linalg/cython_lapack.pyx":5797
 *     void _fortran_slasv2 "F_FUNC(slasv2,SLASV2)"(s *f, s *g, s *h, s *ssmin, s *ssmax, s *snr, s *csr, s *snl, s *csl) nogil
 * cdef void slasv2(s *f, s *g, s *h, s *ssmin, s *ssmax, s *snr, s *csr, s *snl, s *csl) nogil:
 *     _fortran_slasv2(f, g, h, ssmin, ssmax, snr, csr, snl, csl)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasv2,SLASV2)(__pyx_v_f, __pyx_v_g, __pyx_v_h, __pyx_v_ssmin, __pyx_v_ssmax, __pyx_v_snr, __pyx_v_csr, __pyx_v_snl, __pyx_v_csl);

  /* "scipy/linalg/cython_lapack.pyx":5796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasv2 "F_FUNC(slasv2,SLASV2)"(s *f, s *g, s *h, s *ssmin, s *ssmax, s *snr, s *csr, s *snl, s *csl) nogil
 * cdef void slasv2(s *f, s *g, s *h, s *ssmin, s *ssmax, s *snr, s *csr, s *snl, s *csl) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasv2(f, g, h, ssmin, ssmax, snr, csr, snl, csl)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaswp "F_FUNC(slaswp,SLASWP)"(int *n, s *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void slaswp(int *n, s *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaswp(n, a, lda, k1, k2, ipiv, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slaswp(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_k1, int *__pyx_v_k2, int *__pyx_v_ipiv, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":5802
 *     void _fortran_slaswp "F_FUNC(slaswp,SLASWP)"(int *n, s *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void slaswp(int *n, s *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:
 *     _fortran_slaswp(n, a, lda, k1, k2, ipiv, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slaswp,SLASWP)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_k1, __pyx_v_k2, __pyx_v_ipiv, __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":5801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slaswp "F_FUNC(slaswp,SLASWP)"(int *n, s *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void slaswp(int *n, s *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slaswp(n, a, lda, k1, k2, ipiv, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasy2 "F_FUNC(slasy2,SLASY2)"(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, s *tl, int *ldtl, s *tr, int *ldtr, s *b, int *ldb, s *scale, s *x, int *ldx, s *xnorm, int *info) nogil
 * cdef void slasy2(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, s *tl, int *ldtl, s *tr, int *ldtr, s *b, int *ldb, s *scale, s *x, int *ldx, s *xnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasy2(ltranl, ltranr, isgn, n1, n2, tl, ldtl, tr, ldtr, b, ldb, scale, x, ldx, xnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasy2(int *__pyx_v_ltranl, int *__pyx_v_ltranr, int *__pyx_v_isgn, int *__pyx_v_n1, int *__pyx_v_n2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tl, int *__pyx_v_ldtl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tr, int *__pyx_v_ldtr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_xnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5807
 *     void _fortran_slasy2 "F_FUNC(slasy2,SLASY2)"(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, s *tl, int *ldtl, s *tr, int *ldtr, s *b, int *ldb, s *scale, s *x, int *ldx, s *xnorm, int *info) nogil
 * cdef void slasy2(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, s *tl, int *ldtl, s *tr, int *ldtr, s *b, int *ldb, s *scale, s *x, int *ldx, s *xnorm, int *info) nogil:
 *     _fortran_slasy2(ltranl, ltranr, isgn, n1, n2, tl, ldtl, tr, ldtr, b, ldb, scale, x, ldx, xnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasy2,SLASY2)(__pyx_v_ltranl, __pyx_v_ltranr, __pyx_v_isgn, __pyx_v_n1, __pyx_v_n2, __pyx_v_tl, __pyx_v_ldtl, __pyx_v_tr, __pyx_v_ldtr, __pyx_v_b, __pyx_v_ldb, __pyx_v_scale, __pyx_v_x, __pyx_v_ldx, __pyx_v_xnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasy2 "F_FUNC(slasy2,SLASY2)"(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, s *tl, int *ldtl, s *tr, int *ldtr, s *b, int *ldb, s *scale, s *x, int *ldx, s *xnorm, int *info) nogil
 * cdef void slasy2(bint *ltranl, bint *ltranr, int *isgn, int *n1, int *n2, s *tl, int *ldtl, s *tr, int *ldtr, s *b, int *ldb, s *scale, s *x, int *ldx, s *xnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasy2(ltranl, ltranr, isgn, n1, n2, tl, ldtl, tr, ldtr, b, ldb, scale, x, ldx, xnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasyf "F_FUNC(slasyf,SLASYF)"(char *uplo, int *n, int *nb, int *kb, s *a, int *lda, int *ipiv, s *w, int *ldw, int *info) nogil
 * cdef void slasyf(char *uplo, int *n, int *nb, int *kb, s *a, int *lda, int *ipiv, s *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasyf(uplo, n, nb, kb, a, lda, ipiv, w, ldw, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slasyf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, int *__pyx_v_ldw, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5812
 *     void _fortran_slasyf "F_FUNC(slasyf,SLASYF)"(char *uplo, int *n, int *nb, int *kb, s *a, int *lda, int *ipiv, s *w, int *ldw, int *info) nogil
 * cdef void slasyf(char *uplo, int *n, int *nb, int *kb, s *a, int *lda, int *ipiv, s *w, int *ldw, int *info) nogil:
 *     _fortran_slasyf(uplo, n, nb, kb, a, lda, ipiv, w, ldw, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slasyf,SLASYF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, __pyx_v_kb, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_w, __pyx_v_ldw, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slasyf "F_FUNC(slasyf,SLASYF)"(char *uplo, int *n, int *nb, int *kb, s *a, int *lda, int *ipiv, s *w, int *ldw, int *info) nogil
 * cdef void slasyf(char *uplo, int *n, int *nb, int *kb, s *a, int *lda, int *ipiv, s *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slasyf(uplo, n, nb, kb, a, lda, ipiv, w, ldw, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatbs "F_FUNC(slatbs,SLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, s *ab, int *ldab, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, s *ab, int *ldab, s *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatbs(uplo, trans, diag, normin, n, kd, ab, ldab, x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slatbs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5817
 *     void _fortran_slatbs "F_FUNC(slatbs,SLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, s *ab, int *ldab, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, s *ab, int *ldab, s *x, s *scale, s *cnorm, int *info) nogil:
 *     _fortran_slatbs(uplo, trans, diag, normin, n, kd, ab, ldab, x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slatbs,SLATBS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_x, __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatbs "F_FUNC(slatbs,SLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, s *ab, int *ldab, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, s *ab, int *ldab, s *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatbs(uplo, trans, diag, normin, n, kd, ab, ldab, x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatdf "F_FUNC(slatdf,SLATDF)"(int *ijob, int *n, s *z, int *ldz, s *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void slatdf(int *ijob, int *n, s *z, int *ldz, s *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatdf(ijob, n, z, ldz, rhs, rdsum, rdscal, ipiv, jpiv)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slatdf(int *__pyx_v_ijob, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rdsum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rdscal, int *__pyx_v_ipiv, int *__pyx_v_jpiv) {

  /* "scipy/linalg/cython_lapack.pyx":5822
 *     void _fortran_slatdf "F_FUNC(slatdf,SLATDF)"(int *ijob, int *n, s *z, int *ldz, s *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void slatdf(int *ijob, int *n, s *z, int *ldz, s *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil:
 *     _fortran_slatdf(ijob, n, z, ldz, rhs, rdsum, rdscal, ipiv, jpiv)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slatdf,SLATDF)(__pyx_v_ijob, __pyx_v_n, __pyx_v_z, __pyx_v_ldz, __pyx_v_rhs, __pyx_v_rdsum, __pyx_v_rdscal, __pyx_v_ipiv, __pyx_v_jpiv);

  /* "scipy/linalg/cython_lapack.pyx":5821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatdf "F_FUNC(slatdf,SLATDF)"(int *ijob, int *n, s *z, int *ldz, s *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void slatdf(int *ijob, int *n, s *z, int *ldz, s *rhs, s *rdsum, s *rdscal, int *ipiv, int *jpiv) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatdf(ijob, n, z, ldz, rhs, rdsum, rdscal, ipiv, jpiv)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatps "F_FUNC(slatps,SLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, s *ap, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatps(char *uplo, char *trans, char *diag, char *normin, int *n, s *ap, s *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatps(uplo, trans, diag, normin, n, ap, x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slatps(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5827
 *     void _fortran_slatps "F_FUNC(slatps,SLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, s *ap, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatps(char *uplo, char *trans, char *diag, char *normin, int *n, s *ap, s *x, s *scale, s *cnorm, int *info) nogil:
 *     _fortran_slatps(uplo, trans, diag, normin, n, ap, x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slatps,SLATPS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, __pyx_v_ap, __pyx_v_x, __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatps "F_FUNC(slatps,SLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, s *ap, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatps(char *uplo, char *trans, char *diag, char *normin, int *n, s *ap, s *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatps(uplo, trans, diag, normin, n, ap, x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatrd "F_FUNC(slatrd,SLATRD)"(char *uplo, int *n, int *nb, s *a, int *lda, s *e, s *tau, s *w, int *ldw) nogil
 * cdef void slatrd(char *uplo, int *n, int *nb, s *a, int *lda, s *e, s *tau, s *w, int *ldw) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatrd(uplo, n, nb, a, lda, e, tau, w, ldw)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slatrd(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, int *__pyx_v_ldw) {

  /* "scipy/linalg/cython_lapack.pyx":5832
 *     void _fortran_slatrd "F_FUNC(slatrd,SLATRD)"(char *uplo, int *n, int *nb, s *a, int *lda, s *e, s *tau, s *w, int *ldw) nogil
 * cdef void slatrd(char *uplo, int *n, int *nb, s *a, int *lda, s *e, s *tau, s *w, int *ldw) nogil:
 *     _fortran_slatrd(uplo, n, nb, a, lda, e, tau, w, ldw)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slatrd,SLATRD)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, __pyx_v_a, __pyx_v_lda, __pyx_v_e, __pyx_v_tau, __pyx_v_w, __pyx_v_ldw);

  /* "scipy/linalg/cython_lapack.pyx":5831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatrd "F_FUNC(slatrd,SLATRD)"(char *uplo, int *n, int *nb, s *a, int *lda, s *e, s *tau, s *w, int *ldw) nogil
 * cdef void slatrd(char *uplo, int *n, int *nb, s *a, int *lda, s *e, s *tau, s *w, int *ldw) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatrd(uplo, n, nb, a, lda, e, tau, w, ldw)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatrs "F_FUNC(slatrs,SLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, s *a, int *lda, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatrs(char *uplo, char *trans, char *diag, char *normin, int *n, s *a, int *lda, s *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatrs(uplo, trans, diag, normin, n, a, lda, x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slatrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5837
 *     void _fortran_slatrs "F_FUNC(slatrs,SLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, s *a, int *lda, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatrs(char *uplo, char *trans, char *diag, char *normin, int *n, s *a, int *lda, s *x, s *scale, s *cnorm, int *info) nogil:
 *     _fortran_slatrs(uplo, trans, diag, normin, n, a, lda, x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slatrs,SLATRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_x, __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatrs "F_FUNC(slatrs,SLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, s *a, int *lda, s *x, s *scale, s *cnorm, int *info) nogil
 * cdef void slatrs(char *uplo, char *trans, char *diag, char *normin, int *n, s *a, int *lda, s *x, s *scale, s *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatrs(uplo, trans, diag, normin, n, a, lda, x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatrz "F_FUNC(slatrz,SLATRZ)"(int *m, int *n, int *l, s *a, int *lda, s *tau, s *work) nogil
 * cdef void slatrz(int *m, int *n, int *l, s *a, int *lda, s *tau, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatrz(m, n, l, a, lda, tau, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slatrz(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":5842
 *     void _fortran_slatrz "F_FUNC(slatrz,SLATRZ)"(int *m, int *n, int *l, s *a, int *lda, s *tau, s *work) nogil
 * cdef void slatrz(int *m, int *n, int *l, s *a, int *lda, s *tau, s *work) nogil:
 *     _fortran_slatrz(m, n, l, a, lda, tau, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slatrz,SLATRZ)(__pyx_v_m, __pyx_v_n, __pyx_v_l, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":5841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slatrz "F_FUNC(slatrz,SLATRZ)"(int *m, int *n, int *l, s *a, int *lda, s *tau, s *work) nogil
 * cdef void slatrz(int *m, int *n, int *l, s *a, int *lda, s *tau, s *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slatrz(m, n, l, a, lda, tau, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slauu2 "F_FUNC(slauu2,SLAUU2)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void slauu2(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slauu2(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slauu2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5847
 *     void _fortran_slauu2 "F_FUNC(slauu2,SLAUU2)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void slauu2(char *uplo, int *n, s *a, int *lda, int *info) nogil:
 *     _fortran_slauu2(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slauu2,SLAUU2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slauu2 "F_FUNC(slauu2,SLAUU2)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void slauu2(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slauu2(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slauum "F_FUNC(slauum,SLAUUM)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void slauum(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slauum(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_slauum(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5852
 *     void _fortran_slauum "F_FUNC(slauum,SLAUUM)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void slauum(char *uplo, int *n, s *a, int *lda, int *info) nogil:
 *     _fortran_slauum(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(slauum,SLAUUM)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_slauum "F_FUNC(slauum,SLAUUM)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void slauum(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_slauum(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sopgtr "F_FUNC(sopgtr,SOPGTR)"(char *uplo, int *n, s *ap, s *tau, s *q, int *ldq, s *work, int *info) nogil
 * cdef void sopgtr(char *uplo, int *n, s *ap, s *tau, s *q, int *ldq, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sopgtr(uplo, n, ap, tau, q, ldq, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sopgtr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5857
 *     void _fortran_sopgtr "F_FUNC(sopgtr,SOPGTR)"(char *uplo, int *n, s *ap, s *tau, s *q, int *ldq, s *work, int *info) nogil
 * cdef void sopgtr(char *uplo, int *n, s *ap, s *tau, s *q, int *ldq, s *work, int *info) nogil:
 *     _fortran_sopgtr(uplo, n, ap, tau, q, ldq, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sopgtr,SOPGTR)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_tau, __pyx_v_q, __pyx_v_ldq, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sopgtr "F_FUNC(sopgtr,SOPGTR)"(char *uplo, int *n, s *ap, s *tau, s *q, int *ldq, s *work, int *info) nogil
 * cdef void sopgtr(char *uplo, int *n, s *ap, s *tau, s *q, int *ldq, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sopgtr(uplo, n, ap, tau, q, ldq, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sopmtr "F_FUNC(sopmtr,SOPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, s *ap, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sopmtr(char *side, char *uplo, char *trans, int *m, int *n, s *ap, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sopmtr(side, uplo, trans, m, n, ap, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sopmtr(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5862
 *     void _fortran_sopmtr "F_FUNC(sopmtr,SOPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, s *ap, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sopmtr(char *side, char *uplo, char *trans, int *m, int *n, s *ap, s *tau, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_sopmtr(side, uplo, trans, m, n, ap, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sopmtr,SOPMTR)(__pyx_v_side, __pyx_v_uplo, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_ap, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sopmtr "F_FUNC(sopmtr,SOPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, s *ap, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sopmtr(char *side, char *uplo, char *trans, int *m, int *n, s *ap, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sopmtr(side, uplo, trans, m, n, ap, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorg2l "F_FUNC(sorg2l,SORG2L)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorg2l(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorg2l(m, n, k, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorg2l(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5867
 *     void _fortran_sorg2l "F_FUNC(sorg2l,SORG2L)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorg2l(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sorg2l(m, n, k, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorg2l,SORG2L)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorg2l "F_FUNC(sorg2l,SORG2L)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorg2l(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorg2l(m, n, k, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorg2r "F_FUNC(sorg2r,SORG2R)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorg2r(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorg2r(m, n, k, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorg2r(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5872
 *     void _fortran_sorg2r "F_FUNC(sorg2r,SORG2R)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorg2r(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sorg2r(m, n, k, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorg2r,SORG2R)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorg2r "F_FUNC(sorg2r,SORG2R)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorg2r(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorg2r(m, n, k, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgbr "F_FUNC(sorgbr,SORGBR)"(char *vect, int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgbr(char *vect, int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgbr(vect, m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorgbr(char *__pyx_v_vect, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5877
 *     void _fortran_sorgbr "F_FUNC(sorgbr,SORGBR)"(char *vect, int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgbr(char *vect, int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sorgbr(vect, m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorgbr,SORGBR)(__pyx_v_vect, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgbr "F_FUNC(sorgbr,SORGBR)"(char *vect, int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgbr(char *vect, int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgbr(vect, m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorghr "F_FUNC(sorghr,SORGHR)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorghr(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorghr(n, ilo, ihi, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorghr(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5882
 *     void _fortran_sorghr "F_FUNC(sorghr,SORGHR)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorghr(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sorghr(n, ilo, ihi, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorghr,SORGHR)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorghr "F_FUNC(sorghr,SORGHR)"(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorghr(int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorghr(n, ilo, ihi, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgl2 "F_FUNC(sorgl2,SORGL2)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorgl2(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgl2(m, n, k, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorgl2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5887
 *     void _fortran_sorgl2 "F_FUNC(sorgl2,SORGL2)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorgl2(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sorgl2(m, n, k, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorgl2,SORGL2)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgl2 "F_FUNC(sorgl2,SORGL2)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorgl2(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgl2(m, n, k, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorglq "F_FUNC(sorglq,SORGLQ)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorglq(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorglq(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorglq(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5892
 *     void _fortran_sorglq "F_FUNC(sorglq,SORGLQ)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorglq(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sorglq(m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorglq,SORGLQ)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorglq "F_FUNC(sorglq,SORGLQ)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorglq(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorglq(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgql "F_FUNC(sorgql,SORGQL)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgql(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgql(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorgql(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5897
 *     void _fortran_sorgql "F_FUNC(sorgql,SORGQL)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgql(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sorgql(m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorgql,SORGQL)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgql "F_FUNC(sorgql,SORGQL)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgql(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgql(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgqr "F_FUNC(sorgqr,SORGQR)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgqr(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgqr(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorgqr(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5902
 *     void _fortran_sorgqr "F_FUNC(sorgqr,SORGQR)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgqr(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sorgqr(m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorgqr,SORGQR)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgqr "F_FUNC(sorgqr,SORGQR)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgqr(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgqr(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgr2 "F_FUNC(sorgr2,SORGR2)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorgr2(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgr2(m, n, k, a, lda, tau, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorgr2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5907
 *     void _fortran_sorgr2 "F_FUNC(sorgr2,SORGR2)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorgr2(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:
 *     _fortran_sorgr2(m, n, k, a, lda, tau, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorgr2,SORGR2)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgr2 "F_FUNC(sorgr2,SORGR2)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil
 * cdef void sorgr2(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgr2(m, n, k, a, lda, tau, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgrq "F_FUNC(sorgrq,SORGRQ)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgrq(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgrq(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorgrq(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5912
 *     void _fortran_sorgrq "F_FUNC(sorgrq,SORGRQ)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgrq(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sorgrq(m, n, k, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorgrq,SORGRQ)(__pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgrq "F_FUNC(sorgrq,SORGRQ)"(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgrq(int *m, int *n, int *k, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgrq(m, n, k, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgtr "F_FUNC(sorgtr,SORGTR)"(char *uplo, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgtr(char *uplo, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgtr(uplo, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorgtr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5917
 *     void _fortran_sorgtr "F_FUNC(sorgtr,SORGTR)"(char *uplo, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgtr(char *uplo, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_sorgtr(uplo, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorgtr,SORGTR)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorgtr "F_FUNC(sorgtr,SORGTR)"(char *uplo, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void sorgtr(char *uplo, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorgtr(uplo, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorm2l "F_FUNC(sorm2l,SORM2L)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorm2l(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorm2l(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorm2l(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5922
 *     void _fortran_sorm2l "F_FUNC(sorm2l,SORM2L)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorm2l(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_sorm2l(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorm2l,SORM2L)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorm2l "F_FUNC(sorm2l,SORM2L)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorm2l(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorm2l(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorm2r "F_FUNC(sorm2r,SORM2R)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorm2r(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorm2r(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorm2r(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5927
 *     void _fortran_sorm2r "F_FUNC(sorm2r,SORM2R)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorm2r(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_sorm2r(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorm2r,SORM2R)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorm2r "F_FUNC(sorm2r,SORM2R)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorm2r(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorm2r(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormbr "F_FUNC(sormbr,SORMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormbr(char *vect, char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormbr(vect, side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormbr(char *__pyx_v_vect, char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5932
 *     void _fortran_sormbr "F_FUNC(sormbr,SORMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormbr(char *vect, char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:
 *     _fortran_sormbr(vect, side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormbr,SORMBR)(__pyx_v_vect, __pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormbr "F_FUNC(sormbr,SORMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormbr(char *vect, char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormbr(vect, side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormhr "F_FUNC(sormhr,SORMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormhr(side, trans, m, n, ilo, ihi, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormhr(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5937
 *     void _fortran_sormhr "F_FUNC(sormhr,SORMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:
 *     _fortran_sormhr(side, trans, m, n, ilo, ihi, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormhr,SORMHR)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormhr "F_FUNC(sormhr,SORMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormhr(side, trans, m, n, ilo, ihi, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorml2 "F_FUNC(sorml2,SORML2)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorml2(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorml2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sorml2(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5942
 *     void _fortran_sorml2 "F_FUNC(sorml2,SORML2)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorml2(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_sorml2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sorml2,SORML2)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sorml2 "F_FUNC(sorml2,SORML2)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sorml2(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sorml2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormlq "F_FUNC(sormlq,SORMLQ)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormlq(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormlq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormlq(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5947
 *     void _fortran_sormlq "F_FUNC(sormlq,SORMLQ)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormlq(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:
 *     _fortran_sormlq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormlq,SORMLQ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormlq "F_FUNC(sormlq,SORMLQ)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormlq(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormlq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormql "F_FUNC(sormql,SORMQL)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormql(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormql(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormql(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5952
 *     void _fortran_sormql "F_FUNC(sormql,SORMQL)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormql(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:
 *     _fortran_sormql(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormql,SORMQL)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormql "F_FUNC(sormql,SORMQL)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormql(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormql(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormqr "F_FUNC(sormqr,SORMQR)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormqr(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormqr(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormqr(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5957
 *     void _fortran_sormqr "F_FUNC(sormqr,SORMQR)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormqr(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:
 *     _fortran_sormqr(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormqr,SORMQR)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormqr "F_FUNC(sormqr,SORMQR)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormqr(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormqr(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormr2 "F_FUNC(sormr2,SORMR2)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sormr2(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormr2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormr2(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5962
 *     void _fortran_sormr2 "F_FUNC(sormr2,SORMR2)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sormr2(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_sormr2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormr2,SORMR2)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormr2 "F_FUNC(sormr2,SORMR2)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sormr2(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormr2(side, trans, m, n, k, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormr3 "F_FUNC(sormr3,SORMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sormr3(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormr3(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormr3(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5967
 *     void _fortran_sormr3 "F_FUNC(sormr3,SORMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sormr3(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:
 *     _fortran_sormr3(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormr3,SORMR3)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormr3 "F_FUNC(sormr3,SORMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil
 * cdef void sormr3(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormr3(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormrq "F_FUNC(sormrq,SORMRQ)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormrq(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormrq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormrq(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5972
 *     void _fortran_sormrq "F_FUNC(sormrq,SORMRQ)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormrq(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:
 *     _fortran_sormrq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormrq,SORMRQ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormrq "F_FUNC(sormrq,SORMRQ)"(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormrq(char *side, char *trans, int *m, int *n, int *k, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormrq(side, trans, m, n, k, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormrz "F_FUNC(sormrz,SORMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormrz(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormrz(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormrz(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5977
 *     void _fortran_sormrz "F_FUNC(sormrz,SORMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormrz(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:
 *     _fortran_sormrz(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormrz,SORMRZ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormrz "F_FUNC(sormrz,SORMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormrz(char *side, char *trans, int *m, int *n, int *k, int *l, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormrz(side, trans, m, n, k, l, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormtr "F_FUNC(sormtr,SORMTR)"(char *side, char *uplo, char *trans, int *m, int *n, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormtr(char *side, char *uplo, char *trans, int *m, int *n, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormtr(side, uplo, trans, m, n, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sormtr(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5982
 *     void _fortran_sormtr "F_FUNC(sormtr,SORMTR)"(char *side, char *uplo, char *trans, int *m, int *n, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormtr(char *side, char *uplo, char *trans, int *m, int *n, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:
 *     _fortran_sormtr(side, uplo, trans, m, n, a, lda, tau, c, ldc, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sormtr,SORMTR)(__pyx_v_side, __pyx_v_uplo, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_c, __pyx_v_ldc, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sormtr "F_FUNC(sormtr,SORMTR)"(char *side, char *uplo, char *trans, int *m, int *n, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil
 * cdef void sormtr(char *side, char *uplo, char *trans, int *m, int *n, s *a, int *lda, s *tau, s *c, int *ldc, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sormtr(side, uplo, trans, m, n, a, lda, tau, c, ldc, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbcon "F_FUNC(spbcon,SPBCON)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void spbcon(char *uplo, int *n, int *kd, s *ab, int *ldab, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbcon(uplo, n, kd, ab, ldab, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbcon(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5987
 *     void _fortran_spbcon "F_FUNC(spbcon,SPBCON)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void spbcon(char *uplo, int *n, int *kd, s *ab, int *ldab, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_spbcon(uplo, n, kd, ab, ldab, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbcon,SPBCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbcon "F_FUNC(spbcon,SPBCON)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void spbcon(char *uplo, int *n, int *kd, s *ab, int *ldab, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbcon(uplo, n, kd, ab, ldab, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbequ "F_FUNC(spbequ,SPBEQU)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil
 * cdef void spbequ(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbequ(uplo, n, kd, ab, ldab, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbequ(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5992
 *     void _fortran_spbequ "F_FUNC(spbequ,SPBEQU)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil
 * cdef void spbequ(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil:
 *     _fortran_spbequ(uplo, n, kd, ab, ldab, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbequ,SPBEQU)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbequ "F_FUNC(spbequ,SPBEQU)"(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil
 * cdef void spbequ(char *uplo, int *n, int *kd, s *ab, int *ldab, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbequ(uplo, n, kd, ab, ldab, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":5996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbrfs "F_FUNC(spbrfs,SPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spbrfs(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbrfs(uplo, n, kd, nrhs, ab, ldab, afb, ldafb, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_afb, int *__pyx_v_ldafb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":5997
 *     void _fortran_spbrfs "F_FUNC(spbrfs,SPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spbrfs(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_spbrfs(uplo, n, kd, nrhs, ab, ldab, afb, ldafb, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbrfs,SPBRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_afb, __pyx_v_ldafb, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":5996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbrfs "F_FUNC(spbrfs,SPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spbrfs(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbrfs(uplo, n, kd, nrhs, ab, ldab, afb, ldafb, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbstf "F_FUNC(spbstf,SPBSTF)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbstf(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbstf(uplo, n, kd, ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbstf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6002
 *     void _fortran_spbstf "F_FUNC(spbstf,SPBSTF)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbstf(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:
 *     _fortran_spbstf(uplo, n, kd, ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbstf,SPBSTF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbstf "F_FUNC(spbstf,SPBSTF)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbstf(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbstf(uplo, n, kd, ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbsv "F_FUNC(spbsv,SPBSV)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void spbsv(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbsv(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6007
 *     void _fortran_spbsv "F_FUNC(spbsv,SPBSV)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void spbsv(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:
 *     _fortran_spbsv(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbsv,SPBSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbsv "F_FUNC(spbsv,SPBSV)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void spbsv(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbsv(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbsvx "F_FUNC(spbsvx,SPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbsvx(fact, uplo, n, kd, nrhs, ab, ldab, afb, ldafb, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_afb, int *__pyx_v_ldafb, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6012
 *     void _fortran_spbsvx "F_FUNC(spbsvx,SPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_spbsvx(fact, uplo, n, kd, nrhs, ab, ldab, afb, ldafb, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbsvx,SPBSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_afb, __pyx_v_ldafb, __pyx_v_equed, __pyx_v_s, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbsvx "F_FUNC(spbsvx,SPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *afb, int *ldafb, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbsvx(fact, uplo, n, kd, nrhs, ab, ldab, afb, ldafb, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbtf2 "F_FUNC(spbtf2,SPBTF2)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbtf2(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbtf2(uplo, n, kd, ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbtf2(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6017
 *     void _fortran_spbtf2 "F_FUNC(spbtf2,SPBTF2)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbtf2(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:
 *     _fortran_spbtf2(uplo, n, kd, ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbtf2,SPBTF2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbtf2 "F_FUNC(spbtf2,SPBTF2)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbtf2(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbtf2(uplo, n, kd, ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbtrf "F_FUNC(spbtrf,SPBTRF)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbtrf(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbtrf(uplo, n, kd, ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbtrf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6022
 *     void _fortran_spbtrf "F_FUNC(spbtrf,SPBTRF)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbtrf(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:
 *     _fortran_spbtrf(uplo, n, kd, ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbtrf,SPBTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbtrf "F_FUNC(spbtrf,SPBTRF)"(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil
 * cdef void spbtrf(char *uplo, int *n, int *kd, s *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbtrf(uplo, n, kd, ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbtrs "F_FUNC(spbtrs,SPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void spbtrs(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbtrs(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spbtrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6027
 *     void _fortran_spbtrs "F_FUNC(spbtrs,SPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void spbtrs(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:
 *     _fortran_spbtrs(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spbtrs,SPBTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spbtrs "F_FUNC(spbtrs,SPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void spbtrs(char *uplo, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spbtrs(uplo, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spocon "F_FUNC(spocon,SPOCON)"(char *uplo, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void spocon(char *uplo, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spocon(uplo, n, a, lda, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spocon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6032
 *     void _fortran_spocon "F_FUNC(spocon,SPOCON)"(char *uplo, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void spocon(char *uplo, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_spocon(uplo, n, a, lda, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spocon,SPOCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spocon "F_FUNC(spocon,SPOCON)"(char *uplo, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void spocon(char *uplo, int *n, s *a, int *lda, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spocon(uplo, n, a, lda, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spoequ "F_FUNC(spoequ,SPOEQU)"(int *n, s *a, int *lda, s *s, s *scond, s *amax, int *info) nogil
 * cdef void spoequ(int *n, s *a, int *lda, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spoequ(n, a, lda, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spoequ(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6037
 *     void _fortran_spoequ "F_FUNC(spoequ,SPOEQU)"(int *n, s *a, int *lda, s *s, s *scond, s *amax, int *info) nogil
 * cdef void spoequ(int *n, s *a, int *lda, s *s, s *scond, s *amax, int *info) nogil:
 *     _fortran_spoequ(n, a, lda, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spoequ,SPOEQU)(__pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spoequ "F_FUNC(spoequ,SPOEQU)"(int *n, s *a, int *lda, s *s, s *scond, s *amax, int *info) nogil
 * cdef void spoequ(int *n, s *a, int *lda, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spoequ(n, a, lda, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sporfs "F_FUNC(sporfs,SPORFS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sporfs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sporfs(uplo, n, nrhs, a, lda, af, ldaf, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sporfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_af, int *__pyx_v_ldaf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6042
 *     void _fortran_sporfs "F_FUNC(sporfs,SPORFS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sporfs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sporfs(uplo, n, nrhs, a, lda, af, ldaf, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sporfs,SPORFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sporfs "F_FUNC(sporfs,SPORFS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sporfs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sporfs(uplo, n, nrhs, a, lda, af, ldaf, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sposv "F_FUNC(sposv,SPOSV)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void sposv(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sposv(uplo, n, nrhs, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sposv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6047
 *     void _fortran_sposv "F_FUNC(sposv,SPOSV)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void sposv(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:
 *     _fortran_sposv(uplo, n, nrhs, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sposv,SPOSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sposv "F_FUNC(sposv,SPOSV)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void sposv(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sposv(uplo, n, nrhs, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sposvx "F_FUNC(sposvx,SPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sposvx(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sposvx(fact, uplo, n, nrhs, a, lda, af, ldaf, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sposvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_af, int *__pyx_v_ldaf, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6052
 *     void _fortran_sposvx "F_FUNC(sposvx,SPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sposvx(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sposvx(fact, uplo, n, nrhs, a, lda, af, ldaf, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sposvx,SPOSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_equed, __pyx_v_s, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sposvx "F_FUNC(sposvx,SPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sposvx(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sposvx(fact, uplo, n, nrhs, a, lda, af, ldaf, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spotf2 "F_FUNC(spotf2,SPOTF2)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotf2(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spotf2(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spotf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6057
 *     void _fortran_spotf2 "F_FUNC(spotf2,SPOTF2)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotf2(char *uplo, int *n, s *a, int *lda, int *info) nogil:
 *     _fortran_spotf2(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spotf2,SPOTF2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spotf2 "F_FUNC(spotf2,SPOTF2)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotf2(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spotf2(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spotrf "F_FUNC(spotrf,SPOTRF)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotrf(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spotrf(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spotrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6062
 *     void _fortran_spotrf "F_FUNC(spotrf,SPOTRF)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotrf(char *uplo, int *n, s *a, int *lda, int *info) nogil:
 *     _fortran_spotrf(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spotrf,SPOTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spotrf "F_FUNC(spotrf,SPOTRF)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotrf(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spotrf(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spotri "F_FUNC(spotri,SPOTRI)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotri(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spotri(uplo, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spotri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6067
 *     void _fortran_spotri "F_FUNC(spotri,SPOTRI)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotri(char *uplo, int *n, s *a, int *lda, int *info) nogil:
 *     _fortran_spotri(uplo, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spotri,SPOTRI)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spotri "F_FUNC(spotri,SPOTRI)"(char *uplo, int *n, s *a, int *lda, int *info) nogil
 * cdef void spotri(char *uplo, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spotri(uplo, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spotrs "F_FUNC(spotrs,SPOTRS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void spotrs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spotrs(uplo, n, nrhs, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spotrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6072
 *     void _fortran_spotrs "F_FUNC(spotrs,SPOTRS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void spotrs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:
 *     _fortran_spotrs(uplo, n, nrhs, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spotrs,SPOTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spotrs "F_FUNC(spotrs,SPOTRS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void spotrs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spotrs(uplo, n, nrhs, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sppcon "F_FUNC(sppcon,SPPCON)"(char *uplo, int *n, s *ap, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sppcon(char *uplo, int *n, s *ap, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sppcon(uplo, n, ap, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sppcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6077
 *     void _fortran_sppcon "F_FUNC(sppcon,SPPCON)"(char *uplo, int *n, s *ap, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sppcon(char *uplo, int *n, s *ap, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_sppcon(uplo, n, ap, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sppcon,SPPCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sppcon "F_FUNC(sppcon,SPPCON)"(char *uplo, int *n, s *ap, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sppcon(char *uplo, int *n, s *ap, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sppcon(uplo, n, ap, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sppequ "F_FUNC(sppequ,SPPEQU)"(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, int *info) nogil
 * cdef void sppequ(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sppequ(uplo, n, ap, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sppequ(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6082
 *     void _fortran_sppequ "F_FUNC(sppequ,SPPEQU)"(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, int *info) nogil
 * cdef void sppequ(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, int *info) nogil:
 *     _fortran_sppequ(uplo, n, ap, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sppequ,SPPEQU)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sppequ "F_FUNC(sppequ,SPPEQU)"(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, int *info) nogil
 * cdef void sppequ(char *uplo, int *n, s *ap, s *s, s *scond, s *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sppequ(uplo, n, ap, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spprfs "F_FUNC(spprfs,SPPRFS)"(char *uplo, int *n, int *nrhs, s *ap, s *afp, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spprfs(char *uplo, int *n, int *nrhs, s *ap, s *afp, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spprfs(uplo, n, nrhs, ap, afp, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_afp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6087
 *     void _fortran_spprfs "F_FUNC(spprfs,SPPRFS)"(char *uplo, int *n, int *nrhs, s *ap, s *afp, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spprfs(char *uplo, int *n, int *nrhs, s *ap, s *afp, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_spprfs(uplo, n, nrhs, ap, afp, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spprfs,SPPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_afp, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spprfs "F_FUNC(spprfs,SPPRFS)"(char *uplo, int *n, int *nrhs, s *ap, s *afp, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void spprfs(char *uplo, int *n, int *nrhs, s *ap, s *afp, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spprfs(uplo, n, nrhs, ap, afp, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sppsv "F_FUNC(sppsv,SPPSV)"(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void sppsv(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sppsv(uplo, n, nrhs, ap, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sppsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6092
 *     void _fortran_sppsv "F_FUNC(sppsv,SPPSV)"(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void sppsv(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:
 *     _fortran_sppsv(uplo, n, nrhs, ap, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sppsv,SPPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sppsv "F_FUNC(sppsv,SPPSV)"(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void sppsv(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sppsv(uplo, n, nrhs, ap, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sppsvx "F_FUNC(sppsvx,SPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sppsvx(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sppsvx(fact, uplo, n, nrhs, ap, afp, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sppsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_afp, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6097
 *     void _fortran_sppsvx "F_FUNC(sppsvx,SPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sppsvx(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sppsvx(fact, uplo, n, nrhs, ap, afp, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sppsvx,SPPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_afp, __pyx_v_equed, __pyx_v_s, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sppsvx "F_FUNC(sppsvx,SPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sppsvx(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, char *equed, s *s, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sppsvx(fact, uplo, n, nrhs, ap, afp, equed, s, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spptrf "F_FUNC(spptrf,SPPTRF)"(char *uplo, int *n, s *ap, int *info) nogil
 * cdef void spptrf(char *uplo, int *n, s *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spptrf(uplo, n, ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6102
 *     void _fortran_spptrf "F_FUNC(spptrf,SPPTRF)"(char *uplo, int *n, s *ap, int *info) nogil
 * cdef void spptrf(char *uplo, int *n, s *ap, int *info) nogil:
 *     _fortran_spptrf(uplo, n, ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spptrf,SPPTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spptrf "F_FUNC(spptrf,SPPTRF)"(char *uplo, int *n, s *ap, int *info) nogil
 * cdef void spptrf(char *uplo, int *n, s *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spptrf(uplo, n, ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spptri "F_FUNC(spptri,SPPTRI)"(char *uplo, int *n, s *ap, int *info) nogil
 * cdef void spptri(char *uplo, int *n, s *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spptri(uplo, n, ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6107
 *     void _fortran_spptri "F_FUNC(spptri,SPPTRI)"(char *uplo, int *n, s *ap, int *info) nogil
 * cdef void spptri(char *uplo, int *n, s *ap, int *info) nogil:
 *     _fortran_spptri(uplo, n, ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spptri,SPPTRI)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spptri "F_FUNC(spptri,SPPTRI)"(char *uplo, int *n, s *ap, int *info) nogil
 * cdef void spptri(char *uplo, int *n, s *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spptri(uplo, n, ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spptrs "F_FUNC(spptrs,SPPTRS)"(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void spptrs(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spptrs(uplo, n, nrhs, ap, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6112
 *     void _fortran_spptrs "F_FUNC(spptrs,SPPTRS)"(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void spptrs(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:
 *     _fortran_spptrs(uplo, n, nrhs, ap, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spptrs,SPPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spptrs "F_FUNC(spptrs,SPPTRS)"(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void spptrs(char *uplo, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spptrs(uplo, n, nrhs, ap, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptcon "F_FUNC(sptcon,SPTCON)"(int *n, s *d, s *e, s *anorm, s *rcond, s *work, int *info) nogil
 * cdef void sptcon(int *n, s *d, s *e, s *anorm, s *rcond, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptcon(n, d, e, anorm, rcond, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sptcon(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6117
 *     void _fortran_sptcon "F_FUNC(sptcon,SPTCON)"(int *n, s *d, s *e, s *anorm, s *rcond, s *work, int *info) nogil
 * cdef void sptcon(int *n, s *d, s *e, s *anorm, s *rcond, s *work, int *info) nogil:
 *     _fortran_sptcon(n, d, e, anorm, rcond, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sptcon,SPTCON)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptcon "F_FUNC(sptcon,SPTCON)"(int *n, s *d, s *e, s *anorm, s *rcond, s *work, int *info) nogil
 * cdef void sptcon(int *n, s *d, s *e, s *anorm, s *rcond, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptcon(n, d, e, anorm, rcond, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spteqr "F_FUNC(spteqr,SPTEQR)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void spteqr(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spteqr(compz, n, d, e, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spteqr(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6122
 *     void _fortran_spteqr "F_FUNC(spteqr,SPTEQR)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void spteqr(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_spteqr(compz, n, d, e, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spteqr,SPTEQR)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spteqr "F_FUNC(spteqr,SPTEQR)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void spteqr(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spteqr(compz, n, d, e, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptrfs "F_FUNC(sptrfs,SPTRFS)"(int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *info) nogil
 * cdef void sptrfs(int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptrfs(n, nrhs, d, e, df, ef, b, ldb, x, ldx, ferr, berr, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sptrfs(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_df, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ef, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6127
 *     void _fortran_sptrfs "F_FUNC(sptrfs,SPTRFS)"(int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *info) nogil
 * cdef void sptrfs(int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *info) nogil:
 *     _fortran_sptrfs(n, nrhs, d, e, df, ef, b, ldb, x, ldx, ferr, berr, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sptrfs,SPTRFS)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_df, __pyx_v_ef, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptrfs "F_FUNC(sptrfs,SPTRFS)"(int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *info) nogil
 * cdef void sptrfs(int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptrfs(n, nrhs, d, e, df, ef, b, ldb, x, ldx, ferr, berr, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptsv "F_FUNC(sptsv,SPTSV)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil
 * cdef void sptsv(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptsv(n, nrhs, d, e, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sptsv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6132
 *     void _fortran_sptsv "F_FUNC(sptsv,SPTSV)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil
 * cdef void sptsv(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil:
 *     _fortran_sptsv(n, nrhs, d, e, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sptsv,SPTSV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptsv "F_FUNC(sptsv,SPTSV)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil
 * cdef void sptsv(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptsv(n, nrhs, d, e, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptsvx "F_FUNC(sptsvx,SPTSVX)"(char *fact, int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *info) nogil
 * cdef void sptsvx(char *fact, int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptsvx(fact, n, nrhs, d, e, df, ef, b, ldb, x, ldx, rcond, ferr, berr, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sptsvx(char *__pyx_v_fact, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_df, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ef, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6137
 *     void _fortran_sptsvx "F_FUNC(sptsvx,SPTSVX)"(char *fact, int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *info) nogil
 * cdef void sptsvx(char *fact, int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *info) nogil:
 *     _fortran_sptsvx(fact, n, nrhs, d, e, df, ef, b, ldb, x, ldx, rcond, ferr, berr, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sptsvx,SPTSVX)(__pyx_v_fact, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_df, __pyx_v_ef, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptsvx "F_FUNC(sptsvx,SPTSVX)"(char *fact, int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *info) nogil
 * cdef void sptsvx(char *fact, int *n, int *nrhs, s *d, s *e, s *df, s *ef, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptsvx(fact, n, nrhs, d, e, df, ef, b, ldb, x, ldx, rcond, ferr, berr, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spttrf "F_FUNC(spttrf,SPTTRF)"(int *n, s *d, s *e, int *info) nogil
 * cdef void spttrf(int *n, s *d, s *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spttrf(n, d, e, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spttrf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6142
 *     void _fortran_spttrf "F_FUNC(spttrf,SPTTRF)"(int *n, s *d, s *e, int *info) nogil
 * cdef void spttrf(int *n, s *d, s *e, int *info) nogil:
 *     _fortran_spttrf(n, d, e, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spttrf,SPTTRF)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spttrf "F_FUNC(spttrf,SPTTRF)"(int *n, s *d, s *e, int *info) nogil
 * cdef void spttrf(int *n, s *d, s *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spttrf(n, d, e, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spttrs "F_FUNC(spttrs,SPTTRS)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil
 * cdef void spttrs(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spttrs(n, nrhs, d, e, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_spttrs(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6147
 *     void _fortran_spttrs "F_FUNC(spttrs,SPTTRS)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil
 * cdef void spttrs(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil:
 *     _fortran_spttrs(n, nrhs, d, e, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(spttrs,SPTTRS)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_spttrs "F_FUNC(spttrs,SPTTRS)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil
 * cdef void spttrs(int *n, int *nrhs, s *d, s *e, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_spttrs(n, nrhs, d, e, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptts2 "F_FUNC(sptts2,SPTTS2)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb) nogil
 * cdef void sptts2(int *n, int *nrhs, s *d, s *e, s *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptts2(n, nrhs, d, e, b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sptts2(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":6152
 *     void _fortran_sptts2 "F_FUNC(sptts2,SPTTS2)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb) nogil
 * cdef void sptts2(int *n, int *nrhs, s *d, s *e, s *b, int *ldb) nogil:
 *     _fortran_sptts2(n, nrhs, d, e, b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sptts2,SPTTS2)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, __pyx_v_b, __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":6151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sptts2 "F_FUNC(sptts2,SPTTS2)"(int *n, int *nrhs, s *d, s *e, s *b, int *ldb) nogil
 * cdef void sptts2(int *n, int *nrhs, s *d, s *e, s *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sptts2(n, nrhs, d, e, b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_srscl "F_FUNC(srscl,SRSCL)"(int *n, s *sa, s *sx, int *incx) nogil
 * cdef void srscl(int *n, s *sa, s *sx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_srscl(n, sa, sx, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_srscl(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sa, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sx, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":6157
 *     void _fortran_srscl "F_FUNC(srscl,SRSCL)"(int *n, s *sa, s *sx, int *incx) nogil
 * cdef void srscl(int *n, s *sa, s *sx, int *incx) nogil:
 *     _fortran_srscl(n, sa, sx, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(srscl,SRSCL)(__pyx_v_n, __pyx_v_sa, __pyx_v_sx, __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":6156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_srscl "F_FUNC(srscl,SRSCL)"(int *n, s *sa, s *sx, int *incx) nogil
 * cdef void srscl(int *n, s *sa, s *sx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_srscl(n, sa, sx, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbev "F_FUNC(ssbev,SSBEV)"(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssbev(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbev(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssbev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6162
 *     void _fortran_ssbev "F_FUNC(ssbev,SSBEV)"(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssbev(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_ssbev(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssbev,SSBEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbev "F_FUNC(ssbev,SSBEV)"(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssbev(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbev(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbevd "F_FUNC(ssbevd,SSBEVD)"(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssbevd(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbevd(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssbevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6167
 *     void _fortran_ssbevd "F_FUNC(ssbevd,SSBEVD)"(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssbevd(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_ssbevd(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssbevd,SSBEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbevd "F_FUNC(ssbevd,SSBEVD)"(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssbevd(char *jobz, char *uplo, int *n, int *kd, s *ab, int *ldab, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbevd(jobz, uplo, n, kd, ab, ldab, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbevx "F_FUNC(ssbevx,SSBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, s *ab, int *ldab, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void ssbevx(char *jobz, char *range, char *uplo, int *n, int *kd, s *ab, int *ldab, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbevx(jobz, range, uplo, n, kd, ab, ldab, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssbevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6172
 *     void _fortran_ssbevx "F_FUNC(ssbevx,SSBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, s *ab, int *ldab, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void ssbevx(char *jobz, char *range, char *uplo, int *n, int *kd, s *ab, int *ldab, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_ssbevx(jobz, range, uplo, n, kd, ab, ldab, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssbevx,SSBEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_q, __pyx_v_ldq, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbevx "F_FUNC(ssbevx,SSBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, s *ab, int *ldab, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void ssbevx(char *jobz, char *range, char *uplo, int *n, int *kd, s *ab, int *ldab, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbevx(jobz, range, uplo, n, kd, ab, ldab, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbgst "F_FUNC(ssbgst,SSBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *x, int *ldx, s *work, int *info) nogil
 * cdef void ssbgst(char *vect, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *x, int *ldx, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbgst(vect, uplo, n, ka, kb, ab, ldab, bb, ldbb, x, ldx, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssbgst(char *__pyx_v_vect, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6177
 *     void _fortran_ssbgst "F_FUNC(ssbgst,SSBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *x, int *ldx, s *work, int *info) nogil
 * cdef void ssbgst(char *vect, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *x, int *ldx, s *work, int *info) nogil:
 *     _fortran_ssbgst(vect, uplo, n, ka, kb, ab, ldab, bb, ldbb, x, ldx, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssbgst,SSBGST)(__pyx_v_vect, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, __pyx_v_ab, __pyx_v_ldab, __pyx_v_bb, __pyx_v_ldbb, __pyx_v_x, __pyx_v_ldx, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbgst "F_FUNC(ssbgst,SSBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *x, int *ldx, s *work, int *info) nogil
 * cdef void ssbgst(char *vect, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *x, int *ldx, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbgst(vect, uplo, n, ka, kb, ab, ldab, bb, ldbb, x, ldx, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbgv "F_FUNC(ssbgv,SSBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbgv(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssbgv(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6182
 *     void _fortran_ssbgv "F_FUNC(ssbgv,SSBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_ssbgv(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssbgv,SSBGV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, __pyx_v_ab, __pyx_v_ldab, __pyx_v_bb, __pyx_v_ldbb, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbgv "F_FUNC(ssbgv,SSBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbgv(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbgvd "F_FUNC(ssbgvd,SSBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbgvd(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssbgvd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6187
 *     void _fortran_ssbgvd "F_FUNC(ssbgvd,SSBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_ssbgvd(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssbgvd,SSBGVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, __pyx_v_ab, __pyx_v_ldab, __pyx_v_bb, __pyx_v_ldbb, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbgvd "F_FUNC(ssbgvd,SSBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbgvd(jobz, uplo, n, ka, kb, ab, ldab, bb, ldbb, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbgvx "F_FUNC(ssbgvx,SSBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void ssbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbgvx(jobz, range, uplo, n, ka, kb, ab, ldab, bb, ldbb, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssbgvx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6192
 *     void _fortran_ssbgvx "F_FUNC(ssbgvx,SSBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void ssbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_ssbgvx(jobz, range, uplo, n, ka, kb, ab, ldab, bb, ldbb, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssbgvx,SSBGVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, __pyx_v_ab, __pyx_v_ldab, __pyx_v_bb, __pyx_v_ldbb, __pyx_v_q, __pyx_v_ldq, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbgvx "F_FUNC(ssbgvx,SSBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void ssbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, s *ab, int *ldab, s *bb, int *ldbb, s *q, int *ldq, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbgvx(jobz, range, uplo, n, ka, kb, ab, ldab, bb, ldbb, q, ldq, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbtrd "F_FUNC(ssbtrd,SSBTRD)"(char *vect, char *uplo, int *n, int *kd, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *work, int *info) nogil
 * cdef void ssbtrd(char *vect, char *uplo, int *n, int *kd, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbtrd(vect, uplo, n, kd, ab, ldab, d, e, q, ldq, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssbtrd(char *__pyx_v_vect, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6197
 *     void _fortran_ssbtrd "F_FUNC(ssbtrd,SSBTRD)"(char *vect, char *uplo, int *n, int *kd, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *work, int *info) nogil
 * cdef void ssbtrd(char *vect, char *uplo, int *n, int *kd, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *work, int *info) nogil:
 *     _fortran_ssbtrd(vect, uplo, n, kd, ab, ldab, d, e, q, ldq, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssbtrd,SSBTRD)(__pyx_v_vect, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_d, __pyx_v_e, __pyx_v_q, __pyx_v_ldq, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssbtrd "F_FUNC(ssbtrd,SSBTRD)"(char *vect, char *uplo, int *n, int *kd, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *work, int *info) nogil
 * cdef void ssbtrd(char *vect, char *uplo, int *n, int *kd, s *ab, int *ldab, s *d, s *e, s *q, int *ldq, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssbtrd(vect, uplo, n, kd, ab, ldab, d, e, q, ldq, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspcon "F_FUNC(sspcon,SSPCON)"(char *uplo, int *n, s *ap, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sspcon(char *uplo, int *n, s *ap, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspcon(uplo, n, ap, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6202
 *     void _fortran_sspcon "F_FUNC(sspcon,SSPCON)"(char *uplo, int *n, s *ap, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sspcon(char *uplo, int *n, s *ap, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_sspcon(uplo, n, ap, ipiv, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspcon,SSPCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspcon "F_FUNC(sspcon,SSPCON)"(char *uplo, int *n, s *ap, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void sspcon(char *uplo, int *n, s *ap, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspcon(uplo, n, ap, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspev "F_FUNC(sspev,SSPEV)"(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sspev(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspev(jobz, uplo, n, ap, w, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6207
 *     void _fortran_sspev "F_FUNC(sspev,SSPEV)"(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sspev(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_sspev(jobz, uplo, n, ap, w, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspev,SSPEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspev "F_FUNC(sspev,SSPEV)"(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sspev(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspev(jobz, uplo, n, ap, w, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspevd "F_FUNC(sspevd,SSPEVD)"(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sspevd(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspevd(jobz, uplo, n, ap, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6212
 *     void _fortran_sspevd "F_FUNC(sspevd,SSPEVD)"(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sspevd(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_sspevd(jobz, uplo, n, ap, w, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspevd,SSPEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspevd "F_FUNC(sspevd,SSPEVD)"(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sspevd(char *jobz, char *uplo, int *n, s *ap, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspevd(jobz, uplo, n, ap, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspevx "F_FUNC(sspevx,SSPEVX)"(char *jobz, char *range, char *uplo, int *n, s *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sspevx(char *jobz, char *range, char *uplo, int *n, s *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspevx(jobz, range, uplo, n, ap, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6217
 *     void _fortran_sspevx "F_FUNC(sspevx,SSPEVX)"(char *jobz, char *range, char *uplo, int *n, s *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sspevx(char *jobz, char *range, char *uplo, int *n, s *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_sspevx(jobz, range, uplo, n, ap, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspevx,SSPEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspevx "F_FUNC(sspevx,SSPEVX)"(char *jobz, char *range, char *uplo, int *n, s *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sspevx(char *jobz, char *range, char *uplo, int *n, s *ap, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspevx(jobz, range, uplo, n, ap, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspgst "F_FUNC(sspgst,SSPGST)"(int *itype, char *uplo, int *n, s *ap, s *bp, int *info) nogil
 * cdef void sspgst(int *itype, char *uplo, int *n, s *ap, s *bp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspgst(itype, uplo, n, ap, bp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspgst(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6222
 *     void _fortran_sspgst "F_FUNC(sspgst,SSPGST)"(int *itype, char *uplo, int *n, s *ap, s *bp, int *info) nogil
 * cdef void sspgst(int *itype, char *uplo, int *n, s *ap, s *bp, int *info) nogil:
 *     _fortran_sspgst(itype, uplo, n, ap, bp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspgst,SSPGST)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_bp, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspgst "F_FUNC(sspgst,SSPGST)"(int *itype, char *uplo, int *n, s *ap, s *bp, int *info) nogil
 * cdef void sspgst(int *itype, char *uplo, int *n, s *ap, s *bp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspgst(itype, uplo, n, ap, bp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspgv "F_FUNC(sspgv,SSPGV)"(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sspgv(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspgv(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspgv(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6227
 *     void _fortran_sspgv "F_FUNC(sspgv,SSPGV)"(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sspgv(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_sspgv(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspgv,SSPGV)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_bp, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspgv "F_FUNC(sspgv,SSPGV)"(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sspgv(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspgv(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspgvd "F_FUNC(sspgvd,SSPGVD)"(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sspgvd(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspgvd(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspgvd(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6232
 *     void _fortran_sspgvd "F_FUNC(sspgvd,SSPGVD)"(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sspgvd(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_sspgvd(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspgvd,SSPGVD)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_bp, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspgvd "F_FUNC(sspgvd,SSPGVD)"(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sspgvd(int *itype, char *jobz, char *uplo, int *n, s *ap, s *bp, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspgvd(itype, jobz, uplo, n, ap, bp, w, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspgvx "F_FUNC(sspgvx,SSPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, s *ap, s *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sspgvx(int *itype, char *jobz, char *range, char *uplo, int *n, s *ap, s *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspgvx(itype, jobz, range, uplo, n, ap, bp, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspgvx(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6237
 *     void _fortran_sspgvx "F_FUNC(sspgvx,SSPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, s *ap, s *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sspgvx(int *itype, char *jobz, char *range, char *uplo, int *n, s *ap, s *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_sspgvx(itype, jobz, range, uplo, n, ap, bp, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspgvx,SSPGVX)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_bp, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspgvx "F_FUNC(sspgvx,SSPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, s *ap, s *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sspgvx(int *itype, char *jobz, char *range, char *uplo, int *n, s *ap, s *bp, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspgvx(itype, jobz, range, uplo, n, ap, bp, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssprfs "F_FUNC(ssprfs,SSPRFS)"(char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void ssprfs(char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssprfs(uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6242
 *     void _fortran_ssprfs "F_FUNC(ssprfs,SSPRFS)"(char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void ssprfs(char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_ssprfs(uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssprfs,SSPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_afp, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssprfs "F_FUNC(ssprfs,SSPRFS)"(char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void ssprfs(char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssprfs(uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspsv "F_FUNC(sspsv,SSPSV)"(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sspsv(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspsv(uplo, n, nrhs, ap, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6247
 *     void _fortran_sspsv "F_FUNC(sspsv,SSPSV)"(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sspsv(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil:
 *     _fortran_sspsv(uplo, n, nrhs, ap, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspsv,SSPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspsv "F_FUNC(sspsv,SSPSV)"(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void sspsv(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspsv(uplo, n, nrhs, ap, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspsvx "F_FUNC(sspsvx,SSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sspsvx(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sspsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6252
 *     void _fortran_sspsvx "F_FUNC(sspsvx,SSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sspsvx(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_sspsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sspsvx,SSPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_afp, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sspsvx "F_FUNC(sspsvx,SSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void sspsvx(char *fact, char *uplo, int *n, int *nrhs, s *ap, s *afp, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sspsvx(fact, uplo, n, nrhs, ap, afp, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssptrd "F_FUNC(ssptrd,SSPTRD)"(char *uplo, int *n, s *ap, s *d, s *e, s *tau, int *info) nogil
 * cdef void ssptrd(char *uplo, int *n, s *ap, s *d, s *e, s *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssptrd(uplo, n, ap, d, e, tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssptrd(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6257
 *     void _fortran_ssptrd "F_FUNC(ssptrd,SSPTRD)"(char *uplo, int *n, s *ap, s *d, s *e, s *tau, int *info) nogil
 * cdef void ssptrd(char *uplo, int *n, s *ap, s *d, s *e, s *tau, int *info) nogil:
 *     _fortran_ssptrd(uplo, n, ap, d, e, tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssptrd,SSPTRD)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_d, __pyx_v_e, __pyx_v_tau, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssptrd "F_FUNC(ssptrd,SSPTRD)"(char *uplo, int *n, s *ap, s *d, s *e, s *tau, int *info) nogil
 * cdef void ssptrd(char *uplo, int *n, s *ap, s *d, s *e, s *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssptrd(uplo, n, ap, d, e, tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssptrf "F_FUNC(ssptrf,SSPTRF)"(char *uplo, int *n, s *ap, int *ipiv, int *info) nogil
 * cdef void ssptrf(char *uplo, int *n, s *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssptrf(uplo, n, ap, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6262
 *     void _fortran_ssptrf "F_FUNC(ssptrf,SSPTRF)"(char *uplo, int *n, s *ap, int *ipiv, int *info) nogil
 * cdef void ssptrf(char *uplo, int *n, s *ap, int *ipiv, int *info) nogil:
 *     _fortran_ssptrf(uplo, n, ap, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssptrf,SSPTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssptrf "F_FUNC(ssptrf,SSPTRF)"(char *uplo, int *n, s *ap, int *ipiv, int *info) nogil
 * cdef void ssptrf(char *uplo, int *n, s *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssptrf(uplo, n, ap, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssptri "F_FUNC(ssptri,SSPTRI)"(char *uplo, int *n, s *ap, int *ipiv, s *work, int *info) nogil
 * cdef void ssptri(char *uplo, int *n, s *ap, int *ipiv, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssptri(uplo, n, ap, ipiv, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6267
 *     void _fortran_ssptri "F_FUNC(ssptri,SSPTRI)"(char *uplo, int *n, s *ap, int *ipiv, s *work, int *info) nogil
 * cdef void ssptri(char *uplo, int *n, s *ap, int *ipiv, s *work, int *info) nogil:
 *     _fortran_ssptri(uplo, n, ap, ipiv, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssptri,SSPTRI)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssptri "F_FUNC(ssptri,SSPTRI)"(char *uplo, int *n, s *ap, int *ipiv, s *work, int *info) nogil
 * cdef void ssptri(char *uplo, int *n, s *ap, int *ipiv, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssptri(uplo, n, ap, ipiv, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssptrs "F_FUNC(ssptrs,SSPTRS)"(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void ssptrs(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssptrs(uplo, n, nrhs, ap, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6272
 *     void _fortran_ssptrs "F_FUNC(ssptrs,SSPTRS)"(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void ssptrs(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil:
 *     _fortran_ssptrs(uplo, n, nrhs, ap, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssptrs,SSPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssptrs "F_FUNC(ssptrs,SSPTRS)"(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void ssptrs(char *uplo, int *n, int *nrhs, s *ap, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssptrs(uplo, n, nrhs, ap, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstebz "F_FUNC(sstebz,SSTEBZ)"(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *abstol, s *d, s *e, int *m, int *nsplit, s *w, int *iblock, int *isplit, s *work, int *iwork, int *info) nogil
 * cdef void sstebz(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *abstol, s *d, s *e, int *m, int *nsplit, s *w, int *iblock, int *isplit, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstebz(range, order, n, vl, vu, il, iu, abstol, d, e, m, nsplit, w, iblock, isplit, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstebz(char *__pyx_v_range, char *__pyx_v_order, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, int *__pyx_v_m, int *__pyx_v_nsplit, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, int *__pyx_v_iblock, int *__pyx_v_isplit, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6277
 *     void _fortran_sstebz "F_FUNC(sstebz,SSTEBZ)"(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *abstol, s *d, s *e, int *m, int *nsplit, s *w, int *iblock, int *isplit, s *work, int *iwork, int *info) nogil
 * cdef void sstebz(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *abstol, s *d, s *e, int *m, int *nsplit, s *w, int *iblock, int *isplit, s *work, int *iwork, int *info) nogil:
 *     _fortran_sstebz(range, order, n, vl, vu, il, iu, abstol, d, e, m, nsplit, w, iblock, isplit, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstebz,SSTEBZ)(__pyx_v_range, __pyx_v_order, __pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_d, __pyx_v_e, __pyx_v_m, __pyx_v_nsplit, __pyx_v_w, __pyx_v_iblock, __pyx_v_isplit, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstebz "F_FUNC(sstebz,SSTEBZ)"(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *abstol, s *d, s *e, int *m, int *nsplit, s *w, int *iblock, int *isplit, s *work, int *iwork, int *info) nogil
 * cdef void sstebz(char *range, char *order, int *n, s *vl, s *vu, int *il, int *iu, s *abstol, s *d, s *e, int *m, int *nsplit, s *w, int *iblock, int *isplit, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstebz(range, order, n, vl, vu, il, iu, abstol, d, e, m, nsplit, w, iblock, isplit, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstedc "F_FUNC(sstedc,SSTEDC)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstedc(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstedc(compz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstedc(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6282
 *     void _fortran_sstedc "F_FUNC(sstedc,SSTEDC)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstedc(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_sstedc(compz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstedc,SSTEDC)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstedc "F_FUNC(sstedc,SSTEDC)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstedc(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstedc(compz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstegr "F_FUNC(sstegr,SSTEGR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstegr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstegr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6287
 *     void _fortran_sstegr "F_FUNC(sstegr,SSTEGR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstegr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_sstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstegr,SSTEGR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstegr "F_FUNC(sstegr,SSTEGR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstegr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstein "F_FUNC(sstein,SSTEIN)"(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sstein(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstein(n, d, e, m, w, iblock, isplit, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstein(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, int *__pyx_v_iblock, int *__pyx_v_isplit, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6292
 *     void _fortran_sstein "F_FUNC(sstein,SSTEIN)"(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sstein(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_sstein(n, d, e, m, w, iblock, isplit, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstein,SSTEIN)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_m, __pyx_v_w, __pyx_v_iblock, __pyx_v_isplit, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstein "F_FUNC(sstein,SSTEIN)"(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sstein(int *n, s *d, s *e, int *m, s *w, int *iblock, int *isplit, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstein(n, d, e, m, w, iblock, isplit, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstemr "F_FUNC(sstemr,SSTEMR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, s *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstemr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, s *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstemr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_nzc, int *__pyx_v_isuppz, int *__pyx_v_tryrac, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6297
 *     void _fortran_sstemr "F_FUNC(sstemr,SSTEMR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, s *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstemr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, s *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_sstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstemr,SSTEMR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_nzc, __pyx_v_isuppz, __pyx_v_tryrac, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstemr "F_FUNC(sstemr,SSTEMR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, s *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstemr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, int *m, s *w, s *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssteqr "F_FUNC(ssteqr,SSTEQR)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssteqr(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssteqr(compz, n, d, e, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssteqr(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6302
 *     void _fortran_ssteqr "F_FUNC(ssteqr,SSTEQR)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssteqr(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_ssteqr(compz, n, d, e, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssteqr,SSTEQR)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssteqr "F_FUNC(ssteqr,SSTEQR)"(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void ssteqr(char *compz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssteqr(compz, n, d, e, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssterf "F_FUNC(ssterf,SSTERF)"(int *n, s *d, s *e, int *info) nogil
 * cdef void ssterf(int *n, s *d, s *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssterf(n, d, e, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssterf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6307
 *     void _fortran_ssterf "F_FUNC(ssterf,SSTERF)"(int *n, s *d, s *e, int *info) nogil
 * cdef void ssterf(int *n, s *d, s *e, int *info) nogil:
 *     _fortran_ssterf(n, d, e, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssterf,SSTERF)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssterf "F_FUNC(ssterf,SSTERF)"(int *n, s *d, s *e, int *info) nogil
 * cdef void ssterf(int *n, s *d, s *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssterf(n, d, e, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstev "F_FUNC(sstev,SSTEV)"(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sstev(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstev(jobz, n, d, e, z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstev(char *__pyx_v_jobz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6312
 *     void _fortran_sstev "F_FUNC(sstev,SSTEV)"(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sstev(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:
 *     _fortran_sstev(jobz, n, d, e, z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstev,SSTEV)(__pyx_v_jobz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstev "F_FUNC(sstev,SSTEV)"(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil
 * cdef void sstev(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstev(jobz, n, d, e, z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstevd "F_FUNC(sstevd,SSTEVD)"(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstevd(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstevd(jobz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstevd(char *__pyx_v_jobz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6317
 *     void _fortran_sstevd "F_FUNC(sstevd,SSTEVD)"(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstevd(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_sstevd(jobz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstevd,SSTEVD)(__pyx_v_jobz, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstevd "F_FUNC(sstevd,SSTEVD)"(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstevd(char *jobz, int *n, s *d, s *e, s *z, int *ldz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstevd(jobz, n, d, e, z, ldz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstevr "F_FUNC(sstevr,SSTEVR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstevr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstevr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstevr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6322
 *     void _fortran_sstevr "F_FUNC(sstevr,SSTEVR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstevr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_sstevr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstevr,SSTEVR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstevr "F_FUNC(sstevr,SSTEVR)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void sstevr(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstevr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstevx "F_FUNC(sstevx,SSTEVX)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sstevx(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstevx(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_sstevx(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6327
 *     void _fortran_sstevx "F_FUNC(sstevx,SSTEVX)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sstevx(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_sstevx(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(sstevx,SSTEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_sstevx "F_FUNC(sstevx,SSTEVX)"(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil
 * cdef void sstevx(char *jobz, char *range, int *n, s *d, s *e, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_sstevx(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssycon "F_FUNC(ssycon,SSYCON)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void ssycon(char *uplo, int *n, s *a, int *lda, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssycon(uplo, n, a, lda, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssycon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6332
 *     void _fortran_ssycon "F_FUNC(ssycon,SSYCON)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void ssycon(char *uplo, int *n, s *a, int *lda, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_ssycon(uplo, n, a, lda, ipiv, anorm, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssycon,SSYCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssycon "F_FUNC(ssycon,SSYCON)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void ssycon(char *uplo, int *n, s *a, int *lda, int *ipiv, s *anorm, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssycon(uplo, n, a, lda, ipiv, anorm, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyev "F_FUNC(ssyev,SSYEV)"(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *info) nogil
 * cdef void ssyev(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyev(jobz, uplo, n, a, lda, w, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssyev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6337
 *     void _fortran_ssyev "F_FUNC(ssyev,SSYEV)"(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *info) nogil
 * cdef void ssyev(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *info) nogil:
 *     _fortran_ssyev(jobz, uplo, n, a, lda, w, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssyev,SSYEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_w, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyev "F_FUNC(ssyev,SSYEV)"(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *info) nogil
 * cdef void ssyev(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyev(jobz, uplo, n, a, lda, w, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyevd "F_FUNC(ssyevd,SSYEVD)"(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssyevd(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyevd(jobz, uplo, n, a, lda, w, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssyevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6342
 *     void _fortran_ssyevd "F_FUNC(ssyevd,SSYEVD)"(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssyevd(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_ssyevd(jobz, uplo, n, a, lda, w, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssyevd,SSYEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_w, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyevd "F_FUNC(ssyevd,SSYEVD)"(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssyevd(char *jobz, char *uplo, int *n, s *a, int *lda, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyevd(jobz, uplo, n, a, lda, w, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyevr "F_FUNC(ssyevr,SSYEVR)"(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssyevr(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyevr(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssyevr(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6347
 *     void _fortran_ssyevr "F_FUNC(ssyevr,SSYEVR)"(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssyevr(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_ssyevr(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssyevr,SSYEVR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyevr "F_FUNC(ssyevr,SSYEVR)"(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssyevr(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, int *isuppz, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyevr(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyevx "F_FUNC(ssyevx,SSYEVX)"(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void ssyevx(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyevx(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssyevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6352
 *     void _fortran_ssyevx "F_FUNC(ssyevx,SSYEVX)"(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void ssyevx(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_ssyevx(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssyevx,SSYEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyevx "F_FUNC(ssyevx,SSYEVX)"(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void ssyevx(char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyevx(jobz, range, uplo, n, a, lda, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygs2 "F_FUNC(ssygs2,SSYGS2)"(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void ssygs2(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygs2(itype, uplo, n, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssygs2(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6357
 *     void _fortran_ssygs2 "F_FUNC(ssygs2,SSYGS2)"(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void ssygs2(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil:
 *     _fortran_ssygs2(itype, uplo, n, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssygs2,SSYGS2)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygs2 "F_FUNC(ssygs2,SSYGS2)"(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void ssygs2(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygs2(itype, uplo, n, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygst "F_FUNC(ssygst,SSYGST)"(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void ssygst(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygst(itype, uplo, n, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssygst(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6362
 *     void _fortran_ssygst "F_FUNC(ssygst,SSYGST)"(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void ssygst(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil:
 *     _fortran_ssygst(itype, uplo, n, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssygst,SSYGST)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygst "F_FUNC(ssygst,SSYGST)"(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void ssygst(int *itype, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygst(itype, uplo, n, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygv "F_FUNC(ssygv,SSYGV)"(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *info) nogil
 * cdef void ssygv(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygv(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssygv(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6367
 *     void _fortran_ssygv "F_FUNC(ssygv,SSYGV)"(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *info) nogil
 * cdef void ssygv(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *info) nogil:
 *     _fortran_ssygv(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssygv,SSYGV)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_w, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygv "F_FUNC(ssygv,SSYGV)"(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *info) nogil
 * cdef void ssygv(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygv(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygvd "F_FUNC(ssygvd,SSYGVD)"(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssygvd(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygvd(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssygvd(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6372
 *     void _fortran_ssygvd "F_FUNC(ssygvd,SSYGVD)"(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssygvd(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_ssygvd(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssygvd,SSYGVD)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_w, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygvd "F_FUNC(ssygvd,SSYGVD)"(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ssygvd(int *itype, char *jobz, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *w, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygvd(itype, jobz, uplo, n, a, lda, b, ldb, w, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygvx "F_FUNC(ssygvx,SSYGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void ssygvx(int *itype, char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygvx(itype, jobz, range, uplo, n, a, lda, b, ldb, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssygvx(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6377
 *     void _fortran_ssygvx "F_FUNC(ssygvx,SSYGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void ssygvx(int *itype, char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_ssygvx(itype, jobz, range, uplo, n, a, lda, b, ldb, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssygvx,SSYGVX)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, __pyx_v_z, __pyx_v_ldz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssygvx "F_FUNC(ssygvx,SSYGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil
 * cdef void ssygvx(int *itype, char *jobz, char *range, char *uplo, int *n, s *a, int *lda, s *b, int *ldb, s *vl, s *vu, int *il, int *iu, s *abstol, int *m, s *w, s *z, int *ldz, s *work, int *lwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssygvx(itype, jobz, range, uplo, n, a, lda, b, ldb, vl, vu, il, iu, abstol, m, w, z, ldz, work, lwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyrfs "F_FUNC(ssyrfs,SSYRFS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void ssyrfs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyrfs(uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssyrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6382
 *     void _fortran_ssyrfs "F_FUNC(ssyrfs,SSYRFS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void ssyrfs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_ssyrfs(uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssyrfs,SSYRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssyrfs "F_FUNC(ssyrfs,SSYRFS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void ssyrfs(char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssyrfs(uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssysv "F_FUNC(ssysv,SSYSV)"(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, s *work, int *lwork, int *info) nogil
 * cdef void ssysv(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssysv(uplo, n, nrhs, a, lda, ipiv, b, ldb, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssysv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6387
 *     void _fortran_ssysv "F_FUNC(ssysv,SSYSV)"(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, s *work, int *lwork, int *info) nogil
 * cdef void ssysv(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, s *work, int *lwork, int *info) nogil:
 *     _fortran_ssysv(uplo, n, nrhs, a, lda, ipiv, b, ldb, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssysv,SSYSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssysv "F_FUNC(ssysv,SSYSV)"(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, s *work, int *lwork, int *info) nogil
 * cdef void ssysv(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssysv(uplo, n, nrhs, a, lda, ipiv, b, ldb, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssysvx "F_FUNC(ssysvx,SSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ssysvx(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssysvx(fact, uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssysvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6392
 *     void _fortran_ssysvx "F_FUNC(ssysvx,SSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ssysvx(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_ssysvx(fact, uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssysvx,SSYSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_af, __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssysvx "F_FUNC(ssysvx,SSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ssysvx(char *fact, char *uplo, int *n, int *nrhs, s *a, int *lda, s *af, int *ldaf, int *ipiv, s *b, int *ldb, s *x, int *ldx, s *rcond, s *ferr, s *berr, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssysvx(fact, uplo, n, nrhs, a, lda, af, ldaf, ipiv, b, ldb, x, ldx, rcond, ferr, berr, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytd2 "F_FUNC(ssytd2,SSYTD2)"(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, int *info) nogil
 * cdef void ssytd2(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytd2(uplo, n, a, lda, d, e, tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssytd2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6397
 *     void _fortran_ssytd2 "F_FUNC(ssytd2,SSYTD2)"(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, int *info) nogil
 * cdef void ssytd2(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, int *info) nogil:
 *     _fortran_ssytd2(uplo, n, a, lda, d, e, tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssytd2,SSYTD2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tau, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytd2 "F_FUNC(ssytd2,SSYTD2)"(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, int *info) nogil
 * cdef void ssytd2(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytd2(uplo, n, a, lda, d, e, tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytf2 "F_FUNC(ssytf2,SSYTF2)"(char *uplo, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void ssytf2(char *uplo, int *n, s *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytf2(uplo, n, a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssytf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6402
 *     void _fortran_ssytf2 "F_FUNC(ssytf2,SSYTF2)"(char *uplo, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void ssytf2(char *uplo, int *n, s *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_ssytf2(uplo, n, a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssytf2,SSYTF2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytf2 "F_FUNC(ssytf2,SSYTF2)"(char *uplo, int *n, s *a, int *lda, int *ipiv, int *info) nogil
 * cdef void ssytf2(char *uplo, int *n, s *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytf2(uplo, n, a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytrd "F_FUNC(ssytrd,SSYTRD)"(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void ssytrd(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytrd(uplo, n, a, lda, d, e, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssytrd(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6407
 *     void _fortran_ssytrd "F_FUNC(ssytrd,SSYTRD)"(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void ssytrd(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_ssytrd(uplo, n, a, lda, d, e, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssytrd,SSYTRD)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_d, __pyx_v_e, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytrd "F_FUNC(ssytrd,SSYTRD)"(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void ssytrd(char *uplo, int *n, s *a, int *lda, s *d, s *e, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytrd(uplo, n, a, lda, d, e, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytrf "F_FUNC(ssytrf,SSYTRF)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil
 * cdef void ssytrf(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytrf(uplo, n, a, lda, ipiv, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssytrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6412
 *     void _fortran_ssytrf "F_FUNC(ssytrf,SSYTRF)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil
 * cdef void ssytrf(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil:
 *     _fortran_ssytrf(uplo, n, a, lda, ipiv, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssytrf,SSYTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytrf "F_FUNC(ssytrf,SSYTRF)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil
 * cdef void ssytrf(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytrf(uplo, n, a, lda, ipiv, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytri "F_FUNC(ssytri,SSYTRI)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *info) nogil
 * cdef void ssytri(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytri(uplo, n, a, lda, ipiv, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssytri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6417
 *     void _fortran_ssytri "F_FUNC(ssytri,SSYTRI)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *info) nogil
 * cdef void ssytri(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *info) nogil:
 *     _fortran_ssytri(uplo, n, a, lda, ipiv, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssytri,SSYTRI)(__pyx_v_uplo, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytri "F_FUNC(ssytri,SSYTRI)"(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *info) nogil
 * cdef void ssytri(char *uplo, int *n, s *a, int *lda, int *ipiv, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytri(uplo, n, a, lda, ipiv, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytrs "F_FUNC(ssytrs,SSYTRS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void ssytrs(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytrs(uplo, n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ssytrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6422
 *     void _fortran_ssytrs "F_FUNC(ssytrs,SSYTRS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void ssytrs(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:
 *     _fortran_ssytrs(uplo, n, nrhs, a, lda, ipiv, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ssytrs,SSYTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_ipiv, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ssytrs "F_FUNC(ssytrs,SSYTRS)"(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil
 * cdef void ssytrs(char *uplo, int *n, int *nrhs, s *a, int *lda, int *ipiv, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ssytrs(uplo, n, nrhs, a, lda, ipiv, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stbcon "F_FUNC(stbcon,STBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, s *ab, int *ldab, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void stbcon(char *norm, char *uplo, char *diag, int *n, int *kd, s *ab, int *ldab, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stbcon(norm, uplo, diag, n, kd, ab, ldab, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stbcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6427
 *     void _fortran_stbcon "F_FUNC(stbcon,STBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, s *ab, int *ldab, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void stbcon(char *norm, char *uplo, char *diag, int *n, int *kd, s *ab, int *ldab, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_stbcon(norm, uplo, diag, n, kd, ab, ldab, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stbcon,STBCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_ab, __pyx_v_ldab, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stbcon "F_FUNC(stbcon,STBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, s *ab, int *ldab, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void stbcon(char *norm, char *uplo, char *diag, int *n, int *kd, s *ab, int *ldab, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stbcon(norm, uplo, diag, n, kd, ab, ldab, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stbrfs "F_FUNC(stbrfs,STBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void stbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stbrfs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stbrfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6432
 *     void _fortran_stbrfs "F_FUNC(stbrfs,STBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void stbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_stbrfs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stbrfs,STBRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stbrfs "F_FUNC(stbrfs,STBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void stbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stbrfs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stbtrs "F_FUNC(stbtrs,STBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void stbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stbtrs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stbtrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6437
 *     void _fortran_stbtrs "F_FUNC(stbtrs,STBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void stbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:
 *     _fortran_stbtrs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stbtrs,STBTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, __pyx_v_ab, __pyx_v_ldab, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stbtrs "F_FUNC(stbtrs,STBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil
 * cdef void stbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, s *ab, int *ldab, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stbtrs(uplo, trans, diag, n, kd, nrhs, ab, ldab, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgevc "F_FUNC(stgevc,STGEVC)"(char *side, char *howmny, bint *select, int *n, s *s, int *lds, s *p, int *ldp, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil
 * cdef void stgevc(char *side, char *howmny, bint *select, int *n, s *s, int *lds, s *p, int *ldp, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgevc(side, howmny, select, n, s, lds, p, ldp, vl, ldvl, vr, ldvr, mm, m, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stgevc(char *__pyx_v_side, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, int *__pyx_v_lds, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_p, int *__pyx_v_ldp, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6442
 *     void _fortran_stgevc "F_FUNC(stgevc,STGEVC)"(char *side, char *howmny, bint *select, int *n, s *s, int *lds, s *p, int *ldp, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil
 * cdef void stgevc(char *side, char *howmny, bint *select, int *n, s *s, int *lds, s *p, int *ldp, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil:
 *     _fortran_stgevc(side, howmny, select, n, s, lds, p, ldp, vl, ldvl, vr, ldvr, mm, m, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stgevc,STGEVC)(__pyx_v_side, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, __pyx_v_s, __pyx_v_lds, __pyx_v_p, __pyx_v_ldp, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgevc "F_FUNC(stgevc,STGEVC)"(char *side, char *howmny, bint *select, int *n, s *s, int *lds, s *p, int *ldp, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil
 * cdef void stgevc(char *side, char *howmny, bint *select, int *n, s *s, int *lds, s *p, int *ldp, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgevc(side, howmny, select, n, s, lds, p, ldp, vl, ldvl, vr, ldvr, mm, m, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgex2 "F_FUNC(stgex2,STGEX2)"(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *j1, int *n1, int *n2, s *work, int *lwork, int *info) nogil
 * cdef void stgex2(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *j1, int *n1, int *n2, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgex2(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, j1, n1, n2, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stgex2(int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_j1, int *__pyx_v_n1, int *__pyx_v_n2, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6447
 *     void _fortran_stgex2 "F_FUNC(stgex2,STGEX2)"(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *j1, int *n1, int *n2, s *work, int *lwork, int *info) nogil
 * cdef void stgex2(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *j1, int *n1, int *n2, s *work, int *lwork, int *info) nogil:
 *     _fortran_stgex2(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, j1, n1, n2, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stgex2,STGEX2)(__pyx_v_wantq, __pyx_v_wantz, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_j1, __pyx_v_n1, __pyx_v_n2, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgex2 "F_FUNC(stgex2,STGEX2)"(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *j1, int *n1, int *n2, s *work, int *lwork, int *info) nogil
 * cdef void stgex2(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *j1, int *n1, int *n2, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgex2(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, j1, n1, n2, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgexc "F_FUNC(stgexc,STGEXC)"(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *ifst, int *ilst, s *work, int *lwork, int *info) nogil
 * cdef void stgexc(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *ifst, int *ilst, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgexc(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, ifst, ilst, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stgexc(int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ifst, int *__pyx_v_ilst, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6452
 *     void _fortran_stgexc "F_FUNC(stgexc,STGEXC)"(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *ifst, int *ilst, s *work, int *lwork, int *info) nogil
 * cdef void stgexc(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *ifst, int *ilst, s *work, int *lwork, int *info) nogil:
 *     _fortran_stgexc(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, ifst, ilst, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stgexc,STGEXC)(__pyx_v_wantq, __pyx_v_wantz, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_ifst, __pyx_v_ilst, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgexc "F_FUNC(stgexc,STGEXC)"(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *ifst, int *ilst, s *work, int *lwork, int *info) nogil
 * cdef void stgexc(bint *wantq, bint *wantz, int *n, s *a, int *lda, s *b, int *ldb, s *q, int *ldq, s *z, int *ldz, int *ifst, int *ilst, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgexc(wantq, wantz, n, a, lda, b, ldb, q, ldq, z, ldz, ifst, ilst, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsen "F_FUNC(stgsen,STGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, int *m, s *pl, s *pr, s *dif, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void stgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, int *m, s *pl, s *pr, s *dif, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsen(ijob, wantq, wantz, select, n, a, lda, b, ldb, alphar, alphai, beta, q, ldq, z, ldz, m, pl, pr, dif, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stgsen(int *__pyx_v_ijob, int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphar, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alphai, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_pr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dif, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6457
 *     void _fortran_stgsen "F_FUNC(stgsen,STGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, int *m, s *pl, s *pr, s *dif, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void stgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, int *m, s *pl, s *pr, s *dif, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_stgsen(ijob, wantq, wantz, select, n, a, lda, b, ldb, alphar, alphai, beta, q, ldq, z, ldz, m, pl, pr, dif, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stgsen,STGSEN)(__pyx_v_ijob, __pyx_v_wantq, __pyx_v_wantz, __pyx_v_select, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_alphar, __pyx_v_alphai, __pyx_v_beta, __pyx_v_q, __pyx_v_ldq, __pyx_v_z, __pyx_v_ldz, __pyx_v_m, __pyx_v_pl, __pyx_v_pr, __pyx_v_dif, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsen "F_FUNC(stgsen,STGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, int *m, s *pl, s *pr, s *dif, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void stgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *alphar, s *alphai, s *beta, s *q, int *ldq, s *z, int *ldz, int *m, s *pl, s *pr, s *dif, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsen(ijob, wantq, wantz, select, n, a, lda, b, ldb, alphar, alphai, beta, q, ldq, z, ldz, m, pl, pr, dif, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsja "F_FUNC(stgsja,STGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, s *a, int *lda, s *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, s *u, int *ldu, s *v, int *ldv, s *q, int *ldq, s *work, int *ncycle, int *info) nogil
 * cdef void stgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, s *a, int *lda, s *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, s *u, int *ldu, s *v, int *ldv, s *q, int *ldq, s *work, int *ncycle, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsja(jobu, jobv, jobq, m, p, n, k, l, a, lda, b, ldb, tola, tolb, alpha, beta, u, ldu, v, ldv, q, ldq, work, ncycle, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stgsja(char *__pyx_v_jobu, char *__pyx_v_jobv, char *__pyx_v_jobq, int *__pyx_v_m, int *__pyx_v_p, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tola, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tolb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_beta, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_ncycle, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6462
 *     void _fortran_stgsja "F_FUNC(stgsja,STGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, s *a, int *lda, s *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, s *u, int *ldu, s *v, int *ldv, s *q, int *ldq, s *work, int *ncycle, int *info) nogil
 * cdef void stgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, s *a, int *lda, s *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, s *u, int *ldu, s *v, int *ldv, s *q, int *ldq, s *work, int *ncycle, int *info) nogil:
 *     _fortran_stgsja(jobu, jobv, jobq, m, p, n, k, l, a, lda, b, ldb, tola, tolb, alpha, beta, u, ldu, v, ldv, q, ldq, work, ncycle, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stgsja,STGSJA)(__pyx_v_jobu, __pyx_v_jobv, __pyx_v_jobq, __pyx_v_m, __pyx_v_p, __pyx_v_n, __pyx_v_k, __pyx_v_l, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_tola, __pyx_v_tolb, __pyx_v_alpha, __pyx_v_beta, __pyx_v_u, __pyx_v_ldu, __pyx_v_v, __pyx_v_ldv, __pyx_v_q, __pyx_v_ldq, __pyx_v_work, __pyx_v_ncycle, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsja "F_FUNC(stgsja,STGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, s *a, int *lda, s *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, s *u, int *ldu, s *v, int *ldv, s *q, int *ldq, s *work, int *ncycle, int *info) nogil
 * cdef void stgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, s *a, int *lda, s *b, int *ldb, s *tola, s *tolb, s *alpha, s *beta, s *u, int *ldu, s *v, int *ldv, s *q, int *ldq, s *work, int *ncycle, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsja(jobu, jobv, jobq, m, p, n, k, l, a, lda, b, ldb, tola, tolb, alpha, beta, u, ldu, v, ldv, q, ldq, work, ncycle, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsna "F_FUNC(stgsna,STGSNA)"(char *job, char *howmny, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *dif, int *mm, int *m, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void stgsna(char *job, char *howmny, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *dif, int *mm, int *m, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsna(job, howmny, select, n, a, lda, b, ldb, vl, ldvl, vr, ldvr, s, dif, mm, m, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stgsna(char *__pyx_v_job, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dif, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6467
 *     void _fortran_stgsna "F_FUNC(stgsna,STGSNA)"(char *job, char *howmny, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *dif, int *mm, int *m, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void stgsna(char *job, char *howmny, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *dif, int *mm, int *m, s *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_stgsna(job, howmny, select, n, a, lda, b, ldb, vl, ldvl, vr, ldvr, s, dif, mm, m, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stgsna,STGSNA)(__pyx_v_job, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_s, __pyx_v_dif, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsna "F_FUNC(stgsna,STGSNA)"(char *job, char *howmny, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *dif, int *mm, int *m, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void stgsna(char *job, char *howmny, bint *select, int *n, s *a, int *lda, s *b, int *ldb, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *dif, int *mm, int *m, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsna(job, howmny, select, n, a, lda, b, ldb, vl, ldvl, vr, ldvr, s, dif, mm, m, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsy2 "F_FUNC(stgsy2,STGSY2)"(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *iwork, int *pq, int *info) nogil
 * cdef void stgsy2(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *iwork, int *pq, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsy2(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, rdsum, rdscal, iwork, pq, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stgsy2(char *__pyx_v_trans, int *__pyx_v_ijob, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, int *__pyx_v_ldd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, int *__pyx_v_lde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_f, int *__pyx_v_ldf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rdsum, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rdscal, int *__pyx_v_iwork, int *__pyx_v_pq, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6472
 *     void _fortran_stgsy2 "F_FUNC(stgsy2,STGSY2)"(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *iwork, int *pq, int *info) nogil
 * cdef void stgsy2(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *iwork, int *pq, int *info) nogil:
 *     _fortran_stgsy2(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, rdsum, rdscal, iwork, pq, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stgsy2,STGSY2)(__pyx_v_trans, __pyx_v_ijob, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_c, __pyx_v_ldc, __pyx_v_d, __pyx_v_ldd, __pyx_v_e, __pyx_v_lde, __pyx_v_f, __pyx_v_ldf, __pyx_v_scale, __pyx_v_rdsum, __pyx_v_rdscal, __pyx_v_iwork, __pyx_v_pq, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsy2 "F_FUNC(stgsy2,STGSY2)"(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *iwork, int *pq, int *info) nogil
 * cdef void stgsy2(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *rdsum, s *rdscal, int *iwork, int *pq, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsy2(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, rdsum, rdscal, iwork, pq, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsyl "F_FUNC(stgsyl,STGSYL)"(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *dif, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void stgsyl(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *dif, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsyl(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, dif, work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stgsyl(char *__pyx_v_trans, int *__pyx_v_ijob, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_d, int *__pyx_v_ldd, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_e, int *__pyx_v_lde, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_f, int *__pyx_v_ldf, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_dif, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6477
 *     void _fortran_stgsyl "F_FUNC(stgsyl,STGSYL)"(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *dif, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void stgsyl(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *dif, s *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_stgsyl(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, dif, work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stgsyl,STGSYL)(__pyx_v_trans, __pyx_v_ijob, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_c, __pyx_v_ldc, __pyx_v_d, __pyx_v_ldd, __pyx_v_e, __pyx_v_lde, __pyx_v_f, __pyx_v_ldf, __pyx_v_scale, __pyx_v_dif, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stgsyl "F_FUNC(stgsyl,STGSYL)"(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *dif, s *work, int *lwork, int *iwork, int *info) nogil
 * cdef void stgsyl(char *trans, int *ijob, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *d, int *ldd, s *e, int *lde, s *f, int *ldf, s *scale, s *dif, s *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stgsyl(trans, ijob, m, n, a, lda, b, ldb, c, ldc, d, ldd, e, lde, f, ldf, scale, dif, work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stpcon "F_FUNC(stpcon,STPCON)"(char *norm, char *uplo, char *diag, int *n, s *ap, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void stpcon(char *norm, char *uplo, char *diag, int *n, s *ap, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stpcon(norm, uplo, diag, n, ap, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stpcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6482
 *     void _fortran_stpcon "F_FUNC(stpcon,STPCON)"(char *norm, char *uplo, char *diag, int *n, s *ap, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void stpcon(char *norm, char *uplo, char *diag, int *n, s *ap, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_stpcon(norm, uplo, diag, n, ap, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stpcon,STPCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_ap, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stpcon "F_FUNC(stpcon,STPCON)"(char *norm, char *uplo, char *diag, int *n, s *ap, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void stpcon(char *norm, char *uplo, char *diag, int *n, s *ap, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stpcon(norm, uplo, diag, n, ap, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stprfs "F_FUNC(stprfs,STPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void stprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stprfs(uplo, trans, diag, n, nrhs, ap, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stprfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6487
 *     void _fortran_stprfs "F_FUNC(stprfs,STPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void stprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_stprfs(uplo, trans, diag, n, nrhs, ap, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stprfs,STPRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stprfs "F_FUNC(stprfs,STPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void stprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stprfs(uplo, trans, diag, n, nrhs, ap, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stptri "F_FUNC(stptri,STPTRI)"(char *uplo, char *diag, int *n, s *ap, int *info) nogil
 * cdef void stptri(char *uplo, char *diag, int *n, s *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stptri(uplo, diag, n, ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stptri(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6492
 *     void _fortran_stptri "F_FUNC(stptri,STPTRI)"(char *uplo, char *diag, int *n, s *ap, int *info) nogil
 * cdef void stptri(char *uplo, char *diag, int *n, s *ap, int *info) nogil:
 *     _fortran_stptri(uplo, diag, n, ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stptri,STPTRI)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_ap, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stptri "F_FUNC(stptri,STPTRI)"(char *uplo, char *diag, int *n, s *ap, int *info) nogil
 * cdef void stptri(char *uplo, char *diag, int *n, s *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stptri(uplo, diag, n, ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stptrs "F_FUNC(stptrs,STPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void stptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stptrs(uplo, trans, diag, n, nrhs, ap, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stptrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6497
 *     void _fortran_stptrs "F_FUNC(stptrs,STPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void stptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:
 *     _fortran_stptrs(uplo, trans, diag, n, nrhs, ap, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stptrs,STPTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, __pyx_v_ap, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stptrs "F_FUNC(stptrs,STPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil
 * cdef void stptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *ap, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stptrs(uplo, trans, diag, n, nrhs, ap, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strcon "F_FUNC(strcon,STRCON)"(char *norm, char *uplo, char *diag, int *n, s *a, int *lda, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void strcon(char *norm, char *uplo, char *diag, int *n, s *a, int *lda, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strcon(norm, uplo, diag, n, a, lda, rcond, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6502
 *     void _fortran_strcon "F_FUNC(strcon,STRCON)"(char *norm, char *uplo, char *diag, int *n, s *a, int *lda, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void strcon(char *norm, char *uplo, char *diag, int *n, s *a, int *lda, s *rcond, s *work, int *iwork, int *info) nogil:
 *     _fortran_strcon(norm, uplo, diag, n, a, lda, rcond, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strcon,STRCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_rcond, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strcon "F_FUNC(strcon,STRCON)"(char *norm, char *uplo, char *diag, int *n, s *a, int *lda, s *rcond, s *work, int *iwork, int *info) nogil
 * cdef void strcon(char *norm, char *uplo, char *diag, int *n, s *a, int *lda, s *rcond, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strcon(norm, uplo, diag, n, a, lda, rcond, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strevc "F_FUNC(strevc,STREVC)"(char *side, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil
 * cdef void strevc(char *side, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strevc(side, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, mm, m, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strevc(char *__pyx_v_side, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6507
 *     void _fortran_strevc "F_FUNC(strevc,STREVC)"(char *side, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil
 * cdef void strevc(char *side, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil:
 *     _fortran_strevc(side, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, mm, m, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strevc,STREVC)(__pyx_v_side, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strevc "F_FUNC(strevc,STREVC)"(char *side, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil
 * cdef void strevc(char *side, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, int *mm, int *m, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strevc(side, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, mm, m, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strexc "F_FUNC(strexc,STREXC)"(char *compq, int *n, s *t, int *ldt, s *q, int *ldq, int *ifst, int *ilst, s *work, int *info) nogil
 * cdef void strexc(char *compq, int *n, s *t, int *ldt, s *q, int *ldq, int *ifst, int *ilst, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strexc(compq, n, t, ldt, q, ldq, ifst, ilst, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strexc(char *__pyx_v_compq, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_ifst, int *__pyx_v_ilst, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6512
 *     void _fortran_strexc "F_FUNC(strexc,STREXC)"(char *compq, int *n, s *t, int *ldt, s *q, int *ldq, int *ifst, int *ilst, s *work, int *info) nogil
 * cdef void strexc(char *compq, int *n, s *t, int *ldt, s *q, int *ldq, int *ifst, int *ilst, s *work, int *info) nogil:
 *     _fortran_strexc(compq, n, t, ldt, q, ldq, ifst, ilst, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strexc,STREXC)(__pyx_v_compq, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_q, __pyx_v_ldq, __pyx_v_ifst, __pyx_v_ilst, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strexc "F_FUNC(strexc,STREXC)"(char *compq, int *n, s *t, int *ldt, s *q, int *ldq, int *ifst, int *ilst, s *work, int *info) nogil
 * cdef void strexc(char *compq, int *n, s *t, int *ldt, s *q, int *ldq, int *ifst, int *ilst, s *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strexc(compq, n, t, ldt, q, ldq, ifst, ilst, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strrfs "F_FUNC(strrfs,STRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void strrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strrfs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strrfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_berr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6517
 *     void _fortran_strrfs "F_FUNC(strrfs,STRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void strrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:
 *     _fortran_strrfs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, x, ldx, ferr, berr, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strrfs,STRRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_x, __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strrfs "F_FUNC(strrfs,STRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil
 * cdef void strrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, s *x, int *ldx, s *ferr, s *berr, s *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strrfs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, x, ldx, ferr, berr, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strsen "F_FUNC(strsen,STRSEN)"(char *job, char *compq, bint *select, int *n, s *t, int *ldt, s *q, int *ldq, s *wr, s *wi, int *m, s *s, s *sep, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void strsen(char *job, char *compq, bint *select, int *n, s *t, int *ldt, s *q, int *ldq, s *wr, s *wi, int *m, s *s, s *sep, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strsen(job, compq, select, n, t, ldt, q, ldq, wr, wi, m, s, sep, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strsen(char *__pyx_v_job, char *__pyx_v_compq, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_wi, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sep, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6522
 *     void _fortran_strsen "F_FUNC(strsen,STRSEN)"(char *job, char *compq, bint *select, int *n, s *t, int *ldt, s *q, int *ldq, s *wr, s *wi, int *m, s *s, s *sep, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void strsen(char *job, char *compq, bint *select, int *n, s *t, int *ldt, s *q, int *ldq, s *wr, s *wi, int *m, s *s, s *sep, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_strsen(job, compq, select, n, t, ldt, q, ldq, wr, wi, m, s, sep, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strsen,STRSEN)(__pyx_v_job, __pyx_v_compq, __pyx_v_select, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_q, __pyx_v_ldq, __pyx_v_wr, __pyx_v_wi, __pyx_v_m, __pyx_v_s, __pyx_v_sep, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strsen "F_FUNC(strsen,STRSEN)"(char *job, char *compq, bint *select, int *n, s *t, int *ldt, s *q, int *ldq, s *wr, s *wi, int *m, s *s, s *sep, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void strsen(char *job, char *compq, bint *select, int *n, s *t, int *ldt, s *q, int *ldq, s *wr, s *wi, int *m, s *s, s *sep, s *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strsen(job, compq, select, n, t, ldt, q, ldq, wr, wi, m, s, sep, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strsna "F_FUNC(strsna,STRSNA)"(char *job, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *sep, int *mm, int *m, s *work, int *ldwork, int *iwork, int *info) nogil
 * cdef void strsna(char *job, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *sep, int *mm, int *m, s *work, int *ldwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strsna(job, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, s, sep, mm, m, work, ldwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strsna(char *__pyx_v_job, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_sep, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_ldwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6527
 *     void _fortran_strsna "F_FUNC(strsna,STRSNA)"(char *job, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *sep, int *mm, int *m, s *work, int *ldwork, int *iwork, int *info) nogil
 * cdef void strsna(char *job, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *sep, int *mm, int *m, s *work, int *ldwork, int *iwork, int *info) nogil:
 *     _fortran_strsna(job, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, s, sep, mm, m, work, ldwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strsna,STRSNA)(__pyx_v_job, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, __pyx_v_t, __pyx_v_ldt, __pyx_v_vl, __pyx_v_ldvl, __pyx_v_vr, __pyx_v_ldvr, __pyx_v_s, __pyx_v_sep, __pyx_v_mm, __pyx_v_m, __pyx_v_work, __pyx_v_ldwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strsna "F_FUNC(strsna,STRSNA)"(char *job, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *sep, int *mm, int *m, s *work, int *ldwork, int *iwork, int *info) nogil
 * cdef void strsna(char *job, char *howmny, bint *select, int *n, s *t, int *ldt, s *vl, int *ldvl, s *vr, int *ldvr, s *s, s *sep, int *mm, int *m, s *work, int *ldwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strsna(job, howmny, select, n, t, ldt, vl, ldvl, vr, ldvr, s, sep, mm, m, work, ldwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strsyl "F_FUNC(strsyl,STRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *scale, int *info) nogil
 * cdef void strsyl(char *trana, char *tranb, int *isgn, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strsyl(trana, tranb, isgn, m, n, a, lda, b, ldb, c, ldc, scale, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strsyl(char *__pyx_v_trana, char *__pyx_v_tranb, int *__pyx_v_isgn, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_scale, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6532
 *     void _fortran_strsyl "F_FUNC(strsyl,STRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *scale, int *info) nogil
 * cdef void strsyl(char *trana, char *tranb, int *isgn, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *scale, int *info) nogil:
 *     _fortran_strsyl(trana, tranb, isgn, m, n, a, lda, b, ldb, c, ldc, scale, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strsyl,STRSYL)(__pyx_v_trana, __pyx_v_tranb, __pyx_v_isgn, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_c, __pyx_v_ldc, __pyx_v_scale, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strsyl "F_FUNC(strsyl,STRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *scale, int *info) nogil
 * cdef void strsyl(char *trana, char *tranb, int *isgn, int *m, int *n, s *a, int *lda, s *b, int *ldb, s *c, int *ldc, s *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strsyl(trana, tranb, isgn, m, n, a, lda, b, ldb, c, ldc, scale, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strti2 "F_FUNC(strti2,STRTI2)"(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil
 * cdef void strti2(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strti2(uplo, diag, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strti2(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6537
 *     void _fortran_strti2 "F_FUNC(strti2,STRTI2)"(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil
 * cdef void strti2(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil:
 *     _fortran_strti2(uplo, diag, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strti2,STRTI2)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strti2 "F_FUNC(strti2,STRTI2)"(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil
 * cdef void strti2(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strti2(uplo, diag, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strtri "F_FUNC(strtri,STRTRI)"(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil
 * cdef void strtri(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strtri(uplo, diag, n, a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strtri(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6542
 *     void _fortran_strtri "F_FUNC(strtri,STRTRI)"(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil
 * cdef void strtri(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil:
 *     _fortran_strtri(uplo, diag, n, a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strtri,STRTRI)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strtri "F_FUNC(strtri,STRTRI)"(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil
 * cdef void strtri(char *uplo, char *diag, int *n, s *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strtri(uplo, diag, n, a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strtrs "F_FUNC(strtrs,STRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void strtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strtrs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_strtrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6547
 *     void _fortran_strtrs "F_FUNC(strtrs,STRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void strtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:
 *     _fortran_strtrs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(strtrs,STRTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, __pyx_v_a, __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_strtrs "F_FUNC(strtrs,STRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil
 * cdef void strtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, s *a, int *lda, s *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_strtrs(uplo, trans, diag, n, nrhs, a, lda, b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stzrzf "F_FUNC(stzrzf,STZRZF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void stzrzf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stzrzf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_stzrzf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_s *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6552
 *     void _fortran_stzrzf "F_FUNC(stzrzf,STZRZF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void stzrzf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:
 *     _fortran_stzrzf(m, n, a, lda, tau, work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(stzrzf,STZRZF)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_stzrzf "F_FUNC(stzrzf,STZRZF)"(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil
 * cdef void stzrzf(int *m, int *n, s *a, int *lda, s *tau, s *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_stzrzf(m, n, a, lda, tau, work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zbdsqr "F_FUNC(zbdsqr,ZBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, npy_complex128 *vt, int *ldvt, npy_complex128 *u, int *ldu, npy_complex128 *c, int *ldc, d *rwork, int *info) nogil
 * cdef void zbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, z *vt, int *ldvt, z *u, int *ldu, z *c, int *ldc, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zbdsqr(uplo, n, ncvt, nru, ncc, d, e, <npy_complex128*>vt, ldvt, <npy_complex128*>u, ldu, <npy_complex128*>c, ldc, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zbdsqr(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ncvt, int *__pyx_v_nru, int *__pyx_v_ncc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6557
 *     void _fortran_zbdsqr "F_FUNC(zbdsqr,ZBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, npy_complex128 *vt, int *ldvt, npy_complex128 *u, int *ldu, npy_complex128 *c, int *ldc, d *rwork, int *info) nogil
 * cdef void zbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, z *vt, int *ldvt, z *u, int *ldu, z *c, int *ldc, d *rwork, int *info) nogil:
 *     _fortran_zbdsqr(uplo, n, ncvt, nru, ncc, d, e, <npy_complex128*>vt, ldvt, <npy_complex128*>u, ldu, <npy_complex128*>c, ldc, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zbdsqr,ZBDSQR)(__pyx_v_uplo, __pyx_v_n, __pyx_v_ncvt, __pyx_v_nru, __pyx_v_ncc, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_vt), __pyx_v_ldvt, ((npy_complex128 *)__pyx_v_u), __pyx_v_ldu, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zbdsqr "F_FUNC(zbdsqr,ZBDSQR)"(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, npy_complex128 *vt, int *ldvt, npy_complex128 *u, int *ldu, npy_complex128 *c, int *ldc, d *rwork, int *info) nogil
 * cdef void zbdsqr(char *uplo, int *n, int *ncvt, int *nru, int *ncc, d *d, d *e, z *vt, int *ldvt, z *u, int *ldu, z *c, int *ldc, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zbdsqr(uplo, n, ncvt, nru, ncc, d, e, <npy_complex128*>vt, ldvt, <npy_complex128*>u, ldu, <npy_complex128*>c, ldc, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zdrscl "F_FUNC(zdrscl,ZDRSCL)"(int *n, d *sa, npy_complex128 *sx, int *incx) nogil
 * cdef void zdrscl(int *n, d *sa, z *sx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zdrscl(n, sa, <npy_complex128*>sx, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zdrscl(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sa, __pyx_t_double_complex *__pyx_v_sx, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":6562
 *     void _fortran_zdrscl "F_FUNC(zdrscl,ZDRSCL)"(int *n, d *sa, npy_complex128 *sx, int *incx) nogil
 * cdef void zdrscl(int *n, d *sa, z *sx, int *incx) nogil:
 *     _fortran_zdrscl(n, sa, <npy_complex128*>sx, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zdrscl,ZDRSCL)(__pyx_v_n, __pyx_v_sa, ((npy_complex128 *)__pyx_v_sx), __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":6561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zdrscl "F_FUNC(zdrscl,ZDRSCL)"(int *n, d *sa, npy_complex128 *sx, int *incx) nogil
 * cdef void zdrscl(int *n, d *sa, z *sx, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zdrscl(n, sa, <npy_complex128*>sx, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbbrd "F_FUNC(zgbbrd,ZGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *pt, int *ldpt, npy_complex128 *c, int *ldc, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, z *ab, int *ldab, d *d, d *e, z *q, int *ldq, z *pt, int *ldpt, z *c, int *ldc, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbbrd(vect, m, n, ncc, kl, ku, <npy_complex128*>ab, ldab, d, e, <npy_complex128*>q, ldq, <npy_complex128*>pt, ldpt, <npy_complex128*>c, ldc, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbbrd(char *__pyx_v_vect, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_ncc, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_pt, int *__pyx_v_ldpt, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6567
 *     void _fortran_zgbbrd "F_FUNC(zgbbrd,ZGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *pt, int *ldpt, npy_complex128 *c, int *ldc, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, z *ab, int *ldab, d *d, d *e, z *q, int *ldq, z *pt, int *ldpt, z *c, int *ldc, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgbbrd(vect, m, n, ncc, kl, ku, <npy_complex128*>ab, ldab, d, e, <npy_complex128*>q, ldq, <npy_complex128*>pt, ldpt, <npy_complex128*>c, ldc, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbbrd,ZGBBRD)(__pyx_v_vect, __pyx_v_m, __pyx_v_n, __pyx_v_ncc, __pyx_v_kl, __pyx_v_ku, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_pt), __pyx_v_ldpt, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbbrd "F_FUNC(zgbbrd,ZGBBRD)"(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *pt, int *ldpt, npy_complex128 *c, int *ldc, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbbrd(char *vect, int *m, int *n, int *ncc, int *kl, int *ku, z *ab, int *ldab, d *d, d *e, z *q, int *ldq, z *pt, int *ldpt, z *c, int *ldc, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbbrd(vect, m, n, ncc, kl, ku, <npy_complex128*>ab, ldab, d, e, <npy_complex128*>q, ldq, <npy_complex128*>pt, ldpt, <npy_complex128*>c, ldc, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbcon "F_FUNC(zgbcon,ZGBCON)"(char *norm, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbcon(char *norm, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbcon(norm, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbcon(char *__pyx_v_norm, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6572
 *     void _fortran_zgbcon "F_FUNC(zgbcon,ZGBCON)"(char *norm, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbcon(char *norm, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgbcon(norm, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, anorm, rcond, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbcon,ZGBCON)(__pyx_v_norm, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbcon "F_FUNC(zgbcon,ZGBCON)"(char *norm, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbcon(char *norm, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbcon(norm, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbequ "F_FUNC(zgbequ,ZGBEQU)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void zgbequ(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbequ(m, n, kl, ku, <npy_complex128*>ab, ldab, r, c, rowcnd, colcnd, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbequ(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6577
 *     void _fortran_zgbequ "F_FUNC(zgbequ,ZGBEQU)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void zgbequ(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:
 *     _fortran_zgbequ(m, n, kl, ku, <npy_complex128*>ab, ldab, r, c, rowcnd, colcnd, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbequ,ZGBEQU)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbequ "F_FUNC(zgbequ,ZGBEQU)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void zgbequ(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbequ(m, n, kl, ku, <npy_complex128*>ab, ldab, r, c, rowcnd, colcnd, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbrfs "F_FUNC(zgbrfs,ZGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbrfs(trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbrfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_afb, int *__pyx_v_ldafb, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6582
 *     void _fortran_zgbrfs "F_FUNC(zgbrfs,ZGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgbrfs(trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbrfs,ZGBRFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_afb), __pyx_v_ldafb, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbrfs "F_FUNC(zgbrfs,ZGBRFS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbrfs(char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbrfs(trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbsv "F_FUNC(zgbsv,ZGBSV)"(int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgbsv(int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbsv(n, kl, ku, nrhs, <npy_complex128*>ab, ldab, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbsv(int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6587
 *     void _fortran_zgbsv "F_FUNC(zgbsv,ZGBSV)"(int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgbsv(int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zgbsv(n, kl, ku, nrhs, <npy_complex128*>ab, ldab, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbsv,ZGBSV)(__pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbsv "F_FUNC(zgbsv,ZGBSV)"(int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgbsv(int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbsv(n, kl, ku, nrhs, <npy_complex128*>ab, ldab, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbsvx "F_FUNC(zgbsvx,ZGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbsvx(fact, trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, ipiv, equed, r, c, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbsvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_afb, int *__pyx_v_ldafb, int *__pyx_v_ipiv, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6592
 *     void _fortran_zgbsvx "F_FUNC(zgbsvx,ZGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgbsvx(fact, trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, ipiv, equed, r, c, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbsvx,ZGBSVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_afb), __pyx_v_ldafb, __pyx_v_ipiv, __pyx_v_equed, __pyx_v_r, __pyx_v_c, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbsvx "F_FUNC(zgbsvx,ZGBSVX)"(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgbsvx(char *fact, char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, int *ipiv, char *equed, d *r, d *c, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbsvx(fact, trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, ipiv, equed, r, c, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbtf2 "F_FUNC(zgbtf2,ZGBTF2)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void zgbtf2(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbtf2(m, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbtf2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6597
 *     void _fortran_zgbtf2 "F_FUNC(zgbtf2,ZGBTF2)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void zgbtf2(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, int *info) nogil:
 *     _fortran_zgbtf2(m, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbtf2,ZGBTF2)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbtf2 "F_FUNC(zgbtf2,ZGBTF2)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void zgbtf2(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbtf2(m, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbtrf "F_FUNC(zgbtrf,ZGBTRF)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void zgbtrf(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbtrf(m, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbtrf(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6602
 *     void _fortran_zgbtrf "F_FUNC(zgbtrf,ZGBTRF)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void zgbtrf(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, int *info) nogil:
 *     _fortran_zgbtrf(m, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbtrf,ZGBTRF)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbtrf "F_FUNC(zgbtrf,ZGBTRF)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, int *ipiv, int *info) nogil
 * cdef void zgbtrf(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbtrf(m, n, kl, ku, <npy_complex128*>ab, ldab, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbtrs "F_FUNC(zgbtrs,ZGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbtrs(trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgbtrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6607
 *     void _fortran_zgbtrs "F_FUNC(zgbtrs,ZGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zgbtrs(trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgbtrs,ZGBTRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgbtrs "F_FUNC(zgbtrs,ZGBTRS)"(char *trans, int *n, int *kl, int *ku, int *nrhs, npy_complex128 *ab, int *ldab, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgbtrs(char *trans, int *n, int *kl, int *ku, int *nrhs, z *ab, int *ldab, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgbtrs(trans, n, kl, ku, nrhs, <npy_complex128*>ab, ldab, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgebak "F_FUNC(zgebak,ZGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, npy_complex128 *v, int *ldv, int *info) nogil
 * cdef void zgebak(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, z *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgebak(job, side, n, ilo, ihi, scale, m, <npy_complex128*>v, ldv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgebak(char *__pyx_v_job, char *__pyx_v_side, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, int *__pyx_v_m, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6612
 *     void _fortran_zgebak "F_FUNC(zgebak,ZGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, npy_complex128 *v, int *ldv, int *info) nogil
 * cdef void zgebak(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, z *v, int *ldv, int *info) nogil:
 *     _fortran_zgebak(job, side, n, ilo, ihi, scale, m, <npy_complex128*>v, ldv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgebak,ZGEBAK)(__pyx_v_job, __pyx_v_side, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_m, ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgebak "F_FUNC(zgebak,ZGEBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, npy_complex128 *v, int *ldv, int *info) nogil
 * cdef void zgebak(char *job, char *side, int *n, int *ilo, int *ihi, d *scale, int *m, z *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgebak(job, side, n, ilo, ihi, scale, m, <npy_complex128*>v, ldv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgebal "F_FUNC(zgebal,ZGEBAL)"(char *job, int *n, npy_complex128 *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil
 * cdef void zgebal(char *job, int *n, z *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgebal(job, n, <npy_complex128*>a, lda, ilo, ihi, scale, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgebal(char *__pyx_v_job, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6617
 *     void _fortran_zgebal "F_FUNC(zgebal,ZGEBAL)"(char *job, int *n, npy_complex128 *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil
 * cdef void zgebal(char *job, int *n, z *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil:
 *     _fortran_zgebal(job, n, <npy_complex128*>a, lda, ilo, ihi, scale, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgebal,ZGEBAL)(__pyx_v_job, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgebal "F_FUNC(zgebal,ZGEBAL)"(char *job, int *n, npy_complex128 *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil
 * cdef void zgebal(char *job, int *n, z *a, int *lda, int *ilo, int *ihi, d *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgebal(job, n, <npy_complex128*>a, lda, ilo, ihi, scale, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgebd2 "F_FUNC(zgebd2,ZGEBD2)"(int *m, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *work, int *info) nogil
 * cdef void zgebd2(int *m, int *n, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgebd2(m, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgebd2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_tauq, __pyx_t_double_complex *__pyx_v_taup, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6622
 *     void _fortran_zgebd2 "F_FUNC(zgebd2,ZGEBD2)"(int *m, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *work, int *info) nogil
 * cdef void zgebd2(int *m, int *n, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *work, int *info) nogil:
 *     _fortran_zgebd2(m, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgebd2,ZGEBD2)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_tauq), ((npy_complex128 *)__pyx_v_taup), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgebd2 "F_FUNC(zgebd2,ZGEBD2)"(int *m, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *work, int *info) nogil
 * cdef void zgebd2(int *m, int *n, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgebd2(m, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgebrd "F_FUNC(zgebrd,ZGEBRD)"(int *m, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgebrd(int *m, int *n, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgebrd(m, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgebrd(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_tauq, __pyx_t_double_complex *__pyx_v_taup, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6627
 *     void _fortran_zgebrd "F_FUNC(zgebrd,ZGEBRD)"(int *m, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgebrd(int *m, int *n, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgebrd(m, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgebrd,ZGEBRD)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_tauq), ((npy_complex128 *)__pyx_v_taup), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgebrd "F_FUNC(zgebrd,ZGEBRD)"(int *m, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgebrd(int *m, int *n, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgebrd(m, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgecon "F_FUNC(zgecon,ZGECON)"(char *norm, int *n, npy_complex128 *a, int *lda, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgecon(char *norm, int *n, z *a, int *lda, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgecon(norm, n, <npy_complex128*>a, lda, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgecon(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6632
 *     void _fortran_zgecon "F_FUNC(zgecon,ZGECON)"(char *norm, int *n, npy_complex128 *a, int *lda, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgecon(char *norm, int *n, z *a, int *lda, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgecon(norm, n, <npy_complex128*>a, lda, anorm, rcond, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgecon,ZGECON)(__pyx_v_norm, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgecon "F_FUNC(zgecon,ZGECON)"(char *norm, int *n, npy_complex128 *a, int *lda, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgecon(char *norm, int *n, z *a, int *lda, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgecon(norm, n, <npy_complex128*>a, lda, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeequ "F_FUNC(zgeequ,ZGEEQU)"(int *m, int *n, npy_complex128 *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void zgeequ(int *m, int *n, z *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeequ(m, n, <npy_complex128*>a, lda, r, c, rowcnd, colcnd, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeequ(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6637
 *     void _fortran_zgeequ "F_FUNC(zgeequ,ZGEEQU)"(int *m, int *n, npy_complex128 *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void zgeequ(int *m, int *n, z *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:
 *     _fortran_zgeequ(m, n, <npy_complex128*>a, lda, r, c, rowcnd, colcnd, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeequ,ZGEEQU)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeequ "F_FUNC(zgeequ,ZGEEQU)"(int *m, int *n, npy_complex128 *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil
 * cdef void zgeequ(int *m, int *n, z *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeequ(m, n, <npy_complex128*>a, lda, r, c, rowcnd, colcnd, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgees "F_FUNC(zgees,ZGEES)"(char *jobvs, char *sort, _zselect1 *select, int *n, npy_complex128 *a, int *lda, int *sdim, npy_complex128 *w, npy_complex128 *vs, int *ldvs, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgees(char *jobvs, char *sort, zselect1 *select, int *n, z *a, int *lda, int *sdim, z *w, z *vs, int *ldvs, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgees(jobvs, sort, <_zselect1*>select, n, <npy_complex128*>a, lda, sdim, <npy_complex128*>w, <npy_complex128*>vs, ldvs, <npy_complex128*>work, lwork, rwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgees(char *__pyx_v_jobvs, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_zselect1 *__pyx_v_select, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_sdim, __pyx_t_double_complex *__pyx_v_w, __pyx_t_double_complex *__pyx_v_vs, int *__pyx_v_ldvs, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6642
 *     void _fortran_zgees "F_FUNC(zgees,ZGEES)"(char *jobvs, char *sort, _zselect1 *select, int *n, npy_complex128 *a, int *lda, int *sdim, npy_complex128 *w, npy_complex128 *vs, int *ldvs, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgees(char *jobvs, char *sort, zselect1 *select, int *n, z *a, int *lda, int *sdim, z *w, z *vs, int *ldvs, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:
 *     _fortran_zgees(jobvs, sort, <_zselect1*>select, n, <npy_complex128*>a, lda, sdim, <npy_complex128*>w, <npy_complex128*>vs, ldvs, <npy_complex128*>work, lwork, rwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgees,ZGEES)(__pyx_v_jobvs, __pyx_v_sort, ((_zselect1 *)__pyx_v_select), __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_sdim, ((npy_complex128 *)__pyx_v_w), ((npy_complex128 *)__pyx_v_vs), __pyx_v_ldvs, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgees "F_FUNC(zgees,ZGEES)"(char *jobvs, char *sort, _zselect1 *select, int *n, npy_complex128 *a, int *lda, int *sdim, npy_complex128 *w, npy_complex128 *vs, int *ldvs, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgees(char *jobvs, char *sort, zselect1 *select, int *n, z *a, int *lda, int *sdim, z *w, z *vs, int *ldvs, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgees(jobvs, sort, <_zselect1*>select, n, <npy_complex128*>a, lda, sdim, <npy_complex128*>w, <npy_complex128*>vs, ldvs, <npy_complex128*>work, lwork, rwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeesx "F_FUNC(zgeesx,ZGEESX)"(char *jobvs, char *sort, _zselect1 *select, char *sense, int *n, npy_complex128 *a, int *lda, int *sdim, npy_complex128 *w, npy_complex128 *vs, int *ldvs, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgeesx(char *jobvs, char *sort, zselect1 *select, char *sense, int *n, z *a, int *lda, int *sdim, z *w, z *vs, int *ldvs, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeesx(jobvs, sort, <_zselect1*>select, sense, n, <npy_complex128*>a, lda, sdim, <npy_complex128*>w, <npy_complex128*>vs, ldvs, rconde, rcondv, <npy_complex128*>work, lwork, rwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeesx(char *__pyx_v_jobvs, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_zselect1 *__pyx_v_select, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_sdim, __pyx_t_double_complex *__pyx_v_w, __pyx_t_double_complex *__pyx_v_vs, int *__pyx_v_ldvs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcondv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6647
 *     void _fortran_zgeesx "F_FUNC(zgeesx,ZGEESX)"(char *jobvs, char *sort, _zselect1 *select, char *sense, int *n, npy_complex128 *a, int *lda, int *sdim, npy_complex128 *w, npy_complex128 *vs, int *ldvs, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgeesx(char *jobvs, char *sort, zselect1 *select, char *sense, int *n, z *a, int *lda, int *sdim, z *w, z *vs, int *ldvs, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:
 *     _fortran_zgeesx(jobvs, sort, <_zselect1*>select, sense, n, <npy_complex128*>a, lda, sdim, <npy_complex128*>w, <npy_complex128*>vs, ldvs, rconde, rcondv, <npy_complex128*>work, lwork, rwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeesx,ZGEESX)(__pyx_v_jobvs, __pyx_v_sort, ((_zselect1 *)__pyx_v_select), __pyx_v_sense, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_sdim, ((npy_complex128 *)__pyx_v_w), ((npy_complex128 *)__pyx_v_vs), __pyx_v_ldvs, __pyx_v_rconde, __pyx_v_rcondv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeesx "F_FUNC(zgeesx,ZGEESX)"(char *jobvs, char *sort, _zselect1 *select, char *sense, int *n, npy_complex128 *a, int *lda, int *sdim, npy_complex128 *w, npy_complex128 *vs, int *ldvs, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgeesx(char *jobvs, char *sort, zselect1 *select, char *sense, int *n, z *a, int *lda, int *sdim, z *w, z *vs, int *ldvs, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeesx(jobvs, sort, <_zselect1*>select, sense, n, <npy_complex128*>a, lda, sdim, <npy_complex128*>w, <npy_complex128*>vs, ldvs, rconde, rcondv, <npy_complex128*>work, lwork, rwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeev "F_FUNC(zgeev,ZGEEV)"(char *jobvl, char *jobvr, int *n, npy_complex128 *a, int *lda, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeev(char *jobvl, char *jobvr, int *n, z *a, int *lda, z *w, z *vl, int *ldvl, z *vr, int *ldvr, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeev(jobvl, jobvr, n, <npy_complex128*>a, lda, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeev(char *__pyx_v_jobvl, char *__pyx_v_jobvr, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_w, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6652
 *     void _fortran_zgeev "F_FUNC(zgeev,ZGEEV)"(char *jobvl, char *jobvr, int *n, npy_complex128 *a, int *lda, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeev(char *jobvl, char *jobvr, int *n, z *a, int *lda, z *w, z *vl, int *ldvl, z *vr, int *ldvr, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zgeev(jobvl, jobvr, n, <npy_complex128*>a, lda, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeev,ZGEEV)(__pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_w), ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeev "F_FUNC(zgeev,ZGEEV)"(char *jobvl, char *jobvr, int *n, npy_complex128 *a, int *lda, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeev(char *jobvl, char *jobvr, int *n, z *a, int *lda, z *w, z *vl, int *ldvl, z *vr, int *ldvr, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeev(jobvl, jobvr, n, <npy_complex128*>a, lda, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeevx "F_FUNC(zgeevx,ZGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, z *a, int *lda, z *w, z *vl, int *ldvl, z *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeevx(balanc, jobvl, jobvr, sense, n, <npy_complex128*>a, lda, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeevx(char *__pyx_v_balanc, char *__pyx_v_jobvl, char *__pyx_v_jobvr, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_w, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abnrm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcondv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6657
 *     void _fortran_zgeevx "F_FUNC(zgeevx,ZGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, z *a, int *lda, z *w, z *vl, int *ldvl, z *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zgeevx(balanc, jobvl, jobvr, sense, n, <npy_complex128*>a, lda, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeevx,ZGEEVX)(__pyx_v_balanc, __pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_sense, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_w), ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_scale, __pyx_v_abnrm, __pyx_v_rconde, __pyx_v_rcondv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeevx "F_FUNC(zgeevx,ZGEEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, z *a, int *lda, z *w, z *vl, int *ldvl, z *vr, int *ldvr, int *ilo, int *ihi, d *scale, d *abnrm, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeevx(balanc, jobvl, jobvr, sense, n, <npy_complex128*>a, lda, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, ilo, ihi, scale, abnrm, rconde, rcondv, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgehd2 "F_FUNC(zgehd2,ZGEHD2)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgehd2(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgehd2(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgehd2(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6662
 *     void _fortran_zgehd2 "F_FUNC(zgehd2,ZGEHD2)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgehd2(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zgehd2(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgehd2,ZGEHD2)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgehd2 "F_FUNC(zgehd2,ZGEHD2)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgehd2(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgehd2(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgehrd "F_FUNC(zgehrd,ZGEHRD)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgehrd(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgehrd(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgehrd(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6667
 *     void _fortran_zgehrd "F_FUNC(zgehrd,ZGEHRD)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgehrd(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgehrd(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgehrd,ZGEHRD)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgehrd "F_FUNC(zgehrd,ZGEHRD)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgehrd(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgehrd(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelq2 "F_FUNC(zgelq2,ZGELQ2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgelq2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelq2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgelq2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6672
 *     void _fortran_zgelq2 "F_FUNC(zgelq2,ZGELQ2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgelq2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zgelq2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgelq2,ZGELQ2)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelq2 "F_FUNC(zgelq2,ZGELQ2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgelq2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelq2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelqf "F_FUNC(zgelqf,ZGELQF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgelqf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelqf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgelqf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6677
 *     void _fortran_zgelqf "F_FUNC(zgelqf,ZGELQF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgelqf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgelqf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgelqf,ZGELQF)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelqf "F_FUNC(zgelqf,ZGELQF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgelqf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelqf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgels "F_FUNC(zgels,ZGELS)"(char *trans, int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgels(char *trans, int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgels(trans, m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgels(char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6682
 *     void _fortran_zgels "F_FUNC(zgels,ZGELS)"(char *trans, int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgels(char *trans, int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgels(trans, m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgels,ZGELS)(__pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgels "F_FUNC(zgels,ZGELS)"(char *trans, int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgels(char *trans, int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgels(trans, m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelsd "F_FUNC(zgelsd,ZGELSD)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *s, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *info) nogil
 * cdef void zgelsd(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, d *s, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelsd(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, s, rcond, rank, <npy_complex128*>work, lwork, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgelsd(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6687
 *     void _fortran_zgelsd "F_FUNC(zgelsd,ZGELSD)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *s, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *info) nogil
 * cdef void zgelsd(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, d *s, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *iwork, int *info) nogil:
 *     _fortran_zgelsd(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, s, rcond, rank, <npy_complex128*>work, lwork, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgelsd,ZGELSD)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_s, __pyx_v_rcond, __pyx_v_rank, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelsd "F_FUNC(zgelsd,ZGELSD)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *s, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *info) nogil
 * cdef void zgelsd(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, d *s, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelsd(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, s, rcond, rank, <npy_complex128*>work, lwork, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelss "F_FUNC(zgelss,ZGELSS)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *s, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgelss(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, d *s, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelss(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, s, rcond, rank, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgelss(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6692
 *     void _fortran_zgelss "F_FUNC(zgelss,ZGELSS)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *s, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgelss(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, d *s, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zgelss(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, s, rcond, rank, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgelss,ZGELSS)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_s, __pyx_v_rcond, __pyx_v_rank, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelss "F_FUNC(zgelss,ZGELSS)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *s, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgelss(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, d *s, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelss(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, s, rcond, rank, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelsy "F_FUNC(zgelsy,ZGELSY)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *jpvt, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgelsy(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *jpvt, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelsy(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, jpvt, rcond, rank, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgelsy(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_jpvt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6697
 *     void _fortran_zgelsy "F_FUNC(zgelsy,ZGELSY)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *jpvt, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgelsy(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *jpvt, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zgelsy(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, jpvt, rcond, rank, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgelsy,ZGELSY)(__pyx_v_m, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_jpvt, __pyx_v_rcond, __pyx_v_rank, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgelsy "F_FUNC(zgelsy,ZGELSY)"(int *m, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *jpvt, d *rcond, int *rank, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgelsy(int *m, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *jpvt, d *rcond, int *rank, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgelsy(m, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, jpvt, rcond, rank, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeql2 "F_FUNC(zgeql2,ZGEQL2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgeql2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeql2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeql2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6702
 *     void _fortran_zgeql2 "F_FUNC(zgeql2,ZGEQL2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgeql2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zgeql2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeql2,ZGEQL2)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeql2 "F_FUNC(zgeql2,ZGEQL2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgeql2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeql2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeqlf "F_FUNC(zgeqlf,ZGEQLF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgeqlf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeqlf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeqlf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6707
 *     void _fortran_zgeqlf "F_FUNC(zgeqlf,ZGEQLF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgeqlf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgeqlf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeqlf,ZGEQLF)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeqlf "F_FUNC(zgeqlf,ZGEQLF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgeqlf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeqlf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeqp3 "F_FUNC(zgeqp3,ZGEQP3)"(int *m, int *n, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeqp3(int *m, int *n, z *a, int *lda, int *jpvt, z *tau, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeqp3(m, n, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeqp3(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6712
 *     void _fortran_zgeqp3 "F_FUNC(zgeqp3,ZGEQP3)"(int *m, int *n, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeqp3(int *m, int *n, z *a, int *lda, int *jpvt, z *tau, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zgeqp3(m, n, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeqp3,ZGEQP3)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_jpvt, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeqp3 "F_FUNC(zgeqp3,ZGEQP3)"(int *m, int *n, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgeqp3(int *m, int *n, z *a, int *lda, int *jpvt, z *tau, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeqp3(m, n, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeqr2 "F_FUNC(zgeqr2,ZGEQR2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgeqr2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeqr2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeqr2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6717
 *     void _fortran_zgeqr2 "F_FUNC(zgeqr2,ZGEQR2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgeqr2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zgeqr2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeqr2,ZGEQR2)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeqr2 "F_FUNC(zgeqr2,ZGEQR2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgeqr2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeqr2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeqrf "F_FUNC(zgeqrf,ZGEQRF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgeqrf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeqrf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgeqrf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6722
 *     void _fortran_zgeqrf "F_FUNC(zgeqrf,ZGEQRF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgeqrf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgeqrf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgeqrf,ZGEQRF)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgeqrf "F_FUNC(zgeqrf,ZGEQRF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgeqrf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgeqrf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgerfs "F_FUNC(zgerfs,ZGERFS)"(char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgerfs(char *trans, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgerfs(trans, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgerfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6727
 *     void _fortran_zgerfs "F_FUNC(zgerfs,ZGERFS)"(char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgerfs(char *trans, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgerfs(trans, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgerfs,ZGERFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgerfs "F_FUNC(zgerfs,ZGERFS)"(char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgerfs(char *trans, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgerfs(trans, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgerq2 "F_FUNC(zgerq2,ZGERQ2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgerq2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgerq2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgerq2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6732
 *     void _fortran_zgerq2 "F_FUNC(zgerq2,ZGERQ2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgerq2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zgerq2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgerq2,ZGERQ2)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgerq2 "F_FUNC(zgerq2,ZGERQ2)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zgerq2(int *m, int *n, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgerq2(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgerqf "F_FUNC(zgerqf,ZGERQF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgerqf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgerqf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgerqf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6737
 *     void _fortran_zgerqf "F_FUNC(zgerqf,ZGERQF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgerqf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgerqf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgerqf,ZGERQF)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgerqf "F_FUNC(zgerqf,ZGERQF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgerqf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgerqf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesc2 "F_FUNC(zgesc2,ZGESC2)"(int *n, npy_complex128 *a, int *lda, npy_complex128 *rhs, int *ipiv, int *jpiv, d *scale) nogil
 * cdef void zgesc2(int *n, z *a, int *lda, z *rhs, int *ipiv, int *jpiv, d *scale) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesc2(n, <npy_complex128*>a, lda, <npy_complex128*>rhs, ipiv, jpiv, scale)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgesc2(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_rhs, int *__pyx_v_ipiv, int *__pyx_v_jpiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale) {

  /* "scipy/linalg/cython_lapack.pyx":6742
 *     void _fortran_zgesc2 "F_FUNC(zgesc2,ZGESC2)"(int *n, npy_complex128 *a, int *lda, npy_complex128 *rhs, int *ipiv, int *jpiv, d *scale) nogil
 * cdef void zgesc2(int *n, z *a, int *lda, z *rhs, int *ipiv, int *jpiv, d *scale) nogil:
 *     _fortran_zgesc2(n, <npy_complex128*>a, lda, <npy_complex128*>rhs, ipiv, jpiv, scale)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgesc2,ZGESC2)(__pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_rhs), __pyx_v_ipiv, __pyx_v_jpiv, __pyx_v_scale);

  /* "scipy/linalg/cython_lapack.pyx":6741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesc2 "F_FUNC(zgesc2,ZGESC2)"(int *n, npy_complex128 *a, int *lda, npy_complex128 *rhs, int *ipiv, int *jpiv, d *scale) nogil
 * cdef void zgesc2(int *n, z *a, int *lda, z *rhs, int *ipiv, int *jpiv, d *scale) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesc2(n, <npy_complex128*>a, lda, <npy_complex128*>rhs, ipiv, jpiv, scale)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesdd "F_FUNC(zgesdd,ZGESDD)"(char *jobz, int *m, int *n, npy_complex128 *a, int *lda, d *s, npy_complex128 *u, int *ldu, npy_complex128 *vt, int *ldvt, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *info) nogil
 * cdef void zgesdd(char *jobz, int *m, int *n, z *a, int *lda, d *s, z *u, int *ldu, z *vt, int *ldvt, z *work, int *lwork, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesdd(jobz, m, n, <npy_complex128*>a, lda, s, <npy_complex128*>u, ldu, <npy_complex128*>vt, ldvt, <npy_complex128*>work, lwork, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgesdd(char *__pyx_v_jobz, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_double_complex *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6747
 *     void _fortran_zgesdd "F_FUNC(zgesdd,ZGESDD)"(char *jobz, int *m, int *n, npy_complex128 *a, int *lda, d *s, npy_complex128 *u, int *ldu, npy_complex128 *vt, int *ldvt, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *info) nogil
 * cdef void zgesdd(char *jobz, int *m, int *n, z *a, int *lda, d *s, z *u, int *ldu, z *vt, int *ldvt, z *work, int *lwork, d *rwork, int *iwork, int *info) nogil:
 *     _fortran_zgesdd(jobz, m, n, <npy_complex128*>a, lda, s, <npy_complex128*>u, ldu, <npy_complex128*>vt, ldvt, <npy_complex128*>work, lwork, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgesdd,ZGESDD)(__pyx_v_jobz, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, ((npy_complex128 *)__pyx_v_u), __pyx_v_ldu, ((npy_complex128 *)__pyx_v_vt), __pyx_v_ldvt, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesdd "F_FUNC(zgesdd,ZGESDD)"(char *jobz, int *m, int *n, npy_complex128 *a, int *lda, d *s, npy_complex128 *u, int *ldu, npy_complex128 *vt, int *ldvt, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *info) nogil
 * cdef void zgesdd(char *jobz, int *m, int *n, z *a, int *lda, d *s, z *u, int *ldu, z *vt, int *ldvt, z *work, int *lwork, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesdd(jobz, m, n, <npy_complex128*>a, lda, s, <npy_complex128*>u, ldu, <npy_complex128*>vt, ldvt, <npy_complex128*>work, lwork, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesv "F_FUNC(zgesv,ZGESV)"(int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgesv(int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesv(n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgesv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6752
 *     void _fortran_zgesv "F_FUNC(zgesv,ZGESV)"(int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgesv(int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zgesv(n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgesv,ZGESV)(__pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesv "F_FUNC(zgesv,ZGESV)"(int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgesv(int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesv(n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesvd "F_FUNC(zgesvd,ZGESVD)"(char *jobu, char *jobvt, int *m, int *n, npy_complex128 *a, int *lda, d *s, npy_complex128 *u, int *ldu, npy_complex128 *vt, int *ldvt, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgesvd(char *jobu, char *jobvt, int *m, int *n, z *a, int *lda, d *s, z *u, int *ldu, z *vt, int *ldvt, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesvd(jobu, jobvt, m, n, <npy_complex128*>a, lda, s, <npy_complex128*>u, ldu, <npy_complex128*>vt, ldvt, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgesvd(char *__pyx_v_jobu, char *__pyx_v_jobvt, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_double_complex *__pyx_v_vt, int *__pyx_v_ldvt, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6757
 *     void _fortran_zgesvd "F_FUNC(zgesvd,ZGESVD)"(char *jobu, char *jobvt, int *m, int *n, npy_complex128 *a, int *lda, d *s, npy_complex128 *u, int *ldu, npy_complex128 *vt, int *ldvt, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgesvd(char *jobu, char *jobvt, int *m, int *n, z *a, int *lda, d *s, z *u, int *ldu, z *vt, int *ldvt, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zgesvd(jobu, jobvt, m, n, <npy_complex128*>a, lda, s, <npy_complex128*>u, ldu, <npy_complex128*>vt, ldvt, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgesvd,ZGESVD)(__pyx_v_jobu, __pyx_v_jobvt, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, ((npy_complex128 *)__pyx_v_u), __pyx_v_ldu, ((npy_complex128 *)__pyx_v_vt), __pyx_v_ldvt, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesvd "F_FUNC(zgesvd,ZGESVD)"(char *jobu, char *jobvt, int *m, int *n, npy_complex128 *a, int *lda, d *s, npy_complex128 *u, int *ldu, npy_complex128 *vt, int *ldvt, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zgesvd(char *jobu, char *jobvt, int *m, int *n, z *a, int *lda, d *s, z *u, int *ldu, z *vt, int *ldvt, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesvd(jobu, jobvt, m, n, <npy_complex128*>a, lda, s, <npy_complex128*>u, ldu, <npy_complex128*>vt, ldvt, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesvx "F_FUNC(zgesvx,ZGESVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgesvx(char *fact, char *trans, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesvx(fact, trans, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, equed, r, c, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgesvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6762
 *     void _fortran_zgesvx "F_FUNC(zgesvx,ZGESVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgesvx(char *fact, char *trans, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgesvx(fact, trans, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, equed, r, c, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgesvx,ZGESVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, __pyx_v_equed, __pyx_v_r, __pyx_v_c, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgesvx "F_FUNC(zgesvx,ZGESVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgesvx(char *fact, char *trans, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, char *equed, d *r, d *c, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgesvx(fact, trans, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, equed, r, c, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetc2 "F_FUNC(zgetc2,ZGETC2)"(int *n, npy_complex128 *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void zgetc2(int *n, z *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetc2(n, <npy_complex128*>a, lda, ipiv, jpiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgetc2(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_jpiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6767
 *     void _fortran_zgetc2 "F_FUNC(zgetc2,ZGETC2)"(int *n, npy_complex128 *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void zgetc2(int *n, z *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:
 *     _fortran_zgetc2(n, <npy_complex128*>a, lda, ipiv, jpiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgetc2,ZGETC2)(__pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_jpiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetc2 "F_FUNC(zgetc2,ZGETC2)"(int *n, npy_complex128 *a, int *lda, int *ipiv, int *jpiv, int *info) nogil
 * cdef void zgetc2(int *n, z *a, int *lda, int *ipiv, int *jpiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetc2(n, <npy_complex128*>a, lda, ipiv, jpiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetf2 "F_FUNC(zgetf2,ZGETF2)"(int *m, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zgetf2(int *m, int *n, z *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetf2(m, n, <npy_complex128*>a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgetf2(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6772
 *     void _fortran_zgetf2 "F_FUNC(zgetf2,ZGETF2)"(int *m, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zgetf2(int *m, int *n, z *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_zgetf2(m, n, <npy_complex128*>a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgetf2,ZGETF2)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetf2 "F_FUNC(zgetf2,ZGETF2)"(int *m, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zgetf2(int *m, int *n, z *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetf2(m, n, <npy_complex128*>a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetrf "F_FUNC(zgetrf,ZGETRF)"(int *m, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zgetrf(int *m, int *n, z *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetrf(m, n, <npy_complex128*>a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgetrf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6777
 *     void _fortran_zgetrf "F_FUNC(zgetrf,ZGETRF)"(int *m, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zgetrf(int *m, int *n, z *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_zgetrf(m, n, <npy_complex128*>a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgetrf,ZGETRF)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetrf "F_FUNC(zgetrf,ZGETRF)"(int *m, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zgetrf(int *m, int *n, z *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetrf(m, n, <npy_complex128*>a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetri "F_FUNC(zgetri,ZGETRI)"(int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgetri(int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetri(n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgetri(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6782
 *     void _fortran_zgetri "F_FUNC(zgetri,ZGETRI)"(int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgetri(int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgetri(n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgetri,ZGETRI)(__pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetri "F_FUNC(zgetri,ZGETRI)"(int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgetri(int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetri(n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetrs "F_FUNC(zgetrs,ZGETRS)"(char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgetrs(char *trans, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetrs(trans, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgetrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6787
 *     void _fortran_zgetrs "F_FUNC(zgetrs,ZGETRS)"(char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgetrs(char *trans, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zgetrs(trans, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgetrs,ZGETRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgetrs "F_FUNC(zgetrs,ZGETRS)"(char *trans, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgetrs(char *trans, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgetrs(trans, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggbak "F_FUNC(zggbak,ZGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, npy_complex128 *v, int *ldv, int *info) nogil
 * cdef void zggbak(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, z *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggbak(job, side, n, ilo, ihi, lscale, rscale, m, <npy_complex128*>v, ldv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zggbak(char *__pyx_v_job, char *__pyx_v_side, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rscale, int *__pyx_v_m, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6792
 *     void _fortran_zggbak "F_FUNC(zggbak,ZGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, npy_complex128 *v, int *ldv, int *info) nogil
 * cdef void zggbak(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, z *v, int *ldv, int *info) nogil:
 *     _fortran_zggbak(job, side, n, ilo, ihi, lscale, rscale, m, <npy_complex128*>v, ldv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zggbak,ZGGBAK)(__pyx_v_job, __pyx_v_side, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_m, ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggbak "F_FUNC(zggbak,ZGGBAK)"(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, npy_complex128 *v, int *ldv, int *info) nogil
 * cdef void zggbak(char *job, char *side, int *n, int *ilo, int *ihi, d *lscale, d *rscale, int *m, z *v, int *ldv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggbak(job, side, n, ilo, ihi, lscale, rscale, m, <npy_complex128*>v, ldv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggbal "F_FUNC(zggbal,ZGGBAL)"(char *job, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil
 * cdef void zggbal(char *job, int *n, z *a, int *lda, z *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggbal(job, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, ilo, ihi, lscale, rscale, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zggbal(char *__pyx_v_job, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6797
 *     void _fortran_zggbal "F_FUNC(zggbal,ZGGBAL)"(char *job, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil
 * cdef void zggbal(char *job, int *n, z *a, int *lda, z *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil:
 *     _fortran_zggbal(job, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, ilo, ihi, lscale, rscale, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zggbal,ZGGBAL)(__pyx_v_job, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggbal "F_FUNC(zggbal,ZGGBAL)"(char *job, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil
 * cdef void zggbal(char *job, int *n, z *a, int *lda, z *b, int *ldb, int *ilo, int *ihi, d *lscale, d *rscale, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggbal(job, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, ilo, ihi, lscale, rscale, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgges "F_FUNC(zgges,ZGGES)"(char *jobvsl, char *jobvsr, char *sort, _zselect2 *selctg, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *sdim, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vsl, int *ldvsl, npy_complex128 *vsr, int *ldvsr, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgges(char *jobvsl, char *jobvsr, char *sort, zselect2 *selctg, int *n, z *a, int *lda, z *b, int *ldb, int *sdim, z *alpha, z *beta, z *vsl, int *ldvsl, z *vsr, int *ldvsr, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgges(jobvsl, jobvsr, sort, <_zselect2*>selctg, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, sdim, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vsl, ldvsl, <npy_complex128*>vsr, ldvsr, <npy_complex128*>work, lwork, rwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgges(char *__pyx_v_jobvsl, char *__pyx_v_jobvsr, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_zselect2 *__pyx_v_selctg, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_sdim, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_vsl, int *__pyx_v_ldvsl, __pyx_t_double_complex *__pyx_v_vsr, int *__pyx_v_ldvsr, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6802
 *     void _fortran_zgges "F_FUNC(zgges,ZGGES)"(char *jobvsl, char *jobvsr, char *sort, _zselect2 *selctg, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *sdim, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vsl, int *ldvsl, npy_complex128 *vsr, int *ldvsr, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgges(char *jobvsl, char *jobvsr, char *sort, zselect2 *selctg, int *n, z *a, int *lda, z *b, int *ldb, int *sdim, z *alpha, z *beta, z *vsl, int *ldvsl, z *vsr, int *ldvsr, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:
 *     _fortran_zgges(jobvsl, jobvsr, sort, <_zselect2*>selctg, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, sdim, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vsl, ldvsl, <npy_complex128*>vsr, ldvsr, <npy_complex128*>work, lwork, rwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgges,ZGGES)(__pyx_v_jobvsl, __pyx_v_jobvsr, __pyx_v_sort, ((_zselect2 *)__pyx_v_selctg), __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_sdim, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_vsl), __pyx_v_ldvsl, ((npy_complex128 *)__pyx_v_vsr), __pyx_v_ldvsr, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgges "F_FUNC(zgges,ZGGES)"(char *jobvsl, char *jobvsr, char *sort, _zselect2 *selctg, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *sdim, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vsl, int *ldvsl, npy_complex128 *vsr, int *ldvsr, npy_complex128 *work, int *lwork, d *rwork, bint *bwork, int *info) nogil
 * cdef void zgges(char *jobvsl, char *jobvsr, char *sort, zselect2 *selctg, int *n, z *a, int *lda, z *b, int *ldb, int *sdim, z *alpha, z *beta, z *vsl, int *ldvsl, z *vsr, int *ldvsr, z *work, int *lwork, d *rwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgges(jobvsl, jobvsr, sort, <_zselect2*>selctg, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, sdim, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vsl, ldvsl, <npy_complex128*>vsr, ldvsr, <npy_complex128*>work, lwork, rwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggesx "F_FUNC(zggesx,ZGGESX)"(char *jobvsl, char *jobvsr, char *sort, _zselect2 *selctg, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *sdim, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vsl, int *ldvsl, npy_complex128 *vsr, int *ldvsr, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void zggesx(char *jobvsl, char *jobvsr, char *sort, zselect2 *selctg, char *sense, int *n, z *a, int *lda, z *b, int *ldb, int *sdim, z *alpha, z *beta, z *vsl, int *ldvsl, z *vsr, int *ldvsr, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggesx(jobvsl, jobvsr, sort, <_zselect2*>selctg, sense, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, sdim, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vsl, ldvsl, <npy_complex128*>vsr, ldvsr, rconde, rcondv, <npy_complex128*>work, lwork, rwork, iwork, liwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zggesx(char *__pyx_v_jobvsl, char *__pyx_v_jobvsr, char *__pyx_v_sort, __pyx_t_5scipy_6linalg_13cython_lapack_zselect2 *__pyx_v_selctg, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_sdim, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_vsl, int *__pyx_v_ldvsl, __pyx_t_double_complex *__pyx_v_vsr, int *__pyx_v_ldvsr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcondv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6807
 *     void _fortran_zggesx "F_FUNC(zggesx,ZGGESX)"(char *jobvsl, char *jobvsr, char *sort, _zselect2 *selctg, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *sdim, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vsl, int *ldvsl, npy_complex128 *vsr, int *ldvsr, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void zggesx(char *jobvsl, char *jobvsr, char *sort, zselect2 *selctg, char *sense, int *n, z *a, int *lda, z *b, int *ldb, int *sdim, z *alpha, z *beta, z *vsl, int *ldvsl, z *vsr, int *ldvsr, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:
 *     _fortran_zggesx(jobvsl, jobvsr, sort, <_zselect2*>selctg, sense, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, sdim, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vsl, ldvsl, <npy_complex128*>vsr, ldvsr, rconde, rcondv, <npy_complex128*>work, lwork, rwork, iwork, liwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zggesx,ZGGESX)(__pyx_v_jobvsl, __pyx_v_jobvsr, __pyx_v_sort, ((_zselect2 *)__pyx_v_selctg), __pyx_v_sense, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_sdim, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_vsl), __pyx_v_ldvsl, ((npy_complex128 *)__pyx_v_vsr), __pyx_v_ldvsr, __pyx_v_rconde, __pyx_v_rcondv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggesx "F_FUNC(zggesx,ZGGESX)"(char *jobvsl, char *jobvsr, char *sort, _zselect2 *selctg, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *sdim, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vsl, int *ldvsl, npy_complex128 *vsr, int *ldvsr, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil
 * cdef void zggesx(char *jobvsl, char *jobvsr, char *sort, zselect2 *selctg, char *sense, int *n, z *a, int *lda, z *b, int *ldb, int *sdim, z *alpha, z *beta, z *vsl, int *ldvsl, z *vsr, int *ldvsr, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *iwork, int *liwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggesx(jobvsl, jobvsr, sort, <_zselect2*>selctg, sense, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, sdim, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vsl, ldvsl, <npy_complex128*>vsr, ldvsr, rconde, rcondv, <npy_complex128*>work, lwork, rwork, iwork, liwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggev "F_FUNC(zggev,ZGGEV)"(char *jobvl, char *jobvr, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zggev(char *jobvl, char *jobvr, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *vl, int *ldvl, z *vr, int *ldvr, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggev(jobvl, jobvr, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zggev(char *__pyx_v_jobvl, char *__pyx_v_jobvr, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6812
 *     void _fortran_zggev "F_FUNC(zggev,ZGGEV)"(char *jobvl, char *jobvr, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zggev(char *jobvl, char *jobvr, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *vl, int *ldvl, z *vr, int *ldvr, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zggev(jobvl, jobvr, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zggev,ZGGEV)(__pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggev "F_FUNC(zggev,ZGGEV)"(char *jobvl, char *jobvr, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zggev(char *jobvl, char *jobvr, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *vl, int *ldvl, z *vr, int *ldvr, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggev(jobvl, jobvr, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggevx "F_FUNC(zggevx,ZGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void zggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *vl, int *ldvl, z *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *iwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggevx(balanc, jobvl, jobvr, sense, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, <npy_complex128*>work, lwork, rwork, iwork, bwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zggevx(char *__pyx_v_balanc, char *__pyx_v_jobvl, char *__pyx_v_jobvr, char *__pyx_v_sense, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rscale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abnrm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_bbnrm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rconde, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcondv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_bwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6817
 *     void _fortran_zggevx "F_FUNC(zggevx,ZGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void zggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *vl, int *ldvl, z *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *iwork, bint *bwork, int *info) nogil:
 *     _fortran_zggevx(balanc, jobvl, jobvr, sense, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, <npy_complex128*>work, lwork, rwork, iwork, bwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zggevx,ZGGEVX)(__pyx_v_balanc, __pyx_v_jobvl, __pyx_v_jobvr, __pyx_v_sense, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_ilo, __pyx_v_ihi, __pyx_v_lscale, __pyx_v_rscale, __pyx_v_abnrm, __pyx_v_bbnrm, __pyx_v_rconde, __pyx_v_rcondv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_bwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggevx "F_FUNC(zggevx,ZGGEVX)"(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, npy_complex128 *work, int *lwork, d *rwork, int *iwork, bint *bwork, int *info) nogil
 * cdef void zggevx(char *balanc, char *jobvl, char *jobvr, char *sense, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *vl, int *ldvl, z *vr, int *ldvr, int *ilo, int *ihi, d *lscale, d *rscale, d *abnrm, d *bbnrm, d *rconde, d *rcondv, z *work, int *lwork, d *rwork, int *iwork, bint *bwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggevx(balanc, jobvl, jobvr, sense, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, <npy_complex128*>work, lwork, rwork, iwork, bwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggglm "F_FUNC(zggglm,ZGGGLM)"(int *n, int *m, int *p, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *d, npy_complex128 *x, npy_complex128 *y, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggglm(int *n, int *m, int *p, z *a, int *lda, z *b, int *ldb, z *d, z *x, z *y, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggglm(n, m, p, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>d, <npy_complex128*>x, <npy_complex128*>y, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zggglm(int *__pyx_v_n, int *__pyx_v_m, int *__pyx_v_p, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_x, __pyx_t_double_complex *__pyx_v_y, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6822
 *     void _fortran_zggglm "F_FUNC(zggglm,ZGGGLM)"(int *n, int *m, int *p, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *d, npy_complex128 *x, npy_complex128 *y, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggglm(int *n, int *m, int *p, z *a, int *lda, z *b, int *ldb, z *d, z *x, z *y, z *work, int *lwork, int *info) nogil:
 *     _fortran_zggglm(n, m, p, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>d, <npy_complex128*>x, <npy_complex128*>y, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zggglm,ZGGGLM)(__pyx_v_n, __pyx_v_m, __pyx_v_p, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_x), ((npy_complex128 *)__pyx_v_y), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggglm "F_FUNC(zggglm,ZGGGLM)"(int *n, int *m, int *p, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *d, npy_complex128 *x, npy_complex128 *y, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggglm(int *n, int *m, int *p, z *a, int *lda, z *b, int *ldb, z *d, z *x, z *y, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggglm(n, m, p, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>d, <npy_complex128*>x, <npy_complex128*>y, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgghrd "F_FUNC(zgghrd,ZGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *info) nogil
 * cdef void zgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgghrd(compq, compz, n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgghrd(char *__pyx_v_compq, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6827
 *     void _fortran_zgghrd "F_FUNC(zgghrd,ZGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *info) nogil
 * cdef void zgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *info) nogil:
 *     _fortran_zgghrd(compq, compz, n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgghrd,ZGGHRD)(__pyx_v_compq, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgghrd "F_FUNC(zgghrd,ZGGHRD)"(char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *info) nogil
 * cdef void zgghrd(char *compq, char *compz, int *n, int *ilo, int *ihi, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgghrd(compq, compz, n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgglse "F_FUNC(zgglse,ZGGLSE)"(int *m, int *n, int *p, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, npy_complex128 *d, npy_complex128 *x, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgglse(int *m, int *n, int *p, z *a, int *lda, z *b, int *ldb, z *c, z *d, z *x, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgglse(m, n, p, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, <npy_complex128*>d, <npy_complex128*>x, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgglse(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_p, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_c, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_x, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6832
 *     void _fortran_zgglse "F_FUNC(zgglse,ZGGLSE)"(int *m, int *n, int *p, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, npy_complex128 *d, npy_complex128 *x, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgglse(int *m, int *n, int *p, z *a, int *lda, z *b, int *ldb, z *c, z *d, z *x, z *work, int *lwork, int *info) nogil:
 *     _fortran_zgglse(m, n, p, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, <npy_complex128*>d, <npy_complex128*>x, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgglse,ZGGLSE)(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_c), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_x), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgglse "F_FUNC(zgglse,ZGGLSE)"(int *m, int *n, int *p, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, npy_complex128 *d, npy_complex128 *x, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zgglse(int *m, int *n, int *p, z *a, int *lda, z *b, int *ldb, z *c, z *d, z *x, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgglse(m, n, p, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, <npy_complex128*>d, <npy_complex128*>x, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggqrf "F_FUNC(zggqrf,ZGGQRF)"(int *n, int *m, int *p, npy_complex128 *a, int *lda, npy_complex128 *taua, npy_complex128 *b, int *ldb, npy_complex128 *taub, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggqrf(int *n, int *m, int *p, z *a, int *lda, z *taua, z *b, int *ldb, z *taub, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggqrf(n, m, p, <npy_complex128*>a, lda, <npy_complex128*>taua, <npy_complex128*>b, ldb, <npy_complex128*>taub, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zggqrf(int *__pyx_v_n, int *__pyx_v_m, int *__pyx_v_p, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_taua, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_taub, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6837
 *     void _fortran_zggqrf "F_FUNC(zggqrf,ZGGQRF)"(int *n, int *m, int *p, npy_complex128 *a, int *lda, npy_complex128 *taua, npy_complex128 *b, int *ldb, npy_complex128 *taub, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggqrf(int *n, int *m, int *p, z *a, int *lda, z *taua, z *b, int *ldb, z *taub, z *work, int *lwork, int *info) nogil:
 *     _fortran_zggqrf(n, m, p, <npy_complex128*>a, lda, <npy_complex128*>taua, <npy_complex128*>b, ldb, <npy_complex128*>taub, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zggqrf,ZGGQRF)(__pyx_v_n, __pyx_v_m, __pyx_v_p, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_taua), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_taub), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggqrf "F_FUNC(zggqrf,ZGGQRF)"(int *n, int *m, int *p, npy_complex128 *a, int *lda, npy_complex128 *taua, npy_complex128 *b, int *ldb, npy_complex128 *taub, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggqrf(int *n, int *m, int *p, z *a, int *lda, z *taua, z *b, int *ldb, z *taub, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggqrf(n, m, p, <npy_complex128*>a, lda, <npy_complex128*>taua, <npy_complex128*>b, ldb, <npy_complex128*>taub, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggrqf "F_FUNC(zggrqf,ZGGRQF)"(int *m, int *p, int *n, npy_complex128 *a, int *lda, npy_complex128 *taua, npy_complex128 *b, int *ldb, npy_complex128 *taub, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggrqf(int *m, int *p, int *n, z *a, int *lda, z *taua, z *b, int *ldb, z *taub, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggrqf(m, p, n, <npy_complex128*>a, lda, <npy_complex128*>taua, <npy_complex128*>b, ldb, <npy_complex128*>taub, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zggrqf(int *__pyx_v_m, int *__pyx_v_p, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_taua, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_taub, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6842
 *     void _fortran_zggrqf "F_FUNC(zggrqf,ZGGRQF)"(int *m, int *p, int *n, npy_complex128 *a, int *lda, npy_complex128 *taua, npy_complex128 *b, int *ldb, npy_complex128 *taub, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggrqf(int *m, int *p, int *n, z *a, int *lda, z *taua, z *b, int *ldb, z *taub, z *work, int *lwork, int *info) nogil:
 *     _fortran_zggrqf(m, p, n, <npy_complex128*>a, lda, <npy_complex128*>taua, <npy_complex128*>b, ldb, <npy_complex128*>taub, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zggrqf,ZGGRQF)(__pyx_v_m, __pyx_v_p, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_taua), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_taub), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zggrqf "F_FUNC(zggrqf,ZGGRQF)"(int *m, int *p, int *n, npy_complex128 *a, int *lda, npy_complex128 *taua, npy_complex128 *b, int *ldb, npy_complex128 *taub, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zggrqf(int *m, int *p, int *n, z *a, int *lda, z *taua, z *b, int *ldb, z *taub, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zggrqf(m, p, n, <npy_complex128*>a, lda, <npy_complex128*>taua, <npy_complex128*>b, ldb, <npy_complex128*>taub, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtcon "F_FUNC(zgtcon,ZGTCON)"(char *norm, int *n, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zgtcon(char *norm, int *n, z *dl, z *d, z *du, z *du2, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtcon(norm, n, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgtcon(char *__pyx_v_norm, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_du, __pyx_t_double_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6847
 *     void _fortran_zgtcon "F_FUNC(zgtcon,ZGTCON)"(char *norm, int *n, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zgtcon(char *norm, int *n, z *dl, z *d, z *du, z *du2, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:
 *     _fortran_zgtcon(norm, n, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, anorm, rcond, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgtcon,ZGTCON)(__pyx_v_norm, __pyx_v_n, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_du), ((npy_complex128 *)__pyx_v_du2), __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtcon "F_FUNC(zgtcon,ZGTCON)"(char *norm, int *n, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zgtcon(char *norm, int *n, z *dl, z *d, z *du, z *du2, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtcon(norm, n, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtrfs "F_FUNC(zgtrfs,ZGTRFS)"(char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *dlf, npy_complex128 *df, npy_complex128 *duf, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgtrfs(char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *dlf, z *df, z *duf, z *du2, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtrfs(trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>dlf, <npy_complex128*>df, <npy_complex128*>duf, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgtrfs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_du, __pyx_t_double_complex *__pyx_v_dlf, __pyx_t_double_complex *__pyx_v_df, __pyx_t_double_complex *__pyx_v_duf, __pyx_t_double_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6852
 *     void _fortran_zgtrfs "F_FUNC(zgtrfs,ZGTRFS)"(char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *dlf, npy_complex128 *df, npy_complex128 *duf, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgtrfs(char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *dlf, z *df, z *duf, z *du2, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgtrfs(trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>dlf, <npy_complex128*>df, <npy_complex128*>duf, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgtrfs,ZGTRFS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_du), ((npy_complex128 *)__pyx_v_dlf), ((npy_complex128 *)__pyx_v_df), ((npy_complex128 *)__pyx_v_duf), ((npy_complex128 *)__pyx_v_du2), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtrfs "F_FUNC(zgtrfs,ZGTRFS)"(char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *dlf, npy_complex128 *df, npy_complex128 *duf, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgtrfs(char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *dlf, z *df, z *duf, z *du2, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtrfs(trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>dlf, <npy_complex128*>df, <npy_complex128*>duf, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtsv "F_FUNC(zgtsv,ZGTSV)"(int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgtsv(int *n, int *nrhs, z *dl, z *d, z *du, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtsv(n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgtsv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_du, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6857
 *     void _fortran_zgtsv "F_FUNC(zgtsv,ZGTSV)"(int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgtsv(int *n, int *nrhs, z *dl, z *d, z *du, z *b, int *ldb, int *info) nogil:
 *     _fortran_zgtsv(n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgtsv,ZGTSV)(__pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_du), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtsv "F_FUNC(zgtsv,ZGTSV)"(int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgtsv(int *n, int *nrhs, z *dl, z *d, z *du, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtsv(n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtsvx "F_FUNC(zgtsvx,ZGTSVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *dlf, npy_complex128 *df, npy_complex128 *duf, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgtsvx(char *fact, char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *dlf, z *df, z *duf, z *du2, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtsvx(fact, trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>dlf, <npy_complex128*>df, <npy_complex128*>duf, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgtsvx(char *__pyx_v_fact, char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_du, __pyx_t_double_complex *__pyx_v_dlf, __pyx_t_double_complex *__pyx_v_df, __pyx_t_double_complex *__pyx_v_duf, __pyx_t_double_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6862
 *     void _fortran_zgtsvx "F_FUNC(zgtsvx,ZGTSVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *dlf, npy_complex128 *df, npy_complex128 *duf, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgtsvx(char *fact, char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *dlf, z *df, z *duf, z *du2, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zgtsvx(fact, trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>dlf, <npy_complex128*>df, <npy_complex128*>duf, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgtsvx,ZGTSVX)(__pyx_v_fact, __pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_du), ((npy_complex128 *)__pyx_v_dlf), ((npy_complex128 *)__pyx_v_df), ((npy_complex128 *)__pyx_v_duf), ((npy_complex128 *)__pyx_v_du2), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtsvx "F_FUNC(zgtsvx,ZGTSVX)"(char *fact, char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *dlf, npy_complex128 *df, npy_complex128 *duf, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zgtsvx(char *fact, char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *dlf, z *df, z *duf, z *du2, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtsvx(fact, trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>dlf, <npy_complex128*>df, <npy_complex128*>duf, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgttrf "F_FUNC(zgttrf,ZGTTRF)"(int *n, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, int *info) nogil
 * cdef void zgttrf(int *n, z *dl, z *d, z *du, z *du2, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgttrf(n, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgttrf(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_du, __pyx_t_double_complex *__pyx_v_du2, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6867
 *     void _fortran_zgttrf "F_FUNC(zgttrf,ZGTTRF)"(int *n, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, int *info) nogil
 * cdef void zgttrf(int *n, z *dl, z *d, z *du, z *du2, int *ipiv, int *info) nogil:
 *     _fortran_zgttrf(n, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgttrf,ZGTTRF)(__pyx_v_n, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_du), ((npy_complex128 *)__pyx_v_du2), __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgttrf "F_FUNC(zgttrf,ZGTTRF)"(int *n, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, int *info) nogil
 * cdef void zgttrf(int *n, z *dl, z *d, z *du, z *du2, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgttrf(n, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgttrs "F_FUNC(zgttrs,ZGTTRS)"(char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgttrs(char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *du2, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgttrs(trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgttrs(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_du, __pyx_t_double_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6872
 *     void _fortran_zgttrs "F_FUNC(zgttrs,ZGTTRS)"(char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgttrs(char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *du2, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zgttrs(trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgttrs,ZGTTRS)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_du), ((npy_complex128 *)__pyx_v_du2), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgttrs "F_FUNC(zgttrs,ZGTTRS)"(char *trans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zgttrs(char *trans, int *n, int *nrhs, z *dl, z *d, z *du, z *du2, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgttrs(trans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtts2 "F_FUNC(zgtts2,ZGTTS2)"(int *itrans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb) nogil
 * cdef void zgtts2(int *itrans, int *n, int *nrhs, z *dl, z *d, z *du, z *du2, int *ipiv, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtts2(itrans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zgtts2(int *__pyx_v_itrans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_du, __pyx_t_double_complex *__pyx_v_du2, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":6877
 *     void _fortran_zgtts2 "F_FUNC(zgtts2,ZGTTS2)"(int *itrans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb) nogil
 * cdef void zgtts2(int *itrans, int *n, int *nrhs, z *dl, z *d, z *du, z *du2, int *ipiv, z *b, int *ldb) nogil:
 *     _fortran_zgtts2(itrans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zgtts2,ZGTTS2)(__pyx_v_itrans, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_du), ((npy_complex128 *)__pyx_v_du2), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":6876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zgtts2 "F_FUNC(zgtts2,ZGTTS2)"(int *itrans, int *n, int *nrhs, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *du2, int *ipiv, npy_complex128 *b, int *ldb) nogil
 * cdef void zgtts2(int *itrans, int *n, int *nrhs, z *dl, z *d, z *du, z *du2, int *ipiv, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zgtts2(itrans, n, nrhs, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>du2, ipiv, <npy_complex128*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbev "F_FUNC(zhbev,ZHBEV)"(char *jobz, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbev(char *jobz, char *uplo, int *n, int *kd, z *ab, int *ldab, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbev(jobz, uplo, n, kd, <npy_complex128*>ab, ldab, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhbev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6882
 *     void _fortran_zhbev "F_FUNC(zhbev,ZHBEV)"(char *jobz, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbev(char *jobz, char *uplo, int *n, int *kd, z *ab, int *ldab, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:
 *     _fortran_zhbev(jobz, uplo, n, kd, <npy_complex128*>ab, ldab, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhbev,ZHBEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbev "F_FUNC(zhbev,ZHBEV)"(char *jobz, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbev(char *jobz, char *uplo, int *n, int *kd, z *ab, int *ldab, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbev(jobz, uplo, n, kd, <npy_complex128*>ab, ldab, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbevd "F_FUNC(zhbevd,ZHBEVD)"(char *jobz, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhbevd(char *jobz, char *uplo, int *n, int *kd, z *ab, int *ldab, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbevd(jobz, uplo, n, kd, <npy_complex128*>ab, ldab, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhbevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6887
 *     void _fortran_zhbevd "F_FUNC(zhbevd,ZHBEVD)"(char *jobz, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhbevd(char *jobz, char *uplo, int *n, int *kd, z *ab, int *ldab, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zhbevd(jobz, uplo, n, kd, <npy_complex128*>ab, ldab, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhbevd,ZHBEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbevd "F_FUNC(zhbevd,ZHBEVD)"(char *jobz, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhbevd(char *jobz, char *uplo, int *n, int *kd, z *ab, int *ldab, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbevd(jobz, uplo, n, kd, <npy_complex128*>ab, ldab, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbevx "F_FUNC(zhbevx,ZHBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, npy_complex128 *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhbevx(char *jobz, char *range, char *uplo, int *n, int *kd, z *ab, int *ldab, z *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbevx(jobz, range, uplo, n, kd, <npy_complex128*>ab, ldab, <npy_complex128*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhbevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6892
 *     void _fortran_zhbevx "F_FUNC(zhbevx,ZHBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, npy_complex128 *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhbevx(char *jobz, char *range, char *uplo, int *n, int *kd, z *ab, int *ldab, z *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_zhbevx(jobz, range, uplo, n, kd, <npy_complex128*>ab, ldab, <npy_complex128*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhbevx,ZHBEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbevx "F_FUNC(zhbevx,ZHBEVX)"(char *jobz, char *range, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, npy_complex128 *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhbevx(char *jobz, char *range, char *uplo, int *n, int *kd, z *ab, int *ldab, z *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbevx(jobz, range, uplo, n, kd, <npy_complex128*>ab, ldab, <npy_complex128*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbgst "F_FUNC(zhbgst,ZHBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, npy_complex128 *x, int *ldx, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbgst(char *vect, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, z *x, int *ldx, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbgst(vect, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, <npy_complex128*>x, ldx, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhbgst(char *__pyx_v_vect, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6897
 *     void _fortran_zhbgst "F_FUNC(zhbgst,ZHBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, npy_complex128 *x, int *ldx, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbgst(char *vect, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, z *x, int *ldx, z *work, d *rwork, int *info) nogil:
 *     _fortran_zhbgst(vect, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, <npy_complex128*>x, ldx, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhbgst,ZHBGST)(__pyx_v_vect, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_bb), __pyx_v_ldbb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbgst "F_FUNC(zhbgst,ZHBGST)"(char *vect, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, npy_complex128 *x, int *ldx, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbgst(char *vect, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, z *x, int *ldx, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbgst(vect, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, <npy_complex128*>x, ldx, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbgv "F_FUNC(zhbgv,ZHBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbgv(jobz, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhbgv(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6902
 *     void _fortran_zhbgv "F_FUNC(zhbgv,ZHBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:
 *     _fortran_zhbgv(jobz, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhbgv,ZHBGV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_bb), __pyx_v_ldbb, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbgv "F_FUNC(zhbgv,ZHBGV)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhbgv(char *jobz, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbgv(jobz, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbgvd "F_FUNC(zhbgvd,ZHBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbgvd(jobz, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhbgvd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6907
 *     void _fortran_zhbgvd "F_FUNC(zhbgvd,ZHBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zhbgvd(jobz, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhbgvd,ZHBGVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_bb), __pyx_v_ldbb, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbgvd "F_FUNC(zhbgvd,ZHBGVD)"(char *jobz, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhbgvd(char *jobz, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbgvd(jobz, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbgvx "F_FUNC(zhbgvx,ZHBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, npy_complex128 *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, z *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbgvx(jobz, range, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, <npy_complex128*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhbgvx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_ka, int *__pyx_v_kb, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_bb, int *__pyx_v_ldbb, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6912
 *     void _fortran_zhbgvx "F_FUNC(zhbgvx,ZHBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, npy_complex128 *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, z *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_zhbgvx(jobz, range, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, <npy_complex128*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhbgvx,ZHBGVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, __pyx_v_ka, __pyx_v_kb, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_bb), __pyx_v_ldbb, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbgvx "F_FUNC(zhbgvx,ZHBGVX)"(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, npy_complex128 *ab, int *ldab, npy_complex128 *bb, int *ldbb, npy_complex128 *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhbgvx(char *jobz, char *range, char *uplo, int *n, int *ka, int *kb, z *ab, int *ldab, z *bb, int *ldbb, z *q, int *ldq, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbgvx(jobz, range, uplo, n, ka, kb, <npy_complex128*>ab, ldab, <npy_complex128*>bb, ldbb, <npy_complex128*>q, ldq, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbtrd "F_FUNC(zhbtrd,ZHBTRD)"(char *vect, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *work, int *info) nogil
 * cdef void zhbtrd(char *vect, char *uplo, int *n, int *kd, z *ab, int *ldab, d *d, d *e, z *q, int *ldq, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbtrd(vect, uplo, n, kd, <npy_complex128*>ab, ldab, d, e, <npy_complex128*>q, ldq, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhbtrd(char *__pyx_v_vect, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6917
 *     void _fortran_zhbtrd "F_FUNC(zhbtrd,ZHBTRD)"(char *vect, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *work, int *info) nogil
 * cdef void zhbtrd(char *vect, char *uplo, int *n, int *kd, z *ab, int *ldab, d *d, d *e, z *q, int *ldq, z *work, int *info) nogil:
 *     _fortran_zhbtrd(vect, uplo, n, kd, <npy_complex128*>ab, ldab, d, e, <npy_complex128*>q, ldq, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhbtrd,ZHBTRD)(__pyx_v_vect, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhbtrd "F_FUNC(zhbtrd,ZHBTRD)"(char *vect, char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *work, int *info) nogil
 * cdef void zhbtrd(char *vect, char *uplo, int *n, int *kd, z *ab, int *ldab, d *d, d *e, z *q, int *ldq, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhbtrd(vect, uplo, n, kd, <npy_complex128*>ab, ldab, d, e, <npy_complex128*>q, ldq, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhecon "F_FUNC(zhecon,ZHECON)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zhecon(char *uplo, int *n, z *a, int *lda, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhecon(uplo, n, <npy_complex128*>a, lda, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhecon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6922
 *     void _fortran_zhecon "F_FUNC(zhecon,ZHECON)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zhecon(char *uplo, int *n, z *a, int *lda, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:
 *     _fortran_zhecon(uplo, n, <npy_complex128*>a, lda, ipiv, anorm, rcond, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhecon,ZHECON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhecon "F_FUNC(zhecon,ZHECON)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zhecon(char *uplo, int *n, z *a, int *lda, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhecon(uplo, n, <npy_complex128*>a, lda, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zheev "F_FUNC(zheev,ZHEEV)"(char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, d *w, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zheev(char *jobz, char *uplo, int *n, z *a, int *lda, d *w, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zheev(jobz, uplo, n, <npy_complex128*>a, lda, w, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zheev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6927
 *     void _fortran_zheev "F_FUNC(zheev,ZHEEV)"(char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, d *w, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zheev(char *jobz, char *uplo, int *n, z *a, int *lda, d *w, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zheev(jobz, uplo, n, <npy_complex128*>a, lda, w, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zheev,ZHEEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_w, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zheev "F_FUNC(zheev,ZHEEV)"(char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, d *w, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zheev(char *jobz, char *uplo, int *n, z *a, int *lda, d *w, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zheev(jobz, uplo, n, <npy_complex128*>a, lda, w, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zheevd "F_FUNC(zheevd,ZHEEVD)"(char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, d *w, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zheevd(char *jobz, char *uplo, int *n, z *a, int *lda, d *w, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zheevd(jobz, uplo, n, <npy_complex128*>a, lda, w, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zheevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6932
 *     void _fortran_zheevd "F_FUNC(zheevd,ZHEEVD)"(char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, d *w, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zheevd(char *jobz, char *uplo, int *n, z *a, int *lda, d *w, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zheevd(jobz, uplo, n, <npy_complex128*>a, lda, w, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zheevd,ZHEEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_w, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zheevd "F_FUNC(zheevd,ZHEEVD)"(char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, d *w, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zheevd(char *jobz, char *uplo, int *n, z *a, int *lda, d *w, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zheevd(jobz, uplo, n, <npy_complex128*>a, lda, w, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zheevr "F_FUNC(zheevr,ZHEEVR)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, int *isuppz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zheevr(char *jobz, char *range, char *uplo, int *n, z *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, int *isuppz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zheevr(jobz, range, uplo, n, <npy_complex128*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, isuppz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zheevr(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6937
 *     void _fortran_zheevr "F_FUNC(zheevr,ZHEEVR)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, int *isuppz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zheevr(char *jobz, char *range, char *uplo, int *n, z *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, int *isuppz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zheevr(jobz, range, uplo, n, <npy_complex128*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, isuppz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zheevr,ZHEEVR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_isuppz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zheevr "F_FUNC(zheevr,ZHEEVR)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, int *isuppz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zheevr(char *jobz, char *range, char *uplo, int *n, z *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, int *isuppz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zheevr(jobz, range, uplo, n, <npy_complex128*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, isuppz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zheevx "F_FUNC(zheevx,ZHEEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zheevx(char *jobz, char *range, char *uplo, int *n, z *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zheevx(jobz, range, uplo, n, <npy_complex128*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zheevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6942
 *     void _fortran_zheevx "F_FUNC(zheevx,ZHEEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zheevx(char *jobz, char *range, char *uplo, int *n, z *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_zheevx(jobz, range, uplo, n, <npy_complex128*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zheevx,ZHEEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zheevx "F_FUNC(zheevx,ZHEEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zheevx(char *jobz, char *range, char *uplo, int *n, z *a, int *lda, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zheevx(jobz, range, uplo, n, <npy_complex128*>a, lda, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegs2 "F_FUNC(zhegs2,ZHEGS2)"(int *itype, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhegs2(int *itype, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegs2(itype, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhegs2(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6947
 *     void _fortran_zhegs2 "F_FUNC(zhegs2,ZHEGS2)"(int *itype, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhegs2(int *itype, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, int *info) nogil:
 *     _fortran_zhegs2(itype, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhegs2,ZHEGS2)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegs2 "F_FUNC(zhegs2,ZHEGS2)"(int *itype, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhegs2(int *itype, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegs2(itype, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegst "F_FUNC(zhegst,ZHEGST)"(int *itype, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhegst(int *itype, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegst(itype, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhegst(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6952
 *     void _fortran_zhegst "F_FUNC(zhegst,ZHEGST)"(int *itype, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhegst(int *itype, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, int *info) nogil:
 *     _fortran_zhegst(itype, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhegst,ZHEGST)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegst "F_FUNC(zhegst,ZHEGST)"(int *itype, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhegst(int *itype, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegst(itype, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegv "F_FUNC(zhegv,ZHEGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *w, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhegv(int *itype, char *jobz, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *w, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegv(itype, jobz, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, w, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhegv(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6957
 *     void _fortran_zhegv "F_FUNC(zhegv,ZHEGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *w, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhegv(int *itype, char *jobz, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *w, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zhegv(itype, jobz, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, w, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhegv,ZHEGV)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_w, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegv "F_FUNC(zhegv,ZHEGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *w, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhegv(int *itype, char *jobz, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *w, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegv(itype, jobz, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, w, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegvd "F_FUNC(zhegvd,ZHEGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *w, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhegvd(int *itype, char *jobz, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *w, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegvd(itype, jobz, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, w, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhegvd(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6962
 *     void _fortran_zhegvd "F_FUNC(zhegvd,ZHEGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *w, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhegvd(int *itype, char *jobz, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *w, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zhegvd(itype, jobz, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, w, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhegvd,ZHEGVD)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_w, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegvd "F_FUNC(zhegvd,ZHEGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *w, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhegvd(int *itype, char *jobz, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *w, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegvd(itype, jobz, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, w, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegvx "F_FUNC(zhegvx,ZHEGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhegvx(int *itype, char *jobz, char *range, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegvx(itype, jobz, range, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhegvx(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6967
 *     void _fortran_zhegvx "F_FUNC(zhegvx,ZHEGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhegvx(int *itype, char *jobz, char *range, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_zhegvx(itype, jobz, range, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhegvx,ZHEGVX)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhegvx "F_FUNC(zhegvx,ZHEGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhegvx(int *itype, char *jobz, char *range, char *uplo, int *n, z *a, int *lda, z *b, int *ldb, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhegvx(itype, jobz, range, uplo, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zherfs "F_FUNC(zherfs,ZHERFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zherfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zherfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zherfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6972
 *     void _fortran_zherfs "F_FUNC(zherfs,ZHERFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zherfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zherfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zherfs,ZHERFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zherfs "F_FUNC(zherfs,ZHERFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zherfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zherfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhesv "F_FUNC(zhesv,ZHESV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhesv(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhesv(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhesv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6977
 *     void _fortran_zhesv "F_FUNC(zhesv,ZHESV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhesv(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, z *work, int *lwork, int *info) nogil:
 *     _fortran_zhesv(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhesv,ZHESV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhesv "F_FUNC(zhesv,ZHESV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhesv(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhesv(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhesvx "F_FUNC(zhesvx,ZHESVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhesvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhesvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhesvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6982
 *     void _fortran_zhesvx "F_FUNC(zhesvx,ZHESVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhesvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zhesvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhesvx,ZHESVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhesvx "F_FUNC(zhesvx,ZHESVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhesvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhesvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetd2 "F_FUNC(zhetd2,ZHETD2)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tau, int *info) nogil
 * cdef void zhetd2(char *uplo, int *n, z *a, int *lda, d *d, d *e, z *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetd2(uplo, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhetd2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6987
 *     void _fortran_zhetd2 "F_FUNC(zhetd2,ZHETD2)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tau, int *info) nogil
 * cdef void zhetd2(char *uplo, int *n, z *a, int *lda, d *d, d *e, z *tau, int *info) nogil:
 *     _fortran_zhetd2(uplo, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhetd2,ZHETD2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_tau), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetd2 "F_FUNC(zhetd2,ZHETD2)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tau, int *info) nogil
 * cdef void zhetd2(char *uplo, int *n, z *a, int *lda, d *d, d *e, z *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetd2(uplo, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetf2 "F_FUNC(zhetf2,ZHETF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zhetf2(char *uplo, int *n, z *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetf2(uplo, n, <npy_complex128*>a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhetf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6992
 *     void _fortran_zhetf2 "F_FUNC(zhetf2,ZHETF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zhetf2(char *uplo, int *n, z *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_zhetf2(uplo, n, <npy_complex128*>a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhetf2,ZHETF2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetf2 "F_FUNC(zhetf2,ZHETF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zhetf2(char *uplo, int *n, z *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetf2(uplo, n, <npy_complex128*>a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":6996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetrd "F_FUNC(zhetrd,ZHETRD)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhetrd(char *uplo, int *n, z *a, int *lda, d *d, d *e, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetrd(uplo, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhetrd(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":6997
 *     void _fortran_zhetrd "F_FUNC(zhetrd,ZHETRD)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhetrd(char *uplo, int *n, z *a, int *lda, d *d, d *e, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zhetrd(uplo, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhetrd,ZHETRD)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":6996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetrd "F_FUNC(zhetrd,ZHETRD)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhetrd(char *uplo, int *n, z *a, int *lda, d *d, d *e, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetrd(uplo, n, <npy_complex128*>a, lda, d, e, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetrf "F_FUNC(zhetrf,ZHETRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhetrf(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetrf(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhetrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7002
 *     void _fortran_zhetrf "F_FUNC(zhetrf,ZHETRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhetrf(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:
 *     _fortran_zhetrf(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhetrf,ZHETRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetrf "F_FUNC(zhetrf,ZHETRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhetrf(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetrf(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetri "F_FUNC(zhetri,ZHETRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zhetri(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetri(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhetri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7007
 *     void _fortran_zhetri "F_FUNC(zhetri,ZHETRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zhetri(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *info) nogil:
 *     _fortran_zhetri(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhetri,ZHETRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetri "F_FUNC(zhetri,ZHETRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zhetri(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetri(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetrs "F_FUNC(zhetrs,ZHETRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhetrs(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetrs(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhetrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7012
 *     void _fortran_zhetrs "F_FUNC(zhetrs,ZHETRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhetrs(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zhetrs(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhetrs,ZHETRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhetrs "F_FUNC(zhetrs,ZHETRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhetrs(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhetrs(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhgeqz "F_FUNC(zhgeqz,ZHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *t, int *ldt, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *t, int *ldt, z *alpha, z *beta, z *q, int *ldq, z *z, int *ldz, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhgeqz(job, compq, compz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>t, ldt, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhgeqz(char *__pyx_v_job, char *__pyx_v_compq, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7017
 *     void _fortran_zhgeqz "F_FUNC(zhgeqz,ZHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *t, int *ldt, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *t, int *ldt, z *alpha, z *beta, z *q, int *ldq, z *z, int *ldz, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zhgeqz(job, compq, compz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>t, ldt, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhgeqz,ZHGEQZ)(__pyx_v_job, __pyx_v_compq, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhgeqz "F_FUNC(zhgeqz,ZHGEQZ)"(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *t, int *ldt, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zhgeqz(char *job, char *compq, char *compz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *t, int *ldt, z *alpha, z *beta, z *q, int *ldq, z *z, int *ldz, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhgeqz(job, compq, compz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>t, ldt, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpcon "F_FUNC(zhpcon,ZHPCON)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zhpcon(char *uplo, int *n, z *ap, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpcon(uplo, n, <npy_complex128*>ap, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7022
 *     void _fortran_zhpcon "F_FUNC(zhpcon,ZHPCON)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zhpcon(char *uplo, int *n, z *ap, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:
 *     _fortran_zhpcon(uplo, n, <npy_complex128*>ap, ipiv, anorm, rcond, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpcon,ZHPCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7021
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpcon "F_FUNC(zhpcon,ZHPCON)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zhpcon(char *uplo, int *n, z *ap, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpcon(uplo, n, <npy_complex128*>ap, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpev "F_FUNC(zhpev,ZHPEV)"(char *jobz, char *uplo, int *n, npy_complex128 *ap, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpev(char *jobz, char *uplo, int *n, z *ap, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpev(jobz, uplo, n, <npy_complex128*>ap, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpev(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7027
 *     void _fortran_zhpev "F_FUNC(zhpev,ZHPEV)"(char *jobz, char *uplo, int *n, npy_complex128 *ap, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpev(char *jobz, char *uplo, int *n, z *ap, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:
 *     _fortran_zhpev(jobz, uplo, n, <npy_complex128*>ap, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpev,ZHPEV)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7026
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpev "F_FUNC(zhpev,ZHPEV)"(char *jobz, char *uplo, int *n, npy_complex128 *ap, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpev(char *jobz, char *uplo, int *n, z *ap, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpev(jobz, uplo, n, <npy_complex128*>ap, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpevd "F_FUNC(zhpevd,ZHPEVD)"(char *jobz, char *uplo, int *n, npy_complex128 *ap, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhpevd(char *jobz, char *uplo, int *n, z *ap, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpevd(jobz, uplo, n, <npy_complex128*>ap, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpevd(char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7032
 *     void _fortran_zhpevd "F_FUNC(zhpevd,ZHPEVD)"(char *jobz, char *uplo, int *n, npy_complex128 *ap, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhpevd(char *jobz, char *uplo, int *n, z *ap, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zhpevd(jobz, uplo, n, <npy_complex128*>ap, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpevd,ZHPEVD)(__pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7031
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpevd "F_FUNC(zhpevd,ZHPEVD)"(char *jobz, char *uplo, int *n, npy_complex128 *ap, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhpevd(char *jobz, char *uplo, int *n, z *ap, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpevd(jobz, uplo, n, <npy_complex128*>ap, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpevx "F_FUNC(zhpevx,ZHPEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhpevx(char *jobz, char *range, char *uplo, int *n, z *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpevx(jobz, range, uplo, n, <npy_complex128*>ap, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpevx(char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7037
 *     void _fortran_zhpevx "F_FUNC(zhpevx,ZHPEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhpevx(char *jobz, char *range, char *uplo, int *n, z *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_zhpevx(jobz, range, uplo, n, <npy_complex128*>ap, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpevx,ZHPEVX)(__pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7036
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpevx "F_FUNC(zhpevx,ZHPEVX)"(char *jobz, char *range, char *uplo, int *n, npy_complex128 *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhpevx(char *jobz, char *range, char *uplo, int *n, z *ap, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpevx(jobz, range, uplo, n, <npy_complex128*>ap, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpgst "F_FUNC(zhpgst,ZHPGST)"(int *itype, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, int *info) nogil
 * cdef void zhpgst(int *itype, char *uplo, int *n, z *ap, z *bp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpgst(itype, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpgst(int *__pyx_v_itype, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_bp, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7042
 *     void _fortran_zhpgst "F_FUNC(zhpgst,ZHPGST)"(int *itype, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, int *info) nogil
 * cdef void zhpgst(int *itype, char *uplo, int *n, z *ap, z *bp, int *info) nogil:
 *     _fortran_zhpgst(itype, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpgst,ZHPGST)(__pyx_v_itype, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_bp), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7041
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpgst "F_FUNC(zhpgst,ZHPGST)"(int *itype, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, int *info) nogil
 * cdef void zhpgst(int *itype, char *uplo, int *n, z *ap, z *bp, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpgst(itype, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpgv "F_FUNC(zhpgv,ZHPGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpgv(int *itype, char *jobz, char *uplo, int *n, z *ap, z *bp, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpgv(itype, jobz, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpgv(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7047
 *     void _fortran_zhpgv "F_FUNC(zhpgv,ZHPGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpgv(int *itype, char *jobz, char *uplo, int *n, z *ap, z *bp, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:
 *     _fortran_zhpgv(itype, jobz, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpgv,ZHPGV)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_bp), __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7046
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpgv "F_FUNC(zhpgv,ZHPGV)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpgv(int *itype, char *jobz, char *uplo, int *n, z *ap, z *bp, d *w, z *z, int *ldz, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpgv(itype, jobz, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpgvd "F_FUNC(zhpgvd,ZHPGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhpgvd(int *itype, char *jobz, char *uplo, int *n, z *ap, z *bp, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpgvd(itype, jobz, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpgvd(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7052
 *     void _fortran_zhpgvd "F_FUNC(zhpgvd,ZHPGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhpgvd(int *itype, char *jobz, char *uplo, int *n, z *ap, z *bp, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zhpgvd(itype, jobz, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpgvd,ZHPGVD)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_bp), __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7051
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpgvd "F_FUNC(zhpgvd,ZHPGVD)"(int *itype, char *jobz, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zhpgvd(int *itype, char *jobz, char *uplo, int *n, z *ap, z *bp, d *w, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpgvd(itype, jobz, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpgvx "F_FUNC(zhpgvx,ZHPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhpgvx(int *itype, char *jobz, char *range, char *uplo, int *n, z *ap, z *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpgvx(itype, jobz, range, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpgvx(int *__pyx_v_itype, char *__pyx_v_jobz, char *__pyx_v_range, char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_bp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7057
 *     void _fortran_zhpgvx "F_FUNC(zhpgvx,ZHPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhpgvx(int *itype, char *jobz, char *range, char *uplo, int *n, z *ap, z *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_zhpgvx(itype, jobz, range, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpgvx,ZHPGVX)(__pyx_v_itype, __pyx_v_jobz, __pyx_v_range, __pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_bp), __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7056
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpgvx "F_FUNC(zhpgvx,ZHPGVX)"(int *itype, char *jobz, char *range, char *uplo, int *n, npy_complex128 *ap, npy_complex128 *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, npy_complex128 *work, d *rwork, int *iwork, int *ifail, int *info) nogil
 * cdef void zhpgvx(int *itype, char *jobz, char *range, char *uplo, int *n, z *ap, z *bp, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, z *work, d *rwork, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpgvx(itype, jobz, range, uplo, n, <npy_complex128*>ap, <npy_complex128*>bp, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, <npy_complex128*>work, rwork, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhprfs "F_FUNC(zhprfs,ZHPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7062
 *     void _fortran_zhprfs "F_FUNC(zhprfs,ZHPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zhprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhprfs,ZHPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_afp), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7061
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhprfs "F_FUNC(zhprfs,ZHPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpsv "F_FUNC(zhpsv,ZHPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhpsv(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpsv(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7067
 *     void _fortran_zhpsv "F_FUNC(zhpsv,ZHPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhpsv(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zhpsv(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpsv,ZHPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7066
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpsv "F_FUNC(zhpsv,ZHPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhpsv(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpsv(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpsvx "F_FUNC(zhpsvx,ZHPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhpsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7072
 *     void _fortran_zhpsvx "F_FUNC(zhpsvx,ZHPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zhpsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhpsvx,ZHPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_afp), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7071
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhpsvx "F_FUNC(zhpsvx,ZHPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zhpsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhpsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhptrd "F_FUNC(zhptrd,ZHPTRD)"(char *uplo, int *n, npy_complex128 *ap, d *d, d *e, npy_complex128 *tau, int *info) nogil
 * cdef void zhptrd(char *uplo, int *n, z *ap, d *d, d *e, z *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhptrd(uplo, n, <npy_complex128*>ap, d, e, <npy_complex128*>tau, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhptrd(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_tau, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7077
 *     void _fortran_zhptrd "F_FUNC(zhptrd,ZHPTRD)"(char *uplo, int *n, npy_complex128 *ap, d *d, d *e, npy_complex128 *tau, int *info) nogil
 * cdef void zhptrd(char *uplo, int *n, z *ap, d *d, d *e, z *tau, int *info) nogil:
 *     _fortran_zhptrd(uplo, n, <npy_complex128*>ap, d, e, <npy_complex128*>tau, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhptrd,ZHPTRD)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_tau), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7076
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhptrd "F_FUNC(zhptrd,ZHPTRD)"(char *uplo, int *n, npy_complex128 *ap, d *d, d *e, npy_complex128 *tau, int *info) nogil
 * cdef void zhptrd(char *uplo, int *n, z *ap, d *d, d *e, z *tau, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhptrd(uplo, n, <npy_complex128*>ap, d, e, <npy_complex128*>tau, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhptrf "F_FUNC(zhptrf,ZHPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, int *info) nogil
 * cdef void zhptrf(char *uplo, int *n, z *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhptrf(uplo, n, <npy_complex128*>ap, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7082
 *     void _fortran_zhptrf "F_FUNC(zhptrf,ZHPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, int *info) nogil
 * cdef void zhptrf(char *uplo, int *n, z *ap, int *ipiv, int *info) nogil:
 *     _fortran_zhptrf(uplo, n, <npy_complex128*>ap, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhptrf,ZHPTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7081
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhptrf "F_FUNC(zhptrf,ZHPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, int *info) nogil
 * cdef void zhptrf(char *uplo, int *n, z *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhptrf(uplo, n, <npy_complex128*>ap, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhptri "F_FUNC(zhptri,ZHPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zhptri(char *uplo, int *n, z *ap, int *ipiv, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhptri(uplo, n, <npy_complex128*>ap, ipiv, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7087
 *     void _fortran_zhptri "F_FUNC(zhptri,ZHPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zhptri(char *uplo, int *n, z *ap, int *ipiv, z *work, int *info) nogil:
 *     _fortran_zhptri(uplo, n, <npy_complex128*>ap, ipiv, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhptri,ZHPTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7086
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhptri "F_FUNC(zhptri,ZHPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zhptri(char *uplo, int *n, z *ap, int *ipiv, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhptri(uplo, n, <npy_complex128*>ap, ipiv, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhptrs "F_FUNC(zhptrs,ZHPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhptrs(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhptrs(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7092
 *     void _fortran_zhptrs "F_FUNC(zhptrs,ZHPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhptrs(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zhptrs(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhptrs,ZHPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7091
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhptrs "F_FUNC(zhptrs,ZHPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zhptrs(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhptrs(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhsein "F_FUNC(zhsein,ZHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *ifaill, int *ifailr, int *info) nogil
 * cdef void zhsein(char *side, char *eigsrc, char *initv, bint *select, int *n, z *h, int *ldh, z *w, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *ifaill, int *ifailr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhsein(side, eigsrc, initv, select, n, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, ifaill, ifailr, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhsein(char *__pyx_v_side, char *__pyx_v_eigsrc, char *__pyx_v_initv, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_double_complex *__pyx_v_w, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_ifaill, int *__pyx_v_ifailr, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7097
 *     void _fortran_zhsein "F_FUNC(zhsein,ZHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *ifaill, int *ifailr, int *info) nogil
 * cdef void zhsein(char *side, char *eigsrc, char *initv, bint *select, int *n, z *h, int *ldh, z *w, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *ifaill, int *ifailr, int *info) nogil:
 *     _fortran_zhsein(side, eigsrc, initv, select, n, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, ifaill, ifailr, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhsein,ZHSEIN)(__pyx_v_side, __pyx_v_eigsrc, __pyx_v_initv, __pyx_v_select, __pyx_v_n, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex128 *)__pyx_v_w), ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_ifaill, __pyx_v_ifailr, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7096
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhsein "F_FUNC(zhsein,ZHSEIN)"(char *side, char *eigsrc, char *initv, bint *select, int *n, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *ifaill, int *ifailr, int *info) nogil
 * cdef void zhsein(char *side, char *eigsrc, char *initv, bint *select, int *n, z *h, int *ldh, z *w, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *ifaill, int *ifailr, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhsein(side, eigsrc, initv, select, n, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, ifaill, ifailr, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhseqr "F_FUNC(zhseqr,ZHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhseqr(char *job, char *compz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, z *z, int *ldz, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhseqr(job, compz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zhseqr(char *__pyx_v_job, char *__pyx_v_compz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_double_complex *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7102
 *     void _fortran_zhseqr "F_FUNC(zhseqr,ZHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhseqr(char *job, char *compz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, z *z, int *ldz, z *work, int *lwork, int *info) nogil:
 *     _fortran_zhseqr(job, compz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zhseqr,ZHSEQR)(__pyx_v_job, __pyx_v_compz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex128 *)__pyx_v_w), ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7101
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zhseqr "F_FUNC(zhseqr,ZHSEQR)"(char *job, char *compz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zhseqr(char *job, char *compz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, z *z, int *ldz, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zhseqr(job, compz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlabrd "F_FUNC(zlabrd,ZLABRD)"(int *m, int *n, int *nb, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *x, int *ldx, npy_complex128 *y, int *ldy) nogil
 * cdef void zlabrd(int *m, int *n, int *nb, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *x, int *ldx, z *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlabrd(m, n, nb, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>x, ldx, <npy_complex128*>y, ldy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlabrd(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_nb, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_tauq, __pyx_t_double_complex *__pyx_v_taup, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_double_complex *__pyx_v_y, int *__pyx_v_ldy) {

  /* "scipy/linalg/cython_lapack.pyx":7107
 *     void _fortran_zlabrd "F_FUNC(zlabrd,ZLABRD)"(int *m, int *n, int *nb, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *x, int *ldx, npy_complex128 *y, int *ldy) nogil
 * cdef void zlabrd(int *m, int *n, int *nb, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *x, int *ldx, z *y, int *ldy) nogil:
 *     _fortran_zlabrd(m, n, nb, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>x, ldx, <npy_complex128*>y, ldy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlabrd,ZLABRD)(__pyx_v_m, __pyx_v_n, __pyx_v_nb, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_tauq), ((npy_complex128 *)__pyx_v_taup), ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, ((npy_complex128 *)__pyx_v_y), __pyx_v_ldy);

  /* "scipy/linalg/cython_lapack.pyx":7106
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlabrd "F_FUNC(zlabrd,ZLABRD)"(int *m, int *n, int *nb, npy_complex128 *a, int *lda, d *d, d *e, npy_complex128 *tauq, npy_complex128 *taup, npy_complex128 *x, int *ldx, npy_complex128 *y, int *ldy) nogil
 * cdef void zlabrd(int *m, int *n, int *nb, z *a, int *lda, d *d, d *e, z *tauq, z *taup, z *x, int *ldx, z *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlabrd(m, n, nb, <npy_complex128*>a, lda, d, e, <npy_complex128*>tauq, <npy_complex128*>taup, <npy_complex128*>x, ldx, <npy_complex128*>y, ldy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacgv "F_FUNC(zlacgv,ZLACGV)"(int *n, npy_complex128 *x, int *incx) nogil
 * cdef void zlacgv(int *n, z *x, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacgv(n, <npy_complex128*>x, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlacgv(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":7112
 *     void _fortran_zlacgv "F_FUNC(zlacgv,ZLACGV)"(int *n, npy_complex128 *x, int *incx) nogil
 * cdef void zlacgv(int *n, z *x, int *incx) nogil:
 *     _fortran_zlacgv(n, <npy_complex128*>x, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlacgv,ZLACGV)(__pyx_v_n, ((npy_complex128 *)__pyx_v_x), __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":7111
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacgv "F_FUNC(zlacgv,ZLACGV)"(int *n, npy_complex128 *x, int *incx) nogil
 * cdef void zlacgv(int *n, z *x, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacgv(n, <npy_complex128*>x, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacn2 "F_FUNC(zlacn2,ZLACN2)"(int *n, npy_complex128 *v, npy_complex128 *x, d *est, int *kase, int *isave) nogil
 * cdef void zlacn2(int *n, z *v, z *x, d *est, int *kase, int *isave) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacn2(n, <npy_complex128*>v, <npy_complex128*>x, est, kase, isave)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlacn2(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_v, __pyx_t_double_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_est, int *__pyx_v_kase, int *__pyx_v_isave) {

  /* "scipy/linalg/cython_lapack.pyx":7117
 *     void _fortran_zlacn2 "F_FUNC(zlacn2,ZLACN2)"(int *n, npy_complex128 *v, npy_complex128 *x, d *est, int *kase, int *isave) nogil
 * cdef void zlacn2(int *n, z *v, z *x, d *est, int *kase, int *isave) nogil:
 *     _fortran_zlacn2(n, <npy_complex128*>v, <npy_complex128*>x, est, kase, isave)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlacn2,ZLACN2)(__pyx_v_n, ((npy_complex128 *)__pyx_v_v), ((npy_complex128 *)__pyx_v_x), __pyx_v_est, __pyx_v_kase, __pyx_v_isave);

  /* "scipy/linalg/cython_lapack.pyx":7116
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacn2 "F_FUNC(zlacn2,ZLACN2)"(int *n, npy_complex128 *v, npy_complex128 *x, d *est, int *kase, int *isave) nogil
 * cdef void zlacn2(int *n, z *v, z *x, d *est, int *kase, int *isave) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacn2(n, <npy_complex128*>v, <npy_complex128*>x, est, kase, isave)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacon "F_FUNC(zlacon,ZLACON)"(int *n, npy_complex128 *v, npy_complex128 *x, d *est, int *kase) nogil
 * cdef void zlacon(int *n, z *v, z *x, d *est, int *kase) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacon(n, <npy_complex128*>v, <npy_complex128*>x, est, kase)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlacon(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_v, __pyx_t_double_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_est, int *__pyx_v_kase) {

  /* "scipy/linalg/cython_lapack.pyx":7122
 *     void _fortran_zlacon "F_FUNC(zlacon,ZLACON)"(int *n, npy_complex128 *v, npy_complex128 *x, d *est, int *kase) nogil
 * cdef void zlacon(int *n, z *v, z *x, d *est, int *kase) nogil:
 *     _fortran_zlacon(n, <npy_complex128*>v, <npy_complex128*>x, est, kase)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlacon,ZLACON)(__pyx_v_n, ((npy_complex128 *)__pyx_v_v), ((npy_complex128 *)__pyx_v_x), __pyx_v_est, __pyx_v_kase);

  /* "scipy/linalg/cython_lapack.pyx":7121
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacon "F_FUNC(zlacon,ZLACON)"(int *n, npy_complex128 *v, npy_complex128 *x, d *est, int *kase) nogil
 * cdef void zlacon(int *n, z *v, z *x, d *est, int *kase) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacon(n, <npy_complex128*>v, <npy_complex128*>x, est, kase)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacp2 "F_FUNC(zlacp2,ZLACP2)"(char *uplo, int *m, int *n, d *a, int *lda, npy_complex128 *b, int *ldb) nogil
 * cdef void zlacp2(char *uplo, int *m, int *n, d *a, int *lda, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacp2(uplo, m, n, a, lda, <npy_complex128*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlacp2(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":7127
 *     void _fortran_zlacp2 "F_FUNC(zlacp2,ZLACP2)"(char *uplo, int *m, int *n, d *a, int *lda, npy_complex128 *b, int *ldb) nogil
 * cdef void zlacp2(char *uplo, int *m, int *n, d *a, int *lda, z *b, int *ldb) nogil:
 *     _fortran_zlacp2(uplo, m, n, a, lda, <npy_complex128*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlacp2,ZLACP2)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":7126
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacp2 "F_FUNC(zlacp2,ZLACP2)"(char *uplo, int *m, int *n, d *a, int *lda, npy_complex128 *b, int *ldb) nogil
 * cdef void zlacp2(char *uplo, int *m, int *n, d *a, int *lda, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacp2(uplo, m, n, a, lda, <npy_complex128*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacpy "F_FUNC(zlacpy,ZLACPY)"(char *uplo, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb) nogil
 * cdef void zlacpy(char *uplo, int *m, int *n, z *a, int *lda, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacpy(uplo, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlacpy(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":7132
 *     void _fortran_zlacpy "F_FUNC(zlacpy,ZLACPY)"(char *uplo, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb) nogil
 * cdef void zlacpy(char *uplo, int *m, int *n, z *a, int *lda, z *b, int *ldb) nogil:
 *     _fortran_zlacpy(uplo, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlacpy,ZLACPY)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":7131
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacpy "F_FUNC(zlacpy,ZLACPY)"(char *uplo, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb) nogil
 * cdef void zlacpy(char *uplo, int *m, int *n, z *a, int *lda, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacpy(uplo, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacrm "F_FUNC(zlacrm,ZLACRM)"(int *m, int *n, npy_complex128 *a, int *lda, d *b, int *ldb, npy_complex128 *c, int *ldc, d *rwork) nogil
 * cdef void zlacrm(int *m, int *n, z *a, int *lda, d *b, int *ldb, z *c, int *ldc, d *rwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacrm(m, n, <npy_complex128*>a, lda, b, ldb, <npy_complex128*>c, ldc, rwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlacrm(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork) {

  /* "scipy/linalg/cython_lapack.pyx":7137
 *     void _fortran_zlacrm "F_FUNC(zlacrm,ZLACRM)"(int *m, int *n, npy_complex128 *a, int *lda, d *b, int *ldb, npy_complex128 *c, int *ldc, d *rwork) nogil
 * cdef void zlacrm(int *m, int *n, z *a, int *lda, d *b, int *ldb, z *c, int *ldc, d *rwork) nogil:
 *     _fortran_zlacrm(m, n, <npy_complex128*>a, lda, b, ldb, <npy_complex128*>c, ldc, rwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlacrm,ZLACRM)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_b, __pyx_v_ldb, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, __pyx_v_rwork);

  /* "scipy/linalg/cython_lapack.pyx":7136
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacrm "F_FUNC(zlacrm,ZLACRM)"(int *m, int *n, npy_complex128 *a, int *lda, d *b, int *ldb, npy_complex128 *c, int *ldc, d *rwork) nogil
 * cdef void zlacrm(int *m, int *n, z *a, int *lda, d *b, int *ldb, z *c, int *ldc, d *rwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacrm(m, n, <npy_complex128*>a, lda, b, ldb, <npy_complex128*>c, ldc, rwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacrt "F_FUNC(zlacrt,ZLACRT)"(int *n, npy_complex128 *cx, int *incx, npy_complex128 *cy, int *incy, npy_complex128 *c, npy_complex128 *s) nogil
 * cdef void zlacrt(int *n, z *cx, int *incx, z *cy, int *incy, z *c, z *s) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacrt(n, <npy_complex128*>cx, incx, <npy_complex128*>cy, incy, <npy_complex128*>c, <npy_complex128*>s)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlacrt(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_cx, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_cy, int *__pyx_v_incy, __pyx_t_double_complex *__pyx_v_c, __pyx_t_double_complex *__pyx_v_s) {

  /* "scipy/linalg/cython_lapack.pyx":7142
 *     void _fortran_zlacrt "F_FUNC(zlacrt,ZLACRT)"(int *n, npy_complex128 *cx, int *incx, npy_complex128 *cy, int *incy, npy_complex128 *c, npy_complex128 *s) nogil
 * cdef void zlacrt(int *n, z *cx, int *incx, z *cy, int *incy, z *c, z *s) nogil:
 *     _fortran_zlacrt(n, <npy_complex128*>cx, incx, <npy_complex128*>cy, incy, <npy_complex128*>c, <npy_complex128*>s)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlacrt,ZLACRT)(__pyx_v_n, ((npy_complex128 *)__pyx_v_cx), __pyx_v_incx, ((npy_complex128 *)__pyx_v_cy), __pyx_v_incy, ((npy_complex128 *)__pyx_v_c), ((npy_complex128 *)__pyx_v_s));

  /* "scipy/linalg/cython_lapack.pyx":7141
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlacrt "F_FUNC(zlacrt,ZLACRT)"(int *n, npy_complex128 *cx, int *incx, npy_complex128 *cy, int *incy, npy_complex128 *c, npy_complex128 *s) nogil
 * cdef void zlacrt(int *n, z *cx, int *incx, z *cy, int *incy, z *c, z *s) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlacrt(n, <npy_complex128*>cx, incx, <npy_complex128*>cy, incy, <npy_complex128*>c, <npy_complex128*>s)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaed0 "F_FUNC(zlaed0,ZLAED0)"(int *qsiz, int *n, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *qstore, int *ldqs, d *rwork, int *iwork, int *info) nogil
 * cdef void zlaed0(int *qsiz, int *n, d *d, d *e, z *q, int *ldq, z *qstore, int *ldqs, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaed0(qsiz, n, d, e, <npy_complex128*>q, ldq, <npy_complex128*>qstore, ldqs, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaed0(int *__pyx_v_qsiz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_qstore, int *__pyx_v_ldqs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7147
 *     void _fortran_zlaed0 "F_FUNC(zlaed0,ZLAED0)"(int *qsiz, int *n, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *qstore, int *ldqs, d *rwork, int *iwork, int *info) nogil
 * cdef void zlaed0(int *qsiz, int *n, d *d, d *e, z *q, int *ldq, z *qstore, int *ldqs, d *rwork, int *iwork, int *info) nogil:
 *     _fortran_zlaed0(qsiz, n, d, e, <npy_complex128*>q, ldq, <npy_complex128*>qstore, ldqs, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaed0,ZLAED0)(__pyx_v_qsiz, __pyx_v_n, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_qstore), __pyx_v_ldqs, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7146
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaed0 "F_FUNC(zlaed0,ZLAED0)"(int *qsiz, int *n, d *d, d *e, npy_complex128 *q, int *ldq, npy_complex128 *qstore, int *ldqs, d *rwork, int *iwork, int *info) nogil
 * cdef void zlaed0(int *qsiz, int *n, d *d, d *e, z *q, int *ldq, z *qstore, int *ldqs, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaed0(qsiz, n, d, e, <npy_complex128*>q, ldq, <npy_complex128*>qstore, ldqs, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaed7 "F_FUNC(zlaed7,ZLAED7)"(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, npy_complex128 *q, int *ldq, d *rho, int *indxq, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, npy_complex128 *work, d *rwork, int *iwork, int *info) nogil
 * cdef void zlaed7(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, z *q, int *ldq, d *rho, int *indxq, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, z *work, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaed7(n, cutpnt, qsiz, tlvls, curlvl, curpbm, d, <npy_complex128*>q, ldq, rho, indxq, qstore, qptr, prmptr, perm, givptr, givcol, givnum, <npy_complex128*>work, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaed7(int *__pyx_v_n, int *__pyx_v_cutpnt, int *__pyx_v_qsiz, int *__pyx_v_tlvls, int *__pyx_v_curlvl, int *__pyx_v_curpbm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, int *__pyx_v_indxq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_qstore, int *__pyx_v_qptr, int *__pyx_v_prmptr, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7152
 *     void _fortran_zlaed7 "F_FUNC(zlaed7,ZLAED7)"(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, npy_complex128 *q, int *ldq, d *rho, int *indxq, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, npy_complex128 *work, d *rwork, int *iwork, int *info) nogil
 * cdef void zlaed7(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, z *q, int *ldq, d *rho, int *indxq, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, z *work, d *rwork, int *iwork, int *info) nogil:
 *     _fortran_zlaed7(n, cutpnt, qsiz, tlvls, curlvl, curpbm, d, <npy_complex128*>q, ldq, rho, indxq, qstore, qptr, prmptr, perm, givptr, givcol, givnum, <npy_complex128*>work, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaed7,ZLAED7)(__pyx_v_n, __pyx_v_cutpnt, __pyx_v_qsiz, __pyx_v_tlvls, __pyx_v_curlvl, __pyx_v_curpbm, __pyx_v_d, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_rho, __pyx_v_indxq, __pyx_v_qstore, __pyx_v_qptr, __pyx_v_prmptr, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7151
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaed7 "F_FUNC(zlaed7,ZLAED7)"(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, npy_complex128 *q, int *ldq, d *rho, int *indxq, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, npy_complex128 *work, d *rwork, int *iwork, int *info) nogil
 * cdef void zlaed7(int *n, int *cutpnt, int *qsiz, int *tlvls, int *curlvl, int *curpbm, d *d, z *q, int *ldq, d *rho, int *indxq, d *qstore, int *qptr, int *prmptr, int *perm, int *givptr, int *givcol, d *givnum, z *work, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaed7(n, cutpnt, qsiz, tlvls, curlvl, curpbm, d, <npy_complex128*>q, ldq, rho, indxq, qstore, qptr, prmptr, perm, givptr, givcol, givnum, <npy_complex128*>work, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaed8 "F_FUNC(zlaed8,ZLAED8)"(int *k, int *n, int *qsiz, npy_complex128 *q, int *ldq, d *d, d *rho, int *cutpnt, d *z, d *dlamda, npy_complex128 *q2, int *ldq2, d *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, d *givnum, int *info) nogil
 * cdef void zlaed8(int *k, int *n, int *qsiz, z *q, int *ldq, d *d, d *rho, int *cutpnt, d *z, d *dlamda, z *q2, int *ldq2, d *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, d *givnum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaed8(k, n, qsiz, <npy_complex128*>q, ldq, d, rho, cutpnt, z, dlamda, <npy_complex128*>q2, ldq2, w, indxp, indx, indxq, perm, givptr, givcol, givnum, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaed8(int *__pyx_v_k, int *__pyx_v_n, int *__pyx_v_qsiz, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rho, int *__pyx_v_cutpnt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dlamda, __pyx_t_double_complex *__pyx_v_q2, int *__pyx_v_ldq2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, int *__pyx_v_indxp, int *__pyx_v_indx, int *__pyx_v_indxq, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7157
 *     void _fortran_zlaed8 "F_FUNC(zlaed8,ZLAED8)"(int *k, int *n, int *qsiz, npy_complex128 *q, int *ldq, d *d, d *rho, int *cutpnt, d *z, d *dlamda, npy_complex128 *q2, int *ldq2, d *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, d *givnum, int *info) nogil
 * cdef void zlaed8(int *k, int *n, int *qsiz, z *q, int *ldq, d *d, d *rho, int *cutpnt, d *z, d *dlamda, z *q2, int *ldq2, d *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, d *givnum, int *info) nogil:
 *     _fortran_zlaed8(k, n, qsiz, <npy_complex128*>q, ldq, d, rho, cutpnt, z, dlamda, <npy_complex128*>q2, ldq2, w, indxp, indx, indxq, perm, givptr, givcol, givnum, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaed8,ZLAED8)(__pyx_v_k, __pyx_v_n, __pyx_v_qsiz, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_d, __pyx_v_rho, __pyx_v_cutpnt, __pyx_v_z, __pyx_v_dlamda, ((npy_complex128 *)__pyx_v_q2), __pyx_v_ldq2, __pyx_v_w, __pyx_v_indxp, __pyx_v_indx, __pyx_v_indxq, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_givnum, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7156
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaed8 "F_FUNC(zlaed8,ZLAED8)"(int *k, int *n, int *qsiz, npy_complex128 *q, int *ldq, d *d, d *rho, int *cutpnt, d *z, d *dlamda, npy_complex128 *q2, int *ldq2, d *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, d *givnum, int *info) nogil
 * cdef void zlaed8(int *k, int *n, int *qsiz, z *q, int *ldq, d *d, d *rho, int *cutpnt, d *z, d *dlamda, z *q2, int *ldq2, d *w, int *indxp, int *indx, int *indxq, int *perm, int *givptr, int *givcol, d *givnum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaed8(k, n, qsiz, <npy_complex128*>q, ldq, d, rho, cutpnt, z, dlamda, <npy_complex128*>q2, ldq2, w, indxp, indx, indxq, perm, givptr, givcol, givnum, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaein "F_FUNC(zlaein,ZLAEIN)"(bint *rightv, bint *noinit, int *n, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *v, npy_complex128 *b, int *ldb, d *rwork, d *eps3, d *smlnum, int *info) nogil
 * cdef void zlaein(bint *rightv, bint *noinit, int *n, z *h, int *ldh, z *w, z *v, z *b, int *ldb, d *rwork, d *eps3, d *smlnum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaein(rightv, noinit, n, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>v, <npy_complex128*>b, ldb, rwork, eps3, smlnum, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaein(int *__pyx_v_rightv, int *__pyx_v_noinit, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_double_complex *__pyx_v_w, __pyx_t_double_complex *__pyx_v_v, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_eps3, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_smlnum, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7162
 *     void _fortran_zlaein "F_FUNC(zlaein,ZLAEIN)"(bint *rightv, bint *noinit, int *n, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *v, npy_complex128 *b, int *ldb, d *rwork, d *eps3, d *smlnum, int *info) nogil
 * cdef void zlaein(bint *rightv, bint *noinit, int *n, z *h, int *ldh, z *w, z *v, z *b, int *ldb, d *rwork, d *eps3, d *smlnum, int *info) nogil:
 *     _fortran_zlaein(rightv, noinit, n, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>v, <npy_complex128*>b, ldb, rwork, eps3, smlnum, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaein,ZLAEIN)(__pyx_v_rightv, __pyx_v_noinit, __pyx_v_n, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex128 *)__pyx_v_w), ((npy_complex128 *)__pyx_v_v), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_rwork, __pyx_v_eps3, __pyx_v_smlnum, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7161
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaein "F_FUNC(zlaein,ZLAEIN)"(bint *rightv, bint *noinit, int *n, npy_complex128 *h, int *ldh, npy_complex128 *w, npy_complex128 *v, npy_complex128 *b, int *ldb, d *rwork, d *eps3, d *smlnum, int *info) nogil
 * cdef void zlaein(bint *rightv, bint *noinit, int *n, z *h, int *ldh, z *w, z *v, z *b, int *ldb, d *rwork, d *eps3, d *smlnum, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaein(rightv, noinit, n, <npy_complex128*>h, ldh, <npy_complex128*>w, <npy_complex128*>v, <npy_complex128*>b, ldb, rwork, eps3, smlnum, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaesy "F_FUNC(zlaesy,ZLAESY)"(npy_complex128 *a, npy_complex128 *b, npy_complex128 *c, npy_complex128 *rt1, npy_complex128 *rt2, npy_complex128 *evscal, npy_complex128 *cs1, npy_complex128 *sn1) nogil
 * cdef void zlaesy(z *a, z *b, z *c, z *rt1, z *rt2, z *evscal, z *cs1, z *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaesy(<npy_complex128*>a, <npy_complex128*>b, <npy_complex128*>c, <npy_complex128*>rt1, <npy_complex128*>rt2, <npy_complex128*>evscal, <npy_complex128*>cs1, <npy_complex128*>sn1)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaesy(__pyx_t_double_complex *__pyx_v_a, __pyx_t_double_complex *__pyx_v_b, __pyx_t_double_complex *__pyx_v_c, __pyx_t_double_complex *__pyx_v_rt1, __pyx_t_double_complex *__pyx_v_rt2, __pyx_t_double_complex *__pyx_v_evscal, __pyx_t_double_complex *__pyx_v_cs1, __pyx_t_double_complex *__pyx_v_sn1) {

  /* "scipy/linalg/cython_lapack.pyx":7167
 *     void _fortran_zlaesy "F_FUNC(zlaesy,ZLAESY)"(npy_complex128 *a, npy_complex128 *b, npy_complex128 *c, npy_complex128 *rt1, npy_complex128 *rt2, npy_complex128 *evscal, npy_complex128 *cs1, npy_complex128 *sn1) nogil
 * cdef void zlaesy(z *a, z *b, z *c, z *rt1, z *rt2, z *evscal, z *cs1, z *sn1) nogil:
 *     _fortran_zlaesy(<npy_complex128*>a, <npy_complex128*>b, <npy_complex128*>c, <npy_complex128*>rt1, <npy_complex128*>rt2, <npy_complex128*>evscal, <npy_complex128*>cs1, <npy_complex128*>sn1)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaesy,ZLAESY)(((npy_complex128 *)__pyx_v_a), ((npy_complex128 *)__pyx_v_b), ((npy_complex128 *)__pyx_v_c), ((npy_complex128 *)__pyx_v_rt1), ((npy_complex128 *)__pyx_v_rt2), ((npy_complex128 *)__pyx_v_evscal), ((npy_complex128 *)__pyx_v_cs1), ((npy_complex128 *)__pyx_v_sn1));

  /* "scipy/linalg/cython_lapack.pyx":7166
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaesy "F_FUNC(zlaesy,ZLAESY)"(npy_complex128 *a, npy_complex128 *b, npy_complex128 *c, npy_complex128 *rt1, npy_complex128 *rt2, npy_complex128 *evscal, npy_complex128 *cs1, npy_complex128 *sn1) nogil
 * cdef void zlaesy(z *a, z *b, z *c, z *rt1, z *rt2, z *evscal, z *cs1, z *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaesy(<npy_complex128*>a, <npy_complex128*>b, <npy_complex128*>c, <npy_complex128*>rt1, <npy_complex128*>rt2, <npy_complex128*>evscal, <npy_complex128*>cs1, <npy_complex128*>sn1)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaev2 "F_FUNC(zlaev2,ZLAEV2)"(npy_complex128 *a, npy_complex128 *b, npy_complex128 *c, d *rt1, d *rt2, d *cs1, npy_complex128 *sn1) nogil
 * cdef void zlaev2(z *a, z *b, z *c, d *rt1, d *rt2, d *cs1, z *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaev2(<npy_complex128*>a, <npy_complex128*>b, <npy_complex128*>c, rt1, rt2, cs1, <npy_complex128*>sn1)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaev2(__pyx_t_double_complex *__pyx_v_a, __pyx_t_double_complex *__pyx_v_b, __pyx_t_double_complex *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rt2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cs1, __pyx_t_double_complex *__pyx_v_sn1) {

  /* "scipy/linalg/cython_lapack.pyx":7172
 *     void _fortran_zlaev2 "F_FUNC(zlaev2,ZLAEV2)"(npy_complex128 *a, npy_complex128 *b, npy_complex128 *c, d *rt1, d *rt2, d *cs1, npy_complex128 *sn1) nogil
 * cdef void zlaev2(z *a, z *b, z *c, d *rt1, d *rt2, d *cs1, z *sn1) nogil:
 *     _fortran_zlaev2(<npy_complex128*>a, <npy_complex128*>b, <npy_complex128*>c, rt1, rt2, cs1, <npy_complex128*>sn1)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaev2,ZLAEV2)(((npy_complex128 *)__pyx_v_a), ((npy_complex128 *)__pyx_v_b), ((npy_complex128 *)__pyx_v_c), __pyx_v_rt1, __pyx_v_rt2, __pyx_v_cs1, ((npy_complex128 *)__pyx_v_sn1));

  /* "scipy/linalg/cython_lapack.pyx":7171
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaev2 "F_FUNC(zlaev2,ZLAEV2)"(npy_complex128 *a, npy_complex128 *b, npy_complex128 *c, d *rt1, d *rt2, d *cs1, npy_complex128 *sn1) nogil
 * cdef void zlaev2(z *a, z *b, z *c, d *rt1, d *rt2, d *cs1, z *sn1) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaev2(<npy_complex128*>a, <npy_complex128*>b, <npy_complex128*>c, rt1, rt2, cs1, <npy_complex128*>sn1)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlag2c "F_FUNC(zlag2c,ZLAG2C)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex64 *sa, int *ldsa, int *info) nogil
 * cdef void zlag2c(int *m, int *n, z *a, int *lda, c *sa, int *ldsa, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlag2c(m, n, <npy_complex128*>a, lda, <npy_complex64*>sa, ldsa, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlag2c(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_float_complex *__pyx_v_sa, int *__pyx_v_ldsa, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7177
 *     void _fortran_zlag2c "F_FUNC(zlag2c,ZLAG2C)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex64 *sa, int *ldsa, int *info) nogil
 * cdef void zlag2c(int *m, int *n, z *a, int *lda, c *sa, int *ldsa, int *info) nogil:
 *     _fortran_zlag2c(m, n, <npy_complex128*>a, lda, <npy_complex64*>sa, ldsa, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlag2c,ZLAG2C)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex64 *)__pyx_v_sa), __pyx_v_ldsa, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7176
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlag2c "F_FUNC(zlag2c,ZLAG2C)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex64 *sa, int *ldsa, int *info) nogil
 * cdef void zlag2c(int *m, int *n, z *a, int *lda, c *sa, int *ldsa, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlag2c(m, n, <npy_complex128*>a, lda, <npy_complex64*>sa, ldsa, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlags2 "F_FUNC(zlags2,ZLAGS2)"(bint *upper, d *a1, npy_complex128 *a2, d *a3, d *b1, npy_complex128 *b2, d *b3, d *csu, npy_complex128 *snu, d *csv, npy_complex128 *snv, d *csq, npy_complex128 *snq) nogil
 * cdef void zlags2(bint *upper, d *a1, z *a2, d *a3, d *b1, z *b2, d *b3, d *csu, z *snu, d *csv, z *snv, d *csq, z *snq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlags2(upper, a1, <npy_complex128*>a2, a3, b1, <npy_complex128*>b2, b3, csu, <npy_complex128*>snu, csv, <npy_complex128*>snv, csq, <npy_complex128*>snq)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlags2(int *__pyx_v_upper, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a1, __pyx_t_double_complex *__pyx_v_a2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a3, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b1, __pyx_t_double_complex *__pyx_v_b2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_b3, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csu, __pyx_t_double_complex *__pyx_v_snu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csv, __pyx_t_double_complex *__pyx_v_snv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_csq, __pyx_t_double_complex *__pyx_v_snq) {

  /* "scipy/linalg/cython_lapack.pyx":7182
 *     void _fortran_zlags2 "F_FUNC(zlags2,ZLAGS2)"(bint *upper, d *a1, npy_complex128 *a2, d *a3, d *b1, npy_complex128 *b2, d *b3, d *csu, npy_complex128 *snu, d *csv, npy_complex128 *snv, d *csq, npy_complex128 *snq) nogil
 * cdef void zlags2(bint *upper, d *a1, z *a2, d *a3, d *b1, z *b2, d *b3, d *csu, z *snu, d *csv, z *snv, d *csq, z *snq) nogil:
 *     _fortran_zlags2(upper, a1, <npy_complex128*>a2, a3, b1, <npy_complex128*>b2, b3, csu, <npy_complex128*>snu, csv, <npy_complex128*>snv, csq, <npy_complex128*>snq)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlags2,ZLAGS2)(__pyx_v_upper, __pyx_v_a1, ((npy_complex128 *)__pyx_v_a2), __pyx_v_a3, __pyx_v_b1, ((npy_complex128 *)__pyx_v_b2), __pyx_v_b3, __pyx_v_csu, ((npy_complex128 *)__pyx_v_snu), __pyx_v_csv, ((npy_complex128 *)__pyx_v_snv), __pyx_v_csq, ((npy_complex128 *)__pyx_v_snq));

  /* "scipy/linalg/cython_lapack.pyx":7181
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlags2 "F_FUNC(zlags2,ZLAGS2)"(bint *upper, d *a1, npy_complex128 *a2, d *a3, d *b1, npy_complex128 *b2, d *b3, d *csu, npy_complex128 *snu, d *csv, npy_complex128 *snv, d *csq, npy_complex128 *snq) nogil
 * cdef void zlags2(bint *upper, d *a1, z *a2, d *a3, d *b1, z *b2, d *b3, d *csu, z *snu, d *csv, z *snv, d *csq, z *snq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlags2(upper, a1, <npy_complex128*>a2, a3, b1, <npy_complex128*>b2, b3, csu, <npy_complex128*>snu, csv, <npy_complex128*>snv, csq, <npy_complex128*>snq)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlagtm "F_FUNC(zlagtm,ZLAGTM)"(char *trans, int *n, int *nrhs, d *alpha, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *x, int *ldx, d *beta, npy_complex128 *b, int *ldb) nogil
 * cdef void zlagtm(char *trans, int *n, int *nrhs, d *alpha, z *dl, z *d, z *du, z *x, int *ldx, d *beta, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlagtm(trans, n, nrhs, alpha, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>x, ldx, beta, <npy_complex128*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlagtm(char *__pyx_v_trans, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_dl, __pyx_t_double_complex *__pyx_v_d, __pyx_t_double_complex *__pyx_v_du, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":7187
 *     void _fortran_zlagtm "F_FUNC(zlagtm,ZLAGTM)"(char *trans, int *n, int *nrhs, d *alpha, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *x, int *ldx, d *beta, npy_complex128 *b, int *ldb) nogil
 * cdef void zlagtm(char *trans, int *n, int *nrhs, d *alpha, z *dl, z *d, z *du, z *x, int *ldx, d *beta, z *b, int *ldb) nogil:
 *     _fortran_zlagtm(trans, n, nrhs, alpha, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>x, ldx, beta, <npy_complex128*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlagtm,ZLAGTM)(__pyx_v_trans, __pyx_v_n, __pyx_v_nrhs, __pyx_v_alpha, ((npy_complex128 *)__pyx_v_dl), ((npy_complex128 *)__pyx_v_d), ((npy_complex128 *)__pyx_v_du), ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_beta, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":7186
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlagtm "F_FUNC(zlagtm,ZLAGTM)"(char *trans, int *n, int *nrhs, d *alpha, npy_complex128 *dl, npy_complex128 *d, npy_complex128 *du, npy_complex128 *x, int *ldx, d *beta, npy_complex128 *b, int *ldb) nogil
 * cdef void zlagtm(char *trans, int *n, int *nrhs, d *alpha, z *dl, z *d, z *du, z *x, int *ldx, d *beta, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlagtm(trans, n, nrhs, alpha, <npy_complex128*>dl, <npy_complex128*>d, <npy_complex128*>du, <npy_complex128*>x, ldx, beta, <npy_complex128*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlahef "F_FUNC(zlahef,ZLAHEF)"(char *uplo, int *n, int *nb, int *kb, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *w, int *ldw, int *info) nogil
 * cdef void zlahef(char *uplo, int *n, int *nb, int *kb, z *a, int *lda, int *ipiv, z *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlahef(uplo, n, nb, kb, <npy_complex128*>a, lda, ipiv, <npy_complex128*>w, ldw, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlahef(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_w, int *__pyx_v_ldw, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7192
 *     void _fortran_zlahef "F_FUNC(zlahef,ZLAHEF)"(char *uplo, int *n, int *nb, int *kb, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *w, int *ldw, int *info) nogil
 * cdef void zlahef(char *uplo, int *n, int *nb, int *kb, z *a, int *lda, int *ipiv, z *w, int *ldw, int *info) nogil:
 *     _fortran_zlahef(uplo, n, nb, kb, <npy_complex128*>a, lda, ipiv, <npy_complex128*>w, ldw, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlahef,ZLAHEF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, __pyx_v_kb, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_w), __pyx_v_ldw, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7191
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlahef "F_FUNC(zlahef,ZLAHEF)"(char *uplo, int *n, int *nb, int *kb, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *w, int *ldw, int *info) nogil
 * cdef void zlahef(char *uplo, int *n, int *nb, int *kb, z *a, int *lda, int *ipiv, z *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlahef(uplo, n, nb, kb, <npy_complex128*>a, lda, ipiv, <npy_complex128*>w, ldw, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlahqr "F_FUNC(zlahqr,ZLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *info) nogil
 * cdef void zlahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlahqr(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlahqr(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_double_complex *__pyx_v_w, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7197
 *     void _fortran_zlahqr "F_FUNC(zlahqr,ZLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *info) nogil
 * cdef void zlahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, int *info) nogil:
 *     _fortran_zlahqr(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlahqr,ZLAHQR)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex128 *)__pyx_v_w), __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7196
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlahqr "F_FUNC(zlahqr,ZLAHQR)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *info) nogil
 * cdef void zlahqr(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlahqr(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlahr2 "F_FUNC(zlahr2,ZLAHR2)"(int *n, int *k, int *nb, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *t, int *ldt, npy_complex128 *y, int *ldy) nogil
 * cdef void zlahr2(int *n, int *k, int *nb, z *a, int *lda, z *tau, z *t, int *ldt, z *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlahr2(n, k, nb, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>t, ldt, <npy_complex128*>y, ldy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlahr2(int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_nb, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_double_complex *__pyx_v_y, int *__pyx_v_ldy) {

  /* "scipy/linalg/cython_lapack.pyx":7202
 *     void _fortran_zlahr2 "F_FUNC(zlahr2,ZLAHR2)"(int *n, int *k, int *nb, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *t, int *ldt, npy_complex128 *y, int *ldy) nogil
 * cdef void zlahr2(int *n, int *k, int *nb, z *a, int *lda, z *tau, z *t, int *ldt, z *y, int *ldy) nogil:
 *     _fortran_zlahr2(n, k, nb, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>t, ldt, <npy_complex128*>y, ldy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlahr2,ZLAHR2)(__pyx_v_n, __pyx_v_k, __pyx_v_nb, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex128 *)__pyx_v_y), __pyx_v_ldy);

  /* "scipy/linalg/cython_lapack.pyx":7201
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlahr2 "F_FUNC(zlahr2,ZLAHR2)"(int *n, int *k, int *nb, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *t, int *ldt, npy_complex128 *y, int *ldy) nogil
 * cdef void zlahr2(int *n, int *k, int *nb, z *a, int *lda, z *tau, z *t, int *ldt, z *y, int *ldy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlahr2(n, k, nb, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>t, ldt, <npy_complex128*>y, ldy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaic1 "F_FUNC(zlaic1,ZLAIC1)"(int *job, int *j, npy_complex128 *x, d *sest, npy_complex128 *w, npy_complex128 *gamma, d *sestpr, npy_complex128 *s, npy_complex128 *c) nogil
 * cdef void zlaic1(int *job, int *j, z *x, d *sest, z *w, z *gamma, d *sestpr, z *s, z *c) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaic1(job, j, <npy_complex128*>x, sest, <npy_complex128*>w, <npy_complex128*>gamma, sestpr, <npy_complex128*>s, <npy_complex128*>c)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaic1(int *__pyx_v_job, int *__pyx_v_j, __pyx_t_double_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sest, __pyx_t_double_complex *__pyx_v_w, __pyx_t_double_complex *__pyx_v_gamma, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sestpr, __pyx_t_double_complex *__pyx_v_s, __pyx_t_double_complex *__pyx_v_c) {

  /* "scipy/linalg/cython_lapack.pyx":7207
 *     void _fortran_zlaic1 "F_FUNC(zlaic1,ZLAIC1)"(int *job, int *j, npy_complex128 *x, d *sest, npy_complex128 *w, npy_complex128 *gamma, d *sestpr, npy_complex128 *s, npy_complex128 *c) nogil
 * cdef void zlaic1(int *job, int *j, z *x, d *sest, z *w, z *gamma, d *sestpr, z *s, z *c) nogil:
 *     _fortran_zlaic1(job, j, <npy_complex128*>x, sest, <npy_complex128*>w, <npy_complex128*>gamma, sestpr, <npy_complex128*>s, <npy_complex128*>c)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaic1,ZLAIC1)(__pyx_v_job, __pyx_v_j, ((npy_complex128 *)__pyx_v_x), __pyx_v_sest, ((npy_complex128 *)__pyx_v_w), ((npy_complex128 *)__pyx_v_gamma), __pyx_v_sestpr, ((npy_complex128 *)__pyx_v_s), ((npy_complex128 *)__pyx_v_c));

  /* "scipy/linalg/cython_lapack.pyx":7206
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaic1 "F_FUNC(zlaic1,ZLAIC1)"(int *job, int *j, npy_complex128 *x, d *sest, npy_complex128 *w, npy_complex128 *gamma, d *sestpr, npy_complex128 *s, npy_complex128 *c) nogil
 * cdef void zlaic1(int *job, int *j, z *x, d *sest, z *w, z *gamma, d *sestpr, z *s, z *c) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaic1(job, j, <npy_complex128*>x, sest, <npy_complex128*>w, <npy_complex128*>gamma, sestpr, <npy_complex128*>s, <npy_complex128*>c)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlals0 "F_FUNC(zlals0,ZLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, npy_complex128 *b, int *ldb, npy_complex128 *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *rwork, int *info) nogil
 * cdef void zlals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, z *b, int *ldb, z *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlals0(icompq, nl, nr, sqre, nrhs, <npy_complex128*>b, ldb, <npy_complex128*>bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlals0(int *__pyx_v_icompq, int *__pyx_v_nl, int *__pyx_v_nr, int *__pyx_v_sqre, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_bx, int *__pyx_v_ldbx, int *__pyx_v_perm, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, int *__pyx_v_ldgnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_poles, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7212
 *     void _fortran_zlals0 "F_FUNC(zlals0,ZLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, npy_complex128 *b, int *ldb, npy_complex128 *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *rwork, int *info) nogil
 * cdef void zlals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, z *b, int *ldb, z *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *rwork, int *info) nogil:
 *     _fortran_zlals0(icompq, nl, nr, sqre, nrhs, <npy_complex128*>b, ldb, <npy_complex128*>bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlals0,ZLALS0)(__pyx_v_icompq, __pyx_v_nl, __pyx_v_nr, __pyx_v_sqre, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_bx), __pyx_v_ldbx, __pyx_v_perm, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_givnum, __pyx_v_ldgnum, __pyx_v_poles, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_k, __pyx_v_c, __pyx_v_s, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7211
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlals0 "F_FUNC(zlals0,ZLALS0)"(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, npy_complex128 *b, int *ldb, npy_complex128 *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *rwork, int *info) nogil
 * cdef void zlals0(int *icompq, int *nl, int *nr, int *sqre, int *nrhs, z *b, int *ldb, z *bx, int *ldbx, int *perm, int *givptr, int *givcol, int *ldgcol, d *givnum, int *ldgnum, d *poles, d *difl, d *difr, d *z, int *k, d *c, d *s, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlals0(icompq, nl, nr, sqre, nrhs, <npy_complex128*>b, ldb, <npy_complex128*>bx, ldbx, perm, givptr, givcol, ldgcol, givnum, ldgnum, poles, difl, difr, z, k, c, s, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlalsa "F_FUNC(zlalsa,ZLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, npy_complex128 *b, int *ldb, npy_complex128 *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *rwork, int *iwork, int *info) nogil
 * cdef void zlalsa(int *icompq, int *smlsiz, int *n, int *nrhs, z *b, int *ldb, z *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlalsa(icompq, smlsiz, n, nrhs, <npy_complex128*>b, ldb, <npy_complex128*>bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlalsa(int *__pyx_v_icompq, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_bx, int *__pyx_v_ldbx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vt, int *__pyx_v_k, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_difr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_z, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_poles, int *__pyx_v_givptr, int *__pyx_v_givcol, int *__pyx_v_ldgcol, int *__pyx_v_perm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_givnum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7217
 *     void _fortran_zlalsa "F_FUNC(zlalsa,ZLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, npy_complex128 *b, int *ldb, npy_complex128 *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *rwork, int *iwork, int *info) nogil
 * cdef void zlalsa(int *icompq, int *smlsiz, int *n, int *nrhs, z *b, int *ldb, z *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *rwork, int *iwork, int *info) nogil:
 *     _fortran_zlalsa(icompq, smlsiz, n, nrhs, <npy_complex128*>b, ldb, <npy_complex128*>bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlalsa,ZLALSA)(__pyx_v_icompq, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_bx), __pyx_v_ldbx, __pyx_v_u, __pyx_v_ldu, __pyx_v_vt, __pyx_v_k, __pyx_v_difl, __pyx_v_difr, __pyx_v_z, __pyx_v_poles, __pyx_v_givptr, __pyx_v_givcol, __pyx_v_ldgcol, __pyx_v_perm, __pyx_v_givnum, __pyx_v_c, __pyx_v_s, __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7216
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlalsa "F_FUNC(zlalsa,ZLALSA)"(int *icompq, int *smlsiz, int *n, int *nrhs, npy_complex128 *b, int *ldb, npy_complex128 *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *rwork, int *iwork, int *info) nogil
 * cdef void zlalsa(int *icompq, int *smlsiz, int *n, int *nrhs, z *b, int *ldb, z *bx, int *ldbx, d *u, int *ldu, d *vt, int *k, d *difl, d *difr, d *z, d *poles, int *givptr, int *givcol, int *ldgcol, int *perm, d *givnum, d *c, d *s, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlalsa(icompq, smlsiz, n, nrhs, <npy_complex128*>b, ldb, <npy_complex128*>bx, ldbx, u, ldu, vt, k, difl, difr, z, poles, givptr, givcol, ldgcol, perm, givnum, c, s, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlalsd "F_FUNC(zlalsd,ZLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, npy_complex128 *b, int *ldb, d *rcond, int *rank, npy_complex128 *work, d *rwork, int *iwork, int *info) nogil
 * cdef void zlalsd(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, z *b, int *ldb, d *rcond, int *rank, z *work, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlalsd(uplo, smlsiz, n, nrhs, d, e, <npy_complex128*>b, ldb, rcond, rank, <npy_complex128*>work, rwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlalsd(char *__pyx_v_uplo, int *__pyx_v_smlsiz, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, int *__pyx_v_rank, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7222
 *     void _fortran_zlalsd "F_FUNC(zlalsd,ZLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, npy_complex128 *b, int *ldb, d *rcond, int *rank, npy_complex128 *work, d *rwork, int *iwork, int *info) nogil
 * cdef void zlalsd(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, z *b, int *ldb, d *rcond, int *rank, z *work, d *rwork, int *iwork, int *info) nogil:
 *     _fortran_zlalsd(uplo, smlsiz, n, nrhs, d, e, <npy_complex128*>b, ldb, rcond, rank, <npy_complex128*>work, rwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlalsd,ZLALSD)(__pyx_v_uplo, __pyx_v_smlsiz, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_rcond, __pyx_v_rank, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7221
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlalsd "F_FUNC(zlalsd,ZLALSD)"(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, npy_complex128 *b, int *ldb, d *rcond, int *rank, npy_complex128 *work, d *rwork, int *iwork, int *info) nogil
 * cdef void zlalsd(char *uplo, int *smlsiz, int *n, int *nrhs, d *d, d *e, z *b, int *ldb, d *rcond, int *rank, z *work, d *rwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlalsd(uplo, smlsiz, n, nrhs, d, e, <npy_complex128*>b, ldb, rcond, rank, <npy_complex128*>work, rwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlapll "F_FUNC(zlapll,ZLAPLL)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *ssmin) nogil
 * cdef void zlapll(int *n, z *x, int *incx, z *y, int *incy, d *ssmin) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlapll(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, ssmin)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlapll(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ssmin) {

  /* "scipy/linalg/cython_lapack.pyx":7227
 *     void _fortran_zlapll "F_FUNC(zlapll,ZLAPLL)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *ssmin) nogil
 * cdef void zlapll(int *n, z *x, int *incx, z *y, int *incy, d *ssmin) nogil:
 *     _fortran_zlapll(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, ssmin)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlapll,ZLAPLL)(__pyx_v_n, ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, ((npy_complex128 *)__pyx_v_y), __pyx_v_incy, __pyx_v_ssmin);

  /* "scipy/linalg/cython_lapack.pyx":7226
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlapll "F_FUNC(zlapll,ZLAPLL)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *ssmin) nogil
 * cdef void zlapll(int *n, z *x, int *incx, z *y, int *incy, d *ssmin) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlapll(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, ssmin)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlapmt "F_FUNC(zlapmt,ZLAPMT)"(bint *forwrd, int *m, int *n, npy_complex128 *x, int *ldx, int *k) nogil
 * cdef void zlapmt(bint *forwrd, int *m, int *n, z *x, int *ldx, int *k) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlapmt(forwrd, m, n, <npy_complex128*>x, ldx, k)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlapmt(int *__pyx_v_forwrd, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, int *__pyx_v_k) {

  /* "scipy/linalg/cython_lapack.pyx":7232
 *     void _fortran_zlapmt "F_FUNC(zlapmt,ZLAPMT)"(bint *forwrd, int *m, int *n, npy_complex128 *x, int *ldx, int *k) nogil
 * cdef void zlapmt(bint *forwrd, int *m, int *n, z *x, int *ldx, int *k) nogil:
 *     _fortran_zlapmt(forwrd, m, n, <npy_complex128*>x, ldx, k)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlapmt,ZLAPMT)(__pyx_v_forwrd, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_k);

  /* "scipy/linalg/cython_lapack.pyx":7231
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlapmt "F_FUNC(zlapmt,ZLAPMT)"(bint *forwrd, int *m, int *n, npy_complex128 *x, int *ldx, int *k) nogil
 * cdef void zlapmt(bint *forwrd, int *m, int *n, z *x, int *ldx, int *k) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlapmt(forwrd, m, n, <npy_complex128*>x, ldx, k)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqgb "F_FUNC(zlaqgb,ZLAQGB)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void zlaqgb(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqgb(m, n, kl, ku, <npy_complex128*>ab, ldab, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqgb(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":7237
 *     void _fortran_zlaqgb "F_FUNC(zlaqgb,ZLAQGB)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void zlaqgb(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:
 *     _fortran_zlaqgb(m, n, kl, ku, <npy_complex128*>ab, ldab, r, c, rowcnd, colcnd, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqgb,ZLAQGB)(__pyx_v_m, __pyx_v_n, __pyx_v_kl, __pyx_v_ku, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":7236
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqgb "F_FUNC(zlaqgb,ZLAQGB)"(int *m, int *n, int *kl, int *ku, npy_complex128 *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void zlaqgb(int *m, int *n, int *kl, int *ku, z *ab, int *ldab, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqgb(m, n, kl, ku, <npy_complex128*>ab, ldab, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqge "F_FUNC(zlaqge,ZLAQGE)"(int *m, int *n, npy_complex128 *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void zlaqge(int *m, int *n, z *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqge(m, n, <npy_complex128*>a, lda, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqge(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_r, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rowcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_colcnd, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":7242
 *     void _fortran_zlaqge "F_FUNC(zlaqge,ZLAQGE)"(int *m, int *n, npy_complex128 *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void zlaqge(int *m, int *n, z *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:
 *     _fortran_zlaqge(m, n, <npy_complex128*>a, lda, r, c, rowcnd, colcnd, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqge,ZLAQGE)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_r, __pyx_v_c, __pyx_v_rowcnd, __pyx_v_colcnd, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":7241
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqge "F_FUNC(zlaqge,ZLAQGE)"(int *m, int *n, npy_complex128 *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil
 * cdef void zlaqge(int *m, int *n, z *a, int *lda, d *r, d *c, d *rowcnd, d *colcnd, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqge(m, n, <npy_complex128*>a, lda, r, c, rowcnd, colcnd, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqhb "F_FUNC(zlaqhb,ZLAQHB)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhb(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqhb(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqhb(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":7247
 *     void _fortran_zlaqhb "F_FUNC(zlaqhb,ZLAQHB)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhb(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_zlaqhb(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqhb,ZLAQHB)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":7246
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqhb "F_FUNC(zlaqhb,ZLAQHB)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhb(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqhb(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqhe "F_FUNC(zlaqhe,ZLAQHE)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhe(char *uplo, int *n, z *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqhe(uplo, n, <npy_complex128*>a, lda, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqhe(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":7252
 *     void _fortran_zlaqhe "F_FUNC(zlaqhe,ZLAQHE)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhe(char *uplo, int *n, z *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_zlaqhe(uplo, n, <npy_complex128*>a, lda, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqhe,ZLAQHE)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":7251
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqhe "F_FUNC(zlaqhe,ZLAQHE)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhe(char *uplo, int *n, z *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqhe(uplo, n, <npy_complex128*>a, lda, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqhp "F_FUNC(zlaqhp,ZLAQHP)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhp(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqhp(uplo, n, <npy_complex128*>ap, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqhp(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":7257
 *     void _fortran_zlaqhp "F_FUNC(zlaqhp,ZLAQHP)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhp(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_zlaqhp(uplo, n, <npy_complex128*>ap, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqhp,ZLAQHP)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":7256
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqhp "F_FUNC(zlaqhp,ZLAQHP)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqhp(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqhp(uplo, n, <npy_complex128*>ap, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqp2 "F_FUNC(zlaqp2,ZLAQP2)"(int *m, int *n, int *offset, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, d *vn1, d *vn2, npy_complex128 *work) nogil
 * cdef void zlaqp2(int *m, int *n, int *offset, z *a, int *lda, int *jpvt, z *tau, d *vn1, d *vn2, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqp2(m, n, offset, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, vn1, vn2, <npy_complex128*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqp2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_offset, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vn1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vn2, __pyx_t_double_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":7262
 *     void _fortran_zlaqp2 "F_FUNC(zlaqp2,ZLAQP2)"(int *m, int *n, int *offset, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, d *vn1, d *vn2, npy_complex128 *work) nogil
 * cdef void zlaqp2(int *m, int *n, int *offset, z *a, int *lda, int *jpvt, z *tau, d *vn1, d *vn2, z *work) nogil:
 *     _fortran_zlaqp2(m, n, offset, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, vn1, vn2, <npy_complex128*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqp2,ZLAQP2)(__pyx_v_m, __pyx_v_n, __pyx_v_offset, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_jpvt, ((npy_complex128 *)__pyx_v_tau), __pyx_v_vn1, __pyx_v_vn2, ((npy_complex128 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":7261
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqp2 "F_FUNC(zlaqp2,ZLAQP2)"(int *m, int *n, int *offset, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, d *vn1, d *vn2, npy_complex128 *work) nogil
 * cdef void zlaqp2(int *m, int *n, int *offset, z *a, int *lda, int *jpvt, z *tau, d *vn1, d *vn2, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqp2(m, n, offset, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, vn1, vn2, <npy_complex128*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqps "F_FUNC(zlaqps,ZLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, d *vn1, d *vn2, npy_complex128 *auxv, npy_complex128 *f, int *ldf) nogil
 * cdef void zlaqps(int *m, int *n, int *offset, int *nb, int *kb, z *a, int *lda, int *jpvt, z *tau, d *vn1, d *vn2, z *auxv, z *f, int *ldf) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqps(m, n, offset, nb, kb, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, vn1, vn2, <npy_complex128*>auxv, <npy_complex128*>f, ldf)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqps(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_offset, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_jpvt, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vn1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vn2, __pyx_t_double_complex *__pyx_v_auxv, __pyx_t_double_complex *__pyx_v_f, int *__pyx_v_ldf) {

  /* "scipy/linalg/cython_lapack.pyx":7267
 *     void _fortran_zlaqps "F_FUNC(zlaqps,ZLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, d *vn1, d *vn2, npy_complex128 *auxv, npy_complex128 *f, int *ldf) nogil
 * cdef void zlaqps(int *m, int *n, int *offset, int *nb, int *kb, z *a, int *lda, int *jpvt, z *tau, d *vn1, d *vn2, z *auxv, z *f, int *ldf) nogil:
 *     _fortran_zlaqps(m, n, offset, nb, kb, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, vn1, vn2, <npy_complex128*>auxv, <npy_complex128*>f, ldf)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqps,ZLAQPS)(__pyx_v_m, __pyx_v_n, __pyx_v_offset, __pyx_v_nb, __pyx_v_kb, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_jpvt, ((npy_complex128 *)__pyx_v_tau), __pyx_v_vn1, __pyx_v_vn2, ((npy_complex128 *)__pyx_v_auxv), ((npy_complex128 *)__pyx_v_f), __pyx_v_ldf);

  /* "scipy/linalg/cython_lapack.pyx":7266
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqps "F_FUNC(zlaqps,ZLAQPS)"(int *m, int *n, int *offset, int *nb, int *kb, npy_complex128 *a, int *lda, int *jpvt, npy_complex128 *tau, d *vn1, d *vn2, npy_complex128 *auxv, npy_complex128 *f, int *ldf) nogil
 * cdef void zlaqps(int *m, int *n, int *offset, int *nb, int *kb, z *a, int *lda, int *jpvt, z *tau, d *vn1, d *vn2, z *auxv, z *f, int *ldf) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqps(m, n, offset, nb, kb, <npy_complex128*>a, lda, jpvt, <npy_complex128*>tau, vn1, vn2, <npy_complex128*>auxv, <npy_complex128*>f, ldf)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr0 "F_FUNC(zlaqr0,ZLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zlaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr0(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqr0(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_double_complex *__pyx_v_w, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7272
 *     void _fortran_zlaqr0 "F_FUNC(zlaqr0,ZLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zlaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, z *work, int *lwork, int *info) nogil:
 *     _fortran_zlaqr0(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqr0,ZLAQR0)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex128 *)__pyx_v_w), __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7271
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr0 "F_FUNC(zlaqr0,ZLAQR0)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zlaqr0(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr0(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr1 "F_FUNC(zlaqr1,ZLAQR1)"(int *n, npy_complex128 *h, int *ldh, npy_complex128 *s1, npy_complex128 *s2, npy_complex128 *v) nogil
 * cdef void zlaqr1(int *n, z *h, int *ldh, z *s1, z *s2, z *v) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr1(n, <npy_complex128*>h, ldh, <npy_complex128*>s1, <npy_complex128*>s2, <npy_complex128*>v)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqr1(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_double_complex *__pyx_v_s1, __pyx_t_double_complex *__pyx_v_s2, __pyx_t_double_complex *__pyx_v_v) {

  /* "scipy/linalg/cython_lapack.pyx":7277
 *     void _fortran_zlaqr1 "F_FUNC(zlaqr1,ZLAQR1)"(int *n, npy_complex128 *h, int *ldh, npy_complex128 *s1, npy_complex128 *s2, npy_complex128 *v) nogil
 * cdef void zlaqr1(int *n, z *h, int *ldh, z *s1, z *s2, z *v) nogil:
 *     _fortran_zlaqr1(n, <npy_complex128*>h, ldh, <npy_complex128*>s1, <npy_complex128*>s2, <npy_complex128*>v)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqr1,ZLAQR1)(__pyx_v_n, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex128 *)__pyx_v_s1), ((npy_complex128 *)__pyx_v_s2), ((npy_complex128 *)__pyx_v_v));

  /* "scipy/linalg/cython_lapack.pyx":7276
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr1 "F_FUNC(zlaqr1,ZLAQR1)"(int *n, npy_complex128 *h, int *ldh, npy_complex128 *s1, npy_complex128 *s2, npy_complex128 *v) nogil
 * cdef void zlaqr1(int *n, z *h, int *ldh, z *s1, z *s2, z *v) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr1(n, <npy_complex128*>h, ldh, <npy_complex128*>s1, <npy_complex128*>s2, <npy_complex128*>v)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr2 "F_FUNC(zlaqr2,ZLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *ns, int *nd, npy_complex128 *sh, npy_complex128 *v, int *ldv, int *nh, npy_complex128 *t, int *ldt, int *nv, npy_complex128 *wv, int *ldwv, npy_complex128 *work, int *lwork) nogil
 * cdef void zlaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, int *ns, int *nd, z *sh, z *v, int *ldv, int *nh, z *t, int *ldt, int *nv, z *wv, int *ldwv, z *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr2(wantt, wantz, n, ktop, kbot, nw, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, ns, nd, <npy_complex128*>sh, <npy_complex128*>v, ldv, nh, <npy_complex128*>t, ldt, nv, <npy_complex128*>wv, ldwv, <npy_complex128*>work, lwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqr2(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nw, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ns, int *__pyx_v_nd, __pyx_t_double_complex *__pyx_v_sh, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_nh, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, int *__pyx_v_nv, __pyx_t_double_complex *__pyx_v_wv, int *__pyx_v_ldwv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork) {

  /* "scipy/linalg/cython_lapack.pyx":7282
 *     void _fortran_zlaqr2 "F_FUNC(zlaqr2,ZLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *ns, int *nd, npy_complex128 *sh, npy_complex128 *v, int *ldv, int *nh, npy_complex128 *t, int *ldt, int *nv, npy_complex128 *wv, int *ldwv, npy_complex128 *work, int *lwork) nogil
 * cdef void zlaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, int *ns, int *nd, z *sh, z *v, int *ldv, int *nh, z *t, int *ldt, int *nv, z *wv, int *ldwv, z *work, int *lwork) nogil:
 *     _fortran_zlaqr2(wantt, wantz, n, ktop, kbot, nw, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, ns, nd, <npy_complex128*>sh, <npy_complex128*>v, ldv, nh, <npy_complex128*>t, ldt, nv, <npy_complex128*>wv, ldwv, <npy_complex128*>work, lwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqr2,ZLAQR2)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nw, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_ns, __pyx_v_nd, ((npy_complex128 *)__pyx_v_sh), ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, __pyx_v_nh, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, __pyx_v_nv, ((npy_complex128 *)__pyx_v_wv), __pyx_v_ldwv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork);

  /* "scipy/linalg/cython_lapack.pyx":7281
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr2 "F_FUNC(zlaqr2,ZLAQR2)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *ns, int *nd, npy_complex128 *sh, npy_complex128 *v, int *ldv, int *nh, npy_complex128 *t, int *ldt, int *nv, npy_complex128 *wv, int *ldwv, npy_complex128 *work, int *lwork) nogil
 * cdef void zlaqr2(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, int *ns, int *nd, z *sh, z *v, int *ldv, int *nh, z *t, int *ldt, int *nv, z *wv, int *ldwv, z *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr2(wantt, wantz, n, ktop, kbot, nw, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, ns, nd, <npy_complex128*>sh, <npy_complex128*>v, ldv, nh, <npy_complex128*>t, ldt, nv, <npy_complex128*>wv, ldwv, <npy_complex128*>work, lwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr3 "F_FUNC(zlaqr3,ZLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *ns, int *nd, npy_complex128 *sh, npy_complex128 *v, int *ldv, int *nh, npy_complex128 *t, int *ldt, int *nv, npy_complex128 *wv, int *ldwv, npy_complex128 *work, int *lwork) nogil
 * cdef void zlaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, int *ns, int *nd, z *sh, z *v, int *ldv, int *nh, z *t, int *ldt, int *nv, z *wv, int *ldwv, z *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr3(wantt, wantz, n, ktop, kbot, nw, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, ns, nd, <npy_complex128*>sh, <npy_complex128*>v, ldv, nh, <npy_complex128*>t, ldt, nv, <npy_complex128*>wv, ldwv, <npy_complex128*>work, lwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqr3(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nw, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ns, int *__pyx_v_nd, __pyx_t_double_complex *__pyx_v_sh, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, int *__pyx_v_nh, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, int *__pyx_v_nv, __pyx_t_double_complex *__pyx_v_wv, int *__pyx_v_ldwv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork) {

  /* "scipy/linalg/cython_lapack.pyx":7287
 *     void _fortran_zlaqr3 "F_FUNC(zlaqr3,ZLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *ns, int *nd, npy_complex128 *sh, npy_complex128 *v, int *ldv, int *nh, npy_complex128 *t, int *ldt, int *nv, npy_complex128 *wv, int *ldwv, npy_complex128 *work, int *lwork) nogil
 * cdef void zlaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, int *ns, int *nd, z *sh, z *v, int *ldv, int *nh, z *t, int *ldt, int *nv, z *wv, int *ldwv, z *work, int *lwork) nogil:
 *     _fortran_zlaqr3(wantt, wantz, n, ktop, kbot, nw, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, ns, nd, <npy_complex128*>sh, <npy_complex128*>v, ldv, nh, <npy_complex128*>t, ldt, nv, <npy_complex128*>wv, ldwv, <npy_complex128*>work, lwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqr3,ZLAQR3)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nw, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_ns, __pyx_v_nd, ((npy_complex128 *)__pyx_v_sh), ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, __pyx_v_nh, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, __pyx_v_nv, ((npy_complex128 *)__pyx_v_wv), __pyx_v_ldwv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork);

  /* "scipy/linalg/cython_lapack.pyx":7286
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr3 "F_FUNC(zlaqr3,ZLAQR3)"(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, int *ns, int *nd, npy_complex128 *sh, npy_complex128 *v, int *ldv, int *nh, npy_complex128 *t, int *ldt, int *nv, npy_complex128 *wv, int *ldwv, npy_complex128 *work, int *lwork) nogil
 * cdef void zlaqr3(bint *wantt, bint *wantz, int *n, int *ktop, int *kbot, int *nw, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, int *ns, int *nd, z *sh, z *v, int *ldv, int *nh, z *t, int *ldt, int *nv, z *wv, int *ldwv, z *work, int *lwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr3(wantt, wantz, n, ktop, kbot, nw, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, ns, nd, <npy_complex128*>sh, <npy_complex128*>v, ldv, nh, <npy_complex128*>t, ldt, nv, <npy_complex128*>wv, ldwv, <npy_complex128*>work, lwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr4 "F_FUNC(zlaqr4,ZLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zlaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr4(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqr4(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, __pyx_t_double_complex *__pyx_v_w, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7292
 *     void _fortran_zlaqr4 "F_FUNC(zlaqr4,ZLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zlaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, z *work, int *lwork, int *info) nogil:
 *     _fortran_zlaqr4(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqr4,ZLAQR4)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, ((npy_complex128 *)__pyx_v_w), __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7291
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr4 "F_FUNC(zlaqr4,ZLAQR4)"(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, npy_complex128 *h, int *ldh, npy_complex128 *w, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zlaqr4(bint *wantt, bint *wantz, int *n, int *ilo, int *ihi, z *h, int *ldh, z *w, int *iloz, int *ihiz, z *z, int *ldz, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr4(wantt, wantz, n, ilo, ihi, <npy_complex128*>h, ldh, <npy_complex128*>w, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr5 "F_FUNC(zlaqr5,ZLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, npy_complex128 *s, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *v, int *ldv, npy_complex128 *u, int *ldu, int *nv, npy_complex128 *wv, int *ldwv, int *nh, npy_complex128 *wh, int *ldwh) nogil
 * cdef void zlaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, z *s, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, z *v, int *ldv, z *u, int *ldu, int *nv, z *wv, int *ldwv, int *nh, z *wh, int *ldwh) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, <npy_complex128*>s, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>v, ldv, <npy_complex128*>u, ldu, nv, <npy_complex128*>wv, ldwv, nh, <npy_complex128*>wh, ldwh)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqr5(int *__pyx_v_wantt, int *__pyx_v_wantz, int *__pyx_v_kacc22, int *__pyx_v_n, int *__pyx_v_ktop, int *__pyx_v_kbot, int *__pyx_v_nshfts, __pyx_t_double_complex *__pyx_v_s, __pyx_t_double_complex *__pyx_v_h, int *__pyx_v_ldh, int *__pyx_v_iloz, int *__pyx_v_ihiz, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_ldu, int *__pyx_v_nv, __pyx_t_double_complex *__pyx_v_wv, int *__pyx_v_ldwv, int *__pyx_v_nh, __pyx_t_double_complex *__pyx_v_wh, int *__pyx_v_ldwh) {

  /* "scipy/linalg/cython_lapack.pyx":7297
 *     void _fortran_zlaqr5 "F_FUNC(zlaqr5,ZLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, npy_complex128 *s, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *v, int *ldv, npy_complex128 *u, int *ldu, int *nv, npy_complex128 *wv, int *ldwv, int *nh, npy_complex128 *wh, int *ldwh) nogil
 * cdef void zlaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, z *s, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, z *v, int *ldv, z *u, int *ldu, int *nv, z *wv, int *ldwv, int *nh, z *wh, int *ldwh) nogil:
 *     _fortran_zlaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, <npy_complex128*>s, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>v, ldv, <npy_complex128*>u, ldu, nv, <npy_complex128*>wv, ldwv, nh, <npy_complex128*>wh, ldwh)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqr5,ZLAQR5)(__pyx_v_wantt, __pyx_v_wantz, __pyx_v_kacc22, __pyx_v_n, __pyx_v_ktop, __pyx_v_kbot, __pyx_v_nshfts, ((npy_complex128 *)__pyx_v_s), ((npy_complex128 *)__pyx_v_h), __pyx_v_ldh, __pyx_v_iloz, __pyx_v_ihiz, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex128 *)__pyx_v_u), __pyx_v_ldu, __pyx_v_nv, ((npy_complex128 *)__pyx_v_wv), __pyx_v_ldwv, __pyx_v_nh, ((npy_complex128 *)__pyx_v_wh), __pyx_v_ldwh);

  /* "scipy/linalg/cython_lapack.pyx":7296
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqr5 "F_FUNC(zlaqr5,ZLAQR5)"(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, npy_complex128 *s, npy_complex128 *h, int *ldh, int *iloz, int *ihiz, npy_complex128 *z, int *ldz, npy_complex128 *v, int *ldv, npy_complex128 *u, int *ldu, int *nv, npy_complex128 *wv, int *ldwv, int *nh, npy_complex128 *wh, int *ldwh) nogil
 * cdef void zlaqr5(bint *wantt, bint *wantz, int *kacc22, int *n, int *ktop, int *kbot, int *nshfts, z *s, z *h, int *ldh, int *iloz, int *ihiz, z *z, int *ldz, z *v, int *ldv, z *u, int *ldu, int *nv, z *wv, int *ldwv, int *nh, z *wh, int *ldwh) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqr5(wantt, wantz, kacc22, n, ktop, kbot, nshfts, <npy_complex128*>s, <npy_complex128*>h, ldh, iloz, ihiz, <npy_complex128*>z, ldz, <npy_complex128*>v, ldv, <npy_complex128*>u, ldu, nv, <npy_complex128*>wv, ldwv, nh, <npy_complex128*>wh, ldwh)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqsb "F_FUNC(zlaqsb,ZLAQSB)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsb(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqsb(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqsb(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":7302
 *     void _fortran_zlaqsb "F_FUNC(zlaqsb,ZLAQSB)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsb(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_zlaqsb(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqsb,ZLAQSB)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":7301
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqsb "F_FUNC(zlaqsb,ZLAQSB)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsb(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqsb(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqsp "F_FUNC(zlaqsp,ZLAQSP)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsp(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqsp(uplo, n, <npy_complex128*>ap, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqsp(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":7307
 *     void _fortran_zlaqsp "F_FUNC(zlaqsp,ZLAQSP)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsp(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_zlaqsp(uplo, n, <npy_complex128*>ap, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqsp,ZLAQSP)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":7306
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqsp "F_FUNC(zlaqsp,ZLAQSP)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsp(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqsp(uplo, n, <npy_complex128*>ap, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqsy "F_FUNC(zlaqsy,ZLAQSY)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsy(char *uplo, int *n, z *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqsy(uplo, n, <npy_complex128*>a, lda, s, scond, amax, equed)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaqsy(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, char *__pyx_v_equed) {

  /* "scipy/linalg/cython_lapack.pyx":7312
 *     void _fortran_zlaqsy "F_FUNC(zlaqsy,ZLAQSY)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsy(char *uplo, int *n, z *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:
 *     _fortran_zlaqsy(uplo, n, <npy_complex128*>a, lda, s, scond, amax, equed)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaqsy,ZLAQSY)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_equed);

  /* "scipy/linalg/cython_lapack.pyx":7311
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaqsy "F_FUNC(zlaqsy,ZLAQSY)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil
 * cdef void zlaqsy(char *uplo, int *n, z *a, int *lda, d *s, d *scond, d *amax, char *equed) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaqsy(uplo, n, <npy_complex128*>a, lda, s, scond, amax, equed)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlar1v "F_FUNC(zlar1v,ZLAR1V)"(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, npy_complex128 *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil
 * cdef void zlar1v(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, z *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, <npy_complex128*>z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlar1v(int *__pyx_v_n, int *__pyx_v_b1, int *__pyx_v_bn, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lambda_, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ld, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_lld, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gaptol, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_wantnc, int *__pyx_v_negcnt, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ztz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_mingma, int *__pyx_v_r, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_nrminv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_resid, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rqcorr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":7317
 *     void _fortran_zlar1v "F_FUNC(zlar1v,ZLAR1V)"(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, npy_complex128 *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil
 * cdef void zlar1v(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, z *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil:
 *     _fortran_zlar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, <npy_complex128*>z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlar1v,ZLAR1V)(__pyx_v_n, __pyx_v_b1, __pyx_v_bn, __pyx_v_lambda_, __pyx_v_d, __pyx_v_l, __pyx_v_ld, __pyx_v_lld, __pyx_v_pivmin, __pyx_v_gaptol, ((npy_complex128 *)__pyx_v_z), __pyx_v_wantnc, __pyx_v_negcnt, __pyx_v_ztz, __pyx_v_mingma, __pyx_v_r, __pyx_v_isuppz, __pyx_v_nrminv, __pyx_v_resid, __pyx_v_rqcorr, __pyx_v_work);

  /* "scipy/linalg/cython_lapack.pyx":7316
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlar1v "F_FUNC(zlar1v,ZLAR1V)"(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, npy_complex128 *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil
 * cdef void zlar1v(int *n, int *b1, int *bn, d *lambda_, d *d, d *l, d *ld, d *lld, d *pivmin, d *gaptol, z *z, bint *wantnc, int *negcnt, d *ztz, d *mingma, int *r, int *isuppz, d *nrminv, d *resid, d *rqcorr, d *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlar1v(n, b1, bn, lambda_, d, l, ld, lld, pivmin, gaptol, <npy_complex128*>z, wantnc, negcnt, ztz, mingma, r, isuppz, nrminv, resid, rqcorr, work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlar2v "F_FUNC(zlar2v,ZLAR2V)"(int *n, npy_complex128 *x, npy_complex128 *y, npy_complex128 *z, int *incx, d *c, npy_complex128 *s, int *incc) nogil
 * cdef void zlar2v(int *n, z *x, z *y, z *z, int *incx, d *c, z *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlar2v(n, <npy_complex128*>x, <npy_complex128*>y, <npy_complex128*>z, incx, c, <npy_complex128*>s, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlar2v(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_x, __pyx_t_double_complex *__pyx_v_y, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_double_complex *__pyx_v_s, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":7322
 *     void _fortran_zlar2v "F_FUNC(zlar2v,ZLAR2V)"(int *n, npy_complex128 *x, npy_complex128 *y, npy_complex128 *z, int *incx, d *c, npy_complex128 *s, int *incc) nogil
 * cdef void zlar2v(int *n, z *x, z *y, z *z, int *incx, d *c, z *s, int *incc) nogil:
 *     _fortran_zlar2v(n, <npy_complex128*>x, <npy_complex128*>y, <npy_complex128*>z, incx, c, <npy_complex128*>s, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlar2v,ZLAR2V)(__pyx_v_n, ((npy_complex128 *)__pyx_v_x), ((npy_complex128 *)__pyx_v_y), ((npy_complex128 *)__pyx_v_z), __pyx_v_incx, __pyx_v_c, ((npy_complex128 *)__pyx_v_s), __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":7321
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlar2v "F_FUNC(zlar2v,ZLAR2V)"(int *n, npy_complex128 *x, npy_complex128 *y, npy_complex128 *z, int *incx, d *c, npy_complex128 *s, int *incc) nogil
 * cdef void zlar2v(int *n, z *x, z *y, z *z, int *incx, d *c, z *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlar2v(n, <npy_complex128*>x, <npy_complex128*>y, <npy_complex128*>z, incx, c, <npy_complex128*>s, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarcm "F_FUNC(zlarcm,ZLARCM)"(int *m, int *n, d *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, d *rwork) nogil
 * cdef void zlarcm(int *m, int *n, d *a, int *lda, z *b, int *ldb, z *c, int *ldc, d *rwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarcm(m, n, a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, rwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarcm(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork) {

  /* "scipy/linalg/cython_lapack.pyx":7327
 *     void _fortran_zlarcm "F_FUNC(zlarcm,ZLARCM)"(int *m, int *n, d *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, d *rwork) nogil
 * cdef void zlarcm(int *m, int *n, d *a, int *lda, z *b, int *ldb, z *c, int *ldc, d *rwork) nogil:
 *     _fortran_zlarcm(m, n, a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, rwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarcm,ZLARCM)(__pyx_v_m, __pyx_v_n, __pyx_v_a, __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, __pyx_v_rwork);

  /* "scipy/linalg/cython_lapack.pyx":7326
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarcm "F_FUNC(zlarcm,ZLARCM)"(int *m, int *n, d *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, d *rwork) nogil
 * cdef void zlarcm(int *m, int *n, d *a, int *lda, z *b, int *ldb, z *c, int *ldc, d *rwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarcm(m, n, a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, rwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarf "F_FUNC(zlarf,ZLARF)"(char *side, int *m, int *n, npy_complex128 *v, int *incv, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarf(char *side, int *m, int *n, z *v, int *incv, z *tau, z *c, int *ldc, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarf(side, m, n, <npy_complex128*>v, incv, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarf(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_incv, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":7332
 *     void _fortran_zlarf "F_FUNC(zlarf,ZLARF)"(char *side, int *m, int *n, npy_complex128 *v, int *incv, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarf(char *side, int *m, int *n, z *v, int *incv, z *tau, z *c, int *ldc, z *work) nogil:
 *     _fortran_zlarf(side, m, n, <npy_complex128*>v, incv, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarf,ZLARF)(__pyx_v_side, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_v), __pyx_v_incv, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":7331
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarf "F_FUNC(zlarf,ZLARF)"(char *side, int *m, int *n, npy_complex128 *v, int *incv, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarf(char *side, int *m, int *n, z *v, int *incv, z *tau, z *c, int *ldc, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarf(side, m, n, <npy_complex128*>v, incv, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarfb "F_FUNC(zlarfb,ZLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *t, int *ldt, npy_complex128 *c, int *ldc, npy_complex128 *work, int *ldwork) nogil
 * cdef void zlarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, z *v, int *ldv, z *t, int *ldt, z *c, int *ldc, z *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarfb(side, trans, direct, storev, m, n, k, <npy_complex128*>v, ldv, <npy_complex128*>t, ldt, <npy_complex128*>c, ldc, <npy_complex128*>work, ldwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarfb(char *__pyx_v_side, char *__pyx_v_trans, char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_ldwork) {

  /* "scipy/linalg/cython_lapack.pyx":7337
 *     void _fortran_zlarfb "F_FUNC(zlarfb,ZLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *t, int *ldt, npy_complex128 *c, int *ldc, npy_complex128 *work, int *ldwork) nogil
 * cdef void zlarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, z *v, int *ldv, z *t, int *ldt, z *c, int *ldc, z *work, int *ldwork) nogil:
 *     _fortran_zlarfb(side, trans, direct, storev, m, n, k, <npy_complex128*>v, ldv, <npy_complex128*>t, ldt, <npy_complex128*>c, ldc, <npy_complex128*>work, ldwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarfb,ZLARFB)(__pyx_v_side, __pyx_v_trans, __pyx_v_direct, __pyx_v_storev, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_ldwork);

  /* "scipy/linalg/cython_lapack.pyx":7336
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarfb "F_FUNC(zlarfb,ZLARFB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *t, int *ldt, npy_complex128 *c, int *ldc, npy_complex128 *work, int *ldwork) nogil
 * cdef void zlarfb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, z *v, int *ldv, z *t, int *ldt, z *c, int *ldc, z *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarfb(side, trans, direct, storev, m, n, k, <npy_complex128*>v, ldv, <npy_complex128*>t, ldt, <npy_complex128*>c, ldc, <npy_complex128*>work, ldwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarfg "F_FUNC(zlarfg,ZLARFG)"(int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *tau) nogil
 * cdef void zlarfg(int *n, z *alpha, z *x, int *incx, z *tau) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarfg(n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>tau)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarfg(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_tau) {

  /* "scipy/linalg/cython_lapack.pyx":7342
 *     void _fortran_zlarfg "F_FUNC(zlarfg,ZLARFG)"(int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *tau) nogil
 * cdef void zlarfg(int *n, z *alpha, z *x, int *incx, z *tau) nogil:
 *     _fortran_zlarfg(n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>tau)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarfg,ZLARFG)(__pyx_v_n, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, ((npy_complex128 *)__pyx_v_tau));

  /* "scipy/linalg/cython_lapack.pyx":7341
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarfg "F_FUNC(zlarfg,ZLARFG)"(int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *tau) nogil
 * cdef void zlarfg(int *n, z *alpha, z *x, int *incx, z *tau) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarfg(n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>tau)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarft "F_FUNC(zlarft,ZLARFT)"(char *direct, char *storev, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *tau, npy_complex128 *t, int *ldt) nogil
 * cdef void zlarft(char *direct, char *storev, int *n, int *k, z *v, int *ldv, z *tau, z *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarft(direct, storev, n, k, <npy_complex128*>v, ldv, <npy_complex128*>tau, <npy_complex128*>t, ldt)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarft(char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt) {

  /* "scipy/linalg/cython_lapack.pyx":7347
 *     void _fortran_zlarft "F_FUNC(zlarft,ZLARFT)"(char *direct, char *storev, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *tau, npy_complex128 *t, int *ldt) nogil
 * cdef void zlarft(char *direct, char *storev, int *n, int *k, z *v, int *ldv, z *tau, z *t, int *ldt) nogil:
 *     _fortran_zlarft(direct, storev, n, k, <npy_complex128*>v, ldv, <npy_complex128*>tau, <npy_complex128*>t, ldt)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarft,ZLARFT)(__pyx_v_direct, __pyx_v_storev, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt);

  /* "scipy/linalg/cython_lapack.pyx":7346
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarft "F_FUNC(zlarft,ZLARFT)"(char *direct, char *storev, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *tau, npy_complex128 *t, int *ldt) nogil
 * cdef void zlarft(char *direct, char *storev, int *n, int *k, z *v, int *ldv, z *tau, z *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarft(direct, storev, n, k, <npy_complex128*>v, ldv, <npy_complex128*>tau, <npy_complex128*>t, ldt)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarfx "F_FUNC(zlarfx,ZLARFX)"(char *side, int *m, int *n, npy_complex128 *v, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarfx(char *side, int *m, int *n, z *v, z *tau, z *c, int *ldc, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarfx(side, m, n, <npy_complex128*>v, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarfx(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_v, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":7352
 *     void _fortran_zlarfx "F_FUNC(zlarfx,ZLARFX)"(char *side, int *m, int *n, npy_complex128 *v, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarfx(char *side, int *m, int *n, z *v, z *tau, z *c, int *ldc, z *work) nogil:
 *     _fortran_zlarfx(side, m, n, <npy_complex128*>v, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarfx,ZLARFX)(__pyx_v_side, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_v), ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":7351
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarfx "F_FUNC(zlarfx,ZLARFX)"(char *side, int *m, int *n, npy_complex128 *v, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarfx(char *side, int *m, int *n, z *v, z *tau, z *c, int *ldc, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarfx(side, m, n, <npy_complex128*>v, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlargv "F_FUNC(zlargv,ZLARGV)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *c, int *incc) nogil
 * cdef void zlargv(int *n, z *x, int *incx, z *y, int *incy, d *c, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlargv(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, c, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlargv(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":7357
 *     void _fortran_zlargv "F_FUNC(zlargv,ZLARGV)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *c, int *incc) nogil
 * cdef void zlargv(int *n, z *x, int *incx, z *y, int *incy, d *c, int *incc) nogil:
 *     _fortran_zlargv(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, c, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlargv,ZLARGV)(__pyx_v_n, ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, ((npy_complex128 *)__pyx_v_y), __pyx_v_incy, __pyx_v_c, __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":7356
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlargv "F_FUNC(zlargv,ZLARGV)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *c, int *incc) nogil
 * cdef void zlargv(int *n, z *x, int *incx, z *y, int *incy, d *c, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlargv(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, c, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarnv "F_FUNC(zlarnv,ZLARNV)"(int *idist, int *iseed, int *n, npy_complex128 *x) nogil
 * cdef void zlarnv(int *idist, int *iseed, int *n, z *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarnv(idist, iseed, n, <npy_complex128*>x)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarnv(int *__pyx_v_idist, int *__pyx_v_iseed, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_x) {

  /* "scipy/linalg/cython_lapack.pyx":7362
 *     void _fortran_zlarnv "F_FUNC(zlarnv,ZLARNV)"(int *idist, int *iseed, int *n, npy_complex128 *x) nogil
 * cdef void zlarnv(int *idist, int *iseed, int *n, z *x) nogil:
 *     _fortran_zlarnv(idist, iseed, n, <npy_complex128*>x)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarnv,ZLARNV)(__pyx_v_idist, __pyx_v_iseed, __pyx_v_n, ((npy_complex128 *)__pyx_v_x));

  /* "scipy/linalg/cython_lapack.pyx":7361
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarnv "F_FUNC(zlarnv,ZLARNV)"(int *idist, int *iseed, int *n, npy_complex128 *x) nogil
 * cdef void zlarnv(int *idist, int *iseed, int *n, z *x) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarnv(idist, iseed, n, <npy_complex128*>x)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarrv "F_FUNC(zlarrv,ZLARRV)"(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, npy_complex128 *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil
 * cdef void zlarrv(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, z *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, <npy_complex128*>z, ldz, isuppz, work, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarrv(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_l, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pivmin, int *__pyx_v_isplit, int *__pyx_v_m, int *__pyx_v_dol, int *__pyx_v_dou, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_minrgp, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol1, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rtol2, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_werr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_wgap, int *__pyx_v_iblock, int *__pyx_v_indexw, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_gers, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7367
 *     void _fortran_zlarrv "F_FUNC(zlarrv,ZLARRV)"(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, npy_complex128 *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil
 * cdef void zlarrv(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, z *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil:
 *     _fortran_zlarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, <npy_complex128*>z, ldz, isuppz, work, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarrv,ZLARRV)(__pyx_v_n, __pyx_v_vl, __pyx_v_vu, __pyx_v_d, __pyx_v_l, __pyx_v_pivmin, __pyx_v_isplit, __pyx_v_m, __pyx_v_dol, __pyx_v_dou, __pyx_v_minrgp, __pyx_v_rtol1, __pyx_v_rtol2, __pyx_v_w, __pyx_v_werr, __pyx_v_wgap, __pyx_v_iblock, __pyx_v_indexw, __pyx_v_gers, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7366
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarrv "F_FUNC(zlarrv,ZLARRV)"(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, npy_complex128 *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil
 * cdef void zlarrv(int *n, d *vl, d *vu, d *d, d *l, d *pivmin, int *isplit, int *m, int *dol, int *dou, d *minrgp, d *rtol1, d *rtol2, d *w, d *werr, d *wgap, int *iblock, int *indexw, d *gers, z *z, int *ldz, int *isuppz, d *work, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarrv(n, vl, vu, d, l, pivmin, isplit, m, dol, dou, minrgp, rtol1, rtol2, w, werr, wgap, iblock, indexw, gers, <npy_complex128*>z, ldz, isuppz, work, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlartg "F_FUNC(zlartg,ZLARTG)"(npy_complex128 *f, npy_complex128 *g, d *cs, npy_complex128 *sn, npy_complex128 *r) nogil
 * cdef void zlartg(z *f, z *g, d *cs, z *sn, z *r) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlartg(<npy_complex128*>f, <npy_complex128*>g, cs, <npy_complex128*>sn, <npy_complex128*>r)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlartg(__pyx_t_double_complex *__pyx_v_f, __pyx_t_double_complex *__pyx_v_g, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cs, __pyx_t_double_complex *__pyx_v_sn, __pyx_t_double_complex *__pyx_v_r) {

  /* "scipy/linalg/cython_lapack.pyx":7372
 *     void _fortran_zlartg "F_FUNC(zlartg,ZLARTG)"(npy_complex128 *f, npy_complex128 *g, d *cs, npy_complex128 *sn, npy_complex128 *r) nogil
 * cdef void zlartg(z *f, z *g, d *cs, z *sn, z *r) nogil:
 *     _fortran_zlartg(<npy_complex128*>f, <npy_complex128*>g, cs, <npy_complex128*>sn, <npy_complex128*>r)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlartg,ZLARTG)(((npy_complex128 *)__pyx_v_f), ((npy_complex128 *)__pyx_v_g), __pyx_v_cs, ((npy_complex128 *)__pyx_v_sn), ((npy_complex128 *)__pyx_v_r));

  /* "scipy/linalg/cython_lapack.pyx":7371
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlartg "F_FUNC(zlartg,ZLARTG)"(npy_complex128 *f, npy_complex128 *g, d *cs, npy_complex128 *sn, npy_complex128 *r) nogil
 * cdef void zlartg(z *f, z *g, d *cs, z *sn, z *r) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlartg(<npy_complex128*>f, <npy_complex128*>g, cs, <npy_complex128*>sn, <npy_complex128*>r)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlartv "F_FUNC(zlartv,ZLARTV)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *c, npy_complex128 *s, int *incc) nogil
 * cdef void zlartv(int *n, z *x, int *incx, z *y, int *incy, d *c, z *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlartv(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, c, <npy_complex128*>s, incc)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlartv(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_y, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_double_complex *__pyx_v_s, int *__pyx_v_incc) {

  /* "scipy/linalg/cython_lapack.pyx":7377
 *     void _fortran_zlartv "F_FUNC(zlartv,ZLARTV)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *c, npy_complex128 *s, int *incc) nogil
 * cdef void zlartv(int *n, z *x, int *incx, z *y, int *incy, d *c, z *s, int *incc) nogil:
 *     _fortran_zlartv(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, c, <npy_complex128*>s, incc)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlartv,ZLARTV)(__pyx_v_n, ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, ((npy_complex128 *)__pyx_v_y), __pyx_v_incy, __pyx_v_c, ((npy_complex128 *)__pyx_v_s), __pyx_v_incc);

  /* "scipy/linalg/cython_lapack.pyx":7376
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlartv "F_FUNC(zlartv,ZLARTV)"(int *n, npy_complex128 *x, int *incx, npy_complex128 *y, int *incy, d *c, npy_complex128 *s, int *incc) nogil
 * cdef void zlartv(int *n, z *x, int *incx, z *y, int *incy, d *c, z *s, int *incc) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlartv(n, <npy_complex128*>x, incx, <npy_complex128*>y, incy, c, <npy_complex128*>s, incc)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarz "F_FUNC(zlarz,ZLARZ)"(char *side, int *m, int *n, int *l, npy_complex128 *v, int *incv, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarz(char *side, int *m, int *n, int *l, z *v, int *incv, z *tau, z *c, int *ldc, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarz(side, m, n, l, <npy_complex128*>v, incv, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarz(char *__pyx_v_side, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_l, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_incv, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":7382
 *     void _fortran_zlarz "F_FUNC(zlarz,ZLARZ)"(char *side, int *m, int *n, int *l, npy_complex128 *v, int *incv, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarz(char *side, int *m, int *n, int *l, z *v, int *incv, z *tau, z *c, int *ldc, z *work) nogil:
 *     _fortran_zlarz(side, m, n, l, <npy_complex128*>v, incv, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarz,ZLARZ)(__pyx_v_side, __pyx_v_m, __pyx_v_n, __pyx_v_l, ((npy_complex128 *)__pyx_v_v), __pyx_v_incv, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":7381
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarz "F_FUNC(zlarz,ZLARZ)"(char *side, int *m, int *n, int *l, npy_complex128 *v, int *incv, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work) nogil
 * cdef void zlarz(char *side, int *m, int *n, int *l, z *v, int *incv, z *tau, z *c, int *ldc, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarz(side, m, n, l, <npy_complex128*>v, incv, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarzb "F_FUNC(zlarzb,ZLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, npy_complex128 *v, int *ldv, npy_complex128 *t, int *ldt, npy_complex128 *c, int *ldc, npy_complex128 *work, int *ldwork) nogil
 * cdef void zlarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, z *v, int *ldv, z *t, int *ldt, z *c, int *ldc, z *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarzb(side, trans, direct, storev, m, n, k, l, <npy_complex128*>v, ldv, <npy_complex128*>t, ldt, <npy_complex128*>c, ldc, <npy_complex128*>work, ldwork)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarzb(char *__pyx_v_side, char *__pyx_v_trans, char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_ldwork) {

  /* "scipy/linalg/cython_lapack.pyx":7387
 *     void _fortran_zlarzb "F_FUNC(zlarzb,ZLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, npy_complex128 *v, int *ldv, npy_complex128 *t, int *ldt, npy_complex128 *c, int *ldc, npy_complex128 *work, int *ldwork) nogil
 * cdef void zlarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, z *v, int *ldv, z *t, int *ldt, z *c, int *ldc, z *work, int *ldwork) nogil:
 *     _fortran_zlarzb(side, trans, direct, storev, m, n, k, l, <npy_complex128*>v, ldv, <npy_complex128*>t, ldt, <npy_complex128*>c, ldc, <npy_complex128*>work, ldwork)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarzb,ZLARZB)(__pyx_v_side, __pyx_v_trans, __pyx_v_direct, __pyx_v_storev, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_ldwork);

  /* "scipy/linalg/cython_lapack.pyx":7386
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarzb "F_FUNC(zlarzb,ZLARZB)"(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, npy_complex128 *v, int *ldv, npy_complex128 *t, int *ldt, npy_complex128 *c, int *ldc, npy_complex128 *work, int *ldwork) nogil
 * cdef void zlarzb(char *side, char *trans, char *direct, char *storev, int *m, int *n, int *k, int *l, z *v, int *ldv, z *t, int *ldt, z *c, int *ldc, z *work, int *ldwork) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarzb(side, trans, direct, storev, m, n, k, l, <npy_complex128*>v, ldv, <npy_complex128*>t, ldt, <npy_complex128*>c, ldc, <npy_complex128*>work, ldwork)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarzt "F_FUNC(zlarzt,ZLARZT)"(char *direct, char *storev, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *tau, npy_complex128 *t, int *ldt) nogil
 * cdef void zlarzt(char *direct, char *storev, int *n, int *k, z *v, int *ldv, z *tau, z *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarzt(direct, storev, n, k, <npy_complex128*>v, ldv, <npy_complex128*>tau, <npy_complex128*>t, ldt)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlarzt(char *__pyx_v_direct, char *__pyx_v_storev, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt) {

  /* "scipy/linalg/cython_lapack.pyx":7392
 *     void _fortran_zlarzt "F_FUNC(zlarzt,ZLARZT)"(char *direct, char *storev, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *tau, npy_complex128 *t, int *ldt) nogil
 * cdef void zlarzt(char *direct, char *storev, int *n, int *k, z *v, int *ldv, z *tau, z *t, int *ldt) nogil:
 *     _fortran_zlarzt(direct, storev, n, k, <npy_complex128*>v, ldv, <npy_complex128*>tau, <npy_complex128*>t, ldt)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlarzt,ZLARZT)(__pyx_v_direct, __pyx_v_storev, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt);

  /* "scipy/linalg/cython_lapack.pyx":7391
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlarzt "F_FUNC(zlarzt,ZLARZT)"(char *direct, char *storev, int *n, int *k, npy_complex128 *v, int *ldv, npy_complex128 *tau, npy_complex128 *t, int *ldt) nogil
 * cdef void zlarzt(char *direct, char *storev, int *n, int *k, z *v, int *ldv, z *tau, z *t, int *ldt) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlarzt(direct, storev, n, k, <npy_complex128*>v, ldv, <npy_complex128*>tau, <npy_complex128*>t, ldt)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlascl "F_FUNC(zlascl,ZLASCL)"(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlascl(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlascl(type_bn, kl, ku, cfrom, cto, m, n, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlascl(char *__pyx_v_type_bn, int *__pyx_v_kl, int *__pyx_v_ku, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cfrom, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cto, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7397
 *     void _fortran_zlascl "F_FUNC(zlascl,ZLASCL)"(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlascl(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, z *a, int *lda, int *info) nogil:
 *     _fortran_zlascl(type_bn, kl, ku, cfrom, cto, m, n, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlascl,ZLASCL)(__pyx_v_type_bn, __pyx_v_kl, __pyx_v_ku, __pyx_v_cfrom, __pyx_v_cto, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7396
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlascl "F_FUNC(zlascl,ZLASCL)"(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlascl(char *type_bn, int *kl, int *ku, d *cfrom, d *cto, int *m, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlascl(type_bn, kl, ku, cfrom, cto, m, n, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaset "F_FUNC(zlaset,ZLASET)"(char *uplo, int *m, int *n, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *a, int *lda) nogil
 * cdef void zlaset(char *uplo, int *m, int *n, z *alpha, z *beta, z *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaset(uplo, m, n, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaset(char *__pyx_v_uplo, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":7402
 *     void _fortran_zlaset "F_FUNC(zlaset,ZLASET)"(char *uplo, int *m, int *n, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *a, int *lda) nogil
 * cdef void zlaset(char *uplo, int *m, int *n, z *alpha, z *beta, z *a, int *lda) nogil:
 *     _fortran_zlaset(uplo, m, n, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaset,ZLASET)(__pyx_v_uplo, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_a), __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":7401
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaset "F_FUNC(zlaset,ZLASET)"(char *uplo, int *m, int *n, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *a, int *lda) nogil
 * cdef void zlaset(char *uplo, int *m, int *n, z *alpha, z *beta, z *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaset(uplo, m, n, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlasr "F_FUNC(zlasr,ZLASR)"(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, npy_complex128 *a, int *lda) nogil
 * cdef void zlasr(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, z *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlasr(side, pivot, direct, m, n, c, s, <npy_complex128*>a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlasr(char *__pyx_v_side, char *__pyx_v_pivot, char *__pyx_v_direct, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":7407
 *     void _fortran_zlasr "F_FUNC(zlasr,ZLASR)"(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, npy_complex128 *a, int *lda) nogil
 * cdef void zlasr(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, z *a, int *lda) nogil:
 *     _fortran_zlasr(side, pivot, direct, m, n, c, s, <npy_complex128*>a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlasr,ZLASR)(__pyx_v_side, __pyx_v_pivot, __pyx_v_direct, __pyx_v_m, __pyx_v_n, __pyx_v_c, __pyx_v_s, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":7406
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlasr "F_FUNC(zlasr,ZLASR)"(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, npy_complex128 *a, int *lda) nogil
 * cdef void zlasr(char *side, char *pivot, char *direct, int *m, int *n, d *c, d *s, z *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlasr(side, pivot, direct, m, n, c, s, <npy_complex128*>a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlassq "F_FUNC(zlassq,ZLASSQ)"(int *n, npy_complex128 *x, int *incx, d *scale, d *sumsq) nogil
 * cdef void zlassq(int *n, z *x, int *incx, d *scale, d *sumsq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlassq(n, <npy_complex128*>x, incx, scale, sumsq)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlassq(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sumsq) {

  /* "scipy/linalg/cython_lapack.pyx":7412
 *     void _fortran_zlassq "F_FUNC(zlassq,ZLASSQ)"(int *n, npy_complex128 *x, int *incx, d *scale, d *sumsq) nogil
 * cdef void zlassq(int *n, z *x, int *incx, d *scale, d *sumsq) nogil:
 *     _fortran_zlassq(n, <npy_complex128*>x, incx, scale, sumsq)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlassq,ZLASSQ)(__pyx_v_n, ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, __pyx_v_scale, __pyx_v_sumsq);

  /* "scipy/linalg/cython_lapack.pyx":7411
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlassq "F_FUNC(zlassq,ZLASSQ)"(int *n, npy_complex128 *x, int *incx, d *scale, d *sumsq) nogil
 * cdef void zlassq(int *n, z *x, int *incx, d *scale, d *sumsq) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlassq(n, <npy_complex128*>x, incx, scale, sumsq)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaswp "F_FUNC(zlaswp,ZLASWP)"(int *n, npy_complex128 *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void zlaswp(int *n, z *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaswp(n, <npy_complex128*>a, lda, k1, k2, ipiv, incx)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlaswp(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_k1, int *__pyx_v_k2, int *__pyx_v_ipiv, int *__pyx_v_incx) {

  /* "scipy/linalg/cython_lapack.pyx":7417
 *     void _fortran_zlaswp "F_FUNC(zlaswp,ZLASWP)"(int *n, npy_complex128 *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void zlaswp(int *n, z *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:
 *     _fortran_zlaswp(n, <npy_complex128*>a, lda, k1, k2, ipiv, incx)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlaswp,ZLASWP)(__pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_k1, __pyx_v_k2, __pyx_v_ipiv, __pyx_v_incx);

  /* "scipy/linalg/cython_lapack.pyx":7416
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlaswp "F_FUNC(zlaswp,ZLASWP)"(int *n, npy_complex128 *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil
 * cdef void zlaswp(int *n, z *a, int *lda, int *k1, int *k2, int *ipiv, int *incx) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlaswp(n, <npy_complex128*>a, lda, k1, k2, ipiv, incx)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlasyf "F_FUNC(zlasyf,ZLASYF)"(char *uplo, int *n, int *nb, int *kb, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *w, int *ldw, int *info) nogil
 * cdef void zlasyf(char *uplo, int *n, int *nb, int *kb, z *a, int *lda, int *ipiv, z *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlasyf(uplo, n, nb, kb, <npy_complex128*>a, lda, ipiv, <npy_complex128*>w, ldw, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlasyf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, int *__pyx_v_kb, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_w, int *__pyx_v_ldw, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7422
 *     void _fortran_zlasyf "F_FUNC(zlasyf,ZLASYF)"(char *uplo, int *n, int *nb, int *kb, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *w, int *ldw, int *info) nogil
 * cdef void zlasyf(char *uplo, int *n, int *nb, int *kb, z *a, int *lda, int *ipiv, z *w, int *ldw, int *info) nogil:
 *     _fortran_zlasyf(uplo, n, nb, kb, <npy_complex128*>a, lda, ipiv, <npy_complex128*>w, ldw, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlasyf,ZLASYF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, __pyx_v_kb, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_w), __pyx_v_ldw, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7421
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlasyf "F_FUNC(zlasyf,ZLASYF)"(char *uplo, int *n, int *nb, int *kb, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *w, int *ldw, int *info) nogil
 * cdef void zlasyf(char *uplo, int *n, int *nb, int *kb, z *a, int *lda, int *ipiv, z *w, int *ldw, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlasyf(uplo, n, nb, kb, <npy_complex128*>a, lda, ipiv, <npy_complex128*>w, ldw, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatbs "F_FUNC(zlatbs,ZLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, npy_complex128 *ab, int *ldab, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, z *ab, int *ldab, z *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatbs(uplo, trans, diag, normin, n, kd, <npy_complex128*>ab, ldab, <npy_complex128*>x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlatbs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7427
 *     void _fortran_zlatbs "F_FUNC(zlatbs,ZLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, npy_complex128 *ab, int *ldab, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, z *ab, int *ldab, z *x, d *scale, d *cnorm, int *info) nogil:
 *     _fortran_zlatbs(uplo, trans, diag, normin, n, kd, <npy_complex128*>ab, ldab, <npy_complex128*>x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlatbs,ZLATBS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_x), __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7426
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatbs "F_FUNC(zlatbs,ZLATBS)"(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, npy_complex128 *ab, int *ldab, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatbs(char *uplo, char *trans, char *diag, char *normin, int *n, int *kd, z *ab, int *ldab, z *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatbs(uplo, trans, diag, normin, n, kd, <npy_complex128*>ab, ldab, <npy_complex128*>x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatdf "F_FUNC(zlatdf,ZLATDF)"(int *ijob, int *n, npy_complex128 *z, int *ldz, npy_complex128 *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void zlatdf(int *ijob, int *n, z *z, int *ldz, z *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatdf(ijob, n, <npy_complex128*>z, ldz, <npy_complex128*>rhs, rdsum, rdscal, ipiv, jpiv)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlatdf(int *__pyx_v_ijob, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_rhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rdsum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rdscal, int *__pyx_v_ipiv, int *__pyx_v_jpiv) {

  /* "scipy/linalg/cython_lapack.pyx":7432
 *     void _fortran_zlatdf "F_FUNC(zlatdf,ZLATDF)"(int *ijob, int *n, npy_complex128 *z, int *ldz, npy_complex128 *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void zlatdf(int *ijob, int *n, z *z, int *ldz, z *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil:
 *     _fortran_zlatdf(ijob, n, <npy_complex128*>z, ldz, <npy_complex128*>rhs, rdsum, rdscal, ipiv, jpiv)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlatdf,ZLATDF)(__pyx_v_ijob, __pyx_v_n, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_rhs), __pyx_v_rdsum, __pyx_v_rdscal, __pyx_v_ipiv, __pyx_v_jpiv);

  /* "scipy/linalg/cython_lapack.pyx":7431
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatdf "F_FUNC(zlatdf,ZLATDF)"(int *ijob, int *n, npy_complex128 *z, int *ldz, npy_complex128 *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil
 * cdef void zlatdf(int *ijob, int *n, z *z, int *ldz, z *rhs, d *rdsum, d *rdscal, int *ipiv, int *jpiv) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatdf(ijob, n, <npy_complex128*>z, ldz, <npy_complex128*>rhs, rdsum, rdscal, ipiv, jpiv)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatps "F_FUNC(zlatps,ZLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex128 *ap, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatps(char *uplo, char *trans, char *diag, char *normin, int *n, z *ap, z *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatps(uplo, trans, diag, normin, n, <npy_complex128*>ap, <npy_complex128*>x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlatps(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7437
 *     void _fortran_zlatps "F_FUNC(zlatps,ZLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex128 *ap, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatps(char *uplo, char *trans, char *diag, char *normin, int *n, z *ap, z *x, d *scale, d *cnorm, int *info) nogil:
 *     _fortran_zlatps(uplo, trans, diag, normin, n, <npy_complex128*>ap, <npy_complex128*>x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlatps,ZLATPS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_x), __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7436
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatps "F_FUNC(zlatps,ZLATPS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex128 *ap, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatps(char *uplo, char *trans, char *diag, char *normin, int *n, z *ap, z *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatps(uplo, trans, diag, normin, n, <npy_complex128*>ap, <npy_complex128*>x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatrd "F_FUNC(zlatrd,ZLATRD)"(char *uplo, int *n, int *nb, npy_complex128 *a, int *lda, d *e, npy_complex128 *tau, npy_complex128 *w, int *ldw) nogil
 * cdef void zlatrd(char *uplo, int *n, int *nb, z *a, int *lda, d *e, z *tau, z *w, int *ldw) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatrd(uplo, n, nb, <npy_complex128*>a, lda, e, <npy_complex128*>tau, <npy_complex128*>w, ldw)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlatrd(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nb, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_w, int *__pyx_v_ldw) {

  /* "scipy/linalg/cython_lapack.pyx":7442
 *     void _fortran_zlatrd "F_FUNC(zlatrd,ZLATRD)"(char *uplo, int *n, int *nb, npy_complex128 *a, int *lda, d *e, npy_complex128 *tau, npy_complex128 *w, int *ldw) nogil
 * cdef void zlatrd(char *uplo, int *n, int *nb, z *a, int *lda, d *e, z *tau, z *w, int *ldw) nogil:
 *     _fortran_zlatrd(uplo, n, nb, <npy_complex128*>a, lda, e, <npy_complex128*>tau, <npy_complex128*>w, ldw)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlatrd,ZLATRD)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nb, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_e, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_w), __pyx_v_ldw);

  /* "scipy/linalg/cython_lapack.pyx":7441
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatrd "F_FUNC(zlatrd,ZLATRD)"(char *uplo, int *n, int *nb, npy_complex128 *a, int *lda, d *e, npy_complex128 *tau, npy_complex128 *w, int *ldw) nogil
 * cdef void zlatrd(char *uplo, int *n, int *nb, z *a, int *lda, d *e, z *tau, z *w, int *ldw) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatrd(uplo, n, nb, <npy_complex128*>a, lda, e, <npy_complex128*>tau, <npy_complex128*>w, ldw)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatrs "F_FUNC(zlatrs,ZLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex128 *a, int *lda, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatrs(char *uplo, char *trans, char *diag, char *normin, int *n, z *a, int *lda, z *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatrs(uplo, trans, diag, normin, n, <npy_complex128*>a, lda, <npy_complex128*>x, scale, cnorm, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlatrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, char *__pyx_v_normin, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_x, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_cnorm, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7447
 *     void _fortran_zlatrs "F_FUNC(zlatrs,ZLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex128 *a, int *lda, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatrs(char *uplo, char *trans, char *diag, char *normin, int *n, z *a, int *lda, z *x, d *scale, d *cnorm, int *info) nogil:
 *     _fortran_zlatrs(uplo, trans, diag, normin, n, <npy_complex128*>a, lda, <npy_complex128*>x, scale, cnorm, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlatrs,ZLATRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_normin, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_x), __pyx_v_scale, __pyx_v_cnorm, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7446
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatrs "F_FUNC(zlatrs,ZLATRS)"(char *uplo, char *trans, char *diag, char *normin, int *n, npy_complex128 *a, int *lda, npy_complex128 *x, d *scale, d *cnorm, int *info) nogil
 * cdef void zlatrs(char *uplo, char *trans, char *diag, char *normin, int *n, z *a, int *lda, z *x, d *scale, d *cnorm, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatrs(uplo, trans, diag, normin, n, <npy_complex128*>a, lda, <npy_complex128*>x, scale, cnorm, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatrz "F_FUNC(zlatrz,ZLATRZ)"(int *m, int *n, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work) nogil
 * cdef void zlatrz(int *m, int *n, int *l, z *a, int *lda, z *tau, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatrz(m, n, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlatrz(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_l, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work) {

  /* "scipy/linalg/cython_lapack.pyx":7452
 *     void _fortran_zlatrz "F_FUNC(zlatrz,ZLATRZ)"(int *m, int *n, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work) nogil
 * cdef void zlatrz(int *m, int *n, int *l, z *a, int *lda, z *tau, z *work) nogil:
 *     _fortran_zlatrz(m, n, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlatrz,ZLATRZ)(__pyx_v_m, __pyx_v_n, __pyx_v_l, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work));

  /* "scipy/linalg/cython_lapack.pyx":7451
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlatrz "F_FUNC(zlatrz,ZLATRZ)"(int *m, int *n, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work) nogil
 * cdef void zlatrz(int *m, int *n, int *l, z *a, int *lda, z *tau, z *work) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlatrz(m, n, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlauu2 "F_FUNC(zlauu2,ZLAUU2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlauu2(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlauu2(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlauu2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7457
 *     void _fortran_zlauu2 "F_FUNC(zlauu2,ZLAUU2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlauu2(char *uplo, int *n, z *a, int *lda, int *info) nogil:
 *     _fortran_zlauu2(uplo, n, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlauu2,ZLAUU2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7456
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlauu2 "F_FUNC(zlauu2,ZLAUU2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlauu2(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlauu2(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlauum "F_FUNC(zlauum,ZLAUUM)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlauum(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlauum(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zlauum(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7462
 *     void _fortran_zlauum "F_FUNC(zlauum,ZLAUUM)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlauum(char *uplo, int *n, z *a, int *lda, int *info) nogil:
 *     _fortran_zlauum(uplo, n, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zlauum,ZLAUUM)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7461
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zlauum "F_FUNC(zlauum,ZLAUUM)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zlauum(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zlauum(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbcon "F_FUNC(zpbcon,ZPBCON)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbcon(char *uplo, int *n, int *kd, z *ab, int *ldab, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbcon(uplo, n, kd, <npy_complex128*>ab, ldab, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbcon(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7467
 *     void _fortran_zpbcon "F_FUNC(zpbcon,ZPBCON)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbcon(char *uplo, int *n, int *kd, z *ab, int *ldab, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:
 *     _fortran_zpbcon(uplo, n, kd, <npy_complex128*>ab, ldab, anorm, rcond, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbcon,ZPBCON)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7466
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbcon "F_FUNC(zpbcon,ZPBCON)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbcon(char *uplo, int *n, int *kd, z *ab, int *ldab, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbcon(uplo, n, kd, <npy_complex128*>ab, ldab, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbequ "F_FUNC(zpbequ,ZPBEQU)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zpbequ(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbequ(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbequ(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7472
 *     void _fortran_zpbequ "F_FUNC(zpbequ,ZPBEQU)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zpbequ(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil:
 *     _fortran_zpbequ(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbequ,ZPBEQU)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7471
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbequ "F_FUNC(zpbequ,ZPBEQU)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zpbequ(char *uplo, int *n, int *kd, z *ab, int *ldab, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbequ(uplo, n, kd, <npy_complex128*>ab, ldab, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbrfs "F_FUNC(zpbrfs,ZPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbrfs(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbrfs(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_afb, int *__pyx_v_ldafb, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7477
 *     void _fortran_zpbrfs "F_FUNC(zpbrfs,ZPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbrfs(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zpbrfs(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbrfs,ZPBRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_afb), __pyx_v_ldafb, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7476
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbrfs "F_FUNC(zpbrfs,ZPBRFS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbrfs(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbrfs(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbstf "F_FUNC(zpbstf,ZPBSTF)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbstf(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbstf(uplo, n, kd, <npy_complex128*>ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbstf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7482
 *     void _fortran_zpbstf "F_FUNC(zpbstf,ZPBSTF)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbstf(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:
 *     _fortran_zpbstf(uplo, n, kd, <npy_complex128*>ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbstf,ZPBSTF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7481
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbstf "F_FUNC(zpbstf,ZPBSTF)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbstf(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbstf(uplo, n, kd, <npy_complex128*>ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbsv "F_FUNC(zpbsv,ZPBSV)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpbsv(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbsv(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7487
 *     void _fortran_zpbsv "F_FUNC(zpbsv,ZPBSV)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpbsv(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:
 *     _fortran_zpbsv(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbsv,ZPBSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7486
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbsv "F_FUNC(zpbsv,ZPBSV)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpbsv(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbsv(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbsvx "F_FUNC(zpbsvx,ZPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbsvx(fact, uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_afb, int *__pyx_v_ldafb, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7492
 *     void _fortran_zpbsvx "F_FUNC(zpbsvx,ZPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zpbsvx(fact, uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbsvx,ZPBSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_afb), __pyx_v_ldafb, __pyx_v_equed, __pyx_v_s, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7491
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbsvx "F_FUNC(zpbsvx,ZPBSVX)"(char *fact, char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *afb, int *ldafb, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpbsvx(char *fact, char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *afb, int *ldafb, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbsvx(fact, uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>afb, ldafb, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbtf2 "F_FUNC(zpbtf2,ZPBTF2)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbtf2(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbtf2(uplo, n, kd, <npy_complex128*>ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbtf2(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7497
 *     void _fortran_zpbtf2 "F_FUNC(zpbtf2,ZPBTF2)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbtf2(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:
 *     _fortran_zpbtf2(uplo, n, kd, <npy_complex128*>ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbtf2,ZPBTF2)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7496
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbtf2 "F_FUNC(zpbtf2,ZPBTF2)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbtf2(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbtf2(uplo, n, kd, <npy_complex128*>ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbtrf "F_FUNC(zpbtrf,ZPBTRF)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbtrf(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbtrf(uplo, n, kd, <npy_complex128*>ab, ldab, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbtrf(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7502
 *     void _fortran_zpbtrf "F_FUNC(zpbtrf,ZPBTRF)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbtrf(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:
 *     _fortran_zpbtrf(uplo, n, kd, <npy_complex128*>ab, ldab, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbtrf,ZPBTRF)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7501
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbtrf "F_FUNC(zpbtrf,ZPBTRF)"(char *uplo, int *n, int *kd, npy_complex128 *ab, int *ldab, int *info) nogil
 * cdef void zpbtrf(char *uplo, int *n, int *kd, z *ab, int *ldab, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbtrf(uplo, n, kd, <npy_complex128*>ab, ldab, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbtrs "F_FUNC(zpbtrs,ZPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpbtrs(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbtrs(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpbtrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7507
 *     void _fortran_zpbtrs "F_FUNC(zpbtrs,ZPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpbtrs(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:
 *     _fortran_zpbtrs(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpbtrs,ZPBTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7506
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpbtrs "F_FUNC(zpbtrs,ZPBTRS)"(char *uplo, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpbtrs(char *uplo, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpbtrs(uplo, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpocon "F_FUNC(zpocon,ZPOCON)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpocon(char *uplo, int *n, z *a, int *lda, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpocon(uplo, n, <npy_complex128*>a, lda, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpocon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7512
 *     void _fortran_zpocon "F_FUNC(zpocon,ZPOCON)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpocon(char *uplo, int *n, z *a, int *lda, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:
 *     _fortran_zpocon(uplo, n, <npy_complex128*>a, lda, anorm, rcond, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpocon,ZPOCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7511
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpocon "F_FUNC(zpocon,ZPOCON)"(char *uplo, int *n, npy_complex128 *a, int *lda, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpocon(char *uplo, int *n, z *a, int *lda, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpocon(uplo, n, <npy_complex128*>a, lda, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpoequ "F_FUNC(zpoequ,ZPOEQU)"(int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zpoequ(int *n, z *a, int *lda, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpoequ(n, <npy_complex128*>a, lda, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpoequ(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7517
 *     void _fortran_zpoequ "F_FUNC(zpoequ,ZPOEQU)"(int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zpoequ(int *n, z *a, int *lda, d *s, d *scond, d *amax, int *info) nogil:
 *     _fortran_zpoequ(n, <npy_complex128*>a, lda, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpoequ,ZPOEQU)(__pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7516
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpoequ "F_FUNC(zpoequ,ZPOEQU)"(int *n, npy_complex128 *a, int *lda, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zpoequ(int *n, z *a, int *lda, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpoequ(n, <npy_complex128*>a, lda, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zporfs "F_FUNC(zporfs,ZPORFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zporfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zporfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zporfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_af, int *__pyx_v_ldaf, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7522
 *     void _fortran_zporfs "F_FUNC(zporfs,ZPORFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zporfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zporfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zporfs,ZPORFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_af), __pyx_v_ldaf, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7521
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zporfs "F_FUNC(zporfs,ZPORFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zporfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zporfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zposv "F_FUNC(zposv,ZPOSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zposv(char *uplo, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zposv(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zposv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7527
 *     void _fortran_zposv "F_FUNC(zposv,ZPOSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zposv(char *uplo, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:
 *     _fortran_zposv(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zposv,ZPOSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7526
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zposv "F_FUNC(zposv,ZPOSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zposv(char *uplo, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zposv(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zposvx "F_FUNC(zposvx,ZPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zposvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zposvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zposvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_af, int *__pyx_v_ldaf, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7532
 *     void _fortran_zposvx "F_FUNC(zposvx,ZPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zposvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zposvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zposvx,ZPOSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_equed, __pyx_v_s, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7531
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zposvx "F_FUNC(zposvx,ZPOSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zposvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zposvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpotf2 "F_FUNC(zpotf2,ZPOTF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotf2(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpotf2(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpotf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7537
 *     void _fortran_zpotf2 "F_FUNC(zpotf2,ZPOTF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotf2(char *uplo, int *n, z *a, int *lda, int *info) nogil:
 *     _fortran_zpotf2(uplo, n, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpotf2,ZPOTF2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7536
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpotf2 "F_FUNC(zpotf2,ZPOTF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotf2(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpotf2(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpotrf "F_FUNC(zpotrf,ZPOTRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotrf(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpotrf(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpotrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7542
 *     void _fortran_zpotrf "F_FUNC(zpotrf,ZPOTRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotrf(char *uplo, int *n, z *a, int *lda, int *info) nogil:
 *     _fortran_zpotrf(uplo, n, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpotrf,ZPOTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7541
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpotrf "F_FUNC(zpotrf,ZPOTRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotrf(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpotrf(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpotri "F_FUNC(zpotri,ZPOTRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotri(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpotri(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpotri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7547
 *     void _fortran_zpotri "F_FUNC(zpotri,ZPOTRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotri(char *uplo, int *n, z *a, int *lda, int *info) nogil:
 *     _fortran_zpotri(uplo, n, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpotri,ZPOTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7546
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpotri "F_FUNC(zpotri,ZPOTRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void zpotri(char *uplo, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpotri(uplo, n, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpotrs "F_FUNC(zpotrs,ZPOTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpotrs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpotrs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpotrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7552
 *     void _fortran_zpotrs "F_FUNC(zpotrs,ZPOTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpotrs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:
 *     _fortran_zpotrs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpotrs,ZPOTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7551
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpotrs "F_FUNC(zpotrs,ZPOTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpotrs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpotrs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zppcon "F_FUNC(zppcon,ZPPCON)"(char *uplo, int *n, npy_complex128 *ap, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zppcon(char *uplo, int *n, z *ap, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zppcon(uplo, n, <npy_complex128*>ap, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zppcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7557
 *     void _fortran_zppcon "F_FUNC(zppcon,ZPPCON)"(char *uplo, int *n, npy_complex128 *ap, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zppcon(char *uplo, int *n, z *ap, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:
 *     _fortran_zppcon(uplo, n, <npy_complex128*>ap, anorm, rcond, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zppcon,ZPPCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7556
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zppcon "F_FUNC(zppcon,ZPPCON)"(char *uplo, int *n, npy_complex128 *ap, d *anorm, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zppcon(char *uplo, int *n, z *ap, d *anorm, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zppcon(uplo, n, <npy_complex128*>ap, anorm, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zppequ "F_FUNC(zppequ,ZPPEQU)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zppequ(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zppequ(uplo, n, <npy_complex128*>ap, s, scond, amax, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zppequ(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_amax, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7562
 *     void _fortran_zppequ "F_FUNC(zppequ,ZPPEQU)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zppequ(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, int *info) nogil:
 *     _fortran_zppequ(uplo, n, <npy_complex128*>ap, s, scond, amax, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zppequ,ZPPEQU)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_s, __pyx_v_scond, __pyx_v_amax, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7561
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zppequ "F_FUNC(zppequ,ZPPEQU)"(char *uplo, int *n, npy_complex128 *ap, d *s, d *scond, d *amax, int *info) nogil
 * cdef void zppequ(char *uplo, int *n, z *ap, d *s, d *scond, d *amax, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zppequ(uplo, n, <npy_complex128*>ap, s, scond, amax, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpprfs "F_FUNC(zpprfs,ZPPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_afp, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7567
 *     void _fortran_zpprfs "F_FUNC(zpprfs,ZPPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zpprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpprfs,ZPPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_afp), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7566
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpprfs "F_FUNC(zpprfs,ZPPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zpprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zppsv "F_FUNC(zppsv,ZPPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zppsv(char *uplo, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zppsv(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zppsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7572
 *     void _fortran_zppsv "F_FUNC(zppsv,ZPPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zppsv(char *uplo, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:
 *     _fortran_zppsv(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zppsv,ZPPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7571
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zppsv "F_FUNC(zppsv,ZPPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zppsv(char *uplo, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zppsv(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zppsvx "F_FUNC(zppsvx,ZPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zppsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zppsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zppsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_afp, char *__pyx_v_equed, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7577
 *     void _fortran_zppsvx "F_FUNC(zppsvx,ZPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zppsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zppsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zppsvx,ZPPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_afp), __pyx_v_equed, __pyx_v_s, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7576
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zppsvx "F_FUNC(zppsvx,ZPPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, char *equed, d *s, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zppsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, char *equed, d *s, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zppsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, equed, s, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpptrf "F_FUNC(zpptrf,ZPPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void zpptrf(char *uplo, int *n, z *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpptrf(uplo, n, <npy_complex128*>ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7582
 *     void _fortran_zpptrf "F_FUNC(zpptrf,ZPPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void zpptrf(char *uplo, int *n, z *ap, int *info) nogil:
 *     _fortran_zpptrf(uplo, n, <npy_complex128*>ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpptrf,ZPPTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7581
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpptrf "F_FUNC(zpptrf,ZPPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void zpptrf(char *uplo, int *n, z *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpptrf(uplo, n, <npy_complex128*>ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpptri "F_FUNC(zpptri,ZPPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void zpptri(char *uplo, int *n, z *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpptri(uplo, n, <npy_complex128*>ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7587
 *     void _fortran_zpptri "F_FUNC(zpptri,ZPPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void zpptri(char *uplo, int *n, z *ap, int *info) nogil:
 *     _fortran_zpptri(uplo, n, <npy_complex128*>ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpptri,ZPPTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7586
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpptri "F_FUNC(zpptri,ZPPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void zpptri(char *uplo, int *n, z *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpptri(uplo, n, <npy_complex128*>ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpptrs "F_FUNC(zpptrs,ZPPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpptrs(char *uplo, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpptrs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7592
 *     void _fortran_zpptrs "F_FUNC(zpptrs,ZPPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpptrs(char *uplo, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:
 *     _fortran_zpptrs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpptrs,ZPPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7591
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpptrs "F_FUNC(zpptrs,ZPPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpptrs(char *uplo, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpptrs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptcon "F_FUNC(zptcon,ZPTCON)"(int *n, d *d, npy_complex128 *e, d *anorm, d *rcond, d *rwork, int *info) nogil
 * cdef void zptcon(int *n, d *d, z *e, d *anorm, d *rcond, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptcon(n, d, <npy_complex128*>e, anorm, rcond, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zptcon(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_double_complex *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7597
 *     void _fortran_zptcon "F_FUNC(zptcon,ZPTCON)"(int *n, d *d, npy_complex128 *e, d *anorm, d *rcond, d *rwork, int *info) nogil
 * cdef void zptcon(int *n, d *d, z *e, d *anorm, d *rcond, d *rwork, int *info) nogil:
 *     _fortran_zptcon(n, d, <npy_complex128*>e, anorm, rcond, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zptcon,ZPTCON)(__pyx_v_n, __pyx_v_d, ((npy_complex128 *)__pyx_v_e), __pyx_v_anorm, __pyx_v_rcond, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7596
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptcon "F_FUNC(zptcon,ZPTCON)"(int *n, d *d, npy_complex128 *e, d *anorm, d *rcond, d *rwork, int *info) nogil
 * cdef void zptcon(int *n, d *d, z *e, d *anorm, d *rcond, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptcon(n, d, <npy_complex128*>e, anorm, rcond, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpteqr "F_FUNC(zpteqr,ZPTEQR)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, d *work, int *info) nogil
 * cdef void zpteqr(char *compz, int *n, d *d, d *e, z *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpteqr(compz, n, d, e, <npy_complex128*>z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpteqr(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7602
 *     void _fortran_zpteqr "F_FUNC(zpteqr,ZPTEQR)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, d *work, int *info) nogil
 * cdef void zpteqr(char *compz, int *n, d *d, d *e, z *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_zpteqr(compz, n, d, e, <npy_complex128*>z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpteqr,ZPTEQR)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7601
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpteqr "F_FUNC(zpteqr,ZPTEQR)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, d *work, int *info) nogil
 * cdef void zpteqr(char *compz, int *n, d *d, d *e, z *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpteqr(compz, n, d, e, <npy_complex128*>z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptrfs "F_FUNC(zptrfs,ZPTRFS)"(char *uplo, int *n, int *nrhs, d *d, npy_complex128 *e, d *df, npy_complex128 *ef, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zptrfs(char *uplo, int *n, int *nrhs, d *d, z *e, d *df, z *ef, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptrfs(uplo, n, nrhs, d, <npy_complex128*>e, df, <npy_complex128*>ef, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zptrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_double_complex *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_df, __pyx_t_double_complex *__pyx_v_ef, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7607
 *     void _fortran_zptrfs "F_FUNC(zptrfs,ZPTRFS)"(char *uplo, int *n, int *nrhs, d *d, npy_complex128 *e, d *df, npy_complex128 *ef, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zptrfs(char *uplo, int *n, int *nrhs, d *d, z *e, d *df, z *ef, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zptrfs(uplo, n, nrhs, d, <npy_complex128*>e, df, <npy_complex128*>ef, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zptrfs,ZPTRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex128 *)__pyx_v_e), __pyx_v_df, ((npy_complex128 *)__pyx_v_ef), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7606
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptrfs "F_FUNC(zptrfs,ZPTRFS)"(char *uplo, int *n, int *nrhs, d *d, npy_complex128 *e, d *df, npy_complex128 *ef, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zptrfs(char *uplo, int *n, int *nrhs, d *d, z *e, d *df, z *ef, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptrfs(uplo, n, nrhs, d, <npy_complex128*>e, df, <npy_complex128*>ef, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptsv "F_FUNC(zptsv,ZPTSV)"(int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zptsv(int *n, int *nrhs, d *d, z *e, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptsv(n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zptsv(int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_double_complex *__pyx_v_e, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7612
 *     void _fortran_zptsv "F_FUNC(zptsv,ZPTSV)"(int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zptsv(int *n, int *nrhs, d *d, z *e, z *b, int *ldb, int *info) nogil:
 *     _fortran_zptsv(n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zptsv,ZPTSV)(__pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex128 *)__pyx_v_e), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7611
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptsv "F_FUNC(zptsv,ZPTSV)"(int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zptsv(int *n, int *nrhs, d *d, z *e, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptsv(n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptsvx "F_FUNC(zptsvx,ZPTSVX)"(char *fact, int *n, int *nrhs, d *d, npy_complex128 *e, d *df, npy_complex128 *ef, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zptsvx(char *fact, int *n, int *nrhs, d *d, z *e, d *df, z *ef, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptsvx(fact, n, nrhs, d, <npy_complex128*>e, df, <npy_complex128*>ef, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zptsvx(char *__pyx_v_fact, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_double_complex *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_df, __pyx_t_double_complex *__pyx_v_ef, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7617
 *     void _fortran_zptsvx "F_FUNC(zptsvx,ZPTSVX)"(char *fact, int *n, int *nrhs, d *d, npy_complex128 *e, d *df, npy_complex128 *ef, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zptsvx(char *fact, int *n, int *nrhs, d *d, z *e, d *df, z *ef, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zptsvx(fact, n, nrhs, d, <npy_complex128*>e, df, <npy_complex128*>ef, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zptsvx,ZPTSVX)(__pyx_v_fact, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex128 *)__pyx_v_e), __pyx_v_df, ((npy_complex128 *)__pyx_v_ef), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7616
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptsvx "F_FUNC(zptsvx,ZPTSVX)"(char *fact, int *n, int *nrhs, d *d, npy_complex128 *e, d *df, npy_complex128 *ef, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zptsvx(char *fact, int *n, int *nrhs, d *d, z *e, d *df, z *ef, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptsvx(fact, n, nrhs, d, <npy_complex128*>e, df, <npy_complex128*>ef, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpttrf "F_FUNC(zpttrf,ZPTTRF)"(int *n, d *d, npy_complex128 *e, int *info) nogil
 * cdef void zpttrf(int *n, d *d, z *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpttrf(n, d, <npy_complex128*>e, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpttrf(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_double_complex *__pyx_v_e, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7622
 *     void _fortran_zpttrf "F_FUNC(zpttrf,ZPTTRF)"(int *n, d *d, npy_complex128 *e, int *info) nogil
 * cdef void zpttrf(int *n, d *d, z *e, int *info) nogil:
 *     _fortran_zpttrf(n, d, <npy_complex128*>e, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpttrf,ZPTTRF)(__pyx_v_n, __pyx_v_d, ((npy_complex128 *)__pyx_v_e), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7621
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpttrf "F_FUNC(zpttrf,ZPTTRF)"(int *n, d *d, npy_complex128 *e, int *info) nogil
 * cdef void zpttrf(int *n, d *d, z *e, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpttrf(n, d, <npy_complex128*>e, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpttrs "F_FUNC(zpttrs,ZPTTRS)"(char *uplo, int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpttrs(char *uplo, int *n, int *nrhs, d *d, z *e, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpttrs(uplo, n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zpttrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_double_complex *__pyx_v_e, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7627
 *     void _fortran_zpttrs "F_FUNC(zpttrs,ZPTTRS)"(char *uplo, int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpttrs(char *uplo, int *n, int *nrhs, d *d, z *e, z *b, int *ldb, int *info) nogil:
 *     _fortran_zpttrs(uplo, n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zpttrs,ZPTTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex128 *)__pyx_v_e), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7626
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zpttrs "F_FUNC(zpttrs,ZPTTRS)"(char *uplo, int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zpttrs(char *uplo, int *n, int *nrhs, d *d, z *e, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zpttrs(uplo, n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptts2 "F_FUNC(zptts2,ZPTTS2)"(int *iuplo, int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb) nogil
 * cdef void zptts2(int *iuplo, int *n, int *nrhs, d *d, z *e, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptts2(iuplo, n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zptts2(int *__pyx_v_iuplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_double_complex *__pyx_v_e, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb) {

  /* "scipy/linalg/cython_lapack.pyx":7632
 *     void _fortran_zptts2 "F_FUNC(zptts2,ZPTTS2)"(int *iuplo, int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb) nogil
 * cdef void zptts2(int *iuplo, int *n, int *nrhs, d *d, z *e, z *b, int *ldb) nogil:
 *     _fortran_zptts2(iuplo, n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zptts2,ZPTTS2)(__pyx_v_iuplo, __pyx_v_n, __pyx_v_nrhs, __pyx_v_d, ((npy_complex128 *)__pyx_v_e), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb);

  /* "scipy/linalg/cython_lapack.pyx":7631
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zptts2 "F_FUNC(zptts2,ZPTTS2)"(int *iuplo, int *n, int *nrhs, d *d, npy_complex128 *e, npy_complex128 *b, int *ldb) nogil
 * cdef void zptts2(int *iuplo, int *n, int *nrhs, d *d, z *e, z *b, int *ldb) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zptts2(iuplo, n, nrhs, d, <npy_complex128*>e, <npy_complex128*>b, ldb)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zrot "F_FUNC(zrot,ZROT)"(int *n, npy_complex128 *cx, int *incx, npy_complex128 *cy, int *incy, d *c, npy_complex128 *s) nogil
 * cdef void zrot(int *n, z *cx, int *incx, z *cy, int *incy, d *c, z *s) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zrot(n, <npy_complex128*>cx, incx, <npy_complex128*>cy, incy, c, <npy_complex128*>s)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zrot(int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_cx, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_cy, int *__pyx_v_incy, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_c, __pyx_t_double_complex *__pyx_v_s) {

  /* "scipy/linalg/cython_lapack.pyx":7637
 *     void _fortran_zrot "F_FUNC(zrot,ZROT)"(int *n, npy_complex128 *cx, int *incx, npy_complex128 *cy, int *incy, d *c, npy_complex128 *s) nogil
 * cdef void zrot(int *n, z *cx, int *incx, z *cy, int *incy, d *c, z *s) nogil:
 *     _fortran_zrot(n, <npy_complex128*>cx, incx, <npy_complex128*>cy, incy, c, <npy_complex128*>s)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zrot,ZROT)(__pyx_v_n, ((npy_complex128 *)__pyx_v_cx), __pyx_v_incx, ((npy_complex128 *)__pyx_v_cy), __pyx_v_incy, __pyx_v_c, ((npy_complex128 *)__pyx_v_s));

  /* "scipy/linalg/cython_lapack.pyx":7636
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zrot "F_FUNC(zrot,ZROT)"(int *n, npy_complex128 *cx, int *incx, npy_complex128 *cy, int *incy, d *c, npy_complex128 *s) nogil
 * cdef void zrot(int *n, z *cx, int *incx, z *cy, int *incy, d *c, z *s) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zrot(n, <npy_complex128*>cx, incx, <npy_complex128*>cy, incy, c, <npy_complex128*>s)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspcon "F_FUNC(zspcon,ZSPCON)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zspcon(char *uplo, int *n, z *ap, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspcon(uplo, n, <npy_complex128*>ap, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zspcon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7642
 *     void _fortran_zspcon "F_FUNC(zspcon,ZSPCON)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zspcon(char *uplo, int *n, z *ap, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:
 *     _fortran_zspcon(uplo, n, <npy_complex128*>ap, ipiv, anorm, rcond, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zspcon,ZSPCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7641
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspcon "F_FUNC(zspcon,ZSPCON)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zspcon(char *uplo, int *n, z *ap, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspcon(uplo, n, <npy_complex128*>ap, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspmv "F_FUNC(zspmv,ZSPMV)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *ap, npy_complex128 *x, int *incx, npy_complex128 *beta, npy_complex128 *y, int *incy) nogil
 * cdef void zspmv(char *uplo, int *n, z *alpha, z *ap, z *x, int *incx, z *beta, z *y, int *incy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspmv(uplo, n, <npy_complex128*>alpha, <npy_complex128*>ap, <npy_complex128*>x, incx, <npy_complex128*>beta, <npy_complex128*>y, incy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zspmv(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_y, int *__pyx_v_incy) {

  /* "scipy/linalg/cython_lapack.pyx":7647
 *     void _fortran_zspmv "F_FUNC(zspmv,ZSPMV)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *ap, npy_complex128 *x, int *incx, npy_complex128 *beta, npy_complex128 *y, int *incy) nogil
 * cdef void zspmv(char *uplo, int *n, z *alpha, z *ap, z *x, int *incx, z *beta, z *y, int *incy) nogil:
 *     _fortran_zspmv(uplo, n, <npy_complex128*>alpha, <npy_complex128*>ap, <npy_complex128*>x, incx, <npy_complex128*>beta, <npy_complex128*>y, incy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zspmv,ZSPMV)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_y), __pyx_v_incy);

  /* "scipy/linalg/cython_lapack.pyx":7646
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspmv "F_FUNC(zspmv,ZSPMV)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *ap, npy_complex128 *x, int *incx, npy_complex128 *beta, npy_complex128 *y, int *incy) nogil
 * cdef void zspmv(char *uplo, int *n, z *alpha, z *ap, z *x, int *incx, z *beta, z *y, int *incy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspmv(uplo, n, <npy_complex128*>alpha, <npy_complex128*>ap, <npy_complex128*>x, incx, <npy_complex128*>beta, <npy_complex128*>y, incy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspr "F_FUNC(zspr,ZSPR)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *ap) nogil
 * cdef void zspr(char *uplo, int *n, z *alpha, z *x, int *incx, z *ap) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspr(uplo, n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>ap)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zspr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_ap) {

  /* "scipy/linalg/cython_lapack.pyx":7652
 *     void _fortran_zspr "F_FUNC(zspr,ZSPR)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *ap) nogil
 * cdef void zspr(char *uplo, int *n, z *alpha, z *x, int *incx, z *ap) nogil:
 *     _fortran_zspr(uplo, n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>ap)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zspr,ZSPR)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, ((npy_complex128 *)__pyx_v_ap));

  /* "scipy/linalg/cython_lapack.pyx":7651
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspr "F_FUNC(zspr,ZSPR)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *ap) nogil
 * cdef void zspr(char *uplo, int *n, z *alpha, z *x, int *incx, z *ap) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspr(uplo, n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>ap)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsprfs "F_FUNC(zsprfs,ZSPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zsprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsprfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7657
 *     void _fortran_zsprfs "F_FUNC(zsprfs,ZSPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zsprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zsprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsprfs,ZSPRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_afp), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7656
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsprfs "F_FUNC(zsprfs,ZSPRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zsprfs(char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsprfs(uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspsv "F_FUNC(zspsv,ZSPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zspsv(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspsv(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zspsv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7662
 *     void _fortran_zspsv "F_FUNC(zspsv,ZSPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zspsv(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zspsv(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zspsv,ZSPSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7661
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspsv "F_FUNC(zspsv,ZSPSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zspsv(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspsv(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspsvx "F_FUNC(zspsvx,ZSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zspsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zspsvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_afp, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7667
 *     void _fortran_zspsvx "F_FUNC(zspsvx,ZSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zspsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zspsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zspsvx,ZSPSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_afp), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7666
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zspsvx "F_FUNC(zspsvx,ZSPSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *afp, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zspsvx(char *fact, char *uplo, int *n, int *nrhs, z *ap, z *afp, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zspsvx(fact, uplo, n, nrhs, <npy_complex128*>ap, <npy_complex128*>afp, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsptrf "F_FUNC(zsptrf,ZSPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, int *info) nogil
 * cdef void zsptrf(char *uplo, int *n, z *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsptrf(uplo, n, <npy_complex128*>ap, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsptrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7672
 *     void _fortran_zsptrf "F_FUNC(zsptrf,ZSPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, int *info) nogil
 * cdef void zsptrf(char *uplo, int *n, z *ap, int *ipiv, int *info) nogil:
 *     _fortran_zsptrf(uplo, n, <npy_complex128*>ap, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsptrf,ZSPTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7671
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsptrf "F_FUNC(zsptrf,ZSPTRF)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, int *info) nogil
 * cdef void zsptrf(char *uplo, int *n, z *ap, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsptrf(uplo, n, <npy_complex128*>ap, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsptri "F_FUNC(zsptri,ZSPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zsptri(char *uplo, int *n, z *ap, int *ipiv, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsptri(uplo, n, <npy_complex128*>ap, ipiv, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsptri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7677
 *     void _fortran_zsptri "F_FUNC(zsptri,ZSPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zsptri(char *uplo, int *n, z *ap, int *ipiv, z *work, int *info) nogil:
 *     _fortran_zsptri(uplo, n, <npy_complex128*>ap, ipiv, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsptri,ZSPTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7676
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsptri "F_FUNC(zsptri,ZSPTRI)"(char *uplo, int *n, npy_complex128 *ap, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zsptri(char *uplo, int *n, z *ap, int *ipiv, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsptri(uplo, n, <npy_complex128*>ap, ipiv, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsptrs "F_FUNC(zsptrs,ZSPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zsptrs(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsptrs(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsptrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7682
 *     void _fortran_zsptrs "F_FUNC(zsptrs,ZSPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zsptrs(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zsptrs(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsptrs,ZSPTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7681
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsptrs "F_FUNC(zsptrs,ZSPTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *ap, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zsptrs(char *uplo, int *n, int *nrhs, z *ap, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsptrs(uplo, n, nrhs, <npy_complex128*>ap, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zstedc "F_FUNC(zstedc,ZSTEDC)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstedc(char *compz, int *n, d *d, d *e, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zstedc(compz, n, d, e, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zstedc(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_lrwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7687
 *     void _fortran_zstedc "F_FUNC(zstedc,ZSTEDC)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstedc(char *compz, int *n, d *d, d *e, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zstedc(compz, n, d, e, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zstedc,ZSTEDC)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_lrwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7686
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zstedc "F_FUNC(zstedc,ZSTEDC)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, npy_complex128 *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstedc(char *compz, int *n, d *d, d *e, z *z, int *ldz, z *work, int *lwork, d *rwork, int *lrwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zstedc(compz, n, d, e, <npy_complex128*>z, ldz, <npy_complex128*>work, lwork, rwork, lrwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zstegr "F_FUNC(zstegr,ZSTEGR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstegr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zstegr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_abstol, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_isuppz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7692
 *     void _fortran_zstegr "F_FUNC(zstegr,ZSTEGR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstegr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, isuppz, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zstegr,ZSTEGR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_abstol, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_isuppz, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7691
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zstegr "F_FUNC(zstegr,ZSTEGR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, npy_complex128 *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstegr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, d *abstol, int *m, d *w, z *z, int *ldz, int *isuppz, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zstegr(jobz, range, n, d, e, vl, vu, il, iu, abstol, m, w, <npy_complex128*>z, ldz, isuppz, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zstein "F_FUNC(zstein,ZSTEIN)"(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, npy_complex128 *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void zstein(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, z *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zstein(n, d, e, m, w, iblock, isplit, <npy_complex128*>z, ldz, work, iwork, ifail, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zstein(int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, int *__pyx_v_iblock, int *__pyx_v_isplit, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_iwork, int *__pyx_v_ifail, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7697
 *     void _fortran_zstein "F_FUNC(zstein,ZSTEIN)"(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, npy_complex128 *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void zstein(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, z *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:
 *     _fortran_zstein(n, d, e, m, w, iblock, isplit, <npy_complex128*>z, ldz, work, iwork, ifail, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zstein,ZSTEIN)(__pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_m, __pyx_v_w, __pyx_v_iblock, __pyx_v_isplit, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_work, __pyx_v_iwork, __pyx_v_ifail, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7696
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zstein "F_FUNC(zstein,ZSTEIN)"(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, npy_complex128 *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil
 * cdef void zstein(int *n, d *d, d *e, int *m, d *w, int *iblock, int *isplit, z *z, int *ldz, d *work, int *iwork, int *ifail, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zstein(n, d, e, m, w, iblock, isplit, <npy_complex128*>z, ldz, work, iwork, ifail, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zstemr "F_FUNC(zstemr,ZSTEMR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, npy_complex128 *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstemr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, z *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, <npy_complex128*>z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zstemr(char *__pyx_v_jobz, char *__pyx_v_range, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_vu, int *__pyx_v_il, int *__pyx_v_iu, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_w, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_nzc, int *__pyx_v_isuppz, int *__pyx_v_tryrac, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7702
 *     void _fortran_zstemr "F_FUNC(zstemr,ZSTEMR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, npy_complex128 *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstemr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, z *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_zstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, <npy_complex128*>z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zstemr,ZSTEMR)(__pyx_v_jobz, __pyx_v_range, __pyx_v_n, __pyx_v_d, __pyx_v_e, __pyx_v_vl, __pyx_v_vu, __pyx_v_il, __pyx_v_iu, __pyx_v_m, __pyx_v_w, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_nzc, __pyx_v_isuppz, __pyx_v_tryrac, __pyx_v_work, __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7701
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zstemr "F_FUNC(zstemr,ZSTEMR)"(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, npy_complex128 *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void zstemr(char *jobz, char *range, int *n, d *d, d *e, d *vl, d *vu, int *il, int *iu, int *m, d *w, z *z, int *ldz, int *nzc, int *isuppz, bint *tryrac, d *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zstemr(jobz, range, n, d, e, vl, vu, il, iu, m, w, <npy_complex128*>z, ldz, nzc, isuppz, tryrac, work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsteqr "F_FUNC(zsteqr,ZSTEQR)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, d *work, int *info) nogil
 * cdef void zsteqr(char *compz, int *n, d *d, d *e, z *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsteqr(compz, n, d, e, <npy_complex128*>z, ldz, work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsteqr(char *__pyx_v_compz, int *__pyx_v_n, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_d, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_e, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7707
 *     void _fortran_zsteqr "F_FUNC(zsteqr,ZSTEQR)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, d *work, int *info) nogil
 * cdef void zsteqr(char *compz, int *n, d *d, d *e, z *z, int *ldz, d *work, int *info) nogil:
 *     _fortran_zsteqr(compz, n, d, e, <npy_complex128*>z, ldz, work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsteqr,ZSTEQR)(__pyx_v_compz, __pyx_v_n, __pyx_v_d, __pyx_v_e, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_work, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7706
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsteqr "F_FUNC(zsteqr,ZSTEQR)"(char *compz, int *n, d *d, d *e, npy_complex128 *z, int *ldz, d *work, int *info) nogil
 * cdef void zsteqr(char *compz, int *n, d *d, d *e, z *z, int *ldz, d *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsteqr(compz, n, d, e, <npy_complex128*>z, ldz, work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsycon "F_FUNC(zsycon,ZSYCON)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zsycon(char *uplo, int *n, z *a, int *lda, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsycon(uplo, n, <npy_complex128*>a, lda, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsycon(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_anorm, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7712
 *     void _fortran_zsycon "F_FUNC(zsycon,ZSYCON)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zsycon(char *uplo, int *n, z *a, int *lda, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:
 *     _fortran_zsycon(uplo, n, <npy_complex128*>a, lda, ipiv, anorm, rcond, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsycon,ZSYCON)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_anorm, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7711
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsycon "F_FUNC(zsycon,ZSYCON)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, d *anorm, d *rcond, npy_complex128 *work, int *info) nogil
 * cdef void zsycon(char *uplo, int *n, z *a, int *lda, int *ipiv, d *anorm, d *rcond, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsycon(uplo, n, <npy_complex128*>a, lda, ipiv, anorm, rcond, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsymv "F_FUNC(zsymv,ZSYMV)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *a, int *lda, npy_complex128 *x, int *incx, npy_complex128 *beta, npy_complex128 *y, int *incy) nogil
 * cdef void zsymv(char *uplo, int *n, z *alpha, z *a, int *lda, z *x, int *incx, z *beta, z *y, int *incy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsymv(uplo, n, <npy_complex128*>alpha, <npy_complex128*>a, lda, <npy_complex128*>x, incx, <npy_complex128*>beta, <npy_complex128*>y, incy)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsymv(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_y, int *__pyx_v_incy) {

  /* "scipy/linalg/cython_lapack.pyx":7717
 *     void _fortran_zsymv "F_FUNC(zsymv,ZSYMV)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *a, int *lda, npy_complex128 *x, int *incx, npy_complex128 *beta, npy_complex128 *y, int *incy) nogil
 * cdef void zsymv(char *uplo, int *n, z *alpha, z *a, int *lda, z *x, int *incx, z *beta, z *y, int *incy) nogil:
 *     _fortran_zsymv(uplo, n, <npy_complex128*>alpha, <npy_complex128*>a, lda, <npy_complex128*>x, incx, <npy_complex128*>beta, <npy_complex128*>y, incy)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsymv,ZSYMV)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_y), __pyx_v_incy);

  /* "scipy/linalg/cython_lapack.pyx":7716
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsymv "F_FUNC(zsymv,ZSYMV)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *a, int *lda, npy_complex128 *x, int *incx, npy_complex128 *beta, npy_complex128 *y, int *incy) nogil
 * cdef void zsymv(char *uplo, int *n, z *alpha, z *a, int *lda, z *x, int *incx, z *beta, z *y, int *incy) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsymv(uplo, n, <npy_complex128*>alpha, <npy_complex128*>a, lda, <npy_complex128*>x, incx, <npy_complex128*>beta, <npy_complex128*>y, incy)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsyr "F_FUNC(zsyr,ZSYR)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *a, int *lda) nogil
 * cdef void zsyr(char *uplo, int *n, z *alpha, z *x, int *incx, z *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsyr(uplo, n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>a, lda)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsyr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_incx, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda) {

  /* "scipy/linalg/cython_lapack.pyx":7722
 *     void _fortran_zsyr "F_FUNC(zsyr,ZSYR)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *a, int *lda) nogil
 * cdef void zsyr(char *uplo, int *n, z *alpha, z *x, int *incx, z *a, int *lda) nogil:
 *     _fortran_zsyr(uplo, n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>a, lda)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsyr,ZSYR)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_x), __pyx_v_incx, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda);

  /* "scipy/linalg/cython_lapack.pyx":7721
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsyr "F_FUNC(zsyr,ZSYR)"(char *uplo, int *n, npy_complex128 *alpha, npy_complex128 *x, int *incx, npy_complex128 *a, int *lda) nogil
 * cdef void zsyr(char *uplo, int *n, z *alpha, z *x, int *incx, z *a, int *lda) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsyr(uplo, n, <npy_complex128*>alpha, <npy_complex128*>x, incx, <npy_complex128*>a, lda)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsyrfs "F_FUNC(zsyrfs,ZSYRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zsyrfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsyrfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsyrfs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7727
 *     void _fortran_zsyrfs "F_FUNC(zsyrfs,ZSYRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zsyrfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_zsyrfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsyrfs,ZSYRFS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7726
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsyrfs "F_FUNC(zsyrfs,ZSYRFS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void zsyrfs(char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsyrfs(uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsysv "F_FUNC(zsysv,ZSYSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zsysv(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsysv(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsysv(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7732
 *     void _fortran_zsysv "F_FUNC(zsysv,ZSYSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zsysv(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, z *work, int *lwork, int *info) nogil:
 *     _fortran_zsysv(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsysv,ZSYSV)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7731
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsysv "F_FUNC(zsysv,ZSYSV)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zsysv(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsysv(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsysvx "F_FUNC(zsysvx,ZSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zsysvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsysvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsysvx(char *__pyx_v_fact, char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_af, int *__pyx_v_ldaf, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7737
 *     void _fortran_zsysvx "F_FUNC(zsysvx,ZSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zsysvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, int *lwork, d *rwork, int *info) nogil:
 *     _fortran_zsysvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, lwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsysvx,ZSYSVX)(__pyx_v_fact, __pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_af), __pyx_v_ldaf, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_rcond, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7736
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsysvx "F_FUNC(zsysvx,ZSYSVX)"(char *fact, char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *af, int *ldaf, int *ipiv, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *rcond, d *ferr, d *berr, npy_complex128 *work, int *lwork, d *rwork, int *info) nogil
 * cdef void zsysvx(char *fact, char *uplo, int *n, int *nrhs, z *a, int *lda, z *af, int *ldaf, int *ipiv, z *b, int *ldb, z *x, int *ldx, d *rcond, d *ferr, d *berr, z *work, int *lwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsysvx(fact, uplo, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>af, ldaf, ipiv, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, rcond, ferr, berr, <npy_complex128*>work, lwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsytf2 "F_FUNC(zsytf2,ZSYTF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zsytf2(char *uplo, int *n, z *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsytf2(uplo, n, <npy_complex128*>a, lda, ipiv, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsytf2(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7742
 *     void _fortran_zsytf2 "F_FUNC(zsytf2,ZSYTF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zsytf2(char *uplo, int *n, z *a, int *lda, int *ipiv, int *info) nogil:
 *     _fortran_zsytf2(uplo, n, <npy_complex128*>a, lda, ipiv, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsytf2,ZSYTF2)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7741
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsytf2 "F_FUNC(zsytf2,ZSYTF2)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, int *info) nogil
 * cdef void zsytf2(char *uplo, int *n, z *a, int *lda, int *ipiv, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsytf2(uplo, n, <npy_complex128*>a, lda, ipiv, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsytrf "F_FUNC(zsytrf,ZSYTRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zsytrf(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsytrf(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsytrf(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7747
 *     void _fortran_zsytrf "F_FUNC(zsytrf,ZSYTRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zsytrf(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:
 *     _fortran_zsytrf(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsytrf,ZSYTRF)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7746
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsytrf "F_FUNC(zsytrf,ZSYTRF)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zsytrf(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsytrf(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsytri "F_FUNC(zsytri,ZSYTRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zsytri(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsytri(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsytri(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7752
 *     void _fortran_zsytri "F_FUNC(zsytri,ZSYTRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zsytri(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *info) nogil:
 *     _fortran_zsytri(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsytri,ZSYTRI)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7751
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsytri "F_FUNC(zsytri,ZSYTRI)"(char *uplo, int *n, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *work, int *info) nogil
 * cdef void zsytri(char *uplo, int *n, z *a, int *lda, int *ipiv, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsytri(uplo, n, <npy_complex128*>a, lda, ipiv, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsytrs "F_FUNC(zsytrs,ZSYTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zsytrs(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsytrs(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zsytrs(char *__pyx_v_uplo, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_ipiv, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7757
 *     void _fortran_zsytrs "F_FUNC(zsytrs,ZSYTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zsytrs(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:
 *     _fortran_zsytrs(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zsytrs,ZSYTRS)(__pyx_v_uplo, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_ipiv, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7756
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zsytrs "F_FUNC(zsytrs,ZSYTRS)"(char *uplo, int *n, int *nrhs, npy_complex128 *a, int *lda, int *ipiv, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void zsytrs(char *uplo, int *n, int *nrhs, z *a, int *lda, int *ipiv, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zsytrs(uplo, n, nrhs, <npy_complex128*>a, lda, ipiv, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztbcon "F_FUNC(ztbcon,ZTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, npy_complex128 *ab, int *ldab, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztbcon(char *norm, char *uplo, char *diag, int *n, int *kd, z *ab, int *ldab, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztbcon(norm, uplo, diag, n, kd, <npy_complex128*>ab, ldab, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztbcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7762
 *     void _fortran_ztbcon "F_FUNC(ztbcon,ZTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, npy_complex128 *ab, int *ldab, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztbcon(char *norm, char *uplo, char *diag, int *n, int *kd, z *ab, int *ldab, d *rcond, z *work, d *rwork, int *info) nogil:
 *     _fortran_ztbcon(norm, uplo, diag, n, kd, <npy_complex128*>ab, ldab, rcond, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztbcon,ZTBCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7761
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztbcon "F_FUNC(ztbcon,ZTBCON)"(char *norm, char *uplo, char *diag, int *n, int *kd, npy_complex128 *ab, int *ldab, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztbcon(char *norm, char *uplo, char *diag, int *n, int *kd, z *ab, int *ldab, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztbcon(norm, uplo, diag, n, kd, <npy_complex128*>ab, ldab, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztbrfs "F_FUNC(ztbrfs,ZTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztbrfs(uplo, trans, diag, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztbrfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7767
 *     void _fortran_ztbrfs "F_FUNC(ztbrfs,ZTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_ztbrfs(uplo, trans, diag, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztbrfs,ZTBRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7766
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztbrfs "F_FUNC(ztbrfs,ZTBRFS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztbrfs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztbrfs(uplo, trans, diag, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztbtrs "F_FUNC(ztbtrs,ZTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztbtrs(uplo, trans, diag, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztbtrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_kd, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ab, int *__pyx_v_ldab, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7772
 *     void _fortran_ztbtrs "F_FUNC(ztbtrs,ZTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:
 *     _fortran_ztbtrs(uplo, trans, diag, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztbtrs,ZTBTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_kd, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ab), __pyx_v_ldab, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7771
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztbtrs "F_FUNC(ztbtrs,ZTBTRS)"(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, npy_complex128 *ab, int *ldab, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztbtrs(char *uplo, char *trans, char *diag, int *n, int *kd, int *nrhs, z *ab, int *ldab, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztbtrs(uplo, trans, diag, n, kd, nrhs, <npy_complex128*>ab, ldab, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgevc "F_FUNC(ztgevc,ZTGEVC)"(char *side, char *howmny, bint *select, int *n, npy_complex128 *s, int *lds, npy_complex128 *p, int *ldp, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztgevc(char *side, char *howmny, bint *select, int *n, z *s, int *lds, z *p, int *ldp, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgevc(side, howmny, select, n, <npy_complex128*>s, lds, <npy_complex128*>p, ldp, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztgevc(char *__pyx_v_side, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_s, int *__pyx_v_lds, __pyx_t_double_complex *__pyx_v_p, int *__pyx_v_ldp, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7777
 *     void _fortran_ztgevc "F_FUNC(ztgevc,ZTGEVC)"(char *side, char *howmny, bint *select, int *n, npy_complex128 *s, int *lds, npy_complex128 *p, int *ldp, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztgevc(char *side, char *howmny, bint *select, int *n, z *s, int *lds, z *p, int *ldp, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *info) nogil:
 *     _fortran_ztgevc(side, howmny, select, n, <npy_complex128*>s, lds, <npy_complex128*>p, ldp, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztgevc,ZTGEVC)(__pyx_v_side, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, ((npy_complex128 *)__pyx_v_s), __pyx_v_lds, ((npy_complex128 *)__pyx_v_p), __pyx_v_ldp, ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7776
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgevc "F_FUNC(ztgevc,ZTGEVC)"(char *side, char *howmny, bint *select, int *n, npy_complex128 *s, int *lds, npy_complex128 *p, int *ldp, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztgevc(char *side, char *howmny, bint *select, int *n, z *s, int *lds, z *p, int *ldp, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgevc(side, howmny, select, n, <npy_complex128*>s, lds, <npy_complex128*>p, ldp, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgex2 "F_FUNC(ztgex2,ZTGEX2)"(bint *wantq, bint *wantz, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *j1, int *info) nogil
 * cdef void ztgex2(bint *wantq, bint *wantz, int *n, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *j1, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgex2(wantq, wantz, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, j1, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztgex2(int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_j1, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7782
 *     void _fortran_ztgex2 "F_FUNC(ztgex2,ZTGEX2)"(bint *wantq, bint *wantz, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *j1, int *info) nogil
 * cdef void ztgex2(bint *wantq, bint *wantz, int *n, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *j1, int *info) nogil:
 *     _fortran_ztgex2(wantq, wantz, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, j1, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztgex2,ZTGEX2)(__pyx_v_wantq, __pyx_v_wantz, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_j1, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7781
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgex2 "F_FUNC(ztgex2,ZTGEX2)"(bint *wantq, bint *wantz, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *j1, int *info) nogil
 * cdef void ztgex2(bint *wantq, bint *wantz, int *n, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *j1, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgex2(wantq, wantz, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, j1, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgexc "F_FUNC(ztgexc,ZTGEXC)"(bint *wantq, bint *wantz, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *ifst, int *ilst, int *info) nogil
 * cdef void ztgexc(bint *wantq, bint *wantz, int *n, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *ifst, int *ilst, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgexc(wantq, wantz, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, ifst, ilst, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztgexc(int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_ifst, int *__pyx_v_ilst, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7787
 *     void _fortran_ztgexc "F_FUNC(ztgexc,ZTGEXC)"(bint *wantq, bint *wantz, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *ifst, int *ilst, int *info) nogil
 * cdef void ztgexc(bint *wantq, bint *wantz, int *n, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *ifst, int *ilst, int *info) nogil:
 *     _fortran_ztgexc(wantq, wantz, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, ifst, ilst, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztgexc,ZTGEXC)(__pyx_v_wantq, __pyx_v_wantz, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_ifst, __pyx_v_ilst, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7786
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgexc "F_FUNC(ztgexc,ZTGEXC)"(bint *wantq, bint *wantz, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *ifst, int *ilst, int *info) nogil
 * cdef void ztgexc(bint *wantq, bint *wantz, int *n, z *a, int *lda, z *b, int *ldb, z *q, int *ldq, z *z, int *ldz, int *ifst, int *ilst, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgexc(wantq, wantz, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, ifst, ilst, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsen "F_FUNC(ztgsen,ZTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *m, d *pl, d *pr, d *dif, npy_complex128 *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ztgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *q, int *ldq, z *z, int *ldz, int *m, d *pl, d *pr, d *dif, z *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsen(ijob, wantq, wantz, select, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, m, pl, pr, dif, <npy_complex128*>work, lwork, iwork, liwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztgsen(int *__pyx_v_ijob, int *__pyx_v_wantq, int *__pyx_v_wantz, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_z, int *__pyx_v_ldz, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pl, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_pr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dif, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_liwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7792
 *     void _fortran_ztgsen "F_FUNC(ztgsen,ZTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *m, d *pl, d *pr, d *dif, npy_complex128 *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ztgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *q, int *ldq, z *z, int *ldz, int *m, d *pl, d *pr, d *dif, z *work, int *lwork, int *iwork, int *liwork, int *info) nogil:
 *     _fortran_ztgsen(ijob, wantq, wantz, select, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, m, pl, pr, dif, <npy_complex128*>work, lwork, iwork, liwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztgsen,ZTGSEN)(__pyx_v_ijob, __pyx_v_wantq, __pyx_v_wantz, __pyx_v_select, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_alpha), ((npy_complex128 *)__pyx_v_beta), ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_z), __pyx_v_ldz, __pyx_v_m, __pyx_v_pl, __pyx_v_pr, __pyx_v_dif, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_iwork, __pyx_v_liwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7791
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsen "F_FUNC(ztgsen,ZTGSEN)"(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *alpha, npy_complex128 *beta, npy_complex128 *q, int *ldq, npy_complex128 *z, int *ldz, int *m, d *pl, d *pr, d *dif, npy_complex128 *work, int *lwork, int *iwork, int *liwork, int *info) nogil
 * cdef void ztgsen(int *ijob, bint *wantq, bint *wantz, bint *select, int *n, z *a, int *lda, z *b, int *ldb, z *alpha, z *beta, z *q, int *ldq, z *z, int *ldz, int *m, d *pl, d *pr, d *dif, z *work, int *lwork, int *iwork, int *liwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsen(ijob, wantq, wantz, select, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>alpha, <npy_complex128*>beta, <npy_complex128*>q, ldq, <npy_complex128*>z, ldz, m, pl, pr, dif, <npy_complex128*>work, lwork, iwork, liwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsja "F_FUNC(ztgsja,ZTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, npy_complex128 *u, int *ldu, npy_complex128 *v, int *ldv, npy_complex128 *q, int *ldq, npy_complex128 *work, int *ncycle, int *info) nogil
 * cdef void ztgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, z *a, int *lda, z *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, z *u, int *ldu, z *v, int *ldv, z *q, int *ldq, z *work, int *ncycle, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsja(jobu, jobv, jobq, m, p, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, tola, tolb, alpha, beta, <npy_complex128*>u, ldu, <npy_complex128*>v, ldv, <npy_complex128*>q, ldq, <npy_complex128*>work, ncycle, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztgsja(char *__pyx_v_jobu, char *__pyx_v_jobv, char *__pyx_v_jobq, int *__pyx_v_m, int *__pyx_v_p, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tola, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_tolb, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_alpha, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_beta, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_ldu, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_ldv, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_ncycle, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7797
 *     void _fortran_ztgsja "F_FUNC(ztgsja,ZTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, npy_complex128 *u, int *ldu, npy_complex128 *v, int *ldv, npy_complex128 *q, int *ldq, npy_complex128 *work, int *ncycle, int *info) nogil
 * cdef void ztgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, z *a, int *lda, z *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, z *u, int *ldu, z *v, int *ldv, z *q, int *ldq, z *work, int *ncycle, int *info) nogil:
 *     _fortran_ztgsja(jobu, jobv, jobq, m, p, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, tola, tolb, alpha, beta, <npy_complex128*>u, ldu, <npy_complex128*>v, ldv, <npy_complex128*>q, ldq, <npy_complex128*>work, ncycle, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztgsja,ZTGSJA)(__pyx_v_jobu, __pyx_v_jobv, __pyx_v_jobq, __pyx_v_m, __pyx_v_p, __pyx_v_n, __pyx_v_k, __pyx_v_l, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_tola, __pyx_v_tolb, __pyx_v_alpha, __pyx_v_beta, ((npy_complex128 *)__pyx_v_u), __pyx_v_ldu, ((npy_complex128 *)__pyx_v_v), __pyx_v_ldv, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_work), __pyx_v_ncycle, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7796
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsja "F_FUNC(ztgsja,ZTGSJA)"(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, npy_complex128 *u, int *ldu, npy_complex128 *v, int *ldv, npy_complex128 *q, int *ldq, npy_complex128 *work, int *ncycle, int *info) nogil
 * cdef void ztgsja(char *jobu, char *jobv, char *jobq, int *m, int *p, int *n, int *k, int *l, z *a, int *lda, z *b, int *ldb, d *tola, d *tolb, d *alpha, d *beta, z *u, int *ldu, z *v, int *ldv, z *q, int *ldq, z *work, int *ncycle, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsja(jobu, jobv, jobq, m, p, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, tola, tolb, alpha, beta, <npy_complex128*>u, ldu, <npy_complex128*>v, ldv, <npy_complex128*>q, ldq, <npy_complex128*>work, ncycle, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsna "F_FUNC(ztgsna,ZTGSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, d *s, d *dif, int *mm, int *m, npy_complex128 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ztgsna(char *job, char *howmny, bint *select, int *n, z *a, int *lda, z *b, int *ldb, z *vl, int *ldvl, z *vr, int *ldvr, d *s, d *dif, int *mm, int *m, z *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsna(job, howmny, select, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, s, dif, mm, m, <npy_complex128*>work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztgsna(char *__pyx_v_job, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dif, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7802
 *     void _fortran_ztgsna "F_FUNC(ztgsna,ZTGSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, d *s, d *dif, int *mm, int *m, npy_complex128 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ztgsna(char *job, char *howmny, bint *select, int *n, z *a, int *lda, z *b, int *ldb, z *vl, int *ldvl, z *vr, int *ldvr, d *s, d *dif, int *mm, int *m, z *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_ztgsna(job, howmny, select, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, s, dif, mm, m, <npy_complex128*>work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztgsna,ZTGSNA)(__pyx_v_job, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_s, __pyx_v_dif, __pyx_v_mm, __pyx_v_m, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7801
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsna "F_FUNC(ztgsna,ZTGSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, d *s, d *dif, int *mm, int *m, npy_complex128 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ztgsna(char *job, char *howmny, bint *select, int *n, z *a, int *lda, z *b, int *ldb, z *vl, int *ldvl, z *vr, int *ldvr, d *s, d *dif, int *mm, int *m, z *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsna(job, howmny, select, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, s, dif, mm, m, <npy_complex128*>work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsy2 "F_FUNC(ztgsy2,ZTGSY2)"(char *trans, int *ijob, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, npy_complex128 *d, int *ldd, npy_complex128 *e, int *lde, npy_complex128 *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *info) nogil
 * cdef void ztgsy2(char *trans, int *ijob, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, z *d, int *ldd, z *e, int *lde, z *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsy2(trans, ijob, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, <npy_complex128*>d, ldd, <npy_complex128*>e, lde, <npy_complex128*>f, ldf, scale, rdsum, rdscal, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztgsy2(char *__pyx_v_trans, int *__pyx_v_ijob, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_d, int *__pyx_v_ldd, __pyx_t_double_complex *__pyx_v_e, int *__pyx_v_lde, __pyx_t_double_complex *__pyx_v_f, int *__pyx_v_ldf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rdsum, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rdscal, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7807
 *     void _fortran_ztgsy2 "F_FUNC(ztgsy2,ZTGSY2)"(char *trans, int *ijob, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, npy_complex128 *d, int *ldd, npy_complex128 *e, int *lde, npy_complex128 *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *info) nogil
 * cdef void ztgsy2(char *trans, int *ijob, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, z *d, int *ldd, z *e, int *lde, z *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *info) nogil:
 *     _fortran_ztgsy2(trans, ijob, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, <npy_complex128*>d, ldd, <npy_complex128*>e, lde, <npy_complex128*>f, ldf, scale, rdsum, rdscal, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztgsy2,ZTGSY2)(__pyx_v_trans, __pyx_v_ijob, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_d), __pyx_v_ldd, ((npy_complex128 *)__pyx_v_e), __pyx_v_lde, ((npy_complex128 *)__pyx_v_f), __pyx_v_ldf, __pyx_v_scale, __pyx_v_rdsum, __pyx_v_rdscal, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7806
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsy2 "F_FUNC(ztgsy2,ZTGSY2)"(char *trans, int *ijob, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, npy_complex128 *d, int *ldd, npy_complex128 *e, int *lde, npy_complex128 *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *info) nogil
 * cdef void ztgsy2(char *trans, int *ijob, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, z *d, int *ldd, z *e, int *lde, z *f, int *ldf, d *scale, d *rdsum, d *rdscal, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsy2(trans, ijob, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, <npy_complex128*>d, ldd, <npy_complex128*>e, lde, <npy_complex128*>f, ldf, scale, rdsum, rdscal, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsyl "F_FUNC(ztgsyl,ZTGSYL)"(char *trans, int *ijob, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, npy_complex128 *d, int *ldd, npy_complex128 *e, int *lde, npy_complex128 *f, int *ldf, d *scale, d *dif, npy_complex128 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ztgsyl(char *trans, int *ijob, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, z *d, int *ldd, z *e, int *lde, z *f, int *ldf, d *scale, d *dif, z *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsyl(trans, ijob, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, <npy_complex128*>d, ldd, <npy_complex128*>e, lde, <npy_complex128*>f, ldf, scale, dif, <npy_complex128*>work, lwork, iwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztgsyl(char *__pyx_v_trans, int *__pyx_v_ijob, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_d, int *__pyx_v_ldd, __pyx_t_double_complex *__pyx_v_e, int *__pyx_v_lde, __pyx_t_double_complex *__pyx_v_f, int *__pyx_v_ldf, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_dif, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_iwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7812
 *     void _fortran_ztgsyl "F_FUNC(ztgsyl,ZTGSYL)"(char *trans, int *ijob, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, npy_complex128 *d, int *ldd, npy_complex128 *e, int *lde, npy_complex128 *f, int *ldf, d *scale, d *dif, npy_complex128 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ztgsyl(char *trans, int *ijob, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, z *d, int *ldd, z *e, int *lde, z *f, int *ldf, d *scale, d *dif, z *work, int *lwork, int *iwork, int *info) nogil:
 *     _fortran_ztgsyl(trans, ijob, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, <npy_complex128*>d, ldd, <npy_complex128*>e, lde, <npy_complex128*>f, ldf, scale, dif, <npy_complex128*>work, lwork, iwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztgsyl,ZTGSYL)(__pyx_v_trans, __pyx_v_ijob, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_d), __pyx_v_ldd, ((npy_complex128 *)__pyx_v_e), __pyx_v_lde, ((npy_complex128 *)__pyx_v_f), __pyx_v_ldf, __pyx_v_scale, __pyx_v_dif, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_iwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7811
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztgsyl "F_FUNC(ztgsyl,ZTGSYL)"(char *trans, int *ijob, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, npy_complex128 *d, int *ldd, npy_complex128 *e, int *lde, npy_complex128 *f, int *ldf, d *scale, d *dif, npy_complex128 *work, int *lwork, int *iwork, int *info) nogil
 * cdef void ztgsyl(char *trans, int *ijob, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, z *d, int *ldd, z *e, int *lde, z *f, int *ldf, d *scale, d *dif, z *work, int *lwork, int *iwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztgsyl(trans, ijob, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, <npy_complex128*>d, ldd, <npy_complex128*>e, lde, <npy_complex128*>f, ldf, scale, dif, <npy_complex128*>work, lwork, iwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztpcon "F_FUNC(ztpcon,ZTPCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex128 *ap, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztpcon(char *norm, char *uplo, char *diag, int *n, z *ap, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztpcon(norm, uplo, diag, n, <npy_complex128*>ap, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztpcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7817
 *     void _fortran_ztpcon "F_FUNC(ztpcon,ZTPCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex128 *ap, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztpcon(char *norm, char *uplo, char *diag, int *n, z *ap, d *rcond, z *work, d *rwork, int *info) nogil:
 *     _fortran_ztpcon(norm, uplo, diag, n, <npy_complex128*>ap, rcond, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztpcon,ZTPCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7816
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztpcon "F_FUNC(ztpcon,ZTPCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex128 *ap, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztpcon(char *norm, char *uplo, char *diag, int *n, z *ap, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztpcon(norm, uplo, diag, n, <npy_complex128*>ap, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztprfs "F_FUNC(ztprfs,ZTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *ap, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztprfs(uplo, trans, diag, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztprfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7822
 *     void _fortran_ztprfs "F_FUNC(ztprfs,ZTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *ap, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_ztprfs(uplo, trans, diag, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztprfs,ZTPRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7821
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztprfs "F_FUNC(ztprfs,ZTPRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztprfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *ap, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztprfs(uplo, trans, diag, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztptri "F_FUNC(ztptri,ZTPTRI)"(char *uplo, char *diag, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void ztptri(char *uplo, char *diag, int *n, z *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztptri(uplo, diag, n, <npy_complex128*>ap, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztptri(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7827
 *     void _fortran_ztptri "F_FUNC(ztptri,ZTPTRI)"(char *uplo, char *diag, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void ztptri(char *uplo, char *diag, int *n, z *ap, int *info) nogil:
 *     _fortran_ztptri(uplo, diag, n, <npy_complex128*>ap, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztptri,ZTPTRI)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7826
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztptri "F_FUNC(ztptri,ZTPTRI)"(char *uplo, char *diag, int *n, npy_complex128 *ap, int *info) nogil
 * cdef void ztptri(char *uplo, char *diag, int *n, z *ap, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztptri(uplo, diag, n, <npy_complex128*>ap, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztptrs "F_FUNC(ztptrs,ZTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztptrs(uplo, trans, diag, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztptrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7832
 *     void _fortran_ztptrs "F_FUNC(ztptrs,ZTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:
 *     _fortran_ztptrs(uplo, trans, diag, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztptrs,ZTPTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7831
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztptrs "F_FUNC(ztptrs,ZTPTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *ap, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztptrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *ap, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztptrs(uplo, trans, diag, n, nrhs, <npy_complex128*>ap, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrcon "F_FUNC(ztrcon,ZTRCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrcon(char *norm, char *uplo, char *diag, int *n, z *a, int *lda, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrcon(norm, uplo, diag, n, <npy_complex128*>a, lda, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrcon(char *__pyx_v_norm, char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rcond, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7837
 *     void _fortran_ztrcon "F_FUNC(ztrcon,ZTRCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrcon(char *norm, char *uplo, char *diag, int *n, z *a, int *lda, d *rcond, z *work, d *rwork, int *info) nogil:
 *     _fortran_ztrcon(norm, uplo, diag, n, <npy_complex128*>a, lda, rcond, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrcon,ZTRCON)(__pyx_v_norm, __pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_rcond, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7836
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrcon "F_FUNC(ztrcon,ZTRCON)"(char *norm, char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, d *rcond, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrcon(char *norm, char *uplo, char *diag, int *n, z *a, int *lda, d *rcond, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrcon(norm, uplo, diag, n, <npy_complex128*>a, lda, rcond, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrevc "F_FUNC(ztrevc,ZTREVC)"(char *side, char *howmny, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrevc(char *side, char *howmny, bint *select, int *n, z *t, int *ldt, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrevc(side, howmny, select, n, <npy_complex128*>t, ldt, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrevc(char *__pyx_v_side, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7842
 *     void _fortran_ztrevc "F_FUNC(ztrevc,ZTREVC)"(char *side, char *howmny, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrevc(char *side, char *howmny, bint *select, int *n, z *t, int *ldt, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *info) nogil:
 *     _fortran_ztrevc(side, howmny, select, n, <npy_complex128*>t, ldt, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrevc,ZTREVC)(__pyx_v_side, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_mm, __pyx_v_m, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7841
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrevc "F_FUNC(ztrevc,ZTREVC)"(char *side, char *howmny, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, int *mm, int *m, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrevc(char *side, char *howmny, bint *select, int *n, z *t, int *ldt, z *vl, int *ldvl, z *vr, int *ldvr, int *mm, int *m, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrevc(side, howmny, select, n, <npy_complex128*>t, ldt, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, mm, m, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrexc "F_FUNC(ztrexc,ZTREXC)"(char *compq, int *n, npy_complex128 *t, int *ldt, npy_complex128 *q, int *ldq, int *ifst, int *ilst, int *info) nogil
 * cdef void ztrexc(char *compq, int *n, z *t, int *ldt, z *q, int *ldq, int *ifst, int *ilst, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrexc(compq, n, <npy_complex128*>t, ldt, <npy_complex128*>q, ldq, ifst, ilst, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrexc(char *__pyx_v_compq, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, int *__pyx_v_ifst, int *__pyx_v_ilst, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7847
 *     void _fortran_ztrexc "F_FUNC(ztrexc,ZTREXC)"(char *compq, int *n, npy_complex128 *t, int *ldt, npy_complex128 *q, int *ldq, int *ifst, int *ilst, int *info) nogil
 * cdef void ztrexc(char *compq, int *n, z *t, int *ldt, z *q, int *ldq, int *ifst, int *ilst, int *info) nogil:
 *     _fortran_ztrexc(compq, n, <npy_complex128*>t, ldt, <npy_complex128*>q, ldq, ifst, ilst, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrexc,ZTREXC)(__pyx_v_compq, __pyx_v_n, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, __pyx_v_ifst, __pyx_v_ilst, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7846
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrexc "F_FUNC(ztrexc,ZTREXC)"(char *compq, int *n, npy_complex128 *t, int *ldt, npy_complex128 *q, int *ldq, int *ifst, int *ilst, int *info) nogil
 * cdef void ztrexc(char *compq, int *n, z *t, int *ldt, z *q, int *ldq, int *ifst, int *ilst, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrexc(compq, n, <npy_complex128*>t, ldt, <npy_complex128*>q, ldq, ifst, ilst, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrrfs "F_FUNC(ztrrfs,ZTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrrfs(uplo, trans, diag, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrrfs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_ldx, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_ferr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_berr, __pyx_t_double_complex *__pyx_v_work, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7852
 *     void _fortran_ztrrfs "F_FUNC(ztrrfs,ZTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:
 *     _fortran_ztrrfs(uplo, trans, diag, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrrfs,ZTRRFS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_x), __pyx_v_ldx, __pyx_v_ferr, __pyx_v_berr, ((npy_complex128 *)__pyx_v_work), __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7851
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrrfs "F_FUNC(ztrrfs,ZTRRFS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *x, int *ldx, d *ferr, d *berr, npy_complex128 *work, d *rwork, int *info) nogil
 * cdef void ztrrfs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, z *x, int *ldx, d *ferr, d *berr, z *work, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrrfs(uplo, trans, diag, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>x, ldx, ferr, berr, <npy_complex128*>work, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrsen "F_FUNC(ztrsen,ZTRSEN)"(char *job, char *compq, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *q, int *ldq, npy_complex128 *w, int *m, d *s, d *sep, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void ztrsen(char *job, char *compq, bint *select, int *n, z *t, int *ldt, z *q, int *ldq, z *w, int *m, d *s, d *sep, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrsen(job, compq, select, n, <npy_complex128*>t, ldt, <npy_complex128*>q, ldq, <npy_complex128*>w, m, s, sep, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrsen(char *__pyx_v_job, char *__pyx_v_compq, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_w, int *__pyx_v_m, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sep, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7857
 *     void _fortran_ztrsen "F_FUNC(ztrsen,ZTRSEN)"(char *job, char *compq, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *q, int *ldq, npy_complex128 *w, int *m, d *s, d *sep, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void ztrsen(char *job, char *compq, bint *select, int *n, z *t, int *ldt, z *q, int *ldq, z *w, int *m, d *s, d *sep, z *work, int *lwork, int *info) nogil:
 *     _fortran_ztrsen(job, compq, select, n, <npy_complex128*>t, ldt, <npy_complex128*>q, ldq, <npy_complex128*>w, m, s, sep, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrsen,ZTRSEN)(__pyx_v_job, __pyx_v_compq, __pyx_v_select, __pyx_v_n, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_w), __pyx_v_m, __pyx_v_s, __pyx_v_sep, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7856
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrsen "F_FUNC(ztrsen,ZTRSEN)"(char *job, char *compq, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *q, int *ldq, npy_complex128 *w, int *m, d *s, d *sep, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void ztrsen(char *job, char *compq, bint *select, int *n, z *t, int *ldt, z *q, int *ldq, z *w, int *m, d *s, d *sep, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrsen(job, compq, select, n, <npy_complex128*>t, ldt, <npy_complex128*>q, ldq, <npy_complex128*>w, m, s, sep, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrsna "F_FUNC(ztrsna,ZTRSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, d *s, d *sep, int *mm, int *m, npy_complex128 *work, int *ldwork, d *rwork, int *info) nogil
 * cdef void ztrsna(char *job, char *howmny, bint *select, int *n, z *t, int *ldt, z *vl, int *ldvl, z *vr, int *ldvr, d *s, d *sep, int *mm, int *m, z *work, int *ldwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrsna(job, howmny, select, n, <npy_complex128*>t, ldt, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, s, sep, mm, m, <npy_complex128*>work, ldwork, rwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrsna(char *__pyx_v_job, char *__pyx_v_howmny, int *__pyx_v_select, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_t, int *__pyx_v_ldt, __pyx_t_double_complex *__pyx_v_vl, int *__pyx_v_ldvl, __pyx_t_double_complex *__pyx_v_vr, int *__pyx_v_ldvr, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_s, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_sep, int *__pyx_v_mm, int *__pyx_v_m, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_ldwork, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_rwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7862
 *     void _fortran_ztrsna "F_FUNC(ztrsna,ZTRSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, d *s, d *sep, int *mm, int *m, npy_complex128 *work, int *ldwork, d *rwork, int *info) nogil
 * cdef void ztrsna(char *job, char *howmny, bint *select, int *n, z *t, int *ldt, z *vl, int *ldvl, z *vr, int *ldvr, d *s, d *sep, int *mm, int *m, z *work, int *ldwork, d *rwork, int *info) nogil:
 *     _fortran_ztrsna(job, howmny, select, n, <npy_complex128*>t, ldt, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, s, sep, mm, m, <npy_complex128*>work, ldwork, rwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrsna,ZTRSNA)(__pyx_v_job, __pyx_v_howmny, __pyx_v_select, __pyx_v_n, ((npy_complex128 *)__pyx_v_t), __pyx_v_ldt, ((npy_complex128 *)__pyx_v_vl), __pyx_v_ldvl, ((npy_complex128 *)__pyx_v_vr), __pyx_v_ldvr, __pyx_v_s, __pyx_v_sep, __pyx_v_mm, __pyx_v_m, ((npy_complex128 *)__pyx_v_work), __pyx_v_ldwork, __pyx_v_rwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7861
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrsna "F_FUNC(ztrsna,ZTRSNA)"(char *job, char *howmny, bint *select, int *n, npy_complex128 *t, int *ldt, npy_complex128 *vl, int *ldvl, npy_complex128 *vr, int *ldvr, d *s, d *sep, int *mm, int *m, npy_complex128 *work, int *ldwork, d *rwork, int *info) nogil
 * cdef void ztrsna(char *job, char *howmny, bint *select, int *n, z *t, int *ldt, z *vl, int *ldvl, z *vr, int *ldvr, d *s, d *sep, int *mm, int *m, z *work, int *ldwork, d *rwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrsna(job, howmny, select, n, <npy_complex128*>t, ldt, <npy_complex128*>vl, ldvl, <npy_complex128*>vr, ldvr, s, sep, mm, m, <npy_complex128*>work, ldwork, rwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrsyl "F_FUNC(ztrsyl,ZTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, d *scale, int *info) nogil
 * cdef void ztrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, d *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrsyl(trana, tranb, isgn, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, scale, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrsyl(char *__pyx_v_trana, char *__pyx_v_tranb, int *__pyx_v_isgn, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_5scipy_6linalg_13cython_lapack_d *__pyx_v_scale, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7867
 *     void _fortran_ztrsyl "F_FUNC(ztrsyl,ZTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, d *scale, int *info) nogil
 * cdef void ztrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, d *scale, int *info) nogil:
 *     _fortran_ztrsyl(trana, tranb, isgn, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, scale, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrsyl,ZTRSYL)(__pyx_v_trana, __pyx_v_tranb, __pyx_v_isgn, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, __pyx_v_scale, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7866
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrsyl "F_FUNC(ztrsyl,ZTRSYL)"(char *trana, char *tranb, int *isgn, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, npy_complex128 *c, int *ldc, d *scale, int *info) nogil
 * cdef void ztrsyl(char *trana, char *tranb, int *isgn, int *m, int *n, z *a, int *lda, z *b, int *ldb, z *c, int *ldc, d *scale, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrsyl(trana, tranb, isgn, m, n, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, <npy_complex128*>c, ldc, scale, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrti2 "F_FUNC(ztrti2,ZTRTI2)"(char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void ztrti2(char *uplo, char *diag, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrti2(uplo, diag, n, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrti2(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7872
 *     void _fortran_ztrti2 "F_FUNC(ztrti2,ZTRTI2)"(char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void ztrti2(char *uplo, char *diag, int *n, z *a, int *lda, int *info) nogil:
 *     _fortran_ztrti2(uplo, diag, n, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrti2,ZTRTI2)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7871
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrti2 "F_FUNC(ztrti2,ZTRTI2)"(char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void ztrti2(char *uplo, char *diag, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrti2(uplo, diag, n, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrtri "F_FUNC(ztrtri,ZTRTRI)"(char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void ztrtri(char *uplo, char *diag, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrtri(uplo, diag, n, <npy_complex128*>a, lda, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrtri(char *__pyx_v_uplo, char *__pyx_v_diag, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7877
 *     void _fortran_ztrtri "F_FUNC(ztrtri,ZTRTRI)"(char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void ztrtri(char *uplo, char *diag, int *n, z *a, int *lda, int *info) nogil:
 *     _fortran_ztrtri(uplo, diag, n, <npy_complex128*>a, lda, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrtri,ZTRTRI)(__pyx_v_uplo, __pyx_v_diag, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7876
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrtri "F_FUNC(ztrtri,ZTRTRI)"(char *uplo, char *diag, int *n, npy_complex128 *a, int *lda, int *info) nogil
 * cdef void ztrtri(char *uplo, char *diag, int *n, z *a, int *lda, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrtri(uplo, diag, n, <npy_complex128*>a, lda, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrtrs "F_FUNC(ztrtrs,ZTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrtrs(uplo, trans, diag, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztrtrs(char *__pyx_v_uplo, char *__pyx_v_trans, char *__pyx_v_diag, int *__pyx_v_n, int *__pyx_v_nrhs, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int *__pyx_v_ldb, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7882
 *     void _fortran_ztrtrs "F_FUNC(ztrtrs,ZTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:
 *     _fortran_ztrtrs(uplo, trans, diag, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztrtrs,ZTRTRS)(__pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_n, __pyx_v_nrhs, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_b), __pyx_v_ldb, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7881
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztrtrs "F_FUNC(ztrtrs,ZTRTRS)"(char *uplo, char *trans, char *diag, int *n, int *nrhs, npy_complex128 *a, int *lda, npy_complex128 *b, int *ldb, int *info) nogil
 * cdef void ztrtrs(char *uplo, char *trans, char *diag, int *n, int *nrhs, z *a, int *lda, z *b, int *ldb, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztrtrs(uplo, trans, diag, n, nrhs, <npy_complex128*>a, lda, <npy_complex128*>b, ldb, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztzrzf "F_FUNC(ztzrzf,ZTZRZF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void ztzrzf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztzrzf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_ztzrzf(int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7887
 *     void _fortran_ztzrzf "F_FUNC(ztzrzf,ZTZRZF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void ztzrzf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_ztzrzf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(ztzrzf,ZTZRZF)(__pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7886
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_ztzrzf "F_FUNC(ztzrzf,ZTZRZF)"(int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void ztzrzf(int *m, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_ztzrzf(m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zung2l "F_FUNC(zung2l,ZUNG2L)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zung2l(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zung2l(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zung2l(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7892
 *     void _fortran_zung2l "F_FUNC(zung2l,ZUNG2L)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zung2l(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zung2l(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zung2l,ZUNG2L)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7891
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zung2l "F_FUNC(zung2l,ZUNG2L)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zung2l(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zung2l(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zung2r "F_FUNC(zung2r,ZUNG2R)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zung2r(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zung2r(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zung2r(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7897
 *     void _fortran_zung2r "F_FUNC(zung2r,ZUNG2R)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zung2r(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zung2r(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zung2r,ZUNG2R)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7896
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zung2r "F_FUNC(zung2r,ZUNG2R)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zung2r(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zung2r(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungbr "F_FUNC(zungbr,ZUNGBR)"(char *vect, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungbr(char *vect, int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungbr(vect, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zungbr(char *__pyx_v_vect, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7902
 *     void _fortran_zungbr "F_FUNC(zungbr,ZUNGBR)"(char *vect, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungbr(char *vect, int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zungbr(vect, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zungbr,ZUNGBR)(__pyx_v_vect, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7901
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungbr "F_FUNC(zungbr,ZUNGBR)"(char *vect, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungbr(char *vect, int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungbr(vect, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunghr "F_FUNC(zunghr,ZUNGHR)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunghr(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunghr(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunghr(int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7907
 *     void _fortran_zunghr "F_FUNC(zunghr,ZUNGHR)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunghr(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunghr(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunghr,ZUNGHR)(__pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7906
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunghr "F_FUNC(zunghr,ZUNGHR)"(int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunghr(int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunghr(n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungl2 "F_FUNC(zungl2,ZUNGL2)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zungl2(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungl2(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zungl2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7912
 *     void _fortran_zungl2 "F_FUNC(zungl2,ZUNGL2)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zungl2(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zungl2(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zungl2,ZUNGL2)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7911
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungl2 "F_FUNC(zungl2,ZUNGL2)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zungl2(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungl2(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunglq "F_FUNC(zunglq,ZUNGLQ)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunglq(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunglq(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunglq(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7917
 *     void _fortran_zunglq "F_FUNC(zunglq,ZUNGLQ)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunglq(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunglq(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunglq,ZUNGLQ)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7916
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunglq "F_FUNC(zunglq,ZUNGLQ)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunglq(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunglq(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungql "F_FUNC(zungql,ZUNGQL)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungql(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungql(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zungql(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7922
 *     void _fortran_zungql "F_FUNC(zungql,ZUNGQL)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungql(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zungql(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zungql,ZUNGQL)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7921
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungql "F_FUNC(zungql,ZUNGQL)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungql(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungql(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungqr "F_FUNC(zungqr,ZUNGQR)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungqr(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungqr(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zungqr(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7927
 *     void _fortran_zungqr "F_FUNC(zungqr,ZUNGQR)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungqr(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zungqr(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zungqr,ZUNGQR)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7926
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungqr "F_FUNC(zungqr,ZUNGQR)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungqr(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungqr(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungr2 "F_FUNC(zungr2,ZUNGR2)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zungr2(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungr2(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zungr2(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7932
 *     void _fortran_zungr2 "F_FUNC(zungr2,ZUNGR2)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zungr2(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:
 *     _fortran_zungr2(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zungr2,ZUNGR2)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7931
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungr2 "F_FUNC(zungr2,ZUNGR2)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *info) nogil
 * cdef void zungr2(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungr2(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungrq "F_FUNC(zungrq,ZUNGRQ)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungrq(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungrq(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zungrq(int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7937
 *     void _fortran_zungrq "F_FUNC(zungrq,ZUNGRQ)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungrq(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zungrq(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zungrq,ZUNGRQ)(__pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7936
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungrq "F_FUNC(zungrq,ZUNGRQ)"(int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungrq(int *m, int *n, int *k, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungrq(m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungtr "F_FUNC(zungtr,ZUNGTR)"(char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungtr(char *uplo, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungtr(uplo, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zungtr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7942
 *     void _fortran_zungtr "F_FUNC(zungtr,ZUNGTR)"(char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungtr(char *uplo, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:
 *     _fortran_zungtr(uplo, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zungtr,ZUNGTR)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7941
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zungtr "F_FUNC(zungtr,ZUNGTR)"(char *uplo, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zungtr(char *uplo, int *n, z *a, int *lda, z *tau, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zungtr(uplo, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunm2l "F_FUNC(zunm2l,ZUNM2L)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunm2l(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunm2l(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunm2l(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7947
 *     void _fortran_zunm2l "F_FUNC(zunm2l,ZUNM2L)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunm2l(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:
 *     _fortran_zunm2l(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunm2l,ZUNM2L)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7946
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunm2l "F_FUNC(zunm2l,ZUNM2L)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunm2l(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunm2l(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunm2r "F_FUNC(zunm2r,ZUNM2R)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunm2r(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunm2r(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunm2r(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7952
 *     void _fortran_zunm2r "F_FUNC(zunm2r,ZUNM2R)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunm2r(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:
 *     _fortran_zunm2r(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunm2r,ZUNM2R)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7951
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunm2r "F_FUNC(zunm2r,ZUNM2R)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunm2r(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunm2r(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmbr "F_FUNC(zunmbr,ZUNMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmbr(char *vect, char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmbr(vect, side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmbr(char *__pyx_v_vect, char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7957
 *     void _fortran_zunmbr "F_FUNC(zunmbr,ZUNMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmbr(char *vect, char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunmbr(vect, side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmbr,ZUNMBR)(__pyx_v_vect, __pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7956
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmbr "F_FUNC(zunmbr,ZUNMBR)"(char *vect, char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmbr(char *vect, char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmbr(vect, side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmhr "F_FUNC(zunmhr,ZUNMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmhr(side, trans, m, n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmhr(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_ilo, int *__pyx_v_ihi, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7962
 *     void _fortran_zunmhr "F_FUNC(zunmhr,ZUNMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunmhr(side, trans, m, n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmhr,ZUNMHR)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_ilo, __pyx_v_ihi, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7961
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmhr "F_FUNC(zunmhr,ZUNMHR)"(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmhr(char *side, char *trans, int *m, int *n, int *ilo, int *ihi, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmhr(side, trans, m, n, ilo, ihi, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunml2 "F_FUNC(zunml2,ZUNML2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunml2(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunml2(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunml2(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7967
 *     void _fortran_zunml2 "F_FUNC(zunml2,ZUNML2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunml2(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:
 *     _fortran_zunml2(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunml2,ZUNML2)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7966
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunml2 "F_FUNC(zunml2,ZUNML2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunml2(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunml2(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmlq "F_FUNC(zunmlq,ZUNMLQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmlq(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmlq(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmlq(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7972
 *     void _fortran_zunmlq "F_FUNC(zunmlq,ZUNMLQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmlq(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunmlq(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmlq,ZUNMLQ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7971
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmlq "F_FUNC(zunmlq,ZUNMLQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmlq(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmlq(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmql "F_FUNC(zunmql,ZUNMQL)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmql(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmql(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmql(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7977
 *     void _fortran_zunmql "F_FUNC(zunmql,ZUNMQL)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmql(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunmql(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmql,ZUNMQL)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7976
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmql "F_FUNC(zunmql,ZUNMQL)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmql(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmql(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmqr "F_FUNC(zunmqr,ZUNMQR)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmqr(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmqr(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmqr(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7982
 *     void _fortran_zunmqr "F_FUNC(zunmqr,ZUNMQR)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmqr(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunmqr(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmqr,ZUNMQR)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7981
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmqr "F_FUNC(zunmqr,ZUNMQR)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmqr(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmqr(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmr2 "F_FUNC(zunmr2,ZUNMR2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunmr2(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmr2(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmr2(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7987
 *     void _fortran_zunmr2 "F_FUNC(zunmr2,ZUNMR2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunmr2(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:
 *     _fortran_zunmr2(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmr2,ZUNMR2)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7986
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmr2 "F_FUNC(zunmr2,ZUNMR2)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunmr2(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmr2(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmr3 "F_FUNC(zunmr3,ZUNMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunmr3(char *side, char *trans, int *m, int *n, int *k, int *l, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmr3(side, trans, m, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmr3(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7992
 *     void _fortran_zunmr3 "F_FUNC(zunmr3,ZUNMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunmr3(char *side, char *trans, int *m, int *n, int *k, int *l, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:
 *     _fortran_zunmr3(side, trans, m, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmr3,ZUNMR3)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7991
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmr3 "F_FUNC(zunmr3,ZUNMR3)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zunmr3(char *side, char *trans, int *m, int *n, int *k, int *l, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmr3(side, trans, m, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":7996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmrq "F_FUNC(zunmrq,ZUNMRQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmrq(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmrq(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmrq(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":7997
 *     void _fortran_zunmrq "F_FUNC(zunmrq,ZUNMRQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmrq(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunmrq(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmrq,ZUNMRQ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":7996
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmrq "F_FUNC(zunmrq,ZUNMRQ)"(char *side, char *trans, int *m, int *n, int *k, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmrq(char *side, char *trans, int *m, int *n, int *k, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmrq(side, trans, m, n, k, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":8001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmrz "F_FUNC(zunmrz,ZUNMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmrz(char *side, char *trans, int *m, int *n, int *k, int *l, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmrz(side, trans, m, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmrz(char *__pyx_v_side, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, int *__pyx_v_k, int *__pyx_v_l, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":8002
 *     void _fortran_zunmrz "F_FUNC(zunmrz,ZUNMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmrz(char *side, char *trans, int *m, int *n, int *k, int *l, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunmrz(side, trans, m, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmrz,ZUNMRZ)(__pyx_v_side, __pyx_v_trans, __pyx_v_m, __pyx_v_n, __pyx_v_k, __pyx_v_l, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":8001
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmrz "F_FUNC(zunmrz,ZUNMRZ)"(char *side, char *trans, int *m, int *n, int *k, int *l, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmrz(char *side, char *trans, int *m, int *n, int *k, int *l, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmrz(side, trans, m, n, k, l, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":8006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmtr "F_FUNC(zunmtr,ZUNMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmtr(char *side, char *uplo, char *trans, int *m, int *n, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmtr(side, uplo, trans, m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zunmtr(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int *__pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_lwork, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":8007
 *     void _fortran_zunmtr "F_FUNC(zunmtr,ZUNMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmtr(char *side, char *uplo, char *trans, int *m, int *n, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:
 *     _fortran_zunmtr(side, uplo, trans, m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zunmtr,ZUNMTR)(__pyx_v_side, __pyx_v_uplo, __pyx_v_trans, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_a), __pyx_v_lda, ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_lwork, __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":8006
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zunmtr "F_FUNC(zunmtr,ZUNMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex128 *a, int *lda, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *lwork, int *info) nogil
 * cdef void zunmtr(char *side, char *uplo, char *trans, int *m, int *n, z *a, int *lda, z *tau, z *c, int *ldc, z *work, int *lwork, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zunmtr(side, uplo, trans, m, n, <npy_complex128*>a, lda, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, lwork, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":8011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zupgtr "F_FUNC(zupgtr,ZUPGTR)"(char *uplo, int *n, npy_complex128 *ap, npy_complex128 *tau, npy_complex128 *q, int *ldq, npy_complex128 *work, int *info) nogil
 * cdef void zupgtr(char *uplo, int *n, z *ap, z *tau, z *q, int *ldq, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zupgtr(uplo, n, <npy_complex128*>ap, <npy_complex128*>tau, <npy_complex128*>q, ldq, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zupgtr(char *__pyx_v_uplo, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_ldq, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":8012
 *     void _fortran_zupgtr "F_FUNC(zupgtr,ZUPGTR)"(char *uplo, int *n, npy_complex128 *ap, npy_complex128 *tau, npy_complex128 *q, int *ldq, npy_complex128 *work, int *info) nogil
 * cdef void zupgtr(char *uplo, int *n, z *ap, z *tau, z *q, int *ldq, z *work, int *info) nogil:
 *     _fortran_zupgtr(uplo, n, <npy_complex128*>ap, <npy_complex128*>tau, <npy_complex128*>q, ldq, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "_lapack_subroutines.h":
 */
  F_FUNC(zupgtr,ZUPGTR)(__pyx_v_uplo, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_q), __pyx_v_ldq, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":8011
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zupgtr "F_FUNC(zupgtr,ZUPGTR)"(char *uplo, int *n, npy_complex128 *ap, npy_complex128 *tau, npy_complex128 *q, int *ldq, npy_complex128 *work, int *info) nogil
 * cdef void zupgtr(char *uplo, int *n, z *ap, z *tau, z *q, int *ldq, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zupgtr(uplo, n, <npy_complex128*>ap, <npy_complex128*>tau, <npy_complex128*>q, ldq, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":8016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zupmtr "F_FUNC(zupmtr,ZUPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex128 *ap, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zupmtr(char *side, char *uplo, char *trans, int *m, int *n, z *ap, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zupmtr(side, uplo, trans, m, n, <npy_complex128*>ap, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

static void __pyx_f_5scipy_6linalg_13cython_lapack_zupmtr(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_trans, int *__pyx_v_m, int *__pyx_v_n, __pyx_t_double_complex *__pyx_v_ap, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int *__pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int *__pyx_v_info) {

  /* "scipy/linalg/cython_lapack.pyx":8017
 *     void _fortran_zupmtr "F_FUNC(zupmtr,ZUPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex128 *ap, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zupmtr(char *side, char *uplo, char *trans, int *m, int *n, z *ap, z *tau, z *c, int *ldc, z *work, int *info) nogil:
 *     _fortran_zupmtr(side, uplo, trans, m, n, <npy_complex128*>ap, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  F_FUNC(zupmtr,ZUPMTR)(__pyx_v_side, __pyx_v_uplo, __pyx_v_trans, __pyx_v_m, __pyx_v_n, ((npy_complex128 *)__pyx_v_ap), ((npy_complex128 *)__pyx_v_tau), ((npy_complex128 *)__pyx_v_c), __pyx_v_ldc, ((npy_complex128 *)__pyx_v_work), __pyx_v_info);

  /* "scipy/linalg/cython_lapack.pyx":8016
 * cdef extern from "_lapack_subroutines.h":
 *     void _fortran_zupmtr "F_FUNC(zupmtr,ZUPMTR)"(char *side, char *uplo, char *trans, int *m, int *n, npy_complex128 *ap, npy_complex128 *tau, npy_complex128 *c, int *ldc, npy_complex128 *work, int *info) nogil
 * cdef void zupmtr(char *side, char *uplo, char *trans, int *m, int *n, z *ap, z *tau, z *c, int *ldc, z *work, int *info) nogil:             # <<<<<<<<<<<<<<
 *     _fortran_zupmtr(side, uplo, trans, m, n, <npy_complex128*>ap, <npy_complex128*>tau, <npy_complex128*>c, ldc, <npy_complex128*>work, info)
 * 
 */

  /* function exit code */
}

/* "scipy/linalg/cython_lapack.pyx":8022
 * # Python accessible wrappers for testing:
 * 
 * def _test_dlamch(cmach):             # <<<<<<<<<<<<<<
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6linalg_13cython_lapack_1_test_dlamch(PyObject *__pyx_self, PyObject *__pyx_v_cmach); /*proto*/
static PyMethodDef __pyx_mdef_5scipy_6linalg_13cython_lapack_1_test_dlamch = {"_test_dlamch", (PyCFunction)__pyx_pw_5scipy_6linalg_13cython_lapack_1_test_dlamch, METH_O, 0};
static PyObject *__pyx_pw_5scipy_6linalg_13cython_lapack_1_test_dlamch(PyObject *__pyx_self, PyObject *__pyx_v_cmach) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_dlamch (wrapper)", 0);
  __pyx_r = __pyx_pf_5scipy_6linalg_13cython_lapack__test_dlamch(__pyx_self, ((PyObject *)__pyx_v_cmach));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6linalg_13cython_lapack__test_dlamch(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cmach) {
  PyObject *__pyx_v_cmach_bytes = NULL;
  char *__pyx_v_cmach_char;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  char *__pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_dlamch", 0);

  /* "scipy/linalg/cython_lapack.pyx":8024
 * def _test_dlamch(cmach):
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)             # <<<<<<<<<<<<<<
 *     # Now that it is a bytes representation, a non-temporary variable
 *     # must be passed as a part of the function call.
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_cmach);
  __Pyx_GIVEREF(__pyx_v_cmach);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_cmach);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyBytes_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_cmach_bytes = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/linalg/cython_lapack.pyx":8027
 *     # Now that it is a bytes representation, a non-temporary variable
 *     # must be passed as a part of the function call.
 *     cdef char* cmach_char = cmach_bytes             # <<<<<<<<<<<<<<
 *     return dlamch(cmach_char)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_AsString(__pyx_v_cmach_bytes); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_cmach_char = __pyx_t_3;

  /* "scipy/linalg/cython_lapack.pyx":8028
 *     # must be passed as a part of the function call.
 *     cdef char* cmach_char = cmach_bytes
 *     return dlamch(cmach_char)             # <<<<<<<<<<<<<<
 * 
 * def _test_slamch(cmach):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_f_5scipy_6linalg_13cython_lapack_dlamch(__pyx_v_cmach_char)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":8022
 * # Python accessible wrappers for testing:
 * 
 * def _test_dlamch(cmach):             # <<<<<<<<<<<<<<
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("scipy.linalg.cython_lapack._test_dlamch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cmach_bytes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/cython_lapack.pyx":8030
 *     return dlamch(cmach_char)
 * 
 * def _test_slamch(cmach):             # <<<<<<<<<<<<<<
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6linalg_13cython_lapack_3_test_slamch(PyObject *__pyx_self, PyObject *__pyx_v_cmach); /*proto*/
static PyMethodDef __pyx_mdef_5scipy_6linalg_13cython_lapack_3_test_slamch = {"_test_slamch", (PyCFunction)__pyx_pw_5scipy_6linalg_13cython_lapack_3_test_slamch, METH_O, 0};
static PyObject *__pyx_pw_5scipy_6linalg_13cython_lapack_3_test_slamch(PyObject *__pyx_self, PyObject *__pyx_v_cmach) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_slamch (wrapper)", 0);
  __pyx_r = __pyx_pf_5scipy_6linalg_13cython_lapack_2_test_slamch(__pyx_self, ((PyObject *)__pyx_v_cmach));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6linalg_13cython_lapack_2_test_slamch(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cmach) {
  PyObject *__pyx_v_cmach_bytes = NULL;
  char *__pyx_v_cmach_char;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  char *__pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_slamch", 0);

  /* "scipy/linalg/cython_lapack.pyx":8032
 * def _test_slamch(cmach):
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)             # <<<<<<<<<<<<<<
 *     # Now that it is a bytes representation, a non-temporary variable
 *     # must be passed as a part of the function call.
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_cmach);
  __Pyx_GIVEREF(__pyx_v_cmach);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_cmach);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyBytes_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_cmach_bytes = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/linalg/cython_lapack.pyx":8035
 *     # Now that it is a bytes representation, a non-temporary variable
 *     # must be passed as a part of the function call.
 *     cdef char* cmach_char = cmach_bytes             # <<<<<<<<<<<<<<
 *     return slamch(cmach_char)
 */
  __pyx_t_3 = __Pyx_PyObject_AsString(__pyx_v_cmach_bytes); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_cmach_char = __pyx_t_3;

  /* "scipy/linalg/cython_lapack.pyx":8036
 *     # must be passed as a part of the function call.
 *     cdef char* cmach_char = cmach_bytes
 *     return slamch(cmach_char)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_f_5scipy_6linalg_13cython_lapack_slamch(__pyx_v_cmach_char)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "scipy/linalg/cython_lapack.pyx":8030
 *     return dlamch(cmach_char)
 * 
 * def _test_slamch(cmach):             # <<<<<<<<<<<<<<
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("scipy.linalg.cython_lapack._test_slamch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cmach_bytes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_copy_shape;
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_v_hasfields;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":203
 *             # of flags
 * 
 *             if info == NULL: return             # <<<<<<<<<<<<<<
 * 
 *             cdef int copy_shape, i, ndim
 */
  __pyx_t_1 = ((__pyx_v_info == NULL) != 0);
  if (__pyx_t_1) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":206
 * 
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":207
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":209
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":212
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 copy_shape = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 copy_shape = 0
 */
    __pyx_v_copy_shape = 1;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
    goto __pyx_L4;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":214
 *                 copy_shape = 1
 *             else:
 *                 copy_shape = 0             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  /*else*/ {
    __pyx_v_copy_shape = 0;
  }
  __pyx_L4:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":217
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":221
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":224
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if copy_shape:
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":225
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if copy_shape:
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (__pyx_v_copy_shape != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":229
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)malloc((((sizeof(Py_ssize_t)) * ((size_t)__pyx_v_ndim)) * 2)));

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":230
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":231
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":232
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":233
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L11;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":235
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":236
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L11:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":237
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":238
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":239
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":242
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = self.descr
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":243
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = self.descr             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->descr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":246
 *             cdef int offset
 * 
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields and not copy_shape:
 */
  __pyx_v_hasfields = PyDataType_HASFIELDS(__pyx_v_descr);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
  __pyx_t_2 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_copy_shape != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":250
 *             if not hasfields and not copy_shape:
 *                 # do not call releasebuffer
 *                 info.obj = None             # <<<<<<<<<<<<<<
 *             else:
 *                 # need to call releasebuffer
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = Py_None;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
    goto __pyx_L14;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":253
 *             else:
 *                 # need to call releasebuffer
 *                 info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields:
 */
  /*else*/ {
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = ((PyObject *)__pyx_v_self);
  }
  __pyx_L14:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":256
 * 
 *             if not hasfields:
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L20_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_L20_next_or:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L19_bool_binop_done:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_1) {

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":260
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = __pyx_k_b;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":261
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      case NPY_UBYTE:
      __pyx_v_f = __pyx_k_B;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":262
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      case NPY_SHORT:
      __pyx_v_f = __pyx_k_h;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":263
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      case NPY_USHORT:
      __pyx_v_f = __pyx_k_H;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":264
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      case NPY_INT:
      __pyx_v_f = __pyx_k_i;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":265
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      case NPY_UINT:
      __pyx_v_f = __pyx_k_I;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      case NPY_LONG:
      __pyx_v_f = __pyx_k_l;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":267
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      case NPY_ULONG:
      __pyx_v_f = __pyx_k_L;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      case NPY_LONGLONG:
      __pyx_v_f = __pyx_k_q;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":269
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      case NPY_ULONGLONG:
      __pyx_v_f = __pyx_k_Q;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      case NPY_FLOAT:
      __pyx_v_f = __pyx_k_f;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":271
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      case NPY_DOUBLE:
      __pyx_v_f = __pyx_k_d;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      case NPY_LONGDOUBLE:
      __pyx_v_f = __pyx_k_g;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":273
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      case NPY_CFLOAT:
      __pyx_v_f = __pyx_k_Zf;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      case NPY_CDOUBLE:
      __pyx_v_f = __pyx_k_Zd;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":275
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      case NPY_CLONGDOUBLE:
      __pyx_v_f = __pyx_k_Zg;
      break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      case NPY_OBJECT:
      __pyx_v_f = __pyx_k_O;
      break;
      default:

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      break;
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":279
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":282
 *                 return
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)malloc(0xFF));

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_7 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_f = __pyx_t_7;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":288
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":292
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)
 */
    free(__pyx_v_info->format);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":294
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    free(__pyx_v_info->strides);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":771
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":774
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 774; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":780
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 780; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":783
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 783; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":790
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":791
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":795
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":796
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (__pyx_t_6) {

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_6) {

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":813
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":814
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":815
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":816
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":818
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (__pyx_t_6) {

        /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":826
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":828
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":829
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":831
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":832
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":834
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":835
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":837
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":838
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":839
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":841
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":844
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L15:;

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":845
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":849
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":850
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  PyObject *__pyx_v_baseptr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
  __pyx_t_1 = (__pyx_v_base == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":969
 *      cdef PyObject* baseptr
 *      if base is None:
 *          baseptr = NULL             # <<<<<<<<<<<<<<
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 */
    __pyx_v_baseptr = NULL;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
    goto __pyx_L3;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":971
 *          baseptr = NULL
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!             # <<<<<<<<<<<<<<
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 */
  /*else*/ {
    Py_INCREF(__pyx_v_base);

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":972
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base             # <<<<<<<<<<<<<<
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr
 */
    __pyx_v_baseptr = ((PyObject *)__pyx_v_base);
  }
  __pyx_L3:;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":973
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)             # <<<<<<<<<<<<<<
 *      arr.base = baseptr
 * 
 */
  Py_XDECREF(__pyx_v_arr->base);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":974
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_v_arr->base = __pyx_v_baseptr;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arr->base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":978
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return <object>arr.base
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":980
 *         return None
 *     else:
 *         return <object>arr.base             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_arr->base));
    __pyx_r = ((PyObject *)__pyx_v_arr->base);
    goto __pyx_L0;
  }

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "cython_lapack",
    __pyx_k_LAPACK_functions_for_Cython_Usa, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_cmach, __pyx_k_cmach, sizeof(__pyx_k_cmach), 0, 0, 1, 1},
  {&__pyx_n_s_cmach_bytes, __pyx_k_cmach_bytes, sizeof(__pyx_k_cmach_bytes), 0, 0, 1, 1},
  {&__pyx_n_s_cmach_char, __pyx_k_cmach_char, sizeof(__pyx_k_cmach_char), 0, 0, 1, 1},
  {&__pyx_kp_s_home_br_repos_scipy_scipy_linal, __pyx_k_home_br_repos_scipy_scipy_linal, sizeof(__pyx_k_home_br_repos_scipy_scipy_linal), 0, 0, 1, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_linalg_cython_lapack, __pyx_k_scipy_linalg_cython_lapack, sizeof(__pyx_k_scipy_linalg_cython_lapack), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_test_dlamch, __pyx_k_test_dlamch, sizeof(__pyx_k_test_dlamch), 0, 0, 1, 1},
  {&__pyx_n_s_test_slamch, __pyx_k_test_slamch, sizeof(__pyx_k_test_slamch), 0, 0, 1, 1},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__5)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "scipy/linalg/cython_lapack.pyx":8022
 * # Python accessible wrappers for testing:
 * 
 * def _test_dlamch(cmach):             # <<<<<<<<<<<<<<
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)
 */
  __pyx_tuple__7 = PyTuple_Pack(3, __pyx_n_s_cmach, __pyx_n_s_cmach_bytes, __pyx_n_s_cmach_char); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __pyx_codeobj__8 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__7, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_linal, __pyx_n_s_test_dlamch, 8022, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "scipy/linalg/cython_lapack.pyx":8030
 *     return dlamch(cmach_char)
 * 
 * def _test_slamch(cmach):             # <<<<<<<<<<<<<<
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)
 */
  __pyx_tuple__9 = PyTuple_Pack(3, __pyx_n_s_cmach, __pyx_n_s_cmach_bytes, __pyx_n_s_cmach_char); if (unlikely(!__pyx_tuple__9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8030; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_br_repos_scipy_scipy_linal, __pyx_n_s_test_slamch, 8030, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8030; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initcython_lapack(void); /*proto*/
PyMODINIT_FUNC initcython_lapack(void)
#else
PyMODINIT_FUNC PyInit_cython_lapack(void); /*proto*/
PyMODINIT_FUNC PyInit_cython_lapack(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_cython_lapack(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("cython_lapack", __pyx_methods, __pyx_k_LAPACK_functions_for_Cython_Usa, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_scipy__linalg__cython_lapack) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "scipy.linalg.cython_lapack")) {
      if (unlikely(PyDict_SetItemString(modules, "scipy.linalg.cython_lapack", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("cbdsqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cbdsqr, "void (char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbbrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbbrd, "void (char *, int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbcon, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbequ, "void (int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbrfs, "void (char *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbsv, "void (int *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbsvx, "void (char *, char *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbtf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbtf2, "void (int *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbtrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbtrf, "void (int *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgbtrs, "void (char *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgebak", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgebak, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgebal", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgebal, "void (char *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgebd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgebd2, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgebrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgebrd, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgecon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgecon, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeequ, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgees", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgees, "void (char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_cselect1 *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeesx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeesx, "void (char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_cselect1 *, char *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeev, "void (char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeevx, "void (char *, char *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgehd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgehd2, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgehrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgehrd, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgelq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgelq2, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgelqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgelqf, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgels", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgels, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgelsd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgelsd, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgelss", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgelss, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgelsy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgelsy, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeql2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeql2, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeqlf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeqlf, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeqp3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeqp3, "void (int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeqr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeqr2, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgeqrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgeqrf, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgerfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgerfs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgerq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgerq2, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgerqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgerqf, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgesc2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgesc2, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgesdd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgesdd, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgesv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgesv, "void (int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgesvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgesvd, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgesvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgesvx, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgetc2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgetc2, "void (int *, __pyx_t_float_complex *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgetf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgetf2, "void (int *, int *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgetrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgetrf, "void (int *, int *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgetri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgetri, "void (int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgetrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgetrs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cggbak", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cggbak, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cggbal", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cggbal, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgges", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgges, "void (char *, char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_cselect2 *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cggesx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cggesx, "void (char *, char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_cselect2 *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cggev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cggev, "void (char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cggevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cggevx, "void (char *, char *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cggglm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cggglm, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgghrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgghrd, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgglse", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgglse, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cggqrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cggqrf, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cggrqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cggrqf, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgtcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgtcon, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgtrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgtrfs, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgtsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgtsv, "void (int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgtsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgtsvx, "void (char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgttrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgttrf, "void (int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgttrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgttrs, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cgtts2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cgtts2, "void (int *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chbev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chbev, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chbevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chbevd, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chbevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chbevx, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chbgst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chbgst, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chbgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chbgv, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chbgvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chbgvd, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chbgvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chbgvx, "void (char *, char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chbtrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chbtrd, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("checon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_checon, "void (char *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cheev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cheev, "void (char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cheevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cheevd, "void (char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cheevr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cheevr, "void (char *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cheevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cheevx, "void (char *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chegs2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chegs2, "void (int *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chegst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chegst, "void (int *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chegv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chegv, "void (int *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chegvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chegvd, "void (int *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chegvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chegvx, "void (int *, char *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cherfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cherfs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chesv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chesv, "void (char *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chesvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chesvx, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chetd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chetd2, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chetf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chetf2, "void (char *, int *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chetrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chetrd, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chetrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chetrf, "void (char *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chetri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chetri, "void (char *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chetrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chetrs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chgeqz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chgeqz, "void (char *, char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpcon, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpev, "void (char *, char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpevd, "void (char *, char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpevx, "void (char *, char *, char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpgst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpgst, "void (int *, char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpgv, "void (int *, char *, char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpgvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpgvd, "void (int *, char *, char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpgvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpgvx, "void (int *, char *, char *, char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chprfs, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpsv, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chpsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chpsvx, "void (char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chptrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chptrd, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chptrf, "void (char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chptri, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chptrs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chsein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chsein, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("chseqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_chseqr, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clabrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clabrd, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clacgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clacgv, "void (int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clacn2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clacn2, "void (int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clacon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clacon, "void (int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clacp2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clacp2, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clacpy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clacpy, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clacrm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clacrm, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clacrt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clacrt, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cladiv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cladiv, "__pyx_t_float_complex (__pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claed0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claed0, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claed7", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claed7, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claed8", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claed8, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claein, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claesy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claesy, "void (__pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claev2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claev2, "void (__pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clag2z", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clag2z, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clags2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clags2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clagtm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clagtm, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clahef", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clahef, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clahqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clahqr, "void (int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clahr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clahr2, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claic1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claic1, "void (int *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clals0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clals0, "void (int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clalsa", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clalsa, "void (int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clalsd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clalsd, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clangb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clangb, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clange", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clange, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clangt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clangt, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clanhb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clanhb, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clanhe", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clanhe, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clanhp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clanhp, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clanhs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clanhs, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clanht", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clanht, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clansb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clansb, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clansp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clansp, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clansy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clansy, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clantb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clantb, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clantp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clantp, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clantr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clantr, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clapll", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clapll, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clapmt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clapmt, "void (int *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqgb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqgb, "void (int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqge", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqge, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqhb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqhb, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqhe", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqhe, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqhp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqhp, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqp2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqp2, "void (int *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqps", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqps, "void (int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqr0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqr0, "void (int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqr1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqr1, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqr2, "void (int *, int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqr3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqr3, "void (int *, int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqr4", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqr4, "void (int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqr5", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqr5, "void (int *, int *, int *, int *, int *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqsb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqsb, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqsp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqsp, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claqsy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claqsy, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clar1v", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clar1v, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clar2v", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clar2v, "void (int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarcm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarcm, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarf, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarfb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarfb, "void (char *, char *, char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarfg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarfg, "void (int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarft", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarft, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarfx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarfx, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clargv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clargv, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarnv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarnv, "void (int *, int *, int *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarrv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarrv, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clartg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clartg, "void (__pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clartv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clartv, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarz, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarzb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarzb, "void (char *, char *, char *, char *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clarzt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clarzt, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clascl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clascl, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claset", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claset, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clasr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clasr, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("classq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_classq, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("claswp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_claswp, "void (int *, __pyx_t_float_complex *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clasyf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clasyf, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clatbs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clatbs, "void (char *, char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clatdf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clatdf, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clatps", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clatps, "void (char *, char *, char *, char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clatrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clatrd, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clatrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clatrs, "void (char *, char *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clatrz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clatrz, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clauu2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clauu2, "void (char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("clauum", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_clauum, "void (char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbcon, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbequ, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbrfs, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbstf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbstf, "void (char *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbsv, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbsvx, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbtf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbtf2, "void (char *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbtrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbtrf, "void (char *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpbtrs, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpocon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpocon, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpoequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpoequ, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cporfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cporfs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cposv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cposv, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cposvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cposvx, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpotf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpotf2, "void (char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpotrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpotrf, "void (char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpotri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpotri, "void (char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpotrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpotrs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cppcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cppcon, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cppequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cppequ, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpprfs, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cppsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cppsv, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cppsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cppsvx, "void (char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpptrf, "void (char *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpptri, "void (char *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpptrs, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cptcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cptcon, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpteqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpteqr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cptrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cptrfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cptsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cptsv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cptsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cptsvx, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpttrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpttrf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cpttrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cpttrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cptts2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cptts2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("crot", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_crot, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cspcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cspcon, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cspmv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cspmv, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cspr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cspr, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csprfs, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cspsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cspsv, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cspsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cspsvx, "void (char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csptrf, "void (char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csptri, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csptrs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csrscl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csrscl, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cstedc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cstedc, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cstegr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cstegr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cstein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cstein, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cstemr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cstemr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csteqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csteqr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csycon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csycon, "void (char *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csymv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csymv, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csyr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csyr, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csyrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csyrfs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csysv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csysv, "void (char *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csysvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csysvx, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csytf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csytf2, "void (char *, int *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csytrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csytrf, "void (char *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csytri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csytri, "void (char *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("csytrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_csytrs, "void (char *, int *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctbcon, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctbrfs, "void (char *, char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctbtrs, "void (char *, char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctgevc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctgevc, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctgex2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctgex2, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctgexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctgexc, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctgsen", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctgsen, "void (int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctgsja", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctgsja, "void (char *, char *, char *, int *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctgsna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctgsna, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctgsy2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctgsy2, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctgsyl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctgsyl, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctpcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctpcon, "void (char *, char *, char *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctprfs, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctptri, "void (char *, char *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctptrs, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrcon, "void (char *, char *, char *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrevc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrevc, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrexc, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrrfs, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrsen", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrsen, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrsna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrsna, "void (char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrsyl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrsyl, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrti2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrti2, "void (char *, char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrtri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrtri, "void (char *, char *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctrtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctrtrs, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ctzrzf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ctzrzf, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cung2l", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cung2l, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cung2r", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cung2r, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cungbr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cungbr, "void (char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunghr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunghr, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cungl2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cungl2, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunglq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunglq, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cungql", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cungql, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cungqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cungqr, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cungr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cungr2, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cungrq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cungrq, "void (int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cungtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cungtr, "void (char *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunm2l", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunm2l, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunm2r", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunm2r, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmbr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmbr, "void (char *, char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmhr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmhr, "void (char *, char *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunml2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunml2, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmlq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmlq, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmql", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmql, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmqr, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmr2, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmr3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmr3, "void (char *, char *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmrq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmrq, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmrz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmrz, "void (char *, char *, int *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cunmtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cunmtr, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cupgtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cupgtr, "void (char *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("cupmtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_cupmtr, "void (char *, char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dbdsdc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dbdsdc, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dbdsqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dbdsqr, "void (char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ddisna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ddisna, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbbrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbbrd, "void (char *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbcon, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbequ, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbrfs, "void (char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbsv, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbsvx, "void (char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbtf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbtf2, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbtrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbtrf, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgbtrs, "void (char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgebak", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgebak, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgebal", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgebal, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgebd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgebd2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgebrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgebrd, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgecon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgecon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeequ, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgees", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgees, "void (char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_dselect2 *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeesx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeesx, "void (char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_dselect2 *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeev, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeevx, "void (char *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgehd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgehd2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgehrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgehrd, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgelq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgelq2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgelqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgelqf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgels", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgels, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgelsd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgelsd, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgelss", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgelss, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgelsy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgelsy, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeql2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeql2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeqlf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeqlf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeqp3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeqp3, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeqr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeqr2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgeqrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgeqrf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgerfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgerfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgerq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgerq2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgerqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgerqf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgesc2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgesc2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgesdd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgesdd, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgesv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgesv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgesvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgesvd, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgesvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgesvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgetc2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgetc2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgetf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgetf2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgetrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgetrf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgetri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgetri, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgetrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgetrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dggbak", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dggbak, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dggbal", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dggbal, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgges", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgges, "void (char *, char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_dselect3 *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dggesx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dggesx, "void (char *, char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_dselect3 *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dggev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dggev, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dggevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dggevx, "void (char *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dggglm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dggglm, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgghrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgghrd, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgglse", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgglse, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dggqrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dggqrf, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dggrqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dggrqf, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgtcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgtcon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgtrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgtrfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgtsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgtsv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgtsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgtsvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgttrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgttrf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgttrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgttrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dgtts2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dgtts2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dhgeqz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dhgeqz, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dhsein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dhsein, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dhseqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dhseqr, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("disnan", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_disnan, "int (__pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlabad", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlabad, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlabrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlabrd, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlacn2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlacn2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlacon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlacon, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlacpy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlacpy, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dladiv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dladiv, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlae2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlae2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaebz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaebz, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed0, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed1, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed3, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed4", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed4, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed5", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed5, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed6", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed6, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed7", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed7, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed8", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed8, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaed9", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaed9, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaeda", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaeda, "void (int *, int *, int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaein, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaev2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaev2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaexc, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlag2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlag2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlag2s", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlag2s, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlags2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlags2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlagtf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlagtf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlagtm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlagtm, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlagts", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlagts, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlagv2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlagv2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlahqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlahqr, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlahr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlahr2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaic1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaic1, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaln2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaln2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlals0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlals0, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlalsa", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlalsa, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlalsd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlalsd, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlamch", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlamch, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlamrg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlamrg, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaneg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaneg, "int (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlangb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlangb, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlange", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlange, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlangt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlangt, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlanhs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlanhs, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlansb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlansb, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlansp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlansp, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlanst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlanst, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlansy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlansy, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlantb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlantb, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlantp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlantp, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlantr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlantr, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlanv2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlanv2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlapll", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlapll, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlapmt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlapmt, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlapy2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlapy2, "__pyx_t_5scipy_6linalg_13cython_lapack_d (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlapy3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlapy3, "__pyx_t_5scipy_6linalg_13cython_lapack_d (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqgb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqgb, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqge", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqge, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqp2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqp2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqps", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqps, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqr0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqr0, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqr1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqr1, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqr2, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqr3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqr3, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqr4", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqr4, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqr5", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqr5, "void (int *, int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqsb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqsb, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqsp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqsp, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqsy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqsy, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaqtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaqtr, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlar1v", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlar1v, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlar2v", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlar2v, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarf, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarfb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarfb, "void (char *, char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarfg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarfg, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarft", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarft, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarfx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarfx, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlargv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlargv, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarnv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarnv, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarra", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarra, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarrb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarrb, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarrc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarrc, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarrd, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarre", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarre, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarrf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarrj", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarrj, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarrk", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarrk, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarrr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarrr, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarrv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarrv, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlartg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlartg, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlartv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlartv, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaruv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaruv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarz, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarzb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarzb, "void (char *, char *, char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlarzt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlarzt, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlas2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlas2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlascl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlascl, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd0, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd1, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd2, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd3, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd4", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd4, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd5", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd5, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd6", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd6, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd7", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd7, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasd8", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasd8, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasda", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasda, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasdq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasdq, "void (char *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasdt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasdt, "void (int *, int *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaset", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaset, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasq1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasq1, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasq2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasq6", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasq6, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasr, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasrt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasrt, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlassq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlassq, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasv2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasv2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlaswp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlaswp, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasy2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasy2, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlasyf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlasyf, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlatbs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlatbs, "void (char *, char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlatdf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlatdf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlatps", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlatps, "void (char *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlatrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlatrd, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlatrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlatrs, "void (char *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlatrz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlatrz, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlauu2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlauu2, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dlauum", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dlauum, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dopgtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dopgtr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dopmtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dopmtr, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorg2l", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorg2l, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorg2r", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorg2r, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorgbr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorgbr, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorghr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorghr, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorgl2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorgl2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorglq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorglq, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorgql", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorgql, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorgqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorgqr, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorgr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorgr2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorgrq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorgrq, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorgtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorgtr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorm2l", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorm2l, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorm2r", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorm2r, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormbr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormbr, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormhr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormhr, "void (char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dorml2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dorml2, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormlq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormlq, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormql", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormql, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormqr, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormr2, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormr3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormr3, "void (char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormrq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormrq, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormrz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormrz, "void (char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dormtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dormtr, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbcon, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbequ, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbrfs, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbstf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbstf, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbsv, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbsvx, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbtf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbtf2, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbtrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbtrf, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpbtrs, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpocon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpocon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpoequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpoequ, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dporfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dporfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dposv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dposv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dposvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dposvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpotf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpotf2, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpotrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpotrf, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpotri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpotri, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpotrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpotrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dppcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dppcon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dppequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dppequ, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpprfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dppsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dppsv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dppsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dppsvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpptrf, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpptri, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpptrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dptcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dptcon, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpteqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpteqr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dptrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dptrfs, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dptsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dptsv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dptsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dptsvx, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpttrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpttrf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dpttrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dpttrs, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dptts2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dptts2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("drscl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_drscl, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsbev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsbev, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsbevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsbevd, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsbevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsbevx, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsbgst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsbgst, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsbgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsbgv, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsbgvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsbgvd, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsbgvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsbgvx, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsbtrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsbtrd, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsgesv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsgesv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspcon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspev, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspevd, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspevx, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspgst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspgst, "void (int *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspgv, "void (int *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspgvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspgvd, "void (int *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspgvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspgvx, "void (int *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsprfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspsv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dspsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dspsvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsptrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsptrd, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsptrf, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsptri, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsptrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstebz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstebz, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstedc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstedc, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstegr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstegr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstein, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstemr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstemr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsteqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsteqr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsterf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsterf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstev, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstevd, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstevr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstevr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dstevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dstevx, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsycon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsycon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsyev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsyev, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsyevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsyevd, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsyevr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsyevr, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsyevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsyevx, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsygs2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsygs2, "void (int *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsygst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsygst, "void (int *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsygv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsygv, "void (int *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsygvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsygvd, "void (int *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsygvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsygvx, "void (int *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsyrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsyrfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsysv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsysv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsysvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsysvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsytd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsytd2, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsytf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsytf2, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsytrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsytrd, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsytrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsytrf, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsytri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsytri, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dsytrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dsytrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtbcon, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtbrfs, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtbtrs, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtgevc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtgevc, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtgex2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtgex2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtgexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtgexc, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtgsen", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtgsen, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtgsja", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtgsja, "void (char *, char *, char *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtgsna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtgsna, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtgsy2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtgsy2, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtgsyl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtgsyl, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtpcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtpcon, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtprfs, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtptri, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtptrs, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrcon, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrevc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrevc, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrexc, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrrfs, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrsen", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrsen, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrsna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrsna, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrsyl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrsyl, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrti2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrti2, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrtri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrtri, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtrtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtrtrs, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dtzrzf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dtzrzf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dzsum1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_dzsum1, "__pyx_t_5scipy_6linalg_13cython_lapack_d (int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("icmax1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_icmax1, "int (int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ieeeck", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ieeeck, "int (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ilaver", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ilaver, "void (int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("izmax1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_izmax1, "int (int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sbdsdc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sbdsdc, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sbdsqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sbdsqr, "void (char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("scsum1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_scsum1, "__pyx_t_5scipy_6linalg_13cython_lapack_s (int *, __pyx_t_float_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sdisna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sdisna, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbbrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbbrd, "void (char *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbcon, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbequ, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbrfs, "void (char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbsv, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbsvx, "void (char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbtf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbtf2, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbtrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbtrf, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgbtrs, "void (char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgebak", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgebak, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgebal", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgebal, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgebd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgebd2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgebrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgebrd, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgecon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgecon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeequ, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgees", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgees, "void (char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_sselect2 *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeesx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeesx, "void (char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_sselect2 *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeev, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeevx, "void (char *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgehd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgehd2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgehrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgehrd, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgelq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgelq2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgelqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgelqf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgels", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgels, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgelsd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgelsd, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgelss", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgelss, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgelsy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgelsy, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeql2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeql2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeqlf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeqlf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeqp3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeqp3, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeqr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeqr2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgeqrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgeqrf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgerfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgerfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgerq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgerq2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgerqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgerqf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgesc2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgesc2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgesdd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgesdd, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgesv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgesv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgesvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgesvd, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgesvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgesvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgetc2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgetc2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgetf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgetf2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgetrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgetrf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgetri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgetri, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgetrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgetrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sggbak", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sggbak, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sggbal", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sggbal, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgges", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgges, "void (char *, char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_sselect3 *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sggesx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sggesx, "void (char *, char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_sselect3 *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sggev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sggev, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sggevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sggevx, "void (char *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sggglm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sggglm, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgghrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgghrd, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgglse", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgglse, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sggqrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sggqrf, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sggrqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sggrqf, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgtcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgtcon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgtrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgtrfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgtsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgtsv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgtsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgtsvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgttrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgttrf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgttrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgttrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sgtts2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sgtts2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("shgeqz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_shgeqz, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("shsein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_shsein, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("shseqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_shseqr, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slabad", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slabad, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slabrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slabrd, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slacn2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slacn2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slacon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slacon, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slacpy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slacpy, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sladiv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sladiv, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slae2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slae2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaebz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaebz, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed0, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed1, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed3, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed4", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed4, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed5", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed5, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed6", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed6, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed7", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed7, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed8", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed8, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaed9", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaed9, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaeda", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaeda, "void (int *, int *, int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaein, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaev2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaev2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaexc, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slag2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slag2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slag2d", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slag2d, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slags2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slags2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slagtf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slagtf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slagtm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slagtm, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slagts", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slagts, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slagv2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slagv2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slahqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slahqr, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slahr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slahr2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaic1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaic1, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaln2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaln2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slals0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slals0, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slalsa", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slalsa, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slalsd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slalsd, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slamch", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slamch, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slamrg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slamrg, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slangb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slangb, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slange", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slange, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slangt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slangt, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slanhs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slanhs, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slansb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slansb, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slansp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slansp, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slanst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slanst, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slansy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slansy, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slantb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slantb, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slantp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slantp, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slantr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slantr, "__pyx_t_5scipy_6linalg_13cython_lapack_s (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slanv2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slanv2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slapll", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slapll, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slapmt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slapmt, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slapy2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slapy2, "__pyx_t_5scipy_6linalg_13cython_lapack_s (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slapy3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slapy3, "__pyx_t_5scipy_6linalg_13cython_lapack_s (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqgb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqgb, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqge", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqge, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqp2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqp2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqps", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqps, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqr0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqr0, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqr1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqr1, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqr2, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqr3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqr3, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqr4", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqr4, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqr5", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqr5, "void (int *, int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqsb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqsb, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqsp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqsp, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqsy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqsy, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaqtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaqtr, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slar1v", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slar1v, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slar2v", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slar2v, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarf, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarfb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarfb, "void (char *, char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarfg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarfg, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarft", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarft, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarfx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarfx, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slargv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slargv, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarnv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarnv, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarra", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarra, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarrb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarrb, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarrc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarrc, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarrd, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarre", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarre, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarrf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarrj", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarrj, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarrk", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarrk, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarrr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarrr, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarrv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarrv, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slartg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slartg, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slartv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slartv, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaruv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaruv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarz, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarzb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarzb, "void (char *, char *, char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slarzt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slarzt, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slas2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slas2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slascl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slascl, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd0, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd1, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd2, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd3, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd4", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd4, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd5", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd5, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd6", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd6, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd7", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd7, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasd8", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasd8, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasda", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasda, "void (int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasdq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasdq, "void (char *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasdt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasdt, "void (int *, int *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaset", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaset, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasq1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasq1, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasq2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasq6", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasq6, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasr, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasrt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasrt, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slassq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slassq, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasv2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasv2, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slaswp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slaswp, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasy2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasy2, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slasyf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slasyf, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slatbs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slatbs, "void (char *, char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slatdf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slatdf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slatps", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slatps, "void (char *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slatrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slatrd, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slatrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slatrs, "void (char *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slatrz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slatrz, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slauu2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slauu2, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("slauum", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_slauum, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sopgtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sopgtr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sopmtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sopmtr, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorg2l", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorg2l, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorg2r", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorg2r, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorgbr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorgbr, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorghr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorghr, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorgl2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorgl2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorglq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorglq, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorgql", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorgql, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorgqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorgqr, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorgr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorgr2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorgrq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorgrq, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorgtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorgtr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorm2l", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorm2l, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorm2r", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorm2r, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormbr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormbr, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormhr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormhr, "void (char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sorml2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sorml2, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormlq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormlq, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormql", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormql, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormqr, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormr2, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormr3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormr3, "void (char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormrq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormrq, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormrz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormrz, "void (char *, char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sormtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sormtr, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbcon, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbequ, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbrfs, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbstf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbstf, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbsv, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbsvx, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbtf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbtf2, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbtrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbtrf, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spbtrs, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spocon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spocon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spoequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spoequ, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sporfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sporfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sposv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sposv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sposvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sposvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spotf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spotf2, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spotrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spotrf, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spotri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spotri, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spotrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spotrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sppcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sppcon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sppequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sppequ, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spprfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sppsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sppsv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sppsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sppsvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spptrf, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spptri, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spptrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sptcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sptcon, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spteqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spteqr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sptrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sptrfs, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sptsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sptsv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sptsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sptsvx, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spttrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spttrf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("spttrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_spttrs, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sptts2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sptts2, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("srscl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_srscl, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssbev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssbev, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssbevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssbevd, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssbevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssbevx, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssbgst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssbgst, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssbgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssbgv, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssbgvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssbgvd, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssbgvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssbgvx, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssbtrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssbtrd, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspcon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspev, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspevd, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspevx, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspgst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspgst, "void (int *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspgv, "void (int *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspgvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspgvd, "void (int *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspgvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspgvx, "void (int *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssprfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspsv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sspsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sspsvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssptrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssptrd, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssptrf, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssptri, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssptrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstebz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstebz, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstedc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstedc, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstegr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstegr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstein, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstemr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstemr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssteqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssteqr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssterf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssterf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstev, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstevd, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstevr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstevr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("sstevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_sstevx, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssycon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssycon, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssyev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssyev, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssyevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssyevd, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssyevr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssyevr, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssyevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssyevx, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssygs2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssygs2, "void (int *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssygst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssygst, "void (int *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssygv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssygv, "void (int *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssygvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssygvd, "void (int *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssygvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssygvx, "void (int *, char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssyrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssyrfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssysv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssysv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssysvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssysvx, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssytd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssytd2, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssytf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssytf2, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssytrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssytrd, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssytrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssytrf, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssytri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssytri, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ssytrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ssytrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stbcon, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stbrfs, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stbtrs, "void (char *, char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stgevc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stgevc, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stgex2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stgex2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stgexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stgexc, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stgsen", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stgsen, "void (int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stgsja", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stgsja, "void (char *, char *, char *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stgsna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stgsna, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stgsy2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stgsy2, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stgsyl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stgsyl, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stpcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stpcon, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stprfs, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stptri, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stptrs, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strcon, "void (char *, char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strevc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strevc, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strexc, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strrfs, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strsen", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strsen, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strsna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strsna, "void (char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strsyl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strsyl, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strti2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strti2, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strtri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strtri, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("strtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_strtrs, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("stzrzf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_stzrzf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zbdsqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zbdsqr, "void (char *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zdrscl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zdrscl, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbbrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbbrd, "void (char *, int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbcon, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbequ, "void (int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbrfs, "void (char *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbsv, "void (int *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbsvx, "void (char *, char *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbtf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbtf2, "void (int *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbtrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbtrf, "void (int *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgbtrs, "void (char *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgebak", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgebak, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgebal", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgebal, "void (char *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgebd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgebd2, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgebrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgebrd, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgecon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgecon, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeequ, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgees", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgees, "void (char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_zselect1 *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeesx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeesx, "void (char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_zselect1 *, char *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeev, "void (char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeevx, "void (char *, char *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgehd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgehd2, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgehrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgehrd, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgelq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgelq2, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgelqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgelqf, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgels", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgels, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgelsd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgelsd, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgelss", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgelss, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgelsy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgelsy, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeql2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeql2, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeqlf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeqlf, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeqp3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeqp3, "void (int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeqr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeqr2, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgeqrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgeqrf, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgerfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgerfs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgerq2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgerq2, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgerqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgerqf, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgesc2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgesc2, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgesdd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgesdd, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgesv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgesv, "void (int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgesvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgesvd, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgesvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgesvx, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgetc2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgetc2, "void (int *, __pyx_t_double_complex *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgetf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgetf2, "void (int *, int *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgetrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgetrf, "void (int *, int *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgetri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgetri, "void (int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgetrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgetrs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zggbak", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zggbak, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zggbal", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zggbal, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgges", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgges, "void (char *, char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_zselect2 *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zggesx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zggesx, "void (char *, char *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_zselect2 *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zggev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zggev, "void (char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zggevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zggevx, "void (char *, char *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zggglm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zggglm, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgghrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgghrd, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgglse", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgglse, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zggqrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zggqrf, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zggrqf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zggrqf, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgtcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgtcon, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgtrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgtrfs, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgtsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgtsv, "void (int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgtsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgtsvx, "void (char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgttrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgttrf, "void (int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgttrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgttrs, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zgtts2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zgtts2, "void (int *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhbev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhbev, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhbevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhbevd, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhbevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhbevx, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhbgst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhbgst, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhbgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhbgv, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhbgvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhbgvd, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhbgvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhbgvx, "void (char *, char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhbtrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhbtrd, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhecon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhecon, "void (char *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zheev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zheev, "void (char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zheevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zheevd, "void (char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zheevr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zheevr, "void (char *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zheevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zheevx, "void (char *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhegs2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhegs2, "void (int *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhegst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhegst, "void (int *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhegv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhegv, "void (int *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhegvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhegvd, "void (int *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhegvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhegvx, "void (int *, char *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zherfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zherfs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhesv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhesv, "void (char *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhesvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhesvx, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhetd2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhetd2, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhetf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhetf2, "void (char *, int *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhetrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhetrd, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhetrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhetrf, "void (char *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhetri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhetri, "void (char *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhetrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhetrs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhgeqz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhgeqz, "void (char *, char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpcon, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpev", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpev, "void (char *, char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpevd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpevd, "void (char *, char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpevx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpevx, "void (char *, char *, char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpgst", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpgst, "void (int *, char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpgv, "void (int *, char *, char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpgvd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpgvd, "void (int *, char *, char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpgvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpgvx, "void (int *, char *, char *, char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhprfs, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpsv, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhpsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhpsvx, "void (char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhptrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhptrd, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhptrf, "void (char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhptri, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhptrs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhsein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhsein, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zhseqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zhseqr, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlabrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlabrd, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlacgv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlacgv, "void (int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlacn2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlacn2, "void (int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlacon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlacon, "void (int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlacp2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlacp2, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlacpy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlacpy, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlacrm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlacrm, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlacrt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlacrt, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zladiv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zladiv, "__pyx_t_double_complex (__pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaed0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaed0, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaed7", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaed7, "void (int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaed8", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaed8, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaein, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaesy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaesy, "void (__pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaev2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaev2, "void (__pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlag2c", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlag2c, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlags2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlags2, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlagtm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlagtm, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlahef", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlahef, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlahqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlahqr, "void (int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlahr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlahr2, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaic1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaic1, "void (int *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlals0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlals0, "void (int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlalsa", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlalsa, "void (int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlalsd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlalsd, "void (char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlangb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlangb, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlange", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlange, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlangt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlangt, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlanhb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlanhb, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlanhe", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlanhe, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlanhp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlanhp, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlanhs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlanhs, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlanht", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlanht, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlansb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlansb, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlansp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlansp, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlansy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlansy, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlantb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlantb, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlantp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlantp, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlantr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlantr, "__pyx_t_5scipy_6linalg_13cython_lapack_d (char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlapll", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlapll, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlapmt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlapmt, "void (int *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqgb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqgb, "void (int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqge", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqge, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqhb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqhb, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqhe", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqhe, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqhp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqhp, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqp2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqp2, "void (int *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqps", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqps, "void (int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqr0", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqr0, "void (int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqr1", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqr1, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqr2, "void (int *, int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqr3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqr3, "void (int *, int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqr4", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqr4, "void (int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqr5", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqr5, "void (int *, int *, int *, int *, int *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqsb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqsb, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqsp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqsp, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaqsy", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaqsy, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlar1v", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlar1v, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlar2v", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlar2v, "void (int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarcm", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarcm, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarf, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarfb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarfb, "void (char *, char *, char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarfg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarfg, "void (int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarft", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarft, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarfx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarfx, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlargv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlargv, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarnv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarnv, "void (int *, int *, int *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarrv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarrv, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlartg", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlartg, "void (__pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlartv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlartv, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarz, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarzb", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarzb, "void (char *, char *, char *, char *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlarzt", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlarzt, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlascl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlascl, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaset", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaset, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlasr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlasr, "void (char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlassq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlassq, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlaswp", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlaswp, "void (int *, __pyx_t_double_complex *, int *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlasyf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlasyf, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlatbs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlatbs, "void (char *, char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlatdf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlatdf, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlatps", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlatps, "void (char *, char *, char *, char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlatrd", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlatrd, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlatrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlatrs, "void (char *, char *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlatrz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlatrz, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlauu2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlauu2, "void (char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zlauum", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zlauum, "void (char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbcon, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbequ, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbrfs, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbstf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbstf, "void (char *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbsv, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbsvx, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbtf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbtf2, "void (char *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbtrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbtrf, "void (char *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpbtrs, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpocon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpocon, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpoequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpoequ, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zporfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zporfs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zposv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zposv, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zposvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zposvx, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpotf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpotf2, "void (char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpotrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpotrf, "void (char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpotri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpotri, "void (char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpotrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpotrs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zppcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zppcon, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zppequ", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zppequ, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpprfs, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zppsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zppsv, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zppsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zppsvx, "void (char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, char *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpptrf, "void (char *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpptri, "void (char *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpptrs, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zptcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zptcon, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpteqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpteqr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zptrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zptrfs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zptsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zptsv, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zptsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zptsvx, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpttrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpttrf, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zpttrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zpttrs, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zptts2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zptts2, "void (int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zrot", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zrot, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zspcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zspcon, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zspmv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zspmv, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zspr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zspr, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsprfs, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zspsv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zspsv, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zspsvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zspsvx, "void (char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsptrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsptrf, "void (char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsptri, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsptrs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zstedc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zstedc, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zstegr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zstegr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zstein", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zstein, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zstemr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zstemr, "void (char *, char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsteqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsteqr, "void (char *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsycon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsycon, "void (char *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsymv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsymv, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsyr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsyr, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsyrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsyrfs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsysv", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsysv, "void (char *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsysvx", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsysvx, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsytf2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsytf2, "void (char *, int *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsytrf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsytrf, "void (char *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsytri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsytri, "void (char *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zsytrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zsytrs, "void (char *, int *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztbcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztbcon, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztbrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztbrfs, "void (char *, char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztbtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztbtrs, "void (char *, char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztgevc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztgevc, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztgex2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztgex2, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztgexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztgexc, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztgsen", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztgsen, "void (int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztgsja", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztgsja, "void (char *, char *, char *, int *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztgsna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztgsna, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztgsy2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztgsy2, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztgsyl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztgsyl, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztpcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztpcon, "void (char *, char *, char *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztprfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztprfs, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztptri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztptri, "void (char *, char *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztptrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztptrs, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrcon", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrcon, "void (char *, char *, char *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrevc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrevc, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrexc", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrexc, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrrfs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrrfs, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrsen", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrsen, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrsna", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrsna, "void (char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrsyl", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrsyl, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrti2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrti2, "void (char *, char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrtri", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrtri, "void (char *, char *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztrtrs", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztrtrs, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("ztzrzf", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_ztzrzf, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zung2l", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zung2l, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zung2r", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zung2r, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zungbr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zungbr, "void (char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunghr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunghr, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zungl2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zungl2, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunglq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunglq, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zungql", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zungql, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zungqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zungqr, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zungr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zungr2, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zungrq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zungrq, "void (int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zungtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zungtr, "void (char *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunm2l", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunm2l, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunm2r", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunm2r, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmbr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmbr, "void (char *, char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmhr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmhr, "void (char *, char *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunml2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunml2, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmlq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmlq, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmql", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmql, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmqr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmqr, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmr2", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmr2, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmr3", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmr3, "void (char *, char *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmrq", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmrq, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmrz", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmrz, "void (char *, char *, int *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zunmtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zunmtr, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zupgtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zupgtr, "void (char *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("zupmtr", (void (*)(void))__pyx_f_5scipy_6linalg_13cython_lapack_zupmtr, "void (char *, char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_5numpy_dtype)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_flatiter)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_broadcast)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_5numpy_ndarray)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType("numpy", "ufunc", sizeof(PyUFuncObject), 0); if (unlikely(!__pyx_ptype_5numpy_ufunc)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "scipy/linalg/cython_lapack.pyx":8022
 * # Python accessible wrappers for testing:
 * 
 * def _test_dlamch(cmach):             # <<<<<<<<<<<<<<
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6linalg_13cython_lapack_1_test_dlamch, NULL, __pyx_n_s_scipy_linalg_cython_lapack); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_dlamch, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/linalg/cython_lapack.pyx":8030
 *     return dlamch(cmach_char)
 * 
 * def _test_slamch(cmach):             # <<<<<<<<<<<<<<
 *     # This conversion is necessary to handle Python 3 strings.
 *     cmach_bytes = bytes(cmach)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6linalg_13cython_lapack_3_test_slamch, NULL, __pyx_n_s_scipy_linalg_cython_lapack); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8030; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_slamch, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 8030; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/linalg/cython_lapack.pyx":1
 * # This file was generated by _cython_wrapper_generators.py.             # <<<<<<<<<<<<<<
 * # Do not edit this file directly.
 * 
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../virtualenvs/scipy-dev/local/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init scipy.linalg.cython_lapack", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init scipy.linalg.cython_lapack");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        float denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(a, a);
                    case 3:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, a);
                    case 4:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_absf(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        double denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(a, a);
                    case 3:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, a);
                    case 4:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_abs(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) -1, const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
